'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComplainService = void 0;
const common_1 = require("@nestjs/common");
const sequelize_1 = require("sequelize");
const base_service_1 = require("../../core/service/base.service");
const Media_entity_1 = require("../../modules/media/models/Media.entity");
const Channel_entity_1 = require("../../modules/channel/models/Channel.entity");
const User_entity_1 = require("../../modules/user/models/User.entity");
const Process_exception_1 = require("../../middleware/exception/Process.exception");
const config_1 = require("../../config");
const Country_entity_1 = require("../../modules/common/models/Country.entity");
let ComplainService = class ComplainService extends base_service_1.BaseService {
    constructor(complainRepository, complaintAttachmentRepository, complaintCategoryRepository, mediaRepository, channelRepository, userRepository, complaintReplyContentRepository, sequelizeInstance) {
        super();
        this.complainRepository = complainRepository;
        this.complaintAttachmentRepository = complaintAttachmentRepository;
        this.complaintCategoryRepository = complaintCategoryRepository;
        this.mediaRepository = mediaRepository;
        this.channelRepository = channelRepository;
        this.userRepository = userRepository;
        this.complaintReplyContentRepository = complaintReplyContentRepository;
        this.sequelizeInstance = sequelizeInstance;
    }
    async getComplainCategoryByComplainType(complainType) {
        let result = await this.complaintCategoryRepository.scope('baseDetail').findAll({
            where: {
                complainttype: complainType,
                isactive: '1',
                deletedat: null
            }
        });
        return result;
    }
    async setComplain(user, complain) {
        let complaintnumber;
        let result = await this.sequelizeInstance.transaction(async (transaction) => {
            let complainObject = {};
            complainObject.complainttype = complain.complainttype;
            complainObject.channelid = (user && user.channelId != undefined) ? user.channelId : null;
            complainObject.userid = (user && user.userId != undefined) ? user.userId : null;
            complainObject.channeltype = complain.channeltype;
            complainObject.firstname = complain.firstname;
            complainObject.lastname = complain.lastname;
            complainObject.email = (user && user.email != undefined) ? user.email : complain.email;
            complainObject.phoneno = complain.phoneno != undefined ? complain.phoneno : null;
            complainObject.mobileno = complain.mobileno != undefined ? complain.mobileno : null;
            complainObject.countryid = complain.countryid;
            complainObject.subject = complain.subject;
            complainObject.detaildesc = complain.detaildesc;
            complainObject.iprurl = complain.iprurl != undefined ? complain.iprurl : null;
            complainObject.complaintcategoryid = complain.complaintcategoryid;
            complainObject.status = 'pending';
            complainObject.createdat = this.sequelizeInstance.fn('NOW');
            let complainSaved = await this.complainRepository.create(complainObject, {
                transaction
            }).then(async (result) => {
                complaintnumber = await this.sequelizeInstance.query('select \'COMPLAIN\' || ((EXTRACT(YEAR FROM now()):: integer * 10000) + (EXTRACT(MONTH FROM now()):: integer * 100) + (EXTRACT(day FROM now()):: integer * 1)) :: varchar(8) || (count(*) + 1) :: varchar(10) as number from tblcomplaint where to_char(createdat, \'dd/mm/yyyy\') = to_char(now(), \'dd/mm/yyyy\') and "complaintid" <= ' + result.complaintid, {
                    type: this.sequelizeInstance.QueryTypes.SELECT,
                    raw: true,
                });
                result.complaintnumber = complaintnumber[0].number;
                await result.save({
                    transaction
                });
                return result;
            }).catch((e) => {
                throw new Process_exception_1.ProcessException('validate:sql:internalError');
            });
            if (complain.compalainDocument.length > 0) {
                for (let ii = 0; ii < complain.compalainDocument.length; ii++) {
                    let documentObject = {};
                    documentObject.documentPath = complain.compalainDocument[ii].documentPath;
                    documentObject.documentTypeId = complain.compalainDocument[ii].documenttype == 'image' ? config_1.appconstant.DOCUMENTTYPE_IMAGE : config_1.appconstant.DOCUMENTTYPE_DOC;
                    let document = await this.mediaRepository.create(documentObject, {
                        transaction
                    });
                    let complaintAttachmentObject = {};
                    complaintAttachmentObject.attachmentid = document.documentId;
                    complaintAttachmentObject.complaintid = complainSaved.complaintid;
                    await this.complaintAttachmentRepository.create(complaintAttachmentObject, { transaction });
                }
            }
            return complainSaved;
        });
        let complainsaved = await this.complainRepository.findOne({
            where: {
                complaintnumber: complaintnumber[0].number
            },
            include: [{
                    model: this.complaintAttachmentRepository,
                    as: 'complaintAttachment',
                    required: false,
                    include: [{
                            model: this.mediaRepository,
                            required: false,
                        }]
                }, {
                    model: this.complaintCategoryRepository
                }, {
                    model: this.channelRepository,
                    required: false
                }, {
                    model: this.userRepository,
                    required: false
                }, {
                    model: Country_entity_1.Country,
                    as: 'country',
                    attributes: ['countryId', 'country']
                }],
        });
        return result;
    }
    async getCount() {
        return await this.complainRepository.count();
    }
    async getComplain(dto, order) {
        let filter = dto.filter;
        let filters = {
            complainttype: dto.complainttype
        };
        if (dto.search) {
            filters[sequelize_1.Op.or] = {
                complaintnumber: { [sequelize_1.Op.iLike]: '%' + dto.search + '%' },
                firstname: { [sequelize_1.Op.iLike]: '%' + dto.search + '%' },
                lastname: { [sequelize_1.Op.iLike]: '%' + dto.search + '%' },
                subject: { [sequelize_1.Op.iLike]: '%' + dto.search + '%' },
            };
        }
        if (filter.status) {
            filters['status'] = filter.status;
        }
        if (filter.complaintcategoryid) {
            filters['complaintcategoryid'] = filter.complaintcategoryid;
        }
        if (filter.channeltype) {
            filters['channeltype'] = filter.channeltype;
        }
        let results = await this.complainRepository.findAll({
            where: filters,
            include: [{
                    model: this.complaintAttachmentRepository,
                    as: 'complaintAttachment',
                    required: false,
                    include: [{
                            model: this.mediaRepository
                        }]
                }, {
                    model: this.complaintCategoryRepository
                }, {
                    model: this.channelRepository
                }, {
                    model: this.userRepository
                }, {
                    model: Country_entity_1.Country,
                    as: 'country',
                    attributes: ['countryId', 'country']
                }],
            limit: dto.limit,
            offset: dto.offset,
            order: [['complaintid', order || 'desc']]
        });
        let count = 0;
        let aggregations = {};
        if (results) {
            count = await this.complainRepository.count({
                where: filters
            });
            let query = 'SELECT json_build_object(\'status\',to_json(array_agg(distinct("Complain"."status"))), \'channeltype\',to_json(array_agg(distinct("Complain"."channeltype"))), \'category\',to_json(array_agg(distinct(tcc))) )  AS "filters" FROM "tblcomplaint" AS "Complain" left join (select complaintcategoryid as id, complaintcategory as name from "tblcomplaintcategory") tcc on "Complain".complaintcategoryid = tcc."id"';
            query += ' where "Complain"."complainttype" = :complainttype';
            if (dto.search) {
                query += ' AND ("Complain"."complaintnumber" ILIKE :search ';
                query += ' OR "Complain"."firstname" ILIKE :search ';
                query += ' OR "Complain"."lastname" ILIKE :search ';
                query += ' OR "Complain"."subject" ILIKE :search )';
            }
            if (filter.status) {
                query += ' AND "Complain"."status" IN (:status)';
            }
            if (filter.complaintcategoryid) {
                query += ' AND "Complain"."complaintcategoryid" IN (:complaintcategoryid)';
            }
            if (filter.channeltype) {
                query += ' AND "Complain"."channeltype" IN (:channeltype)';
            }
            aggregations = await this.sequelizeInstance.query(query, {
                type: this.sequelizeInstance.QueryTypes.SELECT,
                replacements: {
                    complainttype: dto.complainttype || null,
                    search: '%' + dto.search + '%',
                    status: filter.status || null,
                    complaintcategoryid: filter.complaintcategoryid || null,
                    channeltype: filter.channeltype || null
                }
            }).then(complaints => {
                return complaints[0].filters;
            });
        }
        return {
            rows: results,
            count: count,
            aggregations: aggregations
        };
    }
    async setComplainAction(user, complainId, complainActionDto) {
        let result = await this.complainRepository.findOne({
            where: {
                complaintid: complainId
            },
            include: [{
                    model: this.complaintAttachmentRepository,
                    as: 'complaintAttachment',
                    required: false,
                    include: [{
                            model: this.mediaRepository,
                            required: false,
                        }]
                }, {
                    model: this.complaintCategoryRepository
                }, {
                    model: this.channelRepository,
                    required: false
                }, {
                    model: this.userRepository,
                    required: false
                }, {
                    model: Country_entity_1.Country,
                    as: 'country',
                    attributes: ['countryId', 'country']
                }],
        });
        if (!result) {
            throw new Process_exception_1.ProcessException('Record not found');
        }
        if (complainActionDto.action == 'replied') {
            if (result.status == 'pending' || result.status == 'replied') {
                let emailcontent = {};
                emailcontent.createdat = this.sequelizeInstance.fn('NOW');
                emailcontent.content = complainActionDto.compalaintreply.content;
                emailcontent.complaintid = result.complaintid;
                emailcontent.replyedby = result.user ? result.user.contactId : 1;
                await this.complaintReplyContentRepository.create(emailcontent).then(async (result1) => {
                    return result1;
                }).catch((e) => {
                    throw new Process_exception_1.ProcessException('validate:sql:internalError');
                });
            }
            if (result.status == 'pending') {
                result.status = 'replied';
                await result.save();
            }
        }
        if (complainActionDto.action == 'closed' && result.status != 'closed') {
            result.status = 'closed';
            await result.save();
        }
        return result;
    }
};
ComplainService = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject('ComplainRepository')),
    __param(1, common_1.Inject('ComplaintAttachmentRepository')),
    __param(2, common_1.Inject('ComplaintCategoryRepository')),
    __param(3, common_1.Inject('MediaRepository')),
    __param(4, common_1.Inject('ChannelRepository')),
    __param(5, common_1.Inject('UserRepository')),
    __param(6, common_1.Inject('ComplaintReplyContentRepository')),
    __param(7, common_1.Inject('SequelizeInstance')),
    __metadata("design:paramtypes", [Object, Object, Object, Object, Object, Object, Object, Object])
], ComplainService);
exports.ComplainService = ComplainService;
//# sourceMappingURL=complain.service.js.map