"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseService = void 0;
const sequelize_typescript_1 = require("sequelize-typescript");
const app_constant_1 = require("../../config/app.constant");
const Channel_entity_1 = require("../../modules/channel/models/Channel.entity");
const ChannelJCT_entity_1 = require("../../modules/channel/models/ChannelJCT.entity");
const ChannelSubscription_entity_1 = require("../../modules/channel/models/ChannelSubscription.entity");
const sequelize_1 = require("sequelize");
class BaseService {
    async validateRejectedChannels(channelId, otherChannelId) {
        return await ChannelJCT_entity_1.ChannelJCT.findAll({
            attributes: ['channelJCTId'],
            where: {
                [sequelize_1.Op.and]: [
                    {
                        connectionStatusTypeId: {
                            [sequelize_1.Op.in]: [app_constant_1.appconstant.DISMISS, app_constant_1.appconstant.DISMISS_OTHER_REASON]
                        },
                    },
                    {
                        [sequelize_1.Op.or]: [
                            {
                                channelId: channelId,
                                partnerChannelId: otherChannelId
                            },
                            {
                                channelId: otherChannelId,
                                partnerChannelId: channelId
                            }
                        ]
                    }
                ]
            }
        }).then((results) => {
            return !!(results.length);
        });
    }
    async setSlugName(repository, name, alterNumber) {
        var slugName = name.toLowerCase().replace(/ /g, "-");
        if (alterNumber) {
            slugName = slugName + '-' + alterNumber;
        }
        else {
            alterNumber = 0;
        }
        return await repository.findOne({
            where: {
                slug: slugName
            }
        }).then(async (result) => {
            if (result && result.slug) {
                let replace = +alterNumber + 1;
                return await this.setSlugName(repository, name, replace);
            }
            return slugName;
        });
    }
    async getIdBySlugName(repository, name) {
        return await repository.findOne({
            where: {
                slug: name
            }
        });
    }
    async getCurrentChannelStatus(channelId, features) {
        let whereOption = sequelize_typescript_1.Sequelize.literal(' "channelSubscription"."isActive" = \'1\' AND to_date( to_char("channelSubscription"."endAt",\'dd/mm/yyyy\'),\'dd/mm/yyyy\')  >= to_date(to_char(now(),\'dd/mm/yyyy\'),\'dd/mm/yyyy\') ');
        let channel = await Channel_entity_1.Channel.findOne({
            attributes: ['companyName', 'signUpStatusId', 'subscriptionId', 'planId', 'isActive'],
            where: {
                channelId: channelId
            },
            include: [{
                    model: ChannelSubscription_entity_1.ChannelSubscription,
                    required: false,
                    where: whereOption
                }]
        });
        let myfeatures = {};
        for (let key of Object.keys(features)) {
            if (key.startsWith(channel.subscriptionId + '-')) {
                myfeatures[key] = features[key];
            }
        }
        return {
            "companyName": channel.companyName,
            "channelStatusId": channel.signUpStatusId,
            "planId": channel.planId,
            "subscriptionId": channel.subscriptionId,
            "features": myfeatures,
            "channelIsActive": channel.isActive,
            "subscriptionExpired": !(channel.channelSubscription),
            "channelSubscriptionId": (channel.channelSubscription) ? channel.channelSubscription.channelSubscriptionId : null
        };
    }
    async unsubscribe_token(contactId) {
        const Cryptr = require('cryptr');
        const cryptr = new Cryptr('ch-event-auth');
        return cryptr.encrypt(contactId);
    }
}
exports.BaseService = BaseService;
//# sourceMappingURL=base.service.js.map