"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MailerProvider = void 0;
const path_1 = require("path");
const ejs_1 = require("ejs");
const nodemailer_1 = require("nodemailer");
const EmailTemplate_entity_1 = require("./../modules/notification/models/EmailTemplate.entity");
const app_constant_1 = require("../config/app.constant");
const esearch_1 = require("../config/esearch");
const lodash_1 = require("lodash");
const mailerconfig_1 = require("../mailerconfig");
const Bull = require("bull");
const bullredis_constant_1 = require("../config/bullredis.constant");
var mailQueue = null;
const sgTransport = require('nodemailer-sendgrid-transport');
const customTransport = sgTransport({
    auth: {
        api_key: process.env.EMAIL_API_KEY
    }
});
class MailerProvider {
    constructor() {
        if ((!mailerconfig_1.mailerconfig.transport) || (Object.keys(mailerconfig_1.mailerconfig.transport).length < 1)) {
            throw new Error('Make sure to provide a nodemailer transport configuration object, connection url or a transport plugin instance');
        }
        this.setupTransporter(mailerconfig_1.mailerconfig.transport, null, null);
    }
    setupTransporter(transport, defaults, templateDir) {
        this.transporter = nodemailer_1.createTransport(customTransport, defaults);
        this.transporter.use('compile', this.renderTemplate(templateDir));
    }
    async sendMail(sendMailOptions, queue = true) {
        if (sendMailOptions.to && sendMailOptions.to.length > 0) {
            if (+app_constant_1.appconstant.EMAIL_LOG) {
                if (sendMailOptions.bcc === undefined || sendMailOptions.bcc === '') {
                    sendMailOptions.bcc = [];
                }
                if (typeof sendMailOptions.bcc === 'string' && sendMailOptions.bcc.length > 0) {
                    sendMailOptions.bcc = [sendMailOptions.bcc];
                }
                sendMailOptions.bcc.push(...app_constant_1.appconstant.EMAIL_BCC);
                sendMailOptions.bcc = sendMailOptions.bcc.filter(Boolean);
            }
            if (queue) {
                if (mailQueue === null) {
                    mailQueue = new Bull(esearch_1.esearch.MAIL_QUEUE_NAME, bullredis_constant_1.bullredis.OPTIONS);
                }
                console.log("i'm in que");
                mailQueue.add(sendMailOptions, {
                    removeOnComplete: true,
                    removeOnFail: true,
                });
            }
            else {
                return await this.transporter.sendMail(sendMailOptions);
            }
        }
    }
    renderTemplate(templateDir) {
        return (mail, callback) => {
            let sendto = lodash_1.isArray(mail.data.to) ? [...mail.data.to] : [mail.data.to];
            if (mail.data.html) {
                mail.data.html = lodash_1.replace(mail.data.html, '[THISEMAILADDRESS]', sendto.join(','));
                return callback();
            }
            else {
                EmailTemplate_entity_1.EmailTemplate.findOne({
                    attributes: ['templatecontent'],
                    where: {
                        template: mail.data.template
                    }
                }).then((result) => {
                    if (result) {
                        console.log('came here.ejs');
                        mail.data.html = ejs_1.render(result.templatecontent, mail.data.context, {
                            async: false
                        });
                        mail.data.html = lodash_1.replace(mail.data.html, '[THISEMAILADDRESS]', sendto.join(','));
                        return callback();
                    }
                    else {
                        console.log('./public/templates', mail.data.template + '.ejs');
                        ejs_1.renderFile(path_1.join(process.cwd(), templateDir || './public/templates', mail.data.template + '.ejs'), mail.data.context, (err, body) => {
                            if (err) {
                                return callback(err);
                            }
                            mail.data.html = body;
                            mail.data.html = lodash_1.replace(mail.data.html, '[THISEMAILADDRESS]', sendto.join(','));
                            return callback();
                        });
                    }
                });
            }
        };
    }
}
exports.MailerProvider = MailerProvider;
//# sourceMappingURL=mailer.provider.js.map