'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidationException = void 0;
const moduleErrorMessage = require("./errorMessages");
class ValidationException extends Error {
    constructor(errors, lang) {
        super();
        const messages = this.formatErrors(errors);
        if (!messages)
            throw new Error('Unable to find message code error.');
        Error.captureStackTrace(this, this.constructor);
        this.messages = messages;
    }
    formatErrors(formErrors) {
        let errors = new Array();
        let message = [];
        let childError = [];
        for (let ii = 0; ii < formErrors.length; ii++) {
            if (formErrors[ii].children.length > 0) {
                childError = this.formatErrors(formErrors[ii].children);
            }
            if (formErrors[ii].constraints) {
                message = this.GroupConstraints(formErrors[ii].constraints);
            }
            if (childError.length) {
                errors.push({
                    property: formErrors[ii].property,
                    errors: childError
                });
            }
            else if (message.length) {
                errors.push({
                    property: formErrors[ii].property,
                    errors: message
                });
            }
        }
        return errors;
    }
    GroupConstraints(params) {
        let message = new Array();
        Object.keys(params).forEach((constraint) => {
            var errorMessageConfig = this.getMessageFromMessageCode(params[constraint]);
            if (!errorMessageConfig.code) {
                errorMessageConfig.code = 0;
                errorMessageConfig.message = params[constraint];
            }
            message.push(errorMessageConfig);
        });
        return message;
    }
    getMessageFromMessageCode(messageCode) {
        let errorMessageConfig = {};
        if (typeof messageCode == "string") {
            var message = messageCode.split(":");
            if (moduleErrorMessage[message[0]]) {
                Object.keys(moduleErrorMessage[message[0]]).forEach(key => {
                    if (key === messageCode) {
                        errorMessageConfig = moduleErrorMessage[message[0]][key];
                        return true;
                    }
                });
            }
            return errorMessageConfig;
        }
    }
}
exports.ValidationException = ValidationException;
//# sourceMappingURL=Validation.exception.js.map