"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AccountService = void 0;
const common_1 = require("@nestjs/common");
const Address_entity_1 = require("../../channel/models/Address.entity");
const Channel_entity_1 = require("../../channel/models/Channel.entity");
const ChannelDetail_entity_1 = require("../../channel/models/ChannelDetail.entity");
const ChannelProductGroup_entity_1 = require("../../channel/models/ChannelProductGroup.entity");
const Contact_entity_1 = require("../../user/models/Contact.entity");
const SignUpHistory_entity_1 = require("../../user/models/SignUpHistory.entity");
const User_entity_1 = require("../../user/models/User.entity");
const UserRole_entity_1 = require("../../user/models/UserRole.entity");
const utility_1 = require("../helper/utility");
const sequelize_1 = require("sequelize");
const sequelize_typescript_1 = require("sequelize-typescript");
const Process_exception_1 = require("../../../middleware/exception/Process.exception");
const ChannelProductCategorie_entity_1 = require("../../channel/models/ChannelProductCategorie.entity");
const OldChannel_entity_1 = require("../../channel/models/OldChannel.entity");
var _ = require('lodash');
const crypto_1 = require("crypto");
const OldAddress_entity_1 = require("../../channel/models/OldAddress.entity");
const emailer_service_1 = require("../../emailer/services/emailer.service");
const EventParticipants_entity_1 = require("../../channel/models/EventParticipants.entity");
const EventChannel_entity_1 = require("../../channel/models/EventChannel.entity");
const Cryptr = require('cryptr');
let AccountService = class AccountService {
    constructor(emailerService, userRepository, sequelizeInstance, contactRepository, userRoleRepository, channelRepository, signUpHistorylRepository, addressRepository, channelDetailRepository, channelProductGroupRepository, channelCatRepository, oldChannelRepository, eventParticipantRepository, eventChannelRepository) {
        this.emailerService = emailerService;
        this.userRepository = userRepository;
        this.sequelizeInstance = sequelizeInstance;
        this.contactRepository = contactRepository;
        this.userRoleRepository = userRoleRepository;
        this.channelRepository = channelRepository;
        this.signUpHistorylRepository = signUpHistorylRepository;
        this.addressRepository = addressRepository;
        this.channelDetailRepository = channelDetailRepository;
        this.channelProductGroupRepository = channelProductGroupRepository;
        this.channelCatRepository = channelCatRepository;
        this.oldChannelRepository = oldChannelRepository;
        this.eventParticipantRepository = eventParticipantRepository;
        this.eventChannelRepository = eventChannelRepository;
        this.utility = new utility_1.Utility();
    }
    async create(user, skipVerify = false) {
        let pin = await this.utility.generatePin();
        let userObject = await this.setSignUpObject(user, pin, 14);
        return await this.sequelizeInstance.transaction(async (transaction) => {
            return await this.userRepository.create(userObject, {
                include: [{
                        model: this.contactRepository,
                    }, {
                        model: this.channelRepository,
                        include: [{
                                model: this.signUpHistorylRepository,
                            }]
                    },
                    {
                        model: this.userRoleRepository
                    }
                ],
                transaction,
            }).then(async (userInsetedtObject) => {
                await this.contactRepository.update({ channelId: userInsetedtObject.channelId }, {
                    where: {
                        contactId: userInsetedtObject.contactId
                    },
                    returning: true,
                    transaction
                });
                let regAddressId = await this.addressRepository.create({
                    address: user.address,
                    city: user.city,
                    countryId: user.countryId,
                    postalCode: user.postalCode,
                    state: user.state
                }, { transaction });
                let comapnyRegAddressId = regAddressId.addressId;
                await this.channelRepository.update({ regAddressId: comapnyRegAddressId }, {
                    where: {
                        channelId: userInsetedtObject.channelId
                    },
                    returning: true,
                    transaction
                });
                await this.channelDetailRepository.create({
                    channelId: userInsetedtObject.channelId,
                    webSiteUrl: user.websiteurl,
                    phone1: user.switchPhone,
                    createdAt: new Date()
                }, { transaction });
                let ChannelGroupObject = await this.setChannelProductGroup(user.productFamilies, +userInsetedtObject.channelId);
                await this.channelProductGroupRepository.bulkCreate(ChannelGroupObject, { returning: true, transaction });
                let ChannelCatObject = await this.setChannelProductCat(user.productCategories, +userInsetedtObject.channelId);
                await this.channelCatRepository.bulkCreate(ChannelCatObject, { returning: true, transaction });
                if (user.eventId) {
                    let participants = {
                        eventId: user.eventId,
                        contactId: userInsetedtObject.contactId,
                        channelId: userInsetedtObject.channelId,
                        participantType: user.participantType,
                        channelType: user.channelTypeId,
                        createdAt: new Date()
                    };
                    let eventChannel = {
                        channelId: userInsetedtObject.channelId,
                        eventId: user.eventId,
                        isNew: '1',
                        createdAt: new Date()
                    };
                    await this.eventParticipantRepository.create(participants, { transaction });
                    await this.eventChannelRepository.create(eventChannel, { transaction });
                }
                return userInsetedtObject;
            });
        })
            .catch((err) => {
            console.log(err);
            throw new Process_exception_1.ProcessException('account:singup:tryAgain');
        });
    }
    async setSignUpObject(user, pin, signUpStatus) {
        try {
            let signUpHistory = {};
            signUpHistory.signUpStatusId = signUpStatus;
            signUpHistory.createdAt = new Date();
            let channel = {};
            channel.companyName = user.companyName;
            channel.channelTypeId = user.channelTypeId;
            channel.signUpStatusId = signUpStatus;
            channel.countryId = (user.countryId) ? user.countryId : null;
            channel.isActive = '1';
            channel.createdAt = new Date();
            channel.signUpHistory = signUpHistory;
            channel.hubspotCompanyId = user.hubspotCompanyId;
            channel.planId = 1;
            channel.oldChannelId = user.oldChannelId != undefined ? user.oldChannelId : null;
            channel.isOEM = user.isOEM;
            if (+signUpStatus == 4) {
                channel.isAdminCreated = '1';
            }
            let contact = {};
            contact.firstName = user.firstName;
            contact.lastName = user.lastName;
            contact.email = user.email;
            contact.jobTitle = user.jobPosition;
            contact.phone1 = user.switchPhone;
            contact.mobileNo = user.mobile;
            contact.hubspotContactId = (user.hubspot_contact_id) ? user.hubspot_contact_id : null;
            contact.countryId = user.countryId;
            contact.isActive = '1';
            contact.isDefaultContact = '1';
            contact.createdAt = new Date();
            contact.isShared = '1';
            let userRole = {};
            userRole.roleId = 1;
            let returnObject = {};
            returnObject.contact = contact;
            returnObject.channel = channel;
            returnObject.role = userRole;
            returnObject.email = user.email;
            returnObject.password = user.password;
            returnObject.verificationCode = pin;
            returnObject.verificationSentAt = new Date();
            returnObject.isActive = '1';
            returnObject.createdAt = new Date();
            return returnObject;
        }
        catch (e) {
            throw new Process_exception_1.ProcessException('account:objectCreation:User');
        }
    }
    async checkEmail(email) {
        let result = await this.userRepository.findOne({
            where: {
                email: {
                    [sequelize_1.Op.iLike]: '%' + email + '%'
                },
                deletedAt: null
            }
        });
        console.log(result);
        if (!result) {
            return true;
        }
        return false;
    }
    async checkCompanyName(name) {
        let result = await this.channelRepository.findOne({
            where: sequelize_typescript_1.Sequelize.where(sequelize_typescript_1.Sequelize.fn('lower', sequelize_typescript_1.Sequelize.col('companyName')), sequelize_typescript_1.Sequelize.fn('lower', name)),
        });
        console.log(result);
        if (!result) {
            return true;
        }
        return false;
    }
    async setProductGroup(productFamilies, channelId) {
        try {
            let resultObject = [];
            if (productFamilies) {
                for (let ii = 0; ii < productFamilies.length; ii++) {
                    let innerObject = {};
                    innerObject.channelId = channelId;
                    innerObject.productGroupId = productFamilies[ii];
                    innerObject.createdAt = new Date();
                    resultObject.push(innerObject);
                }
            }
            return resultObject;
        }
        catch (e) {
            throw new Process_exception_1.ProcessException('channel:objectCreation:productGroup');
        }
    }
    async setChannelProductGroup(productGroup, channelId) {
        try {
            productGroup = _.uniq(productGroup);
            let resultObject = [];
            if (productGroup) {
                for (let ii = 0; ii < productGroup.length; ii++) {
                    let innerObject = {};
                    innerObject.channelId = channelId;
                    innerObject.productGroupId = productGroup[ii];
                    innerObject.createdAt = new Date();
                    resultObject.push(innerObject);
                }
            }
            return resultObject;
        }
        catch (e) {
            console.log("err:", e);
            throw new Process_exception_1.ProcessException('channel:objectCreation:channelBrand');
        }
    }
    async setChannelProductCat(productCat, channelId) {
        try {
            let resultObject = [];
            if (productCat) {
                for (let ii = 0; ii < productCat.length; ii++) {
                    let innerObject = {};
                    innerObject.channelId = channelId;
                    innerObject.productCategorieId = productCat[ii];
                    innerObject.createdAt = new Date();
                    resultObject.push(innerObject);
                }
            }
            return resultObject;
        }
        catch (e) {
            throw new Process_exception_1.ProcessException('channel:objectCreation:channelBrand');
        }
    }
    async getCompanyData(name) {
        let result = await this.oldChannelRepository.findOne({
            where: sequelize_typescript_1.Sequelize.where(sequelize_typescript_1.Sequelize.fn('lower', sequelize_typescript_1.Sequelize.col('companyName')), sequelize_typescript_1.Sequelize.fn('lower', name)),
            include: [
                {
                    model: OldAddress_entity_1.OldAddress,
                    as: 'regAddress',
                    attributes: ['address', 'countryId', 'postalCode', 'city']
                }
            ]
        });
        let data = {};
        if (result.oldChannelId) {
        }
        console.log(result);
        return result;
    }
    async forgotPassword(email) {
        console.log('email:', email);
        let user = await this.userRepository.findOne({
            where: {
                email: {
                    [sequelize_1.Op.iLike]: email
                },
                deletedAt: null
            },
            include: [
                {
                    model: Contact_entity_1.Contact,
                    attributes: ['firstName', 'lastName']
                }
            ]
        });
        console.log('user:', user);
        if (!user) {
            console.log('not user');
            throw new Process_exception_1.ProcessException('account:forgotPassword:invalidUser');
        }
        let userObject = {
            resetPasswordToken: crypto_1.randomBytes(20).toString('hex'),
            resetPasswordExpires: this.sequelizeInstance.literal("CURRENT_TIMESTAMP + interval '" + +process.env.RESET_PASSWORD_TOKEN_EXDPIRE + "' day")
        };
        return await this.userRepository.update(userObject, {
            where: {
                userId: +user.userId,
                deletedAt: null
            }
        }).then((result) => {
            let data = {
                name: user.contact.firstName + " " + user.contact.lastName,
                email: email,
                message: 'An e-mail has been sent to ' + email + ' with further instructions.',
                url: process.env.DOMAIN + "resetpassword/" + userObject.resetPasswordToken
            };
            return data;
        }).catch((err) => {
            console.log('eeee', err);
            throw new Process_exception_1.ProcessException('account:forgotPassword:invalidUser');
        });
    }
    async resetPassword(token) {
        let user = await this.userRepository.findOne({
            where: {
                resetPasswordToken: token,
                resetPasswordExpires: {
                    [sequelize_1.Op.gt]: new Date().toUTCString()
                },
                deletedAt: null
            }
        });
        if (!user)
            throw new Process_exception_1.ProcessException('account:resetPassword:tokenExpired');
        return true;
    }
    async resetPasswordUpdate(token, password) {
        let user = await this.userRepository.findOne({
            where: {
                resetPasswordToken: token,
                resetPasswordExpires: {
                    [sequelize_1.Op.gt]: new Date().toUTCString()
                },
                deletedAt: null
            }
        });
        if (!user)
            throw new Process_exception_1.ProcessException('account:resetPassword:tokenExpired');
        user.password = password;
        user.resetPasswordToken = null;
        user.resetPasswordExpires = null;
        return await user.save().then(() => {
            return true;
        }).catch((err) => {
            throw new Process_exception_1.ProcessException('account:resetPassword:tokenExpired');
        });
    }
    async updatePasswordFrom_BO(updatePasswordDto) {
        let user = await User_entity_1.User.findOne({ where: {
                userId: +updatePasswordDto.userId,
                channelId: +updatePasswordDto.channelId,
                contactId: +updatePasswordDto.contactId
            } });
        user.password = updatePasswordDto.password;
        return await user.save().then(() => {
            return true;
        }).catch((err) => {
            return false;
        });
    }
    async encryptAuth(token) {
        const cryptr = new Cryptr('ch-event-auth');
        return cryptr.encrypt(token);
    }
    async decryptAuth(token) {
        const cryptr = new Cryptr('ch-event-auth');
        return cryptr.decrypt(token);
    }
    async isEvent(ContactId, isEvent) {
        return await this.contactRepository.update({ isEvent: isEvent }, {
            where: {
                contactId: ContactId,
            },
            returning: true
        });
    }
};
AccountService = __decorate([
    common_1.Injectable(),
    __param(1, common_1.Inject('UserRepository')),
    __param(2, common_1.Inject('SequelizeInstance')),
    __param(3, common_1.Inject('ContactRepository')),
    __param(4, common_1.Inject('UserRoleRepository')),
    __param(5, common_1.Inject('ChannelRepository')),
    __param(6, common_1.Inject('SignUpHistoryRepository')),
    __param(7, common_1.Inject('AddressRepository')),
    __param(8, common_1.Inject('ChannelDetailRepository')),
    __param(9, common_1.Inject('ChannelProductGroupRepository')),
    __param(10, common_1.Inject('ChannelProductCategoryRepository')),
    __param(11, common_1.Inject('OldChannelRepository')),
    __param(12, common_1.Inject('EventParticipantRepository')),
    __param(13, common_1.Inject('EventChannelRepository')),
    __metadata("design:paramtypes", [emailer_service_1.EmailerService, Object, Object, Object, Object, Object, Object, Object, Object, Object, Object, Object, Object, Object])
], AccountService);
exports.AccountService = AccountService;
//# sourceMappingURL=account.service.js.map