'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IndexController = void 0;
const Channel_entity_1 = require("./../../channel/models/Channel.entity");
const context_service_1 = require("./../../../context/context.service");
const common_1 = require("@nestjs/common");
const validation_pipe_1 = require("./../../../middleware/pipe/validation.pipe");
const cart_service_1 = require("./../service/cart.service");
const addCartProduct_dto_1 = require("./../dto/addCartProduct.dto");
const Auth_exception_1 = require("./../../../middleware/exception/Auth.exception");
const Process_exception_1 = require("./../../../middleware/exception/Process.exception");
const base_controller_1 = require("../../../core/controller/base.controller");
const permissions_constants_1 = require("../../../config/permissions.constants");
const passport_1 = require("@nestjs/passport");
let IndexController = class IndexController extends base_controller_1.BaseController {
    constructor(cart, context) {
        super();
        this.cart = cart;
        this.context = context;
    }
    async _validateCartModule(user) {
        if (user.channelTypeId == Channel_entity_1.Channel.CHANNEL_TYPE_VENDOR) {
            throw new Auth_exception_1.AuthException('Not allowed');
        }
    }
    async findAllCart(req, res) {
        let user = req.user;
        let data = await this.cart.getAllMyCart(user.userId);
        res.status(common_1.HttpStatus.OK).json(data);
    }
    async clearAllCart(req, res) {
        let user = req.user;
        await this.cart.clearCart(user.userId);
        res.status(common_1.HttpStatus.NO_CONTENT).json({
            "status": true
        });
    }
    async clearCart(req, res, id) {
        let user = req.user;
        await this.cart.clearCart(user.userId, id);
        res.status(common_1.HttpStatus.NO_CONTENT).json();
    }
    async addToCart(req, res, product) {
        let user = req.user;
        console.log(user);
        await this._validateCartModule(user);
        if (+product.quantity <= 0) {
            throw new Process_exception_1.ProcessException('quantity should be atleast one');
        }
        let action = (product.action) ? product.action : 'sum';
        await this.cart.addToCart(product.id, user, product.quantity, action);
        res.status(common_1.HttpStatus.CREATED).json({
            "status": true
        });
    }
    async removeProduct(req, res, id) {
        let user = req.user;
        await this.cart.removeItemFromCart(id, user);
        res.status(common_1.HttpStatus.NO_CONTENT).json();
    }
    async getItemCount(req, res) {
        let user = req.user;
        await this._validateCartModule(user);
        let result = await this.cart.returnCount(user.userId);
        res.status(common_1.HttpStatus.OK).json({
            "status": true,
            data: result
        });
    }
};
__decorate([
    common_1.Get('/'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], IndexController.prototype, "findAllCart", null);
__decorate([
    common_1.Delete('/'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], IndexController.prototype, "clearAllCart", null);
__decorate([
    common_1.Delete('/:id'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Param('id', new common_1.ParseIntPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Number]),
    __metadata("design:returntype", Promise)
], IndexController.prototype, "clearCart", null);
__decorate([
    common_1.Post('/item'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body(new validation_pipe_1.ValidationPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, addCartProduct_dto_1.AddToCartProductDto]),
    __metadata("design:returntype", Promise)
], IndexController.prototype, "addToCart", null);
__decorate([
    common_1.Delete('/item/:id'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Param('id', new common_1.ParseIntPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Number]),
    __metadata("design:returntype", Promise)
], IndexController.prototype, "removeProduct", null);
__decorate([
    common_1.Get('/item/count'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], IndexController.prototype, "getItemCount", null);
IndexController = __decorate([
    common_1.Controller('cart'),
    common_1.UseGuards(passport_1.AuthGuard('jwt')),
    __metadata("design:paramtypes", [cart_service_1.CartService,
        context_service_1.ContextService])
], IndexController);
exports.IndexController = IndexController;
//# sourceMappingURL=index.controller.js.map