'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CartService = void 0;
const common_1 = require("@nestjs/common");
const ProductQuote_entity_1 = require("./../models/ProductQuote.entity");
const Process_exception_1 = require("./../../../middleware/exception/Process.exception");
const Product_entity_1 = require("./../../product/models/Product.entity");
const ProductPriceDetail_entity_1 = require("./../../product/models/ProductPriceDetail.entity");
const Brand_entity_1 = require("./../../product/models/Brand.entity");
const Auth_exception_1 = require("./../../../middleware/exception/Auth.exception");
const index_1 = require("../../media/models/index");
const ProductImage_entity_1 = require("../../product/models/ProductImage.entity");
const base_service_1 = require("../../../core/service/base.service");
const app_constant_1 = require("../../../config/app.constant");
const Channel_entity_1 = require("../../channel/models/Channel.entity");
let CartService = class CartService extends base_service_1.BaseService {
    constructor(repository, quoteRepository, sequelizeInstance, productRepository) {
        super();
        this.repository = repository;
        this.quoteRepository = quoteRepository;
        this.sequelizeInstance = sequelizeInstance;
        this.productRepository = productRepository;
    }
    async clearCart(userId, cartId) {
        let where = {
            userid: userId,
            deletedat: null
        };
        if (cartId) {
            where = Object.assign(where, { 'cartid': cartId });
        }
        let result = await this.repository.findAll({
            where: where
        });
        if (!result.length) {
            throw new Process_exception_1.ProcessException('validate:field:notFound');
        }
        try {
            await this.repository.update({
                deletedat: this.sequelizeInstance.fn('NOW')
            }, { where: where });
        }
        catch (e) {
            throw new Process_exception_1.ProcessException('validate:sql:internalError');
        }
    }
    async _validateProduct(productId) {
        let product = await this.productRepository.findByPk(productId, {
            include: [
                {
                    model: ProductPriceDetail_entity_1.ProductPriceDetail,
                    as: 'productPriceDetail',
                    required: true
                }
            ],
        });
        if (!product) {
            throw new Process_exception_1.ProcessException('validate:field:notFound');
        }
        return product;
    }
    async _getCart(sellerChannelId, buyerChannelId, userId, currencyId) {
        let cart = await this.repository.findOrCreate({
            where: {
                sellerchannelid: sellerChannelId,
                buyerchannelid: buyerChannelId,
                userid: userId,
                deletedat: null
            },
            defaults: {
                currencyid: currencyId,
                subtotal: 0,
                grandtotal: 0,
                taxcost: 0,
                discount: 0,
                createdat: this.sequelizeInstance.fn('NOW')
            }
        }).spread((data, created) => {
            return data;
        });
        if (!cart) {
            throw new Process_exception_1.ProcessException('validate:sql:internalError');
        }
        return cart;
    }
    async getAllMyCart(userId) {
        return await this.repository.findAll({
            attributes: ['cartid', 'sellerchannelid', 'currencyid', 'subtotal', 'taxcost', 'discount', 'grandtotal'],
            where: {
                deletedat: null,
                userid: userId
            },
            order: [['createdat', 'DESC']],
            include: [
                {
                    model: ProductQuote_entity_1.ProductQuote,
                    as: 'items',
                    required: true,
                    attributes: ['productid', 'quantity', 'unitprice'],
                    where: {
                        deletedat: null
                    },
                    include: [{
                            model: Product_entity_1.Product,
                            as: 'product',
                            required: true,
                            include: [
                                {
                                    model: Brand_entity_1.Brand,
                                    attributes: ['brandName'],
                                },
                                {
                                    model: ProductImage_entity_1.ProductImage.scope('productImage'),
                                    as: "defaultImage",
                                    where: { deletedat: null },
                                    include: [
                                        {
                                            model: index_1.Media,
                                            where: { deletedAt: null },
                                            required: false
                                        }
                                    ],
                                    required: false
                                }
                            ]
                        }]
                },
                {
                    model: Channel_entity_1.Channel,
                    attributes: ['companyName']
                }
            ]
        }).then((result) => {
            return result;
        });
    }
    async addToCart(productId, user, qty, action) {
        let product = await this._validateProduct(productId);
        let cart = await this._getCart(+product.channelId, +user.channelId, user.userId, product.productPriceDetail.businesscurrencyid);
        if (product.isshowtofreesub == '1' && user.planId == app_constant_1.appconstant.FREE_PLANID) {
            throw new Auth_exception_1.AuthException('Not allowed');
        }
        if (await this.validateRejectedChannels(+user.channelId, +cart.sellerchannelid)) {
            throw new Auth_exception_1.AuthException('Not allowed');
        }
        var limit = product.productPriceDetail.samplethreshold;
        if (product.productPriceDetail.isunlimitsample == '1') {
            limit = app_constant_1.appconstant.MAX_ALLOWED_QTY_FOR_UNLIMITED;
        }
        if (limit < qty) {
            throw new Process_exception_1.ProcessException('This product reached the limit');
        }
        await this.quoteRepository.findOrCreate({
            where: {
                cartid: cart.cartid,
                productid: product.productId,
                deletedat: null
            },
            defaults: {
                quantity: qty,
                unitprice: (product.productPriceDetail.issamplefree == '1') ? 0 : product.productPriceDetail.sampleprice,
                createdat: this.sequelizeInstance.fn('NOW')
            }
        }).spread(async (data, created) => {
            if (!created) {
                let quantityLimit = +data.quantity + qty;
                if (action == 'replace') {
                    quantityLimit = +qty;
                }
                var limit = product.productPriceDetail.samplethreshold;
                if (product.productPriceDetail.isunlimitsample == '1') {
                    limit = app_constant_1.appconstant.MAX_ALLOWED_QTY_FOR_UNLIMITED;
                }
                if (limit < quantityLimit) {
                    throw new Process_exception_1.ProcessException('This product reached the limit');
                }
                data.quantity = +quantityLimit;
                data.updatedat = this.sequelizeInstance.fn('NOW');
                data.unitprice = (product.productPriceDetail.issamplefree == '1') ? 0 : +product.productPriceDetail.sampleprice;
                await data.save();
            }
            return data;
        });
        let subtotal = await this.sequelizeInstance.query('select sum(quantity*unitprice) as subtotal from "tblproductquote" where cartid = :cartId and deletedat is null', {
            raw: true,
            type: this.sequelizeInstance.QueryTypes.SELECT,
            replacements: {
                cartId: cart.cartid
            }
        }).then((result) => {
            return +result[0].subtotal;
        });
        cart.subtotal = subtotal;
        cart.updatedat = this.sequelizeInstance.fn('NOW');
        cart.grandtotal = (+cart.subtotal + +cart.taxcost) - (+cart.discount);
        await cart.save();
    }
    async removeItemFromCart(productId, user) {
        let product = await this._validateProduct(productId);
        let cart = await this._getCart(+product.channelId, +user.channelId, user.userId, product.productPriceDetail.businesscurrencyid);
        let quote = await this.quoteRepository.findOne({
            attributes: ['productquoteid'],
            where: {
                cartid: cart.cartid,
                productid: product.productId
            }
        });
        if (!quote) {
            throw new Process_exception_1.ProcessException('This product not in the cart');
        }
        await quote.destroy();
        let subtotal = await this.sequelizeInstance.query('select sum(quantity*unitprice) as subtotal from "tblproductquote" where cartid = :cartId and deletedat is null', {
            raw: true,
            type: this.sequelizeInstance.QueryTypes.SELECT,
            replacements: {
                cartId: cart.cartid
            }
        }).then((result) => {
            return +result[0].subtotal;
        });
        cart.subtotal = subtotal;
        cart.updatedat = this.sequelizeInstance.fn('NOW');
        cart.grandtotal = (+cart.subtotal + +cart.taxcost) - (+cart.discount);
        await cart.save();
        return;
    }
    async returnCount(userId, cartId) {
        let query = 'select q.cartid, count(q.productid) as product_count, sum(q.quantity) as product_quantity from tblproductquote q inner join tblcart c on c.cartid = q.cartid where q.deletedat IS NULL and c.deletedat IS NULL and c.userid = :userId ';
        if (cartId) {
            query += 'and q.cartid = :cartId ';
        }
        query += 'group by q.cartid';
        let counts = await this.quoteRepository.sequelize.query(query, {
            type: this.sequelizeInstance.QueryTypes.SELECT,
            raw: true,
            replacements: {
                userId: userId,
                cartId: cartId
            }
        }).then((data) => {
            return data;
        });
        console.log("cartcount:", counts);
        return counts;
    }
};
CartService = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject('CartRepository')),
    __param(1, common_1.Inject('ProductQuoteRepository')),
    __param(2, common_1.Inject('SequelizeInstance')),
    __param(3, common_1.Inject('ProductRepository')),
    __metadata("design:paramtypes", [Object, Object, Object, Object])
], CartService);
exports.CartService = CartService;
//# sourceMappingURL=cart.service.js.map