"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChannelAdminController = void 0;
const common_1 = require("@nestjs/common");
const sequelize_1 = require("sequelize");
const config_1 = require("../../../config");
const context_service_1 = require("../../../context/context.service");
const Process_exception_1 = require("../../../middleware/exception/Process.exception");
const channel_admin_service_1 = require("../service/channel.admin.service");
const channel_setting_service_1 = require("../service/channel.setting.service");
const user_service_1 = require("../../user/service/user.service");
const Validation_exception_1 = require("../../../middleware/exception/Validation.exception");
const adminCoupon_dto_1 = require("../dto/adminCoupon.dto");
const subscription_service_1 = require("../service/subscription.service");
const subscriptionFilter_dto_1 = require("../dto/subscriptionFilter.dto");
const tradeLocation_service_1 = require("../service/tradeLocation.service");
const updateUser_dto_1 = require("../../user/dto/updateUser.dto");
const updateSubscription_dto_1 = require("../dto/updateSubscription.dto");
const channel_service_1 = require("../service/channel.service");
const channelDetailOutBound_1 = require("../transformation/channelDetailOutBound");
const product_service_1 = require("../../product/service/product.service");
const fetch = require("node-fetch");
let ChannelAdminController = class ChannelAdminController {
    constructor(channelAdminService, channelSettingService, channelService, subscriptionService, userService, tradeLocationService, productService, sequelizeInstance, context) {
        this.channelAdminService = channelAdminService;
        this.channelSettingService = channelSettingService;
        this.channelService = channelService;
        this.subscriptionService = subscriptionService;
        this.userService = userService;
        this.tradeLocationService = tradeLocationService;
        this.productService = productService;
        this.sequelizeInstance = sequelizeInstance;
        this.context = context;
        this.channelDetailOutBound = new channelDetailOutBound_1.ChannelDetailOutBound();
    }
    async pendingApproval(data, req, res) {
        let channel = await this.channelAdminService.getPendingApprovelChannels(data);
        return res.status(common_1.HttpStatus.OK).send(channel);
    }
    async channelSubscriptionList(req, res, SubscriptionFilterDto) {
        let sort = (req.query.sort && ['asc', 'desc'].includes(req.query.sort.toLowerCase())) ? req.query.sort.toLowerCase() : 'desc';
        sort = 'desc';
        let pagination = {
            limit: req.query.limit,
            offset: req.query.limit * req.query.page,
        };
        let subscriptions = await this.subscriptionService.getAllList(pagination, req.query.search, SubscriptionFilterDto, sort);
        res.status(common_1.HttpStatus.OK).send(subscriptions);
    }
    async pendingChannelsList(userId, req, res) {
        let channel = await this.channelAdminService.pendingChannelsList(userId);
        if (channel) {
            return res.status(common_1.HttpStatus.OK).send(channel);
        }
        throw new Process_exception_1.ProcessException('channel:channelApproval:channelNotFound');
    }
    async oldChannelsList(userId, req, res) {
        let channel = await this.channelAdminService.oldChannelsList(userId);
        if (channel) {
            return res.status(common_1.HttpStatus.OK).send(channel);
        }
        throw new Process_exception_1.ProcessException('channel:channelApproval:channelNotFound');
    }
    async oldChannels(data, req, res) {
        let channel = await this.channelAdminService.oldChannels(data);
        return res.status(common_1.HttpStatus.OK).send(channel);
    }
    async channelApproval(data, req, res) {
        let channel = await this.channelAdminService.channelApproval(data);
        if (channel) {
            return res.status(common_1.HttpStatus.OK).send(true);
        }
        throw new Process_exception_1.ProcessException('channel:channelApproval:channelNotFound');
    }
    async getFilterCountry(data, req, res) {
        let channel = await this.channelAdminService.getFilterCountry(data);
        return res.status(common_1.HttpStatus.OK).send(channel);
    }
    async getProductCat(data, res) {
        let productCat = await this.channelAdminService.getProductCat(data);
        return res.status(common_1.HttpStatus.OK).send(productCat);
    }
    async getSubscriptionPlan(res) {
        let planList = await this.channelAdminService.getSubscriptionPlan();
        return res.status(common_1.HttpStatus.OK).send(planList);
    }
    async introVideoUpload(OldIntro, res) {
        let doc_id = await this.channelAdminService.saveUploadedVideo(OldIntro, OldIntro.user);
        if (doc_id) {
            res.status(common_1.HttpStatus.OK).send(doc_id);
        }
    }
    async medialibraryByContact(req, res) {
        console.log('user', req.query);
        let userMedia = await this.channelAdminService.medialibraryByContact(req.query);
        if (userMedia) {
            res.status(common_1.HttpStatus.OK).send(userMedia);
        }
    }
    async deleteMediaVideo(res, data) {
        let doc_id = await this.channelAdminService.deleteMediaVideo(data);
        res.status(common_1.HttpStatus.OK).send(doc_id);
    }
    async deleteCompanyLogo(res, data) {
        let doc_id = await this.channelAdminService.deleteCompanyLogo(data);
        res.status(common_1.HttpStatus.OK).send(doc_id);
    }
    async getInitialProfile(req, res, data) {
        let channelId = data.channelId;
        let userId = data.userId;
        let user = await this.userService.findUserById(userId, channelId);
        let result = await this.channelSettingService.getInitialProfile(user);
        if (result) {
            res.status(common_1.HttpStatus.OK).json(result);
        }
    }
    async tradeinfo(req, res, data) {
        var _a, _b, _c, _d;
        let channelId = data.channelId;
        let userId = data.userId;
        let user = await this.userService.findUserById(userId, channelId);
        let result = await this.channelSettingService.getTradeInfo(user);
        let brandList = [];
        let productList = [];
        let profileList = [];
        if (result.length > 0) {
            result.map(async (x) => {
                var _a, _b, _c;
                (_a = x.channelBrand) === null || _a === void 0 ? void 0 : _a.map(x => {
                    let brand = {
                        id: x.brandId,
                        name: x.brand.brandName,
                        documentpath: process.env.CDN_URL + x.brand.document.documentPath,
                        isNew: 0
                    };
                    brandList.push(brand);
                });
                let cat = (_b = x.channelProductCategorie) === null || _b === void 0 ? void 0 : _b.map(x => x.productCategorie);
                let group = (_c = x.channelProductGroup) === null || _c === void 0 ? void 0 : _c.map(x => x.productGroup);
                group === null || group === void 0 ? void 0 : group.map(g => {
                    cat === null || cat === void 0 ? void 0 : cat.map(c => {
                        if (c.dataValues.groupid == g.dataValues.groupid) {
                            let product = {
                                childid: c.dataValues.childid,
                                childname: c.dataValues.childname,
                                groupid: c.dataValues.groupid,
                                groupname: g.dataValues.groupname,
                                status: true
                            };
                            productList.push(product);
                        }
                    });
                });
            });
            let pro = (_a = result[0].dataValues.channelCustomerProfile) === null || _a === void 0 ? void 0 : _a.map(x => +x.profileTypeId);
            let clssifiid = (_b = result[0].dataValues.channelCustomerProfile) === null || _b === void 0 ? void 0 : _b.map(x => +x.profileTypeClassificationId);
            let query = 'select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
                + 'when ptc."channelTypeId"=2 then \'vendor\' end as groupname,pt."profileType" as childname,'
                + 'pt."profileTypeId" as childId,cast(false as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" and pt."profileTypeId" in(:ptype) ';
            if (clssifiid.length)
                query += 'and  ptc."profileTypeClassificationId" in(:clssifiid) ';
            profileList = await this.sequelizeInstance.query(query, {
                type: sequelize_1.QueryTypes.SELECT,
                replacements: {
                    typeId: user.channelTypeId,
                    clssifiid: clssifiid,
                    ptype: (user.channelTypeId == 4 || user.channelTypeId == 5) ? result[0].profileTypeId : pro,
                }
            }).then(results => {
                return results;
            });
            let keyRetailerOptions = [];
            let keyRetailerValues = [];
            let keyDistributorOptions = [];
            let keyDistributorValues = [];
            if (result.keyPartners.retailer) {
                if ((_c = result.keyPartners.retailer.channel) === null || _c === void 0 ? void 0 : _c.length) {
                    result.keyPartners.retailer.channel.map(x => {
                        var _a;
                        let brand = {
                            id: x.channelId,
                            name: x.companyName,
                            documentpath: process.env.CDN_URL + ((_a = x.companylogo) === null || _a === void 0 ? void 0 : _a.documentPath),
                            isNew: 0
                        };
                        keyRetailerOptions.push(Object.assign(x, brand));
                    });
                }
            }
            let keyretailer = await this.channelSettingService.getKeyRetailers(+user.channelId, config_1.appconstant.CHANNEL_TYPE_RETAILER);
            if (keyretailer === null || keyretailer === void 0 ? void 0 : keyretailer.length) {
                keyretailer.map(x => {
                    var _a, _b, _c, _d;
                    let logo = x.active ? (_b = (_a = x.active) === null || _a === void 0 ? void 0 : _a.logo) === null || _b === void 0 ? void 0 : _b.documentPath : x.old ? (_c = x.old.logo) === null || _c === void 0 ? void 0 : _c.documentPath : x.new ? (_d = x.new.logo) === null || _d === void 0 ? void 0 : _d.logo_name : '';
                    let companyName = x.active ? x.active.company_name : x.old ? x.old.company_name : x.new ? x.new.company_name : '';
                    let id = x.active ? x.active.id : x.old ? x.old.id : x.new ? x.new.id : '';
                    let brand = {
                        id: id,
                        name: companyName,
                        documentpath: process.env.CDN_URL + logo,
                        isNew: 0
                    };
                    keyRetailerValues.push(Object.assign(x, brand));
                });
            }
            if (result.keyPartners.distributor) {
                if ((_d = result.keyPartners.distributor.channel) === null || _d === void 0 ? void 0 : _d.length) {
                    result.keyPartners.distributor.channel.map(x => {
                        var _a;
                        let brand = {
                            id: x.channelId,
                            name: x.companyName,
                            documentpath: process.env.CDN_URL + ((_a = x.companylogo) === null || _a === void 0 ? void 0 : _a.documentPath),
                            isNew: 0
                        };
                        keyDistributorOptions.push(Object.assign(x, brand));
                    });
                }
            }
            let keyDistributor = await this.channelSettingService.getKeyRetailers(+user.channelId, config_1.appconstant.CHANNEL_TYPE_DISTRIBUTOR);
            if (keyDistributor === null || keyDistributor === void 0 ? void 0 : keyDistributor.length) {
                keyDistributor.map(x => {
                    var _a, _b, _c, _d;
                    let logo = x.active ? (_b = (_a = x.active) === null || _a === void 0 ? void 0 : _a.logo) === null || _b === void 0 ? void 0 : _b.documentPath : x.old ? (_c = x.old.logo) === null || _c === void 0 ? void 0 : _c.documentPath : x.new ? (_d = x.new.logo) === null || _d === void 0 ? void 0 : _d.logo_name : '';
                    let companyName = x.active ? x.active.company_name : x.old ? x.old.company_name : x.new ? x.new.company_name : '';
                    let id = x.active ? x.active.id : x.old ? x.old.id : x.new ? x.new.id : '';
                    let brand = {
                        id: id,
                        name: companyName,
                        documentpath: process.env.CDN_URL + logo,
                        isNew: 0
                    };
                    keyDistributorValues.push(Object.assign(x, brand));
                });
            }
            let profileType = '';
            if (result[0].dataValues.profileTypeId) {
                query = 'select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
                    + 'when ptc."channelTypeId"=2 then \'vendor\' end as groupname,pt."profileType" as childname,'
                    + 'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" where ptc."channelTypeId" in(:typeId) and ptc."profileTypeId"=:profileTypeId;';
                profileType = await this.sequelizeInstance.query(query, {
                    type: sequelize_1.QueryTypes.SELECT,
                    replacements: {
                        typeId: user.channelTypeId,
                        profileTypeId: result[0].dataValues.profileTypeId
                    }
                }).then(results => {
                    return results;
                });
            }
            let data = {
                brand: brandList,
                product: productList,
                profile: profileList,
                empCnt: result[0].dataValues.channelDetail.empCntRange ? result[0].dataValues.channelDetail.empCntRange : result[0].dataValues.channelDetail.empCnt,
                no_of_skus: result[0].dataValues.channelDetail.no_of_skus,
                turnoverRange: result[0].dataValues.channelDetail.turnoverRange,
                revD: result[0].dataValues.channelDetail.revD,
                revI: result[0].dataValues.channelDetail.revI,
                startup: result[0].dataValues.channelDetail.startup,
                no_of_distributors: result[0].dataValues.channelDetail.no_of_distributors,
                no_of_retailers: result[0].dataValues.channelDetail.no_of_retailers,
                maturitylevel: result[0].dataValues.channelDetail.maturitylevel,
                profileTypeId: result[0].dataValues.profileTypeId,
                profileType: profileType,
                onlineStore: result[0].dataValues.channelDetail.turnoverPhyStore,
                turnoverOnStore: result[0].dataValues.channelDetail.turnoverOnStore,
                storeCount: result[0].dataValues.channelDetail.storeCount,
                sales: result[0].dataValues.channelDetail.empCntDSale,
                marketing: result[0].dataValues.channelDetail.empCntISale,
                technical: result[0].dataValues.channelDetail.empCntDMar,
                purchase: result[0].dataValues.channelDetail.empCntIMar,
                keyPartners: { keyRetailerOptions, keyDistributorOptions },
                channelKeyRetailer: keyRetailerValues,
                channelKeyDistributor: keyDistributorValues,
                keyretailers: result[0].dataValues.channelDetail.keyretailers,
                keydistributors: result[0].dataValues.channelDetail.keydistributors,
                serviceListOptions: result.serviceListOptions,
                serviceListValues: result[0].dataValues.serviceList ? result[0].dataValues.serviceList.map(x => ({ id: x.serviceListId, name: x.serviceList })) : [],
                specialityOptions: result.specialityOptions,
                channelSpeciality: result[0].dataValues.channelSpeciality,
                isFrenchTech: result[0].dataValues.isFrenchTech,
                isCompanyPublic: result[0].dataValues.isCompanyPublic
            };
            res.status(common_1.HttpStatus.OK).send(data);
        }
        else {
            res.status(common_1.HttpStatus.OK).send(data);
        }
    }
    async tradelocation(req, res, data) {
        let userId = data.userId;
        let channelId = data.channelId;
        let user = await this.userService.findUserById(userId, channelId);
        let result = await this.channelSettingService.getTradeLocation(user);
        let targetArray = [];
        let sellingArray = [];
        let targetids = [];
        let sellingids = [];
        if (result)
            result[0].channelLocation.map(x => {
                if (x.locationType == 'TARGET' && user.channelTypeId != 4) {
                    targetids.push(x.RegionCountryJCT.region.regionId);
                    let location = {
                        regionCountryJCTId: x.RegionCountryJCT.regionCountryJCTId,
                        groupid: x.RegionCountryJCT.region.regionId,
                        groupname: x.RegionCountryJCT.region.region,
                        childId: x.RegionCountryJCT.country.countryId,
                        childname: x.RegionCountryJCT.country.country,
                        status: true
                    };
                    targetArray.push(location);
                }
                if (x.locationType == 'SELLING' && user.channelTypeId != 4) {
                    sellingids.push(x.RegionCountryJCT.region.regionId);
                    let location = {
                        regionCountryJCTId: x.RegionCountryJCT.regionCountryJCTId,
                        groupid: x.RegionCountryJCT.region.regionId,
                        groupname: x.RegionCountryJCT.region.region,
                        childId: x.RegionCountryJCT.country.countryId,
                        childname: x.RegionCountryJCT.country.country,
                        status: true
                    };
                    sellingArray.push(location);
                }
                if (x.locationType == 'RETAIL' && user.channelTypeId == 4) {
                    sellingids.push(x.RegionCountryJCT.region.regionId);
                    let location = {
                        regionCountryJCTId: x.RegionCountryJCT.regionCountryJCTId,
                        groupid: x.RegionCountryJCT.region.regionId,
                        groupname: x.RegionCountryJCT.region.region,
                        childId: x.RegionCountryJCT.country.countryId,
                        childname: x.RegionCountryJCT.country.country,
                        status: true
                    };
                    sellingArray.push(location);
                }
            });
        let zone = await this.sequelizeInstance.query('select "zoneId","regionId" from "tblZoneRegionJCT" ', {
            type: sequelize_1.QueryTypes.SELECT,
            raw: true,
        }).then(x => {
            let zones = {
                tzone: x.filter(ch => { return targetids.includes(ch.regionId); }),
                szone: x.filter(ch => { return sellingids.includes(ch.regionId); })
            };
            return zones;
        });
        let finalData = {
            zone: zone,
            targetLocation: targetArray,
            sellingLocation: sellingArray
        };
        res.status(common_1.HttpStatus.OK).send(finalData);
    }
    async getUsers(req, res, data) {
        let userId = data.userId;
        let channelId = data.channelId;
        let user = await this.userService.findUserById(userId, channelId);
        let result = await this.channelSettingService.getUsers(user);
        res.status(common_1.HttpStatus.OK).send(result);
    }
    async getDefaultContact(res, req, data) {
        let user = data;
        let result = await this.userService.getDefaultContact(user.channelId);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async getAllContacts(res, req, data) {
        let user = data;
        let result = await this.userService.getAllContacts(user.channelId);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async settings(res, req, data) {
        let userId = data.userId;
        let channelId = data.channelId;
        let user = await this.userService.findUserById(userId, channelId);
        let settings = await this.channelSettingService.settings(user);
        if (settings) {
            return res.status(common_1.HttpStatus.OK).send(settings);
        }
    }
    async getProfileAndSpeciality(req, res, data) {
        let channelId = data.channelId;
        let result = await this.channelSettingService.getProfileAndSpeciality(channelId);
        let final = {};
        if (result) {
            let profileType = '';
            if (result[0].dataValues.profileTypeId) {
                let query = 'select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
                    + 'when ptc."channelTypeId"=2 then \'vendor\' end as groupname,pt."profileType" as childname,'
                    + 'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" where ptc."channelTypeId" in(:typeId) and ptc."profileTypeId"=:profileTypeId;';
                profileType = await this.sequelizeInstance.query(query, {
                    type: sequelize_1.QueryTypes.SELECT,
                    replacements: {
                        typeId: data.channelTypeId,
                        profileTypeId: result[0].dataValues.profileTypeId
                    }
                }).then(results => {
                    return results;
                });
            }
            final = {
                specialityOptions: result.specialityOptions,
                channelSpeciality: result[0].dataValues.channelSpeciality,
                profileTypeId: result[0].dataValues.profileTypeId,
                profileType: profileType,
                isFrenchTech: result[0].dataValues.isFrenchTech
            };
        }
        res.status(common_1.HttpStatus.OK).json(final);
    }
    async setDefaultContact(contactId, res, req, data) {
        let userId = data.userId;
        let channelId = data.channelId;
        let user = await this.userService.findUserById(userId, channelId);
        let result = await this.userService.setDefaultContact(user.channelId, contactId);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async update_Company_Profile(profileData, req, res) {
        let userId = profileData.user.userId;
        let channelId = profileData.user.channelId;
        let user = await this.userService.findUserById(userId, channelId);
        let result = await this.channelSettingService.updateUserCompanyProfile(profileData, user);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async update_company_profile(req, res, data) {
        let userId = data.user.userId;
        let channelId = data.user.channelId;
        let user = await this.userService.findUserById(userId, channelId);
        let result = await this.channelSettingService.update_company_new(user, data);
        console.log('data', data);
        if (result)
            return res.status(common_1.HttpStatus.OK).send('OK');
        else
            return res.status(common_1.HttpStatus.BAD_REQUEST);
    }
    async update_Trade_Info(profileData, req, res) {
        let userId = profileData.user.userId;
        let channelId = profileData.user.channelId;
        let user = await this.userService.findUserById(userId, channelId);
        let result = await this.channelSettingService.updatetradeInformation(profileData, user);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async updateChannelLocation1(channelLocationDto, req, res) {
        let user = channelLocationDto.user;
        let subscriptionFeatureKeys = this.context.get('subscriptionFeatures');
        let query = 'select "contactId" from "tblContact" where "channelId"=:channelId and "isDefaultContact"=\'1\'';
        let contact = await this.sequelizeInstance.query(query, {
            type: sequelize_1.QueryTypes.SELECT,
            replacements: {
                channelId: user.channelId,
            }
        }).then(res => {
            return res[0].contactId;
        });
        let nonsuggestion = [];
        let count;
        let userId = user.userId;
        let channelId = user.channelId;
        let contactId = user.contactId;
        let channelTypeId = user.channel.channelTypeId;
        let channelLocation = await this.channelSettingService.updateChannelLocation(channelLocationDto, Object.assign(user, { contactId: contact }), subscriptionFeatureKeys);
        return res.status(common_1.HttpStatus.OK).send(channelLocation);
    }
    async updateSettings(req, res, settingsDto) {
        let userId = settingsDto.user.userId;
        let channelId = settingsDto.user.channelId;
        let user = await this.userService.findUserById(userId, channelId);
        let query = 'select "contactId" from "tblContact" where "channelId"=:channelId and "isDefaultContact"=\'1\'';
        let contact = await this.sequelizeInstance.query(query, {
            type: sequelize_1.QueryTypes.SELECT,
            replacements: {
                channelId: user.channelId,
            }
        }).then(res => {
            return res[0].contactId;
        });
        let updateSettings = await this.channelSettingService.updateSettings(settingsDto, Object.assign(user, { contactId: contact }));
        if (updateSettings) {
            return res.status(common_1.HttpStatus.OK).send('ok');
        }
    }
    async update_channel_profile(req, res, data) {
        let userId = data.user.userId;
        let channelId = data.user.channelId;
        let user = await this.userService.findUserById(userId, channelId);
        let result = await this.channelSettingService.save_channel_info(data, {}, user);
        if (result)
            return res.status(common_1.HttpStatus.OK).send('OK');
        else
            return res.status(common_1.HttpStatus.BAD_REQUEST);
    }
    async addNewUser(res, req, addUserDto) {
        let nonsuggestion = [];
        let count;
        let userId = addUserDto.user.userId;
        let channelId = addUserDto.user.channelId;
        let user = await this.userService.findUserById(userId, channelId);
        try {
            let result = await this.userService.addUser(addUserDto, user.channelId, user.showId, user.userId, user.channelTypeId);
            let userId = result.userId;
            let channelId = user.channelId;
            let contactId = result.contactId;
            let channelTypeId = user.channelTypeId;
            res.status(common_1.HttpStatus.CREATED).send({
                created: result
            });
        }
        catch (e) {
            console.log("adduser:", e);
        }
    }
    async getUserProfile(req, res, data) {
        let result = await this.channelSettingService.getUserProfile(data.channelId, data.channelTypeId);
        if (result) {
            res.status(common_1.HttpStatus.OK).json(result);
        }
    }
    async getContactLocation(contactId, channelTypeId, req, res) {
        let user = req.user;
        let result;
        result = await this.channelSettingService.getContactLocation(user);
        if (result) {
            let targetArray = result.contactLocation.filter(x => x.locationType == 'TARGET');
            let sellingArray = user.channelTypeId == 4 ? result.contactLocation.filter(x => x.locationType == 'RETAIL') : result.contactLocation.filter(x => x.locationType == 'SELLING');
            let data = {
                targetArray: targetArray,
                sellingArray: sellingArray
            };
            res.status(common_1.HttpStatus.OK).json(data);
        }
    }
    async getCouponList(req, res, couponFilterDto) {
        let result = await this.channelAdminService.getCouponList(couponFilterDto);
        res.status(common_1.HttpStatus.OK).json(result);
    }
    async getCouponByID(id, req, res) {
        let result = await this.channelAdminService.getCouponByID(id);
        if (result)
            return res.status(common_1.HttpStatus.OK).json(result);
        else
            throw new Process_exception_1.ProcessException('channel:channelCoupons:channelCouponsNotFound');
    }
    async AddChannelCoupon(req, res, adminCouponAddDto) {
        let result = await this.channelAdminService.addChannelCoupon(adminCouponAddDto);
        if (result) {
            return res.status(common_1.HttpStatus.CREATED).send();
        }
        else {
            throw new Process_exception_1.ProcessException('channel:channelCoupons:channelCouponsNotFound');
        }
    }
    async UpdateChannelCoupon(id, req, res, adminCouponAddDto) {
        let result = await this.channelAdminService.UpdateChannelCoupon(id, adminCouponAddDto);
        if (result) {
            return res.status(common_1.HttpStatus.OK).send();
        }
        else {
            throw new Process_exception_1.ProcessException('channel:channelCoupons:channelCouponsNotFound');
        }
    }
    async existingKeyRetailerCheck(req, res, KeyRetailer) {
        let result = await this.channelSettingService.existingKeyRetailerCheck(decodeURI(KeyRetailer), +req.query.CountryId, +req.query.channelId, +req.query.partner_type, req.query.partner_id ? +req.query.partner_id : null);
        res.status(common_1.HttpStatus.OK).send(result);
    }
    async searchRegion(searchRegionDto, req, res) {
        let user = searchRegionDto.user;
        let searchRegion = await this.channelSettingService.searchRegion(searchRegionDto, user);
        let finaldata = [];
        if (searchRegion) {
            searchRegion.map(x => {
                let location = {
                    regionCountryJCTId: x.regionCountryJCTId,
                    groupid: x.region.regionId,
                    groupname: x.region.region,
                    childId: x.country.countryId,
                    childname: x.country.country,
                    status: false
                };
                finaldata.push(location);
            });
        }
        return res.status(common_1.HttpStatus.OK).send(finaldata);
    }
    async getBuyerNeed(req, res, reqneed) {
        let result = await this.channelAdminService.getBuyerNeed(req.query);
        res.status(common_1.HttpStatus.OK).send(result);
    }
    async getAllRequestIntro(channelid, req, res) {
        let user = this.context.get('user');
        let result = await this.channelAdminService.getRequestIntro(+channelid);
        res.status(common_1.HttpStatus.OK).send({
            result: result[0][0].requestintro
        });
    }
    async approveChannel(req, res, updateChannelDto) {
        let user = await this.userService.getUser(updateChannelDto.retailerUserId, updateChannelDto.retailerChannelid);
        let updateChannel = await this.channelAdminService.updateChannelConnect(updateChannelDto.sellerChannelid, updateChannelDto, user, true);
        if (updateChannel) {
            return res.status(common_1.HttpStatus.OK).send({ status: true });
        }
        return res.status(common_1.HttpStatus.OK).send({ status: true });
    }
    async getPublicCompanyFlages(req, res, channelId) {
        let flags = await this.channelAdminService.getPublicCompanyFlags(channelId);
        return res.status(common_1.HttpStatus.OK).send(flags);
    }
    async setCompanyPublicFlags(Req, res, data) {
        let result = await this.channelAdminService.setCompanyPublicFlags(data);
        return res.status(common_1.HttpStatus.OK).send({ status: true });
    }
    async getChannelDetail(req, res, data) {
        let channelId = data.channelId;
        let userId = data.userId;
        let user = await this.userService.findUserById(userId, channelId);
        let result = await this.channelSettingService.getChannelDetail(user.channelId);
        if (result) {
            res.status(common_1.HttpStatus.OK).json(result);
        }
    }
    async company(req, res, data) {
        let channelId = data.channelId;
        let userId = data.userId;
        let user = await this.userService.findUserById(userId, channelId);
        console.log('user', user);
        let result = await this.channelSettingService.getCompanyProfile(user);
        let channelPdf = await this.channelSettingService.getCompanyDocument(user.channelId);
        let chImages = await this.channelSettingService.getCompanyImages(user.channelId);
        let channelImages = [];
        if (chImages.length > 0) {
            chImages.map(x => {
                channelImages.push({ documentId: x.documentId, companyMediaId: x.companyMediaId, documentUrl: x.document.documentUrl });
            });
        }
        console.log('channelPdf', channelPdf);
        let channelPdfList = [];
        channelPdf.map(x => {
            let dpath = x.document.documentPath;
            channelPdfList.push({ documentId: x.documentId, name: x.name ? x.name : dpath.split('/').pop().split('#')[0].split('?')[0], channelDocumentId: x.channelDocumentId, document: { documentUrl: x.document.documentUrl } });
        });
        res.status(common_1.HttpStatus.OK).send(Object.assign(result, { channelPdf: channelPdfList }, { channelImages: channelImages }));
    }
    async getChannelCertificateTags(req, res, data) {
        let channelId = data.channelId;
        let userId = data.userId;
        let user = await this.userService.findUserById(userId, channelId);
        let result = await this.channelSettingService.getChannelCertificateTags(user);
        res.status(common_1.HttpStatus.OK).json(result);
    }
    async getChannelProduct(req, res, data) {
        let channelId = data.channelId;
        let userId = data.userId;
        let user = await this.userService.findUserById(userId, channelId);
        console.log("btype:", user.businessType);
        let result = await this.channelSettingService.getChannelProduct(user.channelId, user.businessType);
        if (result) {
            res.status(common_1.HttpStatus.OK).json(result);
        }
    }
    async getChannelBrand(req, res, data) {
        let channelId = data.channelId;
        let userId = data.userId;
        let user = await this.userService.findUserById(userId, channelId);
        let result = await this.channelSettingService.getChannelBrand(user.channelId);
        if (result) {
            res.status(common_1.HttpStatus.OK).json(result);
        }
    }
    async getKeyPartnerOptions(req, res, data) {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        let channelId = data.channelId;
        let userId = data.userId;
        let user = await this.userService.findUserById(userId, channelId);
        let result = await this.channelSettingService.getKeyPartnerOptions(user);
        let keyResellerOptions = [];
        let keyResellerValues = [];
        let keyRetailerOptions = [];
        let keyRetailerValues = [];
        let keyDistributorOptions = [];
        let keyDistributorValues = [];
        if (result.keyPartners.retailer) {
            if ((_a = result.keyPartners.retailer.channel) === null || _a === void 0 ? void 0 : _a.length) {
                result.keyPartners.retailer.channel.map(x => {
                    var _a, _b;
                    let brand = {
                        id: x.channelId,
                        name: x.companyName,
                        documentpath: ((_a = x.companylogo) === null || _a === void 0 ? void 0 : _a.documentPath) ? process.env.CDN_URL + ((_b = x.companylogo) === null || _b === void 0 ? void 0 : _b.documentPath) : null,
                        isNew: 0
                    };
                    keyRetailerOptions.push(Object.assign(x, brand));
                });
            }
            if ((_b = result.keyPartners.retailer.keypartners) === null || _b === void 0 ? void 0 : _b.length) {
                result.keyPartners.retailer.keypartners.map(x => {
                    var _a, _b;
                    let brand = {
                        id: x.channelkeypartnerid,
                        name: x.partnername,
                        documentpath: ((_a = x.companylogo) === null || _a === void 0 ? void 0 : _a.documentPath) ? process.env.CDN_URL + ((_b = x.companylogo) === null || _b === void 0 ? void 0 : _b.documentPath) : null,
                        isNew: 0
                    };
                    keyRetailerOptions.push(Object.assign(x, brand));
                });
            }
        }
        let keyretailer = await this.channelSettingService.getKeyRetailers(+user.channelId, config_1.appconstant.CHANNEL_TYPE_RETAILER);
        if (keyretailer === null || keyretailer === void 0 ? void 0 : keyretailer.length) {
            keyretailer.map(x => {
                var _a, _b, _c, _d;
                let logo = x.active ? (_b = (_a = x.active) === null || _a === void 0 ? void 0 : _a.logo) === null || _b === void 0 ? void 0 : _b.documentPath : x.old ? (_c = x.old.logo) === null || _c === void 0 ? void 0 : _c.documentPath : x.new ? (_d = x.new.logo) === null || _d === void 0 ? void 0 : _d.logo_name : '';
                let companyName = x.active ? x.active.company_name : x.old ? x.old.company_name : x.new ? x.new.company_name : '';
                let id = x.active ? x.active.id : x.old ? x.old.id : x.new ? x.new.id : '';
                let brand = {
                    id: id,
                    name: companyName,
                    documentpath: logo ? process.env.CDN_URL + logo : null,
                    isNew: 0
                };
                keyRetailerValues.push(Object.assign(x, brand));
            });
        }
        if (result.keyPartners.reseller) {
            if ((_c = result.keyPartners.reseller.channel) === null || _c === void 0 ? void 0 : _c.length) {
                (_d = result.keyPartners.reseller.channel) === null || _d === void 0 ? void 0 : _d.map(x => {
                    var _a, _b;
                    let brand = {
                        id: x.channelId,
                        name: x.companyName,
                        documentpath: ((_a = x.companylogo) === null || _a === void 0 ? void 0 : _a.documentPath) ? process.env.CDN_URL + ((_b = x.companylogo) === null || _b === void 0 ? void 0 : _b.documentPath) : null,
                        isNew: 0
                    };
                    keyResellerOptions.push(Object.assign(x, brand));
                });
            }
            if ((_e = result.keyPartners.reseller.keypartners) === null || _e === void 0 ? void 0 : _e.length) {
                (_f = result.keyPartners.reseller.keypartners) === null || _f === void 0 ? void 0 : _f.map(x => {
                    var _a, _b;
                    let brand = {
                        id: x.channelkeypartnerid,
                        name: x.partnername,
                        documentpath: ((_a = x.companylogo) === null || _a === void 0 ? void 0 : _a.documentPath) ? process.env.CDN_URL + ((_b = x.companylogo) === null || _b === void 0 ? void 0 : _b.documentPath) : null,
                        isNew: 0
                    };
                    keyResellerOptions.push(Object.assign(x, brand));
                });
            }
        }
        let keyreseller = await this.channelSettingService.getKeyRetailers(+user.channelId, config_1.appconstant.CHANNEL_TYPE_RESELLER);
        if (keyreseller === null || keyreseller === void 0 ? void 0 : keyreseller.length) {
            keyreseller.map(x => {
                var _a, _b, _c, _d;
                let logo = x.active ? (_b = (_a = x.active) === null || _a === void 0 ? void 0 : _a.logo) === null || _b === void 0 ? void 0 : _b.documentPath : x.old ? (_c = x.old.logo) === null || _c === void 0 ? void 0 : _c.documentPath : x.new ? (_d = x.new.logo) === null || _d === void 0 ? void 0 : _d.logo_name : '';
                let companyName = x.active ? x.active.company_name : x.old ? x.old.company_name : x.new ? x.new.company_name : '';
                let id = x.active ? x.active.id : x.old ? x.old.id : x.new ? x.new.id : '';
                let brand = {
                    id: id,
                    name: companyName,
                    documentpath: logo ? process.env.CDN_URL + logo : null,
                    isNew: 0
                };
                keyResellerValues.push(Object.assign(x, brand));
            });
        }
        if (result.keyPartners.distributor) {
            if ((_g = result.keyPartners.distributor.channel) === null || _g === void 0 ? void 0 : _g.length) {
                result.keyPartners.distributor.channel.map(x => {
                    var _a, _b;
                    let brand = {
                        id: x.channelId,
                        name: x.companyName,
                        documentpath: ((_a = x.companylogo) === null || _a === void 0 ? void 0 : _a.documentPath) ? process.env.CDN_URL + ((_b = x.companylogo) === null || _b === void 0 ? void 0 : _b.documentPath) : null,
                        isNew: 0
                    };
                    keyDistributorOptions.push(Object.assign(x, brand));
                });
            }
            if ((_h = result.keyPartners.distributor.keypartners) === null || _h === void 0 ? void 0 : _h.length) {
                result.keyPartners.distributor.keypartners.map(x => {
                    var _a, _b;
                    let brand = {
                        id: x.channelkeypartnerid,
                        name: x.partnername,
                        documentpath: ((_a = x.companylogo) === null || _a === void 0 ? void 0 : _a.documentPath) ? process.env.CDN_URL + ((_b = x.companylogo) === null || _b === void 0 ? void 0 : _b.documentPath) : null,
                        isNew: 0
                    };
                    keyDistributorOptions.push(Object.assign(x, brand));
                });
            }
        }
        let keyDistributor = await this.channelSettingService.getKeyRetailers(+user.channelId, config_1.appconstant.CHANNEL_TYPE_DISTRIBUTOR);
        if (keyDistributor === null || keyDistributor === void 0 ? void 0 : keyDistributor.length) {
            keyDistributor.map(x => {
                var _a, _b, _c, _d;
                let logo = x.active ? (_b = (_a = x.active) === null || _a === void 0 ? void 0 : _a.logo) === null || _b === void 0 ? void 0 : _b.documentPath : x.old ? (_c = x.old.logo) === null || _c === void 0 ? void 0 : _c.documentPath : x.new ? (_d = x.new.logo) === null || _d === void 0 ? void 0 : _d.logo_name : '';
                let companyName = x.active ? x.active.company_name : x.old ? x.old.company_name : x.new ? x.new.company_name : '';
                let id = x.active ? x.active.id : x.old ? x.old.id : x.new ? x.new.id : '';
                let brand = {
                    id: id,
                    name: companyName,
                    documentpath: logo ? process.env.CDN_URL + logo : null,
                    isNew: 0
                };
                keyDistributorValues.push(Object.assign(x, brand));
            });
        }
        let final = {
            keyPartners: { keyRetailerOptions, keyDistributorOptions, keyResellerOptions },
            channelKeyRetailer: keyRetailerValues,
            channelKeyDistributor: keyDistributorValues,
            channelKeyReseller: keyResellerValues
        };
        return res.status(common_1.HttpStatus.OK).send(final);
    }
    async getChannelProfile(req, res, data) {
        var _a, _b, _c, _d;
        let channelId = data.channelId;
        let userId = data.userId;
        let user = await this.userService.findUserById(userId, channelId);
        let businessType = user.businessType ? user.businessType : '2';
        let ctype = +user.channelTypeId;
        let typeid = [];
        if (ctype == 2 && businessType == 1)
            typeid = [4, 5];
        else if (ctype == 2 && businessType == 2)
            typeid = [4, 5];
        else if (ctype == 2 && businessType == 3)
            typeid = [4, 5];
        else if (ctype == 4)
            typeid = [4];
        else if (ctype == 5)
            typeid = [5];
        else if (ctype == 3 && businessType == 1)
            typeid = [4, 5];
        else if (ctype == 3 && businessType == 2)
            typeid = [4, 5];
        else if (ctype == 3 && businessType == 3)
            typeid = [4, 5];
        let result = await this.channelSettingService.getChannelProfile(user);
        let profileList = [];
        if (result.length > 0) {
            let pro = (_b = (_a = result[0]) === null || _a === void 0 ? void 0 : _a.channelCustomerProfile) === null || _b === void 0 ? void 0 : _b.map(x => +x.profileTypeId);
            let clssifiid = (_d = (_c = result[0]) === null || _c === void 0 ? void 0 : _c.channelCustomerProfile) === null || _d === void 0 ? void 0 : _d.map(x => +x.profileTypeClassificationId);
            let query = 'select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
                + 'when ptc."channelTypeId"=2 then \'vendor\' when ptc."channelTypeId"=5 then \'Reseller\' end as groupname,pt."profileType" as childname,'
                + 'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" and pt."profileTypeId" in(:ptype) and ptc."channelTypeId" in(:typeId) '
                + 'and  ptc."profileTypeClassificationId" in(:clssifiid) ';
            profileList = await this.sequelizeInstance.query(query, {
                type: sequelize_1.QueryTypes.SELECT,
                replacements: {
                    typeId: typeid,
                    clssifiid: clssifiid,
                    ptype: pro,
                }
            }).then(results => {
                return results;
            });
            console.log('profileList', profileList);
        }
        let profileType = '';
        if (result.length > 0 && result[0].profileTypeId) {
            let query = 'select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
                + 'when ptc."channelTypeId"=2 then \'vendor\' when ptc."channelTypeId"=5 then \'Reseller\' end as groupname,pt."profileType" as childname,'
                + 'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" where ptc."channelTypeId" = :typeId and ptc."profileTypeId"=:profileTypeId';
            profileType = await this.sequelizeInstance.query(query, {
                type: sequelize_1.QueryTypes.SELECT,
                replacements: {
                    typeId: user.channelTypeId,
                    profileTypeId: result[0].profileTypeId
                }
            }).then(results => {
                return results;
            });
        }
        let finalData = {
            profile: profileList,
            profileTypeId: result > 0 ? result[0].profileTypeId : null,
            profileType: profileType,
        };
        return res.status(common_1.HttpStatus.OK).send(finalData);
    }
    async getChannelLocation(channelId, req, res) {
        let query = 'select "userId" from "tblUser" where "channelId"=:channelId and "contactId"=(select "contactId" from "tblContact" where "channelId"=:channelId and "isDefaultContact"=\'1\' and "deletedAt" is null)';
        let userId = await this.sequelizeInstance.query(query, {
            type: sequelize_1.QueryTypes.SELECT,
            replacements: {
                channelId: channelId
            }
        }).then(result => {
            return result[0].userId;
        });
        let user = await this.userService.findUserById(userId, channelId);
        let getChannelLocation = await this.tradeLocationService.getChannelLocation(user);
        if (getChannelLocation) {
            return res.status(common_1.HttpStatus.OK).json(getChannelLocation);
        }
        throw new Process_exception_1.ProcessException('channel:channelLocation:noLocations');
    }
    async getContactLocationBo(contactId, req, res, data) {
        let channelId = data.channelId;
        let userId = data.userId;
        let user = await this.userService.findUserById(userId, channelId);
        if (user.contactId != contactId)
            user.contactId = contactId;
        let result;
        result = await this.channelSettingService.getContactLocation(user);
        if (result) {
            let targetArray = result.contactLocation.filter(x => x.locationType == 'TARGET');
            let sellingArray = user.channelTypeId == 4 ? result.contactLocation.filter(x => x.locationType == 'RETAIL') : user.channelTypeId == 5 ? result.contactLocation.filter(x => x.locationType == 'RESELLING') :
                result.contactLocation.filter(x => x.locationType == 'SELLING');
            let data = {
                targetArray: targetArray,
                sellingArray: sellingArray
            };
            res.status(common_1.HttpStatus.OK).json(data);
        }
    }
    async getContactProduct(contactId, channelId, req, res) {
        let result = await this.channelSettingService.getContactProduct(contactId, channelId);
        if (result) {
            res.status(common_1.HttpStatus.OK).json(result);
        }
    }
    async reporting(profileData, req, res, data) {
        let channelId = data.channelId;
        let userId = data.userId;
        let user = await this.userService.findUserById(userId, channelId);
        let result = await this.channelSettingService.reportingusers(user);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async generalInfo(contactId, req, res) {
        let result = await this.channelSettingService.generalInfo(contactId);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async getUserPreference(contactId, channelId, res, req) {
        let result = await this.userService.getUserPreference(contactId, channelId);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async getPreference(res, req, typeId) {
        let result = await this.userService.getPreference(typeId);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async userProfileUpdate(id, userDto, res, req, channelId, userId) {
        let nonsuggestion = [];
        let count;
        let real_user = await this.userService.findUserById(userId, channelId);
        if (Object.keys(userDto).length) {
            let user = await this.userService.getUser(id, real_user.channelId);
            let allowToModifyInterestedCategory = true;
            await this.userService.updateUser(userDto, user, allowToModifyInterestedCategory, real_user);
        }
        res.status(common_1.HttpStatus.OK).send({
            updated: true
        });
    }
    async boEventCreate(req, res, data) {
        console.log('data', data);
        await this.channelAdminService.boEventCreate(data);
        res.status(common_1.HttpStatus.CREATED).send({
            created: true
        });
    }
    async getEventListing(Req, res, data) {
        let result = await this.channelAdminService.getEventListing(data);
        res.status(common_1.HttpStatus.OK).json(result);
    }
    async getEventById(req, res, eventId) {
        let result = await this.channelAdminService.getEventById(eventId);
        res.status(common_1.HttpStatus.OK).json(result);
    }
    async getEventDetailsById(req, res, eventId) {
        let result = await this.channelAdminService.getEventDetailsById(eventId);
        res.status(common_1.HttpStatus.OK).json(result);
    }
    async eventAvailabilityCheck(req, res, eventCatId, eventId) {
        let result = await this.channelAdminService.eventAvailabilityCheck(eventCatId, eventId);
        res.status(common_1.HttpStatus.OK).json(result);
    }
    async getEventChannelDetails(req, res, eventId, channelId) {
        let result = await this.channelAdminService.getEventChannelDetails(channelId, eventId);
        res.status(common_1.HttpStatus.OK).json(result);
    }
    async boEventUsersManage(req, res, data) {
        let result = await this.channelAdminService.boEventUserManage(data);
        res.status(common_1.HttpStatus.OK).json(result);
    }
    async boToEventCatalyst(req, res, data) {
        let result = await this.channelAdminService.boToEventCatalyst(data);
        res.status(common_1.HttpStatus.OK).json(result);
    }
    async setEventContact(id, res) {
        await this.channelService.isEvent(id, '1');
        return res.status(common_1.HttpStatus.OK).send();
    }
    async setEventParticipantStatus(req, res, data) {
        await this.channelService.setEventParticipantStatus(data.eventId, data.channelId, data.contactId);
        return res.status(common_1.HttpStatus.OK).send();
    }
    async offlinePayment(id, updateSubscriptionDto, res) {
        let result = await this.subscriptionService.updateOfflinrPayment(id, updateSubscriptionDto);
        return res.status(common_1.HttpStatus.OK).send(result[0]);
    }
    async companyNameList(req, res, data) {
        console.log('data', data);
        let result = await this.channelAdminService.companyNameList(data.search);
        res.status(common_1.HttpStatus.OK).json(result);
    }
    async getContactSearchList(req, res, data) {
        console.log('data', data);
        let result = await this.channelAdminService.contactNameList(data.search);
        res.status(common_1.HttpStatus.OK).json(result);
    }
    async getContact(data, contactId, res) {
        let contact = await this.channelAdminService.getContact(contactId);
        return res.status(common_1.HttpStatus.OK).send(contact);
    }
    async updateContact(data, res, req, contactId) {
        await this.userService.updateContact(contactId, data);
        res.status(common_1.HttpStatus.OK).send({
            updated: true
        });
    }
    async channelDetail(channelId, req, res) {
        let channelDetail = await this.channelAdminService.getChannel(channelId);
        if (!channelDetail) {
            throw new Process_exception_1.ProcessException('channel:getChannel:channelNotFound');
        }
        res.status(common_1.HttpStatus.OK).send(channelDetail);
    }
    async myProduct(req, res, productFilters) {
        console.log('productFilters', productFilters);
        let userId = productFilters.user.userId;
        let channelId = productFilters.user.channelId;
        let user = await this.userService.findUserById(userId, channelId);
        let order = productFilters.order ? productFilters.order : 'asc';
        let limit = ' LIMIT ' + productFilters.limit;
        let offset = ' OFFSET ' + productFilters.offset;
        let query = 'select count(*) OVER() AS total,tp."product_rank_order",tp."isPublic",tp."productId", tp."channelId", (select "brandName" from "tblBrand" where "id"=tp."brandId"), tp."product",tp."productSKU", tp."isactive", tp."productqualitylevel", pr."businesscurrencyid",pr."retailerprice",(select concat( \'' + process.env.CDN_URL + '\',"documentPath") as "documentUrl" from "tblDocument" where "documentId" in(select "imageid" from "tblproductimage" where "productimageid"=tp."defaultimageid"))from "tblProduct" as tp inner join "tblproductpricedetail" as pr on pr."productId"=tp."productId" where "channelId"= :channelId and tp."deletedat" is null and "isactive"=:isactive ';
        if (productFilters.search)
            query += ' and tp."productId"=' + +productFilters.search;
        if (productFilters.filter.brand)
            query += ' and tp."brandId" in(' + productFilters.filter.brand + ')';
        if (productFilters.filter.family)
            query += ' and tp."productGroupId" in(' + productFilters.filter.family + ')';
        if (productFilters.filter.category)
            query += ' and tp."productCategorieId" in(' + productFilters.filter.category + ')';
        if (productFilters.filter.position)
            query += ' and tp."productqualitylevel" =:quality';
        query += ' order by tp."product_rank_order" ' + order + limit + offset;
        let result = await this.sequelizeInstance.query(query, {
            type: sequelize_1.QueryTypes.SELECT,
            raw: true,
            replacements: {
                channelId: user.channelId,
                isactive: productFilters.isactive.toString(),
                quality: productFilters.filter.position
            }
        });
        let master = result.map(x => x.productSKU);
        let active = await this.sequelizeInstance.query('select count(*) from "tblProduct" where "channelId"=:channelId and "deletedat" is null and "isactive"=:isactive', {
            type: sequelize_1.QueryTypes.SELECT,
            raw: true,
            replacements: {
                channelId: user.channelId,
                isactive: '1'
            }
        });
        let inActive = await this.sequelizeInstance.query('select count(*) from "tblProduct" where "channelId"=:channelId and "deletedat" is null and "isactive"=:isactive', {
            type: sequelize_1.QueryTypes.SELECT,
            raw: true,
            replacements: {
                channelId: user.channelId,
                isactive: '0'
            }
        });
        let tabCount = {
            active: active ? active[0].count : 0,
            inActive: active ? inActive[0].count : 0
        };
        let results;
        if (result.length != 0) {
            results = {
                total: result[0].total,
                result: result,
                tabCount: tabCount,
                master: master
            };
        }
        else {
            results = {
                total: 0,
                result: [],
                master: [],
                tabCount: tabCount
            };
        }
        res.status(common_1.HttpStatus.OK).send(results);
    }
    async myProductsAgg(req, res, isActive, data) {
        let userId = data.userId;
        let channelId = data.channelId;
        let user = await this.userService.findUserById(userId, channelId);
        let result = await this.sequelizeInstance.query('select "productId" from "tblProduct" where "channelId"=:channelId and "deletedat" is null and "isactive"=:isactive', {
            type: sequelize_1.QueryTypes.SELECT,
            raw: true,
            replacements: {
                channelId: user.channelId,
                isactive: isActive.toString()
            }
        }).then(x => {
            return x.map(p => +p.productId);
        });
        console.log('result', result);
        let product = await this.productService.getProductAggr(result);
        if (product) {
            res.status(common_1.HttpStatus.OK).send(product);
        }
    }
    async oldChannelDetail(channelId, req, res) {
        let channelDetail = await this.channelAdminService.getOldChannelDetails(channelId);
        if (!channelDetail) {
            throw new Process_exception_1.ProcessException('channel:getChannel:channelNotFound');
        }
        res.status(common_1.HttpStatus.OK).send(channelDetail);
    }
    async oldChannelDetailView(channelId, req, res) {
        let channelDetail = await this.channelAdminService.getOldChannelDetailView(channelId);
        if (!channelDetail) {
            throw new Process_exception_1.ProcessException('channel:getChannel:channelNotFound');
        }
        res.status(common_1.HttpStatus.OK).send(channelDetail);
    }
    async testPar(channelId, req, res) {
        res.status(common_1.HttpStatus.OK).send(channelId);
    }
    async test(req, res) {
        res.status(common_1.HttpStatus.OK).send(true);
    }
    async testPost(data, req, res) {
        res.status(common_1.HttpStatus.OK).send(data);
    }
    async getPendingEventPayment(req, res) {
        let result = await this.channelService.getPendingEvent();
        console.log(result);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async updateEventPayment(req, res) {
        let result = await this.channelService.updateEventChannel(req.body);
        console.log(result);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async getChannelCountry(req, res, channelId) {
        let query = 'select "countryId", country, "phoneCode", "isoCode" from "tblCountry" where "countryId"=(select "countryId" from "tblChannel" where "channelId"=:channelId limit 1)';
        let result = await this.sequelizeInstance.query(query, {
            type: sequelize_1.QueryTypes.SELECT,
            replacements: {
                channelId: channelId,
            }
        }).then(res => res[0]);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async checkVat(req, res, data) {
        let result = await fetch(process.env.VAT_API_URL + data.vatNumber)
            .then(res => res.json())
            .then(data => {
            if (data.valid) {
                return 1;
            }
            else {
                return 0;
            }
        })
            .catch(err => {
            console.log('vat error', err);
            return 0;
        });
        res.status(common_1.HttpStatus.OK).json({ valid: result });
    }
    async getOldFacts(req, res, oldChannelId) {
        let result = await this.channelAdminService.getOldChannelFacts(oldChannelId);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async getOldProducts(req, res, oldChannelId) {
        let result = await this.channelAdminService.getOldProducts(oldChannelId);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async getOldChannelBrand(req, res, oldChannelId) {
        let result = await this.channelAdminService.getOldChannelBrand(oldChannelId);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async getOldInitialProfile(req, res, data) {
        let result = await this.channelAdminService.getOldInitialProfile(data);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async channelProfileOld(Req, res, data) {
        let user = data;
        let result = await this.channelAdminService.channelProfileOld(data.oldChannelId);
        let ctype = +user.channelTypeId;
        let typeid = [];
        if (ctype == 2)
            typeid = [4, 5];
        else if (ctype == 4)
            typeid = [4];
        else if (ctype == 5)
            typeid = [5];
        else if (ctype == 3)
            typeid = [4, 5];
        let profileList = [];
        if (result) {
            result.map(async (x) => {
                var _a, _b;
                let pro = (_a = result[0].channelCustomerProfile) === null || _a === void 0 ? void 0 : _a.map(x => +x.profileTypeId);
                let clssifiid = (_b = result[0].channelCustomerProfile) === null || _b === void 0 ? void 0 : _b.map(x => +x.profileTypeClassificationId);
                let query = 'select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
                    + 'when ptc."channelTypeId"=2 then \'vendor\' when ptc."channelTypeId"=5 then \'Reseller\' end as groupname,pt."profileType" as childname,'
                    + 'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" and pt."profileTypeId" in(:ptype) and ptc."channelTypeId" in (:typeId)'
                    + 'and  ptc."profileTypeClassificationId" in(:clssifiid) and pt."profileType" != \'\' ';
                profileList = await this.sequelizeInstance.query(query, {
                    type: sequelize_1.QueryTypes.SELECT,
                    replacements: {
                        typeId: typeid,
                        clssifiid: clssifiid,
                        ptype: (user.channelTypeId == 4 || user.channelTypeId == 5) ? result[0].profileTypeId : pro,
                    }
                }).then(results => {
                    return results;
                });
            });
        }
        let profileType = '';
        let query = '';
        if (result[0].profileTypeId) {
            query = 'select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
                + 'when ptc."channelTypeId"=2 then \'vendor\' end as groupname,pt."profileType" as childname,'
                + 'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" where ptc."channelTypeId" = :typeId and ptc."profileTypeId"=:profileTypeId ';
            profileType = await this.sequelizeInstance.query(query, {
                type: sequelize_1.QueryTypes.SELECT,
                replacements: {
                    typeId: user.channelTypeId,
                    profileTypeId: result[0].profileTypeId
                }
            }).then(results => {
                return results;
            });
        }
        let finalData = {
            profile: profileList,
            profileTypeId: result.length > 0 ? result[0].profileTypeId : null,
            profileType: profileType,
        };
        return res.status(common_1.HttpStatus.OK).send(finalData);
    }
    async editBasicOldCompanyProfile(profileData, req, res) {
        let result = await this.channelAdminService.editBasicCompanyOld(profileData);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async saveOldChannelFacts(req, res, data) {
        let result = await this.channelAdminService.saveOldChannelFacts(data);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async updateBannerImage(bannerImg, req, res) {
        let result = await this.channelAdminService.updateBannerImage(bannerImg);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
};
__decorate([
    common_1.Post('pendingApproval'),
    __param(0, common_1.Body()), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "pendingApproval", null);
__decorate([
    common_1.Post('channelSubscription'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body(new common_1.ValidationPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, subscriptionFilter_dto_1.SubscriptionFilterBODto]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "channelSubscriptionList", null);
__decorate([
    common_1.Get('pendingChannelsList/:userId'),
    __param(0, common_1.Param('userId')), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "pendingChannelsList", null);
__decorate([
    common_1.Get('oldChannelsList/:userId'),
    __param(0, common_1.Param('userId')), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "oldChannelsList", null);
__decorate([
    common_1.Post('oldChannels'),
    __param(0, common_1.Body()), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "oldChannels", null);
__decorate([
    common_1.Put('channelApproval'),
    __param(0, common_1.Body()), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "channelApproval", null);
__decorate([
    common_1.Post('getFilterCountry'),
    __param(0, common_1.Body()), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getFilterCountry", null);
__decorate([
    common_1.Post('getProductCat'),
    __param(0, common_1.Body()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getProductCat", null);
__decorate([
    common_1.Get('getSubscriptionPlan'),
    __param(0, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getSubscriptionPlan", null);
__decorate([
    common_1.Post('introVideoUpload'),
    __param(0, common_1.Body()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "introVideoUpload", null);
__decorate([
    common_1.Get('medialibraryByContact'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "medialibraryByContact", null);
__decorate([
    common_1.Post('deleteMediaVideo'),
    __param(0, common_1.Res()), __param(1, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "deleteMediaVideo", null);
__decorate([
    common_1.Post('deleteCompanyLogo'),
    __param(0, common_1.Res()), __param(1, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "deleteCompanyLogo", null);
__decorate([
    common_1.Post('getInitialProfile'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getInitialProfile", null);
__decorate([
    common_1.Post('tradeinfo'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "tradeinfo", null);
__decorate([
    common_1.Post('tradelocation'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "tradelocation", null);
__decorate([
    common_1.Post('getuser'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getUsers", null);
__decorate([
    common_1.Post('defaultcontact'),
    __param(0, common_1.Res()), __param(1, common_1.Req()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getDefaultContact", null);
__decorate([
    common_1.Post('allcontacts'),
    __param(0, common_1.Res()), __param(1, common_1.Req()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getAllContacts", null);
__decorate([
    common_1.Post('settings'),
    __param(0, common_1.Res()), __param(1, common_1.Req()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "settings", null);
__decorate([
    common_1.Post('profileAndSpeciality'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getProfileAndSpeciality", null);
__decorate([
    common_1.Post('setDefaultContact/:contactId'),
    __param(0, common_1.Param('contactId')), __param(1, common_1.Res()), __param(2, common_1.Req()), __param(3, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "setDefaultContact", null);
__decorate([
    common_1.Post('updateCompany'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "update_Company_Profile", null);
__decorate([
    common_1.Post('update_company_profile'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "update_company_profile", null);
__decorate([
    common_1.Post('updateTradeInfo'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "update_Trade_Info", null);
__decorate([
    common_1.Put('updateChannelLocation'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "updateChannelLocation1", null);
__decorate([
    common_1.Post('updatesettings'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body(new common_1.ValidationPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "updateSettings", null);
__decorate([
    common_1.Post('updateChannel'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "update_channel_profile", null);
__decorate([
    common_1.Post('add'),
    __param(0, common_1.Res()), __param(1, common_1.Req()), __param(2, common_1.Body(new common_1.ValidationPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "addNewUser", null);
__decorate([
    common_1.Post('getUserProfile'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getUserProfile", null);
__decorate([
    common_1.Get('getContactLocation/:contactId/:channelTypeId'),
    __param(0, common_1.Param('contactId')), __param(1, common_1.Param('channelTypeId')), __param(2, common_1.Req()), __param(3, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getContactLocation", null);
__decorate([
    common_1.Post('channelCoupons'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body(new common_1.ValidationPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getCouponList", null);
__decorate([
    common_1.Get('channelCoupons/:id'),
    __param(0, common_1.Param('id')), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getCouponByID", null);
__decorate([
    common_1.Post('addCoupon'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body(new common_1.ValidationPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, adminCoupon_dto_1.AdminCouponAddDto]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "AddChannelCoupon", null);
__decorate([
    common_1.Put('channelCoupons/:id'),
    __param(0, common_1.Param('id')), __param(1, common_1.Req()), __param(2, common_1.Res()), __param(3, common_1.Body(new common_1.ValidationPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object, adminCoupon_dto_1.AdminCouponAddDto]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "UpdateChannelCoupon", null);
__decorate([
    common_1.Get('channelKeyPartnerCheck'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Query('KeyRetailer')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "existingKeyRetailerCheck", null);
__decorate([
    common_1.Post('searchRegion'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "searchRegion", null);
__decorate([
    common_1.Get('buyerNeed'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getBuyerNeed", null);
__decorate([
    common_1.Post('request_Intro/:channelid'),
    __param(0, common_1.Param('channelid')), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getAllRequestIntro", null);
__decorate([
    common_1.Post('approveChannel'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body(new common_1.ValidationPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "approveChannel", null);
__decorate([
    common_1.Get('getPublicCompanyFlags/:channelId'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Param('channelId')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getPublicCompanyFlages", null);
__decorate([
    common_1.Post('setCompanyPublicFlags'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "setCompanyPublicFlags", null);
__decorate([
    common_1.Post('getChannelDetail'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getChannelDetail", null);
__decorate([
    common_1.Post('company'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "company", null);
__decorate([
    common_1.Post('getCertificateTag'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getChannelCertificateTags", null);
__decorate([
    common_1.Post('getChannelProduct'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getChannelProduct", null);
__decorate([
    common_1.Post('getChannelBrand'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getChannelBrand", null);
__decorate([
    common_1.Post('keyPartnerOptions'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getKeyPartnerOptions", null);
__decorate([
    common_1.Post('getChannelProfile'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getChannelProfile", null);
__decorate([
    common_1.Get('getChannelLocation/:channelId'),
    __param(0, common_1.Param('channelId')), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getChannelLocation", null);
__decorate([
    common_1.Post('getContactLocation/:contactId'),
    __param(0, common_1.Param('contactId')), __param(1, common_1.Req()), __param(2, common_1.Res()), __param(3, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getContactLocationBo", null);
__decorate([
    common_1.Get('getContactProduct/:contactId/:channelId'),
    __param(0, common_1.Param('contactId')), __param(1, common_1.Param('channelId')), __param(2, common_1.Req()), __param(3, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getContactProduct", null);
__decorate([
    common_1.Post('reporting'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Req()), __param(2, common_1.Res()), __param(3, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "reporting", null);
__decorate([
    common_1.Post('generalInfo'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "generalInfo", null);
__decorate([
    common_1.Get('userpreference/:contactId/:channelId'),
    __param(0, common_1.Param('contactId')), __param(1, common_1.Param('channelId')), __param(2, common_1.Res()), __param(3, common_1.Req()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getUserPreference", null);
__decorate([
    common_1.Get('preference/:typeId'),
    __param(0, common_1.Res()), __param(1, common_1.Req()), __param(2, common_1.Param('typeId')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getPreference", null);
__decorate([
    common_1.Patch('profile/:id/:channelId/:userId'),
    __param(0, common_1.Param('id', new common_1.ParseIntPipe())), __param(1, common_1.Body(new common_1.ValidationPipe())), __param(2, common_1.Res()), __param(3, common_1.Req()), __param(4, common_1.Param('channelId')), __param(5, common_1.Param('userId')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, updateUser_dto_1.UpdateUserDto, Object, Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "userProfileUpdate", null);
__decorate([
    common_1.Post('boEventCreate'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "boEventCreate", null);
__decorate([
    common_1.Post('getEventListing'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getEventListing", null);
__decorate([
    common_1.Get('getEventById/:eventId'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Param('eventId')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getEventById", null);
__decorate([
    common_1.Get('getEventDetails/:eventId'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Param('eventId')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getEventDetailsById", null);
__decorate([
    common_1.Get('eventAvailabilityCheck/:eventCatId/:eventId'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Param('eventCatId')), __param(3, common_1.Param('eventId')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "eventAvailabilityCheck", null);
__decorate([
    common_1.Get('getEventChannelDetails/:channelId/:eventId'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Param('eventId')), __param(3, common_1.Param('channelId')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getEventChannelDetails", null);
__decorate([
    common_1.Post('boEventUsersManage'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "boEventUsersManage", null);
__decorate([
    common_1.Post('boToEventCatalyst'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "boToEventCatalyst", null);
__decorate([
    common_1.Get('setEventContact/:id'),
    __param(0, common_1.Param('id')), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "setEventContact", null);
__decorate([
    common_1.Post('setEventParticipantStatus/'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "setEventParticipantStatus", null);
__decorate([
    common_1.Post('updateSubscription/:id'),
    __param(0, common_1.Param('id')), __param(1, common_1.Body(new common_1.ValidationPipe())), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, updateSubscription_dto_1.BoUpdateSubscriptionDto, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "offlinePayment", null);
__decorate([
    common_1.Post('getCompanySearchList'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "companyNameList", null);
__decorate([
    common_1.Post('getContactSearchList'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getContactSearchList", null);
__decorate([
    common_1.Get('getContact/:contactId'),
    __param(0, common_1.Body()), __param(1, common_1.Param('contactId')), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getContact", null);
__decorate([
    common_1.Patch('updateContact/:contactId'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Res()), __param(2, common_1.Req()), __param(3, common_1.Param('contactId')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "updateContact", null);
__decorate([
    common_1.Get('channelDetail/:channelId'),
    __param(0, common_1.Param('channelId')), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "channelDetail", null);
__decorate([
    common_1.Post('my-products'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body(new common_1.ValidationPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "myProduct", null);
__decorate([
    common_1.Post('myProductsAgg/:isActive'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Param('isActive', new common_1.ParseIntPipe())), __param(3, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Number, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "myProductsAgg", null);
__decorate([
    common_1.Get('oldChannelDetail/:channelId'),
    __param(0, common_1.Param('channelId')), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "oldChannelDetail", null);
__decorate([
    common_1.Get('oldChannelDetailView/:channelId'),
    __param(0, common_1.Param('channelId')), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "oldChannelDetailView", null);
__decorate([
    common_1.Get('test/:channelId'),
    __param(0, common_1.Param('channelId')), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "testPar", null);
__decorate([
    common_1.Get('test'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "test", null);
__decorate([
    common_1.Post('test'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "testPost", null);
__decorate([
    common_1.Post('getPendingEventPayment'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getPendingEventPayment", null);
__decorate([
    common_1.Post('updateEventPayment'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "updateEventPayment", null);
__decorate([
    common_1.Get('channelCountry/:channelId'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Param('channelId')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getChannelCountry", null);
__decorate([
    common_1.Post('checkVat'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "checkVat", null);
__decorate([
    common_1.Get('getOldFacts/:oldChannelId'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Param('oldChannelId')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getOldFacts", null);
__decorate([
    common_1.Get('getOldProducts/:oldChannelId'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Param('oldChannelId')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getOldProducts", null);
__decorate([
    common_1.Get('getOldChannelBrand/:oldChannelId'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Param('oldChannelId')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getOldChannelBrand", null);
__decorate([
    common_1.Post('getOldInitialProfile'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "getOldInitialProfile", null);
__decorate([
    common_1.Post('channelProfileOld'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "channelProfileOld", null);
__decorate([
    common_1.Post('editBasicCompanyOld'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "editBasicOldCompanyProfile", null);
__decorate([
    common_1.Post('saveOldChannelFacts'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "saveOldChannelFacts", null);
__decorate([
    common_1.Post('updateBannerImage'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelAdminController.prototype, "updateBannerImage", null);
ChannelAdminController = __decorate([
    common_1.Controller('channelAdmin'),
    __param(7, common_1.Inject('SequelizeInstance')),
    __metadata("design:paramtypes", [channel_admin_service_1.ChannelAdminService,
        channel_setting_service_1.ChannelSettingService,
        channel_service_1.ChannelService,
        subscription_service_1.SubscriptionService,
        user_service_1.UserService,
        tradeLocation_service_1.TradeLocationService,
        product_service_1.ProductService, Object, context_service_1.ContextService])
], ChannelAdminController);
exports.ChannelAdminController = ChannelAdminController;
//# sourceMappingURL=channel.admin.controller.js.map