"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChannelSettingController = void 0;
const common_1 = require("@nestjs/common");
const passport_1 = require("@nestjs/passport");
const context_service_1 = require("../../../context/context.service");
const setting_dto_1 = require("../dto/setting.dto");
const usersetting_dto_1 = require("../dto/usersetting.dto");
const sequelize_1 = require("sequelize");
const channel_setting_service_1 = require("../service/channel.setting.service");
const channelLocation_dto_1 = require("../dto/channelLocation.dto");
const app_constant_1 = require("../../../config/app.constant");
const settings_dto_1 = require("../dto/settings.dto");
const searchRegion_dto_1 = require("../dto/searchRegion.dto");
const fetch = require("node-fetch");
let ChannelSettingController = class ChannelSettingController {
    constructor(channelSettingService, sequelizeInstance, context) {
        this.channelSettingService = channelSettingService;
        this.sequelizeInstance = sequelizeInstance;
        this.context = context;
    }
    async getInitialProfile(req, res) {
        let result = await this.channelSettingService.getInitialProfile(req.user);
        if (result) {
            res.status(common_1.HttpStatus.OK).json(result);
        }
    }
    async getVat(countryId, req, res) {
        let result = await this.channelSettingService.getVat(countryId);
        if (result) {
            res.status(common_1.HttpStatus.OK).json(result);
        }
    }
    async checkVat(req, res, data) {
        let result = await fetch(process.env.VAT_API_URL + data.vatNumber)
            .then(res => res.json())
            .then(data => {
            if (data.valid) {
                return 1;
            }
            else {
                return 0;
            }
        })
            .catch(err => {
            console.log('vat error', err);
            return 0;
        });
        res.status(common_1.HttpStatus.OK).json({ valid: result });
    }
    async getContactProduct(contactId, req, res) {
        let channelId = req.user.channelId;
        let result = await this.channelSettingService.getContactProduct(contactId, channelId);
        if (result) {
            res.status(common_1.HttpStatus.OK).json(result);
        }
    }
    async getProductProfile(channelId, req, data, res) {
        let result = await this.channelSettingService.getProductProfile(channelId);
        if (result) {
            res.status(common_1.HttpStatus.OK).json(result);
        }
    }
    async getUserProfile(channelId, req, data, res) {
        let user = req.user;
        let result = await this.channelSettingService.getUserProfile(user.channelId, user.channelTypeId);
        if (result) {
            res.status(common_1.HttpStatus.OK).json(result);
        }
    }
    async save_Company_Profile(profileData, req, res) {
        let result = await this.channelSettingService.save_Company_Profile(Object.assign(profileData, Object.assign({}, req.user)));
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async save_User_Profile(profileData, req, res) {
        let result = await this.channelSettingService.save_User_Profile(Object.assign(profileData, { channelTypeId: req.user.channelTypeId }));
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async getProfileList(channelId, typeId, req, res) {
        let channelTypeId = typeId;
        let result = await this.channelSettingService.getProfileList(channelTypeId, channelId);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async getUserStatus(channelId, req, res) {
        let typeId = req.user.channelTypeId;
        let result = await this.channelSettingService.getUserStatus(channelId, typeId);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async getCompanyMediaLibrary(companyId, req, res) {
        let result = await this.channelSettingService.getCompanyMediaLibrary(companyId, "channel");
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async getOldCompanyMediaLibrary(companyId, req, res) {
        let result = await this.channelSettingService.getCompanyMediaLibrary(companyId, "oldChannel");
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async createCompanyLogo(data, res) {
        let doc_id = await this.channelSettingService.createCompanyLogo(data, data.user);
        if (doc_id) {
            res.status(common_1.HttpStatus.OK).send(doc_id);
        }
    }
    async existingBrandCheck(req, res, data) {
        let result = await this.channelSettingService.existingBrandCheck(data);
        res.status(common_1.HttpStatus.OK).send(result);
    }
    async company(req, res, data) {
        let user = req.user;
        let result = await this.channelSettingService.getCompanyProfile(user);
        let channelPdf = await this.channelSettingService.getCompanyDocument(user.channelId);
        let chImages = await this.channelSettingService.getCompanyImages(user.channelId);
        let channelImages = [];
        if (chImages.length > 0) {
            chImages.map(x => {
                channelImages.push({ documentId: x.documentId, companyMediaId: x.companyMediaId, documentUrl: x.document.documentUrl });
            });
        }
        console.log('channelPdf', channelPdf);
        let channelPdfList = [];
        channelPdf.map(x => {
            let dpath = x.document.documentPath;
            channelPdfList.push({ documentId: x.documentId, name: x.name ? x.name : dpath.split('/').pop().split('#')[0].split('?')[0], channelDocumentId: x.channelDocumentId, document: { documentUrl: x.document.documentUrl } });
        });
        res.status(common_1.HttpStatus.OK).send(Object.assign(result, { channelPdf: channelPdfList }, { channelImages: channelImages }));
    }
    async tradeinfo(req, res) {
        var _a, _b, _c, _d;
        let user = req.user;
        let businessType = user.businessType;
        let ctype = +user.channelTypeId;
        let typeid = [];
        if (ctype == 2 && businessType == 1)
            typeid = [5];
        else if (ctype == 2 && businessType == 2)
            typeid = [4];
        else if (ctype == 2 && businessType == 3)
            typeid = [4, 5];
        else if (ctype == 4)
            typeid = [4];
        else if (ctype == 5)
            typeid = [5];
        else if (ctype == 3 && businessType == 1)
            typeid = [5];
        else if (ctype == 3 && businessType == 2)
            typeid = [4];
        else if (ctype == 3 && businessType == 3)
            typeid = [4, 5];
        let result = await this.channelSettingService.getTradeInfo(user);
        let brandList = [];
        let productList = [];
        let profileList = [];
        if (result) {
            result.map(async (x) => {
                var _a, _b, _c;
                (_a = x.channelBrand) === null || _a === void 0 ? void 0 : _a.map(x => {
                    let brand = {
                        id: x.brandId,
                        name: x.brand.brandName,
                        documentpath: process.env.CDN_URL + x.brand.document.documentPath,
                        isNew: 0
                    };
                    brandList.push(brand);
                });
                let cat = (_b = x.channelProductCategorie) === null || _b === void 0 ? void 0 : _b.map(x => x.productCategorie);
                let group = (_c = x.channelProductGroup) === null || _c === void 0 ? void 0 : _c.map(x => x.productGroup);
                group === null || group === void 0 ? void 0 : group.map(g => {
                    cat === null || cat === void 0 ? void 0 : cat.map(c => {
                        if (c.dataValues.groupid == g.dataValues.groupid) {
                            let product = {
                                childid: c.dataValues.childid,
                                childname: c.dataValues.childname,
                                groupid: c.dataValues.groupid,
                                groupname: g.dataValues.groupname,
                                status: true
                            };
                            productList.push(product);
                        }
                    });
                });
            });
            let pro = (_a = result[0].dataValues.channelCustomerProfile) === null || _a === void 0 ? void 0 : _a.map(x => +x.profileTypeId);
            let clssifiid = (_b = result[0].dataValues.channelCustomerProfile) === null || _b === void 0 ? void 0 : _b.map(x => +x.profileTypeClassificationId);
            let query = 'select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
                + 'when ptc."channelTypeId"=2 then \'vendor\' when ptc."channelTypeId"=5 then \'Reseller\' end as groupname,pt."profileType" as childname,'
                + 'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" and pt."profileTypeId" in(:ptype) and ptc."channelTypeId" in(:typeId) ';
            if (clssifiid.length)
                query += 'and  ptc."profileTypeClassificationId" in(:clssifiid) ';
            profileList = await this.sequelizeInstance.query(query, {
                type: sequelize_1.QueryTypes.SELECT,
                replacements: {
                    typeId: typeid,
                    clssifiid: clssifiid,
                    ptype: (req.user.channelTypeId == 4 || req.user.channelTypeId == 5) ? result[0].profileTypeId : pro,
                }
            }).then(results => {
                return results;
            });
            let keyRetailerOptions = [];
            let keyRetailerValues = [];
            let keyDistributorOptions = [];
            let keyDistributorValues = [];
            if (result.keyPartners.retailer) {
                if ((_c = result.keyPartners.retailer.channel) === null || _c === void 0 ? void 0 : _c.length) {
                    result.keyPartners.retailer.channel.map(x => {
                        var _a;
                        let brand = {
                            id: x.channelId,
                            name: x.companyName,
                            documentpath: process.env.CDN_URL + ((_a = x.companylogo) === null || _a === void 0 ? void 0 : _a.documentPath),
                            isNew: 0
                        };
                        keyRetailerOptions.push(Object.assign(x, brand));
                    });
                }
            }
            let keyretailer = await this.channelSettingService.getKeyRetailers(+req.user.channelId, app_constant_1.appconstant.CHANNEL_TYPE_RETAILER);
            if (keyretailer === null || keyretailer === void 0 ? void 0 : keyretailer.length) {
                keyretailer.map(x => {
                    var _a, _b, _c, _d;
                    let logo = x.active ? (_b = (_a = x.active) === null || _a === void 0 ? void 0 : _a.logo) === null || _b === void 0 ? void 0 : _b.documentPath : x.old ? (_c = x.old.logo) === null || _c === void 0 ? void 0 : _c.documentPath : x.new ? (_d = x.new.logo) === null || _d === void 0 ? void 0 : _d.logo_name : '';
                    let companyName = x.active ? x.active.company_name : x.old ? x.old.company_name : x.new ? x.new.company_name : '';
                    let id = x.active ? x.active.id : x.old ? x.old.id : x.new ? x.new.id : '';
                    let brand = {
                        id: id,
                        name: companyName,
                        documentpath: process.env.CDN_URL + logo,
                        isNew: 0
                    };
                    keyRetailerValues.push(Object.assign(x, brand));
                });
            }
            if (result.keyPartners.distributor) {
                if ((_d = result.keyPartners.distributor.channel) === null || _d === void 0 ? void 0 : _d.length) {
                    result.keyPartners.distributor.channel.map(x => {
                        var _a;
                        let brand = {
                            id: x.channelId,
                            name: x.companyName,
                            documentpath: process.env.CDN_URL + ((_a = x.companylogo) === null || _a === void 0 ? void 0 : _a.documentPath),
                            isNew: 0
                        };
                        keyDistributorOptions.push(Object.assign(x, brand));
                    });
                }
            }
            let keyDistributor = await this.channelSettingService.getKeyRetailers(+req.user.channelId, app_constant_1.appconstant.CHANNEL_TYPE_DISTRIBUTOR);
            if (keyDistributor === null || keyDistributor === void 0 ? void 0 : keyDistributor.length) {
                keyDistributor.map(x => {
                    var _a, _b, _c, _d;
                    let logo = x.active ? (_b = (_a = x.active) === null || _a === void 0 ? void 0 : _a.logo) === null || _b === void 0 ? void 0 : _b.documentPath : x.old ? (_c = x.old.logo) === null || _c === void 0 ? void 0 : _c.documentPath : x.new ? (_d = x.new.logo) === null || _d === void 0 ? void 0 : _d.logo_name : '';
                    let companyName = x.active ? x.active.company_name : x.old ? x.old.company_name : x.new ? x.new.company_name : '';
                    let id = x.active ? x.active.id : x.old ? x.old.id : x.new ? x.new.id : '';
                    let brand = {
                        id: id,
                        name: companyName,
                        documentpath: process.env.CDN_URL + logo,
                        isNew: 0
                    };
                    keyDistributorValues.push(Object.assign(x, brand));
                });
            }
            let profileType = '';
            if (result[0].dataValues.profileTypeId) {
                query = 'select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
                    + 'when ptc."channelTypeId"=2 then \'vendor\' end as groupname,pt."profileType" as childname,'
                    + 'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" where ptc."channelTypeId" = :typeId and ptc."profileTypeId"=:profileTypeId';
                profileType = await this.sequelizeInstance.query(query, {
                    type: sequelize_1.QueryTypes.SELECT,
                    replacements: {
                        typeId: req.user.channelTypeId,
                        profileTypeId: result[0].dataValues.profileTypeId
                    }
                }).then(results => {
                    return results;
                });
            }
            let data = {
                brand: brandList,
                product: productList,
                profile: profileList,
                empCnt: result[0].dataValues.channelDetail.empCntRange ? result[0].dataValues.channelDetail.empCntRange : result[0].dataValues.channelDetail.empCnt,
                no_of_skus: result[0].dataValues.channelDetail.no_of_skus,
                turnoverRange: result[0].dataValues.channelDetail.turnoverRange,
                revD: result[0].dataValues.channelDetail.revD,
                revI: result[0].dataValues.channelDetail.revI,
                startup: result[0].dataValues.channelDetail.startup,
                no_of_distributors: result[0].dataValues.channelDetail.no_of_distributors,
                no_of_retailers: result[0].dataValues.channelDetail.no_of_retailers,
                maturitylevel: result[0].dataValues.channelDetail.maturitylevel,
                profileTypeId: result[0].dataValues.profileTypeId,
                profileType: profileType,
                onlineStore: result[0].dataValues.channelDetail.turnoverPhyStore,
                turnoverOnStore: result[0].dataValues.channelDetail.turnoverOnStore,
                storeCount: result[0].dataValues.channelDetail.storeCount,
                sales: result[0].dataValues.channelDetail.empCntDSale,
                marketing: result[0].dataValues.channelDetail.empCntISale,
                technical: result[0].dataValues.channelDetail.empCntDMar,
                purchase: result[0].dataValues.channelDetail.empCntIMar,
                keyPartners: { keyRetailerOptions, keyDistributorOptions },
                channelKeyRetailer: keyRetailerValues,
                channelKeyDistributor: keyDistributorValues,
                keyretailers: result[0].dataValues.channelDetail.keyretailers,
                keydistributors: result[0].dataValues.channelDetail.keydistributors,
                serviceListOptions: result.serviceListOptions,
                serviceListValues: result[0].dataValues.serviceList ? result[0].dataValues.serviceList.map(x => ({ id: x.serviceListId, name: x.serviceList })) : [],
                specialityOptions: result.specialityOptions,
                channelSpeciality: result[0].dataValues.channelSpeciality,
                isFrenchTech: result[0].dataValues.isFrenchTech,
                isCompanyPublic: result[0].dataValues.isCompanyPublic
            };
            res.status(common_1.HttpStatus.OK).send(data);
        }
    }
    async getChannelProduct(req, res) {
        let user = req.user;
        console.log("btype:", user.businessType);
        let result = await this.channelSettingService.getChannelProduct(user.channelId, user.businessType);
        if (result) {
            res.status(common_1.HttpStatus.OK).json(result);
        }
    }
    async getChannelBrand(req, res) {
        let user = req.user;
        let result = await this.channelSettingService.getChannelBrand(user.channelId);
        if (result) {
            res.status(common_1.HttpStatus.OK).json(result);
        }
    }
    async getAdminChannelBrand(id, req, res) {
        let user = await this.getUser(id);
        let result = await this.channelSettingService.getChannelBrand(user.channelId);
        if (result) {
            res.status(common_1.HttpStatus.OK).json(result);
        }
    }
    async getUser(channelId) {
        let query = 'select "firstName","lastName",tc."email",tc."channelId",tc."contactId",tch."planId",tch."subscriptionId",tch."channelTypeId",tu."userId", '
            + 'case when tch."isOEM"=\'1\' then \'B2C\' when tch."isOEM"=\'2\' then \'B2B\' when tch."isOEM"=\'3\' then \'Both\' end, '
            + '(select "documentPath" from "tblDocument" where "documentId"=tc."documentId"), '
            + '(select "roleId" from "tblUserRole" where "userId"=tu."userId") '
            + 'from "tblContact" as tc inner join "tblChannel" as tch on tch."channelId"=tc."channelId" inner join "tblUser" as tu on tu."contactId"=tc."contactId"  where "isDefaultContact"=\'1\' and tc."channelId"=:channelId ';
        let user = await this.sequelizeInstance.query(query, {
            type: sequelize_1.QueryTypes.SELECT,
            raw: true,
            replacements: {
                channelId: channelId
            }
        }).then(result => {
            return result[0];
        }).catch(err => {
            console.log("getUserErr:", err);
        });
        return user;
    }
    async getChannelDetail(req, res) {
        let user = req.user;
        let result = await this.channelSettingService.getChannelDetail(user.channelId);
        if (result) {
            res.status(common_1.HttpStatus.OK).json(result);
        }
    }
    async getChannelCertificateTags(req, res) {
        let user = req.user;
        let result = await this.channelSettingService.getChannelCertificateTags(user);
        res.status(common_1.HttpStatus.OK).json(result);
    }
    async tradelocation(req, res) {
        var _a;
        let user = req.user;
        let result = await this.channelSettingService.getTradeLocation(user);
        let targetArray = [];
        let sellingArray = [];
        let targetids = [];
        let sellingids = [];
        if (result && result.length)
            (_a = result[0].channelLocation) === null || _a === void 0 ? void 0 : _a.map(x => {
                if (x.locationType == 'TARGET' && user.channelTypeId != 4) {
                    targetids.push(x.RegionCountryJCT.region.regionId);
                    let location = {
                        regionCountryJCTId: x.RegionCountryJCT.regionCountryJCTId,
                        groupid: x.RegionCountryJCT.region.regionId,
                        groupname: x.RegionCountryJCT.region.region,
                        childId: x.RegionCountryJCT.country.countryId,
                        childname: x.RegionCountryJCT.country.country,
                        status: true
                    };
                    targetArray.push(location);
                }
                if (x.locationType == 'SELLING' && user.channelTypeId != 4 && user.channelTypeId != 5) {
                    sellingids.push(x.RegionCountryJCT.region.regionId);
                    let location = {
                        regionCountryJCTId: x.RegionCountryJCT.regionCountryJCTId,
                        groupid: x.RegionCountryJCT.region.regionId,
                        groupname: x.RegionCountryJCT.region.region,
                        childId: x.RegionCountryJCT.country.countryId,
                        childname: x.RegionCountryJCT.country.country,
                        status: true
                    };
                    sellingArray.push(location);
                }
                if (x.locationType == 'RETAIL' && user.channelTypeId == 4) {
                    sellingids.push(x.RegionCountryJCT.region.regionId);
                    let location = {
                        regionCountryJCTId: x.RegionCountryJCT.regionCountryJCTId,
                        groupid: x.RegionCountryJCT.region.regionId,
                        groupname: x.RegionCountryJCT.region.region,
                        childId: x.RegionCountryJCT.country.countryId,
                        childname: x.RegionCountryJCT.country.country,
                        status: true
                    };
                    sellingArray.push(location);
                }
                if (x.locationType == 'RESELLING' && user.channelTypeId == 5) {
                    sellingids.push(x.RegionCountryJCT.region.regionId);
                    let location = {
                        regionCountryJCTId: x.RegionCountryJCT.regionCountryJCTId,
                        groupid: x.RegionCountryJCT.region.regionId,
                        groupname: x.RegionCountryJCT.region.region,
                        childId: x.RegionCountryJCT.country.countryId,
                        childname: x.RegionCountryJCT.country.country,
                        status: true
                    };
                    sellingArray.push(location);
                }
            });
        let zone = await this.sequelizeInstance.query('select "zoneId","regionId" from "tblZoneRegionJCT" ', {
            type: sequelize_1.QueryTypes.SELECT,
            raw: true,
        }).then(x => {
            console.log('zones', x);
            let zones = {
                tzone: x.filter(ch => { return targetids.includes(ch.regionId); }),
                szone: x.filter(ch => { return sellingids.includes(ch.regionId); })
            };
            return zones;
        });
        let data = {
            zone: zone,
            targetLocation: targetArray,
            sellingLocation: sellingArray
        };
        res.status(common_1.HttpStatus.OK).send(data);
    }
    async updateChannelLocation1(channelLocationDto, req, res) {
        let user = req.user;
        let subscriptionFeatureKeys = this.context.get('subscriptionFeatures');
        let nonsuggestion = [];
        let count;
        let userId = user.userId;
        let channelId = user.channelId;
        let contactId = user.contactId;
        let channelTypeId = user.channel.channelTypeId;
        let channelLocation = await this.channelSettingService.updateChannelLocation(channelLocationDto, user, subscriptionFeatureKeys);
        return res.status(common_1.HttpStatus.OK).send(channelLocation);
    }
    async getUsers(req, res) {
        let user = req.user;
        let result = await this.channelSettingService.getUsers(user);
        res.status(common_1.HttpStatus.OK).send(result);
    }
    async deleteUser(contactId, req, res) {
        let result = await this.channelSettingService.deleteUser(contactId);
        res.status(common_1.HttpStatus.OK).send(result);
    }
    async getuserLimits(req, res) {
        let user = req.user;
        let result = await this.channelSettingService.getuserLimits(user);
        res.status(common_1.HttpStatus.OK).send(result);
    }
    async update_Company_Profile(profileData, req, res) {
        let result = await this.channelSettingService.updateUserCompanyProfile(profileData, req.user);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async editBasicCompanyProfile(profileData, req, res) {
        let result = await this.channelSettingService.editBasicCompany(profileData, req.user);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async updateBannerImage(bannerImg, req, res) {
        let result = await this.channelSettingService.updateBannerImage(bannerImg, req.user);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async update_Trade_Info(profileData, req, res) {
        let result = await this.channelSettingService.updatetradeInformation(profileData, req.user);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async updateFacts(profileData, req, res) {
        let result = await this.channelSettingService.updateFacts(profileData, req.user);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async getFacts(req, res) {
        let result = await this.channelSettingService.getFacts(req.user);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async reporting(profileData, req, res) {
        let result = await this.channelSettingService.reportingusers(req.user);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async generalInfo(contactId, req, res) {
        let result = await this.channelSettingService.generalInfo(contactId);
        return res.status(common_1.HttpStatus.OK).send(result);
    }
    async getContactLocation(contactId, req, res) {
        let user = req.user;
        if (user.contactId != contactId)
            user.contactId = contactId;
        let result;
        result = await this.channelSettingService.getContactLocation(user);
        if (result) {
            let targetArray = result.contactLocation.filter(x => x.locationType == 'TARGET');
            let sellingArray = user.channelTypeId == 4 ? result.contactLocation.filter(x => x.locationType == 'RETAIL') : user.channelTypeId == 5 ? result.contactLocation.filter(x => x.locationType == 'RESELLING') :
                result.contactLocation.filter(x => x.locationType == 'SELLING');
            let data = {
                targetArray: targetArray,
                sellingArray: sellingArray
            };
            res.status(common_1.HttpStatus.OK).json(data);
        }
    }
    async getPaymentOption(req, res) {
        let result;
        result = await this.channelSettingService.getPaymentOption();
        res.status(common_1.HttpStatus.OK).json(result);
    }
    async getCurrency(req, res) {
        let result;
        result = await this.channelSettingService.getCurrency();
        res.status(common_1.HttpStatus.OK).json(result);
    }
    async existingKeyRetailerCheck(req, res, KeyRetailer) {
        let user = req.user;
        let result = await this.channelSettingService.existingKeyRetailerCheck(decodeURI(KeyRetailer), +req.query.CountryId, +user.channelId, +req.query.partner_type, req.query.partner_id ? +req.query.partner_id : null);
        res.status(common_1.HttpStatus.OK).send(result);
    }
    async updateSettings(req, res, settingsDto) {
        let channelTypeId = req.user.channelTypeId;
        let nonsuggestion = [];
        let count;
        let channelId = req.user.channelId;
        let userId = req.user.userId;
        let contactId = req.user.contactId;
        let user = req.user;
        let updateSettings = await this.channelSettingService.updateSettings(settingsDto, user);
        if (updateSettings) {
            return res.status(common_1.HttpStatus.OK).send('ok');
        }
    }
    async settings(res, req) {
        let settings = await this.channelSettingService.settings(req.user);
        if (settings) {
            return res.status(common_1.HttpStatus.OK).send(settings);
        }
    }
    async searchRegion(searchRegionDto, req, res) {
        let user = req.user;
        let searchRegion = await this.channelSettingService.searchRegion(searchRegionDto, user);
        let finaldata = [];
        if (searchRegion) {
            searchRegion.map(x => {
                let location = {
                    regionCountryJCTId: x.regionCountryJCTId,
                    groupid: x.region.regionId,
                    groupname: x.region.region,
                    childId: x.country.countryId,
                    childname: x.country.country,
                    status: false
                };
                finaldata.push(location);
            });
        }
        return res.status(common_1.HttpStatus.OK).send(finaldata);
    }
    async getProfileAndSpeciality(req, res) {
        let channelId = req.user.channelId;
        let result = await this.channelSettingService.getProfileAndSpeciality(channelId);
        let final = {};
        if (result) {
            let profileType = '';
            if (result[0].dataValues.profileTypeId) {
                let query = 'select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
                    + 'when ptc."channelTypeId"=2 then \'vendor\' end as groupname,pt."profileType" as childname,'
                    + 'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" where ptc."channelTypeId" in(:typeId) and ptc."profileTypeId"=:profileTypeId;';
                profileType = await this.sequelizeInstance.query(query, {
                    type: sequelize_1.QueryTypes.SELECT,
                    replacements: {
                        typeId: req.user.channelTypeId,
                        profileTypeId: result[0].dataValues.profileTypeId
                    }
                }).then(results => {
                    return results;
                });
            }
            final = {
                specialityOptions: result.specialityOptions,
                channelSpeciality: result[0].dataValues.channelSpeciality,
                profileTypeId: result[0].dataValues.profileTypeId,
                profileType: profileType,
                isFrenchTech: result[0].dataValues.isFrenchTech
            };
        }
        res.status(common_1.HttpStatus.OK).json(final);
    }
    async editProfile(req, res, data) {
        let user = req.user;
        let result = await this.channelSettingService.editProfileSave(user, data);
        if (result) {
            return res.status(common_1.HttpStatus.OK).send('OK');
        }
        else {
            return res.status(common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getKeyPartnerOptions(req, res) {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        let user = req.user;
        let result = await this.channelSettingService.getKeyPartnerOptions(user);
        let keyResellerOptions = [];
        let keyResellerValues = [];
        let keyRetailerOptions = [];
        let keyRetailerValues = [];
        let keyDistributorOptions = [];
        let keyDistributorValues = [];
        if (result.keyPartners.retailer) {
            if ((_a = result.keyPartners.retailer.channel) === null || _a === void 0 ? void 0 : _a.length) {
                result.keyPartners.retailer.channel.map(x => {
                    var _a, _b;
                    let brand = {
                        id: x.channelId,
                        name: x.companyName,
                        documentpath: ((_a = x.companylogo) === null || _a === void 0 ? void 0 : _a.documentPath) ? process.env.CDN_URL + ((_b = x.companylogo) === null || _b === void 0 ? void 0 : _b.documentPath) : null,
                        isNew: 0
                    };
                    keyRetailerOptions.push(Object.assign(x, brand));
                });
            }
            if ((_b = result.keyPartners.retailer.keypartners) === null || _b === void 0 ? void 0 : _b.length) {
                result.keyPartners.retailer.keypartners.map(x => {
                    var _a, _b;
                    let brand = {
                        id: x.channelkeypartnerid,
                        name: x.partnername,
                        documentpath: ((_a = x.companylogo) === null || _a === void 0 ? void 0 : _a.documentPath) ? process.env.CDN_URL + ((_b = x.companylogo) === null || _b === void 0 ? void 0 : _b.documentPath) : null,
                        isNew: 0
                    };
                    keyRetailerOptions.push(Object.assign(x, brand));
                });
            }
        }
        let keyretailer = await this.channelSettingService.getKeyRetailers(+req.user.channelId, app_constant_1.appconstant.CHANNEL_TYPE_RETAILER);
        if (keyretailer === null || keyretailer === void 0 ? void 0 : keyretailer.length) {
            keyretailer.map(x => {
                var _a, _b, _c, _d;
                let logo = x.active ? (_b = (_a = x.active) === null || _a === void 0 ? void 0 : _a.logo) === null || _b === void 0 ? void 0 : _b.documentPath : x.old ? (_c = x.old.logo) === null || _c === void 0 ? void 0 : _c.documentPath : x.new ? (_d = x.new.logo) === null || _d === void 0 ? void 0 : _d.logo_name : '';
                let companyName = x.active ? x.active.company_name : x.old ? x.old.company_name : x.new ? x.new.company_name : '';
                let id = x.active ? x.active.id : x.old ? x.old.id : x.new ? x.new.id : '';
                let brand = {
                    id: id,
                    name: companyName,
                    documentpath: logo ? process.env.CDN_URL + logo : null,
                    isNew: 0
                };
                keyRetailerValues.push(Object.assign(x, brand));
            });
        }
        if (result.keyPartners.reseller) {
            if ((_c = result.keyPartners.reseller.channel) === null || _c === void 0 ? void 0 : _c.length) {
                (_d = result.keyPartners.reseller.channel) === null || _d === void 0 ? void 0 : _d.map(x => {
                    var _a, _b;
                    let brand = {
                        id: x.channelId,
                        name: x.companyName,
                        documentpath: ((_a = x.companylogo) === null || _a === void 0 ? void 0 : _a.documentPath) ? process.env.CDN_URL + ((_b = x.companylogo) === null || _b === void 0 ? void 0 : _b.documentPath) : null,
                        isNew: 0
                    };
                    keyResellerOptions.push(Object.assign(x, brand));
                });
            }
            if ((_e = result.keyPartners.reseller.keypartners) === null || _e === void 0 ? void 0 : _e.length) {
                (_f = result.keyPartners.reseller.keypartners) === null || _f === void 0 ? void 0 : _f.map(x => {
                    var _a, _b;
                    let brand = {
                        id: x.channelkeypartnerid,
                        name: x.partnername,
                        documentpath: ((_a = x.companylogo) === null || _a === void 0 ? void 0 : _a.documentPath) ? process.env.CDN_URL + ((_b = x.companylogo) === null || _b === void 0 ? void 0 : _b.documentPath) : null,
                        isNew: 0
                    };
                    keyResellerOptions.push(Object.assign(x, brand));
                });
            }
        }
        let keyreseller = await this.channelSettingService.getKeyRetailers(+req.user.channelId, app_constant_1.appconstant.CHANNEL_TYPE_RESELLER);
        if (keyreseller === null || keyreseller === void 0 ? void 0 : keyreseller.length) {
            keyreseller.map(x => {
                var _a, _b, _c, _d;
                let logo = x.active ? (_b = (_a = x.active) === null || _a === void 0 ? void 0 : _a.logo) === null || _b === void 0 ? void 0 : _b.documentPath : x.old ? (_c = x.old.logo) === null || _c === void 0 ? void 0 : _c.documentPath : x.new ? (_d = x.new.logo) === null || _d === void 0 ? void 0 : _d.logo_name : '';
                let companyName = x.active ? x.active.company_name : x.old ? x.old.company_name : x.new ? x.new.company_name : '';
                let id = x.active ? x.active.id : x.old ? x.old.id : x.new ? x.new.id : '';
                let brand = {
                    id: id,
                    name: companyName,
                    documentpath: logo ? process.env.CDN_URL + logo : null,
                    isNew: 0
                };
                keyResellerValues.push(Object.assign(x, brand));
            });
        }
        if (result.keyPartners.distributor) {
            if ((_g = result.keyPartners.distributor.channel) === null || _g === void 0 ? void 0 : _g.length) {
                result.keyPartners.distributor.channel.map(x => {
                    var _a, _b;
                    let brand = {
                        id: x.channelId,
                        name: x.companyName,
                        documentpath: ((_a = x.companylogo) === null || _a === void 0 ? void 0 : _a.documentPath) ? process.env.CDN_URL + ((_b = x.companylogo) === null || _b === void 0 ? void 0 : _b.documentPath) : null,
                        isNew: 0
                    };
                    keyDistributorOptions.push(Object.assign(x, brand));
                });
            }
            if ((_h = result.keyPartners.distributor.keypartners) === null || _h === void 0 ? void 0 : _h.length) {
                result.keyPartners.distributor.keypartners.map(x => {
                    var _a, _b;
                    let brand = {
                        id: x.channelkeypartnerid,
                        name: x.partnername,
                        documentpath: ((_a = x.companylogo) === null || _a === void 0 ? void 0 : _a.documentPath) ? process.env.CDN_URL + ((_b = x.companylogo) === null || _b === void 0 ? void 0 : _b.documentPath) : null,
                        isNew: 0
                    };
                    keyDistributorOptions.push(Object.assign(x, brand));
                });
            }
        }
        let keyDistributor = await this.channelSettingService.getKeyRetailers(+req.user.channelId, app_constant_1.appconstant.CHANNEL_TYPE_DISTRIBUTOR);
        if (keyDistributor === null || keyDistributor === void 0 ? void 0 : keyDistributor.length) {
            keyDistributor.map(x => {
                var _a, _b, _c, _d;
                let logo = x.active ? (_b = (_a = x.active) === null || _a === void 0 ? void 0 : _a.logo) === null || _b === void 0 ? void 0 : _b.documentPath : x.old ? (_c = x.old.logo) === null || _c === void 0 ? void 0 : _c.documentPath : x.new ? (_d = x.new.logo) === null || _d === void 0 ? void 0 : _d.logo_name : '';
                let companyName = x.active ? x.active.company_name : x.old ? x.old.company_name : x.new ? x.new.company_name : '';
                let id = x.active ? x.active.id : x.old ? x.old.id : x.new ? x.new.id : '';
                let brand = {
                    id: id,
                    name: companyName,
                    documentpath: logo ? process.env.CDN_URL + logo : null,
                    isNew: 0
                };
                keyDistributorValues.push(Object.assign(x, brand));
            });
        }
        let final = {
            keyPartners: { keyRetailerOptions, keyDistributorOptions, keyResellerOptions },
            channelKeyRetailer: keyRetailerValues,
            channelKeyDistributor: keyDistributorValues,
            channelKeyReseller: keyResellerValues
        };
        return res.status(common_1.HttpStatus.OK).send(final);
    }
    async update_company_profile(req, res, data) {
        let user = req.user;
        let result = await this.channelSettingService.update_company_new(user, data);
        console.log('data', data);
        if (result)
            return res.status(common_1.HttpStatus.OK).send('OK');
        else
            return res.status(common_1.HttpStatus.BAD_REQUEST);
    }
    async getChannelProfile(req, res) {
        var _a, _b, _c, _d;
        let user = req.user;
        let businessType = user.businessType ? user.businessType : '2';
        let ctype = +user.channelTypeId;
        let typeid = [];
        if (ctype == 2 && businessType == 1)
            typeid = [4, 5];
        else if (ctype == 2 && businessType == 2)
            typeid = [4, 5];
        else if (ctype == 2 && businessType == 3)
            typeid = [4, 5];
        else if (ctype == 4)
            typeid = [4];
        else if (ctype == 5)
            typeid = [5];
        else if (ctype == 3 && businessType == 1)
            typeid = [4, 5];
        else if (ctype == 3 && businessType == 2)
            typeid = [4, 5];
        else if (ctype == 3 && businessType == 3)
            typeid = [4, 5];
        let result = await this.channelSettingService.getChannelProfile(user);
        let profileList = [];
        if (result.length > 0) {
            let pro = (_b = (_a = result[0]) === null || _a === void 0 ? void 0 : _a.channelCustomerProfile) === null || _b === void 0 ? void 0 : _b.map(x => +x.profileTypeId);
            let clssifiid = (_d = (_c = result[0]) === null || _c === void 0 ? void 0 : _c.channelCustomerProfile) === null || _d === void 0 ? void 0 : _d.map(x => +x.profileTypeClassificationId);
            let query = 'select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
                + 'when ptc."channelTypeId"=2 then \'vendor\' when ptc."channelTypeId"=5 then \'Reseller\' end as groupname,pt."profileType" as childname,'
                + 'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" and pt."profileTypeId" in(:ptype) and ptc."channelTypeId" in(:typeId) '
                + 'and  ptc."profileTypeClassificationId" in(:clssifiid) ';
            profileList = await this.sequelizeInstance.query(query, {
                type: sequelize_1.QueryTypes.SELECT,
                replacements: {
                    typeId: typeid,
                    clssifiid: clssifiid,
                    ptype: pro,
                }
            }).then(results => {
                return results;
            });
            console.log('profileList', profileList);
        }
        let profileType = '';
        if (result.length > 0 && result[0].profileTypeId) {
            let query = 'select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
                + 'when ptc."channelTypeId"=2 then \'vendor\' when ptc."channelTypeId"=5 then \'Reseller\' end as groupname,pt."profileType" as childname,'
                + 'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" where ptc."channelTypeId" = :typeId and ptc."profileTypeId"=:profileTypeId';
            profileType = await this.sequelizeInstance.query(query, {
                type: sequelize_1.QueryTypes.SELECT,
                replacements: {
                    typeId: req.user.channelTypeId,
                    profileTypeId: result[0].profileTypeId
                }
            }).then(results => {
                return results;
            });
        }
        let finalData = {
            profile: profileList,
            profileTypeId: result > 0 ? result[0].profileTypeId : null,
            profileType: profileType,
        };
        return res.status(common_1.HttpStatus.OK).send(finalData);
    }
    async update_channel_profile(req, res, data) {
        let user = req.user;
        let result = await this.channelSettingService.save_channel_info(data, {}, user);
        if (result)
            return res.status(common_1.HttpStatus.OK).send('OK');
        else
            return res.status(common_1.HttpStatus.BAD_REQUEST);
    }
    async getChannelAwards(req, res) {
        let user = req.user;
        let result = await this.channelSettingService.getChannelAwards(user);
        res.status(common_1.HttpStatus.OK).send(result);
    }
    async getProductAwards(req, res) {
        console.log('---');
        let result = await this.channelSettingService.getProductAwards(req.user);
        res.status(common_1.HttpStatus.OK).send(result);
    }
    async getChannelAwardsAdmin(channelId, req, res) {
        let user = {
            channelId: channelId
        };
        let result = await this.channelSettingService.getChannelAwards(user);
        res.status(common_1.HttpStatus.OK).send(result);
    }
    async getProductAwardsAdmin(channelId, req, res) {
        let user = {
            channelId: channelId
        };
        let result = await this.channelSettingService.getProductAwards(user);
        res.status(common_1.HttpStatus.OK).send(result);
    }
    async deleteChannelAwards(req, res, data) {
        let user = req.user;
        let result = await this.channelSettingService.deleteChannelAwards(data);
        res.status(common_1.HttpStatus.OK).send(result);
    }
    async deleteProductAwards(req, res, data) {
        console.log('---');
        let result = await this.channelSettingService.deleteProductAwards(data);
        res.status(common_1.HttpStatus.OK).send(result);
    }
};
__decorate([
    common_1.Get('getInitialProfile'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getInitialProfile", null);
__decorate([
    common_1.Get('getVat/:countryId'),
    __param(0, common_1.Param('countryId')), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getVat", null);
__decorate([
    common_1.Post('checkVat'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "checkVat", null);
__decorate([
    common_1.Get('getContactProduct/:contactId'),
    __param(0, common_1.Param('contactId')), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getContactProduct", null);
__decorate([
    common_1.Get('getProductProfile/:companyId'),
    __param(0, common_1.Param('companyId')), __param(1, common_1.Req()), __param(2, common_1.Param()), __param(3, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getProductProfile", null);
__decorate([
    common_1.Get('getUserProfile/:companyId'),
    __param(0, common_1.Param('companyId')), __param(1, common_1.Req()), __param(2, common_1.Param()), __param(3, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getUserProfile", null);
__decorate([
    common_1.Post('company'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [setting_dto_1.SettingDto, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "save_Company_Profile", null);
__decorate([
    common_1.Post('userprofile'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [usersetting_dto_1.UserSettingDto, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "save_User_Profile", null);
__decorate([
    common_1.Get('profileList/:companyId/:typeId'),
    __param(0, common_1.Param('companyId')), __param(1, common_1.Param('typeId')), __param(2, common_1.Req()), __param(3, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getProfileList", null);
__decorate([
    common_1.Get('checkUserStatus/:companyId'),
    __param(0, common_1.Param('companyId')), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getUserStatus", null);
__decorate([
    common_1.Get('getCompanyMediaLibrary/:companyId'),
    __param(0, common_1.Param('companyId')), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getCompanyMediaLibrary", null);
__decorate([
    common_1.Get('getOldCompanyMediaLibrary/:companyId'),
    __param(0, common_1.Param('companyId')), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getOldCompanyMediaLibrary", null);
__decorate([
    common_1.Post('createCompanyLogo'),
    __param(0, common_1.Body()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "createCompanyLogo", null);
__decorate([
    common_1.Post('channelBrandCheck'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "existingBrandCheck", null);
__decorate([
    common_1.Get('company'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "company", null);
__decorate([
    common_1.Get('tradeinfo'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "tradeinfo", null);
__decorate([
    common_1.Get('getChannelProduct'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getChannelProduct", null);
__decorate([
    common_1.Get('getChannelBrand'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getChannelBrand", null);
__decorate([
    common_1.Get('admin/getChannelBrand/:id'),
    __param(0, common_1.Param('id')), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getAdminChannelBrand", null);
__decorate([
    common_1.Get('getChannelDetail'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getChannelDetail", null);
__decorate([
    common_1.Get('getCertificateTag'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getChannelCertificateTags", null);
__decorate([
    common_1.Get('tradelocation'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "tradelocation", null);
__decorate([
    common_1.Put('updateChannelLocation'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [channelLocation_dto_1.ChannelLocationDto, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "updateChannelLocation1", null);
__decorate([
    common_1.Get('getuser'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getUsers", null);
__decorate([
    common_1.Delete('deleteUser/:contactId'),
    __param(0, common_1.Param('contactId')), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "deleteUser", null);
__decorate([
    common_1.Get('getuserLimits'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getuserLimits", null);
__decorate([
    common_1.Post('updateCompany'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "update_Company_Profile", null);
__decorate([
    common_1.Post('editBasicCompany'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "editBasicCompanyProfile", null);
__decorate([
    common_1.Post('updateBannerImage'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "updateBannerImage", null);
__decorate([
    common_1.Post('updateTradeInfo'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "update_Trade_Info", null);
__decorate([
    common_1.Post('updateFacts'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "updateFacts", null);
__decorate([
    common_1.Get('getFacts'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getFacts", null);
__decorate([
    common_1.Get('reporting'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "reporting", null);
__decorate([
    common_1.Post('generalInfo'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "generalInfo", null);
__decorate([
    common_1.Get('getContactLocation/:contactId'),
    __param(0, common_1.Param('contactId')), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getContactLocation", null);
__decorate([
    common_1.Get('getpaymentoption'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getPaymentOption", null);
__decorate([
    common_1.Get('currency'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getCurrency", null);
__decorate([
    common_1.Get('channelKeyPartnerCheck'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Query('KeyRetailer')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "existingKeyRetailerCheck", null);
__decorate([
    common_1.Post('settings'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body(new common_1.ValidationPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, settings_dto_1.SettingsDto]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "updateSettings", null);
__decorate([
    common_1.Get('settings'),
    __param(0, common_1.Res()), __param(1, common_1.Req()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "settings", null);
__decorate([
    common_1.Post('searchRegion'),
    __param(0, common_1.Body(new common_1.ValidationPipe())), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [searchRegion_dto_1.SearchRegionDto, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "searchRegion", null);
__decorate([
    common_1.Get('profileAndSpeciality'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getProfileAndSpeciality", null);
__decorate([
    common_1.Post('editProfileSave'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "editProfile", null);
__decorate([
    common_1.Get('keyPartnerOptions'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getKeyPartnerOptions", null);
__decorate([
    common_1.Post('update_company_profile'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "update_company_profile", null);
__decorate([
    common_1.Get('getChannelProfile'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getChannelProfile", null);
__decorate([
    common_1.Post('updateChannel'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "update_channel_profile", null);
__decorate([
    common_1.Get('getChannelAwards'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getChannelAwards", null);
__decorate([
    common_1.Get('getProductAwards'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getProductAwards", null);
__decorate([
    common_1.Get('getChannelAwardsAdmin/:channelId'),
    __param(0, common_1.Param('channelId')), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getChannelAwardsAdmin", null);
__decorate([
    common_1.Get('getProductAwardsAdmin/:channelId'),
    __param(0, common_1.Param('channelId')), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "getProductAwardsAdmin", null);
__decorate([
    common_1.Post('deleteChannelAwards'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "deleteChannelAwards", null);
__decorate([
    common_1.Post('deleteProductAwards'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ChannelSettingController.prototype, "deleteProductAwards", null);
ChannelSettingController = __decorate([
    common_1.UseGuards(passport_1.AuthGuard('jwt')),
    common_1.Controller('setting'),
    __param(1, common_1.Inject('SequelizeInstance')),
    __metadata("design:paramtypes", [channel_setting_service_1.ChannelSettingService, Object, context_service_1.ContextService])
], ChannelSettingController);
exports.ChannelSettingController = ChannelSettingController;
//# sourceMappingURL=channel.settings.controller.js.map