"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SubscriptionController = void 0;
const common_1 = require("@nestjs/common");
const passport_1 = require("@nestjs/passport");
const Process_exception_1 = require("../../../middleware/exception/Process.exception");
const subscription_service_1 = require("./../service/subscription.service");
const context_service_1 = require("../../../context/context.service");
const validation_pipe_1 = require("../../../middleware/pipe/validation.pipe");
const subscribe_dto_1 = require("./../dto/subscribe.dto");
const coupon_dto_1 = require("./../dto/coupon.dto");
const app_constant_1 = require("../../../config/app.constant");
const Subscription_entity_1 = require("../models/Subscription.entity");
const BaseHelper_1 = require("../../../core/helper/BaseHelper");
const Auth_exception_1 = require("./../../../middleware/exception/Auth.exception");
const PaymentTransaction_entity_1 = require("../models/PaymentTransaction.entity");
const lodash_1 = require("lodash");
const Stripe_1 = require("../../../lib/Stripe");
const microservices_1 = require("@nestjs/microservices");
const user_service_1 = require("../../user/service/user.service");
let SubscriptionController = class SubscriptionController {
    constructor(context, subscriptionService, userService, algorithmService, profileRepository) {
        this.context = context;
        this.subscriptionService = subscriptionService;
        this.userService = userService;
        this.algorithmService = algorithmService;
        this.profileRepository = profileRepository;
    }
    async subscriptionPlanAllPeriod(channelTypeId, req, res) {
        let subscriptionPlanAllPeriod = await this.subscriptionService.subscriptionPlanAllPeriod(channelTypeId);
        return res.status(common_1.HttpStatus.OK).send(subscriptionPlanAllPeriod);
    }
    async channelAlgm(req) {
        let user = req.user;
        return this.algorithmService.send({
            cmd: 'channel-suggestion-algm'
        }, JSON.stringify(user));
    }
    async offlineRequest(res, req) {
        let user = req.user;
        let offlineRequest = await this.subscriptionService.isOfflineRequestInProcess(user);
        if (offlineRequest) {
            throw new Process_exception_1.ProcessException('Wire transfer request in process. please contact support team');
        }
        return res.status(common_1.HttpStatus.OK).send();
    }
    async subscribe_amt_evaluvate(req, res, subscribeDto) {
        var _a, _b;
        var parser = require('ua-parser-js');
        let user = req.user;
        let offlineRequest = await this.subscriptionService.isOfflineRequestInProcess(user);
        console.log('offlineRequest');
        console.log(offlineRequest.rows);
        if (offlineRequest.count > 0) {
            throw new Process_exception_1.ProcessException('Wire transfer request in process. please contact support team ');
        }
        if (lodash_1.isUndefined(subscribeDto.paymentMethodId)) {
            subscribeDto.paymentMethodId = app_constant_1.appconstant.PAYMENT_METHOD.ONLINE_CREDIT_DEBIT_CART;
        }
        var subscriptionType = await this.subscriptionService.getSubscriptionType(user, +subscribeDto.subscriptionId);
        var subscriptionPlanDetails = await this.subscriptionService.getSubscriptionDetails(+subscribeDto.subscriptionId);
        let amountObj = await this.subscriptionService.calculateAmount(subscribeDto, subscriptionType, user);
        let amount = +amountObj.finalAmount;
        return res.status(common_1.HttpStatus.OK).json({
            amount: +(amount * 100).toFixed(2),
            original_amount: +(amountObj.totalAmount * 100).toFixed(2),
            coupon_discount: +(amountObj.couponDiscount * 100).toFixed(2),
            remainingAmount: +(amountObj.remainingAmount * 100).toFixed(2),
            taxAmount: +(amountObj.taxAmount * 100).toFixed(2),
            paymentMethodId: subscribeDto.paymentMethodId,
            period: subscriptionPlanDetails[0].subscriptionPeriod.periodDesc,
            plan: subscriptionPlanDetails[0].subscriptionPlan.subscriptionPlanType,
            periodId: (_b = (_a = subscriptionPlanDetails[0]) === null || _a === void 0 ? void 0 : _a.subscriptionPeriod) === null || _b === void 0 ? void 0 : _b.subscriptionPeriodId,
            planId: subscriptionPlanDetails[0].subscriptionPlan.subscriptionPlanId,
            vat: app_constant_1.appconstant.VAT_PERCENTAGE,
        });
    }
    async coupanValidation(req, res, coupanDto) {
        let user = req.user;
        var subscriptionType = await this.subscriptionService.getSubscriptionType(user, +coupanDto.subscriptionId);
        let finalAmt = await this.subscriptionService.coupanValidate(coupanDto, user, subscriptionType);
        return res.status(common_1.HttpStatus.OK).json({ finalAmt: finalAmt });
    }
    async subscribe(req, res, subscribeDto) {
        let user = req.user;
        let offlineRequest = await this.subscriptionService.isOfflineRequestInProcess(user);
        if (offlineRequest.count > 0) {
            throw new Process_exception_1.ProcessException('Wire transfer request in process. please contact support team');
        }
        if (lodash_1.isUndefined(subscribeDto.paymentMethodId)) {
            subscribeDto.paymentMethodId = app_constant_1.appconstant.PAYMENT_METHOD.ONLINE_CREDIT_DEBIT_CART;
        }
        if (+user.channel.signUpStatusId != +app_constant_1.appconstant.CHANNEL_APPROVED) {
            throw new Auth_exception_1.AuthException('Not allowed');
        }
        var subscriptionType = await this.subscriptionService.getSubscriptionType(user, +subscribeDto.subscriptionId);
        var subscriptiondetails = await this.subscriptionService.getSubscriptiondetails(subscribeDto.subscriptionId);
        console.log('subscriptionType', subscriptionType);
        console.log('subscriptiondetails', subscriptiondetails.subscriptionPlanId);
        console.log('SUBSCRIPTION_TYPE_NEW', Subscription_entity_1.Subscription.SUBSCRIPTION_TYPE_NEW);
        console.log('planId', user.channel.planId);
        let amountObj = await this.subscriptionService.calculateAmount(subscribeDto, subscriptionType, user);
        let amount = +amountObj.finalAmount;
        let transactionId = null;
        let paymentIntentSecret = null;
        let transaction = null;
        transaction = await this.subscriptionService.getTransactionId(amount, PaymentTransaction_entity_1.PaymentTransaction.PAYMENT_TYPE_SUBSCRIPTION, app_constant_1.appconstant.CHANNELHUB_CHANNEL_ID, user.channel.channelId, user.userId, subscribeDto);
        let channelSubscription = null;
        if (transaction) {
            transactionId = transaction.paymentTransactionId;
            if ((subscribeDto.paymentMethodId == app_constant_1.appconstant.PAYMENT_METHOD.OFFLINE_WIRE_TRANSFER) || subscribeDto.paymentMethodId == app_constant_1.appconstant.PAYMENT_METHOD.ONLINE_CREDIT_DEBIT_CART || subscribeDto.paymentMethodId == app_constant_1.appconstant.PAYMENT_METHOD.STRIPE_ALIPAY || subscribeDto.paymentMethodId == app_constant_1.appconstant.PAYMENT_METHOD.PAYPAL_TRANSFER) {
                channelSubscription = await this.subscriptionService.createChannelSubscription(user, subscribeDto.subscriptionId, subscribeDto.couponCode ? subscribeDto.couponCode : null, transaction.paymentTransactionId, subscribeDto.paymentStatus, subscriptionType, transaction, transaction.cardLastFourDigit);
            }
            let rating = await this.profileRepository.findOne({
                where: {
                    channelId: user.channelId,
                    deletedAt: null,
                    ratingId: 5,
                }
            });
            if (!rating) {
                await this.profileRepository.create({
                    channelId: user.channelId,
                    ratingId: 5,
                    isCompleted: true,
                    createdAt: new Date()
                });
            }
            return res.status(common_1.HttpStatus.OK).json({
                transactionId: transactionId,
                amount: +(amount * 100).toFixed(2),
                original_amount: +(amountObj.totalAmount * 100).toFixed(2),
                coupon_discount: +(amountObj.couponDiscount * 100).toFixed(2),
                remainingAmount: +(amountObj.remainingAmount * 100).toFixed(2),
                taxAmount: +(amountObj.taxAmount * 100).toFixed(2),
                paymentMethodId: subscribeDto.paymentMethodId,
                channelSubscription: channelSubscription,
                planId: +subscriptiondetails.subscriptionPlanId
            });
        }
        else {
            throw new Process_exception_1.ProcessException('Something went wrong');
        }
    }
    async PaymentIntend(req, res) {
        let user = req.user;
        let stripe = new Stripe_1.StripeV2();
        let paymentIntentSecret = await stripe.createPaymentIntend(+(req.body.amount * 100).toFixed(2), app_constant_1.appconstant.CHANNELHUB_CURRENCY_CODE, 'subscription', +user.channelId);
        console.log(paymentIntentSecret.client_secret);
        return res.status(common_1.HttpStatus.OK).json({
            paymentIntentSecret: paymentIntentSecret.client_secret
        });
    }
    async stipeCreatecharge(req, res) {
        let stripe = new Stripe_1.StripeV2();
        var resource = req.body;
        let amount = resource.amount;
        let currency = resource.currency;
        let sourceId = resource.sourceId;
        console.log(sourceId);
        const charge = await stripe.createchargesForSource(amount, currency, sourceId);
        if (resource.channel_transId !== null) {
            let transaction = await this.subscriptionService.UpdateTransactionById(resource.channel_transId, sourceId);
        }
        return res.status(common_1.HttpStatus.OK).json({ charge });
    }
    async addKeyFeature(req, res) {
        let bodyCnt = req.body;
        let result = await this.subscriptionService.insertKeyFeature(bodyCnt.categories, bodyCnt.subscriptionId);
        return res.status(common_1.HttpStatus.OK).json({ result });
    }
    async getKeyFeatures(channelTypeId, subscriptionPeriodId, req, res) {
        let result = await this.subscriptionService.getAllKeyFeatures(channelTypeId, subscriptionPeriodId);
        return res.status(common_1.HttpStatus.OK).json({ result });
    }
    async updateKeyFeature(channelId, subscriptionId, req, res) {
        let result = await this.subscriptionService.updateKeyFeatureForchannel(channelId, subscriptionId);
        return res.status(common_1.HttpStatus.OK).json({ result });
    }
    async getPlansforAdmin(req, res) {
        let result = await this.subscriptionService.getPlansforAdmin();
        return res.status(common_1.HttpStatus.OK).json({ result });
    }
    async getPlansDetailsById(req, res) {
        let subscriptionId = req.body.subscriptionId;
        let result = await this.subscriptionService.getPlansDetailsById(subscriptionId);
        return res.status(common_1.HttpStatus.OK).json({ result });
    }
    async getPlansDetails(req, res) {
        console.log('channelId for key feature');
        console.log(req.user.channel);
        let subscriptionId = req.user.channel.channelSubscription.subscriptionId;
        let result = await this.subscriptionService.getFeaterListId(req.user.channelId);
        let category = await this.subscriptionService.getFeaterCategoryList(result.keyFeature);
        return res.status(common_1.HttpStatus.OK).json({ category });
    }
    async getPlansDetailsBo(req, res, data) {
        let channelId = data.channelId;
        let userId = data.userId;
        let user = await this.userService.findUserById(userId, channelId);
        let subscriptionId = user.channel.channelSubscription.subscriptionId;
        let result = await this.subscriptionService.getFeaterListId(user.channelId);
        let category = await this.subscriptionService.getFeaterCategoryList(result.keyFeature);
        return res.status(common_1.HttpStatus.OK).json({ category });
    }
    async updateSubscription(req, res, subscriptionId) {
        let data = req.body;
        let result = await this.subscriptionService.updateSubscription(data, subscriptionId);
        return res.status(common_1.HttpStatus.OK).json({ result });
    }
    async failedTransaction(req, res) {
        let data = req.body;
        let result = await this.subscriptionService.failedTransaction(data);
        return res.status(common_1.HttpStatus.OK).json({ result });
    }
    async getFeaterList(req, res, channelId) {
        let result = await this.subscriptionService.getFeaterListId(channelId);
        let category = null;
        if (result) {
            category = await this.subscriptionService.getFeaterCategoryList(result.keyFeature);
        }
        return res.status(common_1.HttpStatus.OK).json({ category });
    }
    async updateChannelFeatureList(req, res) {
        let channelId = req.body.channelId;
        let extraFeature = req.body.extraFeature;
        let result = await this.subscriptionService.updateChannelFeatureList(extraFeature, channelId);
        return res.status(common_1.HttpStatus.OK).json({ result });
    }
};
__decorate([
    common_1.Get('subscriptionPlanAllPeriod/:channelTypeId'),
    __param(0, common_1.Param('channelTypeId')), __param(1, common_1.Request()), __param(2, common_1.Response()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], SubscriptionController.prototype, "subscriptionPlanAllPeriod", null);
__decorate([
    common_1.Get('channel_suggestion'),
    __param(0, common_1.Req()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], SubscriptionController.prototype, "channelAlgm", null);
__decorate([
    common_1.Get('channel/isOfflineRequestInProcess'),
    __param(0, common_1.Response()), __param(1, common_1.Req()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], SubscriptionController.prototype, "offlineRequest", null);
__decorate([
    common_1.Post('subscribe_amt_evaluvate'),
    __param(0, common_1.Request()), __param(1, common_1.Response()), __param(2, common_1.Body(new validation_pipe_1.ValidationPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, subscribe_dto_1.SubscribeDto]),
    __metadata("design:returntype", Promise)
], SubscriptionController.prototype, "subscribe_amt_evaluvate", null);
__decorate([
    common_1.Post('coupan_validate'),
    __param(0, common_1.Request()), __param(1, common_1.Response()), __param(2, common_1.Body(new validation_pipe_1.ValidationPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, coupon_dto_1.CouponDto]),
    __metadata("design:returntype", Promise)
], SubscriptionController.prototype, "coupanValidation", null);
__decorate([
    common_1.Post('subscribe'),
    __param(0, common_1.Request()), __param(1, common_1.Response()), __param(2, common_1.Body(new validation_pipe_1.ValidationPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, subscribe_dto_1.SubscribeDto]),
    __metadata("design:returntype", Promise)
], SubscriptionController.prototype, "subscribe", null);
__decorate([
    common_1.Post('createPaymentIntend'),
    __param(0, common_1.Request()), __param(1, common_1.Response()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], SubscriptionController.prototype, "PaymentIntend", null);
__decorate([
    common_1.Post('stipeCreatecharge'),
    __param(0, common_1.Request()), __param(1, common_1.Response()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], SubscriptionController.prototype, "stipeCreatecharge", null);
__decorate([
    common_1.Post('addKeyFeature'),
    __param(0, common_1.Request()), __param(1, common_1.Response()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], SubscriptionController.prototype, "addKeyFeature", null);
__decorate([
    common_1.Get('getAllKeyFeatures/:channelTypeId/:subscriptionPeriodId'),
    __param(0, common_1.Param('channelTypeId')), __param(1, common_1.Param('subscriptionPeriodId')), __param(2, common_1.Request()), __param(3, common_1.Response()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object, Object]),
    __metadata("design:returntype", Promise)
], SubscriptionController.prototype, "getKeyFeatures", null);
__decorate([
    common_1.Get('updateKeyFeature/:channelId/:subscriptionId'),
    __param(0, common_1.Param('channelId')), __param(1, common_1.Param('subscriptionId')), __param(2, common_1.Request()), __param(3, common_1.Response()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object, Object]),
    __metadata("design:returntype", Promise)
], SubscriptionController.prototype, "updateKeyFeature", null);
__decorate([
    common_1.Get('getPlansforAdmin'),
    __param(0, common_1.Request()), __param(1, common_1.Response()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], SubscriptionController.prototype, "getPlansforAdmin", null);
__decorate([
    common_1.Post('getPlansDetailsById'),
    __param(0, common_1.Request()), __param(1, common_1.Response()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], SubscriptionController.prototype, "getPlansDetailsById", null);
__decorate([
    common_1.Get('getPlansDetails'),
    __param(0, common_1.Request()), __param(1, common_1.Response()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], SubscriptionController.prototype, "getPlansDetails", null);
__decorate([
    common_1.Post('getPlansDetailsBo'),
    __param(0, common_1.Request()), __param(1, common_1.Response()), __param(2, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], SubscriptionController.prototype, "getPlansDetailsBo", null);
__decorate([
    common_1.Post('updateSubscription/:subscriptionId'),
    __param(0, common_1.Request()), __param(1, common_1.Response()), __param(2, common_1.Param('subscriptionId')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], SubscriptionController.prototype, "updateSubscription", null);
__decorate([
    common_1.Post('failedTransaction'),
    __param(0, common_1.Request()), __param(1, common_1.Response()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], SubscriptionController.prototype, "failedTransaction", null);
__decorate([
    common_1.Get('getFeaterList/:channelId'),
    __param(0, common_1.Request()), __param(1, common_1.Response()), __param(2, common_1.Param('channelId')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], SubscriptionController.prototype, "getFeaterList", null);
__decorate([
    common_1.Post('updateChannelFeatureList'),
    __param(0, common_1.Request()), __param(1, common_1.Response()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], SubscriptionController.prototype, "updateChannelFeatureList", null);
SubscriptionController = __decorate([
    common_1.UseGuards(passport_1.AuthGuard('jwt')),
    common_1.Controller('channel/subscription'),
    __param(3, common_1.Inject('ALGORITHM_SERVICE')),
    __param(4, common_1.Inject('ProfileRepository')),
    __metadata("design:paramtypes", [context_service_1.ContextService,
        subscription_service_1.SubscriptionService,
        user_service_1.UserService,
        microservices_1.ClientProxy, Object])
], SubscriptionController);
exports.SubscriptionController = SubscriptionController;
//# sourceMappingURL=subscription.controller.js.map