"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChannelHubService = void 0;
const common_1 = require("@nestjs/common");
const lodash_1 = require("lodash");
const app_constant_1 = require("../../../config/app.constant");
const esearch_1 = require("../../../config/esearch");
const Auth_exception_1 = require("../../../middleware/exception/Auth.exception");
const Upgrade_exception_1 = require("../../../middleware/exception/Upgrade.exception");
const Channel_entity_1 = require("../models/Channel.entity");
const ChannelJCT_entity_1 = require("../models/ChannelJCT.entity");
const sequelize_1 = require("sequelize");
const Process_exception_1 = require("../../../middleware/exception/Process.exception");
const profileScore_1 = require("../../../config/profileScore");
const Brand_entity_1 = require("../../product/models/Brand.entity");
const Media_entity_1 = require("../../media/models/Media.entity");
let ChannelHubService = class ChannelHubService {
    constructor(sequelizeInstance, channelRepository, channelJCTRepository, channelDismissReasonRepository, dismissReasonRepository, brandRepository, channelBrandRepository, mediaRepository, buyerRepository, BuyerCategoryRepository) {
        this.sequelizeInstance = sequelizeInstance;
        this.channelRepository = channelRepository;
        this.channelJCTRepository = channelJCTRepository;
        this.channelDismissReasonRepository = channelDismissReasonRepository;
        this.dismissReasonRepository = dismissReasonRepository;
        this.brandRepository = brandRepository;
        this.channelBrandRepository = channelBrandRepository;
        this.mediaRepository = mediaRepository;
        this.buyerRepository = buyerRepository;
        this.BuyerCategoryRepository = BuyerCategoryRepository;
    }
    async connectChannel(channel, user, skipCreate) {
        let allowedSuggestionTypes = esearch_1.esearch.outBoundRequest[+user.channelTypeId];
        let validateChannelType = false;
        allowedSuggestionTypes.forEach((data) => {
            if (+data.type_id == +channel.channelTypeId) {
                validateChannelType = true;
            }
        });
        if (!validateChannelType) {
            throw new Auth_exception_1.AuthException('Not allowed');
        }
        let query = 'select tcs.suggestionfor, tcs.channelsuggestionsid, tcs.suggestioncount, tcs.isfulfilled, tcs.limit, tc."channelSubscriptionId", tc."channelId", to_date( to_char(now(),\'dd/mm/yyyy\'),\'dd/mm/yyyy\') >= to_date( to_char(tc."endAt",\'dd/mm/yyyy\'),\'dd/mm/yyyy\') as expired from "tblChannelSubscription" tc left outer join "tblchannelsuggestions" tcs on tcs."channelsubscriptionid" = tc."channelSubscriptionId" and tcs."suggestionfor" = :suggestionType where tc."channelId" = :channelId and tc."isActive"=\'1\' order by "channelsuggestionsid" desc limit 1';
        let validateSuggestion = await this.sequelizeInstance.query(query, {
            type: this.sequelizeInstance.QueryTypes.SELECT,
            raw: true,
            replacements: {
                suggestionType: app_constant_1.appconstant.CHANNELTYPES[+channel.channelTypeId] + '_OTB',
                channelId: +user.channelId
            }
        }).catch((e) => {
            console.log(e);
        });
        if ((!lodash_1.isUndefined(validateSuggestion.expired) && lodash_1.isBoolean(validateSuggestion.expired) && !validateSuggestion.expired && validateSuggestion.channelsuggestionsid && +validateSuggestion.isfulfilled == 0 && (validateSuggestion.limit == -1 || +validateSuggestion.suggestioncount < +validateSuggestion.limit))
            || (!lodash_1.isUndefined(validateSuggestion.isexpired) && validateSuggestion.isexpired == '0' && validateSuggestion.channelsuggestionsid && +validateSuggestion.isfulfilled == 0 && (validateSuggestion.limit == -1 || +validateSuggestion.suggestioncount < +validateSuggestion.limit)) || (channel.channelTypeId != 2 || user.channelTypeId != 4)) {
            return await this.sequelizeInstance.transaction(async (transaction) => {
                let data = null;
                if (lodash_1.isUndefined(skipCreate)) {
                    data = await ChannelJCT_entity_1.ChannelJCT.bulkCreate([{
                            channelId: +user.channelId,
                            partnerChannelId: +channel.channelId,
                            connectionStatusTypeId: +app_constant_1.appconstant.MYSUGGESTION
                        }], {
                        returning: true, transaction
                    }).catch(async () => {
                    });
                }
                return data;
            }).then((data) => {
                return (data && data[0]) ? data[0] : null;
            });
        }
        else {
            throw new Upgrade_exception_1.UpgradeException('Upgrade you plan to make this action');
        }
    }
    async verifyChannelSuggestion(user, updateChannelDto) {
        let result = await this.channelJCTRepository.findOne({
            where: {
                [sequelize_1.Op.or]: [
                    { partnerChannelId: user.channelId },
                    { channelId: user.channelId },
                ],
                channelJCTId: updateChannelDto.channelJCTId,
                deletedat: null
            },
            include: [
                {
                    model: this.channelRepository,
                    as: 'partnerChannel',
                    where: { deletedAt: null },
                },
                {
                    model: this.channelRepository,
                    as: 'Channel',
                    where: { deletedAt: null },
                },
            ]
        });
        if (!result)
            throw new Process_exception_1.ProcessException('channelHub:updateChannel:channelHubNotFound');
        let partnerChannelTypeId = null;
        if (result.partnerChannel.channelId == user.channelId) {
            partnerChannelTypeId = result.Channel.channelTypeId;
        }
        else {
            partnerChannelTypeId = result.partnerChannel.channelTypeId;
        }
        console.log(partnerChannelTypeId);
        let suggtype = null;
        if (partnerChannelTypeId == app_constant_1.appconstant.CHANNEL_TYPE_DISTRIBUTOR) {
            suggtype = 'DISTRIBUTOR_OTB';
        }
        else if (partnerChannelTypeId == app_constant_1.appconstant.CHANNEL_TYPE_RETAILER) {
            suggtype = 'RETAILER_OTB';
        }
        else {
            suggtype = 'VENDOR_OTB';
        }
        if (user.channelTypeId == app_constant_1.appconstant.CHANNEL_TYPE_VENDOR && (partnerChannelTypeId == app_constant_1.appconstant.CHANNEL_TYPE_DISTRIBUTOR || partnerChannelTypeId == app_constant_1.appconstant.CHANNEL_TYPE_RETAILER)) {
            let query = 'select tcs.suggestioncount, tcs.limit, tcs.endat,tcs.suggestionfor from "tblChannelSubscription" tc left outer join "tblchannelsuggestions" tcs on tcs."channelsubscriptionid" = tc."channelSubscriptionId" where ( tcs."suggestionfor"  = :type OR tcs."suggestionfor"  = :type1 ) AND  tcs.isexpired = :expired AND  tcs.endat >= NOW() and tc."channelId" = ' + user.channelId + '  and tc."isActive"=:isactive ';
            return await this.sequelizeInstance.query(query, {
                type: sequelize_1.QueryTypes.SELECT,
                replacements: {
                    type: 'DISTRIBUTOR_OTB',
                    type1: 'RETAILER_OTB',
                    expired: '0',
                    isactive: '1'
                }
            }).then((queryresult) => {
                var totalcnt = 0;
                queryresult.forEach((sdata) => {
                    if (suggtype == sdata.suggestionfor && sdata.limit == -1) {
                        queryresult['limit'] = 1;
                        queryresult['allowed'] = sdata.limit;
                        return queryresult;
                    }
                    else {
                        totalcnt = totalcnt + parseInt(sdata.suggestioncount);
                        queryresult['limit'] = +totalcnt;
                    }
                    if (suggtype.trim() == sdata.suggestionfor.trim() && sdata.limit != -1) {
                        queryresult['allowed'] = +sdata.limit;
                    }
                });
                return queryresult;
            });
        }
        else if (user.channelTypeId == app_constant_1.appconstant.CHANNEL_TYPE_DISTRIBUTOR && (partnerChannelTypeId == app_constant_1.appconstant.CHANNEL_TYPE_VENDOR || partnerChannelTypeId == app_constant_1.appconstant.CHANNEL_TYPE_RETAILER)) {
            let query = 'select tcs.suggestioncount, tcs.limit, tcs.endat,tcs.suggestionfor  from "tblChannelSubscription" tc left outer join "tblchannelsuggestions" tcs on tcs."channelsubscriptionid" = tc."channelSubscriptionId"  where tcs."suggestionfor" = :type AND   tcs.isexpired = :expired AND  tcs.endat >= NOW() and tc."channelId" = ' + user.channelId + ' and tc."isActive"=:isactive ';
            return await this.sequelizeInstance.query(query, {
                type: sequelize_1.QueryTypes.SELECT,
                replacements: {
                    type: 'RETAILER_OTB',
                    expired: '0',
                    isactive: '1'
                }
            }).then((queryresult) => {
                var totalcnt = 0;
                queryresult.forEach((sdata) => {
                    if (suggtype == sdata.suggestionfor && sdata.limit == -1) {
                        queryresult['limit'] = 1;
                        queryresult['allowed'] = sdata.limit;
                        return queryresult;
                    }
                    else {
                        totalcnt = totalcnt + parseInt(sdata.suggestioncount);
                        queryresult['limit'] = +totalcnt;
                    }
                    if (suggtype.trim() == sdata.suggestionfor.trim() && sdata.limit != -1) {
                        queryresult['allowed'] = +sdata.limit;
                    }
                });
                return queryresult;
            });
        }
        else if (user.channelTypeId == app_constant_1.appconstant.CHANNEL_TYPE_RETAILER) {
            let queryresult = {};
            queryresult['limit'] = 1;
            queryresult['type'] = app_constant_1.appconstant.CHANNEL_TYPE_RETAILER;
            queryresult['allowed'] = 2;
            return queryresult;
        }
        else {
            throw new Auth_exception_1.AuthException('Not allowed vendor');
        }
    }
    async updateChannelConnect(channelId, updateChannelDto, user, suggest_Channel) {
        let result = await this.channelJCTRepository.findOne({
            where: {
                channelJCTId: updateChannelDto.channelJCTId,
                deletedat: null
            },
            include: [
                {
                    model: this.channelRepository,
                    as: 'partnerChannel',
                    where: { deletedAt: null },
                },
                {
                    model: this.channelRepository,
                    as: 'Channel',
                    where: { deletedAt: null },
                },
            ]
        });
        let beforeStatus = result.connectionStatusTypeId;
        let query = 'UPDATE "tblChannelJCT" SET "channelId" = ' + user.channelId + ', "partnerChannelId" = ' + channelId + ', "connectionStatusTypeId" = ' + updateChannelDto.connectionStatusTypeId + ', "updatedAt" = now(), "requested_by" = ' + user.contactId + ', "requested_to" = ' + updateChannelDto.contactid + ', "request_sent_at" = now() WHERE "channelJCTId" = ' + updateChannelDto.channelJCTId;
        await this.sequelizeInstance.query(query, {
            type: sequelize_1.QueryTypes.SELECT
        }).then(async (res) => {
            if (+updateChannelDto.connectionStatusTypeId == app_constant_1.appconstant.REQUEST_INTRO || +updateChannelDto.connectionStatusTypeId == app_constant_1.appconstant.PENDINGMYPROSPECT || +updateChannelDto.connectionStatusTypeId == app_constant_1.appconstant.DISMISS) {
                let delQuery = 'DELETE FROM "tblChannelJCT" WHERE "requested_by" = ' + user.contactId + ' AND "requested_to" = ' + updateChannelDto.contactid + ' AND "connectionStatusTypeId" = 1';
                await this.sequelizeInstance.query(delQuery, {
                    type: sequelize_1.QueryTypes.DELETE
                });
            }
            if (+updateChannelDto.connectionStatusTypeId == app_constant_1.appconstant.DISMISS) {
                let otherReasons = (updateChannelDto.otherreasons) ? updateChannelDto.otherreasons : null;
                await this.channelDismissReasonRepository.create({
                    channelJCTId: result.channelJCTId,
                    dismissreasonid: updateChannelDto.dismissreasonid,
                    otherreasons: otherReasons,
                    updatedby: user.userId
                });
            }
            let updateObject = {
                subscription_queued: '0'
            };
            if (suggest_Channel && +updateChannelDto.connectionStatusTypeId == app_constant_1.appconstant.PENDINGMYPROSPECT) {
                updateObject['suggestion_connect_flag'] = '1';
            }
            if (updateChannelDto.leadManagementTag) {
                if (result.channelId == +user.channelId) {
                    updateObject['leadManagementTag'] = updateChannelDto.leadManagementTag;
                }
                else if (result.partnerChannelId == +user.channelId) {
                    updateObject['partnerLeadManagementTag'] = updateChannelDto.leadManagementTag;
                }
            }
            if (+updateChannelDto.connectionStatusTypeId == app_constant_1.appconstant.PENDINGMYPROSPECT || updateChannelDto.connectionStatusTypeId == app_constant_1.appconstant.REQUEST_INTRO) {
                updateObject['last_remainder_sent_at'] = this.sequelizeInstance.fn('NOW');
                updateObject['no_of_remainders_sent'] = '1';
                updateObject['introVideo'] = updateChannelDto.videoUrl;
            }
            await result.update(updateObject);
            return updateChannelDto;
        });
        return updateChannelDto;
    }
    async profileScore(channelId, channelTypeId, score) {
        var list;
        if (score && +score == 100) {
            return true;
        }
        if (channelTypeId == Channel_entity_1.Channel.CHANNEL_TYPE_VENDOR) {
            list = profileScore_1.profileScore.VENDOR_SCORE;
        }
        else if (channelTypeId == Channel_entity_1.Channel.CHANNEL_TYPE_DISTRIBUTOR) {
            list = profileScore_1.profileScore.DISTRIBUTOR_SCORE;
        }
        else if (channelTypeId == Channel_entity_1.Channel.CHANNEL_TYPE_RETAILER) {
            list = profileScore_1.profileScore.RETAILER_SCORE;
        }
        else {
            return false;
        }
        try {
            let result = await this.sequelizeInstance.query('select ch."channelId", ((CASE WHEN ch."companyName" IS NULL THEN 0 ELSE 1 END) * ' + list.channel.companyName + ') + ((CASE WHEN ch."regAddressId" IS NULL THEN 0 ELSE 1 END) * ' + list.channel.regAddressId + ') + ((CASE WHEN ch."countryId" IS NULL THEN 0 ELSE 1 END) * ' + list.channel.countryId + ') + ((CASE WHEN ch."companyMail" IS NULL THEN 0 ELSE 1 END) * ' + list.channel.companyMail + ') + ((CASE WHEN chd."webSiteUrl" IS NULL THEN 0 ELSE 1 END) * ' + list.detail.webSiteUrl + ') + ((CASE WHEN chd."phone1" IS NULL THEN 0 ELSE 1 END) * ' + list.detail.phone1 + ') +  ((CASE WHEN chd."channelLogoId" IS NULL THEN 0 ELSE 1 END) * ' + list.detail.channelLogoId + ') + ((CASE WHEN chd."channelIconId" IS NULL THEN 0 ELSE 1 END) * ' + list.detail.channelIconId + ') + ((CASE WHEN chd."estYear" IS NULL THEN 0 ELSE 1 END) * ' + list.detail.estYear + ') + ((CASE WHEN chd."regYear" IS NULL THEN 0 ELSE 1 END) * ' + list.detail.regYear + ') + ((CASE WHEN chd."detailDesc" IS NULL THEN 0 ELSE 1 END) * ' + list.detail.detailDesc + ') + ((CASE WHEN chd."empCnt" IS NULL THEN 0 ELSE 1 END) * ' + list.detail.empCnt + ') + ((CASE WHEN chd."turnover" IS NULL THEN 0 ELSE 1 END) * ' + list.detail.turnover + ') + ((CASE WHEN chd."stripeConnectId" IS NULL THEN 0 ELSE 1 END) * ' + list.detail.stripeConnectId + ') + ((CASE WHEN con."firstName" IS NULL THEN 0 ELSE 1 END) * ' + list.contact.firstName + ') + ((CASE WHEN con."lastName" IS NULL THEN 0 ELSE 1 END) * ' + list.contact.lastName + ') + ((CASE WHEN con."mobileNo" IS NULL THEN 0 ELSE 1 END) * ' + list.contact.mobileNo + ') + ((CASE WHEN con."email" IS NULL THEN 0 ELSE 1 END) * ' + list.contact.email + ') + ((CASE WHEN con."jobTitle" IS NULL THEN 0 ELSE 1 END) * ' + list.contact.jobTitle + ') + ((CASE WHEN con."documentId" IS NULL THEN 0 ELSE 1 END) * ' + list.contact.documentId + ') + ((CASE WHEN (select count(chcr."channelId")	from "tblChannelCerificate" chcr where chcr."channelId"=ch."channelId" and chcr."cerificateTypeId"=1 and chcr."deletedAt" is null ) = 0 THEN 0 ELSE 1 END) * ' + list.regCertificate + ') + ((CASE WHEN (select count(chcr."channelId") from "tblChannelCerificate" chcr where chcr."channelId"=ch."channelId" and chcr."cerificateTypeId"=2 and chcr."deletedAt" is null ) = 0 THEN 0 ELSE 1 END) * ' + list.taxCertificate + ') + ((CASE WHEN (select count(chcr."channelId") from "tblChannelAward" chcr where chcr."channelId"=chd."channelId" and chcr."deletedAt" is null ) = 0 THEN 0 ELSE 1 END) * ' + list.award + ') + ((CASE WHEN (select count(chcr."channelId") from "tblChannelSocialLink" chcr where chcr."channelId"=ch."channelId" and chcr."socialSiteId"=1 and chcr."deletedAt" is null ) = 0 THEN 0 ELSE 1 END) * ' + list.linkedin + ') + ((CASE WHEN (select count(chcr."channelId") from "tblChannelSocialLink" chcr where chcr."channelId"=ch."channelId" and chcr."socialSiteId"=2 and chcr."deletedAt" is null ) = 0 THEN 0 ELSE 1 END) * ' + list.googleplus + ') + ((CASE WHEN (select count(chcr."channelId") from "tblChannelSocialLink" chcr where chcr."channelId"=ch."channelId" and chcr."socialSiteId"=3 and chcr."deletedAt" is null ) = 0 THEN 0 ELSE 1 END) * ' + list.instagram + ') + ((CASE WHEN (select count(chcr."channelId") from "tblChannelSocialLink" chcr where chcr."channelId"=ch."channelId" and chcr."socialSiteId"=4 and chcr."deletedAt" is null ) = 0 THEN 0 ELSE 1 END) * ' + list.otherLink + ') + ((CASE WHEN (select count(chcr."channelId") from "tblChannelBrand" chcr where chcr."channelId"=ch."channelId" and chcr."deletedAt" is null ) = 0 THEN 0 ELSE 1 END) * ' + list.brand + ') + ((CASE WHEN (select count(chcr."channelId") from "tblChannelProductGroup" chcr where chcr."channelId"=ch."channelId" and chcr."deletedAt" is null ) = 0 THEN 0 ELSE 1 END) * ' + list.productGroup + ') + ((CASE WHEN (select count(chcr."channelId") from "tblChannelProductCategorie" chcr where chcr."channelId"=ch."channelId" and chcr."deletedAt" is null ) = 0 THEN 0 ELSE 1 END) * ' + list.productCat + ') + ((CASE WHEN (select count(chcr."channelId") from "tblChannelSpeciality" chcr where chcr."channelId"=ch."channelId" and chcr."deletedAt" is null ) = 0 THEN 0 ELSE 1 END) * ' + list.speciality + ') + ((CASE WHEN (select count(chcr."channelId") from "tblChannelCustomerProfile" chcr where chcr."channelId"=ch."channelId" and chcr."deletedAt" is null group by chcr."channelId" having count(chcr."channelId") >=1 ) = 0 THEN 0 ELSE 1 END) * ' + list.channelCustomerProfile + ') + ((CASE WHEN (select count(chcr."channelId") from "tblChannelLocation" chcr where chcr."channelId"=ch."channelId" and chcr."locationType"=\'SELLING\' and chcr."deletedAt" is null group by chcr."channelId" having count(chcr."channelId") >=1 ) = 0 THEN 0 ELSE 1 END) * ' + list.selling + ') + ((CASE WHEN (select count(chcr."channelId") from "tblChannelLocation" chcr where chcr."channelId"=ch."channelId" and chcr."locationType"=\'TARGET\' and chcr."deletedAt" is null group by chcr."channelId" having count(chcr."channelId") >=1) = 0 THEN 0 ELSE 1 END) * ' + list.target + ') + ((CASE WHEN(select count(chcr."channelId") from "tblChannelLocation" chcr where chcr."channelId" = ch."channelId" and chcr."locationType" = \'RETAIL\'  and chcr."deletedAt" is null group by chcr."channelId" having count(chcr."channelId") >= 1) = 0 THEN 0 ELSE 1 END) * ' + list.retail + ') + ((CASE WHEN ch."planId" = 1 THEN 0 ELSE 1 END) * ' + list.channel.planId + ') + ((CASE WHEN (select count(chcr."channelId") from "tblChannelJCT" chcr where chcr."channelId"=ch."channelId" and chcr."connectionStatusTypeId"=4  and chcr."deletedat" is null group by chcr."channelId" having count(chcr."channelId") >3) = 0 THEN 0 ELSE 1 END) * ' + list.partners + ') + ((CASE WHEN (select count(chcr."sellerchannelid") from "tblsampleorder" chcr where chcr."sellerchannelid"=ch."channelId" and chcr."statusid">=6  group by chcr."sellerchannelid" having count(chcr."sellerchannelid") > 2) = 0 THEN 0 ELSE 1 END) * ' + list.sampleOrder + ') + ((CASE WHEN (select count( p."channelId") from tblproductevaluation pe inner join "tblProduct" p on p."productId" = pe.productid where p."channelId" = ch."channelId" and p."deletedat" is null group by  p."channelId" having count( p."channelId")>=2 ) = 0 THEN 0 ELSE 1 END) * ' + list.receivedEvaluation + ') +  ((CASE WHEN (select count(chcr."buyerchannelid") from "tblsampleorder" chcr where chcr."buyerchannelid"=ch."channelId" and chcr."statusid">=3  group by chcr."buyerchannelid" having count(chcr."buyerchannelid") >=1) = 0 THEN 0 ELSE 1 END) * ' + list.sampleRequest + ') + ((CASE WHEN (select count(chcr."channelid") from "tblproductevaluation" chcr where chcr."channelid"=ch."channelId" and chcr."deletedat" is null group by chcr."channelid" having count(chcr."channelid") >1) = 0 THEN 0 ELSE 1 END) * ' + list.myEvaluation + ') as score FROM "tblChannel" ch left join "tblChannelDetail" chd on ch."channelId" = chd."channelId" left join "tblUser" u on u."channelId" = ch."channelId" right join "tblUserRole" ur on ur."roleId" = 1 and u."userId" = ur."userId" left join "tblContact" con on con."contactId" = u."contactId" where "ch"."channelId" = ' + channelId + '; ', { type: this.sequelizeInstance.QueryTypes.SELECT, plain: true }).then(function (results) {
                return results;
            });
            let channel = await this.channelRepository.findByPk(result.channelId);
            channel.profileStrength = result.score;
            await channel.save();
            return true;
        }
        catch (e) {
            console.log(e);
        }
        return false;
    }
    async incrementSuggestionCnt(user, updateChannelDto) {
        let result = await this.channelJCTRepository.findOne({
            where: {
                [sequelize_1.Op.or]: [
                    { partnerChannelId: user.channelId },
                    { channelId: user.channelId },
                ],
                channelJCTId: updateChannelDto.channelJCTId,
                deletedat: null
            },
            include: [
                {
                    model: this.channelRepository,
                    as: 'partnerChannel',
                    where: { deletedAt: null },
                },
                {
                    model: this.channelRepository,
                    as: 'Channel',
                    where: { deletedAt: null },
                },
            ]
        });
        if (!result)
            throw new Process_exception_1.ProcessException('channelHub:updateChannel:channelHubNotFound');
        let partnerChannelTypeId = null;
        if (result.partnerChannel.channelId == user.channelId) {
            partnerChannelTypeId = result.Channel.channelTypeId;
        }
        else {
            partnerChannelTypeId = result.partnerChannel.channelTypeId;
        }
        let suggtype = null;
        if (partnerChannelTypeId == app_constant_1.appconstant.CHANNEL_TYPE_DISTRIBUTOR) {
            suggtype = 'DISTRIBUTOR_OTB';
        }
        else if (partnerChannelTypeId == app_constant_1.appconstant.CHANNEL_TYPE_RETAILER) {
            suggtype = 'RETAILER_OTB';
        }
        else {
            suggtype = 'VENDOR_OTB';
        }
        let query = 'select channelsuggestionsid, suggestioncount from "tblChannelSubscription" tc left outer join "tblchannelsuggestions" tcs on tcs."channelsubscriptionid" = tc."channelSubscriptionId"  where tcs."suggestionfor" = :type AND  tcs.isexpired = :expired AND  tcs.endat >= NOW() and tc."channelId" = ' + user.channelId + ' and tc."isActive"=:isactive ';
        return await this.sequelizeInstance.query(query, {
            type: sequelize_1.QueryTypes.SELECT,
            replacements: {
                type: suggtype,
                expired: '0',
                isactive: '1'
            }
        }).then((queryresult) => {
            if (!queryresult[0]) {
                return true;
            }
            else {
                let channelsuggestionsid = queryresult[0].channelsuggestionsid;
                let suggestioncount = parseInt(queryresult[0].suggestioncount) + 1;
                let incquery = 'update "tblchannelsuggestions"  set "suggestioncount" = :suggestioncount  where  "channelsuggestionsid" = :channelsuggestionsid ';
                return this.sequelizeInstance.query(incquery, {
                    type: sequelize_1.QueryTypes.UPDATE,
                    replacements: {
                        channelsuggestionsid: channelsuggestionsid,
                        suggestioncount: suggestioncount
                    }
                }).then((queryresult) => {
                    return suggestioncount;
                });
            }
        });
    }
    async checkLimit(contactId) {
        let query = 'select "suggestioncount","limit","suggestionfor", "endat" from "tblchannelsuggestions" where "channelsubscriptionid" = (SELECT "channelSubscriptionId" FROM public."tblChannelSubscription" where "channelId" = (select "channelId" from "tblContact" where "contactId"=:contactId) and to_date( to_char("endAt",\'dd/mm/yyyy\'),\'dd/mm/yyyy\') >= to_date(to_char(now(),\'dd/mm/yyyy\'),\'dd/mm/yyyy\') and "isActive" = \'1\') and "isexpired" = \'0\' And  to_date( to_char("endat",\'dd/mm/yyyy\'),\'dd/mm/yyyy\') >= to_date(to_char(now(),\'dd/mm/yyyy\'),\'dd/mm/yyyy\') And "suggestionfor" IN(\'DISTRIBUTOR_OTB\' , \'RETAILER_OTB\' ,  \'VENDOR_OTB\')';
        return await this.sequelizeInstance.query(query, {
            type: sequelize_1.QueryTypes.SELECT,
            replacements: {
                contactId: +contactId,
            }
        });
    }
    async dimissReasons() {
        return await this.dismissReasonRepository.scope('dismissReason').findAll().then((dismissReasons) => {
            return dismissReasons;
        });
    }
    async channelBrands(channelId, user) {
        var whereOptions = {
            channelId: channelId,
            isActive: '1',
            signUpStatusId: app_constant_1.appconstant.CHANNEL_APPROVED,
            deletedAt: null
        };
        if (+user.channelTypeId == app_constant_1.appconstant.CHANNEL_TYPE_HUB) {
            whereOptions = {
                channelId: channelId,
                isActive: '1',
                deletedAt: null
            };
        }
        let channel = await Channel_entity_1.Channel.findOne({
            where: whereOptions
        });
        if (!channel) {
            throw new Auth_exception_1.AuthException('Not allowed');
        }
        let veifiedBrands = [];
        if (channel.channelTypeId == Channel_entity_1.Channel.CHANNEL_TYPE_DISTRIBUTOR) {
        }
        let option = {
            include: [
                {
                    model: this.channelBrandRepository,
                    where: { deletedAt: null, channelId: channelId },
                    required: true
                },
                {
                    model: this.mediaRepository,
                    where: { deletedAt: null },
                    required: true
                }
            ]
        };
        let whereOption = {
            isActive: '1',
            deletedAt: null
        };
        option['where'] = whereOption;
        let channelBrandCount = await this.brandRepository.count(option);
        if (channelBrandCount == 0) {
            return {
                "channelBrand": [],
                "total": 0
            };
        }
        let query = 'SELECT "Brand"."id", "Brand"."brandName", json_build_object(\'documentId\', "document"."documentId", \'documentPath\', "document"."documentPath", \'documentUrl\', :CDN_URL || "document"."documentPath") as "document" FROM "tblBrand" AS "Brand" INNER JOIN "tblChannelBrand" AS "channelBrand" ON "Brand"."id" = "channelBrand"."brandId" AND "channelBrand"."deletedAt" IS NULL AND "channelBrand"."channelId" = :channelId INNER JOIN "tblDocument" AS "document" ON "Brand"."documentId" = "document"."documentId" AND "document"."deletedAt" IS NULL WHERE "Brand"."isActive" = \'1\' AND "Brand"."deletedAt" IS NULL ';
        query += ' ORDER BY "Brand"."id" asc';
        let channelBrand = await this.sequelizeInstance.query(query, {
            type: this.sequelizeInstance.QueryTypes.SELECT,
            model: Brand_entity_1.Brand,
            mapToModel: true,
            replacements: {
                channelId: channelId,
                CDN_URL: process.env.CDN_URL,
            }
        }).then((results) => {
            let formatedBrands = results;
            if (results) {
                formatedBrands = [];
                results.forEach((brand) => {
                    let verified = false;
                    formatedBrands.push(Object.assign(brand.toJSON(), { verified }));
                });
            }
            return formatedBrands;
        });
        let channelBrands;
        if (channelBrand && channelBrand.length == 0) {
            return [];
        }
        channelBrands = {
            "channelBrand": channelBrand,
            "total": +channelBrandCount
        };
        return channelBrands;
    }
    async saveBuyerNeed(requirements) {
        let record = [];
        let result = await this.buyerRepository.create({
            channelid: requirements.channelid,
            contactid: requirements.contactid,
            requirements: requirements.data
        }).then((result) => {
            let catids = requirements.catid;
            catids.map(x => {
                let data = {
                    buyerreqid: result.id,
                    categoryid: x
                };
                record.push(data);
            });
        });
        await this.BuyerCategoryRepository.bulkCreate(record);
    }
    async eventRegister(contactId) {
        let result;
        try {
            let query = 'select (select concat(\'https://d35w8j22j5uly8.cloudfront.net/\',"documentPath") from "tblDocument" where "documentId"=tcon."documentId") as profileImage, '
                + ' "firstName","lastName","jobTitle",tcon."mobileNo",tcon."email",tc."companyName",tc."channelId" as companyId, cd."detailDesc",case when tc."channelTypeId"=2 then \'Vendor\' when tc."channelTypeId"=3 then \'Distributor\' when tc."channelTypeId"=4 then \'Retailer\' end as channelType, '
                + ' (select concat(\'https://d35w8j22j5uly8.cloudfront.net/\',"documentPath") from "tblDocument" where "documentId"=cd."channelLogoId") as companyLogo, '
                + '\'3e683a61-992b-47d2-8ecf-f88c9849f499\' as eventId,\'Confirmed\' as status, '
                + ' case when tc."channelTypeId"=2 then \'Vendor\' when tc."channelTypeId" in(3,4) then \'Buyer\' end as type, '
                + ' (select "country" from "tblCountry" where "countryId"=tc."countryId"),'
                + ' (select "isoCode" from "tblCountry" where "countryId"=tc."countryId"),(select string_agg("productCategorie",\',\') from "tblProductCategories" where "productCategorieId" in '
                + '(Select cpc."productCategorieId" from "tblContactInterestedProductCategory" cpc where cpc."contactId" = tcon."contactId"))as "IntrestedCategory", '
                + '(select string_agg("productCategorie",\',\') from "tblProductCategories" where "productCategorieId" in '
                + '(Select cpc."productCategorieId" from "tblContactProductCategorie" cpc where cpc."contactId" = tcon."contactId")) as "ExistingCategory", '
                + '(select "profileType" from "tblProfileType" where "profileTypeId"=tc."profileTypeId") as "ProfileType", '
                + '(select string_agg(pt."profileType",\',\') from "tblProfileType" pt where "profileTypeId" in (select "profileTypeId" from "tblChannelCustomerProfile" as ccp where ccp."channelId"=tc."channelId")) as "Retail_Customer_Profile", '
                + '(select "password" from "tblUser" where "contactId"=tcon."contactId") '
                + 'from "tblContact" tcon inner join "tblChannel" tc on tc."channelId"=tcon."channelId"	inner join "tblChannelDetail" cd on cd."channelId"=tcon."channelId" where tcon."contactId"=:contactId ';
            result = await this.sequelizeInstance.query(query, {
                type: sequelize_1.QueryTypes.SELECT,
                replacements: {
                    contactId: contactId
                }
            });
        }
        catch (e) {
            console.log("error:", e);
        }
        return result;
    }
};
ChannelHubService = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject('SequelizeInstance')),
    __param(1, common_1.Inject('ChannelRepository')),
    __param(2, common_1.Inject('ChannelJCTRepository')),
    __param(3, common_1.Inject('ChannelDismissReasonRepository')),
    __param(4, common_1.Inject('DismissReasonRepository')),
    __param(5, common_1.Inject('BrandRepository')),
    __param(6, common_1.Inject('ChannelBrandRepository')),
    __param(7, common_1.Inject('MediaRepository')),
    __param(8, common_1.Inject('BuyerRepository')),
    __param(9, common_1.Inject('BuyerCategoryRepository')),
    __metadata("design:paramtypes", [Object, Object, Object, Object, Object, Object, Object, Object, Object, Object])
], ChannelHubService);
exports.ChannelHubService = ChannelHubService;
//# sourceMappingURL=channelHub.service.js.map