'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CheckoutService = void 0;
const common_1 = require("@nestjs/common");
const Process_exception_1 = require("./../../../middleware/exception/Process.exception");
const ProductQuote_entity_1 = require("./../../cart/models/ProductQuote.entity");
const SampleOrderDetail_entity_1 = require("./../models/SampleOrderDetail.entity");
const OrderAddress_entity_1 = require("./../models/OrderAddress.entity");
const Channel_entity_1 = require("./../../channel/models/Channel.entity");
const config_1 = require("./../../../config");
const Product_entity_1 = require("../../product/models/Product.entity");
const sequelize_1 = require("sequelize");
let CheckoutService = class CheckoutService {
    constructor(sequelizeInstance, addressBook, cart, orderRepository) {
        this.sequelizeInstance = sequelizeInstance;
        this.addressBook = addressBook;
        this.cart = cart;
        this.orderRepository = orderRepository;
    }
    async validateAndCollectCart(data, user) {
        let cart = await this.cart.findOne({
            where: {
                deletedat: null,
                userid: user.userId,
                cartid: data.cartId
            },
            include: [
                {
                    model: ProductQuote_entity_1.ProductQuote,
                    as: 'items',
                    where: {
                        deletedat: null
                    },
                    include: [
                        {
                            model: Product_entity_1.Product,
                            as: 'product',
                            required: true,
                            attributes: ['product', 'productSKU', 'isactive']
                        }
                    ]
                },
                {
                    model: Channel_entity_1.Channel,
                    as: 'seller',
                    required: true
                }
            ]
        });
        if (!cart) {
            throw new Process_exception_1.ProcessException("Given cart already processed or not matching with active cart");
        }
        if (cart.grandtotal > config_1.appconstant.ORDER_GRAND_TOTAL_AMOUNT_LIMIT) {
            throw new Process_exception_1.ProcessException("Total Amount should not be greater than " + config_1.appconstant.ORDER_GRAND_TOTAL_AMOUNT_LIMIT);
        }
        if (cart.items) {
            let inactiveProducts = [];
            cart.items.forEach((item) => {
                if (item.product.isactive == '0') {
                    inactiveProducts.push(item.product);
                }
            });
            if (inactiveProducts.length > 0) {
                throw new Process_exception_1.ProcessException({ 'inactiveProduct': inactiveProducts });
            }
        }
        return cart;
    }
    async makeOrder(cart, data, user) {
        let CollectAddressIds = [data.shippingAddressId];
        if (data.billingAddressId != data.shippingAddressId) {
            CollectAddressIds.push(data.billingAddressId);
        }
        let address = await this.addressBook.findAll({
            where: {
                userid: user.userId,
                addressid: {
                    [sequelize_1.Op.in]: CollectAddressIds
                }
            },
            order: [['addressid', 'DESC']]
        });
        let billingaddress = null;
        let shippingaddress = null;
        await address.forEach((cdata) => {
            if (cdata.addressid == data.billingAddressId) {
                billingaddress = cdata.toJSON();
                delete billingaddress['addressid'];
                delete billingaddress['isbillingaddress'];
                delete billingaddress['isshippingaddress'];
            }
            if (cdata.addressid == data.shippingAddressId) {
                shippingaddress = cdata.toJSON();
                delete shippingaddress['addressid'];
                delete shippingaddress['isbillingaddress'];
                delete shippingaddress['isshippingaddress'];
            }
        });
        if (billingaddress == null || shippingaddress == null) {
            throw new Process_exception_1.ProcessException('shipping and billing addresses are mandatory');
        }
        let orderDetails = [];
        cart.items.forEach((item) => {
            let price = +item.quantity * item.unitprice;
            let details = {};
            details.productid = item.productid;
            details.quantity = item.quantity;
            details.finalquantity = item.quantity;
            details.samplepriceperquantity = item.unitprice;
            details.finaltotalprice = +price.toFixed(2);
            details.totalprice = +price.toFixed(2);
            details.createdat = this.sequelizeInstance.fn('NOW');
            orderDetails.push(details);
        });
        let order = await this.sequelizeInstance.transaction(async (transaction) => {
            return await this.orderRepository.create({
                userid: user.userId,
                buyerchannelid: user.channelId,
                sellerchannelid: cart.sellerchannelid,
                billingaddress: billingaddress,
                shippingaddress: shippingaddress,
                statusid: config_1.appconstant.ORDER_STATUS_PENDING_CONFIRMATION,
                currencyid: cart.currencyid,
                subtotalprice: cart.subtotal,
                shippingprice: 0,
                discount: cart.discount,
                othercharge: 0,
                ordertotalprice: cart.grandtotal,
                requestfreesample: (data.requestFreeSample) ? '1' : '0',
                orderdate: this.sequelizeInstance.fn('NOW'),
                queueprocessedat: this.sequelizeInstance.fn('NOW'),
                invoiced: '0',
                orderDetails: orderDetails,
                relation_queued: data['relation_queued']
            }, {
                include: [{
                        model: OrderAddress_entity_1.OrderAddress,
                        as: 'billingaddress'
                    }, {
                        model: OrderAddress_entity_1.OrderAddress,
                        as: 'shippingaddress'
                    }, {
                        model: SampleOrderDetail_entity_1.SampleOrderDetail
                    }],
                transaction
            }).then(async (order) => {
                let ordernumber = await this.sequelizeInstance.query('select \'ORD\' || ((EXTRACT(YEAR FROM now()):: integer * 10000) + (EXTRACT(MONTH FROM now()):: integer * 100) + (EXTRACT(day FROM now()):: integer * 1)) :: varchar(8) || (count(*) + 1) :: varchar(10) as number from tblsampleorder where to_char(orderdate, \'dd/mm/yyyy\') = to_char(now(), \'dd/mm/yyyy\') and "sampleorderid" <= ' + order.sampleorderid, {
                    type: this.sequelizeInstance.QueryTypes.SELECT,
                    raw: true,
                });
                let sellerSubscriptionId = cart.seller.subscriptionId;
                let key = +sellerSubscriptionId + '-' + +config_1.features.SAMPLE_ORDER_MGMT;
                let query = 'select tc."channelSubscriptionId", tc."channelId", count(ts."sampleorderid"), to_date( to_char(now(),\'dd/mm/yyyy\'),\'dd/mm/yyyy\') >= to_date( to_char(tc."endAt",\'dd/mm/yyyy\'),\'dd/mm/yyyy\') as expired from "tblChannelSubscription" tc left outer join "tblsampleorder" ts on tc."channelId" = ts."sellerchannelid" and to_date( to_char(ts.queueprocessedat,\'dd/mm/yyyy\'),\'dd/mm/yyyy\') between to_date( to_char(tc."startedAt",\'dd/mm/yyyy\'),\'dd/mm/yyyy\') AND to_date( to_char(tc."endAt",\'dd/mm/yyyy\'),\'dd/mm/yyyy\') and ts.relation_queued = \'0\' and ts.subscription_queued = \'0\' where tc."channelId" = :sellerchannelid and tc."isActive"=\'1\' group by tc."channelSubscriptionId", tc."channelId", tc."endAt" LIMIT 1';
                let pastOrders = await this.sequelizeInstance.query(query, {
                    type: this.sequelizeInstance.QueryTypes.SELECT,
                    raw: true,
                    replacements: {
                        sellerchannelid: order.sellerchannelid
                    }
                }).then((results) => {
                    return (results[0]) ? results[0] : null;
                });
                order.ordernumber = ordernumber[0].number;
                await order.save({
                    transaction
                });
                cart.deletedat = this.sequelizeInstance.fn('NOW');
                await cart.save({
                    transaction
                });
                return order;
            });
        }).then((result) => {
            return result;
        }).catch((err) => {
            console.log(err);
            throw new Process_exception_1.ProcessException('validate:sql:internalError');
        });
        return order.ordernumber;
    }
};
CheckoutService = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject('SequelizeInstance')),
    __param(1, common_1.Inject('MyAddressBookRepository')),
    __param(2, common_1.Inject('CartRepository')),
    __param(3, common_1.Inject('OrderRepository')),
    __metadata("design:paramtypes", [Object, Object, Object, Object])
], CheckoutService);
exports.CheckoutService = CheckoutService;
//# sourceMappingURL=checkout.service.js.map