'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProductAdminService = void 0;
const common_1 = require("@nestjs/common");
const Process_exception_1 = require("./../../../middleware/exception/Process.exception");
const index_1 = require("../../media/models/index");
const app_constant_1 = require("../../../config/app.constant");
const Validation_exception_1 = require("./../../../middleware/exception/Validation.exception");
const base_service_1 = require("../../../core/service/base.service");
const sequelize_1 = require("sequelize");
let ProductAdminService = class ProductAdminService extends base_service_1.BaseService {
    constructor(productGroupRepository, productCategoryRepository, brandRepository, sequelizeInstance) {
        super();
        this.productGroupRepository = productGroupRepository;
        this.productCategoryRepository = productCategoryRepository;
        this.brandRepository = brandRepository;
        this.sequelizeInstance = sequelizeInstance;
    }
    async createBrand(createBrandDto) {
        createBrandDto['documentId'] = null;
        if (await this.brandRepository.count({
            where: {
                brandName: {
                    [sequelize_1.Op.iLike]: createBrandDto.brandName.trim()
                },
                deletedAt: null
            }
        })) {
            throw new Validation_exception_1.ValidationException([{
                    value: createBrandDto.brandName.trim(),
                    property: 'brandName',
                    children: [],
                    constraints: { unique: 'validate:field:shouldBeUnique' }
                }]);
        }
        if (createBrandDto.brandImage) {
            let brandImage = await index_1.Media.create({
                documentPath: createBrandDto.brandImage,
                documentTypeId: app_constant_1.appconstant.DOCUMENTTYPE_IMAGE,
                dirName: app_constant_1.appconstant.FOLDER_PATH_BRAND
            });
            createBrandDto['documentId'] = brandImage.documentId;
        }
        createBrandDto['createdAt'] = new Date().toUTCString();
        return await this.brandRepository.create(createBrandDto);
    }
    async getBrands(pagination) {
        console.log('pagination', pagination);
        let result = new Object();
        let whereOption = {};
        if (pagination.search) {
            whereOption['brandName'] = { [sequelize_1.Op.iLike]: '%' + pagination.search + '%' };
        }
        if (pagination.filter) {
            whereOption['isActive'] = pagination.filter;
        }
        result.total = await this.brandRepository.scope('brandMaster').count();
        console.log('result.total', result.total);
        let sortField = 'createdAt';
        result.list = await this.brandRepository.scope('brandMaster').findAll({
            where: whereOption,
            limit: pagination.limit,
            offset: pagination.offset,
            order: [[sortField, pagination.orderBy || 'desc']]
        });
        return result;
    }
    async getBrand(id) {
        let result = await this.brandRepository.scope('brandMaster').findByPk(id);
        if (!result) {
            throw new Process_exception_1.ProcessException('validate:field:notFound');
        }
        return result;
    }
    async updateBrand(id, updateBrandDto) {
        let result = await this.brandRepository.scope('brandMaster').findByPk(id);
        if (!result) {
            throw new Process_exception_1.ProcessException('validate:field:notFound');
        }
        if (updateBrandDto.brandName) {
            if (await this.brandRepository.count({
                where: {
                    brandName: {
                        [sequelize_1.Op.iLike]: updateBrandDto.brandName.trim()
                    },
                    deletedAt: null
                }
            })) {
                throw new Validation_exception_1.ValidationException([{
                        value: updateBrandDto.brandName.trim(),
                        property: 'brandName',
                        children: [],
                        constraints: { unique: 'validate:field:shouldBeUnique' }
                    }]);
            }
        }
        if (updateBrandDto.brandImage) {
            let brandImage = await index_1.Media.create({
                documentPath: updateBrandDto.brandImage,
                documentTypeId: app_constant_1.appconstant.DOCUMENTTYPE_IMAGE,
                dirName: app_constant_1.appconstant.FOLDER_PATH_BRAND
            });
            updateBrandDto['documentId'] = brandImage.documentId;
        }
        await result.update(updateBrandDto);
        return result;
    }
};
ProductAdminService = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject('ProductGroupRepository')),
    __param(1, common_1.Inject('ProductCategoryRepository')),
    __param(2, common_1.Inject('BrandRepository')),
    __param(3, common_1.Inject('SequelizeInstance')),
    __metadata("design:paramtypes", [Object, Object, Object, Object])
], ProductAdminService);
exports.ProductAdminService = ProductAdminService;
//# sourceMappingURL=product.admin.service.js.map