'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RejectReasonService = void 0;
const common_1 = require("@nestjs/common");
const Process_exception_1 = require("./../../../middleware/exception/Process.exception");
const Cart_entity_1 = require("./../../cart/models/Cart.entity");
const ProductQuote_entity_1 = require("./../../cart/models/ProductQuote.entity");
const base_service_1 = require("../../../core/service/base.service");
const Wishlist_entity_1 = require("../../wishlist/models/Wishlist.entity");
let RejectReasonService = class RejectReasonService extends base_service_1.BaseService {
    constructor(repository, rejectRepository, sequelizeInstance, productRepository) {
        super();
        this.repository = repository;
        this.rejectRepository = rejectRepository;
        this.sequelizeInstance = sequelizeInstance;
        this.productRepository = productRepository;
    }
    async create(data) {
        return await this.repository.create(this._buildObject(new Object(), data));
    }
    async _validateProduct(productId) {
        let product = await this.productRepository.scope('productReject').findByPk(productId);
        if (!product) {
            throw new Process_exception_1.ProcessException('validate:field:notFound');
        }
        return product;
    }
    async reject(data, user) {
        return await this.sequelizeInstance.transaction(async (transaction) => {
            let product = await this._validateProduct(data.productId);
            try {
                return await this.rejectRepository.create({
                    userid: +user.userId,
                    channelid: +user.channelId,
                    productid: data.productId,
                    rejectreasonid: data.rejectreasonId,
                    otherreasons: data.comment,
                }, {
                    transaction
                }).then(async (data) => {
                    let productId = data.productid;
                    let channelId = data.channelid;
                    await Wishlist_entity_1.Wishlist.destroy({
                        where: {
                            channelid: channelId,
                            productid: productId
                        },
                        transaction
                    });
                    let cart = await Cart_entity_1.Cart.findOne({
                        where: {
                            sellerchannelid: product.channelId,
                            buyerchannelid: channelId,
                            deletedat: null
                        },
                        transaction
                    });
                    if (cart) {
                        await ProductQuote_entity_1.ProductQuote.destroy({
                            where: {
                                cartid: cart.cartid,
                                productid: productId
                            },
                            transaction
                        });
                    }
                });
            }
            catch (e) {
                console.log('e', e);
                let message = 'validate:sql:internalError';
                if (e.errors && (e.errors[0].message == 'productid must be unique')) {
                    message = 'this product already removed';
                }
                throw new Process_exception_1.ProcessException(message);
            }
        });
    }
    async findAll(pagination) {
        return await this.repository.findAll({
            limit: pagination.limit,
            offset: pagination.offset,
            order: [['rejectreasonid', 'DESC']]
        });
    }
    async findOne(options) {
        let result = await this.repository.findOne(options);
        if (!result) {
            throw new Process_exception_1.ProcessException('validate:field:notFound');
        }
        return result;
    }
    async findById(id) {
        let result = await this.repository.findByPk(id);
        if (!result) {
            throw new Process_exception_1.ProcessException('validate:field:notFound');
        }
        return result;
    }
    async update(id, newValue) {
        let result = await this.repository.findByPk(id);
        if (!result) {
            throw new Process_exception_1.ProcessException('validate:field:notFound');
        }
        await this._buildObject(result, newValue).save();
        return result;
    }
    async delete(id) {
        let result = await this.repository.findByPk(id);
        if (!result) {
            throw new Process_exception_1.ProcessException('media:documentTypeId:notFound');
        }
        return await result.destroy();
    }
    async getCount() {
        return await this.repository.count();
    }
    _buildObject(entity, dto) {
        entity.rejectreason = dto.rejectreason;
        return entity;
    }
};
RejectReasonService = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject('RejectReasonRepository')),
    __param(1, common_1.Inject('ProductRejectRepository')),
    __param(2, common_1.Inject('SequelizeInstance')),
    __param(3, common_1.Inject('ProductRepository')),
    __metadata("design:paramtypes", [Object, Object, Object, Object])
], RejectReasonService);
exports.RejectReasonService = RejectReasonService;
//# sourceMappingURL=rejectReason.service.js.map