'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProductServiceOutBound = void 0;
const common_1 = require("@nestjs/common");
const lodash_1 = require("lodash");
class ProductServiceOutBound {
    constructor() {
    }
    async getProductList(productList, pagination, total) {
        try {
            let resultObject = {};
            resultObject.Products = [];
            if (total != undefined) {
                resultObject.total = total.total ? total.total : 0;
                resultObject.activeTotal = total.activeTotal ? total.activeTotal : 0;
                resultObject.inActiveTotal = total.inActiveTotal ? total.inActiveTotal : 0;
            }
            resultObject.offset = pagination.offset || 0;
            resultObject.limit = pagination.limit || 0;
            resultObject.page = pagination.page || 0;
            let productListReturn = [];
            for (let ii = 0; ii < productList.length; ii++) {
                let product = productList[ii].toJSON();
                product['location'] = {};
                product.location['TARGET'] = [];
                product.location['SELLING'] = [];
                product.location['RETAIL'] = [];
                product.locations.map(function (data, index) {
                    product.location[data.ProductLocation.locationtype].push(data.country.country);
                });
                product.locations = product.location;
                productListReturn.push(product);
                delete (product.location);
            }
            resultObject.Products = productListReturn;
            return resultObject;
        }
        catch (e) {
            console.log("Internal Server error at ProductServiceOutBound: " + e);
            throw new common_1.InternalServerErrorException();
        }
    }
    async getProductMasters(productMaster) {
        try {
            let resultObject = {};
            resultObject.productGroup = [];
            resultObject.productCategory = [];
            resultObject.brand = [];
            resultObject.locations = {};
            resultObject.locations.TARGET = [];
            resultObject.locations.SELLING = [];
            resultObject.locations.RETAIL = [];
            resultObject.shareType = [];
            resultObject.customerProfile = {};
            resultObject.customerProfile.DISTRIBUTOR = [];
            resultObject.customerProfile.RETAILER = [];
            resultObject.locations.TARGET.all = '';
            resultObject.locations.SELLING.all = '';
            resultObject.locations.RETAIL.all = '';
            resultObject.customerProfile.DISTRIBUTOR.all = '';
            resultObject.customerProfile.RETAILER.all = '';
            resultObject.shareType.all = '';
            resultObject.unit = [];
            resultObject.discountUnit = [];
            resultObject.unitNames = '';
            resultObject.discountUnitNames = '';
            resultObject.currency = [];
            resultObject.currencyNames = '';
            if (productMaster.productGroup) {
                productMaster.productGroup.map(function (data, index) {
                    resultObject.productGroup.push(data.productGroup.toLowerCase().trim());
                });
            }
            if (productMaster.productCategory) {
                productMaster.productCategory.map(function (data, index) {
                    resultObject.productCategory.push(data.productCategorie.toLowerCase().trim());
                });
            }
            if (productMaster.brand) {
                productMaster.brand.map(function (data, index) {
                    resultObject.brand.push(data.brandName.toLowerCase().trim());
                });
            }
            if (productMaster.channelLocation) {
                productMaster.channelLocation.map(function (data, index) {
                    let separator = '|';
                    if (resultObject.locations[data.locationType].all == '') {
                        separator = '';
                    }
                    if (!resultObject.locations[data.locationType].includes(data.RegionCountryJCT.country.country.toLowerCase().trim())) {
                        resultObject.locations[data.locationType].push(data.RegionCountryJCT.country.country.toLowerCase().trim());
                        resultObject.locations[data.locationType].all = resultObject.locations[data.locationType].all + separator + data.RegionCountryJCT.country.country.toLowerCase().trim();
                    }
                });
            }
            if (productMaster.shareType) {
                productMaster.shareType.map(function (data, index) {
                    let separator = '|';
                    if (resultObject.shareType.all == '') {
                        separator = '';
                    }
                    resultObject.shareType.push(data.sharetype.toLowerCase().trim());
                    resultObject.shareType.all = resultObject.shareType.all + separator + data.sharetype.toLowerCase().trim();
                });
            }
            if (productMaster.channelCustomerProfile) {
                productMaster.channelCustomerProfile.map(function (data, index) {
                    let separator = '|';
                    if (resultObject.customerProfile[data.pTClas.channelType.dataValues.name] == '') {
                        separator = '';
                    }
                    if (data.pTClas.channelType.dataValues.name === 'DISTRIBUTOR' || data.pTClas.channelType.dataValues.name === 'RETAILER') {
                        resultObject.customerProfile[data.pTClas.channelType.dataValues.name].push(data.pTClas.profileType.dataValues.name.toLowerCase().trim());
                        resultObject.customerProfile[data.pTClas.channelType.dataValues.name].all = resultObject.customerProfile[data.pTClas.channelType.dataValues.name].all + separator + data.pTClas.profileType.dataValues.name.toLowerCase().trim();
                    }
                });
            }
            if (productMaster.unit) {
                productMaster.unit.map(function (data, index) {
                    let separator = ',';
                    if (index == 0)
                        separator = '';
                    resultObject.unit.push(data.unit.toLowerCase().trim());
                    resultObject.unitNames = resultObject.unitNames + separator + data.unit.toLowerCase().trim();
                });
            }
            if (productMaster.discountUnit) {
                productMaster.discountUnit.map(function (data, index) {
                    let separator = ',';
                    if (index == 0)
                        separator = '';
                    resultObject.discountUnit.push(data.discountUnit.toLowerCase().trim());
                    resultObject.discountUnitNames = resultObject.discountUnitNames + separator + data.discountUnit.toLowerCase().trim();
                });
            }
            if (productMaster.currency) {
                productMaster.currency.map(function (data, index) {
                    let separator = ',';
                    if (index == 0)
                        separator = '';
                    resultObject.currency.push(data);
                    resultObject.currencyNames = resultObject.currencyNames + separator + data.currency.toLowerCase().trim();
                });
            }
            resultObject['master'] = productMaster;
            return resultObject;
        }
        catch (e) {
            console.log("Internal Server error at ProductServiceOutBound: " + e);
            throw new common_1.InternalServerErrorException();
        }
    }
    async customerProfile(params) {
        let returnObject = {};
        let paramsjson = params.toJSON();
        if (paramsjson.channelCustomerProfile && paramsjson.channelCustomerProfile.length > 0) {
            let channelCustomerProfile = paramsjson.channelCustomerProfile;
            for (let ii = 0; ii < channelCustomerProfile.length; ii++) {
                let resultObject = await this.customerProfileAssaingObject(channelCustomerProfile[ii]);
                let propertyName = channelCustomerProfile[ii].pTClas.channelType.name;
                if (!paramsjson[propertyName]) {
                    paramsjson[propertyName] = new Array();
                    paramsjson[propertyName].push(resultObject);
                }
                else {
                    paramsjson[propertyName].push(resultObject);
                }
            }
        }
        paramsjson['productGroup'] = [];
        if (params.productGroup) {
            paramsjson['productGroup'] = params.productGroup.map((data) => {
                return {
                    "productGroupId": data.productGroupId,
                    "productGroup": data.productGroup
                };
            });
        }
        paramsjson['productCategory'] = [];
        if (params.productCategory) {
            paramsjson['productCategory'] = params.productCategory.map((data) => {
                return {
                    "productCategorieId": data.productCategorieId,
                    "productGroupId": data.productGroupId,
                    "productCategorie": data.productCategorie
                };
            });
        }
        paramsjson['locations'] = [];
        if (params.locations) {
            paramsjson['locations'] = params.locations.map((data) => {
                return {
                    "regionCountryJCTId": data.regionCountryJCTId,
                    "regionId": data.regionId,
                    "countryId": data.countryId,
                    "country": {
                        "countryId": data.country.countryId,
                        "country": data.country.country
                    },
                    "region": {
                        "regionId": data.region.regionId,
                        "region": data.region.region
                    },
                    "ChannelLocation": {
                        "locationType": data.ChannelLocation.locationType
                    }
                };
            });
        }
        return paramsjson;
    }
    async productCustomerProfile(params, user) {
        let returnObject = {};
        let returnImageObject = {};
        let paramsjson = params.toJSON();
        paramsjson.other_info = params.other_info;
        if (paramsjson.productChannelCustomerProfile && paramsjson.productChannelCustomerProfile.length > 0) {
            let productChannelCustomerProfile = paramsjson.productChannelCustomerProfile;
            for (let ii = 0; ii < productChannelCustomerProfile.length; ii++) {
                if (productChannelCustomerProfile[ii].profile) {
                    let resultObject = await this.customerProfileAssaingObject(productChannelCustomerProfile[ii].profile);
                    if (productChannelCustomerProfile[ii].profile.pTClas) {
                        let propertyName = productChannelCustomerProfile[ii].profile.pTClas.channelType.name;
                        if (!paramsjson[propertyName]) {
                            paramsjson[propertyName] = new Array();
                            paramsjson[propertyName].push(resultObject);
                        }
                        else {
                            paramsjson[propertyName].push(resultObject);
                        }
                    }
                }
            }
        }
        if (params.productLocation.length > 0) {
            returnObject['SELLING'] = [];
            returnObject['TARGET'] = [];
            returnObject['RETAIL'] = [];
            for (let ii = 0; ii < params.productLocation.length; ii++) {
                if (params.productLocation[ii].locationtype == 'SELLING') {
                    returnObject['SELLING'].push(params.productLocation[ii].regionCountryJCT.country.country);
                }
                else if (params.productLocation[ii].locationtype == 'TARGET') {
                    returnObject['TARGET'].push(params.productLocation[ii].regionCountryJCT.country.country);
                }
                else if (params.productLocation[ii].locationtype == 'RETAIL') {
                    returnObject['RETAIL'].push(params.productLocation[ii].regionCountryJCT.country.country);
                }
            }
            paramsjson.locations = returnObject;
            paramsjson.productLocation = params.productLocation;
        }
        if (lodash_1.isArray(params.productKeyRetailer) && params.productKeyRetailer.length > 0) {
            paramsjson.productKeyRetailer = params.productKeyRetailer;
        }
        if (lodash_1.isArray(params.productKeyDistributor) && params.productKeyDistributor.length > 0) {
            paramsjson.productKeyDistributor = params.productKeyDistributor;
        }
        if (lodash_1.isArray(params.productKeyReseller) && params.productKeyReseller.length > 0) {
            paramsjson.productKeyReseller = params.productKeyReseller;
        }
        if (params.productAwards.length > 0) {
            paramsjson.productAwards = params.productAwards;
        }
        if (paramsjson.productImage.length > 0) {
            returnImageObject['PRODUCT'] = [];
            returnImageObject['DISPLAY'] = [];
            returnImageObject['PACKING'] = [];
            returnImageObject['BROUCHER'] = [];
            for (let ii = 0; ii < paramsjson.productImage.length; ii++) {
                if (paramsjson.productImage[ii].productimagetype == 'Image') {
                    returnImageObject['PRODUCT'].push(paramsjson.productImage[ii]);
                }
                else if (paramsjson.productImage[ii].productimagetype == 'Display') {
                    returnImageObject['DISPLAY'].push(paramsjson.productImage[ii]);
                }
                else if (paramsjson.productImage[ii].productimagetype == 'Packing') {
                    returnImageObject['PACKING'].push(paramsjson.productImage[ii]);
                }
                else if (paramsjson.productImage[ii].productimagetype == 'Broucher') {
                    paramsjson.productImage[ii].productimagetype = 'Brochure';
                    returnImageObject['BROUCHER'].push(paramsjson.productImage[ii]);
                }
            }
            paramsjson.images = returnImageObject;
        }
        if (paramsjson.productPriceDetail || paramsjson.productPackageInfo) {
            params.unit.map(function (data, index) {
                if (paramsjson.productPriceDetail.retailerpriceunitid && paramsjson.productPriceDetail.retailerpriceunitid == data.unitid) {
                    paramsjson.productPriceDetail['retailerpriceunitdetail'] = {
                        unitid: paramsjson.productPriceDetail.retailerpriceunitid,
                        unit: data.unit
                    };
                }
                if (paramsjson.productPriceDetail.samplepriceunitid && paramsjson.productPriceDetail.samplepriceunitid == data.unitid) {
                    paramsjson.productPriceDetail['samplepriceunitdetail'] = {
                        unitid: paramsjson.productPriceDetail.samplepriceunitid,
                        unit: data.unit
                    };
                }
                if (paramsjson.productPackageInfo && paramsjson.productPackageInfo.orderunitid && paramsjson.productPackageInfo.orderunitid == data.unitid) {
                    paramsjson.productPackageInfo['orderunitdetail'] = {
                        unitid: paramsjson.productPackageInfo.orderunitid,
                        unit: data.unit
                    };
                }
                if (paramsjson.productPackageInfo && paramsjson.productPackageInfo.packageunit && paramsjson.productPackageInfo.packageunit == data.unitid) {
                    paramsjson.productPackageInfo['packageunitdetail'] = {
                        unitid: paramsjson.productPackageInfo.packageunit,
                        unit: data.unit
                    };
                }
                if (paramsjson.productPackageInfo && paramsjson.productPackageInfo.noofpackageinboxunitid && paramsjson.productPackageInfo.noofpackageinboxunitid == data.unitid) {
                    paramsjson.productPackageInfo['noofpackageinboxunitdetail'] = {
                        unitid: paramsjson.productPackageInfo.noofpackageinboxunitid,
                        unit: data.unit
                    };
                }
                if (paramsjson.productPackageInfo && paramsjson.productPackageInfo.noofboxinpalletunitid && paramsjson.productPackageInfo.noofboxinpalletunitid == data.unitid) {
                    paramsjson.productPackageInfo['noofboxinpalletunitdetail'] = {
                        unitid: paramsjson.productPackageInfo.noofboxinpalletunitid,
                        unit: data.unit
                    };
                }
                if (paramsjson.productPackageInfo && paramsjson.productPackageInfo.approximateweightofpalletunitid && paramsjson.productPackageInfo.approximateweightofpalletunitid == data.unitid) {
                    paramsjson.productPackageInfo['approximateweightofpalletunitdetail'] = {
                        unitid: paramsjson.productPackageInfo.approximateweightofpalletunitid,
                        unit: data.unit
                    };
                }
            });
        }
        paramsjson['existRatingAverage'] = 0;
        if (params.productExistRating && params.productExistRating.length > 0) {
            let exitRatingAverage = 0;
            for (let ii = 0; ii < params.productExistRating.length; ii++) {
                exitRatingAverage = exitRatingAverage + +params.productExistRating[ii].existproductrating;
            }
            paramsjson['existRatingAverage'] = (exitRatingAverage / params.productExistRating.length).toFixed(2);
        }
        if (params.channel) {
            paramsjson['companyName'] = params.channel.companyName;
            paramsjson['channelTypeId'] = params.channel.channelTypeId;
            paramsjson['is_french_tech'] = params.channel.isFrenchTech;
            paramsjson['isOEM'] = params.channel.isOEM;
        }
        paramsjson['productType'] = 'Buying';
        if (user) {
            if (paramsjson.channelId == user.channelId) {
                paramsjson['productType'] = 'Selling';
            }
        }
        return paramsjson;
    }
    async publicProductCustomerProfile(params) {
        let returnObject = {};
        let returnImageObject = {};
        let paramsjson = params.toJSON();
        if (paramsjson.productChannelCustomerProfile && paramsjson.productChannelCustomerProfile.length > 0) {
            let productChannelCustomerProfile = paramsjson.productChannelCustomerProfile;
            for (let ii = 0; ii < productChannelCustomerProfile.length; ii++) {
                if (productChannelCustomerProfile[ii].profile) {
                    let resultObject = await this.customerProfileAssaingObject(productChannelCustomerProfile[ii].profile);
                    if (productChannelCustomerProfile[ii].profile.pTClas) {
                        let propertyName = productChannelCustomerProfile[ii].profile.pTClas.channelType.name;
                        if (!paramsjson[propertyName]) {
                            paramsjson[propertyName] = new Array();
                            paramsjson[propertyName].push(resultObject);
                        }
                        else {
                            paramsjson[propertyName].push(resultObject);
                        }
                    }
                }
            }
        }
        if (lodash_1.isArray(params.productKeyRetailer) && params.productKeyRetailer.length > 0) {
            paramsjson.productKeyRetailer = params.productKeyRetailer;
        }
        if (lodash_1.isArray(params.productKeyReseller) && params.productKeyReseller.length > 0) {
            paramsjson.productKeyReseller = params.productKeyReseller;
        }
        if (params.productAwards.length > 0) {
            paramsjson.productAwards = params.productAwards;
        }
        if (paramsjson.productImage.length > 0) {
            returnImageObject['PRODUCT'] = [];
            returnImageObject['DISPLAY'] = [];
            returnImageObject['PACKING'] = [];
            returnImageObject['BROUCHER'] = [];
            for (let ii = 0; ii < paramsjson.productImage.length; ii++) {
                console.log('paramsjson.productImage[ii].productimagetype', paramsjson.productImage[ii]);
                if (paramsjson.productImage[ii].productimagetype == 'Image') {
                    returnImageObject['PRODUCT'].push(paramsjson.productImage[ii]);
                }
                else if (paramsjson.productImage[ii].productimagetype == 'Display') {
                    returnImageObject['DISPLAY'].push(paramsjson.productImage[ii]);
                }
                else if (paramsjson.productImage[ii].productimagetype == 'Packing') {
                    returnImageObject['PACKING'].push(paramsjson.productImage[ii]);
                }
                else if (paramsjson.productImage[ii].productimagetype == 'Broucher' && paramsjson.productImage[ii].image.documentTypeId == 2) {
                    paramsjson.productImage[ii].productimagetype = 'Brochure';
                    returnImageObject['BROUCHER'].push(paramsjson.productImage[ii]);
                }
            }
            paramsjson.images = returnImageObject;
        }
        if (paramsjson.productPriceDetail || paramsjson.productPackageInfo) {
            params.unit.map(function (data, index) {
                if (paramsjson.productPriceDetail.retailerpriceunitid && paramsjson.productPriceDetail.retailerpriceunitid == data.unitid) {
                    paramsjson.productPriceDetail['retailerpriceunitdetail'] = {
                        unitid: paramsjson.productPriceDetail.retailerpriceunitid,
                        unit: data.unit
                    };
                }
                if (paramsjson.productPriceDetail.samplepriceunitid && paramsjson.productPriceDetail.samplepriceunitid == data.unitid) {
                    paramsjson.productPriceDetail['samplepriceunitdetail'] = {
                        unitid: paramsjson.productPriceDetail.samplepriceunitid,
                        unit: data.unit
                    };
                }
                if (paramsjson.productPackageInfo && paramsjson.productPackageInfo.orderunitid && paramsjson.productPackageInfo.orderunitid == data.unitid) {
                    paramsjson.productPackageInfo['orderunitdetail'] = {
                        unitid: paramsjson.productPackageInfo.orderunitid,
                        unit: data.unit
                    };
                }
                if (paramsjson.productPackageInfo && paramsjson.productPackageInfo.packageunit && paramsjson.productPackageInfo.packageunit == data.unitid) {
                    paramsjson.productPackageInfo['packageunitdetail'] = {
                        unitid: paramsjson.productPackageInfo.packageunit,
                        unit: data.unit
                    };
                }
                if (paramsjson.productPackageInfo && paramsjson.productPackageInfo.noofpackageinboxunitid && paramsjson.productPackageInfo.noofpackageinboxunitid == data.unitid) {
                    paramsjson.productPackageInfo['noofpackageinboxunitdetail'] = {
                        unitid: paramsjson.productPackageInfo.noofpackageinboxunitid,
                        unit: data.unit
                    };
                }
                if (paramsjson.productPackageInfo && paramsjson.productPackageInfo.noofboxinpalletunitid && paramsjson.productPackageInfo.noofboxinpalletunitid == data.unitid) {
                    paramsjson.productPackageInfo['noofboxinpalletunitdetail'] = {
                        unitid: paramsjson.productPackageInfo.noofboxinpalletunitid,
                        unit: data.unit
                    };
                }
                if (paramsjson.productPackageInfo && paramsjson.productPackageInfo.approximateweightofpalletunitid && paramsjson.productPackageInfo.approximateweightofpalletunitid == data.unitid) {
                    paramsjson.productPackageInfo['approximateweightofpalletunitdetail'] = {
                        unitid: paramsjson.productPackageInfo.approximateweightofpalletunitid,
                        unit: data.unit
                    };
                }
            });
        }
        paramsjson['existRatingAverage'] = 0;
        if (params.productExistRating && params.productExistRating.length > 0) {
            let exitRatingAverage = 0;
            for (let ii = 0; ii < params.productExistRating.length; ii++) {
                exitRatingAverage = exitRatingAverage + +params.productExistRating[ii].existproductrating;
            }
            paramsjson['existRatingAverage'] = (exitRatingAverage / params.productExistRating.length).toFixed(2);
        }
        if (params.channel) {
            paramsjson['companyName'] = params.channel.companyName;
            paramsjson['channelTypeId'] = params.channel.channelTypeId;
            paramsjson['is_french_tech'] = params.channel.isFrenchTech;
        }
        paramsjson['productType'] = 'Buying';
        return paramsjson;
    }
    async customerProfileAssaingObject(customerparams) {
        let resultObject = {};
        if (customerparams.pTClas) {
            resultObject.Channel_Profile_ID = parseInt(customerparams.pTClas.profileType.id);
            resultObject.Channel_Profile_Name = customerparams.pTClas.profileType.name;
            resultObject.name = customerparams.pTClas.profileType.name;
            resultObject.Channel_Type_ID = parseInt(customerparams.pTClas.channelType.id);
            resultObject.Channel_Type_Name = customerparams.pTClas.channelType.name;
            resultObject.Channel_Profile_Type_Classification_ID = parseInt(customerparams.profileTypeClassification);
            resultObject.Channel_Customer_Profile_ID = customerparams.customerProfileId;
        }
        return resultObject;
    }
}
exports.ProductServiceOutBound = ProductServiceOutBound;
//# sourceMappingURL=productServiceOutBound.js.map