'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MyAddressBookService = void 0;
const common_1 = require("@nestjs/common");
const Process_exception_1 = require("./../../../middleware/exception/Process.exception");
const Country_entity_1 = require("../../common/models/Country.entity");
let MyAddressBookService = class MyAddressBookService {
    constructor(repository, sequelizeInstance) {
        this.repository = repository;
        this.sequelizeInstance = sequelizeInstance;
    }
    async create(data, user) {
        return await this.repository.create(this._buildObject(new Object(), data, user));
    }
    async getCount(user, type) {
        let filter = {
            userid: user.userId
        };
        if (type) {
            if (type == 'billing') {
                filter = Object.assign(filter, { isbillingaddress: '1' });
            }
            else if (type == 'shipping') {
                filter = Object.assign(filter, { isshippingaddress: '1' });
            }
        }
        return await this.repository.count({
            where: filter
        });
    }
    async findAll(user, type) {
        let filter = {
            userid: user.userId
        };
        if (type) {
            if (type == 'billing') {
                filter = Object.assign(filter, { isbillingaddress: '1' });
            }
            else if (type == 'shipping') {
                filter = Object.assign(filter, { isshippingaddress: '1' });
            }
        }
        return await this.repository.findAll({
            where: filter,
            include: [
                {
                    model: Country_entity_1.Country,
                    as: 'country',
                    attributes: ['country']
                }
            ],
            order: [['addressid', 'DESC']]
        });
    }
    async findOne(options) {
        let result = await this.repository.findOne(options);
        if (!result) {
            throw new Process_exception_1.ProcessException('validate:field:notFound');
        }
        return result;
    }
    async findById(id, user) {
        let result = await this.repository.findOne({
            where: {
                userid: user.userId,
                addressid: id
            }
        });
        if (!result) {
            throw new Process_exception_1.ProcessException('validate:field:notFound');
        }
        return result;
    }
    async update(id, newValue, user) {
        let result = await this.findById(id, user);
        await this._buildObject(result, newValue).save();
        return result;
    }
    async delete(id, user) {
        let result = await this.findById(id, user);
        return await result.destroy();
    }
    _buildObject(entity, dto, user) {
        entity.name = dto.name;
        entity.address = dto.address;
        entity.city = dto.city;
        entity.countryid = dto.countryId;
        entity.postalcode = dto.postalCode;
        entity.contactno1 = dto.primary_contactno;
        entity.contactno2 = dto.alternative_contactno;
        entity.isshippingaddress = (dto.isshippingaddress) ? '1' : '0';
        entity.isbillingaddress = (dto.isbillingaddress) ? '1' : '0';
        if (user) {
            entity.userid = user.userId;
            entity.channelid = user.channelId;
        }
        return entity;
    }
};
MyAddressBookService = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject('MyAddressBookRepository')),
    __param(1, common_1.Inject('SequelizeInstance')),
    __metadata("design:paramtypes", [Object, Object])
], MyAddressBookService);
exports.MyAddressBookService = MyAddressBookService;
//# sourceMappingURL=myAddressBook.service.js.map