"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserService = void 0;
const common_1 = require("@nestjs/common");
const User_entity_1 = require("../models/User.entity");
const context_service_1 = require("./../../../context/context.service");
const Channel_entity_1 = require("../../channel/models/Channel.entity");
const AdminMenu_entity_1 = require("../../admin/models/AdminMenu.entity");
const AdminUserCountries_entity_1 = require("../../admin/models/AdminUserCountries.entity");
const RegionCountryJCT_entity_1 = require("../../common/models/RegionCountryJCT.entity");
const Country_entity_1 = require("../../common/models/Country.entity");
const Media_entity_1 = require("../../media/models/Media.entity");
const app_constant_1 = require("../../../config/app.constant");
const userInBound_1 = require("../transformation/userInBound");
const UserRole_entity_1 = require("../models/UserRole.entity");
const SocialSite_entity_1 = require("./../../channel/models/SocialSite.entity");
const ChannelContactSuggestion_entity_1 = require("../../channel/models/ChannelContactSuggestion.entity");
const base_service_1 = require("../../../core/service/base.service");
const Role_entity_1 = require("../models/Role.entity");
const ContactJCT_entity_1 = require("../../channel/models/ContactJCT.entity");
const sequelize_1 = require("sequelize");
const SubscriptionPlan_entity_1 = require("../../channel/models/SubscriptionPlan.entity");
const ChannelSubscription_entity_1 = require("../../channel/models/ChannelSubscription.entity");
const ChannelDetail_entity_1 = require("../../channel/models/ChannelDetail.entity");
const Subscription_entity_1 = require("../../channel/models/Subscription.entity");
const Process_exception_1 = require("../../../middleware/exception/Process.exception");
const ProfileList_entity_1 = require("../../channel/models/ProfileList.entity");
let UserService = class UserService {
    constructor(sequelizeInstance, userRoleRepository, contactLocationRepository, userRepository, contactRepository, channelRepository, adminUserRepository, adminUserAccessRepository, mediaRepository, contactProductGroupRepository, contactInterestedProductGroupRepository, contactInterestedProductCategoryRepository, interestedProductGroupRepository, interestedProductCategoryRepository, contactSocialLinkRepository, contactProductCategoryRepository, userAccessPermissionRepository, contactSuggestionRepository, countryRepository, context) {
        this.sequelizeInstance = sequelizeInstance;
        this.userRoleRepository = userRoleRepository;
        this.contactLocationRepository = contactLocationRepository;
        this.userRepository = userRepository;
        this.contactRepository = contactRepository;
        this.channelRepository = channelRepository;
        this.adminUserRepository = adminUserRepository;
        this.adminUserAccessRepository = adminUserAccessRepository;
        this.mediaRepository = mediaRepository;
        this.contactProductGroupRepository = contactProductGroupRepository;
        this.contactInterestedProductGroupRepository = contactInterestedProductGroupRepository;
        this.contactInterestedProductCategoryRepository = contactInterestedProductCategoryRepository;
        this.interestedProductGroupRepository = interestedProductGroupRepository;
        this.interestedProductCategoryRepository = interestedProductCategoryRepository;
        this.contactSocialLinkRepository = contactSocialLinkRepository;
        this.contactProductCategoryRepository = contactProductCategoryRepository;
        this.userAccessPermissionRepository = userAccessPermissionRepository;
        this.contactSuggestionRepository = contactSuggestionRepository;
        this.countryRepository = countryRepository;
        this.context = context;
        this.userInBound = new userInBound_1.UserInBound();
        this.specialChars = ['@', '#', '%', '$', '*'];
    }
    async findUser(email) {
        console.log(email);
        const user = await this.userRepository.scope('userLogInInfom').findOne({
            include: [
                {
                    model: UserRole_entity_1.UserRole,
                    as: 'role',
                    attributes: ['roleId']
                },
                {
                    model: this.channelRepository.scope('basicInfom'),
                    required: true,
                    where: {
                        isActive: '1',
                        signUpStatusId: 15,
                        deletedAt: null
                    },
                    include: [
                        {
                            model: SubscriptionPlan_entity_1.SubscriptionPlan,
                            as: 'plan',
                            attributes: ['subscriptionPlanId', 'subscriptionPlanType', 'rankOrder']
                        },
                        {
                            model: ChannelSubscription_entity_1.ChannelSubscription,
                            attributes: ['isActive', 'subscriptionId', 'startedAt', 'endAt', 'keyFeature'],
                            where: { isActive: '1' },
                            required: false,
                            include: [
                                {
                                    model: Subscription_entity_1.Subscription,
                                    as: 'subscription',
                                    attributes: ['subscriptionPeriodId']
                                }
                            ]
                        }
                    ]
                }, {
                    model: this.contactRepository.scope('contactInfo'),
                    required: true,
                    include: [{
                            model: this.mediaRepository.scope('profileInfo')
                        }]
                }
            ],
            where: {
                email: {
                    [sequelize_1.Op.iLike]: '%' + email + '%'
                },
                deletedAt: null
            }
        });
        if (user) {
            this.userRepository.update({ lastLogin: Date.now() }, {
                where: {
                    userId: user.userId
                }
            });
        }
        return user;
    }
    async findUserById(userId, channelId) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        const user = await this.userRepository.scope('userLogInInfom').findOne({
            include: [
                {
                    model: UserRole_entity_1.UserRole,
                    as: 'role',
                    attributes: ['roleId']
                },
                {
                    model: this.channelRepository.scope('basicInfom'),
                    required: true,
                    where: {
                        isActive: '1',
                        deletedAt: null
                    },
                    include: [
                        {
                            model: SubscriptionPlan_entity_1.SubscriptionPlan,
                            as: 'plan',
                            attributes: ['subscriptionPlanId', 'subscriptionPlanType', 'rankOrder']
                        },
                        {
                            model: ChannelSubscription_entity_1.ChannelSubscription,
                            attributes: ['isActive', 'subscriptionId', 'startedAt', 'endAt', 'keyFeature'],
                            where: { isActive: '1' },
                            required: false,
                            include: [
                                {
                                    model: Subscription_entity_1.Subscription,
                                    as: 'subscription',
                                    attributes: ['subscriptionPeriodId']
                                }
                            ]
                        }
                    ]
                }, {
                    model: this.contactRepository.scope('contactInfo'),
                    required: true,
                    include: [{
                            model: this.mediaRepository.scope('profileInfo')
                        }]
                }
            ],
            where: { userId: userId, channelId: channelId }
        });
        let userData = {
            userId: user.userId,
            contactId: user.contactId,
            channelId: user.channelId,
            email: user.email,
            isActive: user.isActive,
            planId: (_a = user.channel) === null || _a === void 0 ? void 0 : _a.planId,
            showId: (_b = user.channel) === null || _b === void 0 ? void 0 : _b.showId,
            signUpStatusId: (_c = user.channel) === null || _c === void 0 ? void 0 : _c.signUpStatusId,
            channelTypeId: (_d = user.channel) === null || _d === void 0 ? void 0 : _d.channelTypeId,
            profileTypeId: (_e = user.channel) === null || _e === void 0 ? void 0 : _e.profileTypeId,
            businessType: +((_f = user.channel) === null || _f === void 0 ? void 0 : _f.isOEM),
            lastLogin: user.lastLogin,
            isAlgorithm: user.isAlgorithm,
            firstName: (_g = user.contact) === null || _g === void 0 ? void 0 : _g.firstName,
            lastName: (_h = user.contact) === null || _h === void 0 ? void 0 : _h.lastName,
            document: ((_j = user.contact) === null || _j === void 0 ? void 0 : _j.document) ? user.contact.document : {},
            channel: user.channel,
            roleId: (_k = user.role) === null || _k === void 0 ? void 0 : _k.roleId,
            suggestion_comm: user.suggestion_comm,
        };
        return userData;
    }
    async boUser(id) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        const user = await this.userRepository.scope('userLogInInfom').findOne({
            include: [
                {
                    model: UserRole_entity_1.UserRole,
                    as: 'role',
                    attributes: ['roleId']
                },
                {
                    model: this.channelRepository.scope('basicInfom'),
                    required: true,
                    where: {
                        isActive: '1',
                        deletedAt: null
                    },
                    include: [
                        {
                            model: SubscriptionPlan_entity_1.SubscriptionPlan,
                            as: 'plan',
                            attributes: ['subscriptionPlanId', 'subscriptionPlanType', 'rankOrder']
                        },
                        {
                            model: ChannelSubscription_entity_1.ChannelSubscription,
                            attributes: ['isActive', 'subscriptionId', 'startedAt', 'endAt', 'keyFeature'],
                            where: { isActive: '1' },
                            required: false,
                            include: [
                                {
                                    model: Subscription_entity_1.Subscription,
                                    as: 'subscription',
                                    attributes: ['subscriptionPeriodId']
                                }
                            ]
                        }
                    ]
                }, {
                    model: this.contactRepository.scope('contactInfo'),
                    required: true,
                    include: [{
                            model: this.mediaRepository.scope('profileInfo')
                        }]
                }
            ],
            where: { contactId: id }
        });
        if (user) {
            this.userRepository.update({ lastLogin: Date.now() }, {
                where: {
                    userId: user.userId
                }
            });
        }
        let userData = {};
        userData = {
            userId: user.userId,
            contactId: user.contactId,
            channelId: user.channelId,
            email: user.email,
            isActive: user.isActive,
            planId: (_a = user.channel) === null || _a === void 0 ? void 0 : _a.planId,
            showId: (_b = user.channel) === null || _b === void 0 ? void 0 : _b.showId,
            signUpStatusId: (_c = user.channel) === null || _c === void 0 ? void 0 : _c.signUpStatusId,
            channelTypeId: (_d = user.channel) === null || _d === void 0 ? void 0 : _d.channelTypeId,
            profileTypeId: (_e = user.channel) === null || _e === void 0 ? void 0 : _e.profileTypeId,
            businessType: +((_f = user.channel) === null || _f === void 0 ? void 0 : _f.isOEM),
            lastLogin: user.lastLogin,
            isAlgorithm: user.isAlgorithm,
            firstName: (_g = user.contact) === null || _g === void 0 ? void 0 : _g.firstName,
            lastName: (_h = user.contact) === null || _h === void 0 ? void 0 : _h.lastName,
            document: ((_j = user.contact) === null || _j === void 0 ? void 0 : _j.document) ? user.contact.document : {},
            channel: user.channel,
            roleId: (_k = user.role) === null || _k === void 0 ? void 0 : _k.roleId,
            contact: user.contact,
            suggestion_comm: user.suggestion_comm,
        };
        if (!userData) {
        }
        return userData;
    }
    async findAdmin(email) {
        const user = await this.adminUserRepository.scope().findOne({
            where: { emailId: email }
        });
        return user;
    }
    async findOneById(payload) {
        if (payload.userId)
            return await this.userRepository.findOne({ where: { userId: payload.userId } });
        else
            return await this.adminUserRepository.findOne({ where: { adminUserId: payload.adminUserId } });
    }
    async addUser(userDto, channelId, showId, userId, channelTypeId) {
        let password = await this.getPassword();
        userDto['documentId'] = userDto.contact.documentId;
        let uid;
        let userObject = await this.userInBound.setAddUserObject(userDto, channelId, password, showId, userId);
        let result = await this.sequelizeInstance.transaction(async (transaction) => {
            let user = await this.userRepository.create(userObject, {
                include: [{
                        model: this.contactRepository,
                    }, {
                        model: this.userRoleRepository
                    }],
                transaction,
            });
            uid = user;
            await this.relations(transaction, userDto, user, 'create', true);
            let ChannelcontactsuggestionObject = await this.userInBound.setChannelContactSuggestion(app_constant_1.appconstant.VISIBILITY[+channelTypeId], user.contactId, channelId, user.userId, []);
            if (ChannelcontactsuggestionObject.length) {
                await this.contactSuggestionRepository.bulkCreate(ChannelcontactsuggestionObject, { returning: true, transaction });
            }
            user.contact.empId = showId + user.userId;
            await user.contact.save();
            return user;
        }).then(function (result) {
            Object.assign(result, { password: password });
            return result;
        }).catch(function (e) {
            console.log(e);
        });
        return uid;
    }
    async updateUser(userDto, user, interestedFamily = false, real_user) {
        var activeIds = [];
        var realUser = real_user;
        await this.sequelizeInstance.transaction(async (transaction) => {
            if (userDto.contact && userDto.contact.role && user.role && user.role.roleId != Role_entity_1.Role.ROLE_ACCOUNT_USER) {
                var roles = [
                    Role_entity_1.Role.ROLE_ADMIN_USER,
                    Role_entity_1.Role.ROLE_PRIMARY_USER
                ];
                if (realUser && +realUser.channelTypeId === +app_constant_1.appconstant.CHANNEL_TYPE_HUB) {
                    roles.push(Role_entity_1.Role.ROLE_ACCOUNT_USER);
                    if (userDto.contact.role == Role_entity_1.Role.ROLE_ACCOUNT_USER) {
                        userDto.contact.workNatureId = null;
                    }
                }
                if (!roles.includes(userDto.contact.role)) {
                    userDto.contact.role = Role_entity_1.Role.ROLE_PRIMARY_USER;
                }
                user.role.roleId = userDto.contact.role;
                await user.role.save({
                    transaction
                });
            }
            await this.userRepository.update({
                invitation_comm: +userDto.contact.invitation_comm,
                suggestion_comm: +userDto.contact.suggestion_comm
            }, {
                where: {
                    contactId: user.contactId
                },
                transaction
            }).then(x => console.log('saved', user.contactId));
            let contact = await this.contactRepository.findOne({
                where: {
                    contactId: user.contactId
                },
                include: [
                    {
                        model: this.mediaRepository,
                        required: false
                    }
                ]
            });
            if (userDto.contact && Object.keys(userDto.contact).length) {
                if (userDto.contact.phoneNo) {
                    userDto.contact.phone1 = userDto.contact.phoneNo;
                }
                if (+userDto.contact.rptContactId === 0) {
                    userDto.contact.rptContactId = null;
                }
                if (userDto.contact.documentId) {
                    userDto.contact.documentId = userDto.contact.documentId;
                }
                userDto.contact.isActive = (userDto.contact.isActive) ? userDto.contact.isActive : '1';
                var userUpdate = false;
                if (userDto.contact.email != contact.email) {
                    userUpdate = true;
                }
                else {
                    userDto.contact.email = contact.email;
                }
                await this.contactRepository.update(userDto.contact, {
                    where: {
                        contactId: user.contactId
                    },
                    transaction: transaction
                });
            }
            await this.relations(transaction, userDto, user, 'update', interestedFamily);
            if (userDto.channelContactSuggestion) {
                activeIds = [];
                let nin = [];
                userDto.channelContactSuggestion.forEach((r) => {
                    nin.push(r.id);
                });
                await this.contactSuggestionRepository.update({
                    deletedat: new Date()
                }, {
                    where: {
                        contactid: user.contactId,
                        channelid: user.channelId,
                        sharetypeid: {
                            [sequelize_1.Op.notIn]: nin
                        }
                    }, transaction: transaction
                });
                await this.contactSuggestionRepository.findAll({
                    where: {
                        contactid: user.contactId,
                        channelid: user.channelId,
                        deletedat: null
                    }
                }).then((r) => {
                    r.forEach((d) => {
                        activeIds.push(+d.sharetypeid);
                    });
                });
                let ChannelcontactsuggestionObject = await this.userInBound.setChannelContactSuggestion(userDto.channelContactSuggestion, user.contactId, user.channelId, user.userId, activeIds);
                if (ChannelcontactsuggestionObject.length) {
                    await this.contactSuggestionRepository.bulkCreate(ChannelcontactsuggestionObject, { returning: true, transaction });
                }
            }
            if (userUpdate) {
                await this.userRepository.update({
                    email: userDto.contact.email,
                }, {
                    where: {
                        contactId: +user.contactId
                    },
                    transaction
                });
            }
            if (+realUser.channel.signUpStatusId == app_constant_1.appconstant.CHANNEL_APPROVAL || userDto.ratingId) {
                let defaultdata = {
                    channelId: +realUser.channel.channelId,
                    ratingId: 2,
                    isCompleted: true,
                    createdAt: new Date()
                };
                await ProfileList_entity_1.ProfileList.findOrCreate({ where: { channelId: +realUser.channel.channelId, ratingId: 2 }, defaults: defaultdata }).then((result) => {
                    if (!result[1]) {
                        result[0].update(defaultdata);
                    }
                });
            }
            return user;
        }).then(async (result) => {
            if (userDto.contact.isActive && userDto.contact.isActive == '0') {
                let contactJCT = await ContactJCT_entity_1.ContactJCT.findAll({
                    where: {
                        partnerContactId: user.contactId
                    }
                });
                if (contactJCT.length > 0) {
                    for (var i = 0; i < contactJCT.length; i++) {
                        let notificationObject = {
                            senderId: +user.contactId,
                            receiverId: contactJCT[i].contactId
                        };
                    }
                }
            }
            return result;
        }).catch(function (e) {
            console.log("updateerror:", e);
        });
        return true;
    }
    async getPassword() {
        var generator = require('generate-password');
        var password = generator.generate({
            length: 9,
            numbers: true,
            uppercase: true,
            strict: true
        });
        return password + this.specialChars[Math.floor(Math.random() * this.specialChars.length)];
    }
    async relations(transaction, userDto, user, action, interestedFamily = false) {
        var activeIds = [];
        if (userDto.contact) {
            console.log("link:", userDto);
            let contactSocialLinkObject = await this.userInBound.setSocialLinks(userDto, user.contactId);
            if (contactSocialLinkObject.length) {
                if (action == 'update') {
                    let fin = [];
                    if (userDto.contact.linkedInUrl !== undefined) {
                        fin.push(SocialSite_entity_1.SocialSite.SOCIALSITE_LINKEDIN);
                    }
                    if (userDto.contact.googlePlusUrl !== undefined) {
                        fin.push(SocialSite_entity_1.SocialSite.SOCIALSITE_GOOGLEPLUS);
                    }
                    await this.contactSocialLinkRepository.destroy({
                        where: {
                            contactId: user.contactId,
                            socialSiteId: {
                                [sequelize_1.Op.in]: fin
                            }
                        }, transaction: transaction
                    });
                }
                await this.contactSocialLinkRepository.bulkCreate(contactSocialLinkObject, { returning: true, transaction });
            }
        }
        if (userDto.productFamilies) {
            activeIds = [];
            if (action == 'update') {
                let nin = [];
                userDto.productFamilies.forEach((r) => {
                    nin.push(r.id);
                });
                await this.interestedProductGroupRepository.update({
                    deletedAt: new Date()
                }, {
                    where: {
                        contactId: user.contactId,
                        productGroupId: {
                            [sequelize_1.Op.notIn]: nin
                        }
                    }, transaction: transaction
                });
                await this.interestedProductGroupRepository.findAll({
                    where: {
                        contactId: user.contactId,
                        deletedAt: null
                    }
                }).then((r) => {
                    r.forEach((d) => {
                        activeIds.push(+d.productGroupId);
                    });
                });
            }
            let InterestedProductGroupObject = await this.userInBound.setInterestedProductGroup(userDto.productFamilies, user.channelId, user.contactId, activeIds);
            if (InterestedProductGroupObject.length) {
                await this.interestedProductGroupRepository.bulkCreate(InterestedProductGroupObject, { returning: true, transaction });
            }
        }
        else {
            await this.interestedProductGroupRepository.update({
                deletedAt: new Date()
            }, {
                where: {
                    contactId: user.contactId
                }, transaction: transaction
            });
        }
        if (userDto.productCategories) {
            activeIds = [];
            if (action == 'update') {
                let nin = [];
                userDto.productCategories.forEach((r) => {
                    nin.push(r.productCategoriesId);
                });
                await this.interestedProductCategoryRepository.update({
                    deletedAt: new Date()
                }, {
                    where: {
                        contactId: user.contactId,
                        productCategorieId: {
                            [sequelize_1.Op.notIn]: nin
                        }
                    }, transaction: transaction
                });
                await this.interestedProductCategoryRepository.findAll({
                    where: {
                        contactId: user.contactId,
                        deletedAt: null
                    }
                }).then((r) => {
                    r.forEach((d) => {
                        activeIds.push(+d.productCategorieId);
                    });
                });
            }
            let InterestedProductCategorieObject = await this.userInBound.setInterestedProductCategories(userDto.productCategories, user.channelId, user.contactId, activeIds);
            if (InterestedProductCategorieObject.length) {
                await this.interestedProductCategoryRepository.bulkCreate(InterestedProductCategorieObject, { returning: true, transaction });
            }
        }
        else {
            await this.interestedProductCategoryRepository.update({
                deletedAt: new Date()
            }, {
                where: {
                    contactId: user.contactId
                }, transaction: transaction
            });
        }
        if (userDto.accessPermissions) {
            activeIds = [];
            if (action == 'update') {
                let nin = [];
                userDto.accessPermissions.forEach((r) => {
                    nin.push(r.id);
                });
                await this.userAccessPermissionRepository.update({
                    deletedat: new Date()
                }, {
                    where: {
                        userid: user.userId,
                        accesspermissionid: {
                            [sequelize_1.Op.notIn]: nin
                        }
                    }, transaction: transaction
                });
                await this.userAccessPermissionRepository.findAll({
                    where: {
                        userid: user.userId,
                        deletedat: null
                    }
                }).then((r) => {
                    r.forEach((d) => {
                        activeIds.push(+d.accesspermissionid);
                    });
                });
            }
            let UserAccessPermissionObject = await this.userInBound.setAccessPermissions(userDto.accessPermissions, user.userId, activeIds);
            if (UserAccessPermissionObject.length) {
                await this.userAccessPermissionRepository.bulkCreate(UserAccessPermissionObject, { returning: true, transaction });
            }
            await User_entity_1.User.update({
                tokenModifiedAt: this.sequelizeInstance.fn('NOW')
            }, {
                where: {
                    userId: user.userId
                },
                returning: true, transaction
            });
        }
        if (userDto.locations) {
            if (action == 'update') {
                await this.contactLocationRepository.update({
                    deletedAt: new Date()
                }, {
                    where: {
                        contactId: user.contactId
                    }, transaction: transaction
                });
            }
            let ContactLocationObject = await this.userInBound.setContactLocations(userDto.locations, user.channelId, user.contactId);
            if (ContactLocationObject.length) {
                await this.contactLocationRepository.bulkCreate(ContactLocationObject, { returning: true, transaction });
            }
        }
    }
    async getPreference(typeid) {
        let ids = (typeid == 2) ? [50, 21, 25, 1] : [21, 25, 42, 1];
        let left = [];
        let right = [];
        let children;
        let query = 'select "accesspermissionid" as id,"name","parentaccesspermissionid" as parent from "tblaccesspermission" where ("parentaccesspermissionid" in(:Ids) or "accesspermissionid" in(:Ids)) and "deletedat" is null and "isactive"=\'1\' ';
        let result = await this.sequelizeInstance.query(query, {
            type: sequelize_1.QueryTypes.SELECT,
            replacements: {
                Ids: ids
            }
        }).then(results => {
            return results;
        });
        let child = result;
        if (typeid == 2) {
            result.map(x => {
                if (x.parent == null && (x.id == 50 || x.id == 21)) {
                    children = [];
                    child.map(c => {
                        if (c.parent == x.id) {
                            let obj = {
                                title: c.name,
                                key: c.id,
                                selectable: true
                            };
                            children.push(obj);
                        }
                    });
                    left.push({ title: x.name, key: x.id, selectable: true, children: children });
                }
                if (x.parent == null && (x.id == 25 || x.id == 1)) {
                    children = [];
                    child.map(c => {
                        if (c.parent == x.id) {
                            let obj = {
                                title: c.name,
                                key: c.id,
                                selectable: true
                            };
                            children.push(obj);
                        }
                    });
                    right.push({ title: x.name, key: x.id, selectable: true, children: children });
                }
            });
        }
        else {
            result.map(x => {
                if (x.parent == null && (x.id == 21 || x.id == 25)) {
                    children = [];
                    child.map(c => {
                        if (c.parent == x.id) {
                            let obj = {
                                title: c.name,
                                key: c.id,
                                selectable: true
                            };
                            children.push(obj);
                        }
                    });
                    left.push({ title: x.name, key: x.id, selectable: true, children: children });
                }
                if (x.parent == null && (x.id == 42 || x.id == 1)) {
                    children = [];
                    child.map(c => {
                        if (c.parent == x.id) {
                            let obj = {
                                title: c.name,
                                key: c.id,
                                selectable: true
                            };
                            children.push(obj);
                        }
                    });
                    right.push({ title: x.name, key: x.id, selectable: true, children: children });
                }
            });
        }
        let data = {
            left: left,
            right: right
        };
        return data;
    }
    async getMailPreference() {
        let mail = [
            { title: 'Channel Approve', key: 2, selectable: true }, { title: 'Channel Dismiss', key: 3, selectable: true }, { title: 'Connection Approve', key: 4, selectable: true },
            { title: 'Connection Dismiss', key: 5, selectable: true }
        ];
        let left = [];
        let right = [];
        left.push({ title: 'Email', key: 1, selectable: true, children: mail });
        let data = {
            left: left,
            right: right
        };
        return data;
    }
    async getUserPreference(contactId, channelId) {
        let query = 'select cast("accesspermissionid" as varchar) from "tbluseraccesspermission" as up where "deletedat" is null and "userid" in(select "userId" from "tblUser" where "contactId"=:contactId and "channelId"=:channelId) ';
        let result = await this.sequelizeInstance.query(query, {
            type: sequelize_1.QueryTypes.SELECT,
            replacements: {
                contactId: contactId,
                channelId: channelId
            }
        }).then(results => {
            return results.map(x => x.accesspermissionid);
        });
        return result;
    }
    async getUserPrevilege(contactId) {
        let query = 'select cast("accesspermissionid" as varchar),(select "name" from "tblaccesspermission" where "accesspermissionid"=up."accesspermissionid") from "tbluseraccesspermission" as up where "deletedat" is null and "userid" in(select "userId" from "tblUser" where "contactId"=:contactId) ';
        let result = await this.sequelizeInstance.query(query, {
            type: sequelize_1.QueryTypes.SELECT,
            replacements: {
                contactId: contactId
            }
        }).then(results => {
            return results.map(x => x.accesspermissionid);
        });
        return result;
    }
    async getUser(id, channelId) {
        let user = await this.userRepository.findOne({
            where: {
                contactId: id,
                channelId: channelId
            },
            include: [
                {
                    model: UserRole_entity_1.UserRole,
                    required: true
                }
            ]
        });
        if (!user) {
            console.log("error:", user, id, channelId);
        }
        return user;
    }
    async getDefaultContact(channelId) {
        let user = await this.contactRepository.findOne({
            attributes: ['firstName', 'lastName', 'jobTitle'],
            where: {
                channelId: channelId,
                isDefaultContact: '1'
            },
            include: [
                {
                    model: Country_entity_1.Country,
                    attributes: ['country']
                },
                {
                    model: Media_entity_1.Media,
                    attributes: ['documentPath']
                }
            ]
        });
        let no_of_users = await this.contactRepository.count({
            where: {
                channelId: channelId,
            }
        });
        let contact = {
            user: user,
            userCount: no_of_users
        };
        return contact;
    }
    async getAllContacts(channelId) {
        let user = await this.contactRepository.findAll({
            attributes: ['contactId', 'firstName', 'lastName', 'jobTitle'],
            where: {
                channelId: channelId,
                deletedAt: null,
                isActive: '1'
            },
            include: [
                {
                    model: Media_entity_1.Media,
                    attributes: ['documentPath']
                },
                {
                    model: Country_entity_1.Country,
                    attributes: ['country']
                },
                {
                    model: Channel_entity_1.Channel,
                    attributes: ['companyName'],
                    include: [
                        {
                            model: ChannelDetail_entity_1.ChannelDetail,
                            attributes: ['channelLogoId'],
                            include: [
                                {
                                    model: Media_entity_1.Media,
                                    as: 'channelLogo',
                                    attributes: ['documentPath']
                                }
                            ]
                        }
                    ]
                },
            ]
        });
        return user;
    }
    async setDefaultContact(channelId, contactId) {
        await this.contactRepository.update({ isDefaultContact: '0' }, {
            where: {
                channelId: channelId,
            },
        });
        await this.contactRepository.update({ isDefaultContact: '1' }, {
            where: {
                contactId: contactId,
            },
        });
    }
    async Countries() {
        const countries = await Country_entity_1.Country.findAll({
            attributes: [['countryId', 'id'], ['country', 'name']]
        });
        return countries;
    }
    async getContact(Id) {
        try {
            let Contact = await this.contactRepository.scope('contactInfo').findOne({
                include: [
                    {
                        model: this.countryRepository,
                        attributes: ['isoCode', 'country', 'countryId'],
                        required: false
                    },
                    {
                        model: this.mediaRepository,
                        attributes: ['documentUrl', 'documentPath', 'documentId'],
                        required: false
                    },
                    {
                        model: User_entity_1.User,
                        as: 'user',
                        required: true,
                        attributes: ['userId']
                    }
                ], where: { contactId: Id }
            });
            return Contact;
        }
        catch (e) {
            console.log("Error+", e);
            throw new common_1.InternalServerErrorException();
        }
    }
    async updateContact(contactId, contactData) {
        let contact = await this.contactRepository.findOne({
            where: {
                contactId: contactId
            }
        });
        if (!contact)
            throw new Process_exception_1.ProcessException('User not found');
        if (contactData) {
            let contact = {};
            if (contactData.firstName)
                contact['firstName'] = contactData.firstName;
            if (contactData.lastName)
                contact['lastName'] = contactData.lastName;
            if (contactData.email)
                contact['email'] = contactData.email;
            if (contactData.country)
                contact['countryId'] = contactData.country.id;
            if (contactData.phone)
                contact['phone1'] = contactData.phone;
            if (contactData.mobileNo)
                contact['mobileNo'] = contactData.mobileNo;
            if (contactData.jobTitle)
                contact['jobTitle'] = contactData.jobTitle;
            if (+contactData.rptContactId === 0) {
                contact['rptContactId'] = null;
            }
            else {
                contact['rptContactId'] = +contactData.rptContactId;
            }
            await this.contactRepository.update(contact, {
                where: {
                    contactId: contactId
                }
            });
        }
    }
    async isAlgorithm(userId) {
        this.userRepository.update({ isAlgorithm: '1' }, {
            where: {
                userId: userId
            }
        });
    }
    async checkEmail(email, userId) {
        let result = await this.userRepository.findOne({
            where: {
                email: {
                    [sequelize_1.Op.iLike]: '%' + email + '%'
                },
                userId: {
                    [sequelize_1.Op.not]: +userId
                },
                deletedAt: null
            }
        });
        console.log(result);
        if (!result) {
            return true;
        }
        return false;
    }
};
UserService = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject('SequelizeInstance')),
    __param(1, common_1.Inject('UserRoleRepository')),
    __param(2, common_1.Inject('ContactLocationRepository')),
    __param(3, common_1.Inject('UserRepository')),
    __param(4, common_1.Inject('ContactRepository')),
    __param(5, common_1.Inject('ChannelRepository')),
    __param(6, common_1.Inject('AdminUserRepository')),
    __param(7, common_1.Inject('AdminUserAccessRepository')),
    __param(8, common_1.Inject('MediaRepository')),
    __param(9, common_1.Inject('ContactProductGroupRepository')),
    __param(10, common_1.Inject('ContactInterestedProductGroupRepository')),
    __param(11, common_1.Inject('ContactInterestedProductCategoryRepository')),
    __param(12, common_1.Inject('InterestedProductGroupRepository')),
    __param(13, common_1.Inject('InterestedProductCategoryRepository')),
    __param(14, common_1.Inject('ContactSocialLinkRepository')),
    __param(15, common_1.Inject('ContactProductCategoryRepository')),
    __param(16, common_1.Inject('UserAccessPermissionRepository')),
    __param(17, common_1.Inject('ContactSuggestionRepository')),
    __param(18, common_1.Inject('CountryRepository')),
    __metadata("design:paramtypes", [Object, Object, Object, Object, Object, Object, Object, Object, Object, Object, Object, Object, Object, Object, Object, Object, Object, Object, Object, context_service_1.ContextService])
], UserService);
exports.UserService = UserService;
//# sourceMappingURL=user.service.js.map