'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserInBound = void 0;
const Process_exception_1 = require("../../../middleware/exception/Process.exception");
const Role_entity_1 = require("../../user/models/Role.entity");
const SocialSite_entity_1 = require("./../../channel/models/SocialSite.entity");
const lodash_1 = require("lodash");
class UserInBound {
    async setAddUserObject(user, channelId, password, showId, userId) {
        try {
            let contact = {};
            contact.firstName = user.contact.firstName;
            contact.lastName = user.contact.lastName;
            contact.email = user.contact.email;
            contact.countryId = user.contact.countryId;
            contact.channelId = channelId;
            contact.showId = showId;
            contact.userId = userId;
            contact.phone1 = (user.contact.phoneNo) ? user.contact.phoneNo : null;
            contact.mobileNo = (user.contact.mobileNo) ? user.contact.mobileNo : null;
            contact.jobTitle = user.contact.jobTitle;
            contact.documentId = user.documentId;
            contact.rptContactId = (+user.contact.rptContactId != 0) ? user.contact.rptContactId : null;
            contact.isActive = (user.contact.isActive) ? user.contact.isActive : '1';
            contact.isShared = user.contact.isShared;
            contact.workNatureId = user.contact.workNatureId;
            contact.createdAt = new Date();
            var roles = [
                Role_entity_1.Role.ROLE_ADMIN_USER,
                Role_entity_1.Role.ROLE_PRIMARY_USER
            ];
            if (!roles.includes(user.contact.role)) {
                user.contact.role = Role_entity_1.Role.ROLE_PRIMARY_USER;
            }
            let userRole = {};
            userRole.roleId = user.contact.role;
            let returnObject = {};
            returnObject.contact = contact;
            returnObject.channelId = channelId;
            returnObject.role = userRole;
            returnObject.email = user.contact.email;
            returnObject.password = password;
            returnObject.isActive = '1';
            returnObject.createdAt = new Date();
            returnObject.verificationCode = "0000000";
            returnObject.verificationSentAt = new Date();
            return returnObject;
        }
        catch (e) {
            throw new Process_exception_1.ProcessException('account:objectCreation:User');
        }
    }
    async setProductGroup(productFamilies, contactId, activeIds) {
        try {
            let resultObject = [];
            for (let ii = 0; ii < productFamilies.length; ii++) {
                if (!activeIds.includes(productFamilies[ii].id)) {
                    let innerObject = {};
                    innerObject.contactId = contactId;
                    innerObject.productGroupId = productFamilies[ii].id;
                    innerObject.createdAt = new Date();
                    resultObject.push(innerObject);
                }
            }
            return resultObject;
        }
        catch (e) {
            throw new Process_exception_1.ProcessException('channel:objectCreation:productGroup');
        }
    }
    async setChannelContactSuggestion(channelContactSuggestion, contactId, channelId, userId, activeIds) {
        try {
            let resultObject = [];
            for (let ii = 0; ii < channelContactSuggestion.length; ii++) {
                if (!activeIds.includes(channelContactSuggestion[ii].id)) {
                    let sharetypeid;
                    if (lodash_1.isObject(channelContactSuggestion[ii])) {
                        sharetypeid = channelContactSuggestion[ii].id;
                    }
                    else {
                        sharetypeid = channelContactSuggestion[ii];
                    }
                    let innerObject = {};
                    innerObject.channelid = channelId;
                    innerObject.contactid = contactId;
                    innerObject.sharetypeid = sharetypeid;
                    innerObject.updatedby = userId;
                    innerObject.createdat = new Date();
                    resultObject.push(innerObject);
                }
            }
            return resultObject;
        }
        catch (e) {
            throw new Process_exception_1.ProcessException('channel:objectCreation:channelContactSuggestion');
        }
    }
    async setInterestedProductGroup(productFamilies, channelId, contactId, activeIds) {
        try {
            let resultObject = [];
            for (let ii = 0; ii < productFamilies.length; ii++) {
                if (!activeIds.includes(+productFamilies[ii].id)) {
                    let innerObject = {};
                    innerObject.contactId = contactId;
                    innerObject.channelId = channelId;
                    innerObject.productGroupId = productFamilies[ii].id;
                    innerObject.createdAt = new Date();
                    resultObject.push(innerObject);
                }
            }
            return resultObject;
        }
        catch (e) {
            throw new Process_exception_1.ProcessException('channel:objectCreation:productGroup');
        }
    }
    async setProductCategories(productCategories, contactId, activeIds) {
        try {
            let resultObject = [];
            for (let ii = 0; ii < productCategories.length; ii++) {
                if (!activeIds.includes(productCategories[ii].productCategoriesId)) {
                    let innerObject = {};
                    innerObject.contactId = contactId;
                    innerObject.productCategorieId = productCategories[ii].productCategoriesId;
                    innerObject.createdAt = new Date();
                    resultObject.push(innerObject);
                }
            }
            return resultObject;
        }
        catch (e) {
            throw new Process_exception_1.ProcessException('channel:objectCreation:ProductCategories');
        }
    }
    async setInterestedProductCategories(productCategories, channelId, contactId, activeIds) {
        try {
            let resultObject = [];
            for (let ii = 0; ii < productCategories.length; ii++) {
                if (!activeIds.includes(+productCategories[ii].productCategoriesId)) {
                    let innerObject = {};
                    innerObject.contactId = contactId;
                    innerObject.channelId = channelId;
                    innerObject.productCategorieId = productCategories[ii].productCategoriesId;
                    innerObject.createdAt = new Date();
                    resultObject.push(innerObject);
                }
            }
            return resultObject;
        }
        catch (e) {
            console.log("err:", e);
            throw new Process_exception_1.ProcessException('channel:objectCreation:ProductCategories');
        }
    }
    async setAccessPermissions(accessPermissions, userId, activeIds) {
        try {
            let resultObject = [];
            for (let ii = 0; ii < accessPermissions.length; ii++) {
                if (!activeIds.includes(accessPermissions[ii].id)) {
                    let innerObject = {};
                    innerObject.userid = userId;
                    innerObject.accesspermissionid = accessPermissions[ii].id;
                    innerObject.createdAt = new Date();
                    resultObject.push(innerObject);
                }
            }
            return resultObject;
        }
        catch (e) {
            throw new Process_exception_1.ProcessException('channel:objectCreation:UserAccessPermission');
        }
    }
    async setContactLocations(locations, channelId, contactId) {
        try {
            let resultObject = [];
            for (let ii = 0; ii < locations.length; ii++) {
                for (let ii2 = 0; ii2 < locations[ii].location.length; ii2++) {
                    let innerObject = {};
                    innerObject.channelId = channelId;
                    innerObject.contactId = contactId;
                    innerObject.locationType = locations[ii].locationType;
                    innerObject.regionCountryJCTId = locations[ii].location[ii2];
                    innerObject.createdAt = new Date();
                    resultObject.push(innerObject);
                }
            }
            return resultObject;
        }
        catch (e) {
            console.log(e);
            throw new Process_exception_1.ProcessException('channel:objectCreation:ContactLocation');
        }
    }
    async setSocialLinks(user, contactId) {
        try {
            let resultObject = [];
            if (user.contact && user.contact.linkedInUrl !== undefined) {
                let innerObject = {};
                innerObject.contactId = contactId;
                innerObject.socialSiteId = SocialSite_entity_1.SocialSite.SOCIALSITE_LINKEDIN;
                innerObject.contactSocialLink = user.contact.linkedInUrl;
                innerObject.createdAt = new Date();
                resultObject.push(innerObject);
            }
            if (user.contact && user.contact.googlePlusUrl !== undefined) {
                let innerObject = {};
                innerObject.contactId = contactId;
                innerObject.socialSiteId = SocialSite_entity_1.SocialSite.SOCIALSITE_GOOGLEPLUS;
                innerObject.contactSocialLink = user.contact.googlePlusUrl;
                innerObject.createdAt = new Date();
                resultObject.push(innerObject);
            }
            return resultObject;
        }
        catch (e) {
            throw new Process_exception_1.ProcessException('channel:objectCreation:SocialLink');
        }
    }
}
exports.UserInBound = UserInBound;
//# sourceMappingURL=userInBound.js.map