'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IndexController = void 0;
const Channel_entity_1 = require("./../../channel/models/Channel.entity");
const common_1 = require("@nestjs/common");
const validation_pipe_1 = require("./../../../middleware/pipe/validation.pipe");
const product_dto_1 = require("./../dto/product.dto");
const productMove_dto_1 = require("./../dto/productMove.dto");
const wishlist_service_1 = require("./../service/wishlist.service");
const Auth_exception_1 = require("./../../../middleware/exception/Auth.exception");
const sequelize_1 = require("sequelize");
const group_dto_1 = require("../dto/group.dto");
const passport_1 = require("@nestjs/passport");
const base_controller_1 = require("../../../core/controller/base.controller");
const permissions_constants_1 = require("../../../config/permissions.constants");
let IndexController = class IndexController extends base_controller_1.BaseController {
    constructor(wishlistService) {
        super();
        this.wishlistService = wishlistService;
    }
    async _validateWishlistModule(user) {
        if (user.channelTypeId == Channel_entity_1.Channel.CHANNEL_TYPE_VENDOR) {
            throw new Auth_exception_1.AuthException('Not allowed');
        }
    }
    async getMywishlist(req, res, productFilters) {
        let user = req.user;
        await this._validateWishlistModule(user);
        let rejectedProductIds = await Channel_entity_1.Channel.sequelize.query('select array_agg(productid) as ids from tblproductrejection where channelid = ' + user.channelId, {
            type: sequelize_1.QueryTypes.SELECT,
            raw: true,
        }).then((result) => {
            return (result[0].ids) ? result[0].ids : [];
        });
        let productIds = [];
        await this.wishlistService.findAll(user.userId, productFilters.type, user.channelId, rejectedProductIds).then((products) => {
            products.forEach((product) => {
                productIds.push(product.productid);
            });
        });
        let results = [];
        let total = productIds.length;
        if (productIds.length > 0) {
            results = await this.wishlistService.getSuggestedProduct(productIds, productFilters, user);
        }
        let result = {
            results: results,
            total: total
        };
        res.status(common_1.HttpStatus.OK).send(result);
    }
    async addToMywishlist(req, res, product) {
        let user = req.user;
        await this._validateWishlistModule(user);
        await this.wishlistService.create({
            userid: +user.userId,
            channelid: +user.channelId,
            productid: product.id,
            type: product.type
        }, user);
        res.status(common_1.HttpStatus.CREATED).json({
            "status": true
        });
    }
    async removeFromMywishlist(req, res, id) {
        let user = req.user;
        await this._validateWishlistModule(user);
        await this.wishlistService.delete(id, 'WISHLIST', user);
        res.status(common_1.HttpStatus.NO_CONTENT).json({
            "status": true
        });
    }
    async getItemCount(req, res) {
        let user = req.user;
        let result = await this.wishlistService.count('WISHLIST', user.userId);
        result += await this.wishlistService.channelcount(user);
        res.status(common_1.HttpStatus.OK).json({
            "status": true,
            count: result
        });
    }
    async getProductCount(productId, req, res) {
        let user = req.user;
        let result = await this.wishlistService.productcount(user, productId);
        res.status(common_1.HttpStatus.OK).json({
            "status": true,
            count: result
        });
    }
    async MoveToOtherType(req, res, product) {
        let user = req.user;
        await this._validateWishlistModule(user);
        await this.wishlistService.move(user.channelId, user.userId, product, user);
        res.status(common_1.HttpStatus.OK).json({
            "status": true
        });
    }
    async addChannelGroup(req, res, group) {
        let user = req.user;
        let result = await this.wishlistService.createChannelGroup({
            userId: +user.userId,
            channelId: +user.channelId,
            groupName: group.groupName
        }, user);
        res.status(common_1.HttpStatus.CREATED).json({
            "status": result.channelGroupId
        });
    }
    async getChannelCount(req, res) {
        let user = req.user;
        let result = await this.wishlistService.channelcount(user);
        res.status(common_1.HttpStatus.OK).json({
            "status": true,
            count: result
        });
    }
    async channelGroupList(req, res) {
        let user = req.user;
        let group = await this.wishlistService.getGroupList(user);
        res.status(common_1.HttpStatus.OK).send(group);
    }
    async addFavoriteChannel(req, res, product) {
        let user = req.user;
        await this.wishlistService.createFavoriteChannel({
            userid: +user.userId,
            channelid: +product.channelid,
            channelgroupid: product.channelgroupid,
            type: product.type
        }, user, product.list_type);
        res.status(common_1.HttpStatus.CREATED).json({
            "status": true
        });
    }
    async removeFavoriteChannel(req, res, product) {
        let user = req.user;
        await this.wishlistService.removeFavoriteChannel({
            userid: +user.userId,
            channelid: +product.channelid,
            channelgroupid: product.channelgroupid
        }, user, product.list_type);
        res.status(common_1.HttpStatus.CREATED).json({
            "status": true
        });
    }
    async removeChannelGroup(req, res, product) {
        let user = req.user;
        await this.wishlistService.removeFavoriteGroup(product.channelgroupid);
        res.status(common_1.HttpStatus.NO_CONTENT).json({
            "status": true
        });
    }
};
__decorate([
    common_1.Post(''),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body(new validation_pipe_1.ValidationPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], IndexController.prototype, "getMywishlist", null);
__decorate([
    common_1.Post('add'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body(new validation_pipe_1.ValidationPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, product_dto_1.ProductDto]),
    __metadata("design:returntype", Promise)
], IndexController.prototype, "addToMywishlist", null);
__decorate([
    common_1.Delete(''),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Query('id', new common_1.ParseIntPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], IndexController.prototype, "removeFromMywishlist", null);
__decorate([
    common_1.Get('/count'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], IndexController.prototype, "getItemCount", null);
__decorate([
    common_1.Get('/productcount/:productId'),
    __param(0, common_1.Param('productId')), __param(1, common_1.Req()), __param(2, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], IndexController.prototype, "getProductCount", null);
__decorate([
    common_1.Put(''),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body(new validation_pipe_1.ValidationPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, productMove_dto_1.ProductMoveDto]),
    __metadata("design:returntype", Promise)
], IndexController.prototype, "MoveToOtherType", null);
__decorate([
    common_1.Post('group'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body(new validation_pipe_1.ValidationPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, group_dto_1.GroupDto]),
    __metadata("design:returntype", Promise)
], IndexController.prototype, "addChannelGroup", null);
__decorate([
    common_1.Get('channelcount'),
    __param(0, common_1.Req()), __param(1, common_1.Res()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], IndexController.prototype, "getChannelCount", null);
__decorate([
    common_1.Get('group'),
    __param(0, common_1.Req()), __param(1, common_1.Response()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], IndexController.prototype, "channelGroupList", null);
__decorate([
    common_1.Post('addchannel'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body(new validation_pipe_1.ValidationPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, product_dto_1.ProductDto]),
    __metadata("design:returntype", Promise)
], IndexController.prototype, "addFavoriteChannel", null);
__decorate([
    common_1.Put('removechannel'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body(new validation_pipe_1.ValidationPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, product_dto_1.ProductDto]),
    __metadata("design:returntype", Promise)
], IndexController.prototype, "removeFavoriteChannel", null);
__decorate([
    common_1.Put('removegroup'),
    __param(0, common_1.Req()), __param(1, common_1.Res()), __param(2, common_1.Body(new validation_pipe_1.ValidationPipe())),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, product_dto_1.ProductDto]),
    __metadata("design:returntype", Promise)
], IndexController.prototype, "removeChannelGroup", null);
IndexController = __decorate([
    common_1.Controller('favorite'),
    common_1.UseGuards(passport_1.AuthGuard('jwt')),
    __metadata("design:paramtypes", [wishlist_service_1.WishlistService])
], IndexController);
exports.IndexController = IndexController;
//# sourceMappingURL=index.controller.js.map