'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WishlistService = void 0;
const common_1 = require("@nestjs/common");
const Wishlist_entity_1 = require("../models/Wishlist.entity");
const Process_exception_1 = require("./../../../middleware/exception/Process.exception");
const Auth_exception_1 = require("./../../../middleware/exception/Auth.exception");
const Product_entity_1 = require("./../../product/models/Product.entity");
const ChannelJCT_entity_1 = require("./../../channel/models/ChannelJCT.entity");
const sequelize_1 = require("sequelize");
const base_service_1 = require("../../../core/service/base.service");
const app_constant_1 = require("../../../config/app.constant");
const ProductImage_entity_1 = require("../../product/models/ProductImage.entity");
const models_1 = require("../../media/models");
const ProductAward_entity_1 = require("../../product/models/ProductAward.entity");
const Brand_entity_1 = require("../../product/models/Brand.entity");
let WishlistService = class WishlistService extends base_service_1.BaseService {
    constructor(repository, grouprepository, evaluation, sequelizeInstance, productRepository, productImageRepository, mediaRepository, productAwardRepository, brandRepository) {
        super();
        this.repository = repository;
        this.grouprepository = grouprepository;
        this.evaluation = evaluation;
        this.sequelizeInstance = sequelizeInstance;
        this.productRepository = productRepository;
        this.productImageRepository = productImageRepository;
        this.mediaRepository = mediaRepository;
        this.productAwardRepository = productAwardRepository;
        this.brandRepository = brandRepository;
    }
    async create(data, user) {
        let product = await Product_entity_1.Product.findOne({
            where: {
                productId: data.productid,
                deletedat: null
            }
        });
        if (!product) {
            throw new Auth_exception_1.AuthException('Not allowed');
        }
        if (product.isshowtofreesub == '1' && user.planId == app_constant_1.appconstant.FREE_PLANID) {
            throw new Auth_exception_1.AuthException('Not allowed');
        }
        if (await this.validateRejectedChannels(+data.channelid, +product.channelId)) {
            throw new Auth_exception_1.AuthException('Not allowed');
        }
        try {
            return await this.repository.create(this._buildObject(new Object(), data)).then(async (result) => {
                return result;
            });
        }
        catch (e) {
            let message = 'validate:sql:internalError';
            if (e.errors && (e.errors[0].message == 'productid must be unique' || e.errors[0].message == 'type must be unique')) {
                message = 'this product already in ' + data.type;
            }
            else {
                console.log(e.errors);
            }
            throw new Process_exception_1.ProcessException(message);
        }
    }
    async createChannelGroup(data, user) {
        try {
            return await this.grouprepository.create(this._groupObject(new Object(), data)).then(async (result) => {
                return result;
            });
        }
        catch (e) {
            let message = 'validate:sql:internalError';
            throw new Process_exception_1.ProcessException(message);
        }
    }
    async createFavoriteChannel(data, user, list_type) {
        try {
            return await this.repository.create(this._favoriteObject(new Object(), data)).then(async (result) => {
                return result;
            });
        }
        catch (e) {
            console.log(e.message);
            let message = 'validate:sql:internalError';
            throw new Process_exception_1.ProcessException(e.message);
        }
    }
    async removeFavoriteChannel(data, user, list_type) {
        await this.repository.update({ deletedat: new Date() }, {
            where: {
                channelid: data.channelid,
                userid: data.userid,
                channelgroupid: data.channelgroupid,
                deletedat: null
            }
        });
    }
    async removeFavoriteGroup(id) {
        await this.grouprepository.update({ deletedAt: new Date() }, {
            where: {
                channelGroupId: id,
                deletedAt: null
            }
        });
    }
    async getGroupList(user) {
        let result = [];
        let gtotal = 0;
        let group = await this.grouprepository.findAll({
            where: {
                channelId: user.channelId,
                deletedAt: null
            },
            attributes: ['channelGroupId', 'channelName'],
            order: [['channelGroupId', 'DESC']]
        });
        for (var i = 0, len = group.length; i < len; i++) {
            let total = 0;
            total = await this.repository.count({
                where: {
                    channelgroupid: group[i].channelGroupId,
                    userid: user.userId,
                    deletedat: null
                }
            });
            if (total > 0) {
                gtotal = gtotal + total;
                result.push({ id: group[i].channelGroupId, name: group[i].channelName, count: total });
            }
        }
        let response = {
            result: result,
            gtotal: gtotal
        };
        return response;
    }
    async findAll(userId, type, channelId, rejectedProductIds) {
        let result;
        if (type == 'EVALVATED') {
            result = await this.evaluation.findAll({
                where: {
                    channelid: channelId,
                    productid: {
                        [sequelize_1.Op.not]: rejectedProductIds
                    }
                },
                attributes: ['productid'],
                order: [['productevaluationid', 'DESC']]
            });
        }
        else {
            result = await this.repository.findAll({
                where: {
                    userid: userId,
                    deletedat: null,
                    channelgroupid: null,
                    type: type,
                    productid: {
                        [sequelize_1.Op.not]: rejectedProductIds
                    }
                },
                attributes: ['productid'],
                order: [['productwishlistid', 'DESC']]
            });
        }
        return result;
    }
    async move(channelId, userId, product, user) {
        let count = await this.repository.count({
            where: {
                channelid: channelId,
                productid: product.id,
                type: product.to_type
            }
        });
        if (count > 0) {
            throw new Process_exception_1.ProcessException("Already in BUSINESS");
        }
        let productData = await Product_entity_1.Product.findOne({
            where: {
                productId: product.id,
                deletedat: null
            }
        });
        if (!product) {
            throw new Auth_exception_1.AuthException('Not allowed');
        }
        let evaluateQuery = 'select count(pe.productid) from "tblproductevaluation" pe where pe.channelid = :cahnnelId and pe."productid"= :productId';
        let evaluated = await this.sequelizeInstance.query(evaluateQuery, {
            type: this.sequelizeInstance.QueryTypes.SELECT,
            raw: true,
            replacements: {
                productId: productData.productId,
                cahnnelId: channelId
            }
        }).then(result => {
            return !!(+result[0].count);
        });
        if (!evaluated) {
            throw new Process_exception_1.ProcessException("Product not yet evaluated");
        }
        return await this.create({
            userid: userId,
            channelid: channelId,
            productid: productData.productId,
            type: 'BUSINESS'
        }, user).then(async (data) => {
            await ChannelJCT_entity_1.ChannelJCT.findOne({
                where: {
                    [sequelize_1.Op.or]: [
                        {
                            partnerChannelId: +user.channelId,
                            channelId: +productData.channelId
                        },
                        {
                            partnerChannelId: +productData.channelId,
                            channelId: +user.channelId
                        }
                    ]
                }
            }).then(async (result) => {
                if (result && result.channelJCTId) {
                    if (result.channelId == +user.channelId) {
                        result.leadManagementTag = 'High';
                    }
                    else if (result.partnerChannelId == +user.channelId) {
                        result.partnerLeadManagementTag = 'High';
                    }
                    await result.save();
                }
            });
        }).then(async (result) => {
        }).then(async (result) => {
            return result;
        });
    }
    async delete(id, type, user) {
        let result = await Wishlist_entity_1.Wishlist.destroy({
            where: {
                userid: +user.userId,
                productid: id,
                channelgroupid: null,
                deletedat: null,
                type: type
            }
        });
        if (!result) {
            throw new Process_exception_1.ProcessException('validate:field:notFound');
        }
        try {
        }
        catch (e) {
            throw new Process_exception_1.ProcessException('validate:sql:internalError');
        }
        return;
    }
    _buildObject(entity, dto) {
        entity.userid = dto.userid;
        entity.channelid = dto.channelid;
        entity.productid = dto.productid;
        entity.createdat = this.sequelizeInstance.fn('NOW');
        entity.type = dto.type;
        return entity;
    }
    _groupObject(entity, dto) {
        entity.userId = dto.userId;
        entity.channelId = dto.channelId;
        entity.channelName = dto.groupName;
        entity.createdAt = this.sequelizeInstance.fn('NOW');
        return entity;
    }
    _favoriteObject(entity, dto) {
        entity.userid = dto.userid;
        entity.channelid = dto.channelid;
        entity.channelgroupid = dto.channelgroupid;
        entity.createdat = this.sequelizeInstance.fn('NOW');
        entity.type = dto.type;
        return entity;
    }
    async count(type, userId) {
        return await this.repository.count({
            where: {
                type: type,
                userid: userId,
                deletedat: null
            },
            include: [{
                    model: Product_entity_1.Product,
                    where: {
                        deletedat: null,
                        isactive: '1'
                    }
                }]
        });
    }
    async channelcount(user) {
        var total = 0;
        total = await this.repository.count({
            where: {
                userid: user.userId,
                productid: null,
                deletedat: null
            }
        });
        return total;
    }
    async productcount(user, productId) {
        let total = 0;
        total = await this.repository.count({
            where: {
                channelid: user.channelId,
                productid: productId,
                type: 'WISHLIST',
                deletedat: null
            }
        });
        return total;
    }
    async getSuggestedProduct(ids, data, user) {
        console.log("ids:", ids);
        try {
            return this.productRepository.scope('productInfo').findAll({
                limit: data.limit,
                offset: data.offset,
                where: {
                    deletedat: null,
                    productId: {
                        [sequelize_1.Op.in]: ids
                    }
                },
                include: [
                    {
                        model: this.productImageRepository,
                        as: 'defaultImage',
                        attributes: ['productimageid', 'imageid', 'productimagetype', 'name', 'description'],
                        where: { deletedat: null },
                        required: false,
                        include: [{
                                model: this.mediaRepository,
                                attributes: ['documentUrl', 'documentPath'],
                                where: { deletedAt: null },
                                required: false,
                            }]
                    },
                    {
                        model: this.productAwardRepository,
                        attributes: ['awardDetail'],
                        required: false,
                        where: {
                            deletedAt: null,
                        },
                        include: [
                            {
                                model: this.mediaRepository,
                                attributes: ['documentPath'],
                                required: false,
                            }
                        ]
                    },
                    {
                        model: this.brandRepository,
                        attributes: ['brandName'],
                        required: false,
                        where: {
                            deletedAt: null,
                        },
                        include: [
                            {
                                model: this.mediaRepository,
                                attributes: ['documentPath'],
                                required: false,
                            }
                        ]
                    },
                    {
                        model: Wishlist_entity_1.Wishlist,
                        as: 'wishList',
                        attributes: ['productid'],
                        where: {
                            userid: user.userId,
                            deletedat: null
                        },
                        required: false,
                    }
                ]
            });
        }
        catch (error) {
            console.log('error', error);
            throw new Process_exception_1.ProcessException(error);
        }
    }
};
WishlistService = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject('WishlistRepository')),
    __param(1, common_1.Inject('ChannelGroupRepository')),
    __param(2, common_1.Inject('ProductEvaluationRepository')),
    __param(3, common_1.Inject('SequelizeInstance')),
    __param(4, common_1.Inject('ProductRepository')),
    __param(5, common_1.Inject('ProductImageRepository')),
    __param(6, common_1.Inject('MediaRepository')),
    __param(7, common_1.Inject('ProductAwardRepository')),
    __param(8, common_1.Inject('BrandRepository')),
    __metadata("design:paramtypes", [Object, Object, Object, Object, Object, Object, Object, Object, Object])
], WishlistService);
exports.WishlistService = WishlistService;
//# sourceMappingURL=wishlist.service.js.map