import { Inject, Injectable } from '@nestjs/common';
import { Op, QueryTypes } from "sequelize";
import { AlgmChannelJCT } from './models/AlgmChannelJCT.entity';
import { ChannelMatching } from './models/ChannelMatching.entity';
import { ProductMatching } from './models/ProductMatching.entity';

import { ChannelSuggestion } from './models/ChannelSuggestion.entity';

import { Sequelize } from 'sequelize-typescript';
import { AlgmProductJCT } from './models/AlgmProductJCT.entity';
var _ = require('lodash');


@Injectable()
export class AlgorithmService {
    constructor(
        @Inject('SequelizeInstance') private readonly sequelizeInstance,
        @Inject('ProductMatchingRepository') private readonly productMatchingRepository: typeof ProductMatching,
        @Inject('ChannelMatchingRepository') private readonly channelMatchingRepository: typeof ChannelMatching,
        @Inject('ChannelJctRepository') private readonly channelJctRepository: typeof AlgmChannelJCT,
        @Inject('ProductJctRepository') private readonly productJctRepository: typeof AlgmProductJCT,
    ) { }

    async profile(ctype, btype, chlist, channelId, profileTypeId) {


        let profileMatchedChannels = [];
        let profileMatchedIds = [];
        let typeId = [];

        if (ctype == 2 && btype == 1)
            typeId = [3, 4]
        else if (ctype == 2 && btype == 2)
            typeId = [3, 5]
        else if (ctype == 2 && btype == 3)
            typeId = [3, 5, 4]

        else if (ctype == 3 || ctype == 4 || ctype == 5)
            typeId = [2]
        // else if (ctype == 3 && btype == 2)
        //     typeId = [4]
        // else if (ctype == 3 && btype == 3)
        //     typeId = [5, 4]


        if (ctype == 2) {
            let buyers; let resellerProfileIds; let distChannel;
            if (btype == 1)
                buyers = chlist.filter(ch => { return [4].includes(+ch.channelTypeId) });
            else if (btype == 2)
                buyers = chlist.filter(ch => { return [5].includes(+ch.channelTypeId) });
            else if (btype == 3)
                buyers = chlist.filter(ch => { return [4, 5].includes(+ch.channelTypeId) });


            let distributor = chlist.filter(ch => { return [3].includes(+ch.channelTypeId) });

            let buyerIds = buyers.map(x => +x.channelId);
            let distributorIds = distributor.map(x => +x.channelId);

            console.log("buyerIds:", buyerIds.length);
            console.log("distributorIds:", distributorIds.length);

            let vendorQuery = 'select "profileTypeId" from "tblChannelCustomerProfile" where "deletedAt" is null and "channelId"=:channelId';
            let vendorProfileIds = await this.sequelizeInstance.query(vendorQuery, {
                type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                    channelId: channelId
                }
            }).then(x => {
                return x.map(c => +c.profileTypeId);
            })

            //console.log("vendorProfileIds:", vendorProfileIds);

            if (buyerIds.length > 0) {
                let profileQuery = 'select "profileTypeId","channelId" from "tblChannel" where "deletedAt" is null and "profileTypeId" in (:vendorProfileIds) and "channelId" in(:resellerIds)';
                resellerProfileIds = await this.sequelizeInstance.query(profileQuery, {
                    type: QueryTypes.SELECT,
                    raw: true,
                    replacements: {
                        vendorProfileIds: vendorProfileIds,
                        resellerIds: buyerIds
                    }
                }).then(x => {
                    return x;
                })
                profileMatchedIds.push(...resellerProfileIds.map(x => +x.channelId));
                //console.log("resellerProfileIds:", resellerProfileIds.length);
            }

            if (distributorIds.length) {
                let distQuery = 'select "channelId" from "tblChannelCustomerProfile" where "profileTypeId" in(:vendorProfileIds) and "channelId" in(:distributorIds) and "deletedAt" is null';
                distChannel = await this.sequelizeInstance.query(distQuery, {
                    type: QueryTypes.SELECT,
                    raw: true,
                    replacements: {
                        //resellerProfileIds:resellerProfileIds.map(x => +x.profileTypeId),
                        vendorProfileIds: vendorProfileIds,
                        distributorIds: distributorIds
                    }
                }).then(x => {
                    return x.map(c => +c.channelId);
                })
                profileMatchedIds.push(...distChannel);
                //console.log("distChannel", distChannel.length);
            }

            //  console.log("profileMatchedIds",profileMatchedIds);
        }

        if (ctype == 3) {
            //let buyers = chlist.filter(ch => { return typeId.includes(+ch.channelTypeId) });
            //let distributor = chlist.filter(ch => { return [3].includes(+ch.channelTypeId) });
            let vendor = chlist.filter(ch => { return [2].includes(+ch.channelTypeId) });

            // let buyerIds = buyers.map(x => +x.channelId);
            // let distributorIds = distributor.map(x => +x.channelId);
            let vendorIds = vendor.map(x => +x.channelId);

            let distQuery = 'select "profileTypeId" from "tblChannelCustomerProfile" where "deletedAt" is null and "channelId"=:channelId';
            let distProfileIds = await this.sequelizeInstance.query(distQuery, {
                type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                    channelId: channelId
                }
            }).then(x => {
                return x.map(c => +c.profileTypeId);
            })

            // let profileQuery = 'select "profileTypeId","channelId" from "tblChannel" where "deletedAt" is null and "profileTypeId" in (:distProfileIds) and "channelId" in(:resellerIds)';
            // let resellerProfileIds = await this.sequelizeInstance.query(profileQuery, {
            //     type: QueryTypes.SELECT,
            //     raw: true,
            //     replacements: {
            //         distProfileIds: distProfileIds,
            //         resellerIds: buyerIds
            //     }
            // }).then(x => {
            //     return x;
            // })


            // let distMatchQuery = 'select "channelId" from "tblChannelCustomerProfile" where "profileTypeId" in(:distProfileIds) and "channelId" in(:distributorIds) and "deletedAt" is null';
            // let distChannel = await this.sequelizeInstance.query(distMatchQuery, {
            //     type: QueryTypes.SELECT,
            //     raw: true,
            //     replacements: {
            //         //resellerProfileIds:resellerProfileIds.map(x => +x.profileTypeId),
            //         distProfileIds: distProfileIds,
            //         distributorIds: distributorIds
            //     }
            // }).then(x => {
            //     return x.map(c => +c.channelId);
            // })


            let venMatchQuery = 'select "channelId" from "tblChannelCustomerProfile" where "profileTypeId" in(:distProfileIds) and "channelId" in(:vendorIds) and "deletedAt" is null';
            let venChannel = await this.sequelizeInstance.query(venMatchQuery, {
                type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                    //resellerProfileIds:resellerProfileIds.map(x => +x.profileTypeId),
                    distProfileIds: distProfileIds,
                    vendorIds: vendorIds
                }
            }).then(x => {
                return x.map(c => +c.channelId);
            })
            //            profileMatchedIds.push(...resellerProfileIds.map(x => +x.channelId));
            //           profileMatchedIds.push(...distChannel);
            profileMatchedIds.push(...venChannel);
        }

        if (ctype == 4 || ctype == 5) {
            //console.log("chlist:",chlist);
            let sellers = chlist.filter(ch => { return [2].includes(+ch.channelTypeId) });

            let sellersIds = sellers.map(x => +x.channelId);

            let profileQuery = 'select "channelId" from "tblChannelCustomerProfile" where "deletedAt" is null and "profileTypeId" =:buyerProfileIds and "channelId" in(:sellerIds)';
            let sellerChannel = await this.sequelizeInstance.query(profileQuery, {
                type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                    buyerProfileIds: profileTypeId,
                    sellerIds: sellersIds
                }
            }).then(x => {
                return x.map(c => +c.channelId);
            })

            profileMatchedIds.push(...sellerChannel);
        }

        //console.log("profileMatchedChannels",profileMatchedChannels);
        profileMatchedChannels = chlist.filter(ch => { return profileMatchedIds.includes(+ch.channelId) });

        return profileMatchedChannels;
    }

    async location(ctype, btype, contactId, channelId, chlist) {
        //console.log("profile:",chlist,ctype,btype,contactId);
        //console.log("contacts:",chlist.filter(x => +x.channelTypeId==4));
        //chlist=chlist.filter(x => +x.channelTypeId==4);
        //console.log("contacts:",chlist.map(x => +x.contactId));
        let locationMatchedChannels = []; let srcLocationType = []; let destLocationType = [];
        let targetLocation;
        if (ctype == 2 && btype == 1) {
            destLocationType = ['RETAIL', 'SELLING'];
            srcLocationType = ['TARGET'];
        }
        else if (ctype == 2 && btype == 2) {
            destLocationType = ['RESELLING', 'SELLING'];
            srcLocationType = ['TARGET'];
        }
        else if (ctype == 2 && btype == 3) {
            destLocationType = ['RETAIL', 'RESELLING', 'SELLING'];
            srcLocationType = ['TARGET'];
        }
        // else if(ctype==3&&btype==1)
        // {
        // destLocationType=['SELLING','RESELLING','TARGET'];
        // srcLocationType=['SELLING','TARGET'];    
        // }
        // else if(ctype==3&&btype==2)
        // {
        // destLocationType=['SELLING','RETAIL','TARGET'];
        // srcLocationType=['SELLING','TARGET'];

        // }
        // else if(ctype==3&&btype==3)
        // {
        // destLocationType=['SELLING','RESELLING','RETAIL','TARGET'];
        // srcLocationType=['SELLING','TARGET'];

        // }
        else if (ctype == 4) {
            destLocationType = ['TARGET'];
            srcLocationType = ['RETAIL'];

        }
        else if (ctype == 5) {
            destLocationType = ['TARGET'];
            srcLocationType = ['RESELLING'];

        }

        if (ctype == 3) {
            // let ch = chlist.filter(ch => { return [3, 4, 5].includes(+ch.channelTypeId) });

            // let locationquery = 'Select pc."countryId",cpc."channelId",cpc."contactId" from "tblContactLocation" cpc inner join "tblRegionCountryJCT" pc on pc."regionCountryJCTId" = cpc."regionCountryJCTId" '
            //     + 'where cpc."contactId" in(:contacts) and CAST(cpc."locationType" AS text) in (:location) and cpc."deletedAt" is null';
            // await this.sequelizeInstance.query(locationquery, {
            //     type: QueryTypes.SELECT,
            //     raw: true,
            //     replacements: {
            //         location: ['SELLING', 'RESELLING', 'RETAIL'],
            //         contacts: ch.map(x => +x.contactId),
            //     }
            // }).then(x => {
            //     targetLocation = x//.map(con => +con.countryId));
            // })


            // //Getting Locations of login Contacts
            // locationquery = 'Select pc."countryId" from "tblContactLocation" cpc inner join "tblRegionCountryJCT" pc on pc."regionCountryJCTId" = cpc."regionCountryJCTId" '
            //     + 'where cpc."contactId" =:contactId and cpc."deletedAt" is null and CAST(cpc."locationType" AS text) in(:location) ';

            // let srcLocation = await this.sequelizeInstance.query(locationquery, {
            //     type: QueryTypes.SELECT,
            //     raw: true,
            //     replacements: {
            //         location: ['TARGET'],
            //         contactId: contactId
            //     }
            // }).then(x => {
            //     return x.map(c => +c.countryId);//new Map(x)//.map(con => +con.countryId));
            // })

            // if (srcLocation.length == 0) {
            //     locationquery = 'Select pc."countryId" from "tblChannelLocation" cpc inner join "tblRegionCountryJCT" pc on pc."regionCountryJCTId" = cpc."regionCountryJCTId" '
            //         + 'where cpc."channelId" =:channelId and cpc."deletedAt" is null and CAST(cpc."locationType" AS text) in(:location) ';

            //     srcLocation = await this.sequelizeInstance.query(locationquery, {
            //         type: QueryTypes.SELECT,
            //         raw: true,
            //         replacements: {
            //             location: ['TARGET'],
            //             channelId: channelId
            //         }
            //     }).then(x => {
            //         return x.map(c => +c.countryId);
            //     })

            // }

            // let matchingBuyers = targetLocation.filter(ch => { return srcLocation.includes(+ch.countryId) });

            targetLocation = [];
            let srcLocation = [];
            let ch = chlist.filter(ch => { return [2].includes(+ch.channelTypeId) });

            let locationquery = 'Select pc."countryId",cpc."channelId",cpc."contactId" from "tblContactLocation" cpc inner join "tblRegionCountryJCT" pc on pc."regionCountryJCTId" = cpc."regionCountryJCTId" '
                + 'where cpc."contactId" in(:contacts) and CAST(cpc."locationType" AS text) in (:location) and cpc."deletedAt" is null';
            await this.sequelizeInstance.query(locationquery, {
                type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                    location: ['TARGET'],
                    contacts: ch.map(x => +x.contactId),
                }
            }).then(x => {
                targetLocation = x//.map(con => +con.countryId));
            })

            //Getting Locations of login Contacts
            locationquery = 'Select pc."countryId" from "tblContactLocation" cpc inner join "tblRegionCountryJCT" pc on pc."regionCountryJCTId" = cpc."regionCountryJCTId" '
                + 'where cpc."contactId" =:contactId and cpc."deletedAt" is null and CAST(cpc."locationType" AS text) in(:location) ';

            srcLocation = await this.sequelizeInstance.query(locationquery, {
                type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                    location: ['SELLING'],
                    contactId: contactId
                }
            }).then(x => {
                return x.map(c => +c.countryId);//new Map(x)//.map(con => +con.countryId));
            })

            if (srcLocation.length == 0) {
                locationquery = 'Select pc."countryId" from "tblChannelLocation" cpc inner join "tblRegionCountryJCT" pc on pc."regionCountryJCTId" = cpc."regionCountryJCTId" '
                    + 'where cpc."channelId" =:channelId and cpc."deletedAt" is null and CAST(cpc."locationType" AS text) in(:location) ';

                srcLocation = await this.sequelizeInstance.query(locationquery, {
                    type: QueryTypes.SELECT,
                    raw: true,
                    replacements: {
                        location: ['SELLING'],
                        channelId: channelId
                    }
                }).then(x => {
                    return x.map(c => +c.countryId);
                })

            }


            let matchingVendors = targetLocation.filter(ch => { return srcLocation.includes(+ch.countryId) });

            //locationMatchedChannels.push(...matchingBuyers.map(x => +x.channelId));
            locationMatchedChannels.push(...matchingVendors.map(x => +x.channelId));
        }
        else {
            let locationquery = 'Select pc."countryId",cpc."channelId",cpc."contactId" from "tblContactLocation" cpc inner join "tblRegionCountryJCT" pc on pc."regionCountryJCTId" = cpc."regionCountryJCTId" '
                + 'where cpc."contactId" in(:contacts) and CAST(cpc."locationType" AS text) in (:location) and cpc."deletedAt" is null';
            await this.sequelizeInstance.query(locationquery, {
                type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                    location: destLocationType,
                    contacts: chlist.map(x => +x.contactId),
                }
            }).then(x => {
                targetLocation = x//.map(con => +con.countryId));
            })

            console.log("targetlocation:", targetLocation.length);
            //Getting Locations of login Contacts
            locationquery = 'Select pc."countryId" from "tblContactLocation" cpc inner join "tblRegionCountryJCT" pc on pc."regionCountryJCTId" = cpc."regionCountryJCTId" '
                + 'where cpc."contactId" =:contactId and cpc."deletedAt" is null and CAST(cpc."locationType" AS text) in(:location) ';

            let srcLocation;
            srcLocation = await this.sequelizeInstance.query(locationquery, {
                type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                    location: srcLocationType,
                    contactId: contactId
                }
            }).then(x => {
                return x.map(c => +c.countryId);
            })

            console.log("source:", srcLocation.length);

            if (srcLocation.length == 0) {
                locationquery = 'Select pc."countryId" from "tblChannelLocation" cpc inner join "tblRegionCountryJCT" pc on pc."regionCountryJCTId" = cpc."regionCountryJCTId" '
                    + 'where cpc."channelId" =:channelId and cpc."deletedAt" is null and CAST(cpc."locationType" AS text) in(:location) ';

                srcLocation = await this.sequelizeInstance.query(locationquery, {
                    type: QueryTypes.SELECT,
                    raw: true,
                    replacements: {
                        location: srcLocationType,
                        channelId: channelId
                    }
                }).then(x => {
                    return x.map(c => +c.countryId);
                })
                console.log("source:", srcLocation.length);
            }



            let location = targetLocation.filter(ch => { return srcLocation.includes(+ch.countryId) });
            locationMatchedChannels.push(...location.map(x => +x.channelId));

        }

        //console.log("locationMatchedChannels:", locationMatchedChannels);

        let channels = chlist.filter(ch => { return locationMatchedChannels.includes(+ch.channelId) })
        return channels;
    }

    async product(contactId, channelId, chlist) {
        let product = [];
        let productquery = 'Select cpc."productCategorieId",cpc."contactId" from "tblInterestedProductCategory" cpc where "deletedAt" is null and cpc."contactId" in(:contacts) '
            + 'and cpc."productCategorieId" in (Select cpc."productCategorieId" from "tblInterestedProductCategory" cpc where "deletedAt" is null and cpc."contactId" =:contactId) ';

        await this.sequelizeInstance.query(productquery, {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
                contacts: chlist.map(x => +x.contactId),
                contactId: contactId
            }
        }).then(async (x) => {
            product.push(...x)//_.uniqBy(x,'contactId');
            let filterch = product.map(p => +p.contactId);
            let result = chlist.filter(ch => { return !filterch.includes(+ch.contactId); })

            if (result && result.length) {
                let productquery = 'Select cpc."productCategorieId",cpc."channelId" from "tblChannelProductCategorie" cpc where "deletedAt" is null and cpc."channelId" in(:channels) '
                    + 'and cpc."productCategorieId" in (Select cpc."productCategorieId" from "tblChannelProductCategorie" cpc where "deletedAt" is null and cpc."channelId" =:channelId)';

                await this.sequelizeInstance.query(productquery, {
                    type: QueryTypes.SELECT,
                    raw: true,
                    replacements: {
                        channels: result.map(x => +x.channelId),
                        channelId: channelId
                    }
                }).then(x => {
                    product.push(...x);
                    //console.log("product:",product);
                })
            }
        })

        //console.log("product:", product);
        let productContact = product.map(x => +x.contactId);
        let productChannel = product.map(x => +x.channelId);


        let chproducts = chlist.filter(ch => { return productChannel.includes(+ch.channelId); })

        chproducts.map(x => {
            Object.assign(x, { productInterest: 0 });
        })

        let conproducts = chlist.filter(ch => { return productContact.includes(+ch.contactId); })

        conproducts.map(x => {
            Object.assign(x, { productInterest: 1 });
        })

        let channels = [];
        channels.push(...chproducts); channels.push(...conproducts);


        //console.log("channels:", channels);

        //let channels = chlist.filter(ch => { return (productChannel.includes(+ch.channelId) || productContact.includes(+ch.contactId)); })
        //console.log("pchannels:",channels);
        return channels;
    }

    async channelAlgm1(user) {
        let contactId = user.contactId;
        let typeid = user.channelTypeId;
        let businessType = user.businessType;
        let channelId = user.channelId;
        let planId = user.planId;
        let profileTypeId = user.profileTypeId;

        //console.log("user:",user);

        let channeltype = []; let srcLocationType = []; let destLocationType = [];

        let channels = [];

        if (typeid == 2 && businessType == 1)
            channeltype = [3, 4];
        else if (typeid == 2 && businessType == 2)
            channeltype = [3, 5];
        else if (typeid == 2 && businessType == 3)
            channeltype = [3, 4, 5];
        else if (typeid == 3)// && businessType == 1)
            channeltype = [2];
        // else if (typeid == 3 && businessType == 2)
        //     channeltype = [2, 3, 4];
        // else if (typeid == 3 && businessType == 3)
        //     channeltype = [2, 3, 4, 5];
        else if (typeid == 4)
            channeltype = [2];
        else if (typeid == 5)
            channeltype = [2];


        let channelquery;

        let startup = 'select cd."startup" from "tblChannelDetail" as cd where cd."channelId"=:channelid ';

        let sup = await this.sequelizeInstance.query(startup, {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
                channelid: channelId
            }
        }).then(x => {
            return x[0].startup;
        })

        if (typeid == 2) {
            channelquery = 'select (select "shareStatus" from "tblChannelDetail" where "channelId"=:channelId) as share, ' //(select count(*) from "tblChannel" where "channelId"=tc."channelId" and "channelTypeId" in(select "sharetypeid" from "tblchannelsharepreference" as csp where csp."channelid"=:channelId and csp."deletedat" is null)) as share, '
                + 'tc."channelId",tc."profileStrength",tc."channelTypeId",tc."planId",tcon."contactId" from "tblChannel" as tc inner join "tblContact" as tcon on tcon."channelId"=tc."channelId" and tcon."deletedAt" is null and cast(tcon."isActive" as bigint)=1  where '
                + 'tc."channelTypeId" in(select "sharetypeid"+1 from "tblchannelsharepreference" as csp where csp."channelid"=:channelId and csp."deletedat" is null) and '
                + 'tc."channelId" in(select "channelId" from "tblChannelDetail" where "shareStatus"=true) and '
                //+ 'tc."channelId" in(select csp."channelid" from "tblchannelsharepreference" as csp where "sharetypeid"=1 and csp."channelid"=tc."channelId" and csp."deletedat" is null) and '
                //+ 'and (case when :sup=1 then tc."channelId" in(select "channelId" from "tblChannelDetail" as cd where "deletedAt" is null and cd."startup"=\'1\' and cd."channelId"=tc."channelId") when :sup=0 then tc."channelId" in(select "channelId" from "tblChannelDetail" as cd where "deletedAt" is null and cd."startup"=\'0\' and cd."channelId"=tc."channelId")   end) and '
                + 'tc."channelId" not in(select case when "channelId"=:channelId  then "partnerChannelId" when "partnerChannelId"=:channelId then "channelId" end from "tblChannelJCT" where ("requested_by"=:contactId and "requested_to"=tcon."contactId" and "deletedat" is null and "connectionStatusTypeId" not in(1,5,6))or("requested_to"=:contactId and "requested_by"=tcon."contactId" and "deletedat" is null and "connectionStatusTypeId" not in(1,5,6))) and tc."signUpStatusId" = 15 and tc."channelTypeId" in(:channeltype) and tc."deletedAt" is null and tc."isActive"=\'1\' ';
        }
        else if (typeid == 3) {
            channelquery = 'select (select "shareStatus" from "tblChannelDetail" where "channelId"=:channelId) as share, ' //(select count(*) from "tblChannel" where "channelId"=tc."channelId" and "channelTypeId" in(select "sharetypeid"+1 from "tblchannelsharepreference" as csp where csp."channelid"=:channelId and csp."deletedat" is null)) as share, '
                + 'tc."channelId",tc."profileStrength",tc."channelTypeId",tc."planId",tcon."contactId" from "tblChannel" tc inner join "tblContact" tcon on tcon."channelId"=tc."channelId" and tcon."deletedAt" is null and cast(tcon."isActive" as bigint)=1 where '
                + 'tc."channelId" in(select "channelId" from "tblChannelDetail" where "shareStatus"=true) and '
                //+ 'cast(tc."isOEM" as bigint) in(:businessType) and tc."channelId" in(select csp."channelid" from "tblchannelsharepreference" as csp where "sharetypeid"=2 and csp."channelid"=tc."channelId" and csp."deletedat" is null) and (case when :sup=1 and tc."channelTypeId"=2 then tc."channelId" in(select "channelId" from "tblChannel" where "channelId"=tc."channelId" and "deletedAt" is null) when :sup=0 and tc."channelTypeId"=2 then tc."channelId" in(select "channelId" from "tblChannelDetail" as cd where cd."startup"=\'0\' and cd."deletedAt" is null and cd."channelId"=tc."channelId")  when tc."channelTypeId" in(3,4,5) then tc."channelId" in(select "channelId" from "tblChannel" where "channelId"=tc."channelId") end) and '
                + 'tc."channelId" not in(select case when "channelId"=:channelId then "partnerChannelId" when "partnerChannelId"=:channelId then "channelId" end from "tblChannelJCT" where ("requested_by"=:contactId and "requested_to"=tcon."contactId" and "deletedat" is null and "connectionStatusTypeId" not in(1,5,6))or("requested_to"=:contactId and "requested_by"=tcon."contactId" and "deletedat" is null and "connectionStatusTypeId" not in(1,5,6))) and tc."signUpStatusId" = 15 and tc."channelTypeId" in(:channeltype) and tc."deletedAt" is null and tc."isActive"=\'1\' ';

        }
        else if (typeid == 4) {
            channelquery = 'select (select "shareStatus" from "tblChannelDetail" where "channelId"=:channelId) as share, ' //(select count(*) from "tblChannel" where "channelId"=tc."channelId" and "channelTypeId" in(select "sharetypeid"+1 from "tblchannelsharepreference" as csp where csp."channelid"=:channelId and csp."deletedat" is null)) as share, '
                + 'tc."channelId",tc."profileStrength",tc."channelTypeId",tc."planId",tcon."contactId" from "tblChannel" tc inner join "tblContact" tcon on tcon."channelId"=tc."channelId" and tcon."deletedAt" is null and cast(tcon."isActive" as bigint)=1 where '
                + 'tc."channelId" in(select "channelId" from "tblChannelDetail" where "shareStatus"=true) and '
                //+ 'cast(tc."isOEM" as bigint) in(:businessType) and tc."channelId" in(select csp."channelid" from "tblchannelsharepreference" as csp where "sharetypeid"=3 and csp."channelid"=tc."channelId" and csp."deletedat" is null) and (case when :sup=1 and tc."channelTypeId"=2 then tc."channelId" in(select "channelId" from "tblChannel" where "channelId"=tc."channelId" and "deletedAt" is null) when :sup=0 and tc."channelTypeId"=2 then tc."channelId" in(select "channelId" from "tblChannelDetail" as cd where cd."startup"=\'0\' and cd."deletedAt" is null and cd."channelId"=tc."channelId")  when tc."channelTypeId"=3 then tc."channelId" in(select "channelId" from "tblChannel" where "channelId"=tc."channelId") end) and '
                + 'tc."channelId" not in(select case when "channelId"=:channelId then "partnerChannelId" when "partnerChannelId"=:channelId then "channelId" end from "tblChannelJCT" where ("requested_by"=:contactId and "requested_to"=tcon."contactId" and "deletedat" is null and "connectionStatusTypeId" not in(1,5,6))or("requested_to"=:contactId and "requested_by"=tcon."contactId" and "deletedat" is null and "connectionStatusTypeId" not in(1,5,6))) and tc."signUpStatusId" = 15 and tc."channelTypeId" in(:channeltype) and tc."deletedAt" is null and tc."isActive"=\'1\' ';
        }
        else if (typeid == 5) {
            channelquery = 'select (select "shareStatus" from "tblChannelDetail" where "channelId"=:channelId) as share, ' //(select count(*) from "tblChannel" where "channelId"=tc."channelId" and "channelTypeId" in(select "sharetypeid"+1 from "tblchannelsharepreference" as csp where csp."channelid"=:channelId and csp."deletedat" is null)) as share, '
                + 'tc."channelId",tc."profileStrength",tc."channelTypeId",tc."planId",tcon."contactId" from "tblChannel" tc inner join "tblContact" tcon on tcon."channelId"=tc."channelId" and tcon."deletedAt" is null and cast(tcon."isActive" as bigint)=1 where '
                + 'tc."channelId" in(select "channelId" from "tblChannelDetail" where "shareStatus"=true) and '
                //+ 'cast(tc."isOEM" as bigint) in(:businessType) and tc."channelId" in(select csp."channelid" from "tblchannelsharepreference" as csp where "sharetypeid"=3 and csp."channelid"=tc."channelId" and csp."deletedat" is null) and (case when :sup=1 and tc."channelTypeId"=2 then tc."channelId" in(select "channelId" from "tblChannel" where "channelId"=tc."channelId" and "deletedAt" is null) when :sup=0 and tc."channelTypeId"=2 then tc."channelId" in(select "channelId" from "tblChannelDetail" as cd where cd."startup"=\'0\' and cd."deletedAt" is null and cd."channelId"=tc."channelId")  when tc."channelTypeId"=3 then tc."channelId" in(select "channelId" from "tblChannel" where "channelId"=tc."channelId") end) and '
                + 'tc."channelId" not in(select case when "channelId"=:channelId then "partnerChannelId" when "partnerChannelId"=:channelId then "channelId" end from "tblChannelJCT" where ("requested_by"=:contactId and "requested_to"=tcon."contactId" and "deletedat" is null and "connectionStatusTypeId" not in(1,5,6))or("requested_to"=:contactId and "requested_by"=tcon."contactId" and "deletedat" is null and "connectionStatusTypeId" not in(1,5,6))) and tc."signUpStatusId" = 15 and tc."channelTypeId" in(:channeltype) and tc."deletedAt" is null and tc."isActive"=\'1\' ';
        }

        await this.sequelizeInstance.query(channelquery, {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
                channeltype: channeltype,
                businessType: businessType == 3 ? [1, 2, 3] : [businessType],
                channelId: channelId,
                contactId: contactId,
                sup: sup
            }
        }).then(x => {
            channels.push(...x);
        })

        // console.log("channels:",channels);
        //console.log("channels:", channels.length);
        if (channels.length == 0 && typeid == 2) {
            channelquery = 'select (select "shareStatus" from "tblChannelDetail" where "channelId"=:channelId) as share, ' //(select count(*) from "tblChannel" where "channelId"=tc."channelId" and "channelTypeId" in(select "sharetypeid" from "tblchannelsharepreference" as csp where csp."channelid"=:channelId and csp."deletedat" is null)) as share, '
                + 'tc."channelId",tc."profileStrength",tc."channelTypeId",tc."planId",tcon."contactId" from "tblChannel" as tc inner join "tblContact" as tcon on tcon."channelId"=tc."channelId" and tcon."deletedAt" is null and cast(tcon."isActive" as bigint)=1  where '
                + 'tc."channelTypeId" in(3,4,5) and '
                + 'tc."channelId" in(select "channelId" from "tblChannelDetail" where "shareStatus"=true) and '
                //+ 'tc."channelId" in(select csp."channelid" from "tblchannelsharepreference" as csp where "sharetypeid"=1 and csp."channelid"=tc."channelId" and csp."deletedat" is null) and '
                //+ 'and (case when :sup=1 then tc."channelId" in(select "channelId" from "tblChannelDetail" as cd where "deletedAt" is null and cd."startup"=\'1\' and cd."channelId"=tc."channelId") when :sup=0 then tc."channelId" in(select "channelId" from "tblChannelDetail" as cd where "deletedAt" is null and cd."startup"=\'0\' and cd."channelId"=tc."channelId")   end) and '
                + 'tc."channelId" not in(select case when "channelId"=:channelId  then "partnerChannelId" when "partnerChannelId"=:channelId then "channelId" end from "tblChannelJCT" where ("requested_by"=:contactId and "requested_to"=tcon."contactId" and "deletedat" is null and "connectionStatusTypeId" not in(1,5,6))or("requested_to"=:contactId and "requested_by"=tcon."contactId" and "deletedat" is null and "connectionStatusTypeId" not in(1,5,6))) and tc."signUpStatusId" = 15 and tc."channelTypeId" in(:channeltype) and tc."deletedAt" is null and tc."isActive"=\'1\' ';

            await this.sequelizeInstance.query(channelquery, {
                type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                    channeltype: channeltype,
                    businessType: businessType == 3 ? [1, 2, 3] : [businessType],
                    channelId: channelId,
                    contactId: contactId,
                    sup: sup
                }
            }).then(x => {
                channels.push(...x);
            })

        }

        channels = await this.profile(typeid, businessType, channels, channelId, profileTypeId);
        console.log("prochannels:", channels.length);
        //        console.log("channels:", channels.length);
        channels = await this.location(typeid, businessType, contactId, channelId, channels);
        console.log("locchannels:", channels.length);
        channels = await this.product(contactId, channelId, channels);
        console.log("productchannels:", channels.length);
        let channelJCT = [];

        // let channelJCT = [].concat(channels.map(x => {
        //     return {channelId:channelId,partnerChannelId: x.channelId, requested_to: x.contactId,requested_by:contactId,connectionStatusTypeId: 1};
        // }));
        //console.log("planId:", planId, typeid);

        channels.map(x => {

            let mscore = 0;
            let confidence = '0';
            let dconfidence = '0';

            if ((planId == 1 || planId == 6) && typeid == 2)
                confidence = '1';

            if (planId == 1 && typeid == 3 && x.channelTypeId == 4)
                confidence = '1';

            if (planId == 1 && typeid == 3 && x.channelTypeId == 2) {
                confidence = '0';
                if (x.planId == 1)
                    dconfidence = '1';
            }

            if (planId == 1 && typeid == 4) {
                confidence = '0';
                if (x.planId == 1)
                    dconfidence = '1';
            }

            if (x.productInterest == 1) {
                if (x.profileStrength > 75)
                    mscore = 80 + 20;
                else if (x.profileStrength >= 70 && x.profileStrength <= 75)
                    mscore = 80 + 15;
                else if (x.profileStrength >= 65 && x.profileStrength < 70)
                    mscore = 80 + 10;
                else if (x.profileStrength < 65)
                    mscore = 80 + 8;
            }
            else if (x.productInterest == 0) {
                if (x.profileStrength > 75)
                    mscore = 70 + 20;
                else if (x.profileStrength >= 70 && x.profileStrength <= 75)
                    mscore = 70 + 15;
                else if (x.profileStrength >= 65 && x.profileStrength < 70)
                    mscore = 70 + 10;
                else if (x.profileStrength < 65)
                    mscore = 70 + 8;

            }



            channelJCT.push({ channelId: x.channelId, partnerChannelId: channelId, requested_to: contactId, requested_by: x.contactId, connectionStatusTypeId: 1, isConfidential: confidence, matchingScore: mscore });
            if (x.share) {
                //console.log("share:", x.share);
                channelJCT.push({ channelId: channelId, partnerChannelId: x.channelId, requested_to: x.contactId, requested_by: contactId, connectionStatusTypeId: 1, isConfidential: dconfidence, matchingScore: mscore });
            }

        });


        await this.channelJctRepository.destroy({
            where: {
                requested_to: contactId,
                //channelId: channelId,
                connectionStatusTypeId: 1
            }
        });

        await this.channelJctRepository.destroy({
            where: {
                requested_by: contactId,
                //partnerChannelId: channelId,
                connectionStatusTypeId: 1
            }
        });

        let res = await this.channelJctRepository.bulkCreate(channelJCT);
        //this.ranking({ planId: planId, contactId: contactId, channelId: channelId, channelTypeId: user.channelTypeId });

        //return res;

    }

    async channelAlgm(user) {
        let contactId = user.contactId;
        let typeid = user.channelTypeId;
        let businessType = user.businessType;
        let channelId = user.channelId;
        let planId = user.planId;
        let profileTypeId = user.profileTypeId;
        let result = []; let product = []; let location = []; let profile = [];

        let channeltype = []; let srcLocationType = []; let destLocationType = [];

        let channels = [];
        let srcLocation;
        let targetLocation;

        if (typeid == 2 && businessType == 1) {
            destLocationType = ['RESELLING', 'SELLING'];
            srcLocationType = ['TARGET'];
            channeltype = [3, 5];
        }
        else if (typeid == 2 && businessType == 2) {
            destLocationType = ['RETAIL', 'SELLING'];
            srcLocationType = ['TARGET'];
            channeltype = [3, 4];
        }
        else if (typeid == 2 && businessType == 3) {
            destLocationType = ['RETAIL', 'RESELLING', 'SELLING'];
            srcLocationType = ['TARGET'];
            channeltype = [3, 4, 5];
        }
        else if (typeid == 3 && businessType == 1) {
            destLocationType = ['SELLING', 'RESELLING', 'TARGET'];
            srcLocationType = ['SELLING', 'TARGET'];
            channeltype = [2, 3, 5];
        }
        else if (typeid == 3 && businessType == 2) {
            destLocationType = ['SELLING', 'RETAIL', 'TARGET'];
            srcLocationType = ['SELLING', 'TARGET'];
            channeltype = [2, 3, 4];
        }
        else if (typeid == 3 && businessType == 3) {
            destLocationType = ['SELLING', 'RESELLING', 'RETAIL', 'TARGET'];
            srcLocationType = ['SELLING', 'TARGET'];
            channeltype = [2, 3, 4, 5];
        }
        else if (typeid == 4) {
            destLocationType = ['TARGET'];
            srcLocationType = ['RETAIL'];
            channeltype = [2, 3];
        }
        else if (typeid == 5) {
            destLocationType = ['TARGET'];
            srcLocationType = ['RESELLING'];
            channeltype = [2, 3];
        }

        let channelquery;

        let startup = 'select cd."startup" from "tblChannelDetail" as cd where cd."channelId"=:channelid ';

        let sup = await this.sequelizeInstance.query(startup, {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
                channelid: channelId
            }
        }).then(x => {
            return x[0].startup;
        })

        if (typeid == 2) {
            channelquery = 'select (select count(*) from "tblChannel" where "channelId"=tc."channelId" and "channelTypeId" in(select "sharetypeid"+1 from "tblchannelsharepreference" as csp where csp."channelid"=:channelId and csp."deletedat" is null)) as share, '
                + 'tc."channelId",tc."channelTypeId",tc."planId",tcon."contactId" from "tblChannel" as tc inner join "tblContact" as tcon on tcon."channelId"=tc."channelId" and tcon."deletedAt" is null and cast(tcon."isActive" as bigint)=1  where '
                + 'tc."channelId" in(select csp."channelid" from "tblchannelsharepreference" as csp where "sharetypeid"=1 and csp."channelid"=tc."channelId" and csp."deletedat" is null) and (case when :sup=1 then tc."channelId" in(select "channelId" from "tblChannelDetail" as cd where "deletedAt" is null and cd."startup"=\'1\' and cd."channelId"=tc."channelId") when :sup=0 then tc."channelId" in(select "channelId" from "tblChannelDetail" as cd where "deletedAt" is null and cd."startup"=\'0\' and cd."channelId"=tc."channelId")   end) and '
                + 'tc."channelId" not in(select case when "channelId"=:channelId  then "partnerChannelId" when "partnerChannelId"=:channelId then "channelId" end from "tblChannelJCT" where ("requested_by"=:contactId and "requested_to"=tcon."contactId" and "deletedat" is null and "connectionStatusTypeId" not in(1,5,6))or("requested_to"=:contactId and "requested_by"=tcon."contactId" and "deletedat" is null and "connectionStatusTypeId" not in(1,5,6))) and tc."signUpStatusId" = 15 and tc."channelTypeId" in(3,4) and tc."deletedAt" is null and tc."isActive"=\'1\' ';
        }
        else if (typeid == 3) {
            channelquery = 'select (select count(*) from "tblChannel" where "channelId"=tc."channelId" and "channelTypeId" in(select "sharetypeid"+1 from "tblchannelsharepreference" as csp where csp."channelid"=:channelId and csp."deletedat" is null)) as share, '
                + 'tc."channelId",tc."channelTypeId",tc."planId",tcon."contactId" from "tblChannel" tc inner join "tblContact" tcon on tcon."channelId"=tc."channelId" and tcon."deletedAt" is null and cast(tcon."isActive" as bigint)=1 where '
                + 'tc."channelId" in(select csp."channelid" from "tblchannelsharepreference" as csp where "sharetypeid"=2 and csp."channelid"=tc."channelId" and csp."deletedat" is null) and (case when :sup=1 and tc."channelTypeId"=2 then tc."channelId" in(select "channelId" from "tblChannel" where "channelId"=tc."channelId" and "deletedAt" is null) when :sup=0 and tc."channelTypeId"=2 then tc."channelId" in(select "channelId" from "tblChannelDetail" as cd where cd."startup"=\'0\' and cd."deletedAt" is null and cd."channelId"=tc."channelId")  when tc."channelTypeId"=4 then tc."channelId" in(select "channelId" from "tblChannel" where "channelId"=tc."channelId") end) and '
                + 'tc."channelId" not in(select case when "channelId"=:channelId then "partnerChannelId" when "partnerChannelId"=:channelId then "channelId" end from "tblChannelJCT" where ("requested_by"=:contactId and "requested_to"=tcon."contactId" and "deletedat" is null and "connectionStatusTypeId" not in(1,5,6))or("requested_to"=:contactId and "requested_by"=tcon."contactId" and "deletedat" is null and "connectionStatusTypeId" not in(1,5,6))) and tc."signUpStatusId" = 15 and tc."channelTypeId" in(2,4) and tc."deletedAt" is null and tc."isActive"=\'1\' ';
        }
        else if (typeid == 4) {
            channelquery = 'select (select count(*) from "tblChannel" where "channelId"=tc."channelId" and "channelTypeId" in(select "sharetypeid"+1 from "tblchannelsharepreference" as csp where csp."channelid"=:channelId and csp."deletedat" is null)) as share, '
                + 'tc."channelId",tc."channelTypeId",tc."planId",tcon."contactId" from "tblChannel" tc inner join "tblContact" tcon on tcon."channelId"=tc."channelId" and tcon."deletedAt" is null and cast(tcon."isActive" as bigint)=1 where '
                + 'tc."channelId" in(select csp."channelid" from "tblchannelsharepreference" as csp where "sharetypeid"=3 and csp."channelid"=tc."channelId" and csp."deletedat" is null) and (case when :sup=1 and tc."channelTypeId"=2 then tc."channelId" in(select "channelId" from "tblChannel" where "channelId"=tc."channelId" and "deletedAt" is null) when :sup=0 and tc."channelTypeId"=2 then tc."channelId" in(select "channelId" from "tblChannelDetail" as cd where cd."startup"=\'0\' and cd."deletedAt" is null and cd."channelId"=tc."channelId")  when tc."channelTypeId"=3 then tc."channelId" in(select "channelId" from "tblChannel" where "channelId"=tc."channelId") end) and '
                + 'tc."channelId" not in(select case when "channelId"=:channelId then "partnerChannelId" when "partnerChannelId"=:channelId then "channelId" end from "tblChannelJCT" where ("requested_by"=:contactId and "requested_to"=tcon."contactId" and "deletedat" is null and "connectionStatusTypeId" not in(1,5,6))or("requested_to"=:contactId and "requested_by"=tcon."contactId" and "deletedat" is null and "connectionStatusTypeId" not in(1,5,6))) and tc."signUpStatusId" = 15 and tc."channelTypeId" in(2,3) and tc."deletedAt" is null and tc."isActive"=\'1\' ';
        }
        else if (typeid == 5) {
            channelquery = 'select (select count(*) from "tblChannel" where "channelId"=tc."channelId" and "channelTypeId" in(select "sharetypeid"+1 from "tblchannelsharepreference" as csp where csp."channelid"=:channelId and csp."deletedat" is null)) as share, '
                + 'tc."channelId",tc."channelTypeId",tc."planId",tcon."contactId" from "tblChannel" tc inner join "tblContact" tcon on tcon."channelId"=tc."channelId" and tcon."deletedAt" is null and cast(tcon."isActive" as bigint)=1 where '
                + 'tc."channelId" in(select csp."channelid" from "tblchannelsharepreference" as csp where "sharetypeid"=3 and csp."channelid"=tc."channelId" and csp."deletedat" is null) and (case when :sup=1 and tc."channelTypeId"=2 then tc."channelId" in(select "channelId" from "tblChannel" where "channelId"=tc."channelId" and "deletedAt" is null) when :sup=0 and tc."channelTypeId"=2 then tc."channelId" in(select "channelId" from "tblChannelDetail" as cd where cd."startup"=\'0\' and cd."deletedAt" is null and cd."channelId"=tc."channelId")  when tc."channelTypeId"=3 then tc."channelId" in(select "channelId" from "tblChannel" where "channelId"=tc."channelId") end) and '
                + 'tc."channelId" not in(select case when "channelId"=:channelId then "partnerChannelId" when "partnerChannelId"=:channelId then "channelId" end from "tblChannelJCT" where ("requested_by"=:contactId and "requested_to"=tcon."contactId" and "deletedat" is null and "connectionStatusTypeId" not in(1,5,6))or("requested_to"=:contactId and "requested_by"=tcon."contactId" and "deletedat" is null and "connectionStatusTypeId" not in(1,5,6))) and tc."signUpStatusId" = 15 and tc."channelTypeId" in(2,3) and tc."deletedAt" is null and tc."isActive"=\'1\' ';
        }

        //Getting all Contacts to be matched
        //let channelquery='select (select count(*) from "tblChannel" where "channelId"=tc."channelId" and "channelTypeId" in(select "sharetypeid"+1 from "tblchannelsharepreference" as csp where csp."channelid"=' + channelId + ' and csp."deletedat" is null)) as share,con."contactId",con."channelId",tc."channelTypeId",tc."planId" from "tblContact" as con inner join "tblChannel" as tc on tc."channelId"=con."channelId" and tc."deletedAt" is null and tc."isActive"=\'1\' and tc."signUpStatusId" = 15 and tc."channelTypeId" in(:channeltype) where con."deletedAt" is null and con."isActive"=\'1\' ';
        await this.sequelizeInstance.query(channelquery, {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
                channeltype: channeltype,
                businnessType: businessType == 3 ? [1, 2, 3] : [businessType],
                channelId: channelId,
                contactId: contactId,
                sup: sup
            }
        }).then(x => {
            channels.push(...x);
        })

        //console.log("channels:",channels);
        //Getting Locations of Contacts to be Matched
        this.profile(typeid, businessType, channels, channelId, profileTypeId);

        let locationquery = 'Select pc."countryId",cpc."channelId",cpc."contactId" from "tblContactLocation" cpc inner join "tblRegionCountryJCT" pc on pc."regionCountryJCTId" = cpc."regionCountryJCTId" '
            + 'where cpc."contactId" in(:contacts) and CAST(cpc."locationType" AS text) in (:location) and cpc."deletedAt" is null';

        await this.sequelizeInstance.query(locationquery, {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
                location: destLocationType,
                contacts: channels.map(x => x.contactId),
            }
        }).then(x => {
            //result.push(x.map(con => {con}))
            targetLocation = x//.map(con => +con.countryId));
        })

        //Getting Locations of login Contacts
        locationquery = 'Select pc."countryId" from "tblContactLocation" cpc inner join "tblRegionCountryJCT" pc on pc."regionCountryJCTId" = cpc."regionCountryJCTId" '
            + 'where cpc."contactId" =:contactId and cpc."deletedAt" is null and CAST(cpc."locationType" AS text) in(:location) ';

        srcLocation = await this.sequelizeInstance.query(locationquery, {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
                location: srcLocationType,
                contactId: contactId
            }
        }).then(x => {
            return x.map(c => c.countryId);//new Map(x)//.map(con => +con.countryId));
        })

        location = targetLocation.filter(ch => { return srcLocation.includes(ch.countryId) });

        //product matching for vendor user
        // if(typeid==2)
        // {
        let productquery = 'Select cpc."productCategorieId",cpc."contactId" from "tblInterestedProductCategory" cpc where "deletedAt" is null and cpc."contactId" in(:contacts) '
            + 'and cpc."productCategorieId" in (Select cpc."productCategorieId" from "tblInterestedProductCategory" cpc where "deletedAt" is null and cpc."contactId" =:contactId) ';

        await this.sequelizeInstance.query(productquery, {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
                contacts: channels.map(x => x.contactId),
                contactId: contactId
            }
        }).then(async (x) => {
            product.push(...x)//_.uniqBy(x,'contactId');
            let filterch = product.map(p => +p.contactId);
            result = channels.filter(ch => { return !filterch.includes(ch.contactId); })

            if (result && result.length) {
                let productquery = 'Select cpc."productCategorieId",cpc."channelId" from "tblChannelProductCategorie" cpc where "deletedAt" is null and cpc."channelId" in(:channels) '
                    + 'and cpc."productCategorieId" in (Select cpc."productCategorieId" from "tblChannelProductCategorie" cpc where "deletedAt" is null and cpc."channelId" =:channelId)';

                await this.sequelizeInstance.query(productquery, {
                    type: QueryTypes.SELECT,
                    raw: true,
                    replacements: {
                        channels: result.map(x => x.channelId),
                        channelId: channelId
                    }
                }).then(x => {
                    product.push(...x);
                    //console.log("product:",product);
                })
            }


        })
        //}

        // else if(typeid==3||typeid==4)
        // {

        // let productquery='Select cpc."productCategorieId",cpc."contactId" from "tblInterestedProductCategory" cpc where "deletedAt" is null and cpc."contactId" in(:contacts) '
        // + 'and cpc."productCategorieId" in (Select cpc."productCategorieId" from "tblInterestedProductCategory" cpc where "deletedAt" is null and cpc."contactId" =:contactId) ';

        // await this.sequelizeInstance.query(productquery, { //contact vs contact
        // type: QueryTypes.SELECT,
        // raw: true,
        // replacements: {
        // contacts:channels.map(x => x.contactId),
        // contactId:contactId
        // }
        // }).then(async (x)=>{ //contact vs channel
        // product=x;//_.uniqBy(x,'contactId');
        // let filterch=product.map(p => +p.contactId);
        // result = channels.filter(ch => {return !filterch.includes(ch.contactId);})

        // if(result && result.length!=0)
        // {
        // let productquery='Select cpc."productCategorieId",cpc."channelId" from "tblChannelProductCategorie" cpc where "deletedAt" is null and cpc."channelId" in(:channels) '
        // + 'and cpc."productCategorieId" in (Select cpc."productCategorieId" from "tblContactInterestedProductCategory" cpc where "deletedAt" is null and cpc."contactId" =:contactId) ';

        // await this.sequelizeInstance.query(productquery, {
        // type: QueryTypes.SELECT,
        // raw: true,
        // replacements: {
        // channels:result.map(x => +x.channelId),
        // contactId:contactId
        // }
        // }).then(async (x) => { //channel vs channel
        // product.length?product.push(...x):(product=x); //=_.uniqBy(x,'contactId');
        // let filterch=product.map(p => +p.channelId);
        // let filtercon=product.map(p => +p.contactId);
        // result = channels.filter(ch => {return !filterch.includes(ch.channelId)&&!filtercon.includes(ch.conatctId);})
        // if(result && result.length!=0)
        // {
        // let productquery='Select cpc."productCategorieId",cpc."channelId" from "tblChannelProductCategorie" cpc where "deletedAt" is null and cpc."channelId" in(:channels) '
        // + 'and cpc."productCategorieId" in (Select cpc."productCategorieId" from "tblChannelProductCategorie" cpc where "deletedAt" is null and cpc."channelId" =:channelId) ';

        // await this.sequelizeInstance.query(productquery, {
        // type: QueryTypes.SELECT,
        // raw: true,
        // replacements: {
        // channels:result.map(x => +x.channelId),
        // channelId:channelId
        // }
        // }).then(x => {
        // product.length?product.push(...x):(product=x); //=_.uniqBy(x,'contactId');
        // //console.log("product:",product);

        // })
        // }
        // })
        // }
        // })
        // }
        //end of product matching

        //profile matching
        if (typeid == 2) {
            let profilequery = 'select cp."channelId",tc."profileTypeId" from "tblChannelCustomerProfile" as tc inner join "tblChannelCustomerProfile" as cp on tc."profileTypeId"=cp."profileTypeId" and cp."channelId" in(:channels) and cp."deletedAt" is null where tc."channelId" =:channelId and tc."deletedAt" is null ';
            await this.sequelizeInstance.query(profilequery, {
                type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                    channels: channels.map(x => x.channelId),
                    channelId: channelId
                }
            }).then(async (x) => {
                profile = x;
                let filterch = profile.map(p => +p.channelId);
                result = channels.filter(ch => { return !filterch.includes(ch.channelId); })

                let profilequery = 'select cp."profileTypeId",tc."channelId" from "tblChannel" as tc inner join "tblChannelCustomerProfile" as cp on cp."channelId" =:channelId and tc."profileTypeId"=cp."profileTypeId" and cp."deletedAt" is null where tc."channelId" in (:channels) and tc."channelTypeId"=4';
                await this.sequelizeInstance.query(profilequery, {
                    type: QueryTypes.SELECT,
                    raw: true,
                    replacements: {
                        channels: result.map(x => x.channelId),
                        channelId: channelId
                    }
                }).then(async (x) => {
                    profile.push(...x);
                })
            })

        }
        if (typeid == 3) {
            let profilequery = 'select cp."channelId",tc."profileTypeId" from "tblChannelCustomerProfile" as tc inner join "tblChannelCustomerProfile" as cp on tc."profileTypeId"=cp."profileTypeId" and cp."channelId" in(:channels) and cp."deletedAt" is null where tc."channelId" =:channelId and tc."deletedAt" is null ';
            await this.sequelizeInstance.query(profilequery, {
                type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                    channels: channels.map(x => x.channelId),
                    channelId: channelId
                }
            }).then(async (x) => {
                profile = x;
                let filterch = profile.map(p => +p.channelId);
                result = channels.filter(ch => { return !filterch.includes(ch.channelId); })

                let profilequery = 'select cp."profileTypeId",tc."channelId" from "tblChannel" as tc inner join "tblChannelCustomerProfile" as cp on cp."channelId" =:channelId and tc."profileTypeId"=cp."profileTypeId" and cp."deletedAt" is null where tc."channelId" in (:channels) and tc."channelTypeId"=4';
                await this.sequelizeInstance.query(profilequery, {
                    type: QueryTypes.SELECT,
                    raw: true,
                    replacements: {
                        channels: result.map(x => x.channelId),
                        channelId: channelId
                    }
                }).then(async (x) => {
                    profile.push(...x);
                })
            })

        }

        if (typeid == 4) {
            let profilequery = 'select cp."profileTypeId",cp."channelId" from "tblChannel" as tc inner join "tblChannelCustomerProfile" as cp on cp."channelId" in(:channels) and tc."profileTypeId"=cp."profileTypeId" and cp."deletedAt" is null where tc."channelId" = :channelId ';
            await this.sequelizeInstance.query(profilequery, {
                type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                    channels: channels.map(x => x.channelId),
                    channelId: channelId
                }
            }).then(async (x) => {
                profile = x;
            })

        }

        //end of profile matching
        let locMatch = location.map(x => x.channelId);
        let proMatch = profile.map(x => x.channelId);
        let productContact = product.map(x => x.contactId);
        let productChannel = product.map(x => x.channelId);

        channels = channels.filter(ch => {
            return locMatch.includes(ch.channelId)
                // && proMatch.includes(ch.channelId) &&
                // (productChannel.includes(ch.channelId) || productContact.includes(ch.contactId))
                ;
        })

        channels = channels.filter(ch => { return proMatch.includes(ch.channelId); })

        channels = channels.filter(ch => { return (productChannel.includes(ch.channelId) || productContact.includes(ch.contactId)); })


        let data = {
            channels: channels,
            // Product:product,
            // Location:location,//srcLocation.size,
            // Profile:profile
        }

        // let channelMatch = [].concat(channels.map(x => {
        //     return {channelId:channelId,partnerChannelId: x.channelId, request_to: x.contactId,request_by:contactId,mailStatus: '0'};
        // }));
        let channelJCT = [];

        // let channelJCT = [].concat(channels.map(x => {
        //     return {channelId:channelId,partnerChannelId: x.channelId, requested_to: x.contactId,requested_by:contactId,connectionStatusTypeId: 1};
        // }));

        console.log("planId:", planId, typeid);

        channels.map(x => {

            let confidence = '1';
            let dconfidence = '1';

            if ((planId == 1 || planId == 6) && typeid == 2) {
                console.log("planId:", planId);
                confidence = '0';
            }

            if (planId = 1 && typeid == 3 && x.channelTypeId == 4)
                confidence = '0';

            if (planId == 1 && typeid == 3 && x.channelTypeId == 2) {
                confidence = '1';
                if (x.planId == 1)
                    dconfidence = '0';
            }

            if (planId == 1 && typeid == 4) {
                confidence = '1';
                if (x.planId == 1)
                    dconfidence = '0';
            }

            channelJCT.push({ channelId: x.channelId, partnerChannelId: channelId, requested_to: contactId, requested_by: x.contactId, connectionStatusTypeId: 1, isConfidential: confidence });
            if (x.share > 0)
                channelJCT.push({ channelId: channelId, partnerChannelId: x.channelId, requested_to: x.contactId, requested_by: contactId, connectionStatusTypeId: 1, isConfidential: dconfidence });

        });



        //await this.channelMatchingRepository.bulkCreate(channelMatch);

        //await this.channelJctRepository.destroy()

        await this.channelJctRepository.destroy({
            where: {
                requested_to: contactId,
                //channelId: channelId,
                connectionStatusTypeId: 1
            }
        });

        await this.channelJctRepository.destroy({
            where: {
                requested_by: contactId,
                //partnerChannelId: channelId,
                connectionStatusTypeId: 1
            }
        });

        //console.log("jct:",channelJCT);

        let res = await this.channelJctRepository.bulkCreate(channelJCT);
        this.ranking({ planId: planId, contactId: contactId, channelId: channelId, channelTypeId: user.channelTypeId });
        //console.log("jct:",res);
        return res;
    }

    async productAlgm(user) {

        let contactId = user.contactId;
        let typeid = user.channelTypeId;
        let channelId = user.channelId;
        let profileTypeId = user.profileTypeId;
        let result = []; let innovation = [];
        let query;
        if (typeid == 3) {
            query = 'select tp."productId" from "tblChannelCustomerProfile" as tcp inner join "tblProduct" as tp on tp."deletedat" is null and  tp."channelId"=tcp."channelId" and tp."isSuggestion"=\'1\' '
                //+ 'and tp."productId" in(select "productId" from "tblproductsharepreference" where "sharetypeid"=2 and "deletedat" is null) '

                + 'and tp."productCategorieId" in(select "productCategorieId" from "tblInterestedProductCategory" where "contactId"=' + contactId + ' '
                + 'and "deletedAt" is null union select "productCategorieId" from "tblChannelProductCategorie" where "channelId"=' + channelId + 'and "deletedAt" is null) '

                + 'and tp."productId" in (select "productId" from "tblproductlocation" where "locationtype"=\'TARGET\' and "deletedat" is null '
                + 'and "regionCountryJCTId" in (select "regionCountryJCTId" from "tblContactLocation" where "contactId"=' + contactId + ' and "deletedAt" is null '
                + 'and CAST("locationType" AS text)=\'SELLING\')) '

                + 'and "productId" not in(select "productid" from "tblproductrejection" where "channelid"=' + channelId + ') '
                + 'and "productId" in(select "productId" from "tblProduct" where "deletedat" is null and "isactive"=\'1\')'


                // + 'and tcp."channelId" in(select "channelId" from "tblChannel" where "channelTypeId"=2 '
                // + 'and "deletedAt" is null and "channelId" in(select "channelId" from "tblChannelDetail" where "startup"=\'0\' and "deletedAt" is null) union '
                // + 'select "channelId" from "tblChannel" where "channelTypeId"=2 and "deletedAt" is null and "channelId" in(select "channelId" from "tblChannelDetail" '
                // + 'where "deletedAt" is null and "startup"=(select "startup" from "tblChannelDetail" where "channelId"=' + channelId + '))) '

                + 'and tcp."profileTypeId" in(select "profileTypeId" from "tblChannelCustomerProfile" where "deletedAt" is null and "channelId"=' + channelId + ') where tcp."deletedAt" is null order by tp."updatedat" desc';
        }
        else if (typeid == 4) {

            query = 'select tp."productId" from "tblChannelCustomerProfile" as tcp inner join "tblProduct" as tp on tp."deletedat" is null and  tp."channelId"=tcp."channelId" and tp."isSuggestion"=\'1\' '
                //+ 'and tp."productId" in(select "productId" from "tblproductsharepreference" where "sharetypeid"=2 and "deletedat" is null) '

                + 'and tp."productCategorieId" in(select "productCategorieId" from "tblInterestedProductCategory" where "contactId"=' + contactId + ' '
                + 'and "deletedAt" is null union select "productCategorieId" from "tblChannelProductCategorie" where "channelId"=' + channelId + 'and "deletedAt" is null) '

                + 'and tp."productId" in (select "productId" from "tblproductlocation" where "locationtype"=\'TARGET\' and "deletedat" is null '
                + 'and "regionCountryJCTId" in (select "regionCountryJCTId" from "tblContactLocation" where "contactId"=' + contactId + ' and "deletedAt" is null '
                + 'and CAST("locationType" AS text)=\'RETAIL\')) '

                + 'and "productId" not in(select "productid" from "tblproductrejection" where "channelid"=' + channelId + ') '
                + 'and "productId" in(select "productId" from "tblProduct" where "deletedat" is null and "isactive"=\'1\')'


                // + 'and tcp."channelId" in(select "channelId" from "tblChannel" where "channelTypeId"=2 '
                // + 'and "deletedAt" is null and "channelId" in(select "channelId" from "tblChannelDetail" where "startup"=\'0\' and "deletedAt" is null) union '
                // + 'select "channelId" from "tblChannel" where "channelTypeId"=2 and "deletedAt" is null and "channelId" in(select "channelId" from "tblChannelDetail" '
                // + 'where "deletedAt" is null and "startup"=(select "startup" from "tblChannelDetail" where "channelId"=' + channelId + '))) '

                + 'and tcp."profileTypeId"=' + profileTypeId + 'where tcp."deletedAt" is null order by tp."updatedat" desc';
        }
        else if (typeid == 5) {

            query = 'select tp."productId" from "tblChannelCustomerProfile" as tcp inner join "tblProduct" as tp on tp."deletedat" is null and  tp."channelId"=tcp."channelId" and tp."isSuggestion"=\'1\' '
                //+ 'and tp."productId" in(select "productId" from "tblproductsharepreference" where "sharetypeid"=2 and "deletedat" is null) '

                + 'and tp."productCategorieId" in(select "productCategorieId" from "tblInterestedProductCategory" where "contactId"=' + contactId + ' '
                + 'and "deletedAt" is null union select "productCategorieId" from "tblChannelProductCategorie" where "channelId"=' + channelId + 'and "deletedAt" is null) '

                + 'and tp."productId" in (select "productId" from "tblproductlocation" where "locationtype"=\'TARGET\' and "deletedat" is null '
                + 'and "regionCountryJCTId" in (select "regionCountryJCTId" from "tblContactLocation" where "contactId"=' + contactId + ' and "deletedAt" is null '
                + 'and CAST("locationType" AS text)=\'RESELLING\')) '

                + 'and "productId" not in(select "productid" from "tblproductrejection" where "channelid"=' + channelId + ') '
                + 'and "productId" in(select "productId" from "tblProduct" where "deletedat" is null and "isactive"=\'1\')'


                // + 'and tcp."channelId" in(select "channelId" from "tblChannel" where "channelTypeId"=2 '
                // + 'and "deletedAt" is null and "channelId" in(select "channelId" from "tblChannelDetail" where "startup"=\'0\' and "deletedAt" is null) union '
                // + 'select "channelId" from "tblChannel" where "channelTypeId"=2 and "deletedAt" is null and "channelId" in(select "channelId" from "tblChannelDetail" '
                // + 'where "deletedAt" is null and "startup"=(select "startup" from "tblChannelDetail" where "channelId"=' + channelId + '))) '

                + 'and tcp."profileTypeId"=' + profileTypeId + 'where tcp."deletedAt" is null order by tp."updatedat" desc';
        }


        try {
            result = await this.sequelizeInstance.query(query, {
                type: QueryTypes.SELECT,
                raw: true,
            }).then(x => {
                return x.map(p => +p.productId);
            });

            if (result.length == 0) {
                if (typeid == 3) {
                    query = 'select tp."productId" from "tblChannelCustomerProfile" as tcp inner join "tblProduct" as tp on tp."deletedat" is null and  tp."channelId"=tcp."channelId" and tp."isSuggestion"=\'1\' '
                        //+ 'and tp."productId" in(select "productId" from "tblproductsharepreference" where "sharetypeid"=2 and "deletedat" is null) '

                        + 'and tp."productCategorieId" in(select "productCategorieId" from "tblInterestedProductCategory" where "contactId"=' + contactId + ' '
                        + 'and "deletedAt" is null union select "productCategorieId" from "tblChannelProductCategorie" where "channelId"=' + channelId + 'and "deletedAt" is null) '

                        + 'and tp."productId" in (select "productId" from "tblproductlocation" where "locationtype"=\'TARGET\' and "deletedat" is null '
                        + 'and "regionCountryJCTId" in (select "regionCountryJCTId" from "tblChannelLocation" where "channelId"=' + channelId + ' and "deletedAt" is null '
                        + 'and CAST("locationType" AS text)=\'SELLING\')) '

                        + 'and "productId" not in(select "productid" from "tblproductrejection" where "channelid"=' + channelId + ') '
                        + 'and "productId" in(select "productId" from "tblProduct" where "deletedat" is null and "isactive"=\'1\')'


                        // + 'and tcp."channelId" in(select "channelId" from "tblChannel" where "channelTypeId"=2 '
                        // + 'and "deletedAt" is null and "channelId" in(select "channelId" from "tblChannelDetail" where "startup"=\'0\' and "deletedAt" is null) union '
                        // + 'select "channelId" from "tblChannel" where "channelTypeId"=2 and "deletedAt" is null and "channelId" in(select "channelId" from "tblChannelDetail" '
                        // + 'where "deletedAt" is null and "startup"=(select "startup" from "tblChannelDetail" where "channelId"=' + channelId + '))) '

                        + 'and tcp."profileTypeId" in(select "profileTypeId" from "tblChannelCustomerProfile" where "deletedAt" is null and "channelId"=' + channelId + ') where tcp."deletedAt" is null order by tp."updatedat" desc';
                }
                else if (typeid == 4) {

                    query = 'select tp."productId" from "tblChannelCustomerProfile" as tcp inner join "tblProduct" as tp on tp."deletedat" is null and  tp."channelId"=tcp."channelId" and tp."isSuggestion"=\'1\' '
                        //+ 'and tp."productId" in(select "productId" from "tblproductsharepreference" where "sharetypeid"=2 and "deletedat" is null) '

                        + 'and tp."productCategorieId" in(select "productCategorieId" from "tblInterestedProductCategory" where "contactId"=' + contactId + ' '
                        + 'and "deletedAt" is null union select "productCategorieId" from "tblChannelProductCategorie" where "channelId"=' + channelId + 'and "deletedAt" is null) '

                        + 'and tp."productId" in (select "productId" from "tblproductlocation" where "locationtype"=\'TARGET\' and "deletedat" is null '
                        + 'and "regionCountryJCTId" in (select "regionCountryJCTId" from "tblChannelLocation" where "channelId"=' + channelId + ' and "deletedAt" is null '
                        + 'and CAST("locationType" AS text)=\'RETAIL\')) '

                        + 'and "productId" not in(select "productid" from "tblproductrejection" where "channelid"=' + channelId + ') '
                        + 'and "productId" in(select "productId" from "tblProduct" where "deletedat" is null and "isactive"=\'1\')'

                        // + 'and tcp."channelId" in(select "channelId" from "tblChannel" where "channelTypeId"=2 '
                        // + 'and "deletedAt" is null and "channelId" in(select "channelId" from "tblChannelDetail" where "startup"=\'0\' and "deletedAt" is null) union '
                        // + 'select "channelId" from "tblChannel" where "channelTypeId"=2 and "deletedAt" is null and "channelId" in(select "channelId" from "tblChannelDetail" '
                        // + 'where "deletedAt" is null and "startup"=(select "startup" from "tblChannelDetail" where "channelId"=' + channelId + '))) '

                        + 'and tcp."profileTypeId"=' + profileTypeId + 'where tcp."deletedAt" is null order by tp."updatedat" desc';
                }
                else if (typeid == 5) {

                    query = 'select tp."productId" from "tblChannelCustomerProfile" as tcp inner join "tblProduct" as tp on tp."deletedat" is null and  tp."channelId"=tcp."channelId" and tp."isSuggestion"=\'1\' '
                        //+ 'and tp."productId" in(select "productId" from "tblproductsharepreference" where "sharetypeid"=2 and "deletedat" is null) '

                        + 'and tp."productCategorieId" in(select "productCategorieId" from "tblInterestedProductCategory" where "contactId"=' + contactId + ' '
                        + 'and "deletedAt" is null union select "productCategorieId" from "tblChannelProductCategorie" where "channelId"=' + channelId + 'and "deletedAt" is null) '

                        + 'and tp."productId" in (select "productId" from "tblproductlocation" where "locationtype"=\'TARGET\' and "deletedat" is null '
                        + 'and "regionCountryJCTId" in (select "regionCountryJCTId" from "tblChannelLocation" where "channelId"=' + channelId + ' and "deletedAt" is null '
                        + 'and CAST("locationType" AS text)=\'RESELLING\')) '

                        + 'and "productId" not in(select "productid" from "tblproductrejection" where "channelid"=' + channelId + ') '
                        + 'and "productId" in(select "productId" from "tblProduct" where "deletedat" is null and "isactive"=\'1\')'

                        // + 'and tcp."channelId" in(select "channelId" from "tblChannel" where "channelTypeId"=2 '
                        // + 'and "deletedAt" is null and "channelId" in(select "channelId" from "tblChannelDetail" where "startup"=\'0\' and "deletedAt" is null) union '
                        // + 'select "channelId" from "tblChannel" where "channelTypeId"=2 and "deletedAt" is null and "channelId" in(select "channelId" from "tblChannelDetail" '
                        // + 'where "deletedAt" is null and "startup"=(select "startup" from "tblChannelDetail" where "channelId"=' + channelId + '))) '

                        + 'and tcp."profileTypeId"=' + profileTypeId + 'where tcp."deletedAt" is null order by tp."updatedat" desc';
                }

                result = await this.sequelizeInstance.query(query, {
                    type: QueryTypes.SELECT,
                    raw: true,
                }).then(x => {
                    return x.map(p => +p.productId);
                });

            }

            result = _.uniq(result);

            let productJCT = [];

            if (result.length > 0) {
                let intcat = await this.sequelizeInstance.query('select tp."productId",tp."channelId",(select "profileStrength" as profile_strength from "tblChannel" where "channelId"=tp."channelId") from "tblProduct" as tp where tp."productId" in(:productId) and tp."productCategorieId" in(select "productCategorieId" from "tblInterestedProductCategory" where  "contactId"=' + contactId + ' and "deletedAt" is null)', {
                    type: QueryTypes.SELECT,
                    raw: true,
                    replacements: {
                        productId: result

                    }
                }).then(x => {
                    return x;
                });

                let chcat = await this.sequelizeInstance.query('select tp."productId",tp."channelId",(select "profileStrength" as profile_strength from "tblChannel" where "channelId"=tp."channelId") from "tblProduct" as tp where tp."productId" in(:productId) and tp."productCategorieId" in(select "productCategorieId" from "tblChannelProductCategorie" where "channelId"=' + channelId + ' and "deletedAt" is null)', {
                    type: QueryTypes.SELECT,
                    raw: true,
                    replacements: {
                        productId: result

                    }
                }).then(x => {
                    return x;
                });

                let results = [];
                let mscore;
                let i = 1;

                if (intcat.length > 0) {
                    intcat.map(x => {
                        if (x.profile_strength > 75)
                            mscore = 80 + 20;
                        else if (x.profile_strength >= 70 && x.profile_strength <= 75)
                            mscore = 80 + 15;
                        else if (x.profile_strength >= 65 && x.profile_strength < 70)
                            mscore = 80 + 10;
                        else if (x.profile_strength < 65)
                            mscore = 80 + 8;
                        Object.assign(x, { rank: i++, matchingScore: mscore });
                        // Object.assign(x,{isMatched:true});				
                        // Object.assign(x,{rank:i++});
                    })
                    results.push(...intcat);
                }

                if (chcat.length > 0) {
                    chcat.map(x => {
                        if (x.profile_strength > 75)
                            mscore = 70 + 20;
                        else if (x.profile_strength >= 70 && x.profile_strength <= 75)
                            mscore = 70 + 15;
                        else if (x.profile_strength >= 65 && x.profile_strength < 70)
                            mscore = 70 + 10;
                        else if (x.profile_strength < 65)
                            mscore = 70 + 8;
                        Object.assign(x, { rank: i++, matchingScore: mscore });
                    })
                    results.push(...chcat);
                }

                results = _.uniqBy(results, 'productId');
                results.map(x => {
                    productJCT.push({ channelId: channelId, partnerChannelId: x.channelId, requested_to: x.contactId, requested_by: contactId, connectionStatusTypeId: 1, productId: x.productId, rank: x.rank });
                })

                await this.productJctRepository.destroy({
                    where: {
                        requested_by: contactId,
                        //channelId: channelId,
                        connectionStatusTypeId: 1
                    }
                });
            }

            let res = await this.productJctRepository.bulkCreate(productJCT);

            return res;
        }
        catch (e) {
            console.log("error:", e);
        }
    }

    async channelSuggestionAlgm(user) {


        let contactId = user.contactId;
        let typeid = user.channelTypeId;
        let channelId = user.channelId;
        let result = []; let product = []; let location = []; let profile = [];

        let channeltype = []; let srcLocationType = []; let destLocationType = [];

        let channels = [];
        let srcLocation;
        let targetLocation;

        if (typeid == 2) {
            destLocationType = ['RETAIL', 'SELLING'];
            srcLocationType = ['TARGET'];
            channeltype = [3, 4];
        }
        else if (typeid == 3) {
            destLocationType = ['RETAIL', 'TARGET'];
            srcLocationType = ['SELLING', 'TARGET'];
            channeltype = [2, 4];
        }
        else if (typeid == 4) {
            destLocationType = ['SELLING', 'TARGET'];
            srcLocationType = ['RETAIL'];
            channeltype = [2, 3];
        }

        //Getting all Contacts to be matched
        let channelquery = 'select con."contactId",con."channelId",tc."channelTypeId" from "tblContact" as con inner join "tblChannel" as tc on tc."channelId"=con."channelId" and tc."deletedAt" is null and tc."isActive"=\'1\' and tc."signUpStatusId" = 15 and tc."channelTypeId" in(:channeltype) where con."deletedAt" is null and con."isActive"=\'1\' ';
        await this.sequelizeInstance.query(channelquery, {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
                channeltype: channeltype
            }
        }).then(x => {
            channels.push(...x);
        })
        //console.log("channels:",channels);
        //Getting Locations of Contacts to be Matched
        let locationquery = 'Select pc."countryId",cpc."channelId",cpc."contactId" from "tblContactLocation" cpc inner join "tblRegionCountryJCT" pc on pc."regionCountryJCTId" = cpc."regionCountryJCTId" '
            + 'where cpc."contactId" in(:contacts) and CAST(cpc."locationType" AS text) in (:location) and cpc."deletedAt" is null';

        await this.sequelizeInstance.query(locationquery, {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
                location: destLocationType,
                contacts: channels.map(x => x.contactId),
            }
        }).then(x => {
            //result.push(x.map(con => {con}))
            targetLocation = x//.map(con => +con.countryId));
        })

        //Getting Locations of login Contacts
        locationquery = 'Select pc."countryId" from "tblContactLocation" cpc inner join "tblRegionCountryJCT" pc on pc."regionCountryJCTId" = cpc."regionCountryJCTId" '
            + 'where cpc."contactId" =:contactId and cpc."deletedAt" is null and CAST(cpc."locationType" AS text) in(:location) ';

        await this.sequelizeInstance.query(locationquery, {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
                location: srcLocationType,
                contactId: contactId
            }
        }).then(x => {
            srcLocation = x//new Map(x)//.map(con => +con.countryId));
        })

        srcLocation.map(s => {
            location = targetLocation.filter(t => t.countryId == s.countryId)
        })

        //product matching for vendor user
        // if(typeid==2)
        // {
        let productquery = 'Select cpc."productCategorieId",cpc."contactId" from "tblInterestedProductCategory" cpc where "deletedAt" is null and cpc."contactId" in(:contacts) '
            + 'and cpc."productCategorieId" in (Select cpc."productCategorieId" from "tblInterestedProductCategory" cpc where "deletedAt" is null and cpc."contactId" =:contactId) ';

        await this.sequelizeInstance.query(productquery, {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
                contacts: channels.map(x => x.contactId),
                contactId: contactId
            }
        }).then(async (x) => {
            product.push(...x)//_.uniqBy(x,'contactId');
            let filterch = product.map(p => +p.contactId);
            result = channels.filter(ch => { return !filterch.includes(ch.contactId); })

            if (result && result.length) {
                let productquery = 'Select cpc."productCategorieId",cpc."channelId" from "tblChannelProductCategorie" cpc where "deletedAt" is null and cpc."channelId" in(:channels) '
                    + 'and cpc."productCategorieId" in (Select cpc."productCategorieId" from "tblChannelProductCategorie" cpc where "deletedAt" is null and cpc."channelId" =:channelId)';

                await this.sequelizeInstance.query(productquery, {
                    type: QueryTypes.SELECT,
                    raw: true,
                    replacements: {
                        channels: result.map(x => x.channelId),
                        channelId: channelId
                    }
                }).then(x => {
                    product.push(...x);
                    //console.log("product:",product);
                })
            }


        })
        //}

        // else if(typeid==3||typeid==4)
        // {

        // let productquery='Select cpc."productCategorieId",cpc."contactId" from "tblInterestedProductCategory" cpc where "deletedAt" is null and cpc."contactId" in(:contacts) '
        // + 'and cpc."productCategorieId" in (Select cpc."productCategorieId" from "tblInterestedProductCategory" cpc where "deletedAt" is null and cpc."contactId" =:contactId) ';

        // await this.sequelizeInstance.query(productquery, { //contact vs contact
        // type: QueryTypes.SELECT,
        // raw: true,
        // replacements: {
        // contacts:channels.map(x => x.contactId),
        // contactId:contactId
        // }
        // }).then(async (x)=>{ //contact vs channel
        // product=x;//_.uniqBy(x,'contactId');
        // let filterch=product.map(p => +p.contactId);
        // result = channels.filter(ch => {return !filterch.includes(ch.contactId);})

        // if(result && result.length!=0)
        // {
        // let productquery='Select cpc."productCategorieId",cpc."channelId" from "tblChannelProductCategorie" cpc where "deletedAt" is null and cpc."channelId" in(:channels) '
        // + 'and cpc."productCategorieId" in (Select cpc."productCategorieId" from "tblContactInterestedProductCategory" cpc where "deletedAt" is null and cpc."contactId" =:contactId) ';

        // await this.sequelizeInstance.query(productquery, {
        // type: QueryTypes.SELECT,
        // raw: true,
        // replacements: {
        // channels:result.map(x => +x.channelId),
        // contactId:contactId
        // }
        // }).then(async (x) => { //channel vs channel
        // product.length?product.push(...x):(product=x); //=_.uniqBy(x,'contactId');
        // let filterch=product.map(p => +p.channelId);
        // let filtercon=product.map(p => +p.contactId);
        // result = channels.filter(ch => {return !filterch.includes(ch.channelId)&&!filtercon.includes(ch.conatctId);})
        // if(result && result.length!=0)
        // {
        // let productquery='Select cpc."productCategorieId",cpc."channelId" from "tblChannelProductCategorie" cpc where "deletedAt" is null and cpc."channelId" in(:channels) '
        // + 'and cpc."productCategorieId" in (Select cpc."productCategorieId" from "tblChannelProductCategorie" cpc where "deletedAt" is null and cpc."channelId" =:channelId) ';

        // await this.sequelizeInstance.query(productquery, {
        // type: QueryTypes.SELECT,
        // raw: true,
        // replacements: {
        // channels:result.map(x => +x.channelId),
        // channelId:channelId
        // }
        // }).then(x => {
        // product.length?product.push(...x):(product=x); //=_.uniqBy(x,'contactId');
        // //console.log("product:",product);

        // })
        // }
        // })
        // }
        // })
        // }
        //end of product matching

        //profile matching
        if (typeid == 2) {
            let profilequery = 'select cp."channelId",tc."profileTypeId" from "tblChannelCustomerProfile" as tc inner join "tblChannelCustomerProfile" as cp on tc."profileTypeId"=cp."profileTypeId" and cp."channelId" in(:channels) and cp."deletedAt" is null where tc."channelId" =:channelId and tc."deletedAt" is null ';
            await this.sequelizeInstance.query(profilequery, {
                type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                    channels: channels.map(x => x.channelId),
                    channelId: channelId
                }
            }).then(async (x) => {
                profile = x;
                let filterch = profile.map(p => +p.channelId);
                result = channels.filter(ch => { return !filterch.includes(ch.channelId); })

                let profilequery = 'select cp."profileTypeId",tc."channelId" from "tblChannel" as tc inner join "tblChannelCustomerProfile" as cp on cp."channelId" =:channelId and tc."profileTypeId"=cp."profileTypeId" and cp."deletedAt" is null where tc."channelId" in (:channels) and tc."channelTypeId"=4';
                await this.sequelizeInstance.query(profilequery, {
                    type: QueryTypes.SELECT,
                    raw: true,
                    replacements: {
                        channels: result.map(x => x.channelId),
                        channelId: channelId
                    }
                }).then(async (x) => {
                    profile.push(...x);
                })
            })

        }
        if (typeid == 3) {
            let profilequery = 'select cp."channelId",tc."profileTypeId" from "tblChannelCustomerProfile" as tc inner join "tblChannelCustomerProfile" as cp on tc."profileTypeId"=cp."profileTypeId" and cp."channelId" in(:channels) and cp."deletedAt" is null where tc."channelId" =:channelId and tc."deletedAt" is null ';
            await this.sequelizeInstance.query(profilequery, {
                type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                    channels: channels.map(x => x.channelId),
                    channelId: channelId
                }
            }).then(async (x) => {
                profile = x;
                let filterch = profile.map(p => +p.channelId);
                result = channels.filter(ch => { return !filterch.includes(ch.channelId); })

                let profilequery = 'select cp."profileTypeId",tc."channelId" from "tblChannel" as tc inner join "tblChannelCustomerProfile" as cp on cp."channelId" =:channelId and tc."profileTypeId"=cp."profileTypeId" and cp."deletedAt" is null where tc."channelId" in (:channels) and tc."channelTypeId"=4';
                await this.sequelizeInstance.query(profilequery, {
                    type: QueryTypes.SELECT,
                    raw: true,
                    replacements: {
                        channels: result.map(x => x.channelId),
                        channelId: channelId
                    }
                }).then(async (x) => {
                    profile.push(...x);
                })
            })

        }

        if (typeid == 4) {
            let profilequery = 'select cp."profileTypeId",cp."channelId" from "tblChannel" as tc inner join "tblChannelCustomerProfile" as cp on cp."channelId" in(:channels) and tc."profileTypeId"=cp."profileTypeId" and cp."deletedAt" is null where tc."channelId" = :channelId ';
            await this.sequelizeInstance.query(profilequery, {
                type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                    channels: channels.map(x => x.channelId),
                    channelId: channelId
                }
            }).then(async (x) => {
                profile = x;
            })

        }

        //end of profile matching
        let locMatch = location.map(x => x.channelId);
        let proMatch = profile.map(x => x.channelId);
        let productContact = product.map(x => x.contactId);
        let productChannel = product.map(x => x.channelId);

        channels = channels.filter(ch => {
            return locMatch.includes(ch.channelId)
                // && proMatch.includes(ch.channelId) &&
                // (productChannel.includes(ch.channelId) || productContact.includes(ch.contactId))
                ;
        })

        channels = channels.filter(ch => { return proMatch.includes(ch.channelId); })

        channels = channels.filter(ch => { return (productChannel.includes(ch.channelId) || productContact.includes(ch.contactId)); })


        let data = {
            channels: channels,
            // Product:product,
            // Location:location,//srcLocation.size,
            // Profile:profile
        }

        let isConfidentialFlag = (+user.channel.plan.subscriptionPlanId === 1) ? '0' : 1;
        let channelMatch = [].concat(channels.map(x => {
            return { channelId: channelId, partnerChannelId: x.channelId, request_to: x.contactId, request_by: contactId, mailStatus: '0', isConfidential: isConfidentialFlag };
        }));
        // console.log('channelMatch');
        // console.log(channelMatch);
        await this.channelJctRepository.bulkCreate(channelMatch);
        //  await this.createSuggestions(user);

        return data;

    }
    async createSuggestions(user) {
        var channelId = null;
        channelId = user.channelId


        console.log("=============================");
        console.log(channelId);
        console.log("=============================");
        let suggestionChannelTypes: {
            2: [
                { type: 'DISTRIBUTOR', type_id: 3, shareTypeId: 1, suggestionId: 8, limit: 50 },
                { type: 'RETAILER', type_id: 4, shareTypeId: 1, suggestionId: 8, limit: 50 }
            ],
            3: [
                { type: 'VENDOR', type_id: 2, shareTypeId: 2, suggestionId: 5, limit: 50 },
                { type: 'RETAILER', type_id: 4, shareTypeId: 2, suggestionId: 8, limit: 50 }
            ],
            4: [
                { type: 'VENDOR', type_id: 2, shareTypeId: 3, suggestionId: 5, limit: 50 },
                { type: 'DISTRIBUTOR', type_id: 3, shareTypeId: 3, suggestionId: 5, limit: 50 }
            ]
        };

        let subscriptionFeatures = {
            REDIS_KEY: 'SubscriptionFeatures',
            SUGGESTION_ID: [
                5,
                8,
                76,
                77,
                79,
                81
            ],
            OUTBOUND_IDS: [
                76,
                77,
                79,
                81
            ]
        };

        let insert = [];
        let queue = [];
        let currentDate = Sequelize.literal('CURRENT_TIMESTAMP');
        let endDate = Sequelize.literal("CURRENT_TIMESTAMP + interval '" + 30 + "' day"); // TODO: make 30 as a configuration

        let query = 'SELECT tcsub."channelSubscriptionId", tcsub."subscriptionId", tcsub."endAt", tc."channelId", tc."channelTypeId" from "tblChannelSubscription" tcsub INNER JOIN "tblChannel" tc ON tc."channelId" = tcsub."channelId" AND tc."channelTypeId" IN (2,3,4) WHERE tc."signUpStatusId"=\'15\' and tcsub."isActive" = \'1\' AND tcsub."startedAt" < now() AND tcsub."endAt" > now() AND NOT EXISTS( SELECT * FROM tblchannelsuggestions tcsug WHERE tcsug.channelsubscriptionid = tcsub."channelSubscriptionId" AND tcsug."startedat" < now() AND tcsug."endat" > now())';


        if (channelId) {
            query += ' AND tc."channelId" = ' + channelId;
        }

        await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
            raw: true,

        }).then(async (results) => {
            if (results && results.length) {
                // get limits
                var limits: any = {};

                let SubscriptionDetailQuery = 'SELECT "keyValue", "subscriptionId", "subscriptionFeatureId" from "tblSubscriptionDetail" where  "subscriptionFeatureId" IN ' + subscriptionFeatures.SUGGESTION_ID;

                // await SubscriptionDetail.findAll({
                //     attributes: ['keyValue', 'subscriptionId', 'subscriptionFeatureId'],
                //     where: {
                //         subscriptionFeatureId: {
                //             $in: subscriptionFeatures.SUGGESTION_ID
                //         }
                //     }

                await this.sequelizeInstance.query(SubscriptionDetailQuery, {
                    type: QueryTypes.SELECT,
                    raw: true,
                }).then((datas) => {
                    datas.forEach((data) => {
                        if (!limits[data.subscriptionFeatureId]) {
                            limits[data.subscriptionFeatureId] = {
                                [data.subscriptionId]: data.keyValue
                            };
                        } else {
                            limits[data.subscriptionFeatureId][data.subscriptionId] = data.keyValue;
                        }
                    })
                });

                var lastMonth = [];
                await results.forEach(async (data) => {
                    let suggestionfor = suggestionChannelTypes[data.channelTypeId];
                    let sepateLimits = {};
                    await suggestionfor.forEach((suggession) => {
                        insert.push({
                            channelsubscriptionid: data.channelSubscriptionId,
                            startedat: currentDate,
                            endat: endDate,
                            suggestionfor: suggession.type,
                            suggestioncount: 0,
                            limit: limits[suggession.suggestionId][data.subscriptionId],
                            createdat: currentDate
                        });

                        sepateLimits[suggession.type] = (limits[suggession.suggestionId][data.subscriptionId]) ? limits[suggession.suggestionId][data.subscriptionId] : 0;
                    });
                    lastMonth.push(data.channelSubscriptionId);
                });

                //add Expired
                await ChannelSuggestion.update({
                    isexpired: '1'
                }, {
                    where: {
                        channelsubscriptionid: {
                            $in: lastMonth
                        },
                        isexpired: '0'
                    }
                })

                //insert into database
                await ChannelSuggestion.bulkCreate(insert);




                // suggest queued orders
                // queue.forEach(async (data) => {
                //     await ClearSampleOrderQueue({
                //         data: {
                //             channelId: +data.channelId,
                //             subscriptionId: +data.subscriptionId
                //         }
                //     }, () => {
                //         console.log("queued orders processed")
                //     });
                // });
            } else {
                // if (channelId) {
                //     canSuggest = true;
                // }
            }
        });
    }

    async ranking(contactData) {

        //console.log("contact:",contactData);
        let channelId; let contactId;
        let contactIds; let channelIds;
        let channeltype = [];
        channelId = +contactData.channelId; contactId = +contactData.contactId;
        let planId = +contactData.planId;


        // let chtype=await this.sequelizeInstance.query('select "channelTypeId" from "tblChannel" where "channelId"=:chid ', {
        // type: QueryTypes.SELECT,
        // replacements: {
        // chid:channelId
        // }
        // });
        // let typeid=+contactData.channelTypeId;//chtype.map(x => x.channelTypeId);
        // if(typeid==2)
        // channeltype=[3,4];
        // else if(typeid==3)
        // channeltype=[2];
        // else if(typeid==4)
        // channeltype=[2];

        try {
            let contacts = await this.sequelizeInstance.query('select * from "tblChannelJCT" where "requested_to"=:contactId and "connectionStatusTypeId"=1 and "deletedat" is null ', {
                type: QueryTypes.SELECT,
                replacements: {
                    contactId: contactId
                }
            });



            contactIds = contacts.map(x => x.requested_by);
            channelIds = contacts.map(x => x.channelId);

            let contactids = contactIds.toString(); let channelids = channelIds.toString();

            //console.log("matching:", contactids);

            let result = await this.sequelizeInstance.query('select * from "EventMatchingAlgm"(:chid,:conid,:plan,:conids,:chids)', {
                type: QueryTypes.SELECT,
                replacements: {
                    conid: contactId,
                    chid: channelId,
                    plan: planId,
                    conids: contactids,
                    chids: channelids
                }
            });

            let i = 1;
            let results = [];
            let mscore;



            if (result[0].EventMatchingAlgm != null) {

                //result[0].EventMatchingAlgm.matched = _.orderBy(result[0].EventMatchingAlgm.matched, ['companyname'],['asc']);
                //result[0].EventMatchingAlgm.nonmatched = _.orderBy(result[0].EventMatchingAlgm.nonmatched, ['companyname'],['asc']);


                if (result[0].EventMatchingAlgm.intmatched != null) {
                    result[0].EventMatchingAlgm.intmatched.map(x => {
                        if (x.profile_strength > 75)
                            mscore = 80 + 20;
                        else if (x.profile_strength >= 70 && x.profile_strength <= 75)
                            mscore = 80 + 15;
                        else if (x.profile_strength >= 65 && x.profile_strength < 70)
                            mscore = 80 + 10;
                        else if (x.profile_strength < 65)
                            mscore = 80 + 8;

                        Object.assign(x, { isInterested: true, isMatched: true, rank: i++, matchingScore: mscore });
                        // Object.assign(x,{isMatched:true});				
                        // Object.assign(x,{rank:i++});
                    })
                    results.push(...result[0].EventMatchingAlgm.intmatched);
                }

                if (result[0].EventMatchingAlgm.matched != null) {
                    result[0].EventMatchingAlgm.matched.map(x => {
                        if (x.profile_strength > 75)
                            mscore = 70 + 20;
                        else if (x.profile_strength >= 70 && x.profile_strength <= 75)
                            mscore = 70 + 15;
                        else if (x.profile_strength >= 65 && x.profile_strength < 70)
                            mscore = 70 + 10;
                        else if (x.profile_strength < 65)
                            mscore = 70 + 8;
                        Object.assign(x, { isInterested: false, isMatched: true, rank: i++, matchingScore: mscore });
                    })
                    results.push(...result[0].EventMatchingAlgm.matched);
                }

                if (result[0].EventMatchingAlgm.nonmatched != null) {
                    result[0].EventMatchingAlgm.nonmatched.map(x => {
                        if (x.profile_strength > 75)
                            mscore = 60 + 20;
                        else if (x.profile_strength >= 70 && x.profile_strength <= 75)
                            mscore = 60 + 15;
                        else if (x.profile_strength >= 65 && x.profile_strength < 70)
                            mscore = 60 + 10;
                        else if (x.profile_strength < 65)
                            mscore = 60 + 8;
                        Object.assign(x, { isInterested: false, isMatched: false, rank: i++, matchingScore: mscore });
                    });
                    results.push(...result[0].EventMatchingAlgm.nonmatched);
                }

            }
            // let data={
            //  //   companyId:channelId,
            //  //   contactId:contactId,
            //     participant:results.length>0?results:[]                
            // }

            results.map(async x => {
                //console.log("rank:",contactId,x.contactid,x.rank);		  
                await AlgmChannelJCT.update({
                    rank: x.rank,
                    matchingScore: x.matchingScore
                }, {
                    where: {
                        requested_to: contactId,
                        requested_by: x.contactid,
                        connectionStatusTypeId: 1
                    }
                }
                );
            })

            //console.log("rank:",data);		
        }
        catch (e) {
            console.log("error:", e);
            return [];

        }
    }



}
