import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, BelongsTo, ForeignKey, HasMany, AfterCreate, Scopes } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
// import { Channel } from './Channel.entity';
// import { PaymentTransaction } from './PaymentTransaction.entity';
// import { Subscription } from './Subscription.entity';
// import { SubscriptionType } from './SubscriptionType.entity';
// import { Contact } from '../../user/models/Contact.entity';
// import { Coupon } from './Coupon.entity';
// import { AutoRenewalTracking } from './AutoRenewalTracking.entity';
import { ChannelSuggestion } from './ChannelSuggestion.entity';


const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblChannelSubscription' } as IDefineOptions;

@Table({
	tableName: "tblChannelSubscription"
})

@Scopes({
	channelDetail: {
		attributes: ['subscriptionId', 'actualAmount', 'finalAmount', 'paymentTransactionId', 'startedAt', 'startedAt', 'endAt', 'isActive']
	}
})

export class ChannelSubscription extends Model<ChannelSubscription>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	channelSubscriptionId: number

	//@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	channelId: number

	//@BelongsTo(() => Channel)
	//channel: Channel;

	//@HasMany(() => Contact, 'channelId')
	//contact: Contact[];

	//@ForeignKey(() => Subscription)
	@Column(DataType.BIGINT)
	subscriptionId: number

	// @BelongsTo(() => Subscription)
	// subscription: Subscription;

	// @ForeignKey(() => SubscriptionType)
	@Column(DataType.BIGINT)
	subscriptionTypeId: number

	// @BelongsTo(() => SubscriptionType)
	// subscriptionType: SubscriptionType;

	@AllowNull(false)
	@Column(DataType.DECIMAL(15, 2))
	actualAmount: number

	@Column(DataType.DECIMAL(15, 2))
	taxAmount: number

	@Column(DataType.CHAR(100))
	taxNumber: string

	@Column(DataType.DECIMAL(15, 2))
	carryForwardAmount: number

	//@ForeignKey(() => Coupon)
	@Column(DataType.INTEGER)
	couponId: number

	// @BelongsTo(() => Coupon)
	// coupon: Coupon

	@Column(DataType.DECIMAL(15, 2))
	couponAmount: number

	@Column(DataType.DECIMAL(15, 2))
	totalDiscountedAmount: number

	@AllowNull(false)
	@Column(DataType.DECIMAL(15, 2))
	finalAmount: number

	//@ForeignKey(() => PaymentTransaction)
	@Column(DataType.INTEGER)
	paymentTransactionId: number

	// @BelongsTo(() => PaymentTransaction)
	// paymentTransaction: PaymentTransaction

	@Column(DataType.TIME)
	startedAt: any

	@Column(DataType.INTEGER)
	additionalDays: number

	@Column(DataType.TIME)
	endAt: any

	@Column
	isActive: string

	@Column
	isPaid: string

	@Column(DataType.TIME)
	changedAt: number

	@ForeignKey(() => ChannelSubscription)
	@Column(DataType.INTEGER)
	oldChannelSubscriptionId: number

	@BelongsTo(() => ChannelSubscription)
	oldChannelSubscription: ChannelSubscription

	@HasMany(() => ChannelSuggestion)
	suggestions: ChannelSuggestion[];

	// @HasMany(() => AutoRenewalTracking)
	// autoRenewalTracking: AutoRenewalTracking;

	@Column(DataType.BIGINT)
	updatedBy: number

	@AfterCreate
	static async deactiveOldRecords(instance: ChannelSubscription) {
		if (instance.oldChannelSubscriptionId && instance.isPaid == '1') {
			let oldChannelSubscription = await ChannelSubscription.findByPk(instance.oldChannelSubscriptionId);

			if (oldChannelSubscription) {
				oldChannelSubscription.isActive = '0'
				await oldChannelSubscription.save();
			}
		}
	}
}
