import { Module } from '@nestjs/common';
import { ConfigModule } from '@nestjs/config';
import { AppController } from './app.controller';
//import { RedisModule } from './redis/redis.module';
import { AppService } from './app.service';
import { DatabaseModule } from './core/database/database.module';
import { AuthModule } from './modules/auth/auth.module';
import { ContextModule } from './context/context.module';
import { ChannelModule } from './modules/channel/channel.module';
import { UserModule } from './modules/user/user.module';
import { MediaModule } from './modules/media/media.module';
import { OrderModule } from './modules/order/order.module';
import { CommonModule } from './modules/common/common.module';
import { ProductModule } from './modules/product/product.module';
import { AccountModule } from './modules/account/account.module';
import { NotificationModule } from './modules/notification/notification.module';
import { HubspotModule } from './hubspot/hubspot.module';
import { ActivityModule } from './modules/activity/activity.module';
import { APP_FILTER, APP_INTERCEPTOR, APP_PIPE } from '@nestjs/core';
import { HttpErrorFilter } from './shared/http-error.filter';
import { LoggingInterceptor } from './shared/logging.interceptor';
import { EmailerModule } from './modules/emailer/emailer.module';
import { ValidationPipe } from './middleware/pipe/validation.pipe';
import { AdminModule } from './modules/admin/admin.module';
import { MulterModule } from '@nestjs/platform-express';
// import { FavouriteModule } from './modules/favourite/favourite.module';
import { AlgorithmModule } from './algorithm/algorithm.module';
import { MeetingModule } from './modules/meeting/meeting.module';
import { CartModule } from './modules/cart/cart.module';
import { WishlistModule } from './modules/wishlist/wishlist.module';
//import { BullQueueModule } from './bull/bull.module';
import { bullconfig } from './bull.config';
import { WalletRechargeModule } from './modules/walletRecharge/walletRecharge.module';
import { MailerserviceModule } from './mailerservice/mailerservice.module';
import { ComplainModule } from './complains/complain.module';
import { ErrorModule } from './error/error.module';
import { ReportModule } from './modules/report/report.module';

@Module({
  imports: [
    ConfigModule.forRoot({ isGlobal: true }),
    DatabaseModule,
    //RedisModule,
    ContextModule,
    AuthModule,
    ChannelModule,
    UserModule,
    MediaModule,
    OrderModule,
    CommonModule,
    ProductModule,
    AccountModule,
    NotificationModule,
    HubspotModule,
    ActivityModule,
    EmailerModule,
    AdminModule,
    MeetingModule,
    CartModule,
    MulterModule.register({
      dest:'./upload_files'
    }),
    // FavouriteModule,
    AlgorithmModule,
    WishlistModule,
    WalletRechargeModule,
   // BullQueueModule.forRoot(bullconfig),
    MailerserviceModule,
    ComplainModule,
    ErrorModule,
    ReportModule
  //  UsersModule,
],   
   controllers: [AppController],
   providers: [AppService,
    {
      provide:APP_FILTER,
      useClass:HttpErrorFilter,
    },
    {
      provide:APP_INTERCEPTOR,
      useClass:LoggingInterceptor,
    },
    {
      provide: APP_PIPE,
      useClass: ValidationPipe,
    }
  ],
})
export class AppModule {}
