'use strict';

import {
	UseGuards,
	Controller,
	Post,
	Request,
	Response,
	Body,
	Put,
	Get,
	Param,
	HttpStatus,
	ValidationPipe
} from '@nestjs/common';
import { ComplainService } from '../service/complain.service';
import { BaseController } from '../../core';
import { ComplainActionDto } from '../dto/complainAction.dto';
import { PaginationTransform } from 'src/core/transformation/paginationTransformation';
import { ComplainsFilterDto } from '../dto/complainsFilter.dto';
import { ComplainDto } from '../dto/complain.dto';

@Controller('master')
export class ComplainAdminController extends BaseController {
	private paginationTransform = new PaginationTransform();
	constructor(
		private complainService: ComplainService
	) { super(); }

	@Post('/')
	public async getComplainList(@Request() req, @Response() res, @Body(new ValidationPipe()) dto: any) {
		// ComplainsFilterDto
		let pagination = {
			limit:dto.limit,
			offset:dto.offset,
			search:dto.search
		}
		let sort = (req.query.sort && ['asc', 'desc'].includes(req.query.sort.toLowerCase())) ? req.query.sort.toLowerCase() : 'desc';
		let result: any = await this.complainService.getComplain(dto, sort);

		res.status(HttpStatus.OK).json(
			this.paginationTransform.make(
				result.rows,
				pagination,
				result.count,
				result.aggregations
			)
		);
	}

	@Post('/action/:complainId')
	public async setComplainAction(@Param('complainId') complainId, @Request() req, @Response() res, @Body(new ValidationPipe()) complainActionDto: ComplainActionDto) {
		let user = req.user;
		const result = await this.complainService.setComplainAction(user, complainId, complainActionDto);
		return res.status(HttpStatus.OK).json(result);
	}

	@Get('/:complaintype')
	public async getComplainCategoryByComplainType(@Param('complaintype') complaintype, @Request() req, @Response() res) {
		const result = await this.complainService.getComplainCategoryByComplainType(complaintype);
		return res.status(HttpStatus.OK).json(result);
	}

	@Post('/complain')
	public async setComplain(@Request() req, @Response() res, @Body(new ValidationPipe()) channelApprovalDto: ComplainDto) {
		let user = req.user;
		const result = await this.complainService.setComplain(user,channelApprovalDto);
		return res.status(HttpStatus.OK).json(result);
	}
}
