'use strict';

import {
	Controller,
	Post,
	Get,
	HttpStatus,
	Request,
	Response,
	Param,
	Body,
	ValidationPipe,
	UseGuards,
} from '@nestjs/common';
import { BaseController } from '../../core';
import { ComplainService } from '../service/complain.service';
import { ComplainDto } from '../dto/complain.dto';
import { AuthGuard } from '@nestjs/passport';

@UseGuards(AuthGuard('jwt'))
@Controller('complain')
export class ComplainController extends BaseController {

	constructor(
		private complainService: ComplainService
	) {
		super();
	}

	@Get('/category/:complaintype')
	public async getComplainCategoryByComplainType(@Param('complaintype') complaintype, @Request() req, @Response() res) {
		const result = await this.complainService.getComplainCategoryByComplainType(complaintype);
		return res.status(HttpStatus.OK).json(result);
	}

	@Post('/')
	public async setComplain(@Request() req, @Response() res, @Body(new ValidationPipe()) channelApprovalDto: ComplainDto) {
		let user = req.user;
		const result = await this.complainService.setComplain(user,channelApprovalDto);
		return res.status(HttpStatus.OK).json(result);
	}
}
