import { IsNotEmpty, IsString, IsNumber, IsOptional, ArrayNotEmpty, ValidateNested, IsEnum, MaxLength } from 'class-validator';
import { Type } from 'class-transformer';
import { CompalainDocumentDto } from './compalainDocument.dto';

export enum ComplainType {
	ContactType,
	Complaint,
	IPR
}

export enum ChannelType {
	GUEST,
	HUB,
	VENDOR,
	DISTRIBUTOR,
	RETAILER,
	SALESREP,
	SIGNUP
}

export class ComplainDto {


	@IsNotEmpty({ message: "complain:complainttype:shouldNotEmpty" })
	@IsString({ message: "complain:complainttype:shouldbeString" })
	@IsEnum(ComplainType, { message: "Please choose anyone ComplainType from the list ( ContactType, Complaint, IPR)" })
	complainttype: string;

	@IsOptional()
	@IsNumber({}, { message: "complain:channelid:shouldBeNumber" })
	channelid: number;

	@IsOptional()
	@IsNumber({}, { message: "complain:userid:shouldBeNumber" })
	userid: number;

	@IsNotEmpty({ message: "complain:channeltype:shouldNotEmpty" })
	@IsString({ message: "complain:channeltype:shouldbeString" })
	@IsEnum(ChannelType, { message: "Please choose anyone ChannelType from the list ( GUEST, HUB, VENDOR, DISTRIBUTOR, RETAILER , SALESREP, SIGNUP )" })
	channeltype: string;

	@IsString({ message: "complain:firstname:shouldbeString" })
	@IsNotEmpty({ message: "complain:firstname:shouldNotEmpty" })
	firstname: string;

	@IsString({ message: "complain:lastname:shouldbeString" })
	@IsNotEmpty({ message: "complain:lastname:shouldNotEmpty" })
	lastname: string;

	@IsNotEmpty({ message: "complain:email:shouldNotEmpty" })
	@IsString({ message: "complain:email:shouldbeString" })
	email: string;

	@IsString({ message: "complain:email:shouldbeString" })
	@IsOptional()
	phoneno: string;

	@IsString({ message: "complain:email:shouldbeString" })
	@IsOptional()
	mobileno: string;

	@IsNotEmpty({ message: "complain:countryid:shouldNotEmpty" })
	@IsNumber({}, { message: "complain:countryid:shouldBeNumber" })
	countryid: number;

	@IsNotEmpty({ message: "complain:subject:shouldNotEmpty" })
	@IsString({ message: "complain:subject:shouldbeString" })
	@MaxLength(255, { message: "validate:field:validateMaxLength" })
	subject: string;

	@IsNotEmpty({ message: "complain:detaildesc:shouldNotEmpty" })
	@IsString({ message: "complain:detaildesc:shouldbeString" })
	@MaxLength(8000, { message: "validate:field:validateMaxLength" })
	detaildesc: string;

	@IsString({ message: "complain:iprurl:shouldbeString" })
	@IsOptional()
	iprurl: string;

	@IsNotEmpty({ message: "complain:complaintcategoryid:shouldNotEmpty" })
	@IsNumber({}, { message: "complain:complaintcategoryid:shouldBeNumber" })
	complaintcategoryid: number;

	// @IsNotEmpty({ message: "complain:status:shouldNotEmpty" })
	// @IsString({ message: "complain:status:shouldbeString" })
	// @IsEnum(ComplainActions, { message: "Please choose anyone ComplainActions from the list (  pending, Replied, closed)" })
	// @ApiModelProperty()
	// status: string;

	@IsOptional()
	@ValidateNested()
	@Type(() => CompalainDocumentDto)
	readonly compalainDocument: CompalainDocumentDto[];

}
