import { ArrayNotEmpty, IsOptional, IsString, IsNumber, IsEnum, ValidateIf, ValidateNested, IsNotEmpty } from 'class-validator';

export enum ComplainActions {
	pending,
	replied,
	closed
}

export enum ComplainType {
	ContactType,
	Complaint,
	IPR
}

export enum ChannelType {
	GUEST,
	HUB,
	VENDOR,
	DISTRIBUTOR,
	RETAILER,
	SALESREP,
	SIGNUP
}

export class ComplainsFilterDto {

	@IsNotEmpty({ message: "complain:complainttype:shouldNotEmpty" })
	@IsString({ message: "complain:complainttype:shouldbeString" })
	@IsEnum(ComplainType, { message: "Please choose anyone ComplainType from the list ( ContactType, Complaint, IPR)" })
	complainttype: string;

	@IsOptional()
	@ArrayNotEmpty({ message: "validate:field:shouldBeArray" })
	@IsEnum(ComplainActions, {
		each: true,
		message: "Please choose anyone status from the list ( pending, replied, closed )"
	})
	readonly status: string;

	@IsOptional()
	@ArrayNotEmpty({ message: "validate:field:shouldBeArray" })
	@IsNumber({}, {
		each: true,
		message: "complain:complaintcategoryid:shouldBeNumber"
	})
	complaintcategoryid: number;

	@IsOptional()
	@ArrayNotEmpty({ message: "validate:field:shouldBeArray" })
	@IsEnum(ChannelType, {
		each: true,
		message: "Please choose anyone ChannelType from the list ( GUEST, HUB, VENDOR, DISTRIBUTOR, RETAILER , SALESREP, SIGNUP )"
	})
	channeltype: string;
}
