'use strict';
import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, Scopes, ForeignKey, BelongsTo ,HasMany } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { ComplaintCategory } from '../models/ComplaintCategory.entity';
import { ComplaintAttachment } from '../models/ComplaintAttachment.entity';
import { ComplaintReplyContent } from '../models/ComplaintReplyContent.entity';
import { Country } from 'src/modules/common/models/Country.entity';
import { User } from 'src/modules/user/models/User.entity';
import { Channel } from 'src/modules/channel/models/Channel.entity';

const tableOptions: IDefineOptions = { timestamps: false, tableName: 'tblcomplaint' } as IDefineOptions;
@Table(tableOptions)

@Scopes({ 
    baseDetail: {  }
})

export class Complain extends Model<Complain>{
    
    @PrimaryKey
    @AutoIncrement
    @AllowNull(false)
    @Column(DataType.BIGINT)
    complaintid: number


    @Column(DataType.CHAR(150))
    complaintnumber: string;

    @AllowNull(false)
    @Column(DataType.ENUM('ContactType','Complaint','IPR'))
    complainttype: string;

    @ForeignKey(() => Channel)
    @Column(DataType.BIGINT)
    channelid: number;

    @BelongsTo(() => Channel)
    channel: Channel;

    @ForeignKey(() => User)
    @Column(DataType.BIGINT)
    userid: number;

    @BelongsTo(() => User)
    user: User;

    @Column(DataType.ENUM('VENDOR','DISTRIBUTOR','RETAILER' ,'SALESREP'))
    channeltype: string;

    @Column(DataType.CHAR(255))
    firstname: string;

    @Column
    lastname: string;

    @Column
    email:string;

    @Column
    phoneno: string;

    @Column
    mobileno: string;

    @ForeignKey(() => Country)
    @Column(DataType.BIGINT)
    countryid: number;

    @BelongsTo(() => Country)
    country: Country;

    @Column(DataType.CHAR(255))
    subject: string;

    @Column(DataType.CHAR(8000))
    detaildesc: string;
    
    @Column
    iprurl: string;

    @ForeignKey(() => ComplaintCategory)
    @Column(DataType.BIGINT)
    complaintcategoryid : number;

    @BelongsTo(() => ComplaintCategory)
    complaintcategory: ComplaintCategory;
    
    @Column(DataType.ENUM('pending','replied','closed'))
    status: string;
    
    @Column(DataType.DATE)
    createdat: Date;

    @Column(DataType.DATE)
    updatedat: Date;

    @Column(DataType.DATE)
    deletedat: Date;

    @HasMany(() => ComplaintAttachment, 'complaintid')
    complaintAttachment: ComplaintAttachment;
    
    @HasMany(() => ComplaintReplyContent, 'complaintid')
	complaintReplyContent: ComplaintReplyContent;

}