'use strict';

import { Injectable, Inject } from '@nestjs/common';
import { Complain } from '../models/Complain.entity';
import { ComplaintAttachment } from '../models/ComplaintAttachment.entity';
import { ComplaintCategory } from '../models/ComplaintCategory.entity';
import { ComplainDto } from '../dto/complain.dto';
import { ComplaintReplyContent } from '../models/ComplaintReplyContent.entity'
import { ComplainActionDto } from '../dto/complainAction.dto';
import { Op } from "sequelize";
import { BaseService } from 'src/core/service/base.service';
import { Media } from 'src/modules/media/models/Media.entity';
import { Channel } from 'src/modules/channel/models/Channel.entity';
import { User } from 'src/modules/user/models/User.entity';
import { ProcessException } from 'src/middleware/exception/Process.exception';
import { appconstant } from 'src/config';
import { Country } from 'src/modules/common/models/Country.entity';

@Injectable()
export class ComplainService extends BaseService {
	constructor(
		@Inject('ComplainRepository') private readonly complainRepository: typeof Complain,
		@Inject('ComplaintAttachmentRepository') private readonly complaintAttachmentRepository: typeof ComplaintAttachment,
		@Inject('ComplaintCategoryRepository') private readonly complaintCategoryRepository: typeof ComplaintCategory,
		@Inject('MediaRepository') private readonly mediaRepository: typeof Media,
		@Inject('ChannelRepository') private readonly channelRepository: typeof Channel,
		@Inject('UserRepository') private readonly userRepository: typeof User,
		@Inject('ComplaintReplyContentRepository') private readonly complaintReplyContentRepository: typeof ComplaintReplyContent,
		@Inject('SequelizeInstance') private readonly sequelizeInstance,
		// @Inject('NotificationService') private readonly notificationService,
	) {
		super();
	}

	public async getComplainCategoryByComplainType(complainType: string): Promise<any> {

		let result = await this.complaintCategoryRepository.scope('baseDetail').findAll({
			where: {
				complainttype: complainType,
				isactive: '1',
				deletedat: null
			}
		});

		return result;
	}

	public async setComplain(user: any, complain: ComplainDto): Promise<any> {
		let complaintnumber;

		let result: any = await this.sequelizeInstance.transaction(async transaction => {

			//saving Complain
			let complainObject = <Complain>{};

			complainObject.complainttype = complain.complainttype;
			complainObject.channelid = (user && user.channelId != undefined) ? user.channelId : null;
			complainObject.userid = (user && user.userId != undefined) ? user.userId : null;
			complainObject.channeltype = complain.channeltype;
			complainObject.firstname = complain.firstname;
			complainObject.lastname = complain.lastname;
			complainObject.email = (user && user.email != undefined) ? user.email : complain.email;
			complainObject.phoneno = complain.phoneno != undefined ? complain.phoneno : null;
			complainObject.mobileno = complain.mobileno != undefined ? complain.mobileno : null;
			complainObject.countryid = complain.countryid;
			complainObject.subject = complain.subject;
			complainObject.detaildesc = complain.detaildesc;
			complainObject.iprurl = complain.iprurl != undefined ? complain.iprurl : null;
			complainObject.complaintcategoryid = complain.complaintcategoryid;
			complainObject.status = 'pending';
			complainObject.createdat = this.sequelizeInstance.fn('NOW');


			let complainSaved = await this.complainRepository.create(complainObject, {
				// returning: true,
				transaction
			}).then(async (result) => {

				// update complain number
				complaintnumber = await this.sequelizeInstance.query('select \'COMPLAIN\' || ((EXTRACT(YEAR FROM now()):: integer * 10000) + (EXTRACT(MONTH FROM now()):: integer * 100) + (EXTRACT(day FROM now()):: integer * 1)) :: varchar(8) || (count(*) + 1) :: varchar(10) as number from tblcomplaint where to_char(createdat, \'dd/mm/yyyy\') = to_char(now(), \'dd/mm/yyyy\') and "complaintid" <= ' + result.complaintid, {
					type: this.sequelizeInstance.QueryTypes.SELECT,
					raw: true,
				});
				result.complaintnumber = complaintnumber[0].number;

				await result.save({
					transaction
				});

				return result;

			}).catch((e) => {
				throw new ProcessException('validate:sql:internalError')
			});


			if (complain.compalainDocument.length > 0) {
				for (let ii = 0; ii < complain.compalainDocument.length; ii++) {

					//moving Complain document tem to public bugket
					let documentObject = <Media>{};
					documentObject.documentPath = complain.compalainDocument[ii].documentPath;
					documentObject.documentTypeId = complain.compalainDocument[ii].documenttype == 'image' ? appconstant.DOCUMENTTYPE_IMAGE : appconstant.DOCUMENTTYPE_DOC;

					let document = await this.mediaRepository.create(documentObject, { 
						// returning: true, 
						transaction });

					//saving Complain attachment
					let complaintAttachmentObject = <ComplaintAttachment>{};
					complaintAttachmentObject.attachmentid = document.documentId;
					complaintAttachmentObject.complaintid = complainSaved.complaintid;

					await this.complaintAttachmentRepository.create(complaintAttachmentObject, { transaction });
				}
			}

			return complainSaved;

		});

		let complainsaved = await this.complainRepository.findOne<Complain>({
			where: {
				complaintnumber: complaintnumber[0].number
			},
			include: [{
				model: this.complaintAttachmentRepository,
				as: 'complaintAttachment',
				required: false,
				include: [{
					model: this.mediaRepository,
					required: false,
				}]
			}, {
				model: this.complaintCategoryRepository
			}, {
				model: this.channelRepository,
				required: false
			}, {
				model: this.userRepository,
				required: false
			}, {
				model: Country,
				as: 'country',
				attributes: ['countryId', 'country']
			}],
		});

		// TODO: make notifications for this action
		// let requestObject = <IMailNotification>{};
		// requestObject.complain = complainsaved;
		// this.notificationService.sendNotificationRouter(requestObject, notification.COMPLAINTS);

		return result;
	}

	public async getCount(): Promise<any> {
		return await this.complainRepository.count();
	}

	public async getComplain( dto, order?): Promise<any> {
		let filter = dto.filter;
		let filters = {
			complainttype: dto.complainttype
		};

		if (dto.search) {
			filters[Op.or] = {
				complaintnumber: {[Op.iLike]: '%' + dto.search + '%' },
				firstname: { [Op.iLike]: '%' + dto.search + '%' },
				lastname: { [Op.iLike]: '%' + dto.search + '%' },
				subject: { [Op.iLike]: '%' + dto.search + '%' },
			};
		}

		if (filter.status) {
			filters['status'] = filter.status;
			// {
			// 	[Op.in]: filter.status
			// };
		}

		if (filter.complaintcategoryid) {
			filters['complaintcategoryid'] = filter.complaintcategoryid;
			// {
			// 	[Op.in]: filter.complaintcategoryid
			// };
		}

		if (filter.channeltype) {
			filters['channeltype'] = filter.channeltype;
			// {
			// 	[Op.in]: filter.channeltype
			// };
		}

		let results = await this.complainRepository.findAll<Complain>({
			where: filters,
			include: [{
				model: this.complaintAttachmentRepository,
				as: 'complaintAttachment',
				required: false,
				include: [{
					model: this.mediaRepository
				}]
			}, {
				model: this.complaintCategoryRepository
			}, {
				model: this.channelRepository
			}, {
				model: this.userRepository
			}, {
				model: Country,
				as: 'country',
				attributes: ['countryId', 'country']
			}],
			limit: dto.limit,
			offset: dto.offset,
			order: [['complaintid', order || 'desc']]
		});

		let count = 0;
		let aggregations = {};
		if (results) {
			count = await this.complainRepository.count({
				where: filters
			});

			let query = 'SELECT json_build_object(\'status\',to_json(array_agg(distinct("Complain"."status"))), \'channeltype\',to_json(array_agg(distinct("Complain"."channeltype"))), \'category\',to_json(array_agg(distinct(tcc))) )  AS "filters" FROM "tblcomplaint" AS "Complain" left join (select complaintcategoryid as id, complaintcategory as name from "tblcomplaintcategory") tcc on "Complain".complaintcategoryid = tcc."id"';

			query += ' where "Complain"."complainttype" = :complainttype'

			if (dto.search) {
				query += ' AND ("Complain"."complaintnumber" ILIKE :search ';
				query += ' OR "Complain"."firstname" ILIKE :search ';
				query += ' OR "Complain"."lastname" ILIKE :search ';
				query += ' OR "Complain"."subject" ILIKE :search )'
			}

			if (filter.status) {
				query += ' AND "Complain"."status" IN (:status)'
			}

			if (filter.complaintcategoryid) {
				query += ' AND "Complain"."complaintcategoryid" IN (:complaintcategoryid)'
			}

			if (filter.channeltype) {
				query += ' AND "Complain"."channeltype" IN (:channeltype)'
			}

			aggregations = await this.sequelizeInstance.query(query, {
				type: this.sequelizeInstance.QueryTypes.SELECT,
				replacements: {
					complainttype: dto.complainttype || null,
					search: '%' + dto.search + '%',
					status: filter.status || null,
					complaintcategoryid: filter.complaintcategoryid || null,
					channeltype: filter.channeltype || null
				}
			}).then(complaints => {
				return complaints[0].filters;
			});
		}

		return {
			rows: results,
			count: count,
			aggregations: aggregations
		};

	}

	public async setComplainAction(user: any, complainId: number, complainActionDto: ComplainActionDto): Promise<any> {
		let result = await this.complainRepository.findOne<Complain>({
			where: {
				complaintid: complainId
			},
			include: [{
				model: this.complaintAttachmentRepository,
				as: 'complaintAttachment',
				required: false,
				include: [{
					model: this.mediaRepository,
					required: false,
				}]
			}, {
				model: this.complaintCategoryRepository
			}, {
				model: this.channelRepository,
				required: false
			}, {
				model: this.userRepository,
				required: false
			}, {
				model: Country,
				as: 'country',
				attributes: ['countryId', 'country']
			}],
		});

		if (!result) {
			throw new ProcessException('Record not found');
		}

		if (complainActionDto.action == 'replied') {
			if (result.status == 'pending' || result.status == 'replied') {

				let emailcontent = <ComplaintReplyContent>{};
				emailcontent.createdat = this.sequelizeInstance.fn('NOW');
				emailcontent.content = complainActionDto.compalaintreply.content;
				emailcontent.complaintid = result.complaintid;
				emailcontent.replyedby = result.user?result.user.contactId :1;

				await this.complaintReplyContentRepository.create(
					emailcontent
				).then(async (result1) => {

					// TODO: make notifications for this action
					// let requestObject = <IMailNotification>{};
					// requestObject.complain = result;
					// requestObject.complain_replay = result1;
					// this.notificationService.sendNotificationRouter(requestObject, notification.COMPLAINTS_REPLAY);

					return result1;
				}).catch((e) => {
					throw new ProcessException('validate:sql:internalError')
				});


			}

			if (result.status == 'pending') {
				result.status = 'replied';
				await result.save();
			}
		}

		if (complainActionDto.action == 'closed' && result.status != 'closed') {
			result.status = 'closed';
			await result.save();
		}

		return result;
	}

}
