'use strict'

import { appconstant } from '../config/app.constant';

// Notification Attributes details
export interface INotificationConfigAttributes {
	formatContact: any,
	messageType: number,
	mail: boolean,
	socket: boolean,
	recipients: {
		requested: INotificationUser | null,
		receiver: INotificationUser | null,
	}
}

// Notification sending list
export interface INotificationEnvironmentConfig {
	SUGGESTION_CHANNEL: INotificationConfigAttributes,
	SUGGESTION_PRODUCT: INotificationConfigAttributes,
	ADDTOPROSPECTS: INotificationConfigAttributes,
	REQUEST_TO_MYCHANNEL: INotificationConfigAttributes,
	REQUEST_TO_MYPROSPECT: INotificationConfigAttributes,
	REQUEST_TO_MYPROSPECT_REMAINDER: INotificationConfigAttributes,
	MYPROSPECT_APPROVAL: INotificationConfigAttributes,
	MYPROSPECT_DECLINE: INotificationConfigAttributes,
	MYCHANNEL_APPROVAL: INotificationConfigAttributes,
	MYCHANNEL_DECLINE: INotificationConfigAttributes,
	DISMISS_CHANNEL: INotificationConfigAttributes,
	CHANGE_OF_CONTACT: INotificationConfigAttributes,
	RATING: INotificationConfigAttributes,
	MESSAGING: INotificationConfigAttributes,
	NOT_INTEREST_IN_THIS_PRODUCT: INotificationConfigAttributes,
	NOT_INTEREST_IN_THIS_PRODUCT_AF_EVA: INotificationConfigAttributes,
	ORDER_CHECKOUT: INotificationConfigAttributes,
	ORDER_APPROVAL: INotificationConfigAttributes,
	ORDER_PAID: INotificationConfigAttributes,
	ORDER_CANCEL: INotificationConfigAttributes,
	ORDER_PACKED: INotificationConfigAttributes,
	ORDER_SHIPPED: INotificationConfigAttributes,
	ORDER_DELIVERED: INotificationConfigAttributes,
	ORDER_REFUND_APPROVED: INotificationConfigAttributes,
	ORDER_REFUND_REJECTED: INotificationConfigAttributes,
	ORDER_REFUND_PENDDING: INotificationConfigAttributes,
	ORDER_RE_CREATION: INotificationConfigAttributes,
	ORDER_TRANSFER: INotificationConfigAttributes,
	ORDER_RE_TRANSFER: INotificationConfigAttributes,
	ORDER_MAIL_INVOICE: INotificationConfigAttributes,
	COMPLAINTS: INotificationConfigAttributes,
	CHANNEL_APPROVED: INotificationConfigAttributes,
	CHANNEL_REJECT: INotificationConfigAttributes,
	CONTACT_MAIL: INotificationConfigAttributes,
	ORDER_EVALUATED: INotificationConfigAttributes,
	MAILMESSAGE: INotificationConfigAttributes,
	COMPLAINTS_REPLAY: INotificationConfigAttributes,
	ORDER_REFUND_CANCEL: INotificationConfigAttributes,
	CHANNEL_APPROVAL_PENDING: INotificationConfigAttributes,
	CLAIM_APPROVED: INotificationConfigAttributes,
	CLAIM_REJECTED: INotificationConfigAttributes,
	CHANNEL_RECOMMENDATION: INotificationConfigAttributes,
	PRODUCT_READY_FOR_BUSSINESS: INotificationConfigAttributes,
	CHANNEL_INVITATION: INotificationConfigAttributes,
	CHANNEL_INVITATION_VERIFY: INotificationConfigAttributes,
	CHANNEL_INVITATION_REMAINDER: INotificationConfigAttributes,
	MAYBE_LATER: INotificationConfigAttributes,
	DEACTIVATE_CHANNEL: INotificationConfigAttributes
}

// notifaction receviing User interface
export interface INotificationUser {
	requesteduser?: boolean,
	receiveruser?: boolean,
	company?: boolean,
	accountowner?: boolean,
	reportinguser?: boolean,
	channelsuperadmin?: boolean,
	channelaccountowner?: boolean
}

// Notification configuration
export const notification: INotificationEnvironmentConfig = {
	MAILMESSAGE: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.MAILMESSAGE,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: false,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: false,
				reportinguser: false,
			}
		}
	},
	SUGGESTION_CHANNEL: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.SUGGESTION_CHANNEL,
		messageType: appconstant.MESSAGE_TYPE.SUGGESTION,
		mail: false,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			}
		}
	},
	SUGGESTION_PRODUCT: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.SUGGESTION_PRODUCT,
		messageType: appconstant.MESSAGE_TYPE.SUGGESTION,
		mail: false,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: true,
				reportinguser: true,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: true,
				reportinguser: true,
			}
		}
	},
	ADDTOPROSPECTS: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.ADDTOPROSPECTS,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: false,
		socket: false,
		recipients: {
			requested: {
				requesteduser: false,
				company: false,
				accountowner: false,
				reportinguser: false,
			},
			receiver: {
				receiveruser: false,
				company: false,
				accountowner: false,
				reportinguser: false,
			}
		}
	},
	REQUEST_TO_MYPROSPECT: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.REQUEST_TO_MYPROSPECT,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: {
				requesteduser: false,
				company: false,
				accountowner: false,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: false,
				reportinguser: false,
			}
		}
	},
	MAYBE_LATER: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.MAYBE_LATER,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: {
				requesteduser: false,
				company: false,
				accountowner: false,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: false,
				reportinguser: false,
			}
		}
	},
	DEACTIVATE_CHANNEL: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.DEACTIVATE_CHANNEL,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: {
				requesteduser: false,
				company: false,
				accountowner: false,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: false,
				reportinguser: false,
			}
		}
	},
	REQUEST_TO_MYPROSPECT_REMAINDER: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.REQUEST_TO_MYPROSPECT_REMAINDER,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: {
				requesteduser: false,
				company: false,
				accountowner: false,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: false,
				reportinguser: false,
			}
		}
	},
	MYPROSPECT_APPROVAL: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.MYPROSPECT_APPROVAL,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: {
				requesteduser: false,
				company: false,
				accountowner: false,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: false,
				reportinguser: false,
			}
		}
	},
	MYPROSPECT_DECLINE: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.MYPROSPECT_DECLINE,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: {
				requesteduser: false,
				company: false,
				accountowner: false,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: false,
				reportinguser: false,
			}
		}
	},
	REQUEST_TO_MYCHANNEL: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.REQUEST_TO_MYCHANNEL,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: {
				requesteduser: false,
				company: false,
				accountowner: false,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: false,
				reportinguser: false,
			}
		}
	},
	MYCHANNEL_APPROVAL: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.MYCHANNEL_APPROVAL,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: false,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: false,
				reportinguser: false,
			}
		}
	},
	MYCHANNEL_DECLINE: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.MYCHANNEL_DECLINE,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: false,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: false,
				reportinguser: false,
			}
		}
	},
	DISMISS_CHANNEL: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.DISMISS_CHANNEL,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: false,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: false,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: false,
				reportinguser: false,
			}
		}
	},
	CHANGE_OF_CONTACT: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.CHANGE_OF_CONTACT,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: false,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: false,
				reportinguser: false,
			},
			receiver: {
				receiveruser: false,
				company: false,
				accountowner: false,
				reportinguser: false,
			}
		}
	},
	RATING: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.RATING,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: true,
				reportinguser: true,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: true,
				reportinguser: true,
			}
		}
	},
	MESSAGING: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.MESSAGING,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: true,
				reportinguser: true,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: true,
				reportinguser: true,
			}
		}
	},
	NOT_INTEREST_IN_THIS_PRODUCT: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.NOT_INTEREST_IN_THIS_PRODUCT,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: false,
		socket: true,
		recipients: {
			requested: {
				requesteduser: false
			},
			receiver: {
				receiveruser: true
			}
		}
	},
	NOT_INTEREST_IN_THIS_PRODUCT_AF_EVA: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.NOT_INTEREST_IN_THIS_PRODUCT_AF_EVA,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: {
				requesteduser: false
			},
			receiver: {
				receiveruser: true
			}
		}
	},
	ORDER_CHECKOUT: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.ORDER_CHECKOUT,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			}
		}
	},
	ORDER_APPROVAL: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.ORDER_APPROVAL,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: {
				requesteduser: false,
				company: false,
				accountowner: false,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			}
		}
	},
	ORDER_PAID: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.ORDER_PAID,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: false,
		socket: true,
		recipients: {
			requested: {
				requesteduser: false,
				company: false,
				accountowner: false,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			}
		}
	},
	ORDER_CANCEL: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.ORDER_CANCEL,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: false,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			}
		}
	},
	ORDER_PACKED: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.ORDER_PACKED,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: false,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			}
		}
	},
	ORDER_SHIPPED: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.ORDER_SHIPPED,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: false,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			}
		}
	},
	ORDER_DELIVERED: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.ORDER_DELIVERED,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: false,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			}
		}
	},
	ORDER_REFUND_APPROVED: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.ORDER_REFUND_APPROVED,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: false,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			}
		}
	},
	ORDER_REFUND_REJECTED: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.ORDER_REFUND_REJECTED,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: false,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			}
		}
	},
	ORDER_REFUND_CANCEL: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.ORDER_REFUND_CANCEL,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: false,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			}
		}
	},
	ORDER_REFUND_PENDDING: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.ORDER_REFUND_PENDDING,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: false,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			}
		}
	},
	ORDER_RE_CREATION: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.ORDER_RE_CREATION,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: false,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			}
		}
	},
	ORDER_TRANSFER: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.ORDER_TRANSFER,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			}
		}
	},
	ORDER_RE_TRANSFER: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.ORDER_RE_TRANSFER,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			}
		}
	},
	ORDER_MAIL_INVOICE: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.ORDER_MAIL_INVOICE,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: false,
				reportinguser: false,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: true,
				reportinguser: false,
			}
		}
	},
	COMPLAINTS: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.COMPLAINTS,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: null,
			receiver: null
		}
	},
	COMPLAINTS_REPLAY: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.COMPLAINTS_REPLAY,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: null,
			receiver: null
		}
	},
	CHANNEL_APPROVED: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.CHANNEL_APPROVED,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		socket: true,
		mail: true,
		recipients: {
			requested: null,
			receiver: {
				company: false,
				accountowner: true,
			}
		}
	},
	CHANNEL_REJECT: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.CHANNEL_REJECT,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		socket: true,
		mail: true,
		recipients: {
			requested: null,
			receiver: {
				company: false,
				accountowner: true,
			}
		}
	},
	CONTACT_MAIL: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.MAILMESSAGE,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		socket: true,
		mail: true,
		recipients: {
			requested: {
				requesteduser: true,
				company: false,
				accountowner: true,
				reportinguser: true,
			},
			receiver: {
				receiveruser: true,
				company: false,
				accountowner: true,
				reportinguser: true,
			}
		}
	},
	ORDER_EVALUATED: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.ORDER_EVALUATED,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		socket: true,
		mail: true,
		recipients: {
			requested: {
				requesteduser: false
			},
			receiver: {
				receiveruser: true
			}
		}
	},
	CHANNEL_APPROVAL_PENDING: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.CHANNEL_APPROVAL_PENDING,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		socket: true,
		mail: true,
		recipients: {
			requested: null,
			receiver: {
				company: false,
				accountowner: true,
			}
		}
	},
	CLAIM_APPROVED: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.CLAIM_APPROVED,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		socket: false,
		mail: true,
		recipients: {
			requested: null,
			receiver: {
				company: false,
				accountowner: true,
			}
		}
	},
	CLAIM_REJECTED: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.CLAIM_REJECTED,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		socket: false,
		mail: true,
		recipients: {
			requested: null,
			receiver: {
				company: false,
				accountowner: true,
			}
		}
	},
	CHANNEL_RECOMMENDATION: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.CHANNEL_RECOMMENDED,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		socket: true,
		mail: true,
		recipients: {
			requested: null,
			receiver: {
				company: false,
				accountowner: true,
			}
		}
	},
	CHANNEL_INVITATION: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.CHANNEL_INVITATION,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		socket: false,
		mail: true,
		recipients: {
			requested: null,
			receiver: {
				receiveruser: true
			}
		}
	},
	CHANNEL_INVITATION_REMAINDER: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.CHANNEL_INVITATION_REMAINDER,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		socket: false,
		mail: true,
		recipients: {
			requested: null,
			receiver: {
				receiveruser: true
			}
		}
	},
	CHANNEL_INVITATION_VERIFY: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.CHANNEL_INVITATION_VERIFY,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		socket: true,
		mail: true,
		recipients: {
			requested: null,
			receiver: {
				receiveruser: true,
			}
		}
	},
	PRODUCT_READY_FOR_BUSSINESS: {
		formatContact: appconstant.MESSAGE_FORMATE_CONTACTIDS.PRODUCT_READY_FOR_BUSSINESS,
		messageType: appconstant.MESSAGE_TYPE.ACTION,
		mail: true,
		socket: true,
		recipients: {
			requested: {
				requesteduser: false
			},
			receiver: {
				receiveruser: true
			}
		}
	}
};

// Socket events
export const socketEvent = {
	EVENT: 'event'
}
