import { Sequelize } from 'sequelize-typescript';
import { appconstant } from 'src/config/app.constant';
import { Channel } from 'src/modules/channel/models/Channel.entity';
import { ChannelJCT } from 'src/modules/channel/models/ChannelJCT.entity';
import { ChannelSubscription } from 'src/modules/channel/models/ChannelSubscription.entity';
import {Op, QueryTypes } from "sequelize";
export abstract class BaseService {

	async validateRejectedChannels(channelId, otherChannelId) {
		return await ChannelJCT.findAll({
			attributes: ['channelJCTId'],
			where: {
				[Op.and]: [
					{
						connectionStatusTypeId: {
							[Op.in]: [appconstant.DISMISS, appconstant.DISMISS_OTHER_REASON]
						},
					},
					{
						[Op.or]: [
							{
								channelId: channelId,
								partnerChannelId: otherChannelId
							},
							{
								channelId: otherChannelId,
								partnerChannelId: channelId
							}
						]
					}
				]
			}
		}).then((results) => {
			return !!(results.length);
		});
	}

	public async setSlugName(repository, name: any, alterNumber?) {

		var slugName = name.toLowerCase().replace(/ /g, "-");

		if (alterNumber) {
			slugName = slugName + '-' + alterNumber;
		} else {
			alterNumber = 0;
		}

		return await repository.findOne({
			where: {
				slug: slugName
			}
		}).then(async (result) => {
			if (result && result.slug) {
				let replace = +alterNumber + 1;
				return await this.setSlugName(repository, name, replace);
			}
			return slugName;
		});
	}

	public async getIdBySlugName(repository, name) {

		return await repository.findOne({
			where: {
				slug: name
			}
		});
	}

	async getCurrentChannelStatus(channelId, features) {
		let whereOption: any = Sequelize.literal(' "channelSubscription"."isActive" = \'1\' AND to_date( to_char("channelSubscription"."endAt",\'dd/mm/yyyy\'),\'dd/mm/yyyy\')  >= to_date(to_char(now(),\'dd/mm/yyyy\'),\'dd/mm/yyyy\') ');

		let channel = await Channel.findOne({
			attributes: ['companyName', 'signUpStatusId', 'subscriptionId', 'planId', 'isActive'],
			where: {
				channelId: channelId
			},
			include: [{
				model: ChannelSubscription,
				required: false,
				where: whereOption
			}]
		});

		// subscription features
		let myfeatures = {};
		for (let key of Object.keys(features)) {
			if (key.startsWith(channel.subscriptionId + '-')) {
				myfeatures[key] = features[key];
			}
		}

		return {
			"companyName": channel.companyName,
			"channelStatusId": channel.signUpStatusId,
			"planId": channel.planId,
			"subscriptionId": channel.subscriptionId,
			"features": myfeatures,
			"channelIsActive": channel.isActive,
			"subscriptionExpired": !(channel.channelSubscription),
			"channelSubscriptionId": (channel.channelSubscription) ? channel.channelSubscription.channelSubscriptionId : null
		}
	}

	async unsubscribe_token(contactId){
        const Cryptr = require('cryptr');
        
		const cryptr = new Cryptr('ch-event-auth');
		return cryptr.encrypt(contactId);
    }
}
