'use strict';

import { Inject, Controller, Post, Get, HttpStatus, Request, Response, Param, Body, UsePipes, InternalServerErrorException, UseGuards } from '@nestjs/common';
// import { ApiBearerAuth, ApiTags, ApiOperation, ApiParam, ApiHeader } from '@nestjs/swagger';
import { json } from 'sequelize';
import { SignupDto } from 'src/modules/account/dto/signup.dto';
// import { HubspotService } from '../helper/hubspot.service';
import { AccountService } from '../../modules/account/service/account.service'
// import { BaseController } from '../../modules/core';
import { HubspotImportHelper } from '../helper/hubspotImport.helper';
// import { ProcessException } from '../../middleware/exception/Process.exception';

// @ApiTags('Hubspot')
// @ApiBearerAuth()
@Controller('Hubspot')

export class HubspotController 
// extends BaseController 
{
	constructor(
		private accountService: AccountService
	) {
		// super();
	}

	@Get('getCompany/:companyId')
	public async getCompanyDetails(@Param('companyId') companyId, @Response() res) {
		// console.log(companyId); //2433637650
		let hubspotClient = new HubspotImportHelper();
		let responseData = await hubspotClient.getContactWithThisEmail(companyId);
		let hubspotData = JSON.parse(responseData.data);
		 if(responseData.response != 200)
			return res.status(HttpStatus.BAD_REQUEST).send(hubspotData);
			
		let email = hubspotData.properties.email.value;
		let cheackmail = await  this.accountService.checkEmail(email);
		let data;
		if (!cheackmail) {
			console.log('already exist');
			// throw new ProcessException('account:singup:emailAleradytaken');
			return res.status(HttpStatus.BAD_REQUEST).send({code: 1413, message: 'Given mail id already exist' });
		} else {
			let company = [];
			if(hubspotData.properties.associatedcompanyid)
			 company= await hubspotClient.getCompanyDetails(hubspotData.properties.associatedcompanyid.value);

			data={
				contactVid:hubspotData.vid,
				is_contact:hubspotData.iscontact,
				properties:hubspotData.properties,
				company:company
			}
		}
		return res.status(HttpStatus.OK).send(data);
	}
	
	@Post('createCompany')
	// @ApiHeader({ name: 'Hubspot', description: 'Create Hubspot Company', required: true })
	public async createCompany(@Body() signUpData:SignupDto, @Response() res){
		let hubspotClient = new HubspotImportHelper();
			let companyId = await hubspotClient.createCompany(signUpData);
			let contactId = await hubspotClient.impotContactToHubSpot(signUpData);
			let data = await hubspotClient.mergeContactToCompany(contactId, companyId);
			return res.status(HttpStatus.CREATED).send(data);
	}

	@Get('getCompanyWithId/:companyId')
	public async getCompanyWithId(@Param('companyId') companyId, @Response() res){
		let hubspotClient = new HubspotImportHelper();
		let companyData = await hubspotClient.getCompanyDetails(companyId);
			return res.status(HttpStatus.OK).send(companyData);
	}

	@Get('getCompanyList')
	public async getCompanyList(@Response() res){
		let hubspotClient = new HubspotImportHelper();
		let url= 'https://api.hubapi.com/crm/v3/objects/companies';
		// let url = 'https://api.hubapi.com/crm/v3/objects/companies?limit=100&after=5652414383';
		// let url = 'https://api.hubapi.com/properties/v1/companies/properties'
		// let url = "https://api.hubapi.com/companies/v2/companies/paged";
		let data=[];
		let check=false;
	 	for(var i=0;i<100;i++){
			if(check)break;
			let value= await hubspotClient.getCompanyList(url);
			if(value.paging && value.paging.next){
			url=value.paging.next.link
			}else{
				check = true
			}
			data.push(...value.results);
			}
		let result=[]
		if(data && data.length){
			result = data;
			result = result.map(x=>{
				return {id:x.id, name:x.properties.name, type:2}
			})
		}
		return res.status(HttpStatus.OK).send(result);
	}
}
