import * as AWS from 'aws-sdk';
import { ProcessException } from 'src/middleware/exception/Process.exception';
var fs = require('fs');
let path = require("path");

export class S3Lib {
	private options = {
		accessKeyId: process.env.AWS_ACCESS_KEY,
		secretAccessKey: process.env.AWS_SECRET_ACCESS_KEY,
		region: process.env.AWS_REGION
	};

	public fs = require('fs');
	private s3 = new AWS.S3(this.options);

	constructor() {

	}

	async CopyObjectPublic(sourcePath, destinationPath): Promise<boolean> {
		try {
			let params = {
				Bucket: process.env.PUBLIC_AWS_BUCKET,
				CopySource: process.env.PRIVATE_AWS_BUCKET + sourcePath,
				Key: destinationPath
			};
			let resultPromise = await this.s3.copyObject(params).promise();
			if (resultPromise) {
				return true;
			}
			return false;
		} catch (e) {
			console.log(e);
			return false;
		}
	}
	async CopyObjectTempToPlublic(sourcePath, destinationPath): Promise<boolean> {
		try {
			let params = {
				Bucket: process.env.PUBLIC_AWS_BUCKET,
				CopySource: process.env.TEM_AWS_BUCKET + sourcePath,
				Key: destinationPath,
				ACL: "public-read"
			};
			console.log('destinationPath',destinationPath);
			let resultPromise = await this.s3.copyObject(params).promise();
			if (resultPromise) {
				return true;
			}
			return false;
		} catch (e) {
			console.log(e);
			return false;
		}
	}
	async FindObject(key): Promise<boolean> {
		try {
			let params = {
				Bucket: process.env.PRIVATE_AWS_BUCKET,
				Key: key
			};
			let resultPromise = await this.s3.headObject(params).promise();
			if (resultPromise) {
				return true;
			}
			return false;
		} catch (e) {
			console.log(e);
			return false;
		}
	}

	async downloadFromTempBucket(sourcePath, fileName): Promise<boolean> {
		try {
            console.log(sourcePath);
			var params = {
				Bucket:process.env.PRIVATE_AWS_BUCKET,
				Key: sourcePath+'/'+fileName
			   };
			   
			let fileObject = await new Promise((resolve, reject) => {
				const destPath = path.resolve(__dirname, '..', '..')+ '/.tmp/upload_s3/'+fileName;
				this.s3.getObject(params)
				  .createReadStream()
				  .pipe(fs.createWriteStream(destPath))
				  .on('close', () => {
					resolve(destPath)
				})
			})
			if (fileObject) {
			    console.log('downloadFromTempBucket success!!!');
				return true;
			}
			console.log('downloadFromTempBucket false!!!');
			return false;
		} catch (e) {
			console.log(e);
			return false;
		}
	}

	async moveFromLocal(file_path, s3_key, id) {
		let self = this;
		try {
			return await self.s3.upload({
				Key: s3_key,
				Body: file_path,
				Bucket: id===1? process.env.PRIVATE_AWS_BUCKET:process.env.TEM_AWS_BUCKET,
				// ContentEncoding: 'base64',
				// ContentType: 'application/pdf'
			}, (e,d)=>{
				if(e){
					throw new ProcessException('Soming went wrong');
					}
				return true
		}).promise();
		} catch (error) {
			throw new ProcessException('Soming went wrong');
		}
	}

	
	async moveFromPlublic(file_path, s3_key) {
		let self = this;
		try {
			return await self.s3.upload({
				Key: s3_key,
				Body: file_path,
				Bucket: process.env.PUBLIC_AWS_BUCKET,
				ACL: "public-read",

			}, (e,d)=>{
				if(e){
					throw new ProcessException('Soming went wrong');
					}
				return true
		}).promise();
		} catch (error) {
			throw new ProcessException('Soming went wrong');
		}
	}
	
	async deleteObject(pathname, bucket){
		const params = {
			Bucket:bucket,
			Key: pathname,
		   }
		   await this.s3.deleteObject(params).promise()
	}


	async uploadZipToTempBucket(sourcePath, fileName): Promise<any> {
	    var rimraf = require('rimraf');
		let tempPath = path.resolve(__dirname, '..', '..') + '/.tmp/upload_s3/';
		try {
			let newfileName = Date.now();
			let zipFolderPath = tempPath + fileName;
			let newFolderPath = tempPath + newfileName;
			
			var AdmZip = require('adm-zip');
			var zip = new AdmZip(zipFolderPath);
			
			if(await zip.extractAllTo(newFolderPath,true)){
				 rimraf(zipFolderPath, this.responseRimraf());
			}
            
            console.log('uploadZipToTempBucket Success!!!');
			// rimraf(zipFolderPath, this.responseRimraf());
			// rimraf(newFolderPath, this.responseRimraf());
			console.log('uploadZipToTempBucket deleted!!!');
			let newPath = '/'+newfileName;
			return { newPath: newPath };

		} catch (e) {
			console.log(e);
		}
	}

	private responseRimraf(): any {
		return function (e) {
			if (e) {
				console.log("error while file delete");
				return false;
			}
			return true;
		};
	}


	async callforUnzip(newFolderPath){
		let tempPath = path.resolve(__dirname, '..', '..') + '/.tmp/upload_s3';
		let newfileName = Date.now();
		let imageUploaded = await fs.readdir(tempPath+newFolderPath, (err, files) => {
			if (err)
             	console.log(err);
			else { 
				const delay = ms => new Promise(_ => setTimeout(_, ms));
						for (const imageFileName of files) {
							const filePath = newFolderPath+'/'+ imageFileName;
							try{
								fs.readdir(tempPath+filePath, (error, fileContent) => {
									if (error) { throw error; }
									try{
										for (const imgs of fileContent) {
											var filePaths = tempPath + filePath +'/'+ imgs;
											fs.readFile(filePaths, (error, fileContent) => {
												if (error) { throw error; }
												this.s3.putObject({
												Bucket: process.env.TEM_AWS_BUCKET,
												Key: 'product-import-zip-s3'+newFolderPath +'/'+ imgs,
												Body: fileContent
												}, (res) => {
													console.log(`Successfully uploaded '${filePaths}'!`);
												});
											});
									}  
									} catch (e){
										console.log('upload error ===== ', e)
									}
								});
							} catch(e){
								console.log(e);
							}
						}
						}
						})
						console.log('uploadZipToTempBucket Success!!!', newFolderPath);
						var rimraf = require('rimraf');
						//	rimraf(zipFolderPath, this.responseRimraf());
						rimraf(tempPath+newFolderPath, this.responseRimraf());
				}

}
