
import { Setting } from './../modules/common/models/Setting.entity';
import { PaymentException } from '../middleware/exception/Payment.exception';
import * as Stripe from 'stripe';
import { payment } from '../config/payment.constant';
const { Op } = require('sequelize');
//const Cryptr = require('cryptr');

export class StripeV2 {
	private stripe = null;
    private redisInstance;
    
	private minimumAmount = 0.50;

	// constructor(redisInstance) {
	// 	this.redisInstance = redisInstance;
    // }

    public async loadKeys() {
		//return await this.redisInstance.fetch("DB:CONFIG:STRIPE").then(async (data: any) => {
			//const cryptr = new Cryptr(process.env.DB_CONFIG_SECRET);
			//console.log(cryptr);
			
			//if (data === null) {
				let result = await Setting.findAll({
					attributes: ['key', 'value'],
					where: {
						key: {
							[Op.in]: ['STRIPE_SECRET_KEY', 'STRIPE_CONNECT_APPLICATION_KEY', 'STRIPE_WEBHOOK_SECRET', 'STRIPE_WEBHOOK_CONNECTED_SECRET']
						}
					}
				}).then((result) => {
					var format = {};
					for (let data of result) {
						format[data.key] = data.value
					}

					return format;
				});

				//await this.redisInstance.set("DB:CONFIG:STRIPE", cryptr.encrypt(JSON.stringify(result)));

				return result;
			//}

			//return JSON.parse(cryptr.decrypt(data));
		//});
	}
    
    public async connect() {
		if(this.stripe === null) {
			//let keys = await this.loadKeys();

			//this.stripe = new Stripe((keys['STRIPE_SECRET_KEY']) ? keys['STRIPE_SECRET_KEY'] : process.env.STRIPE_SECRET_KEY);\
			//this.stripe = new Stripe(process.env.STRIPE_SECRET_KEY);
			this.stripe = require('stripe')(process.env.STRIPE_SECRET_KEY);
		}		
    }
    

    // stripe connect
    public async getStripeConnectUrl() {
		//let keys = await this.loadKeys();

		//let STRIPE_APPLICATION_KEY = (keys['STRIPE_CONNECT_APPLICATION_KEY']) ? keys['STRIPE_CONNECT_APPLICATION_KEY'] : process.env.STRIPE_CONNECT_APPLICATION_KEY

		let STRIPE_APPLICATION_KEY = process.env.STRIPE_CONNECT_APPLICATION_KEY

		return process.env.STRIPE_CONNECT_AUTHORIZE_URI + "?response_type=code&client_id=" + STRIPE_APPLICATION_KEY + "&scope=read_write";
    }
    
    public async getAuthStripeToken(code): Promise<any> {

		let keys = await this.loadKeys();

		// Post the authorization code to Stripe to complete the authorization flow.
		let request = require('request');
		return await new Promise(function(resolve, reject) {
			// let result:any = {err:'',accountId:''}
			request.post(process.env.STRIPE_CONNECT_TOKEN_URI, {
				form: {
					grant_type: 'authorization_code',
					client_id:  process.env.STRIPE_CONNECT_APPLICATION_KEY,
					client_secret:  process.env.STRIPE_SECRET_KEY,
					code: code
				},
				json: true
			}, (err, response, body) => {
				resolve(body);
			});
		});
    }
    // stripe connect
    

    // stripe intend
    public async createPaymentIntend(amount, currency, entity, entity_id) {
		// connect stripe
		await this.connect();

		if (amount < this.minimumAmount) {
			throw new PaymentException("stripe:minimum:amount");
		}

        const intent = await this.stripe.paymentIntents.create({
			amount: amount,
			currency: currency,
			setup_future_usage: 'off_session',
			metadata: {
				entity_id: entity_id,
				entity: entity
			}
		});
        
        return intent;
	}

	public async createPaymentIntendWithCustomer(amount, currency, entity, entity_id, customerId, paymentMethodId) {
		// connect stripe
		await this.connect();

		if (amount < this.minimumAmount) {
			throw new PaymentException("stripe:minimum:amount");
		}

        const intent = await this.stripe.paymentIntents.create({
			amount: amount,
			currency: currency,
			payment_method_types: ['card'],
			customer: customerId,
			payment_method: paymentMethodId,
			off_session: true,
			confirm: true,
			metadata: {
				entity_id: entity_id,
				entity: entity
			}
		});
        
        return intent;
	}
	
    // stripe intend using connected account Id
    public async createPaymentIntendWIthConnectedAccount(amount, currency, entity, entity_id, accountId) {
		// connect stripe
		await this.connect();

		if (amount < this.minimumAmount) {
			throw new PaymentException("stripe:minimum:amount");
		}

        const intent = await this.stripe.paymentIntents.create({
			amount: amount,
			currency: currency,
			payment_method_types: ['card'],
			// application_fee: payment.application_fee,
			metadata: {
				entity_id: entity_id,
				entity: entity
			}
		}, {
			stripe_account: accountId,
		});
        
        return intent;
    }

	public async processWebHook(signature, body, connected = false) {
		let keys = await this.loadKeys();

		// connect stripe
		await this.connect();
		let secret = keys['STRIPE_WEBHOOK_SECRET'];
		if(connected) {
			secret = keys['STRIPE_WEBHOOK_CONNECTED_SECRET'];
		}

		return await this.stripe.webhooks.constructEvent(body, signature, Buffer.from(secret));
	}

	public async addOrUpdateCustomer(channel, intent) {
		// connect stripe
		await this.connect();

		if(channel.stripeCustomerId) {
			await this.stripe.paymentMethods.attach(
				intent.payment_method,
				{
				  customer: channel.stripeCustomerId,
				}
			);
		} else {
			// This creates a new Customer and attaches the PaymentMethod in one API call.
			const customer = await this.stripe.customers.create({
				payment_method: intent.payment_method,
				email: channel.companyMail,
				description: 'Customer for ' + channel.companyName,
				name: channel.companyName,
				metadata: {
					channel_id: channel.channelId,
					channel_name: channel.companyName
				}
			});

			channel.stripeCustomerId = customer.id;
		}

		return channel.stripeCustomerId;
	}


	public async createchargesForSource(amount,currency,sourceId) {
		// connect stripe
		
	//	this.stripe = new Stripe(process.env.STRIPE_SECRET_KEY);
	this.stripe = require('stripe')(process.env.STRIPE_SECRET_KEY);
		//await this.connect();
		
        const charge = await this.stripe.charges.create({
			amount: amount,
			currency: currency,
			source: sourceId,
			});
        
        return charge;
	}
}