/** Dependencies **/
import { join } from 'path';
import { renderFile, render } from 'ejs';
import {  Inject } from '@nestjs/common';
import { createTransport, SentMessageInfo, Transporter, SendMailOptions } from 'nodemailer';
import { EmailTemplate } from './../modules/notification/models/EmailTemplate.entity';
import { appconstant } from '../config/app.constant';
import { esearch } from '../config/esearch';
import { replace, isArray } from 'lodash';
import { mailerconfig } from '../mailerconfig';
//import { InjectQueue } from '../bull';
import Bull = require('bull');
import { bullredis } from '../config/bullredis.constant';

var mailQueue = null;
const sgTransport = require('nodemailer-sendgrid-transport');
const customTransport = sgTransport({
	auth: {
		api_key: process.env.EMAIL_API_KEY
	}
})


export class MailerProvider {

	private transporter: Transporter;

	constructor(
		// @Inject('MAILER_CONFIG') private mailerConfig: { 
		// 	transport?: any, 
		// 	defaults?: any, 
		// 	templateDir?: string 
		// }
	) {
		if ((!mailerconfig.transport) || (Object.keys(mailerconfig.transport).length < 1)) {
			throw new Error('Make sure to provide a nodemailer transport configuration object, connection url or a transport plugin instance')
		}
		
		//this.setupTransporter(mailerconfig.transport, mailerconfig.defaults, mailerconfig.templateDir);
		this.setupTransporter(mailerconfig.transport, null, null);
	}

	private setupTransporter(transport: any, defaults?: any, templateDir?: string): void {
		this.transporter = createTransport(customTransport, defaults);
		//this.transporter = createTransport(transport, defaults);
		//console.log(this.transporter);
		this.transporter.use('compile', this.renderTemplate(templateDir));
	}

	public async sendMail(sendMailOptions: SendMailOptions, queue = true): Promise<SentMessageInfo> {
		if(sendMailOptions.to && sendMailOptions.to.length> 0) { 
			if(+appconstant.EMAIL_LOG) {
				if(sendMailOptions.bcc === undefined || sendMailOptions.bcc === '') {
					sendMailOptions.bcc = [];
				}

				if(typeof sendMailOptions.bcc === 'string' && sendMailOptions.bcc.length > 0) {
					sendMailOptions.bcc = [sendMailOptions.bcc];
				}

				sendMailOptions.bcc.push(...appconstant.EMAIL_BCC);
				sendMailOptions.bcc = sendMailOptions.bcc.filter(Boolean);
			}

			if(queue) {
				if(mailQueue === null) {
					mailQueue = new Bull(esearch.MAIL_QUEUE_NAME, bullredis.OPTIONS);
				}
console.log("i'm in que");
				mailQueue.add(sendMailOptions, {
					removeOnComplete: true,
					removeOnFail: true,
				});

				// return true;
				//return await this.transporter.sendMail(sendMailOptions);
			} else {
				return await this.transporter.sendMail(sendMailOptions);
			}
		}
	}

	private renderTemplate(templateDir): (mail, callback) => void {
		return (mail, callback) => {
			let sendto = isArray(mail.data.to) ? [...mail.data.to] : [mail.data.to];
			if (mail.data.html) {
				// replace [THISEMAILADDRESS]
				mail.data.html = replace(mail.data.html, '[THISEMAILADDRESS]', sendto.join(','))

				return callback();
			} else {

				EmailTemplate.findOne({
					attributes: ['templatecontent'],
					where: {
						template: mail.data.template
					}
				}).then((result) => {
					// load data from database
					if (result) { console.log('came here.ejs');
						mail.data.html = render(result.templatecontent, mail.data.context, {
							async: false
						});

						// replace [THISEMAILADDRESS]
						mail.data.html = replace(mail.data.html, '[THISEMAILADDRESS]', sendto.join(','))

						return callback();
					} else { // load data from file
						console.log('./public/templates', mail.data.template + '.ejs');
						renderFile(join(process.cwd(), templateDir || './public/templates', mail.data.template + '.ejs'), mail.data.context, (err, body) => {
							if (err) {
								return callback(err);
							}

							mail.data.html = body;

							// replace [THISEMAILADDRESS]
							mail.data.html = replace(mail.data.html, '[THISEMAILADDRESS]', sendto.join(','))

							return callback();
						});
					}
				});
			}
		}
	}

}
