import { Body, Controller, Get, Inject, Param, Post, Req, Res, UseGuards } from '@nestjs/common';
import { ClientProxy } from '@nestjs/microservices';
import { AuthGuard } from '@nestjs/passport';
import { renderFile } from 'ejs';
import { join } from 'path';
import { ContextService } from 'src/context/context.service';
import { MailerService } from './mailerservice.service';
import { Op, QueryTypes } from "sequelize";
let fs = require('fs');

//@UseGuards(AuthGuard('jwt'))
@Controller('mailerservice')
export class MailerserviceController {

  //For microservice Code
    // constructor(
    //     @Inject('MAIL_SERVICE') private mailService: ClientProxy,
    //     private context: ContextService,
    //   ) {}]

    //Non Micro
    constructor(
      private readonly mailService: MailerService,
      @Inject('SequelizeInstance') private readonly sequelizeInstance,
    ) { }

  // @Get('mail')
  //   async mail(@Req() req) {
  //       let msg = {
  //           to: ['nagakin97@gmail.com','nagaraj.va@gmail.com'], 
  //           from: process.env.EMAIL_FROM,
  //           subject: '🍩 Donuts, at the big donut 🍩',
  //           templateId:'d-bbe981a1ae2e4bcd88bd0c3e7f076c57',
  //           text: 'Fresh donuts are out of the oven. Get them while they’re hot!',
  //           html: '<p>Fresh donuts are out of the oven. Get them while they’re <em>hot!</em></p>',
  //           data:{
  //              name:"nagaraj",
  //              inbound_contact_name:"kin",
  //              inbound_company_name:"start"
  //           }
  //     };

  //   return this.mailService.send({
  //     cmd: 'mail'
  //   },JSON.stringify(msg))
  // }  

  //Micro
  // @Post('Add_Product_Mail')
  // async Add_Product_Mail(@Req() req, @Body() data) {                      
  //   return this.mailService.send({
  //     cmd: 'addproductmail'
  //   },JSON.stringify(data))
  // }  

  //Non Micro
  @Post('Add_Product_Mail')
  async Add_Product_Mail(@Req() req, @Body() data) {

    let query = 'select "firstName" as name,email from "tblContact" where "contactId"=(select "contactId" from "tblUser" where "userId"=:userId) ';

    let result = await this.sequelizeInstance.query(query, {
      type: QueryTypes.SELECT,
      raw: true,
      replacements: {
        userId: data.userId
      }
    });

    let msg = {
      to: result[0].email,
      cc: process.env.MAIL_CC,
      from: process.env.EMAIL_FROM,
      templateId: 'd-aa5e57c256a641fdbfbd7486722165e1',
      dynamicTemplateData: {
        name: result[0].name,
      }
    };
    return this.mailService.sendEmail(msg);
  }

  // Micro
  // @Post('After_Signup_Mail')
  // async After_Signup_Mail(@Req() req, @Body() data) {                      
  //   return this.mailService.send({
  //     cmd: 'aftersignupmail'
  //   },JSON.stringify(data))
  // }  

  //Non Micro
  @Post('After_Signup_Mail')
  async After_Signup_Mail(@Req() req, @Body() data) {                      
    let msg = {
      to: data.to,
      cc: process.env.MAIL_CC,
      from: process.env.EMAIL_FROM,
      templateId: 'd-a20e8735754e47edb328e72b9c650772',
      dynamicTemplateData: {
        name: data.name,
      }
    };
    return this.mailService.sendEmail(msg);
  }  

  // Micro
  // @Post('Pending_Approval_Mail')
  // async Pending_Approval_Mail(@Req() req, @Body() data) {
  //   return this.mailService.send({
  //     cmd: 'pendingapprovalmail'
  //   }, JSON.stringify(data))
  // }

  //Non Micro
  @Post('Pending_Approval_Mail')
  async Pending_Approval_Mail(@Req() req, @Body() data) {
    let bquery = 'select "companyName",case when "channelTypeId"=3 then \'Distributor\' when "channelTypeId"=4 then \'Retailer\' when "channelTypeId"=5 then \'Reseller\' end as "companyType", '
      + '(select "country" from "tblCountry" where "countryId"=tc."countryId"), '
      + '(select concat(\'' + process.env.REACT_APP_CDN_URL + '\',"documentPath") from "tblDocument" where "documentId" in (select "channelLogoId" from "tblChannelDetail" where "channelId"=tc."channelId")) as "companyLogo" '
      + 'from "tblChannel" as tc where "channelId" in(select "channelId" from "tblUser" where "email"=:email) ';

    let channel = await this.sequelizeInstance.query(bquery, {
      type: QueryTypes.SELECT,
      raw: true,
      replacements: {
        email: data.email
      }
    });

    console.log("channels:", channel);

    let msg = {
      to: data.to,
      cc: process.env.MAIL_CC,
      from: process.env.EMAIL_FROM,
      templateId: 'd-a20e8735754e47edb328e72b9c650772',
      dynamicTemplateData: {
        companyName: channel[0].companyName,
        companyType: channel[0].companyType,
        country: channel[0].country,
        companyLogo: channel[0].companyLogo,
      }
    };
    return this.mailService.sendEmail(msg);
  }


  // Micro
  // @Post('Buyer_Following_Vendor')
  // async Buyer_Following_Vendor(@Req() req, @Body() data) {                      
  //   return this.mailService.send({
  //     cmd: 'followingvendor'
  //   },JSON.stringify(data))
  // }

  //Non Micro
  @Post('Buyer_Following_Vendor')
  async Buyer_Following_Vendor(@Req() req, @Body() data) {

    let query = 'select "email","firstName" as name from "tblContact" where "channelId"=:channelId and "isActive"=\'1\'  and "deletedAt" is null ';

    let result = await this.sequelizeInstance.query(query, {
      type: QueryTypes.SELECT,
      raw: true,
      replacements: {
        channelId: data.channelId
      }
    });

    result.map(x => {

      let msg = {
        to: x.email,
        cc: process.env.MAIL_CC,
        from: process.env.EMAIL_FROM,
        templateId: 'd-b3517a78038c43839b0e267f0b62dfa0',
        dynamicTemplateData: {
          name: x.name,
        }
      };
      this.mailService.sendEmail(msg);
    })

    return true;//this.mailService.sendEmail(msg);       
  }



  // Micro
  // @Post('Reset_Password_Mail')
  // async Reset_Password_Mail(@Req() req, @Body() data) {
  //   return this.mailService.send({
  //     cmd: 'resetpassword'
  //   }, JSON.stringify(data))
  // }

  //Non Micro
  @Post('Reset_Password_Mail')
  async Reset_Password_Mail(@Req() req, @Body() data) {
    let msg = {
      to: data.email,
      cc: process.env.MAIL_CC,
      from: process.env.EMAIL_FROM,
      templateId: 'd-bf3f35948d4e4154aa22239acd8b0b66',
      dynamicTemplateData: {
        name: data.name,
        profileurl: data.url
      }
    };
    return this.mailService.sendEmail(msg);
  }

  //Micro
  // @Post('Buyer_Favorites_Product')
  // async Buyer_Favorites_Product(@Req() req, @Body() data) {                      
  //   return this.mailService.send({
  //     cmd: 'favoriteproduct'
  //   },JSON.stringify(data))
  // }  

  //Non Micro
  @Post('Buyer_Favorites_Product')
  async Buyer_Favorites_Product(@Req() req, @Body() data) {                      

    let query = 'select "email","firstName" as name from "tblContact" where "channelId"=(select "channelId" from "tblProduct" where "productId"=:productId) and "isActive"=\'1\'  and "deletedAt" is null ';

    let result = await this.sequelizeInstance.query(query, {
      type: QueryTypes.SELECT,
      raw: true,
      replacements: {
        productId: data.productId
      }
    });

    result.map(x => {

      let msg = {
        to: x.email,
        cc: process.env.MAIL_CC,
        from: process.env.EMAIL_FROM,
        templateId: 'd-5b41e3fb45424532b5957d5e3eb943ce',
        dynamicTemplateData: {
          name: x.name,
        }
      };
      this.mailService.sendEmail(msg);
    })

    return true;//this.mailService.sendEmail(msg);       
  }


  //Micro
  // @Post('Channel_Approval_Mail')
  // async Channel_Approval_Mail(@Req() req, @Body() data) {
  //   return this.mailService.send({
  //     cmd: 'afterapprovalmail'
  //   },JSON.stringify(data))
  // }  

  //Non Micro
  @Post('Channel_Approval_Mail')
  async Channel_Approval_Mail(@Req() req, @Body() data) {
    let query = 'select "firstName" as name from "tblContact" where "contactId"=:contactId ';

    let result = await this.sequelizeInstance.query(query, {
      type: QueryTypes.SELECT,
      raw: true,
      replacements: {
        contactId: data.contactId
      }
    });

    let msg = {
      to: data.to,
      cc: process.env.MAIL_CC,
      from: process.env.EMAIL_FROM,
      templateId: 'd-fadcddce4e6a41e9a73c027c7ca6ed95',
      dynamicTemplateData: {
        name: result[0].name,
        profileurl: data.profileurl
      }
    };
    return this.mailService.sendEmail(msg);
  }

  //Micro
  // @Post('Channel_Dismiss_Mail')
  // async Channel_Dismiss_Mail(@Req() req, @Body() data) {
  //   return this.mailService.send({
  //     cmd: 'afterdismissmail'
  //   }, JSON.stringify(data))
  // }

  //Non Micro
  @Post('Channel_Dismiss_Mail')
  async Channel_Dismiss_Mail(@Req() req, @Body() data) {
    let query = 'select "firstName" as name from "tblContact" where "contactId"=:contactId ';

    let result = await this.sequelizeInstance.query(query, {
      type: QueryTypes.SELECT,
      raw: true,
      replacements: {
        contactId: data.contactId
      }
    });

    let msg = {
      to: data.to,
      cc: process.env.MAIL_CC,
      from: process.env.EMAIL_FROM,
      templateId: 'd-e5136f3dba5044e4b8e87e2b160d61a4',
      dynamicTemplateData: {
        name: result[0].name,
        //profileurl: data.profileurl
      }
    };
    return this.mailService.sendEmail(msg);
  }


  //Micro
  //@Post('Add_User_Mail')
  // async Add_User_Mail(@Req() req,@Body() data) {
  //   return this.mailService.send({
  //     cmd: 'addusermail'
  //   },JSON.stringify(data))
  // }    

  //Non Micro
  @Post('Add_User_Mail')
  async Add_User_Mail(@Req() req,@Body() data) {                          
    let query;
    if (data.accountOwner == 0)
      query = 'select "firstName" as name from "tblContact" where "isDefaultContact"=\'1\' ';
    else
      query = 'select "firstName" as name from "tblContact" where "contactId"=:contactId ';

    let result = await this.sequelizeInstance.query(query, {
      type: QueryTypes.SELECT,
      raw: true,
      replacements: {
        contactId: data.accountOwner
      }
    });

    let msg = {
      to: data.to,
      cc: process.env.MAIL_CC,
      from: process.env.EMAIL_FROM,
      templateId: 'd-959d91d142794da6b90e11b37933d4ec',
      dynamicTemplateData: {
        name: data.name,
        profileurl: data.profileurl,
        userId: data.userId,
        password: data.password,
        accountOwner: result[0].name
      }
    };
    return this.mailService.sendEmail(msg);

  }    


  //Non Micro
  @Post('Add_BO_User')
  async Add_BO_User(@Req() req, @Body() data) {
    // let query;
    // query = 'select concat("fName",\' \',"lName") as name from "tblAdminUsers" where "adminUserId"=:adminUserId ';

    // let result = await this.sequelizeInstance.query(query, {
    //   type: QueryTypes.SELECT,
    //   raw: true,
    //   replacements: {
    //     adminUserId: data.adminUserId
    //   }
    // });

    let msg = {
      to: data.result.to,
      cc: process.env.MAIL_CC,
      from: process.env.EMAIL_FROM,
      templateId: 'd-f83af20e08f94e2aad9d4f2899f7b7b4',
      dynamicTemplateData: {
        name: data.result.name,
        profileurl: data.result.profileurl,
        userId: data.result.userId,
        password: data.result.password,
        accountOwner: data.result.accountOwner
      }
    };
    return this.mailService.sendEmail(msg);
  }

  //Micro
  // @Post('Buyer_Suggestion_Mail')
  // async Buyer_Suggestion_Mail(@Req() req,@Body() data) {                          

  //   return this.mailService.send({
  //     cmd: 'buyersuggestionmail'
  //   },JSON.stringify(data))
  // }    



  //Micro
  // @Post('Product_Suggestion_Mail')
  // async Product_Suggestion_Mail(@Req() req,@Body() data) {                      

  //   //console.log("user:",data);
  //   return this.mailService.send({
  //     cmd: 'productsuggestionmail'
  //   },JSON.stringify(data))
  // }

  //Non Micro
  @Post('Product_Suggestion_Mail')
  async Product_Suggestion_Mail(@Req() req,@Body() data) {                      

    let bquery = 'select pc."productCategorie",concat(tcon."firstName",\' \',tcon."lastName") as name,tcon."email" from "tblInterestedProductCategory" as ipc inner join "tblProductCategories" as pc '
      + 'on pc."productCategorieId"=ipc."productCategorieId" inner join "tblContact" as tcon on tcon."contactId"=ipc."contactId" '
      + 'where ipc."contactId"=:contactId and ipc."deletedAt" is null';

    let productCat = await this.sequelizeInstance.query(bquery, {
      type: QueryTypes.SELECT,
      raw: true,
      replacements: {
        contactId: data.contactId,
      }
    });

    let name, email, matchedProducts, more = 1;
    productCat.map(x => {
      name = x.name;
      email = x.email;
      if (more <= 4 && more > 1)
        matchedProducts = matchedProducts + " - " + x.productCategorie;
      if (more == 1)
        matchedProducts = x.productCategorie;

      more = more + 1;
    })

    let query = 'select tp."product" as "productName",tp."productSKU",(select "companyName" from "tblChannel" where "channelId"="partnerChannelId"), '
      + '(select "country" from "tblCountry" where "countryId" in(select "countryId" from "tblChannel" where "channelId"="partnerChannelId")), '
      + '(select regexp_replace("detailDesc", E\'<[^>]+>\', \'\', \'gi\') as "tweetPitch" from "tblChannelDetail" where "channelId"="partnerChannelId"), '
      + '(select "documentPath" from "tblDocument" where "documentId"=tp."defaultimageid") as "productImage", '
      + '(select "retailerprice" from "tblproductpricedetail" where "productId"=tp."productId") as price, '
      + '(select sum("existproductrating")/count(*) from "tblproductexistrating" where "productId"=tp."productId") as rating '
      + 'from "tblProductJCT" as pjct inner join "tblProduct" as tp on tp."productId"=pjct."productId" '
      + 'where "requested_by"=:contactId limit 3';

    let product = await this.sequelizeInstance.query(query, {
      type: QueryTypes.SELECT,
      raw: true,
      replacements: {
        //channelId:data.channelId,
        contactId: data.contactId
      }
    });

    if (product && product.length > 0) {
      product.map(x => {
        Object.assign(x, {
          productUrl: process.env.DOMAIN + "/productDetailView/" + x.productSKU,
          keyRetailer: "Amazon, Flipkart, Acuchen, Carrefour"
        });
        if (x.rating) {
          let rat = "";
          for (let i = 0; i < x.rating; i++)
            rat += "*";

          x.rating = rat;
        }

        x.productImage = process.env.REACT_APP_CDN_URL + x.productImage;
      })

      let result = {
        name: name,
        matchedProducts: matchedProducts,
        more: more - 1 > 4 ? more - 1 : 0,
        product: product
      }

      //console.log("result:",result,email);

      let msg = {
        to: email,
        cc: process.env.MAIL_CC,
        from: process.env.EMAIL_FROM,
        templateId: 'd-452c8584e1584a06ba59e8cce363cb51',
        dynamicTemplateData: result
      };
      this.mailService.sendEmail(msg);
    }

    return true;
  }

  //Micro
  // @Post('Connection_Request_Mail')
  // async Connection_Request_Mail(@Req() req, @Body() data) {
  //   console.log("connection:", data);
  //   return this.mailService.send({
  //     cmd: 'connectionrequestmail'
  //   }, JSON.stringify(data))
  // }

  //Non Micro
  @Post('Connection_Request_Mail')
  async Connection_Request_Mail(@Req() req, @Body() data) {
    let query; let tempId;
    if (data.typeId == 3) {
      tempId = 'd-68e4ffef524849958a0ac8a55ee4381d';
      query = 'select concat(\'' + process.env.DOMAIN + '/admin/channel/channelDetail/\',cd."channelId") as "companyUrl","companyName",regexp_replace("detailDesc", E\'<[^>]+>\', \'\', \'gi\') as detailDesc,(select concat(\'' + process.env.REACT_APP_CDN_URL + '\',"documentPath") from "tblDocument" where "documentId"=cd."channelLogoId")as "companyLogo","firstName"as name,"jobTitle",(select "country" from "tblCountry" where "countryId"=tc."countryId"), '
        + '(select array_to_json(array_agg(row_to_json(br))) from (select tb."brandName" as item_name,concat(\'' + process.env.REACT_APP_CDN_URL + '\',td."documentPath") as image_Url from "tblBrand" tb inner join "tblDocument" td on td."documentId"=tb."documentId" where "id" in(select "brandId" from "tblChannelBrand" where "channelId"=:req_channelId and "deletedAt" is null))br) as brand,'
        + '(select array_to_json(array_agg(row_to_json(x))) from (Select concat(pc."productCategorie",\' \',\'- \') as item_name from "tblChannelProductCategorie" cpc inner join "tblProductCategories" pc on pc."productCategorieId" = cpc."productCategorieId" where cpc."channelId"=:req_channelId  and cpc."deletedAt" is null)x) as company_cat, '
        + '(select array_to_json(array_agg(row_to_json(z))) from (Select concat(pt."profileType",\' \',\'- \') as item_name from "tblProfileType" pt where "profileTypeId" in (select "profileTypeId" from "tblChannelCustomerProfile" cp where cp."channelId"=:req_channelId and cp."deletedAt" is null))z) as retail_customer_profile '
        + 'from "tblContact" as tc inner join "tblChannel" as ch on ch."channelId"=tc."channelId" inner join "tblChannelDetail" as cd on cd."channelId"=tc."channelId" where "contactId"=:req_contactId ';
    }
    else if (data.typeId == 4 || data.typeId == 5) {
      tempId = 'd-d565f18e12514d4c9491cdad31c43116';
      query = 'select concat(\'' + process.env.DOMAIN + '/admin/channel/channelDetail/\',cd."channelId") as "companyUrl","companyName",regexp_replace("detailDesc", E\'<[^>]+>\', \'\', \'gi\') as detailDesc,(select concat(\'' + process.env.REACT_APP_CDN_URL + '\',"documentPath") from "tblDocument" where "documentId"=cd."channelLogoId")as "companyLogo","firstName"as name,"jobTitle",(select "country" from "tblCountry" where "countryId"=tc."countryId"), '
        //+ '(select array_to_json(array_agg(row_to_json(br))) from (select tb."brandName" as item_name,concat(\'' + process.env.REACT_APP_CDN_URL + '\',td."documentPath") as image_Url from "tblBrand" tb inner join "tblDocument" td on td."documentId"=tb."documentId" where "id" in(select "brandId" from "tblChannelBrand" where "channelId"=:req_channelId and "deletedAt" is null))br) as brand,'
        + '(select array_to_json(array_agg(row_to_json(x))) from (Select concat(pc."productCategorie",\' \',\'- \') as item_name from "tblChannelProductCategorie" cpc inner join "tblProductCategories" pc on pc."productCategorieId" = cpc."productCategorieId" where cpc."channelId"=:req_channelId  and cpc."deletedAt" is null)x) as company_cat, '
        //+ '(select array_to_json(array_agg(row_to_json(z))) from (Select pt."profileType" as item_name from "tblProfileType" pt where "profileTypeId" in (select "profileTypeId" from "tblChannelCustomerProfile" cp where cp."channelId"=:req_channelId and cp."deletedAt" is null))z) as retail_customer_profile '
        + 'from "tblContact" as tc inner join "tblChannel" as ch on ch."channelId"=tc."channelId" inner join "tblChannelDetail" as cd on cd."channelId"=tc."channelId" where "contactId"=:req_contactId ';
    }

    let result = await this.sequelizeInstance.query(query, {
      type: QueryTypes.SELECT,
      raw: true,
      replacements: {
        req_contactId: data.req_contact,
        req_channelId: data.req_channel
      }
    });

    let templateData = result[0];

    if (templateData?.company_cat?.length > 0) {
      let lastCategory = templateData.company_cat[templateData.company_cat.length - 1];
      templateData.company_cat[templateData.company_cat.length - 1].item_name = lastCategory.item_name.trim().slice(0, -1);
    }
    if (data.typeId == 3 && templateData?.retail_customer_profile?.length > 0) {
      let lastProfile = templateData.retail_customer_profile[templateData.retail_customer_profile.length - 1];
      templateData.retail_customer_profile[templateData.retail_customer_profile.length - 1].item_name = lastProfile.item_name.trim().slice(0, -1);
    }

    Object.assign(templateData, {
      message: data.note, receiver_name: data.receiver_name,
      receiver_photo: data.receiver_photo, connect_url: process.env.DOMAIN + '/admin/mynetwork'
    });

    //console.log("conn:", result[0]);

    let msg = {
      to: data.receiver_mail,
      cc: process.env.MAIL_CC,
      from: process.env.EMAIL_FROM,
      templateId: tempId,
      dynamicTemplateData: templateData
    };
    return this.mailService.sendEmail(msg);
  }

  //Micro
  // @Post('Connection_Dismiss_Mail')
  // async Connection_Dismiss_Mail(@Req() req, @Body() data) {
  //   console.log("connection:", data);
  //   return this.mailService.send({
  //     cmd: 'connectiondismissmail'
  //   }, JSON.stringify(data))
  // }

  //Non Micro
  @Post('Connection_Dismiss_Mail')
  async Connection_Dismiss_Mail(@Req() req, @Body() data) {
    let query = 'select "companyName",(select concat(\'' + process.env.REACT_APP_CDN_URL + '\',"documentPath") from "tblDocument" where "documentId"=cd."channelLogoId")as "companyLogo","firstName"as "contactName","jobTitle",(select "country" from "tblCountry" where "countryId"=tc."countryId") '
      + 'from "tblContact" as tc inner join "tblChannel" as ch on ch."channelId"=tc."channelId" inner join "tblChannelDetail" as cd on cd."channelId"=tc."channelId" where "contactId"=:req_contactId ';


    let result = await this.sequelizeInstance.query(query, {
      type: QueryTypes.SELECT,
      raw: true,
      replacements: {
        req_contactId: data.req_contact
        //req_channelId: data.req_channel
      }
    });

    let templateData = result[0];
    Object.assign(templateData, {
      receiver_name: data.receiver_name,
      companyType: 'VENDOR'
    });

    //console.log("conn:", result[0]);

    let msg = {
      to: data.receiver_mail,
      cc: process.env.MAIL_CC,
      from: process.env.EMAIL_FROM,
      templateId: 'd-1313b0cf495d449284e98e982ebc3c73',
      dynamicTemplateData: templateData
    };
    return this.mailService.sendEmail(msg);
  }


  //Micro
  // @Post('Connection_Approve_Mail')
  // async Connection_Approve_Mail(@Req() req, @Body() data) {
  //   console.log("connection:", data);
  //   return this.mailService.send({
  //     cmd: 'connectionapprovemail'
  //   }, JSON.stringify(data))
  // }

  //Non Micro
  @Post('Connection_Approve_Mail')
  async Connection_Approve_Mail(@Req() req, @Body() data) {
    let query = 'select concat(\'' + process.env.DOMAIN + '/admin/channel/channelDetail/\',tc."channelId") as "companyUrl","companyName",(select concat(\'' + process.env.REACT_APP_CDN_URL + '\',"documentPath") from "tblDocument" where "documentId"=tc."documentId")as "profile_url","firstName"as "contactName","jobTitle",(select "country" from "tblCountry" where "countryId"=tc."countryId") '
      + 'from "tblContact" as tc inner join "tblChannel" as ch on ch."channelId"=tc."channelId"  where "contactId"=:req_contactId ';

    let result = await this.sequelizeInstance.query(query, {
      type: QueryTypes.SELECT,
      raw: true,
      replacements: {
        req_contactId: data.req_contact
        //req_channelId: data.req_channel
      }
    });

    let templateData = result[0];
    Object.assign(templateData, {
      receiver_name: data.receiver_name,
      companyType: 'VENDOR'
      // receiver_photo: data.receiver_photo, connect_url: process.env.DOMAIN + '/admin/mynetwork'
    });
    //console.log("conn:", result[0]);
    let msg = {
      to: data.receiver_mail,
      cc: process.env.MAIL_CC,
      from: process.env.EMAIL_FROM,
      templateId: 'd-3a0481920ac04363a7be12a935ed20c2',
      dynamicTemplateData: templateData
    };
    return this.mailService.sendEmail(msg);
  }

  //Micro
  // @Post('Guest_Product_Enquiry')
  // async Guest_Product_Enquiry(@Req() Req, @Body() data){

  //   return this.mailService.send({
  //     cmd: 'guestproductenquiry'
  //   },JSON.stringify(data));
  // }

  //Non Micro
  @Post('Guest_Product_Enquiry')
  async Guest_Product_Enquiry(@Req() Req, @Body() data) {

    let query = 'select "product" from "tblProduct" where "productId"=:productId';

    let result = await this.sequelizeInstance.query(query, {
      type: QueryTypes.SELECT,
      raw: true,
      replacements: {
        productId: data.productId
      }
    }).then(res => {
      return res[0];
    });

    let msg = {
      to: data.email,
      cc: process.env.MAIL_CC,
      from: process.env.EMAIL_FROM,
      templateId: 'd-61f83a7b6627419cbbe8361f6188e343',
      dynamicTemplateData: {
        name: data.email.split('@')[0],
        productname: result.product
      }
    }

    this.mailService.sendEmail(msg);

    return true;

  }


  //Micro
  // @Post('subscriptionEmail')
  // async subscriptionEmail(@Req() Req, @Body() data){
  //   return this.mailService.send({
  //     cmd: 'subscriptionemail'
  //   },JSON.stringify(data));
  // }

  //Non Micro
  @Post('subscriptionEmail')
  async subscriptionEmail(@Req() Req, @Body() data) {

    let query = 'select u."email", con."firstName",p."transactionNumber", p."cardLastFourDigit", p."cardType", p."invoiceno" , p."amount" , sp."subscriptionPlanType", spr."periodDesc" from "tblChannel" c ';
    query += ' left join "tblUser" u on u."channelId" = c."channelId" ';
    query += ' left join "tblChannelSubscription" cs on cs."channelId" = c."channelId" AND cs."isActive" =:isActive';
    query += ' left join "tblSubscription" s on cs."subscriptionId" = s."subscriptionId" ';
    query += ' left join "tblSubscriptionPeriod" spr on spr."subscriptionPeriodId" = s."subscriptionPeriodId" ';
    query += ' left join "tblSubscriptionPlan" sp on sp."subscriptionPlanId" = c."planId" ';
    query += ' left join "tblPaymentTransaction" p on p."paymentTransactionId" = cs."paymentTransactionId"  ';
    query += ' left join "tblContact" con on con."channelId" = c."channelId" where c."channelId"=:channelId ';

    let result = await this.sequelizeInstance.query(query, {
      type: QueryTypes.SELECT,
      raw: true,
      replacements: {
        channelId: data.id,
        isActive: '1'
        //req_channelId: data.req_channel
      }
      });

    // console.log(result);
    if (result.length > 0) {
      let receiptURL = process.env.CDN_URL +'invoice/'+ data.id+'/' + result.channelSubscriptionId + '-' + result.subscriptionId + '.pdf';
      let msg = {
        to: result[0].email,//, 'meena@yopmail.com', //data.to,
        cc: process.env.MAIL_CC,
        from: process.env.EMAIL_FROM,
        templateId: 'd-30bb5ef59bd04f7d9622d153105514fe',
        dynamicTemplateData: {
          receiver_name: result[0].firstName,
          plan: result[0].subscriptionPlanType,
          receipt: result[0].invoiceno,
          channel_type: 'Vendor',
          transaction_id: result[0].transactionNumber,
          card_number: result[0].cardLastFourDigit,
          amount: result[0].amount,
          period: result[0].periodDesc,
          receipt_url: receiptURL,
          login_url: process.env.HOST_URL
        }
      };
      // console.log(msg);
      this.mailService.sendEmail(msg);
      return true;
    }
  }

  //Micro
  // @Post('OtherpaymentEmail')
  // async OtherpaymentEmail(@Req() Req, @Body() data){
  //   return this.mailService.send({
  //     cmd: 'otherpaymentEmail'
  //   },JSON.stringify(data));
  // }

  //Non Micro
  @Post('OtherpaymentEmail')
  async OtherpaymentEmail(@Req() Req, @Body() data) {
    let query = 'select u."email", con."firstName",p."transactionNumber", p."cardLastFourDigit", p."cardType", p."invoiceno" , p."amount" , sp."subscriptionPlanType", spr."periodDesc" from "tblChannel" c ';
    query += ' left join "tblUser" u on u."channelId" = c."channelId" ';
    query += ' left join "tblChannelSubscription" cs on cs."channelId" = c."channelId" AND cs."isActive" =:isActive';
    query += ' left join "tblSubscription" s on cs."subscriptionId" = s."subscriptionId" ';
    query += ' left join "tblSubscriptionPeriod" spr on spr."subscriptionPeriodId" = s."subscriptionPeriodId" ';
    query += ' left join "tblSubscriptionPlan" sp on sp."subscriptionPlanId" = c."planId" ';
    query += ' left join "tblPaymentTransaction" p on p."paymentTransactionId" = cs."paymentTransactionId"  ';
    query += ' left join "tblContact" con on con."channelId" = c."channelId" where c."channelId"=:channelId ';

    let result = await this.sequelizeInstance.query(query, {
      type: QueryTypes.SELECT,
      raw: true,
      replacements: {
        channelId: data.id,
        isActive: '1'
        //req_channelId: data.req_channel
      }
    });

    //console.log(result);
    if (result.length > 0) {
      let msg = {
        to: result[0].email,
        cc: process.env.MAIL_CC,
        from: process.env.EMAIL_FROM,
        templateId: 'd-9172ba27165e4308af67962675327289',
        dynamicTemplateData: {
          receivrecharge_amounter_name: result[0].firstName,
          transaction_id: result[0].subscriptionPlanType,
          user_balance: result[0].invoiceno,
          receipt_url: 'Vendor',
          name: result[0].transactionNumber,
          login_url: process.env.HOST_URL
        }
      };
      // console.log(msg);
      return this.mailService.sendEmail(msg);
    }
  }




}
