import { NestFactory } from '@nestjs/core';
import { AppModule } from './app.module';
import { ValidationPipe } from './middleware/pipe/validation.pipe';
import { urlencoded, json } from 'express';
import { NestApplicationOptions } from '@nestjs/common/interfaces/nest-application-options.interface';
/* context */
const contextService = require('request-context');


async function bootstrap() {
  	/* Nest application Option */
	const nestOptions: NestApplicationOptions = {
		cors: {
			origin: '*',
			methods: 'GET,HEAD,PUT,PATCH,POST,DELETE,OPTIONS',
			maxAge: 86400,
			preflightContinue: false,
			optionsSuccessStatus: 204,
		}, bodyParser: true
	};
  const app = await NestFactory.create(AppModule, nestOptions);
  app.useGlobalPipes(new ValidationPipe());
  app.enableCors({
    allowedHeaders: ['*']
	});
  app.setGlobalPrefix('api/v2');
  app.use(json({limit: '50mb'}));
  app.use(contextService.middleware('request'));
  await app.listen(process.env.PORT || 3000);
}
bootstrap().then(() => console.log('Application is listening on port 3000'));
