'use strict';

import * as moduleErrorMessage from './errorMessages';
import { HttpStatus } from '@nestjs/common';

export class MessageCodeError extends Error {
	public code: number
	public messageCode: string;
	public httpStatus: number;
	public errorMessage: string;
	public messages: any

	constructor(messageCode: any, httpStatus: HttpStatus, lang?: string) {
		super();

		const errorMessageConfig = this.getMessageFromMessageCode(messageCode);

		Error.captureStackTrace(this, this.constructor);

		if (!errorMessageConfig.code) {
			errorMessageConfig.code = 0;
			errorMessageConfig.message = messageCode;
		}

		// format error
		this.code = errorMessageConfig.code;
		this.message = errorMessageConfig.message;
	}

	/**
	 * @description: Find the error config by the given message code and Lang code.
	 * @param {any} messageCode
	 * @return {IErrorMessages}
	*/
	private getMessageFromMessageCode(messageCode: any): any {
		let errorMessageConfig: any = {};
		if (typeof messageCode == "string") {
			var message = messageCode.split(":");
			if (moduleErrorMessage[message[0]]) {
				Object.keys(moduleErrorMessage[message[0]]).forEach(key => {
					if (key === messageCode) {
						errorMessageConfig = moduleErrorMessage[message[0]][key];
						return true;
					}
				});
			}
		}

		return errorMessageConfig;
	}
}
