'use strict';
import * as moduleErrorMessage from './errorMessages';
import { ValidationError } from 'class-validator';
import { IErrorMessages } from './errorMessages/interface/IErrorMessages';

export class ValidationException extends Error {
	public messages: any[];

	constructor(errors: any, lang?: string) {
		super();
		const messages = this.formatErrors(errors);
		if (!messages) throw new Error('Unable to find message code error.');

		Error.captureStackTrace(this, this.constructor);

		// formated error message
		this.messages = messages;
	}

	/**
	* @description: format the form validation errors to client processable format .
	* @param {Object} constraints
	* @return Array<IErrorMessages>
	*/
	protected formatErrors(formErrors: ValidationError[]): Array<IErrorMessages> {
		let errors = new Array();
		let message = [];
		let childError = [];

		for (let ii = 0; ii < formErrors.length; ii++) {

			if (formErrors[ii].children.length > 0) {
				childError = this.formatErrors(formErrors[ii].children);
			}

			if (formErrors[ii].constraints) {
				message = this.GroupConstraints(formErrors[ii].constraints);
			}

			if (childError.length) {
				errors.push({
					property: formErrors[ii].property,
					errors: childError
				});
			} else if (message.length) {
				errors.push({
					property: formErrors[ii].property,
					errors: message
				});
			}
		}
		return errors;
	}

	/**
	 * @description: Group the error By Constraints.
	 * @param {Object} constraints
	 * @return Array<IErrorMessages>
	 */
	protected GroupConstraints(params: any): Array<IErrorMessages> {
		let message = new Array();
		Object.keys(params).forEach((constraint) => {

			var errorMessageConfig = this.getMessageFromMessageCode(params[constraint]);

			if (!errorMessageConfig.code) {
				errorMessageConfig.code = 0;
				errorMessageConfig.message = params[constraint];
			}

			message.push(errorMessageConfig);
		});

		return message;
	}

	/**
	 * @description: Find the error config by the given message code.
	 * @param {string} messageCode
	 * @return {IErrorMessages}
	 */
	protected getMessageFromMessageCode(messageCode: string): any {
		let errorMessageConfig: any = {};
		if (typeof messageCode == "string") {
			var message = messageCode.split(":");
			if (moduleErrorMessage[message[0]]) {
				Object.keys(moduleErrorMessage[message[0]]).forEach(key => {
					if (key === messageCode) {
						errorMessageConfig = moduleErrorMessage[message[0]][key];
						return true;
					}
				});
			}
			return errorMessageConfig;
		}
	}
}
