import { Module } from '@nestjs/common';
import { databaseProviders } from 'src/core/database/database.providers';
import { addressProvider, oldAddressProvider } from '../channel/provider/address.provider';
import { channelDetailProvider, channelProductCatProvider, channelProductGroupProvider, oldChannelProvider } from '../channel/provider/channel.provider';
import { EmailerModule } from '../emailer/emailer.module';
import { EmailerService } from '../emailer/services/emailer.service';
import { SignUpHistoryProviders } from '../user/provider/SignUpHistory.providers';
import { UserProviders } from '../user/provider/user.provider';
import { UserRoleProviders } from '../user/provider/userRole.providers';
import { UserModule } from '../user/user.module';
import { AccountController } from './controller/account.controller';
import { AccountService } from './service/account.service';
import { eventProvider } from '../channel/provider/event.provider';

@Module({
  controllers: [AccountController],
  providers: [
    AccountService,
    EmailerService,
    ...databaseProviders,
    ...UserProviders,
    ...UserRoleProviders,
    ...SignUpHistoryProviders,
    ...addressProvider,
    ...oldAddressProvider,
    ...channelDetailProvider,
    ...channelProductGroupProvider,
    ...channelProductCatProvider,
    ...oldChannelProvider,
    ...eventProvider
  ],
	exports: [
		AccountService
  ],
  imports:[UserModule, EmailerModule]
})
export class AccountModule {}
