import { Controller, Get, Res, HttpStatus, Post, Body, Inject, Param } from '@nestjs/common';
import { databaseConfig } from 'src/core/database/database.config';
import { AccountService } from '../service/account.service';
import { SignupDto } from '../dto/signup.dto';
import { HubspotImportHelper } from 'src/hubspot/helper/hubspotImport.helper';
import { ValidationPipe } from 'src/middleware/pipe/validation.pipe';
import { ApiParam } from '@nestjs/swagger';
import { CheckEmailPipe } from '../pipe/account.pipe';
import { ForgotPasswordDto } from '../dto/forgotPassword.dto';
import { ResetPasswordDto } from '../dto/resetPassword.dto';
import { ProcessException } from 'src/middleware/exception/Process.exception';
import { ResetPasswordBoDto } from '../dto/resetPasswordBo.dto';
var request = require('request');
@Controller('account')
export class AccountController {

    constructor(private readonly accountService : AccountService){}
    
    @Post()
    async signUp(@Body(new ValidationPipe()) signUpData:SignupDto, @Res() res){
        // console.log(signUpData);
        let hubspotClient = new HubspotImportHelper();
        if(!signUpData.hubspotCompanyId){
        signUpData.hubspotCompanyId = await hubspotClient.createCompany(signUpData);
		console.log('signUpData.hubspotCompanyId', signUpData.hubspotCompanyId)
		if(!signUpData.hubspotCompanyId)
		throw new ProcessException('creating hubspot error');
        }else{
        await hubspotClient.updateCompany(signUpData, signUpData.hubspotCompanyId)
        }
        if(!signUpData.hubspot_contact_id){
        signUpData.hubspot_contact_id = await hubspotClient.impotContactToHubSpot(signUpData);
		console.log('signUpData.hubspot_contact_id', signUpData.hubspot_contact_id);
		if(!signUpData.hubspot_contact_id)
		throw new ProcessException('creating hubspot contact error');
        }else{
         await hubspotClient.UpdateContactToHubSpot(signUpData, signUpData.hubspot_contact_id)
        }
        await hubspotClient.mergeContactToCompany(signUpData.hubspot_contact_id,signUpData.hubspotCompanyId);
		let cheackmail = await this.accountService.checkEmail(signUpData.email);
		if(!cheackmail) {
            res.status(HttpStatus.BAD_REQUEST).send({code: 1413, message: 'Given mail id already exist' });
			// throw new ProcessException('account:singup:emailAleradytaken');
		}else{
		await this.accountService.create(signUpData);
		res.status(HttpStatus.CREATED).send('ok');
		}
    }

    @Get('checkmail/:email')
	public async checkmail(@Param('email', new CheckEmailPipe()) email, @Res() res) {
        let result = await this.accountService.checkEmail(email.trim());
        console.log('email', email)

		res.status(HttpStatus.OK).send({
			isavialble: result
		});
    }
    
    @Post('checkCompanyName')
	public async checkCompanyName(@Body() data, @Res() res) {
        let result = await this.accountService.checkCompanyName(data.name.trim());

		res.status(HttpStatus.OK).send({
			isavialble: result
		});
	}

    @Post('getCompanyData')
	public async getCompanyData(@Body() data, @Res() res) {
        let result = await this.accountService.getCompanyData(data.name.trim());

		res.status(HttpStatus.OK).send(result);
	}
     
	@Post('forgotPassword')
	public async forgotPassword(@Res() res, @Body(new ValidationPipe()) forgotPasswordDto: ForgotPasswordDto) {
        console.log('forgotPasswordDto', forgotPasswordDto);
		let result=await this.accountService.forgotPassword(forgotPasswordDto.email);
		console.log('result', result);
		return res.status(HttpStatus.OK).send(result);
	}

	@Get('resetPassword/:token')
	public async resetPassword(@Param('token') token, @Res() res) {
		let result = await this.accountService.resetPassword(token);

		return res.status(HttpStatus.OK).send({
			valid: result
		});
	}
	@Post('resetPassword/:token')
	public async resetPasswordUpdate(@Param('token') token, @Res() res, @Body(new ValidationPipe()) resetPasswordDto: ResetPasswordDto) {
		await this.accountService.resetPasswordUpdate(token, resetPasswordDto.password);

		return res.status(HttpStatus.OK).send({
			'message': 'New Password updated successfully'
		});
	}

	@Post('resetPasswordBO')
	public async resetPasswordUpdateBO(@Res() res, @Body(new ValidationPipe()) resetPasswordDto: ResetPasswordBoDto) {
		await this.accountService.updatePasswordFrom_BO(resetPasswordDto);

		return res.status(HttpStatus.OK).send({
			'message': 'New Password updated successfully'
		});
	}
		
	@Post('getEventRegister')
	async getEventRegister(@Body(new ValidationPipe()) data: any, @Res() res) {

		var options = {
			'method': 'GET',
			'url': data.url,
			'headers': {
				'Authorization': data.token.token_type + ' ' + data.token.access_token
			}
			};
			request(options, function (error, response) {
			if (error) throw new Error(error);
			
			return res.status(HttpStatus.OK).send(response.body);
			});
	}
		@Post('postEventRegister')
		async postEventRegister(@Body(new ValidationPipe()) data: any, @Res() res) {

			var options = {
				'method': 'POST',
				'url': data.url,
				'headers': {
					'Authorization': data.token.token_type + ' ' + data.token.access_token,
					'Content-Type' : 'application/json'
				},
				body: JSON.stringify(data.body)
				};
				request(options, function (error, response) {
				if (error) throw new Error(error);

				return res.status(HttpStatus.OK).send(response.body);
				});
		}
		@Post('eventcatalystToken')
		async eventcatalystToken(@Body(new ValidationPipe()) data: any, @Res() res) {
			let body = new URLSearchParams();
			body.set('grant_type', process.env.EVENTCAT_GRANT_TYPE);
			body.set('username', process.env.EVENTCAT_USERNAME);
			body.set('password', process.env.EVENTCAT_PASSWORD);
			var options = {
				'method': 'POST',
				'url': data.url,
				'headers': {
					'Content-Type': 'application/x-www-form-urlencoded'
				},
				body: body.toString()
				};
				request(options, function (error, response) {
				if (error) throw new Error(error);

				return res.status(HttpStatus.OK).send(response.body);
				});
		}

		
	@Get('getPublicToken/:id')
	public async getPublicToken(@Param('id') id, @Res() res) {
		let token = await this.accountService.encryptAuth(id);
		return res.status(HttpStatus.OK).send({
			token: token
		});
	}

	@Get('checkAuth/:token')
	public async checkPublicAuth(@Param('token') token, @Res() res) {
		let id = await this.accountService.decryptAuth(token);
		return res.status(HttpStatus.OK).send({
			id: id
		});
	}
	
	@Get('setEventContact/:id')
	public async setEventContact(@Param('id') id, @Res() res) {
		await this.accountService.isEvent(id, '1');
		return res.status(HttpStatus.OK).send();
	}
}
