import { IsNotEmpty, IsEmail, MinLength, MaxLength, IsString, IsAlphanumeric, IsNumber, Matches, IsOptional, ArrayNotEmpty } from 'class-validator';
import { ApiProperty } from '@nestjs/swagger';

export class SignupDto {

	@MinLength(2, { message: "account:singup:fisrtNameMinLeng" })
	@MaxLength(30, { message: "account:singup:fisrtNameMaxLeng" })
	@IsNotEmpty({ message: "account:singup:fisrtNameIsNotEmpty" })
	@IsString({ message: "account:singup:firstnameonlyacceptStringOnly" })
	@ApiProperty()
	firstName: string;

	@MaxLength(30, { message: "account:singup:lastNameMaxLeng" })
	@IsNotEmpty({ message: "account:singup:lastNameIsNotEmpty" })
	@IsString({ message: "account:singup:lastnameonlyacceptStringOnly" })
	@ApiProperty()
	lastName: string;

	@IsEmail({}, { message: "account:singup:emailInvalide" })
	@IsNotEmpty({ message: "account:singup:emailIsNotEmpty" })
	@ApiProperty()
	email: string;

	@IsNotEmpty({ message: "account:singup:passwordIsNotEmpty" })
	@MinLength(8, { message: "account:singup:passwordMinLen" })
	@MaxLength(25, { message: "account:singup:passwordMaxLen" })
	@Matches(RegExp("^(?=.*?[A-Z])(?=.*?[a-z])(?=.*?[0-9])(?=.*?[#?!@$%^&*-]).{8,25}$"), { message: "account:singup:passwordMatches" })
	@ApiProperty()
	password: string;

	@MinLength(2, { message: "account:singup:companyNameMinLeng" })
	@IsNotEmpty({ message: "account:singup:companyNameIsNotEmpty" })
	@IsString({ message: "account:singup:companynameonlyacceptStringOnly" })
	@ApiProperty()
	companyName: string;

	@IsNotEmpty({ message: "account:singup:companyAddressIsNotEmpty" })
	@MaxLength(255, { message: "account:singup:companyAddressMaxLeng" })
    @ApiProperty()
	readonly address: string;
	
	@MinLength(1, { message: "account:singup:jobPositionMinLeng" })
	@MaxLength(80, { message: "account:singup:jobPositionMaxLeng" })
	@IsNotEmpty({ message: "account:singup:jobPositionIsNotEmpty" })
	@ApiProperty()
	jobPosition: string;

	@IsNotEmpty({ message: "account:singup:cityIsNotEmpty" })
    @ApiProperty()
	city: string;
	
	@IsOptional()
	@IsNotEmpty({ message: "account:singup:stateIsNotEmpty" })
    @ApiProperty()
	state: string;
	
    @IsNotEmpty({ message: "account:singup:countryIdIsNotEmpty" })
    @IsNumber({}, { message: "account:singup:countryIdonlyacceptInt" })
    @ApiProperty()
	countryId: number;
	
	@IsNotEmpty({ message: "account:singup:postalcodeIsNotEmpty" })
	@MinLength(3, { message: "account:singup:postalcodeMinLen" })
	@MaxLength(12, { message: "account:singup:postalcodeMaxLen" })
    @ApiProperty()
	postalCode: string;
	
	@IsNotEmpty({ message: "account:singup:websiteurlIsNotEmpty" })
	@ApiProperty()
	readonly websiteurl: string;

	@IsNotEmpty({ message: "account:singup:mobileIsNotEmpty" })
	@MinLength(4, { message: "account:singup:mobileMinLen" })
	@MaxLength(20, { message: "account:singup:mobileMaxLen" })
	@ApiProperty()
	mobile:string;

	@ApiProperty()
	channelTypeId:number;

	@IsOptional()
	@ApiProperty()
	oldChannelId:number;

	@ApiProperty()
	hubspot_contact_id: number;
	
	@ApiProperty()
	hubspotCompanyId:number;

	@ApiProperty()
	switchPhone:string;
	
	@ArrayNotEmpty({ message: "validate:field:shouldBeArray" })
	@IsNumber({}, {
		each: true,
		message: "validate:field:shouldBeInt"
	})
	@ApiProperty({ type: 'number', isArray: true })
	readonly productFamilies: number[];

	@ArrayNotEmpty({ message: "validate:field:shouldBeArray" })
	@IsNumber({}, {
		each: true,
		message: "validate:field:shouldBeInt"
	})
	@ApiProperty({ type: 'number', isArray: true })
	readonly productCategories: number[];
	
	@ApiProperty()
	readonly isOEM: string;

	//events 
	@IsOptional()
	readonly eventId: number;

	@IsOptional()
	readonly participantType: number;
}
