import { Inject, Injectable } from '@nestjs/common';
import { Address } from 'src/modules/channel/models/Address.entity';
import { Channel } from 'src/modules/channel/models/Channel.entity';
import { ChannelDetail } from 'src/modules/channel/models/ChannelDetail.entity';
import { ChannelProductGroup } from 'src/modules/channel/models/ChannelProductGroup.entity';
import { Contact } from 'src/modules/user/models/Contact.entity';
import { SignUpHistory } from 'src/modules/user/models/SignUpHistory.entity';
import { User } from 'src/modules/user/models/User.entity';
import { UserRole } from 'src/modules/user/models/UserRole.entity';
import { SignupDto } from '../dto/signup.dto';
import { Utility } from '../helper/utility';
import { Op } from 'sequelize';
import { Sequelize } from 'sequelize-typescript';
import { ProcessException } from 'src/middleware/exception/Process.exception';
import { ChannelProductCategorie } from 'src/modules/channel/models/ChannelProductCategorie.entity';
import { OldChannel } from 'src/modules/channel/models/OldChannel.entity';
var _ = require('lodash');
import { createCipheriv, randomBytes } from 'crypto';
import { OldAddress } from 'src/modules/channel/models/OldAddress.entity';
import { EmailerService } from 'src/modules/emailer/services/emailer.service';
import { EventParticipants } from 'src/modules/channel/models/EventParticipants.entity';
import { EventChannel } from 'src/modules/channel/models/EventChannel.entity';
const Cryptr = require('cryptr');

@Injectable()
export class AccountService {
	private utility = new Utility();
    constructor(
		private readonly emailerService : EmailerService,
		@Inject('UserRepository') private readonly userRepository: typeof User,
		@Inject('SequelizeInstance') private readonly sequelizeInstance,
		@Inject('ContactRepository') private readonly contactRepository: typeof Contact,
		@Inject('UserRoleRepository') private readonly userRoleRepository: typeof UserRole,
		@Inject('ChannelRepository') private readonly channelRepository: typeof Channel,
		@Inject('SignUpHistoryRepository') private readonly signUpHistorylRepository: typeof SignUpHistory,
		@Inject('AddressRepository') private readonly addressRepository: typeof Address,
		@Inject('ChannelDetailRepository') private readonly channelDetailRepository: typeof ChannelDetail,
		@Inject('ChannelProductGroupRepository') private readonly channelProductGroupRepository: typeof ChannelProductGroup,
		@Inject('ChannelProductCategoryRepository') private readonly channelCatRepository: typeof ChannelProductCategorie,
		@Inject('OldChannelRepository') private readonly oldChannelRepository: typeof OldChannel,
		@Inject('EventParticipantRepository') private readonly eventParticipantRepository: typeof EventParticipants,
		@Inject('EventChannelRepository') private readonly eventChannelRepository: typeof EventChannel,
		){
        }
		async create(user: SignupDto, skipVerify = false): Promise<any> {
			// generate 4 digit pin
			let pin = await this.utility.generatePin();
	
			// construct user object
			let userObject = await this.setSignUpObject(user, pin, 14);
	
			//  create user, contact and channel
			return await this.sequelizeInstance.transaction(async transaction => {
				return await this.userRepository.create(userObject, {
					include: [{
						model: this.contactRepository,
					}, {
						model: this.channelRepository,
						include: [{
							model: this.signUpHistorylRepository,
						}]
					},
					{
						model: this.userRoleRepository
					}
					],
					transaction,
				}).then(async (userInsetedtObject) => {
					await this.contactRepository.update({ channelId: userInsetedtObject.channelId }, {
						where: {
							contactId: userInsetedtObject.contactId
						},
						returning: true,
						transaction
					});
					let regAddressId = await this.addressRepository.create(
						{
							address: user.address,
							city: user.city,
							countryId: user.countryId,
							postalCode: user.postalCode,
							state: user.state
						},
						{  transaction }
					);
					let comapnyRegAddressId = regAddressId.addressId;
					await this.channelRepository.update({ regAddressId: comapnyRegAddressId }, {
						where: {
							channelId: userInsetedtObject.channelId
						},
						returning: true,
						transaction
					});
					await this.channelDetailRepository.create({
						channelId : userInsetedtObject.channelId,
						webSiteUrl : user.websiteurl,
						phone1 : user.switchPhone,
						createdAt : new Date()
					}, { transaction });

				// update product group & category
					// let ChannelProductGroupObject: ChannelProductGroup[] = await this.setProductGroup(user.productFamilies, +userInsetedtObject.channelId);
					// await this.channelProductGroupRepository.bulkCreate(ChannelProductGroupObject, { returning: true, transaction });

				let ChannelGroupObject: ChannelProductGroup[] = await this.setChannelProductGroup(user.productFamilies, +userInsetedtObject.channelId);
                await this.channelProductGroupRepository.bulkCreate(ChannelGroupObject, { returning: true, transaction });

                let ChannelCatObject: ChannelProductCategorie[] = await this.setChannelProductCat(user.productCategories, +userInsetedtObject.channelId);
				await this.channelCatRepository.bulkCreate(ChannelCatObject, { returning: true, transaction });
				
				//event setting
				if(user.eventId){
					let participants = {
						eventId: user.eventId,
						contactId: userInsetedtObject.contactId,
						channelId: userInsetedtObject.channelId,
						participantType: user.participantType,
						channelType: user.channelTypeId,
						createdAt: new Date()
					}
					let eventChannel = {
						channelId: userInsetedtObject.channelId,
						eventId: user.eventId,
						isNew: '1',
						createdAt: new Date()
					}
					await this.eventParticipantRepository.create(participants, { transaction })
					await this.eventChannelRepository.create(eventChannel, { transaction })
				}
				

					return userInsetedtObject;
				})
			})
			.catch((err) => {
				console.log(err)
				throw new ProcessException('account:singup:tryAgain');
			});
		}
	async setSignUpObject(user: SignupDto, pin: string, signUpStatus: number): Promise<User> {
		try {

			let signUpHistory = <SignUpHistory>{};

			//signUpHistory object
			signUpHistory.signUpStatusId = signUpStatus;
			signUpHistory.createdAt = new Date()

			let channel = <Channel>{};
			//chennal object
			channel.companyName=user.companyName;
			channel.channelTypeId=user.channelTypeId
			channel.signUpStatusId = signUpStatus;
			// channel.channelTypeId = Channel.CHANNEL_TYPE_SIGNUP;
			channel.countryId = (user.countryId) ? user.countryId : null;
			//
			channel.isActive = '1';
			channel.createdAt = new Date();
			channel.signUpHistory = signUpHistory;
			channel.hubspotCompanyId=user.hubspotCompanyId;
			channel.planId=1;
			channel.oldChannelId= user.oldChannelId != undefined?user.oldChannelId:null;
			channel.isOEM= user.isOEM
			if(+signUpStatus == 4) {
				channel.isAdminCreated = '1';
			}

			let contact = <Contact>{};
			//contact Object
			contact.firstName = user.firstName;
			contact.lastName = user.lastName;
			contact.email = user.email;
			contact.jobTitle = user.jobPosition;
			contact.phone1= user.switchPhone;
			contact.mobileNo= user.mobile;
			contact.hubspotContactId = (user.hubspot_contact_id) ? user.hubspot_contact_id : null;
			contact.countryId = user.countryId;
			contact.isActive = '1';
			contact.isDefaultContact = '1'; // make default contact when signup
			contact.createdAt = new Date();
			contact.isShared = '1';
			let userRole = <UserRole>{};
			userRole.roleId = 1;

			let returnObject = <User>{};
			// user object
			returnObject.contact = contact;
			returnObject.channel = channel;
			returnObject.role = userRole;
			returnObject.email = user.email;
			returnObject.password = user.password;
			returnObject.verificationCode = pin;
			returnObject.verificationSentAt = new Date();
			returnObject.isActive = '1';
			returnObject.createdAt = new Date();

			// console.log("returnObject",returnObject);
			return returnObject;
		} catch (e) {
			throw new ProcessException('account:objectCreation:User');
		}
	}
    async checkEmail(email: string) {
        let result = await this.userRepository.findOne<User>({
			where: {
				email: 
				{
					[Op.iLike]: '%' +email+ '%'
				},
				deletedAt : null 
			}
		});
console.log(result)
		if (!result) {
			return true;
		}

		return false;
	}

	async checkCompanyName(name: string) {
        let result = await this.channelRepository.findOne<Channel>({
			where:Sequelize.where(Sequelize.fn('lower', Sequelize.col('companyName')), Sequelize.fn('lower', name)),
		});
		console.log(result)
		if (!result) {
			return true;
		}

		return false;
	}

	async setProductGroup(productFamilies: any, channelId: number): Promise<ChannelProductGroup[]> {
		try {
			let resultObject = <Array<ChannelProductGroup>>[];
			if (productFamilies) {
				for (let ii = 0; ii < productFamilies.length; ii++) {
					let innerObject = <ChannelProductGroup>{};
					innerObject.channelId = channelId;
					innerObject.productGroupId = productFamilies[ii];
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:productGroup');
		}
	}

	async setChannelProductGroup(productGroup: any, channelId: number): Promise<ChannelProductGroup[]> {
		try {
            //console.log("producdtGroup:",productGroup);
            productGroup=_.uniq(productGroup);
            
			let resultObject = <Array<ChannelProductGroup>>[];
			if (productGroup) {
				for (let ii = 0; ii < productGroup.length; ii++) {
					let innerObject = <ChannelProductGroup>{};
					innerObject.channelId = channelId;
					innerObject.productGroupId = productGroup[ii];
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
            console.log("err:",e);
			throw new ProcessException('channel:objectCreation:channelBrand');
		}
    }
    
    async setChannelProductCat(productCat: any, channelId: number): Promise<ChannelProductCategorie[]> {
		try {
			let resultObject = <Array<ChannelProductCategorie>>[];
			if (productCat) {
				for (let ii = 0; ii < productCat.length; ii++) {
					let innerObject = <ChannelProductCategorie>{};
					innerObject.channelId = channelId;
					innerObject.productCategorieId = productCat[ii];
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:channelBrand');
		}
    }

	async getCompanyData(name: string) {
        let result = await this.oldChannelRepository.findOne<OldChannel>({
			where:Sequelize.where(Sequelize.fn('lower', Sequelize.col('companyName')), Sequelize.fn('lower', name)),
			include:[
				{
					model: OldAddress,
					as: 'regAddress',
					attributes: ['address', 'countryId', 'postalCode', 'city']
				}
			]
		});
		let data = {};
		if(result.oldChannelId){
			//to be finished

		}
		console.log(result)


		return result;
	}

	async forgotPassword(email: string): Promise<any> {
		console.log('email:', email);
		let user = await this.userRepository.findOne<User>({
			where: {
				email: {
					[Op.iLike]: email
				},
				deletedAt : null 
			},
			include: [
				{
					model: Contact,
					attributes: ['firstName', 'lastName']
				}
			]
		});
		console.log('user:', user);
		if (!user) {
			console.log('not user');
			throw new ProcessException('account:forgotPassword:invalidUser');
		}

		let userObject = {
			resetPasswordToken: randomBytes(20).toString('hex'),
			resetPasswordExpires: this.sequelizeInstance.literal("CURRENT_TIMESTAMP + interval '" + +process.env.RESET_PASSWORD_TOKEN_EXDPIRE + "' day")
		}

		return await this.userRepository.update(userObject, {
			where: {
				userId: +user.userId,
				deletedAt : null 
			}
		}).then((result) => {

			let data = {
				name: user.contact.firstName + " " + user.contact.lastName,
				email: email,
				message: 'An e-mail has been sent to ' + email + ' with further instructions.',
				url: process.env.DOMAIN + "resetpassword/" + userObject.resetPasswordToken
			}
			//this.emailerService.email('Password Recovery Mail', email, 'Forget_Password', data)
		    // this.sendRecoveEmail(userObject, email, user.contactId);


			return data;//userObject.resetPasswordToken;
		}).catch((err) => {
			console.log('eeee', err);
			throw new ProcessException('account:forgotPassword:invalidUser');
		});
	}
	async resetPassword(token: string): Promise<any> {
		let user = await this.userRepository.findOne<User>({
			where: {
				resetPasswordToken: token,
				resetPasswordExpires: {
					[Op.gt]: new Date().toUTCString()
				},
				deletedAt:null
			}
		});

		if (!user)
			throw new ProcessException('account:resetPassword:tokenExpired');

		return true;
	}
	async resetPasswordUpdate(token: string, password: string): Promise<any> {
		let user = await this.userRepository.findOne<User>({
			where: {
				resetPasswordToken: token,
				resetPasswordExpires: {
					[Op.gt]: new Date().toUTCString()
				},
				deletedAt:null
			}
		});

		if (!user) throw new ProcessException('account:resetPassword:tokenExpired');

		// update password and reset token
		user.password = password;
		user.resetPasswordToken = null;
		user.resetPasswordExpires = null;

		return await user.save().then(() => {
			return true;
		}).catch((err) => {
			throw new ProcessException('account:resetPassword:tokenExpired');
		});
	}


	async updatePasswordFrom_BO(updatePasswordDto) {

			let user = await User.findOne({where: {
				userId: +updatePasswordDto.userId,
				channelId: +updatePasswordDto.channelId,
				contactId: +updatePasswordDto.contactId
			}});

			// update new password
			user.password = updatePasswordDto.password;
			return await user.save().then(() => {
				return true;
			}).catch((err) => {
				return false;
			});
	}


	async encryptAuth(token){
		const cryptr = new Cryptr('ch-event-auth');
		return cryptr.encrypt(token);
	}
	
	async decryptAuth(token){
		const cryptr = new Cryptr('ch-event-auth');
		return cryptr.decrypt(token);
	}

	async isEvent(ContactId, isEvent){
		return await this.contactRepository.update({ isEvent:isEvent},{
			where:{
				contactId : ContactId,
			},
			returning: true
		});
	}

}
