import { User } from '../../user/models/User.entity';
import { ProcessException } from '../../../middleware/exception/Process.exception';
import { appconstant } from "../../../config";
import { IAccountOutBound, ILoginResponse } from './Interface/IAccountOutBound';


export class AccountOutBound implements IAccountOutBound {

	async setLoginResponse(user: User, token: string, productCount: number = 0, signUpStatusId?: number, isAnyInbound?: boolean): Promise<ILoginResponse> {
		try {
			let returnObject = <ILoginResponse>{};
			returnObject.token = 'Bearer ' + token;
			returnObject.channelId = parseInt(user.channelId.toString());
			returnObject.currency = (user.contact.channel.channelDetail) ? +user.contact.channel.channelDetail.currencyid : null;
			returnObject.contactId = parseInt(user.contactId.toString());
			returnObject.role = (user.roles) ? user.roles[0] : null;
			returnObject.permissions = user.permissions;
			returnObject.profile = user.contact;
			let channel = user.contact.channel;
			returnObject.oldChannel = !!(user.contact.channel.oldChannelId);
			returnObject.channelStatusId = (signUpStatusId && signUpStatusId > 0) ? signUpStatusId : parseInt(channel.signUpStatusId.toString());
			returnObject.sourcingProfile = appconstant.SOURCING_CONTACT;
			returnObject.sellingProfile = appconstant.SELLING_CONTACT;
			returnObject.otherProfile = appconstant.OTHER_CONTACT;
			returnObject.isAnyInbound = (isAnyInbound != undefined)? isAnyInbound : false;
			returnObject.productCount = (user.contact.channel.actAsDistributor === '1') ? (productCount === 0) ? 1 : productCount : productCount;
			return returnObject;
		} catch (e) {
			throw new ProcessException('account:objectCreation:userResponse');
		}
	}
}
