'use strict';

//import { BaseTransformation } from "../../core/transformation/BaseTransformation";
import { ProcessException } from '../../../middleware/exception/Process.exception';

import { User } from '../../user/models/User.entity';
import { Contact } from '../../user/models/Contact.entity';
import { Channel } from '../../channel/models/Channel.entity';
import { SignUpHistory } from '../../user/models/SignUpHistory.entity';
import { SignupDto } from '../dto/signup.dto';
import { UserRole } from '../../user/models/UserRole.entity';
import { Role } from '../../user/models/Role.entity';
import { appconstant } from "../../../config";
import { IAccountSerivceInBound, IMailObject, IUserToken } from './Interface/IAccountSerivceInBound';

export class AccountSerivceInBound implements IAccountSerivceInBound {
	

	async setPayload(user: User): Promise<IUserToken> {
		try {
			let returnObject: IUserToken = <IUserToken>{};
			returnObject.id = user.userId;
			returnObject.email = user.email;
			returnObject.channelId = user.channelId;
			returnObject.contactId = user.contactId;
			returnObject.tokenModifiedAt = user.tokenModifiedAt.getTime();

			return returnObject;
		} catch (e) {
			throw new ProcessException('account:objectCreation:userToken');
		}
	}

	async setDataSession(user: User): Promise<any> {
		let returnObject: any = {};
		let channel = user.channel || user.contact.channel;

		returnObject.channelTypeId = (channel) ? channel.channelTypeId : null;
		returnObject.role = (user.roles) ? user.roles[0].toJSON() : null;
		returnObject.plan = (channel) ? channel.planId : null;
		returnObject.workNatureId = (user.contact) ? user.contact.workNatureId : null;
		returnObject.permissions = (user.permissions) ? user.permissions.map(a => +a.accesspermissionid) : null;

		return returnObject;
	}

	async setSignUpObject(user: SignupDto, pin: string, signUpStatus: number): Promise<User> {
		try {

			let signUpHistory = <SignUpHistory>{};

			//signUpHistory object
			signUpHistory.signUpStatusId = signUpStatus;
			signUpHistory.createdAt = new Date()

			let channel = <Channel>{};
			//chennal object
			channel.signUpStatusId = signUpStatus;
			channel.channelTypeId = Channel.CHANNEL_TYPE_SIGNUP;
			//channel.countryId = (user.country) ? user.country : null;
			//
			channel.isActive = '1';
			channel.createdAt = new Date();
			channel.signUpHistory = signUpHistory;
			if(+signUpStatus == appconstant.SUBSCRIPTION) {
				channel.isAdminCreated = '1';
			}

			let contact = <Contact>{};
			//contact Object
			contact.firstName = user.firstName;
			contact.lastName = user.lastName;
			contact.email = user.email;
			contact.hubspotContactId = (user.hubspot_contact_id) ? user.hubspot_contact_id : null;
			//contact.countryId = user.country;
			contact.isActive = '1';
			contact.isDefaultContact = '1'; // make default contact when signup
			contact.createdAt = new Date();

			let userRole = <UserRole>{};
			userRole.roleId = Role.ROLE_ACCOUNT_USER;

			let returnObject = <User>{};
			// user object
			returnObject.contact = contact;
			returnObject.channel = channel;
			returnObject.role = userRole;
			returnObject.email = user.email;
			returnObject.password = user.password;
			returnObject.verificationCode = pin;
			returnObject.verificationSentAt = new Date();
			returnObject.isActive = '1';
			returnObject.createdAt = new Date();

			return returnObject;
		} catch (e) {
			throw new ProcessException('account:objectCreation:User');
		}
	}

	async setMailObject(email: string, pin: string): Promise<IMailObject> {
		let returnObject = <IMailObject>{};
		returnObject.pin = pin;
		returnObject.email = email;
		returnObject.tokenExpires = Date.now() + 3600000 * 24

		return returnObject;
	}
}
