import {
    Table, Column, PrimaryKey, Model, AutoIncrement, DataType, BeforeCreate,
    AllowNull,
    BeforeUpdate, ForeignKey, HasOne, HasMany, Scopes, BelongsTo, BelongsToMany, AfterUpdate, AfterBulkUpdate
} from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import {AdminMenu } from './AdminMenu.entity';
import {AdminUserAccess} from './AdminUserAccess.entity';
import { AdminUserChanneltype } from './AdminUserChanneltype.entity';
import { AdminUserCountries } from './AdminUserCountries.entity';
import { AdminUserNotification } from './AdminUserNotification.entity';
import * as crypto from 'crypto';

const tableOptions: IDefineOptions = { tableName: 'tblAdminUsers' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
    
    userLogInInfom: {
		attributes: ['adminUserId', 'jobTitle', 'permissions','password','emailId']
	}

}))

export class AdminUsers extends Model<AdminUsers>{
	
	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	adminUserId: number	
		
	@Column(DataType.CHAR(255))
    fName: string
    
    @Column(DataType.CHAR(255))
    lName: string
    
    @Column(DataType.CHAR(255))
	jobTitle: string

    @Column(DataType.CHAR(255))
    role: string;
    
    @Column(DataType.CHAR(255))
    emailId: string
    
    @Column(DataType.CHAR(255))
    password: string

    @Column
    isVerified: string

    @Column
    isActive: string

    @Column
    deletedAt: Date

    @Column(DataType.CHAR(255))
    reportingTo: string;

    @Column(DataType.BIGINT)
    accessId: number;
       
    @HasMany(() => AdminUserCountries, 'adminUserId')
    managedCountries: AdminUserCountries[];

    @HasMany(() => AdminUserNotification, 'adminUserId')
    preference: AdminUserNotification[];
    
    @HasMany(() => AdminUserChanneltype, 'adminUserId')
	managedChannelTypes: AdminUserChanneltype[];

    @BeforeCreate
    //@BeforeUpdate
    public static async hashPassword(user: AdminUsers, options: any) {
        if (user.password) {
            var hash = crypto.createHmac('sha512', process.env.USER_PASSWORD_SALT_ROUNT); /** Hashing algorithm sha512 */
            hash.update(user.password);
            user.password = hash.digest('hex');
        }
    }

	// @BeforeCreate
	// @BeforeUpdate
	// public static async dateUpdate(user: AdminUsers, options: any) {
	// 	user.updatedAt = Sequelize.fn('NOW');
	// }
    
	}
