import { Inject, Injectable } from '@nestjs/common';
import { AdminUsers } from '../models/AdminUsers.entity';
import {Op, QueryTypes } from "sequelize";
import { AdminUserCountries } from '../models/AdminUserCountries.entity';
import { AdminUserChanneltype } from '../models/AdminUserChanneltype.entity';
import { Country } from 'src/modules/common/models/Country.entity';
import { ProcessException } from 'src/middleware/exception/Process.exception';
import { AdminUserNotification } from '../models/AdminUserNotification.entity';

@Injectable()
export class AdminService {
    private specialChars = ['@', '#', '%', '$', '*'];
    constructor(
        @Inject('SequelizeInstance') private readonly sequelizeInstance,
        @Inject('AdminUserRepository') private readonly adminUserRepository: typeof AdminUsers,        
        @Inject('AdminUserCountryRepository') private readonly adminUserCountryRepository: typeof AdminUserCountries,        
        @Inject('AdminUserChannelRepository') private readonly AdminUserChannelRepository: typeof AdminUserChanneltype,
        @Inject('AdminUserNotificationRepository') private readonly adminUserNotificationRepository: typeof AdminUserNotification,

        //@Inject('SequelizeInstance') private readonly sequelizeInstance
    ){}
    
    public async userListing(data) { 
        try{
            let filter = data.filter;
        
            const users = await this.adminUserRepository.scope().findAndCountAll({
                limit: data.limit,
                offset: data.page,
                //required:(data.search)?true:false,
                where: (data.search)?{adminUserId:+data.search}:{},
                distinct:true,             
                include: [{
                    model: AdminUserCountries,
                    attributes: [['countryId', 'id']],
                    required:(filter.countryId)?true:false,
                    where:(filter.countryId)?{countryId: {[Op.in]: filter.countryId }}:{},
                
                include:[{
                    model: Country,                
                    attributes: [['countryId', 'id'], ['country', 'name']],
                    required:false,                
                    }],
                },                                      
                {
                model: AdminUserChanneltype,
                    attributes: [['typeId', 'id'], ['channelType', 'name']],
                required:(filter.channelTypeId)?true:false,
                where: (filter.channelTypeId)?{typeId:+filter.channelTypeId}:{},
                    },

                    {
                        model: AdminUserNotification,
                        attributes: [['notificationId', 'id']],
                        required: false,

                    }
                ]
        
            });      
                return users;
        }catch(e){
            console.log(e)
        }
    }

    async updateContact(contactId, data) {

        let contactData = data.userData;
        let preference = data.preference;
        console.log("contactData:", contactData);
        console.log("preference:", preference);

        let result;
        //update avatar
        let contact = await this.adminUserRepository.findOne({
            where: {
                adminUserId: contactId
            }
        });


        if (!contact && contactData) {
            let password: string = await this.getPassword();
            let contact = {}; let managedCty = []; let managedType = []; let notification = [];
            if (contactData.firstName)
                contact['fName'] = contactData.firstName;
            if (contactData.lastName)
                contact['lName'] = contactData.lastName;
            if (contactData.email)
                contact['emailId'] = contactData.email;
            // if (contactData.password)
            //     contact['password'] = contactData.country.id;
            if (contactData.role)
                contact['role'] = contactData.role.name;

            contact['accessId'] = contactData.role.id;
            contact['isActive'] = '1';
            contact['password'] = password;

            if (contactData.role.id == 2) {
                let ven = contactData.managedChannelType.filter(x => x.id == 2);
                if (ven.length > 0)
                    contact['accessId'] = 2;
                else
                    contact['accessId'] = 3;
            }

            await this.adminUserRepository.create(contact).then((results) => {
                contactId = results['dataValues'].adminUserId;

                result = {
                    name: contactData.firstName + ' ' + contactData.lastName,
                    profileurl: process.env.DOMAIN,
                    userId: contactData.email,
                    to: contactData.email,
                    password: password,
                    accountOwner: 'ChannelHub'
                }

                if (contactData.managedChannelType && contactData.managedChannelType?.length > 0) {
                    contactData.managedChannelType.map(x => {
                        managedType.push({ adminUserId: contactId, typeId: x.id, channelType: x.name });
                    })
                }

                if (contactData.managedCountry && contactData.managedCountry?.length > 0) {
                    contactData.managedCountry.map(x => {
                        managedCty.push({ adminUserId: contactId, countryId: x.id });
                    })
                }

                if (preference && preference.length > 0) {
                    preference.map(x => {
                        notification.push({ adminUserId: contactId, notificationId: x })
                    })
                }

            })


            if (managedCty)
                await this.adminUserCountryRepository.bulkCreate(managedCty);

            if (managedType)
                await this.AdminUserChannelRepository.bulkCreate(managedType);

            if (notification)
                await this.adminUserNotificationRepository.bulkCreate(notification);


        }
        else if (contactData) {
            let contact = {}; let managedCty = []; let managedType = []; let notification = [];
            if (contactData.firstName)
                contact['fName'] = contactData.firstName;
            if (contactData.lastName)
                contact['lName'] = contactData.lastName;
            if (contactData.email)
                contact['emailId'] = contactData.email;
            // if (contactData.password)
            //     contact['password'] = contactData.country.id;
            if (contactData.role)
                contact['role'] = contactData.role.name;

            contact['accessId'] = contactData.role.id;

            if (contactData.role.id == 2) {
                let ven = contactData.managedChannelType.filter(x => x.id == 2);
                if (ven.length > 0)
                    contact['accessId'] = 2;
                else
                    contact['accessId'] = 3;
            }


            if (contactData.managedChannelType && contactData.managedChannelType?.length > 0) {
                contactData.managedChannelType.map(x => {
                    managedType.push({ adminUserId: contactId, typeId: x.id, channelType: x.name });
                })
            }

            if (contactData.managedCountry && contactData.managedCountry?.length > 0) {
                contactData.managedCountry.map(x => {
                    managedCty.push({ adminUserId: contactId, countryId: x.id });
                })
            }

            if (preference && preference.length > 0) {
                preference.map(x => {
                    notification.push({ adminUserId: contactId, notificationId: x })
                })
            }

            // if (contactData.mobileNo)
            //     contact['mobileNo'] = contactData.mobileNo;
            // if (contactData.jobTitle)
            //     contact['jobTitle'] = contactData.jobTitle;

            // if (+contactData.rptContactId === 0) {
            //     contact['rptContactId'] = null;
            // } else {
            //     contact['rptContactId'] = +contactData.rptContactId;
            // }

            await this.adminUserRepository.update(contact, {
                where: {
                    adminUserId: contactId
                }
            });

            await this.adminUserCountryRepository.destroy({
                where: {
                    adminUserId: contactId
                }
            });

            if (managedCty)
                await this.adminUserCountryRepository.bulkCreate(managedCty);

            await this.AdminUserChannelRepository.destroy({
                where: {
                    adminUserId: contactId
                }
            });

            if (managedType)
                await this.AdminUserChannelRepository.bulkCreate(managedType);

            await this.adminUserNotificationRepository.destroy({
                where: {
                    adminUserId: contactId
                }
            });

            if (notification)
                await this.adminUserNotificationRepository.bulkCreate(notification);

        }
        return result;
    }

    public async userCountries() {       
       
        const countries = await this.adminUserCountryRepository.scope().findAll({             
            
            include:[{
                model: Country,                                
                attributes: [['countryId','id'],['country','name']],
                required: false, 
                }],

                attributes: []

            
        
        });

        return countries;
    }

    public async searchList() {       
               const nameList = await this.adminUserRepository.scope().findAll({                                    
                attributes: [['adminUserId','id'],'fName','lName'],
                where:{ isActive: '1', deletedAt: null},
        });

        return nameList;
    }

    public async menuListing(accessId) {
        
        let routes=[];
        let cquery = 'select * from "tblAdminUserAccess" as aua inner join "tblAdminMenu" as am on am."moduleId"=aua."moduleId" where aua."accessId"=:accessId order by am."moduleId"';
        await this.sequelizeInstance.query(cquery, {
			type: QueryTypes.SELECT,
			replacements: {
				accessId: accessId				
			}
		}).then((results) => {
            //console.log("userlists:",results);
            results.map(x => {
                let menu={};
                if(x.allowView==='1')
                {
                    if(x.parentMenuId==0&&x.isParent=='0')
                    {
                        menu={
                            collapse: x.collapse,
                            name: x.menuName,
                            icon: x.icon,
                            state: x.state,
                            path: x.path,                      
                            miniName: x.miniName,
                            component: x.component,
                            layout: x.layout
                        }                        
                    }
                    else if(x.parentMenuId==0&&x.isParent=='1')
                    {
                        let childviews=[];                        
                        let child=results.filter(p => p.parentMenuId==x.moduleId)                        
                        child.map(c => {
                            if(c.allowView==='1')
                            {
                            let view={
                                path: c.path,
                                name: c.menuName,
                                miniName: c.miniName,
                                component: c.component,
                                layout: c.layout
                            }
                            childviews.push(view);
                            }
                        })
                        menu={
                            collapse: x.collapse,
                            name: x.menuName,
                            icon: x.icon,
                            state: x.state,
                            views: childviews
                        }
                    }                    
                    routes.push(menu);
                }
            })
			
        });            
        return routes;
    }    

    async getPassword() {
        var generator = require('generate-password');

        var password = generator.generate({
            length: 9,
            numbers: true,
            uppercase: true,
            strict: true
        });

        return password + this.specialChars[Math.floor(Math.random() * this.specialChars.length)];
    }
}
