'use strict';

import { Post, Get, Put, Delete, HttpStatus, Request, Response, Param } from '@nestjs/common';
import { BaseController } from 'src/core';


export abstract class ApiController extends BaseController {
	public apiService;

	constructor() {
		super();
	}

	public async assignService(service) {
		this.apiService = service;
	}

	@Get('/')
	public async getAll(@Request() req, @Response() res) {
		let total = 0;
		this.apiService.getCount().then(function(c) {
			total = c;
		});
		let pagination = {
			page:1,
			limit:20
		}
		const list = await this.apiService.findAll(pagination);

		res.setHeader('app-pagination-page', pagination.page);
		res.setHeader('app-pagination-limit', pagination.limit);
		res.setHeader('app-pagination-total', total);

		return res.status(HttpStatus.OK).json(list);
	}

	@Get('/:id')
	public async getById(@Param('id') id, @Request() req, @Response() res) {
		const row = await this.apiService.findById(id);
		return res.status(HttpStatus.OK).json(row);
	}

	@Post('/')
	public async create(@Request() req, @Response() res, data) {
		let result = await this.apiService.create(data);
		return res.status(HttpStatus.CREATED).send(result);
	}

	@Put('/:id')
	public async update(id, data, @Request() req, @Response() res) {
		const row = await this.apiService.update(id, data);
		return res.status(HttpStatus.OK).json(row);
	}

	@Delete('/:id')
	public async delete(@Param('id') id, @Request() req, @Response() res) {
		const row = await this.apiService.delete(id);
		if (row) {
			return res.status(HttpStatus.NO_CONTENT).json();
		} else {
			return res.status(HttpStatus.NOT_FOUND).send();
		}
	}
}
