'use strict';

import { Module, RequestMethod } from '@nestjs/common';
import { PassportModule } from '@nestjs/passport';
import { AuthService } from './service/auth/auth.service';
import { AuthController } from './controller/auth/auth.controller';
import { LocalStrategy } from './local.strategy';
import { UserModule } from '../user/user.module';
//import { BaseModule } from '../core/module/base.module';
import { ApplicationProviders } from './provider/application.providers';
import { JwtModule, JwtService } from '@nestjs/jwt';
import { JwtStrategy } from './jwt.strategy';

import { ChannelModule } from '../channel/channel.module';
//import { OrderModule } from '../order/order.module';

@Module({
	controllers: [
		AuthController
	],
	providers: [
		AuthService,
		LocalStrategy,
		JwtStrategy,		
		...ApplicationProviders,
		
	],
	exports: [
		AuthService				
	],
	imports:[
		PassportModule,
		UserModule,
		ChannelModule,
		JwtModule.register({
            secret: process.env.JWTKEY,
            signOptions: { expiresIn: process.env.TOKEN_EXPIRATION },
        })
	]
})
export class AuthModule  {
	// constructor() {
	// 	super();
	// }
}
