
import { Controller,Get,UseGuards,Post,Request, Headers,Req, Res, HttpStatus, Param } from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import { AuthService } from '../../service/auth/auth.service';
import { SubscriptionService } from '../../../channel/service/subscription.service';

@Controller('auth')
export class AuthController {

    constructor(private readonly authService: AuthService,
        private readonly subscriptionService: SubscriptionService,) { }

    @UseGuards(AuthGuard('local'))
    @Post('login')
    async login(@Request() req) {
        
        return await this.authService.login(req.user);
    }

    @UseGuards(AuthGuard('jwt'))
    @Get()
    async findAll() {
        // get all posts in the db
        return await this.authService.findAll();
    }

    @UseGuards(AuthGuard('jwt'))
    @Get('token')
    async valid() {
        // get all posts in the db
        console.log('------------token------------');
        return await true;
    }

    @UseGuards(AuthGuard('jwt'))
    @Get('logout')
    async logout(){
        return await this.authService.logout();
    }

    @Post('webhook')
	async webhook(@Headers('stripe-signature') signature, @Req() request: any, @Res() response) {
        try {
            return await this.authService.stripeWebhook(signature, request, response);
        } catch (err) {
                
            console.log(err);

            // invalid signature
            response.status(400).end();
            return;
        }
	}

	@Post('webhook_connected')
	async webhookConnected(@Headers('stripe-signature') signature, @Req() request: any, @Res() response) {
        return await this.authService.stripeWebhookConnected(signature, request, response);
	}

    @Post('directlogin/:id')		
	//@Roles(Channel.BACKEND_CHANNEL)
	public async directLogin(@Param('id') id,@Res() res) {
		console.log("directlogin:",id);
		const user = await this.authService.directlogin(id);

		res.status(HttpStatus.OK).json(user);
	}

    @Get('subscriptionPlanAllPeriod/:channelTypeId')
    public async subscriptionPlanAllPeriod(@Param('channelTypeId') channelTypeId, @Request() req, @Res() res) {
		let subscriptionPlanAllPeriod = await this.subscriptionService.subscriptionPlanAllPeriod(channelTypeId);
		return res.status(HttpStatus.OK).send(subscriptionPlanAllPeriod);
	}
}
