import { Injectable,Inject } from '@nestjs/common';
import { Application } from '../../models/Application.entity';
import { JwtService } from '@nestjs/jwt';
import { UserService } from '../../../user/service/user.service';
import * as crypto from 'crypto';
import { ContextService } from '../../../../context/context.service';
import { User } from '../../../user/models/User.entity';
import { StripeV2 } from '../../../../lib/Stripe';
import { PaymentTransaction } from '../../../channel/models/PaymentTransaction.entity';
import { ChannelSubscription } from '../../../channel/models/ChannelSubscription.entity';
import { Sequelize } from 'sequelize-typescript';
import { appconstant } from '../../../../config/app.constant';
import { Subscription } from '../../../channel/models/Subscription.entity';
import { AutoRenewalTracking } from '../../../channel/models/AutoRenewalTracking.entity';
import { SubscriptionService } from '../../../channel/service/subscription.service';
import { Channel } from '../../../channel/models/Channel.entity';

@Injectable()
export class AuthService {
    private  isAdmin =  false; 
    constructor(@Inject('ApplicationRepository') private readonly applicationRepository: typeof Application,
    @Inject('AutoRenewalTrackingRepository') private readonly autoRenewalTrackingRepository: typeof AutoRenewalTracking,
   
    private readonly userService: UserService,
        private readonly jwtService: JwtService,
        private readonly contextService: ContextService,
        private readonly subscriptionService: SubscriptionService
    ) { }

    async validateUser(username: string, pass: string,userType:string) {
        // find if user exist with this email

        let user;
        if(userType==="-user")
        {
            user= await this.userService.findUser(username);

            if (!user)         
                return null;            
        }
        else if(userType==="-admin")
        {     
            user = await this.userService.findAdmin(username);   
            this.isAdmin=true;     
            if (!user) 
                return null;        
        }
        else
            return null;       
        

        // find if user password match
        const match = await this.comparePassword(pass, user.password);
        if (!match) {
            return null;
        }

        // tslint:disable-next-line: no-string-literal
        const { password, ...result } = user['dataValues'];
        return result;
    }

	async directlogin(id)
	{
		
		let user= await this.userService.boUser(id);		
		const token = await this.generateToken(user);
		let result={
			token:token,
			user:user
		}
		return result;
	}

    private async comparePassword(enteredPassword, dbPassword) {
        let hash = crypto.createHmac('sha512', process.env.USER_PASSWORD_SALT_ROUNT);
        hash.update(enteredPassword.trim());
        console.log("lp:",enteredPassword);
            console.log("dtap:",dbPassword);
		let passwordHash = hash.digest('hex');
		if (dbPassword.trim() != passwordHash) {
            console.log("ep:",passwordHash);
            console.log("dp:",dbPassword);
            return false;
            
        }
        
        return true;
    }

    public async login(user) {
        const token = await this.generateToken(user);

        // if(token && this.isAdmin==false && user)
        //     this.contextService.set(`user${user.contactId}`, { userId: user.userId,
        //     contactId: user.contactId,
        //     channelId: user.channelId,
        //     email: user.email,
        //     isActive: user.isActive,
        //     planId: user.channel.planId,
        //     signUpStatusId: user.channel.signUpStatusId,
        //     channelTypeId: user.channel.channelTypeId,
        //     profileTypeId: user.channel.profileTypeId,
        // });
        console.log('setting', user);
        
        this.isAdmin=false;

        return { user, token };
    }    
    
    async findAll(): Promise<Application[]> {
        return await this.applicationRepository.findAll<Application>({
        	attributes: ['id','secretKey'],
        });
        
    }

    private async generateToken(user) {
        const token = await this.jwtService.signAsync(user);
        return token;
    }

    public async logout(){
        return this.contextService.set('user', {});
    }
    async stripeWebhook(signature, request, response) {
		let event = null;
		const stripe = new StripeV2();
		// try { 
		// 	event = await stripe.processWebHook(signature, request);
		// 	console.log('event == ', event);
		// } catch (err) {
		// 	console.log('err == ', err);
        //     // invalid signature
		// 	response.status(400).end();
		// 	return;
		// }

		// console.log(request.body);
		// console.log(request.body.type);
		let intent = null;
		switch (request.body.type) {
			case 'payment_intent.succeeded':
				intent = request.body.data.object;
				console.log("Succeeded:", intent.id);
				//update cart details
				PaymentTransaction.findOne({
					where: {
						paymentIntent: intent.id,
						status: '0'
					}
				}).then((transaction) => {
					console.log("transaction.entity ============>", transaction.entity)
					console.log(" transaction.entityId ============>", transaction.entityId)
					console.log(" transaction.couponCode ============>", transaction.couponCode)
					try {
						if(transaction) {
							transaction.status = "1";
							transaction.transactionNumber = intent.charges.data[0].id;
							// transaction.paymentMethod = intent.charges.data[0].payment_method;
							transaction.cardLastFourDigit = intent.charges.data[0].payment_method_details.card.last4;
							transaction.cardType = intent.charges.data[0].payment_method_details.card.brand;
							transaction.save().then(async () => {
								//let helper = new BaseHelper();
								let user = await User.findOne<User>({ where: { userId: +transaction.updatedBy } });
								let channel = await Channel.findOne<Channel>({ where: { channelId: user.channelId } });
								// user.channel = await helper.buildUserObject(user, this.redisInstance);
								let stripePaymentMethodId = intent.charges.data[0].payment_method;	
	
																
								if(transaction.entity == 'subscription') {

									// get stripe customerId

									
									// let stripeCustomerId = await stripe.addOrUpdateCustomer(channel, intent);

									// console.log('stripeCustomerId == ',stripeCustomerId);
									// if(channel.stripeCustomerId == null) {
									// 	channel.stripeCustomerId = stripeCustomerId;
									// 	channel.save();
									// }	
									//var subscriptionType = await this.subscriptionService.getSubscriptionType(user, transaction.entityId);
						
									// //create channel subscription
									ChannelSubscription.findOne({
										where: {
											paymentTransactionId: transaction.paymentTransactionId
										}
									}).then(async (channelsubscription) => {

										channelsubscription.isActive = "1";
										channelsubscription.isPaid = "1";
										channelsubscription.save();
										
										let oldSubscriptionId = this.subscriptionService.updateOldTransac(channelsubscription.oldChannelSubscriptionId);

										
											
									});

						
									//if (channelSubscription) {
										// profile score
									//	this.channelService.profileScore(user.channelId, user.channel.channelTypeId, user.channel.profileStrength);
									//}
						
									// update company info to elastic search
									//this.channelIndexService.index(user.channelId);
								} else if(transaction.entity == 'subscription_autorenewal') {
									let autoRenewal = await this.autoRenewalTrackingRepository.findByPk(transaction.couponCode);
	
									//create channel subscription
									// var channelSubscription = await this.subscriptionService.createChannelSubscription(
									// 	user,
									// 	transaction.entityId,
									// 	null,
									// 	transaction.paymentTransactionId,
									// 	Subscription.SUBSCRIPTION_TYPE_RENEWAL,
									// 	transaction,
									// 	transaction.cardLastFourDigit,
									// 	user.channel.stripeCustomerId,
									// 	stripePaymentMethodId
									// );
	
									console.log("channel subscription created");
	
									//update auto renewal status
									autoRenewal.update({ 
										statusid: appconstant.AUTO_RENEWAL_TRACKING_STATUS.COMPLETED,
										updatedat: Sequelize.fn('NOW'),
										paymenttransactionid: transaction.paymentTransactionId
									});	
								}
							});
						}
					} catch(e) {
						console.log(e);
					}

				});
				
				break;
			case 'payment_intent.payment_failed':
				intent = event.data.object;
				const message = intent.last_payment_error && intent.last_payment_error.message;
				console.log('Failed:', intent.id, message);
				
				//update cart details
				PaymentTransaction.findOne({
					where: {
						paymentIntent: intent.id,
						status: '0',
						message: message
					}
				}).then((transaction) => {
					if(transaction) {

						console.log("============>", transaction.entity)
						console.log("============>", transaction.entityId)
						console.log("============>", transaction.couponCode)			

						transaction.invoiceno=null;
						transaction.cardLastFourDigit = intent.last_payment_error.payment_method.card.last4;
						transaction.cardType = intent.last_payment_error.payment_method.card.brand;
						transaction.message = message;

						transaction.save().then(async () => {
							if(transaction.entity == 'subscription_autorenewal') {
								let autoRenewal = await this.autoRenewalTrackingRepository.findByPk(transaction.couponCode);
								autoRenewal.update({ statusid: appconstant.AUTO_RENEWAL_TRACKING_STATUS.FAILED });
							}
						});
					}
				});

				break;
		}
		console.log('============= webhook done =====================');
		response.sendStatus(200);
	}

	async stripeWebhookConnected(signature, request, response) {
		let event = null;
		const stripe = new StripeV2();
		try {
			// process intend
			event = await stripe.processWebHook(signature, request.rawBody, true);
		} catch (err) {
			// invalid signature
			response.status(400).end();
			return;
		}

		let intent = null;
		switch (event['type']) {
			case 'payment_intent.succeeded':
				intent = event.data.object;
				console.log("Succeeded:", intent.id);

				//update cart details
				PaymentTransaction.findOne({
					where: {
						paymentIntent: intent.id,
						status: '0'
					}
				}).then((transaction) => {
					if(transaction) {

						console.log("============>", transaction.entity)
						console.log("============>", transaction.entityId)
						console.log("============>", transaction.couponCode)			

						transaction.status = "1";
						transaction.transactionNumber = intent.charges.data[0].id;
						// transaction.paymentMethod = intent.charges.data[0].payment_method;
						transaction.cardLastFourDigit = intent.charges.data[0].payment_method_details.card.last4;
						transaction.cardType = intent.charges.data[0].payment_method_details.card.brand;
						transaction.save().then(async () => {
							//let helper = new BaseHelper();
							let user = await User.findOne<User>({ where: { userId: +transaction.updatedBy } });
							//user.channel = await helper.buildUserObject(user, this.redisInstance);
	
							//this.sampleService.paymentReceived(transaction.entityId, user, transaction.paymentTransactionId);
						});
					}
				});
				
				break;
			case 'payment_intent.payment_failed':
				intent = event.data.object;
				const message = intent.last_payment_error && intent.last_payment_error.message;
				console.log('Failed:', intent.id, message);
				
				//update cart details
				PaymentTransaction.findOne({
					where: {
						paymentIntent: intent.id,
						status: '0'
					}
				}).then((transaction) => {
					if(transaction) {

						console.log("============>", transaction.entity)
						console.log("============>", transaction.entityId)
						console.log("============>", transaction.couponCode)			

						transaction.invoiceno=null;
						transaction.cardLastFourDigit = intent.last_payment_error.payment_method.card.last4;
						transaction.cardType = intent.last_payment_error.payment_method.card.brand;
						transaction.message = message;
	
						transaction.save();
					}
				});

				break;
		}

		response.sendStatus(200);
	}

	
}
