'use strict';


import { Channel } from './../../channel/models/Channel.entity';
import { ContextService } from './../../../context/context.service';
import { Param, Put, Controller, Post, Get, HttpStatus, Response, Query, UseGuards, Res, Req, Body, Delete, ParseIntPipe } from '@nestjs/common';
import { ValidationPipe } from './../../../middleware/pipe/validation.pipe';
import { CartService } from './../service/cart.service';
import { AddToCartProductDto } from './../dto/addCartProduct.dto';
import { AuthException } from './../../../middleware/exception/Auth.exception';
import { ProcessException } from './../../../middleware/exception/Process.exception';
import { Role } from './../../user/models/Role.entity';
import { BaseController } from 'src/core/controller/base.controller';
import { permissions } from 'src/config/permissions.constants';
import { AuthGuard } from '@nestjs/passport';

@Controller('cart')
@UseGuards(AuthGuard('jwt'))
export class IndexController extends BaseController {

	constructor(
		private cart: CartService,
		private context: ContextService
	) {
		super();
	}

	async _validateCartModule(user) {
		if (user.channelTypeId == Channel.CHANNEL_TYPE_VENDOR) {
			throw new AuthException('Not allowed');
		}

		// let currentUserPermissions = this.context.get('dataSession').permissions;
		// let currentUserRole = this.context.get('dataSession').role;

		// if (currentUserRole.roleId != Role.ROLE_ACCOUNT_USER) {
		// 	if (!currentUserPermissions.includes(permissions.PRODUCTS_ADD_TO_CART)) {
		// 		throw new AuthException('Not allowed');
		// 	}
		// }
	}

	@Get('/')
	//@Roles(Channel.FRONTEND_CHANNEL)
	async findAllCart(@Req() req, @Res() res) {
		let user = req.user;

		// user preference
	//	await this._validateCartModule(user);

		let data = await this.cart.getAllMyCart(user.userId);

		res.status(HttpStatus.OK).json(data);
	}

	@Delete('/')
	//@Roles(Channel.FRONTEND_CHANNEL)
	async clearAllCart(@Req() req, @Res() res) {
		let user = req.user;

		// user preference
		//await this._validateCartModule(user);

		await this.cart.clearCart(user.userId);

		res.status(HttpStatus.NO_CONTENT).json({
			"status": true
		});
	}

	@Delete('/:id')
	//@Roles(Channel.FRONTEND_CHANNEL)	
	async clearCart(@Req() req, @Res() res, @Param('id', new ParseIntPipe()) id: number) {
		let user = req.user;

		// user preference
		//await this._validateCartModule(user);

		await this.cart.clearCart(user.userId, id);

		res.status(HttpStatus.NO_CONTENT).json();
	}

	@Post('/item')
	// @Roles(Channel.FRONTEND_CHANNEL)
	async addToCart(@Req() req, @Res() res, @Body(new ValidationPipe()) product: AddToCartProductDto) {
		let user = req.user;
		console.log(user);
		// user preference
		await this._validateCartModule(user);

		if (+product.quantity <= 0) {
			throw new ProcessException('quantity should be atleast one');
		}

		let action = (product.action) ? product.action : 'sum';
		await this.cart.addToCart(product.id, user, product.quantity, action);

		res.status(HttpStatus.CREATED).json({
			"status": true
		});
	}

	@Delete('/item/:id')
	//@Roles(Channel.FRONTEND_CHANNEL)
	async removeProduct(@Req() req, @Res() res, @Param('id', new ParseIntPipe()) id: number) {
		let user = req.user;

		// user preference
		//await this._validateCartModule(user);

		await this.cart.removeItemFromCart(id, user);

		res.status(HttpStatus.NO_CONTENT).json();
	}

	@Get('/item/count')
	//@Roles(Channel.FRONTEND_CHANNEL)
	async getItemCount(@Req() req, @Res() res) {
		let user = req.user;

		// user preference
		await this._validateCartModule(user);

		let result = await this.cart.returnCount(user.userId);

		res.status(HttpStatus.OK).json({
			"status": true,
			data: result
		});
	}
}
