'use strict';

import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, BelongsTo, HasMany, BeforeCreate, BeforeUpdate } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { ProductQuote } from './ProductQuote.entity';
import { Channel } from './../../channel/models/Channel.entity';

var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamps: false, tableName: 'tblcart' } as IDefineOptions;

@Table(tableOptions)
export class Cart extends Model<Cart>
{
	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	cartid: number

	@AllowNull(false)
	@Column(DataType.BIGINT)
	sellerchannelid: number

	@AllowNull(false)
	@Column(DataType.BIGINT)
	buyerchannelid: number

	@AllowNull(false)
	@Column(DataType.BIGINT)
	userid: number

	@Column(DataType.BIGINT)
	currencyid: number

	@AllowNull(false)
	@Column(DataType.DECIMAL(15, 2))
	subtotal: number

	@AllowNull(false)
	@Column(DataType.DECIMAL(15, 2))
	taxcost: number

	@AllowNull(false)
	@Column(DataType.DECIMAL(15, 2))
	discount: number

	@AllowNull(false)
	@Column(DataType.DECIMAL(15, 2))
	grandtotal: number

	@Column
	request_free: string

	@Column(DataType.DATE)
	deletedat: Date;

	@AllowNull(false)
	@Column(DataType.DATE)
	createdat: Date;

	@Column(DataType.DATE)
	updatedat: Date;

	@HasMany(() => ProductQuote, 'cartid')
	items: ProductQuote[]

	@BelongsTo(() => Channel, 'sellerchannelid')
	seller: Channel

	@BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(cart: Cart, options: any) { 
		cart.updatedat = Sequelize.fn('NOW'); 
	}
}
