import { Module } from '@nestjs/common';
import { CommonProviders, CountryProviders } from '../common/provider/common.provider';
import { UserProviders } from '../user/provider/user.provider';
import { ChannelAdminController } from './controller/channel.admin.controller';
import { addressProvider, oldAddressProvider } from './provider/address.provider';
import { settingProvider } from './provider/settting.provider';
import { channelCertificateTypeProvider, channelDetailProvider, ChannelDocumentProvider, channelProductCatProvider, channelProductGroupProvider, channelProvider, ChannelTypeProviders, CompanyMediaProvider, ChannelSubscriptionProviders, channelSocialLinkProvider, channelAwardProvider, channelProductCategorieProvider, channelServiceListProvider, channelSpecialityProvider, channelB2BProvider, ChannelPaymentTermProvider, ChannelPaymentOptionTypeProvider, ChannelSharePreferenceProvider, sharetypeProvider, profileTypeClassificationProvider, channelSuggestionProvider, buyerNeedProvider, buyerReqCategoryProvider, CompanyMediaImagesProvider, GuestRequirementProvider } from './provider/channel.provider';
import { ChannelAdminService } from './service/channel.admin.service';
import { ChannelSettingService } from './service/channel.setting.service';
import { ChannelSettingController } from './controller/channel.settings.controller';
import { ContextService } from '../../context/context.service';
import { ContextModule } from 'src/context/context.module';
import { MediaModule } from '../media/media.module';
import { databaseProviders } from 'src/core/database/database.providers';
import { ProductService } from '../product/service/product.service';
import { brandProvider } from '../product/provider/brand.providers';
import { ProductModule } from '../product/product.module';
import { SubscriptionController } from './controller/subscription.controller';
import { SubscriptionService } from './service/subscription.service';
import {
	SubscriptionProviders,
    SubscriptionPeriodProviders,
    SubscriptionPlanProviders,
    SubscriptionExtraFeatureProviders,
    SubscriptionExtraFeatureCategoryProviders,
    CouponProviders,
    SubscriptionDetailProviders,
    AutoRenewalTrackingProviders } from './provider/subscription.provider';
    
import { ChannelController } from './controller/channel.controller';
import { ChannelService } from './service/channel.service';
import { CurrencyProviders, PaymentTransactionProviders } from './provider/paymentTransaction.provider';
import { ChannelDismissReasonProvider, ChannelJCTProvider, ConnectionStatusTypeProvider, ContactJCTProvider, DismissReasonProvider, EndCustomersB2bProvider, EndCustomersB2bTypeProvider } from './provider/channelHub.provider';
import { SignUpStatusProviders } from '../user/provider/SignUpStatus.providers';
import { productProvider } from '../product/provider/product.providers';
import { ChannelKeyPartnerService } from './service/channelKeyPartner.service';
import { ChannelHubService } from './service/channelHub.service';
import { UserModule } from '../user/user.module';
import { MailerProvider } from '../../mailer/mailer.provider';
import { serviceListProvider } from './provider/serviceList.provider';
import { TradeLocationService } from './service/tradeLocation.service';
import { TradeLocationController } from './controller/tradeLocation.controller';
import { ZoneProvider } from '../common/provider/location.provider';
import { ChannelKeyPartnerController } from './controller/channelKeyPartner.controller';
import { ChannelRelationController } from './controller/channelRelation.controller';
import { AlgorithmModule } from '../../algorithm/algorithm.module';
import { algorithmProvider } from './provider/algorithm.provider';
import { GuestController } from './controller/guest.controller';
import { UserWalletAmountProviders,
    OtherPaymentProviders,
    WalletTransactionProviders,
    WalletUsageTypeProviders } from '../walletRecharge/provider/walletRecharge.providers';
import { PublicChannelController } from './controller/public.channel.controller';
import { PublicChannelService } from './service/public.channel.service';
import { eventProvider } from './provider/event.provider';

@Module({
    controllers:[ChannelController,ChannelAdminController,ChannelSettingController, SubscriptionController, TradeLocationController, ChannelKeyPartnerController, ChannelRelationController, GuestController, PublicChannelController],
    imports: [ContextModule, MediaModule,ProductModule,UserModule, AlgorithmModule],
    providers:[ContextService,
        ChannelKeyPartnerService,
        PublicChannelService,
        ...addressProvider,
        //...brandProvider,
        ...oldAddressProvider,
        ...CurrencyProviders,
        ...channelDetailProvider,
        ...channelProductGroupProvider,
        ...channelProvider,
        ...CountryProviders,
        ...CommonProviders,
        ...UserProviders,
        ...settingProvider,
        ...CompanyMediaProvider,
        ...CompanyMediaImagesProvider,
        ...databaseProviders,
        ...ChannelDocumentProvider,
        ...SubscriptionProviders,
        ...SubscriptionPeriodProviders,
        ...SubscriptionPlanProviders,
        ...SubscriptionExtraFeatureProviders,
        ...SubscriptionExtraFeatureCategoryProviders,
        ...AutoRenewalTrackingProviders,
        ...SubscriptionDetailProviders,
        ...PaymentTransactionProviders,
        ...ChannelSubscriptionProviders,
        ...ConnectionStatusTypeProvider,
        ...ChannelJCTProvider,
        ...ContactJCTProvider,
        ...DismissReasonProvider,
        ...ChannelDismissReasonProvider,
        ...SignUpStatusProviders,
        ...ChannelJCTProvider,
        ...ChannelTypeProviders,
        ...channelCertificateTypeProvider,
        ...productProvider,
        ...CouponProviders,
        ...channelSocialLinkProvider,
        ...channelAwardProvider,
        ...channelProductCategorieProvider,
        ...channelServiceListProvider,
        ...channelSpecialityProvider,
        ...channelB2BProvider,
        ...serviceListProvider,
        ...ChannelPaymentOptionTypeProvider,
        ...ChannelPaymentTermProvider,	    
        ...ChannelSharePreferenceProvider,
        ...sharetypeProvider,
        ...ZoneProvider,
        ...profileTypeClassificationProvider,
        ...channelSuggestionProvider,
        ...algorithmProvider,
        ...UserWalletAmountProviders,
        ...buyerNeedProvider,
        ...buyerReqCategoryProvider,
        ...EndCustomersB2bTypeProvider,
        ...EndCustomersB2bProvider,
        ...GuestRequirementProvider,
        ...eventProvider,
        MailerProvider,
        ChannelAdminService,ChannelSettingService, SubscriptionService,ChannelService, ChannelHubService,TradeLocationService,
    ],
    exports:[ChannelHubService,
            ChannelService,
            SubscriptionService,
            ChannelKeyPartnerService,
            ...addressProvider,
            ...oldAddressProvider,
            ...channelDetailProvider,
            ...channelProductCatProvider,
            ...channelProductGroupProvider,
            ...PaymentTransactionProviders,
            ...ChannelSubscriptionProviders,
            ...ConnectionStatusTypeProvider,
            ...AutoRenewalTrackingProviders,
            ...ChannelJCTProvider,
            ...ContactJCTProvider,
            ...DismissReasonProvider,
            ...ChannelDismissReasonProvider,
            ...CouponProviders,
            ...sharetypeProvider,
            ...channelSuggestionProvider,
            ...algorithmProvider,
            ...UserWalletAmountProviders,
            ...GuestRequirementProvider,
            ...eventProvider,
            // ...EndCustomersB2bProvider,
            MailerProvider
        ]
})
export class ChannelModule {}

