import { Body, Controller, Get, HttpStatus, Inject, Param, ParseIntPipe, Patch, Post, Put, Query, Req, Res, UseGuards, ValidationPipe } from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import { ApiQuery } from '@nestjs/swagger';
import {Op, QueryTypes } from "sequelize";
import { appconstant } from 'src/config';
import { ContextService } from 'src/context/context.service';
import { ProcessException } from 'src/middleware/exception/Process.exception';
import { ChannelAdminService } from '../service/channel.admin.service';
import { ChannelSettingService } from '../service/channel.setting.service';
import { UserService } from '../../user/service/user.service';
import { CompanyUpdateDto } from '../dto/company.update.dto';
import { profile } from 'console';
import { ValidationException } from 'src/middleware/exception/Validation.exception';
import { AdminCouponAddDto } from '../dto/adminCoupon.dto';
import { SubscriptionService } from '../service/subscription.service';
import { SubscriptionFilterBODto } from '../dto/subscriptionFilter.dto';
import { TradeLocationService } from '../service/tradeLocation.service';
import { UpdateUserDto } from 'src/modules/user/dto/updateUser.dto';
import { BoUpdateSubscriptionDto } from '../dto/updateSubscription.dto';
import { ChannelService } from '../service/channel.service';
import { ChannelDetailOutBound } from '../transformation/channelDetailOutBound';
import { ProductService } from 'src/modules/product/service/product.service';

const fetch = require("node-fetch");

// @UseGuards(AuthGuard('jwt'))
@Controller('channelAdmin')
export class ChannelAdminController {
    private channelDetailOutBound = new ChannelDetailOutBound();
constructor(
    private channelAdminService:ChannelAdminService,
    private channelSettingService:ChannelSettingService,
    private channelService: ChannelService,
    private readonly subscriptionService: SubscriptionService,
    private userService: UserService,
    private tradeLocationService: TradeLocationService,
    private productService: ProductService,
    @Inject('SequelizeInstance') private readonly sequelizeInstance,
    
    private context: ContextService,) {}
    @Post('pendingApproval')
    async pendingApproval(@Body() data, @Req() req,@Res() res){
        let channel = await this.channelAdminService.getPendingApprovelChannels(data);
		return res.status(HttpStatus.OK).send(channel);
    }

    @Post('channelSubscription')	
	//@Roles(Channel.BACKEND_CHANNEL)
	public async channelSubscriptionList(@Req() req, @Res() res, @Body(new ValidationPipe()) SubscriptionFilterDto: SubscriptionFilterBODto) {

        //console.log("sub:",req.query.);
		let sort = (req.query.sort && ['asc', 'desc'].includes(req.query.sort.toLowerCase())) ? req.query.sort.toLowerCase() : 'desc';
        sort='desc';
        //console.log("sort:",sort);
        let pagination={
            limit:req.query.limit,
            offset:req.query.limit*req.query.page,
        }
		let subscriptions = await this.subscriptionService.getAllList(pagination,req.query.search, SubscriptionFilterDto, sort);

		res.status(HttpStatus.OK).send(subscriptions
			//this.paginationTransform.make(subscriptions.rows, pagination, subscriptions.count, subscriptions.aggregations)
		);
	}

    @Get('pendingChannelsList/:userId')
    async pendingChannelsList(@Param('userId') userId,@Req() req,@Res() res){
        let channel = await this.channelAdminService.pendingChannelsList(userId)
        if(channel){
            return res.status(HttpStatus.OK).send(channel);
        }
        throw new ProcessException('channel:channelApproval:channelNotFound');
    }

    //oldChannel listing start
    @Get('oldChannelsList/:userId')
    async oldChannelsList(@Param('userId') userId,@Req() req,@Res() res){
        let channel = await this.channelAdminService.oldChannelsList(userId)
        if(channel){
            return res.status(HttpStatus.OK).send(channel);
        }
        throw new ProcessException('channel:channelApproval:channelNotFound');
    }

    @Post('oldChannels')
    async oldChannels(@Body() data, @Req() req,@Res() res){
        let channel = await this.channelAdminService.oldChannels(data);
		return res.status(HttpStatus.OK).send(channel);
    }
    //oldChannel listing end

    @Put('channelApproval')
    async channelApproval(@Body() data, @Req() req,@Res() res){
        let channel = await this.channelAdminService.channelApproval(data);
        if(channel){
            return res.status(HttpStatus.OK).send(true);
        }
        throw new ProcessException('channel:channelApproval:channelNotFound');
    }

    @Post('getFilterCountry')
    async getFilterCountry(@Body() data, @Req() req,@Res() res){
        let channel = await this.channelAdminService.getFilterCountry(data)
		return res.status(HttpStatus.OK).send(channel);
    }

    @Post('getProductCat')
    async getProductCat(@Body() data, @Res() res){
        let productCat = await this.channelAdminService.getProductCat(data)
		return res.status(HttpStatus.OK).send(productCat);
    }

    @Get('getSubscriptionPlan')
    async getSubscriptionPlan(@Res() res){
        let planList = await this.channelAdminService.getSubscriptionPlan();
		return res.status(HttpStatus.OK).send(planList);
    }


    @Post('introVideoUpload')
	public async introVideoUpload(@Body() OldIntro, @Res() res) {		
	// let user = this.context.get('user');
	let doc_id = await this.channelAdminService.saveUploadedVideo(OldIntro, OldIntro.user);
		if(doc_id){
			res.status(HttpStatus.OK).send(doc_id);
		}
	}

	@Get('medialibraryByContact')
	public async medialibraryByContact(@Req() req, @Res() res) {		
    // let user = this.context.get('user');

    console.log('user',req.query);
	let userMedia = await this.channelAdminService.medialibraryByContact(req.query);
		if(userMedia){
			res.status(HttpStatus.OK).send(userMedia);
		}
    }
    @Post('deleteMediaVideo')
	public async deleteMediaVideo(@Res() res, @Body() data: any) {		
		let doc_id = await this.channelAdminService.deleteMediaVideo(data);
		res.status(HttpStatus.OK).send(doc_id);
    }
    @Post('deleteCompanyLogo')
	public async deleteCompanyLogo(@Res() res, @Body() data: any) {		
		let doc_id = await this.channelAdminService.deleteCompanyLogo(data);
		res.status(HttpStatus.OK).send(doc_id);
	}

    // //company profile
    // @Post('company')
    // public async company(@Req() req, @Res() res, @Body() data) {
    // let user=data;
    // console.log('user', user);
    // let result = await this.channelAdminService.getCompanyProfile(user);
    // let channelPdf = await this.channelSettingService.getCompanyDocument(user.channelId);
    
    // let channelPdfList = [];
    // channelPdf.map(x=>{
    //     let dpath = x.document.documentPath;
    //     channelPdfList.push({documentId: x.documentId, name: x.name?x.name: dpath.split('/').pop().split('#')[0].split('?')[0], channelDocumentId: x.channelDocumentId,document:{documentUrl: dpath.includes('/product/broucher')?dpath:x.document.documentUrl} });
    // })

    // res.status(HttpStatus.OK).send(Object.assign(result, {channelPdf: channelPdfList}));
    // }

    @Post('getInitialProfile')	
    public async getInitialProfile(@Req() req,@Res() res, @Body() data) {
        
        let channelId = data.channelId;
        let userId = data.userId;
        let user = await this.userService.findUserById(userId, channelId);

        let result = await this.channelSettingService.getInitialProfile(user);
         if (result) {
            res.status(HttpStatus.OK).json(result);
        } 
    }    

    //trade info
    @Post('tradeinfo')
    public async tradeinfo(@Req() req, @Res() res, @Body() data) {
    let channelId = data.channelId;
    let userId = data.userId;
    let user = await this.userService.findUserById(userId, channelId);

    let result = await this.channelSettingService.getTradeInfo(user);

    let brandList=[];let productList=[];let profileList=[];

    
    if(result.length>0)
    {
        result.map(async x => {

        x.channelBrand?.map(x => {
            let brand={
                id:x.brandId,
                name:x.brand.brandName,
                documentpath:process.env.CDN_URL+x.brand.document.documentPath,
                isNew:0
            }
            brandList.push(brand);
        })


        let cat=x.channelProductCategorie?.map(x => x.productCategorie);
        let group=x.channelProductGroup?.map(x => x.productGroup);
        


    
        group?.map(g => {
            cat?.map(c => {
                if(c.dataValues.groupid==g.dataValues.groupid)
                {
                    //console.log(c);
                    
                    let product={
                        childid:c.dataValues.childid,
                        childname:c.dataValues.childname,
                        groupid:c.dataValues.groupid,
                        groupname:g.dataValues.groupname,
                        status:true
                    }
                    productList.push(product);
                }
                    
            })
        })

    })

    //console.log("result:",result[0]);
    
    let pro=result[0].dataValues.channelCustomerProfile?.map(x => +x.profileTypeId);
    let clssifiid=result[0].dataValues.channelCustomerProfile?.map(x => +x.profileTypeClassificationId);

        let query='select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
        +'when ptc."channelTypeId"=2 then \'vendor\' end as groupname,pt."profileType" as childname,'
        +'pt."profileTypeId" as childId,cast(false as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" and pt."profileTypeId" in(:ptype) ';

        if(clssifiid.length)
            query += 'and  ptc."profileTypeClassificationId" in(:clssifiid) ';
        
        profileList=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,           
            replacements: {
                typeId: user.channelTypeId,
                clssifiid: clssifiid,
                ptype: (user.channelTypeId==4 || user.channelTypeId==5) ? result[0].profileTypeId : pro,     
            }
            }).then(results => {     
                //console.log("result:",pro,req.user.channelTypeId,results);   
               return results;
            });

    let keyRetailerOptions = [];
    let keyRetailerValues = [];

    let keyDistributorOptions = [];
    let keyDistributorValues = [];

    if(result.keyPartners.retailer){
        if(result.keyPartners.retailer.channel?.length)
        {
            result.keyPartners.retailer.channel.map(x => {
                let brand={
                    id:x.channelId,
                    name:x.companyName,
                    documentpath:process.env.CDN_URL+x.companylogo?.documentPath,
                    isNew:0
                }
                keyRetailerOptions.push(Object.assign(x,brand));
            })
        }
    }

    let keyretailer = await this.channelSettingService.getKeyRetailers(+user.channelId, appconstant.CHANNEL_TYPE_RETAILER);
    if(keyretailer?.length){
        keyretailer.map(x => {
            let logo = x.active? x.active?.logo?.documentPath : x.old? x.old.logo?.documentPath : x.new? x.new.logo?.logo_name : '';
            let companyName = x.active? x.active.company_name : x.old? x.old.company_name : x.new? x.new.company_name : '';
            let id = x.active? x.active.id : x.old? x.old.id : x.new? x.new.id : '';
            let brand={
                id:id,
                name:companyName,
                documentpath:process.env.CDN_URL+logo,
                isNew:0
            }
            keyRetailerValues.push(Object.assign(x,brand));
        })
    }


    if(result.keyPartners.distributor){
        if(result.keyPartners.distributor.channel?.length)
        {
            result.keyPartners.distributor.channel.map(x => {
                let brand={
                    id:x.channelId,
                    name:x.companyName,
                    documentpath:process.env.CDN_URL+x.companylogo?.documentPath,
                    isNew:0
                }
                keyDistributorOptions.push(Object.assign(x,brand));
            })
        }

    }    

    let keyDistributor = await this.channelSettingService.getKeyRetailers(+user.channelId, appconstant.CHANNEL_TYPE_DISTRIBUTOR)
    if(keyDistributor?.length){
        
            keyDistributor.map(x => {
                let logo = x.active? x.active?.logo?.documentPath : x.old? x.old.logo?.documentPath : x.new? x.new.logo?.logo_name : '';
                let companyName = x.active? x.active.company_name : x.old? x.old.company_name : x.new? x.new.company_name : '';
                let id = x.active? x.active.id : x.old? x.old.id : x.new? x.new.id : '';
                let brand={
                    id:id,
                    name:companyName,
                    documentpath:process.env.CDN_URL+logo,
                    isNew:0
                }
                keyDistributorValues.push(Object.assign(x,brand));
            })

    }

    let profileType = '';

    if(result[0].dataValues.profileTypeId){
    query='select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
        +'when ptc."channelTypeId"=2 then \'vendor\' end as groupname,pt."profileType" as childname,'
        +'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" where ptc."channelTypeId" in(:typeId) and ptc."profileTypeId"=:profileTypeId;';
      
         profileType=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,           
            replacements: {
                typeId: user.channelTypeId,
                profileTypeId: result[0].dataValues.profileTypeId
            }
            }).then(results => {        
               return results;
            });
        }

    let data={
        brand:brandList,
        product:productList,
        profile:profileList,
        empCnt:result[0].dataValues.channelDetail.empCntRange?result[0].dataValues.channelDetail.empCntRange:result[0].dataValues.channelDetail.empCnt,
        no_of_skus:result[0].dataValues.channelDetail.no_of_skus,
        turnoverRange:result[0].dataValues.channelDetail.turnoverRange,
        revD:result[0].dataValues.channelDetail.revD,
        revI:result[0].dataValues.channelDetail.revI,
        startup:result[0].dataValues.channelDetail.startup,
        no_of_distributors:result[0].dataValues.channelDetail.no_of_distributors,
        no_of_retailers:result[0].dataValues.channelDetail.no_of_retailers,
        maturitylevel: result[0].dataValues.channelDetail.maturitylevel,
        profileTypeId: result[0].dataValues.profileTypeId,
        profileType: profileType,
        onlineStore: result[0].dataValues.channelDetail.turnoverPhyStore,
        turnoverOnStore: result[0].dataValues.channelDetail.turnoverOnStore,
        storeCount: result[0].dataValues.channelDetail.storeCount,
        sales: result[0].dataValues.channelDetail.empCntDSale,
        marketing: result[0].dataValues.channelDetail.empCntISale,
        technical: result[0].dataValues.channelDetail.empCntDMar,
        purchase: result[0].dataValues.channelDetail.empCntIMar,
        keyPartners: {keyRetailerOptions, keyDistributorOptions},
        channelKeyRetailer: keyRetailerValues,
		channelKeyDistributor: keyDistributorValues,
        keyretailers: result[0].dataValues.channelDetail.keyretailers,
        keydistributors: result[0].dataValues.channelDetail.keydistributors,
        serviceListOptions: result.serviceListOptions,
        serviceListValues: result[0].dataValues.serviceList?result[0].dataValues.serviceList.map(x => ({id: x.serviceListId, name: x.serviceList})):[],
        specialityOptions: result.specialityOptions,
        channelSpeciality: result[0].dataValues.channelSpeciality,
        isFrenchTech: result[0].dataValues.isFrenchTech,
        isCompanyPublic: result[0].dataValues.isCompanyPublic
    }

    res.status(HttpStatus.OK).send(data);
    }
    else{
        res.status(HttpStatus.OK).send(data);
    }
        
}


//trade location
@Post('tradelocation')   
public async tradelocation(@Req() req, @Res() res, @Body() data) {    
    // let user=data;

    let userId = data.userId;
    let channelId = data.channelId;

    let user = await this.userService.findUserById(userId, channelId);

    let result = await this.channelSettingService.getTradeLocation(user);    
    let targetArray=[];let sellingArray=[];
    
    let targetids=[];let sellingids=[];
    if(result)
    result[0].channelLocation.map(x => {
     

        if(x.locationType=='TARGET' && user.channelTypeId!=4)
        {
            targetids.push(x.RegionCountryJCT.region.regionId);

            let location={
                regionCountryJCTId:x.RegionCountryJCT.regionCountryJCTId,
                groupid:x.RegionCountryJCT.region.regionId,
                groupname:x.RegionCountryJCT.region.region,
                childId:x.RegionCountryJCT.country.countryId,
                childname:x.RegionCountryJCT.country.country,
                status:true
            }
            targetArray.push(location);
        }
        if(x.locationType=='SELLING' && user.channelTypeId!=4)
        {
            sellingids.push(x.RegionCountryJCT.region.regionId);
            let location={
                regionCountryJCTId:x.RegionCountryJCT.regionCountryJCTId,
                groupid:x.RegionCountryJCT.region.regionId,
                groupname:x.RegionCountryJCT.region.region,
                childId:x.RegionCountryJCT.country.countryId,
                childname:x.RegionCountryJCT.country.country,
                status:true
            }
            sellingArray.push(location);
        }
        if(x.locationType=='RETAIL' && user.channelTypeId==4)
        {
            sellingids.push(x.RegionCountryJCT.region.regionId);
            let location={
                regionCountryJCTId:x.RegionCountryJCT.regionCountryJCTId,
                groupid:x.RegionCountryJCT.region.regionId,
                groupname:x.RegionCountryJCT.region.region,
                childId:x.RegionCountryJCT.country.countryId,
                childname:x.RegionCountryJCT.country.country,
                status:true
            }
            sellingArray.push(location);
        }
    })

    let zone=await this.sequelizeInstance.query('select "zoneId","regionId" from "tblZoneRegionJCT" ' , {
        type: QueryTypes.SELECT,
        raw: true,
        }).then(x=>{            
            let zones={
                tzone:x.filter(ch => {return targetids.includes(ch.regionId);}),
                szone:x.filter(ch => {return sellingids.includes(ch.regionId);})
            }
            return zones;
        })

    let finalData={
        zone:zone,
        targetLocation:targetArray,
        sellingLocation:sellingArray
    }    
    
    res.status(HttpStatus.OK).send(finalData);
}

//get user
@Post('getuser')
public async getUsers(@Req() req, @Res() res, @Body() data) {    
    let userId = data.userId;
    let channelId = data.channelId;

    let user = await this.userService.findUserById(userId, channelId);
    let result = await this.channelSettingService.getUsers(user);    
    
    res.status(HttpStatus.OK).send(result);
}

//default user
@Post('defaultcontact')	
	public async getDefaultContact(@Res() res,@Req() req, @Body() data) {
        let user=data;
        let result=await this.userService.getDefaultContact(user.channelId);		
		return res.status(HttpStatus.OK).send(result);
    }

//all contacts
@Post('allcontacts')	
public async getAllContacts(@Res() res,@Req() req, @Body() data) {
    let user=data;
    let result=await this.userService.getAllContacts(user.channelId);		
    return res.status(HttpStatus.OK).send(result);
}

//get settings
@Post('settings')	
	public async settings(@Res() res,@Req() req, @Body() data) {

        let userId = data.userId;
        let channelId = data.channelId;

        let user = await this.userService.findUserById(userId, channelId);

		let settings = await this.channelSettingService.settings(user);
		if (settings) {
			return res.status(HttpStatus.OK).send(settings);
		}
		//throw new ProcessException('channel:settings:settingsNotFound');
	}

    @Post('profileAndSpeciality')
    public async getProfileAndSpeciality(@Req() req, @Res() res, @Body() data){
        let channelId = data.channelId;
        let result = await this.channelSettingService.getProfileAndSpeciality(channelId);
    
        let final = {};
        if(result){
    
            let profileType = '';
    
            if(result[0].dataValues.profileTypeId){
            let query='select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
            +'when ptc."channelTypeId"=2 then \'vendor\' end as groupname,pt."profileType" as childname,'
            +'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" where ptc."channelTypeId" in(:typeId) and ptc."profileTypeId"=:profileTypeId;';
          
             profileType=await this.sequelizeInstance.query(query, {
                type: QueryTypes.SELECT,           
                replacements: {
                    typeId: data.channelTypeId,
                    profileTypeId: result[0].dataValues.profileTypeId
                }
                }).then(results => {        
                   return results;
                });
            }
    
            final = {
            specialityOptions: result.specialityOptions,
            channelSpeciality: result[0].dataValues.channelSpeciality,
            profileTypeId: result[0].dataValues.profileTypeId,
            profileType: profileType,
            isFrenchTech: result[0].dataValues.isFrenchTech
            }
        }
    
        res.status(HttpStatus.OK).json(final);
    }
    
    

//default contact settings
@Post('setDefaultContact/:contactId')	
public async setDefaultContact(@Param('contactId') contactId,@Res() res,@Req() req, @Body() data) {
    let userId = data.userId;
    let channelId = data.channelId;

    let user = await this.userService.findUserById(userId, channelId);
    let result=await this.userService.setDefaultContact(user.channelId,contactId);		
    return res.status(HttpStatus.OK).send(result);
}

//updating company profile
@Post('updateCompany')
async update_Company_Profile(@Body(new ValidationPipe()) profileData:any, @Req() req,@Res() res){

        let userId = profileData.user.userId;
        let channelId = profileData.user.channelId;

        let user = await this.userService.findUserById(userId, channelId);

        //console.log("profileData:",profileData);
        // let result=await this.channelSettingService.update_Company_Profile(Object.assign(profileData, {contactId: req.user.contactId, channelId: req.user.channelId}));
        let result=await this.channelSettingService.updateUserCompanyProfile(profileData, user);
        
        return res.status(HttpStatus.OK).send(result);
}

//updating company settings for distributor from bo
@Post('update_company_profile')
public async update_company_profile(@Req() req, @Res() res, @Body() data){

    let userId = data.user.userId;
    let channelId = data.user.channelId;

    let user = await this.userService.findUserById(userId, channelId);

    // let user = req.user;
    let result = await this.channelSettingService.update_company_new(user, data);

    console.log('data', data);

    if(result)
        return res.status(HttpStatus.OK).send('OK');
    else    
        return res.status(HttpStatus.BAD_REQUEST);
}

//updating Trade Info
@Post('updateTradeInfo')
async update_Trade_Info(@Body(new ValidationPipe()) profileData:any, @Req() req,@Res() res){

        let userId = profileData.user.userId;
        let channelId = profileData.user.channelId;
        let user = await this.userService.findUserById(userId, channelId);

        // let query = 'select "contactId" from "tblContact" where "channelId"=:channelId and "isDefaultContact"=\'1\'';
        // let contact = await this.sequelizeInstance.query(query, {
        //     type: QueryTypes.SELECT,           
        //     replacements: {
        //         channelId: profileData.user.channelId,
        //     }
        // }
        // ).then(res => {
        //     return res[0].contactId;
        // });

        let result=await this.channelSettingService.updatetradeInformation(profileData, user);
        return res.status(HttpStatus.OK).send(result);
}

//trade location
@Put('updateChannelLocation')		
	public async updateChannelLocation1(@Body(new ValidationPipe()) channelLocationDto: any, @Req() req, @Res() res) {
		let user = channelLocationDto.user;
		let subscriptionFeatureKeys = this.context.get('subscriptionFeatures');

        let query = 'select "contactId" from "tblContact" where "channelId"=:channelId and "isDefaultContact"=\'1\'';
        let contact = await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,           
            replacements: {
                channelId: user.channelId,
            }
        }
        ).then(res => {
            return res[0].contactId;
        });


		// trade locations
		//await this._validateTradeLocationProfile();

		let nonsuggestion=[];let count;
		let userId = user.userId;
			let channelId = user.channelId;
			let contactId =user.contactId;
			let channelTypeId = user.channel.channelTypeId;
			

        let channelLocation = await this.channelSettingService.updateChannelLocation(channelLocationDto, Object.assign(user, {contactId: contact}), subscriptionFeatureKeys);
        
		// if (channelLocation) {
		// 	this.channelService.profileScore(user.channelId, user.channel.channelTypeId, user.channel.profileStrength).then(() => {
		// 		// update company info to elastic search
		// 		this.channelIndexService.index(user.channelId);
		// 	});

			//this.tradeLocationService.getNonSuggestion(this.context.get('user').channelId,this.context.get('dataSession').plan,contactId,userId,channelTypeId);		

			// audit log
			// if (user.channel.signUpStatusId == appconstant.CHANNEL_APPROVED && this.context.get('real-user')) {
			// 	this.auditLog.addAuditLog((channelLocationDto && channelLocationDto.reason) ? channelLocationDto.reason : null, auditlog.MODULE.BO_TRADE_LOCATION, req);
			// }

			return res.status(HttpStatus.OK).send(channelLocation);
		
	}

    //settings
    @Post('updatesettings')	
	public async updateSettings(@Req() req, @Res() res, @Body(new ValidationPipe()) settingsDto: any) {
        
        let userId = settingsDto.user.userId;
        let channelId = settingsDto.user.channelId;
        let user = await this.userService.findUserById(userId, channelId);

        let query = 'select "contactId" from "tblContact" where "channelId"=:channelId and "isDefaultContact"=\'1\'';
        let contact = await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,           
            replacements: {
                channelId: user.channelId,
            }
        }
        ).then(res => {
            return res[0].contactId;
        });

		    // update company info to elastic search
			//this.context.get('user');
		// trade locations
		//await this._validateSettingProfile();

		let updateSettings = await this.channelSettingService.updateSettings(settingsDto, Object.assign(user, {contactId: contact}));
		if (updateSettings) {

			//this.channelIndexService.index(this.context.get('user').channelId);			
			
			//this.getNonSuggestion(user.channelId,user.channel.planId,user.contactId,user.userId,user.channel.channelTypeId);
		   
			// audit log
			// if (this.context.get('real-user')) {
			// 	this.auditLog.addAuditLog((settingsDto && settingsDto.reason) ? settingsDto.reason : null, auditlog.MODULE.BO_SETTINGS, req);
			// }

			return res.status(HttpStatus.OK).send('ok');
		}
		//throw new ProcessException('channel:settings:settingsNotUpdated');
	}


    //update channel
    @Post('updateChannel')
    public async update_channel_profile(@Req() req, @Res() res, @Body() data){
    let userId = data.user.userId;
    let channelId = data.user.channelId;
    let user = await this.userService.findUserById(userId, channelId);
    let result = await this.channelSettingService.save_channel_info(data, {}, user);
    if(result)
        return res.status(HttpStatus.OK).send('OK');
    else    
        return res.status(HttpStatus.BAD_REQUEST);
    }

    // //getting reporting to contacts
    // @Post('reporting')
    // async reporting(@Body(new ValidationPipe()) profileData:any, @Req() req,@Res() res){
    //     let result=await this.channelSettingService.reportingusers(profileData);
    //     return res.status(HttpStatus.OK).send(result);
    // }


    //user tabs
    @Post('add')	
	public async addNewUser(@Res() res,@Req() req, @Body(new ValidationPipe()) addUserDto: any) {

		let nonsuggestion=[];let count;
		// let user = addUserDto.user;

        let userId = addUserDto.user.userId;
        let channelId = addUserDto.user.channelId;
        let user = await this.userService.findUserById(userId, channelId);
			
		// user magt preference
		//await this._validateUserMgmtProfile();

		//await this._validateSubscriptionFeatures();
		
		try{
		let result = await this.userService.addUser(addUserDto, user.channelId, user.showId, user.userId, user.channelTypeId);
		//console.log(result);
		//update index
		//this.ChannelIndexService.index(user.channelId);		

		//if(Object.keys(result.userId).length != 0)
	

		let userId=result.userId;
		let channelId = user.channelId;
		let contactId = result.contactId;
		let channelTypeId = user.channelTypeId;

		// //update rank and custom_sort_order table
		// await this.sequelizeInstance.query('select * from "setrank"(' + contactId + ')', {
		// 	type: Sequelize.QueryTypes.SELECT
		// });

		//this.userService.getNonSuggestion(channelId,this.context.get('dataSession').plan,contactId,userId,channelTypeId);


		// if(result.userId != 0){
		// 	let hubspotCompanyId = user.channel.hubspotCompanyId;
		// 	console.log("end");

			
		// 	let contactData = await this.getHubspotContact(addUserDto, user.contactId, user.userId, user.channelId, hubspotCompanyId);
		// }
		
		res.status(HttpStatus.CREATED).send({
			created: result
		});
	}catch(e)
	{
		console.log("adduser:",e);
	}
    }

//location options for myprofile bo
@Post('getUserProfile')	
    public async getUserProfile(@Req() req , @Res() res, @Body() data) {		

        let result = await this.channelSettingService.getUserProfile(data.channelId, data.channelTypeId);
         if (result) {
            res.status(HttpStatus.OK).json(result);
        } 
}    


//contactlocation for user edit
    @Get('getContactLocation/:contactId/:channelTypeId')	
    public async getContactLocation(@Param('contactId') contactId, @Param('channelTypeId') channelTypeId, @Req() req,@Res() res) {		
        let user = req.user;
        let result;
        result = await this.channelSettingService.getContactLocation(user);
        //console.log('reult',result);
         if (result) {
            let targetArray = result.contactLocation.filter(x => x.locationType == 'TARGET');
            let sellingArray = user.channelTypeId == 4  ? result.contactLocation.filter(x => x.locationType == 'RETAIL') :result.contactLocation.filter(x => x.locationType == 'SELLING');
            let data = {
                targetArray : targetArray,
                sellingArray : sellingArray
            }
            res.status(HttpStatus.OK).json(data);
        } 
}    

//Get Channel Coupon List
@Post('channelCoupons')
//@Roles(Channel.BACKEND_CHANNEL)
public async getCouponList(@Req() req, @Res() res,@Body(new ValidationPipe()) couponFilterDto: any) {//CouponFilterDto) {

    let result = await this.channelAdminService.getCouponList(couponFilterDto);
    //let output: any = this.paginationTransform.make(result.rows, pagination, result.count, result.aggregation);

    res.status(HttpStatus.OK).json(result);
}

	// Get Channel Coupon By Id
	@Get('channelCoupons/:id')
	//@Roles(Channel.BACKEND_CHANNEL)
	public async getCouponByID(@Param('id') id, @Req() req, @Res() res) {
		let result = await this.channelAdminService.getCouponByID(id);
		if (result)
			return res.status(HttpStatus.OK).json(result);
		else
			throw new ProcessException('channel:channelCoupons:channelCouponsNotFound');
	}


//Add coupon list
@Post('addCoupon')
//@Roles(Channel.BACKEND_CHANNEL)
public async AddChannelCoupon(@Req() req, @Res() res, @Body(new ValidationPipe()) adminCouponAddDto: AdminCouponAddDto) {
    let result = await this.channelAdminService.addChannelCoupon(adminCouponAddDto);
    if (result) {
        return res.status(HttpStatus.CREATED).send();
    } else {
        throw new ProcessException('channel:channelCoupons:channelCouponsNotFound');
    }
}

//UPdate coupon list
@Put('channelCoupons/:id')
//@Roles(Channel.BACKEND_CHANNEL)
public async UpdateChannelCoupon(@Param('id') id, @Req() req, @Res() res, @Body(new ValidationPipe()) adminCouponAddDto: AdminCouponAddDto) {
    let result = await this.channelAdminService.UpdateChannelCoupon(id, adminCouponAddDto);
    if (result) {
        return res.status(HttpStatus.OK).send();
    } else {
        throw new ProcessException('channel:channelCoupons:channelCouponsNotFound');
    }
}

//keypartnercheck

@Get('channelKeyPartnerCheck')
public async existingKeyRetailerCheck(@Req() req, @Res() res, @Query('KeyRetailer') KeyRetailer) {

let result = await this.channelSettingService.existingKeyRetailerCheck(decodeURI(KeyRetailer), +req.query.CountryId, +req.query.channelId, +req.query.partner_type, req.query.partner_id ? +req.query.partner_id : null);
res.status(HttpStatus.OK).send(result);

}

@Post('searchRegion')
public async searchRegion(@Body(new ValidationPipe()) searchRegionDto: any, @Req() req, @Res() res) {
    let user = searchRegionDto.user;
    let searchRegion = await this.channelSettingService.searchRegion(searchRegionDto, user);

    let finaldata = [];

    if(searchRegion){
        searchRegion.map(x => {
            let location={
                regionCountryJCTId:x.regionCountryJCTId,
                groupid:x.region.regionId,
                groupname:x.region.region,
                childId:x.country.countryId,
                childname:x.country.country,
                status:false
            }
            finaldata.push(location);
    })
    }
    
    return res.status(HttpStatus.OK).send(finaldata);
}    

@Get('buyerNeed')
public async getBuyerNeed(@Req() req, @Res() res, @Body() reqneed: any) {		
let result=await this.channelAdminService.getBuyerNeed(req.query);
res.status(HttpStatus.OK).send(result);
}

@Post('request_Intro/:channelid')
public async getAllRequestIntro(@Param('channelid') channelid,@Req() req, @Res() res) {
	let user = this.context.get('user');
	let result = await this.channelAdminService.getRequestIntro(+channelid);
	res.status(HttpStatus.OK).send({
		result: result[0][0].requestintro });
}

@Post('approveChannel')	
	public async approveChannel(@Req() req, @Res() res, @Body(new ValidationPipe()) updateChannelDto: any) {
		let user = await this.userService.getUser(updateChannelDto.retailerUserId, updateChannelDto.retailerChannelid);

			 let updateChannel = await this.channelAdminService.updateChannelConnect(updateChannelDto.sellerChannelid, updateChannelDto, user, true);
			if (updateChannel) {

			// 	this.channelService.profileScore(user.channelId, user.channel.channelTypeId, user.channel.profileStrength);
				//this.channelIndexService.hubspotCompanyUpdate(user.channelId, user);
				//ChannelIndexService.updateChannelRelationCount(+user.channelId);
				return res.status(HttpStatus.OK).send({status:true});
			}
			return res.status(HttpStatus.OK).send({status:true});
	 }	

@Get('getPublicCompanyFlags/:channelId')
public async getPublicCompanyFlages(@Req() req, @Res() res, @Param('channelId') channelId){
    let flags = await this.channelAdminService.getPublicCompanyFlags(channelId);

    return res.status(HttpStatus.OK).send(flags);
}

@Post('setCompanyPublicFlags')
public async setCompanyPublicFlags(@Req() Req, @Res() res, @Body() data){
    let result = await this.channelAdminService.setCompanyPublicFlags(data);

    return res.status(HttpStatus.OK).send({status: true});
}

//bo company settings related services

@Post('getChannelDetail')	
    public async getChannelDetail(@Req() req, @Res() res, @Body() data) {		
        let channelId = data.channelId; 
        let userId = data.userId;
        let user = await this.userService.findUserById(userId, channelId);
        
        let result = await this.channelSettingService.getChannelDetail(user.channelId);
         if (result) {
            res.status(HttpStatus.OK).json(result);
        } 
}

@Post('company')
public async company(@Req() req, @Res() res, @Body() data) {

    let channelId = data.channelId; 
    let userId = data.userId;
    let user = await this.userService.findUserById(userId, channelId);

    console.log('user', user);
    
    let result = await this.channelSettingService.getCompanyProfile(user);
    let channelPdf = await this.channelSettingService.getCompanyDocument(user.channelId);
    let chImages = await this.channelSettingService.getCompanyImages(user.channelId);
    let channelImages=[];
    if(chImages.length>0){
    chImages.map(x=>{
        channelImages.push({documentId: x.documentId, companyMediaId: x.companyMediaId, documentUrl:x.document.documentUrl });
    })
    }
    console.log('channelPdf',channelPdf);
    let channelPdfList = [];
    channelPdf.map(x=>{
        let dpath = x.document.documentPath;
        channelPdfList.push({documentId: x.documentId, name: x.name?x.name: dpath.split('/').pop().split('#')[0].split('?')[0], channelDocumentId: x.channelDocumentId,document:{documentUrl:x.document.documentUrl} });
    })

    res.status(HttpStatus.OK).send(Object.assign(result, {channelPdf: channelPdfList},{channelImages:channelImages}));
}

@Post('getCertificateTag')
    public async getChannelCertificateTags(@Req() req, @Res() res, @Body() data){
        let channelId = data.channelId; 
        let userId = data.userId;
        let user = await this.userService.findUserById(userId, channelId);
        let result = await this.channelSettingService.getChannelCertificateTags(user);
        
        res.status(HttpStatus.OK).json(result);
    }

@Post('getChannelProduct')	
public async getChannelProduct(@Req() req,@Res() res, @Body() data) {		
    let channelId = data.channelId; 
    let userId = data.userId;
    let user = await this.userService.findUserById(userId, channelId);
    console.log("btype:",user.businessType);
    let result = await this.channelSettingService.getChannelProduct(user.channelId,user.businessType);
        if (result) {
        res.status(HttpStatus.OK).json(result);
    } 
}

@Post('getChannelBrand')	
    public async getChannelBrand(@Req() req, @Res() res, @Body() data) {		
        let channelId = data.channelId; 
        let userId = data.userId;
        let user = await this.userService.findUserById(userId, channelId);
        
        let result = await this.channelSettingService.getChannelBrand(user.channelId);
         if (result) {
            res.status(HttpStatus.OK).json(result);
        } 
}

//key partner options list
@Post('keyPartnerOptions')
public async getKeyPartnerOptions(@Req() req, @Res() res, @Body() data){
    let channelId = data.channelId; 
    let userId = data.userId;
    let user = await this.userService.findUserById(userId, channelId);

    let result = await this.channelSettingService.getKeyPartnerOptions(user);

    let keyResellerOptions = [];
    let keyResellerValues = [];

    let keyRetailerOptions = [];
    let keyRetailerValues = [];

    let keyDistributorOptions = [];
    let keyDistributorValues = [];

    if(result.keyPartners.retailer){
        if(result.keyPartners.retailer.channel?.length)
        {
            result.keyPartners.retailer.channel.map(x => {
                let brand={
                    id:x.channelId,
                    name:x.companyName,
                    documentpath:x.companylogo?.documentPath ? process.env.CDN_URL+x.companylogo?.documentPath : null,
                    isNew:0
                }
                keyRetailerOptions.push(Object.assign(x,brand));
            })
        }

        if(result.keyPartners.retailer.keypartners?.length)
        {
            result.keyPartners.retailer.keypartners.map(x => {
                let brand={
                    id:x.channelkeypartnerid,
                    name:x.partnername,
                    documentpath:x.companylogo?.documentPath ? process.env.CDN_URL+x.companylogo?.documentPath : null ,
                    isNew:0
                }
                keyRetailerOptions.push(Object.assign(x,brand));
            })
        }
    }

    let keyretailer = await this.channelSettingService.getKeyRetailers(+user.channelId, appconstant.CHANNEL_TYPE_RETAILER);
    if(keyretailer?.length){
        keyretailer.map(x => {
            let logo = x.active? x.active?.logo?.documentPath : x.old? x.old.logo?.documentPath : x.new? x.new.logo?.logo_name : '';
            let companyName = x.active? x.active.company_name : x.old? x.old.company_name : x.new? x.new.company_name : '';
            let id = x.active? x.active.id : x.old? x.old.id : x.new? x.new.id : '';
            let brand={
                id:id,
                name:companyName,
                documentpath: logo ? process.env.CDN_URL+logo : null,
                isNew:0
            }
            keyRetailerValues.push(Object.assign(x,brand));
        })
    }

    if(result.keyPartners.reseller){
        if(result.keyPartners.reseller.channel?.length)
        {
            result.keyPartners.reseller.channel?.map(x => {
                let brand={
                    id:x.channelId,
                    name:x.companyName,
                    documentpath: x.companylogo?.documentPath ? process.env.CDN_URL+x.companylogo?.documentPath : null,
                    isNew:0
                }
                keyResellerOptions.push(Object.assign(x,brand));
            })
        }
        if(result.keyPartners.reseller.keypartners?.length)
        {
            result.keyPartners.reseller.keypartners?.map(x => {
                let brand={
                    id:x.channelkeypartnerid,
                    name:x.partnername,
                    documentpath:x.companylogo?.documentPath ? process.env.CDN_URL+x.companylogo?.documentPath : null,
                    isNew:0
                }
                keyResellerOptions.push(Object.assign(x,brand));
            })
        }
    }

    let keyreseller = await this.channelSettingService.getKeyRetailers(+user.channelId, appconstant.CHANNEL_TYPE_RESELLER);
    if(keyreseller?.length){
        keyreseller.map(x => {
            let logo = x.active? x.active?.logo?.documentPath : x.old? x.old.logo?.documentPath : x.new? x.new.logo?.logo_name : '';
            let companyName = x.active? x.active.company_name : x.old? x.old.company_name : x.new? x.new.company_name : '';
            let id = x.active? x.active.id : x.old? x.old.id : x.new? x.new.id : '';
            let brand={
                id:id,
                name:companyName,
                documentpath: logo ? process.env.CDN_URL+logo : null,
                isNew:0
            }
            keyResellerValues.push(Object.assign(x,brand));
        })
    }


    if(result.keyPartners.distributor){
        if(result.keyPartners.distributor.channel?.length)
        {
            result.keyPartners.distributor.channel.map(x => {
                let brand={
                    id:x.channelId,
                    name:x.companyName,
                    documentpath:x.companylogo?.documentPath ? process.env.CDN_URL+x.companylogo?.documentPath : null,
                    isNew:0
                }
                keyDistributorOptions.push(Object.assign(x,brand));
            })
        }
        if(result.keyPartners.distributor.keypartners?.length)
        {
            result.keyPartners.distributor.keypartners.map(x => {
                let brand={
                    id:x.channelkeypartnerid,
                    name:x.partnername,
                    documentpath:x.companylogo?.documentPath ? process.env.CDN_URL+x.companylogo?.documentPath : null,
                    isNew:0
                }
                keyDistributorOptions.push(Object.assign(x,brand));
            })
        }

    }    

    let keyDistributor = await this.channelSettingService.getKeyRetailers(+user.channelId, appconstant.CHANNEL_TYPE_DISTRIBUTOR)
    if(keyDistributor?.length){
        
            keyDistributor.map(x => {
                let logo = x.active? x.active?.logo?.documentPath : x.old? x.old.logo?.documentPath : x.new? x.new.logo?.logo_name : '';
                let companyName = x.active? x.active.company_name : x.old? x.old.company_name : x.new? x.new.company_name : '';
                let id = x.active? x.active.id : x.old? x.old.id : x.new? x.new.id : '';
                let brand={
                    id:id,
                    name:companyName,
                    documentpath: logo ? process.env.CDN_URL+logo : null,
                    isNew:0
                }
                keyDistributorValues.push(Object.assign(x,brand));
            })

    }

    let final = {
        keyPartners: {keyRetailerOptions, keyDistributorOptions, keyResellerOptions},
        channelKeyRetailer: keyRetailerValues,
		channelKeyDistributor: keyDistributorValues,
        channelKeyReseller: keyResellerValues
    }   

    return res.status(HttpStatus.OK).send(final);

}

//get channel profile
@Post('getChannelProfile')
public async getChannelProfile(@Req() req, @Res() res, @Body() data){
    let channelId = data.channelId; 
    let userId = data.userId;
    let user = await this.userService.findUserById(userId, channelId);

    let businessType=user.businessType?user.businessType:'2';
    let ctype=+user.channelTypeId;
    
    let typeid=[];
    if(ctype==2 && businessType==1)
        typeid=[4,5]
    else if(ctype==2 && businessType==2)
        typeid=[4,5]
    else if(ctype==2 && businessType==3)
        typeid=[4,5]
    else if(ctype==4)
        typeid=[4]
    else if(ctype==5)
        typeid=[5]
    else if(ctype==3 && businessType==1)
        typeid=[4,5]
    else if(ctype==3 && businessType==2)
        typeid=[4,5]
    else if(ctype==3 && businessType==3)
        typeid=[4,5]

    let result = await this.channelSettingService.getChannelProfile(user);
    let profileList = [];


    if(result.length > 0){
        let pro=result[0]?.channelCustomerProfile?.map(x => +x.profileTypeId);
        let clssifiid=result[0]?.channelCustomerProfile?.map(x => +x.profileTypeClassificationId);


        let query='select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
        +'when ptc."channelTypeId"=2 then \'vendor\' when ptc."channelTypeId"=5 then \'Reseller\' end as groupname,pt."profileType" as childname,'
        +'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" and pt."profileTypeId" in(:ptype) and ptc."channelTypeId" in(:typeId) '
        + 'and  ptc."profileTypeClassificationId" in(:clssifiid) ';
        
        profileList=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,           
            replacements: {
                typeId: typeid,
                clssifiid:clssifiid,
                ptype: pro,     
            }
            }).then(results => {     
                //console.log("result:",pro,req.user.channelTypeId,results);   
               return results;
            });

        console.log('profileList', profileList);
    }
    let profileType = '';
    
    if(result.length > 0 && result[0].profileTypeId){

    let query='select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
        +'when ptc."channelTypeId"=2 then \'vendor\' when ptc."channelTypeId"=5 then \'Reseller\' end as groupname,pt."profileType" as childname,'
        +'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" where ptc."channelTypeId" = :typeId and ptc."profileTypeId"=:profileTypeId';
      
         profileType=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,           
            replacements: {
                typeId: user.channelTypeId,
                profileTypeId: result[0].profileTypeId
            }
            }).then(results => {        
               return results;
            });
        }

        let finalData = {
            profile:profileList,
            profileTypeId: result>0?result[0].profileTypeId:null,
            profileType: profileType,
        }

        return res.status(HttpStatus.OK).send(finalData);
    }
 

//get channel location bo product form
@Get('getChannelLocation/:channelId')
	public async getChannelLocation(@Param('channelId') channelId, @Req() req, @Res() res) {
        let query = 'select "userId" from "tblUser" where "channelId"=:channelId and "contactId"=(select "contactId" from "tblContact" where "channelId"=:channelId and "isDefaultContact"=\'1\' and "deletedAt" is null)';
        let userId = await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,           
            replacements: {
                channelId: channelId
            }
        }).then(result => {
            return result[0].userId
        });

        let user = await this.userService.findUserById(userId, channelId);

		let getChannelLocation = await this.tradeLocationService.getChannelLocation(user);
		if (getChannelLocation) {
			return res.status(HttpStatus.OK).json(getChannelLocation);
		}
		throw new ProcessException('channel:channelLocation:noLocations');
	}


    //contact location
    @Post('getContactLocation/:contactId')	
    public async getContactLocationBo(@Param('contactId') contactId, @Req() req,@Res() res, @Body() data) {		
        let channelId = data.channelId; 
        let userId = data.userId;
        let user = await this.userService.findUserById(userId, channelId);

        if(user.contactId!=contactId)
            user.contactId=contactId;
        let result;
        result = await this.channelSettingService.getContactLocation(user);
        //console.log('reult',result);
         if (result) {
            let targetArray = result.contactLocation.filter(x => x.locationType == 'TARGET');
            let sellingArray = user.channelTypeId == 4  ? result.contactLocation.filter(x => x.locationType == 'RETAIL') : user.channelTypeId ==5 ? result.contactLocation.filter(x => x.locationType == 'RESELLING') : 
            result.contactLocation.filter(x => x.locationType == 'SELLING');
            let data = {
                targetArray : targetArray,
                sellingArray : sellingArray
            }
            res.status(HttpStatus.OK).json(data);
        } 
}

//get contact product
@Get('getContactProduct/:contactId/:channelId')	
    public async getContactProduct(@Param('contactId') contactId, @Param('channelId') channelId, @Req() req,@Res() res) {		        
        let result = await this.channelSettingService.getContactProduct(contactId,channelId);
         if (result) {
            res.status(HttpStatus.OK).json(result);
        } 
}  

//getting reporting to contacts
@Post('reporting')
async reporting(@Body(new ValidationPipe()) profileData:any, @Req() req,@Res() res, @Body() data){
    let channelId = data.channelId; 
    let userId = data.userId;
    let user = await this.userService.findUserById(userId, channelId);
    //console.log("profileData:",profileData);
        let result=await this.channelSettingService.reportingusers(user);
        return res.status(HttpStatus.OK).send(result);
}

//getting contactDetails for usermanage general form
@Post('generalInfo')
async generalInfo(@Body(new ValidationPipe()) contactId:any, @Req() req, @Res() res){
    let result = await this.channelSettingService.generalInfo(contactId);
    return res.status(HttpStatus.OK).send(result);
}

//contact preference bo
@Get('userpreference/:contactId/:channelId')	
	public async getUserPreference(@Param('contactId') contactId,@Param('channelId') channelId,@Res() res,@Req() req,) {
        // let user=req.user;
        let result=await this.userService.getUserPreference(contactId, channelId);		
		return res.status(HttpStatus.OK).send(result);
    }

//preference
@Get('preference/:typeId')	
public async getPreference(@Res() res,@Req() req, @Param('typeId') typeId) {
    let result=await this.userService.getPreference(typeId);		
    return res.status(HttpStatus.OK).send(result);
}

//update my profile user

@Patch('profile/:id/:channelId/:userId')	
	public async userProfileUpdate(@Param('id', new ParseIntPipe()) id, @Body(new ValidationPipe()) userDto: UpdateUserDto, @Res() res, @Req() req, @Param('channelId') channelId, @Param('userId') userId) {
		// user magt preference
	//	await this._validateUserMgmtProfile();
		let nonsuggestion=[];let count;	
	//	await this._validateSubscriptionFeatures(true);

        let real_user = await this.userService.findUserById(userId, channelId);        

		
		if (Object.keys(userDto).length) {
			let user = await this.userService.getUser(id,real_user.channelId);
			

			let allowToModifyInterestedCategory = true;
			// check if the request from back office
			// if (this.context.get('real-user')) {
			// 	allowToModifyInterestedCategory = true
			// }

			await this.userService.updateUser(userDto, user, allowToModifyInterestedCategory,real_user);
			//let channelTypeId=this.context.get('user').channel.channelTypeId;
			// update index
			//this.ChannelIndexService.index(this.context.get('user').channelId);

			//this.userService.getNonSuggestion(user.channelId,this.context.get('dataSession').plan,id,user.userId,channelTypeId);

			// audit log
			// if (this.context.get('real-user')) {
			// 	this.auditLog.addAuditLog((userDto && userDto.reason) ? userDto.reason : null, auditlog.MODULE.BO_USER_MANAGEMENT, req);
			// }
			
			//let hubspotClient = new HubspotImportHelper();
			//let channelId = this.context.get('user').channelId;
			//let contactId = this.context.get('user').contactId;
			//console.log(userDto.contact.email);
			//hubspotClient.getContactWithThisEmail(userDto.contact.email).then(async function(existContactDetails) {
			//var obj = JSON.parse(existContactDetails);
			//console.log(obj.vid);
				// if(obj.vid){
				// 	hubspotClient.UpdateContactToHubSpot(userDto.contact, contactId, user.userId, channelId, obj.vid).then(async function(contactDetails) {
				// 	}); 
				// } else {
				// 	hubspotClient.impotContactToHubSpot(userDto.contact, contactId, user.userId, channelId).then(async function(contactDetails) {
				// 	}); 
				// }
			//}); 
		}

		res.status(HttpStatus.OK).send({
			updated: true
		});
	}


    //events creation BO

    @Post('boEventCreate')
    public async boEventCreate(@Req() req, @Res() res, @Body() data){
        console.log('data', data)
        await this.channelAdminService.boEventCreate(data);
        
        res.status(HttpStatus.CREATED).send({
            created: true
        });
    }

    //Event Listing
    @Post('getEventListing')
    public async getEventListing(@Req() Req, @Res() res, @Body() data){

        let result = await this.channelAdminService.getEventListing(data);

        res.status(HttpStatus.OK).json(result);
    }

    //Event By Id
    @Get('getEventById/:eventId')
    public async getEventById(@Req() req, @Res() res, @Param('eventId') eventId)
    {
        let result = await this.channelAdminService.getEventById(eventId)
        res.status(HttpStatus.OK).json(result);
    }

    //Event Details Manage
    @Get('getEventDetails/:eventId')
    public async getEventDetailsById(@Req() req, @Res() res, @Param('eventId') eventId)
    {
        let result = await this.channelAdminService.getEventDetailsById(eventId)
        res.status(HttpStatus.OK).json(result);
    }

    //eventAvailabilityCheck
    @Get('eventAvailabilityCheck/:eventCatId/:eventId')
    public async eventAvailabilityCheck(@Req() req, @Res() res, @Param('eventCatId') eventCatId, @Param('eventId') eventId){
        let result = await this.channelAdminService.eventAvailabilityCheck(eventCatId, eventId);
        res.status(HttpStatus.OK).json(result);
    }

    //getEventChannelDetails
    @Get('getEventChannelDetails/:channelId/:eventId')
    public async getEventChannelDetails(@Req() req, @Res() res, @Param('eventId') eventId, @Param('channelId') channelId)
    {
        let result = await this.channelAdminService.getEventChannelDetails(channelId, eventId)
        res.status(HttpStatus.OK).json(result);
    }

    //saving event contacts from bo
    @Post('boEventUsersManage')
    public async boEventUsersManage(@Req() req, @Res() res, @Body() data)
    {
        let result = await this.channelAdminService.boEventUserManage(data);
        res.status(HttpStatus.OK).json(result);
    }

    //sending to eventCatalyst from bo
    @Post('boToEventCatalyst')
    public async boToEventCatalyst(@Req() req, @Res() res, @Body() data){
        let result = await this.channelAdminService.boToEventCatalyst(data);
        res.status(HttpStatus.OK).json(result);
    }


    //setting event contact
    @Get('setEventContact/:id')
	public async setEventContact(@Param('id') id, @Res() res) {
		await this.channelService.isEvent(id, '1');
		return res.status(HttpStatus.OK).send();
	}

    //setting event contact
    @Post('setEventParticipantStatus/')
	public async setEventParticipantStatus(@Req() req, @Res() res, @Body() data) {
		await this.channelService.setEventParticipantStatus(data.eventId, data.channelId, data.contactId);
		return res.status(HttpStatus.OK).send();
	}

    

    //UPdate wire transfer in billing
    @Post('updateSubscription/:id')
	//@Roles(Channel.BACKEND_CHANNEL)
	public async offlinePayment(@Param('id') id, @Body(new ValidationPipe()) updateSubscriptionDto: BoUpdateSubscriptionDto, @Res() res) {
        let result =await this.subscriptionService.updateOfflinrPayment(id, updateSubscriptionDto);
        return res.status(HttpStatus.OK).send(result[0]);
	}
    @Post('getCompanySearchList')
    public async companyNameList(@Req() req, @Res() res,@Body() data: any) {		
        console.log('data', data);
        let result = await this.channelAdminService.companyNameList(data.search);
            res.status(HttpStatus.OK).json(result);
    }
    @Post('getContactSearchList')
    public async getContactSearchList(@Req() req, @Res() res,@Body() data: any) {		
        console.log('data', data);
        let result = await this.channelAdminService.contactNameList(data.search);
            res.status(HttpStatus.OK).json(result);
    }
    @Get('getContact/:contactId')
    async getContact(@Body() data, @Param('contactId') contactId, @Res() res){
        let contact = await this.channelAdminService.getContact(contactId);
		return res.status(HttpStatus.OK).send(contact);
    }

    @Patch('updateContact/:contactId')	
	public async updateContact(@Body(new ValidationPipe()) data: any, @Res() res, @Req() req,@Param('contactId') contactId) {       

			await this.userService.updateContact(contactId, data);

		res.status(HttpStatus.OK).send({
			updated: true
		});
	}

    @Get('channelDetail/:channelId')
	public async channelDetail(@Param('channelId') channelId, @Req() req, @Res() res) {
		let channelDetail = await this.channelAdminService.getChannel(channelId);
		if (!channelDetail) {
			throw new ProcessException('channel:getChannel:channelNotFound');
		}
		res.status(HttpStatus.OK).send(channelDetail);
	}


    //products for individual vendor in bo myproduct
    @Post('my-products')
	async myProduct(@Req() req, @Res() res, @Body(new ValidationPipe()) productFilters: any) {
		console.log('productFilters', productFilters);
        let userId = productFilters.user.userId;
        let channelId = productFilters.user.channelId;

        let user = await this.userService.findUserById(userId, channelId);
		
		// this.validateProductLimit(user);
		let order=productFilters.order?productFilters.order:'asc'
		let limit = ' LIMIT ' + productFilters.limit;
		let offset = ' OFFSET ' + productFilters.offset;
		//console.log('productFilters', productFilters);
		let query = 'select count(*) OVER() AS total,tp."product_rank_order",tp."isPublic",tp."productId", tp."channelId", (select "brandName" from "tblBrand" where "id"=tp."brandId"), tp."product",tp."productSKU", tp."isactive", tp."productqualitylevel", pr."businesscurrencyid",pr."retailerprice",(select concat( \'' + process.env.CDN_URL + '\',"documentPath") as "documentUrl" from "tblDocument" where "documentId" in(select "imageid" from "tblproductimage" where "productimageid"=tp."defaultimageid"))from "tblProduct" as tp inner join "tblproductpricedetail" as pr on pr."productId"=tp."productId" where "channelId"= :channelId and tp."deletedat" is null and "isactive"=:isactive ';
		if(productFilters.search)
		query += ' and tp."productId"='+ +productFilters.search ;
		if(productFilters.filter.brand)
		query += ' and tp."brandId" in('+ productFilters.filter.brand +')';
		if(productFilters.filter.family)
		query += ' and tp."productGroupId" in('+ productFilters.filter.family +')';
		if(productFilters.filter.category)
		query += ' and tp."productCategorieId" in('+ productFilters.filter.category +')';
		if(productFilters.filter.position)
		query += ' and tp."productqualitylevel" =:quality';

		query += ' order by tp."product_rank_order" '+order +limit+offset;

		let result=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
			channelId:user.channelId,
			isactive:productFilters.isactive.toString(),
			quality:productFilters.filter.position
		}
		});
		let master = result.map(x => x.productSKU);
		let active=await this.sequelizeInstance.query('select count(*) from "tblProduct" where "channelId"=:channelId and "deletedat" is null and "isactive"=:isactive', {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
			channelId:user.channelId,
			isactive:'1'
		}
		})
		let inActive=await this.sequelizeInstance.query('select count(*) from "tblProduct" where "channelId"=:channelId and "deletedat" is null and "isactive"=:isactive', {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
			channelId:user.channelId,
			isactive:'0'
		}
		})
			let tabCount={
				active: active?active[0].count:0,
				inActive: active?inActive[0].count:0
			}
		let results;
		if(result.length!=0){
			results={
				total:result[0].total,
				result:result,
				tabCount: tabCount,
				master: master
			}
		}else{
			results={
				total:0,
				result:[],
				master: [],
				tabCount:tabCount
			}
		}
		res.status(HttpStatus.OK).send(results);
	}

    //myproduct search list for vendor bo
    @Post('myProductsAgg/:isActive')
	async myProductsAgg(@Req() req, @Res() res, @Param('isActive', new ParseIntPipe()) isActive: number, @Body() data) {
		
        let userId = data.userId;
        let channelId = data.channelId;

        let user = await this.userService.findUserById(userId, channelId);

		let result=await this.sequelizeInstance.query('select "productId" from "tblProduct" where "channelId"=:channelId and "deletedat" is null and "isactive"=:isactive', {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
			channelId:user.channelId,
			isactive:isActive.toString()
		}
		}).then(x=>{	
			return x.map(p => +p.productId);
			})
		console.log('result', result);
		let product = await this.productService.getProductAggr(result);
		if(product){
			res.status(HttpStatus.OK).send(product);
		}
	}


    //old channel detail
    @Get('oldChannelDetail/:channelId')
	public async oldChannelDetail(@Param('channelId') channelId, @Req() req, @Res() res) {
        let channelDetail = await this.channelAdminService.getOldChannelDetails(channelId);
		if (!channelDetail) {
			throw new ProcessException('channel:getChannel:channelNotFound');
		}
		res.status(HttpStatus.OK).send(channelDetail);
	}

    //old channel detail view
    @Get('oldChannelDetailView/:channelId')
	public async oldChannelDetailView(@Param('channelId') channelId, @Req() req, @Res() res) {
        let channelDetail = await this.channelAdminService.getOldChannelDetailView(channelId);
		if (!channelDetail) {
			throw new ProcessException('channel:getChannel:channelNotFound');
		}
		res.status(HttpStatus.OK).send(channelDetail);
	}

    @Get('test/:channelId')
	public async testPar(@Param('channelId') channelId, @Req() req, @Res() res) {
		res.status(HttpStatus.OK).send(channelId);
	}

    @Get('test')
	public async test(@Req() req, @Res() res) {
		res.status(HttpStatus.OK).send(true);
	}

    @Post('test')
	public async testPost(@Body(new ValidationPipe()) data: any,@Req() req, @Res() res) {
		res.status(HttpStatus.OK).send(data);
	}
    @Post('getPendingEventPayment')
	public async getPendingEventPayment(@Req() req, @Res() res) {
		let result = await this.channelService.getPendingEvent();
        console.log(result);
		return res.status(HttpStatus.OK).send(result);
	}

    @Post('updateEventPayment')
	public async updateEventPayment(@Req() req, @Res() res) {
		let result = await this.channelService.updateEventChannel(req.body);
        console.log(result);
		return res.status(HttpStatus.OK).send(result);
	}

    @Get('channelCountry/:channelId')
    async getChannelCountry(@Req() req, @Res() res, @Param('channelId') channelId){
        let query = 'select "countryId", country, "phoneCode", "isoCode" from "tblCountry" where "countryId"=(select "countryId" from "tblChannel" where "channelId"=:channelId limit 1)';
        let result = await this.sequelizeInstance.query(query, {
          type: QueryTypes.SELECT,           
          replacements: {
              channelId: channelId,
          }

        }).then(res => res[0]);
        return res.status(HttpStatus.OK).send(result);
    }

    @Post('checkVat')
    public async checkVat(@Req() req, @Res() res, @Body() data){
    
        let result = await fetch(process.env.VAT_API_URL+data.vatNumber)
        .then(res => res.json())
        .then(data => {
          if(data.valid){
            return 1;
          }else{
            return 0;
          }
            
        })
        .catch(err => {
          console.log('vat error', err);
          return 0;
        });

      
    res.status(HttpStatus.OK).json({valid: result});
      

    
}

    //getting oldchannelfacts
    @Get('getOldFacts/:oldChannelId')
    async getOldFacts(@Req() req, @Res() res, @Param('oldChannelId') oldChannelId){
        let result = await this.channelAdminService.getOldChannelFacts(oldChannelId);
        return res.status(HttpStatus.OK).send(result);
    }

    //getting oldChannelProduct section
    @Get('getOldProducts/:oldChannelId')
    async getOldProducts(@Req() req, @Res() res, @Param('oldChannelId') oldChannelId){
        let result = await this.channelAdminService.getOldProducts(oldChannelId);
        return res.status(HttpStatus.OK).send(result);
    }

    //getting oldChannelBrands section
    @Get('getOldChannelBrand/:oldChannelId')
    async getOldChannelBrand(@Req() req, @Res() res, @Param('oldChannelId') oldChannelId){
        let result = await this.channelAdminService.getOldChannelBrand(oldChannelId);
        return res.status(HttpStatus.OK).send(result);
    }

    //old channel initial profile
    @Post('getOldInitialProfile')
    async getOldInitialProfile(@Req() req, @Res() res, @Body() data){
        let result = await this.channelAdminService.getOldInitialProfile(data);
        return res.status(HttpStatus.OK).send(result);
    }

    //old Channel profile and speciality
    @Post('channelProfileOld')
    async channelProfileOld(@Req() Req, @Res() res, @Body() data){

        let user = data;
        let result = await this.channelAdminService.channelProfileOld(data.oldChannelId);

        let ctype=+user.channelTypeId;
    
    let typeid=[];
    if(ctype==2)
        typeid=[4,5]
    else if(ctype==4)
        typeid=[4]
    else if(ctype==5)
        typeid=[5]
    else if(ctype==3)
        typeid=[4,5]
    
        let profileList = [];

    if(result)
    {
        result.map(async x => {

    
    let pro=result[0].channelCustomerProfile?.map(x => +x.profileTypeId);
    let clssifiid=result[0].channelCustomerProfile?.map(x => +x.profileTypeClassificationId);

        let query='select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
        +'when ptc."channelTypeId"=2 then \'vendor\' when ptc."channelTypeId"=5 then \'Reseller\' end as groupname,pt."profileType" as childname,'
        +'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" and pt."profileTypeId" in(:ptype) and ptc."channelTypeId" in (:typeId)'
        + 'and  ptc."profileTypeClassificationId" in(:clssifiid) and pt."profileType" != \'\' ';
        
        profileList=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,           
            replacements: {
                typeId: typeid,
                clssifiid:clssifiid,
                ptype: (user.channelTypeId==4||user.channelTypeId==5) ? result[0].profileTypeId : pro,     
            }
            }).then(results => {     
                //console.log("result:",pro,req.user.channelTypeId,results);   
               return results;
            });

        });
    }

    let profileType = '';
    let query = '';
    
    if(result[0].profileTypeId){

    query='select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
        +'when ptc."channelTypeId"=2 then \'vendor\' end as groupname,pt."profileType" as childname,'
        +'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" where ptc."channelTypeId" = :typeId and ptc."profileTypeId"=:profileTypeId ';
      
         profileType=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,           
            replacements: {
                typeId: user.channelTypeId,
                profileTypeId: result[0].profileTypeId
            }
            }).then(results => {        
               return results;
            });
        }

        let finalData = {
            profile:profileList,
            profileTypeId: result.length>0?result[0].profileTypeId:null,
            profileType: profileType,
        }        


        return res.status(HttpStatus.OK).send(finalData);
    }

    //old channel speciality
    // @Post

    //Edit old company profile
    @Post('editBasicCompanyOld')
    async editBasicOldCompanyProfile(@Body(new ValidationPipe()) profileData:any, @Req() req,@Res() res){
        let result=await this.channelAdminService.editBasicCompanyOld(profileData);
        return res.status(HttpStatus.OK).send(result);
    }

    //saving old channel facts
    @Post('saveOldChannelFacts')
    async saveOldChannelFacts(@Req() req, @Res() res, @Body() data){

        let result = await this.channelAdminService.saveOldChannelFacts(data);
        return res.status(HttpStatus.OK).send(result);

    }

    //Edit BANNER IMAGE
    @Post('updateBannerImage')
    async updateBannerImage(@Body(new ValidationPipe()) bannerImg:any, @Req() req,@Res() res){
        let result=await this.channelAdminService.updateBannerImage(bannerImg);

        return res.status(HttpStatus.OK).send(result);
}    
}
