import { Body, Controller, Get, HttpStatus, Inject, Param, ParseIntPipe, Post, Put, Req, Res, UseGuards, ValidationPipe } from '@nestjs/common';
import { ChannelService } from '../service/channel.service';
import { QueryTypes } from "sequelize";
import { AuthGuard } from '@nestjs/passport';
import { ProcessException } from 'src/middleware/exception/Process.exception';
import { BaseController } from 'src/core';
import { ContextService } from 'src/context/context.service';
import { Channel } from '../models/Channel.entity';
import { includes } from 'lodash';
import { ChannelHubDetailOutBound } from '../transformation/channelHubDetailOutBound';
import { appconstant } from 'src/config/app.constant';
import { AuthException } from 'src/middleware/exception/Auth.exception';
import { ChannelHubService } from '../service/channelHub.service';
import { UpdateChannelDto } from '../dto/updateChannel.dto';
import _ = require('lodash');
@UseGuards(AuthGuard('jwt'))
@Controller('channel')
export class ChannelController extends BaseController {
	private channelHubDetailOutBound = new ChannelHubDetailOutBound();
constructor(private channelService:ChannelService,
	private channelHubService : ChannelHubService,
	@Inject('SequelizeInstance') private readonly sequelizeInstance,
	private context: ContextService,) {
		super();
	}


	@Post('getSearchList/:viewId/:planId')
	async getSearchList(@Req() req, @Res() res,@Param('viewId', new ParseIntPipe()) viewId: number,@Param('planId', new ParseIntPipe()) planId: number,@Body() data){        
		//let channel = await this.channelService.getSearchList(viewId,planId,data,req.user);
		let channel = await this.channelService.getSearchList(viewId,planId,data,req.user);
            return res.status(HttpStatus.OK).send(channel);      

    }

	@Post('getNonMember/:viewId/:planId')
	async getNonMember(@Req() req, @Res() res,@Param('viewId', new ParseIntPipe()) viewId: number,@Param('planId', new ParseIntPipe()) planId: number,@Body() data){        
		//let channel = await this.channelService.getSearchList(viewId,planId,data,req.user);
		let channel = await this.channelService.nonMemberList(viewId,planId,data,req.user);
            return res.status(HttpStatus.OK).send(channel);      

    }

	@Get('topcompany/:keyword')
	async getTopCompany(@Req() req, @Res() res,@Param('keyword') keyword){    
		
		let type=req.user.channelTypeId;
		let btype = req.user.businessType;
		let channel = await this.channelService.getTopCompany(keyword, type, btype);
            return res.status(HttpStatus.OK).send(channel);    

    }

	@Get('topproduct/:keyword')
	async getTopProduct(@Req() req, @Res() res,@Param('keyword') keyword){        
		let channel = await this.channelService.getTopProduct(keyword);
            return res.status(HttpStatus.OK).send(channel);    

    }


	// @Get('suggAlgm')
	// async suggAlgm(@Req() req, @Res() res){                
    //         let channel = await this.channelService.suggAlgm();
    //         return res.status(HttpStatus.OK).send(channel);      
    // }


    @Get('getSearchAggregation/:viewId')
	async getSearchAggregation(@Req() req, @Res() res,@Param('viewId', new ParseIntPipe()) viewId: number){
        let channel = await this.channelService.getSearchAggregation(viewId,req.user);
        return res.status(HttpStatus.OK).send(channel);      
    }

	//Channel Filter Services

	@Post('chFilter/:viewId/:channelType')
	async chFilter(@Req() req, @Res() res, @Param('viewId', new ParseIntPipe()) viewId: number, @Param('channelType', new ParseIntPipe()) channelType: number, @Body() data) {
		let channel = await this.channelService.chFilterBase(viewId, channelType, req.user, data);
		return res.status(HttpStatus.OK).send(channel);
	}


	@Get('channeltype')
	async channeltype(@Req() req, @Res() res){
		let type=req.user.channelTypeId;
        //let channel = await this.channelService.chFilterBase(viewId,req.user);
		let result = await this.channelService.getchannelTypes(type);
        return res.status(HttpStatus.OK).send(result);      
    }

	@Post('brandFilter')
	async brandFilter(@Req() req, @Res() res,@Body() data){
        //let channel = await this.channelService.chFilterBase(viewId,req.user);
		let result = await this.channelService.getBrandFilter(data);
        return res.status(HttpStatus.OK).send(result);      
    }

	@Post('locationFilter')
	async locationFilter(@Req() req, @Res() res,@Body() data){
        //let channel = await this.channelService.chFilterBase(viewId,req.user);
		let result = await this.channelService.getLocationFilter(data);
        return res.status(HttpStatus.OK).send(result);      
    }

	@Post('countryFilter')
	async countryFilter(@Req() req, @Res() res,@Body() data){
        //let channel = await this.channelService.chFilterBase(viewId,req.user);
		let result = await this.channelService.getCountryFilter(data);
        return res.status(HttpStatus.OK).send(result);      
    }

	@Post('specialFilter')
	async specialFilter(@Req() req, @Res() res,@Body() data){
        //let channel = await this.channelService.chFilterBase(viewId,req.user);
		let result = await this.channelService.getSpecialFilter(data);
        return res.status(HttpStatus.OK).send(result);      
    }

	@Post('productFilter')
	async productFilter(@Req() req, @Res() res,@Body() data){
		let user = req.user;
        //let channel = await this.channelService.chFilterBase(viewId,req.user);
		let result = await this.channelService.getProductFilter(data, user.channelId);
		return res.status(HttpStatus.OK).send(result);
	}

	@Post('interestedProductFilter')
	async interestedProductFilter(@Req() req, @Res() res, @Body() data) {
		let user = req.user;
		//let channel = await this.channelService.chFilterBase(viewId,req.user);
		let result = await this.channelService.getInterestedProduct(data, user.contactId);
        return res.status(HttpStatus.OK).send(result);      
    }

	@Post('retailFilter')
	async retailFilter(@Req() req, @Res() res,@Body() data){
        //let channel = await this.channelService.chFilterBase(viewId,req.user);
		let result = await this.channelService.getRetailFilter(data);
        return res.status(HttpStatus.OK).send(result);      
    }

	@Post('targetFilter')
	async targetFilter(@Req() req, @Res() res, @Body() data) {

		let result = await this.channelService.getTargetFilter(req.user, data);
		return res.status(HttpStatus.OK).send(result);
	}
	//End of Channel Filter Services

	//Contact Filter Services
	@Get('conFilter')
	async conFilter(@Req() req, @Res() res){
        let channel = await this.channelService.contactFilterBase(req.user);
        return res.status(HttpStatus.OK).send(channel);      
    }

	@Post('conSpecialFilter')
	async conSpecialFilter(@Req() req, @Res() res,@Body() data){
        //let channel = await this.channelService.chFilterBase(viewId,req.user);
		let result = await this.channelService.getContactSpecialFilter(data);
        return res.status(HttpStatus.OK).send(result);      
    }

	@Post('conCountryFilter')
	async conCountryFilter(@Req() req, @Res() res,@Body() data){
        //let channel = await this.channelService.chFilterBase(viewId,req.user);
		let result = await this.channelService.getContactCountryFilter(data);
        return res.status(HttpStatus.OK).send(result);      
    }
	
	@Get('conNameListFilter')
	async conNameList(@Req() req, @Res() res){
        let channel = await this.channelService.contactFilterBase(req.user);
		
		let list = await this.channelService.conNameListFilter(channel);
        return res.status(HttpStatus.OK).send(list);      
    }
	
	//End of Contact Filter Services

    @Get('getTapCount/:contactId')
	async getTapCount(@Req() req, @Res() res,@Param('contactId', new ParseIntPipe()) contactId: number){
		 let user = req.user;
		 console.log('user', user);
		 let channelId=user.channelId;
		 let typeid=user.channelTypeId;
		 let profileTypeId=user.profileTypeId;

		//non member
		let channeltype = [];
		if(typeid==2)
		channeltype=[3,4];
		else if(typeid == 3 || typeid == 4 || typeid == 5)
		channeltype=[2];

		// else if(typeid==3)
		// channeltype=[2,3,4];
		// else if(typeid==4)
		// channeltype=[2,3];		

		let inquery; let otherquery; let sugg; let nonmember; let favorites; let chfavorites; let dismissCnt;		 
		inquery = 'select count("productId") from "tblProduct" where "deletedat" is null and "isInovation"=\'1\' and "isactive"=\'1\' and "productId" not in(select "productid" from "tblproductrejection" where "channelid"=:channelId) ';
		otherquery = 'select count("productId") from "tblProduct" where "deletedat" is null and "isactive"=\'1\' and "productId" not in(select "productid" from "tblproductrejection" where "channelid"=:channelId) and "productId" not in(select "productId" from "tblProductJCT" where "requested_by"=:contactId) ';
		sugg = 'select count("productId") from "tblProductJCT" where "productId" in(select "productId" from "tblProduct" where "deletedat" is null and "isactive"=\'1\') and "requested_by"=:contactId and "productId" not in(select "productid" from "tblproductrejection" where "channelid"=:channelId)';
		 nonmember = 'select count("oldChannelId") from "tblOldChannel" where "channelTypeId" in(:channelTypeId) and "isActive"=\'1\' and "deletedAt" is null'; // "updatedAt" '+data.order+' ' ;
		 favorites = 'select count("channelid") from "tblproductwishlist" where "userid"=:userId and "deletedat" is null'
		chfavorites = 'select count("channelid") from "tblproductwishlist" where "userid"=:userId and "deletedat" is null and "productid" is null'
		 dismissCnt = 'select count("channelJCTId") from "tblChannelJCT" where "channelId"=:channelId and "connectionStatusTypeId" = 5 and "deletedat" is null'
		 
		 let incount = 0;
		 let othercount = 0;
		 let suggcount = 0;
		 let nonmember_count = 0;
		 let favorite_count = 0;
		let chfavorite_count = 0;

		 if(typeid != 2){
		 incount=await this.sequelizeInstance.query(inquery, {
		 type: QueryTypes.SELECT,
		 raw: true,
			 replacements: {
				 channelId: channelId
			 }
		 }).then(x=>{
		 //console.log("products:",x.length);
		 return x[0].count;
		 })
		 othercount=await this.sequelizeInstance.query(otherquery, {
			type: QueryTypes.SELECT,
			raw: true,
			 replacements: {
				 channelId: channelId,
				 contactId: contactId

			 }
			}).then(x=>{
			//console.log("products:",x.length);
			return x[0].count;
			})
		
			suggcount=await this.sequelizeInstance.query(sugg, {
				type: QueryTypes.SELECT,
				raw: true,
				replacements:{
					contactId: contactId,
					channelId: channelId
				}
				}).then(x=>{
				//console.log("products:",x.length);
				return x[0].count;
				})
		
		
		}

		nonmember_count=await this.sequelizeInstance.query(nonmember, {
			type: QueryTypes.SELECT,
			raw: true,
			replacements:{
				channelTypeId:channeltype
			}
			}).then(x=>{
			//console.log("products:",x.length);
			return x[0].count;
			})

		favorite_count = await this.sequelizeInstance.query(chfavorites, {
				type: QueryTypes.SELECT,
				raw: true,
				replacements:{
					userId: user.userId
				}
				}).then(x=>{
				//console.log("products:",x.length);
				return x[0].count;
				})
	    
				dismissCnt=await this.sequelizeInstance.query(dismissCnt, {
					type: QueryTypes.SELECT,
					raw: true,
					replacements:{
						channelId: user.channelId
					}
					}).then(x=>{
					//console.log("products:",x.length);
					return x[0].count;
					})		

		// chfavorite_count = await this.sequelizeInstance.query(chfavorites, {
		// 	type: QueryTypes.SELECT,
		// 	raw: true,
		// 	replacements: {
		// 		userId: user.userId
		// 	}
		// }).then(x => {
		// 	//console.log("products:",x.length);
		// 	return x[0].count;
		// })


		var result = await this.sequelizeInstance.query('select * from "getnetworkchannellist"(11,'+contactId +',0,0)', {
			type: QueryTypes.SELECT 			
		});

		res.status(HttpStatus.OK).json(Object.assign({
			counts:result[0].getnetworkchannellist,
			product_suggestion_count:suggcount,
			product_innovation_count:incount,
			product_others_count:othercount,
			all_product_count: +suggcount + +othercount,
			nonmember_count: nonmember_count,
			//chfavorite_count: chfavorite_count
			favorite_count: favorite_count,
			dismiss_count:dismissCnt
		}));	
	}

	@Get('getDashboardcount')
	public async getDashboard(@Req() req, @Res() res) {
		let user = req.user;
		let result = await this.channelService.getDashboardcount(user);
		if (result) {

			let query = 'select tc."startedAt",tc."endAt",tp."subscriptionPlanType"  from "tblChannelSubscription" tc join "tblSubscription" ts on tc."subscriptionId"=ts."subscriptionId" '
			 + 'inner join "tblSubscriptionPlan" tp on tp."subscriptionPlanId"=ts."subscriptionPlanId" '
			+ 'where tc."isActive"=\'1\' and tc."channelId"=:channelId' ;

			let res1 = await this.sequelizeInstance.query(query, {
				type: QueryTypes.SELECT,
				replacements: {
					channelId: user.channelId
				}
			}).then(x => {
				return x[0];
			});
			console.log('req.user.channel.channelSubscription', req.user.channel.channelSubscription);
			let featureList = {'featureList':req.user.channel.channelSubscription.keyFeature? req.user.channel.channelSubscription.keyFeature[appconstant.SubscriptionCharcteristicFeature.DASHBOARD_PROFILE_PRODUCT]:null};

			query = 'select  "month","count" from "dashboard" as pv WHERE "channelid"=:channelId and "visitid"=:channelId and "visittype"=1 and "user_id" not in(select "userId" from "tblUser" where ("channelId"=:channelId) or ("isActive"=\'0\' and "deletedAt" is null)) order by "month"';
			let graphData = await this.sequelizeInstance.query(query, {
				type: QueryTypes.SELECT,
				replacements: {
					channelId: user.channelId,
				}
			}).then(x => {
				console.log("datasets:", x);
				let newv = [{ month: 1, count: 0 }, { month: 2, count: 0 }, { month: 3, count: 0 }, { month: 4, count: 0 }, { month: 5, count: 0 }, { month: 6, count: 0 },
				{ month: 7, count: 0 }, { month: 8, count: 0 }, { month: 9, count: 0 }, { month: 10, count: 0 }, { month: 11, count: 0 }, { month: 12, count: 0 }];
				let uniqv = [{ month: 1, count: 0 }, { month: 2, count: 0 }, { month: 3, count: 0 }, { month: 4, count: 0 }, { month: 5, count: 0 }, { month: 6, count: 0 },
				{ month: 7, count: 0 }, { month: 8, count: 0 }, { month: 9, count: 0 }, { month: 10, count: 0 }, { month: 11, count: 0 }, { month: 12, count: 0 }];

				let uniqvisit = x.filter(n => n.count > 1);
				if (uniqvisit?.length) {
					uniqv.map(master => {
						uniqvisit.map(data => {
							if (master.month == data.month)
								master.count = data.count
						})
					})
				}
				let newvisit = x.filter(n => n.count == 1);
				if (newvisit?.length) {
					newv.map(master => {
						newvisit.map(data => {
							if (master.month == data.month)
								master.count = data.count
						})
					})
				}

				let datasets = [
					{
						label: "New Visits",
						data: newv.map(x => x.count),
						borderColor: '#fb6340'
					},
					{
						label: "Unique Visits",
						data: uniqv.map(x => x.count),
						borderColor: '#11cdef',
					},

				]
				return datasets;
			});

			let final = { ...result, ...res1, ...featureList, graphData: graphData };
			
			res.status(HttpStatus.OK).json(final);
		} else {
			throw new ProcessException("notification:presetmail:listNotfound");
		}
	}


	@Get('getFoCompanyList/:viewId/:planId')
	async getFoCompanyList(@Req() req, @Res() res,@Param('viewId', new ParseIntPipe()) viewId: number,@Param('planId', new ParseIntPipe()) planId: number){
        console.log('res.query',req.query);
        let companyList = await this.channelService.getFoCompanyList(viewId,planId,Object.assign(req.query, {...req.user}));
			res.status(HttpStatus.OK).send(companyList);
	}
	@Get('channelDetailView/:channelId')
	public async getChannelDetailView(@Param('channelId') channelId, @Req() req, @Res() res) {
		let user = {
			channelId: req.user.channelId,
			contactId: req.user.contactId,
			channelTypeId: req.user.channelTypeId,
			businessType: req.user.businessType,
			userId: req.user.userId
		};
		if (+channelId != +user.channelId) {
			await this.verifyRejectedChannels(user, +channelId);
		}

		let channelDetail = await this.channelService.getChannel(channelId, user);
		if (!channelDetail) {
			throw new ProcessException('channel:getChannel:channelNotFound');
		}

		res.status(HttpStatus.OK).send(channelDetail);
	}

	@Post('channelContacts/:channelId')
	public async getContacts(@Param('channelId') channelId, @Res() res, @Body(new ValidationPipe()) contactFilter: any, @Req() req) {
		let user = req.user;
		let defaultContact = null;
		let contactList: any = null;
		let skip = true;
		console.log('user', user);
		if(user.hasOwnProperty('adminUserId')){
			user={
				channelTypeId:1,
				channelId:1,
				contactId:1,
				userId:1
			}
		}

		if (user.channelTypeId == Channel.CHANNEL_TYPE_HUB) { // || +user.channelId == +channelId
			skip = false;
		}

		if (skip && +user.channelId != +channelId) {
			let ChannelTypeId = await this.getChannelType(channelId, 'channelTypeId');
			if (ChannelTypeId == Channel.CHANNEL_TYPE_VENDOR || (
				ChannelTypeId == Channel.CHANNEL_TYPE_DISTRIBUTOR && user.channelTypeId == +Channel.CHANNEL_TYPE_RETAILER
			)) {
				contactFilter.contactType = 'SELLING';
			} else if (ChannelTypeId == Channel.CHANNEL_TYPE_RETAILER || (
				ChannelTypeId == Channel.CHANNEL_TYPE_DISTRIBUTOR && user.channelTypeId == +Channel.CHANNEL_TYPE_VENDOR
			)) {
				contactFilter.contactType = 'SOURCING';
			}
		}

		let total = await this.channelService.getChannelContactCount(channelId, req.query, contactFilter, skip);
		if (total > 0) {
			contactList = await this.channelService.getChannelContactsList(channelId, req.query, contactFilter, skip);
			defaultContact = await this.channelService.getDefaultContact(channelId, user, contactFilter);
			if (defaultContact && !includes(contactList.contactIds, ''+defaultContact)) {
				defaultContact = null;
			}
		}

		let newlist=await this.channelService.getrequestid((contactList)?contactList.list:[],user);

		let result = await this.channelHubDetailOutBound.getContactList(
			(newlist) ? newlist : [],
			req.query,
			total,
			defaultContact
		);
		res.status(HttpStatus.OK).send(Object.assign(result));
	}

	@Post('updateChannel/:channelId')
	public async updateChannel(@Param('channelId') channelId, @Req() req, @Res() res, @Body(new ValidationPipe()) updateChannelDto: UpdateChannelDto) {
		let user = req.user;
		console.log('user', user);
		let suggest_Channel:boolean = true;
		let channel = await Channel.findOne({
			where: {
				channelId: channelId,
				isActive: '1',
				deletedAt: null,
				signUpStatusId: appconstant.CHANNEL_APPROVED
			}
		});
		
		if (!channel) {
			throw new ProcessException('channel:getChannel:channelNotFound');
		}

		var counted = false;
		let channelJCTId = (updateChannelDto.channelJCTId) ? updateChannelDto.channelJCTId : null;
		console.log('channelJCTId', channelJCTId);
		if (!channelJCTId && (updateChannelDto.connectionStatusTypeId == appconstant.PENDINGMYPROSPECT || updateChannelDto.connectionStatusTypeId == appconstant.REQUEST_INTRO || updateChannelDto.connectionStatusTypeId == appconstant.DISMISS)) {
			let channelJct = await this.channelHubService.connectChannel(channel, user);
				if (channelJct) {
					updateChannelDto.channelJCTId = +channelJct.channelJCTId;
					suggest_Channel = false;
				}
				counted = true;
			}
			let limitStatus = false;
			let limitcheck = null;
			let limit = null;
			limitcheck = await this.channelHubService.verifyChannelSuggestion(user, updateChannelDto);
			limit = limitcheck.limit;
			if ((updateChannelDto.connectionStatusTypeId == appconstant.PENDINGMYPROSPECT || updateChannelDto.connectionStatusTypeId == appconstant.REQUEST_INTRO)&& channel.channelTypeId!=2) {	
				limitStatus = true;
			}
		    if(( limitStatus == true) && (limit != null && limitcheck.allowed !=-1 && limit >= limitcheck.allowed) && (channel.channelTypeId!=2 || user.channelTypeId !=4)){
				throw new AuthException('Upgrade you plan to make this action');
			}
			 let updateChannel = await this.channelHubService.updateChannelConnect(channelId, updateChannelDto, user, suggest_Channel);
			if (updateChannel) {
				// profile score
				this.channelHubService.profileScore(user.channelId, user.channelTypeId, user.profileStrength);
				if(limitStatus){
				await this.channelHubService.incrementSuggestionCnt(user, updateChannelDto);
				}
				// this.channelIndexService.hubspotCompanyUpdate(user.channelId, user);
				let suggestion_lst = await this.channelHubService.checkLimit(user.contactId);
				// ChannelIndexService.updateChannelRelationCount(+user.channelId);
				return res.status(HttpStatus.OK).send(Object.assign({channelJCTId:updateChannel.channelJCTId,suggestion_lst:suggestion_lst}));
			}
	 }
	 @Get('dimissReasons')
	 public async dimissReasons(@Req() req, @Res() res) {
		 let dimissReasons = await this.channelHubService.dimissReasons();
		 if (dimissReasons) {
			 return res.status(HttpStatus.OK).send(dimissReasons);
		 }
		 throw new ProcessException('channelHub:dimissReasons:notFound');
	 }

	 @Post('channelBrands/:channelId')
	 public async channelBrands(@Param('channelId') channelId, @Res() res, @Req() req) {
		 let user = req.user;//this.context.get('user');
 
		 let channelBrands = await this.channelHubService.channelBrands(+channelId, user);
 
		 res.status(HttpStatus.OK).send(channelBrands);
	 }

	 @Get('getSugOutboundcount/:contactId')
	 public async getSugOutboundcount(@Param('contactId') contactId, @Req() req, @Res() res) {
		let sug_Count = await this.channelHubService.checkLimit(contactId);
		 res.status(HttpStatus.OK).send(sug_Count);
	 }
	 @Post('chfavorite/:groupid')
	 async getChannelFavorite(@Req() req, @Param('groupid') groupid: number, @Res() res, @Body(new ValidationPipe()) channelFilter: any){
	 
		 let user = req.user;
		 let contactId=user.contactId;
		 let channelId=user.channelId;let userId=user.userId;
		 let filterch=[];let aggregation={};let result;
		 //select all non member channel no's		
		 let channelquery = 'select array_agg("channelid") as ch from "tblproductwishlist" where "userid"=' + user.userId + ' and "channelgroupid"=' + groupid + 'and "deletedat" is null';
		 let  chlist= await this.sequelizeInstance.query(channelquery, {
			 type: QueryTypes.SELECT,
			 raw: true			
		 }).then((result) => {			
				 return result[0].ch;
		 });
		 
		 if(channelFilter.brandId||channelFilter.locationId||channelFilter.countryId||channelFilter.specialityId||channelFilter.productCategoryId||channelFilter.retailId)
		 {
 
			 if(channelFilter.brandId)
			 {
				 await this.sequelizeInstance.query('select "oldChannelId" as ch from "tblOldChannelBrand" where "brandId" in(:brandid) and "oldChannelId" in(:channelid)', {
					 type: QueryTypes.SELECT,
					 raw: true,
					 replacements: {
						 channelid:chlist,
						 branid:channelFilter.brandId
					 }
				 }).then((result) => {
					 filterch.push(...result[0].ch);
				 });	
			 }
			 if(channelFilter.productCategoryId)
			 {
				 await this.sequelizeInstance.query('select "oldChannelId" as ch from "tblOldChannelProductCategorie" cpc where cpc."productCategorieId" in(:productcatid) and "oldChannelId" in(:channelid) ', {
					 type: QueryTypes.SELECT,
					 raw: true,
					 replacements: {
						 channelid:(filterch)?filterch:chlist,
						 productcatid:channelFilter.productCategoryId
					 }
				 }).then((result) => {
					 filterch.push(...result[0].ch);
				 });	
			 }
			 if(channelFilter.retailId)
			 {
				 await this.sequelizeInstance.query('select "oldChannelId" as ch from "tblOldChannelCustomerProfile" where "profileTypeId" in(:retailprofileid) and "oldChannelId" in(:channelid)', {
					 type: QueryTypes.SELECT,
					 raw: true,
					 replacements: {
						 channelid:(filterch)?filterch:chlist,
						 retailprofileid:channelFilter.retailId
					 }
				 }).then((result) => {
					 filterch.push(...result[0].ch);
				 });	
			 }
			 if(channelFilter.countryId)
			 {
				 await this.sequelizeInstance.query('select "oldChannelId" as ch from from "tblOldChannel" where "countryId" in(:countryid) and "oldChannelId" in(:channelid)', {
					 type: QueryTypes.SELECT,
					 raw: true,
					 replacements: {
						 channelid:(filterch)?filterch:chlist,
						 countryid:channelFilter.countryId
					 }
				 }).then((result) => {
					 filterch.push(...result[0].ch);
				 });	
			 }
			 if(channelFilter.specialityId)
			 {
				 await this.sequelizeInstance.query('select "oldChannelId" from "tblOldContact" where "oldContactId" in(Select "oldContactId" from "tblOldContactInterestedProductCategory" where "productCategorieId" in(:specialityid)) and "oldChannelId" in(:channelid)', {
					 type: QueryTypes.SELECT,
					 raw: true,
					 replacements: {
						 channelid:(filterch)?filterch:chlist,
						 specialityid:channelFilter.specialityId
					 }
				 }).then((result) => {
					 filterch.push(...result[0].ch);
				 });	
			 }
			 if(channelFilter.locationId)
			 {
				 await this.sequelizeInstance.query('select "oldChannelId" as ch from from "tblOldChannel" where "countryId" in(:locationid) and "oldChannelId" in(:channelid)', {
					 type: QueryTypes.SELECT,
					 raw: true,
					 replacements: {
						 channelid:(filterch)?filterch:chlist,
						 locationid:channelFilter.locationId
					 }
				 }).then((result) => {
					 filterch.push(...result[0].ch);
				 });	
			 }
			  chlist = chlist.filter(ch => {return filterch.includes(+ch);})
		 }
		 else if(channelFilter.page==0)
		 {
			 let brandquery='select tb."brandName" as item_name,tb."id" as brandid from "tblBrand" tb '
			 +'inner join "tblDocument" td on td."documentId"=tb."documentId" where id in (select "brandId" from "tblChannelBrand" where "channelId" in(:channelid))  ';	
 
			 let productcatquery='Select pc."productCategorie" as item_name,pc."productCategorieId" as productcatid from "tblChannelProductCategorie" cpc '
			 +'inner join "tblProductCategories" pc on pc."productCategorieId" = cpc."productCategorieId" '
			 +'where cpc."channelId" in(:channelid)';
 
			 let retailcustquery='Select pt."profileType" as item_name,pt."profileTypeId" as retailcustid from "tblProfileType" pt where "profileTypeId" in '
			 +'(select "profileTypeId" from "tblChannelCustomerProfile" where "channelId" in(:channelid)) ';
 
			 let countryquery='select tc."country" as item_name,ch."countryId" as countryid from "tblChannel" ch inner join "tblCountry" tc on tc."countryId"=ch."countryId" '
			 +'where ch."channelId" in(:channelid) ';
 
			 let intcatquery='select "productCategorie" as item_name,"productCategorieId" as procatid  from "tblProductCategories"  where "productCategorieId" in '
			 +'(Select cpc."productCategorieId" from "tblContactInterestedProductCategory" cpc where cpc."contactId" in(select "contactId" from "tblContact" where "channelId" in(:channelid) and "deletedAt" is null)) ';
 
			 let lookupquery='select "country" as item_name,"countryId" as countryid from "tblCountry"  where "countryId" in '
			 +'(Select pc."countryId" from "tblContactLocation" cpc inner join "tblRegionCountryJCT" pc on pc."regionCountryJCTId" = cpc."regionCountryJCTId" '
			 + 'inner join "tblChannel" ch on ch."channelId" in(:channelid) where cpc."channelId" in(:channelid) and cpc."locationType"=\'SELLING\') ';
			 
			 result= await this.sequelizeInstance.query(brandquery, {
				 type: QueryTypes.SELECT,
				 raw: true,
				 replacements: {
					 channelid:chlist
				 }
			 })
			 let brandlist=_.uniqBy(result, 'item_name');
 
			 result= await this.sequelizeInstance.query(productcatquery, {
				 type: QueryTypes.SELECT,
				 raw: true,
				 replacements: {
					 channelid:chlist
				 }
			 })		
			 let productcatlist=_.uniqBy(result, 'item_name');
 
			 result= await this.sequelizeInstance.query(retailcustquery, {
				 type: QueryTypes.SELECT,
				 raw: true,
				 replacements: {
					 channelid:chlist
				 }
			 })		
			 let retailcustlist=_.uniqBy(result, 'item_name');
 
			 result= await this.sequelizeInstance.query(countryquery, {
				 type: QueryTypes.SELECT,
				 raw: true,
				 replacements: {
					 channelid:chlist
				 }
			 })		
			 let countrylist=_.uniqBy(result, 'item_name');
 
			 result= await this.sequelizeInstance.query(intcatquery, {
				 type: QueryTypes.SELECT,
				 raw: true,
				 replacements: {
					 channelid:chlist
				 }
			 })		
			 let intcatlist=_.uniqBy(result, 'item_name');
 
			 result= await this.sequelizeInstance.query(lookupquery, {
				 type: QueryTypes.SELECT,
				 raw: true,
				 replacements: {
					 channelid:chlist
				 }
			 })		
			 let lookuplist=_.uniqBy(result, 'item_name');
 
			 aggregation={
				 brandIds:brandlist,
				 locactionIds:lookuplist,
				 specialIds:intcatlist,
				 countryIds:countrylist,
				 retailIds:retailcustlist,
				 productIds:productcatlist
			 };	
		 }
				 
		 let companyquery = 'Select ch."channelId" as cid,case when chd."turnover" is null then chd."turnoverRange" when chd."turnover" is not null then cast(chd."turnover" as text) end as turnover,ch."companyName",ch."countryId", tc."country", tc."isoCode", (select "profileType" from "tblProfileType" where "profileTypeId"=ch."profileTypeId"), cht."channelType",ch."channelTypeId", '
		 +'chd."detailDesc", chd."webSiteUrl", chd."startup",(select array_to_json(array_agg(row_to_json(a))) from  (select SUBSTRING ( cast(chd."maturitylevel" as TEXT) ,2 , length(cast(chd."maturitylevel" as TEXT))-2) as item_name)a) as looking_startups, '		 
		 +'(Select td."documentPath" as logo from "tblChannelDetail" chd inner join "tblDocument" td on td."documentId" in(chd."channelLogoId") where chd."channelId"=ch."channelId"), '
		 +'(select array_to_json(array_agg(row_to_json(br))) from (select tb."brandName" as item_name,td."documentPath" as image_Url from "tblBrand" tb inner join "tblDocument" td on td."documentId"=tb."documentId" where "id" in(select "brandId" from "tblChannelBrand" where "channelId"=ch."channelId"))br) as brand, '
		 +'(select array_to_json(array_agg(row_to_json(x))) from (Select pc."productCategorie" as item_name from "tblChannelProductCategorie" cpc inner join "tblProductCategories" pc on pc."productCategorieId" = cpc."productCategorieId" where cpc."channelId"=ch."channelId")x) as company_cat, '
		 +'(select array_to_json(array_agg(row_to_json(y))) from (select tp."product" as item_name,td."documentPath" as image_url from "tblProduct" as tp inner join "tblDocument" as td on td."documentId" in(select "imageid" from "tblproductimage" where "productimageid"=tp."defaultimageid") where "channelId"=ch."channelId" and "deletedat" is null and "isactive"=\'1\')y) as key_products, '
		 +'(select array_to_json(array_agg(row_to_json(z))) from (Select pt."profileType" as item_name from "tblProfileType" pt where "profileTypeId" in (select "profileTypeId" from "tblChannelCustomerProfile" cp where cp."channelId"=ch."channelId"))z) as retail_customer_profile, '
		 +'(select array_to_json(array_agg(row_to_json(t))) from (select CASE when tkjct."existchannelid" is not null then json_build_object(\'partner_type\',tkjct."partner_type",\'id\',tkjct."keypartnerid",\'companyName\',tc."companyName",\'documentPath\',tcdoc."documentPath") end as Active, '
		 +'CASE when tkjct."oldchannelid" is not null then json_build_object(\'partner_type\',tkjct."partner_type",\'id\',tkjct."keypartnerid",\'companyName\',toc."companyName",\'documentPath\',tocdoc."documentPath") end as Old, '
		 +'CASE when tkjct."channelkeypartnerid" is not null then json_build_object(\'partner_type\',tkjct."partner_type",\'id\',tkjct."keypartnerid",\'companyName\',tckr."partnername",\'documentPath\',tckrd."documentPath") end as Custom '
		 +'from "tblKeyPartnerJCT" tkjct left join "tblChannel" tc on tc."channelId" = tkjct."existchannelid" left join "tblChannelDetail" tcd on tcd."channelId" = tc."channelId" left join "tblDocument" tcdoc on tcdoc."documentId" = tcd."channelLogoId" '
		 +'left join "tblOldChannel" toc on toc."oldChannelId" = tkjct."oldchannelid" left join "tblOldChannelDetail" tocd on tocd."oldChannelId" = toc."oldChannelId" left join "tblDocument" tocdoc on tocdoc."documentId" = tocd."channelLogoId" '
		 +'left join "tblchannelkeypartner" tckr on tckr."channelkeypartnerid" = tkjct."channelkeypartnerid"	left join "tblDocument" tckrd on tckrd."documentId" = tckr."documentid"	where tkjct."channelid" =ch."oldChannelId"  and tkjct."partner_type" in(4,3) and tkjct."deletedat" is null)t) as key_retailer, '
		 
		 +'(select array_to_json(array_agg(row_to_json(con))) from(select toc."contactId",toc."firstName",toc."lastName",toc."jobTitle",toc."email", '
		 +'(select "country" from "tblCountry" where "countryId"=(select "countryId" from "tblContact" where "contactId"=toc."contactId" )), '
		 +'(Select td."documentPath"  from "tblContact" utc inner join "tblDocument" td on td."documentId" = utc."documentId" where utc."contactId" = toc."contactId"), '			
		 
		 +'(select array_to_json(array_agg(row_to_json(p))) from (select "productCategorie" as item_name from "tblProductCategories"  where "productCategorieId" in '
		 +'(Select cpc."productCategorieId" from "tblContactProductCategorie" cpc where cpc."contactId" = toc."contactId"))p) as ext_pname, '
 
		 +'(select array_to_json(array_agg(row_to_json(i)))  from (select "productCategorie" from "tblProductCategories" where "productCategorieId" in (Select cpc."productCategorieId" from "tblContactInterestedProductCategory" cpc where cpc."contactId" = toc."contactId"))i) as int_pname, '
		 
		 +'(select array_to_json(array_agg(row_to_json(x))) from (select "country" as item_name from "tblCountry"  where "countryId" in (Select pc."countryId" from "tblContactLocation" cpc inner join "tblRegionCountryJCT" pc on pc."regionCountryJCTId" = cpc."regionCountryJCTId" '
		 +'inner join "tblChannel" ch on ch."channelId"=toc."channelId" where cpc."channelId" = toc."channelId" and cpc."locationType"=\'TARGET\'))x) as look_location '
		 +'from "tblContact" toc where toc."contactId"=(select tu."contactId" from "tblUser" tu where tu."contactId" in(select tcc."contactId" from "tblContact" tcc where tcc."channelId"=ch."channelId" and tcc."isDefaultContact"=\'1\')and tu."userId" in(select "userId" from "tblUserRole" rl where rl."roleId"=1) ))con) as contacts '
 
		 +'from "tblChannel" ch inner join "tblChannelDetail" chd on chd."channelId" = ch."channelId" inner join "tblCountry" tc on tc."countryId" = ch."countryId" '
			 + 'inner join "tblChannelType"cht on cht."channelTypeId" = ch."channelTypeId" where ch."channelId" in(:channelid)';
			 console.log('chlist',chlist);
		if(chlist && chlist.length>0){
		 chlist=chlist.slice(channelFilter.offset).slice(0,channelFilter.limit);

		 result= await this.sequelizeInstance.query(companyquery, {
			 type: QueryTypes.SELECT,			
			 raw: true,			
			  replacements: {
				  channelid:chlist
			 }
		 });		
 		}else 
		 result=[];
		 
		 let resultformat={
			 result:result,
			 aggregation:aggregation			
		 }	
		 res.status(HttpStatus.OK).send(resultformat);	
	 }

	 @Post('favoriteList/:viewId/:planId')
	 async getChannelFavoriteList(@Req() req, @Param('groupid') groupid: number, @Res() res, @Body(new ValidationPipe()) channelFilter: any, @Param('viewId') isVariableWidth, @Param('planId') planId){
	 
		 let user = req.user;
		 let contactId=user.contactId;
		 let channelId=user.channelId;let userId=user.userId;
		 let filterch=[];let aggregation={};let result;
		 //select all non member channel no's		
		 let channelquery='select array_agg("channelid") as ch from "tblproductwishlist" where "userid"=:userId and "deletedat" is null';
		 let  chlist= await this.sequelizeInstance.query(channelquery, {
			 type: QueryTypes.SELECT,
			 raw: true,
			 replacements:{
				 userId: user.userId
			 }			
		 }).then((result) => {			
				 return result[0].ch;
		 });
		 

		 if(channelFilter.brandId||channelFilter.locationId||channelFilter.countryId||channelFilter.specialityId||channelFilter.productCategoryId||channelFilter.retailId)
		 {
 
			 if(channelFilter.brandId)
			 {
				 await this.sequelizeInstance.query('select "oldChannelId" as ch from "tblOldChannelBrand" where "brandId" in(:brandid) and "oldChannelId" in(:channelid)', {
					 type: QueryTypes.SELECT,
					 raw: true,
					 replacements: {
						 channelid:chlist,
						 branid:channelFilter.brandId
					 }
				 }).then((result) => {
					 filterch.push(...result[0].ch);
				 });	
			 }
			 if(channelFilter.productCategoryId)
			 {
				 await this.sequelizeInstance.query('select "oldChannelId" as ch from "tblOldChannelProductCategorie" cpc where cpc."productCategorieId" in(:productcatid) and "oldChannelId" in(:channelid) ', {
					 type: QueryTypes.SELECT,
					 raw: true,
					 replacements: {
						 channelid:(filterch)?filterch:chlist,
						 productcatid:channelFilter.productCategoryId
					 }
				 }).then((result) => {
					 filterch.push(...result[0].ch);
				 });	
			 }
			 if(channelFilter.retailId)
			 {
				 await this.sequelizeInstance.query('select "oldChannelId" as ch from "tblOldChannelCustomerProfile" where "profileTypeId" in(:retailprofileid) and "oldChannelId" in(:channelid)', {
					 type: QueryTypes.SELECT,
					 raw: true,
					 replacements: {
						 channelid:(filterch)?filterch:chlist,
						 retailprofileid:channelFilter.retailId
					 }
				 }).then((result) => {
					 filterch.push(...result[0].ch);
				 });	
			 }
			 if(channelFilter.countryId)
			 {
				 await this.sequelizeInstance.query('select "oldChannelId" as ch from from "tblOldChannel" where "countryId" in(:countryid) and "oldChannelId" in(:channelid)', {
					 type: QueryTypes.SELECT,
					 raw: true,
					 replacements: {
						 channelid:(filterch)?filterch:chlist,
						 countryid:channelFilter.countryId
					 }
				 }).then((result) => {
					 filterch.push(...result[0].ch);
				 });	
			 }
			 if(channelFilter.specialityId)
			 {
				 await this.sequelizeInstance.query('select "oldChannelId" from "tblOldContact" where "oldContactId" in(Select "oldContactId" from "tblOldContactInterestedProductCategory" where "productCategorieId" in(:specialityid)) and "oldChannelId" in(:channelid)', {
					 type: QueryTypes.SELECT,
					 raw: true,
					 replacements: {
						 channelid:(filterch)?filterch:chlist,
						 specialityid:channelFilter.specialityId
					 }
				 }).then((result) => {
					 filterch.push(...result[0].ch);
				 });	
			 }
			 if(channelFilter.locationId)
			 {
				 await this.sequelizeInstance.query('select "oldChannelId" as ch from from "tblOldChannel" where "countryId" in(:locationid) and "oldChannelId" in(:channelid)', {
					 type: QueryTypes.SELECT,
					 raw: true,
					 replacements: {
						 channelid:(filterch)?filterch:chlist,
						 locationid:channelFilter.locationId
					 }
				 }).then((result) => {
					 filterch.push(...result[0].ch);
				 });	
			 }
			  chlist = chlist.filter(ch => {return filterch.includes(+ch);})
		 }
		 else if(channelFilter.page==0)
		 {
			 let brandquery='select tb."brandName" as item_name,tb."id" as brandid from "tblBrand" tb '
			 +'inner join "tblDocument" td on td."documentId"=tb."documentId" where id in (select "brandId" from "tblChannelBrand" where "channelId" in(:channelid))  ';	
 
			 let productcatquery='Select pc."productCategorie" as item_name,pc."productCategorieId" as productcatid from "tblChannelProductCategorie" cpc '
			 +'inner join "tblProductCategories" pc on pc."productCategorieId" = cpc."productCategorieId" '
			 +'where cpc."channelId" in(:channelid)';
 
			 let retailcustquery='Select pt."profileType" as item_name,pt."profileTypeId" as retailcustid from "tblProfileType" pt where "profileTypeId" in '
			 +'(select "profileTypeId" from "tblChannelCustomerProfile" where "channelId" in(:channelid)) ';
 
			 let countryquery='select tc."country" as item_name,ch."countryId" as countryid from "tblChannel" ch inner join "tblCountry" tc on tc."countryId"=ch."countryId" '
			 +'where ch."channelId" in(:channelid) ';
 
			 let intcatquery='select "productCategorie" as item_name,"productCategorieId" as procatid  from "tblProductCategories"  where "productCategorieId" in '
			 +'(Select cpc."productCategorieId" from "tblContactInterestedProductCategory" cpc where cpc."contactId" in(select "contactId" from "tblContact" where "channelId" in(:channelid) and "deletedAt" is null)) ';
 
			 let lookupquery='select "country" as item_name,"countryId" as countryid from "tblCountry"  where "countryId" in '
			 +'(Select pc."countryId" from "tblContactLocation" cpc inner join "tblRegionCountryJCT" pc on pc."regionCountryJCTId" = cpc."regionCountryJCTId" '
			 + 'inner join "tblChannel" ch on ch."channelId" in(:channelid) where cpc."channelId" in(:channelid) and cpc."locationType"=\'SELLING\') ';
			 
			 result= await this.sequelizeInstance.query(brandquery, {
				 type: QueryTypes.SELECT,
				 raw: true,
				 replacements: {
					 channelid:chlist
				 }
			 })
			 let brandlist=_.uniqBy(result, 'item_name');
 
			 result= await this.sequelizeInstance.query(productcatquery, {
				 type: QueryTypes.SELECT,
				 raw: true,
				 replacements: {
					 channelid:chlist
				 }
			 })		
			 let productcatlist=_.uniqBy(result, 'item_name');
 
			 result= await this.sequelizeInstance.query(retailcustquery, {
				 type: QueryTypes.SELECT,
				 raw: true,
				 replacements: {
					 channelid:chlist
				 }
			 })		
			 let retailcustlist=_.uniqBy(result, 'item_name');
 
			 result= await this.sequelizeInstance.query(countryquery, {
				 type: QueryTypes.SELECT,
				 raw: true,
				 replacements: {
					 channelid:chlist
				 }
			 })		
			 let countrylist=_.uniqBy(result, 'item_name');
 
			 result= await this.sequelizeInstance.query(intcatquery, {
				 type: QueryTypes.SELECT,
				 raw: true,
				 replacements: {
					 channelid:chlist
				 }
			 })		
			 let intcatlist=_.uniqBy(result, 'item_name');
 
			 result= await this.sequelizeInstance.query(lookupquery, {
				 type: QueryTypes.SELECT,
				 raw: true,
				 replacements: {
					 channelid:chlist
				 }
			 })		
			 let lookuplist=_.uniqBy(result, 'item_name');
 
			 aggregation={
				 brandIds:brandlist,
				 locactionIds:lookuplist,
				 specialIds:intcatlist,
				 countryIds:countrylist,
				 retailIds:retailcustlist,
				 productIds:productcatlist
			 };	
		 }
				 
		 let companyquery = 'Select (select distinct("matchingScore") from "tblChannelJCT" where (("partnerChannelId"=41 and "channelId"=779) or ("partnerChannelId"=779 and "channelId"=41)) and "deletedat" is null and "connectionStatusTypeId"=1 and "matchingScore" is not null) as matchingScore,ch."channelId" as cid,case when chd."turnover" is null then chd."turnoverRange" when chd."turnover" is not null then cast(chd."turnover" as text) end as turnover,ch."companyName" as companyname,ch."countryId", tc."country", tc."isoCode", pt."profileType", cht."channelType" as channeltype,ch."channelTypeId", '
			 + 'chd."detailDesc" as detaildesc, chd."webSiteUrl" as websiteurl, chd."startup",(select array_to_json(array_agg(row_to_json(a))) from  (select SUBSTRING ( cast(chd."maturitylevel" as TEXT) ,2 , length(cast(chd."maturitylevel" as TEXT))-2) as item_name)a) as looking_startups, '
		 +'(Select td."documentPath" as logo from "tblChannelDetail" chd inner join "tblDocument" td on td."documentId" in(chd."channelLogoId") where chd."channelId"=ch."channelId"), '
		 +'(select array_to_json(array_agg(row_to_json(br))) from (select tb."brandName" as item_name,td."documentPath" as image_Url from "tblBrand" tb inner join "tblDocument" td on td."documentId"=tb."documentId" where "id" in(select "brandId" from "tblChannelBrand" where "channelId"=ch."channelId"))br) as brand, '
		 +'(select array_to_json(array_agg(row_to_json(x))) from (Select pc."productCategorie" as item_name from "tblChannelProductCategorie" cpc inner join "tblProductCategories" pc on pc."productCategorieId" = cpc."productCategorieId" where cpc."channelId"=ch."channelId")x) as company_cat, '
		 +'(select array_to_json(array_agg(row_to_json(y))) from (select tp."product" as item_name,td."documentPath" as image_url from "tblProduct" as tp inner join "tblDocument" as td on td."documentId" in(select "imageid" from "tblproductimage" where "productimageid"=tp."defaultimageid") where "channelId"=ch."channelId" and "deletedat" is null and "isactive"=\'1\')y) as key_products, '
		 +'(select array_to_json(array_agg(row_to_json(z))) from (Select pt."profileType" as item_name from "tblProfileType" pt where "profileTypeId" in (select "profileTypeId" from "tblChannelCustomerProfile" cp where cp."channelId"=ch."channelId"))z) as retail_customer_profile, '
		 +'(select array_to_json(array_agg(row_to_json(t))) from (select CASE when tkjct."existchannelid" is not null then json_build_object(\'partner_type\',tkjct."partner_type",\'id\',tkjct."keypartnerid",\'companyName\',tc."companyName",\'documentPath\',tcdoc."documentPath") end as Active, '
		 +'CASE when tkjct."oldchannelid" is not null then json_build_object(\'partner_type\',tkjct."partner_type",\'id\',tkjct."keypartnerid",\'companyName\',toc."companyName",\'documentPath\',tocdoc."documentPath") end as Old, '
		 +'CASE when tkjct."channelkeypartnerid" is not null then json_build_object(\'partner_type\',tkjct."partner_type",\'id\',tkjct."keypartnerid",\'companyName\',tckr."partnername",\'documentPath\',tckrd."documentPath") end as Custom '
		 +'from "tblKeyPartnerJCT" tkjct left join "tblChannel" tc on tc."channelId" = tkjct."existchannelid" left join "tblChannelDetail" tcd on tcd."channelId" = tc."channelId" left join "tblDocument" tcdoc on tcdoc."documentId" = tcd."channelLogoId" '
		 +'left join "tblOldChannel" toc on toc."oldChannelId" = tkjct."oldchannelid" left join "tblOldChannelDetail" tocd on tocd."oldChannelId" = toc."oldChannelId" left join "tblDocument" tocdoc on tocdoc."documentId" = tocd."channelLogoId" '
		 +'left join "tblchannelkeypartner" tckr on tckr."channelkeypartnerid" = tkjct."channelkeypartnerid"	left join "tblDocument" tckrd on tckrd."documentId" = tckr."documentid"	where tkjct."channelid" =ch."oldChannelId"  and tkjct."partner_type" in(4,3) and tkjct."deletedat" is null)t) as key_retailer, '
		 
		 +'(select array_to_json(array_agg(row_to_json(con))) from(select toc."contactId",toc."firstName",toc."lastName",toc."jobTitle",toc."email", '
		 +'(select "country" from "tblCountry" where "countryId"=(select "countryId" from "tblContact" where "contactId"=toc."contactId" )), '
		 +'(Select td."documentPath"  from "tblContact" utc inner join "tblDocument" td on td."documentId" = utc."documentId" where utc."contactId" = toc."contactId"), '			
		 
		 +'(select array_to_json(array_agg(row_to_json(p))) from (select "productCategorie" as item_name from "tblProductCategories"  where "productCategorieId" in '
		 +'(Select cpc."productCategorieId" from "tblContactProductCategorie" cpc where cpc."contactId" = toc."contactId"))p) as ext_pname, '
 
		 +'(select array_to_json(array_agg(row_to_json(i)))  from (select "productCategorie" from "tblProductCategories" where "productCategorieId" in (Select cpc."productCategorieId" from "tblContactInterestedProductCategory" cpc where cpc."contactId" = toc."contactId"))i) as int_pname, '
		 
		 +'(select array_to_json(array_agg(row_to_json(x))) from (select "country" as item_name from "tblCountry"  where "countryId" in (Select pc."countryId" from "tblContactLocation" cpc inner join "tblRegionCountryJCT" pc on pc."regionCountryJCTId" = cpc."regionCountryJCTId" '
		 +'inner join "tblChannel" ch on ch."channelId"=toc."channelId" where cpc."channelId" = toc."channelId" and cpc."locationType"=\'TARGET\'))x) as look_location '
		 +'from "tblContact" toc where toc."contactId"=(select tu."contactId" from "tblUser" tu where tu."contactId" in(select tcc."contactId" from "tblContact" tcc where tcc."channelId"=ch."channelId" and tcc."isDefaultContact"=\'1\')and tu."userId" in(select "userId" from "tblUserRole" rl where rl."roleId"=1) ))con) as contacts, wlst."channelgroupid" as favorite '
 
		 +'from "tblChannel" ch inner join "tblChannelDetail" chd on chd."channelId" = ch."channelId" inner join "tblCountry" tc on tc."countryId" = ch."countryId" '
		 +'inner join "tblProfileType"pt on pt."profileTypeId" = ch."profileTypeId" inner join "tblChannelType"cht on cht."channelTypeId" = ch."channelTypeId" ' +' inner join "tblproductwishlist" wlst on wlst."channelid" = ch."channelId" where "userid"='+user.userId+' and "deletedat" is null'; ' and ch."channelId" in(:channelid)'
		 
 
		 let total = chlist.length;

		 chlist=chlist.slice(channelFilter.offset).slice(0,channelFilter.limit);

		 result= await this.sequelizeInstance.query(companyquery, {
			 type: QueryTypes.SELECT,			
			 raw: true,			
			  replacements: {
				  channelid:chlist
			 }
		 });		
 
		 let resultformat={
			 results:result,
			 aggregation:aggregation,
			 total: total
		 }	
		 res.status(HttpStatus.OK).send(resultformat);	
	 }

	 @Get('shareTypeList')
	 public async shareTypeList(@Req() req, @Res() res) {
		 let shareTypeList = await this.channelService.getShareTypeList();
		 return res.status(HttpStatus.OK).send(shareTypeList);
	 }

	 @Post('allcontacts')
    public async getAllContacts(@Res() res, @Req() req, @Body() data){

		let list = await this.channelService.getSearchList1(data, req.user);

		let total = 0;
		total = list.total;
		let channelList = list.result.map(x => x.channel_id);
		console.log('channelList', channelList);
		let contacts = [];
		if(channelList.length)
         contacts = await this.channelService.contactFilterBase1(req.user, channelList, data.order);		
        if(contacts){

			contacts=contacts.slice(data.offset).slice(0,data.limit);
			let result = await this.channelService.getAllContacts(contacts);
			if(result){
				let final = {
					result : result,
					total : total
				}
				return res.status(HttpStatus.OK).send(final);
			}
            
        }
    }
	@Post('buyerNeed')
	public async buyerNeed(@Req() req, @Res() res, @Body() reqneed: any) {		
	await this.channelHubService.saveBuyerNeed(reqneed);
	res.status(HttpStatus.OK).send();
	}

	@Post('profileView/:viewId')
	public async profileView(@Req() req, @Res() res, @Body() data, @Param ('viewId') viewId)
	{
		let user = req.user;
		let total = 0;
		let query = '' ;
		let final = {
			result : [],
			total : 0,
			contactList: []
		};
		let result = [];

		if(viewId == '1')
			query = 'select array_agg("contactId") from "tblUser" where "userId" in (select "user_id" from "dashboard" where "channelid"=:channelId and "visittype"=1) and "isActive"=\'1\' and "deletedAt" is null and "channelId"!=:channelId';
		if(viewId=='2')
			query = 'select array_agg("contactId") from "tblUser" where "userId" in (select "user_id" from "dashboard" where "channelid"=:channelId and "visittype"=2) and "isActive"=\'1\' and "deletedAt" is null and "channelId"!=:channelId';
		if(viewId=='3'){
			query = 'SELECT array_agg("contactId") from "tblUser" where "userId" in (select "userid" FROM "tblproductwishlist" WHERE "channelgroupid" IS NOT NULL AND "channelid" = :channelId  AND "deletedat" IS NULL)';
		}


		let contact = await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT,
			replacements: {
				channelId: user.channelId
			}
		}).then(x => x[0].array_agg);

		let list = await this.channelService.getSearchListProfile(Object.assign(data, {contactId : contact}), user);

		let contactList = contact;

		total = list.total;
		let channelIds = list.result.map(x => x.contactId);


		let temp = [];

		if(channelIds.length){		
				temp = channelIds.slice(data.offset,data.limit);
		}
         
		// let contact_ids = [];
		// if(channelIds.length){		
		// 		temp = channelIds.slice(data.offset,data.limit);
		// 		temp.map(x=>contact_ids.push({contact_id:x}));
		// 		console.log('contact_ids', contact_ids);
		// }


		if(temp.length){
			result = await this.channelService.getAllContacts(temp);
			let newlist=await this.channelService.getrequestid(result?result:[],user);
			 final = {
				result : newlist,
				total : total,
				contactList: contactList
			}
		}
			 
		res.status(HttpStatus.OK).send(final);
	}

	@Get('billingAddress')
	public async getBillingAddress(@Res() res, @Req() req){
		let user = req.user;

		let result = await this.channelService.billingAddress(user);

		let finalres = {};

		if(result.length){
			finalres = {
				address: result[0].billAddress.address,
				addressId: result[0].billAddressId,
				city: result[0].billAddress.city,
				country: result[0].billAddress.country,
				countryId: result[0].billAddress.countryId,
				postalCode: result[0].billAddress.postalCode,
				state: result[0].billAddress.state
			}
		}

		res.status(HttpStatus.OK).json(finalres);
	}
	@Get('eventRegister/:contactId')
	public async eventRegister(@Param('contactId') contactId,@Req() req, @Res() res) {
		//console.log("request:",req.headers.mytoken);
	let result = await this.channelHubService.eventRegister(contactId);
	return res.status(HttpStatus.OK).send(result);
	}
	@Get('setEventContact/:id')
	public async setEventContact(@Param('id') id, @Res() res) {
		await this.channelService.isEvent(id, '1');
		return res.status(HttpStatus.OK).send();
	}

	@Get('getEventListing')
	public async getEventListing(@Req() req, @Res() res){
		let result = await this.channelService.getEventListing(req.user.channelId, req.user.contactId);
		return res.status(HttpStatus.OK).send(result);
	}

	@Get('eventPackages/:eventId')
	public async getEventPackages(@Param('eventId') eventId, @Req() req, @Res() res){
		let result = await this.channelService.getEventPackages(req.user, eventId);
		return res.status(HttpStatus.OK).send(result);
	}

	//getEventChannelDetails
    @Get('getEventChannelDetails/:eventId')
    public async getEventChannelDetails(@Req() req, @Res() res, @Param('eventId') eventId)
    {
        let result = await this.channelService.getEventChannelDetails(req.user.channelId, eventId)
        res.status(HttpStatus.OK).json(result);
    }

	//eventRegistration
	@Post('postEventRegistration')
	public async postEventRegistration(@Req() req, @Res() res, @Body() data){
		let result = await this.channelService.postEventRegistraion(req.user, data);
		res.status(HttpStatus.OK).send(result);
	}

	  //sending to eventCatalyst from fo
	  @Post('foToEventCatalyst')
	  public async foToEventCatalyst(@Req() req, @Res() res, @Body() data){
		  let result = await this.channelService.foToEventCatalyst(data);
		  res.status(HttpStatus.OK).json(result);
	  }
  
	  //setting event contact
	  @Post('setEventParticipantStatus/')
	  public async setEventParticipantStatus(@Req() req, @Res() res, @Body() data) {
		  await this.channelService.setEventParticipantStatus(data.eventId, data.channelId, data.contactId);
		  return res.status(HttpStatus.OK).send();
	  }

	  //getting eventChannelDetails for fo
	  @Get('getSelectedEventDetails/:eventId')
	  public async getSelectedEventDetails(@Req() req, @Res() res, @Param('eventId') eventId){
		  let result = await this.channelService.getSelectedEventDetails(req.user, eventId);
		  return res.status(HttpStatus.OK).send(result);
	  }

	  @Post('eventPayment')
	  async eventpaymentTransaction(@Req() req, @Res() res) {
		let result = await this.channelService.eventRegistration(req.user);
		console.log(result);
		return res.status(HttpStatus.OK).send(result);
	  }

	  @Get('channelCountry')
	  async getChannelCountry(@Req() req, @Res() res){
		  let query = 'select "countryId", country, "phoneCode", "isoCode" from "tblCountry" where "countryId"=(select "countryId" from "tblChannel" where "channelId"=:channelId limit 1)';
		  let result = await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT,           
			replacements: {
				channelId: req.user.channelId,
			}

		  }).then(res => res[0]);
		  return res.status(HttpStatus.OK).send(result);
	  }
  
	  @Get('contactDetails/:contactId')
	  public async getContactProfile(@Param('contactId') contactId, @Req() req, @Res() res) {
		  let user = req.user;

		  let contact = await this.channelService.getContactDetail(contactId);
  
		  let skip = true;
		  let channelContactFilter: any = {};
			console.log('contact',contact);
			console.log('user',user);
		  if (user.channelTypeId == Channel.CHANNEL_TYPE_HUB || +user.channelId == +contact.channelId) {
			  skip = false;
		  }
  
		  if (skip) {
			  let ChannelTypeId = await this.getChannelType(+contact.channelId, 'channelTypeId');
			  if (ChannelTypeId == Channel.CHANNEL_TYPE_VENDOR || (
				  ChannelTypeId == Channel.CHANNEL_TYPE_DISTRIBUTOR && user.channelTypeId == +Channel.CHANNEL_TYPE_RETAILER
			  )) {
				  channelContactFilter.contactType = 'SELLING';
			  } else if (ChannelTypeId == Channel.CHANNEL_TYPE_RETAILER || (
				  ChannelTypeId == Channel.CHANNEL_TYPE_DISTRIBUTOR && user.channelTypeId == +Channel.CHANNEL_TYPE_VENDOR
			  )) {
				  channelContactFilter.contactType = 'SOURCING';
			  }
		  }
  
		//   let defaultContact = await this.channelHubService.getDefaultContact(contact.channelId, user, channelContactFilter);
  
		//   let relation=await this.channelHubService.getrequestid(contactId,user);
		  res.status(HttpStatus.OK).send(Object.assign(contact.toJSON(), {
			  productCategories: contact.productCategories,
			  IntProductCategories: contact.IntProductCategories,
			  locations: contact.locations,
			//   defaultContact: +defaultContact,
			//   relation
		  }));
	  }


}