import { Body, Controller, Get, HttpStatus, Inject, Param, Post, Delete, Put, Query, Req, Res, UseGuards, ValidationPipe } from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import { ContextService } from '../../../context/context.service';
import { SettingDto } from '../dto/setting.dto';
import { UserSettingDto } from '../dto/usersetting.dto';
import {Op, QueryTypes } from "sequelize";

//import { ApiQuery } from '@nestjs/swagger';
//import { ValidationPipe } from 'src/middleware/pipe/validation.pipe';
//import { ProcessException } from 'src/middleware/exception/Process.exception';
import { ChannelSettingService } from '../service/channel.setting.service';
import { ChannelLocationDto } from '../dto/channelLocation.dto';
import { appconstant } from 'src/config/app.constant';
import { TradeInformationUpdateDto } from '../dto/tradeInformation.update.dto';
import { CompanyUpdateDto } from '../dto/company.update.dto';
import { SettingsDto } from '../dto/settings.dto';
import { SearchRegionDto } from '../dto/searchRegion.dto';

const fetch = require("node-fetch");

@UseGuards(AuthGuard('jwt'))
@Controller('setting')
export class ChannelSettingController {
constructor(private channelSettingService:ChannelSettingService,
    @Inject('SequelizeInstance') private readonly sequelizeInstance,
    private context: ContextService) {}
    
@Get('getInitialProfile')	
    public async getInitialProfile(@Req() req,@Res() res) {		
        let result = await this.channelSettingService.getInitialProfile(req.user);
         if (result) {
            res.status(HttpStatus.OK).json(result);
        } 
}    

@Get('getVat/:countryId')	
public async getVat(@Param('countryId') countryId,@Req() req,@Res() res) {		
    let result = await this.channelSettingService.getVat(countryId);
    if (result) {
       res.status(HttpStatus.OK).json(result);
   } 
}

@Post('checkVat')
public async checkVat(@Req() req, @Res() res, @Body() data){
    
    let result = await fetch(process.env.VAT_API_URL+data.vatNumber)
        .then(res => res.json())
        .then(data => {
          if(data.valid){
            return 1;
          }else{
            return 0;
          }
            
        })
        .catch(err => {
          console.log('vat error', err);
          return 0;
        });

      
    res.status(HttpStatus.OK).json({valid: result});
      

    
}

@Get('getContactProduct/:contactId')	
    public async getContactProduct(@Param('contactId') contactId, @Req() req,@Res() res) {		        
        let channelId=req.user.channelId;
        let result = await this.channelSettingService.getContactProduct(contactId,channelId);
         if (result) {
            res.status(HttpStatus.OK).json(result);
        } 
}    


@Get('getProductProfile/:companyId')	
    public async getProductProfile(@Param('companyId') channelId,@Req() req, @Param() data , @Res() res) {		
        let result = await this.channelSettingService.getProductProfile(channelId);
         if (result) {
            res.status(HttpStatus.OK).json(result);
        } 
}    

@Get('getUserProfile/:companyId')	
    public async getUserProfile(@Param('companyId') channelId,@Req() req, @Param() data , @Res() res) {		
        let user = req.user;
        let result = await this.channelSettingService.getUserProfile(user.channelId, user.channelTypeId);
         if (result) {
            res.status(HttpStatus.OK).json(result);
        } 
}    

@Post('company')
async save_Company_Profile(@Body(new ValidationPipe()) profileData:SettingDto, @Req() req,@Res() res){

    //console.log("profileData:",profileData);
        let result=await this.channelSettingService.save_Company_Profile(Object.assign(profileData,{...req.user}));
        return res.status(HttpStatus.OK).send(result);
}

@Post('userprofile')
async save_User_Profile(@Body(new ValidationPipe()) profileData:UserSettingDto, @Req() req,@Res() res){

    //console.log("profileData:",profileData);
        let result=await this.channelSettingService.save_User_Profile(Object.assign(profileData, {channelTypeId: req.user.channelTypeId}));
        return res.status(HttpStatus.OK).send(result);
}

@Get('profileList/:companyId/:typeId')
async getProfileList(@Param('companyId') channelId,@Param('typeId') typeId,@Req() req,@Res() res){
    let channelTypeId=typeId;
    //this.context.set('user', {data:'hello'});
    
        let result=await this.channelSettingService.getProfileList(channelTypeId,channelId);
        return res.status(HttpStatus.OK).send(result);
}

    @Get('checkUserStatus/:companyId')
    async getUserStatus(@Param('companyId') channelId, @Req() req, @Res() res) {
        let typeId = req.user.channelTypeId;
        let result = await this.channelSettingService.getUserStatus(channelId, typeId);
        return res.status(HttpStatus.OK).send(result);
    }


@Get('getCompanyMediaLibrary/:companyId')
async getCompanyMediaLibrary(@Param('companyId') companyId,@Req() req,@Res() res){
    // this.context.set('user', {data:'hello'});
    
        let result=await this.channelSettingService.getCompanyMediaLibrary(companyId, "channel");
        return res.status(HttpStatus.OK).send(result);
}

//getoldchannelmedia
@Get('getOldCompanyMediaLibrary/:companyId')
async getOldCompanyMediaLibrary(@Param('companyId') companyId,@Req() req,@Res() res){
    // this.context.set('user', {data:'hello'});
    
        let result=await this.channelSettingService.getCompanyMediaLibrary(companyId, "oldChannel");
        return res.status(HttpStatus.OK).send(result);
}

@Post('createCompanyLogo')
public async createCompanyLogo(@Body() data, @Res() res) {		
// let user = req.user;
let doc_id = await this.channelSettingService.createCompanyLogo(data, data.user);
    if(doc_id){
        res.status(HttpStatus.OK).send(doc_id);
    }
}

@Post('channelBrandCheck')
public async existingBrandCheck(@Req() req, @Res() res, @Body() data) {
    //console.log('data', data);
    let result = await this.channelSettingService.existingBrandCheck(data);
    res.status(HttpStatus.OK).send(result);
}

@Get('company')
public async company(@Req() req, @Res() res, @Body() data) {
    let user=req.user;
    let result = await this.channelSettingService.getCompanyProfile(user);
    let channelPdf = await this.channelSettingService.getCompanyDocument(user.channelId);
    let chImages = await this.channelSettingService.getCompanyImages(user.channelId);
    let channelImages=[];
    if(chImages.length>0){
    chImages.map(x=>{
        channelImages.push({documentId: x.documentId, companyMediaId: x.companyMediaId, documentUrl:x.document.documentUrl });
    })
    }
    console.log('channelPdf',channelPdf);
    let channelPdfList = [];
    channelPdf.map(x=>{
        let dpath = x.document.documentPath;
        channelPdfList.push({documentId: x.documentId, name: x.name?x.name: dpath.split('/').pop().split('#')[0].split('?')[0], channelDocumentId: x.channelDocumentId,document:{documentUrl:x.document.documentUrl} });
    })

    res.status(HttpStatus.OK).send(Object.assign(result, {channelPdf: channelPdfList},{channelImages:channelImages}));
}

@Get('tradeinfo')
public async tradeinfo(@Req() req, @Res() res) {
    let user=req.user;
    let businessType=user.businessType;
    let ctype=+user.channelTypeId;
    
    let typeid=[];
    if(ctype==2 && businessType==1)
        typeid=[5]
    else if(ctype==2 && businessType==2)
        typeid=[4]
    else if(ctype==2 && businessType==3)
        typeid=[4,5]
    else if(ctype==4)
        typeid=[4]
    else if(ctype==5)
        typeid=[5]
    else if(ctype==3 && businessType==1)
        typeid=[5]
    else if(ctype==3 && businessType==2)
        typeid=[4]
    else if(ctype==3 && businessType==3)
        typeid=[4,5]

        
    let result = await this.channelSettingService.getTradeInfo(user);

    let brandList=[];let productList=[];let profileList=[];

    
    if(result)
    {
        result.map(async x => {

        x.channelBrand?.map(x => {
            let brand={
                id:x.brandId,
                name:x.brand.brandName,
                documentpath:process.env.CDN_URL+x.brand.document.documentPath,
                isNew:0
            }
            brandList.push(brand);
        })


        let cat=x.channelProductCategorie?.map(x => x.productCategorie);
        let group=x.channelProductGroup?.map(x => x.productGroup);
        


    
        group?.map(g => {
            cat?.map(c => {
                if(c.dataValues.groupid==g.dataValues.groupid)
                {
                    //console.log(c);
                    
                    let product={
                        childid:c.dataValues.childid,
                        childname:c.dataValues.childname,
                        groupid:c.dataValues.groupid,
                        groupname:g.dataValues.groupname,
                        status:true
                    }
                    productList.push(product);
                }
                    
            })
        })

    })

    //console.log("result:",result[0]);
    
    let pro=result[0].dataValues.channelCustomerProfile?.map(x => +x.profileTypeId);
    let clssifiid=result[0].dataValues.channelCustomerProfile?.map(x => +x.profileTypeClassificationId);

        let query='select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
        +'when ptc."channelTypeId"=2 then \'vendor\' when ptc."channelTypeId"=5 then \'Reseller\' end as groupname,pt."profileType" as childname,'
        +'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" and pt."profileTypeId" in(:ptype) and ptc."channelTypeId" in(:typeId) '
        if(clssifiid.length)
            query += 'and  ptc."profileTypeClassificationId" in(:clssifiid) ';
        
        profileList=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,           
            replacements: {
                typeId: typeid,
                clssifiid: clssifiid,
                ptype: (req.user.channelTypeId==4||req.user.channelTypeId==5) ? result[0].profileTypeId : pro,     
            }
            }).then(results => {     
                //console.log("result:",pro,req.user.channelTypeId,results);   
               return results;
            });

    let keyRetailerOptions = [];
    let keyRetailerValues = [];

    let keyDistributorOptions = [];
    let keyDistributorValues = [];

    if(result.keyPartners.retailer){
        if(result.keyPartners.retailer.channel?.length)
        {
            result.keyPartners.retailer.channel.map(x => {
                let brand={
                    id:x.channelId,
                    name:x.companyName,
                    documentpath:process.env.CDN_URL+x.companylogo?.documentPath,
                    isNew:0
                }
                keyRetailerOptions.push(Object.assign(x,brand));
            })
        }
    }

    let keyretailer = await this.channelSettingService.getKeyRetailers(+req.user.channelId, appconstant.CHANNEL_TYPE_RETAILER);
    if(keyretailer?.length){
        keyretailer.map(x => {
            let logo = x.active? x.active?.logo?.documentPath : x.old? x.old.logo?.documentPath : x.new? x.new.logo?.logo_name : '';
            let companyName = x.active? x.active.company_name : x.old? x.old.company_name : x.new? x.new.company_name : '';
            let id = x.active? x.active.id : x.old? x.old.id : x.new? x.new.id : '';
            let brand={
                id:id,
                name:companyName,
                documentpath:process.env.CDN_URL+logo,
                isNew:0
            }
            keyRetailerValues.push(Object.assign(x,brand));
        })
    }


    if(result.keyPartners.distributor){
        if(result.keyPartners.distributor.channel?.length)
        {
            result.keyPartners.distributor.channel.map(x => {
                let brand={
                    id:x.channelId,
                    name:x.companyName,
                    documentpath:process.env.CDN_URL+x.companylogo?.documentPath,
                    isNew:0
                }
                keyDistributorOptions.push(Object.assign(x,brand));
            })
        }

    }    

    let keyDistributor = await this.channelSettingService.getKeyRetailers(+req.user.channelId, appconstant.CHANNEL_TYPE_DISTRIBUTOR)
    if(keyDistributor?.length){
        
            keyDistributor.map(x => {
                let logo = x.active? x.active?.logo?.documentPath : x.old? x.old.logo?.documentPath : x.new? x.new.logo?.logo_name : '';
                let companyName = x.active? x.active.company_name : x.old? x.old.company_name : x.new? x.new.company_name : '';
                let id = x.active? x.active.id : x.old? x.old.id : x.new? x.new.id : '';
                let brand={
                    id:id,
                    name:companyName,
                    documentpath:process.env.CDN_URL+logo,
                    isNew:0
                }
                keyDistributorValues.push(Object.assign(x,brand));
            })

    }

    let profileType = '';
    
    if(result[0].dataValues.profileTypeId){

    query='select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
        +'when ptc."channelTypeId"=2 then \'vendor\' end as groupname,pt."profileType" as childname,'
        +'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" where ptc."channelTypeId" = :typeId and ptc."profileTypeId"=:profileTypeId';
      
         profileType=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,           
            replacements: {
                typeId: req.user.channelTypeId,
                profileTypeId: result[0].dataValues.profileTypeId
            }
            }).then(results => {        
               return results;
            });
        }

    let data={
        brand:brandList,
        product:productList,
        profile:profileList,
        empCnt:result[0].dataValues.channelDetail.empCntRange?result[0].dataValues.channelDetail.empCntRange:result[0].dataValues.channelDetail.empCnt,
        no_of_skus:result[0].dataValues.channelDetail.no_of_skus,
        turnoverRange:result[0].dataValues.channelDetail.turnoverRange,
        revD:result[0].dataValues.channelDetail.revD,
        revI:result[0].dataValues.channelDetail.revI,
        startup:result[0].dataValues.channelDetail.startup,
        no_of_distributors:result[0].dataValues.channelDetail.no_of_distributors,
        no_of_retailers:result[0].dataValues.channelDetail.no_of_retailers,
        maturitylevel: result[0].dataValues.channelDetail.maturitylevel,
        profileTypeId: result[0].dataValues.profileTypeId,
        profileType: profileType,
        onlineStore: result[0].dataValues.channelDetail.turnoverPhyStore,
        turnoverOnStore: result[0].dataValues.channelDetail.turnoverOnStore,
        storeCount: result[0].dataValues.channelDetail.storeCount,
        sales: result[0].dataValues.channelDetail.empCntDSale,
        marketing: result[0].dataValues.channelDetail.empCntISale,
        technical: result[0].dataValues.channelDetail.empCntDMar,
        purchase: result[0].dataValues.channelDetail.empCntIMar,
        keyPartners: {keyRetailerOptions, keyDistributorOptions},
        channelKeyRetailer: keyRetailerValues,
		channelKeyDistributor: keyDistributorValues,
        keyretailers: result[0].dataValues.channelDetail.keyretailers,
        keydistributors: result[0].dataValues.channelDetail.keydistributors,
        serviceListOptions: result.serviceListOptions,
        serviceListValues: result[0].dataValues.serviceList?result[0].dataValues.serviceList.map(x => ({id: x.serviceListId, name: x.serviceList})):[],
        specialityOptions: result.specialityOptions,
        channelSpeciality: result[0].dataValues.channelSpeciality,
        isFrenchTech: result[0].dataValues.isFrenchTech,
        isCompanyPublic: result[0].dataValues.isCompanyPublic
    }

    res.status(HttpStatus.OK).send(data);
    }
    
}

@Get('getChannelProduct')	
    public async getChannelProduct(@Req() req,@Res() res) {		
        let user = req.user;
        console.log("btype:",user.businessType);
        let result = await this.channelSettingService.getChannelProduct(user.channelId,user.businessType);
         if (result) {
            res.status(HttpStatus.OK).json(result);
        } 
}

@Get('getChannelBrand')	
    public async getChannelBrand(@Req() req, @Res() res) {		
        let user = req.user;
        
        let result = await this.channelSettingService.getChannelBrand(user.channelId);
         if (result) {
            res.status(HttpStatus.OK).json(result);
        } 
}

@Get('admin/getChannelBrand/:id')	
    public async getAdminChannelBrand(@Param('id') id,@Req() req, @Res() res) {		
        let user = await this.getUser(id);
        
        let result = await this.channelSettingService.getChannelBrand(user.channelId);
         if (result) {
            res.status(HttpStatus.OK).json(result);
        } 
}

public async getUser(channelId) {
    let query='select "firstName","lastName",tc."email",tc."channelId",tc."contactId",tch."planId",tch."subscriptionId",tch."channelTypeId",tu."userId", '
    + 'case when tch."isOEM"=\'1\' then \'B2C\' when tch."isOEM"=\'2\' then \'B2B\' when tch."isOEM"=\'3\' then \'Both\' end, '
    + '(select "documentPath" from "tblDocument" where "documentId"=tc."documentId"), '
    + '(select "roleId" from "tblUserRole" where "userId"=tu."userId") '
    + 'from "tblContact" as tc inner join "tblChannel" as tch on tch."channelId"=tc."channelId" inner join "tblUser" as tu on tu."contactId"=tc."contactId"  where "isDefaultContact"=\'1\' and tc."channelId"=:channelId ';


    let user = await this.sequelizeInstance.query(query, {
        type: QueryTypes.SELECT,
        raw: true,
        replacements: {
            channelId: channelId				
        }
    }).then(result => {
        //console.log("getUserResult:",result);
        return result[0];
    }).catch(err => {
        console.log("getUserErr:",err);
    });

    return user;
}




@Get('getChannelDetail')	
    public async getChannelDetail(@Req() req, @Res() res) {		
        let user = req.user;
        let result = await this.channelSettingService.getChannelDetail(user.channelId);
         if (result) {
            res.status(HttpStatus.OK).json(result);
        } 
}

@Get('getCertificateTag')
    public async getChannelCertificateTags(@Req() req, @Res() res){
        let user = req.user;
        let result = await this.channelSettingService.getChannelCertificateTags(user);
        
        res.status(HttpStatus.OK).json(result);
    }


@Get('tradelocation')   
public async tradelocation(@Req() req, @Res() res) {    
    let user=req.user;
    let result = await this.channelSettingService.getTradeLocation(user);    
    let targetArray=[];let sellingArray=[];
    
    let targetids=[];let sellingids=[];
    if(result && result.length)
    result[0].channelLocation?.map(x => {
     

        if(x.locationType=='TARGET' && user.channelTypeId!=4)
        {
            targetids.push(x.RegionCountryJCT.region.regionId);

            let location={
                regionCountryJCTId:x.RegionCountryJCT.regionCountryJCTId,
                groupid:x.RegionCountryJCT.region.regionId,
                groupname:x.RegionCountryJCT.region.region,
                childId:x.RegionCountryJCT.country.countryId,
                childname:x.RegionCountryJCT.country.country,
                status:true
            }
            targetArray.push(location);
        }
        if(x.locationType=='SELLING' && user.channelTypeId!=4 && user.channelTypeId!=5)
        {
            sellingids.push(x.RegionCountryJCT.region.regionId);
            let location={
                regionCountryJCTId:x.RegionCountryJCT.regionCountryJCTId,
                groupid:x.RegionCountryJCT.region.regionId,
                groupname:x.RegionCountryJCT.region.region,
                childId:x.RegionCountryJCT.country.countryId,
                childname:x.RegionCountryJCT.country.country,
                status:true
            }
            sellingArray.push(location);
        }
        if(x.locationType=='RETAIL' && user.channelTypeId==4)
        {
            sellingids.push(x.RegionCountryJCT.region.regionId);
            let location={
                regionCountryJCTId:x.RegionCountryJCT.regionCountryJCTId,
                groupid:x.RegionCountryJCT.region.regionId,
                groupname:x.RegionCountryJCT.region.region,
                childId:x.RegionCountryJCT.country.countryId,
                childname:x.RegionCountryJCT.country.country,
                status:true
            }
            sellingArray.push(location);
        }
        if(x.locationType=='RESELLING' && user.channelTypeId==5)
        {
            sellingids.push(x.RegionCountryJCT.region.regionId);
            let location={
                regionCountryJCTId:x.RegionCountryJCT.regionCountryJCTId,
                groupid:x.RegionCountryJCT.region.regionId,
                groupname:x.RegionCountryJCT.region.region,
                childId:x.RegionCountryJCT.country.countryId,
                childname:x.RegionCountryJCT.country.country,
                status:true
            }
            sellingArray.push(location);
        }
    })

    let zone=await this.sequelizeInstance.query('select "zoneId","regionId" from "tblZoneRegionJCT" ' , {
        type: QueryTypes.SELECT,
        raw: true,
        }).then(x=>{      
            console.log('zones', x);
            let zones={
                tzone:x.filter(ch => {return targetids.includes(ch.regionId);}),
                szone:x.filter(ch => {return sellingids.includes(ch.regionId);})
            }
            return zones;
        })

    let data={
        zone:zone,
        targetLocation:targetArray,
        sellingLocation:sellingArray
    }    
    
    res.status(HttpStatus.OK).send(data);
}

@Put('updateChannelLocation')		
	public async updateChannelLocation1(@Body(new ValidationPipe()) channelLocationDto: ChannelLocationDto, @Req() req, @Res() res) {
		let user = req.user;
		let subscriptionFeatureKeys = this.context.get('subscriptionFeatures');

		// trade locations
		//await this._validateTradeLocationProfile();

		let nonsuggestion=[];let count;
		let userId = user.userId;
			let channelId = user.channelId;
			let contactId =user.contactId;
			let channelTypeId = user.channel.channelTypeId;
			

        let channelLocation = await this.channelSettingService.updateChannelLocation(channelLocationDto, user, subscriptionFeatureKeys);
        
		// if (channelLocation) {
		// 	this.channelService.profileScore(user.channelId, user.channel.channelTypeId, user.channel.profileStrength).then(() => {
		// 		// update company info to elastic search
		// 		this.channelIndexService.index(user.channelId);
		// 	});

			//this.tradeLocationService.getNonSuggestion(this.context.get('user').channelId,this.context.get('dataSession').plan,contactId,userId,channelTypeId);		

			// audit log
			// if (user.channel.signUpStatusId == appconstant.CHANNEL_APPROVED && this.context.get('real-user')) {
			// 	this.auditLog.addAuditLog((channelLocationDto && channelLocationDto.reason) ? channelLocationDto.reason : null, auditlog.MODULE.BO_TRADE_LOCATION, req);
			// }

			return res.status(HttpStatus.OK).send(channelLocation);
		
	}


@Get('getuser')
public async getUsers(@Req() req, @Res() res) {    
    let user=req.user;
    let result = await this.channelSettingService.getUsers(user);    
    
    res.status(HttpStatus.OK).send(result);
}

    @Delete('deleteUser/:contactId')
    public async deleteUser(@Param('contactId') contactId, @Req() req, @Res() res) {
        let result = await this.channelSettingService.deleteUser(contactId);
        res.status(HttpStatus.OK).send(result);
    }

@Get('getuserLimits')
public async getuserLimits(@Req() req, @Res() res) {   
    let user=req.user;
    let result = await this.channelSettingService.getuserLimits(user);    
    
    res.status(HttpStatus.OK).send(result);
}

//updating company profile
@Post('updateCompany')
async update_Company_Profile(@Body(new ValidationPipe()) profileData:any, @Req() req,@Res() res){

    //console.log("profileData:",profileData);
        // let result=await this.channelSettingService.update_Company_Profile(Object.assign(profileData, {contactId: req.user.contactId, channelId: req.user.channelId}));
        let result=await this.channelSettingService.updateUserCompanyProfile(profileData, req.user);
        
        return res.status(HttpStatus.OK).send(result);
}

//Edit company profile
@Post('editBasicCompany')
async editBasicCompanyProfile(@Body(new ValidationPipe()) profileData:any, @Req() req,@Res() res){
        let result=await this.channelSettingService.editBasicCompany(profileData, req.user);
        return res.status(HttpStatus.OK).send(result);
}

//Edit company profile
@Post('updateBannerImage')
async updateBannerImage(@Body(new ValidationPipe()) bannerImg:any, @Req() req,@Res() res){
        let result=await this.channelSettingService.updateBannerImage(bannerImg, req.user);

        return res.status(HttpStatus.OK).send(result);
}

//updating Trade Info
@Post('updateTradeInfo')
async update_Trade_Info(@Body(new ValidationPipe()) profileData:any, @Req() req,@Res() res){

    //console.log("profileData:",profileData);
        // let result=await this.channelSettingService.updatetradeInformation(Object.assign(profileData.data, {contactId: req.user.contactId, channelId: req.user.channelId, channelType: req.user.channelTypeId}), req.user);
        let result=await this.channelSettingService.updatetradeInformation(profileData, req.user);
        return res.status(HttpStatus.OK).send(result);
}

//Facts abd Figures update
@Post('updateFacts')
async updateFacts(@Body(new ValidationPipe()) profileData:any, @Req() req,@Res() res){
        let result=await this.channelSettingService.updateFacts(profileData, req.user);
        return res.status(HttpStatus.OK).send(result);
}

@Get('getFacts')
async getFacts(@Req() req,@Res() res){
        let result=await this.channelSettingService.getFacts(req.user);
        return res.status(HttpStatus.OK).send(result);
}

//getting reporting to contacts
@Get('reporting')
async reporting(@Body(new ValidationPipe()) profileData:any, @Req() req,@Res() res){

    //console.log("profileData:",profileData);
        let result=await this.channelSettingService.reportingusers(req.user);
        return res.status(HttpStatus.OK).send(result);
}

//getting contactDetails for usermanage general form
@Post('generalInfo')
async generalInfo(@Body(new ValidationPipe()) contactId:any, @Req() req, @Res() res){
    let result = await this.channelSettingService.generalInfo(contactId);
    return res.status(HttpStatus.OK).send(result);
}

@Get('getContactLocation/:contactId')	
    public async getContactLocation(@Param('contactId') contactId, @Req() req,@Res() res) {		
        let user = req.user;
        if(user.contactId!=contactId)
            user.contactId=contactId;
        let result;
        result = await this.channelSettingService.getContactLocation(user);
        //console.log('reult',result);
         if (result) {
            let targetArray = result.contactLocation.filter(x => x.locationType == 'TARGET');
            let sellingArray = user.channelTypeId == 4  ? result.contactLocation.filter(x => x.locationType == 'RETAIL') : user.channelTypeId ==5 ? result.contactLocation.filter(x => x.locationType == 'RESELLING') : 
            result.contactLocation.filter(x => x.locationType == 'SELLING');
            let data = {
                targetArray : targetArray,
                sellingArray : sellingArray
            }
            res.status(HttpStatus.OK).json(data);
        } 
}    


//payment

@Get('getpaymentoption')
public async getPaymentOption(@Req() req,@Res() res) {
//let user = req.user;
let result;
result = await this.channelSettingService.getPaymentOption();

res.status(HttpStatus.OK).json(result);
}

@Get('currency')
public async getCurrency(@Req() req,@Res() res) {
//let user = req.user;
let result;
result = await this.channelSettingService.getCurrency();

res.status(HttpStatus.OK).json(result);
}

//keypartnercheck

@Get('channelKeyPartnerCheck')
public async existingKeyRetailerCheck(@Req() req, @Res() res, @Query('KeyRetailer') KeyRetailer) {
    let user = req.user;
    let result = await this.channelSettingService.existingKeyRetailerCheck(decodeURI(KeyRetailer), +req.query.CountryId, +user.channelId, +req.query.partner_type, req.query.partner_id ? +req.query.partner_id : null);
    res.status(HttpStatus.OK).send(result);
}

@Post('settings')	
	public async updateSettings(@Req() req, @Res() res, @Body(new ValidationPipe()) settingsDto: SettingsDto) {

        let channelTypeId =req.user.channelTypeId;
		let nonsuggestion=[];let count;
		let channelId = req.user.channelId;
		let userId = req.user.userId;
		let contactId = req.user.contactId;
		    // update company info to elastic search
			let user=req.user;//this.context.get('user');
		// trade locations
		//await this._validateSettingProfile();

		let updateSettings = await this.channelSettingService.updateSettings(settingsDto, user);
		if (updateSettings) {

			//this.channelIndexService.index(this.context.get('user').channelId);			
			
			//this.getNonSuggestion(user.channelId,user.channel.planId,user.contactId,user.userId,user.channel.channelTypeId);
		   
			// audit log
			// if (this.context.get('real-user')) {
			// 	this.auditLog.addAuditLog((settingsDto && settingsDto.reason) ? settingsDto.reason : null, auditlog.MODULE.BO_SETTINGS, req);
			// }

			return res.status(HttpStatus.OK).send('ok');
		}
		//throw new ProcessException('channel:settings:settingsNotUpdated');
	}

@Get('settings')	
	public async settings(@Res() res,@Req() req) {
		let settings = await this.channelSettingService.settings(req.user);
		if (settings) {
			return res.status(HttpStatus.OK).send(settings);
		}
		//throw new ProcessException('channel:settings:settingsNotFound');
	}

@Post('searchRegion')
public async searchRegion(@Body(new ValidationPipe()) searchRegionDto: SearchRegionDto, @Req() req, @Res() res) {
    let user = req.user;
    let searchRegion = await this.channelSettingService.searchRegion(searchRegionDto, user);

    let finaldata = [];

    if(searchRegion){
        searchRegion.map(x => {
            let location={
                regionCountryJCTId:x.regionCountryJCTId,
                groupid:x.region.regionId,
                groupname:x.region.region,
                childId:x.country.countryId,
                childname:x.country.country,
                status:false
            }
            finaldata.push(location);
    })
    }
    
    return res.status(HttpStatus.OK).send(finaldata);
}    

@Get('profileAndSpeciality')
public async getProfileAndSpeciality(@Req() req, @Res() res){
    let channelId = req.user.channelId;
    let result = await this.channelSettingService.getProfileAndSpeciality(channelId);

    let final = {};
    if(result){

        let profileType = '';

        if(result[0].dataValues.profileTypeId){
        let query='select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
        +'when ptc."channelTypeId"=2 then \'vendor\' end as groupname,pt."profileType" as childname,'
        +'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" where ptc."channelTypeId" in(:typeId) and ptc."profileTypeId"=:profileTypeId;';
      
         profileType=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,           
            replacements: {
                typeId: req.user.channelTypeId,
                profileTypeId: result[0].dataValues.profileTypeId
            }
            }).then(results => {        
               return results;
            });
        }

        final = {
        specialityOptions: result.specialityOptions,
        channelSpeciality: result[0].dataValues.channelSpeciality,
        profileTypeId: result[0].dataValues.profileTypeId,
        profileType: profileType,
        isFrenchTech: result[0].dataValues.isFrenchTech
        }
    }

    res.status(HttpStatus.OK).json(final);
}

 
//save profile edit

@Post('editProfileSave')
public async editProfile(@Req() req, @Res() res, @Body() data){
    let user = req.user;
    let result = await this.channelSettingService.editProfileSave(user, data);

    if(result){
        return res.status(HttpStatus.OK).send('OK');
    }else{
        return res.status(HttpStatus.BAD_REQUEST);
    }

}

//key partner options list
@Get('keyPartnerOptions')
public async getKeyPartnerOptions(@Req() req, @Res() res){
    let user = req.user;
    let result = await this.channelSettingService.getKeyPartnerOptions(user);

    let keyResellerOptions = [];
    let keyResellerValues = [];

    let keyRetailerOptions = [];
    let keyRetailerValues = [];

    let keyDistributorOptions = [];
    let keyDistributorValues = [];

    if(result.keyPartners.retailer){
        if(result.keyPartners.retailer.channel?.length)
        {
            result.keyPartners.retailer.channel.map(x => {
                let brand={
                    id:x.channelId,
                    name:x.companyName,
                    documentpath:x.companylogo?.documentPath ? process.env.CDN_URL+x.companylogo?.documentPath : null ,
                    isNew:0
                }
                keyRetailerOptions.push(Object.assign(x,brand));
            })
        }

        if(result.keyPartners.retailer.keypartners?.length)
        {
            result.keyPartners.retailer.keypartners.map(x => {
                let brand={
                    id:x.channelkeypartnerid,
                    name:x.partnername,
                    documentpath:x.companylogo?.documentPath ? process.env.CDN_URL+x.companylogo?.documentPath : null ,
                    isNew:0
                }
                keyRetailerOptions.push(Object.assign(x,brand));
            })
        }
    }

    let keyretailer = await this.channelSettingService.getKeyRetailers(+req.user.channelId, appconstant.CHANNEL_TYPE_RETAILER);
    if(keyretailer?.length){
        keyretailer.map(x => {
            let logo = x.active? x.active?.logo?.documentPath : x.old? x.old.logo?.documentPath : x.new? x.new.logo?.logo_name : '';
            let companyName = x.active? x.active.company_name : x.old? x.old.company_name : x.new? x.new.company_name : '';
            let id = x.active? x.active.id : x.old? x.old.id : x.new? x.new.id : '';
            let brand={
                id:id,
                name:companyName,
                documentpath:logo ? process.env.CDN_URL+logo : null,
                isNew:0
            }
            keyRetailerValues.push(Object.assign(x,brand));
        })
    }

    if(result.keyPartners.reseller){
        if(result.keyPartners.reseller.channel?.length)
        {
            result.keyPartners.reseller.channel?.map(x => {
                let brand={
                    id:x.channelId,
                    name:x.companyName,
                    documentpath:x.companylogo?.documentPath ? process.env.CDN_URL+x.companylogo?.documentPath : null,
                    isNew:0
                }
                keyResellerOptions.push(Object.assign(x,brand));
            })
        }
        if(result.keyPartners.reseller.keypartners?.length)
        {
            result.keyPartners.reseller.keypartners?.map(x => {
                let brand={
                    id:x.channelkeypartnerid,
                    name:x.partnername,
                    documentpath:x.companylogo?.documentPath ? process.env.CDN_URL+x.companylogo?.documentPath : null,
                    isNew:0
                }
                keyResellerOptions.push(Object.assign(x,brand));
            })
        }
    }

    let keyreseller = await this.channelSettingService.getKeyRetailers(+req.user.channelId, appconstant.CHANNEL_TYPE_RESELLER);
    if(keyreseller?.length){
        keyreseller.map(x => {
            let logo = x.active? x.active?.logo?.documentPath : x.old? x.old.logo?.documentPath : x.new? x.new.logo?.logo_name : '';
            let companyName = x.active? x.active.company_name : x.old? x.old.company_name : x.new? x.new.company_name : '';
            let id = x.active? x.active.id : x.old? x.old.id : x.new? x.new.id : '';
            let brand={
                id:id,
                name:companyName,
                documentpath: logo ? process.env.CDN_URL+logo : null,
                isNew:0
            }
            keyResellerValues.push(Object.assign(x,brand));
        })
    }


    if(result.keyPartners.distributor){
        if(result.keyPartners.distributor.channel?.length)
        {
            result.keyPartners.distributor.channel.map(x => {
                let brand={
                    id:x.channelId,
                    name:x.companyName,
                    documentpath:x.companylogo?.documentPath ? process.env.CDN_URL+x.companylogo?.documentPath : null,
                    isNew:0
                }
                keyDistributorOptions.push(Object.assign(x,brand));
            })
        }

        if(result.keyPartners.distributor.keypartners?.length)
        {
            result.keyPartners.distributor.keypartners.map(x => {
                let brand={
                    id:x.channelkeypartnerid,
                    name:x.partnername,
                    documentpath:x.companylogo?.documentPath ? process.env.CDN_URL+x.companylogo?.documentPath : null,
                    isNew:0
                }
                keyDistributorOptions.push(Object.assign(x,brand));
            })
        }

    }    

    let keyDistributor = await this.channelSettingService.getKeyRetailers(+req.user.channelId, appconstant.CHANNEL_TYPE_DISTRIBUTOR)
    if(keyDistributor?.length){
        
            keyDistributor.map(x => {
                let logo = x.active? x.active?.logo?.documentPath : x.old? x.old.logo?.documentPath : x.new? x.new.logo?.logo_name : '';
                let companyName = x.active? x.active.company_name : x.old? x.old.company_name : x.new? x.new.company_name : '';
                let id = x.active? x.active.id : x.old? x.old.id : x.new? x.new.id : '';
                let brand={
                    id:id,
                    name:companyName,
                    documentpath:logo ? process.env.CDN_URL+logo : null,
                    isNew:0
                }
                keyDistributorValues.push(Object.assign(x,brand));
            })

    }

    let final = {
        keyPartners: {keyRetailerOptions, keyDistributorOptions, keyResellerOptions},
        channelKeyRetailer: keyRetailerValues,
		channelKeyDistributor: keyDistributorValues,
        channelKeyReseller: keyResellerValues
    }   

    return res.status(HttpStatus.OK).send(final);

}

@Post('update_company_profile')
public async update_company_profile(@Req() req, @Res() res, @Body() data){
    let user = req.user;
    let result = await this.channelSettingService.update_company_new(user, data);

    console.log('data', data);

    if(result)
        return res.status(HttpStatus.OK).send('OK');
    else    
        return res.status(HttpStatus.BAD_REQUEST);
}

@Get('getChannelProfile')
public async getChannelProfile(@Req() req, @Res() res){
    let user = req.user;

    let businessType=user.businessType?user.businessType:'2';
    let ctype=+user.channelTypeId;
    
    let typeid=[];
    if(ctype==2 && businessType==1)
        typeid=[4,5]
    else if(ctype==2 && businessType==2)
        typeid=[4,5]
    else if(ctype==2 && businessType==3)
        typeid=[4,5]
    else if(ctype==4)
        typeid=[4]
    else if(ctype==5)
        typeid=[5]
    else if(ctype==3 && businessType==1)
        typeid=[4,5]
    else if(ctype==3 && businessType==2)
        typeid=[4,5]
    else if(ctype==3 && businessType==3)
        typeid=[4,5]

    let result = await this.channelSettingService.getChannelProfile(user);
    let profileList = [];


    if(result.length > 0){
        let pro=result[0]?.channelCustomerProfile?.map(x => +x.profileTypeId);
        let clssifiid=result[0]?.channelCustomerProfile?.map(x => +x.profileTypeClassificationId);


        let query='select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
        +'when ptc."channelTypeId"=2 then \'vendor\' when ptc."channelTypeId"=5 then \'Reseller\' end as groupname,pt."profileType" as childname,'
        +'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" and pt."profileTypeId" in(:ptype) and ptc."channelTypeId" in(:typeId) '
        + 'and  ptc."profileTypeClassificationId" in(:clssifiid) ';
        
        profileList=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,           
            replacements: {
                typeId: typeid,
                clssifiid:clssifiid,
                ptype: pro,     
            }
            }).then(results => {     
                //console.log("result:",pro,req.user.channelTypeId,results);   
               return results;
            });

        console.log('profileList', profileList);
    }
    let profileType = '';
    
    if(result.length > 0 && result[0].profileTypeId){

    let query='select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
        +'when ptc."channelTypeId"=2 then \'vendor\' when ptc."channelTypeId"=5 then \'Reseller\' end as groupname,pt."profileType" as childname,'
        +'pt."profileTypeId" as childId,cast(true as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" where ptc."channelTypeId" = :typeId and ptc."profileTypeId"=:profileTypeId';
      
         profileType=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,           
            replacements: {
                typeId: req.user.channelTypeId,
                profileTypeId: result[0].profileTypeId
            }
            }).then(results => {        
               return results;
            });
        }

        let finalData = {
            profile:profileList,
            profileTypeId: result>0?result[0].profileTypeId:null,
            profileType: profileType,
        }

        return res.status(HttpStatus.OK).send(finalData);
    }
 
    
    @Post('updateChannel')
    public async update_channel_profile(@Req() req, @Res() res, @Body() data){
    let user = req.user;
    let result = await this.channelSettingService.save_channel_info(data, {}, user);
    if(result)
        return res.status(HttpStatus.OK).send('OK');
    else    
        return res.status(HttpStatus.BAD_REQUEST);
}

@Get('getChannelAwards')
public async getChannelAwards(@Req() req, @Res() res) {   
    let user=req.user;
    let result = await this.channelSettingService.getChannelAwards(user);    
    
    res.status(HttpStatus.OK).send(result);
}

@Get('getProductAwards')
public async getProductAwards(@Req() req, @Res() res) {
    console.log('---')
let result = await this.channelSettingService.getProductAwards(req.user);
res.status(HttpStatus.OK).send(result);
}


@Get('getChannelAwardsAdmin/:channelId')
public async getChannelAwardsAdmin(@Param('channelId') channelId,@Req() req, @Res() res) {   
    let user={
        channelId:channelId
    };
    let result = await this.channelSettingService.getChannelAwards(user);
    
    res.status(HttpStatus.OK).send(result);
}

@Get('getProductAwardsAdmin/:channelId')
public async getProductAwardsAdmin(@Param('channelId') channelId,@Req() req, @Res() res) {
    let user={
        channelId:channelId
    };
let result = await this.channelSettingService.getProductAwards(user);
res.status(HttpStatus.OK).send(result);
}

@Post('deleteChannelAwards')
public async deleteChannelAwards(@Req() req, @Res() res, @Body() data) {   
    let user=req.user;
    let result = await this.channelSettingService.deleteChannelAwards(data);    
    
    res.status(HttpStatus.OK).send(result);
}

@Post('deleteProductAwards')
public async deleteProductAwards(@Req() req, @Res() res, @Body() data) {
    console.log('---')
let result = await this.channelSettingService.deleteProductAwards(data);
res.status(HttpStatus.OK).send(result);
}
}