'use strict';

import { UseGuards, Controller, Post, Get, HttpStatus, Request, Response, Param, Body, Put, Query, ValidationPipe, Req } from '@nestjs/common';
import { Channel } from '../models/Channel.entity';
import { isUndefined } from 'lodash';
import { AuthGuard } from '@nestjs/passport';
import { BaseController } from 'src/core';
import { TradeInformationInBound } from '../transformation/tradeInformationInBound';
import { ChannelKeyPartnerService } from '../service/channelKeyPartner.service';
import { appconstant } from 'src/config';
import { AuthException } from 'src/middleware/exception/Auth.exception';
import { ChannelDetail } from '../models/ChannelDetail.entity';

@UseGuards(AuthGuard('jwt'))
@Controller('')
export class ChannelKeyPartnerController extends BaseController {
	private tradeInformationInBound = new TradeInformationInBound();
	constructor(
		private channelKeyPartnerService: ChannelKeyPartnerService,
	) {
		super();
	}

	// @Get('retailerChannelSearch')
	// @ApiImplicitQuery({ name: 'limit', description: 'Limit', required: false })
	// @ApiImplicitQuery({ name: 'page', description: 'Page', required: false })
	// @ApiImplicitQuery({ name: 'search', description: 'search text', required: false })
	// public async retailerChannelSearch(@Response() res, @PaginationWithSearch() pagination: PaginationWithSearchAndfilterOptions) {
	// 	let user = req.user;;
	// 	if (user.channel.channelTypeId == appconstant.CHANNEL_TYPE_RETAILER) {
	// 		throw new AuthException('Not allowed');
	// 	}

	// 	let result = await this.channelKeyPartnerService.searchRetailer(pagination, appconstant.CHANNEL_TYPE_RETAILER);
	// 	res.status(HttpStatus.OK).json(result);
	// }

	@Get('partnerChannelSearch/:type')
	public async distributorChannelSearch(@Param('type') type, @Req() req, @Response() res) {
		let user = req.user;
		console.log('type', type);
		if (user.channel.channelTypeId == appconstant.CHANNEL_TYPE_RETAILER) {
			throw new AuthException('Not allowed ');
		}

		if (![appconstant.CHANNEL_TYPE_RETAILER, appconstant.CHANNEL_TYPE_DISTRIBUTOR, appconstant.CHANNEL_TYPE_RESELLER].includes(+type)) {
			throw new AuthException('Not allowed by type '+ type);
		}

		let result = await this.channelKeyPartnerService.searchRetailer({limit:50,offset:1}, +type);
		res.status(HttpStatus.OK).json(result);
	}


	@Get('partnerChannelSearchBo')
	public async distributorChannelSearchBO(@Request() req, @Response() res) {
		let user = req.query.channelTypeId;
		if (user.channelTypeId == appconstant.CHANNEL_TYPE_RETAILER) {
			throw new AuthException('Not allowed');
		}

		if (![appconstant.CHANNEL_TYPE_RETAILER, appconstant.CHANNEL_TYPE_DISTRIBUTOR, appconstant.CHANNEL_TYPE_RESELLER].includes(+req.query.partner_type)) {
			throw new AuthException('Not allowed');
		}

		let result = await this.channelKeyPartnerService.searchRetailer({limit:50,offset:1}, +req.query.partner_type);
		res.status(HttpStatus.OK).json(result);
	}
	// @Get('channelKeyPartnerCheck')
	// public async existingBrandCheck(@Request() req, @Response() res, @Query('KeyRetailer') KeyRetailer) {
	// 	let user = req.user;;
	// 	let result = await this.channelKeyPartnerService.existingKeyRetailerCheck(decodeURI(KeyRetailer), +req.query.CountryId, +user.channelId, +req.query.partner_type, isUndefined(req.query.partner_id) ? null: +req.query.partner_id);
	// 	res.status(HttpStatus.OK).send(result);
	// }

	// @Post('newKeyRetailer')
	// public async addNewKeyRetailer(@Response() res, @Body(new ValidationPipe()) channelKeyRetailer: ChannelKeyRetailerDto) {
	// 	let user = req.user;;
	// 	if (user.channel.channelTypeId == appconstant.CHANNEL_TYPE_RETAILER) {
	// 		throw new AuthException('Not allowed');
	// 	}

	// 	await this.tradeInformationInBound.keyRetailers(appconstant.CHANNEL_TYPE_RETAILER, [channelKeyRetailer], user.channelId, user.userId).then((newKeyRetailers) => {
	// 		res.status(HttpStatus.OK).json({
	// 			success: (newKeyRetailers[0]) ? true : false,
	// 			keyretailerid: newKeyRetailers[0]
	// 		});
	// 	});
	// }

	@Get('keyPartners/:channelId')
	public async getAllMyKeyRetailer(@Param('channelId') channelId, @Request() req, @Response() res) {
		let result = await this.channelKeyPartnerService.getKeyRetailers(channelId, +req.query.partner_type);
		let keyRetailerValues = [];
		if(result?.length){
			result.map(x => {
				let logo = x.active? x.active?.logo?.documentPath : x.old? x.old.logo?.documentPath : x.new? x.new.logo?.logo_name : '';
				let imageid = x.active? x.active?.logo?.logo_id : x.old? x.old.logo?.logo_id : x.new? x.new.logo?.logo_id : '';
				let companyName = x.active? x.active.company_name : x.old? x.old.company_name : x.new? x.new.company_name : '';
				let id = x.active? x.active.id : x.old? x.old.id : x.new? x.new.id : '';
				let brand={
					id:id,
					name:companyName,
					channelId:x.active? x.active.id:null,
					oldchannelid: x.old? x.old.id:null,
					newChannelId:x.new? x.new.id :null,
					documentpath:process.env.CDN_URL+logo,
					image:logo,
					imageid:imageid,
					countryid: x.active? x.active.country.country_id : x.old? x.old.country.country_id : x.new? x.new.country.country_id : '',
					isNew:0
				}
				keyRetailerValues.push(Object.assign(x,brand));
			})
		}
		// load key retailer description from channelDetail table
		let keyretailerDescription = await ChannelDetail.findOne({
			where: {
				channelId: +channelId
			}
		});

		res.status(HttpStatus.OK).json({
			keyretailerDescription: (keyretailerDescription) ? keyretailerDescription.keyretailers : '',
			channelKeyRetailers: keyRetailerValues
		});
	}

	// @Get('keyPartner/:partnerId')
	// public async getKeyPartnerById(@Param('partnerId') partnerId, @Request() req, @Response() res) {
	// 	res.status(HttpStatus.OK).json(
	// 		await this.channelKeyPartnerService.getKeyPartner(+partnerId)
	// 	);
	// }

	// @Put('keyPartner/:partnerId')
	// public async updateKeyPartnerById(@Param('partnerId') partnerId, @Request() req, @Response() res, @Body(new ValidationPipe()) channelKeyPartner: ChannelKeyRetailerDto) {
	// 	res.status(HttpStatus.OK).json(
	// 		await this.channelKeyPartnerService.updateKeyPartner(+partnerId, channelKeyPartner)
	// 	);
	// }

	// @Get('keyDataCleaning')
	// public async keyDataCleaning(@Request() req, @Response() res) {
	// 	let result = await this.channelKeyPartnerService.getDataCleanning(pagination);
	// 	res.status(HttpStatus.OK).json(result);
	// }

	// @Post('keyDataUpdate')
	// public async keyDataUpdate(@Request() req, @Response() res, @Body(new ValidationPipe()) keyDataUpdate: any) {
	// 	let result = await this.channelKeyPartnerService.updatecustomchannelkey(keyDataUpdate);
	// 	console.log('keyDataUpdate',result);
	// 	return res.status(HttpStatus.OK).json({
	// 		success: result
	// 	});
	// }
}
