'use strict';

import { Controller, Post, Request, Response, Body, HttpStatus, UseGuards, Param, Get, Query } from '@nestjs/common';
import { ChannelService } from './../service/channel.service';
import { ProductServiceOutBound } from '../../product/transformation/productServiceOutBound';
import { ContextService } from '../../../context/context.service';
import { BaseController } from 'src/core';
import { QueryPipe } from 'src/middleware/pipe/Query.pipe';
import { AuthGuard } from '@nestjs/passport';

@UseGuards(AuthGuard('jwt'))
@Controller('')
export class ChannelRelationController extends BaseController {
	private productServiceOutBound = new ProductServiceOutBound();
	constructor(
		private channelService: ChannelService,
		private context: ContextService
	) {
		super();
	}

	@Get('channel-infos')
	public async getRelations(@Request() req, @Response() res, @Query(new QueryPipe()) filter) {
		let user = req.user;

		//get channel from header 
		let channelId = user.channelId;
		let productFilter: any = false;
		let resultOject: any = "";
		(!filter.productFilter) ? productFilter = false: productFilter = filter.productFilter;
		let result = await this.channelService.findById(channelId, productFilter);
		resultOject = await this.productServiceOutBound.customerProfile(result);
		return res.status(HttpStatus.OK).json(resultOject);
	}
	@Get('channel-infos-bo/:channelId')
	public async getRelationsBo(@Param('channelId') channelId,@Request() req, @Response() res, @Query(new QueryPipe()) filter) {
		//let user = this.context.get('user');
		//get channel from header 
		//let channelId = user.channelId;
	
		let productFilter: any = false;
		let resultOject: any = "";
		(!filter.productFilter) ? productFilter = false: productFilter = filter.productFilter;
		let result = await this.channelService.findById(+channelId, productFilter);
		resultOject = await this.productServiceOutBound.customerProfile(result);
		return res.status(HttpStatus.OK).json(resultOject);
	}
}
