import { Body, Controller, Get, HttpStatus, Inject, Param, ParseIntPipe, Post, Req, Res, ValidationPipe } from '@nestjs/common';
import { ChannelSettingService } from '../service/channel.setting.service';
import { ProductService } from '../../product/service/product.service';
import {Op, QueryTypes } from "sequelize";
import { UserService } from '../../user/service/user.service';

@Controller('guest')
export class GuestController {

    constructor(private channelSettingService:ChannelSettingService,
        private productService:ProductService,
		private userService:UserService,
        @Inject('SequelizeInstance') private readonly sequelizeInstance,
        ){}

    @Get('buyershighlight/:viewId')
    public async BuyersHighlight(@Param ('viewId') viewId, @Req() Req, @Res() res){
   // console.log('get', viewId)
    let result = await this.channelSettingService.buyerHighlights(viewId);
    return res.status(HttpStatus.OK).send(result);
    }

	@Post('buyersaggr/:viewId/:parent')
	public async buyersaggr(@Param('viewId') viewId, @Param('parent') parent, @Req() Req, @Res() res, @Body() data) {

		let result = await this.channelSettingService.buyersaggr(viewId, parent, data.data);
    return res.status(HttpStatus.OK).send(result);
    }

	@Post('productsaggr/:parent')
	public async productsaggr(@Param('parent') parent, @Req() Req, @Res() res, @Body() data) {
		let result = await this.channelSettingService.productsaggr(parent, data.data);
    return res.status(HttpStatus.OK).send(result);
    }


    @Get('productshighlight/:btype')
    public async ProductsHighlight(@Param ('btype') btype, @Req() Req, @Res() res){
    let result = await this.channelSettingService.productsHighlights(btype);
    return res.status(HttpStatus.OK).send(result);
    }
    

    @Post('getBuyerSearchList/:viewId')
	async getSearchList(@Req() req, @Res() res,@Param('viewId', new ParseIntPipe()) viewId: number,@Body() data){        
		let channel = await this.channelSettingService.getBuyerSearchList(viewId,data);
            return res.status(HttpStatus.OK).send(channel);      

    }

    @Post('categoriesFilter')
    async getCategories(@Req() req, @Res() res, @Body() data){
        let categorie = await this.channelSettingService.getCategoriesFilter(data);
        return res.status(HttpStatus.OK).send(categorie);
    }

    @Post('brandsFilter')
    async getBrands(@Req() req, @Res() res, @Body() data){
        let brand = await this.channelSettingService.getBrandFilter(data);
        return res.status(HttpStatus.OK).send(brand);
    }

	@Post('countryFilter')
	public async getCountry(@Req() req, @Res() res, @Body() data) {
		let country = await this.channelSettingService.getCountryFilter(data);
		return res.status(HttpStatus.OK).send(country);
	}

    @Post('my-products')
	async myProduct(@Req() req, @Res() res, @Body(new ValidationPipe()) productFilters: any) {
		console.log('productFilters', productFilters);

		let order=productFilters.order?productFilters.order:'asc'
		let limit = ' LIMIT ' + productFilters.limit;
		let offset = ' OFFSET ' + productFilters.offset;
		let orderby = ' order by "'+productFilters.orderby+'" ';
		console.log('productFilters', productFilters);
		let query = 'select count(*) OVER() AS total, tp."productId",tp."businessType",(select "brandName" from "tblBrand" where "id"=tp."brandId"), tp."product",tp."productSKU", tp."isactive", tp."productqualitylevel", pr."businesscurrencyid",pr."retailerprice",(select concat( \'' + process.env.CDN_URL + '\',"documentPath") as "documentUrl" from "tblDocument" where "documentId" in(select "imageid" from "tblproductimage" where "productimageid"=tp."defaultimageid")), (select sum("existproductrating")/count(*) as existproductrating from "tblproductexistrating" where "productId"=tp."productId"), (select "country" from "tblCountry" where "countryId" = (select "countryId" from "tblChannel" where "channelId" = tp."channelId" limit 1) limit 1) as country ' +
		'from "tblProduct" as tp inner join "tblproductpricedetail" as pr on pr."productId"=tp."productId" where tp."deletedat" is null and "isactive"=:isactive';
		if(productFilters.search)
		query += ' and tp."productId"='+ +productFilters.search ;
		if(productFilters.filter.brandId)
		query += ' and tp."brandId" in('+ productFilters.filter.brandId +')';
		if(productFilters.filter.family)
		query += ' and tp."productGroupId" in('+ productFilters.filter.family +')';
		if(productFilters.filter.productCategoryId)
		query += ' and tp."productCategorieId" in('+ productFilters.filter.productCategoryId +')';

		if (productFilters.filter.businessType)
			query += ' and tp."businessType" in(' + productFilters.filter.businessType + ')';

		if(productFilters.filter.position)
		query += ' and tp."productqualitylevel" =:quality';
		if(productFilters.filter.hot_topic)
		{
			if(productFilters.filter.hot_topic.includes(1))
			query += ' and tp."eco_friendly"=\'1\' ';
			if(productFilters.filter.hot_topic.includes(2))
			query += ' and tp."work_from_home"=\'1\' ';
			if(productFilters.filter.hot_topic.includes(3))
			query += ' and tp."life_style"=\'1\' ';
			if(productFilters.filter.hot_topic.includes(4))
			query += ' and tp."french_tech"=\'1\' ';
			if(productFilters.filter.hot_topic.includes(5))
			query += ' and tp."taiwan_excellence"=\'1\' ';
		}
		if(productFilters.featured){
			query += ' and tp."isFeatured"=\'1\' ';
		}

		if(productFilters.orderby)
		query += orderby+order +limit+offset;
		else
		query += ' order by tp."product" '+order +limit+offset;
		let results;
		
		let result=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
			isactive:'1',
			quality:productFilters.filter.position
		}
		});
		let active=await this.sequelizeInstance.query('select count(*) from "tblProduct" where "deletedat" is null and "isactive"=:isactive', {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
			isactive:'1'
		}
		})
		let inActive=await this.sequelizeInstance.query('select count(*) from "tblProduct" where "deletedat" is null and "isactive"=:isactive', {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
			isactive:'0'
		}
		})
			let tabCount={
				active: active?active[0].count:0,
				inActive: active?inActive[0].count:0
			}
		// let results;
		if(result.length!=0){
			results={
				total:result[0].total,
				result:result,
				tabCount:tabCount
			}
		}else{
			results={
				total:0,
				result:[],
				tabCount:tabCount
			}
		}
	
	
		
		res.status(HttpStatus.OK).send(results);
	}

	//product only slider
	@Post('my-products-slider')
	async myProductSlider(@Req() req, @Res() res, @Body(new ValidationPipe()) productFilters: any) {


		let order=productFilters.order?productFilters.order:'asc'
		let limit = ' LIMIT ' + productFilters.limit;
		let offset = ' OFFSET ' + productFilters.offset;
		let orderby = ' order by random()';
		let query = 'select count(*) OVER() AS total, tp."productId",(select "brandName" from "tblBrand" where "id"=tp."brandId"), tp."product",tp."productSKU", tp."isactive", tp."productqualitylevel", pr."businesscurrencyid",pr."retailerprice",(select concat( \'' + process.env.CDN_URL + '\',"documentPath") as "documentUrl" from "tblDocument" where "documentId" in(select "imageid" from "tblproductimage" where "productimageid"=tp."defaultimageid")), (select sum("existproductrating")/count(*) as existproductrating from "tblproductexistrating" where "productId"=tp."productId"), (select "country" from "tblCountry" where "countryId" = (select "countryId" from "tblChannel" where "channelId" = tp."channelId" limit 1) limit 1) as country ' + 
		'from "tblProduct" as tp inner join "tblproductpricedetail" as pr on pr."productId"=tp."productId" where tp."deletedat" is null and "isactive"=:isactive';

		if(productFilters.search)
		query += ' and tp."productId"='+ +productFilters.search ;
		if(productFilters.filter.brandId)
		query += ' and tp."brandId" in('+ productFilters.filter.brandId +')';
		if(productFilters.filter.family)
		query += ' and tp."productGroupId" in('+ productFilters.filter.family +')';
		if(productFilters.filter.productCategoryId)
		query += ' and tp."productCategorieId" in('+ productFilters.filter.productCategoryId +')';
		if(productFilters.filter.position)
		query += ' and tp."productqualitylevel" =:quality';

		if(productFilters.featured){
			query += ' and tp."isFeatured"=\'1\' ';
		}

		if(productFilters.orderby)
		query += orderby+order +limit+offset;
		else
		query += ' order by tp."product" '+order +limit+offset;
		let results;
		
		

		let result=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
			isactive:'1',
		}
		});
		
		
			
		// let results;
		if(result.length!=0){
			results={
				total:result[0].total,
				result:result,
				
			}
		}else{
			results={
				total:0,
				result:[],
				
			}
		}
	
	
		
		res.status(HttpStatus.OK).send(results);
	}

    //brand Filter for product
    @Post('brandFilter')
	async brandFilter(@Req() req, @Res() res,@Body() data){
        //let channel = await this.channelService.chFilterBase(viewId,req.user);
		let result = await this.productService.getBrandFilter(data.data);
        return res.status(HttpStatus.OK).send(result);      
    }

    //category filter for product
    @Post('categoryFilter')
	async categoryFilter(@Req() req, @Res() res,@Body() data){
        //let channel = await this.channelService.chFilterBase(viewId,req.user);
		let result = await this.productService.getCategoryFilter(data.data);
        return res.status(HttpStatus.OK).send(result);      
    }

	@Post('categoryGuestFilter')
	async categoryGuestFilter(@Req() req, @Res() res, @Body() data) {
		//let channel = await this.channelService.chFilterBase(viewId,req.user);
		let result = await this.productService.getCategoryGuestFilter(data.data);
		return res.status(HttpStatus.OK).send(result);
	}


    //family filter for product
    @Post('familyFilter')
	async familyFilter(@Req() req, @Res() res,@Body() data){
        //let channel = await this.channelService.chFilterBase(viewId,req.user);
		let result = await this.channelSettingService.getFamilyFilter(data.data);
		// let result = await this.productService.getFamilyFilter(data.data);
        return res.status(HttpStatus.OK).send(result);      
    }

	@Get('country')	
	public async userCountry(@Req() req, @Res() res) {
		let result = await this.userService.Countries();
		if (result) {
			res.status(HttpStatus.OK).json(result);
		} else {
			//throw new ProcessException("notification:presetmail:listNotfound");
		}
    }

	@Get('categoryDetails/:groupId')
	public async categoryDetails(@Req() Req, @Res() res, @Param('groupId') groupId){
		let result = await this.channelSettingService.getCategoryDetails(groupId);
	}

	@Get('getProducts/:businessType')	
     public async getProducts(@Req() req, @Param() data , @Res() res) {		
        let result = await this.channelSettingService.getProducts(+data.businessType);
         if (result) {
            res.status(HttpStatus.OK).json(result);
         } else {
              //throw new ProcessException("notification:presetmail:listNotfound");
          }
    }

	@Get('getCategories/:businessType')	
     public async getCategorieList(@Req() req, @Param() data , @Res() res) {		
        let result = await this.channelSettingService.getCategories(+data.businessType);
         if (result) {
            res.status(HttpStatus.OK).json(result);
         } else {
              //throw new ProcessException("notification:presetmail:listNotfound");
          }
    }
	

	@Post('getProductByGroup')
	public async getProductByGroup(@Res() res, @Body() filter) {
		let query ='select * from (select pc."productCategorieId", pc."productCategorie", pc."businessType", pc."image", (select count("productCategorieId") as total from "tblProduct" where "productCategorieId"=pc."productCategorieId") as total from "tblProductCategories" as pc where pc."productGroupId"=:groupId and pc."businessType" in(:type) and pc."isActive"=\'1\') as productgroup where total > 0 '
		let result=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
			groupId:filter.groupId,
			type: [filter.type, 3]
		}
		});
		res.status(HttpStatus.OK).json(result);
	}

	@Post('guestUserRequirement')
	public async guestUserRequirement(@Res() res, @Body() data) {
		let result = await this.channelSettingService.guestUserRequirement(data);
		res.status(HttpStatus.OK).json(result);
	}
	@Get('topcompany/:keyword/:btype')
	async getTopCompany(@Req() req, @Res() res,@Param('keyword') keyword, @Param('btype') btype){    
		
		// let type=req.user.channelTypeId;
		let channel = await this.channelSettingService.getTopCompany(keyword,btype);
            return res.status(HttpStatus.OK).send(channel);    

    }

	// @Get('topproduct/:keyword')
	// async getTopProduct(@Req() req, @Res() res,@Param('keyword') keyword){        
	// 	let channel = await this.channelSettingService.getTopProduct(keyword);
    //         return res.status(HttpStatus.OK).send(channel);    

    // }

	@Post('matchingVendors')
	async getMatchingVendors(@Req() req, @Res() res, @Body() data){
		let result = await this.channelSettingService.getMatchingVendors(data.channelId, data);
		return res.status(HttpStatus.OK).send(result);
	}

	

	@Post('matchingBrands')
	async getMatchingBrands(@Req() req, @Res() res, @Body() data){
		let result = await this.channelSettingService.getMatchingBrands(data.channelId, data);
		return res.status(HttpStatus.OK).send(result);
	}

	@Post('featuredVendors')
	async getFeaturedVendors(@Req() req, @Res() res, @Body() data){
		let result = await this.channelSettingService.getFeaturedVendors(data.data);
		return res.status(HttpStatus.OK).send(result);
	}

	@Post('buyerSearchCount')
	async buyerSearchCount(@Req() req, @Res() res, @Body() data)
	{
		let result = await this.channelSettingService.getBuyerSearchCount(data);
		return res.status(HttpStatus.OK).send(result);
	}
}
