import { Controller, Get, HttpStatus, Param, Req, Res, Request, ParseIntPipe, Inject, Body, ValidationPipe, Response, Post } from '@nestjs/common';
import { ProcessException } from 'src/middleware/exception/Process.exception';
import { BaseController } from 'src/core';
import _ = require('lodash');
import { PublicChannelService } from '../service/public.channel.service';
import { Op, QueryTypes } from "sequelize";

@Controller('publicChannel')
export class PublicChannelController extends BaseController {
constructor(private channelService:PublicChannelService,
    @Inject('SequelizeInstance') private readonly sequelizeInstance,) {
		super();
	}

	@Get('channelDetailView/:channelId')
	public async getChannelDetailView(@Param('channelId') channelId, @Req() req, @Res() res) {
		let channelDetail = await this.channelService.getChannel(channelId);
		if (!channelDetail) {
			throw new ProcessException('channel:getChannel:channelNotFound');
		}

		res.status(HttpStatus.OK).send(channelDetail);
	}
    
	@Post('channelBrands/:channelId')
	public async channelBrands(@Param('channelId') channelId, @Res() res, @Req() req) {
		let user = req.user;//this.context.get('user');

		let channelBrands = await this.channelService.channelBrands(+channelId);

		res.status(HttpStatus.OK).send(channelBrands);
	}

	@Get('eventChannelDetailView/:channelId')
	public async getEventChannelDetailView(@Param('channelId') channelId, @Req() req, @Res() res) {

		let channelDetail = await this.channelService.getChannel(channelId);
		if (!channelDetail) {
			throw new ProcessException('channel:getChannel:channelNotFound');
		}

		res.status(HttpStatus.OK).send(channelDetail);
	}


	@Post('eventMatching')
	public async eventMatching(@Request() req, @Response() res, @Body(new ValidationPipe()) contactData: any) {
		if (req.headers.eventtoken == "VBzL1GQPiotFRTbgUSWP") {
			let result = await this.channelService.eventMatching(contactData);
			return res.status(HttpStatus.CREATED).send(result);
		}
	}

	@Get('eventListing/:typeId')
	public async eventListing(@Param('typeId') typeId, @Request() req, @Response() res) {
		let result = await this.channelService.eventListing(typeId);
		return res.status(HttpStatus.OK).send(result);
	}

	@Get('getPassword/:contactId')
	public async getPassword(@Param('contactId') contactId, @Request() req, @Response() res) {
		if (req.headers.eventtoken == "VBzL1GQPiotFRTbgUSWP") {
			let result = await this.channelService.getPassword(contactId);
			return res.status(HttpStatus.OK).send(result);
		}
	}


}