import { Controller, Post, Get, HttpStatus, Request, Response, Param, Query, Inject, Body, UsePipes, InternalServerErrorException, UseGuards, Req } from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import { ProcessException } from 'src/middleware/exception/Process.exception';
import { SubscriptionService } from './../service/subscription.service';
import { ContextService } from '../../../context/context.service';
import { ValidationPipe } from 'src/middleware/pipe/validation.pipe';
import { SubscribeDto } from './../dto/subscribe.dto';
import { CouponDto } from './../dto/coupon.dto';
import { appconstant } from '../../../config/app.constant';
//import { AuthException } from './../../../middleware/exception/Auth.exception';
import { Subscription } from '../models/Subscription.entity';
import { BaseHelper } from "src/core/helper/BaseHelper";
import { AuthException } from './../../../middleware/exception/Auth.exception';
import { PaymentTransaction } from '../models/PaymentTransaction.entity';
//import { ChannelIndexService } from './../service/channelIndex.service';
import { isUndefined } from 'lodash';
import { StripeV2 } from '../../../lib/Stripe';
import { ClientProxy } from '@nestjs/microservices';
import { ProfileList } from '../models/ProfileList.entity';
import { UserService } from 'src/modules/user/service/user.service';

@UseGuards(AuthGuard('jwt'))
@Controller('channel/subscription')
export class SubscriptionController {
    constructor(
		private context: ContextService,
		private readonly subscriptionService: SubscriptionService,
		private userService: UserService,
		@Inject('ALGORITHM_SERVICE') private algorithmService: ClientProxy,
		@Inject('ProfileRepository') private readonly profileRepository: typeof ProfileList,
		//private channelIndexService: ChannelIndexService,
	) {}
    @Get('subscriptionPlanAllPeriod/:channelTypeId')
    public async subscriptionPlanAllPeriod(@Param('channelTypeId') channelTypeId, @Request() req, @Response() res) {
		let subscriptionPlanAllPeriod = await this.subscriptionService.subscriptionPlanAllPeriod(channelTypeId);
		return res.status(HttpStatus.OK).send(subscriptionPlanAllPeriod);
	}

	@Get('channel_suggestion')
    async channelAlgm(@Req() req) {
      let user = req.user;        		
    return this.algorithmService.send({
      cmd: 'channel-suggestion-algm'
    },JSON.stringify(user));

	
  }


	@Get('channel/isOfflineRequestInProcess')
	
	public async offlineRequest(@Response() res, @Req() req) {
		let user = req.user;

		let offlineRequest = await this.subscriptionService.isOfflineRequestInProcess(user);
		if (offlineRequest) {
			throw new ProcessException('Wire transfer request in process. please contact support team');
		}

		return res.status(HttpStatus.OK).send();
	}

	@Post('subscribe_amt_evaluvate')
	public async subscribe_amt_evaluvate(@Request() req, @Response() res, @Body(new ValidationPipe()) subscribeDto: SubscribeDto) {
		var parser = require('ua-parser-js');
        
		let user = req.user;
		let offlineRequest = await this.subscriptionService.isOfflineRequestInProcess(user);

		console.log('offlineRequest');
		console.log(offlineRequest.rows);
		if(offlineRequest.count > 0) {
			throw new ProcessException('Wire transfer request in process. please contact support team ');
		}
	
		// let helper = new BaseHelper();
		// await helper.lazyload(user, 'channel', null, true);

		if(isUndefined(subscribeDto.paymentMethodId)) {
			subscribeDto.paymentMethodId = appconstant.PAYMENT_METHOD.ONLINE_CREDIT_DEBIT_CART;
		}

		// subscription only happend after approval

		
		// if (+user.channel.signUpStatusId != +appconstant.CHANNEL_APPROVED) {
		// 	throw new AuthException('Not allowed');
		// }

		 var subscriptionType = await this.subscriptionService.getSubscriptionType(user, +subscribeDto.subscriptionId);
		// if (subscriptionType === Subscription.SUBSCRIPTION_TYPE_NEW && user.channel.planId != null) {
		// 	throw new ProcessException('channel:already:subscribed');
		// }
		var subscriptionPlanDetails = await this.subscriptionService.getSubscriptionDetails(+subscribeDto.subscriptionId);
		
		// get finalAmount
		let amountObj = await this.subscriptionService.calculateAmount(subscribeDto, subscriptionType, user);
		let amount = +amountObj.finalAmount;

		return res.status(HttpStatus.OK).json({

			amount: +(amount * 100).toFixed(2),
			original_amount: +(amountObj.totalAmount * 100).toFixed(2),
			coupon_discount: +(amountObj.couponDiscount * 100).toFixed(2),
			remainingAmount: +(amountObj.remainingAmount * 100).toFixed(2),
			taxAmount: +(amountObj.taxAmount * 100).toFixed(2),
			paymentMethodId: subscribeDto.paymentMethodId,
			period: subscriptionPlanDetails[0].subscriptionPeriod.periodDesc,
			plan: subscriptionPlanDetails[0].subscriptionPlan.subscriptionPlanType,
			periodId: subscriptionPlanDetails[0]?.subscriptionPeriod?.subscriptionPeriodId,
			planId: subscriptionPlanDetails[0].subscriptionPlan.subscriptionPlanId,
			vat: appconstant.VAT_PERCENTAGE,
		});
	}

	@Post('coupan_validate')
	public async coupanValidation(@Request() req, @Response() res, @Body(new ValidationPipe()) coupanDto: CouponDto) {
		let user = req.user;
		var subscriptionType = await this.subscriptionService.getSubscriptionType(user, +coupanDto.subscriptionId);
		let finalAmt = await this.subscriptionService.coupanValidate(coupanDto, user, subscriptionType);
		return res.status(HttpStatus.OK).json({finalAmt: finalAmt});
	}


	@Post('subscribe')
	public async subscribe(@Request() req, @Response() res, @Body(new ValidationPipe()) subscribeDto: SubscribeDto) {
		let user = req.user;

		//this.channelIndexService.hubspotCompanyUpdate(user.channelId, user);
		let offlineRequest = await this.subscriptionService.isOfflineRequestInProcess(user);
		if (offlineRequest.count > 0) {
			throw new ProcessException('Wire transfer request in process. please contact support team');
		}

		// let helper = new BaseHelper();
		// await helper.lazyload(user, 'channel', null, true);
		
		if (isUndefined(subscribeDto.paymentMethodId)) {
			subscribeDto.paymentMethodId = appconstant.PAYMENT_METHOD.ONLINE_CREDIT_DEBIT_CART;
		}
		// subscription only happend after approval
		if (+user.channel.signUpStatusId != +appconstant.CHANNEL_APPROVED) {
			throw new AuthException('Not allowed');
		}
		var subscriptionType = await this.subscriptionService.getSubscriptionType(user, +subscribeDto.subscriptionId);
		var subscriptiondetails = await this.subscriptionService.getSubscriptiondetails(subscribeDto.subscriptionId);
		
		console.log('subscriptionType',subscriptionType);
		console.log('subscriptiondetails',subscriptiondetails.subscriptionPlanId);
		console.log('SUBSCRIPTION_TYPE_NEW',Subscription.SUBSCRIPTION_TYPE_NEW);
		console.log('planId',user.channel.planId);
		// if (subscriptionType === Subscription.SUBSCRIPTION_TYPE_NEW && user.channel.planId != null) {
		// 	throw new ProcessException('channel:already:subscribed');
		// }

		
		let amountObj = await this.subscriptionService.calculateAmount(subscribeDto, subscriptionType, user);
		let amount = +amountObj.finalAmount;
		let transactionId = null
		let paymentIntentSecret = null;	
		let transaction = null;
	
			transaction = await this.subscriptionService.getTransactionId(
			amount,
			PaymentTransaction.PAYMENT_TYPE_SUBSCRIPTION,
			appconstant.CHANNELHUB_CHANNEL_ID,
			user.channel.channelId,
			user.userId,
			subscribeDto);	
			
			let channelSubscription = null;
            if(transaction){
				transactionId = transaction.paymentTransactionId;	
				if (( subscribeDto.paymentMethodId == appconstant.PAYMENT_METHOD.OFFLINE_WIRE_TRANSFER) || subscribeDto.paymentMethodId == appconstant.PAYMENT_METHOD.ONLINE_CREDIT_DEBIT_CART || subscribeDto.paymentMethodId == appconstant.PAYMENT_METHOD.STRIPE_ALIPAY || subscribeDto.paymentMethodId == appconstant.PAYMENT_METHOD.PAYPAL_TRANSFER) {

					channelSubscription = await this.subscriptionService.createChannelSubscription(
						user,
						subscribeDto.subscriptionId,
						subscribeDto.couponCode ? subscribeDto.couponCode : null,
						transaction.paymentTransactionId,
						subscribeDto.paymentStatus,
						subscriptionType,
						transaction,
						transaction.cardLastFourDigit
					);
					//this.channelIndexService.index(user.channelId);

				}
				let rating = await this.profileRepository.findOne({
					where:{
						channelId : user.channelId,
						deletedAt:null,
						ratingId : 5,
					}
				});
				if(!rating){
					await this.profileRepository.create({
						channelId : user.channelId,
						ratingId : 5,
						isCompleted : true,
						createdAt : new Date()
					});
				}
				return res.status(HttpStatus.OK).json({
					transactionId: transactionId,
					amount: +(amount * 100).toFixed(2),
					original_amount: +(amountObj.totalAmount * 100).toFixed(2),
					coupon_discount: +(amountObj.couponDiscount * 100).toFixed(2),
					remainingAmount: +(amountObj.remainingAmount * 100).toFixed(2),
					taxAmount: +(amountObj.taxAmount * 100).toFixed(2),
					paymentMethodId: subscribeDto.paymentMethodId,
					channelSubscription:channelSubscription,
					planId:+subscriptiondetails.subscriptionPlanId
				});
			}else{
				throw new ProcessException('Something went wrong');
			}
		
	}

	@Post('createPaymentIntend')
	public async PaymentIntend(@Request() req, @Response() res) {
		let user = req.user;
		let stripe = new StripeV2();
		
		let paymentIntentSecret = await stripe.createPaymentIntend( +(req.body.amount * 100).toFixed(2), appconstant.CHANNELHUB_CURRENCY_CODE, 'subscription', +user.channelId);
		console.log(paymentIntentSecret.client_secret);
		return res.status(HttpStatus.OK).json({
			paymentIntentSecret: paymentIntentSecret.client_secret
		});
	}		


	@Post('stipeCreatecharge')
	public async stipeCreatecharge(@Request() req, @Response() res) {
		let stripe = new StripeV2();
		var resource = req.body;
		let amount = resource.amount;
		let currency = resource.currency;
		let sourceId = resource.sourceId;
		console.log(sourceId);
		const charge = await stripe.createchargesForSource(amount,currency,sourceId);
		//update transaction table with alipay source ID
		if(resource.channel_transId !== null){
			let transaction = await this.subscriptionService.UpdateTransactionById(resource.channel_transId, sourceId);
		}
		return res.status(HttpStatus.OK).json({ charge });
	}


	@Post('addKeyFeature')
	public async addKeyFeature(@Request() req, @Response() res) {
		let bodyCnt = req.body
		let result = await this.subscriptionService.insertKeyFeature(bodyCnt.categories, bodyCnt.subscriptionId);
		return res.status(HttpStatus.OK).json({ result });
	}

	@Get('getAllKeyFeatures/:channelTypeId/:subscriptionPeriodId')
	public async getKeyFeatures(@Param('channelTypeId') channelTypeId, @Param('subscriptionPeriodId') subscriptionPeriodId,@Request() req, @Response() res) {
		let result = await this.subscriptionService.getAllKeyFeatures(channelTypeId, subscriptionPeriodId);
		return res.status(HttpStatus.OK).json({ result });
	}

	@Get('updateKeyFeature/:channelId/:subscriptionId')
	public async updateKeyFeature(@Param('channelId') channelId, @Param('subscriptionId') subscriptionId,@Request() req, @Response() res) {
		let result = await this.subscriptionService.updateKeyFeatureForchannel(channelId, subscriptionId);
		// let result = await this.subscriptionService.updateKeyFeatureForchannel(channelId, subscriptionId);
		return res.status(HttpStatus.OK).json({ result });
	}
	
	//admin
	@Get('getPlansforAdmin')
	public async getPlansforAdmin(@Request() req, @Response() res) {
		let result = await this.subscriptionService.getPlansforAdmin();
		return res.status(HttpStatus.OK).json({ result });
	}

	@Post('getPlansDetailsById')
	public async getPlansDetailsById(@Request() req, @Response() res) {
		let subscriptionId = req.body.subscriptionId;
		let result = await this.subscriptionService.getPlansDetailsById(subscriptionId);
		return res.status(HttpStatus.OK).json({ result });
	}

	@Get('getPlansDetails')
	public async getPlansDetails(@Request() req, @Response() res) {
		console.log('channelId for key feature');
		console.log(req.user.channel);
		let subscriptionId = req.user.channel.channelSubscription.subscriptionId;

		

		let result = await this.subscriptionService.getFeaterListId(req.user.channelId);
		let category = await this.subscriptionService.getFeaterCategoryList(result.keyFeature);
		return res.status(HttpStatus.OK).json({ category });
		
	}

	@Post('getPlansDetailsBo')
	public async getPlansDetailsBo(@Request() req, @Response() res, @Body() data) {

		let channelId = data.channelId;
		let userId = data.userId;
		let user = await this.userService.findUserById(userId, channelId);

		let subscriptionId = user.channel.channelSubscription.subscriptionId;
		let result = await this.subscriptionService.getFeaterListId(user.channelId);
		let category = await this.subscriptionService.getFeaterCategoryList(result.keyFeature);
		return res.status(HttpStatus.OK).json({ category });
		
	}

	@Post('updateSubscription/:subscriptionId')
	public async updateSubscription(@Request() req, @Response() res, @Param('subscriptionId') subscriptionId) {
		let data = req.body;
		let result = await this.subscriptionService.updateSubscription(data , subscriptionId);
		return res.status(HttpStatus.OK).json({ result });
	}

	@Post('failedTransaction')
	public async failedTransaction(@Request() req, @Response() res) {
		let data = req.body;
		let result = await this.subscriptionService.failedTransaction(data);
		return res.status(HttpStatus.OK).json({ result });
	}

	@Get('getFeaterList/:channelId')
	public async getFeaterList(@Request() req, @Response() res, @Param('channelId') channelId) {
		let result = await this.subscriptionService.getFeaterListId(channelId);
		let category = null;
		if(result){
		category = await this.subscriptionService.getFeaterCategoryList(result.keyFeature);
		}
		return res.status(HttpStatus.OK).json({ category });
	}

	@Post('updateChannelFeatureList')
	public async updateChannelFeatureList(@Request() req, @Response() res) {
		let channelId = req.body.channelId;
		let extraFeature = req.body.extraFeature;
		let result = await this.subscriptionService.updateChannelFeatureList(extraFeature, channelId);
		return res.status(HttpStatus.OK).json({ result });
	}
    
	
	
}