import {
	IsString,
	IsEmail,
	IsNotEmpty,
	MinLength,
	MaxLength,
	Min,
	Max,
	IsNumber,
	ArrayNotEmpty,
	ValidateIf,
	ValidateNested,
	IsOptional,
	IsDate,
	IsDateString,
	IsAlphanumeric,
	Length
} from 'class-validator';
import { Type } from 'class-transformer';
import { CompanyCertificateDto } from './companycertificate.dto';
import { CompanyAwardsDto } from './companyawards.dto';
import { CompanyB2BDto } from './companyB2B.dto';
import { IsNumeric } from 'sequelize-typescript';
import { ChannelDocumentDto } from './channelDocument.dto';
import { CertificateTagDto } from './certificateTag.dto';

export class CompanyUpdateDto {

	@IsNumber({}, { message: "channel:companyProfile:companyImageIdIsNumber" })
	@IsOptional()
	readonly company_image_id: number;

	@IsOptional()
	@IsNotEmpty({ message: "channel:companyProfile:companyLogoUrlIsNotEmpty" })
	readonly company_logo_url: string;

	// @IsOptional()
	// @IsNumber({}, { message: "channel:companyProfile:companyIconIdIsNumber" })
	// @ApiProperty()
	// readonly company_icon_id: number;
	//
	// @IsOptional()
	// @IsNotEmpty({ message: "channel:companyProfile:companyIconUrlIsNotEmpty" })
	// @ApiProperty()
	// readonly company_icon_url: string;
	//
	// @IsNotEmpty({ message: "channel:companyProfile:companyOwnerFirstNameIsNotEmpty" })
	// @ApiProperty()
	// readonly company_owner_first_name: string;
	//
	// @IsNotEmpty({ message: "channel:companyProfile:companyOwnerLastNameIsNotEmpty" })
	// @ApiProperty()
	// readonly company_owner_last_name: string;

	@IsOptional()
	@MaxLength(255, { message: "channel:subscription:companyNameMaxLen" })
	@IsNotEmpty({ message: "channel:subscription:companyNameIsNotEmpty" })
	readonly company_name: string;

	@IsOptional()
	@IsEmail({}, { message: "channel:subscription:companyMailInvalid" })
	// @MinLength(5, { message: "channel:subscription:companyMailMinLen" })
	@MaxLength(255, { message: "channel:subscription:companyMailMaxLen" })
	readonly company_email: string;

	@MinLength(5, { message: "channel:subscription:comapanyWebSideMinLen" })
	@MaxLength(255, { message: "channel:subscription:comapanyWebSideMaxLen" })
	readonly company_website_url: string;

	@IsOptional()
	@ValidateIf(o => o.company_phone1 !== "")
	@IsNotEmpty({ message: "channel:subscription:companyPhoneIsNotEmpty" })
	@MinLength(5, { message: "channel:subscription:companyPhoneMinLen" })
	@MaxLength(30, { message: "channel:subscription:companyPhoneMaxLen" })
	readonly company_phone1: string;

	@IsOptional()
	@ValidateIf(o => o.company_phone2 !== "")
	@IsNotEmpty({ message: "channel:subscription:companyPhone2IsNotEmpty" })
	@MinLength(5, { message: "channel:subscription:companyPhone2MinLen" })
	@MaxLength(30, { message: "channel:subscription:companyPhone2MaxLen" })
	readonly company_phone2: string;

	@IsOptional()
	@ValidateIf(o => o.company_mobile_no !== "")
	@MinLength(5, { message: "channel:companyProfile:companyMobileNoMinLen" })
	@MaxLength(30, { message: "channel:companyProfile:companyMobileNoMaxLen" })
	readonly company_mobile_no: string;

	@MinLength(5, { message: "channel:subscription:describtionMinLen" })
	@MaxLength(5000, { message: "channel:subscription:describtionMaxLen" })
	readonly company_about: string;

	@IsOptional()
	@MinLength(5, { message: "channel:subscription:describtionMinLen" })
	@MaxLength(5000, { message: "channel:subscription:describtionMaxLen" })
	readonly company_expectation: string;

	@IsOptional()
	@Max(9999, {
		message: 'channel:companyProfile:shouldBeFourDigitNumber'
	})
	@IsNumber({}, { message: "channel:companyProfile:shouldBeNumber" })
	// @IsNotEmpty({ message: "channel:companyProfile:yearOfEstabilishmentIsNotEmpty" })
	readonly year_of_estabilishment: number;

	// @Max(9999, {
	// 	message: 'channel:companyProfile:shouldBeFourDigitNumber'
	// })
	// @IsNumber({}, { message: "channel:companyProfile:shouldBeNumber" })
	// @IsNotEmpty({ message: "channel:companyProfile:yearOfRegistrationIsNotEmpty" })
	// @ApiProperty()
	// readonly year_of_registration: number;

	@IsNotEmpty({ message: "channel:companyProfile:companyRegAddressIdIsNotEmpty" })
	@IsNumber({}, { message: "channel:companyProfile:companyRegAddressIdIsNumber" })
	readonly reg_address_id: number;

	// @IsNotEmpty({ message: "channel:companyProfile:companyAddressAddressIsNotEmpty" })
	readonly reg_address: string;

	@IsNotEmpty({ message: "channel:companyProfile:companyRegCityIsNotEmpty" })
	readonly reg_city: string;

	@IsNotEmpty({ message: "channel:companyProfile:companyAddresscountryIdIsNotEmpty" })
	@IsNumber({}, { message: "channel:companyProfile:companyAddresscountryIdonlyacceptInt" })
	readonly reg_country_id: number;

	@IsNotEmpty({ message: "channel:companyProfile:companyAddresspostalcodeIsNotEmpty" })
	readonly reg_postal_code: number;

	@IsOptional()
	@IsNumber({}, { message: "channel:companyProfile:companyBillAddressIdIsNumber" })
	readonly bill_address_id: number;

	@IsOptional()
	// @IsNotEmpty({ message: "channel:companyProfile:companyAddressAddressIsNotEmpty" })
	readonly bill_address: string;

	@IsOptional()
	@IsNotEmpty({ message: "channel:companyProfile:companyBillCityIsNotEmpty" })
	readonly bill_city: string;

	@IsOptional()
	@IsNotEmpty({ message: "channel:companyProfile:companyAddresscountryIdIsNotEmpty" })
	@IsNumber({}, { message: "channel:companyProfile:companyAddresscountryIdonlyacceptInt" })
	readonly bill_country_id: number;

	@IsOptional()
	@IsNotEmpty({ message: "channel:companyProfile:companyAddresspostalcodeIsNotEmpty" })
	readonly bill_postal_code: number;

	@IsNotEmpty({ message: "channel:companyProfile:isSameAddressIsNotEmpty" })
	@IsNumber({}, { message: "channel:companyProfile:isSameAddressIsNumber" })
	readonly is_same_address: number;

	@IsOptional()
	@IsNumber({}, { message: "channel:companyProfile:companySocialLinkedinIdIsNumber" })
	readonly social_linkedin_id: number;

	@IsOptional()
	@ValidateIf(o => o.social_linkedin_url !== "")
	@IsNotEmpty({ message: "channel:companyProfile:companySocialLinkedinUrlIsNotEmpty" })
	// @MinLength(5, { message: "channel:companyProfile:companySocialLinkedinUrlMinLen" })
	@MaxLength(255, { message: "channel:companyProfile:companySocialLinkedinUrlMaxLen" })
	readonly social_linkedin_url: string;

	@IsOptional()
	@IsNumber({}, { message: "channel:companyProfile:companySocialGoogleplusIdIsNumber" })
	readonly social_googleplus_id: number;

	@IsOptional()
	@ValidateIf(o => o.social_googleplus_url !== "")
	@IsNotEmpty({ message: "channel:companyProfile:companySocialGoogleplusUrlIsNotEmpty" })
	@MinLength(5, { message: "channel:companyProfile:companySocialGoogleplusUrlMinLen" })
	@MaxLength(255, { message: "channel:companyProfile:companySocialGoogleplusUrlMaxLen" })
	readonly social_googleplus_url: string;

	@IsOptional()
	@IsNumber({}, { message: "channel:companyProfile:companySocialInstagramIdIsNumber" })
	readonly social_instagram_id: number;

	@IsOptional()
	@ValidateIf(o => o.social_instagram_url !== "")
	@IsNotEmpty({ message: "channel:companyProfile:companySocialInstagramUrlIsNotEmpty" })
	@MinLength(5, { message: "channel:companyProfile:companySocialInstagramUrlMinLen" })
	@MaxLength(255, { message: "channel:companyProfile:companySocialInstagramUrlMaxLen" })
	readonly social_instagram_url: string;

	@IsOptional()
	@IsNumber({}, { message: "channel:companyProfile:companySocialfacebookIdIsNumber" })
	readonly social_facebook_id: number;

	@IsOptional()
	@ValidateIf(o => o.social_facebook_url !== "")
	@IsNotEmpty({ message: "channel:companyProfile:companySocialfacebookUrlIsNotEmpty" })
	@MinLength(5, { message: "channel:companyProfile:companySocialfacebookUrlMinLen" })
	@MaxLength(255, { message: "channel:companyProfile:companySocialfacebookUrlMaxLen" })
	readonly social_facebook_url: string;

	@IsOptional()
	@IsNumber({}, { message: "channel:companyProfile:companySocialtwitterIdIsNumber" })
	readonly social_twitter_id: number;

	@IsOptional()
	@ValidateIf(o => o.social_twitter_url !== "")
	@IsNotEmpty({ message: "channel:companyProfile:companySocialtwitterUrlIsNotEmpty" })
	@MinLength(5, { message: "channel:companyProfile:companySocialtwitterUrlMinLen" })
	@MaxLength(255, { message: "channel:companyProfile:companySocialtwitterUrlMaxLen" })
	readonly social_twitter_url: string;

	@IsOptional()
	@IsNotEmpty({ message: "channel:subscription:firstNameNotEmpty" })
	@MinLength(1, { message: "channel:subscription:firstNameMinLen" })
	@MaxLength(30, { message: "channel:subscription:firstNameMaxLen" })
	readonly contact_first_name: string;

	@IsOptional()
	@IsNotEmpty({ message: "channel:subscription:lastNameIsNotEmpty" })
	@MinLength(1, { message: "channel:subscription:lastNameMinLen" })
	@MaxLength(30, { message: "channel:subscription:lastNameMaxLen" })
	readonly contact_last_name: string;

	@IsOptional()
	@ValidateIf(o => o.contact_phone1 !== "")
	@IsNotEmpty({ message: "channel:subscription:phoneIsNotEmpty" })
	@MinLength(5, { message: "channel:subscription:phoneMinLen" })
	@MaxLength(30, { message: "channel:subscription:phoneMaxLen" })
	readonly contact_phone1: string;

	@IsOptional()
	@ValidateIf(o => o.contact_phone2 !== "")
	@MinLength(5, { message: "channel:subscription:phoneMinLen" })
	@MaxLength(30, { message: "channel:subscription:phoneMaxLen" })
	readonly contact_phone2: string;

	@IsOptional()
	@IsNotEmpty({ message: "channel:companyProfile:companyContactMobileNoIsNotEmpty" })
	@MinLength(5, { message: "channel:companyProfile:companyContactMobileNoMinLen" })
	@MaxLength(30, { message: "channel:companyProfile:companyContactMobileNoMaxLen" })
	readonly contact_mobile_no: string;

	@IsOptional()
	@IsNotEmpty({ message: "channel:subscription:jobTitleIsNotEmpty" })
	@MinLength(2, { message: "channel:subscription:jobTitleMinLen" })
	@MaxLength(255, { message: "channel:subscription:jobTitleMaxLen" })
	readonly contact_job_title: string;

	@IsOptional()
	@IsNumber({}, { message: "channel:companyProfile:contactLinkedIdIsNumber" })
	readonly contact_linkedin_id: number;

	@IsOptional()
	@ValidateIf(o => o.contact_linkedin_url !== "")
	@IsNotEmpty({ message: "channel:companyProfile:linkedInIsNotEmpty" })
	// @MinLength(5, { message: "channel:companyProfile:linkedInMinLen" })
	@MaxLength(255, { message: "channel:companyProfile:linkedInMaxLen" })
	readonly contact_linkedin_url: string;

	@IsOptional()
	@IsNumber({}, { message: "channel:companyProfile:contactProfileImageIdIsNumber" })
	readonly contact_profile_image_id: number;

	@IsOptional()
	@IsNotEmpty({ message: "channel:companyProfile:imageIsNotEmpty" })
	readonly contact_profile_image_url: string;

	// @ArrayNotEmpty({ message: "channel:companyProfile:companyRegistrationCertificateIsNotEmpty" })
	@ValidateNested()
	@Type(() => CompanyCertificateDto)
	readonly registration_certificate: CompanyCertificateDto[];

	@IsOptional()
	@ArrayNotEmpty({ message: "channel:companyProfile:companyTaxCertificateIsNotEmpty" })
	@ValidateNested()
	@Type(() => CompanyCertificateDto)
	readonly tax_certificate: CompanyCertificateDto[];

	@IsOptional()
	@ValidateNested()
	@Type(() => CompanyAwardsDto)
	readonly award_details: CompanyAwardsDto[];

	@IsOptional()
	@ArrayNotEmpty({ message: "channel:companyProfile:companyB2BUrlIsNotEmpty" })
	@ValidateNested()
	@Type(() => CompanyB2BDto)
	readonly B2B_url: CompanyB2BDto[];

	@IsOptional()
	@ArrayNotEmpty({ message: "channel:companyProfile:companyDeletedRegistrationCertificateIsNotEmpty" })
	readonly deleted_registration_certificate: number[];

	@IsOptional()
	@ArrayNotEmpty({ message: "channel:companyProfile:companyDeletedTaxCertificateIsNotEmpty" })
	readonly deleted_tax_certificate: number[];

	@IsOptional()
	@ArrayNotEmpty({ message: "channel:companyProfile:companyDeletedAwardDetailsIsNotEmpty" })
	readonly deleted_award_details: number[];

	@IsOptional()
	@ArrayNotEmpty({ message: "channel:companyProfile:companyDeletedB2BIsNotEmpty" })
	readonly deleted_B2B: number[];

	@IsOptional()
	@ArrayNotEmpty({ message: "validate:field:shouldBeEmptyArray" })
	@ValidateNested()
	@Type(() => ChannelDocumentDto)
	readonly channel_document: ChannelDocumentDto[];

	@IsOptional()
	@ArrayNotEmpty({ message: "validate:field:shouldBeEmptyArray" })
	// @ValidateNested()
	// @Type(() => ChannelDocumentDto)
	readonly companyImages: any[];

	@IsOptional()
	@ArrayNotEmpty({ message: "validate:field:shouldBeEmptyArray" })
	readonly deleted_channel_documents: number[];


	@IsOptional()
	@ArrayNotEmpty({ message: "validate:field:shouldBeEmptyArray" })
	readonly deleted_companyImages
	@IsOptional()
	@MaxLength(1000, { message: "validate:field:validateMaxLength" })
	reason: string

	@IsOptional()
	company_introvideo: string

	@IsOptional()
	bannerImage : any

	@IsOptional()
	certificatesList : CertificateTagDto[];

	@IsOptional()
	deletedTags : number[];

	@IsOptional()
	companyDesc : string

	
	@IsOptional()
	isOEM : string

	@IsOptional()
	is_company_public: string

	@IsOptional()
	ratingId: number
}
