import { IsNotEmpty, IsEmail, MinLength, MaxLength, IsString, IsAlphanumeric, IsNumber, Matches, IsOptional, ArrayNotEmpty } from 'class-validator';
import { ApiProperty } from '@nestjs/swagger';
import { Type } from 'class-transformer';
import { ChannelProfileDto } from './SaveProfile/ChannelProfile.dto';

export class SettingDto {
	
	@IsNotEmpty({ message: "channel:companyProfile:yearOfEstabilishmentIsNotEmpty" })	
	@ApiProperty()
	yearEstablished: number;

	@IsNotEmpty({ message: "channel:tradeinformation:startupIsNotEmpty" })	
	@ApiProperty()
	areYouStartUp: string;
	
	@IsNotEmpty({ message: "channel:tradeinformation:employeesRangeNotEmpty" })
	@ApiProperty()
	no_of_staffs: string;    

    @IsNotEmpty({ message: "channel:tradeinformation:annualTurnoverRangeNotEmpty" })
	@ApiProperty()
    annualTurnOver: string;
    
    @IsNotEmpty({ message: "channel:profile:registrationNumberNotEmpty" })
	@ApiProperty()
    companyRegNumber: string;
    
	//@IsNotEmpty({ message: "channel:profile:vatNumberNotEmpty" })
	@IsOptional()
	@ApiProperty()
    vatNumber: string;

    @IsOptional()
    //@IsNotEmpty({ message: "channel:profile:companyLogoUrlIsNotEmpty" })
	@ApiProperty()
    companyLogo: string;

    @IsNotEmpty({ message: "channel:profile:tweetPitchNotEmpty" })
	@ApiProperty()
    tweetPitch: string;

    @IsOptional()
    //@IsNotEmpty({ message: "channel:profile:videoUrlIsNotEmpty" })
	@ApiProperty()
    introVideo: string;

    //@IsNotEmpty({ message: "channel:profile:companyDescriptionNotEmpty" })
	@ApiProperty()
    companyDescription: string;

    @IsOptional()
	@ApiProperty()
	readonly productBrand: string;


    // @ArrayNotEmpty({ message: "validate:field:shouldBeArray" })
	// @IsNumber({}, {
	// 	each: true,
	// 	message: "validate:field:shouldBeInt"
	// })
	// @ApiProperty({ type: 'number', isArray: true })
	// readonly brands: number[];

	@IsOptional()
    @ArrayNotEmpty({ message: "validate:field:shouldBeArray" })
	@IsNumber({}, {
		each: true,
		message: "validate:field:shouldBeInt"
	})
	@ApiProperty({ type: 'number', isArray: true })
	readonly productGroup: number[];

	@IsOptional()
	@ArrayNotEmpty({ message: "validate:field:shouldBeArray" })
	@IsNumber({}, {
		each: true,
		message: "validate:field:shouldBeInt"
	})
	@ApiProperty({ type: 'number', isArray: true })
	readonly productCat: number[];


	@ApiProperty({ type: ChannelProfileDto })
	@IsNotEmpty({ message: "channel:product:channelProfileIsNotEmpty" })	
	@Type(() => ChannelProfileDto)
	readonly relevantChannelProfile: ChannelProfileDto;

    // @ArrayNotEmpty({ message: "validate:field:shouldBeArray" })
	// @IsNumber({}, {
	// 	each: true,
	// 	message: "validate:field:shouldBeInt"
	// })
	// @ApiProperty({ type: 'number', isArray: true })
	// readonly relevantChannelProfile: number[];

	@IsOptional()
    @IsNumber({}, {
		each: true,
		message: "validate:field:shouldBeInt"
	})
	@ApiProperty()
	no_of_skus: number;

	@IsOptional()
	@ApiProperty({ type: ChannelProfileDto })
	@IsNotEmpty({ message: "channel:product:channelProfileIsNotEmpty" })	
	@Type(() => ChannelProfileDto)
	readonly relevantDistributorProfile: ChannelProfileDto;    

}
