import {
	IsNotEmpty,
	IsNumber,
	ArrayNotEmpty,
	ValidateNested,
	ValidateIf,
	IsBoolean,
	IsOptional,
	MaxLength
} from 'class-validator';
import { Type } from 'class-transformer';

import { PaymentTypeDto } from './paymentType.dto';
import { PaymentTermsDto } from './paymentTerms.dto';
//import { UserVisibilityDto } from './userVisibility.dto';
import { ShareDto } from './share.dto';
//import { SampleRequestsDto } from './sampleRequests.dto';
import { appconstant } from '../../../config/index';

export class SettingsDto {

	@IsOptional()
	@ArrayNotEmpty({ message: "channel:settings:paymentTypeArrayIsNotEmpty" })
	@ValidateNested()
	@Type(() => PaymentTypeDto)
	readonly paymentType: PaymentTypeDto[];


	@IsOptional()
	@ArrayNotEmpty({ message: "channel:settings:paymentTermsArrayIsNotEmpty" })
	@ValidateNested()
	@Type(() => PaymentTermsDto)
	readonly paymentTerms: PaymentTermsDto[];

	@IsNotEmpty({ message: "channel:settings:currencyIdIsNotEmpty" })
	@IsNumber({}, { message: "channel:settings:currencyIdIsNumber" })
	readonly currencyId: string;

	// @IsNotEmpty({ message: "validate:field:shouldNotEmpty" })
	// @IsBoolean({ message: "validate:field:shouldBeBoolean" })
	// @ApiModelProperty()
	// readonly autoRenewalOff: boolean;

	// @ApiModelProperty({ type: UserVisibilityDto, isArray: true })
	// @ArrayNotEmpty({ message: "channel:settings:userVisibilityArrayIsNotEmpty" })
	// @ValidateNested()
	// @Type(() => UserVisibilityDto)
	// readonly userVisibility: UserVisibilityDto[];

	@IsNotEmpty({ message: "channel:settings:shareStatusIsNotEmpty" })
	@IsBoolean({ message: "channel:settings:shareStatusIsBoolean" })	
	readonly shareStatus: boolean;

	@ValidateIf(o => o.shareStatus === true)	
	@ArrayNotEmpty({ message: "channel:settings:shareArrayIsNotEmpty" })
	@ValidateNested()
	@Type(() => ShareDto)
	readonly share: ShareDto[];

	@IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })	
	readonly channelType: number;

	// @ValidateIf(o => o.channelType === appconstant.CHANNEL_TYPE_VENDOR)	
	// @ArrayNotEmpty({ message: "channel:settings:sampleRequestsArrayIsNotEmpty" })
	// @ValidateNested()
	// @Type(() => SampleRequestsDto)
	// readonly sampleRequests: SampleRequestsDto[];

	
	readonly otherOptions: string;

	@IsOptional()
	@MaxLength(1000, { message: "validate:field:validateMaxLength" })
	reason: string
}
