import { IsString, IsNotEmpty, IsNumber, ValidateIf, ArrayNotEmpty, ValidateNested } from 'class-validator';
import { Type } from 'class-transformer';
import { ApiProperty } from '@nestjs/swagger';
import { appconstant } from 'src/config/app.constant';
import { TradeInformation_Product_FamiliesDto } from './tradeInformation_Product_Families.dto';
import { TradeInformation_Product_CategoriesDto } from './tradeInformation_Product_Categories.dto';
import { TradeInformation_Customer_ProfilesDto } from './tradeInformation_Customer_Profiles.dto';
import { TradeInformation_VendorDto } from './tradeInformation_Vendor.dto';
import { TradeInformation_DistributorDto } from './tradeInformation_Distributor.dto';
import { TradeInformation_RetailerDto } from './tradeInformation_Retailer.dto';


export class TradeInformationDto {

    @ApiProperty({ type: TradeInformation_Product_FamiliesDto, isArray: true })
    @ArrayNotEmpty({ message: "channel:tradeinformation:productFamiliesArray" })
    @IsNotEmpty({ message: "channel:tradeinformation:productFamiliesArrayNotEmpty" })
    @ValidateNested()
    @Type(() => TradeInformation_Product_FamiliesDto)
    readonly productFamilies: TradeInformation_Product_FamiliesDto[];

    @ApiProperty({ type: TradeInformation_Product_CategoriesDto, isArray: true })
    @ArrayNotEmpty({ message: "channel:tradeinformation:productCategoriesArray" })
    @IsNotEmpty({ message: "channel:tradeinformation:productCategoriesNotEmpty" })
    @ValidateNested()
    @Type(() => TradeInformation_Product_CategoriesDto)
    readonly productCategories: TradeInformation_Product_CategoriesDto[];

    @ValidateIf(o => o.channelType !== appconstant.CHANNEL_TYPE_RETAILER)
    @ApiProperty({ type: TradeInformation_Customer_ProfilesDto, isArray: true })
    @ArrayNotEmpty({ message: "channel:tradeinformation:customerProfilesArray" })
    @IsNotEmpty({ message: "channel:tradeinformation:customerProfilesNotEmpty" })
    @ValidateNested()
    @Type(() => TradeInformation_Customer_ProfilesDto)
    readonly customerProfiles: TradeInformation_Customer_ProfilesDto[];

    @IsNotEmpty({ message: "channel:tradeinformation:channelTypeNotEmpty" })
    @IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })
    @ApiProperty()
    readonly channelType: number;

    @IsNotEmpty({ message: "validate:field:shouldNotEmpty" })
    @IsString({ message: "validate:field:shouldBeString" })
    @ApiProperty()
    readonly is_french_tech: string;

    @ValidateIf(o => o.channelType === appconstant.CHANNEL_TYPE_VENDOR)
    @ApiProperty({ type: TradeInformation_VendorDto })
    @ValidateNested()
    @Type(() => TradeInformation_VendorDto)
    readonly tradeInformationVendor: TradeInformation_VendorDto;

    @ValidateIf(o => o.channelType === appconstant.CHANNEL_TYPE_DISTRIBUTOR)
    @ApiProperty({ type: TradeInformation_DistributorDto })
    @ValidateNested()
    @Type(() => TradeInformation_DistributorDto)
    readonly tradeInformationDistributor: TradeInformation_DistributorDto;

    @ValidateIf(o => o.channelType === appconstant.CHANNEL_TYPE_RETAILER)
    @ApiProperty({ type: TradeInformation_RetailerDto })
    @ValidateNested()
    @Type(() => TradeInformation_RetailerDto)
    readonly tradeInformationRetailer: TradeInformation_RetailerDto;
}