import {
	IsString,
	IsEmail,
	IsNotEmpty,
	MinLength,
	MaxLength,
	Min,
	Max,
	IsNumber,
	ValidateIf,
	IsArray,
	Length,
	IsOptional,
	ArrayNotEmpty,
	ValidateNested
} from 'class-validator';
import { Type } from 'class-transformer';
import { ApiProperty } from '@nestjs/swagger';
import { appconstant } from '../../../config/app.constant';
import { TradeInformation_Product_FamiliesUpdateDto } from './tradeInformation_Product_Families.update.dto';
import { TradeInformation_Product_CategoriesUpdateDto } from './tradeInformation_Product_Categories.update.dto';
import { TradeInformation_Customer_ProfilesUpdateDto } from './tradeInformation_Customer_Profiles.update.dto';
import { TradeInformation_VendorUpdateDto } from './tradeInformation_Vendor.Update.dto';
import { TradeInformation_DistributorUpdateDto } from './tradeInformation_Distributor.update.dto';
import { TradeInformation_RetailerUpdateDto } from './tradeInformation_Retailer.update.dto';


export class TradeInformationUpdateDto {

	@ApiProperty({ type: TradeInformation_Product_FamiliesUpdateDto, isArray: true })
	@IsOptional()
	@ValidateNested()
	@Type(() => TradeInformation_Product_FamiliesUpdateDto)
	readonly productFamilies: TradeInformation_Product_FamiliesUpdateDto[];

	@ApiProperty({ type: TradeInformation_Product_CategoriesUpdateDto, isArray: true })
	@IsOptional()
	@ValidateNested()
	@Type(() => TradeInformation_Product_CategoriesUpdateDto)
	readonly productCategories: TradeInformation_Product_CategoriesUpdateDto[];

	@ApiProperty({ type: TradeInformation_Customer_ProfilesUpdateDto, isArray: true })
	@IsOptional()
	@ValidateNested()
	@Type(() => TradeInformation_Customer_ProfilesUpdateDto)
	readonly customerProfiles: TradeInformation_Customer_ProfilesUpdateDto[];

	@IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })
	@IsOptional()
	@ApiProperty()
	readonly channelType: number;

	@IsNotEmpty({ message: "validate:field:shouldNotEmpty" })
	@IsString({ message: "validate:field:shouldBeString" })
	@ApiProperty()
	readonly is_french_tech: string;

	@ValidateIf(o => o.channelType === appconstant.CHANNEL_TYPE_VENDOR)
	@ApiProperty({ type: TradeInformation_VendorUpdateDto })
	@IsOptional()
	@ValidateNested()
	@Type(() => TradeInformation_VendorUpdateDto)
	readonly tradeInformationVendor: TradeInformation_VendorUpdateDto;

	@ValidateIf(o => o.channelType === appconstant.CHANNEL_TYPE_DISTRIBUTOR)
	@ApiProperty({ type: TradeInformation_DistributorUpdateDto })
	@IsOptional()
	@ValidateNested()
	@Type(() => TradeInformation_DistributorUpdateDto)
	readonly tradeInformationDistributor: TradeInformation_DistributorUpdateDto;

	@ValidateIf(o => o.channelType === appconstant.CHANNEL_TYPE_RETAILER)
	@ApiProperty({ type: TradeInformation_RetailerUpdateDto })
	@IsOptional()
	@ValidateNested()
	@Type(() => TradeInformation_RetailerUpdateDto)
	readonly tradeInformationRetailer: TradeInformation_RetailerUpdateDto;

	@IsOptional()
	@MaxLength(1000, { message: "validate:field:validateMaxLength" })
	@ApiProperty()
	reason: string
}
