import { IsString, IsNotEmpty, MaxLength, IsNumber, ValidateIf, Min, ArrayNotEmpty, IsOptional, ValidateNested, IsEnum, Max } from 'class-validator';
import { Type } from 'class-transformer';
import { ApiProperty } from '@nestjs/swagger';
import { TradeInformation_List_of_ServicesUpdateDto } from './tradeInformation_List_of_Services.update.dto';
import { TradeInformation_Brand_NameUpdateDto } from './tradeInformation_Brand_Name.update.dto';
import { TradeInformation_Distributor_SpecialityUpdateDto } from './tradeInformation_Distributor_Speciality.update.dto';
import { ChannelKeyRetailerDto } from './channelkeyretailer.dto';

export enum maturityLevel {
	PROTOTYPE,
	S_BACKER,
	S_VOLUME
}

export class TradeInformation_DistributorUpdateDto {

	@ApiProperty({ type: TradeInformation_Brand_NameUpdateDto, isArray: true })
	@ValidateNested()
	@IsOptional()
	@Type(() => TradeInformation_Brand_NameUpdateDto)
	readonly brand: TradeInformation_Brand_NameUpdateDto[];

	@ApiProperty({ type: TradeInformation_List_of_ServicesUpdateDto, isArray: true })
	@ValidateNested()
	@IsOptional()
	@Type(() => TradeInformation_List_of_ServicesUpdateDto)
	readonly listOfServices: TradeInformation_List_of_ServicesUpdateDto[];

	@ApiProperty({ type: TradeInformation_Distributor_SpecialityUpdateDto, isArray: true })
	@ValidateNested()
	@IsOptional()
	@Type(() => TradeInformation_Distributor_SpecialityUpdateDto)
	readonly distributorSpeciality: TradeInformation_Distributor_SpecialityUpdateDto[];

	@IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })
	@IsOptional()
	@ApiProperty()
	readonly profileTypeId: number;

	@ValidateIf(o => o.employeesCount === undefined)
	@IsOptional()
	@ApiProperty()
	readonly employeesRange: string;

	@ValidateIf(o => o.employeesRange === undefined)
	@Min(1, { message: "channel:tradeinformationVendor:minEmployeesCount" })
	@IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })
	@IsOptional()
	@ApiProperty()
	readonly employeesCount: number;

	@Min(1, { message: "channel:tradeinformationVendor:mindomesticSales" })
	@IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })
	@IsOptional()
	@ApiProperty()
	readonly domesticSales: number;

	@Min(1, { message: "channel:tradeinformationVendor:mininternationalSales" })
	@IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })
	@IsOptional()
	@ApiProperty()
	readonly internationalSales: number;

	@Min(1, { message: "channel:tradeinformationVendor:mindomesticMarketing" })
	@IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })
	@IsOptional()
	@ApiProperty()
	readonly domesticMarketing: number;

	@Min(1, { message: "channel:tradeinformationVendor:mininternationalMarketing" })
	@IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })
	@IsOptional()
	@ApiProperty()
	readonly internationalMarketing: number;

	@ValidateIf(o => o.annualTurnover === undefined)
	@IsOptional()
	@ApiProperty()
	readonly annualTurnoverRange: string;

	@ValidateIf(o => o.annualTurnoverRange === undefined)
	@IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })
	@IsOptional()
	@ApiProperty()
	readonly annualTurnover: number;

	@IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })
	@IsOptional()
	@ApiProperty()
	readonly domesticRevenue: number;

	@IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })
	@IsOptional()
	@ApiProperty()
	readonly internationalRevenue: number;

	@IsNotEmpty({ message: "channel:tradeinformation:startupIsNotEmpty" })
	@IsString({ message: "channel:tradeinformation:startupIsString" })
	@ApiProperty()
	readonly startup: string;

	@ValidateIf(o => o.startup === '1')
	@IsEnum(maturityLevel, {
		each: true,
		message: "channel:tradeinformation:maturityLevelIsEnum"
	})
	@ApiProperty()
	readonly maturitylevel: string[];

	@IsOptional()
	@ApiProperty({ type: ChannelKeyRetailerDto, isArray: true })
	@ArrayNotEmpty({ message: "channel:product:productKeyRetailerArrayNotEmpty" })
	@IsNotEmpty({ message: "channel:product:productKeyRetailerIsNotEmpty" })
	@ValidateNested()
	@Type(() => ChannelKeyRetailerDto)
	readonly channelKeyRetailer: ChannelKeyRetailerDto[];

	@IsOptional()
	@ArrayNotEmpty({ message: "channel:product:deletedkeyretailerIsNotEmpty" })
	@ApiProperty({ type: 'number', isArray: true })
	readonly deletedkeyretailer: number[];

	@IsOptional()
	@MaxLength(2000, {
		message: "validate:field:validateMaxLength"
	})
	@IsString({ message: "validate:field:shouldBeString" })
	@ApiProperty()
	readonly keyretailers: string

	// @IsOptional()
	// @Min(1, { message: "validate:field:validateMinLength" })
	@Max(9999999, { message: "validate:field:validateMaxLength" })
	@ApiProperty()
	readonly no_of_retailers: number;
}
