import { IsString, IsEmail, IsNotEmpty, MinLength, MaxLength, IsNumber, ValidateIf, Min, IsOptional, ArrayNotEmpty, ValidateNested, IsEnum, Max } from 'class-validator';
import { Type } from 'class-transformer';
import { ApiProperty } from '@nestjs/swagger';
import { TradeInformation_Retailer_SpecialityDto } from './tradeInformation_Retailer_Speciality.dto';

export enum maturityLevel {
	PROTOTYPE,
	S_BACKER,
	S_VOLUME
}

export class TradeInformation_RetailerDto {

	@IsOptional()
	@ApiProperty({ type: TradeInformation_Retailer_SpecialityDto, isArray: true })
	@IsNotEmpty({ message: "channel:tradeinformation:retailerSpecialityNotEmpty" })
	@ArrayNotEmpty({ message: "channel:tradeinformation:retailerSpecialityArrayNotEmpty" })
	@ValidateNested()
	@Type(() => TradeInformation_Retailer_SpecialityDto)
	readonly speciality: TradeInformation_Retailer_SpecialityDto[];

	@IsNotEmpty({ message: "channel:tradeinformation:profileTypeIdNotEmpty" })
	@IsNumber({}, { message: "channel:subscription:profileTypeIdonlyacceptInt" })
	@ApiProperty()
	readonly profileTypeId: number;

	@ValidateIf(o => o.employeesCount === undefined)
	@IsNotEmpty({ message: "channel:tradeinformation:employeesRangeNotEmpty" })
	@ApiProperty()
	readonly employeesRange: string;

	@ValidateIf(o => o.employeesRange === undefined)
	@IsNotEmpty({ message: "channel:tradeinformation:employeesCountNotEmpty" })
	@Min(1, { message: "channel:tradeinformationVendor:minEmployeesCount" })
	@ApiProperty()
	readonly employeesCount: number;

	@IsOptional()
	@ValidateIf(o => o.annualTurnover === undefined)
	@IsNotEmpty({ message: "channel:tradeinformation:annualTurnoverRangeNotEmpty" })
	@ApiProperty()
	readonly annualTurnoverRange: string;

	@IsOptional()
	@ValidateIf(o => o.annualTurnoverRange === undefined)
	@IsNotEmpty({ message: "channel:tradeinformation:annualTurnoverNotEmpty" })
	@ApiProperty()
	readonly annualTurnover: number;

	@IsOptional()
	@ApiProperty()
	readonly physicalStore: number;

	@IsOptional()
	@ApiProperty()
	readonly OnlineStore: number;

	@IsNotEmpty({ message: "channel:tradeinformation:startupIsNotEmpty" })
	@IsString({ message: "channel:tradeinformation:startupIsString" })
	@ApiProperty()
	readonly startup: string;

	@ValidateIf(o => o.startup === '1')
	@IsEnum(maturityLevel, { 
		each: true, 
		message: "channel:tradeinformation:maturityLevelIsEnum" 
	})
	@ApiProperty()
	readonly maturitylevel: string[];

	@Min(1, { message: "channel:tradeinformation:minStoreCount" })
	@Max(999999999, { message: "channel:tradeinformation:maxStoreCount" })
	@ApiProperty()
	readonly storeCount: number;
}
