import { IsString, IsNotEmpty, MaxLength, IsNumber, ValidateIf, Min, IsOptional, ArrayNotEmpty, ValidateNested, IsEnum, Max } from 'class-validator';
import { Type } from 'class-transformer';
import { TradeInformation_Brand_NameUpdateDto } from './tradeInformation_Brand_Name.update.dto';
import { ApiProperty } from '@nestjs/swagger';
import { ChannelKeyRetailerDto } from './channelkeyretailer.dto';

export enum maturityLevel {
	PROTOTYPE,
	S_BACKER,
	S_VOLUME
}

export class TradeInformation_VendorUpdateDto {

	@ApiProperty({ type: TradeInformation_Brand_NameUpdateDto, isArray: true })
	@IsOptional()
	@ValidateNested()
	@Type(() => TradeInformation_Brand_NameUpdateDto)
	readonly brand: TradeInformation_Brand_NameUpdateDto[];

	@ValidateIf(o => o.employeesCount === undefined)
	@IsOptional()
	@ApiProperty()
	readonly employeesRange: string;

	@ValidateIf(o => o.employeesRange === undefined)
	@IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })
	@Min(1, { message: "channel:tradeinformationVendor:minEmployeesCount" })
	@IsOptional()
	@ApiProperty()
	readonly employeesCount: number;

	@Min(1, { message: "channel:tradeinformationVendor:mindomesticSales" })
	@IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })
	@IsOptional()
	@ApiProperty()
	readonly domesticSales: number;

	@IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })
	@IsOptional()
	@ApiProperty()
	readonly profileTypeId: number;

	@Min(1, { message: "channel:tradeinformationVendor:mininternationalSales" })
	@IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })
	@IsOptional()
	@ApiProperty()
	readonly internationalSales: number;

	@Min(1, { message: "channel:tradeinformationVendor:mindomesticMarketing" })
	@IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })
	@IsOptional()
	@ApiProperty()
	readonly domesticMarketing: number;

	@Min(1, { message: "channel:tradeinformationVendor:mininternationalMarketing" })
	@IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })
	@IsOptional()
	@ApiProperty()
	readonly internationalMarketing: number;

	@ValidateIf(o => o.annualTurnover === undefined)
	@IsOptional()
	@ApiProperty()
	readonly annualTurnoverRange: string;

	@ValidateIf(o => o.annualTurnoverRange === undefined)
	@IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })
	@IsOptional()
	@ApiProperty()
	readonly annualTurnover: number;

	@IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })
	@IsOptional()
	@ApiProperty()
	readonly domesticRevenue: number;

	@IsNumber({}, { message: "channel:tradeinformation:channelTypeIsNumber" })
	@IsOptional()
	@ApiProperty()
	readonly internationalRevenue: number;

	@IsNotEmpty({ message: "channel:tradeinformation:startupIsNotEmpty" })
	@IsString({ message: "channel:tradeinformation:startupIsString" })
	@ApiProperty()
	readonly startup: string;

	@ValidateIf(o => o.startup === '1')
	@IsEnum(maturityLevel, {
		each: true,
		message: "channel:tradeinformation:maturityLevelIsEnum"
	})
	@ApiProperty()
	readonly maturitylevel: string[];

	@IsOptional()
	@ApiProperty({ type: ChannelKeyRetailerDto, isArray: true })
	@ArrayNotEmpty({ message: "channel:product:productKeyRetailerArrayNotEmpty" })
	@IsNotEmpty({ message: "channel:product:productKeyRetailerIsNotEmpty" })
	@ValidateNested()
	@Type(() => ChannelKeyRetailerDto)
	readonly channelKeyRetailer: ChannelKeyRetailerDto[];

	@IsOptional()
	@ArrayNotEmpty({ message: "channel:product:deletedkeyretailerIsNotEmpty" })
	@ApiProperty({ type: 'number', isArray: true })
	readonly deletedkeyretailer: number[];

	@IsOptional()
	@MaxLength(2000, {
		message: "validate:field:validateMaxLength"
	})
	@IsString({ message: "validate:field:shouldBeString" })
	@ApiProperty()
	readonly keyretailers: string

	// @IsOptional()
	// @Min(1, { message: "validate:field:validateMinLength" })
	@Max(9999, { message: "validate:field:validateMaxLength" })
	@ApiProperty()
	readonly no_of_skus: number;

	// @IsOptional()
	// @Min(1, { message: "validate:field:validateMinLength" })
	@Max(9999999, { message: "validate:field:validateMaxLength" })
	@ApiProperty()
	readonly no_of_distributors: number;

	// @IsOptional()
	// @Min(1, { message: "validate:field:validateMinLength" })
	@Max(9999999, { message: "validate:field:validateMaxLength" })
	@ApiProperty()
	readonly no_of_retailers: number;

	@IsOptional()
	@ApiProperty({ type: ChannelKeyRetailerDto, isArray: true })
	@ArrayNotEmpty({ message: "channel:product:productKeyRetailerArrayNotEmpty" })
	@IsNotEmpty({ message: "channel:product:productKeyRetailerIsNotEmpty" })
	@ValidateNested()
	@Type(() => ChannelKeyRetailerDto)
	readonly channelKeyDistributor: ChannelKeyRetailerDto[];

	@IsOptional()
	@ArrayNotEmpty({ message: "channel:product:deletedkeyretailerIsNotEmpty" })
	@ApiProperty({ type: 'number', isArray: true })
	readonly deletedkeydistributor: number[];

	@IsOptional()
	@MaxLength(2000, {
		message: "validate:field:validateMaxLength"
	})
	@IsString({ message: "validate:field:shouldBeString" })
	@ApiProperty()
	readonly keydistributors: string

}
