import { IsString, IsEmail, IsNotEmpty, MinLength, MaxLength, IsNumber, ValidateIf, Min, IsOptional, ArrayNotEmpty, ValidateNested, IsBoolean, IsEnum, Max } from 'class-validator';
import { Type } from 'class-transformer';
import { TradeInformation_Brand_NameDto } from './tradeInformation_Brand_Name.dto';
import { ApiProperty } from '@nestjs/swagger';
import { ChannelKeyRetailerDto } from './channelkeyretailer.dto';
export enum maturityLevel {
	PROTOTYPE,
	S_BACKER,
	S_VOLUME
}

export class TradeInformation_VendorDto {

	@ApiProperty({ type: TradeInformation_Brand_NameDto, isArray: true })
	@ArrayNotEmpty({ message: "channel:tradeinformation:BrandArray" })
	@IsNotEmpty({ message: "channel:tradeinformation:BrandNotEmpty" })
	@ValidateNested()
	@Type(() => TradeInformation_Brand_NameDto)
	readonly brand: TradeInformation_Brand_NameDto[];

	@ValidateIf(o => o.employeesCount === undefined)
	@IsNotEmpty({ message: "channel:tradeinformation:employeesRangeNotEmpty" })
	@ApiProperty()
	readonly employeesRange: string;

	@ValidateIf(o => o.employeesRange === undefined)
	@IsNotEmpty({ message: "channel:tradeinformation:employeesCountNotEmpty" })
	@Min(1, { message: "channel:tradeinformationVendor:minEmployeesCount" })
	@ApiProperty()
	readonly employeesCount: number;

	@IsNotEmpty({ message: "channel:tradeinformation:profileTypeIdNotEmpty" })
	@IsNumber({}, { message: "channel:subscription:profileTypeIdonlyacceptInt" })
	@ApiProperty()
	readonly profileTypeId: number;

	@IsOptional()
	@Min(1, { message: "channel:tradeinformationVendor:mindomesticSales" })
	@ApiProperty()
	readonly domesticSales: number;

	@IsOptional()
	@Min(1, { message: "channel:tradeinformationVendor:mininternationalSales" })
	@ApiProperty()
	readonly internationalSales: number;

	@IsOptional()
	@Min(1, { message: "channel:tradeinformationVendor:mindomesticMarketing" })
	@ApiProperty()
	readonly domesticMarketing: number;

	@IsOptional()
	@Min(1, { message: "channel:tradeinformationVendor:mininternationalMarketing" })
	@ApiProperty()
	readonly internationalMarketing: number;

	@IsOptional()
	@ValidateIf(o => o.annualTurnover === undefined)
	@IsNotEmpty({ message: "channel:tradeinformation:annualTurnoverRangeNotEmpty" })
	@ApiProperty()
	readonly annualTurnoverRange: string;

	@IsOptional()
	@ValidateIf(o => o.annualTurnoverRange === undefined)
	@IsNotEmpty({ message: "channel:tradeinformation:annualTurnoverNotEmpty" })
	@ApiProperty()
	readonly annualTurnover: number;

	@IsOptional()
	@ApiProperty()
	readonly domesticRevenue: number;

	@IsOptional()
	@ApiProperty()
	readonly internationalRevenue: number;

	@IsNotEmpty({ message: "channel:tradeinformation:startupIsNotEmpty" })
	@IsString({ message: "channel:tradeinformation:startupIsString" })
	@ApiProperty()
	readonly startup: string;

	@ValidateIf(o => o.startup === '1')
	@IsEnum(maturityLevel, { 
		each: true, 
		message: "channel:tradeinformation:maturityLevelIsEnum" 
	})
	@ApiProperty()
	readonly maturitylevel: string[];

	@IsOptional()
	@ApiProperty({ type: ChannelKeyRetailerDto, isArray: true })
	@ArrayNotEmpty({ message: "channel:product:productKeyRetailerArrayNotEmpty" })
	@IsNotEmpty({ message: "channel:product:productKeyRetailerIsNotEmpty" })
	@ValidateNested()
	@Type(() => ChannelKeyRetailerDto)
	readonly channelKeyRetailer: ChannelKeyRetailerDto[];

	@IsOptional()
	@ArrayNotEmpty({ message: "channel:product:deletedkeyretailerIsNotEmpty" })
	@ApiProperty({ type: 'number', isArray: true })
	readonly deletedkeyretailer: number[];

	@IsOptional()
	@MaxLength(2000, {
		message: "validate:field:validateMaxLength"
	})
	@IsString({ message: "validate:field:shouldBeString" })
	@ApiProperty()
	readonly keyretailers: string

	// @IsOptional()
	// @Min(1, { message: "validate:field:validateMinLength" })
	@Max(9999, { message: "validate:field:validateMaxLength" })
	@ApiProperty()
	readonly no_of_skus: number;

	// @IsOptional()
	// @Min(1, { message: "validate:field:validateMinLength" })
	@Max(9999999, { message: "validate:field:validateMaxLength" })
	@ApiProperty()
	readonly no_of_distributors: number;

	// @IsOptional()
	// @Min(1, { message: "validate:field:validateMinLength" })
	@Max(9999999, { message: "validate:field:validateMaxLength" })
	@ApiProperty()
	readonly no_of_retailers: number;

	@IsOptional()
	@ApiProperty({ type: ChannelKeyRetailerDto, isArray: true })
	@ArrayNotEmpty({ message: "channel:product:productKeyRetailerArrayNotEmpty" })
	@IsNotEmpty({ message: "channel:product:productKeyRetailerIsNotEmpty" })
	@ValidateNested()
	@Type(() => ChannelKeyRetailerDto)
	readonly channelKeyDistributor: ChannelKeyRetailerDto[];

	@IsOptional()
	@ArrayNotEmpty({ message: "channel:product:deletedkeyretailerIsNotEmpty" })
	@ApiProperty({ type: 'number', isArray: true })
	readonly deletedkeydistributor: number[];

	@IsOptional()
	@MaxLength(2000, {
		message: "validate:field:validateMaxLength"
	})
	@IsString({ message: "validate:field:shouldBeString" })
	@ApiProperty()
	readonly keydistributors: string
}
