import { Table, Column, Model, DataType, BelongsTo, ForeignKey, AllowNull, BeforeCreate, BeforeUpdate } from 'sequelize-typescript';
import { ApiProperty } from '@nestjs/swagger';
import { Channel } from './Channel.entity';
import { PaymentTransaction } from './PaymentTransaction.entity';
import { AutoRenewalTrackingStatus } from './AutoRenewalTrackingStatus.entity';
import { ChannelSubscription } from './ChannelSubscription.entity';
import { UUIDV4 } from 'sequelize/types';

var Sequelize = require('sequelize'); 

@Table({
	tableName: "tblautorenewaltracking"
})

export class AutoRenewalTracking extends Model<AutoRenewalTracking> {

	@Column({
		type: DataType.INTEGER,
		allowNull: false,
		// defaultValue:UUIDV4,//autoIncrement: true,
		unique: true,
		primaryKey: true,
	})
	@ApiProperty()
	autorenewaltrackingid: number

	@ForeignKey(() => Channel)
	@Column(DataType.INTEGER)
	channelid: number

	@BelongsTo(() => Channel)
	channel: Channel

	@ForeignKey(() => ChannelSubscription)
	@Column(DataType.INTEGER)
	channelsubscriptionid: number

	@BelongsTo(() => ChannelSubscription)
	channelSubscription: ChannelSubscription
	
	@Column({
		type: DataType.INTEGER,
	})
	@ForeignKey(() => AutoRenewalTrackingStatus)
	statusid: number

	@ApiProperty()
	@Column(DataType.INTEGER)
	remindercount: number

	@ForeignKey(() => PaymentTransaction)
	@Column(DataType.INTEGER)
	paymenttransactionid: number

	@Column(DataType.TIME)
	endat: number

	@BelongsTo(() => PaymentTransaction)
	paymentTransaction: PaymentTransaction

    @Column(DataType.DATE)
    createdat:Date

    @Column(DataType.DATE)
	updatedat:Date
	
	@Column(DataType.DATE)
	deletedat:Date
	
	@BeforeCreate  
	@BeforeUpdate  
	public static async dateUpdate(autoRenewalTracking: AutoRenewalTracking, options: any) {  
		autoRenewalTracking.updatedat = Sequelize.fn('NOW');  
	}
	
}
