import {Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, ForeignKey, HasOne, HasMany, Scopes, BelongsTo, BelongsToMany, AfterUpdate, AfterBulkUpdate } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Address } from './Address.entity';
import { ProfileType } from './ProfileType.entity'
import { SignUpHistory } from '../../user/models/SignUpHistory.entity';
import { ChannelType } from './ChannelType.entity';
import { ChannelDetail } from './ChannelDetail.entity';
import { ChannelSubscription } from './ChannelSubscription.entity';
import { Contact } from '../../user/models/Contact.entity';
import { ChannelCertificate } from './ChannelCertificate.entity';
import { ChannelSocialLink } from './ChannelSocialLink.entity';
import { ChannelB2B } from './ChannelB2B.entity';
import { ChannelAward } from './ChannelAward.entity';
import { User } from '../../user/models/User.entity';
import { ChannelProductCategorie } from '../models/ChannelProductCategorie.entity';
import { ChannelProductGroup } from '../models/ChannelProductGroup.entity';
import { ChannelServiceList } from '../models/ChannelServiceList.entity';
import { ChannelSpeciality } from '../models/ChannelSpeciality.entity';
import { SignUpStatus } from '../../user/models/SignUpStatus.entity';
import { Country } from '../../common/models/Country.entity';
import { ChannelBrand } from '../models/ChannelBrand.entity';
import { ChannelCustomerProfile } from '../models/ChannelCustomerProfile.entity';
import { ChannelLocation } from '../models/ChannelLocation.entity';
import { ProductGroup } from './../../product/models/ProductGroup.entity';
import { ProductCategory } from './../../product/models/ProductCategory.entity';
import { RegionCountryJCT } from './../../common/models/RegionCountryJCT.entity';
import { ServiceList } from './ServiceList.entity';
import { Brand } from '../../product/models/Brand.entity';
import { ChannelPaymentOptionType } from './ChannelPaymentOptionType.entity';
import { ChannelPaymentTerm } from './ChannelPaymentTerm.entity';
import { ChannelJCT } from './ChannelJCT.entity';
import { ShareType } from './ShareType.entity';
import { ChannelSharePreference } from './ChannelSharePreference.entity';
import { SubscriptionPlan } from './SubscriptionPlan.entity';
import { Subscription } from './Subscription.entity';
import { PaymentOptionType } from './PaymentOptionType.entity';
import { PaymentTerm } from './PaymentTerm.entity';
import { ReferenceContact } from './../../user/models/ReferenceContact.entity';
// import { RedisHelper } from './../../../redis/helper/redis.helper';
import { Claim } from './Claim.entity';
import { ChannelDocument } from './ChannelDocument.entity';
import { EventParticipants } from './EventParticipants.entity';

const tableOptions: IDefineOptions = { tableName: 'tblChannel' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
	//Used
	basicInfom: {
		attributes: ['companyName', 'showId', 'channelId', 'planId', 'signUpStatusId', 'channelTypeId', 'profileTypeId', 'regAddressId', 'createdAt', 'updatedAt', 'isOEM', 'hubspotCompanyId', 'billAddressId', 'stripeCustomerId']
	},
	basicInfomWithStripeCustomer: {
		attributes: ['channelId', 'signUpStatusId', 'channelTypeId', 'profileTypeId', 'isFrenchTech', 'regAddressId', 'stripeCustomerId']
	},
	subscription: {
		include: [ChannelSubscription]
	},
	profileInfom: {
		attributes: ['channelId', 'signUpStatusId', 'channelTypeId', 'companyName', 'companyName', 'companyMail', 'countryId', 'showId', 'profileTypeId', 'regAddressId', 'billAddressId', 'isActive', 'isFrenchTech']
	},
	channelDetail: {
		attributes: ['channelId', 'companyName', 'companyMail', 'signUpStatusId', 'channelTypeId', 'countryId', 'profileTypeId', 'isActive', 'oldChannelId', 'deletedAt', 'isFrenchTech', 'createdAt', 'isOEM']
	},
	orderInfo: {
		attributes: ['channelId', 'companyName', 'companyMail', 'countryId', 'showId', 'profileTypeId', 'channelTypeId', 'isActive', 'oldChannelId', 'profileStrength', 'isFrenchTech']
	}
}))

export class Channel extends Model<Channel>{

	static CHANNEL_TYPE_HUB = 1;
	static CHANNEL_TYPE_VENDOR = 2;
	static CHANNEL_TYPE_DISTRIBUTOR = 3;
	static CHANNEL_TYPE_RETAILER = 4;
	static CHANNEL_TYPE_SALESREP = 5;
	static CHANNEL_TYPE_SIGNUP = 6;

	static REDIS_KEY = 'CHANNEL';

	static FRONTEND_CHANNEL = [
		Channel.CHANNEL_TYPE_VENDOR,
		Channel.CHANNEL_TYPE_DISTRIBUTOR,
		Channel.CHANNEL_TYPE_RETAILER,
		Channel.CHANNEL_TYPE_SALESREP,
		Channel.CHANNEL_TYPE_SIGNUP
	];

	static BACKEND_CHANNEL = [
		Channel.CHANNEL_TYPE_HUB
	];

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	channelId: number

	@ForeignKey(() => SignUpStatus)
	@Column(DataType.BIGINT)
	signUpStatusId: number;

	@BelongsTo(() => SignUpStatus)
	signUpStatus: SignUpStatus;

	@ForeignKey(() => SubscriptionPlan)
	@Column(DataType.BIGINT)
	planId: number;

	@Column(DataType.BIGINT)
	hubspotCompanyId: number;
	
	@BelongsTo(() => SubscriptionPlan)
	plan: SubscriptionPlan;

	@ForeignKey(() => Subscription)
	@Column(DataType.BIGINT)
	subscriptionId: number;

	@BelongsTo(() => Subscription)
	subscription: Subscription;

	@Column(DataType.CHAR(255))
	companyName: string

	@Column(DataType.CHAR(255))
	companyMail: string

	@ForeignKey(() => Country)
	@Column(DataType.BIGINT)
	countryId: number;

	@BelongsTo(() => Country)
	country: Country;

	@Column(DataType.CHAR(255))
	showId: string

	@Column(DataType.BIGINT)
	oldChannelId: number

	@ForeignKey(() => ProfileType)
	@Column(DataType.BIGINT)
	profileTypeId: number;

	@BelongsTo(() => ProfileType)
	profileType: ProfileType;

	@ForeignKey(() => ChannelType)
	@Column(DataType.BIGINT)
	channelTypeId: number

	@BelongsTo(() => ChannelType)
	channelType: ChannelType[];

	@HasOne(() => ChannelSubscription)
	channelSubscription: ChannelSubscription;

	@ForeignKey(() => Address)
	@Column(DataType.BIGINT)
	regAddressId: number

	@BelongsTo(() => Address, 'regAddressId')
	regAddress: Address;

	@ForeignKey(() => Address)
	@Column(DataType.BIGINT)
	billAddressId: number

	@BelongsTo(() => Address, 'billAddressId')
	billAddress: Address;

	@Column
	isActive: string;

	@Column(DataType.DATE)
	deletedAt: Date

	@Column(DataType.DATE)
	createdAt: Date

	@Column(DataType.DATE)
	updatedAt: Date

	@Column(DataType.CHAR(255))
	slug: string

	@Column
	isFrenchTech: string;

	@Column
	isCompanyPublic: string;

	@Column
	isAdminCreated: string;

	@Column
	actAsDistributor: string;

	@HasMany(() => SignUpHistory, 'signupId')
	signUpHistory: SignUpHistory;

	@HasMany(() => ChannelCertificate, 'channelId')
	channelCertificate: ChannelCertificate[];

	@HasMany(() => ChannelSocialLink, 'channelId')
	channelSocial: ChannelSocialLink[];

	@HasMany(() => ChannelB2B, 'channelId')
	channelB2B: ChannelB2B[];

	@HasMany(() => ChannelAward, 'channelId')
	channelAward: ChannelAward[];

	@HasOne(() => ChannelDetail, 'channelId')
	channelDetail: ChannelDetail;

	@HasMany(() => ChannelSpeciality, 'channelId')
	channelSpeciality: ChannelSpeciality[];

	@HasMany(() => ChannelCustomerProfile, 'channelId')
	channelCustomerProfile: ChannelCustomerProfile[];

	@HasMany(() => ChannelProductCategorie, 'channelId')
	channelProductCategorie: ChannelProductCategorie[];

	@HasMany(() => ChannelLocation, 'channelId')
	channelLocation: ChannelLocation[];

	@HasMany(() => ChannelServiceList, 'channelId')
	channelServiceList: ChannelServiceList[];

	@HasOne(() => Contact) // TODO: Should return default contact
	contact: Contact;

	@HasMany(() => Contact, 'channelId')
	contacts: Contact[];

	@HasMany(() => ChannelBrand, 'channelId')
	channelBrand: ChannelBrand[];

	@HasMany(() => ChannelProductGroup, 'channelId')
	channelProductGroup: ChannelProductGroup[];

	// @HasMany(() => ChannelPaymentOptionType, 'channelid')
	// channelPaymentOptionType: ChannelPaymentOptionType[];

	@BelongsToMany(() => PaymentOptionType, {
		through: {
			model: () => ChannelPaymentOptionType,
			scope: {
				deletedat: null
			}
		}
	})
	channelPaymentOptionType: PaymentOptionType[];

	// @HasMany(() => ChannelPaymentTerm, 'channelid')
	// channelPaymentTerm: ChannelPaymentTerm[];

	@BelongsToMany(() => PaymentTerm, {
		through: {
			model: () => ChannelPaymentTerm,
			scope: {
				deletedat: null
			}
		}
	})
	channelPaymentTerm: PaymentTerm[];

	@Column(DataType.INTEGER)
	profileStrength: number

	@HasMany(() => User, 'channelId')
	user: User;

	@BelongsToMany(() => ProductGroup, {
		through: {
			model: () => ChannelProductGroup,
			scope: {
				deletedAt: null
			}
		}
	})
	productGroup: ProductGroup[];

	@BelongsToMany(() => ProductCategory, {
		through: {
			model: () => ChannelProductCategorie,
			scope: {
				deletedAt: null
			}
		}
	})
	productCategory: ProductCategory[];

	@BelongsToMany(() => ProfileType, {
		through: {
			model: () => ChannelCustomerProfile,
			scope: {
				deletedAt: null
			}
		}
	})
	customerProfile: ProfileType[];

	@BelongsToMany(() => ServiceList, {
		through: {
			model: () => ChannelServiceList,
			scope: {
				deletedAt: null
			}
		}
	})
	serviceList: ServiceList[];

	@BelongsToMany(() => ProductGroup, {
		through: {
			model: () => ChannelSpeciality,
			scope: {
				deletedAt: null
			}
		}
	})
	speciality: ProductGroup[];

	@BelongsToMany(() => Brand, {
		through: {
			model: () => ChannelBrand,
			scope: {
				deletedAt: null
			}
		}
	})
	brand: Brand[];

	@BelongsToMany(() => ShareType, {
		through: {
			model: () => ChannelSharePreference,
			scope: {
				deletedat: null
			}
		}
	})
	shareType: ShareType[];

	@BelongsToMany(() => RegionCountryJCT, {
		through: {
			model: () => ChannelLocation,
			scope: {
				deletedAt: null
			}
		}
	})
	locations: RegionCountryJCT[];

	@HasOne(() => ChannelJCT, 'partnerChannelId')
	channelJCT: ChannelJCT[];

	@HasOne(() => ChannelJCT, 'channelId')
	channelJCTPartner: ChannelJCT[];

	@HasMany(() => ReferenceContact, 'channelid')
	salesrepReference: ReferenceContact[];

	@HasOne(() => Claim, 'channelId')
	claim: Claim;

	@Column(DataType.CHAR(255))
	stripeCustomerId: string

	@Column(DataType.CHAR(255))
	stripePaymentMethodId: string

	@Column
	autoRenewalOff: string;

	@HasMany(() => ChannelDocument, 'channelId')
	channelDocument: ChannelDocument[];

	@HasMany(() => EventParticipants, 'channelId')
	participants: EventParticipants[];

	// Don't change labels, it related to suggestion
	get channelLabel(): string {
		var label = '';
		if (this.channelTypeId == Channel.CHANNEL_TYPE_HUB) {
			label = 'Hub';
		} else if (this.channelTypeId == Channel.CHANNEL_TYPE_VENDOR) {
			label = 'Vendor';
		} else if (this.channelTypeId == Channel.CHANNEL_TYPE_DISTRIBUTOR) {
			label = 'Distributor';
		} else if (this.channelTypeId == Channel.CHANNEL_TYPE_RETAILER) {
			label = 'Retailer';
		} else if (this.channelTypeId == Channel.CHANNEL_TYPE_SALESREP) {
			label = 'Salesrep';
		} else if (this.channelTypeId == Channel.CHANNEL_TYPE_SIGNUP) {
			label = 'Signup';
		}

		return label;
	}

	@AfterUpdate
	@AfterBulkUpdate
	static async removeRedisKey(instance) {
		// let redis = new RedisHelper();

		let channelId = +instance.channelId || null;
		if (instance.type == 'BULKUPDATE') {
			channelId = +instance.where.channelId || null;
		}

		if (channelId) {
			// await redis.delete(Channel.REDIS_KEY + ":ID:" + channelId);
		}
	}
	@Column(DataType.CHAR(255))
	isOEM: string

	@Column
	isPublic: string;

	@Column
	isFeatured: string;
}
