import { Table, 
    Column, 
    PrimaryKey, 
    Model, 
    AutoIncrement, 
    DataType, 
    BeforeCreate, 
    AllowNull, 
    BeforeUpdate,
    ForeignKey,
    BelongsTo,
    Scopes
  } from 'sequelize-typescript';
  import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
  import { Channel } from './Channel.entity';
  
  var Sequelize = require('sequelize');
  
  const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblChannelAward' } as IDefineOptions;
  
  @Table(tableOptions)

  @Scopes(() => ({
    profileInfo: {
      attributes: ['channelAwardId', 'channelId', 'awardDetail', 'issuedBy', 'validUpto']
    }
  }))
  
export class ChannelAward extends Model<ChannelAward>{

  @PrimaryKey
  @AutoIncrement
  @AllowNull(false)
  @Column(DataType.BIGINT)
  channelAwardId:number

  @ForeignKey(() => Channel) 
  @Column(DataType.BIGINT)
  channelId:number

  @BelongsTo(() => Channel)
  channel: Channel[];

  @Column(DataType.CHAR(8000))
  awardDetail:string

  @Column(DataType.CHAR(255))
  issuedBy:string

  @Column(DataType.DATE)
  validUpto:Date

  @ForeignKey(() => Channel) 
  @Column(DataType.BIGINT)
  documentId:number

  @Column(DataType.DATE)
  deletedAt:Date

  @Column(DataType.DATE)
  createdAt:Date

  @Column(DataType.DATE)
  updatedAt:Date

  @Column
  isLifeTime: string;
    
  @BeforeCreate 
  @BeforeUpdate 
  public static async dateUpdate(channelAward: ChannelAward, options: any) { 
    channelAward.updatedAt = Sequelize.fn('NOW'); 
  }

}