import {
  Table,
  Column,
  PrimaryKey,
  Model,
  AutoIncrement,
  DataType,
  BeforeCreate,
  AllowNull,
  BeforeUpdate,
  ForeignKey,
  HasMany,
  BelongsTo,
  HasOne,
  Scopes
} from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
// import { DataTypeTime } from 'sequelize';
import { Channel } from './Channel.entity';
import { Brand } from '../../product/models/Brand.entity';

var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblChannelBrand' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
  channelDetail: {
    attributes: ['channelId', 'brandId', 'deletedAt']
  },
  tradeinfo: {
    attributes: {
      exclude: ['deletedAt', 'createdAt', 'updatedAt']
    }
  }
}))

export class ChannelBrand extends Model<ChannelBrand>{

  @PrimaryKey
  @AutoIncrement
  @AllowNull(false)
  @Column(DataType.BIGINT)
  channelBrandId: number

  @ForeignKey(() => Channel)
  @Column(DataType.BIGINT)
  channelId: number

  @BelongsTo(() => Channel)
  channel: Channel[];

  @ForeignKey(() => Brand)
  @Column(DataType.BIGINT)
  brandId: number

  @BelongsTo(() => Brand)
  brand: Brand;

  @Column(DataType.DATE)
  deletedAt: Date

  @Column(DataType.DATE)
  createdAt: Date

  @Column(DataType.DATE)
  updatedAt: Date

  @BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(channelBrand: ChannelBrand, options: any) { 
		channelBrand.updatedAt = Sequelize.fn('NOW'); 
	}

}