import { Table, 
    Column, 
    PrimaryKey, 
    Model, 
    AutoIncrement, 
    DataType, 
    BeforeCreate, 
    AllowNull, 
    BeforeUpdate,
    ForeignKey,
    HasMany,
    BelongsTo,
    Scopes  
  } from 'sequelize-typescript';
  import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
  import { Channel } from './Channel.entity';
  import { CertificateType } from './CertificateType.entity';
  
  var Sequelize = require('sequelize');
  
  const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblChannelCerificate' } as IDefineOptions;
  
  @Table(tableOptions)

  @Scopes(() => ({
    profileInfo: {
      attributes: ['channelCerificateId', 'channelId', 'cerificateTypeId', 'cerificateName', 'cerificateNumber', 'cerificateDesc']
    }
  }))
  
  export class ChannelCertificate extends Model<ChannelCertificate>{
  
    @PrimaryKey
    @AutoIncrement
    @AllowNull(false)
    @Column(DataType.BIGINT)
    channelCerificateId:number
    
    @ForeignKey(() => Channel) 
    @Column(DataType.BIGINT)
    channelId:number
  
    @BelongsTo(() => Channel)
    channel: Channel[];

    @ForeignKey(() => CertificateType) 
    @Column(DataType.BIGINT)
    cerificateTypeId:number
  
    @BelongsTo(() => CertificateType)
    certificateType: CertificateType;
  
    @AllowNull(false)
    @Column(DataType.CHAR(255))
    cerificateName:string

    @AllowNull(false)
    @Column(DataType.CHAR(255))
    cerificateNumber:string

    @Column(DataType.CHAR(8000))
    cerificateDesc:string

    @Column(DataType.BIGINT)
    documentId:string

    @Column(DataType.DATE)
    deletedAt:Date

    @Column(DataType.DATE)
    createdAt:Date

    @Column(DataType.DATE)
    updatedAt:Date

    @BeforeCreate 
    @BeforeUpdate 
    public static async dateUpdate(channelCertificate: ChannelCertificate, options: any) { 
      channelCertificate.updatedAt = Sequelize.fn('NOW'); 
    }

    @Column
    vat_verified: string

  }