import { Table, 
    Column, 
    PrimaryKey, 
    Model, 
    AutoIncrement, 
    DataType, 
    BeforeCreate, 
    AllowNull, 
    BeforeUpdate,
    ForeignKey,
    BelongsTo,
    Scopes
  } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Channel } from './Channel.entity';
import { Contact } from '../../user/models/Contact.entity';
import { ShareType } from './ShareType.entity';
var Sequelize = require('sequelize');
  
const tableOptions: IDefineOptions = { timestamps: false, tableName: 'tblchannelcontactsuggestion' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
  contactSuggestionInfo: {
    attributes: ['channelcontactsuggestion', 'channelid', 'contactid', 'sharetypeid', 'updatedby']
  }
}))

export class ChannelContactSuggestion extends Model<ChannelContactSuggestion>{

  @PrimaryKey
  @AutoIncrement
  @AllowNull(false)
  @Column(DataType.BIGINT)
  channelcontactsuggestion:number

  @ForeignKey(() => Channel) 
  @Column(DataType.BIGINT)
  channelid:number

  @BelongsTo(() => Channel)
  channel: Channel[];

  @ForeignKey(() => Contact) 
  @Column(DataType.BIGINT)
  contactid:number

  @BelongsTo(() => Contact)
  contact: Contact[];

  @ForeignKey(() => ShareType) 
  @Column(DataType.BIGINT)
  sharetypeid:number

  @BelongsTo(() => ShareType)
  sharetype: ShareType[];

  @Column(DataType.BIGINT)
  updatedby:number

  @Column(DataType.DATE)
  deletedat:Date
  
  @Column(DataType.DATE)
  createdat:Date

  @Column(DataType.DATE)
  updatedat:Date

  @BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(channelContactSuggestion: ChannelContactSuggestion, options: any) { 
		channelContactSuggestion.updatedat = Sequelize.fn('NOW'); 
	}

}