import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, ForeignKey, BelongsTo, Scopes } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Channel } from './Channel.entity';
import { Media } from '../../media/models/Media.entity';
import { Currency } from './Currency.entity';

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblChannelDetail' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
	basicInfom: {
		attributes: ['channelDetailId', 'channelLogoId', 'channelIconId', 'estYear', 'regYear', 'detailDesc', 'webSiteUrl', 'expectations', 'storeCount', 'no_of_retailers', 'no_of_skus', 'no_of_distributors', 'no_of_vendors']
	},
	channelDetail: {
		attributes: ['channelLogoId', 'channelIconId', 'webSiteUrl', 'estYear', 'regYear', 'detailDesc', 'expectations', 'empCntRange', 'empCnt', 'empCntDSale', 'empCntISale', 'empCntDMar', 'empCntIMar', 'turnoverRange', 'turnover', 'revD', 'revI', 'deletedAt', 'updatedAt', 'turnoverPhyStore', 'turnoverOnStore', 'startup', 'storeCount', 'maturitylevel', 'no_of_retailers', 'no_of_skus', 'no_of_distributors', 'no_of_vendors', 'introvideoId']
	},
	tradeinfo: {
		attributes: {
			exclude: ['deletedAt', 'createdAt', 'updatedAt']
		}
	},
	orderInfo: {
		attributes: ['channelLogoId']
	},
	settings: {
		attributes: ['shareStatus']
	}

}))
export class ChannelDetail extends Model<ChannelDetail>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	channelDetailId: number

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	channelId: number

	@BelongsTo(() => Channel)
	channel: Channel[];

	@ForeignKey(() => Media)
	@Column(DataType.BIGINT)
	channelLogoId: number

	@BelongsTo(() => Media, 'channelLogoId')
	channelLogo: Media;

	@ForeignKey(() => Media)
	@Column(DataType.BIGINT)
	channelIconId: number

	@BelongsTo(() => Media, 'channelIconId')
	channelIcon: Media;

	@ForeignKey(() => Media)
	@Column(DataType.BIGINT)
	channelBannerId: number

	@BelongsTo(() => Media, 'channelBannerId')
	channelBanner: Media;

	@Column(DataType.CHAR(255))
	phone1: string

	@Column(DataType.CHAR(255))
	phone2: string

	@Column(DataType.CHAR(255))
	mobileNo: string

	@Column(DataType.CHAR(255))
	webSiteUrl: string

	@Column(DataType.INTEGER)
	estYear: number

	@Column(DataType.INTEGER)
	regYear: number

	@Column(DataType.CHAR(80000))
	detailDesc: string

	@Column(DataType.CHAR(2000))
	keyretailers: string

	@Column(DataType.CHAR(2000))
	keydistributors: string

	@Column(DataType.CHAR(80000))
	expectations: string

	@Column(DataType.CHAR(10000))
	companyDesc: string

	@Column(DataType.CHAR(100))
	empCntRange: string

	@Column(DataType.INTEGER)
	empCnt: number

	@Column(DataType.INTEGER)
	empCntDSale: number

	@Column(DataType.INTEGER)
	empCntISale: number

	@Column(DataType.INTEGER)
	empCntDMar: number

	@Column(DataType.INTEGER)
	empCntIMar: number

	@Column(DataType.CHAR(100))
	turnoverRange: string

	@Column(DataType.DECIMAL(50, 2))
	turnover: number

	@Column(DataType.INTEGER)
	revD: number

	@Column(DataType.INTEGER)
	revI: number;

	@Column(DataType.INTEGER)
	turnoverPhyStore: number;

	@Column(DataType.INTEGER)
	turnoverOnStore: number;

	@Column(DataType.DATE)
	deletedAt: Date

	@AllowNull(false)
	@Column(DataType.DATE)
	createdAt: Date

	@Column(DataType.DATE)
	updatedAt: Date;

	@Column(DataType.CHAR(255))
	stripeConnectId: string;

	@Column(DataType.CHAR(255))
	paymentoptionothers: string

	@ForeignKey(() => Currency)
	@Column(DataType.BIGINT)
	currencyid: number

	@BelongsTo(() => Currency)
	currency: Currency[];

	@Column(DataType.CHAR(255))
	ownerFirstName: string

	@Column(DataType.CHAR(255))
	ownerLastName: string

	@Column
	startup: string;

	@Column({
		type: DataType.ARRAY({
			type: DataType.ENUM('PROTOTYPE', 'S_BACKER', 'S_VOLUME')
		})
	})
	maturitylevel: string[]

	@Column(DataType.INTEGER)
	storeCount: number;

	@Column(DataType.INTEGER)
	no_of_skus: number;

	@Column(DataType.INTEGER)
	no_of_vendors: number;

	@Column(DataType.INTEGER)
	no_of_distributors: number;

	@Column(DataType.INTEGER)
	no_of_retailers: number;

	@ForeignKey(() => Media)
	@Column(DataType.BIGINT)
	introvideoId: number

	@BelongsTo(() => Media, 'introvideoId')
	introvideo: Media;

	@Column(DataType.STRING)
	isPhysicalSales: string;

	@Column(DataType.INTEGER)
	physicalSales: number;

	@Column(DataType.CHAR(2000))
	keyresellers: string


	@Column(DataType.INTEGER)
	no_of_resellers: number;

	@Column(DataType.BOOLEAN)
	shareStatus: boolean;

	@Column(DataType.STRING)
	bannerURL: string

}
