import {
    Table,
    Column,
    PrimaryKey,
    Model,
    AutoIncrement,
    DataType,
    AllowNull,
    ForeignKey,
    BelongsTo
  } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Media } from '../../media/models';
import { Channel } from './Channel.entity';
import { OldChannel } from './OldChannel.entity';

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblChannelDocument' } as IDefineOptions;

@Table(tableOptions)


export class ChannelDocument extends Model<ChannelDocument>{

    @PrimaryKey
    @AutoIncrement
    @AllowNull(false)
    @Column(DataType.BIGINT)
    channelDocumentId: number

    @ForeignKey(() => Channel)
    @Column(DataType.BIGINT)
    channelId: number

    @ForeignKey(() => OldChannel)
    @Column(DataType.BIGINT)
    oldChannelId: number

    @BelongsTo(() => Channel)
    channel: Channel[];

    @ForeignKey(() => Media)
    @Column(DataType.BIGINT)
    documentId: number

    @Column(DataType.CHAR(255))
    name: string

    @Column(DataType.CHAR(255))
    description: string

    @BelongsTo(() => Media)
    document: Media;

    @Column(DataType.BIGINT)
    updatedBy: number

    @Column(DataType.DATE)
    createdAt: Date

    @Column(DataType.DATE)
    updatedAt: Date;

    @Column(DataType.DATE)
    deletedAt: Date

}