import { BeforeCreate, BeforeUpdate, Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, ForeignKey, BelongsTo, HasOne, BelongsToMany } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Channel } from './Channel.entity';
import { ConnectionStatusType } from './ConnectionStatusType.entity';
import { ContactJCT } from './ContactJCT.entity';
var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { tableName: 'tblChannelJCT' } as IDefineOptions;

@Table(tableOptions)

export class ChannelJCT extends Model<ChannelJCT>
{
	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	channelJCTId: number;

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	channelId: number;

	@BelongsTo(() => Channel, 'channelId')
	Channel: Channel;

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	partnerChannelId: number;

	@BelongsTo(() => Channel, 'partnerChannelId')
	partnerChannel: Channel;

	@ForeignKey(() => ConnectionStatusType)
	@Column(DataType.BIGINT)
	connectionStatusTypeId: number;

	@Column
	leadManagementTag: string;

	@Column
	partnerLeadManagementTag: string;

	@Column
	note: string;

	@BelongsTo(() => ConnectionStatusType)
	connectionStatusType: ConnectionStatusType;

	@HasOne(() => ContactJCT, 'channelJCTId')
	contactJCT: ContactJCT[];

	@Column(DataType.DATE)
	deletedat: Date

	@AllowNull(false)
	@Column(DataType.DATE)
	createdAt: Date

	@Column(DataType.DATE)
	updatedAt: Date

	@Column
	subscription_queued: string

	@Column(DataType.DATE)
	last_remainder_sent_at: Date

	@Column(DataType.INTEGER)
	no_of_remainders_sent: number

	@Column(DataType.BIGINT)
	requested_to: number;

	@Column(DataType.BIGINT)
	requested_by: number;

	@Column(DataType.INTEGER)
	request_sent_at: Date
	
	@Column(DataType.INTEGER)
	suggestion_connect_flag: number

	@Column
	isConfidential: string;

	@Column(DataType.BIGINT)
	introVideo: number;
	
	@Column(DataType.BIGINT)
	transToContact: number;

	@Column(DataType.BIGINT)
	transFromContact: number;


	@BeforeCreate
	@BeforeUpdate
	public static async dateUpdate(channelBrand: ChannelJCT, options: any) {
		channelBrand.updatedAt = Sequelize.fn('NOW');
	}
}
