import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, ForeignKey, BelongsTo, Scopes } from 'sequelize-typescript';
import { ChannelType } from './ChannelType.entity';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Channel } from './Channel.entity';
import { Country } from '../../common/models/Country.entity';
import { Media } from '../../media/models';

const tableOptions: IDefineOptions = { timestamps: false, tableName: 'tblchannelkeypartner' } as IDefineOptions;
@Table(tableOptions)
@Scopes(() => ({
	productInfo: {
		attributes: ['channelkeypartnerid', 'channelId', 'partnername', 'partner_type', 'countryid', 'documentid', 'updatedby', 'deletedat']
	}
}))

export class ChannelKeyPartner extends Model<ChannelKeyPartner> {

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	channelkeypartnerid: number

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	channelId: number

	@BelongsTo(() => Channel)
	channel: Channel;

	@Column(DataType.CHAR(255))
	partnername: string

	@ForeignKey(() => ChannelType)
	@Column(DataType.BIGINT)
	partner_type: number

	@ForeignKey(() => Country)
	@Column(DataType.BIGINT)
	countryid: number

	@BelongsTo(() => Country)
	country: Country;

	@ForeignKey(() => Media)
	@Column(DataType.BIGINT)
	documentid: number

	@BelongsTo(() => Media)
	partnerImage: Media;

	@Column(DataType.BIGINT)
	updatedby: number

	@AllowNull(false)
	@Column(DataType.DATE)
	createdat: Date

	@Column(DataType.DATE)
	updatedat: Date;

	@Column(DataType.DATE)
	deletedat: Date
}
