import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, ForeignKey, BelongsTo, Scopes, BeforeCreate, BeforeUpdate } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Channel } from './Channel.entity';
import { RegionCountryJCT } from '../../common/models/RegionCountryJCT.entity';
import { User } from './../../user/models/User.entity';
var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { tableName: 'tblChannelLocation' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
	regionCountryJCT: {
		attributes: ['regionCountryJCTId', 'locationType']
	},
	channelDetail: {
		attributes: ['channelLocationId', 'locationType', 'regionCountryJCTId', 'deletedAt']
	}
}))

export class ChannelLocation extends Model<ChannelLocation>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	channelLocationId: number

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	channelId: number;

	@BelongsTo(() => Channel)
	Channel: Channel[];

	@Column
	locationType: string

	@ForeignKey(() => RegionCountryJCT)
	@Column(DataType.BIGINT)
	regionCountryJCTId: number

	@BelongsTo(() => RegionCountryJCT)
	RegionCountryJCT: RegionCountryJCT[];

	@Column(DataType.DATE)
	deletedAt: Date

	@AllowNull(false)
	@Column(DataType.DATE)
	createdAt: Date

	@Column(DataType.DATE)
	updatedAt: Date

	@ForeignKey(() => User)
	@Column(DataType.BIGINT)
	updatedBy: number

	@BelongsTo(() => User)
	User: User;

	@BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(channelLocation: ChannelLocation, options: any) { 
		channelLocation.updatedAt = Sequelize.fn('NOW'); 
	}
}
