import {
  Table,
  Column,
  PrimaryKey,
  Model,
  AutoIncrement,
  DataType,
  BeforeCreate,
  AllowNull,
  BeforeUpdate,
  ForeignKey,
  BelongsTo,
  Scopes
} from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Channel } from './Channel.entity';
import { ProductCategory } from '../../product/models/ProductCategory.entity';
var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblChannelProductCategorie' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
  channelDetail: {
    attributes: ['channelProductCategorieId', 'productCategorieId', 'deletedAt']
  },
  tradeinfo: {
    attributes: {
      exclude: ['deletedAt', 'createdAt', 'updatedAt']
    }
  }
}))

export class ChannelProductCategorie extends Model<ChannelProductCategorie>{

  @PrimaryKey
  @AutoIncrement
  @AllowNull(false)
  @Column(DataType.BIGINT)
  channelProductCategorieId: number

  @ForeignKey(() => Channel)
  @Column(DataType.BIGINT)
  channelId: number;

  @BelongsTo(() => Channel)
  channel: Channel[];

  @ForeignKey(() => ProductCategory)
  @Column(DataType.BIGINT)
  productCategorieId: number;

  @BelongsTo(() => ProductCategory)
  productCategorie: ProductCategory;

  @Column(DataType.DATE)
  deletedAt: Date

  @Column(DataType.DATE)
  createdAt: Date

  @Column(DataType.DATE)
  updatedAt: Date

  @BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(channelProductCategorie: ChannelProductCategorie, options: any) { 
		channelProductCategorie.updatedAt = Sequelize.fn('NOW'); 
	}

}