import {
  Table,
  Column,
  PrimaryKey,
  Model,
  AutoIncrement,
  DataType,
  BeforeCreate,
  AllowNull,
  BeforeUpdate,
  ForeignKey,
  Scopes,
  BelongsTo
} from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
// import { DataTypeTime } from 'sequelize';
import { Channel } from './Channel.entity';
import { ProductGroup } from '../../product/models/ProductGroup.entity';
var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblChannelProductGroup' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
  tradeinfo: {
    attributes: {
      exclude: ['deletedAt', 'createdAt', 'updatedAt']
    }
  }
}))

export class ChannelProductGroup extends Model<ChannelProductGroup>{

  @PrimaryKey
  @AutoIncrement
  @AllowNull(false)
  @Column(DataType.BIGINT)
  channelProductGroupId: number

  @ForeignKey(() => Channel)
  @Column(DataType.BIGINT)
  channelId: number

  @BelongsTo(() => Channel)
  channel: Channel[];

  @ForeignKey(() => ProductGroup)
  @Column(DataType.BIGINT)
  productGroupId: number

  @BelongsTo(() => ProductGroup)
  productGroup: ProductGroup;

  @Column(DataType.DATE)
  deletedAt: Date

  @Column(DataType.DATE)
  createdAt: Date

  @Column(DataType.DATE)
  updatedAt: Date

  @BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(channelProductGroup: ChannelProductGroup, options: any) { 
		channelProductGroup.updatedAt = Sequelize.fn('NOW'); 
	}

}