import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, ForeignKey, BelongsTo } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Channel } from './Channel.entity';
import { Country } from '../../common/models/Country.entity';
import { Address } from './Address.entity';
import { User } from '../../user/models/User.entity';
import { ChannelType } from './ChannelType.entity';

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblchannelrecommended' } as IDefineOptions;

@Table(tableOptions)
export class ChannelRecommended extends Model<ChannelRecommended> {

	static CHANNEL_TAG_NEW = 'New';
	static CHANNEL_TAG_ACTIVE = 'Active';
	static CHANNEL_TAG_EXISTING = 'Existing';

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	channelrecommendid: number

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	existchannelid: number

	@BelongsTo(() => Channel, 'existchannelid')
	existchannel: Channel;

	@Column(DataType.BIGINT)
	oldchannelid: number

	@Column({
		type: new DataType.VIRTUAL(DataType.CHAR(255), ['existchannelid', 'oldchannelid']),
		get: function () {
			if (this.get('existchannelid')) {
				return ChannelRecommended.CHANNEL_TAG_ACTIVE
			} else if (this.get('oldchannelid')) {
				return ChannelRecommended.CHANNEL_TAG_EXISTING
			} else {
				return ChannelRecommended.CHANNEL_TAG_NEW
			}
		}
	})
	tag: string

	// @ForeignKey(() => Channel)
	// @Column(DataType.BIGINT)
	// vendorchannelid: number

	// @BelongsTo(() => Channel, 'vendorchannelid')
	// vendorchannel: Channel[];

	// @ForeignKey(() => Channel)
	// @Column(DataType.BIGINT)
	// retailerchannelid: number

	// @BelongsTo(() => Channel, 'retailerchannelid')
	// retailerchannel: Channel[];

	@Column(DataType.CHAR(255))
	companyname: string

	@Column(DataType.CHAR(255))
	contactname: string

	@ForeignKey(() => Country)
	@Column(DataType.BIGINT)
	countryid: number;

	@BelongsTo(() => Country, 'countryid')
	country: Country;

	@Column(DataType.CHAR(255))
	city: string

	@Column(DataType.CHAR(255))
	mail: string

	@Column(DataType.CHAR(255))
	mobileno: string

	@ForeignKey(() => Address)
	@Column(DataType.BIGINT)
	addressid: number

	@BelongsTo(() => Address, 'addressid')
	address: Address;

	@Column(DataType.CHAR(255))
	websiteurl: string

	@Column(DataType.BIGINT)
	updatedby: number

	@Column(DataType.DATE)
	createdat: Date

	@Column(DataType.DATE)
	updatedat: Date;

	@Column(DataType.DATE)
	deletedat: Date

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	suggestedto: number

	@BelongsTo(() => Channel, 'suggestedto')
	suggestedtochannel: Channel;

	@ForeignKey(() => ChannelType)
	@Column(DataType.BIGINT)
	suggestedto_type: number

	@ForeignKey(() => User)
	@Column(DataType.BIGINT)
	suggestedby: number

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	suggestedby_channelid: number

	@BelongsTo(() => Channel, 'suggestedby_channelid')
	suggestedbychannel: Channel;

	@ForeignKey(() => ChannelType)
	@Column(DataType.BIGINT)
	suggestedby_type: number

	@ForeignKey(() => ChannelType)
	@Column(DataType.BIGINT)
	suggested_type: number

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	active_id: number

	@BelongsTo(() => Channel, 'active_id')
	active_channel: Channel;

	@Column(DataType.DATE)
	last_remainder_sent_at: Date

	@Column(DataType.INTEGER)
	no_of_remainders_sent: number

	@Column
	partner_request: string

	@ForeignKey(() => ChannelRecommended)
	@Column(DataType.BIGINT)
	child_invitation: number

	@Column(DataType.CHAR(255))
	occasion: string

	@Column(DataType.CHAR(255))
	note: string
}
