import {
  Table,
  Column,
  PrimaryKey,
  Model,
  AutoIncrement,
  DataType,
  BeforeCreate,
  AllowNull,
  BeforeUpdate,
  ForeignKey,
  BelongsTo,
  Scopes
} from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
// import { DataTypeTime } from 'sequelize';
import { Channel } from './Channel.entity';
import { ServiceList } from './ServiceList.entity';
var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblChannelServiceList' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
	channelDetail: {
		attributes: ['channelServiceListId', 'serviceListId', 'deletedAt']
  },
  tradeinfo: {
    attributes: {
      exclude: ['deletedAt', 'createdAt', 'updatedAt']
    }
  }
}))

export class ChannelServiceList extends Model<ChannelServiceList>{

  @PrimaryKey
  @AutoIncrement
  @AllowNull(false)
  @Column(DataType.BIGINT)
  channelServiceListId: number

  @ForeignKey(() => Channel)
  @Column(DataType.BIGINT)
  channelId: number

  @BelongsTo(() => Channel)
  channel: Channel[];

  @ForeignKey(() => ServiceList)
  @Column(DataType.BIGINT)
  serviceListId: number

  @BelongsTo(() => ServiceList)
  serviceList: ServiceList;

  @Column(DataType.DATE)
  deletedAt: Date

  @Column(DataType.DATE)
  createdAt: Date

  @Column(DataType.DATE)
  updatedAt: Date

  @BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(channelServiceList: ChannelServiceList, options: any) { 
		channelServiceList.updatedAt = Sequelize.fn('NOW'); 
	}

}