import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, BelongsTo, ForeignKey, Scopes } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Channel } from './Channel.entity';
import { ShareType } from './ShareType.entity';
import { User } from './../../user/models/User.entity';

const tableOptions: IDefineOptions = { tableName: 'tblchannelsharepreference', timestamps:false } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
	settings: {
		attributes: ['sharetypeid']
	}
}))

export class ChannelSharePreference extends Model<ChannelSharePreference>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	channelsharepreferenceid: number

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	channelid: number

	@BelongsTo(() => Channel)
	channel: Channel[];

	@ForeignKey(() => ShareType)
	@Column(DataType.BIGINT)
	sharetypeid: number

	@BelongsTo(() => ShareType)
	sharetype: ShareType[];

	@ForeignKey(() => User)
	@Column(DataType.BIGINT)
	updatedby: number

	@BelongsTo(() => User, 'updatedby')
	User: User;

	@Column(DataType.DATE)
	deletedat: Date

	@Column(DataType.DATE)
	createdat: Date

	@Column(DataType.DATE)
	updatedat: Date

}
