import {
  Table,
  Column,
  PrimaryKey,
  Model,
  AutoIncrement,
  DataType,
  BeforeCreate,
  AllowNull,
  BeforeUpdate,
  ForeignKey,
  BelongsTo,
  Scopes
} from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Channel } from './Channel.entity';
import { ProductGroup } from '../../product/models/ProductGroup.entity';
var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblChannelSpeciality' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
  channelDetail: {
    attributes: ['channelSpecialityId', 'productGroupId', 'deletedAt']
  },
  tradeinfo: {
    attributes: {
      exclude: ['deletedAt', 'createdAt', 'updatedAt']
    }
  }
}))

export class ChannelSpeciality extends Model<ChannelSpeciality>{

  @PrimaryKey
  @AutoIncrement
  @AllowNull(false)
  @Column(DataType.BIGINT)
  channelSpecialityId: number

  @ForeignKey(() => Channel)
  @Column(DataType.BIGINT)
  channelId: number

  @BelongsTo(() => Channel)
  channel: Channel[];

  @ForeignKey(() => ProductGroup)
  @Column(DataType.BIGINT)
  productGroupId: number

  @BelongsTo(() => ProductGroup)
  productGroup: ProductGroup;

  @Column(DataType.DATE)
  deletedAt: Date

  @Column(DataType.DATE)
  createdAt: Date

  @Column(DataType.DATE)
  updatedAt: Date

  @BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(channelSpeciality: ChannelSpeciality, options: any) { 
		channelSpeciality.updatedAt = Sequelize.fn('NOW'); 
	}

}