import {
	Table,
	Column,
	PrimaryKey,
	Model,
	AutoIncrement,
	DataType,
	AllowNull,
	HasMany,
	ForeignKey,
	BelongsTo,
	Length
} from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Channel } from './Channel.entity';
import { ClaimDocument } from './ClaimDocument.entity';
import { SignUpStatus } from '../../user/models/SignUpStatus.entity';

const tableOptions: IDefineOptions = { tableName: 'tblClaim' } as IDefineOptions;

@Table(tableOptions)

export class Claim extends Model<Claim>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	claimId: number;

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	channelId: number;

	@BelongsTo(() => Channel, 'channelId')
	channel: Channel;

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	claimChannelId: number;

	@BelongsTo(() => Channel, 'claimChannelId')
	claimChannel: Channel;

	@HasMany(() => ClaimDocument, 'claimId')
	claimDocument: ClaimDocument[];

	@Length({ max: 8000 })
	@Column
	claimDesc: string;

	@ForeignKey(() => SignUpStatus)
	@Column(DataType.BIGINT)
	claimStatusId: number;

	@BelongsTo(() => SignUpStatus)
	claimStatus: SignUpStatus;

	@Length({ max: 1000 })
	@Column
	dismissReason: string;

}
