import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, BelongsTo, ForeignKey, Scopes, BeforeCreate, BeforeUpdate } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';

const Sequelize = require('sequelize');

import { ChannelType } from './ChannelType.entity';
import { SubscriptionPlan } from './SubscriptionPlan.entity';
import { SubscriptionType } from '../models/SubscriptionType.entity';
import { SubscriptionPeriod } from './SubscriptionPeriod.entity';

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblChannelSubscription' } as IDefineOptions;

@Table({
	tableName: "tblCoupon"
})

@Scopes(() => ({
	channelDetail: {
		attributes: ['couponDesc', 'couponCode', 'couponOfferValue', 'discountUnit']
	},
	couponCheck: {
		attributes: ['couponId', 'couponDesc', 'couponOfferValue', 'discountUnit']
	}
}))

export class Coupon extends Model<Coupon>{
	static DISCOUNT_UNIT_AMT = 1;
	static DISCOUNT_UNIT_PER = 2;

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	couponId: number

	@AllowNull(false)
	@Column(DataType.CHAR(255))
	couponDesc: string

	@AllowNull(false)
	@Column(DataType.CHAR(100))
	couponCode: string

	@AllowNull(false)
	@Column(DataType.INTEGER)
	couponOfferValue: number

	@Column
	isActive: number;

	@AllowNull(false)
	@Column(DataType.TIME)
	fromDate

	@AllowNull(false)
	@Column(DataType.TIME)
	toDate

	@AllowNull(false)
	@ForeignKey(() => SubscriptionType)
	@Column(DataType.BIGINT)
	subscriptionTypeId: number

	@BelongsTo(() => SubscriptionType)
	subscriptionType: SubscriptionType;

	@ForeignKey(() => ChannelType)
	@Column(DataType.BIGINT)
	channelTypeId: number

	@BelongsTo(() => ChannelType)
	channelType: ChannelType;

	@ForeignKey(() => SubscriptionPlan)
	@Column(DataType.BIGINT)
	subscriptionPlanId: number

	@BelongsTo(() => SubscriptionPlan)
	subscriptionPlan: SubscriptionPlan;

	@ForeignKey(() => SubscriptionPeriod)
	@Column(DataType.BIGINT)
	subscriptionPeriodId: number

	@BelongsTo(() => SubscriptionPeriod)
	subscriptionPeriod: SubscriptionPeriod;

	@AllowNull(false)
	@Column(DataType.INTEGER)
	discountUnit: number

	@AllowNull(false)
	@Column(DataType.INTEGER)
	userThreshold: number

	@AllowNull(false)
	@Column(DataType.INTEGER)
	usageThreshold: number

	@Column(DataType.DATE)
	deletedAt: Date;

	@Column(DataType.DATE)
	updatedAt: Date;

	public async getDiscountedAmount(amount) {
		return amount - await this.getOfferAmount(amount);
	}

	public async getOfferAmount(amount) {
		if (this.discountUnit == Coupon.DISCOUNT_UNIT_AMT) {
			return this.couponOfferValue;
		} else if (this.discountUnit == Coupon.DISCOUNT_UNIT_PER) {
			return amount * (this.couponOfferValue / 100);
		}
	}

	@BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(coupon: Coupon, options: any) { 
		coupon.updatedAt = Sequelize.fn('NOW'); 
	}

}
