import {
    Table,
    Column,
    PrimaryKey,
    Model,
    AutoIncrement,
    DataType,
    BeforeCreate,
    AllowNull,
    BeforeUpdate,
    ForeignKey,
    HasMany,
    BelongsTo,
    Scopes,
  } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Country } from 'src/modules/common/models/Country.entity';
import { Media } from 'src/modules/media/models';
import { EventChannel } from './EventChannel.entity';
import { EventPackage } from './EventPackage.entity';
import { EventParticipants } from './EventParticipants.entity';

//import { DataTypeTime } from 'sequelize';

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblevents' } as IDefineOptions;

@Table(tableOptions)

export class EventBo extends Model<EventBo>{

    @PrimaryKey
    @AutoIncrement
    @AllowNull(false)
    @Column(DataType.BIGINT)
    id: number

    @Column(DataType.CHAR(255))
    name: string

    @Column(DataType.CHAR(255))
    description: string

    @Column(DataType.DATE)
	createdAt: Date

	@Column(DataType.DATE)
	updatedAt: Date

    @Column(DataType.DATE)
	deletedAt: Date

    @Column(DataType.BIGINT)
    type: number

    @Column(DataType.DATE)
    fromDate: Date

    @Column(DataType.DATE)
    toDate: Date

    @Column(DataType.TIME)
    fromTime: Date

    @Column(DataType.TIME)
    toTime: Date

    @Column(DataType.CHAR(255))
    timezone: string

    @Column(DataType.CHAR(500))
    address: string

    @Column(DataType.CHAR(255))
    city: string

    @ForeignKey(() => Country)
	@Column(DataType.BIGINT)
	countryId: number;

	@BelongsTo(() => Country)
	country: Country;

    @Column(DataType.CHAR(10))
    zipcode: string

    
    @Column(DataType.BIGINT)
    eventLogoId: number

    @BelongsTo(() => Media, {as : 'logo', foreignKey: 'eventLogoId'})
    logo: Media

    // @ForeignKey(() => Media)
    @Column(DataType.BIGINT)
    eventBannerId: number

    @BelongsTo(() => Media, {as: 'banner', foreignKey: 'eventBannerId'})
    banner: Media

    @HasMany(() => EventPackage, 'eventId')
	packages: EventPackage[];

    @HasMany(() => EventParticipants, 'eventId')
	participants: EventParticipants[];

    @HasMany(() => EventChannel, 'eventId')
	channels: EventChannel[];

    @Column(DataType.CHAR(500))
    eventCatalystId: string
}