import {
    Table,
    Column,
    PrimaryKey,
    Model,
    AutoIncrement,
    DataType,
    BeforeCreate,
    AllowNull,
    BeforeUpdate,
    ForeignKey,
    HasMany,
    BelongsTo,
    Scopes,
  } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Contact } from 'src/modules/user/models/Contact.entity';
import { Channel } from './Channel.entity';
import { EventBo } from './EventBo.entity';
import { EventPackage } from './EventPackage.entity';
import { EventParticipants } from './EventParticipants.entity';
//import { DataTypeTime } from 'sequelize';

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblEventChannel' } as IDefineOptions;

@Table(tableOptions)

export class EventChannel extends Model<EventChannel>{

    @PrimaryKey
    @AutoIncrement
    @AllowNull(false)
    @Column(DataType.BIGINT)
    id: number

    @ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	channelId: number

    @BelongsTo(()=>Channel, {foreignKey: 'channelId'})
    channel: Channel[]

    @ForeignKey(() => EventBo)
    @Column(DataType.BIGINT)
	eventId: number

    @ForeignKey(() => EventPackage)
    @Column(DataType.BIGINT)
	packageId: number

    @BelongsTo(()=>EventPackage, {foreignKey: 'packageId'})
    package: EventPackage

    @BelongsTo(()=>EventBo, {foreignKey: 'eventId'})
    event: EventBo

    @Column
    isNew: string

    @Column(DataType.DATE)
	createdAt: Date

    @Column(DataType.DATE)
	deletedAt: Date

    @Column(DataType.DATE)
    updatedAt: Date

    @HasMany(() => EventParticipants, {foreignKey: 'eventId'})
	participants: EventParticipants[];

    @Column(DataType.INTEGER)
    additionalMembers: number

    @Column(DataType.BIGINT)
    paymenttransactionId: number

    @Column
    paymentstatus: string;

    @Column(DataType.INTEGER)
    paymentMethod: number

}