import {
    Table,
    Column,
    PrimaryKey,
    Model,
    AutoIncrement,
    DataType,
    BeforeCreate,
    AllowNull,
    BeforeUpdate,
    ForeignKey,
    HasMany,
    BelongsTo,
    Scopes,
  } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Contact } from 'src/modules/user/models/Contact.entity';
import { Channel } from './Channel.entity';
import { EventBo } from './EventBo.entity';
import { EventPackage } from './EventPackage.entity';
//import { DataTypeTime } from 'sequelize';

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblEventParticipants' } as IDefineOptions;

@Table(tableOptions)

export class EventParticipants extends Model<EventParticipants>{

    @PrimaryKey
    @AutoIncrement
    @AllowNull(false)
    @Column(DataType.BIGINT)
    eventParticipantId: number

    @ForeignKey(() => Contact)
	@Column(DataType.BIGINT)
	contactId: number

    @BelongsTo(()=>Contact, {foreignKey: 'contactId'})
    contacts: Contact[]

    @ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	channelId: number

    @BelongsTo(()=>Channel, {foreignKey: 'channelId'})
    channels: Channel[]

    @ForeignKey(() => EventBo)
    @Column(DataType.BIGINT)
	eventId: number

    @ForeignKey(() => EventPackage)
    @Column(DataType.BIGINT)
	packageId: number

    @BelongsTo(()=>EventPackage, {foreignKey: 'packageId'})
    package: EventPackage[]

    @Column(DataType.DATE)
	createdAt: Date

    @Column(DataType.DATE)
	deletedAt: Date

    @Column(DataType.NUMBER)
    participantType: number

    @Column(DataType.NUMBER)
    channelType: number

    @Column(DataType.BIGINT)
    users: number

    @Column
    status: string

    @HasMany(() => Channel, {foreignKey: 'channelId'})
	participants: Channel[];

}