import { Table, Column, Model, DataType, BelongsTo, ForeignKey, AllowNull, Scopes } from 'sequelize-typescript';
import { PaymentType } from './PaymentType.entity';
import { Sequelize } from 'sequelize-typescript';
import { PaymentTransaction } from './PaymentTransaction.entity';
import { Channel } from './Channel.entity';

@Table({
	timestamps: false,
	tableName: "tblFailureTransaction"
})


export class FailureTransaction extends Model<FailureTransaction> {
	static PAYMENT_TYPE_SUBSCRIPTION = 1;
	static PAYMENT_TYPE_TRADESHOW = 2;
	static PAYMENT_TYPE_SAMPLES = 3;
	static PAYMENT_TYPE_SALESREPPAY = 4;

	@Column({
		type: DataType.INTEGER,
		allowNull: false,
		// defaultValue:UUIDV4,
		autoIncrement: true,
		unique: true,
		primaryKey: true,
	})
	failureTransactionId: number

	@Column({
		type: DataType.INTEGER,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false
	})
	paymentMethodId: number;

	@ForeignKey(() => PaymentTransaction)
	@Column(DataType.BIGINT)
	paymentTransactionId: number;

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	channelId: number;


	@AllowNull(true)
	@Column(DataType.CHAR(255))
	response: string

	@Column({
		type: DataType.DATE,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		defaultValue: Sequelize.literal('CURRENT_TIMESTAMP')
	})
	createdAt: string

	
	@BelongsTo(() => PaymentTransaction)
    paymentTransaction: PaymentTransaction[];
 
	@BelongsTo(() => Channel)
    channel: Channel[];
	
	
}
