import {
  Table,
  Column,
  PrimaryKey,
  Model,
  AutoIncrement,
  DataType,
  AllowNull,
  BelongsTo,
  HasMany,
  HasOne,
  ForeignKey,
  Scopes,
  BelongsToMany
} from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Address } from './Address.entity';
import { OldAddress } from './OldAddress.entity';
import { Channel } from './Channel.entity';
import { OldContact } from './OldContact.entity';
import { OldChannelDetail } from './OldChannelDetail.entity';
import { Country } from '../../common/models/Country.entity';
import { OldChannelCustomerProfile } from './OldChannelCustomerProfile.entity';
import { ProfileType } from './ProfileType.entity';
import { OldChannelSpeciality } from './OldChannelSpeciality.entity';
import { ProductGroup } from 'src/modules/product/models/ProductGroup.entity';

const tableOptions: IDefineOptions = { tableName: 'tblOldChannel' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
	//Used
	basicInform: {
		attributes: ['companyName','oldChannelId']
	}

}))

export class OldChannel extends Model<OldChannel>{

  @PrimaryKey
  @AutoIncrement
  @AllowNull(false)
  @Column(DataType.BIGINT)
  oldChannelId: number

  @Column(DataType.CHAR(255))
  companyName: string

  @AllowNull(false)
  @Column(DataType.CHAR(255))
  companyMail: string

  @AllowNull(false)
  @ForeignKey(() => Country)
  @Column(DataType.BIGINT)
  countryId: string

  @BelongsTo(() => Country)
	country: Country;

  @Column(DataType.BIGINT)
  channelTypeId: number

  @Column(DataType.BIGINT)
  regAddressId: number

  @Column(DataType.BIGINT)
  billAddressId: number

  @BelongsTo(() => OldAddress, {
    as: 'regAddress',
    foreignKey: 'regAddressId',
    targetKey: 'oldAddressId'
  })
  regAddress: OldAddress[];

  @BelongsTo(() => OldAddress, {
    as: 'billAddress',
    foreignKey: 'billAddressId',
    targetKey: 'oldAddressId'
  })
  billAddress: OldAddress[];

  @HasMany(() => Channel, 'oldChannelId')
  channel: Channel;
  
  @HasMany(() => OldContact, 'oldChannelId')
  oldContacts: OldContact[];
  
  @HasOne(() => OldChannelDetail, 'oldChannelId')
  oldChannelDetail: OldChannelDetail;
  
  @Column
	isActive: string;

	@Column(DataType.DATE)
	deletedAt: Date

  @Column(DataType.INTEGER)
  profileTypeId: number

  @HasMany(() => OldChannelCustomerProfile, 'oldChannelId')
	channelCustomerProfile: OldChannelCustomerProfile[];

  @BelongsToMany(() => ProfileType, {
		through: {
			model: () => OldChannelCustomerProfile,
			scope: {
				deletedAt: null
			}
		}
	})
	customerProfile: ProfileType[];

  @HasMany(() => OldChannelSpeciality, 'oldChannelId')
	channelSpeciality: OldChannelSpeciality[];

  @BelongsToMany(() => ProductGroup, {
		through: {
			model: () => OldChannelSpeciality,
			scope: {
				deletedAt: null
			}
		}
	})
	speciality: ProductGroup[];

}