import {
  Table,
  Column,
  PrimaryKey,
  Model,
  AutoIncrement,
  DataType,
  BeforeCreate,
  AllowNull,
  BeforeUpdate,
  ForeignKey,
  BelongsTo,
  Scopes
} from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { OldChannel } from './OldChannel.entity';
import { ProfileType } from './ProfileType.entity';
import { ProfileTypeClassification } from './ProfileTypeClassification.entity';
var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblOldChannelCustomerProfile' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
  channelDetail: {
    attributes: ['oldCustomerProfileId' as 'customerProfileId', 'profileTypeId', 'profileTypeClassificationId', 'deletedAt']
  },
  tradeinfo: {
    attributes: {
      exclude: ['deletedAt', 'createdAt', 'updatedAt']
    }
  }
}))

export class OldChannelCustomerProfile extends Model<OldChannelCustomerProfile>{

  @PrimaryKey
  @AutoIncrement
  @AllowNull(false)
  @Column(DataType.BIGINT)
  oldCustomerProfileId: number

  @ForeignKey(() => OldChannel)
  @Column(DataType.BIGINT)
  oldChannelId: number

  @BelongsTo(() => OldChannel)
  channel: OldChannel[];

  @ForeignKey(() => ProfileType)
  @Column(DataType.BIGINT)
  profileTypeId: number

  @BelongsTo(() => ProfileType)
  profileType: ProfileType;

  @ForeignKey(() => ProfileTypeClassification)
  @Column(DataType.BIGINT)
  profileTypeClassificationId: number

  @BelongsTo(() => ProfileTypeClassification)
  pTClas: ProfileTypeClassification;

  @Column(DataType.DATE)
  deletedAt: Date

  @Column(DataType.DATE)
  createdAt: Date

  @Column(DataType.DATE)
  updatedAt: Date

  @BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(oldChannelCustomerProfile: OldChannelCustomerProfile, options: any) { 
		oldChannelCustomerProfile.updatedAt = Sequelize.fn('NOW'); 
	}

}