import {
	Table,
	Column,
	PrimaryKey,
	Model,
	AutoIncrement,
	DataType,
	AllowNull,
	ForeignKey,
	BelongsTo,
} from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Media } from '../../media/models/Media.entity';
import { Currency } from './Currency.entity';
import { OldChannel } from './OldChannel.entity';

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblOldChannelDetail' } as IDefineOptions;

@Table(tableOptions)

export class OldChannelDetail extends Model<OldChannelDetail>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	OldChannelDetailId: number

	@ForeignKey(() => OldChannel)
	@Column(DataType.BIGINT)
	oldChannelId: number;

	@BelongsTo(() => OldChannel)
	oldChannel: OldChannel;

	@ForeignKey(() => Media)
	@Column(DataType.BIGINT)
	channelLogoId: number

	@BelongsTo(() => Media, 'channelLogoId')
	channelLogo: Media;

	@ForeignKey(() => Media)
	@Column(DataType.BIGINT)
	channelIconId: number

	@BelongsTo(() => Media, 'channelIconId')
	channelIcon: Media;

	@Column(DataType.CHAR(255))
	phone1: string

	@Column(DataType.CHAR(255))
	phone2: string

	@Column(DataType.CHAR(255))
	mobileNo: string

	@Column(DataType.CHAR(255))
	webSiteUrl: string

	@Column(DataType.INTEGER)
	estYear: number

	@Column(DataType.INTEGER)
	regYear: number

	@Column(DataType.CHAR(80000))
	detailDesc: string

	@Column(DataType.CHAR(100))
	empCntRange: string

	@Column(DataType.INTEGER)
	empCnt: number

	@Column(DataType.INTEGER)
	empCntDSale: number

	@Column(DataType.INTEGER)
	empCntISale: number

	@Column(DataType.INTEGER)
	empCntDMar: number

	@Column(DataType.INTEGER)
	empCntIMar: number

	@Column(DataType.CHAR(100))
	turnoverRange: string

	@Column(DataType.INTEGER)
	turnover: number

	@Column(DataType.INTEGER)
	revD: number

	@Column(DataType.INTEGER)
	revI: number;

	@Column(DataType.INTEGER)
	turnoverPhyStore: number;

	@Column(DataType.INTEGER)
	turnoverOnStore: number;

	@Column(DataType.DATE)
	deletedAt: Date

	@AllowNull(false)
	@Column(DataType.DATE)
	createdAt: Date

	@Column(DataType.DATE)
	updatedAt: Date;

	@Column(DataType.CHAR(255))
	stripeConnectId: string;

	@Column(DataType.CHAR(255))
	paymentoptionothers: string

	@ForeignKey(() => Currency)
	@Column(DataType.BIGINT)
	currencyid: number

	@BelongsTo(() => Currency)
	currency: Currency[];

	@Column(DataType.CHAR(255))
	ownerFirstName: string

	@Column(DataType.CHAR(255))
	ownerLastName: string

	@Column
	startup: string;

	@Column({
		type: DataType.ARRAY({
			type: DataType.ENUM('PROTOTYPE', 'S_BACKER', 'S_VOLUME')
		})
	})
	maturitylevel: string[]

	@Column(DataType.INTEGER)
	storeCount: number;

	@Column(DataType.INTEGER)
	no_of_skus: number;

	@Column(DataType.BIGINT)
	introvideoId: number

	@Column(DataType.BIGINT)
	channelBannerId: number

}
