import {
  Table,
  Column,
  PrimaryKey,
  Model,
  AutoIncrement,
  DataType,
  BeforeCreate,
  AllowNull,
  BeforeUpdate,
  ForeignKey,
  BelongsTo,
  Scopes
} from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { OldChannel } from './OldChannel.entity';
import { ProductGroup } from '../../product/models/ProductGroup.entity';
var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblOldChannelSpeciality' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
  channelDetail: {
    attributes: ['oldChannelSpecialityId', 'productGroupId', 'deletedAt']
  },
  tradeinfo: {
    attributes: {
      exclude: ['deletedAt', 'createdAt', 'updatedAt']
    }
  }
}))

export class OldChannelSpeciality extends Model<OldChannelSpeciality>{

  @PrimaryKey
  @AutoIncrement
  @AllowNull(false)
  @Column(DataType.BIGINT)
  oldChannelSpecialityId: number

  @ForeignKey(() => OldChannel)
  @Column(DataType.BIGINT)
  oldChannelId: number

  @BelongsTo(() => OldChannel)
  channel: OldChannel[];

  @ForeignKey(() => ProductGroup)
  @Column(DataType.BIGINT)
  productGroupId: number

  @BelongsTo(() => ProductGroup)
  productGroup: ProductGroup;

  @Column(DataType.DATE)
  deletedAt: Date

  @Column(DataType.DATE)
  createdAt: Date

  @Column(DataType.DATE)
  updatedAt: Date

  @BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(channelSpeciality: OldChannelSpeciality, options: any) { 
		channelSpeciality.updatedAt = Sequelize.fn('NOW'); 
	}

}