import {
	Table,
	Column,
	PrimaryKey,
	Model,
	AutoIncrement,
	DataType,
	AllowNull,
	HasMany,
	ForeignKey,
	BelongsTo,
} from 'sequelize-typescript';

import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Country } from '../../common/models/Country.entity';
import { Media } from '../../media/models/Media.entity';
import { Responsibility } from '../../user/models/Responsibility.entity';
import { OldChannel } from './OldChannel.entity';

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblOldContact' } as IDefineOptions;

@Table(tableOptions)

export class OldContact extends Model<OldContact>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	oldContactId: number;

	@ForeignKey(() => OldChannel)
	@Column(DataType.BIGINT)
	oldChannelId: number;

	@BelongsTo(() => OldChannel)
	oldChannel: OldChannel	;

	@Column
	firstName: string;

	@Column
	lastName: string;

	@Column
	phone1: string;

	@Column
	phone2: string;

	@Column
	mobileNo: string;

	@Column({
		type: DataType.CHAR(255),
		allowNull: false,
		unique: { name: 'email', msg: 'account:singup:emailAleradytaken' }
	})
	email: string;

	@ForeignKey(() => Responsibility)
	@Column(DataType.BIGINT)
	responsibilityId: number;

	@BelongsTo(() => Responsibility)
	responsibility: Media;

	@Column(DataType.BIGINT)
	workNatureId: number;

	@Column
	jobTitle: string;

	@Column
	linkedIn: string;

	@Column
	googleplus: string;

	@ForeignKey(() => Media)
	@Column(DataType.BIGINT)
	documentId: number;

	@BelongsTo(() => Media, 'documentId')
	document: Media;

	@ForeignKey(() => Country)
	@Column(DataType.BIGINT)
	countryId: number;

	@BelongsTo(() => Country)
	country: Country;

	@Column
	isDefaultContact: string;

	@Column
	isShared: string;

	@Column
	isActive: string;

	@Column(DataType.DATE)
	deletedAt: Date

	@AllowNull(false)
	@Column(DataType.DATE)
	createdAt: Date

	@Column(DataType.DATE)
	updatedAt: Date

}
