import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, BeforeCreate, BeforeUpdate, Scopes, HasMany } from 'sequelize-typescript';
import { ApiProperty } from '@nestjs/swagger';
import { ProfileList } from './ProfileList.entity';
var Sequelize = require('sequelize');

@Table({
	tableName: "tblProfileCreationRatings"
})

@Scopes(() => ({
	setting: {
		attributes: ['ratingId', 'order','icon','heading','desc','value']
	}
}))

export class ProfileRatings extends Model<ProfileRatings>{

	@PrimaryKey
	@AutoIncrement
	@Column
	@ApiProperty()
    ratingId: number
    
	@Column({type: DataType.BIGINT})
	@ApiProperty()
    order: number;

	@Column({type: DataType.CHAR(255)})
	@ApiProperty()
    icon: string;
    
    @Column({type: DataType.CHAR(255)})
	@ApiProperty()
    heading: string;
    
    @Column({type:DataType.CHAR(500)})
	@ApiProperty()
	desc: string;

	@Column({type: DataType.BIGINT})
	@ApiProperty()
    value: number;
    
    @Column({type: DataType.BIGINT})
	@ApiProperty()
	channelType: number;    

	@Column({
		type: DataType.DATE,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		defaultValue: Sequelize.literal('CURRENT_TIMESTAMP')
	})
	createdAt: string

	@Column(DataType.DATE)
	updatedAt: Date

	@Column(DataType.DATE)
	deletedAt: Date
	
	@BeforeCreate
	@BeforeUpdate
	public static async dateUpdate(rating: ProfileRatings, options: any) {
		rating.updatedAt = Sequelize.fn('NOW');
	}   
	
	@HasMany(() => ProfileList, 'ratingId')
	ProfileList: ProfileList[];    

}
