import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, BeforeCreate, BeforeUpdate, Scopes, ForeignKey, BelongsTo } from 'sequelize-typescript';
import { ApiProperty } from '@nestjs/swagger';
import { Channel } from './Channel.entity';
import { ProfileRatings } from './ProfileCreationRating.entity';
var Sequelize = require('sequelize');

@Table({
	tableName: "tblProfileList"
})

@Scopes(() => ({
	basicList: {
		attributes: ['ratingId', 'channelId', 'isCompleted','value']
	}
}))

export class ProfileList extends Model<ProfileList>{

	@PrimaryKey
	@AutoIncrement
	@Column
	@ApiProperty()
    listId: number
    
    @ForeignKey(() => Channel) 
    @Column(DataType.BIGINT)
    channelId:number
  
    @BelongsTo(() => Channel)
    channel: Channel[];

    @ForeignKey(() => ProfileRatings) 
    @Column(DataType.BIGINT)
    ratingId:number
  
    @BelongsTo(() => ProfileRatings)
    profilelist: ProfileRatings[];

    @Column({type: DataType.BIGINT})
	@ApiProperty()
    value: number;

	@Column({type: DataType.BOOLEAN})
	@ApiProperty()
    isCompleted: boolean;        

	@Column({
		type: DataType.DATE,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		defaultValue: Sequelize.literal('CURRENT_TIMESTAMP')
	})
	createdAt: string

	@Column(DataType.DATE)
    updatedAt: Date
    
    @Column(DataType.DATE)
	deletedAt: Date
	
	@BeforeCreate
	@BeforeUpdate
	public static async dateUpdate(rating: ProfileList, options: any) {
		rating.updatedAt = Sequelize.fn('NOW');
    }   
    

}
