import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, Scopes, HasMany, BeforeCreate, BeforeUpdate } from 'sequelize-typescript';
import { ProfileTypeClassification } from './ProfileTypeClassification.entity';
import { ApiProperty } from '@nestjs/swagger';
var Sequelize = require('sequelize');
import { ValidationException } from './../../../middleware/exception/Validation.exception';

@Table({
	tableName: "tblProfileType"
})

@Scopes(() => ({
	profileTypeInfom: {
		attributes: [['profileTypeId', 'id'], ['profileType', 'name']]
	},
	channelDetail: {
		attributes: ['profileTypeId', 'profileType', 'deletedAt']
	},
	tradeinfo: {
		attributes: {
			exclude: ['deletedAt', 'createdAt', 'updatedAt']
		}
	},
	profileTypeMaster: {
		where: {
			deletedAt: null
		},
		attributes: {
			exclude: ['deletedAt', 'createdAt', 'updatedAt']
		}
	}

}))

export class ProfileType extends Model<ProfileType>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	@ApiProperty()
	profileTypeId: number;

	@Column({
		type: DataType.CHAR(255),
		allowNull: false,
		unique: { name: 'profileType', msg: 'validate:field:shouldBeUnique' }
	})
	@ApiProperty()
	profileType: string;

	@HasMany(() => ProfileTypeClassification, 'profileTypeId')
	profileTypeClass: ProfileTypeClassification[];

	@Column(DataType.DATE)
	deletedAt: Date;

	@Column(DataType.DATE)
	createdAt: Date;

	@Column(DataType.DATE)
	updatedAt: Date;

	@Column(DataType.INTEGER)
	businessType: number

	@BeforeCreate
	@BeforeUpdate
	public static async dateUpdate(instance: ProfileType, options: any) {
		instance.updatedAt = Sequelize.fn('NOW');

		let woptions = {
			where: {
				profileType: {
					$ilike: instance.profileType.trim()
				},
				deletedAt: null
			}
		};

		if (instance.profileTypeId) {
			woptions['where']['profileTypeId'] = {
				$not: instance.profileTypeId
			}
		}

		if (await ProfileType.count(woptions)) {
			throw new ValidationException([{
				value: instance.profileType.trim(),
				property: 'profileType',
				children: [],
				constraints:
					{ unique: 'validate:field:shouldBeUnique' }
			}]);
		}
	}

}
