import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, BelongsTo, ForeignKey, Scopes, BeforeCreate, BeforeUpdate } from 'sequelize-typescript';
import { ProfileType } from './ProfileType.entity';
import { ChannelType } from './ChannelType.entity';
import { ApiProperty } from '@nestjs/swagger';
var Sequelize = require('sequelize');
import { ValidationException } from './../../../middleware/exception/Validation.exception';

@Table({
	tableName: "tblProfileTypeClassification"
})

@Scopes(() => ({
	profileTypeClassificationInfom: {
		attributes: ['profileTypeClassificationId', 'profileTypeId', 'channelTypeId', 'isDefault']
	},
	profileTypeClassificationMaster: {
		where: {
			deletedAt: null
		},
		attributes: {
			exclude: ['deletedAt', 'createdAt', 'updatedAt']
		},
		include: [
			{
				model:ProfileType,
				attributes: ['profileTypeId', 'profileType'],
			},
			{
				model:ChannelType,
			}
		]
	}
}))

export class ProfileTypeClassification extends Model<ProfileTypeClassification>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	@ApiProperty()
	profileTypeClassificationId: number;

	@ForeignKey(() => ProfileType)
	@Column({
		type: DataType.BIGINT,
		allowNull: false,
		unique: { name: 'profileType', msg: 'validate:field:shouldBeUnique' }
	})
	@ApiProperty()
	profileTypeId: string;

	@BelongsTo(() => ProfileType)
	profileType: ProfileType;

	@ForeignKey(() => ChannelType)
	@Column({
		type: DataType.BIGINT,
		allowNull: false,
		unique: { name: 'profileType', msg: 'validate:field:shouldBeUnique' }
	})
	@ApiProperty()
	channelTypeId: number;

	@BelongsTo(() => ChannelType)
	channelType: ChannelType;

	@Column
	@ApiProperty()
	isDefault: string;

	@Column(DataType.DATE)
	deletedAt: Date;

	@Column(DataType.DATE)
	createdAt: Date;

	@Column(DataType.DATE)
	updatedAt: Date;

	@BeforeCreate
	@BeforeUpdate
	public static async dateUpdate(instance: ProfileTypeClassification, options: any) {
		instance.updatedAt = Sequelize.fn('NOW');

		let woptions = {
			where: {
				channelTypeId: +instance.channelTypeId,
				profileTypeId: +instance.profileTypeId,
				deletedAt: null
			}
		};

		if (instance.profileTypeClassificationId) {
			woptions['where']['profileTypeClassificationId'] = {
				$not: instance.profileTypeClassificationId
			}
		}

		if (await ProfileTypeClassification.count(woptions)) {
			throw new ValidationException([{
				value: +instance.profileTypeId,
				property: 'profileTypeId',
				children: [],
				constraints:
					{ unique: 'validate:field:shouldBeUnique' }
			},
			{
				value: +instance.channelTypeId,
				property: 'channelTypeId',
				children: [],
				constraints:
					{ unique: 'validate:field:shouldBeUnique' }
			}]);
		}
	}

}
