import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, BelongsTo, ForeignKey, BeforeCreate, BeforeUpdate } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Subscription } from './Subscription.entity';
import { SubscriptionFeatures } from './SubscriptionFeatures.entity';
var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblSubscriptionDetail' } as IDefineOptions;

@Table({
	tableName: "tblSubscriptionDetail"
})

export class SubscriptionDetail extends Model<SubscriptionDetail>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	subscriptionDetailId: number

	@ForeignKey(() => Subscription)
	@Column(DataType.BIGINT)
	subscriptionId: number

	@BelongsTo(() => Subscription)
	subscription: Subscription[];

	@ForeignKey(() => SubscriptionFeatures)
	@Column(DataType.BIGINT)
	subscriptionFeatureId: number

	@BelongsTo(() => SubscriptionFeatures)
	subscriptionFeatures: SubscriptionFeatures[];

	@Column(DataType.CHAR(255))
	keyDesc: string

	@Column(DataType.CHAR(255))
	keyValue: string

	@Column(DataType.BOOLEAN)
	isActive: boolean

	@Column(DataType.DATE)
	deletedAt: Date

	@Column(DataType.DATE)
	updatedAt: Date;

	@Column(DataType.BIGINT)
	updatedBy: number

	@BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(subscriptionDetail: SubscriptionDetail, options: any) { 
		subscriptionDetail.updatedAt = Sequelize.fn('NOW'); 
	}

}
