import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, BelongsTo, ForeignKey, AllowNull, Scopes, BeforeCreate, BeforeUpdate } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Subscription } from './Subscription.entity';
import { SubscriptionExtraFeatureCategory } from './SubscriptionExtraFeatureCategories.entity';
var Sequelize = require('sequelize');
import { ValidationException } from './../../../middleware/exception/Validation.exception';

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblSubscriptionExtraFeature' } as IDefineOptions;

@Table({
	tableName: "tblSubscriptionExtraFeature"
})

export class SubscriptionExtraFeature extends Model<SubscriptionExtraFeature>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column({
		type: DataType.BIGINT,
		allowNull: false,
		// defaultValue:UUIDV4,//autoIncrement: true,
		unique: true,
		primaryKey: true,
	})
	subscriptionExtraFeatureId: number

    @ForeignKey(() => Subscription)
	@Column(DataType.BIGINT)
    subscriptionId: number

    @ForeignKey(() => SubscriptionExtraFeatureCategory)
	@Column(DataType.BIGINT)
	subscriptionExtraFeatureCategoryId: number

	@BelongsTo(() => SubscriptionExtraFeatureCategory)
	subscriptionExtraFeatureCategory: SubscriptionExtraFeatureCategory;
	
	@Column(DataType.CHAR(255))
    feature_value: string

	@Column(DataType.CHAR(255))
	feature_label: string;
    
    @Column
	isActive: string

	@Column(DataType.DATE)
	deletedAt: Date;

	@Column(DataType.DATE)
	updatedAt: Date;

	@Column(DataType.DATE)
	createdAt: Date

	@BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(SubscriptionExtraFeature: SubscriptionExtraFeature, options: any) { 
		SubscriptionExtraFeature.updatedAt = Sequelize.fn('NOW'); 
	}

}
