import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, BeforeCreate, BeforeUpdate } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { ValidationException } from './../../../middleware/exception/Validation.exception';
var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblSubscriptionFeatureCategories' } as IDefineOptions;

@Table({
	tableName: "tblSubscriptionFeatureCategories"
})

export class SubscriptionFeatureCategory extends Model<SubscriptionFeatureCategory>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	subscriptionFeatureCategoryId: number

	@AllowNull(false)
	@Column(DataType.CHAR(255))
	subscriptionFeatureCategory: string

	@BeforeCreate
	@BeforeUpdate
	public static async dateUpdate(instance: SubscriptionFeatureCategory) {
		let options = {
			where: {
				subscriptionFeatureCategory: {
					$ilike: instance.subscriptionFeatureCategory.trim()
				}
			}
		};

		if (instance.subscriptionFeatureCategoryId) {
			options['where']['subscriptionFeatureCategoryId'] = {
				$not: instance.subscriptionFeatureCategoryId
			}
		}

		if (await SubscriptionFeatureCategory.count(options)) {
			throw new ValidationException([{
				value: instance.subscriptionFeatureCategory.trim(),
				property: 'subscriptionFeatureCategory',
				children: [],
				constraints:
					{ unique: 'validate:field:shouldBeUnique' }
			}]);
		}
	}
}
