import { Table, Column, PrimaryKey, BelongsToMany, Model, AutoIncrement, DataType, AllowNull, BeforeCreate, BeforeUpdate, ForeignKey, BelongsTo } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { SubscriptionFeatureCategory } from './SubscriptionFeatureCategory.entity';
import { SubscriptionDetail } from './SubscriptionDetail.entity';
import { Subscription } from './/Subscription.entity';
var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblSubscriptionFeatures' } as IDefineOptions;

@Table({
	tableName: "tblSubscriptionFeatures"
})

export class SubscriptionFeatures extends Model<SubscriptionFeatures>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	subscriptionFeatureId: number

	@AllowNull(false)
	@Column({
		type: DataType.CHAR(255),
		allowNull: false,
		unique: { name: 'subscriptionFeature', msg: 'validate:field:shouldBeUnique' }
	})
	subscriptionFeature: string

	@ForeignKey(() => SubscriptionFeatureCategory)
	@Column({
		type: DataType.BIGINT,
		allowNull: false,
		unique: { name: 'subscriptionFeature', msg: 'validate:field:shouldBeUnique' }
	})
	subscriptionFeatureCategoryId: number

	@BelongsTo(() => SubscriptionFeatureCategory)
	subscriptionFeatureCategory: SubscriptionFeatureCategory;

	@Column(DataType.BOOLEAN)
	isActive: boolean

	@Column(DataType.DATE)
	deletedAt: Date

	@Column(DataType.DATE)
	updatedAt: Date;

	@Column(DataType.BIGINT)
	updatedBy: number

	@BelongsToMany(() => Subscription, {
		through: {
			model: () => SubscriptionDetail,
			scope: {
				deletedAt: null
			}
		}
	})
	subscription: Subscription[];

	@BeforeCreate
	@BeforeUpdate
	public static async dateUpdate(subscriptionFeatures: SubscriptionFeatures, options: any) {
		subscriptionFeatures.updatedAt = Sequelize.fn('NOW');
	}

}
