import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, BeforeCreate, BeforeUpdate, Scopes } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
var Sequelize = require('sequelize');
import { ValidationException } from './../../../middleware/exception/Validation.exception';

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblSubscriptionPeriod' } as IDefineOptions;

@Table({
	timestamps: true,
	tableName: "tblSubscriptionPeriod"
})

@Scopes(() => ({
	subscriptionPeriodMaster: {
		attributes: ['subscriptionPeriodId', 'periodDesc', 'periodValueInDays', 'isActive', 'updatedAt']
	}
}))

export class SubscriptionPeriod extends Model<SubscriptionPeriod>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	subscriptionPeriodId: number

	@AllowNull(false)
	@Column(DataType.CHAR(255))
	periodDesc: string

	@Column(DataType.INTEGER)
	periodValueInDays: number

	@Column
	isActive: string;

	@Column(DataType.INTEGER)
	order: number;

	@Column(DataType.DATE)
	deletedAt: Date

	@Column(DataType.DATE)
	updatedAt: Date;

	@Column(DataType.BIGINT)
	updatedBy: number

	@BeforeCreate
	@BeforeUpdate
	public static async dateUpdate(instance: SubscriptionPeriod) {
		instance.updatedAt = Sequelize.fn('NOW');

		let options = {
			where: {
				periodDesc: {
					$ilike: instance.periodDesc.trim()
				},
				deletedAt: null
			}
		};

		if (instance.subscriptionPeriodId) {
			options['where']['subscriptionPeriodId'] = {
				$not: instance.subscriptionPeriodId
			}
		}

		if (await SubscriptionPeriod.count(options)) {
			throw new ValidationException([{
				value: instance.periodDesc.trim(),
				property: 'periodDesc',
				children: [],
				constraints:
					{ unique: 'validate:field:shouldBeUnique' }
			}]);
		}
	}

}
