import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, Scopes, BeforeCreate, BeforeUpdate } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { ApiProperty } from '@nestjs/swagger';
var Sequelize = require('sequelize');
import { ValidationException } from './../../../middleware/exception/Validation.exception';
import { UUIDV4 } from 'sequelize/types';

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblSubscriptionPlan' } as IDefineOptions;

@Table({
	tableName: "tblSubscriptionPlan"
})

@Scopes(() => ({
	channelDetail: {
		attributes: ['subscriptionPlanType', 'isActive', 'deletedAt']
	},
	orderInfo: {
		attributes: ['subscriptionPlanType']
	},
	subscriptionPlanMaster: {
		attributes: ['subscriptionPlanId', 'subscriptionPlanType', 'rankOrder', 'isActive']
	}
}))

export class SubscriptionPlan extends Model<SubscriptionPlan>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column({
		type: DataType.BIGINT,
		allowNull: false,
		// defaultValue:UUIDV4,//autoIncrement: true,
		unique: true,
		primaryKey: true,
	})
	@ApiProperty()
	subscriptionPlanId: number

	@Column({
		type: DataType.CHAR(255),
		allowNull: false,
		unique: { name: 'subscriptionPlanType', msg: 'validate:field:shouldBeUnique' }
	})
	@ApiProperty()
	subscriptionPlanType: string

	@Column(DataType.INTEGER)
	rankOrder: number

	@Column
	@ApiProperty()
	isActive: string

	@Column(DataType.DATE)
	deletedAt: Date;

	@Column(DataType.DATE)
	updatedAt: Date;

	@Column(DataType.BIGINT)
	updatedBy: number

	@BeforeCreate
	@BeforeUpdate
	public static async dateUpdate(instance: SubscriptionPlan) {
		instance.updatedAt = Sequelize.fn('NOW');

		let options = {
			where: {
				subscriptionPlanType: {
					$ilike: instance.subscriptionPlanType.trim()
				},
				deletedAt: null
			}
		};

		if (instance.subscriptionPlanId) {
			options['where']['subscriptionPlanId'] = {
				$not: instance.subscriptionPlanId
			}
		}

		if (await SubscriptionPlan.count(options)) {
			throw new ValidationException([{
				value: instance.subscriptionPlanType.trim(),
				property: 'subscriptionPlanType',
				children: [],
				constraints:
					{ unique: 'validate:field:shouldBeUnique' }
			}]);
		}
	}

}
