import { Table, Column, Model, DataType, BelongsTo, ForeignKey, AllowNull, Scopes, AutoIncrement, PrimaryKey } from 'sequelize-typescript';
import { ApiProperty } from '@nestjs/swagger';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Channel } from './Channel.entity';
import { User } from '../../user/models/User.entity';
import { Sequelize } from 'sequelize-typescript';
import { Currency } from './Currency.entity';

const channelValidation = {
	isAvailable: async (value: string, next: Function): Promise<any> => {
		const isExist = await Channel.findOne({ where: { channelId: value } });
		if (!isExist) {
			next('channel:notFound');
		}
		next();
	},
};
const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblUserWalletAmount' } as IDefineOptions;
@Table({
	tableName: "tblUserWalletAmount"
})

@Scopes(() => ({
	channelDetail: {
		attributes: ['userId', 'channelId', 'amount', 'updatedAt']
	},
}))

export class UserWalletAmount extends Model<UserWalletAmount> {
	

	@PrimaryKey
    @AutoIncrement
    @AllowNull(false)
    @Column(DataType.BIGINT)
	id: number

	@Column({
		type: DataType.INTEGER,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		validate: channelValidation
	})
	channelId: number

	@ApiProperty()
	@AllowNull(false)
	@Column(DataType.DECIMAL(15, 2))
	amount: number

	@Column({
		type: DataType.INTEGER,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		validate: {
			isAvailable: async (value: string, next: Function): Promise<any> => {
				const isExist = await User.findOne({ where: { userId: value } });
				if (!isExist) {
					next('account:resetPassword:userNotFound');
				}
				next();
			},
		}
	})
	userId: number

	@Column({
		type: DataType.DATE,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		defaultValue: Sequelize.literal('CURRENT_TIMESTAMP')
	})
	updatedAt: string
			
}
