import { Table, Column, Model, DataType, BelongsTo, ForeignKey, AllowNull, Scopes, AutoIncrement, PrimaryKey } from 'sequelize-typescript';
import { ApiProperty } from '@nestjs/swagger';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { PaymentType } from './PaymentType.entity';
import { Channel } from './Channel.entity';
import { User } from '../../user/models/User.entity';
import { Sequelize } from 'sequelize-typescript';
import { Currency } from './Currency.entity';
import { WalletUsageType } from './WalletUsageType.entity';

const channelValidation = {
	isAvailable: async (value: string, next: Function): Promise<any> => {
		const isExist = await Channel.findOne({ where: { channelId: value } });
		if (!isExist) {
			next('channel:notFound');
		}
		next();
	},
};
const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblWalletRecharge' } as IDefineOptions;
@Table({
	tableName: "tblWalletTransaction"
})

@Scopes(() => ({
	channelDetail: {
		attributes: ['payerUserId', 'payeeChannelId', 'amount', 'currencyid', 'walletUsageType', 'updatedAt']
	},
}))

export class WalletTransaction extends Model<WalletTransaction> {
	
	@ForeignKey(() => WalletUsageType)
	@Column(DataType.BIGINT)
	walletUsageTypeId: number

	@PrimaryKey
    @AutoIncrement
    @AllowNull(false)
    @Column(DataType.BIGINT)
	walletTransactionId: number

	@Column({
		type: DataType.INTEGER,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		validate: {
			isAvailable: async (value: string, next: Function): Promise<any> => {
				const isExist = await User.findOne({ where: { userId: value } });
				if (!isExist) {
					next('account:resetPassword:userNotFound');
				}
				next();
			},
		}
	})
	payerUserId: number

	@Column({
		type: DataType.INTEGER,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		validate: channelValidation
	})
	payeeChannelId: number

	@ApiProperty()
	@AllowNull(false)
	@Column(DataType.DECIMAL(15, 2))
	amount: number

	
	@AllowNull(true)
	@Column(DataType.CHAR(255))
	message: string

	

	@Column({
		type: DataType.INTEGER,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		validate: {
			isAvailable: async (value: string, next: Function): Promise<any> => {
				const isExist = await Currency.findOne({ where: { currencyid: value } });
				if (!isExist) {
					next('currency:notFound');
				}
				next();
			},
		}
	})
	@ForeignKey(() => Currency)
	currencyid: number

	@Column({
		type: DataType.DATE,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		defaultValue: Sequelize.literal('CURRENT_TIMESTAMP')
	})
	updatedAt: string
	
}
