import { Inject, Injectable, InternalServerErrorException } from '@nestjs/common';
import { Country } from 'src/modules/common/models/Country.entity';
import { ProductGroup } from 'src/modules/product/models/ProductGroup.entity';
import { Contact } from 'src/modules/user/models/Contact.entity';
import { Address } from '../models/Address.entity';
import { Channel } from '../models/Channel.entity';
import { ChannelDetail } from '../models/ChannelDetail.entity';
import { ChannelProductGroup } from '../models/ChannelProductGroup.entity';
import { Op, fn, QueryTypes } from 'sequelize';
import { Media } from '../../media/models/index';
import { CompanyMedia } from '../models/CompanyMedia.entity';
import { ProcessException } from 'src/middleware/exception/Process.exception';
import { ChannelDocument } from '../models/ChannelDocument.entity';
import { ContactProductGroup } from 'src/modules/user/models/ContactProductGroup.entity';
import { User } from 'src/modules/user/models/User.entity';
import { UserRole } from 'src/modules/user/models/UserRole.entity';
import { ChannelJCT } from '../models/ChannelJCT.entity';
import { ChannelDismissReason } from '../models/ChannelDismissReason.entity';
import { notification } from './../../../config/notificaton.constant';

import { appconstant } from 'src/config';
import { Coupon } from '../models/Coupon.entity';
import { SubscriptionPlan } from '../models/SubscriptionPlan.entity';
import { ChannelType } from '../models/ChannelType.entity';
import { SubscriptionType } from '../models/SubscriptionType.entity';
import { ValidationException } from 'src/middleware/exception/Validation.exception';
import { EventBo } from '../models/EventBo.entity';
import { EventPackage } from '../models/EventPackage.entity';
import { EventParticipants } from '../models/EventParticipants.entity';
import { EventChannel } from '../models/EventChannel.entity';
import { ChannelB2B } from '../models/ChannelB2B.entity';
import { ProfileType } from '../models/ProfileType.entity';
import { Brand } from 'src/modules/product/models/Brand.entity';
import { ChannelAward } from '../models/ChannelAward.entity';
import { RegionCountryJCT } from 'src/modules/common/models/RegionCountryJCT.entity';
import { ContactSocialLink } from 'src/modules/user/models/ContactSocialLink.entity';
import { SocialSite } from '../models/SocialSite.entity';
import { CertificateType } from '../models/CertificateType.entity';
import { PaymentTransaction } from '../models/PaymentTransaction.entity';
import { Product } from 'src/modules/product/models/Product.entity';
import { SampleOrder } from 'src/modules/order/models/SampleOrder.entity';
import { ProductEvaluation } from 'src/modules/product/models/ProductEvaluation.entity';
import { ChannelKeyPartnerService } from './channelKeyPartner.service';
import { Utility } from './../helper/utility';
import { Subscription } from '../models/Subscription.entity';
import { ProductCategory } from 'src/modules/product/models/ProductCategory.entity';
import { Role } from 'src/modules/user/models/Role.entity';
import { CompanyMediaImages } from '../models/CompanyMediaImages';
import { ChannelDetailOutBound } from '../transformation/channelDetailOutBound';
import { OldChannel } from '../models/OldChannel.entity';
import { OldContact } from '../models/OldContact.entity';

import { TradeInformationInBound } from '../transformation/tradeInformationInBound';
import { OldChannelDetail } from '../models/OldChannelDetail.entity';
import { OldAddress } from '../models/OldAddress.entity';
import { OldChannelSocialLink } from '../models/OldChannelSocialLink.entity';

import { isBoolean, isUndefined } from 'lodash';
import { OldChannelSpeciality } from '../models/OldChannelSpeciality.entity';
import { OldChannelCustomerProfile } from '../models/OldChannelCustomerProfile.entity';

var _ = require('lodash');
@Injectable()
export class ChannelAdminService {
    private channelDetailOutBound = new ChannelDetailOutBound();
    private tradeInformationInBound = new TradeInformationInBound();
    constructor(
        @Inject('SequelizeInstance') private readonly sequelizeInstance,
        @Inject('ChannelRepository') private readonly channelRepository: typeof Channel,
        @Inject('ChannelDetailRepository') private readonly channelDetailRepository: typeof ChannelDetail,
        @Inject('ContactRepository') private readonly contactRepository: typeof Contact,
        @Inject('AddressRepository') private readonly addressRepository: typeof Address,
        @Inject('ChannelProductGroupRepository') private readonly channelProductGroupRepository: typeof ChannelProductGroup,
        @Inject('CountryRepository') private readonly countryRepository: typeof Country,
        @Inject('ProductGroupRepository') private readonly productGroupRepository: typeof ProductGroup,
        @Inject('CompanyMediaRepository') private readonly companyMediaRepository: typeof CompanyMedia,
        @Inject('MediaRepository') private readonly mediaRepository: typeof Media,
        @Inject('ChannelDocumentRepository') private readonly channelDocumentRepository: typeof ChannelDocument,
        @Inject('CouponRepository') private readonly couponRepository: typeof Coupon,
        @Inject('ChannelJCTRepository') private readonly channelJCTRepository: typeof ChannelJCT,
        @Inject('ChannelDismissReasonRepository') private readonly channelDismissReasonRepository: typeof ChannelDismissReason,
        @Inject('EventParticipantRepository') private readonly eventParticipantRepository: typeof EventParticipants,
		@Inject('EventChannelRepository') private readonly eventChannelRepository: typeof EventChannel,
        @Inject('ProductRepository') private readonly productRepository: typeof Product,
        @Inject('SubscriptionRepository') private readonly subscriptionRepository: typeof Subscription,
        @Inject('ProductCategoryRepository') private readonly productCategoryRepository: typeof ProductCategory,
        @Inject('ChannelCertificateTypeRepository') private readonly channelCertificateTypeRepository: typeof CertificateType,
        @Inject('CompanyMediaImagesRepository') private readonly companyMediaImagesRepository: typeof CompanyMediaImages,
        @Inject('OldChannelRepository') private readonly oldChannelRepository: typeof OldChannel,
        private readonly channelKeyPartnerService: ChannelKeyPartnerService,
    ){

    }

    async getPendingApprovelChannels(data){           
        try{
        
        let cquery='select "typeId" from "tblAdminUserChanneltype" where "adminUserId"=:userId';
        let chtypes=await this.sequelizeInstance.query(cquery, {
			type: QueryTypes.SELECT,
			replacements: {
				userId: data.userId			
			}
		}).then((results) => {
            return results.map(x => +x.typeId);
        });      

        cquery='select "countryId" from "tblAdminUserCountries" where "adminUserId"=:userId';
        let countries=await this.sequelizeInstance.query(cquery, {
			type: QueryTypes.SELECT,
			replacements: {
				userId: data.userId			
			}
		}).then((results) => {
            return results.map(x => +x.countryId);
        });      

            let filter = data.filter;
            let channelFilter = {
                signUpStatusId: {
                    [Op.in]: [14,15]//data.signupstatusid                    
                },
                
                isActive: '1',
                deletedAt: null
                
            };

            if (countries&&countries.length>0) {
                channelFilter['countryId'] = { [Op.in]: countries }
            }

            if (chtypes&&chtypes.length>0) {
                channelFilter['channelTypeId'] = { [Op.in]: chtypes }
            }

            if (data.search) {
                channelFilter['channelId'] = +data.search
            }
            if (filter.channelTypeId) {
                channelFilter['channelTypeId'] = { [Op.in]: [+filter.channelTypeId] }
                //channelFilter['channelTypeId'] = +filter.channelTypeId;
            }
            if (filter.is_OEM) {
                channelFilter['isOEM'] = filter.is_OEM;
            }
            if (filter.countryId) {
                channelFilter['countryId'] = { [Op.in]: filter.countryId }
            }

            if (filter.planId) {
                channelFilter['planId'] = { [Op.in]: filter.planId }
            }

            let product_Categories_Filter = {
                deletedAt: null
            }
            if (filter.product_Categories) {
                product_Categories_Filter['productGroupId'] = { [Op.in]: filter.product_Categories }
            }
			let channel = await this.channelRepository.scope('basicInfom').findAndCountAll<Channel>({
                        limit: data.limit,
                        offset: data.page,
                        where: channelFilter,
                        distinct:true,
                        attributes: ['isFeatured', 'isPublic'],
                        order: [
                            ['signUpStatusId', 'ASC'],
                            ['updatedAt', 'desc']
                        ],
                    include: [
                        {
                                model: User,
                            attributes: ['userId', 'contactId'],
                            where: { deletedAt: null },

                        },
                        {
                          model:this.contactRepository.scope('contactInfo'),
                          as:'contacts',
                          where: { deletedAt: null },
                          required:true,
                          include:[
                              {
                                  model: Media,
                                  attributes: ['documentPath', 'documentId', 'thumbnailPath'],
                                  required:false,
                              },
                              {
                                  model: ContactProductGroup,
                                  where:{
                                      deletedAt: null
                                  },
                                  include: [{
                                      model: ProductGroup,
                                      attributes: ['productGroup']
                                  }],
                                  required:false,
                              },
                              {
                                  model: this.countryRepository,
                                  attributes: ['country'],
                                  required:false,
                              },
                              {
                                  model: User,
                                  attributes: ['contactId', 'userId', 'lastLogin'],
                                  include: [{
                                      model: UserRole,
                                      attributes: ['roleId']
                                  }]
                              },
                              {
                                  model: EventParticipants,
                                  required: false,
                                  where: {
                                      deletedAt: null
                                  }   
                              }
                          ]
                        },
                        { model: this.countryRepository.scope('regionCountryJCT'),
                            as: 'country',
                            where: { deletedAt: null },
                            required: true },
                        {
                            model: this.channelDetailRepository,
                            attributes: ['webSiteUrl','phone1'],
                            where: { deletedAt: null },
                            required: true
                            },
                            {
                                model:this.addressRepository,
                                as: 'regAddress',
                                required: true
                            },
                            {
                                model:this.channelProductGroupRepository,
                                attributes:['channelProductGroupId'],
                                where:product_Categories_Filter,
                                required: true,
                                include:[{
                                    model:this.productGroupRepository.scope('productInfo'),
                                    where: { deletedAt: null },
                                    required: false
                                }]
                            }
                        ]
            })

            return channel;
        }catch(e){
            console.log(e)
        }
    }    

    
    async pendingChannelsList(userId){

        let cquery='select "typeId" from "tblAdminUserChanneltype" where "adminUserId"=:userId';
        let chtypes=await this.sequelizeInstance.query(cquery, {
			type: QueryTypes.SELECT,
			replacements: {
				userId: userId		
			}
		}).then((results) => {
            return results.map(x => +x.typeId);
        });      

        cquery='select "countryId" from "tblAdminUserCountries" where "adminUserId"=:userId';
        let countries=await this.sequelizeInstance.query(cquery, {
			type: QueryTypes.SELECT,
			replacements: {
				userId: userId			
			}
		}).then((results) => {
            return results.map(x => +x.countryId);
        });      

        let channelFilter = {
            signUpStatusId: {
                [Op.in]: [14,15]//data.signupstatusid                    
            },
            
            isActive: '1',
            deletedAt: null            
        };

        if (countries&&countries.length>0) {
            channelFilter['countryId'] = { [Op.in]: countries }
        }

        if (chtypes&&chtypes.length>0) {
            channelFilter['channelTypeId'] = { [Op.in]: chtypes }
        }


        return await this.channelRepository.findAll<Channel>({
            where: channelFilter,
            attributes:[["companyName","name"], ["channelId", "id"]],
            include:[{
                model:this.contactRepository,
                as:'contacts',
                where: { deletedAt: null, isDefaultContact:'1' },
                required:true,
                attributes:["firstName","lastName", "contactId"],
            }]
        })
    }

    async channelApproval(data){
        if (data.signUpStatusId == 0)
            return await this.channelRepository.update({ isActive: '0', deletedAt: Date() },
                {
                    where: {
                        channelId: +data.channelId
                    }
                })
        else
            return await this.channelRepository.update({ signUpStatusId: data.signUpStatusId },
                {
                    where: {
                        channelId: +data.channelId
                    }
                })

    }

    async getFilterCountry(data){

        let cquery='select "countryId" from "tblAdminUserCountries" where "adminUserId"=:userId';
        let countries=await this.sequelizeInstance.query(cquery, {
			type: QueryTypes.SELECT,
			replacements: {
				userId: data.userId			
			}
		}).then((results) => {
            return results.map(x => +x.countryId);
        });      

            let channelFilter = {
                deletedAt: null                
            };

            if (countries&&countries.length>0) {
                channelFilter['countryId'] = { [Op.in]: countries }
            }


        return await this.countryRepository.scope('regionCountryJCT').findAll({
            where: channelFilter,
            order: [
                ['priority', 'ASC']
            ]
        })
    }

    async getProductCat(data){

        let channels=[];      

        if(data.accessId==3||data.accessId==4)
        {            
            if(data.accessId==4)
            {
                let cquery='select "countryId" from "tblAdminUserCountries" where "adminUserId"=:userId';
                await this.sequelizeInstance.query(cquery, {
                    type: QueryTypes.SELECT,
                    replacements: {
                        userId: data.userId			
                    }
                }).then(async(results) => {
                    cquery='select "channelId" from "tblChannel" where "isActive"=\'1\' and "deletedAt" is null and "signUpStatusId" in(14,15) and "countryId" in(:countries) and "channelTypeId" in(select "typeId" from "tblAdminUserChanneltype" where "adminUserId"=:userId)';
                    await this.sequelizeInstance.query(cquery, {
                        type: QueryTypes.SELECT,
                        replacements: {
                            userId: data.userId,
                            countries:results.map(x => +x.countryId)

                        }
                    }).then((results) => {                        
                        channels.push(...results.map(x => +x.channelId));
                    });      
                });          
            }
            else
            {
                let cquery='select "channelId" from "tblChannel" where "isActive"=\'1\' and "deletedAt" is null and "signUpStatusId" in(14,15) and "channelTypeId" in(select "typeId" from "tblAdminUserChanneltype" where "adminUserId"=:userId)';
                await this.sequelizeInstance.query(cquery, {
                    type: QueryTypes.SELECT,
                    replacements: {
                        userId: data.userId
                    }
                }).then((results) => {                        
                    channels.push(...results.map(x => +x.channelId));
                });      
            }            
        }

        let channelFilter = {
            deletedAt: null            
        };

        if (channels&&channels.length>0) {
            channelFilter['channelId'] = { [Op.in]: channels }
        }

        let productIds = [];
                  productIds = await this.channelProductGroupRepository.findAll({
                    where: channelFilter,
                        attributes:["productGroupId"],
                    include: [{
                        model:this.channelRepository,
                        required: true,
                        where: {
                            signUpStatusId: {
                                [Op.in]: data.signupstatusid
                            },
                            isActive: '1',
                            deletedAt: null
                            },
                            attributes:[]
                    }]
                })

                if(productIds){
                    productIds = productIds.map(x=>x.productGroupId)
                    let result = await this.productGroupRepository.findAll({
                        where:{
                            productGroupId:{ [Op.in]:productIds}
                        },
                        attributes:[["productGroup", "name"], ["productGroupId", "id"]]
                    })
                    return result;
                }else{
                    return [];
                }
    }

    async getSubscriptionPlan(){
        return await SubscriptionPlan.findAll<SubscriptionPlan>({
            attributes: [['subscriptionPlanId','id'], ['subscriptionPlanType','name']]            
        })
    }

    async medialibraryByContact(user){

        let cond = {
            ...(user.channelId && {channelId: +user.channelId}),
            ...(user.contactId && {contactId: +user.contactId}),
            ...(user.oldChannelId && {oldChannelId: +user.oldChannelId}),
            deletedAt: null
        }

		return await this.companyMediaRepository.findAll<CompanyMedia>({
			where: cond,
			include: [{
				model: Media,
				as: 'document',
				where: { deletedAt: null },
				required: false
			},
			{
				model: Media,
				as: 'snap',
				where: { deletedAt: null },
				required: false
			},],
			order: [['companyMediaId', 'DESC']]
		});	
	}

	async saveDocument(reqVideo){
		let updatedDocument = await this.mediaRepository.create({
			documentPath: reqVideo.Key,
			documentTypeId: 3,
			dirName: 'connectionVideos'
		});

		await this.companyMediaRepository.update({
			snapId: updatedDocument.documentId
		}, { where: {
			documentId: reqVideo.mediaId
		    }
		});
		return updatedDocument;
	}

	async deleteMediaVideo(data){
		return await this.companyMediaRepository.update({
			deletedAt: Date()
		}, { where: {
			documentId: data.documentId
		    }
		});
	}
	
	async saveUploadedVideo(reqVideo, user){

        let cond = {
            ...(user.channelId && {channelId: +user.channelId}),
            ...(user.contactId && {contactId: +user.contactId}),
            ...(user.oldChannelId && {oldChannelId: +user.oldChannelId}),
        }

        return this.sequelizeInstance.transaction(async transaction => {
		let updatedDocument = await this.mediaRepository.create({
			documentPath: reqVideo.Key,
			documentTypeId: 3,
			dirName: 'introVideos'
		}, transaction);
		let companyMedia = await this.companyMediaRepository.create({
			documentId: updatedDocument.documentId,
			// contactId: +user.contactId,
			// channelId: +user.channelId,
            ...cond,
			snapId: null
        }, transaction);
        
    let data = {
        documentId: companyMedia.documentId,
        contactId: companyMedia.contactId,
        channelId: companyMedia.channelId,
        oldChannelId: companyMedia.oldChannelId,
        snapId: companyMedia.snapId,
        companyMediaId:companyMedia.companyMediaId,
        document:updatedDocument
    };
        return data;
    })
    .catch((err) => {
        console.log(err)
        throw new ProcessException('account:singup:tryAgain');
    });
    }

    async deleteCompanyLogo(data){
		return await this.channelDocumentRepository.update({
			deletedAt: Date()
		}, { where: {
			channelDocumentId: data.channelDocumentId
		    }
		});
	}

//getting company profile for bo

async getCompanyProfile(user: any): Promise<any> {
    //console.log("user:",user);
    let companyProfile: any = '';
    let channelAward: any[] = [];
    let data = '';
    let channelStatusArray: Array<number> = [appconstant.ACCOUNTSETUP, appconstant.ACCOUNTSETUP_COMPANYPROFILE, appconstant.ACCOUNTSETUP_TRADEINFORMATION, appconstant.ACCOUNTSETUP_TRADELOCATION, appconstant.CHANNEL_APPROVAL, appconstant.CHANNEL_APPROVED, appconstant.CLAIM_APPROVED];
    try {
        let userSignUpStatus = channelStatusArray.includes(parseInt(user.signUpStatusId));
        if (userSignUpStatus) {
            companyProfile = await this.sequelizeInstance.query("select * from fngetchannelprofile(" + user.userId + "," + parseInt(user.signUpStatusId) + ",'" + process.env.CDN_URL + "');");
            companyProfile = companyProfile[0][0].fngetchannelprofile;

            console.log('companyProfle', companyProfile);

            if(companyProfile.channel.channelAward){
                channelAward  = await Promise.all(companyProfile.channel.channelAward.map(async x => {
                       
                       if(x.documentId){
                           let data = await 
                            this.sequelizeInstance.query('select "documentId","documentPath" from "tblDocument" where "documentId"='+x.documentId, {
                               type: QueryTypes.SELECT,
                               raw: true,
                               }).then(y=>{            
                                   console.log('img',x, y, );
                                   return Object.assign(x, {awardImg: y[0].documentPath, isOld:true, lifeTime: x.lifeTime == '1' ? true : false});
                               })
                           return data;
                       }
                       else{
                           return Object.assign(x, {awardImg: null, isOld:true, lifeTime: x.lifeTime == '1' ? true : false});
                       }
                           
                   }))
               }

               let bannerImg = [];
               console.log('banner', companyProfile.channel.channelDetail.channelBannerId)
               if(companyProfile.channel.channelDetail.channelBannerId){
                bannerImg = await this.sequelizeInstance.query('select "documentId","documentPath" from "tblDocument" where "documentId"='+companyProfile.channel.channelDetail.channelBannerId, {
                            type: QueryTypes.SELECT,
                            raw: true,
                            }).then(y=>{            
                                console.log('img', y );
                                return y[0];
                            })
               }

                   // channelAward = await Promise.all(channelAward);
                   // console.log('channelAward', channelAward);
                   companyProfile['bannerImg'] = bannerImg;
                   companyProfile['channelAward'] = channelAward;
           }
    } catch (e) {
        console.log("Error+", e);
        //throw new InternalServerErrorException();
    }
    if (!companyProfile) {
        throw new ProcessException('channel:getCompanyProfile:companyNotFound');
    }
    return companyProfile;
}

//Get All admin coupns list
public async getCouponList(couponFilterDto): Promise<any> {
    //const Op = Sequelize.Op;

    let where = {
        deletedAt: null,
    };

    // if (pagination.search) {
    //     where[Op.or] = [
    //         {
    //             couponCode: {
    //                 $ilike: '%' + pagination.search + '%'
    //             }
    //         },
    //         {
    //             couponDesc: {
    //                 $ilike: '%' + pagination.search + '%'
    //             }
    //         }
    //     ]
    // }

    // if (couponFilterDto.subscriptionTypeId) {
    //     where['subscriptionTypeId'] = {
    //         $in: couponFilterDto.subscriptionTypeId
    //     }
    // }

    // if (couponFilterDto.channelTypeId) {
    //     where['channelTypeId'] = {
    //         $in: couponFilterDto.channelTypeId
    //     }
    // }

    // if (couponFilterDto.subscriptionPlanId) {
    //     where['subscriptionPlanId'] = {
    //         $in: couponFilterDto.subscriptionPlanId
    //     }
    // }

    // if (couponFilterDto.isActive) {
    //     where['isActive'] = couponFilterDto.isActive;
    // }

    // if (couponFilterDto.fromDate) {
    //     where['fromDate'] = {
    //         $gte: couponFilterDto.fromDate
    //     };
    // }

    // if (couponFilterDto.toDate) {
    //     where['toDate'] = {
    //         $lte: couponFilterDto.toDate + ' 23:59:59'
    //     };
    // }

    let options = {
        include: [{
            model: SubscriptionPlan,
            attributes: ['subscriptionPlanId', 'subscriptionPlanType']
        }, {
            model: ChannelType,
            attributes: ['channelTypeId', 'channelType']
        },
        {
            model: SubscriptionType,
            attributes: ['subscriptionTypeId', 'subscriptionType']
        }],
        where: where
    };

    let result: any = {};

    result.count = await Coupon.count(options);

    result.aggregation = await Coupon.findAll<Coupon>({
        attributes: [[this.sequelizeInstance.literal('json_build_object(\'plan\',to_json(array_agg(distinct("subscriptionPlan".*))), \'channel_type\',to_json(array_agg(distinct("channelType".*))), \'subscription_type\',to_json(array_agg(distinct("subscriptionType".*))))'), 'filters']],
        include: [{
            model: SubscriptionPlan,
            required: true,
            attributes: []
        }, {
            model: ChannelType,
            required: true,
            attributes: []
        },
        {
            model: SubscriptionType,
            required: true,
            attributes: []
        }],
        where: where,
        raw: true
    }).then((result: any) => {

        if (result.filters?.plan) {
            result.filters.plan = result.filters.plan.map((record) => {
                return {
                    subscriptionPlanId: record.subscriptionPlanId,
                    subscriptionPlanType: record.subscriptionPlanType
                }
            })
        }

        return result.filters;
    });

     options['limit'] = couponFilterDto.limit;
     options['offset'] = couponFilterDto.page;

    // let sortField = 'couponId';
    // let sortOrder = [];
    // if (pagination.orderBy && ['couponDesc', 'couponCode', 'couponOfferValue', 'isActive', 'usageThreshold', 'userThreshold', 'fromDate', 'toDate', 'subscriptionPlanType', 'subscriptionType', 'channelType'].includes(pagination.orderBy)) {
    //     sortField = pagination.orderBy;
    //     if (pagination.orderBy == 'subscriptionPlanType') {
    //         sortOrder.push(...['subscriptionPlan', sortField, pagination.order]);
    //     } else if (pagination.orderBy == 'subscriptionType') {
    //         sortOrder.push(...['subscriptionType', sortField, pagination.order]);
    //     } else if (pagination.orderBy == 'channelType') {
    //         sortOrder.push(...['channelType', sortField, pagination.order]);
    //     }
    // }

    // if (sortOrder.length === 0) {
    //     sortOrder.push(...[sortField, pagination.order]);
    // }

//    options['order'] = [sortOrder];

    result.rows = await Coupon.findAll<Coupon>(options);

    return result;
}

//Get admin coupns By ID
public async getCouponByID(couponid: number): Promise<any> {
    try {
        let result = await Coupon.findAll<Coupon>({
            include: [{
                model: SubscriptionPlan,
                attributes: ['subscriptionPlanId', 'subscriptionPlanType']
            }, {
                model: ChannelType,
                attributes: ['channelTypeId', 'channelType']
            },
            {
                model: SubscriptionType,
                attributes: ['subscriptionTypeId', 'subscriptionType']
            }],
            where: {
                couponId: couponid
            }
        });
        return result;
    } catch (e) {
      //  throw new InternalServerErrorException();
    }
}

//Add coupon in admin
public async addChannelCoupon(adminCouponAddDto: any): Promise<any> {

    if (await this.couponRepository.count({
        where: {
            couponCode: {
                [Op.iLike]: adminCouponAddDto.couponCode.trim()
            },
            channelTypeId: +adminCouponAddDto.channelTypeId,
            // subscriptionPlanId: +adminCouponAddDto.subscriptionPlanId,
            deletedAt: null
        }
    })) {
        throw new ValidationException([{
            value: adminCouponAddDto.couponCode.trim(),
            property: 'couponCode',
            children: [],
            constraints:
                { unique: 'validate:field:shouldBeUnique' }
        }]);
    }

    try {
        let coupon = [];
        if (adminCouponAddDto.subscriptionPlans && adminCouponAddDto.subscriptionPlans.length > 0) {
            for (var i = 0; i < adminCouponAddDto.subscriptionPlans.length; i++) {
                for (var j = 0; j < adminCouponAddDto.subscriptionPeriods.length; j++) {
                    let couponObject = {};
                    couponObject['couponDesc'] = adminCouponAddDto.couponDesc;
                    couponObject['couponCode'] = adminCouponAddDto.couponCode;
                    couponObject['couponOfferValue'] = adminCouponAddDto.couponOfferValue;
                    couponObject['isActive'] = adminCouponAddDto.isActive;
                    couponObject['fromDate'] = adminCouponAddDto.fromDate;
                    couponObject['toDate'] = adminCouponAddDto.toDate;
                    couponObject['subscriptionTypeId'] = adminCouponAddDto.subscriptionTypeId;
                    couponObject['discountUnit'] = adminCouponAddDto.discountUnit;
                    couponObject['userThreshold'] = (adminCouponAddDto.userThreshold) ? adminCouponAddDto.userThreshold : 0;
                    couponObject['usageThreshold'] = (adminCouponAddDto.usageThreshold) ? adminCouponAddDto.usageThreshold : 0;
                    couponObject['channelTypeId'] = adminCouponAddDto.channelTypeId;
                    couponObject['subscriptionPlanId'] = adminCouponAddDto.subscriptionPlans[i];
                    couponObject['subscriptionPeriodId'] = adminCouponAddDto.subscriptionPeriods[j];
                    coupon.push(couponObject);
                }
            }
        } else {
            coupon.push(adminCouponAddDto);
        }
        return await this.couponRepository.bulkCreate(coupon);
    } catch (e) {

        console.log("cerr:",e);
       // throw new InternalServerErrorException();
    }
}


//Update Coupon By Id
public async UpdateChannelCoupon(couponid: number, adminCouponAddDto: any): Promise<any> {
    if (await this.couponRepository.count({
        where: {
            couponCode: {
                [Op.iLike]: adminCouponAddDto.couponCode.trim()
            },
            channelTypeId: +adminCouponAddDto.channelTypeId,
            subscriptionPlanId: +adminCouponAddDto.subscriptionPlanId,
            subscriptionPeriodId: +adminCouponAddDto.subscriptionPeriodId,
            deletedAt: null,
            couponId: {
                [Op.not]: couponid
            }
        }
    })) {
        throw new ValidationException([{
            value: adminCouponAddDto.couponCode.trim(),
            property: 'couponCode',
            children: [],
            constraints:
                { unique: 'validate:field:shouldBeUnique' }
        }]);
    }

    let result = await this.couponRepository.update<Coupon>(adminCouponAddDto, {
        where: {
            couponId: couponid
        }
    });

    return result;
}

async getBuyerNeed(filter){
    let limit = ' LIMIT ' + filter.limit;
    let offset = ' OFFSET ' + filter.offset;
    let query = 'select count(*) OVER() AS total, bn."id",bn."channelid",bn."contactid",bn."requirements",tc."companyName",tcon."firstName",tcon."lastName", '
    +'(select td."documentPath" from "tblContact" utc inner join "tblDocument" td on td."documentId" = utc."documentId" where utc."contactId" = tcon."contactId") as con_image '
    +'from "buyingneed" as bn inner join "tblChannel" tc on tc."channelId"=bn."channelid" inner join "tblContact" tcon on tcon."contactId"=bn."contactid"';
    query += limit+offset;
    let list=await this.sequelizeInstance.query(query, {
        type: QueryTypes.SELECT
    });
     
    let result=[];
    if(list){
        for(var i=0;i<list.length;i++){
            query='select "productCategorie" from "tblProductCategories" where "productCategorieId" in(select "categoryid" from "buyerreqcategory" where "buyerreqid"=:ids group by "categoryid")';
    
            let category=await this.sequelizeInstance.query(query, {
                type: QueryTypes.SELECT,
                replacements: {
                    ids:list[i].id
                }
            });
        
            let obj={...list[i],...{category:category.map(x =>x.productCategorie)}};
            result.push(obj);
        }
        return {result:result, total:list[0].total}
    }else{
        return {result:result, total:0}
    }
}

async getRequestIntro(channelid){
    let Request_Intro = await this.sequelizeInstance.query('select * from requestintro('+channelid+')');
    return Request_Intro;
}

async updateChannelConnect(channelId, updateChannelDto, user, suggest_Channel) {

    let result: any = await this.channelJCTRepository.findOne<ChannelJCT>(
        {
            where: {
                channelJCTId: updateChannelDto.channelJCTId,
                deletedat: null
            },
            include: [
                {
                    model: this.channelRepository,
                    as: 'partnerChannel',
                    where: { deletedAt: null },
                },
                {
                    model: this.channelRepository,
                    as: 'Channel',
                    where: { deletedAt: null },
                },
                // {
                // 	model: this.contactJCTRepository,
                // 	as: 'contactJCT',
                // 	where: { contactId: user.contactId },
                // 	required: false
                // }
            ]
        });

    let beforeStatus: number = result.connectionStatusTypeId;
    let query = 'UPDATE "tblChannelJCT" SET "channelId" = '+user.channelId+', "partnerChannelId" = '+channelId+', "connectionStatusTypeId" = '+updateChannelDto.connectionStatusTypeId+', "updatedAt" = now(), "requested_by" = '+user.contactId+', "requested_to" = '+updateChannelDto.contactid+', "request_sent_at" = now() WHERE "channelJCTId" = '+updateChannelDto.channelJCTId;
    let count = await this.sequelizeInstance.query( query, {
        type: QueryTypes.SELECT
    }).then(async (res) => {
        console.log('this is test');
        if (+updateChannelDto.connectionStatusTypeId == appconstant.REQUEST_INTRO || +updateChannelDto.connectionStatusTypeId == appconstant.PENDINGMYPROSPECT || +updateChannelDto.connectionStatusTypeId == appconstant.DISMISS) {

            let delQuery = 'DELETE FROM "tblChannelJCT" WHERE "requested_by" = ' + user.contactId + ' AND "requested_to" = ' + updateChannelDto.contactid + ' AND "connectionStatusTypeId" = 1';
            await this.sequelizeInstance.query(delQuery, {
                type: QueryTypes.DELETE
            })
        }

        if (+updateChannelDto.connectionStatusTypeId == appconstant.DISMISS) {
            let otherReasons = (updateChannelDto.otherreasons) ? updateChannelDto.otherreasons : null;
            await this.channelDismissReasonRepository.create({
                channelJCTId: result.channelJCTId,
                dismissreasonid: updateChannelDto.dismissreasonid,
                otherreasons: otherReasons,
                updatedby: user.userId
            });
        }

        let updateObject = {
            subscription_queued: '0'
        }

        if (suggest_Channel && +updateChannelDto.connectionStatusTypeId == appconstant.PENDINGMYPROSPECT) {
            updateObject['suggestion_connect_flag'] = '1';
        }

        if (updateChannelDto.leadManagementTag) {
            if (result.channelId == +user.channelId) {
                updateObject['leadManagementTag'] = updateChannelDto.leadManagementTag;
            } else if (result.partnerChannelId == +user.channelId) {
                updateObject['partnerLeadManagementTag'] = updateChannelDto.leadManagementTag;
            }
        }

        // connection remainder
        if (+updateChannelDto.connectionStatusTypeId == appconstant.PENDINGMYPROSPECT || updateChannelDto.connectionStatusTypeId == appconstant.REQUEST_INTRO) {
            updateObject['last_remainder_sent_at'] = this.sequelizeInstance.fn('NOW');
            updateObject['no_of_remainders_sent'] = '1';
            updateObject['introVideo'] = updateChannelDto.videoUrl;
        }
        // console.log('updateObject');
        // console.log(updateObject);
        await result.update(updateObject);

        let notificationObject = {
            senderId: user.contactId,
            receiverId: updateChannelDto.contactid,
            note: updateChannelDto.note
        };
        let status: any = updateChannelDto.connectionStatusTypeId;
        let notificationAction = null;

        
        switch (status) {
            case appconstant.PENDINGMYPROSPECT:
                notificationAction = notification.REQUEST_TO_MYPROSPECT;
                break;
            case appconstant.MAYBELATER:
                notificationAction = notification.MAYBE_LATER;
                break;
            case appconstant.DISMISS:
                if (beforeStatus == appconstant.PENDINGMYPROSPECT) {
                    notificationAction = notification.MYPROSPECT_DECLINE;
                } else if (beforeStatus == appconstant.MYCHANNEL) {
                    notificationAction = notification.DISMISS_CHANNEL;
                } else if (beforeStatus == appconstant.PENDINGAPPROVAL) {
                    notificationAction = notification.MYCHANNEL_DECLINE;
                }

                break;
            case appconstant.MYPROSPECT:
                notificationAction = notification.MYPROSPECT_APPROVAL;
                break;
            case appconstant.PENDINGAPPROVAL:
                notificationAction = notification.REQUEST_TO_MYCHANNEL;
                break;

            case appconstant.MYCHANNEL:
                notificationAction = notification.MYCHANNEL_APPROVAL;
                break;
            default:
                break;
        }
        // if (notificationAction) {
        //     var contactDetail = await this.checkDNDContact(updateChannelDto.contactid);
        //     updateChannelDto.dnd = contactDetail.isConnectionMail;
        //     if(contactDetail.isConnectionMail != '0' ){
        //         this.notificationService.sendNotificationRouter(notificationObject, notificationAction);
        //     }
        // }

        return updateChannelDto;
    });

    return updateChannelDto;
}

async getPublicCompanyFlags(channelId){
    let query = 'Select "channelId", "isFeatured", "isPublic" from "tblChannel" where "channelId"=:channelId';

    let result = await this.sequelizeInstance.query(query, {
        type: QueryTypes.SELECT,
        replacements: {
            channelId: channelId
        }
    }).then(res => res[0]);

    return result;
}

async setCompanyPublicFlags(data){

    let save = {
        isPublic: data.isPublic,
        isFeatured: data.isFeatured
    }

    let result = await this.channelRepository.update(save, {where:{
        channelId: data.channelId
    }}).then(res => res);

    return result;
}

//event create
public async boEventCreate(data){

    let event = data.event;
    
    var eventId: number;
    let availability = false;
    if(event.eventCatalystId){
        let available = 'select "id" from "tblevents" where "eventCatalystId"=:eventCatId and "deletedAt" is null limit 1';

        let ifAvailable = await this.sequelizeInstance.query(available, {
        type: QueryTypes.SELECT,
        raw: true,
        replacements: {
            eventCatId: event.eventCatalystId
        }
        });
        
        console.log('ifAvailable', ifAvailable)
        
        if(event.eventId){
            if(ifAvailable.length && ifAvailable[0].id != event.eventId){
                return {error: 'Event already Registered'}
            }
            else{
                eventId = event.eventId;
                await EventBo.update(event, {
                    where:{
                        id: event.eventId,
                    }
                })
            }
        }
        else{
            eventId = await EventBo.create(event).then(res => res.id);
        }

    }else{
        if(event.eventId){
            eventId = event.eventId;
            await EventBo.update(event, {
                where:{
                    id: event.eventId
                }
            })
        }else{
            eventId = await EventBo.create(event).then(res => res.id);
        }
    }

    

    let bannerId: any = null;
						let companyBanner;
						if (event.eventBannerId) {
							bannerId = event.eventBannerId;
						} else {
							// save comapny logo detail to document table

							let oldBanner = await EventBo.findOne({
								where: {id: eventId},
								attributes: ['eventBannerId']
							}).then( res => {
								console.log('banner', res.eventBannerId);
								return res.eventBannerId;
							}).catch(err => console.log('banner error', err));

							if(oldBanner){
								await this.mediaRepository.update({deletedAt: new Date()}, {where: {
									documentId: +oldBanner
								}});
							}

							companyBanner = await this.mediaRepository.create({
								documentPath: event.eventBanner,
								documentTypeId: appconstant.DOCUMENTTYPE_IMAGE,
								dirName: appconstant.FOLDER_PATH_COMPANY_BANNER
							});
                            console.log('banner', companyBanner);
								bannerId = companyBanner.documentId;
						}
					

    let logoId: any = null;

    
    if ( event.eventLogoId ) {
        logoId = event.eventLogoId;
    } else {
        // save comapny logo detail to document table

        let oldBanner = await EventBo.findOne({
            where: {id: eventId},
            attributes: ['eventLogoId']
        }).then( res => {
            console.log('banner', res.eventLogoId);
            return res.eventLogoId;
        }).catch(err => console.log('banner error', err));

        if(oldBanner){
            await this.mediaRepository.update({deletedAt: new Date()}, {where: {
                documentId: +oldBanner
            }});
        }

        companyBanner = await this.mediaRepository.create({
            documentPath: event.eventLogo,
            documentTypeId: appconstant.DOCUMENTTYPE_IMAGE,
            dirName: appconstant.FOLDER_PATH_COMPANY_LOGO
        });
        console.log('banner', companyBanner);
            logoId = companyBanner.documentId;
    }

    // console.log('bannerId', bannerId, logoId);

    await EventBo.update({
        eventLogoId: logoId,
        eventBannerId: bannerId ,
        updatedAt: new Date()
    }, {where:{
        id: eventId,
        deletedAt: null
    }});
    
    //deleting packages which are not selected
    let oldPackages = await EventPackage.findAll({
        where: {
            eventId: eventId,
            deletedAt: null
        },
        attributes: ['packageId']
    }).then(res => {console.log(res); return _.map(res, (x)=>x.packageId)});

    if(oldPackages?.length){
        console.log('oldPacks', oldPackages)
        let activeIds = [];
        data.packages.map(x => {
            oldPackages.map(y => {
                if(x.packageId == y)
                    activeIds.push(x.packageId);
            })
        });

        //deleting inactive packages
        await EventPackage.update({
            deletedAt: new Date(),
            updatedAt: new Date()
        }, {
            where:{
                eventId: eventId,
                packageId: {
                    [Op.notIn]: activeIds
            }
        }
        }).then(res => console.log('cleared inactive packs', res)).catch(e => console.log('error ', e , activeIds));
    }



    data.packages.map(async x => {
        if(x.packageId){
            await EventPackage.update({
                ...x, updatedAt: new Date()
            },{
                where: {
                    packageId: x.packageId,
                    deletedAt: null
                }
            })
        }else{
            await EventPackage.create({...x, eventId: eventId});
        }
    })


    return true;
}

//getEventListing

public async getEventListing(eventFilter){
    //const Op = Sequelize.Op;

    let where = {
        deletedAt: null,
    };

    // if (pagination.search) {
    //     where[Op.or] = [
    //         {
    //             couponCode: {
    //                 $ilike: '%' + pagination.search + '%'
    //             }
    //         },
    //         {
    //             couponDesc: {
    //                 $ilike: '%' + pagination.search + '%'
    //             }
    //         }
    //     ]
    // }

    // if (couponFilterDto.subscriptionTypeId) {
    //     where['subscriptionTypeId'] = {
    //         $in: couponFilterDto.subscriptionTypeId
    //     }
    // }

    // if (couponFilterDto.channelTypeId) {
    //     where['channelTypeId'] = {
    //         $in: couponFilterDto.channelTypeId
    //     }
    // }

    // if (couponFilterDto.subscriptionPlanId) {
    //     where['subscriptionPlanId'] = {
    //         $in: couponFilterDto.subscriptionPlanId
    //     }
    // }

    // if (couponFilterDto.isActive) {
    //     where['isActive'] = couponFilterDto.isActive;
    // }

    // if (couponFilterDto.fromDate) {
    //     where['fromDate'] = {
    //         $gte: couponFilterDto.fromDate
    //     };
    // }

    // if (couponFilterDto.toDate) {
    //     where['toDate'] = {
    //         $lte: couponFilterDto.toDate + ' 23:59:59'
    //     };
    // }

    // let options = {
    //     include: [{
    //         model: SubscriptionPlan,
    //         attributes: ['subscriptionPlanId', 'subscriptionPlanType']
    //     }, {
    //         model: ChannelType,
    //         attributes: ['channelTypeId', 'channelType']
    //     },
    //     {
    //         model: SubscriptionType,
    //         attributes: ['subscriptionTypeId', 'subscriptionType']
    //     }],
    //     where: where
    // };

    let options = {
        include: [
            {
                model: EventPackage,
                where:{
                    deletedAt: null
                },
                required: false
            },
            {
                model: Country,
                attributes: ['countryId', 'country'],
                required: false
            },
            {
                model: Media,
                as: 'logo',
                where: {
                    deletedAt: null
                },
                required: false
            },
            {
                model: Media,
                as: 'banner',
                where: {
                    deletedAt: null
                },
                required: false
            }
        ]
    }

    let result: any = {};

    result.count = await EventBo.count(options);
    console.log('result.count', result.count)

    result.aggregation = await EventBo.findAll<EventBo>({
        include: [
            {
                model: EventPackage,
                where:{
                    deletedAt: null
                },
                required: false
            },
            {
                model: Country,
                attributes: ['countryId', 'country'],
                required: false
            },
            {
                model: Media,
                as: 'logo',
                where: {
                    deletedAt: null
                },
                required: false
            },
            {
                model: Media,
                as: 'banner',
                where: {
                    deletedAt: null
                },
                required: false
            }
        ]
    });

     options['limit'] = eventFilter.limit;
     options['offset'] = eventFilter.page;


    result.rows = await EventBo.findAll<EventBo>(options);

    return result;
}

//getEventById
async getEventById(eventId){
    let result: any = {};

    result = await EventBo.findAll<EventBo>({
        where:{
            id: eventId,
            deletedAt: null
        },
        include: [
            {
                model: EventPackage,
                where:{
                    deletedAt: null
                },
                required: false
            },
            {
                model: Country,
                attributes: ['countryId', 'country'],
                required: false
            },
            {
                model: Media,
                as: 'logo',
                attributes: ['documentId', 'documentPath'],
                where: {
                    deletedAt: null
                },
                required: false
            },
            {
                model: Media,
                as: 'banner',
                attributes: ['documentId', 'documentPath'],
                where: {
                    deletedAt: null
                },
                required: false
            }
        ]
    });

    return result;
}

async getEventDetailsById(eventId){
    let result: any = {};

    result.eventDetails = await EventBo.findAll<EventBo>({
        where:{
            id: eventId,
            deletedAt: null
        },
        include: [
            {
                model: EventPackage,
                where:{
                    deletedAt: null
                },
                required: false
            },
            {
                model: Country,
                attributes: ['countryId', 'country'],
                required: false
            },
            {
                model: Media,
                as: 'logo',
                attributes: ['documentId', 'documentPath'],
                where: {
                    deletedAt: null
                },
                required: false
            },
            {
                model: Media,
                as: 'banner',
                attributes: ['documentId', 'documentPath'],
                where: {
                    deletedAt: null
                },
                required: false
            },
            // {
            //     model: EventChannel,
            //     include: [{
            //         model: Channel,
            //         include: [
            //         {
            //             model: EventParticipants,
            //             include:[
            //                 {
            //                     model: Contact,
            //                     include: [{
            //                         model: Media,
            //                         required: false
            //                     }]
            //                 },
            //                 {
            //                     model: EventPackage,
            //                     required: false
            //                 }
            //             ]
            //         }
            //     ]
            //     },
                
            // ]
            // },
            
        ]
    }).then(res => res[0]);

    let eventChannels = await EventChannel.findAll<EventChannel>({
        where: {
            eventId: eventId,
            deletedAt: null
        },
        include: [
            {
                model: Channel,
                include: [
                {
                    model: Contact,
                    as: 'contacts',
                    include: [{
                        model: EventParticipants,
                        required: false,
                        include:[
                            {
                                    model: EventPackage,
                                    required: false
                            }
                        ]
                    },
                    {
                        
                        model: Media,
                        required: false

                    }
                    ]
                }
            ]
            },
        ]
    });

    result.eventChannels = eventChannels;

    return result;
}

//eventAvailabilityCheck
async eventAvailabilityCheck(eventCatId, eventId){
    let result = [];
    let available = 'select * from "tblevents" where "eventCatalystId"=:eventCatId and "deletedAt" is null limit 1'  
    if(eventId){
        available = 'select "id" from "tblevents" where "eventCatalystId"=:eventCatId and "deletedAt" is null limit 1'
    }

    result = await this.sequelizeInstance.query(available, {
        type: QueryTypes.SELECT,
        raw: true,
        replacements: {
            eventCatId: eventCatId,
        }
    });

    if(result.length){
        if(result[0].id == eventId)
        return true;
        else
        return false;
    }
    else    
        return true;

}

//getEventChannelDetails
async getEventChannelDetails(channelId, eventId){
    let result: any={participants: [], contacts: []};

    let participantQuery = 'select tc."contactId" as "id", concat(tc."firstName",  \' \', tc."lastName") as "name", td."documentPath", tpar."eventParticipantId" from "tblEventParticipants" tpar inner join "tblContact" tc on tc."contactId"=tpar."contactId" left join "tblDocument" td on td."documentId"=tc."documentId" where tpar."channelId"=:channelId and tpar."eventId"=:eventId';

    result.participants = await this.sequelizeInstance.query(participantQuery, {
        type: QueryTypes.SELECT,
        raw: true,
        replacements: {
            channelId: channelId,
            eventId: eventId
        }
    }).then(res => res);

    let contactsQuery = 'select tc."contactId" as "id", concat(tc."firstName", \' \', tc."lastName") as "name", td."documentPath"  from "tblContact" tc left join "tblDocument" td on td."documentId"=tc."documentId" where tc."channelId"=:channelId and tc."deletedAt" is null';

    result.contacts = await this.sequelizeInstance.query(contactsQuery, {
        type: QueryTypes.SELECT,
        raw: true,
        replacements: {
            channelId: channelId,
        }
    });

    return result;

}

//boEventUserManage
async boEventUserManage(data){
    console.log(data);

    let contactId = data.contactId;
    let channelId = data.channelId;
    let eventId = data.eventId;
    let participantType = data.pType;
    let channelType = data.cType;


        let registeredQuery = 'select "contactId", "eventParticipantId", status from "tblEventParticipants" where "channelId"=:channelId and "eventId"=:eventId  and "deletedAt" is null';

        let oldIds = await this.sequelizeInstance.query(registeredQuery, {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
                channelId: channelId,
                eventId: eventId
            }
        });

        let activeId = [];
        
        oldIds.map(x => {
            contactId.map(y => {
                if(x.contactId == y)
                    activeId.push(y);
            })
        })

        //removing deleted contacts
        await this.eventParticipantRepository.destroy({where: {
            eventId: eventId,
            channelId: channelId,
            contactId: {
                [Op.notIn]: activeId
            }
        }});

        console.log('activeId', activeId)
        //adding new contacts
        let newRegisteredUsers = [];
        if(activeId.length)
          contactId.map(x => {
              if(activeId.filter(y => x == y).length == 0)
                newRegisteredUsers.push(x);
          })
        else 
         newRegisteredUsers = contactId;

        if(newRegisteredUsers.length){
            let newContacts = newRegisteredUsers.map(x => {
                let data = {contactId: +x, channelId: +channelId, eventId: +eventId, channelType: +channelType, participantType: +participantType}

                return data;
            });
            console.log('new users', newContacts)
            await this.eventParticipantRepository.bulkCreate(newContacts);
        }

    

    return true;
}

//sending data to eventCatalyst
async boToEventCatalyst(contacts){
   

    let query='select (select concat(\'https://d35w8j22j5uly8.cloudfront.net\',"documentPath") from "tblDocument" where "documentId"=tcon."documentId") as profileImage, '
    + ' "firstName","lastName","jobTitle","contactId",tcon."mobileNo",tcon."email",tc."companyName",tc."channelId" as companyId, cd."detailDesc",case when tc."channelTypeId"=2 then \'Vendor\' when tc."channelTypeId"=3 then \'Distributor\' when tc."channelTypeId"=4 then \'Retailer\' end as channelType, '
    + ' (select concat(\'https://d35w8j22j5uly8.cloudfront.net\',"documentPath") from "tblDocument" where "documentId"=cd."channelLogoId") as companyLogo, '
    + '\'3e683a61-992b-47d2-8ecf-f88c9849f499\' as eventId,\'Confirmed\' as status, '
    + ' case when tc."channelTypeId"=2 then \'Vendor\' when tc."channelTypeId" in(3,4) then \'Buyer\' end as type, '
    + ' (select "country" from "tblCountry" where "countryId"=tc."countryId"),'
    + ' (select "isoCode" from "tblCountry" where "countryId"=tc."countryId"),(select string_agg("productCategorie",\',\') from "tblProductCategories" where "productCategorieId" in '
    + '(Select cpc."productCategorieId" from "tblContactInterestedProductCategory" cpc where cpc."contactId" = tcon."contactId"))as "IntrestedCategory", '
    
    + '(select string_agg("productCategorie",\',\') from "tblProductCategories" where "productCategorieId" in '
    + '(Select cpc."productCategorieId" from "tblContactProductCategorie" cpc where cpc."contactId" = tcon."contactId")) as "ExistingCategory", '
    
    + '(select "profileType" from "tblProfileType" where "profileTypeId"=tc."profileTypeId") as "ProfileType", '
    
    + '(select string_agg(pt."profileType",\',\') from "tblProfileType" pt where "profileTypeId" in (select "profileTypeId" from "tblChannelCustomerProfile" as ccp where ccp."channelId"=tc."channelId")) as "Retail_Customer_Profile", '
    
    + '(select "password" from "tblUser" where "contactId"=tcon."contactId") '
    
    + 'from "tblContact" tcon inner join "tblChannel" tc on tc."channelId"=tcon."channelId"	inner join "tblChannelDetail" cd on cd."channelId"=tcon."channelId" where tcon."contactId" in (:contactId)';	

    let result:any;

    if(contacts.length){
        result=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
            replacements: {
            contactId:contacts
            }
            });
    }

    return result;
}

async companyNameList(search: string) {
    return await this.channelRepository.findAll<Channel>({
        where:{
            companyName: { [Op.iLike]: '%' +search+ '%' },
            deletedAt : null,
            isActive : '1'
        },
        attributes: ['companyName', 'channelId', 'channelTypeId', 'isOEM'],
        include:[
            {
				model: Country,
				attributes:['country']
			},
            {
                model: this.channelDetailRepository,
                attributes: ['channelLogoId'],
                where: { deletedAt: null },
                required: true,
                include: [
                    {
                        model: this.mediaRepository,
                        as: 'channelLogo',
                        attributes:['documentUrl', 'documentPath'],
                        required: false
                    }
                ]
                },
                {
                    model: User,
                    attributes: ['userId', 'contactId'],
                    include: [{
                        model: UserRole,
                        attributes: ['roleId'],
                        where:{
                            roleId:1
                        },
                        required: true
                    }]
                }
        ],
        limit : 10
    });
}

async contactNameList(search: string) {
    return await this.contactRepository.findAll<Contact>({
        where:{
            [Op.or]:{
                firstName: { [Op.iLike]: '%' +search+ '%' },
                lastName: { [Op.iLike]: '%' +search+ '%' }
            },
            deletedAt : null,
            isActive : '1'
        },
        attributes: ['firstName', 'lastName', 'contactId', 'jobTitle'],
        include:[
            {
				model: Media,
				attributes:['documentUrl', 'documentPath']
			},
			{
				model: Country,
				attributes:['country']
			},
            {
                model: this.channelRepository,
                attributes: ['companyName', 'channelId', 'channelTypeId'],
                where: { deletedAt: null },
                required: true,
                }
        ],
        limit : 10
    });
}

async getContact(contactId){
       return await this.contactRepository.scope('contactInfo').findOne({
            where: { 
                contactId: contactId,
                deletedAt: null 
            },
            include:[
                {
                    model: Media,
                    attributes: ['documentPath', 'documentId', 'thumbnailPath'],
                    required:false,
                },
                {
                    model: ContactProductGroup,
                    where:{
                        deletedAt: null
                    },
                    include: [{
                        model: ProductGroup,
                        attributes: ['productGroup']
                    }],
                    required:false,
                },
                {
                    model: this.countryRepository,
                    attributes: ['country'],
                    required:false,
                },
                {
                    model: User,
                    attributes: ['contactId', 'userId'],
                    include: [{
                        model: UserRole,
                        attributes: ['roleId']
                    }]
                }
            ]
        })
}
public async getChannel(channelId: number): Promise<any> {

    try {
        let channel = await this.channelRepository.scope('channelDetail').findOne<Channel>({
            where:
            {
                channelId: channelId,
                deletedAt: null
            },
            include: [
				{
                    model: ChannelB2B,
                    as:'channelB2B',
                    attributes: ['channelB2BId', 'channelId', 'channelB2B'],
                    where: { deletedAt: null },
                    required: false,
                },
                { model: Country, as: 'country', where: { deletedAt: null }, required: false },
                { model: Address, as: 'regAddress'},
                { model: ProfileType.scope('channelDetail'), as: 'profileType', attributes: ['profileType'], where: { deletedAt: null }, required: false },
                {
					model: this.productGroupRepository,
					as: 'speciality',
					attributes: ['productGroupId', 'productGroup'],
					required: false,
					where: { isActive: '1' },
					through: { attributes: [], where: { deletedAt: null } }
				},
				{
					model: this.productCategoryRepository,
					as: 'productCategory',
					attributes: ['productCategorieId', 'productCategorie'],
					required: false,
					where: { isActive: '1' },
					order:'productCategorie',
					through: { attributes: [], where: { deletedAt: null } }
				},
				{
					model: ChannelAward,
					attributes: ['awardDetail', 'issuedBy', 'validUpto', 'isLifeTime'],
					where: { deletedAt: null },
					required: false
				}
            ]
        });
        if (!channel) return false;


        if (!channel.channelDetail) {
            await new Utility().lazyload(channel, 'channelDetail', {
                where: { deletedAt: null },
                required: false,
                attributes: ['channelLogoId', 'channelIconId', 'webSiteUrl', 'estYear', 'regYear', 'detailDesc', 'expectations', 'companyDesc', 'empCntRange', 'empCnt', 'empCntDSale', 'empCntISale', 'empCntDMar', 'empCntIMar', 'turnoverRange', 'turnover', 'revD', 'revI', 'deletedAt', 'turnoverPhyStore', 'turnoverOnStore', 'startup', 'storeCount', 'maturitylevel', 'keyretailers', 'keyresellers', 'keydistributors', 'no_of_retailers', 'no_of_resellers', 'no_of_skus', 'no_of_distributors', 'no_of_vendors', 'introvideoId', 'channelBannerId', 'phone1', 'mobileNo'],
                include: [{
                    model: Media,
                    as: 'channelLogo',
                    attributes: ['documentUrl', 'documentPath'],
                    where: { deletedAt: null },
                    required: false
                },
                {
                    model: Media,
                    as: 'channelIcon',
                    where: { deletedAt: null },
                    attributes: ['documentUrl'],
                    required: false
                },
                {
                    model: Media,
                    as: 'introvideo',
                    where: { deletedAt: null },
                    // attributes: ['documentUrl'],
                    required: false
                },
                {
                    model: Media,
                    as: 'channelBanner',
                    where: { deletedAt: null },
                    // attributes: ['documentUrl'],
                    required: false
                }
                ],
            });
        }

        if (!channel.serviceList) {
            await new Utility().lazyload(channel, 'serviceList', {
                through: { attributes: [], where: { deletedAt: null } },
                required: false,
                attributes: ['serviceListId', 'serviceList']
            });
        }

        if (!channel.channelLocation) {
            await new Utility().lazyload(channel, 'channelLocation', {
                where: { deletedAt: null },
                required: false,
                attributes: ['locationType'],
                include: [{
                    model: RegionCountryJCT,
                    where: { deletedAt: null },
                    required: false,
                    include: [
                        {
                            model: Country,
                            where: { deletedAt: null },
                            attributes: ['countryId', 'country', 'priority'],
                            required: false
                        }
                    ],
                }]
            });
        }

        if (!channel.channelBrand) {
            await new Utility().lazyload(channel, 'channelBrand', {
                where: { deletedAt: null },
                required: false,
                include: [{
                    model: Brand.scope('channelDetail'),
                    as: 'brand',
                    where: { deletedAt: null, isActive: '1' },
                    required: false,
                    include: [{
                        model: Media,
                        as: 'document',
                        where: { deletedAt: null },
                        required: false
                    }]
                }]
            });
        }
        if (!channel.contacts) {
            await new Utility().lazyload(channel, 'contacts', {
                where: { deletedAt: null, isActive: '1' },
                required: true,
                attributes: ['contactId', 'firstName', 'lastName', 'phone1', 'mobileNo', 'email', 'responsibilityId', 'linkedIn', 'jobTitle', 'googleplus', 'isDefaultContact', 'empId', 'isActive', 'deletedAt', 'documentId', 'countryId'],
                include: [
                    {
                        model: User.scope('channelDetail'),
                        where: { deletedAt: null },
                        required: true,
                        include: [
                            {
                                model: UserRole,
                                where: {
                                    roleId: Role.ROLE_ACCOUNT_USER
                                },
                                required: true,
                            }]
                    },
                    {
                        model: ContactSocialLink,
                        where: { deletedAt: null },
                        required: false,
                        include: [{
                            model: SocialSite.scope('channelDetail'),
                            where: { deletedAt: null },
                            required: false
                        }]
                    },
                    {
                        model: Media,
                        as: 'document',
                        // attributes: ['documentUrl'],
                        required: false,
                    },
                    {
                        model: this.productCategoryRepository.scope('contactProductInfom'),
                        as: 'IntProductCategories',
                        where: { deletedAt: null, isActive: "1" },
                        required: false,
                        through: { attributes: [], where: { deletedAt: null } }

                    },
                    { model: Country, as: 'country', where: { deletedAt: null }, required: false },
                ]
            });
        }
        if (!channel.channelSocial) {
            await new Utility().lazyload(channel, 'channelSocial', {
                where: { deletedAt: null },
                required: false,
                include: [{
                    model: SocialSite.scope('channelDetail'),
                    where: { deletedAt: null },
                    attributes: ['socialSite'],
                    required: false
                }]
            });
        }

        if (!channel.channelCertificate) {
            await new Utility().lazyload(channel, 'channelCertificate', {
                where: { deletedAt: null },
                attributes: ['cerificateTypeId', 'cerificateName', 'cerificateNumber', 'cerificateDesc'],
                required: false
            });
        }

        if (!channel.channelSubscription) {
            await new Utility().lazyload(channel, 'channelSubscription', {
                where: { isActive: '1' },
                required: false,
                include: [{
                    model: this.subscriptionRepository.scope('channelDetail'),
                    as: 'subscription',
                    where: { deletedAt: null, isActive: '1' },
                    required: false,
                    include: [{
                        model: SubscriptionPlan.scope('channelDetail'),
                        as: 'subscriptionPlan',
                        where: { deletedAt: null, isActive: '1' },
                        required: false
                    }],
                }]
            });
        }

        if (!channel.channelDocument) {
            await new Utility().lazyload(channel, 'channelDocument', {
                attributes: ['channelDocumentId', 'documentId', 'name', 'description'],
                where: { deletedAt: null },
                required: false,
                include: [{
                    attributes: ['documentId', 'documentPath', 'documentUrl'],
                    where: { deletedAt: null, documentTypeId:2 },
                    model: this.mediaRepository,
                    required: true,
                }]
            });
        }

        channel.toJSON();

        channel['channelKeyRetailer'] = await this.channelKeyPartnerService.getKeyRetailers(+channel.channelId, appconstant.CHANNEL_TYPE_RETAILER);
        channel['channelKeyReseller'] = await this.channelKeyPartnerService.getKeyRetailers(+channel.channelId, appconstant.CHANNEL_TYPE_RESELLER);
        channel['channelKeyDistributor'] = await this.channelKeyPartnerService.getKeyRetailers(+channel.channelId, appconstant.CHANNEL_TYPE_DISTRIBUTOR);

        let custquery = 'select tpt."profileType", tptc."channelTypeId" from "tblChannelCustomerProfile" tcp inner join "tblProfileTypeClassification" tptc on tcp."profileTypeClassificationId"=tptc."profileTypeClassificationId" inner join "tblProfileType" tpt on tpt."profileTypeId"=tptc."profileTypeId" where tcp."channelId"=:channelId and tcp."deletedAt" is null'

        channel['customerProfile'] = await await this.sequelizeInstance.query(custquery, {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
                channelId: +channel.channelId
            }
            });

        if (channel.channelTypeId == appconstant.CHANNEL_TYPE_DISTRIBUTOR || channel.channelTypeId == appconstant.CHANNEL_TYPE_RETAILER) {
            channel['sampleOrderCount'] = await this.getSampleOrderCount(channelId);
            channel['sampleEvaluationCount'] = await this.getSampleEvaluationCount(channelId);
        }

        channel['productCount'] = await this.productRepository.count({
            where: {
                channelId: channelId,
                isactive: '1',
                deletedat: null
            }
        });

        channel['certificateType'] = await this.channelCertificateTypeRepository.findAll({
            where: { deletedAt: null }
        });

        let query='select "productCategorieId", "productCategorie" from "tblProductCategories" where "productCategorieId" in(select "productCategorieId" from "tblInterestedProductCategory" where "channelId"=:channelId and "deletedAt" is null) order by "productCategorie"';
        
        let userProductCat= await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
                channelId: channelId
            }
            });
                let companyImages= await this.companyMediaImagesRepository.findAll<CompanyMediaImages>({
                    where: {
                        channelId: +channel.channelId,
                        deletedAt: null
                    },
                    include: [{
                        model: Media,
                        where: { 
                            documentTypeId:1,
                            deletedAt: null
                        },
                        required: true
                    }],
                    order: [['companyMediaId', 'DESC']]
                });
                let resultObject: any = await this.channelDetailOutBound.channelDetail(channel, userProductCat, companyImages);
        return resultObject;
    } catch (e) {
        console.log("Error+ ", e);
        throw new InternalServerErrorException();
    }
}
	
async getSampleOrderCount(channelId: number): Promise<number> {
    try {
        return await SampleOrder.count(
            {
                where: {
                    buyerchannelid: channelId,
                    statusid: { [Op.ne]: appconstant.ORDER_STATUS_CANCELED }
                }
            });
    } catch (e) {
        console.log("Error+ ", e);
        throw new InternalServerErrorException();
    }
}

async getSampleEvaluationCount(channelId: number): Promise<number> {
    try {
        return await ProductEvaluation.count(
            {
                where: {
                    channelid: channelId,
                    deletedat: null
                }
            });
    } catch (e) {
        console.log("Error+ ", e);
        throw new InternalServerErrorException();
    }
}

//OldChannel search
async oldChannelsList(userId){

    let cquery='select "typeId" from "tblAdminUserChanneltype" where "adminUserId"=:userId';
    let chtypes=await this.sequelizeInstance.query(cquery, {
        type: QueryTypes.SELECT,
        replacements: {
            userId: userId		
        }
    }).then((results) => {
        return results.map(x => +x.typeId);
    });      

    cquery='select "countryId" from "tblAdminUserCountries" where "adminUserId"=:userId';
    let countries=await this.sequelizeInstance.query(cquery, {
        type: QueryTypes.SELECT,
        replacements: {
            userId: userId			
        }
    }).then((results) => {
        return results.map(x => +x.countryId);
    });      

    let channelFilter = {        
        isActive: '1',
        deletedAt: null            
    };

    if (countries&&countries.length>0) {
        channelFilter['countryId'] = { [Op.in]: countries }
    }

    if (chtypes&&chtypes.length>0) {
        channelFilter['channelTypeId'] = { [Op.in]: chtypes }
    }

    return await this.oldChannelRepository.findAll<OldChannel>({
        where: channelFilter,
        attributes:[["companyName","name"], ["oldChannelId", "id"]],
        include:[{
            model:OldContact,
            as:'oldContacts',
            where: { deletedAt: null, isDefaultContact:'1' },
            required:false,
            attributes:["firstName","lastName", "oldContactId"],
        }]
    })
}

async oldChannels(data){
    // let user = userData;
    let channelFilter=data.filter;
    // let contactId=user.contactId;
    // let channelId=user.channelId;
    // let typeid=user.channelTypeId;
    let userId=1; //user.userId;
    let filterch=[];let channeltype=[2,3,4];
    let aggregation={};let channelquery;

    // else if(typeid==3)
    // channeltype=[2,3,4];
    // else if(typeid==4)
    // channeltype=[2,3];
    
    let order=data.order;			
    channelquery='select "oldChannelId" as channel_id from "tblOldChannel" where "channelTypeId" in(:channelTypeId) and "isActive"=\'1\' and "deletedAt" is null'; // "updatedAt" '+data.order+' ' ;
                 
            
    let result= await this.sequelizeInstance.query(channelquery, {
    type: QueryTypes.SELECT,
    raw: true,
    replacements: {		
    channelTypeId:channeltype
    }
    }).then(x=>{						
        return  _.uniqBy(x,'channel_id');
    })

    

    if(data.search)
    {			
        let  searchchannellist=await this.sequelizeInstance.query('select "oldChannelId" as channel_id from "tblOldChannel" where "oldChannelId" = (:search)', {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
            search: +data.search,
            }
            })
        let chl=searchchannellist.map(x => x.channel_id);
        result = result.filter(ch => {return chl.includes(ch.channel_id);})			
    }		
    if(channelFilter.typeId && result)
    {
        let chlist = result.map(x => +x.channel_id);		
        let ctype='select array_agg("oldChannelId") as ch from "tblOldChannel" where "channelTypeId" in(:channeltypeid) and "oldChannelId" in(:channelid)';
    await this.sequelizeInstance.query(ctype, {
        type: QueryTypes.SELECT,
    raw: true,
    replacements: {
    channelid:chlist,
    channeltypeid:+channelFilter.typeId
    }
    }).then((result) => {
        if(result[0].ch)
    filterch.push(...result[0].ch);
    });
    result = result.filter(ch => {return filterch.includes(ch.channel_id);})
    }
    
    if((channelFilter.turnOver||channelFilter.brandId||channelFilter.locationId||channelFilter.countryId||channelFilter.specialityId||channelFilter.productCategoryId||channelFilter.retailerprofileId)&& result) {

    let chlist = result.map(x => +x.channel_id);		
    
    if(channelFilter.turnOver)
    {
    let query;
        if(channelFilter.turnOver=='<1M€')
        {
            query='select array_agg("oldChannelId") as ch from "tblOldChannelDetail" where "oldChannelId" in(:channelid) and case when "turnoverRange"=\'<1M€\' or cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)<1 or "turnover"<1 then cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)<1 or "turnoverRange"=\'<1M€\' or "turnover"<1 end';
        }
        else if(channelFilter.turnOver=='1-10M€')
        {					
            query='select  array_agg("oldChannelId") as ch from "tblOldChannelDetail" where "oldChannelId" in(:channelid) and case when "turnoverRange"=\'1-10M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\'))as int)>1 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<10) or ("turnover">1 and "turnover"<10) then "turnoverRange"=\'1-10M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\'))as int)>1 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<10) or ("turnover">1 and "turnover"<10) end';
        }
        else if(channelFilter.turnOver=='10-100M€')
        {
            query='select  array_agg("oldChannelId") as ch from "tblOldChannelDetail" where "oldChannelId" in(:channelid) and "turnoverRange"!=\'>100M€\' and case when "turnoverRange"=\'10-100M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\')) as int)>10 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<=100) or ("turnover">10 and "turnover"<=100) then "turnoverRange"=\'10-100M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\')) as int)>10 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<=100) or ("turnover">10 and "turnover"<=100) end';
        }
        else if(channelFilter.turnOver=='>100M€')
        {
            query='select array_agg("oldChannelId") as ch from "tblOldChannelDetail" where "oldChannelId" in(:channelid) and case when "turnoverRange"=\'>100M€\' or cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)>100 or "turnover">100 then cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)>100 or "turnoverRange"=\'>100M€\' or "turnover">100 end';
        }
        
    await this.sequelizeInstance.query(query, {
        type: QueryTypes.SELECT,
    raw: true,
    replacements: {
    channelid:chlist
    //channeltypeid:channelFilter.typeId
    }
    }).then((result) => {

        if(result[0].ch){
            filterch = [];
            filterch.push(...result[0].ch);
        }else
        filterch = [];
    });
    }

    if(channelFilter.brandId) {
    let brand='select array_agg("oldChannelId") as ch from "tblOldChannelBrand" where "brandId" in(:brandid) and "oldChannelId" in(:channelid)';
    await this.sequelizeInstance.query(brand, {
        type: QueryTypes.SELECT,
    raw: true,
    replacements: {
    channelid:chlist,
    brandid:channelFilter.brandId
    }
    }).then((result) => {
        if(result[0].ch){
            filterch = [];
    filterch.push(...result[0].ch);
    }else
    filterch = [];

    });
    }
    if(channelFilter.productCategoryId) {
    await this.sequelizeInstance.query('select array_agg("oldChannelId") as ch from "tblOldChannelProductCategorie" cpc where cpc."productCategorieId" in(:productcatid) and "oldChannelId" in(:channelid) ', {
        type: QueryTypes.SELECT,
    raw: true,
    replacements: {
    channelid:(filterch.length>0)?filterch:chlist,
    productcatid:channelFilter.productCategoryId
    }
    }).then((result) => {
        if(result[0].ch){
            filterch = [];
    filterch.push(...result[0].ch);
        }else
        filterch = [];

    });
    }
    if(channelFilter.retailerprofileId) {
    await this.sequelizeInstance.query('select array_agg("oldChannelId") as ch from "tblOldChannelCustomerProfile" where "profileTypeId" in(:retailprofileid) and "oldChannelId" in(:channelid)', {
        type: QueryTypes.SELECT,
    raw: true,
    replacements: {
    channelid:(filterch.length>0)?filterch:chlist,
    retailprofileid:channelFilter.retailerprofileId
    }
    }).then((result) => {
        if(result[0].ch){
            filterch = [];
    filterch.push(...result[0].ch);
        }else
        filterch = [];

    });
    }

    if(channelFilter.countryId) {
    await this.sequelizeInstance.query('select array_agg("oldChannelId") as ch from "tblOldChannel" where "countryId" in(:countryid) and "oldChannelId" in(:channelid)', {
        type: QueryTypes.SELECT,
    raw: true,
    replacements: {
    channelid:(filterch.length>0)?filterch:chlist,
    countryid:channelFilter.countryId
    }
    }).then((result) => {
        if(result[0].ch){
            filterch = [];
    filterch.push(...result[0].ch);
        }else
        filterch = [];
    });
    }
    if(channelFilter.specialityId) {
    await this.sequelizeInstance.query('select array_agg(distinct "oldChannelId") as ch from "tblOldContact" where "oldContactId" in(Select "oldContactId" from "tblContactInterestedProductCategory" where "productCategorieId" in(:specialityid)) and "oldChannelId" in(:channelid)', {
        type: QueryTypes.SELECT,
    raw: true,
    replacements: {
    channelid:(filterch.length>0)?filterch:chlist,
    specialityid:channelFilter.specialityId
    }
    }).then((result) => {
        if(result[0].ch){
            filterch = [];
    filterch.push(...result[0].ch);
        }else
        filterch = [];
    });
    }
    if(channelFilter.locationId) {
    await this.sequelizeInstance.query('select array_agg("oldChannelId") as ch from "tblOldChannel" where "countryId" in(:locationid) and "oldChannelId" in(:channelid)', {
        type: QueryTypes.SELECT,
    raw: true,
    replacements: {
    channelid:(filterch.length>0)?filterch:chlist,
    locationid:channelFilter.locationId
    }
    }).then((result) => {
        if(result[0].ch){
            filterch = [];
    filterch.push(...result[0].ch);
        }else
        filterch = [];
    });
    }
    // if(!channelFilter.connectionStatusId)
        result = result.filter(ch => {return filterch.includes(ch.channel_id);})
    }

    
    let total = result?result.length:0;
    let final=[];
            
    if(result){		

        let ordered = await this.sequelizeInstance.query('select "oldChannelId" from "tblOldChannel" where "oldChannelId" in(:channelid)  order by "companyName" asc', {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
            channelid:result.map(x => x.channel_id),
            }						
        }).then(res => {
            return res.map(x=>x.oldChannelId)
            });

        //   result.sort(function(a, b){  
        // 	return ordered.indexOf(a) - ordered.indexOf(b);
        //   });

        result = await this.mapOrder(result, ordered, 'channel_id');

        result=result.slice(data.page).slice(0,data.limit);
        
        
    
     for(let i=0;i<result.length;i++)
     {
         let status='Non Suggestion';		 
         let mscore=null;let relation_status=null;let isConfidential=1;let channeljct=null;let transToContact=null;let note=null;
        // console.log("status:",status);
        let companyquery;

        try{						
            companyquery='Select '+mscore+' as matchingScore,'+relation_status+' as relation_status,\'Non Member\' as status,'+isConfidential+' as isConfidential,'+channeljct+' as channel_jct_id,'+transToContact+' as transToContact,'+note+' as note,ch."oldChannelId" as cid,case when chd."turnover" is null then chd."turnoverRange" when chd."turnover" is not null then cast(chd."turnover" as text) end as turnover,ch."companyName" as companyname,ch."countryId" as countryid, tc."country" as country, tc."isoCode", pt."profileType" as profiletype, cht."channelType" as channeltype,ch."channelTypeId" as channeltypeid,'
            +'chd."detailDesc" as detaildesc, chd."webSiteUrl" as websiteurl, chd."startup",(select array_to_json(array_agg(row_to_json(a))) from (select SUBSTRING ( cast(chd."maturitylevel" as TEXT) ,2 , length(cast(chd."maturitylevel" as TEXT))-2) as item_name)a) as looking_startups, '

            //+'(select doc."documentPath" from "tblDocument" as doc  where doc."documentId"='+result[i].introVideo+' ) as introvideo, '

            +'(select distinct channelgroupid as favorite from "tblproductwishlist" fav where fav."userid"='+userId+' and fav."channelid"=ch."oldChannelId" and "productid" is null and "deletedat" is null fetch first row only), '
            +'(Select td."documentPath" as logo from "tblOldChannelDetail" chd inner join "tblDocument" td on td."documentId" in(chd."channelLogoId") where chd."oldChannelId"=ch."oldChannelId"), '
    
            +'(select array_to_json(array_agg(row_to_json(c))) from (Select td."documentPath",con."isDefaultContact",con."firstName",con."lastName",con."jobTitle" from "tblOldContact" con left join "tblDocument" td on td."documentId" in(con."documentId") where con."oldChannelId"=ch."oldChannelId")c)as contacts, '

            +'(select array_to_json(array_agg(row_to_json(x))) from (Select pc."productCategorie" as item_name from "tblOldChannelProductCategorie" cpc inner join "tblProductCategories" pc on pc."productCategorieId" = cpc."productCategorieId" where cpc."oldChannelId"=ch."oldChannelId")x) as company_cat, '
    
            +'(select array_to_json(array_agg(row_to_json(z))) from (Select pt."profileType" as item_name from "tblProfileType" pt where "profileTypeId" in (select "profileTypeId" from "tblOldChannelCustomerProfile" as ccp where ccp."oldChannelId"=ch."oldChannelId"))z) as retail_customer_profile '
            
    
            +'from "tblOldChannel" ch inner join "tblOldChannelDetail" chd on chd."oldChannelId" = ch."oldChannelId" inner join "tblCountry" tc on tc."countryId" = ch."countryId" '
            +'inner join "tblProfileType"pt on pt."profileTypeId" = ch."profileTypeId" inner join "tblChannelType" as cht on cht."channelTypeId" = ch."channelTypeId" where ch."oldChannelId" in(:channelid)';
         
    let res= await this.sequelizeInstance.query(companyquery, {
        type: QueryTypes.SELECT,
            raw: true,
            replacements: {
             channelid:result[i].channel_id,
            // contactid:result[i].contact_id,
            // status:result[i].status,
            // note:(result[i].note=='')?null:result[i].note
             }
            });

            final.push(...res);

        }
        catch(e)
        {
            console.log("error:",e);
            // console.log("status:",result[i])

        }
    }		
    
}else
result=[];


// final.map(x => {		
// 	//console.log("typeid:",x.channeltypeid);
// 	if((x.relation_status==null||x.relation_status==1)&&((x.isconfidential!=1 && (viewId==8||viewId==9||viewId==10||viewId==6||viewId==7) && planId==1&&(typeid==2||(typeid==3&&x.channeltypeid==4))) 
// 	||(planId==7&&x.isconfidential!=1&&(typeid==2||(typeid==3&&x.channeltypeid==4))&&(viewId==6||viewId==7))
// 	||(planId==7&&x.isconfidential!=1&&data.page!=1&&(typeid==2||(typeid==3&&x.channeltypeid==4))&&(viewId==8||viewId==9||viewId==10))))
// 	{
// 		x.logo=null;
// 		x.companyname='XXXXXXXXXXXXXXXXX';
// 		x.country='XXXXXXXXXXXXXXXXX';
// 		x.contacts.map(c => {
// 			c.firstName='XXXXXXXXXXXXXXXXX';
// 			c.lastName='XXXXXXXXXXXXXXXXX';
// 			c.jobTitle='XXXXXXXXXXXXXXXXX';
// 		})
// 	}
// })

    let resultformat={
    results:final,
    aggregation:aggregation,
    total:total
    }

    return resultformat;
}


//get old channel detail view
public async getOldChannelDetailView(oldChannelId){

    let result : any;

    let oldchanneldetails = await this.getOldChannelDetails(oldChannelId);
    let facts = await this.getOldChannelFacts(oldChannelId);

    let socialLinks = [];
    
    if(oldchanneldetails.channelSocial?.length){
        socialLinks = oldchanneldetails.channelSocial.map(x => {
            return {social : x.socialSite?.socialSite, link: x.channelSocialLink}
        })
    }

    result = {
        ...oldchanneldetails,
        ...facts,
        ...(socialLinks.length && {socialLinks: socialLinks})
    }

    return result;

}

//get channel details
public async getOldChannelDetails(oldChannelId : number) : Promise<any> {
    
    let result : any;

    let query = `select (select json_build_object('addressId', "oldAddressId", 'address', "address", 'city', "city",'countryId', "countryId",'postalCode', "postalCode") as "regAddress" from "tblOldAddress" where "oldAddressId"=ch."regAddressId"),

     (select (array_agg(to_json(q))) from (select 
        ocs.*, to_json(s.*) "socialSite" from "tblOldChannelSocialLink" ocs inner join "tblSocialSite" s on s."socialSiteId"= ocs."socialSiteId" where ocs."oldChannelId"= ch."oldChannelId" )q) as "channelSocial",

    (select "documentPath" from "tblDocument" where "documentId"=chd."channelLogoId") as "channelLogo",
    (select "documentPath" from "tblDocument" where "documentId"=chd."introvideoId") as "introVideo",
    (select "documentPath" from "tblDocument" where "documentId"=chd."channelBannerId") as "banner",
    case when ch."channelTypeId"=2 then 'VENDOR'
    when ch."channelTypeId"=3 then 'DISTRIBUTOR'
    when ch."channelTypeId"=4 then 'RETAILER'
    when ch."channelTypeId"=5 then 'RESELLER' end as "channelType",
    cc."country", cc."phoneCode", cc."isoCode", 
    ch."companyName",ch."companyMail", "detailDesc","webSiteUrl","phone1" as  "phoneno","estYear" ,"mobileNo", "empCntRange", "empCnt",
     "turnover", "turnoverRange", "startup", "storeCount", "introvideoId", "channelBannerId", ch."channelTypeId",ch."oldChannelId" from "tblOldChannelDetail" as chd
    inner join "tblOldChannel" as ch on ch."oldChannelId"=chd."oldChannelId" inner join "tblCountry" as cc on cc."countryId"=ch."countryId" where ch."oldChannelId"=:oldChannelId`;

    try{

        result= await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                 oldChannelId: oldChannelId,
                // contactid:result[i].contact_id,
                // status:result[i].status,
                // note:(result[i].note=='')?null:result[i].note
                 }
                }).then(res => res[0]);
    
    }catch(e){
        console.log("error", e);
    }

     return result;

}

//getting factsdetails
public async getOldChannelFacts(oldChannelId : number) : Promise<any> {
    
    let result : any;

    let query = `select 
    "estYear" , "empCntRange", "empCnt", "maturitylevel",
     "turnover", "turnoverRange", "startup", "storeCount", ch."channelTypeId",ch."oldChannelId" from "tblOldChannelDetail" as chd
    inner join "tblOldChannel" as ch on ch."oldChannelId"=chd."oldChannelId" inner join "tblCountry" as cc on cc."countryId"=ch."countryId" where ch."oldChannelId"=:oldChannelId`;

    try{

        result= await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                 oldChannelId: oldChannelId,
                // contactid:result[i].contact_id,
                // status:result[i].status,
                // note:(result[i].note=='')?null:result[i].note
                 }
                }).then(res => res[0]);
    
    }catch(e){
        console.log("error", e);
    }

     return result;

}

//get oldchannel products
public async getOldProducts(oldChannelId){
    let result = {productGroup : []};

    let query = `select pg."productGroupId" as groupId,pg."productGroup" as groupname,pc."productCategorieId" as childId,pc."productCategorie" as childname,cast(false as boolean) as status from "tblProductGroup" as pg inner join
    "tblProductCategories" as pc on pg."productGroupId"=pc."productGroupId" where pc."productCategorieId" in(select "productCategorieId" from "tblOldChannelProductCategorie" where "oldChannelId"=:oldChannelId)`;

    try{

    result.productGroup= await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                 oldChannelId: oldChannelId,
                // contactid:result[i].contact_id,
                // status:result[i].status,
                // note:(result[i].note=='')?null:result[i].note
                 }
                }).then(res => res);
    
    }catch(e){
        console.log("error", e);
    }

     return result;
}

//getting old channel brands
public async getOldChannelBrand(oldChannelId) {
    let result = {brand: []};

    let query = `select "brandName","id",json_build_object('document', doc.*) as document, doc."documentId" as doc_id,doc."documentPath" as documentPath from "tblBrand" as br inner join "tblDocument" as doc on doc."documentId"=br."documentId" where br."id" in
    (select "brandId" from "tblOldChannelBrand" where "oldChannelId"=:oldChannelId)`;

    try{

        result.brand= await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                 oldChannelId: oldChannelId,
                // contactid:result[i].contact_id,
                // status:result[i].status,
                // note:(result[i].note=='')?null:result[i].note
                 }
                }).then(res => res);
    
    }catch(e){
        console.log("error", e);
    }

     return result;
}

//getting old channel profile and speciality
public async channelProfileOld(oldChannelId){


    let tradeinfo = await OldChannel.findAll<OldChannel>({
        attributes:['oldChannelId', 'profileTypeId'],                    
        where:{
            oldChannelId:oldChannelId
        },
        include: [
            {
                model:OldChannelCustomerProfile,
                as:'channelCustomerProfile',
                attributes: ['profileTypeId','profileTypeClassificationId'],
                // include: [
                //     {
                //         model:this.profileTypeRepository,
                //         attributes:['profileType']
                //     }
                // ]
                where:{
                    deletedAt: null
                }

            },
            {
                model:OldChannelSpeciality,
                as:'channelSpeciality',
                where: {deletedAt: null},
                required: false,
                include: [
                    {
                        model:this.productGroupRepository,
                        as:'productGroup',
                        attributes:[['productGroupId','id'],['productGroup','name']]
                    }
                ]
            },

        ]

    })

        let query = 'select * from "tblProductGroup" where "deletedAt" is null and "isActive"=\'1\'';

		let specialityOptions = await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT
		}).then(res => res);

        tradeinfo = Object.assign(tradeinfo, {specialityOptions: specialityOptions});
		return tradeinfo;

}



public async specialityOld(data){

    let tradeinfo = {};

    let query = 'select * from "tblProductGroup" where "deletedAt" is null and "isActive"=\'1\'';
		let specialityOptions = await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT
		}).then(res => res);

	let final = Object.assign(tradeinfo, {specialityOptions: specialityOptions});
    return final;
}

//oldChannel initial Profile
public async getOldInitialProfile(data){
    let user = data;
    // console.log('user', user);
    let distProfile;
let channelTypeId=user.channelTypeId;
// let businessType=user.businessType;
// let isOEM=user.channel.isOEM;
let channelId=+user.oldChannelId;
let type=[];
let btype=[];

console.log('userdata', data);

// if(businessType==1)
//     btype=[1,3];
// else if(businessType==2)
//     btype=[2,3];
// else if(businessType==3)
//     btype=[1,2,3];
// else	
//     btype=[-1]

// if((channelTypeId==2||channelTypeId==3)&&businessType==1)
//    type=[4,5];	
// else if((channelTypeId==2||channelTypeId==3)&&businessType==2)	
//     type=[4, 5];			
// else if((channelTypeId==2||channelTypeId==3)&&businessType==3)	
//     type=[4,5];    			
// else if(channelTypeId==4)
//     type=[4];
// else if(channelTypeId==5)
//     type=[5];
// else	
//     type=[-1];
    
//Code for get productgroup and its categories of company for company profile addnew
let query='select pg."productGroupId" as groupId,pg."productGroup" as groupname,pc."productCategorieId" as childId,pc."productCategorie" as childname,cast(false as boolean) as status from "tblProductGroup" as pg inner join' 
    +' "tblProductCategories" as pc on pg."productGroupId"=pc."productGroupId" where pg."deletedAt" is null and pg."productGroup"!=\'\'';
  
    let productGroup=await this.sequelizeInstance.query(query, {
        type: QueryTypes.SELECT,  
        // replacements: {
        //     businessType: btype     
        // }         
        }).then(results => {        
           return results;
        });

//     // let productGroup = await this.productGroupRepository.scope('productInfo').findAll({
    //     where: {
    //         deletedAt:null
    //     },
    //     include: [
    //         {
    //           model:this.productCatRepository.scope('productCategoryInfo'),                                      
    //           where: { deletedAt: null},
    //           //required:true
    //         }],
    // });
    
    
    // //Code for get productBrand of company for company profile addnew
    let url='https://d35w8j22j5uly8.cloudfront.net/';//process.env.CDN_URL;
  query ='select "brandName" as name,"id",doc."documentId" as doc_id,concat(\''+url+'\',doc."documentPath") as documentPath from "tblBrand" as br inner join "tblDocument" as doc on doc."documentId"=br."documentId" where br."deletedAt" is null';

  let brand=await this.sequelizeInstance.query(query, {
    type: QueryTypes.SELECT,        
    }).then(results => {        
       return results;
    });
    // let productBrand = await this.brandRepository.scope('basicInfom').findAll({
    //     where: {
    //         deletedAt:null
    //     },
        
    // });

    // //Code for get profile data of company for company profile addnew

    if(channelTypeId == 4 || channelTypeId ==5){
        query='select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
        +'when ptc."channelTypeId"=2 then \'vendor\' when ptc."channelTypeId"=5 then \'Reseller\' end as groupname,pt."profileType" as childname,'
            + 'pt."profileTypeId" as childId,cast(false as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId"';
    }else{
        query='select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
        +'when ptc."channelTypeId"=2 then \'vendor\' when ptc."channelTypeId"=5 then \'Reseller\' end as groupname,pt."profileType" as childname,'
            + 'pt."profileTypeId" as childId,cast(false as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId"';
    }
    let profileList=await this.sequelizeInstance.query(query, {
        type: QueryTypes.SELECT,           
        replacements: {
            // typeId: type,
            //businessType: (!isOEM||isOEM==3)?[1,2,3]:isOEM
        }
        }).then(results => {        
           return results;
        });
    if(channelTypeId==3)
    {
        query='select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
        +'when ptc."channelTypeId"=2 then \'vendor\' when ptc."channelTypeId"=5 then \'Reseller\' end as groupname,pt."profileType" as childname,'
        +'pt."profileTypeId" as childId,cast(false as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId";';
      
        distProfile=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,           
            replacements: {
                // typeId: [3]
            }
            }).then(results => {        
               return results;
            });
    }

    // let profileList = await this.profileTypeRepository.scope('profileTypeInfom').findAll({
    //     where: {
    //         deletedAt:null
    //     },
    //     include: [
    //         {
    //           model:this.profileTypeClassiRepository.scope('profileTypeClassificationInfom'),
    //           where: { deletedAt: null},//channelTypeId:[3,2]},
    //           required:true
    //         }],
    // });   

    //Code for get registration number of company for company profile addnew

query = 'select "registrationcertificate","isVat" from "tblCountry" as cty inner join "tblOldChannel" as ch on ch."countryId"=cty."countryId" where ch."oldChannelId"=:channelId';
  
  let country=await this.sequelizeInstance.query(query, {
  type: QueryTypes.SELECT,
  replacements: {
    channelId: channelId     
}
  }).then(results => {        
     return results;
  }); 



    let result={
        productGroup:productGroup,
        productBrand:brand,
        profileList:profileList,
        distProfile:distProfile,
        country:country
    }

    return result;    
}


//saving oldchanneldetails
public async editBasicCompanyOld (requestObject) : Promise<any>{

            let companyProfile: any = '';
			let companyProfileJson: any = '';
			let user = requestObject.user;

			
			console.log('log', user);
			// if (realUser && realUser.channel && +realUser.channel.channelTypeId === +appconstant.CHANNEL_TYPE_HUB) {
				// before update verify if the company name already exist

				let result = await this.oldChannelRepository.findOne({
					where: {
						companyName: requestObject.company_name.trim(),
						oldChannelId: {
							[Op.not]: +user.oldChannelId
						}
					}
				});
				if (result) {
					throw new ProcessException('channel:profile:companyNameAlreadyExist');
				}

			// }
			// return true;
			try {
				let result: any = '';
                let compQuery = `select (select json_build_object('addressId', "oldAddressId", 'address', "address", 'city', "city",'countryId', "countryId",'postalCode', "postalCode") as "regAddress" from "tblOldAddress" where "oldAddressId"=ch."regAddressId"),

                (select (array_agg(to_json(q))) from (select 
                   ocs.*, to_json(s.*) "socialSite" from "tblOldChannelSocialLink" ocs inner join "tblSocialSite" s on s."socialSiteId"= ocs."socialSiteId" where ocs."oldChannelId"= ch."oldChannelId" )q) as "channelSocial",
           
               (select "documentPath" from "tblDocument" where "documentId"=chd."channelLogoId") as "channelLogo",
               (select "documentPath" from "tblDocument" where "documentId"=chd."introvideoId") as "introVideo",
               (select "documentPath" from "tblDocument" where "documentId"=chd."channelBannerId") as "banner",
               case when ch."channelTypeId"=2 then 'VENDOR'
               when ch."channelTypeId"=3 then 'DISTRIBUTOR'
               when ch."channelTypeId"=4 then 'RETAILER'
               when ch."channelTypeId"=5 then 'RESELLER' end as "channelType",
               cc."country", cc."phoneCode", cc."isoCode", 
               ch."companyName",ch."companyMail", "detailDesc","webSiteUrl","phone1" as  "phoneno","estYear" ,"mobileNo", "empCntRange", "empCnt",
                "turnover", "turnoverRange", "startup", "storeCount", "introvideoId", "channelBannerId", ch."channelTypeId",ch."oldChannelId" from "tblOldChannelDetail" as chd
               inner join "tblOldChannel" as ch on ch."oldChannelId"=chd."oldChannelId" inner join "tblCountry" as cc on cc."countryId"=ch."countryId" where ch."oldChannelId"=:oldChannelId`;

				companyProfile = await this.sequelizeInstance.query(compQuery, {
                            type: QueryTypes.SELECT,
                            raw: true,
                            replacements: {
                            oldChannelId: +user.oldChannelId,
                            // contactid:result[i].contact_id,
                            // status:result[i].status,
                            // note:(result[i].note=='')?null:result[i].note
                            }
                            }).then(res => res[0]);
                

	            console.log('companyProfile', companyProfile);
				if (companyProfile) {
					companyProfileJson = companyProfile; //.toJSON();
					result = await this.sequelizeInstance.transaction(async transaction => {
						let comapnyLogoId: any = null;
						let comapnyRegAddressId: any = null;
	
						// update company registration address to address table
						if (requestObject.reg_address_id) {
							await OldAddress.update({
								address: requestObject.reg_address,
								city: requestObject.reg_city,
								countryId: requestObject.reg_country_id,
								postalCode: requestObject.reg_postal_code,
							},
								{
									where: { oldAddressId: companyProfile.regAddress.addressId },
									returning: true, transaction,
								});
							comapnyRegAddressId = companyProfile.regAddress.addressId;
						}
	
						// update channel information's address to channel table
						let companyInfoUpdate: any = {
							companyMail: requestObject.company_email || null,
							regAddressId: comapnyRegAddressId,
							countryId: +requestObject.reg_country_id,
							companyName : requestObject.company_name.trim()
						};
	
						await this.oldChannelRepository.update(companyInfoUpdate,
							{
								where: { oldChannelId: companyProfile.oldChannelId }, returning: true, transaction,
							});
	
						if (requestObject.social_linkedin_id) {
							// update channel social linkedin profile details to channel social link table
							await OldChannelSocialLink.update({
								socialSiteId: appconstant.SOCIAL_SITE_TYPE_LINKEDIN,
								oldChannelId: companyProfile.oldChannelId,
								channelSocialLink: requestObject.social_linkedin_url
							},
								{
									where: { oldChannelSocialLinkId: requestObject.social_linkedin_id },
									returning: true, transaction,
								});
						} else {
							// save channel social linkedin profile details to channel social link table
							if (requestObject.social_linkedin_url && Object.keys(requestObject.social_linkedin_url).length > 0) {
								await OldChannelSocialLink.create({
									socialSiteId: appconstant.SOCIAL_SITE_TYPE_LINKEDIN,
									oldChannelId: companyProfile.oldChannelId,
									channelSocialLink: requestObject.social_linkedin_url
								},
									{
										transaction
									});
							}
						}
	
						// save channel social google plus profile details to channel social link table
						if (requestObject.social_googleplus_id) {
							// update channel social google plus profile details to channel social link table
							await OldChannelSocialLink.update({
								socialSiteId: appconstant.SOCIAL_SITE_TYPE_GOOGLEPLUS,
								oldChannelId: companyProfile.oldChannelId,
								channelSocialLink: requestObject.social_googleplus_url
							},
								{
									where: { oldChannelSocialLinkId: requestObject.social_googleplus_id },
									returning: true, transaction,
								});
						} else {
							// save channel social google plus profile details to channel social link table
							if (requestObject.social_googleplus_url && Object.keys(requestObject.social_googleplus_url).length > 0) {
								await OldChannelSocialLink.create({
									socialSiteId: appconstant.SOCIAL_SITE_TYPE_GOOGLEPLUS,
									oldChannelId: companyProfile.oldChannelId,
									channelSocialLink: requestObject.social_googleplus_url
								},
									{
										 transaction
									});
							}
						}
	
						if (requestObject.social_instagram_id) {
							// update channel social instagram profile details to channel social link table
							await OldChannelSocialLink.update({
								oldChannelId: companyProfile.oldChannelId,
								channelSocialLink: requestObject.social_instagram_url
							},
								{
									where: { oldChannelSocialLinkId: requestObject.social_instagram_id },
									returning: true, transaction,
								});
						} else {
							// save channel social instagram profile details to channel social link table
							if (requestObject.social_instagram_url && Object.keys(requestObject.social_instagram_url).length > 0) {
								await OldChannelSocialLink.create({
									socialSiteId: appconstant.SOCIAL_SITE_TYPE_INSTAGRAM,
									oldChannelId: companyProfile.oldChannelId,
									channelSocialLink: requestObject.social_instagram_url
								},
									{
										transaction
									});
							}
						}

						if (requestObject.social_twitter_id) {
							// update channel social twitter profile details to channel social link table
							await OldChannelSocialLink.update({
								oldChannelId: companyProfile.oldChannelId,
								channelSocialLink: requestObject.social_twitter_url
							},
								{
									where: { oldChannelSocialLinkId: requestObject.social_twitter_id },
									returning: true, transaction,
								});
						} else {
							// save channel social twitter profile details to channel social link table
							if (requestObject.social_twitter_url && Object.keys(requestObject.social_twitter_url).length > 0) {
								await OldChannelSocialLink.create({
									socialSiteId: appconstant.SOCIAL_SITE_TYPE_TWITTER,
									oldChannelId: companyProfile.oldChannelId,
									channelSocialLink: requestObject.social_twitter_url
								},
									{
										transaction
									});
							}
						}

						if (requestObject.social_facebook_id) {
							// update channel social facebook profile details to channel social link table
							await OldChannelSocialLink.update({
								oldChannelId: companyProfile.oldChannelId,
								channelSocialLink: requestObject.social_facebook_url
							},
								{
									where: { oldChannelSocialLinkId: requestObject.social_facebook_id },
									returning: true, transaction,
								});
						} else {
							// save channel social facebook profile details to channel social link table
							if (requestObject.social_facebook_url && Object.keys(requestObject.social_facebook_url).length > 0) {
								await OldChannelSocialLink.create({
									socialSiteId: appconstant.SOCIAL_SITE_TYPE_FACEBOOK,
									oldChannelId: companyProfile.oldChannelId,
									channelSocialLink: requestObject.social_facebook_url
								},
									{
										transaction
									});
							}
						}
						


	
						// if (requestObject.B2B_url && Object.keys(requestObject.B2B_url).length > 0) {
	
						// 	for (let ii = 0; ii < requestObject.B2B_url.length; ii++) {
						// 		if (requestObject.B2B_url[ii].channel_B2B_id) {
						// 			// update channel B2B details to channel B2B table
	
						// 			await this.channelB2BRepository.update({
						// 				channelId: companyProfile.channelId,
						// 				channelB2B: requestObject.B2B_url[ii].channelB2B
						// 			},
						// 				{
						// 					where: { channelB2BId: requestObject.B2B_url[ii].channel_B2B_id },
						// 					returning: true, transaction,
						// 				});
						// 		} else {
						// 			// save channel B2B details to channel B2B table
	
						// 			await this.channelB2BRepository.create({
						// 				channelId: companyProfile.channelId,
						// 				channelB2B: requestObject.B2B_url[ii].channelB2B
						// 			},
						// 				{
						// 					 transaction
						// 				});
						// 		}
						// 	}
						// }
						// if (requestObject.deleted_B2B && requestObject.deleted_B2B.length > 0) {
						// 	for (let ii = 0; ii < requestObject.deleted_B2B.length; ii++) {
						// 		if (requestObject.deleted_B2B[ii]) {
						// 			// update channel B2B details to channel B2B table
						// 			await this.channelB2BRepository.update({
						// 				channelId: companyProfile.channelId,
						// 				deletedAt: new Date()
						// 			},
						// 				{
						// 					where: { channelB2BId: requestObject.deleted_B2B[ii] },
						// 					returning: true, transaction,
						// 				});
						// 		}
						// 	}
						// }
						
						if (companyProfileJson.channelLogo) {
							comapnyLogoId = companyProfileJson.channelLogoId;
						}
						// save comapny logo detail to document table
						if (requestObject.company_logo_url) {
							comapnyLogoId = +requestObject.company_logo_url;
						}	
	
						// update channel detals address to channel detail table
						return await OldChannelDetail.update({
							channelLogoId: comapnyLogoId,
							phone1: requestObject.company_phone1,
							phone2: (isUndefined(requestObject.company_phone2))? null : requestObject.company_phone2,
							mobileNo: (isUndefined(requestObject.company_mobile_no))? null : requestObject.company_mobile_no,
							webSiteUrl: requestObject.company_website_url,
							introvideoId: requestObject.company_introvideo,
							detailDesc: requestObject.company_about,//sanitizeHtml(requestObject.company_about, appconstant.SANITIZE_HTML),
						},
							{
								where: { oldChannelId: companyProfile.oldChannelId }, returning: true, transaction,
							});
	
					});
					if (result) {
						return true;
					} else {
						return false;
					}
				}
			} catch (e) {
				console.log("Error+", e);
				throw new InternalServerErrorException();
				
			}
			if (!companyProfile) {
				throw new ProcessException('channel:getCompanyProfile:companyNotFound');
			}

    return true;
}

//saving old channel facts
public async saveOldChannelFacts(data: any) {
    console.log('data to be saved...', data);

    let user = data.user;

    try {
        
        
            return await this.sequelizeInstance.transaction(async transaction => {
                let contacts = [];
                
                let ChannelDetailObject = <OldChannelDetail>{};
                ChannelDetailObject = await this.tradeInformationInBound.setFactsOldChannelDetail(data);

                if (ChannelDetailObject) {
                    return await OldChannelDetail.update(ChannelDetailObject, { where: { oldChannelId: user.oldChannelId, deletedAt: null }, returning: true, transaction });
                }
            });

    } catch (e) {
        console.log("Error+", e);
        throw new InternalServerErrorException();
    }

    return true;
}

//update oldchannel banner
public async updateBannerImage(requestObject: any): Promise<boolean> {
    let user = requestObject.user;
    let bannerId: any = null;
        let companyBanner;
            let oldBanner = await OldChannelDetail.findOne({
                where: {oldChannelId: user.oldChannelId},
                attributes: ['channelBannerId']
            })
            return await this.sequelizeInstance.transaction(async transaction => {
                console.log('oldBanner',oldBanner);
            if(oldBanner){
                await this.mediaRepository.update({deletedAt: new Date()}, {where: {
                    documentId: +oldBanner.channelBannerId
                },returning: true, transaction});
            }

            companyBanner = await this.mediaRepository.create({
                documentPath: requestObject.bannerUrl,
                documentTypeId: appconstant.DOCUMENTTYPE_IMAGE,
                dirName: appconstant.FOLDER_PATH_COMPANY_BANNER
            },
                {
                     transaction,
                });
                bannerId = companyBanner.documentId;
                await OldChannelDetail.update({channelBannerId:bannerId}, {
                    where: {oldChannelId: user.oldChannelId},returning: true, transaction
                });
                return companyBanner;
            });
}


//helper functions

/**
 * Sort array of objects based on another array
 */

 async mapOrder (array, order, key) {
  
	array.sort( function (a, b) {
	  var A = a[key], B = b[key];
	  
	  if (order.indexOf(A) > order.indexOf(B)) {
		return 1;
	  } else {
		return -1;
	  }
	  
	});
	
	return array;
  };


}
