import { Inject, Injectable, InternalServerErrorException } from '@nestjs/common';
import {Op, QueryTypes } from "sequelize";
import { appconstant } from 'src/config/app.constant';
import { Country } from 'src/modules/common/models/Country.entity';
import { RegionCountryJCT } from 'src/modules/common/models/RegionCountryJCT.entity';
import { Media } from 'src/modules/media/models/Media.entity';
import { Brand } from 'src/modules/product/models/Brand.entity';
import { Product } from 'src/modules/product/models/Product.entity';
import { ProductCategory } from 'src/modules/product/models/ProductCategory.entity';
import { ProductGroup } from 'src/modules/product/models/ProductGroup.entity';
import { SignUpStatus } from 'src/modules/user/models/SignUpStatus.entity';
import { Utility } from '../helper/utility';
import { CertificateType } from '../models/CertificateType.entity';
import { Channel } from '../models/Channel.entity';
import { ChannelAward } from '../models/ChannelAward.entity';
import { ChannelType } from '../models/ChannelType.entity';
import { ContactJCT } from '../models/ContactJCT.entity';
import { ProfileType } from '../models/ProfileType.entity';
import { SocialSite } from '../models/SocialSite.entity';
import { Subscription } from '../models/Subscription.entity';
import { SubscriptionPlan } from '../models/SubscriptionPlan.entity';
import { ChannelHubDetailOutBound } from '../transformation/channelHubDetailOutBound';
import { ChannelKeyPartnerService } from './channelKeyPartner.service';
import { Contact } from 'src/modules/user/models/Contact.entity';
import { ShareType } from '../models/ShareType.entity';
import { ChannelCustomerProfile } from '../models/ChannelCustomerProfile.entity';
import { ProfileTypeClassification } from '../models/ProfileTypeClassification.entity';
import { Address } from '../models/Address.entity';
import { ChannelB2B } from '../models/ChannelB2B.entity';
import { CompanyMediaImages } from '../models/CompanyMediaImages';
import { AuthException } from 'src/middleware/exception/Auth.exception';
import { ChannelBrand } from '../models/ChannelBrand.entity';
import { SampleOrder } from 'src/modules/order/models/SampleOrder.entity';
import { ProductEvaluation } from 'src/modules/product/models/ProductEvaluation.entity';
const Cryptr = require('cryptr');

var _ = require('lodash');

@Injectable()
export class PublicChannelService {
	private channelHubDetailOutBound = new ChannelHubDetailOutBound();
    constructor(
        @Inject('SequelizeInstance') private readonly sequelizeInstance,
		@Inject('ChannelRepository') private readonly channelRepository: typeof Channel,
		@Inject('SignUpStatusRepository') private readonly signUpStatusRepository: typeof SignUpStatus,
		@Inject('ChannelTypeRepository') private readonly channelTypeRepository: typeof ChannelType,
		@Inject('ProductGroupRepository') private readonly productGroupRepository: typeof ProductGroup,
		@Inject('ProductCategoryRepository') private readonly productCategoryRepository: typeof ProductCategory,
		@Inject('ProfileTypeRepository') private readonly profileTypeRepository: typeof ProfileType,
		@Inject('ContactJCTRepository') private readonly contactJCTRepository: typeof ContactJCT,
		@Inject('SubscriptionRepository') private readonly subscriptionRepository: typeof Subscription,
		@Inject('MediaRepository') private readonly mediaRepository: typeof Media,
		@Inject('ChannelCertificateTypeRepository') private readonly channelCertificateTypeRepository: typeof CertificateType,
		@Inject('ProductRepository') private readonly productRepository: typeof Product,
		@Inject('SharetypeRepository') private readonly sharetypeRepository: typeof ShareType,
		@Inject('ChannelCustomerProfileRepository') private readonly channelCustomerProfileRepository: typeof ChannelCustomerProfile,
		@Inject('ProfileTypeClassificationRepository') private readonly profileTypeClassificationRepository: typeof ProfileTypeClassification,
		@Inject('BrandRepository') private readonly brandRepository: typeof Brand,
		@Inject('ContactRepository') private readonly contactRepository: typeof Contact,
		// @Inject('AddressRepository') private readonly addressRepository: typeof Address,
		@Inject('ChannelBrandRepository') private readonly channelBrandRepository: typeof ChannelBrand,
		private readonly channelKeyPartnerService: ChannelKeyPartnerService,
		@Inject('CompanyMediaImagesRepository') private readonly companyMediaImagesRepository: typeof CompanyMediaImages,
    ){}

	async getChannel(channelId: number): Promise<any> {
		console.log('channelId',+channelId);
		let channel = await this.channelRepository.scope('channelDetail').findOne<Channel>({
			where:
			{
				channelId: +channelId,
				deletedAt: null,
				signUpStatusId: appconstant.CHANNEL_APPROVED,
				isActive: '1',
				channelTypeId: {
					[Op.in]: [appconstant.CHANNEL_TYPE_VENDOR, appconstant.CHANNEL_TYPE_DISTRIBUTOR, appconstant.CHANNEL_TYPE_RETAILER, appconstant.CHANNEL_TYPE_RESELLER]
				}
			},
			include: [
				{
				model: ChannelB2B,
				as:'channelB2B',
				attributes: ['channelB2BId', 'channelId', 'channelB2B'],
				where: { deletedAt: null },
				required: false,
			},
				{ model: this.signUpStatusRepository },
				{ model: this.channelTypeRepository },
				{ model: Country, as: 'country', where: { deletedAt: null } },
				{ model: Address, as: 'regAddress'},
				{ model: ProfileType.scope('channelDetail'), as: 'profileType', attributes: ['profileType'], where: { deletedAt: null }, required: false },
				{
					model: this.productGroupRepository,
					as: 'speciality',
					attributes: ['productGroupId', 'productGroup'],
					required: false,
					where: { isActive: '1' },
					through: { attributes: [], where: { deletedAt: null } }
				},
				{
					model: this.productCategoryRepository,
					as: 'productCategory',
					attributes: ['productCategorieId', 'productCategorie'],
					required: false,
					where: { isActive: '1' },
					order:'productCategorie',
					through: { attributes: [], where: { deletedAt: null } }
				},
				{
					model: ChannelAward,
					attributes: ['awardDetail', 'issuedBy', 'validUpto', 'isLifeTime'],
					where: { deletedAt: null },
					required: false
				}
			]
		});
		if (!channel) return false;
		console.log('.......channel,,,,,,,,,');
		try {
			if (!channel.channelDetail) {
				await new Utility().lazyload(channel, 'channelDetail', {
					where: { deletedAt: null },
					required: false,
					attributes: ['channelLogoId', 'channelIconId', 'webSiteUrl', 'estYear', 'regYear', 'detailDesc', 'expectations', 'companyDesc', 'empCntRange', 'empCnt', 'empCntDSale', 'empCntISale', 'empCntDMar', 'empCntIMar', 'turnoverRange', 'turnover', 'revD', 'revI', 'deletedAt', 'turnoverPhyStore', 'turnoverOnStore', 'startup', 'storeCount', 'maturitylevel', 'keyretailers', 'keyresellers', 'keydistributors', 'no_of_retailers', 'no_of_resellers', 'no_of_skus', 'no_of_distributors', 'no_of_vendors', 'introvideoId', 'channelBannerId', 'phone1', 'mobileNo', 'bannerURL'],
					include: [{
						model: Media,
						as: 'channelLogo',
						attributes: ['documentUrl', 'documentPath'],
						where: { deletedAt: null },
						required: false
					},
					{
						model: Media,
						as: 'channelIcon',
						where: { deletedAt: null },
						attributes: ['documentUrl'],
						required: false
					},
					{
						model: Media,
						as: 'introvideo',
						where: { deletedAt: null },
						// attributes: ['documentUrl'],
						required: false
					},
					{
						model: Media,
						as: 'channelBanner',
						where: { deletedAt: null },
						// attributes: ['documentUrl'],
						required: false
					}
					],
				});
			}

			if (!channel.serviceList) {
				await new Utility().lazyload(channel, 'serviceList', {
					through: { attributes: [], where: { deletedAt: null } },
					required: false,
					attributes: ['serviceListId', 'serviceList']
				});
			}

			if (!channel.channelLocation) {
				await new Utility().lazyload(channel, 'channelLocation', {
					where: { deletedAt: null },
					required: false,
					attributes: ['locationType'],
					include: [{
						model: RegionCountryJCT,
						where: { deletedAt: null },
						required: false,
						include: [
							{
								model: Country,
								where: { deletedAt: null },
								attributes: ['countryId', 'country', 'priority'],
								required: false
							}
						],
					}]
				});
			}

			if (!channel.channelBrand) {
				await new Utility().lazyload(channel, 'channelBrand', {
					where: { deletedAt: null },
					required: false,
					include: [{
						model: Brand.scope('channelDetail'),
						as: 'brand',
						where: { deletedAt: null, isActive: '1' },
						required: false,
						include: [{
							model: Media,
							as: 'document',
							where: { deletedAt: null },
							required: false
						}]
					}]
				});
			}

			if (!channel.channelSocial) {
				await new Utility().lazyload(channel, 'channelSocial', {
					where: { deletedAt: null },
					required: false,
					include: [{
						model: SocialSite.scope('channelDetail'),
						where: { deletedAt: null },
						attributes: ['socialSite'],
						required: false
					}]
				});
			}

			if (!channel.channelCertificate) {
				await new Utility().lazyload(channel, 'channelCertificate', {
					where: { deletedAt: null },
					attributes: ['cerificateTypeId', 'cerificateName', 'cerificateNumber', 'cerificateDesc'],
					required: false
				});
			}

			if (!channel.channelSubscription) {
				await new Utility().lazyload(channel, 'channelSubscription', {
					where: { isActive: '1' },
					required: false,
					include: [{
						model: this.subscriptionRepository.scope('channelDetail'),
						as: 'subscription',
						where: { deletedAt: null, isActive: '1' },
						required: false,
						include: [{
							model: SubscriptionPlan.scope('channelDetail'),
							as: 'subscriptionPlan',
							where: { deletedAt: null, isActive: '1' },
							required: false
						}],
					}]
				});
			}
			if (!channel.channelDocument) {
				await new Utility().lazyload(channel, 'channelDocument', {
					attributes: ['channelDocumentId', 'documentId', 'name', 'description'],
					where: { deletedAt: null },
					required: false,
					include: [{
						attributes: ['documentId', 'documentPath', 'documentUrl'],
						where: { deletedAt: null, documentTypeId:2 },
						model: this.mediaRepository,
						required: true,
					}]
				});
			}
			channel.toJSON();
			
			channel['channelKeyRetailer'] = await this.channelKeyPartnerService.getKeyRetailers(+channel.channelId, appconstant.CHANNEL_TYPE_RETAILER);
			channel['channelKeyReseller'] = await this.channelKeyPartnerService.getKeyRetailers(+channel.channelId, appconstant.CHANNEL_TYPE_RESELLER);
			channel['channelKeyDistributor'] = await this.channelKeyPartnerService.getKeyRetailers(+channel.channelId, appconstant.CHANNEL_TYPE_DISTRIBUTOR);

			let custquery = 'select tpt."profileType", tptc."channelTypeId" from "tblChannelCustomerProfile" tcp inner join "tblProfileTypeClassification" tptc on tcp."profileTypeClassificationId"=tptc."profileTypeClassificationId" inner join "tblProfileType" tpt on tpt."profileTypeId"=tptc."profileTypeId" where tcp."channelId"=:channelId and tcp."deletedAt" is null'

			channel['customerProfile'] = await await this.sequelizeInstance.query(custquery, {
				type: QueryTypes.SELECT,
				raw: true,
				replacements: {
					channelId: +channel.channelId
				}
				});

			if (channel.channelTypeId == appconstant.CHANNEL_TYPE_DISTRIBUTOR || channel.channelTypeId == appconstant.CHANNEL_TYPE_RETAILER) {
				channel['sampleOrderCount'] = await this.getSampleOrderCount(channelId);
				channel['sampleEvaluationCount'] = await this.getSampleEvaluationCount(channelId);
			}

			channel['productCount'] = await this.productRepository.count({
				where: {
					channelId: channelId,
					isactive: '1',
					deletedat: null
				}
			});

			channel['certificateType'] = await this.channelCertificateTypeRepository.findAll({
				where: { deletedAt: null }
			});

			let query='select "productCategorieId", "productCategorie" from "tblProductCategories" where "productCategorieId" in(select "productCategorieId" from "tblInterestedProductCategory" where "channelId"=:channelId and "deletedAt" is null) order by "productCategorie"';
			
			let userProductCat= await this.sequelizeInstance.query(query, {
				type: QueryTypes.SELECT,
				raw: true,
				replacements: {
					channelId: channelId
				}
				});
				let companyImages= await this.companyMediaImagesRepository.findAll<CompanyMediaImages>({
					where: {
						channelId: +channel.channelId,
						deletedAt: null
					},
					include: [{
						model: Media,
						where: { 
							documentTypeId:1,
							deletedAt: null
						},
						required: true
					}],
					order: [['companyMediaId', 'DESC']]
				});
			let channelTransformed: any = await this.channelHubDetailOutBound.publicChannelDetail(channel, userProductCat, companyImages);
			
			return channelTransformed;
		} catch (e) {
			console.log("Error+ ", e);
			throw new InternalServerErrorException();
		}
	}
	async getSampleOrderCount(channelId: number): Promise<number> {
		try {
			return await SampleOrder.count(
				{
					where: {
						buyerchannelid: channelId,
						statusid: { [Op.ne]: appconstant.ORDER_STATUS_CANCELED }
					}
				});
		} catch (e) {
			console.log("Error+ ", e);
			throw new InternalServerErrorException();
		}
	}

	async getSampleEvaluationCount(channelId: number): Promise<number> {
		try {
			return await ProductEvaluation.count(
				{
					where: {
						channelid: channelId,
						deletedat: null
					}
				});
		} catch (e) {
			console.log("Error+ ", e);
			throw new InternalServerErrorException();
		}
	}
	async channelBrands(channelId: number) {
		var whereOptions: any = {
			channelId: channelId,
			isActive: '1',
			signUpStatusId: appconstant.CHANNEL_APPROVED,
			deletedAt: null
		};

		let channel = await Channel.findOne({
			where: whereOptions
		});

		if (!channel) {
			throw new AuthException('Not allowed');
		}

		let veifiedBrands = [];
		if (channel.channelTypeId == Channel.CHANNEL_TYPE_DISTRIBUTOR) {
			// let myChannelIds = await this.getChannelRelatedIds(this.redisInstance, channelId, null, null, Channel.CHANNEL_TYPE_VENDOR, appconstant.MYCHANNEL);
			// if (Object.keys(myChannelIds).length > 0) {
			// 	let brandQuery = 'SELECT array_agg(distinct "Brand"."id") as brands FROM "tblBrand" AS "Brand" INNER JOIN "tblChannelBrand" AS "channelBrand" ON "Brand"."id" = "channelBrand"."brandId" AND "channelBrand"."deletedAt" IS NULL AND "channelBrand"."channelId" IN (:channelIds) WHERE "Brand"."isActive" = \'1\' AND "Brand"."deletedAt" IS NULL ';
			// 	veifiedBrands = await this.sequelizeInstance.query(brandQuery, {
			// 		type: this.sequelizeInstance.QueryTypes.SELECT,
			// 		raw: true,
			// 		replacements: {
			// 			channelIds: Object.keys(myChannelIds)
			// 		}
			// 	}).then((result) => {
			// 		return (result && result[0]) ? result[0].brands : []
			// 	});
			// }
		}

		let option = {
			include: [
				{
					model: this.channelBrandRepository,
					where: { deletedAt: null, channelId: channelId },
					required: true
				},
				{
					model: this.mediaRepository,
					where: { deletedAt: null },
					required: true
				}
			]
		};

		let whereOption: any = {
			isActive: '1',
			deletedAt: null
		};

		option['where'] = whereOption;

		let channelBrandCount = await this.brandRepository.count(option);
		if (channelBrandCount == 0) {
			return {
				"channelBrand": [],
				"total": 0
			};
		}
		let query = 'SELECT "Brand"."id", "Brand"."brandName", json_build_object(\'documentId\', "document"."documentId", \'documentPath\', "document"."documentPath", \'documentUrl\', :CDN_URL || "document"."documentPath") as "document" FROM "tblBrand" AS "Brand" INNER JOIN "tblChannelBrand" AS "channelBrand" ON "Brand"."id" = "channelBrand"."brandId" AND "channelBrand"."deletedAt" IS NULL AND "channelBrand"."channelId" = :channelId INNER JOIN "tblDocument" AS "document" ON "Brand"."documentId" = "document"."documentId" AND "document"."deletedAt" IS NULL WHERE "Brand"."isActive" = \'1\' AND "Brand"."deletedAt" IS NULL '

		query += ' ORDER BY "Brand"."id" asc';

		// query += ' LIMIT :limit OFFSET :offset';

		let channelBrand: any = await this.sequelizeInstance.query(query, {
			type: this.sequelizeInstance.QueryTypes.SELECT,
			model: Brand,
			mapToModel: true,
			replacements: {
				channelId: channelId,
				// limit: pagination.limit,
				// offset: pagination.offset,
				// search: '%' +  + '%',
				CDN_URL: process.env.CDN_URL,
				// id: []
			}
		}).then((results) => {
			let formatedBrands = results;
			if (results) {
				formatedBrands = [];
				results.forEach((brand) => {
					let verified = false;
					// if (veifiedBrands.includes(brand.id)) {
					// 	verified = true;
					// }

					formatedBrands.push(Object.assign(brand.toJSON(), { verified }))
				})
			}

			return formatedBrands;
		});

		let channelBrands: any;
		if (channelBrand && channelBrand.length == 0) {
			return [];
		}

		channelBrands = {
			"channelBrand": channelBrand,
			"total": +channelBrandCount
		};

		return channelBrands;
	}

	async eventMatching(contactData) {


		let event = []; let channelId; let contactId;
		let contactIds; let channelIds;
		let channeltype = [];
		channelId = contactData.companyId; contactId = contactData.contactId;

		try {
			let chtype = await this.sequelizeInstance.query('select "channelTypeId" from "tblChannel" where "channelId"=:chid ', {
				type: QueryTypes.SELECT,
				replacements: {
					chid: channelId
				}
			});
			let typeid = chtype.map(x => x.channelTypeId);
			if (typeid == 2)
				channeltype = [3, 4];
			else if (typeid == 3)
				channeltype = [2];
			else if (typeid == 4)
				channeltype = [2];


			let contacts = await this.sequelizeInstance.query('select * from "tblContact" where "isActive"=\'1\' and "isEvent"=\'1\' and "deletedAt" is null and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "deletedAt" is null and "signUpStatusId"=15) ', {
				type: QueryTypes.SELECT,
				replacements: {
					channelTypeId: channeltype
				}
			});

			contactIds = contacts.map(x => x.contactId);
			channelIds = contacts.map(x => x.channelId);

			let contactids = contactIds.toString(); let channelids = channelIds.toString();


			let result = await this.sequelizeInstance.query('select * from "EventMatchingAlgm"(:chid,:conid,:plan,:conids,:chids)', {
				type: QueryTypes.SELECT,
				replacements: {
					conid: contactId,
					chid: channelId,
					plan: 0,
					conids: contactids,
					chids: channelids
				}
			});

			let i = 1;
			let results = [];
			let mscore;

			if (result[0].EventMatchingAlgm != null) {

				//result[0].EventMatchingAlgm.matched = _.orderBy(result[0].EventMatchingAlgm.matched, ['companyname'],['asc']);
				//result[0].EventMatchingAlgm.nonmatched = _.orderBy(result[0].EventMatchingAlgm.nonmatched, ['companyname'],['asc']);


				if (result[0].EventMatchingAlgm.intmatched != null) {
					result[0].EventMatchingAlgm.intmatched.map(x => {
						if (x.profile_strength > 75)
							mscore = 80 + 20;
						else if (x.profile_strength >= 70 && x.profile_strength <= 75)
							mscore = 80 + 15;
						else if (x.profile_strength >= 65 && x.profile_strength < 70)
							mscore = 80 + 10;
						else if (x.profile_strength < 65)
							mscore = 80 + 8;

						Object.assign(x, { isInterested: true, isMatched: true, rank: i++, matchingScore: mscore });
						// Object.assign(x,{isMatched:true});				
						// Object.assign(x,{rank:i++});
					})
					results.push(...result[0].EventMatchingAlgm.intmatched);
				}

				if (result[0].EventMatchingAlgm.matched != null) {
					result[0].EventMatchingAlgm.matched.map(x => {
						if (x.profile_strength > 75)
							mscore = 70 + 20;
						else if (x.profile_strength >= 70 && x.profile_strength <= 75)
							mscore = 70 + 15;
						else if (x.profile_strength >= 65 && x.profile_strength < 70)
							mscore = 70 + 10;
						else if (x.profile_strength < 65)
							mscore = 70 + 8;
						Object.assign(x, { isInterested: false, isMatched: true, rank: i++, matchingScore: mscore });
					})
					results.push(...result[0].EventMatchingAlgm.matched);
				}

				if (result[0].EventMatchingAlgm.nonmatched != null) {
					result[0].EventMatchingAlgm.nonmatched.map(x => {
						if (x.profile_strength > 75)
							mscore = 60 + 20;
						else if (x.profile_strength >= 70 && x.profile_strength <= 75)
							mscore = 60 + 15;
						else if (x.profile_strength >= 65 && x.profile_strength < 70)
							mscore = 60 + 10;
						else if (x.profile_strength < 65)
							mscore = 60 + 8;
						Object.assign(x, { isInterested: false, isMatched: false, rank: i++, matchingScore: mscore });
					});
					results.push(...result[0].EventMatchingAlgm.nonmatched);
				}

			}
			let data = {
				companyId: channelId,
				contactId: contactId,
				participant: results.length > 0 ? results : []
			}
			return data;
		}
		catch (e) {
			//console.log("error:",e);
			return [];

		}

	}

	async eventListing(typeid) {
		let channeltype = []; let result = [];
		//let ids = [224, 528, 534, 716, 756, 776, 183, 590, 815, 819, 845, 860, 776, 590, 993, 994, 1014, 1033, 1040, 1014, 994, 993, 993, 993, 993, 993, 993, 1040, 1040, 994, 994, 994, 994, 994, 994, 1062, 1062, 1062, 993, 993, 1128, 1137, 1143, 534, 1151, 534, 1034, 534, 534, 534, 534, 994, 994, 994, 994, 994, 1158, 1158, 1161, 1033, 993, 534, 845, 1161, 993, 993, 1151, 1034, 1164, 1062, 1062, 1016, 1040, 1164, 1164, 1164, 1164, 1170, 1170, 1170, 1014, 183, 1174, 1175, 993, 993, 716, 1137, 1137, 1137, 1016, 1040, 756, 756, 756, 756, 860, 860, 1174, 1174, 1179, 1180, 1180, 1180, 1180, 1180, 1180, 1180, 1180, 1179, 1179, 1179, 1179, 1180, 1180, 1180, 1180, 1180, 1180, 1180, 1179, 1179, 1175];
		if (typeid == 2)
			channeltype = [2];
		else if (typeid == 3)
			channeltype = [3];
		else if (typeid == 4)
			channeltype = [4];
		else if (typeid == 1)
			channeltype = [3, 4];


		try {
			let query = 'select tc."channelId",tc."companyName",(select "country" from "tblCountry" where "countryId"=tc."countryId") as "Country",cd."detailDesc" as tweetpitch, cd."webSiteUrl" as websiteurl, '
				+ '(select concat(\'https://d35w8j22j5uly8.cloudfront.net/\',"documentPath") from "tblDocument" where "documentId"=cd."channelLogoId") as companyLogo, '
				+ 'case when tc."channelTypeId"=2 then \'Vendor\' when tc."channelTypeId"=3 then \'Distributor\' when tc."channelTypeId"=4 then \'Retailer\' end as channelType, '
				+ '(select array_to_json(array_agg(row_to_json(c))) from(select concat("firstName",\' \',"lastName",\',\',"jobTitle") as contact,(Select array_to_json(array_agg(row_to_json(x))) from(select pc."productCategorie" as item_name from "tblContactInterestedProductCategory" cpc inner join "tblProductCategories" pc on pc."productCategorieId" = cpc."productCategorieId" where cpc."contactId"=tcon."contactId" and "isEvent"=\'1\')x) as intcat, '
				+ '(select array_to_json(array_agg(row_to_json(x))) from (Select pc."productCategorie" as item_name from "tblContactProductCategorie" cpc inner join "tblProductCategories" pc on pc."productCategorieId" = cpc."productCategorieId" where cpc."contactId"=tcon."contactId")x) as cat '
				//+ '(select array_to_json(array_agg(row_to_json(x))) from (Select pc."productCategorie" as item_name from "tblChannelProductCategorie" cpc inner join "tblProductCategories" pc on pc."productCategorieId" = cpc."productCategorieId" where cpc."channelId"=tc."channelId")x) as chcat '
				+ ' from "tblContact" where "channelId"=tc."channelId" and tcon."isEvent"=\'1\')c) as contact, '
				//+ '(select case when "isInterested"=\'1\' and "isMatched"=\'1\' then 100 when "isInterested"=\'0\' and "isMatched"=\'1\' then 75 when "isInterested"=\'0\' and "isMatched"=\'0\' then 25 end as macthingscore from "tblEventMatching" where "pchannelId"=tc."channelId"), '
				+ '(select array_to_json(array_agg(row_to_json(z))) from (Select pt."profileType" as item_name from "tblProfileType" pt where "profileTypeId" in (select "profileTypeId" from "tblChannelCustomerProfile" as ccp where ccp."channelId"=tc."channelId"))z) as retail_customer_profile, '
				+ 'tc."profileStrength" as "profileScore" from "tblContact" tcon inner join "tblChannel" tc on tc."channelId"=tcon."channelId" and tc."channelTypeId" in(:channelTypeId) '
				+ 'inner join "tblChannelDetail" cd on cd."channelId"=tcon."channelId" where tcon."isEvent"=\'1\' and tcon."isActive"=\'1\' order by tc."companyName" asc';

			result = await this.sequelizeInstance.query(query, {
				type: QueryTypes.SELECT,
				replacements: {
					channelTypeId: channeltype

				}
			});


			// const cryptr = new Cryptr('ch-event-auth');
			// ids.map(x => {
			// 	let url = "https://d2y4uc0ee09il5.cloudfront.net/channelview/" + x + "?token=" + cryptr.encrypt(x);
			// 	result.push(url);
			// });
		}
		catch (e) {
			// return [];
			console.log("error:", e);
		}
		return result;

	}

	async getPassword(contactId) {
		let query = 'select "password" from "tblUser" where "contactId"=:contactId';
		let password = await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT,
			replacements: {
				contactId: contactId
			}
		}).then(x => {
			return x[0].password;
		});

		return password;
	}


	}
