'use strict';

import { Injectable, Inject, InternalServerErrorException } from '@nestjs/common';
import { Region } from 'src/modules/common/models/Region.entity';
import { Zone } from 'src/modules/common/models/Zone.entity';
import {Op, QueryTypes } from "sequelize";
// var nodeUnique = require('node-unique-array');

// import { isBoolean, isUndefined } from 'lodash';
import { ChannelLocation } from '../models/ChannelLocation.entity';
import { ContactLocation } from 'src/modules/user/models/ContactLocation.entity';
import { Country } from 'src/modules/common/models/Country.entity';


@Injectable()
export class TradeLocationService {
	constructor(
		@Inject('SequelizeInstance') private readonly sequelizeInstance,
		@Inject('ChannelLocationRepository') private readonly channelLocationRepository: typeof ChannelLocation,
		@Inject('ContactLocationRepository') private readonly contactLocationRepository: typeof ContactLocation,
		@Inject('ZoneRepository') private readonly zoneRepository: typeof Zone,
	) {
		// super();
	}
	async getChannelLocation(user: any): Promise<any> {
		try {
			let locations = {};
			let channelLocation = await this.contactLocationRepository.scope('contactLocationInfo').findAll<any>({
				where: {
					channelId: user.channelId,
					deletedAt: null
				}
			});
			var newLocations = { 'TARGET': [], 'SELLING': [], 'RETAIL': [] };
			if (channelLocation && channelLocation.length > 0) {
				channelLocation.forEach(function (obj) {
					newLocations[obj.locationType].push(obj.regionCountryJCTId);
				});

				for (var i in newLocations) {
					let locationObj: any = await this.getZoneObject(newLocations[i]);
					newLocations[i] = locationObj;
				}
			}
			locations['currentLocation'] = newLocations;

			let oldLocations = { 'TARGET': [], 'SELLING': [], 'RETAIL': [] };
			if (+user.channel.oldChannelId) {

				let query = 'select to_json( array_agg((d.*))) from( '
					+ 'select "locationType", array_to_json( array_agg(to_json(c.loc))) from( '
					+ '		select  "locationType", json_build_object ( \'zoneId\', "zoneId" ,\'zone\' ,zone ,\'region\', region ) loc from( '
					+ '		select '
					+ '		"locationType", '
					+ '		z."zoneId", '
					+ '		z.zone, '
					+ '			(select array_to_json(array_agg(to_json(q))) from ( '
					+ '				select '
					+ '					r1."regionId", '
					+ '					r1."region", '
					+ '					zr1."zoneRegionJCTId", '
					+ '					json_build_object(\'zoneRegionJCTId\', zr1."zoneRegionJCTId")  "ZoneRegionJCT",	'
					+ '					(select to_json(array_agg(a.* )) from '
					+ '					   (select c1.* , '
					+ '							(select to_json(b.*) from (select * from "tblRegionCountryJCT" rc2 where rc2."regionCountryJCTId" =rc1."regionCountryJCTId"  )b) "RegionCountryJCT" '
					+ '							from "tblCountry" c1 where c1."countryId" = rc1."countryId")a) country '
					+ '					from '
					+ '					"tblOldChannelLocation"  ocl1 '
					+ '					inner join "tblRegionCountryJCT" rc1   on rc1."regionCountryJCTId" = ocl1."regionCountryJCTId" '
					+ '					inner join "tblZoneRegionJCT" zr1 ON rc1."regionId" = zr1."regionId" '
					+ '					JOIN "tblRegion" r1 ON r1."regionId" = zr1."regionId" '
					+ '					where zr1."zoneId" = z."zoneId" and ocl1."locationType" = ocl."locationType" '
					+ '					and ocl1."oldChannelId" = ocl."oldChannelId" '
					+ '				)q '
					+ '			)region '
					+ '		from  '
					+ '		"tblOldChannelLocation"  ocl '
					+ '		inner join "tblRegionCountryJCT" rc   on rc."regionCountryJCTId" = ocl."regionCountryJCTId" '
					+ '		inner join "tblZoneRegionJCT" zr ON rc."regionId" = zr."regionId" '
					+ '		JOIN "tblZone" z ON z."zoneId" = zr."zoneId" '
					+ '		where ocl."oldChannelId" = :oldChannelId '
					+ '		Group by "locationType",z."zoneId",ocl."oldChannelId"  '
					+ '		)b  '
					+ ')c group by "locationType" )d ;';

				let result = await this.sequelizeInstance.query(query, {
					type: QueryTypes.SELECT,
					replacements: {
						oldChannelId: +user.channel.oldChannelId,
					}
				}).then((result) => {
					return result[0].to_json;
				});

				if (result && result.length > 0) {
					for (var j = 0; j < result.length; j++) {
						oldLocations[result[j].locationType] = result[j].array_to_json;
					}
				}

			}
			locations['oldLocation'] = oldLocations;
			return locations;
		} catch (e) {
			console.log("Error+", e);
			throw new InternalServerErrorException();
		}
	}
	async getZoneObject(locations: any) {
		return await this.zoneRepository.scope('zones').findAll<any>({
			include: [{
				model: Region,
				as: 'region',
				where: {
					deletedAt: null,
				},
				through: {
					attributes: ['zoneRegionJCTId'],
					where: { deletedAt: null }
				},
				include: [{
					model: Country,
					as: 'country',
					where:
					{
						deletedAt: null
					},
					through: {
						attributes: ['regionCountryJCTId', 'regionId', 'countryId', 'deletedAt'],
						where: {
							deletedAt: null,
							regionCountryJCTId: { [Op.in]: locations }
						}
					},
				}]
			}],
			where: {
				deletedAt: null
			}
		});
	}
}
