'use strict';

//import { BaseTransformation } from "../../core/transformation/BaseTransformation";
import { Media } from '../../media/models/Media.entity';
import { ProcessException } from '../../../middleware/exception/Process.exception';
import { Contact } from '../../user/models/Contact.entity';
import { Claim } from '../models/Claim.entity';
import { ClaimDocument } from '../models/ClaimDocument.entity';
import { SignUpHistory } from '../../user/models/SignUpHistory.entity';
import { Channel } from "../models/Channel.entity";
import { ITradeInformationInBound } from './Interface/ITradeInformationInBound';
import { ChannelProductGroup } from '../models/ChannelProductGroup.entity';
import { ChannelCustomerProfile } from '../models/ChannelCustomerProfile.entity';
import { ChannelProductCategorie } from '../models/ChannelProductCategorie.entity';
import { ChannelServiceList } from '../models/ChannelServiceList.entity';
import { ChannelDetail } from '../models/ChannelDetail.entity';
import { ChannelSpeciality } from '../models/ChannelSpeciality.entity';
import { ChannelBrand } from '../models/ChannelBrand.entity';
//import { AnyFindOptions } from "sequelize";
import { ContactProductGroup } from "../../user/models/ContactProductGroup.entity";
import { ContactProductCategory } from "../../user/models/ContactProductCategory.entity";
import { ChannelKeyPartner } from '../models/ChannelKeyPartner.entity';
import { ProductKeyPartnerJCT } from './../../product/models/ProductKeyPartnerJCT.entity';
//import { appconstant } from './../../../config';
import { isUndefined } from 'lodash';
import { KeyPartnerJCT } from '../models/KeyPartnerJCT.entity';
import { appconstant } from 'src/config/app.constant';
import {Op} from "sequelize";
import { OldChannelDetail } from '../models/OldChannelDetail.entity';

export class TradeInformationInBound implements ITradeInformationInBound {
	// constructor() {
	// 	super();
	// }

	async setProductGroup(productFamilies: any, channelId: number): Promise<ChannelProductGroup[]> {
		try {
			let resultObject = <Array<ChannelProductGroup>>[];
			if (productFamilies) {
				for (let ii = 0; ii < productFamilies.length; ii++) {
					let innerObject = <ChannelProductGroup>{};
					innerObject.channelId = parseInt(channelId.toString());
					innerObject.productGroupId = productFamilies[ii].id;
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:productGroup');
		}
	}

	async setProductCategories(productCategories: any, channelId: number): Promise<ChannelProductCategorie[]> {
		try {
			let resultObject = <Array<ChannelProductCategorie>>[];
			if (productCategories) {
				for (let ii = 0; ii < productCategories.length; ii++) {
					let innerObject = <ChannelProductCategorie>{};
					innerObject.channelId = parseInt(channelId.toString());
					innerObject.productCategorieId = productCategories[ii].productCategoriesId;
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:ProductCategories');
		}
	}

	async setContactProductGroup(productFamilies: any, contactId: number): Promise<ContactProductGroup[]> {
		try {
			let resultObject = <Array<ContactProductGroup>>[];
			if (productFamilies) {
				for (let ii = 0; ii < productFamilies.length; ii++) {
					let innerObject = <ContactProductGroup>{};
					innerObject.contactId = parseInt(contactId.toString());
					innerObject.productGroupId = productFamilies[ii].id;
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:productGroup');
		}
	}

	async setContactProductCategories(productCategories: any, contactId: number): Promise<ContactProductCategory[]> {
		try {
			let resultObject = <Array<ContactProductCategory>>[];
			if (productCategories) {
				for (let ii = 0; ii < productCategories.length; ii++) {
					let innerObject = <ContactProductCategory>{};
					innerObject.contactId = parseInt(contactId.toString());
					innerObject.productCategorieId = productCategories[ii].productCategoriesId;
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:ProductCategories');
		}
	}

	async setCustomerProfiles(customerProfiles: any, channelId: number): Promise<ChannelCustomerProfile[]> {
		try {
			let resultObject = <Array<ChannelCustomerProfile>>[];
			if (customerProfiles) {
				for (let ii = 0; ii < customerProfiles.length; ii++) {
					let innerObject = <ChannelCustomerProfile>{};
					innerObject.channelId = parseInt(channelId.toString());
					innerObject.profileTypeId = customerProfiles[ii].profileTypeId;
					innerObject.profileTypeClassificationId = customerProfiles[ii].profileTypeClassificationId;
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}
			return resultObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:CustomerProfiles');
		}
	}

	async setSpeciality(speciality: any, channelId: number): Promise<ChannelSpeciality[]> {
		try {
			let resultObject = <Array<ChannelSpeciality>>[];
			if (speciality) {
				for (let ii = 0; ii < speciality.length; ii++) {
					let innerObject = <ChannelSpeciality>{};
					innerObject.channelId = parseInt(channelId.toString());
					innerObject.productGroupId = speciality[ii].specialityId;
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
			console.log(e);
			throw new ProcessException('channel:objectCreation:Speciality');
		}
	}

	async setServiceList(listOfServices: any, channelId: number): Promise<ChannelServiceList[]> {
		try {
			let resultObject = <Array<ChannelServiceList>>[];
			if (listOfServices) {
				for (let ii = 0; ii < listOfServices.length; ii++) {
					let innerObject = <ChannelServiceList>{};
					innerObject.channelId = parseInt(channelId.toString());
					innerObject.serviceListId = listOfServices[ii].id;
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:serviceList');
		}
	}

	async keyRetailers(partner_type: number, retailerList: any, channelId: number, userId, deletedKeys?) {
		try {
			if (deletedKeys) {
				await KeyPartnerJCT.update({
					deletedat: new Date()
				}, {
						where: {
							keypartnerid: {
								[Op.in]: deletedKeys
							},
							partner_type: partner_type,
							channelid: +channelId,
							deletedat: null
						},
						returning: true
					}).then(async (datas) => {
						if (!isUndefined(datas[1])) {
							let deletedCKeys = [];
							let deletedProductKeys = [];
							datas[1].forEach((row) => {
								if (row.channelkeypartnerid) deletedCKeys.push(row.channelkeypartnerid);
								deletedProductKeys.push(row.keypartnerid);
							});

							if (deletedCKeys.length) {
								await ChannelKeyPartner.update({
									deletedat: new Date()
								}, {
										where: {
											channelkeypartnerid: {
												[Op.in]: deletedCKeys
											},
											channelId: +channelId,
											deletedat: null
										}
									});
							}

							if (deletedProductKeys.length) {
								await ProductKeyPartnerJCT.destroy({
									where: {
										keypartnerid: {
											[Op.in]: deletedProductKeys
										}
									}
								});
							}
						}
					});
			}

			if (retailerList) {
				var newKeyRetailers = [];
				for (let ii = 0; ii < retailerList.length; ii++) {
					if (isUndefined(retailerList[ii].existchannelid) && isUndefined(retailerList[ii].oldchannelid) && isUndefined(retailerList[ii].keypartnerid)) {
						let innerObject = new ChannelKeyPartner;

						if (retailerList[ii].channelkeypartnerid) {
							innerObject = await ChannelKeyPartner.findByPk(+retailerList[ii].channelkeypartnerid);

							if (innerObject) {
								innerObject.channelkeypartnerid = retailerList[ii].channelkeypartnerid;
								innerObject.updatedat = new Date();
							} else {
								innerObject = new ChannelKeyPartner;
								innerObject.partner_type = partner_type; //appconstant.CHANNEL_TYPE_RETAILER;
								innerObject.createdat = new Date();
							}
						} else {
							innerObject.partner_type = partner_type; // appconstant.CHANNEL_TYPE_RETAILER;
							innerObject.createdat = new Date();
						}

						innerObject.updatedby = userId;
						innerObject.channelId = +channelId;
						innerObject.partnername = retailerList[ii].partnername
						innerObject.countryid = retailerList[ii].countryid;

						if (retailerList[ii].image) {
							let media = new Media;
							media.documentTypeId = appconstant.DOCUMENTTYPE_IMAGE;
							media.documentPath = (retailerList[ii].image) ? retailerList[ii].image : null;
							media.dirName = appconstant.FOLDER_PATH_CHANNEL_KEY_RETAILER_IMAGE;

							await media.save();

							innerObject.documentid = media.documentId;
						} else if (retailerList[ii].imageid) {
							innerObject.documentid = retailerList[ii].imageid;
						}

						await innerObject.save();

						retailerList[ii].channelkeypartnerid = innerObject.channelkeypartnerid;
					}

					if (isUndefined(retailerList[ii].keypartnerid)) {
						let keyRetailer = new KeyPartnerJCT;
						let verify: any = {
							channelid: +channelId,
							partner_type: partner_type,
							deletedat: null
						};

						keyRetailer.channelid = +channelId;
						keyRetailer.partner_type = partner_type;
						keyRetailer.createdat = new Date();
						if (retailerList[ii].channelkeypartnerid) {
							keyRetailer.channelkeypartnerid = retailerList[ii].channelkeypartnerid;
							verify.channelkeypartnerid = +retailerList[ii].channelkeypartnerid;
						}
						if (retailerList[ii].existchannelid) {
							keyRetailer.existchannelid = retailerList[ii].existchannelid;
							verify.existchannelid = +retailerList[ii].existchannelid;
						} 
						if (retailerList[ii].oldchannelid) { 
							keyRetailer.oldchannelid = retailerList[ii].oldchannelid;
							verify.oldchannelid = +retailerList[ii].oldchannelid;
						}
						let verifyResult = await KeyPartnerJCT.findOne({
							where: verify
						});					

						if(verifyResult) {
							keyRetailer = verifyResult;
						} else {
							await keyRetailer.save();							
						}
						newKeyRetailers.push(+keyRetailer.keypartnerid);
					}
					if (!isUndefined(retailerList[ii].keypartnerid) && !isUndefined(retailerList[ii].channelkeypartnerid) && isUndefined(retailerList[ii].imageid)) {
						console.log("Key Pater", retailerList[ii].channelkeypartnerid)
						let keyRetailer = new KeyPartnerJCT;
						let verify: any = {
							channelid: +channelId,
							partner_type: partner_type,
							deletedat: null
						};

						keyRetailer.channelid = +channelId;
						keyRetailer.partner_type = partner_type;
						keyRetailer.createdat = new Date();
						if (retailerList[ii].channelkeypartnerid) {
							console.log("channelkeypartnerid", retailerList[ii].channelkeypartnerid)
							keyRetailer.channelkeypartnerid = retailerList[ii].channelkeypartnerid;
							verify.channelkeypartnerid = +retailerList[ii].channelkeypartnerid;
						}
						if (retailerList[ii].existchannelid) {
							keyRetailer.existchannelid = retailerList[ii].existchannelid;
							verify.existchannelid = +retailerList[ii].existchannelid;
						} 
						if (retailerList[ii].oldchannelid) { 
							keyRetailer.oldchannelid = retailerList[ii].oldchannelid;
							verify.oldchannelid = +retailerList[ii].oldchannelid;
						}
						let verifyResult = await KeyPartnerJCT.findOne({
							where: verify
						});					
						console.log("verifyResult", verifyResult)
						if(verifyResult) {
							keyRetailer = verifyResult;
						} else {
							await keyRetailer.save();							
						}
						newKeyRetailers.push(+keyRetailer.keypartnerid);
					}
				}
			}

			return newKeyRetailers;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:keyretailers' + e);
		}
	}

	async setChannelDetail(channelDetail: any, channelId: number, channelTypeId): Promise<ChannelDetail> {
		try {
			let resultObject = <ChannelDetail>{};

			if (channelDetail.employeesRange && !channelDetail.employeesCount) {
				resultObject.empCntRange = channelDetail.employeesRange;
			} 
			else if(channelDetail.employeesCount){
				resultObject.empCntRange = '';
			}

			if (channelDetail.employeesCount && !channelDetail.employeesRange) {
				resultObject.empCnt = channelDetail.employeesCount;
			 } 
			 else if(channelDetail.employeesRange){
				resultObject.empCnt = null;
			}
			if(channelTypeId==2 || channelTypeId==3 || channelTypeId==4 || channelTypeId==5){
			// resultObject.empCntDSale = null;
			if (channelDetail.domesticSales) {
				resultObject.empCntDSale = channelDetail.domesticSales;
			}

			// resultObject.empCntISale = null;
			if (channelDetail.internationalSales) {
				resultObject.empCntISale = channelDetail.internationalSales;
			}

			// resultObject.empCntDMar = null;
			if (channelDetail.domesticMarketing) {
				resultObject.empCntDMar = channelDetail.domesticMarketing;
			}

			// resultObject.empCntIMar = null;
			if (channelDetail.internationalMarketing) {
				resultObject.empCntIMar = channelDetail.internationalMarketing;
			}
			if (channelDetail.startup) {
				resultObject.startup = channelDetail.startup;
			}

			if (channelDetail.startup == '1' && channelDetail.maturitylevel) {
				resultObject.maturitylevel = channelDetail.maturitylevel;
			} 
			}
			if (channelDetail.annualTurnoverRange && !channelDetail.annualTurnover) {
				resultObject.turnoverRange = channelDetail.annualTurnoverRange;
			 } 
			 else if(channelDetail.annualTurnover){
				resultObject.turnoverRange = '';
			}

			if (channelDetail.annualTurnover && !channelDetail.annualTurnoverRange) {
				resultObject.turnover = channelDetail.annualTurnover;
			 } 
			 else if(channelDetail.annualTurnoverRange){
				resultObject.turnover = null;
			}

			if (!isUndefined(channelDetail.domesticRevenue)) {
				resultObject.revD = channelDetail.domesticRevenue;
			}

			if (!isUndefined(channelDetail.internationalRevenue)) {
				resultObject.revI = channelDetail.internationalRevenue;
			}

			if (!isUndefined(channelDetail.physicalStore)) {
				resultObject.turnoverPhyStore = channelDetail.physicalStore;
			}

			// resultObject.turnoverOnStore = null;
			if (channelDetail.OnlineStore) {
				resultObject.turnoverOnStore = channelDetail.OnlineStore;
			}

			// else {
			// 	resultObject.maturitylevel = [];
			// }

			if (!isUndefined(channelDetail.storeCount)) {
				resultObject.storeCount = channelDetail.storeCount
			}

			if (!isUndefined(channelDetail.no_of_retailers)) {
				resultObject.no_of_retailers = channelDetail.no_of_retailers
			}

			if (!isUndefined(channelDetail.no_of_skus)) {
				resultObject.no_of_skus = channelDetail.no_of_skus
			}

			if (!isUndefined(channelDetail.no_of_distributors)) {
				resultObject.no_of_distributors = channelDetail.no_of_distributors
			}

			if (!isUndefined(channelDetail.no_of_resellers)) {
				resultObject.no_of_resellers = +channelDetail.no_of_resellers
			}

			if (!isUndefined(channelDetail.no_of_vendors)) {
				resultObject.no_of_vendors = channelDetail.no_of_vendors
			}

			if (!isUndefined(channelDetail.keyretailers)) {
				resultObject.keyretailers = channelDetail.keyretailers
			}

			if (!isUndefined(channelDetail.keyresellers)) {
				resultObject.keyresellers = channelDetail.keyresellers
			}

			if (!isUndefined(channelDetail.keydistributors)) {
				resultObject.keydistributors = channelDetail.keydistributors
			}
			
			if (!isUndefined(channelDetail.year_of_estabilishment)) {
				resultObject.estYear = +channelDetail.year_of_estabilishment
			}

			resultObject.updatedAt = new Date()

			return resultObject

		} catch (e) {
			throw new ProcessException('channel:objectCreation:ChannelDetail');
		}
	}

	public async setDocumentObject(docTempPath: string, doctype: number, folderPath?: string): Promise<Media> {
		try {
			let mediaObject = <Media>{};
			mediaObject.documentPath = docTempPath;
			mediaObject.documentTypeId = doctype;
			if (folderPath) {
				mediaObject.dirName = folderPath;
			}
			return mediaObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:media');
		}
	}

	public async  setProductGroupIds(productGroup: any): Promise<Array<any>> {
		try {
			let ProductGroupIds = [];
			if (productGroup) {
				for (let ii = 0; ii < productGroup.length; ii++) {
					ProductGroupIds.push(parseInt(productGroup[ii].productGroupId));
				}
			}
			return ProductGroupIds;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:media');
		}
	}

	async setChannelBrand(brandId: number, channelId: number): Promise<ChannelBrand> {
		try {
			let resultObject = <ChannelBrand>{};

			resultObject.brandId = brandId;
			resultObject.channelId = channelId;
			resultObject.createdAt = new Date();

			return resultObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:channelBrand');
		}
	}

	public async  setProductCategoryIds(ProductGroupCategore: any): Promise<Array<any>> {
		try {
			let ProductCategoreIds = [];
			if (ProductGroupCategore) {
				for (let ii = 0; ii < ProductGroupCategore.length; ii++) {
					ProductCategoreIds.push(parseInt(ProductGroupCategore[ii].productCategorieId));
				}
			}

			return ProductCategoreIds;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:media');
		}
	}

	public async setCustomerProfilesIds(customerProfiles: any): Promise<Array<any>> {
		try {
			let CustomerProfilesIds = [];
			if (customerProfiles) {
				for (let ii = 0; ii < customerProfiles.length; ii++) {
					CustomerProfilesIds.push(parseInt(customerProfiles[ii].profileTypeId));
				}
			}
			return CustomerProfilesIds;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:media');
		}
	}

	public async setServiceListIds(serviceList: any): Promise<Array<any>> {
		try {
			let ServiceListIds = [];
			if (serviceList) {
				for (let ii = 0; ii < serviceList.length; ii++) {
					ServiceListIds.push(parseInt(serviceList[ii].serviceListId));
				}
			}
			return ServiceListIds;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:media');
		}
	}

	public async setbrandIds(channelBrand: any): Promise<Array<any>> {
		try {
			let brandIds = [];
			if (channelBrand) {
				for (let ii = 0; ii < channelBrand.length; ii++) {
					brandIds.push(parseInt(channelBrand[ii].id));
				}
			}
			return brandIds;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:media');
		}
	}

	public async setchannelbrandIds(channelBrand: any): Promise<Array<any>> {
		try {
			let brandIds = [];
			if (channelBrand) {
				for (let ii = 0; ii < channelBrand.length; ii++) {
					brandIds.push(parseInt(channelBrand[ii].brandId));
				}
			}
			return brandIds;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:media');
		}
	}
	async setFactsChannelDetail(channelDetail: any): Promise<ChannelDetail> {
		try {
			let resultObject = <ChannelDetail>{};

			if (channelDetail.employeesRange && !channelDetail.employeesCount) {
				resultObject.empCntRange = channelDetail.employeesRange;
			} 
			else if(channelDetail.employeesCount){
				resultObject.empCntRange = '';
			}

			if (channelDetail.employeesCount && !channelDetail.employeesRange) {
				resultObject.empCnt = channelDetail.employeesCount;
			 } 
			 else if(channelDetail.employeesRange){
				resultObject.empCnt = null;
			}

			if (channelDetail.annualTurnoverRange && !channelDetail.annualTurnover) {
				resultObject.turnoverRange = channelDetail.annualTurnoverRange;
			 } 
			 else if(channelDetail.annualTurnover){
				resultObject.turnoverRange = '';
			}

			if (channelDetail.annualTurnover && !channelDetail.annualTurnoverRange) {
				resultObject.turnover = channelDetail.annualTurnover;
			 } 
			 else if(channelDetail.annualTurnoverRange){
				resultObject.turnover = null;
			}

			if (channelDetail.startup) {
				resultObject.startup = channelDetail.startup;
			}

			if (channelDetail.year_of_estabilishment) {
				resultObject.estYear = channelDetail.year_of_estabilishment;
			}

			if (channelDetail.startup == '1' && channelDetail.maturitylevel) {
				resultObject.maturitylevel = channelDetail.maturitylevel;
			}

			if (!isUndefined(channelDetail.storeCount)) {
				resultObject.storeCount = channelDetail.storeCount
			}

			if (!isUndefined(channelDetail.no_of_skus)) {
				resultObject.no_of_skus = channelDetail.no_of_skus
			}

			resultObject.updatedAt = new Date()

			return resultObject

		} catch (e) {
			throw new ProcessException('channel:objectCreation:ChannelDetail');
		}
	}

	//old channel facts
	async setFactsOldChannelDetail(channelDetail: any): Promise<OldChannelDetail> {
		try {
			let resultObject = <OldChannelDetail>{};

			if (channelDetail.employeesRange && !channelDetail.employeesCount) {
				resultObject.empCntRange = channelDetail.employeesRange;
			} 
			else if(channelDetail.employeesCount){
				resultObject.empCntRange = '';
			}

			if (channelDetail.employeesCount && !channelDetail.employeesRange) {
				resultObject.empCnt = channelDetail.employeesCount;
			 } 
			 else if(channelDetail.employeesRange){
				resultObject.empCnt = null;
			}

			if (channelDetail.annualTurnoverRange && !channelDetail.annualTurnover) {
				resultObject.turnoverRange = channelDetail.annualTurnoverRange;
			 } 
			 else if(channelDetail.annualTurnover){
				resultObject.turnoverRange = '';
			}

			if (channelDetail.annualTurnover && !channelDetail.annualTurnoverRange) {
				resultObject.turnover = channelDetail.annualTurnover;
			 } 
			 else if(channelDetail.annualTurnoverRange){
				resultObject.turnover = null;
			}

			if (channelDetail.startup) {
				resultObject.startup = channelDetail.startup;
			}

			if (channelDetail.year_of_estabilishment) {
				resultObject.estYear = channelDetail.year_of_estabilishment;
			}

			if (channelDetail.startup == '1' && channelDetail.maturitylevel) {
				resultObject.maturitylevel = channelDetail.maturitylevel;
			}

			if (!isUndefined(channelDetail.storeCount)) {
				resultObject.storeCount = channelDetail.storeCount
			}

			if (!isUndefined(channelDetail.no_of_skus)) {
				resultObject.no_of_skus = channelDetail.no_of_skus
			}

			resultObject.updatedAt = new Date()

			return resultObject

		} catch (e) {
			throw new ProcessException('channel:objectCreation:ChannelDetail');
		}
	}
}
