import { Table, Column, PrimaryKey, Model, AutoIncrement, Scopes, DataType, HasOne, BelongsToMany } from 'sequelize-typescript';
import { Inject } from '@nestjs/common';
import { ApiProperty } from '@nestjs/swagger';
import { Region } from './Region.entity';
import { RegionCountryJCT } from './RegionCountryJCT.entity';

@Table({ timestamps:false,
	tableName: "tblCountry"
})

@Scopes(() => ({
	regionCountryJCT: {
		attributes: [['country', 'name'], ['countryId', 'id'], ['isoCode', 'countryCode'], 'priority', 'phoneCode']
	},
	orderInfo: {
		attributes: ['country']
	},
	country: {
		where: { deletedAt: null },
		attributes: ['countryId', 'country'],
		include: [
			{
				model: Region,
				required: false,
				attributes: ['regionId', 'region'],
				where: { deletedAt: null },
			}
		]
	},
}))
export class Country extends Model<Country> {

	public static REDIS_KEY = 'country';

	@PrimaryKey
	@AutoIncrement
	@Column
	@ApiProperty()
	countryId: number

	@Column({
		type: DataType.CHAR(255),
		allowNull: false,
		unique: { name: 'country', msg: 'validate:field:shouldBeUnique' }
	})
	@ApiProperty()
	country: string;

	@Column(DataType.DATE)
	deletedAt: Date

	@Column
	isVat: string

	@Column
	registrationcertificate: string

	@Column
	phoneCode: string

	@Column
	isoCode: string

	@Column
	priority: number

	@BelongsToMany(() => Region, { through: () => RegionCountryJCT })
	region: Region[];

	@HasOne(() => RegionCountryJCT, 'countryId')
	regionCountryJCT: RegionCountryJCT;
}
