import { Table, Column, PrimaryKey, Model, AutoIncrement, BelongsToMany, Scopes, ForeignKey, DataType, HasOne } from 'sequelize-typescript';
import { Country } from './Country.entity';
import { Zone } from './Zone.entity';
import { Channel } from '../../channel/models/Channel.entity';
import { RegionCountryJCT } from './RegionCountryJCT.entity';
import { ZoneRegionJCT } from './ZoneRegionJCT.entity';
import { ApiProperty } from '@nestjs/swagger';

@Table({
	tableName: "tblRegion", timestamps:false
})

@Scopes(() => ({
	zoneRegionJCT: {
		attributes: ['regionId', 'region', 'channelId']
	},
	region: {
		where: { deletedAt: null },
		attributes: ['regionId', 'region'],
		include: [
			{
				model: Zone,
				required: false,
				attributes: ['zoneId', 'zone'],
				where: { deletedAt: null }
			}
		]
	},
}))

export class Region extends Model<Region>{

	@PrimaryKey
	@AutoIncrement
	@Column
	@ApiProperty()
	regionId: number

	@Column({
		type: DataType.CHAR(150),
		allowNull: false,
		unique: { name: 'region', msg: 'validate:field:shouldBeUnique' }
	})
	@ApiProperty()
	region: string;

	@ForeignKey(() => Channel)
	@Column
	channelId: number

	@BelongsToMany(() => Country, { through: () => RegionCountryJCT })
	country: Country[];

	@BelongsToMany(() => Zone, { through: () => ZoneRegionJCT })
	zone: Zone[];

    @HasOne(() => ZoneRegionJCT, 'regionId')
	zoneRegionJCT: ZoneRegionJCT;

}
