import { Table, Column, PrimaryKey, Model, AutoIncrement, BelongsTo, ForeignKey, DataType, Scopes } from 'sequelize-typescript';
import { Country } from './Country.entity';
import { Region } from './Region.entity';
import { ApiProperty } from '@nestjs/swagger';

@Table({
	tableName: "tblRegionCountryJCT",
	timestamps:false
})

@Scopes(() => ({
	channelDetail: {
	  attributes: ['regionCountryJCTId', 'regionId', 'countryId', 'deletedAt']
	}
  }))

export class RegionCountryJCT extends Model<RegionCountryJCT>{

	@PrimaryKey
	@AutoIncrement
	@Column
	@ApiProperty()
	regionCountryJCTId: number

	@ForeignKey(() => Region)
	@Column({
		type: DataType.BIGINT,
		allowNull: false,
		unique: { name: 'regionId_countryId', msg: 'validate:field:shouldBeUnique' }
	})
	@ApiProperty()
	regionId: number;

	@BelongsTo(() => Region)
	region: Region[];

	@ForeignKey(() => Country)
	@Column({
		type: DataType.BIGINT,
		allowNull: false,
		unique: { name: 'regionId_countryId', msg: 'validate:field:shouldBeUnique' }
	})
	@ApiProperty()
	countryId: number

	@BelongsTo(() => Country)
	country: Country[];

	@Column(DataType.DATE)
	deletedAt: Date

}
