import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, BeforeCreate, BeforeUpdate, Scopes } from 'sequelize-typescript';
import { ApiProperty } from '@nestjs/swagger';
var Sequelize = require('sequelize');

@Table({
	tableName: "tblsettings"
})

@Scopes(() => ({
	setting: {
		attributes: ['settingid', 'key', 'value']
	}
}))

export class Setting extends Model<Setting>{

	@PrimaryKey
	@AutoIncrement
	@Column
	@ApiProperty()
	settingid: number

	@Column({
		type: DataType.CHAR(255),
		allowNull: false,
		unique: { name: 'key', msg: 'validate:field:shouldBeUnique' }
	})
	@ApiProperty()
	key: string;

	@Column({
		type: DataType.CHAR(500),
		allowNull: false,
	})
	@ApiProperty()
	value: string;

	@Column({
		type: DataType.CHAR(255),
		allowNull: true,
	})
	@ApiProperty()
	label: string;

	@Column(DataType.DATE)
	createdat: Date

	@Column(DataType.DATE)
	updatedat: Date

	@Column(DataType.BIGINT)
	updatedby: number

	@BeforeCreate
	@BeforeUpdate
	public static async dateUpdate(setting: Setting, options: any) {
		setting.updatedat = Sequelize.fn('NOW');
	}

}
