import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, Scopes, BelongsToMany } from 'sequelize-typescript';
import { Region } from './Region.entity';
import { ZoneRegionJCT } from './ZoneRegionJCT.entity';
import { ApiProperty } from '@nestjs/swagger';

@Table({
	tableName: "tblZone"
})

@Scopes(() => ({
	zones: {
		attributes: ['zoneId', 'zone']
	},
	noZones: {
		attributes: []
	},
}))

export class Zone extends Model<Zone>{

	@PrimaryKey
	@AutoIncrement
	@Column
	@ApiProperty()
	zoneId: number

	@Column({
		type: DataType.CHAR(150),
		allowNull: false,
		unique: { name: 'zone', msg: 'validate:field:shouldBeUnique' }
	})
	@ApiProperty()
	zone: string;

	@Column(DataType.DATE)
	deletedAt: Date

	@BelongsToMany(() => Region, { through: () => ZoneRegionJCT })
	region: Region[];

}
