import { Injectable,Inject } from '@nestjs/common';
import { Brand } from 'src/modules/product/models/Brand.entity';
import { ProductGroup } from 'src/modules/product/models/ProductGroup.entity';
import { OldChannel } from '../../channel/models/OldChannel.entity';
import {Op, QueryTypes } from "sequelize";
import { Country } from '../models/Country.entity';
import { ProductCategory } from 'src/modules/product/models/ProductCategory.entity';
import { ProfileType } from 'src/modules/channel/models/ProfileType.entity';
import { ProfileTypeClassification } from 'src/modules/channel/models/ProfileTypeClassification.entity';

@Injectable()
export class CommonService {
    constructor(
        @Inject('SequelizeInstance') private readonly sequelizeInstance,
        @Inject('OldChannelRepository') private readonly oldChannelRepository: typeof OldChannel,
        @Inject('ProductGroupRepository') private readonly productGroupRepository: typeof ProductGroup,
        @Inject('CountryRepository') private readonly countryRepository: typeof Country,
        //@Inject('SequelizeInstance') private readonly sequelizeInstance
    ){}
    
    public async getSignup(channelTypeId) {
        
        let company = await this.oldChannelRepository.scope('basicInform').findAll({
            where: {
                deletedAt:null,
                channelTypeId: channelTypeId
            }
        });        

        let query='select pg."productGroupId" as groupId,pg."productGroup" as groupname,pc."productCategorieId" as childId,pc."productCategorie" as childname,cast(false as boolean) as status from "tblProductGroup" as pg inner join' 
         +' "tblProductCategories" as pc on pg."productGroupId"=pc."productGroupId" where pg."deletedAt" is null and pg."productGroup"!=\'\' and pg."businessType"=3 ';

         let productGroup=await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT,		
		}).then((results) => {  
			return results; 
        });
        // let productGroup = await this.productGroupRepository.scope('productInfo').findAll({
        //     where: {
        //         deletedAt:null
        //     }
        // });        
        // let brand = await this.brandRepository.scope('basicInfom').findAll({
        //     where: {
        //         deletedAt:null
        //     }
        // });        

        let country = await this.countryRepository.scope('regionCountryJCT').findAll({
            where: {
                deletedAt: null
            },
            order: [
                ['priority', 'ASC']
            ]
        });
        let companyList = [];
        if(company && company.length){
			companyList = company.map(x=>{
				return {id:x.oldChannelId, name:x.companyName, type:1}
			})
		}
        let data={
            productGroup:productGroup,
            //brand:brand,
            company:companyList,
            country:country
        }

        
        return data;
    }

    public async getProducts(businessType) {     
        
        let type = [];
        if(businessType==1)
        type=['1', '3']
        if(businessType==2)
        type=['2', '3']
        if(businessType==3)
        type=['1', '2', '3']

        let query='select pg."productGroupId" as groupId,pg."productGroup" as groupname,pc."productCategorieId" as childId,pc."productCategorie" as childname,cast(false as boolean) as status from "tblProductGroup" as pg inner join' 
         +' "tblProductCategories" as pc on pg."productGroupId"=pc."productGroupId" where pg."deletedAt" is null and pg."productGroup"!=\'\' and pg."businessType" in (:businessType) ';

         let productGroup=await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT,	
            replacements: {
				businessType:type			
			}	
		}).then((results) => {  
			return results; 
        });
        
        let data={
            productGroup:productGroup,
        }

        
        return data;
    }
    
    public async getOldChannelProductGroup(email)
    {
         
         let query='select pg."productGroup" as "firstLetter",pg."productGroupId" as groupId,pg."productGroup" as groupname,pc."productCategorieId" as childId,pc."productCategorie" as childname,cast(true as boolean) as status from "tblProductGroup" as pg inner join "tblProductCategories" as pc on pg."productGroupId"=pc."productGroupId" '
         + 'and pg."productGroupId" in(select "productGroupId" from "tblOldContact" as oc inner join "tblOldChannelProductGroup" as pg on oc."oldChannelId"=pg."oldChannelId" where oc."email" like :mail) '
         + 'and pc."productCategorieId" in(select "productCategorieId" from "tblOldContact" as oc inner join "tblOldChannelProductCategorie" as pg on oc."oldChannelId"=pg."oldChannelId" where oc."email" like :mail)';

        //let query='select "productGroup" as name,"productGroupId" as id from "tblProductGroup" where "productGroupId" in(select "productgroupid" from "tblOldContact" as oc inner join "tbloldchannelproductgroup" as pg on oc."oldChannelId"=pg."oldchannelid" where oc."email" like :mail)';
        
        let productGroup=await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT,
			replacements: {
				mail: '%' + email + '%'
				// d_userid: d_userid,
				// d_subsid: d_subsid
			}
		}).then((results) => {
  //          console.log("progductgrour:",results);
			return results; 
        });

        
        
        let cquery='select "companyName" as name,oc."oldChannelId" as id,"jobTitle","address","postalCode",ad."city","country",oc."countryId",oc."firstName",oc."lastName","isoCode","webSiteUrl" from "tblOldContact" as oc inner join "tblOldChannel" as ch on oc."oldChannelId"=ch."oldChannelId" inner join "tblOldAddress" as ad on ad."oldAddressId"=ch."regAddressId" inner join "tblCountry" as cty on cty."countryId"=oc."countryId" inner join "tblOldChannelDetail" as cd on cd."oldChannelId"=oc."oldChannelId"  where oc."email" like :mail';
        let companyAddress=await this.sequelizeInstance.query(cquery, {
			type: QueryTypes.SELECT,
			replacements: {
				mail: '%' + email + '%'
				// d_userid: d_userid,
				// d_subsid: d_subsid
			}
		}).then((results) => {
//            console.log("companyaddr:",results);
			return results;
        });

        let result={
            productGroup:productGroup,            
            companyAddress:companyAddress
        }

        return result;
    }

    async findAllCountry(){
        return await this.countryRepository.scope('regionCountryJCT').findAll({
            where: {
                deletedAt: null
            },
            order: [
                ['priority', 'ASC']
            ]
        })
    }
}
