'use strict';

import { DataType, Table, Column, Model, ForeignKey, BelongsTo, BeforeCreate, BeforeUpdate, Scopes, AutoIncrement, AllowNull, PrimaryKey } from 'sequelize-typescript';
import { ApiProperty } from '@nestjs/swagger';
import { MediaType } from './MediaType.entity';
// import { ProcessException } from './../../../middleware/exception/Process.exception';
import { S3Lib } from './../../../lib/S3Lib';
import { Utility } from './../helper/utility';
import { ProcessException } from 'src/middleware/exception/Process.exception';
import { FetchUploadS3 } from 'src/lib/FetchUploadS3';

@Table({
	// timestamps: false,
	tableName: "tblDocument"
})

@Scopes(() => ({
	profileInfo: {
		attributes: ['documentId', 'documentTypeId', 'documentPath', 'documentUrl']
	},
	orderInfo: {
		attributes: ['documentUrl']
	}
}))

export class Media extends Model<Media>{
	// updateMedia
	updatedId = null;

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	// @ApiProperty()
	documentId: number

	@Column(DataType.INTEGER)
		// allowNull: false,
		// autoIncrement: false,
		// unique: false,
		// primaryKey: false,
		// validate: {
		// 	isAvailable: async (value: string, next: Function): Promise<any> => {
		// 		const isExist = await MediaType.findOne({ where: { documentTypeId: value } });
		// 		if (!isExist) {
		// 			next('media:documentTypeId:notFound');
		// 		}
		// 		next();
		// 	},
		// }
	// })
	// @ApiProperty()
	@ForeignKey(() => MediaType)
	documentTypeId: number

	@Column({
		type: DataType.CHAR(255),
		allowNull: false,
		unique: false
	})
	@ApiProperty()
	documentPath: string

	@BelongsTo(() => MediaType)
	documentType: MediaType;

	@Column(DataType.DATE)
	deletedAt: Date;

	@Column(DataType.DATE)
	createdAt: Date;

	@Column(DataType.DATE)
	updatedAt: Date;

	@Column({
		type: DataType.CHAR(255),
		// allowNull: false,
		unique: false
	})
	@ApiProperty()
	thumbnailPath: string

	@Column({
		type: new DataType.VIRTUAL(DataType.CHAR(255), ['documentPath']),
		get: function() {
			return process.env.CDN_URL + this.get('documentPath');
		}
	})
	documentUrl: string

	@Column({
		type: new DataType.VIRTUAL(DataType.CHAR(255), ['thumbnailPath']),
		get: function() {
			if(this.get('documentTypeId')==1)
			return process.env.CDN_URL + this.get('thumbnailPath');
		}
	})
	thumbnailUrl: string

	@Column({
		type: new DataType.VIRTUAL(DataType.CHAR(255), ['dirName']),
		set: function(value) {
			this.setDataValue('dirName', value);
		}
	})
	public dirName: string;

	@BeforeCreate
	static async moveFromPublicBucket(instance: Media) {
		let s3 = new S3Lib();
		var fus3 = new FetchUploadS3();
		// format name
		let helper = new Utility();
		let path = instance.documentPath;
		let newPath = await helper.getMediaPath(instance);
		let destinationPath = process.env.AWS_BUCKET_KEY_PREFIX + '/' + newPath;
		// let buffer = await fus3.getS3urlToBuffer(path, process.env.TEM_AWS_BUCKET);
		let moveAction = await s3.CopyObjectTempToPlublic("/" + encodeURI(instance.documentPath.trim()), destinationPath);
		// let moveAction = await s3.moveFromPlublic(buffer, destinationPath);
		if (moveAction) {
			instance.documentPath = destinationPath;
		}
		if(instance.documentTypeId ==1){
		let thambnailPath = await helper.getThambnailPath(instance);
		let destinationThamnailPath = 'thambnail/' + thambnailPath;
		let buffer = await fus3.getS3urlToBuffer(path, process.env.TEM_AWS_BUCKET);
		await fus3.upload(buffer, destinationThamnailPath, 100, 100, process.env.PUBLIC_AWS_BUCKET);
		// console.log('thambnail',thambnail);
		// if (thambnail) {
			instance.thumbnailPath = destinationThamnailPath;
		// }
	}
		// s3.deleteObject(path, process.env.TEM_AWS_BUCKET)
	}

	@BeforeUpdate
	static async blockMediaUpdate(instance: Media) {
		//insert new records
		let newMedia = new Media();
		newMedia.documentTypeId = instance.documentTypeId;
		newMedia.documentPath = instance.documentPath;
		newMedia.dirName = instance.dirName;
		await newMedia.save();

		//return new documentId
		instance.updatedId = newMedia.documentId;

		if(instance.documentId === 1) {
			// delete old records
			// TODO: write cron to remove files from s3 Bucket
			instance.deletedAt = new Date();
			instance.documentPath = instance.previous('documentPath');
		}
	}
}
