'use strict';

//import * as crypto from 'crypto';
import { Injectable, Inject, InternalServerErrorException } from '@nestjs/common';
import { MeetingReq } from '../models/MeetingReq.entity';
import { Contact } from '../../user/models/Contact.entity';
import { MeetingInBound } from './../transformation/meetingInBound';
import { ProcessException } from '../../../middleware/exception/Process.exception';
import { Channel } from './../../channel/models/Channel.entity';
import {Op, QueryTypes } from "sequelize";

@Injectable()
export class MeetingService {//implements IMeetingService {
	private meetingInBound = new MeetingInBound();
	constructor(
        @Inject('MeetingRepository') private readonly meetingRepository: typeof MeetingReq,
		@Inject('ContactRepository') private readonly contactRepository: typeof Contact,
	    @Inject('ChannelRepository') private readonly channelRepository: typeof Channel,
		@Inject('SequelizeInstance') private readonly sequelizeInstance,
		// @Inject('MailerProvider') private readonly mailerProvider
	) { }
	
    async addMeetingReq(meetingDto) {
		let meetingObject = await this.meetingInBound.setAddMeetingObject(meetingDto);//, channelId, password, showId, userId);
		let result: any = await this.sequelizeInstance.transaction(async transaction => {
			await this.meetingRepository.create(meetingObject);
		}).then(function(result) {
			return result;
		}).catch(function(e) {
			console.log(e);
			throw new ProcessException(e.message);
		});

		// Send email to account owner and the new user
		// this.sendMeetingEmail(meetingDto);

		return true;
    }
    
    // async sendMeetingEmail(meetingDto) {
	// 	try {
	// 		//add cc to account owner
	// 		let sender = await this.getContactMail(meetingDto.senderContactId);
	// 		console.log("sender", sender);
	// 		let receiver = await this.getContactMail(meetingDto.receiverContactId);
	// 		console.log("receiver", receiver);
	// 		let companyreceiver = await this.getCompanyMail(meetingDto.receiverChannelId);
	// 		// console.log("companyreceiver", companyreceiver);
	// 		let companysender = await this.getCompanyMail(meetingDto.senderChannelId);
	// 		// console.log("companysender", companysender);
	// 		let cc = companyreceiver.companyMail;
	// 		//if (owner && owner.email != user.email) {
	// 		//	cc = sender.email;
	// 		//}

	// 		//let helper = new Utility();
	// 		//await helper.lazyload(owner, 'contact');
	// 		let unsub_token = null;
	// 		if(meetingDto.receiverContactId){
	// 			unsub_token = await this.unsubscribe_token(meetingDto.receiverContactId);
	// 		}
	// 		let result: any = await this.mailerProvider.sendMail({
	// 			to: process.env.ASK_MEETING_TO,
	// 			// cc: cc,
	// 			from: process.env.EMAIL_FROM,//process.env.EMAIL_FROM,
	// 			bcc: [process.env.HUBSPOT_BCC],
	// 			subject: 'please setup a call with '+companyreceiver.companyName+ ' from '+companysender.companyName ,
	// 			template: 'MEETING_REQUEST_MAIL',
	// 			context: {
	// 				//title: "Meeting Request",
	// 				Requester_Company:companysender.companyName,
	// 				Requester_Username:sender.firstName,
	// 				Requester_Email:sender.email,
					
	// 				Receiver_Company:companyreceiver.companyName,
	// 				Receiver_Username:receiver.firstName,
	// 				Receiver_Email:receiver.email,
	// 				unsub_url: process.env.FRONTEND_BASE + '/auth/unsubscribe/'+meetingDto.receiverContactId+'?token='+unsub_token,
	// 				//cdnUrl: process.env.CDN_URL,
	// 				//banner: process.env.EMAIL_BANNER,
	// 				//home_url: process.env.FRONTEND_BASE,
	// 			//	full_name: userDto.contact.firstName, // + ' ' + userDto.contact.lastName
	// 				//login_url: process.env.FRONTEND_BASE + "/" + process.env.FRONTEND_LOGIN,
	// 				//email: user.email,
	// 				//password: password,
	// 				//ownername: owner.contact.firstName + ' ' + owner.contact.lastName,
	// 				//login_url_path: "/" + process.env.FRONTEND_LOGIN,
	// 			}
	// 		}).catch((e) => {
	// 			console.log(e);
	// 		});

	// 		if (result) {
	// 			return true;
	// 		}
	// 	} catch (e) {
	// 		console.log("Error + ", e);
	// 	}

	// 	return false;
	// }

	async getMeetingList(filter){
		let limit = ' LIMIT ' + filter.limit;
		let offset = ' OFFSET ' + filter.offset;
		let q ='select count(*) OVER() AS full_count, mr."mettingReqId" as meetingReqId, mr."isMeetingConfirm", ch."companyName" as Requester_Company,tc."firstName" as Requester_Username,tc."email" as Requester_Email,rch."companyName" as Receiver_Company,rtc."firstName" as Receiver_Username,rtc."email" as Receiver_Email from "tblMeetingReq" as mr inner join "tblContact" as tc on  mr."senderContactId"=tc."contactId" inner join "tblChannel" as ch on mr."senderChannelId"=ch."channelId" inner join "tblContact" as rtc on  mr."receiverContactId"=rtc."contactId"  inner join "tblChannel" as rch on mr."receiverChannelId"=rch."channelId" where mr."mettingReqId"=mr."mettingReqId" and mr."isMeetingConfirm" != 1 and mr."deletedAt" is null';
		q += limit+offset;
		return await this.sequelizeInstance.query(q, {
			type: QueryTypes.SELECT
		});
	}

	async updateMeeting(confirmMeeting:any): Promise<any>
	{
		await this.meetingRepository.update({meeting_Scheduled_At : confirmMeeting.meeting_Scheduled_At,isMeetingConfirm:confirmMeeting.isMeetingConfirm }, {
			where: {							
				mettingReqId: confirmMeeting.meetingReqId,
				deletedAt: null
			}//, returning: true
		});
		
	}

   /* async contactMail(reqObject: SendMailDto, user): Promise<boolean> {
		let result = await this.notificationService.sendNotificationRouter(Object.assign({
			senderId: +user.contactId
		}, reqObject), notification.MAILMESSAGE);

		if (result) {
			return true;
		}

		throw new ProcessException("notification:presetmail:mailNotAbletoSend");
    }*/
    
    async getContactMail(contactId) {
		return await this.contactRepository.findOne<Contact>({
			where: {
				contactId: contactId
			},
			
		});
	}

	async getCompanyMail(channelId) {
		return await this.channelRepository.findOne<Channel>({
			where: {
				channelId: channelId
			},
			
		});
	}
        
        
	}