import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, BeforeCreate, AllowNull, BeforeUpdate, ForeignKey, BelongsTo, Scopes } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { SampleOrder } from './SampleOrder.entity';

var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblordershipment' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
	orderInfo: {
		attributes: ['ordershipmentid', 'name', 'trackingno', 'trackingurl', 'trackingurl', 'comments']
	}
}))

export class OrderShipment extends Model<OrderShipment>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	ordershipmentid: number

	@ForeignKey(() => SampleOrder)
	@Column(DataType.BIGINT)
	orderid: number

	@BelongsTo(() => SampleOrder, 'orderid')
	order: SampleOrder;

	@Column(DataType.CHAR(255))
	name: string

	@Column(DataType.CHAR(255))
	trackingno: string

	@Column(DataType.CHAR(255))
	trackingurl: string

	@Column(DataType.CHAR(5000))
	comments: string

	@AllowNull(false)
	@Column(DataType.DATE)
	createdat: Date

	@Column(DataType.DATE)
	updatedat: Date;

	@BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(orderShipment: OrderShipment, options: any) { 
		orderShipment.updatedat = Sequelize.fn('NOW'); 
	}
}
