import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, ForeignKey, BelongsTo, Scopes, HasOne, HasMany } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Channel } from '../../channel/models/Channel.entity';
import { User } from '../../user/models/User.entity';
import { DiscountUnit } from '../../product/models/DiscountUnit.entity';
import { PaymentTransaction } from '../../channel/models/PaymentTransaction.entity';
import { OrderShipment } from './OrderShipment.entity';
import { OrderStatus } from './OrderStatus.entity';
import { SampleOrderDetail } from './SampleOrderDetail.entity';
import { OrderAddress } from './OrderAddress.entity';
import { Contact } from './../../user/models/Contact.entity';
import { Currency } from '../../channel/models/Currency.entity';
const tableOptions: IDefineOptions = { timestamps: false, tableName: 'tblsampleorder' } as IDefineOptions;

@Table(tableOptions)

export class SampleOrder extends Model<SampleOrder>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	sampleorderid: number

	@Column(DataType.CHAR(255))
	ordernumber: string

	@ForeignKey(() => User)
	@Column(DataType.BIGINT)
	userid: number

	@BelongsTo(() => User)
	user: User;

	buyer_contact: Contact;
	seller_contact: Contact;
	contact: Contact;

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	sellerchannelid: number

	@BelongsTo(() => Channel, 'sellerchannelid')
	sellerchannel: Channel;

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	buyerchannelid: number

	@BelongsTo(() => Channel, 'buyerchannelid')
	buyerchannel: Channel;

	@ForeignKey(() => OrderAddress)
	@Column(DataType.BIGINT)
	billingaddressid: number

	@BelongsTo(() => OrderAddress, 'billingaddressid')
	billingaddress: OrderAddress;

	@ForeignKey(() => OrderAddress)
	@Column(DataType.BIGINT)
	shippingaddressid: number

	@BelongsTo(() => OrderAddress, 'shippingaddressid')
	shippingaddress: OrderAddress;

	@ForeignKey(() => OrderStatus)
	@Column(DataType.BIGINT)
	statusid: number

	@BelongsTo(() => OrderStatus)
	status: OrderStatus;

	@ForeignKey(() => Currency)
	@Column(DataType.BIGINT)
	currencyid: number

	@BelongsTo(() => Currency)
	currency: Currency;

	@Column(DataType.BIGINT)
	subtotalprice: number

	@Column(DataType.BIGINT)
	shippingprice: number

	@Column(DataType.BIGINT)
	discount: number

	@ForeignKey(() => DiscountUnit)
	@Column(DataType.BIGINT)
	discountunitid: number

	@BelongsTo(() => DiscountUnit, 'discountunitid')
	discountunitdetail: DiscountUnit;

	@Column(DataType.BIGINT)
	othercharge: number

	@Column(DataType.BIGINT)
	ordertotalprice: number

	@ForeignKey(() => PaymentTransaction)
	@Column(DataType.BIGINT)
	paymenttranscationid: number

	@BelongsTo(() => PaymentTransaction)
	paymenttranscation: PaymentTransaction;

	@Column
	requestfreesample: string

	@Column(DataType.CHAR(5000))
	comments: string

	@AllowNull(false)
	@Column(DataType.DATE)
	orderdate: Date

	@Column(DataType.DATE)
	updatedat: Date;

	@HasMany(() => SampleOrderDetail, 'sampleorderid')
	orderDetails: SampleOrderDetail[];

	@HasMany(() => OrderShipment, 'orderid')
	shipments: OrderShipment[];

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	canceledby: number

	@BelongsTo(() => Channel, 'canceledby')
	canceled: Channel;

	@Column
	transferred: string

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	transferredto: number

	@BelongsTo(() => Channel, 'transferredto')
	transferredChannel: number

	@Column(DataType.DATE)
	transferredat: Date;

	@Column
	relation_queued: string

	@Column
	subscription_queued: string

	@Column(DataType.DATE)
	queueprocessedat: Date;

	@Column
	invoiced: string;

	@Column(DataType.CHAR(256))
	paymentDetail: string
}
