import {
	Table,
	Column,
	PrimaryKey,
	Model,
	AutoIncrement,
	DataType,
	BeforeCreate,
	AllowNull,
	BeforeUpdate,
	ForeignKey,
	BelongsTo,
	Scopes,
} from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Product } from '../../product/models/Product.entity';
import { SampleOrder } from './SampleOrder.entity';

var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamps: false, tableName: 'tblsampleorderdetail' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
	orderInfo: {
		attributes: ['sampleorderdetailid', 'sampleorderid', 'productid', 'retailerprice', 'retailerpriceunitid', 'distributormarginpercent', 'distributormarginpercentunitid', 'retailermarginpercent', 'retailermarginpercentunitid', 'quantity', 'finalquantity', 'samplepriceperquantity', 'finaltotalprice', 'totalprice']
	}
}))

export class SampleOrderDetail extends Model<SampleOrderDetail>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	sampleorderdetailid: number

	@ForeignKey(() => SampleOrder)
	@Column(DataType.BIGINT)
	sampleorderid: number

	@BelongsTo(() => SampleOrder)
	order: SampleOrder;

	@ForeignKey(() => Product)
	@Column(DataType.BIGINT)
	productid: number

	@BelongsTo(() => Product)
	product: Product;

	@Column(DataType.BIGINT)
	quantity: number

	@Column(DataType.BIGINT)
	finalquantity: number

	@Column(DataType.FLOAT(15, 2))
	samplepriceperquantity: number

	@Column(DataType.FLOAT(15, 2))
	finaltotalprice: number

	@Column(DataType.FLOAT(15, 2))
	totalprice: number

	@AllowNull(false)
	@Column(DataType.DATE)
	createdat: Date

	@Column(DataType.DATE)
	updatedat: Date;

	@Column(DataType.DATE)
	deletedat: Date;

	@BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(sampleOrderDetail: SampleOrderDetail, options: any) { 
		sampleOrderDetail.updatedat = Sequelize.fn('NOW'); 
	}
}
