'use strict';

import { Inject, Injectable } from '@nestjs/common';
//import { BaseService } from '../../core';
import { ProcessException } from './../../../middleware/exception/Process.exception';
import { MyAddressBook } from './../../user/models/MyAddressBook.entity';
import { ProductQuote } from './../../cart/models/ProductQuote.entity';
import { Cart } from './../../cart/models/Cart.entity';
import { SampleOrder } from './../models/SampleOrder.entity';
import { SampleOrderDetail } from './../models/SampleOrderDetail.entity';
import { OrderAddress } from './../models/OrderAddress.entity';
//import { ProductIndexService } from './../../product/service/productIndex.service';
import { Channel } from './../../channel/models/Channel.entity';
import { AuthException } from './../../../middleware/exception/Auth.exception';
import { ChannelSuggestion } from './../../channel/models/ChannelSuggestion.entity';
import { ChannelJCT } from './../../channel/models/ChannelJCT.entity';
//import { notification } from './../../../config/notificaton.constant';
import { NotificationService } from '../../notification/service/notification.service';
//import { IMailNotification } from '../../notification/transformation/Interface/INotificatonSerivceInBound';
import { Contact } from './../../user/models/Contact.entity';
import { Currency } from '../../channel/models/Currency.entity';
import { appconstant, features } from './../../../config';
import { Brand } from '../../product/models/Brand.entity';
import { ProductGroup } from '../../product/models/ProductGroup.entity';
import { ProductCategory } from '../../product/models/ProductCategory.entity';
import { ProductPriceDetail } from '../../product/models/ProductPriceDetail.entity';
import { ProductImage } from '../../product/models/ProductImage.entity';
import { Unit } from '../../product/models/Unit.entity';
import { DiscountUnit } from '../../product/models/DiscountUnit.entity';
import { PaymentTransaction } from '../../channel/models/PaymentTransaction.entity';
import { PaymentType } from '../../channel/models/PaymentType.entity';
import { Media } from '../../media/models/Media.entity';
import { Product } from '../../product/models/Product.entity';
import { User } from '../../user/models/User.entity';
//import { BaseHelper } from './../../core/helper/BaseHelper';
//import { ChannelHubService } from './../../channel/service/channelHub.service';
import { isBoolean } from 'lodash';
import {Op, QueryTypes } from "sequelize";

@Injectable()
export class CheckoutService {
	constructor(
		//private productIndex: ProductIndexService,
		//private channelHubService: ChannelHubService,
		@Inject('SequelizeInstance') private readonly sequelizeInstance,
		@Inject('MyAddressBookRepository') private readonly addressBook: typeof MyAddressBook,
		@Inject('CartRepository') private readonly cart: typeof Cart,
		@Inject('OrderRepository') private readonly orderRepository: typeof SampleOrder,
		// @Inject('RedisInstance') private readonly redisInstance,
		//@Inject('OrderDetailRepository') private readonly orderDetailRepository: typeof SampleOrderDetail,
		// @Inject('ChannelRepository') private readonly channelRepository: typeof Channel,
		// @Inject('UserRepository') private readonly userRepository: typeof User,
		// @Inject('ContactRepository') private readonly contactRepository: typeof Contact,
		//@Inject('orderAddressRepository') private readonly orderaddressRepository: typeof OrderAddress,
		// @Inject('ProductRepository') private readonly productRepository: typeof Product,
		// @Inject('ProductGroupRepository') private readonly productGroupRepository: typeof ProductGroup,
		// @Inject('ProductCategoryRepository') private readonly productCategoryRepository: typeof ProductCategory,
		// @Inject('BrandRepository') private readonly brandRepository: typeof Brand,
		// @Inject('ProductPriceDetailRepository') private readonly productPriceDetailRepository: typeof ProductPriceDetail,
		// @Inject('ProductImageRepository') private readonly productImageRepository: typeof ProductImage,
		// @Inject('MediaRepository') private readonly mediaRepository: typeof Media,
		// @Inject('UnitRepository') private readonly unitRepository: typeof Unit,
		// @Inject('DiscountUnitRepository') private readonly discountUnitRepository: typeof DiscountUnit,
		// @Inject('PaymentTransactionRepository') private readonly paymentTransactionRepository: typeof PaymentTransaction,
		// @Inject('PaymentTypeRepository') private readonly paymentTypeRepository: typeof PaymentType,
		//private notificationService: NotificationService
	) {
		
	}


	async validateAndCollectCart(data, user) {
		//load cart data
		let cart = await this.cart.findOne({
			where: {
				deletedat: null,
				userid: user.userId,
				cartid: data.cartId
			},
			include: [
				{
					model: ProductQuote,
					as: 'items',
					where: {
						deletedat: null
					},
					include: [
						{
							model: Product,
							as: 'product',
							required: true,
							attributes: ['product', 'productSKU', 'isactive']
						}
					]
				},
				{
					model: Channel,
					as: 'seller',
					required: true
				}
			]
		});

		if (!cart) {
			throw new ProcessException("Given cart already processed or not matching with active cart");
		}

		if (cart.grandtotal > appconstant.ORDER_GRAND_TOTAL_AMOUNT_LIMIT) {
			throw new ProcessException("Total Amount should not be greater than " + appconstant.ORDER_GRAND_TOTAL_AMOUNT_LIMIT);
		}

		//Inactive product check
		if (cart.items) {
			let inactiveProducts = [];
			cart.items.forEach((item) => {
				if (item.product.isactive == '0') {
					inactiveProducts.push(item.product);
				}
			});

			if (inactiveProducts.length > 0) {
				throw new ProcessException({ 'inactiveProduct': inactiveProducts });
			}
		}

		return cart;
	}

	async makeOrder(cart, data, user) { //, subscriptionFeatureKeys) {

		let CollectAddressIds = [data.shippingAddressId];
		if (data.billingAddressId != data.shippingAddressId) {
			CollectAddressIds.push(data.billingAddressId);
		}

		// load Address from user AddressBook
		let address = await this.addressBook.findAll({
			where: {
				userid: user.userId,
				addressid: {
					[Op.in]: CollectAddressIds
				}
			},
			order: [['addressid', 'DESC']]
		});
		
		let billingaddress:any = null;
		let shippingaddress:any = null;
		await address.forEach((cdata) => {
			if (cdata.addressid == data.billingAddressId) {
				billingaddress = cdata.toJSON();

				delete billingaddress['addressid'];
				delete billingaddress['isbillingaddress'];
				delete billingaddress['isshippingaddress'];
			}

			if (cdata.addressid == data.shippingAddressId) {
				shippingaddress = cdata.toJSON();

				delete shippingaddress['addressid'];
				delete shippingaddress['isbillingaddress'];
				delete shippingaddress['isshippingaddress'];
			}
		});

		if (billingaddress == null || shippingaddress == null) {
			throw new ProcessException('shipping and billing addresses are mandatory')
		}

		let orderDetails: SampleOrderDetail[] = [];
		cart.items.forEach((item) => {
			let price = +item.quantity * item.unitprice;

			let details = <SampleOrderDetail>{};

			details.productid = item.productid;
			details.quantity = item.quantity;
			details.finalquantity = item.quantity;
			details.samplepriceperquantity = item.unitprice;
			details.finaltotalprice = +price.toFixed(2);
			details.totalprice = +price.toFixed(2);
			details.createdat = this.sequelizeInstance.fn('NOW');

			orderDetails.push(details);
		})

		let order = await this.sequelizeInstance.transaction(async transaction => {
			return await this.orderRepository.create({
				userid: user.userId,
				buyerchannelid: user.channelId,
				sellerchannelid: cart.sellerchannelid,
				billingaddress: billingaddress,
				shippingaddress: shippingaddress,
				statusid: appconstant.ORDER_STATUS_PENDING_CONFIRMATION,
				currencyid: cart.currencyid,
				subtotalprice: cart.subtotal,
				shippingprice: 0,
				discount: cart.discount,
				othercharge: 0,
				ordertotalprice: cart.grandtotal,
				requestfreesample: (data.requestFreeSample) ? '1' : '0',
				orderdate: this.sequelizeInstance.fn('NOW'),
				queueprocessedat: this.sequelizeInstance.fn('NOW'),
				invoiced: '0',
				orderDetails: orderDetails,
				relation_queued: data['relation_queued']
			}, {
					include: [{
						model: OrderAddress,
						as: 'billingaddress'
					}, {
						model: OrderAddress,
						as: 'shippingaddress'
					}, {
						model: SampleOrderDetail
					}],
					transaction
				}).then(async (order) => {
					// update order number
					let ordernumber = await this.sequelizeInstance.query('select \'ORD\' || ((EXTRACT(YEAR FROM now()):: integer * 10000) + (EXTRACT(MONTH FROM now()):: integer * 100) + (EXTRACT(day FROM now()):: integer * 1)) :: varchar(8) || (count(*) + 1) :: varchar(10) as number from tblsampleorder where to_char(orderdate, \'dd/mm/yyyy\') = to_char(now(), \'dd/mm/yyyy\') and "sampleorderid" <= ' + order.sampleorderid, {
						type: this.sequelizeInstance.QueryTypes.SELECT,
						raw: true,
					});

					// Verify subscription
					let sellerSubscriptionId = cart.seller.subscriptionId;
					let key = +sellerSubscriptionId + '-' + +features.SAMPLE_ORDER_MGMT;

					let query = 'select tc."channelSubscriptionId", tc."channelId", count(ts."sampleorderid"), to_date( to_char(now(),\'dd/mm/yyyy\'),\'dd/mm/yyyy\') >= to_date( to_char(tc."endAt",\'dd/mm/yyyy\'),\'dd/mm/yyyy\') as expired from "tblChannelSubscription" tc left outer join "tblsampleorder" ts on tc."channelId" = ts."sellerchannelid" and to_date( to_char(ts.queueprocessedat,\'dd/mm/yyyy\'),\'dd/mm/yyyy\') between to_date( to_char(tc."startedAt",\'dd/mm/yyyy\'),\'dd/mm/yyyy\') AND to_date( to_char(tc."endAt",\'dd/mm/yyyy\'),\'dd/mm/yyyy\') and ts.relation_queued = \'0\' and ts.subscription_queued = \'0\' where tc."channelId" = :sellerchannelid and tc."isActive"=\'1\' group by tc."channelSubscriptionId", tc."channelId", tc."endAt" LIMIT 1';

					let pastOrders = await this.sequelizeInstance.query(query, {
						type: this.sequelizeInstance.QueryTypes.SELECT,
						raw: true,
						replacements: {
							sellerchannelid: order.sellerchannelid
						}
					}).then((results) => {
						return (results[0]) ? results[0] : null;
					});

					// if (isBoolean(pastOrders.expired) && pastOrders.expired || (isBoolean(pastOrders.expired) && !pastOrders.expired && subscriptionFeatureKeys[key] && subscriptionFeatureKeys[key] != -1 && +pastOrders.count >= subscriptionFeatureKeys[key])) {
					// 	order.subscription_queued = '1';
					// }

					order.ordernumber = ordernumber[0].number;
					await order.save({
						transaction
					});

					// delete the cart
					cart.deletedat = this.sequelizeInstance.fn('NOW');
					await cart.save({
						transaction
					});

					// index to calculate
					// cart.items.forEach((item) => {
					// 	// start product index
					// 	this.productIndex.index(+item.productid);
					// });

					return order;
				});
		}).then((result) => {
			return result;
		}).catch((err) => {
			console.log(err);
			throw new ProcessException('validate:sql:internalError');
		});

		// let notiObject = <IMailNotification>{};
		// notiObject.receiverId = await this.channelHubService.getDefaultContact(order.sellerchannelid, user, {
		// 	contactType: 'SELLING'
		// });
		// notiObject.senderId = user.contactId;
		// this.sendnotification(notiObject, order.ordernumber, user);

		return order.ordernumber;
	}

	// public async sendnotification(notiObject, ordernumber, user): Promise<void> {

	// 	let filter = <SampleOrder>{
	// 		buyerchannelid: user.channelId,
	// 		ordernumber: ordernumber
	// 	};

	// 	let order = await this.orderRepository.findOne<SampleOrder>({
	// 		where: filter,
	// 		include: [{
	// 			model: this.orderDetailRepository,
	// 			as: 'orderDetails',
	// 			include: [
	// 				{
	// 					model: this.productRepository,
	// 					include: [
	// 						{
	// 							model: this.productPriceDetailRepository.scope('productExclude'),
	// 							where: { deletedat: null },
	// 							required: false,
	// 							include: [
	// 								{
	// 									model: this.unitRepository,
	// 									as: 'retailerpriceunitdetail',
	// 									required: false
	// 								},
	// 								{
	// 									model: this.unitRepository,
	// 									as: 'samplepriceunitdetail',
	// 									required: false
	// 								},
	// 								{
	// 									model: this.discountUnitRepository,
	// 									as: 'distributorunitdetail',
	// 									required: false
	// 								},
	// 								{
	// 									model: this.discountUnitRepository,
	// 									as: 'retailerunitdetail',
	// 									required: false
	// 								},
	// 								{
	// 									model: Currency
	// 								}
	// 							],
	// 						},
	// 						{
	// 							model: this.productImageRepository.scope('productImage'),
	// 							as: "defaultImage",
	// 							where: { deletedat: null },
	// 							include: [
	// 								{
	// 									model: this.mediaRepository,
	// 									where: { deletedAt: null },
	// 									required: false
	// 								}
	// 							],
	// 							required: true
	// 						},
	// 						{
	// 							model: this.productGroupRepository,
	// 							as: 'productGroup',
	// 							where: { deletedAt: null },
	// 							required: true
	// 						},
	// 						{
	// 							model: this.productCategoryRepository,
	// 							as: 'productCategory',
	// 							where: { deletedAt: null },
	// 							required: true
	// 						},
	// 						{
	// 							model: this.brandRepository,
	// 							where: { deletedAt: null },
	// 							required: true
	// 						}
	// 					]
	// 				}
	// 			]

	// 		}, {
	// 			model: this.channelRepository,
	// 			as: 'buyerchannel',
	// 			required: true,
	// 			include: [
	// 				{
	// 					model: this.contactRepository.scope('basicInfom'),
	// 					as: 'contacts',
	// 					where: { contactId: user.contactId },
	// 					required: false
	// 				}
	// 			]
	// 		}, {
	// 			model: this.channelRepository,
	// 			as: 'sellerchannel',
	// 			required: true,
	// 			include: [
	// 				{
	// 					model: this.contactRepository.scope('basicInfom'),
	// 					as: 'contacts',
	// 					where: { isDefaultContact: '1' },
	// 					required: false
	// 				}
	// 			]
	// 		}, {
	// 			model: this.userRepository.scope('userBasicInfom'),
	// 			include: [
	// 				{
	// 					model: this.contactRepository.scope('senderContactMail')
	// 				}
	// 			]
	// 		}, {
	// 			model: this.orderaddressRepository.scope('orderInfo'),
	// 			as: 'shippingaddress'
	// 		}, {
	// 			model: this.paymentTransactionRepository,
	// 			include: [
	// 				{
	// 					model: this.paymentTypeRepository
	// 				}
	// 			]
	// 		}
	// 		]
	// 	});

	// 	// lazyload currency
	// 	let helper = new BaseHelper();
	// 	order = await helper.lazyload(order, 'currency');

	// 	notiObject.order = order;
	// 	this.notificationService.sendNotificationRouter(notiObject, notification.ORDER_CHECKOUT);
	// }
}
