'use strict';

import { Controller, Post, Request, Response, Param, Body, Put, UseGuards, HttpStatus, Get, Delete, ValidationPipe, Res, Req } from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import { CreateBrandDto } from '../dto/createBrand.dto';
import { UpdateBrandDto } from '../dto/UpdateBrand.dto';
import { ProductAdminService } from '../service/product.admin.service';

@UseGuards(AuthGuard('jwt'))
@Controller('master')
export class ProductAdminController {
	constructor(
		private productAdminService: ProductAdminService
	) {	}

	@Post('/brand')
	public async brandCreate(@Request() req, @Response() res, @Body(new ValidationPipe()) createBrandDto: CreateBrandDto) {
		let result = await this.productAdminService.createBrand(createBrandDto);
		return res.status(HttpStatus.CREATED).send(result);
	}

	@Get('getBrands')
	async getBrands(@Req() req, @Res() res) {
		// let showall = false;
		// if (req.query.showall == 'true') {
		// 	showall = true;
		// }
		console.log('req',req);
		let result: any = await this.productAdminService.getBrands(req.query);
		res.status(HttpStatus.OK).json(
			// this.paginationTransform.make(result.list, pagination, result.total)
			result
		);
	}

	@Get('/brand/:id')
	public async getBrand(@Param('id') id, @Request() req, @Response() res) {
		const row = await this.productAdminService.getBrand(id);
		return res.status(HttpStatus.OK).json(row);
	}

	@Put('/brand/:id')
	public async brandUpdate(@Param('id') id, @Request() req, @Response() res, @Body(new ValidationPipe()) updateBrandDto: UpdateBrandDto) {
		let result = await this.productAdminService.updateBrand(id, updateBrandDto);
		return res.status(HttpStatus.CREATED).send(result);
	}

}
