'use strict';

import { Inject, Controller, Get, HttpStatus, Request, Response, Param, ParseIntPipe, Post, ValidationPipe, Body } from '@nestjs/common';
import { ProductService } from '../service/product.service';
import { ProductServiceOutBound } from '../transformation/productServiceOutBound';
import { QueryTypes } from "sequelize";

@Controller('public')
export class PublicProductController {
	private productServiceOutBound = new ProductServiceOutBound();
	constructor(
		private productService: ProductService,
		@Inject('SequelizeInstance') private readonly sequelizeInstance,
	) {
	}

	@Get('Product/:sku')
	public async getProduct(@Param('sku') sku, @Request() req, @Response() res) {
		let productDetail = await this.productService.getProductBySKU(sku);
		console.log('productDetail',productDetail);
		return res.status(HttpStatus.OK).json(
			await this.productServiceOutBound.publicProductCustomerProfile(
				await this.productService.getPublicProduct(sku)
			)
		);
	}

	@Post('channel-products/:channelId')
	async channelProductFindAll(@Request() req, @Param('channelId', new ParseIntPipe()) channelId: number, @Response() res, @Body(new ValidationPipe()) productFilters: any) {
		// let user = req.user;
		let query = 'select tp."brandId",tp."productId",tp."product", tp."productSKU", tp."productqualitylevel", pr."businesscurrencyid",pr."retailerprice", br."brandName",(select concat( \'' + process.env.CDN_URL + '\',"documentPath") as "brandDocumentUrl" from "tblDocument" where "documentId" = br."documentId"),(select concat( \'' + process.env.CDN_URL + '\',"documentPath") as "documentUrl" from "tblDocument" where "documentId" in(select "imageid" from "tblproductimage" where "productimageid"=tp."defaultimageid"))from "tblProduct" as tp inner join "tblBrand" as br on br."id"=tp."brandId" inner join "tblproductpricedetail" as pr on pr."productId"=tp."productId" where "channelId"= :channelId and tp."deletedat" is null and "isactive"=\'1\' and "isPublic"=\'1\'';
		// query += ' and tp."productId" not in(select "productid" as "productId" from "tblproductrejection" where channelid = ' + user.channelId+')';
		if(productFilters.excludedProductSku)
		query += ' and tp."productSKU" !=:excludedProductSku';
		// if(type!=2)
		// query += ' and tp."productId" in(select "productId" from "tblproductsharepreference" where sharetypeid = :sharedId and "productId" = tp."productId" and "deletedat" IS NULL)';
		let result=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
			raw: true,
		replacements: {
			channelId:channelId,
			excludedProductSku:productFilters.excludedProductSku?productFilters.excludedProductSku:null,
		}
		})
		res.status(HttpStatus.OK).send(result);
	}
	
	@Get('channel-similarproducts/:channelId')
	async channelSimilarProduct(@Request() req, @Param('channelId', new ParseIntPipe()) channelId: number, @Response() res) {
		let products=[];
		let query = 'select tp."brandId" from "tblProduct" as tp where "channelId"= :channelId and tp."deletedat" is null and "isactive"=\'1\'';
		// query += ' and tp."productId" not in(select "productid" as "productId" from "tblproductrejection" where channelid = ' + user.channelId+')';
		// query += ' and tp."productId" in(select "productId" from "tblproductsharepreference" where sharetypeid = :sharedId and "productId" = tp."productId" and "deletedat" IS NULL)';
		let result=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
			raw: true,
		replacements: {
			channelId:channelId
		}
		})
		if(result.length>0){
			let brandIds=result.map(x=>+x.brandId);
			console.log('brandIds', brandIds);
			let query = 'select tp."brandId",tp."productId",tp."product", tp."productSKU", tp."productqualitylevel", pr."businesscurrencyid",pr."retailerprice", br."brandName",(select concat( \'' + process.env.CDN_URL + '\',"documentPath") as "brandDocumentUrl" from "tblDocument" where "documentId" = br."documentId"),(select concat( \'' + process.env.CDN_URL + '\',"documentPath") as "documentUrl" from "tblDocument" where "documentId" in(select "imageid" from "tblproductimage" where "productimageid"=tp."defaultimageid"))from "tblProduct" as tp inner join "tblBrand" as br on br."id"=tp."brandId" inner join "tblproductpricedetail" as pr on pr."productId"=tp."productId" where "channelId"!= :channelId and "brandId" in(:brandIds) and tp."deletedat" is null and "isactive"=\'1\' and "isPublic"=\'1\'';
			// query += ' and tp."productId" not in(select "productid" as "productId" from "tblproductrejection" where channelid = ' + user.channelId+')';
			// query += ' and tp."productId" in(select "productId" from "tblproductsharepreference" where sharetypeid = :sharedId and "productId" = tp."productId" and "deletedat" IS NULL)';
			products=await this.sequelizeInstance.query(query, {
				type: QueryTypes.SELECT,
				raw: true,
			replacements: {
				channelId:channelId,
				// sharedId:type==2?1:type==3?2:type==4?3:4,
				brandIds:brandIds
			}
			});
		}
		res.status(HttpStatus.OK).send(products);
	}
}
