import {
	IsString,
	IsEmail,
	IsNotEmpty,
	MinLength,
	MaxLength,
	Min,
	Max,
	IsNumber,
	ArrayNotEmpty,
	ValidateIf,
	ValidateNested,
	IsOptional,
	IsDate,
	IsDateString
} from 'class-validator';
import { Type } from 'class-transformer';
import { ApiProperty } from '@nestjs/swagger';
import { ProductTagDto } from './productTag.dto';
export class AddProductBasicInfoDto {

	@IsNotEmpty({
		message: "channel:product:productNameIsNotEmpty",
		groups: ["import"]
	})
	@MinLength(3, {
		message: "channel:product:productNameMinLen",
		groups: ["import"]
	})
	@MaxLength(256, {
		message: "channel:product:productNameMaxLen",
		groups: ["import"]
	})
	@IsString({ message: "channel:product:productNameIsString" })
	@ApiProperty()
	readonly productname: string;

	// @IsNotEmpty({
	// 	message: "Product Short Description should not be empty",
	// 	groups: ["import"]
	// })
	// @MaxLength(256, {
	// 	message: "Product Short Description  Must Have Max 30 Characters",
	// 	groups: ["import"]
	// })
	@IsString({ message: "channel:product:productNameIsString" })
	@ApiProperty()
	readonly productshortdec: string;

	@IsNotEmpty({ message: "channel:product:productStatusIsNotEmpty" })
	@IsString({ message: "channel:product:productStatusIsString" })
	@ApiProperty()
	readonly productstatus: string;

	@IsOptional()
	@IsString({ message: "channel:product:productPitchAwardIsString" })
	@ApiProperty()
	readonly product_pitch_award: string;

	@IsNotEmpty({
		message: "channel:product:productGroupIsNotEmpty",
		groups: ["import"]
	})
	@IsNumber({}, { message: "channel:product:productGroupIsNumber" })
	@ApiProperty()
	readonly productgroup: number;

	@IsNotEmpty({
		message: "channel:product:productCategoryIsNotEmpty",
		groups: ["import"]
	})
	@IsNumber({}, { message: "channel:product:productCategoryIsNumber" })
	@ApiProperty()
	readonly productcategory: number;

	@IsNotEmpty({
		message: "channel:product:productBrandIsNotEmpty",
		groups: ["import"]
	})
	@IsNumber({}, { message: "channel:product:productBrandIsNumber" })
	@ApiProperty()
	readonly productbrand: number;

	@IsOptional()
	// @MinLength(5, {
	// 	message: "channel:product:productDescMinLen",
	// 	groups: ["import"]
	// })
	// @MaxLength(8000, {
	// 	message: "channel:product:productDescMaxLen",
	// 	groups: ["import"]
	// })
	@IsString({ message: "channel:product:productDescIsString" })
	@ApiProperty()
	readonly productdesc: string;

	@IsOptional()
	@MaxLength(2000, {
		message: "validate:field:validateMaxLength"
	})
	@IsString({ message: "validate:field:shouldBeString" })
	@ApiProperty()
	readonly keyretailers: string

	@IsOptional()
	@MaxLength(2000, {
		message: "validate:field:validateMaxLength"
	})
	@IsString({ message: "validate:field:shouldBeString" })
	@ApiProperty()
	readonly keydistributors: string

	@IsOptional()
	@MaxLength(2000, {
		message: "validate:field:validateMaxLength"
	})
	@IsString({ message: "validate:field:shouldBeString" })
	@ApiProperty()
	readonly keyresellers: string

	// @IsNotEmpty({
	// 	message: "channel:product:productUspIsNotEmpty",
	// 	groups: ["import"]
	// })
	// @MinLength(15, {
	// 	message: "channel:product:productUspMinLen",
	// 	groups: ["import"]
	// })
	// @MaxLength(5000, {
	// 	message: "channel:product:productUspMaxLen",
	// 	groups: ["import"]
	// })
	@IsOptional()
	@IsString({ message: "channel:product:productUspIsString" })
	@ApiProperty()
	readonly productusp: string;

	@IsOptional()
	@ApiProperty({ type: ProductTagDto, isArray: true })
	@ArrayNotEmpty({ message: "channel:product:productTagArrayNotEmpty" })
	@IsNotEmpty({ message: "channel:product:productTagIsNotEmpty" })
	@ValidateNested()
	@Type(() => ProductTagDto)
	readonly producttag: ProductTagDto[];

	@IsOptional()
	@IsNumber({}, { message: "channel:product:productBusinessTypeIsNumber" })
	@ApiProperty()
	readonly businessType: number;


}
