import { IsString, IsEmail, IsNotEmpty, MinLength, MaxLength, Min, Max, IsNumber, IsInt, ArrayNotEmpty, ValidateIf, ValidateNested, IsOptional, IsDate, IsDateString } from 'class-validator';
import { Type } from 'class-transformer';
//import { ApiProperty } from '@nestjs/swagger';
import { AddProductBasicInfoDto } from './AddProduct/addProductBasicInfo.dto';
import { AddProductArticleReviewDto } from './AddProduct/addProductArticleReview.dto';
import { AddProductImageDto } from './AddProduct/addProductImage.dto';
import { AddProductBrochureDto } from './AddProduct/addProductBrochure.dto';
import { AddProductPreferencesDto } from './AddProduct/addProductPreferences.dto';
import { AddProductCodeDto } from './AddProduct/addProductCode.dto';
import { AddProductPriceDto } from './AddProduct/addProductPrice.dto';
import { AddProductPackageInfoDto } from './AddProduct/addProductPackageInfo.dto';
import { AddProductCompetingDto } from './AddProduct/addProductCompeting.dto';

import { AddProductPositionDto } from './AddProduct/addProductPosition.dto';
import { AddProductRelevantChannelDto } from './AddProduct/addProductRelevantChannel.dto';
import { AddProductSellingLocationDto } from './AddProduct/addProductSellingLocation.dto';
import { AddProductTargetLocationDto } from './AddProduct/addProductTargetLocation.dto';
import { AddProductAvailabilityDto } from './AddProduct/addProductAvailability.dto';

import { ProductAwardsDto } from './productAwards.dto';
import { ChannelKeyRetailerDto } from '../../channel/dto/channelkeyretailer.dto';
import { ApiProperty } from '@nestjs/swagger';

export class AddProductDto {

	@ApiProperty({ type: AddProductBasicInfoDto })
	@IsNotEmpty({ message: "channel:product:productBasicInfoIsNotEmpty" })
	@ValidateNested({ groups: ["import"] })
	@Type(() => AddProductBasicInfoDto)
	readonly productBasicInfo: AddProductBasicInfoDto;

	@ApiProperty()
	@IsNotEmpty({ message: "channel:product:productBasicInfoIsNotEmpty" })
	@ValidateNested({ groups: ["import"] })
	readonly ProductInfo:any;


	@IsOptional({ groups: ["import"] })
	@ApiProperty({ type: AddProductArticleReviewDto })
	@IsNotEmpty({ message: "channel:product:productArticleReviewIsNotEmpty" })
	@ValidateNested({ groups: ["import"] })
	@Type(() => AddProductArticleReviewDto)
	readonly productArticleReview: AddProductArticleReviewDto;

	@ApiProperty({ type: AddProductImageDto })
	@IsNotEmpty({ message: "channel:product:productImagesIsNotEmpty" })
	// @ValidateNested({ groups: ["import"] })
	@Type(() => AddProductImageDto)
	readonly productImages: AddProductImageDto;

	@IsOptional()
	@ApiProperty({ type: AddProductBrochureDto })
	@IsNotEmpty({ message: "channel:product:productBrochureIsNotEmpty" })
	@ValidateNested()
	@Type(() => AddProductBrochureDto)
	readonly productBrochure: AddProductBrochureDto;

	@ApiProperty({ type: AddProductPreferencesDto, isArray: true })
	@IsNotEmpty({ message: "channel:product:productPreferencesIsNotEmpty" })
	@ValidateNested()
	@Type(() => AddProductPreferencesDto)
	readonly productPreferences: AddProductPreferencesDto;

	@ApiProperty({ type: AddProductCodeDto })
	@IsNotEmpty({ message: "channel:product:productCodesIsNotEmpty" })
	@ValidateNested()
	@Type(() => AddProductCodeDto)
	readonly productCode: AddProductCodeDto;

	@ApiProperty({ type: AddProductPriceDto })
	@IsNotEmpty({ message: "channel:product:productPriceIsNotEmpty" })
	@ValidateNested()
	@Type(() => AddProductPriceDto)
	readonly productPrice: AddProductPriceDto;

	@ApiProperty({ type: AddProductPackageInfoDto })
	@IsNotEmpty({ message: "channel:product:productPackageInfoIsNotEmpty" })
	@ValidateNested()
	@Type(() => AddProductPackageInfoDto)
	readonly productPackageInfo: AddProductPackageInfoDto;

	@IsOptional()
	@ApiProperty({ type: AddProductCompetingDto })
	@IsNotEmpty({ message: "channel:product:productCompetingIsNotEmpty" })
	@ValidateNested()
	@Type(() => AddProductCompetingDto)
	readonly productCompeting: AddProductCompetingDto;

	@ApiProperty({ type: AddProductPositionDto })
	@IsNotEmpty({ message: "channel:product:productPositionIsNotEmpty" })
	@ValidateNested()
	@Type(() => AddProductPositionDto)
	readonly productPosition: AddProductPositionDto;

	@IsOptional()
	@ApiProperty({ type: AddProductRelevantChannelDto })
	@IsNotEmpty({ message: "channel:product:productRelevantChannelIsNotEmpty" })
	@ValidateNested()
	@Type(() => AddProductRelevantChannelDto)
	readonly productRelevantChannel: AddProductRelevantChannelDto;

	@IsOptional()
	@ApiProperty({ type: AddProductSellingLocationDto })
	@IsNotEmpty({ message: "channel:product:productSellingLocationIsNotEmpty" })
	@ValidateNested()
	@Type(() => AddProductSellingLocationDto)
	readonly productSellingLocation: AddProductSellingLocationDto;

	@IsOptional()
	@ApiProperty({ type: AddProductTargetLocationDto })
	@IsNotEmpty({ message: "channel:product:productTargetLocationIsNotEmpty" })
	@ValidateNested()
	@Type(() => AddProductTargetLocationDto)
	readonly productTargetLocation: AddProductTargetLocationDto;

	@ApiProperty({ type: AddProductAvailabilityDto })
	@IsNotEmpty({ message: "channel:product:productAvailabilityIsNotEmpty" })
	@ValidateNested()
	@Type(() => AddProductAvailabilityDto)
	readonly productAvailability: AddProductAvailabilityDto;

	@IsOptional()
	@ApiProperty({ type: ProductAwardsDto, description: "product awards", isArray: true })
	@ValidateNested()
	@Type(() => ProductAwardsDto)
	readonly award_details: ProductAwardsDto[];

	// @IsOptional()
	// @IsInt({
	// 	each: true,
	// 	message: "validate:field:shouldBeInt"
	// })
	// @ApiProperty({ type: 'number', isArray: true })
	// readonly keyRetailers: number

	@IsOptional()
	@ApiProperty({ type: ChannelKeyRetailerDto, isArray: true })
	@ArrayNotEmpty({ message: "channel:product:productKeyRetailerArrayNotEmpty" })
	@IsNotEmpty({ message: "channel:product:productKeyRetailerIsNotEmpty" })
	@ValidateNested()
	@Type(() => ChannelKeyRetailerDto)
	readonly keyRetailers: ChannelKeyRetailerDto[];

	@IsOptional()
	@ApiProperty({ type: ChannelKeyRetailerDto, isArray: true })
	@ArrayNotEmpty({ message: "channel:product:productKeyRetailerArrayNotEmpty" })
	@IsNotEmpty({ message: "channel:product:productKeyRetailerIsNotEmpty" })
	@ValidateNested()
	@Type(() => ChannelKeyRetailerDto)
	readonly keyDistributors: ChannelKeyRetailerDto[];

}
