import {
    Table,
    Column,
    PrimaryKey,
    Model,
    AutoIncrement,
    DataType,
    BeforeCreate,
    AllowNull,
    BeforeUpdate,
    ForeignKey,
    HasMany,
    BelongsTo,
    Scopes,
  } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
//import { DataTypeTime } from 'sequelize';
import { ApiProperty } from '@nestjs/swagger';
import { Product } from './Product.entity';
import { Event } from './Event.entity';
import { Media } from './../../media/models/Media.entity';
import { Brand } from './Brand.entity';
import { ProductCategory } from './ProductCategory.entity';
import { ProductImage } from './ProductImage.entity';

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'Award_Results' } as IDefineOptions;


@Scopes(() => ({
	productInfo: {
		attributes: ['productId', 'channelId', 'productGroupId', 'productCategorieId', 'product', 'brandId', 'productSKU', 'productkey', 'isshowtofreesub', 'isactive', 'hasanyvariant', 'variantdetails', 'productqualitylevel', 'defaultimageid', 'updatedBy', 'deletedat', 'product_pitch_award']
	},
	orderInfo: {
		attributes: ['productId', 'product', 'productSKU', 'productGroupId', 'productCategorieId', 'brandId', 'defaultimageid']
	},
	productRatingInfo: {
		attributes: ['productId', 'channelId', 'productGroupId', 'productCategorieId', 'product', 'brandId', 'productSKU', 'productkey', 'isactive', 'defaultimageid', 'productqualitylevel']
	},
	productReject: {
		attributes: {
			exclude: ['deletedat', 'createdat', 'updatedat', 'updatedby']
		},
		include: [
			{
				model:Brand,
				as: 'brand'
			},
			{
				model:ProductCategory,
				as: 'productCategory'
			},
			{
				model:ProductImage,
				as: 'defaultImage',
				include: [{
					model:Media,
					as: 'image'
				}]
			}
		]
	}
}))
export class AwardResult extends Model<AwardResult>{

    @PrimaryKey
    @AutoIncrement
    @AllowNull(false)
    @Column(DataType.BIGINT)
    id: number

    @ForeignKey(() => Product)
	@Column(DataType.BIGINT)
	@ApiProperty()
    product_id: number;
    
    @BelongsTo(() => Product)
	product: Product;

	@Column({
		type: DataType.CHAR(255),
		allowNull: false,
	})
	@ApiProperty()
	result: string

	@ForeignKey(() => Event)
	@Column(DataType.BIGINT)
	@ApiProperty()
    event_id: number;
    
	@AllowNull(false)
	@Column(DataType.DATE)
	created_at: Date

	@Column(DataType.DATE)
	updated_at: Date;

	@Column(DataType.DATE)
	deleted_at: Date;
}

