'use strict';

import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, ForeignKey, BelongsTo, Scopes, HasMany, BeforeCreate, BeforeUpdate } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Media } from '../../media/models/Media.entity';
import { ChannelBrand } from "../../channel/models/ChannelBrand.entity";
//import { ElasticQueueHelper } from './../../../search/helper/elasticQueue.helper';
//import { esearch } from './../../../config';
import { SpotDealBrand } from "../../product/models/SpotDealBrand.entity";

var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamps: false, tableName: 'tblBrand' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
	basicInfom: {
		attributes: ['id', ['brandName', 'name'], 'isAssigned', 'isVerified'],
	},
	channelDetail: {
		attributes: ['id', 'brandName', 'isActive', 'isVerified', 'deletedAt']
	},
	tradeinfo: {
		attributes: {
			exclude: ['deletedAt', 'createdAt', 'updatedAt']
		}
	},
	brandMaster: {
		where: {
			deletedAt: null
		},
		attributes: {
			exclude: ['deletedAt', 'createdAt', 'updatedAt']
		},
		include: [
			{
				model:Media,
				attributes: ['documentId', 'documentPath', 'thumbnailUrl', 'documentUrl'],
				required: false
			}
		]
	},
	orderInfo: {
		attributes: ['brandName']
	}
}))

export class Brand extends Model<Brand>{
	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	id: number

	@Column({
		type: DataType.CHAR(255),
		allowNull: false,
		unique: { name: 'brandName', msg: 'validate:field:shouldBeUnique' }
	})
	brandName: string

	@ForeignKey(() => Media)
	@Column(DataType.BIGINT)
	documentId: number;

	@BelongsTo(() => Media)
	document: Media;

	@Column
	isAssigned: string;

	@Column
	isVerified: string;

	@Column
	isActive: string;

	@Column(DataType.DATE)
	deletedAt: Date;

	@AllowNull(false)
	@Column(DataType.DATE)
	createdAt: Date;

	@Column(DataType.DATE)
	updatedAt: Date;

	@HasMany(() => ChannelBrand, 'brandId')
	channelBrand: ChannelBrand;

	// static connectEsClient() {
	// 	return new ElasticQueueHelper();
	// }

	// @AfterCreate
	// @AfterBulkCreate
	// static async indexCreate(instance: Brand) {
	// 	this.connectEsClient().addToProcessQueue(
	// 		'create',
	// 		esearch.brand.INDEX_NAME,
	// 		instance
	// 	);
	// }
	//
	// @AfterUpdate
	// @AfterBulkUpdate
	// static async indexUpdate(instance: Brand) {
	// 	this.connectEsClient().addToProcessQueue(
	// 		'update',
	// 		esearch.brand.INDEX_NAME,
	// 		instance
	// 	);
	// }
	//
	// @AfterDelete
	// @AfterBulkDelete
	// static async indexDelete(instance: Brand) {
	// 	this.connectEsClient().addToProcessQueue(
	// 		'delete',
	// 		esearch.brand.INDEX_NAME,
	// 		instance
	// 	);
	// }

	@BeforeCreate
	@BeforeUpdate
	public static async dateUpdate(brand: Brand, options: any) {
		brand.updatedAt = Sequelize.fn('NOW');
	}
}
