import {
	Table,
	Column,
	PrimaryKey,
	Model,
	AutoIncrement,
	DataType,
	AllowNull,
	ForeignKey,
	BelongsTo,
	Scopes,
	HasOne,
	HasMany,
	BelongsToMany,
	BeforeCreate,
	BeforeUpdate
} from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
//import { DataTypeTime } from 'sequelize';
import { Channel } from '../../channel/models/Channel.entity';
import { ProductGroup } from './ProductGroup.entity';
import { ProductCategory } from './ProductCategory.entity';
import { Brand } from './Brand.entity';
import { ChannelBrand } from '../../channel/models/ChannelBrand.entity';
import { ProductPriceDetail } from './ProductPriceDetail.entity';
//import { HasMany } from 'sequelize-typescript/lib/annotations/association/HasMany';
import { ProductImage } from './ProductImage.entity';
import { ProductDetail } from './ProductDetail.entity';
import { ProductTag } from './ProductTag.entity';
import { ProductArticleReview } from './ProductArticleReview.entity';
import { ProductExistRating } from './ProductExistRating.entity';
import { ProductUniversalCode } from './ProductUniversalCode.entity';
import { ProductPackageInfo } from './ProductPackageInfo.entity';
import { ProductCompeting } from './ProductCompeting.entity';
import { ProductChannelCustomerProfile } from './ProductChannelCustomerProfile.entity';
import { ProductLocation } from './ProductLocation.entity';
import { ProductSharePreference } from './ProductSharePreference.entity';
import { Media } from './../../media/models/Media.entity';
import { RegionCountryJCT } from './../../common/models/RegionCountryJCT.entity';
import { ShareType } from './../../channel/models/ShareType.entity';
import { appconstant } from '../../../config/app.constant';
import { ProductReject } from './ProductReject.entity';
import { ProductAward } from './ProductAward.entity';
import { ProductJCT } from './ProductJCT.entity';
import { Wishlist } from 'src/modules/wishlist/models/Wishlist.entity';

var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamps: false, tableName: 'tblProduct' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
	productInfo: {
		attributes: ['productId', 'channelId', 'productGroupId', 'productCategorieId', 'product', 'brandId', 'productSKU', 'productkey', 'isshowtofreesub', 'isactive', 'hasanyvariant', 'variantdetails', 'productqualitylevel', 'defaultimageid', 'updatedBy', 'deletedat', 'product_pitch_award', 'product_rank_order', 'isSuggestion', 'isInovation', 'businessType']
	},
	orderInfo: {
		attributes: ['productId', 'product', 'productSKU', 'productGroupId', 'productCategorieId', 'brandId', 'defaultimageid']
	},
	productRatingInfo: {
		attributes: ['productId', 'channelId', 'productGroupId', 'productCategorieId', 'product', 'brandId', 'productSKU', 'productkey', 'isactive', 'defaultimageid', 'productqualitylevel']
	},
	productReject: {
		attributes: {
			exclude: ['deletedat', 'createdat', 'updatedat', 'updatedby']
		},
		include: [
			{
				model:Brand,
				as: 'brand'
			},
			{
				model:ProductCategory,
				as: 'productCategory'
			},
			{
				model:ProductImage,
				as: 'defaultImage',
				include: [{
					model:Media,
					as: 'image'
				}]
			}
		]
	}
}))

export class Product extends Model<Product>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	productId: number

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	channelId: number

	@BelongsTo(() => Channel)
	channel: Channel;

	@ForeignKey(() => ProductGroup)
	@Column(DataType.BIGINT)
	productGroupId: number

	@BelongsTo(() => ProductGroup)
	productGroup: ProductGroup;

	@ForeignKey(() => ProductCategory)
	@Column(DataType.BIGINT)
	productCategorieId: number

	@BelongsTo(() => ProductCategory)
	productCategory: ProductCategory;

	@Column(DataType.CHAR(255))
	product: string

	@ForeignKey(() => Brand)
	@Column(DataType.BIGINT)
	brandId: number

	@BelongsTo(() => Brand)
	brand: Brand;

	@Column(DataType.CHAR(255))
	productSKU: string

	@Column(DataType.CHAR(255))
	productkey: string

	@Column
	isshowtofreesub: string;

	@Column
	isactive: string;

	@Column
	hasanyvariant: string;

	@Column(DataType.CHAR(255))
	variantdetails: string

	@Column
	productqualitylevel: string;

	@Column
	product_pitch_award: string;

	@Column
	isSuggestion: string;

	@Column
	isInovation: string

	@Column
	isPublic: string;

	@Column
	isFeatured: string;

	@Column
	eco_friendly: string;

	@Column
	life_style: string;

	@Column
	work_from_home: string;

	@Column
	french_tech: string;

	@Column
	taiwan_excellence: string;

	@Column
	show_in_home_page: string;
	
	@Column(DataType.BIGINT)
	product_rank_order: number

	@HasOne(() => ProductPriceDetail, 'productId')
	productPriceDetail: ProductPriceDetail;

	@HasMany(() => ProductImage, 'productId')
	productImage: ProductImage[];

	@HasMany(() => ProductAward, 'productId')
	productAwards: ProductAward[]

	@HasMany(() => ProductJCT, 'productId')
	productJCT: ProductJCT[]

	@HasMany(() => Wishlist, 'productid')
	wishList: Wishlist[]

	@ForeignKey(() => ProductImage)
	@Column(DataType.BIGINT)
	defaultimageid: number

	@BelongsTo(() => ProductImage, 'defaultimageid')
	defaultImage: ProductImage;

	@Column(DataType.BIGINT)
	updatedBy: number

	@Column(DataType.BIGINT)
	businessType: number

	@AllowNull(false)
	@Column(DataType.DATE)
	createdat: Date

	@Column(DataType.DATE)
	updatedat: Date;

	@Column(DataType.DATE)
	deletedat: Date;

	@Column(DataType.CHAR(255))
	slug: string

	@HasOne(() => ProductDetail, 'productId')
	productDetail: ProductDetail;

	@HasMany(() => ProductTag, 'productId')
	productTag: ProductTag[];

	@HasMany(() => ProductArticleReview, 'productId')
	productArticleReview: ProductArticleReview[];

	@HasMany(() => ProductExistRating, 'productId')
	productExistRating: ProductExistRating[];

	@HasMany(() => ProductSharePreference, 'productId')
	productSharePreference: ProductSharePreference[];

	@HasMany(() => ProductUniversalCode, 'productId')
	productUniversalCode: ProductUniversalCode[];

	@HasOne(() => ProductPackageInfo, 'productId')
	productPackageInfo: ProductPackageInfo;

	@HasMany(() => ProductCompeting, 'productId')
	productCompeting: ProductCompeting[];

	@HasMany(() => ProductChannelCustomerProfile, 'productId')
	productChannelCustomerProfile: ProductChannelCustomerProfile[];

	@HasMany(() => ProductLocation, 'productId')
	productLocation: ProductLocation[];

	@BelongsToMany(() => RegionCountryJCT, {
		through: {
			model: () => ProductLocation,
			scope: {
				deletedat: null
			}
		}
	})
	locations: RegionCountryJCT[];

	@BelongsToMany(() => RegionCountryJCT, {
		through: {
			model: () => ProductLocation,
			scope: {
				deletedat: null,
				locationtype: appconstant.LOCATION_TYPE_SELLING
			}
		}
	})
	selling_locations: RegionCountryJCT[];

	@BelongsToMany(() => RegionCountryJCT, {
		through: {
			model: () => ProductLocation,
			scope: {
				deletedat: null,
				locationtype: appconstant.LOCATION_TYPE_TARGET
			}
		}
	})
	target_locations: RegionCountryJCT[];

	@BelongsToMany(() => Media, {
		through: {
			model: () => ProductImage,
			scope: {
				deletedat: null
			}
		}
	})
	images: Media[];

	@BelongsToMany(() => ShareType, {
		through: {
			model: () => ProductSharePreference,
			scope: {
				deletedat: null
			}
		}
	})
	shareType: ShareType[];

	@HasMany(() => ProductReject, 'productid')
	productReject: ProductReject[];

	@BeforeCreate
	@BeforeUpdate
	public static async dateUpdate(product: Product, options: any) {
		product.updatedat = Sequelize.fn('NOW');
	}

}
