import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, BeforeCreate, AllowNull, BeforeUpdate, ForeignKey, BelongsTo, Scopes } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Product } from './Product.entity';
import { Media } from './../../media/models/Media.entity';
var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblProductAward' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
	profileInfo: {
		attributes: ['productAwardId', 'productId', 'awardDetail', 'issuedBy', 'validUpto']
	}
}))

export class ProductAward extends Model<ProductAward>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	productAwardId: number

	@ForeignKey(() => Product)
	@Column(DataType.BIGINT)
	productId: number

	@BelongsTo(() => Product)
	product: Product[];

	@Column(DataType.CHAR(8000))
	awardDetail: string

	@Column(DataType.CHAR(255))
	issuedBy: string

	@Column(DataType.DATE)
	validUpto: Date

	@ForeignKey(() => Media)
	@Column(DataType.BIGINT)
	documentId: number

	@BelongsTo(() => Media, 'documentId')
	document: Media;

	@Column(DataType.DATE)
	deletedAt: Date

	@Column(DataType.DATE)
	createdAt: Date

	@Column(DataType.DATE)
	updatedAt: Date

	@Column
	isLifeTime: string;

	@BeforeCreate
	@BeforeUpdate
	public static async dateUpdate(productAward: ProductAward, options: any) {
		productAward.updatedAt = Sequelize.fn('NOW');
	}

}
