import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull,BelongsTo,ForeignKey,Scopes,HasMany, BelongsToMany } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { SampleOrderDetail } from '../../order/models/SampleOrderDetail.entity';
import { Product } from './Product.entity';
import { Contact } from '../../user/models/Contact.entity';
import { ProductEvaluationRating } from './ProductEvaluationRating.entity';
import { Channel } from '../../channel/models/Channel.entity';


const tableOptions: IDefineOptions = { tableName: 'tblproductevaluation' } as IDefineOptions;
  
@Table(tableOptions)

@Scopes(() => ({
    productInfo: {
        attributes: ['productevaluationid', 'productid', 'review', 'evaluatedby','createdat']
    }
}))
  
export class ProductEvaluation extends Model<ProductEvaluation>{
  
    @PrimaryKey
    @AutoIncrement
    @AllowNull(false)
    @Column(DataType.BIGINT)
    productevaluationid:number;
    
    @Column(DataType.BIGINT)
    @ForeignKey(() => SampleOrderDetail)
    sampleorderdetailid:number;

    @BelongsTo(() => SampleOrderDetail)
    sampleOrderDetail: SampleOrderDetail;
    
    @Column(DataType.BIGINT)
    @ForeignKey(() => Product)
    productid: number;

    @BelongsTo(() => Product)
    product: Product;

    @Column(DataType.BIGINT)
    @ForeignKey(() => Contact)
    evaluatedby: number;

    @BelongsTo(() => Contact)
    contact: Contact;

    @Column(DataType.BIGINT)
    review: string;

    @AllowNull(false)
    @Column(DataType.DATE)
    createdat: Date

    @Column(DataType.DATE)
    updatedat: Date;

    @Column(DataType.DATE)
    deletedat: Date;

    @Column(DataType.BIGINT)
    @ForeignKey(() => Channel)
    channelid: number;

    @BelongsTo(() => Channel)
    channel: Channel;

    @HasMany(() => ProductEvaluationRating, 'productevaluationid')
	productEvaluationRating: ProductEvaluationRating[];

}
