'use strict';
import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, Scopes, HasMany, BeforeCreate, BeforeUpdate } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
// import { ApiProperty } from '@nestjs/swagger';
//import { Unique } from 'sequelize-typescript/lib/annotations/Unique';
import { ValidationException } from './../../../middleware/exception/Validation.exception';
import { ProductCategory } from './ProductCategory.entity';

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblProductGroup' } as IDefineOptions;
@Table(tableOptions)
@Scopes(() => ({
	productInfo: {
		attributes: [['productGroupId', 'id'], ['productGroup', 'name']]
	},
	channelDetail: {
		attributes: ['productGroupId', 'productGroup', 'deletedAt', 'isActive']
	},
	contactProductInfom: {
		attributes: ['productGroupId', 'productGroup']
	},
	tradeinfo: {
		attributes: {
			include: [['productGroupId', 'productGroupId'], ['productGroup', 'productGroup'], ['isActive', 'isActive']],
			exclude: ['deletedAt', 'createdAt', 'updatedAt']
		}
	},
	channelDetailList: {
		attributes: ['productGroup']
	},
	excludeDeleted: {
		where: {
			deletedAt: null
		},
		attributes: ['productGroupId', 'productGroup', 'isActive']
	},
}))

export class ProductGroup extends Model<ProductGroup>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	productGroupId: number

	@Column({
		type: DataType.CHAR(255),
		allowNull: false,
		unique: { name: 'productGroup', msg: 'validate:field:shouldBeUnique' }
	})
	productGroup: string;

	@Column
	isActive: string;

	@Column(DataType.DATE)
	deletedAt: Date;

	@Column(DataType.DATE)
	createdAt: Date;

	@Column(DataType.DATE)
	updatedAt: Date;

	@Column
	businessType: string;
	
	@HasMany(() => ProductCategory)
	categories: ProductCategory[]

	@BeforeCreate
	@BeforeUpdate
	static async makeUniqueValidation(instance: ProductGroup) {
		let options = {
			where: {
				productGroup: {
					$ilike: instance.productGroup.trim()
				},
				deletedAt: null
			}
		};

		if (instance.productGroupId) {
			options['where']['productGroupId'] = {
				$not: instance.productGroupId
			}
		}

		if (await ProductGroup.count(options)) {
			throw new ValidationException([{
				value: instance.productGroup.trim(),
				property: 'productGroup',
				children: [],
				constraints:
					{ unique: 'validate:field:shouldBeUnique' }
			}]);
		}
	}
}
