import {
    Table,
    Column,
    PrimaryKey,
    Model,
    AutoIncrement,
    DataType,
    BeforeCreate,
    AllowNull,
    BeforeUpdate,
    ForeignKey,
    HasMany,
    BelongsTo,
    Scopes,
  } from 'sequelize-typescript';
  import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
//import { DataTypeTime } from 'sequelize';
import { Product } from './Product.entity';
import { Media } from '../../media/models';

const tableOptions: IDefineOptions = { timestamps: false, tableName: 'tblproductimage' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
    productInfo: {
        attributes: ['productimageid', 'productId', 'imageid', 'productimagetype', 'updatedby', 'deletedat', 'name', 'description']
    },
    orderInfo: {
        attributes: ['productimageid', 'productId', 'imageid', 'productimagetype', 'sortorder', 'updatedby', 'deletedat', 'name', 'description']
    },
    productImage:{
        attributes: ['productimageid', 'productId', 'imageid', 'productimagetype', 'name', 'description']
    }
}))

export class ProductImage extends Model<ProductImage>{

    @PrimaryKey
    @AutoIncrement
    @AllowNull(false)
    @Column(DataType.BIGINT)
    productimageid: number

    @ForeignKey(() => Product)
    @Column(DataType.BIGINT)
    productId: number

    @BelongsTo(() => Product)
    product: Product[];

    @ForeignKey(() => Media)
    @Column(DataType.BIGINT)
    imageid: number

    @BelongsTo(() => Media)
    image: Media;

    @Column
    productimagetype: string

    @Column(DataType.CHAR(255))
    name: string

    @Column(DataType.CHAR(255))
    description: string

    @Column(DataType.BIGINT)
    updatedby: number

    @Column(DataType.BIGINT)
    isdefault: string

    @Column(DataType.BIGINT)
    sortorder: number

    @AllowNull(false)
    @Column(DataType.DATE)
    createdat: Date

    @Column(DataType.DATE)
    updatedat: Date;

    @Column(DataType.DATE)
    deletedat: Date

}