import {
    Table,
    Column,
    PrimaryKey,
    Model,
    AutoIncrement,
    DataType,
    BeforeCreate,
    AllowNull,
    BeforeUpdate,
    ForeignKey,
    HasMany,
    BelongsTo,
    Scopes,
  } from 'sequelize-typescript';
  import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
//import { DataTypeTime } from 'sequelize';
import { Product } from './Product.entity';
import { Unit } from './Unit.entity';

const tableOptions: IDefineOptions = { timestamps: false, tableName: 'tblproductpackageinfo' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
    productInfo: {
        attributes: ['productpackageinfoid', 'productId', 'minorderquantity', 'orderunitid', 'packagesize', 'packageunit', 'noofboxinpallet', 'noofboxinpalletunitid', 'noofpackageinbox', 'noofpackageinboxunitid', 'approximateweightofpallet', 'approximateweightofpalletunitid', 'length', 'breadth', 'height', 'weight', 'updatedby', 'deletedat']
    }
}))
  
export class ProductPackageInfo extends Model<ProductPackageInfo>{

    @PrimaryKey
    @AutoIncrement
    @AllowNull(false)
    @Column(DataType.BIGINT)
    productpackageinfoid: number

    @ForeignKey(() => Product)
    @Column(DataType.BIGINT)
    productId: number

    @BelongsTo(() => Product)
    product: Product[];

    @Column(DataType.FLOAT(10,2))
    minorderquantity: number

    @ForeignKey(() => Unit)
    @Column(DataType.BIGINT)
    orderunitid: number

    @BelongsTo(() => Unit, 'orderunitid')
    orderunitdetail: Unit;

    @Column(DataType.FLOAT(10,2))
    packagesize: number

    @ForeignKey(() => Unit)
    @Column(DataType.BIGINT)
    packageunit: number

    @BelongsTo(() => Unit, 'packageunit')
    packageunitdetail: Unit;

    @Column(DataType.FLOAT(10,2))
    noofpackageinbox: number

    @ForeignKey(() => Unit)
    @Column(DataType.BIGINT)
    noofpackageinboxunitid: number

    @BelongsTo(() => Unit, 'noofpackageinboxunitid')
    noofpackageinboxunitdetail: Unit;

    @Column(DataType.FLOAT(10,2))
    noofboxinpallet: number

    @ForeignKey(() => Unit)
    @Column(DataType.BIGINT)
    noofboxinpalletunitid: number

    @BelongsTo(() => Unit, 'noofboxinpalletunitid')
    noofboxinpalletunitdetail: Unit;

    @Column(DataType.FLOAT(10,2))
    approximateweightofpallet: number

    @ForeignKey(() => Unit)
    @Column(DataType.BIGINT)
    approximateweightofpalletunitid: number

    @BelongsTo(() => Unit, 'approximateweightofpalletunitid')
    approximateweightofpalletunitdetail: Unit;

    @Column(DataType.FLOAT(10,2))
    length: number

    @Column(DataType.FLOAT(10,2))
    breadth: number
    
    @Column(DataType.FLOAT(10,2))
    height: number

    @Column(DataType.FLOAT(10,2))
    weight: number

    @Column(DataType.BIGINT)
    updatedby: number

    @AllowNull(false)
    @Column(DataType.DATE)
    createdat: Date

    @Column(DataType.DATE)
    updatedat: Date;

    @Column(DataType.DATE)
    deletedat: Date

}