import {
	Table,
	Column,
	PrimaryKey,
	Model,
	AutoIncrement,
	DataType,
	BeforeCreate,
	AllowNull,
	BeforeUpdate,
	ForeignKey,
	HasMany,
	BelongsTo,
	Scopes,
} from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
//import { DataTypeTime } from 'sequelize';
import { Product } from './Product.entity';
import { Currency } from '../../channel/models/Currency.entity';
import { Unit } from './Unit.entity';
import { DiscountUnit } from './DiscountUnit.entity';

const tableOptions: IDefineOptions = { timestamps: false, tableName: 'tblproductpricedetail' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
	productInfo: {
		attributes: ['productpricedetailid', 'productId', 'businesscurrencyid', 'retailerprice', 'retailerpriceunitid', 'issamplefree', 'sampleprice', 'samplepriceunitid', 'distributormarginpercent', 'distributormarginpercentunitid', 'retailermarginpercent', 'retailermarginpercentunitid', 'samplethreshold', 'isunlimitsample', 'updatedby', 'deletedat']
	},
    productExclude: {
        attributes: {
          exclude: ['deletedat', 'createdat', 'updatedat', 'updatedby']
        }
	},
	productPriceInfo: {
		attributes: {
			exclude: ['deletedat', 'createdat', 'updatedat', 'updatedby']
		  },
		  include:[
			{
				model:Unit,
				as : 'retailerpriceunitdetail'
			},
			{
				model:Unit,
				as : 'distributorunitdetail'
			},
			{
				model:Unit,
				as : 'retailerunitdetail'
			}
		  ]
	}	
}))

export class ProductPriceDetail extends Model<ProductPriceDetail>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	productpricedetailid: number

	@ForeignKey(() => Product)
	@Column(DataType.BIGINT)
	productId: number

	@BelongsTo(() => Product)
	product: Product[];

	@ForeignKey(() => Currency)
	@Column(DataType.BIGINT)
	businesscurrencyid: number

	@BelongsTo(() => Currency)
	businesscurrency: Currency;

	@Column(DataType.FLOAT(15, 2))
	retailerprice: number

	@ForeignKey(() => Unit)
	@Column(DataType.BIGINT)
	retailerpriceunitid: number

	@BelongsTo(() => Unit, 'retailerpriceunitid')
	retailerpriceunitdetail: Unit;

	@Column
	issamplefree: string;

	@Column(DataType.FLOAT(15, 2))
	sampleprice: number

	@ForeignKey(() => Unit)
	@Column(DataType.BIGINT)
	samplepriceunitid: number

	@BelongsTo(() => Unit, 'samplepriceunitid')
	samplepriceunitdetail: Unit;

	@Column(DataType["DOUBLE PRECISION"])
	distributormarginpercent: number

	@ForeignKey(() => DiscountUnit)
	@Column(DataType.BIGINT)
	distributormarginpercentunitid: number

	@BelongsTo(() => DiscountUnit, 'distributormarginpercentunitid')
	distributorunitdetail: DiscountUnit;

	@Column(DataType["DOUBLE PRECISION"])
	retailermarginpercent: number

	@ForeignKey(() => DiscountUnit)
	@Column(DataType.BIGINT)
	retailermarginpercentunitid: number

	@BelongsTo(() => DiscountUnit, 'retailermarginpercentunitid')
	retailerunitdetail: DiscountUnit;

	@Column(DataType.FLOAT(15, 2))
	samplethreshold: number

	@Column
	isunlimitsample: string

	@Column(DataType.BIGINT)
	updatedby: number

	@AllowNull(false)
	@Column(DataType.DATE)
	createdat: Date

	@Column(DataType.DATE)
	updatedat: Date;

	@Column(DataType.DATE)
	deletedat: Date

}
