import {
    Table,
    Column,
    PrimaryKey,
    Model,
    AutoIncrement,
    DataType,
    BeforeCreate,
    AllowNull,
    BeforeUpdate,
    ForeignKey,
    BelongsTo,
    Scopes,
  } from 'sequelize-typescript';
  import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Product } from './Product.entity';
import { ShareType } from '../../channel/models/ShareType.entity';
var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamps: false, tableName: 'tblproductsharepreference' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
    productInfo: {
        attributes: ['productsharepreferenceid', 'productId', 'sharetypeid', 'updatedby', 'deletedat']
    }
}))
  
export class ProductSharePreference extends Model<ProductSharePreference>{

    @PrimaryKey
    @AutoIncrement
    @AllowNull(false)
    @Column(DataType.BIGINT)
    productsharepreferenceid: number

    @ForeignKey(() => Product)
    @Column(DataType.BIGINT)
    productId: number

    @BelongsTo(() => Product)
    product: Product[];

    @ForeignKey(() => ShareType) 
    @Column(DataType.BIGINT)
    sharetypeid:number

    @BelongsTo(() => ShareType)
    sharetype: ShareType[];

    @Column(DataType.BIGINT)
    updatedby: number

    @AllowNull(false)
    @Column(DataType.DATE)
    createdat: Date

    @Column(DataType.DATE)
    updatedat: Date;

    @Column(DataType.DATE)
    deletedat: Date

    @BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(productSharePreference: ProductSharePreference, options: any) { 
		productSharePreference.updatedat = Sequelize.fn('NOW'); 
	}

}