import {
	Table,
	Column,
	PrimaryKey,
	Model,
	AutoIncrement,
	DataType,
	AllowNull,
	ForeignKey,
	BelongsTo,
	HasMany,
	Scopes,
	HasOne,
	BelongsToMany,
	BeforeCreate,
	BeforeUpdate
} from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
//import { DataTypeTime } from 'sequelize';
import { Channel } from '../../channel/models/Channel.entity';

import { Brand } from './Brand.entity';
import { ChannelBrand } from '../../channel/models/ChannelBrand.entity';
import { ProductPriceDetail } from './ProductPriceDetail.entity';
//import { HasMany } from 'sequelize-typescript/lib/annotations/association/HasMany';
import { SpotDealBrand } from './SpotDealBrand.entity';
import { SpotDealImages } from './SpotDealImages.entity';

import { Media } from './../../media/models/Media.entity';

import { ShareType } from './../../channel/models/ShareType.entity';
import { appconstant } from '../../../config/app.constant';

import { Unit } from './Unit.entity';
import { Currency } from '../../channel/models/Currency.entity';
 
var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblSpotDeals' } as IDefineOptions;

@Table(tableOptions)

// @Scopes({
// 	productInfo: {
// 		attributes: ['productId', 'channelId', 'productGroupId', 'productCategorieId', 'product', 'brandId', 'productSKU', 'productkey', 'isshowtofreesub', 'isactive', 'hasanyvariant', 'variantdetails', 'productqualitylevel', 'defaultimageid', 'updatedBy', 'deletedat', 'product_pitch_award','product_rank_order','isSuggestion','isInovation']
// 	},
// })

export class SpotDeals extends Model<SpotDeals>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	id: number

	@Column(DataType.CHAR(255))
	product: string

	@Column(DataType.CHAR(500))
	productcategory: string

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	channelId: number

	@BelongsTo(() => Channel)
	brand: Channel;

	@Column(DataType.CHAR(500))
	keywords: string


	@Column(DataType.CHAR(6000))
	spotdealdesc: string

	@Column(DataType.BIGINT)
	quantity: number

	@ForeignKey(() => Unit)
	@Column(DataType.BIGINT)
	samplepriceunitid: number

	@BelongsTo(() => Unit, 'samplepriceunitid')
	samplepriceunitdetail: Unit;

	@Column(DataType.FLOAT(15, 2))
	price: number

	@ForeignKey(() => Currency)
	@Column(DataType.BIGINT)
	businesscurrencyid: number

	@BelongsTo(() => Currency)
	businesscurrency: Currency;
	
	@Column(DataType.DATE)
	startduration: Date;

	@Column(DataType.DATE)
	endduration: Date;

	@Column(DataType.CHAR(6000))
	shippingdetail: string

	@Column(DataType.CHAR(6000))
	paymentdetail: string

	@Column(DataType.CHAR(6000))
	productlocation: string

	@ForeignKey(() => Media)
    @Column(DataType.BIGINT)
    spotimageid: number;

    @BelongsTo(() => Media)
	image: Media;

	@HasMany(() => SpotDealBrand, 'spotId')
	spotBrand: SpotDealBrand[];

	@HasMany(() => SpotDealImages, 'spotId')
	spotImage: SpotDealImages[];

}
