import { Module } from '@nestjs/common';
import { databaseProviders } from 'src/core/database/database.providers';
import { MediaModule } from '../media/media.module';
import { ProductController } from './controller/product.controller';
import { brandProvider } from './provider/brand.providers';
import { awardResultProvider, channelTypeProvider, discountUnitProvider, productArticleReviewProvider, productAwardProvider, productChannelCustomerProfileProvider, productCompetingProvider, productDetailProvider, productExistRatingProvider, productImageProvider, productJctProvider, productKeyPartnerJCTProvider, productLocationProvider, ProductMediaLibraryProvider, productPackageInfoProvider, productPriceDetailProvider, productProvider, productSharePreferenceProvider, productTagProvider, productUniversalCodeProvider, ProfileProgressProvider, profileTypeClassificationProvider, profileTypeProvider, unitProvider } from './provider/product.providers';
import { productEvaluationProvider, productEvaluationRatingProvider } from './provider/evaluation.providers';
import { productGroupProviders, RejectReasonProviders, ProductRejectProviders } from './provider/productGroup.providers';
import { ProductService } from './service/product.service';
import { ChannelService } from '../channel/service/channel.service';
import { channelCertificateTypeProvider, channelProductCategorieProvider, channelProvider, ChannelSubscriptionProviders, CompanyMediaImagesProvider, PaymentOptionTypeProvider, PaymentTermProvider, sharetypeProvider } from '../channel/provider/channel.provider';
import { SubscriptionPlanProviders, SubscriptionProviders } from '../channel/provider/subscription.provider';
import { SignUpStatusProviders } from '../user/provider/SignUpStatus.providers';
import { settingProvider } from '../channel/provider/settting.provider';
import { ContactJCTProvider } from '../channel/provider/channelHub.provider';
import { ChannelKeyPartnerService } from '../channel/service/channelKeyPartner.service';
import { CountryProviders } from '../common/provider/common.provider';
import { RegionCountryJCTProvider } from '../common/provider/location.provider';
import { RejectReasonService } from './service/rejectReason.service';
import { RejectController } from './controller/reject.controller';
import { UnitController } from './controller/unit.controller';
import { UnitService } from './service/unit.services';
import { ProductImportController } from './controller/productImport.controller';
import { ProductImportService } from './service/productImport.service';
import { PublicProductController } from './controller/public.product.controller';
import {
	SubscriptionExtraFeatureProviders
    } from '../channel/provider/subscription.provider';
import { ProductAdminController } from './controller/product.admin.controller';
import { ProductAdminService } from './service/product.admin.service';
@Module({

    controllers: [ProductController, RejectController, UnitController, ProductImportController, PublicProductController, ProductAdminController],
    imports: [ MediaModule],
	  providers: [ProductService,
		ChannelService,
		ChannelKeyPartnerService,
		RejectReasonService,
		UnitService,
		ProductImportService,
		ProductAdminService,
		...productJctProvider,
		...channelProvider,
    ...productProvider,...databaseProviders,
	...productAwardProvider,
	...SubscriptionProviders,
	...PaymentOptionTypeProvider,
		...ProductRejectProviders,
		...SignUpStatusProviders,
		...RejectReasonProviders,
		...ChannelSubscriptionProviders,
		...settingProvider,
		...ContactJCTProvider,
		...channelCertificateTypeProvider,
		...SubscriptionPlanProviders,
		...PaymentTermProvider,
		...CountryProviders,
		...RegionCountryJCTProvider,
		// ...productCategoryProviders,
		...productGroupProviders,
		...ProductMediaLibraryProvider,
		...brandProvider,
		...ProfileProgressProvider,
		...awardResultProvider,
		...productDetailProvider,
		...productTagProvider,
		...productArticleReviewProvider,
		...productExistRatingProvider,
		...discountUnitProvider,
		...productChannelCustomerProfileProvider,
		...productCompetingProvider,
		...productImageProvider,
		...productKeyPartnerJCTProvider,
		...productLocationProvider,
		...productPackageInfoProvider,
		...productPriceDetailProvider,
		...productSharePreferenceProvider,
		...productUniversalCodeProvider,
		...unitProvider,
		...channelTypeProvider,
		...profileTypeProvider,
		...profileTypeClassificationProvider,
		...productEvaluationProvider,
		...productEvaluationRatingProvider,
		...sharetypeProvider,
		...channelProductCategorieProvider,
        ...CompanyMediaImagesProvider,
		...SubscriptionExtraFeatureProviders
		// ...spotDealsProvider,
		// ...spotDealBrandProvider,
        // ...spotDealImageProvider
	],
	exports: [ProductService,...brandProvider, ...productEvaluationProvider,
		...productEvaluationRatingProvider,...productProvider]
})
export class ProductModule {}
