'use strict';

import { Injectable, Inject } from '@nestjs/common';
import { ProductGroup } from '../models/ProductGroup.entity';
import { ProcessException } from './../../../middleware/exception/Process.exception';
import { ProductCategory } from '../models/ProductCategory.entity';
import { CreateBrandDto } from './../dto/createBrand.dto';
import { Brand } from '../../product/models/Brand.entity';
import { Media } from '../../media/models/index';
import { appconstant } from '../../../config/app.constant';
import { ValidationException } from './../../../middleware/exception/Validation.exception';
import { UpdateBrandDto } from '../dto/UpdateBrand.dto';
import { BaseService } from 'src/core/service/base.service';
import {Op, QueryTypes } from "sequelize";

@Injectable()
export class ProductAdminService extends BaseService {
	constructor(
		@Inject('ProductGroupRepository') private readonly productGroupRepository: typeof ProductGroup,
		@Inject('ProductCategoryRepository') private readonly productCategoryRepository: typeof ProductCategory,
		@Inject('BrandRepository') private readonly brandRepository: typeof Brand,
		@Inject('SequelizeInstance') private readonly sequelizeInstance
	) {
		super();
	}

	async createBrand(createBrandDto: CreateBrandDto): Promise<any> {
		createBrandDto['documentId'] = null;

		if (await this.brandRepository.count({
			where: {
				brandName: {
					[Op.iLike]: createBrandDto.brandName.trim()
				},
				deletedAt: null
			}
		})) {
			throw new ValidationException([{
				value: createBrandDto.brandName.trim(),
				property: 'brandName',
				children: [],
				constraints:
					{ unique: 'validate:field:shouldBeUnique' }
			}]);
		}

		if (createBrandDto.brandImage) {
			let brandImage = await Media.create({
				documentPath: createBrandDto.brandImage,
				documentTypeId: appconstant.DOCUMENTTYPE_IMAGE,
				dirName: appconstant.FOLDER_PATH_BRAND
			});
			createBrandDto['documentId'] = brandImage.documentId;
		}
		createBrandDto['createdAt'] = new Date().toUTCString();

		return await this.brandRepository.create(createBrandDto);
	}

	public async getBrands(pagination): Promise<Array<any>> {
console.log('pagination', pagination);
		let result: any = new Object();

		let whereOption: any = {};
		// whereOption[Op.or] = [
		// 	{
		// 		'documentId': null
		// 	},
		// 	{
		// 		'documentId': 1
		// 	}
		// ];

		if (pagination.search) {
			whereOption['brandName'] = { [Op.iLike]: '%' + pagination.search + '%' }
		}
		if (pagination.filter) {
			whereOption['isActive'] = pagination.filter
		}

		// get showall query param 
		// if(showall === true) {
		// 	delete whereOption[Op.or];
		// }
		// { where: whereOption }
		result.total = await this.brandRepository.scope('brandMaster').count();
		console.log('result.total', result.total);
		let sortField = 'createdAt';
		// let sortOrder ;
		// if (pagination.orderBy && ['id', 'brandName', 'isActive', 'updatedAt'].includes(pagination.orderBy)) {
		// 	sortField = pagination.orderBy;
		// }

		// if (sortOrder.length === 0) {
		// 	sortOrder.push(...[sortField, pagination.order]);
		// }

		result.list = await this.brandRepository.scope('brandMaster').findAll<Brand>({
			where: whereOption,
			limit: pagination.limit,
			offset: pagination.offset,
			order: [[sortField, pagination.orderBy || 'desc']]
		});
		return result;
	}

	public async getBrand(id: number): Promise<any | null> {
		let result = await this.brandRepository.scope('brandMaster').findByPk<Brand>(id);
		if (!result) {
			throw new ProcessException('validate:field:notFound');
		}
		return result;
	}

	public async updateBrand(id: number, updateBrandDto: UpdateBrandDto): Promise<any | null> {
		let result = await this.brandRepository.scope('brandMaster').findByPk<Brand>(id);
		if (!result) {
			throw new ProcessException('validate:field:notFound');
		}
		if (updateBrandDto.brandName) {
			if (await this.brandRepository.count({
				where: {
					brandName: {
						[Op.iLike]: updateBrandDto.brandName.trim()
					},
					deletedAt: null
				}
			})) {
				throw new ValidationException([{
					value: updateBrandDto.brandName.trim(),
					property: 'brandName',
					children: [],
					constraints:
						{ unique: 'validate:field:shouldBeUnique' }
				}]);
			}
		}

		if (updateBrandDto.brandImage) {
			let brandImage = await Media.create({
				documentPath: updateBrandDto.brandImage,
				documentTypeId: appconstant.DOCUMENTTYPE_IMAGE,
				dirName: appconstant.FOLDER_PATH_BRAND
			});
			updateBrandDto['documentId'] = brandImage.documentId;
		}

		await result.update(updateBrandDto);
		return result;
	}
}
