'use strict';

import {  Inject, Injectable, InternalServerErrorException, NotFoundException } from '@nestjs/common';
import { Product } from '../models/Product.entity';
import { Channel } from '../../channel/models/Channel.entity';
import { ProductPriceDetail } from '../models/ProductPriceDetail.entity';
import { ProductImage } from '../models/ProductImage.entity';

import { Media } from '../../media/models/index';

import { ProcessException } from '../../../middleware/exception/Process.exception';

import { AddProductDto } from '../dto/addProduct.dto';
import { ProductSerivceInBound } from '../transformation/productSerivceInBound';
import { ProductDetail } from '../models/ProductDetail.entity';
import { ProductTag } from '../models/ProductTag.entity';
import { ProductArticleReview } from '../models/ProductArticleReview.entity';
import { ProductExistRating } from '../models/ProductExistRating.entity';
import { ProductSharePreference } from '../models/ProductSharePreference.entity';
import { ProductUniversalCode } from '../models/ProductUniversalCode.entity';
import { ProductPackageInfo } from '../models/ProductPackageInfo.entity';

import { ProductChannelCustomerProfile } from '../models/ProductChannelCustomerProfile.entity';
import { ProductLocation } from '../models/ProductLocation.entity';

import { ChannelSubscription } from '../../channel/models/ChannelSubscription.entity';
import { Subscription } from '../../channel/models/Subscription.entity';
import { SubscriptionPlan } from '../../channel/models/SubscriptionPlan.entity';
import { Country } from '../../common/models/Country.entity';
import { PaymentOptionType } from '../../channel/models/PaymentOptionType.entity';
import { PaymentTerm } from '../../channel/models/PaymentTerm.entity';
import { DiscountUnit } from '../models/DiscountUnit.entity';

import { ChannelType } from '../../channel/models/ChannelType.entity';
import _ = require('lodash');
import { appconstant } from 'src/config/app.constant';
import { ProfileList } from 'src/modules/channel/models/ProfileList.entity';
import { Brand } from '../models/Brand.entity';
import { ProductMediaLibrary } from '../models/ProductMediaLibrary.entity';
import {Op, QueryTypes } from "sequelize";
import { Utility } from 'src/modules/channel/helper/utility';
import { TradeInformationInBound } from 'src/modules/channel/transformation/tradeInformationInBound';
import { ProductKeyPartnerJCT } from '../models/ProductKeyPartnerJCT.entity';
import { features } from 'src/config/features.constants';
import { BaseService } from 'src/core/service/base.service';
import { ChannelService } from 'src/modules/channel/service/channel.service';
import { Contact } from 'src/modules/user/models/Contact.entity';
import { ProductGroup } from '../models/ProductGroup.entity';
import { ProductCategory } from '../models/ProductCategory.entity';
import { ChannelCustomerProfile } from 'src/modules/channel/models/ChannelCustomerProfile.entity';
import { ProfileTypeClassification } from 'src/modules/channel/models/ProfileTypeClassification.entity';
import { ProfileType } from 'src/modules/channel/models/ProfileType.entity';
import { RegionCountryJCT } from 'src/modules/common/models/RegionCountryJCT.entity';
import { Unit } from '../models/Unit.entity';
import { plainToClass } from 'class-transformer';
import { ProductServiceOutBound } from '../transformation/productServiceOutBound';
import { Currency } from '../../channel/models/Currency.entity';
import { ShareType } from '../../channel/models/ShareType.entity';
import { S3Lib } from "src/lib/S3Lib";
import { validate } from "class-validator";
import { ValidationException } from './../../../middleware/exception/Validation.exception';
import { ProductAward } from './../models/ProductAward.entity';
import { ImportProductImageDto } from '../dto/importProductImage.dto';
// var inArray = require('in-array');
let fs = require('fs');
let path = require("path");

@Injectable()
export class ProductImportService extends BaseService  {
    private productServiceInBound = new ProductSerivceInBound();
    private tradeInformationInBound = new TradeInformationInBound();
    private utility = new Utility();
	private productServiceOutBound = new ProductServiceOutBound();
	//private tradeInformationInBound = new TradeInformationInBound();
	//private utility = new Utility();
	constructor(
		// private productIndex: ProductIndexService,
		private channelService: ChannelService,
		// private channelIndexService: ChannelIndexService,
         //private context: ContextService,
         
		@Inject('ProductCategoryRepository') private readonly productCategoryRepository: typeof ProductCategory,
		@Inject('ProductGroupRepository') private readonly productGroupRepository: typeof ProductGroup,
        @Inject('BrandRepository') private readonly brandRepository: typeof Brand,        
		@Inject('SequelizeInstance') private readonly sequelizeInstance,
        @Inject('ProductRepository') private readonly productRepository: typeof Product,
        @Inject('ProfileRepository') private readonly profileRepository: typeof ProfileList,
		// @Inject('AwardRepository') private readonly awardRepository: typeof AwardResult,
		@Inject('ChannelRepository') private readonly channelRepository: typeof Channel,
		//@Inject('ChannelLocationRepository') private readonly channelLocationRepository: typeof ChannelLocation,
		@Inject('ProductPriceDetailRepository') private readonly productPriceDetailRepository: typeof ProductPriceDetail,
		@Inject('ProductImageRepository') private readonly productImageRepository: typeof ProductImage,
		@Inject('MediaRepository') private readonly mediaRepository: typeof Media,
		@Inject('ProductDetailRepository') private readonly productDetailRepository: typeof ProductDetail,
		@Inject('ProductTagRepository') private readonly productTagRepository: typeof ProductTag,
		@Inject('ProductArticleReviewRepository') private readonly productArticleReviewRepository: typeof ProductArticleReview,
		@Inject('ProductExistRatingRepository') private readonly productExistRatingRepository: typeof ProductExistRating,
		@Inject('ProductSharePreferenceRepository') private readonly productSharePreferenceRepository: typeof ProductSharePreference,
		@Inject('ProductUniversalCodeRepository') private readonly productUniversalCodeRepository: typeof ProductUniversalCode,
		@Inject('ProductPackageInfoRepository') private readonly productPackageInfoRepository: typeof ProductPackageInfo,
		// @Inject('ProductCompetingRepository') private readonly productCompetingRepository: typeof ProductCompeting,
		@Inject('ProductChannelCustomerProfileRepository') private readonly productChannelCustomerProfileRepository: typeof ProductChannelCustomerProfile,
        @Inject('ProductLocationRepository') private readonly productLocationRepository: typeof ProductLocation,
        @Inject('ProductMediaLibraryRepository') private readonly productMediaLibraryRepository: typeof ProductMediaLibrary,
		//@Inject('UserAccessPermissionRepository') private readonly userAccessPermissionRepository: typeof UserAccessPermission,
		// @Inject('PreferenceRepository') private readonly preferenceRepository: typeof AccessPermission,
		// @Inject('ContactRepository') private readonly contactRepository: typeof Contact,
		@Inject('ChannelSubscriptionRepository') private readonly channelSubscriptionRepository: typeof ChannelSubscription,
		@Inject('SubscriptionRepository') private readonly subscriptionRepository: typeof Subscription,
		@Inject('SubscriptionPlanRepository') private readonly subscriptionPlanRepository: typeof SubscriptionPlan,
		@Inject('CountryRepository') private readonly countryRepository: typeof Country,
		@Inject('RegionCountryJCTRepository') private readonly regionCountryJCTRepository: typeof RegionCountryJCT,
		@Inject('PaymentOptionTypeRepository') private readonly paymentOptionTypeRepository: typeof PaymentOptionType,
		@Inject('PaymentTermRepository') private readonly paymentTermRepository: typeof PaymentTerm,
		@Inject('UnitRepository') private readonly unitRepository: typeof Unit,
		@Inject('DiscountUnitRepository') private readonly discountUnitRepository: typeof DiscountUnit,
		@Inject('ProfileTypeClassificationRepository') private readonly profileTypeClassificationRepository: typeof ProfileTypeClassification,
		@Inject('ProfileTypeRepository') private readonly profileTypeRepository: typeof ProfileType,
		@Inject('ChannelTypeRepository') private readonly channelTypeRepository: typeof ChannelType,
		@Inject('ChannelCustomerProfileRepository') private readonly channelCustomerProfileRepository: typeof ChannelCustomerProfile,
		@Inject('ProductAwardRepository') private readonly productAwardRepository: typeof ProductAward,
	) {
		super();
	}


    async productImport(products: any, user: any): Promise<any> {
		//Stripe connect Id check
		//var stripConnectionCheck = await this.stripeConnectCheck(user);
		let pcnt = 0
		if(products.result.length > 0){
			for (let i = 0; i < products.result.length; i++) {
				if((products.result[i].product) && products.result[i].product != '') {
					pcnt = pcnt +1;
				}
		    }
		}
		console.log('Product Import count',pcnt);
		if (products && pcnt > 0) {

			let key = +user.channel.subscriptionId + '-' + +features.PRODUCT_SHEET;
			let subscriptionFeatureKeys = user.productCount;
			if (subscriptionFeatureKeys && subscriptionFeatureKeys[key]) {
				let productCount = await this.productRepository.count({ where: { channelId: +user.channelId, deletedat: null } });
				if (productCount >= +subscriptionFeatureKeys[key]) {
					throw new ProcessException({ message: ' Based on your Subscription, you are eligible to add ' + +subscriptionFeatureKeys[key] + ' products. Already you have ' + productCount + ' products.', eligibleCount: subscriptionFeatureKeys[key], exitCount: productCount });
				} 
				if (((+pcnt + +productCount) > +subscriptionFeatureKeys[key])) {

					let eligible = ((+subscriptionFeatureKeys[key] - productCount) >= 1) ? 'You are able to add only ' + (+subscriptionFeatureKeys[key] - productCount) + ' products' : '';

					throw new ProcessException({ message: 'Based on your Subscription, you are eligible to add ' + +subscriptionFeatureKeys[key] + ' products. Already you have ' + productCount + ' products and uploading product count is ' + pcnt + '. ' + eligible, eligibleCount: +subscriptionFeatureKeys[key], exitCount: +productCount, uploadCount: +pcnt });
				}
			}
			try {
				let premaster = await this.channelSettings(user);
				let masters = await this.productServiceOutBound.getProductMasters(premaster);
				let returnObject: any = { 'uploaded': [], 'totalCount': pcnt, 'uploadedCount': 0, 'uninsertedCount': 0, 'uninsertedFileName': '' };
				let unInsertedProducts = [];

				for (let i = 0; i < pcnt; i++) {
					
					let message = '';
					let product = products.result[i]; 
					var productCheck = await this.productImportCheck(product, masters);
					
					if (productCheck.status) {

						
						let productObject = await this.productServiceInBound.setProductImportObject(product, masters, products.imageFolder);
						const object = plainToClass(AddProductDto, productObject);
						//var error = null;
						var error = await validate(object, {
							groups: ["import"]
						}).then(errors => {
							return new ValidationException(errors);
						});

						let productAdd: any = false;
						if (!error || !error.messages || error.messages.length === 0) {
							try {
								productAdd = await this.addProduct(productObject, user, null, true);
							} catch (e) {

								console.log('product upload error ', e);
								error = e;
							}

							if (productAdd) {
								returnObject.uploaded.push({ productId: productAdd.data.productId, productSKU: productAdd.data.productSKU });
								returnObject.uploadedCount = returnObject.uploadedCount + 1;
							}
						}

						if (!productAdd) {
							let matchingFields = {
								'productname': 'product',
								'productgroup': 'productGroup',
								'productcategory': 'productCategory',
								'productbrand': 'productBrand',
								'productdesc': 'productDesc',
								'productusp': 'tweetPitch',
								'reviewername': 'productArticleReview_Name',
								'articleurl': 'productArticleReview_URL',
								'logo': 'productArticleReview_Logo',
								'0': '1',
								'1': '2',
								'2': '3',
								'3': '4'
							}

							if (!message) {
								message = 'Validate uploaded image names with input OR Re-download the samplesheet and try again.';
								if (error && error.messages && error.messages.length > 0) {
									message = '\n';
									error.messages.forEach((data) => {
										if (data.errors) {
											if (matchingFields[data.property]) {
												message += '\n  ' + matchingFields[data.property] + '\n';
											}
											data.errors.forEach((sdata) => {
												if (sdata.errors) {
													if (matchingFields[sdata.property]) {
														message += '\n  ' + matchingFields[sdata.property] + '\n';
													}
													sdata.errors.forEach((tdata) => {
														if (tdata.errors) {
															if (matchingFields[tdata.property]) {
																message += '\n  ' + matchingFields[tdata.property] + '\n';
															}

															tdata.errors.forEach((fdata) => {
																if (fdata.errors) {
																	if (matchingFields[fdata.property]) {
																		message += '\n   ' + matchingFields[fdata.property] + '\n';
																	}

																	fdata.errors.forEach((fidata) => {
																		if (fidata.errors) {
																			if (matchingFields[fidata.property]) {
																				message += '\n   ' + matchingFields[fidata.property] + '\n';
																			}
																		} else {
																			message += '    * ' + fidata.message + '\n';
																		}
																	});
																} else {
																	message += '    * ' + fdata.message + '\n';
																}
															});
														} else {
															message += '    * ' + tdata.message + '\n';
														}
													});
												} else {
													message += '    * ' + sdata.message + '\n';
												}
											});
										} else {
											message += '    * ' + data.message + '\n';
										}
									});
								}
							}
							product['errors'] = message;
							unInsertedProducts.push(product);
							returnObject.uninsertedCount = returnObject.uninsertedCount + 1;
						}
					} else {
						product['errors'] = productCheck.errors;
						if (productCheck.productRowCheck) {
							unInsertedProducts.push(product);
							returnObject.uninsertedCount = returnObject.uninsertedCount + 1;
						} else {
							returnObject.totalCount = +returnObject.totalCount - 1;
						}
					}
				}

				if (unInsertedProducts.length > 0) {
					let fileName = 'uninserted_products_' + user.channelId + '.xlsx';
					await this.sampleOrUninsertedProducts(unInsertedProducts, fileName, 'uninserted');
					returnObject.uninsertedFileName = fileName;
				}

				fs.unlink(path.resolve(__dirname, '..', '..', '..', '..') + '/.tmp/upload_s3/' + products.fileName, function (err) {
					if (err) {
						console.log("error while file delete");
						return false;
					}
					return true;
				});


				return returnObject;

			} catch (e) {
				console.log("Error+ ", e);
				throw new InternalServerErrorException();
			}
		} else {
			throw new ProcessException('product:productImport:zeroRows');
		}
	}

    async productImageUpload(importProductImageDto: ImportProductImageDto) {
		let imageFolder = 'product-import-zip-s3';
		console.log(importProductImageDto.fileName)
		if (!importProductImageDto.fileName) {
			throw new ProcessException('product:productImport:imageFolderName');
		}
		if (importProductImageDto.fileName) {
			let s3 = new S3Lib();
			let moveAction = await s3.downloadFromTempBucket(imageFolder, importProductImageDto.fileName);
			if (moveAction) {
				let moveImages = await s3.uploadZipToTempBucket(imageFolder, importProductImageDto.fileName);
				if (moveImages) {
					return moveImages;
				}
			}
		} {
			return 'Something went Wrong';
		}
	}

    public async productImportSample(user){
        try {
			let masters = await this.productServiceOutBound.getProductMasters(await this.channelSettings(user));
			//let fileName = 'sample_products_' + user.channelId + '.xlsx';
			let fileName = 'sample_products_' + user.channelId + '.xlsm';
			await this.cloneSampleProductsImportSht(masters, fileName, 'sample', user.channelId);
			//await this.sampleOrUninsertedProducts(masters, fileName, 'sample');
			return { 'sampleFileName': fileName };

		} catch (e) {
			console.log("Error+ ", e);
			throw new InternalServerErrorException();
		}
    }

    async cloneSampleProductsImportSht(data: any, fileName: any, type: any, channelId: any): Promise<any> {
		var fs = require('fs');
		var cpyfilename = 'sample_products_import.xlsm'
		var newfilename = fileName;
		fs.copyFile(path.resolve(__dirname, '..', '..', '..', '..') + '/public/templates/' + cpyfilename, path.resolve(__dirname, '..', '..', '..', '..') + '/.tmp/upload_s3/' + newfilename, function (err) {
			if (err) {
				console.log(err);
				return false;
			} else { console.log('brand');
				var pt = path.resolve(__dirname, '..', '..', '..', '..') + '/.tmp/upload_s3/' + newfilename;
				const XLSX = require('xlsx');
				var productCat = [];
				var brand = [];
				var currency = [];
				var shareType = [];
				const workbook = XLSX.readFile(pt, { bookVBA: true });

				var ws_name = "Masters";
				for (let i = 0; i < data.productCategory.length; i++) {
					productCat.push(data.productCategory[i]);
				}

				for (let i = 0; i < data.brand.length; i++) {
					brand.push(data.brand[i]);
				}

				if (data.currency.length > 0) {
					for (let i = 0; i < data.currency.length; i++) {
						currency.push(data.currency[i].currency);
					}
				}

				if (data.shareType.length > 0) {

					for (let i = 0; i < data.shareType.length; i++) {
						console.log(data.shareType[i]);
						shareType.push(data.shareType[i]);
					} shareType.push(data.shareType.all);
				}

				console.log(brand);
				/* make worksheet */
				var ws_data = [
					productCat,
					brand,
					currency,
					shareType,
					[data.productCategory.length],
					[data.brand.length],
					[data.currency.length],
					[data.shareType.length]
				];
				var ws = XLSX.utils.aoa_to_sheet(ws_data);

				/* Add the worksheet to the workbook */
				XLSX.utils.book_append_sheet(workbook, ws, ws_name);

				XLSX.writeFile(workbook, path.resolve(__dirname, '..', '..', '..', '..') + '/.tmp/upload_s3/' + newfilename);
				console.log('Completed ...');
				

				
			}
			
			return true;
		});
	}

    public async channelSettings(user: any) {
		try {
			let result = await this.channelRepository.findOne<Channel>({
				where: {
					channelId: user.channelId,
					deletedAt: null
				},
				include: [
					/*{
						model: ChannelDetail,
						as: 'channelDetail',
						required: false,
						attributes: ['currencyid'],
						include: [{
							model: Currency,
							required: false,
							attributes: ['currency']
						}]
					},*/
					{
						model: ProductGroup.scope('defaultScope'), as: 'productGroup', required: false, where: { deletedAt: null }, through: { attributes: [], where: { deletedAt: null } }, attributes: ['productGroupId', 'productGroup']
					},
					{
						model: ProductCategory, as: 'productCategory', required: false, where: { deletedAt: null }, through: { attributes: [], where: { deletedAt: null } }, attributes: ['productCategorieId', 'productGroupId', 'productCategorie']
					},
					{
						model: Brand, as: 'brand', required: false, where: { deletedAt: null }, through: { attributes: [], where: { deletedAt: null } }, attributes: ['id', 'brandName']
					},
					/*{
						model: this.channelLocationRepository,
						where: { deletedAt: null },
						required: false,
						include: [{
							model: this.regionCountryJCTRepository,
							required: false,
							where: { deletedAt: null },
							include: [
								{ model: Country, required: false, attributes: ['countryId', 'country'] }
							]
						}]
					},*/
					{
						model: ShareType, as: 'shareType', required: false, through: { attributes: [], where: { deletedat: null } }, attributes: ['sharetypeid', 'sharetype']
					},
					/*{
						model: this.channelCustomerProfileRepository.scope('channelDetail'),
						where: { deletedAt: null },
						required: false,
						include: [
							{
								model: this.profileTypeClassificationRepository.scope('profileTypeClassificationInfom'),
								where: { deletedAt: null },
								required: false,
								include: [
									{
										model: this.profileTypeRepository.scope('profileTypeInfom'),
										where: { deletedAt: null },
										required: false,
									},
									{
										model: this.channelTypeRepository.scope('channelTypeInfom')
									}
								]
							}
						]
					},*/
				],
				attributes: ['channelId']
			});

			// if (!result.channelDetail) {
			// 	await new Utility().lazyload(result, 'channelDetail', {
			// 		required: false,
			// 		attributes: ['currencyid'],
			// 		include: [{
			// 			model: Currency,
			// 			required: false,
			// 			attributes: ['currency']
			// 		}]
			// 	});
			// }

			// if (!result.channelLocation) {
			// 	await new Utility().lazyload(result, 'channelLocation', {
			// 		where: { deletedAt: null },
			// 		required: false,
			// 		include: [{
			// 			model: this.regionCountryJCTRepository,
			// 			required: false,
			// 			where: { deletedAt: null },
			// 			include: [
			// 				{ model: Country, required: false, attributes: ['countryId', 'country'] }
			// 			]
			// 		}]
			// 	});
			// }

			// if (!result.channelCustomerProfile) {
			// 	await new Utility().lazyload(result, 'channelCustomerProfile', {
			// 		where: { deletedAt: null },
			// 		required: false,
			// 		include: [
			// 			{
			// 				model: this.profileTypeClassificationRepository.scope('profileTypeClassificationInfom'),
			// 				where: { deletedAt: null },
			// 				required: false,
			// 				include: [
			// 					{
			// 						model: this.profileTypeRepository.scope('profileTypeInfom'),
			// 						where: { deletedAt: null },
			// 						required: false,
			// 					},
			// 					{
			// 						model: this.channelTypeRepository.scope('channelTypeInfom')
			// 					}
			// 				]
			// 			}
			// 		]
			// 	});
			// }

			result['unit'] = await this.unitRepository.findAll();
			result['discountUnit'] = await this.discountUnitRepository.findAll();
			result['currency'] = await Currency.findAll();
			return result;

		} catch (e) {
			console.log("Error+ ", e);
			throw new InternalServerErrorException();
		}
	}

	public async productFileUpload(){
		
	}

	async sampleOrUninsertedProducts(data: any, fileName: any, type: any): Promise<any> {

		// const fields = ['product', 'productCategory', 'productBrand', 'productDesc', 'usp', 'productArticleReview_Name_1', 'productArticleReview_URL_1', 'productArticleReview_Logo_1', 'productArticleReview_Name_2', 'productArticleReview_URL_2', 'productArticleReview_Logo_2', 'productArticleReview_Name_3', 'productArticleReview_URL_3', 'productArticleReview_Logo_3', 'productArticleReview_Name 4', 'productArticleReview_URL_4', 'productArticleReview_Logo_4', 'productImage', 'productDisplayImage', 'productPackagingImage', 'productBrochure_1', 'productBrochure_2', 'productBrochure_3', 'productVisibility', 'isShowToFreeSub', 'EAN', 'UPC', 'inventoryCode', 'hasAnyVariant', 'variantDetails', 'currency', 'retailerPrice', 'retailerPriceUnit', 'isSampleFree', 'samplePrice', 'samplePriceUnit', 'isUnlimitSample', 'sampleThreshold', 'productQualityLevel', 'minOrderQuantity', 'minOrderQuantityUnit', 'productRelevantRetailerChannel', 'productSellingLocation', 'productTargetLocation', 'isAvailabelImmediate', 'availabelFrom', 'availabilityComment'];
		const fields = ['product', 'productCategory', 'productBrand', 'productDesc', 'tweetPitch', 'EAN', 'inventoryCode', 'hasAnyVariant', 'variantDetails', 'retailerPrice', 'retailerPriceUnit', 'isSampleFree', 'samplePrice', 'samplePriceUnit', 'isUnlimitSample', 'sampleThreshold', 'productQualityLevel', 'minOrderQuantity', 'minOrderQuantityUnit', 'isAvailabelImmediate', 'availabelFrom', 'availabilityComment', 'productImage', 'productDisplayImage', 'productPackagingImage'];

		var xl = require('excel4node');
		var wb = new xl.Workbook();
		var ws = wb.addWorksheet('Products');
		var ws2 = wb.addWorksheet('Masters');

		wb.createStyle({
			font: {
				size: 10
			}
		});

		if (type == 'sample') {

			for (let i = 0; i < fields.length; i++) {
				ws.cell(1, i + 1).string(fields[i]);
			}

			// ws2.cell(1, 1).string('Product Group').style({ font: { bold: true } });
			// for (let i = 0; i < data.productGroup.length; i++) {
			// 	ws2.cell(i + 2, 1).string(data.productGroup[i]);
			// }

			ws2.cell(1, 1).string('Product Category').style({ font: { bold: true } });
			for (let i = 0; i < data.productCategory.length; i++) {
				ws2.cell(i + 2, 1).string(data.productCategory[i]);
			}

			ws2.cell(1, 2).string('Product Brand').style({ font: { bold: true } });
			for (let i = 0; i < data.brand.length; i++) {
				ws2.cell(i + 2, 2).string(data.brand[i]);
			}

			if (data.unit.length > 0) {
				ws2.cell(1, 3).string('Units').style({ font: { bold: true } });
				for (let i = 0; i < data.unit.length; i++) {
					ws2.cell(i + 2, 3).string(data.unit[i]);
				}
			}

			if (data.discountUnit.length > 0) {
				ws2.cell(1, 4).string('Discount Units').style({ font: { bold: true } });
				for (let i = 0; i < data.discountUnit.length; i++) {
					ws2.cell(i + 2, 4).string(data.discountUnit[i]);
				}
			}

			// if (data.locations.TARGET.length > 0) {
			// 	ws2.cell(1, 5).string('Target Location').style({ font: { bold: true } });
			// 	ws2.cell(2, 5).string(data.locations.TARGET.all);
			// }

			// if (data.locations.SELLING.length > 0) {
			// 	ws2.cell(1, 6).string('Selling Location').style({ font: { bold: true } });
			// 	ws2.cell(2, 6).string(data.locations.SELLING.all);
			// }

			// if (data.locations.RETAIL.length > 0) {
			// 	ws2.cell(1, 7).string('Retail Location').style({ font: { bold: true } });
			// 	ws2.cell(2, 7).string(data.locations.RETAIL.all);
			// }

			// if (data.shareType.length > 0) {
			// 	ws2.cell(1, 8).string('Product Visibility').style({ font: { bold: true } });
			// 	ws2.cell(2, 8).string(data.shareType.all);
			// }

			// if (data.customerProfile.DISTRIBUTOR.length > 0) {
			// 	ws2.cell(1, 9).string('Relevant Distributor Channel').style({ font: { bold: true } });
			// 	ws2.cell(2, 9).string(data.customerProfile.DISTRIBUTOR.all);
			// }

			// if (data.customerProfile.RETAILER.length > 0) {
			// 	ws2.cell(1, 10).string('Relevant Retailer Channel').style({ font: { bold: true } });
			// 	ws2.cell(2, 10).string(data.customerProfile.RETAILER.all);
			// }

			// if (data.currency.length > 0) {
			// 	ws2.cell(1, 11).string('Currency').style({ font: { bold: true } });
			// 	for (let i = 0; i < data.currency.length; i++) {
			// 		ws2.cell(i + 2, 11).string(data.currency[i].currency);
			// 	}
			// }

			// ws.addDataValidation({
			// 	type: 'list',
			// 	prompt: 'Choose from dropdown',
			// 	error: 'Invalid choice was chosen',
			// 	showDropDown: true,
			// 	sqref: 'B2:B5000',
			// 	formulas: ["=Masters!$A$2:$A$100"]
			// });

			ws.addDataValidation({
				type: 'list',
				prompt: 'Choose from dropdown',
				error: 'Invalid choice was chosen',
				showDropDown: true,
				sqref: 'B2:B5000',
				allowBlank: false,
				formulas: ["=Masters!$A$2:$A$100"]
			});

			ws.addDataValidation({
				type: 'list',
				prompt: 'Choose from dropdown',
				error: 'Invalid choice was chosen',
				showDropDown: true,
				sqref: 'C2:C5000',
				formulas: ["=Masters!$B$2:$B$100"]
			});

			// ws.addDataValidation({
			// 	type: 'list',
			// 	prompt: 'Choose from dropdown',
			// 	error: 'Invalid choice was chosen',
			// 	showDropDown: true,
			// 	sqref: 'Y2:Y5000',
			// 	formulas: ["yes,no"]
			// });

			ws.addDataValidation({
				type: 'list',
				prompt: 'Choose from dropdown',
				error: 'Invalid choice was chosen',
				showDropDown: true,
				sqref: 'H2:H5000',
				formulas: ["no,yes"]
			});

			// ws.addDataValidation({
			// 	type: 'list',
			// 	prompt: 'Choose from dropdown',
			// 	error: 'Invalid choice was chosen',
			// 	showDropDown: true,
			// 	sqref: 'AE2:AE5000',
			// 	formulas: [
			// 		data.currencyNames
			// 	]
			// });

			ws.addDataValidation({
				type: 'list',
				prompt: 'Choose from dropdown',
				error: 'Invalid choice was chosen',
				showDropDown: true,
				sqref: 'K2:K5000',
				formulas: [
					data.unitNames
				]
			});

			/*ws.addDataValidation({
				type: 'list',
				prompt: 'Choose from dropdown',
				error: 'Invalid choice was chosen',
				showDropDown: true,
				sqref: 'AJ2:AJ5000',
				formulas: [
					data.discountUnitNames
				]
			});

			ws.addDataValidation({
				type: 'list',
				prompt: 'Choose from dropdown',
				error: 'Invalid choice was chosen',
				showDropDown: true,
				sqref: 'AL2:AL5000',
				formulas: [
					data.discountUnitNames
				]
			});*/

			ws.addDataValidation({
				type: 'list',
				prompt: 'Choose from dropdown',
				error: 'Invalid choice was chosen',
				showDropDown: true,
				sqref: 'L2:L5000',
				formulas: ["yes,no"]
			});

			ws.addDataValidation({
				type: 'list',
				prompt: 'Choose from dropdown',
				error: 'Invalid choice was chosen',
				showDropDown: true,
				sqref: 'N2:N5000',
				formulas: [
					data.unitNames
				]
			});

			ws.addDataValidation({
				type: 'list',
				prompt: 'Choose from dropdown',
				error: 'Invalid choice was chosen',
				showDropDown: true,
				sqref: 'O2:O5000',
				formulas: ["yes,no"]
			});

			ws.addDataValidation({
				type: 'list',
				prompt: 'Choose from dropdown',
				error: 'Invalid choice was chosen',
				showDropDown: true,
				sqref: 'Q2:Q5000',
				formulas: [
					appconstant.PRODUCT_QUALITY_LEVEL_TYPE
				]
			});

			ws.addDataValidation({
				type: 'list',
				prompt: 'Choose from dropdown',
				error: 'Invalid choice was chosen',
				showDropDown: true,
				sqref: 'S2:S5000',
				formulas: [
					data.unitNames
				]
			});

			/*ws.addDataValidation({
				type: 'list',
				prompt: 'Choose from dropdown',
				error: 'Invalid choice was chosen',
				showDropDown: true,
				sqref: 'AV2:AV5000',
				formulas: [
					data.unitNames
				]
			});

			ws.addDataValidation({
				type: 'list',
				prompt: 'Choose from dropdown',
				error: 'Invalid choice was chosen',
				showDropDown: true,
				sqref: 'AX2:AX5000',
				formulas: [
					data.unitNames
				]
			});*/

			ws.addDataValidation({
				type: 'list',
				prompt: 'Choose from dropdown',
				error: 'Invalid choice was chosen',
				showDropDown: true,
				sqref: 'T2:T5000',
				formulas: ["yes,no"]
			});

			// if (data.channelCurrency) {
			// 	ws.cell(2, 31, 2, 31).string(data.channelCurrency);
			// }

			// ws.cell(2, 24, 2, 24).string(data.shareType.all).style({ border: { bottom: { style: 'thin', color: '#D4762C' } } });
			// //ws.cell(2, 43, 5000, 43).string(data.customerProfile.DISTRIBUTOR.all).style({ border: { bottom: { style: 'thin', color: '#D4762C' } } });
			// ws.cell(2, 42, 2, 42).string(data.customerProfile.RETAILER.all).style({ border: { bottom: { style: 'thin', color: '#D4762C' } } });
			// ws.cell(2, 43, 2, 43).string(data.locations.SELLING.all).style({ border: { bottom: { style: 'thin', color: '#D4762C' } } });
			// ws.cell(2, 44, 2, 44).string(data.locations.TARGET.all).style({ border: { bottom: { style: 'thin', color: '#D4762C' } } });

		} else {
			const uifields = ['product', 'errors'];
			data.map(function (data, index) {
				if (!data.errors) {
					data.errors = 'Invalid Images or Master data';
				} else {
					data.errors = 'Invalid columns : ' + data.errors;
				}
				for (var j = 0; j < uifields.length; j++) {
					if (index == 0) {
						ws.cell(1, j + 1).string(uifields[j]);
					}
					ws.cell(2 + index, 1 + j).string((data[uifields[j]]) ? data[uifields[j]] : '');
				}
			});

			ws.cell(2, 2, 5000, 2).style({ font: { color: '#FF0000' } });
		}

		await wb.writeToBuffer().then(function (buffer) {
			fs.writeFileSync(path.resolve(__dirname, '..', '..', '..', '..') + '/.tmp/upload_s3/' + fileName, buffer, 'utf-8', function (err) {
				if (err) throw err;
				return true;
			});
		});
	}

	public async addProduct(addProductDto: AddProductDto, user: any, subscriptionFeatureKeys?, importFlag?): Promise<any> {
		let product: any = "";
		let productUpdate: any = "";
		let result: object = { error: true };

		let key = +user.channel.subscriptionId + '-' + +features.PRODUCT_SHEET;
		if (subscriptionFeatureKeys && subscriptionFeatureKeys[key]) {
			let productCount = await this.productRepository.count({ where: { channelId: +user.channelId, deletedat: null } });
			if (productCount >= +subscriptionFeatureKeys[key]) {
				throw new ProcessException({ message: 'Based on your Subscription, you are eligible to add ' + +subscriptionFeatureKeys[key] + ' products. Already you have ' + productCount + ' products.', eligibleCount: subscriptionFeatureKeys[key], exitCount: productCount });
			}
		}

		key = +user.channel.subscriptionId + '-' + +features.PRODUCT_VIDEO_BY_SHEET;
		if (subscriptionFeatureKeys && subscriptionFeatureKeys[key]) {
			let videoUrl = (addProductDto.productImages && addProductDto.productImages.videourl && addProductDto.productImages.videourl.length > 0);
			if (!subscriptionFeatureKeys[key] && videoUrl) {
				throw new ProcessException({ message: 'Based on your Subscription, you are not eligible to add videos in product.' });
			}
		}

		//load channel currency
		// if (user.channel.channelDetail === undefined) {
		// 	await this.utility.lazyload(user.channel, 'channelDetail');
		// }

		let productObject = await this.productServiceInBound.setProductObject(addProductDto, user);

		productObject['slug'] = await this.setSlugName(this.productRepository, addProductDto.productBasicInfo.productname);

		productUpdate = await this.sequelizeInstance.transaction(async transaction => {
			// Add product

			console.log('productObjectproductObjectproductObjectproductObject');
			console.log(productObject.productImage);
			product = await this.productRepository.create(productObject, {
				include: [
					{
						model: this.productDetailRepository
					},
					{
						model: this.productTagRepository
					},
					{
						model: this.productImageRepository,
						as: "productImage",
						include: [
							{
								model: this.mediaRepository

							}
						]
					},
					{
						model: this.productSharePreferenceRepository
					},
					{
						model: this.productUniversalCodeRepository

					},
					{
						model: this.productArticleReviewRepository,
						include: [
							{
								model: this.mediaRepository

							}
						]
					},
					{
						model: this.productExistRatingRepository,
						include: [
							{
								model: this.mediaRepository

							}
						]

					},
					{
						model: this.productPriceDetailRepository

					},
					{
						model: this.productPackageInfoRepository

					},
					// {
					// 	model: this.productCompetingRepository
					//
					// },
					{
						model: this.productChannelCustomerProfileRepository

					}
				],
				//returning: true,
				transaction,
			});

			// add product awards
			// if (addProductDto.award_details && Object.keys(addProductDto.award_details).length > 0) {
			// 	for (let ii = 0; ii < addProductDto.award_details.length; ii++) {
			// 	//	let valid_upto = (addProductDto.award_details[ii].valid_upto) ? addProductDto.award_details[ii].valid_upto : null;

			// 		//save awards image
			// 		if (addProductDto.award_details[ii].image) {
			// 			let media = <Media>{};
			// 			media.documentTypeId = appconstant.DOCUMENTTYPE_IMAGE;
			// 			media.documentPath = (addProductDto.award_details[ii].image) ? addProductDto.award_details[ii].image : null;
			// 			media.dirName = appconstant.FOLDER_PATH_PRODUCT_AWARDS;

			// 		} else {
			// 			addProductDto.award_details[ii].imageId = null
			// 		}

			// 	}
			// }
			// Add product SKU and slug
			let productUpdateObject = await this.productServiceInBound.setProductUpdateObject(addProductDto, product);

			// check make product only active after image upload
			if (product.isactive == '1') {
				await this.productImageRepository.count({
					where: {
						productId: product.productId,
						deletedat: null,
						productimagetype: {
							//$in: [appconstant.PRODUCT_IMAGE_TYPE]
							[Op.in]: [appconstant.PRODUCT_IMAGE_TYPE]
						},
					},
					group: ['productimagetype'],
					transaction
				}).then((count: any) => {
					if (count && count.length != 1) {
						productUpdateObject.isactive = '0';
					}
				});
			}

			productUpdate = await this.productRepository.update(productUpdateObject,
				{
					where: { productId: product.productId },
					returning: true,
					transaction
				}
			);
			return productUpdate;
		}).then(async (productUpdate) => {
			return productUpdate;
		}).catch(function (e) {
			console.log("Error+", e);
			if (importFlag) {
				return false;
			} else {
				throw new ProcessException(e.message);
			}
		});

		if (productUpdate) {
			if (productUpdate[1] && productUpdate[1][0]) {
				let productReturnData = productUpdate[1][0];
				productUpdate = await this.setProductDefaultImage(productReturnData);

				// add key retailers
				await this.updateKeyRetailers(productReturnData, addProductDto, user, appconstant.CHANNEL_TYPE_RETAILER);
				await this.updateKeyRetailers(productReturnData, addProductDto, user, appconstant.CHANNEL_TYPE_DISTRIBUTOR);

				result = { success: true, data: productReturnData };
			}
		} else {
			result = productUpdate;
		}

		return result;
	}

	public async setProductDefaultImage(product) {
		let productDefaultImage = <ProductImage>{};
		let productDefaultImageId: any = "";
		try {
			productDefaultImage = await this.productImageRepository.scope('productInfo').findOne<ProductImage>({
				where: { productId: parseInt(product.productId), productimagetype: appconstant.PRODUCT_IMAGE_TYPE, isdefault: '1', deletedat: null }
			});
			if (productDefaultImage) {
				if (productDefaultImage && productDefaultImage.productimageid) {
					productDefaultImageId = productDefaultImage.productimageid;

					product.defaultimageid = productDefaultImageId

					// Add product default image
					await product.save();
				}
			}
		} catch (e) {
			console.log("Error+ setProductDefaultImage", e);
			throw new ProcessException(e.message);
		}
		return product;
	}

	public async getKeyRetailerIds(channelId, partnerType) {
		return await this.sequelizeInstance.query('select array_agg(keypartnerid) as ids from "tblKeyPartnerJCT" where channelid = :channelId and partner_type = :partnerType and deletedat is null', {
			type: QueryTypes.SELECT,
			replacements: {
				channelId: channelId,
				partnerType: partnerType
			}
		}).then((result) => {
			return (result[0].ids) ? result[0].ids : [];
		});
	}

	async updateKeyRetailers(product, dto, user, partner_type) {
		// get key retailer list using channelId
		let keyRetailers = await this.getKeyRetailerIds(+product.channelId, partner_type);
		let dtoKeyPartners = (+partner_type == +appconstant.CHANNEL_TYPE_RETAILER) ? dto.keyRetailers : dto.keyDistributors;
		let dtoDeleteKeyPartners = (+partner_type == +appconstant.CHANNEL_TYPE_RETAILER) ? dto.deletedkeyretailer : dto.deletedkeydistributor

		if (dtoKeyPartners) {
			var receivedNewKeyRetailers = await this.tradeInformationInBound.keyRetailers(partner_type, dtoKeyPartners, +product.channelId, +user.userId);

			// filter valid keyretailerid
			// let validRetailers = map(filter(dtoKeyPartners, (key) => {
			// 	return (key.keypartnerid && keyRetailers.includes('' + key.keypartnerid))
			// }), 'keypartnerid');

			// // delete all the keyretailerIds and update
			// await ProductKeyPartnerJCT.destroy({
			// 	where: {
			// 		productid: product.productId,
			// 		keypartnerid: {
			// 			$in: keyRetailers
			// 		}
			// 	}
			// }).then(async () => {
			// 	let keyretailers = [];
			// 	validRetailers.forEach((retailer) => {
			// 		keyretailers.push({
			// 			productid: +product.productId,
			// 			channelid: +product.channelId,
			// 			keypartnerid: retailer
			// 		})
			// 	});

			// 	console.log(receivedNewKeyRetailers);

			// 	if (receivedNewKeyRetailers.length > 0) {
			// 		receivedNewKeyRetailers.forEach((retailer) => {
			// 			keyretailers.push({
			// 				productid: +product.productId,
			// 				channelid: +product.channelId,
			// 				keypartnerid: retailer
			// 			})
			// 		});
			// 	}

			// 	if (keyretailers) {
			// 		await ProductKeyPartnerJCT.bulkCreate(keyretailers);
			// 		// console.log(keyretailers);					
			// 	}
			// });
		} else if (dtoDeleteKeyPartners) {
			// filter valid keyretailerid
			// let validDeletedRetailers = filter(dtoDeleteKeyPartners, (key) => {
			// 	return keyRetailers.includes('' + key)
			// });

			// await ProductKeyPartnerJCT.destroy({
			// 	where: {
			// 		productid: product.productId,
			// 		keypartnerid: {
			// 			$in: validDeletedRetailers
			// 		}
			// 	}
			// });
		}
	}


	async productImportCheck(products: any, masters: any): Promise<any> {
		try {
			

			let errors: string = '';

			errors += (!products.tweetPitch) ? '\n  tweetPitch \n    * Can\'t be to empty,' : '';
			errors += (products.tweetPitch && products.tweetPitch.trim().length >= 140) ? '\n  tweetPitch \n    * Can\'t be greater than 140,' : '';
			
			errors += (products.inventoryCode && (products.inventoryCode.trim().length < 3 || products.inventoryCode.trim().length > 38)) ? '\n  inventoryCode  \n    * should have 3 to 38 characters' : '';
			
			errors += (!products.hasAnyVariant) ? '\n  hasAnyVariant \n    * Can\'t be to empty,' : '';
			errors += (products.hasAnyVariant && products.hasAnyVariant.toLowerCase().trim() == 'yes' && !products.variantDetails) ? '\n  variantDetails \n    * Can\'t be to empty when hasAnyVariant set "yes",' : '';
			
			errors += (!products.retailerPrice) ? '\n retailerPrice \n    * Can\'t be to empty,' : '';
			
			errors += (!products.isSampleFree) ? '\n isSampleFree \n    * Can\'t be to empty,' : '';
			errors += (products.isSampleFree && products.isSampleFree.toLowerCase().trim() == 'no' && !products.samplePrice) ? '\n samplePrice \n    * Can\'t be to empty when isSampleFree set "no",' : '';
			errors += (!products.isUnlimitSample) ? '\n isUnlimitSample \n    * Can\'t be to empty,' : '';
			//errors += (products.isUnlimitSample && products.isUnlimitSample.toLowerCase().trim() == 'no' && !products.sampleThreshold) ? '\n sampleThreshold \n    * Can\'t be to empty when isUnlimitSample set "no",' : '';
			errors += (!products.productQualityLevel) ? '\n productQualityLevel \n    * Can\'t be to empty,' : '';
			errors += (!products.minOrderQuantity) ? '\n minOrderQuantity \n    * Can\'t be to empty,' : '';
			errors += (!products.minOrderQuantityUnit) ? '\n minOrderQuantityUnit \n    * Can\'t be to empty,' : '';
			
			errors += (!products.isAvailabelImmediate) ? '\n isAvailabelImmediate \n    * Can\'t be to empty,' : '';
			errors += (products.isAvailabelImmediate && products.isAvailabelImmediate.toLowerCase().trim() == 'no' && !products.availabelFrom) ? 'availabelFrom,' : '';
			
			// errors += (!products.productCategory || (products.productCategory && !includes(masters.productCategory, products.productCategory.toLowerCase().trim()))) ? 'productCategory,' : '';
			// errors += (!products.productBrand || (products.productBrand && !includes(masters.brand, products.productBrand.toLowerCase().trim()))) ? 'productBrand,' : '';
			// errors += (isBoolean(targetLocationCheck) && !targetLocationCheck) ? '\n  Invalid Target location,' : '';
			// errors += (isBoolean(sellingLocationCheck) && !sellingLocationCheck) ? '\n  Invalid Selling location,' : '';
			// errors += (isBoolean(retailLocationCheck) && !retailLocationCheck) ? '\n  Invalid Retail location,' : '';
			// errors += (isBoolean(shareTypeCheck) && !shareTypeCheck) ? '\n  Invalid Share Preference,' : '';
			//errors += (productRelevantDistributorCheck == false) ? 'Invalid Relevant Distributors,' : '';
			// errors += (isBoolean(productRelevantRetailerCheck) && !productRelevantRetailerCheck) ? '\n  Invalid Relevant Retailers,' : '';

			if (products.product) {
				let productcheck = await this.productRepository.findAll({
					where: {
						product: products.product.trim(),
						deletedat: null
					}
				});


				if (productcheck && productcheck.length > 0) {
					errors += 'Product Name already exist in ChannelHub '+products.product.trim();
				}
			}

			let productRowCheck = (!products.product) ? 0 : 1;

			if (errors != '') {
				return { status: false, errors: errors, productRowCheck: productRowCheck };
			}
			return { status: true };
		} catch (e) {
			console.log("Error+ ", e);
			throw new InternalServerErrorException();
		}
	}
}