'use strict';

import { Injectable, Inject } from '@nestjs/common';
import { ProcessException } from './../../../middleware/exception/Process.exception';
import { RejectReason } from '../models/RejectReason.entity';
import { ProductReject } from './../models/ProductReject.entity';
import { Cart } from './../../cart/models/Cart.entity';
import { ProductQuote } from './../../cart/models/ProductQuote.entity';
import { Product } from './../../product/models/Product.entity';
import { Sequelize } from 'sequelize-typescript';
import { isNull } from 'lodash';
import {Op, QueryTypes } from "sequelize";
import { BaseService } from 'src/core/service/base.service';
import { Wishlist } from 'src/modules/wishlist/models/Wishlist.entity';

@Injectable()
export class RejectReasonService extends BaseService {
	constructor(
		@Inject('RejectReasonRepository') private readonly repository: typeof RejectReason,
		@Inject('ProductRejectRepository') private readonly rejectRepository: typeof ProductReject,
		@Inject('SequelizeInstance') private readonly sequelizeInstance,
		@Inject('ProductRepository') private readonly productRepository: typeof Product,
		// @Inject('NotificationService') private readonly notificationService,
	) {
		super();
	}

	async create(data): Promise<any> {
		return await this.repository.create<RejectReason>(this._buildObject(new Object(), data));
	}

	async _validateProduct(productId) {
		let product = await this.productRepository.scope('productReject').findByPk(productId);

		if (!product) {
			throw new ProcessException('validate:field:notFound');
		}

		return product;
	}

	async reject(data, user) {
		return await this.sequelizeInstance.transaction(async transaction => {
			// let requestObject: IMailNotification = {};
			// requestObject.reason = data.comment;
			let product = await this._validateProduct(data.productId);

			try {
				return await this.rejectRepository.create<ProductReject>({
					userid: +user.userId,
					channelid: +user.channelId,
					productid: data.productId,
					rejectreasonid: data.rejectreasonId,
					otherreasons: data.comment,
				}, {
						transaction
					}).then(async (data) => {
						let productId = data.productid;
						let channelId = data.channelid;

						// if (requestObject.reason == '') {
						// 	let reason = await this.repository.findByPk(data.rejectreasonid);
						// 	requestObject.reason = reason.rejectreason;
						// }

						// requestObject.senderId = user.contactId;
						// requestObject.receiverId = await this.sequelizeInstance.query('select "contactId", "isDefaultContact", "workNatureId"  from "tblContact" CON where "channelId" = :partnerChannelId', {
						// 	type: QueryTypes.SELECT,
						// 	replacements: {
						// 		partnerChannelId: product.channelId
						// 	}
						// }).then((result) => {
						// 	let otherUsers = [];
						// 	if (result && result.length > 0) {
						// 		if (result.length == 1) {
						// 			return +result[0].contactId;
						// 		} else {
						// 			result.forEach((contact) => {
						// 				if (contact.isDefaultContact == '1') {
						// 					otherUsers = [contact];
						// 					return false
						// 				} else if (contact.workNatureId != null) {
						// 					otherUsers.push(contact);
						// 				}
						// 			});

						// 			return +otherUsers[0].contactId;
						// 		}
						// 	} else {
						// 		return null;
						// 	}
						// });

						// if (isNull(requestObject.receiverId)) {
						// 	delete requestObject.receiverId;
						// }

						// let evaluateQuery = 'select count(pe.productid) from "tblproductevaluation" pe where pe.channelid = :channelId and pe."productid"= :productId';
						// let evaluated = await this.sequelizeInstance.query(evaluateQuery, {
						// 	type: this.sequelizeInstance.QueryTypes.SELECT,
						// 	raw: true,
						// 	replacements: {
						// 		productId: productId,
						// 		channelId: channelId
						// 	}
						// }).then(result => {
						// 	return !!(+result[0].count);
						// });

						// requestObject.senderId = +user.contactId;
						// requestObject.productId = productId;
						// let action = notification.NOT_INTEREST_IN_THIS_PRODUCT;
						// if (evaluated) {
						// 	action = notification.NOT_INTEREST_IN_THIS_PRODUCT_AF_EVA;

						// 	// notification removed for general not interested product #M181063
						// 	this.notificationService.sendNotificationRouter(requestObject, action);
						// }

						// this.notificationService.sendNotificationRouter(requestObject, action);

						// Remove the products from wishlist based on channelId
						await Wishlist.destroy({
							where: {
								channelid: channelId,
								productid: productId
							},
							transaction
						});

						// Remove the products from cart based on channelId
						let cart = await Cart.findOne({
							where: {
								sellerchannelid: product.channelId,
								buyerchannelid: channelId,
								deletedat: null
							},
							transaction
						});

						if (cart) {
							await ProductQuote.destroy({
								where: {
									cartid: cart.cartid,
									productid: productId
								},
								transaction
							});
						}
					});
			} catch (e) {
				console.log('e',e)
				let message = 'validate:sql:internalError';
				if (e.errors && (e.errors[0].message == 'productid must be unique')) {
					message = 'this product already removed';
				}

				throw new ProcessException(message);
			}
		});
	}

	public async findAll(pagination): Promise<Array<RejectReason>> {
		return await this.repository.findAll<RejectReason>({
			limit: pagination.limit,
			offset: pagination.offset,
			order: [['rejectreasonid', 'DESC']]
		});
	}

	public async findOne(options: Object): Promise<RejectReason | null> {
		let result = await this.repository.findOne<RejectReason>(options);
		if (!result) {
			throw new ProcessException('validate:field:notFound');
		}

		return result;
	}

	public async findById(id: number): Promise<RejectReason | null> {
		let result = await this.repository.findByPk<RejectReason>(id);
		if (!result) {
			throw new ProcessException('validate:field:notFound');
		}

		return result;
	}

	public async update(id: number, newValue): Promise<RejectReason | null> {
		let result = await this.repository.findByPk<RejectReason>(id);
		if (!result) {
			throw new ProcessException('validate:field:notFound');
		}

		await this._buildObject(result, newValue).save();

		return result;
	}

	public async delete(id: number): Promise<void> {
		let result = await this.repository.findByPk<RejectReason>(id);
		if (!result) {
			throw new ProcessException('media:documentTypeId:notFound');
		}

		return await result.destroy();
	}

	public async getCount() {
		return await this.repository.count();
	}

	private _buildObject(entity, dto) {
		entity.rejectreason = dto.rejectreason;

		return entity;
	}
}
