'use strict';

import { Injectable, Inject } from '@nestjs/common';
import { Unit } from '../models/Unit.entity';
import { UnitDto } from '../dto/unit.dto';
import { ProcessException } from 'src/middleware/exception/Process.exception';

@Injectable()
export class UnitService {
	constructor(
		@Inject('UnitRepository') private readonly repository: typeof Unit,
		@Inject('SequelizeInstance') private readonly sequelizeInstance
	) {
		// super();
	}

	async create(data: UnitDto): Promise<any> {
		return await this.repository.create<Unit>(this._buildObject(new Object(), data));
	}

	public async findAll(pagination): Promise<Array<Unit>> {
		return await this.repository.findAll<Unit>({
			limit: pagination.limit,
			offset: pagination.offset,
			order: [['unitid', 'DESC']]
		});
	}

	public async findOne(options: Object): Promise<Unit | null> {
		let result = await this.repository.findOne<Unit>(options);
		if (!result) {
			throw new ProcessException('validate:field:notFound');
		}

		return result;
	}

	public async findByPk(id: number): Promise<Unit | null> {
		let result = await this.repository.findByPk<Unit>(id);
		if (!result) {
			throw new ProcessException('validate:field:notFound');
		}

		return result;
	}

	public async update(id: number, newValue): Promise<Unit | null> {
		let result = await this.repository.findByPk<Unit>(id);
		if (!result) {
			throw new ProcessException('validate:field:notFound');
		}

		await this._buildObject(result, newValue).save();

		return result;
	}

	public async delete(id: number): Promise<void> {
		let result = await this.repository.findByPk<Unit>(id);
		if (!result) {
			throw new ProcessException('media:documentTypeId:notFound');
		}

		return await result.destroy();
	}

	public async getCount() {
		return await this.repository.count();
	}

	private _buildObject(entity, dto) {
		entity.unit = dto.unit;

		return entity;
	}
}
