'use strict';

import { Controller, HttpStatus, Get, Post, Request, Response, Param, Body, Put, UseGuards, Delete } from '@nestjs/common';
//import { ApiImplicitQuery, ApiBearerAuth, ApiUseTags, ApiOperation, ApiImplicitParam, ApiImplicitHeader, ApiResponse } from '@nestjs/swagger';

import { ValidationPipe } from '../../../middleware/pipe/validation.pipe';

import { MyAddressBookService } from './../service/myAddressBook.service';
import { MyAddressBookDto } from './../dto/myAddressBook.dto';
import { AuthGuard } from '@nestjs/passport';
//import { Pagination } from './../../common/decorators/pagination.decorator';
//import { PaginationOptions } from './../../common/decorators/pagination.model';
//import { ContextService } from './../../../context/context.service';
//import { PaginationTransform } from './../../core/transformation/paginationTransformation';

@Controller('addressbook')
//@UseGuards(RolesGuard)
@UseGuards(AuthGuard('jwt'))
export class AddressBookController {

//	private paginationTransform = new PaginationTransform();

	constructor(
		private service: MyAddressBookService,
	) {
		//super();
	}

	@Get('/')	
	// @Roles(Channel.BACKEND_CHANNEL)
	public async getAll(@Request() req, @Response() res) {
		let user = req.user;
		let total = 0;
		
		
		this.service.getCount(user, (req.query.type) ? req.query.type : null).then(function(c) {
			total = c;
		});

		const list = await this.service.findAll(user, (req.query.type) ? req.query.type : null);

		return res.status(HttpStatus.OK).send(list
			//this.paginationTransform.make(list, pagination, total)
		);
	}

	@Get('/:id')	
	public async getById(@Param('id') id, @Request() req, @Response() res) {
		let user = req.user;

		const row = await this.service.findById(id, user);
		return res.status(HttpStatus.OK).json(row);
	}

	@Post('/')	
	public async create(@Request() req, @Response() res, @Body(new ValidationPipe()) dto: MyAddressBookDto) {
		let user = req.user;

		let result = await this.service.create(dto, user);
		return res.status(HttpStatus.CREATED).send(result);
	}

	@Put('/:id')	
	public async update(@Param('id') id, @Request() req, @Response() res, @Body(new ValidationPipe()) dto: MyAddressBookDto) {
		let user = req.user;

		const row = await this.service.update(id, dto, user);
		return res.status(HttpStatus.OK).json(row);
	}

	@Delete('/:id')	
	public async delete(@Param('id') id, @Request() req, @Response() res) {
		let user = req.user;

		let row = await this.service.delete(id, user);
		// if (row) {
		// 	return res.status(HttpStatus.NO_CONTENT).json();
		// } else {
		// 	return res.status(HttpStatus.NOT_FOUND).send();
		// }
		return res.status(HttpStatus.OK).send();
	}

	
}
