import { ArrayNotEmpty, ValidateNested, IsOptional, IsNotEmpty, IsNumber, ValidateIf } from 'class-validator';
//import { ApiModelProperty } from '@nestjs/swagger';
import { AddContactDto } from './AddUser/contact.dto';
import { Type } from 'class-transformer';
import { TradeInformation_Product_FamiliesDto } from './../../channel/dto/tradeInformation_Product_Families.dto';
import { TradeInformation_Product_CategoriesDto } from './../../channel/dto/tradeInformation_Product_Categories.dto';
import { AccessPermissionDto } from './AddUser/accesspermission.dto';
import { ChannelLocationDto } from './../../channel/dto/channelLocation.dto';

import { includes } from 'lodash';
import { appconstant } from 'src/config/app.constant';

export class AddUserDto {

	@ValidateNested()
	@Type(() => AddContactDto)
//	@ApiModelProperty({ type: AddContactDto })
	contact: AddContactDto

	@ValidateIf(o => (includes(appconstant.SOURCING_CONTACT, o.contact.workNatureId) && o.channelTypeId == appconstant.CHANNEL_TYPE_DISTRIBUTOR) || o.channelTypeId == appconstant.CHANNEL_TYPE_VENDOR || o.channelTypeId == appconstant.CHANNEL_TYPE_RETAILER)
//	@ApiModelProperty({ type: TradeInformation_Product_FamiliesDto, isArray: true })
	@ArrayNotEmpty({ message: "channel:tradeinformation:productFamiliesArray" })
	@IsNotEmpty({ message: "channel:tradeinformation:productFamiliesArrayNotEmpty" })
	@ValidateNested()
	@Type(() => TradeInformation_Product_FamiliesDto)
	productFamilies: TradeInformation_Product_FamiliesDto[];

	@ValidateIf(o => (includes(appconstant.SOURCING_CONTACT, o.contact.workNatureId) && o.channelTypeId == appconstant.CHANNEL_TYPE_DISTRIBUTOR) || o.channelTypeId == appconstant.CHANNEL_TYPE_VENDOR || o.channelTypeId == appconstant.CHANNEL_TYPE_RETAILER)
//	@ApiModelProperty({ type: TradeInformation_Product_CategoriesDto, isArray: true })
	@ArrayNotEmpty({ message: "channel:tradeinformation:productCategoriesArray" })
	@IsNotEmpty({ message: "channel:tradeinformation:productCategoriesNotEmpty" })
	@ValidateNested()
	@Type(() => TradeInformation_Product_CategoriesDto)
	productCategories: TradeInformation_Product_CategoriesDto[];

	@IsOptional()
	@ValidateIf(o => (includes(appconstant.SOURCING_CONTACT, o.contact.workNatureId) && o.channelTypeId == appconstant.CHANNEL_TYPE_DISTRIBUTOR) || o.channelTypeId == appconstant.CHANNEL_TYPE_VENDOR || o.channelTypeId == appconstant.CHANNEL_TYPE_RETAILER)
//	@ApiModelProperty({ type: TradeInformation_Product_FamiliesDto, isArray: true })
	@ArrayNotEmpty({ message: "validate:field:shouldBeEmptyArray" })
	@IsNotEmpty({ message: "validate:field:shouldNotEmpty" })
	@ValidateNested()
	@Type(() => TradeInformation_Product_FamiliesDto)
	interestedProductFamilies: TradeInformation_Product_FamiliesDto[];

	@IsOptional()
	@ValidateIf(o => (includes(appconstant.SOURCING_CONTACT, o.contact.workNatureId) && o.channelTypeId == appconstant.CHANNEL_TYPE_DISTRIBUTOR) || o.channelTypeId == appconstant.CHANNEL_TYPE_VENDOR || o.channelTypeId == appconstant.CHANNEL_TYPE_RETAILER)
//	@ApiModelProperty({ type: TradeInformation_Product_CategoriesDto, isArray: true })
	@ArrayNotEmpty({ message: "validate:field:shouldBeEmptyArray" })
	@IsNotEmpty({ message: "validate:field:shouldNotEmpty" })
	@ValidateNested()
	@Type(() => TradeInformation_Product_CategoriesDto)
	interestedProductCategories: TradeInformation_Product_CategoriesDto[];

	@IsOptional()
//	@ApiModelProperty({ type: AccessPermissionDto, isArray: true })
	@ArrayNotEmpty({ message: "validate:field:shouldBeArray" })
	@IsNotEmpty({ message: "validate:field:shouldNotEmpty" })
	@ValidateNested()
	@Type(() => AccessPermissionDto)
	readonly accessPermissions: AccessPermissionDto[];

	@ValidateIf(o => (includes([...appconstant.SELLING_CONTACT, ...appconstant.OTHER_CONTACT], o.contact.workNatureId) && o.channelTypeId == appconstant.CHANNEL_TYPE_DISTRIBUTOR) || (o.channelTypeId == appconstant.CHANNEL_TYPE_VENDOR))
//	@ApiModelProperty({ type: ChannelLocationDto, isArray: true })
	@ArrayNotEmpty({ message: "validate:field:shouldBeArray" })
	@IsNotEmpty({ message: "validate:field:shouldNotEmpty" })
	@ValidateNested()
	@Type(() => ChannelLocationDto)
	locations: ChannelLocationDto[];

	@IsNumber({}, { message: "validate:field:shouldBeNumber" })
	@IsNotEmpty({ message: "validate:field:shouldNotEmpty" })
//	@ApiModelProperty()
	readonly channelTypeId: number;
}
