'use strict';
import {
	Table,
	Column,
	PrimaryKey,
	Model,
	AutoIncrement,
	DataType,
	BeforeCreate,
	AllowNull,
	BeforeUpdate,
	HasMany,
	ForeignKey,
	BelongsTo
} from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamps: false, tableName: 'tblaccesspermission' } as IDefineOptions;

@Table(tableOptions)
export class AccessPermission extends Model<AccessPermission>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	accesspermissionid: number

	@Column({
		type: DataType.CHAR(30),
		allowNull: false,
		unique: { name: 'accesspermissionid', msg: 'validate:field:shouldBeUnique' }
	})
	name: string;

	@Column
	isactive: string;

	@ForeignKey(() => AccessPermission)
	@Column(DataType.BIGINT)
	parentaccesspermissionid: number

	@BelongsTo(() => AccessPermission)
	AccessPermission: AccessPermission;

	@HasMany(() => AccessPermission)
	ChildAccessPermission: AccessPermission[];

	@Column(DataType.DATE)
	deletedat: Date;

	@Column(DataType.DATE)
	createdat: Date;

	@Column(DataType.DATE)
	updatedat: Date;

	@BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(accessPermission: AccessPermission, options: any) { 
		accessPermission.updatedat = Sequelize.fn('NOW'); 
	}
}
