import {
	Table,
	Column,
	PrimaryKey,
	Model,
	AutoIncrement,
	DataType,
	BeforeCreate,
	AllowNull,
	HasMany,
	ForeignKey,
	Scopes,
	BelongsTo,
	BelongsToMany,
	HasOne,
	BeforeUpdate
} from 'sequelize-typescript';
import { Channel } from '../../channel/models/Channel.entity';
import { ChannelType } from '../../channel/models/ChannelType.entity';
import { ProfileType } from '../../channel/models/ProfileType.entity';
import { ChannelSubscription } from './../../channel/models/ChannelSubscription.entity';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Country } from '../../common/models/Country.entity';
import { Media } from '../../media/models/Media.entity';
import { ContactProductGroup } from './ContactProductGroup.entity';
import { ContactProductCategory } from './ContactProductCategory.entity';
import { ProductGroup } from './../../product/models/ProductGroup.entity';
import { ProductCategory } from './../../product/models/ProductCategory.entity';
import { Responsibility } from '../../user/models/Responsibility.entity';
import { ContactSocialLink } from './ContactSocialLink.entity';
import { ContactLocation } from './ContactLocation.entity';
import { ChannelContactSuggestion } from '../../channel/models/ChannelContactSuggestion.entity';
import { User } from './../models/User.entity';
import { UserRole } from './../models/UserRole.entity';
import { RegionCountryJCT } from './../../common/models/RegionCountryJCT.entity';
import { ContactInterestedProductGroup } from './ContactInterestedProductGroup.entity';
import { ContactInterestedProductCategory } from './ContactInterestedProductCategory.entity';
import { Subscription } from './../../channel/models/Subscription.entity';
import { SubscriptionPlan } from './../../channel/models/SubscriptionPlan.entity';
import { ChannelDetail } from '../../channel/models/ChannelDetail.entity';
import { Role } from './Role.entity';
import { EventParticipants } from 'src/modules/channel/models/EventParticipants.entity';

var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblContact' } as IDefineOptions;

@Table(tableOptions)

@Scopes(() => ({
	basicInfom: {
		attributes: ['contactId', 'channelId']
	},
	contactInfo: {
		attributes: ['contactId', 'firstName', 'lastName', 'phone1', 'phone2', 'mobileNo', 'email', 'jobTitle', 'linkedIn', 'googleplus', 'workNatureId', 'isShared', 'isActive', 'countryId', 'documentId', 'empId', 'hubspotContactId', 'rptContactId']
	},
	senderContactMail: {
		attributes: ['contactId', 'channelId', 'rptContactId', 'firstName', 'lastName', 'phone1', 'phone2', 'mobileNo', 'email', 'jobTitle', 'workNatureId', 'documentId', 'countryId', 'isConnectionMail','isSuggestMail', 'isFeatureMail','isProfileMail'],
		include: [
			{
				model:Channel,
				include: [
					{
						model:ChannelType,
					},
					{
						model:ProfileType,
						as: 'profileType'
					},
					{
						model:User,
						attributes: ["userId", "email"],
						include: [{
							model: UserRole,
							attributes: [],
							where: {
								roleId: Role.ROLE_ACCOUNT_USER
							}
						}, {
							model:Contact,
							attributes: ['contactId', 'channelId', 'firstName', 'lastName', 'email']
						}]
					}
				]
			}, {
				model:Country,
				attributes: ['countryId', 'country'],
			},
			{
				model:Media,
			}
		]
	},
	ReceverContactMail: {
		attributes: ['contactId', 'channelId', 'rptContactId', 'firstName', 'lastName', 'phone1', 'phone2', 'mobileNo', 'email', 'jobTitle', 'workNatureId', 'documentId', 'countryId', 'isConnectionMail','isSuggestMail', 'isFeatureMail','isProfileMail'],
	},
	companyWithActivePlan: {
		attributes: {
			exclude: ['deletedAt', 'createdAt', 'updatedAt']
		},
		include: [
			{
				model:Channel,
				where: {
					deletedAt: null
				},
				attributes: {
					exclude: ['deletedAt', 'createdAt', 'updatedAt', 'stripeCustomerId']
				},
				include: [
					{
						model:SubscriptionPlan,
						as: 'plan',
						attributes: ['subscriptionPlanId', 'subscriptionPlanType', 'rankOrder']
					},
					{
						model:Country,
						required: false,
						attributes: ['country', 'countryId']
					},
					{
						model:ChannelSubscription,
						attributes: ['isActive', 'subscriptionId', 'startedAt', 'endAt'],
						where: { isActive: '1' },
						required: false
					},
					{
						model:ChannelType,
						required: true
					},
					{
						model:ChannelDetail,
						required: false,
						attributes: {
							exclude: ['channelDetailId', 'channelId', 'deletedAt', 'createdAt', 'updatedAt']
						},
						include: [{
							model:Media,
							as: 'channelLogo',
							where: { deletedAt: null },
							required: false,
							attributes: ['documentId', 'documentPath']
						},
						{
							model:Media,
							as: 'channelIcon',
							where: { deletedAt: null },
							required: false,
							attributes: ['documentId', 'documentPath']
						}]
					}
				]
			},
			{
				model:Media,
				attributes: ['documentId', 'documentPath'],
				required: false
			}
		]
	},
	channelDetail: {
		attributes: ['contactId', 'firstName', 'lastName', 'phone1', 'mobileNo', 'email', 'responsibilityId', 'linkedIn', 'jobTitle', 'googleplus', 'isDefaultContact', 'empId', 'isActive', 'deletedAt', 'documentId', 'isEvent']
	},
	orderInfo: {
		attributes: ['firstName', 'lastName', 'jobTitle', 'countryId']
	},
	productInfo: {
		attributes: ['contactId', 'firstName', 'lastName', 'jobTitle', 'isDefaultContact', 'deletedAt', 'documentId']
	}

}))

export class Contact extends Model<Contact>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	contactId: number;

	@BelongsToMany(() => ProductGroup, {
		through: () => ContactProductGroup
	})
	productGroups: ProductGroup[];

	@BelongsToMany(() => ProductCategory, {
		through: () => ContactProductCategory
	})
	productCategories: ProductCategory[];

	@ForeignKey(() => Contact)
	@Column(DataType.BIGINT)
	rptContactId: number;

	@BelongsTo(() => Contact, 'rptContactId')
	reportPerson: Contact

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	channelId: number;

	@BelongsTo(() => Channel)
	channel: Channel;

	@AllowNull(true)
	@Column
	firstName: string;

	@AllowNull(true)
	@Column
	lastName: string;

	@Column
	phone1: string;

	@Column
	phone2: string;

	@Column
	mobileNo: string;

	@Column({
		type: DataType.CHAR(255),
		allowNull: false,
		unique: { name: 'email', msg: 'account:singup:emailAleradytaken' }
	})
	email: string;

	@ForeignKey(() => Responsibility)
	@Column(DataType.BIGINT)
	responsibilityId: number;

	@BelongsTo(() => Responsibility)
	responsibility: Media;

	@Column(DataType.BIGINT)
	workNatureId: number;

	@Column(DataType.BIGINT)
	hubspotContactId: number;

	@Column
	jobTitle: string;

	@Column
	linkedIn: string;

	@Column
	googleplus: string;

	@ForeignKey(() => Media)
	@Column(DataType.BIGINT)
	documentId: number;

	@BelongsTo(() => Media, 'documentId')
	document: Media;

	@ForeignKey(() => Country)
	@Column(DataType.BIGINT)
	countryId: number;

	@BelongsTo(() => Country)
	country: Country;

	@Column
	isDefaultContact: string;

	@Column
	isShared: string;

	@Column
	isActive: string;

	@Column(DataType.DATE)
	deletedAt: Date

	@AllowNull(false)
	@Column(DataType.DATE)
	createdAt: Date

	@Column(DataType.DATE)
	updatedAt: Date

	@HasMany(() => ContactSocialLink, 'contactId')
	contactSocialLink: ContactSocialLink[];

	@HasMany(() => ContactLocation, 'contactId')
	contactLocation: ContactLocation[];

	@BelongsToMany(() => RegionCountryJCT, {
		through: () => ContactLocation
	})
	locations: RegionCountryJCT[];

	@BelongsToMany(() => RegionCountryJCT, {
		through: () => ContactLocation
	})
	selling_locations: RegionCountryJCT[];

	@BelongsToMany(() => RegionCountryJCT, {
		through: () => ContactLocation,
	})
	target_locations: RegionCountryJCT[];

	@BelongsToMany(() => RegionCountryJCT, {
		through: () => ContactLocation,
	})
	retail_locations: RegionCountryJCT[];

	@HasMany(() => ChannelContactSuggestion, 'contactid')
	channelContactSuggestion: ChannelContactSuggestion[];

	@HasOne(() => User, 'contactId')
	user: User

	@HasMany(() => ContactProductGroup, 'contactId')
	productGroup: ContactProductGroup[];

	@HasMany(() => ContactProductCategory, 'contactId')
	productCategory: ContactProductCategory[];

	@HasMany(() => ContactInterestedProductGroup, 'contactId')
	IntProductGroup: ContactInterestedProductGroup[];

	@HasMany(() => ContactInterestedProductCategory, 'contactId')
	IntProductCategory: ContactInterestedProductCategory[];

	@BelongsToMany(() => ProductCategory, {
		through: () => ContactInterestedProductCategory
	})
	IntProductCategories: ProductCategory[];

	@BelongsToMany(() => ProductGroup, {
		through: () => ContactInterestedProductGroup
	})
	IntProductGroups: ProductGroup[];

	@Column({
		type: new DataType.VIRTUAL(DataType.INTEGER, ['userId']),
		set: function(value) {
			this.setDataValue('userId', value);
		}
	})
	userId: number;

	@Column
	empId: string;

	@Column({
		type: new DataType.VIRTUAL(DataType.CHAR(255), ['showId']),
		set: function(value) {
			this.setDataValue('showId', value);
		}
	})
	showId: string;

	@BeforeCreate
	static async buildEmpIdLogic(instance: Contact) {
		if (instance.showId && instance.userId) {
			instance.empId = Contact.empIdLogic(instance.showId, instance.userId);
		}
	}

	static empIdLogic(showId, userId) {
		return "#" + showId + userId;
	}

	@BeforeCreate
	@BeforeUpdate
	public static async dateUpdate(contact: Contact, options: any) {
		contact.updatedAt = Sequelize.fn('NOW');
	}

	@Column
	isEvent: string;
	
	@Column
	isConnectionMail: string;

	@Column
	isSuggestMail: string;

	@Column
	isFeatureMail: string;

	@Column
	isProfileMail: string;

	//event participant
	@HasMany(() => EventParticipants, {foreignKey:'contactId'})
	eventParticipants: EventParticipants[];
	

}
