import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, ForeignKey, BelongsTo, Scopes, BeforeCreate, BeforeUpdate  } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Channel } from './../../channel/models/Channel.entity';
import { Contact } from './Contact.entity';
import { RegionCountryJCT } from './../../common/models/RegionCountryJCT.entity';
import { User } from './../../user/models/User.entity';
var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { tableName: 'tblContactLocation' } as IDefineOptions;
@Table(tableOptions)
@Scopes({
	contactLocationInfo: {
		attributes: ['contactLocationId', 'channelId', 'contactId', 'locationType', 'regionCountryJCTId']
	}
})
export class ContactLocation extends Model<ContactLocation>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	contactLocationId: number

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	channelId: number;

	@BelongsTo(() => Channel)
	Channel: Channel[];

	@ForeignKey(() => Contact)
	@Column(DataType.BIGINT)
	contactId: number;

	@BelongsTo(() => Contact)
	Contact: Contact[];

	@Column
	locationType: string

	@ForeignKey(() => RegionCountryJCT)
	@Column(DataType.BIGINT)
	regionCountryJCTId: number

	@BelongsTo(() => RegionCountryJCT)
	RegionCountryJCT: RegionCountryJCT[];

	@Column(DataType.DATE)
	deletedAt: Date

	@AllowNull(false)
	@Column(DataType.DATE)
	createdAt: Date

	@Column(DataType.DATE)
	updatedAt: Date

	@ForeignKey(() => User)
	@Column(DataType.BIGINT)
	updatedBy: number

	@BelongsTo(() => User, 'updatedBy')
	User: User;

	@BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(contactLocation: ContactLocation, options: any) { 
		contactLocation.updatedAt = Sequelize.fn('NOW'); 
	}
}
