import {
	Table,
	Column,
	PrimaryKey,
	Model,
	AutoIncrement,
	DataType,
	BeforeCreate,
	AllowNull,
	BeforeUpdate,
	ForeignKey,
	BelongsTo,
	Scopes
} from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Contact } from './../../user/models/Contact.entity';
import { ProductCategory } from '../../product/models/ProductCategory.entity';
var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblContactProductCategorie' } as IDefineOptions;

@Table(tableOptions)

@Scopes({
	contactProductInfom: {
		attributes: ['contactProductCategorieId', 'contactId', 'productCategorieId']
	}
})

export class ContactProductCategory extends Model<ContactProductCategory>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	contactProductCategorieId: number

	@ForeignKey(() => Contact)
	@Column(DataType.BIGINT)
	contactId: number

	@BelongsTo(() => Contact)
	contact: Contact[];

	@ForeignKey(() => ProductCategory)
	@Column(DataType.BIGINT)
	productCategorieId: number

	@BelongsTo(() => ProductCategory)
	productCategorie: ProductCategory;

	@Column(DataType.DATE)
	deletedAt: Date

	@Column(DataType.DATE)
	createdAt: Date

	@Column(DataType.DATE)
	updatedAt: Date

	@BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(contactProductCategory: ContactProductCategory, options: any) { 
		contactProductCategory.updatedAt = Sequelize.fn('NOW'); 
	}

}
