import {
	Table,
	Column,
	PrimaryKey,
	Model,
	AutoIncrement,
	DataType,
	BeforeCreate,
	AllowNull,
	BeforeUpdate,
	ForeignKey,
	BelongsTo,
	Scopes
} from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Contact } from './../../user/models/Contact.entity';
import { ProductGroup } from '../../product/models/ProductGroup.entity';
var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblContactProductGroup' } as IDefineOptions;

@Table(tableOptions)

@Scopes({
	contactProductInfom: {
		attributes: ['contactProductGroupId', 'contactId', 'productGroupId']
	}
})

export class ContactProductGroup extends Model<ContactProductGroup>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	contactProductGroupId: number

	@ForeignKey(() => Contact)
	@Column(DataType.BIGINT)
	contactId: number

	@BelongsTo(() => Contact)
	contact: Contact[];

	@ForeignKey(() => ProductGroup)
	@Column(DataType.BIGINT)
	productGroupId: number

	@BelongsTo(() => ProductGroup)
	productGroup: ProductGroup;

	@Column(DataType.DATE)
	deletedAt: Date

	@Column(DataType.DATE)
	createdAt: Date

	@Column(DataType.DATE)
	updatedAt: Date

	@BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(contactProductGroup: ContactProductGroup, options: any) { 
		contactProductGroup.updatedAt = Sequelize.fn('NOW'); 
	}

}
