import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, BeforeCreate, AllowNull, BeforeUpdate, ForeignKey, HasMany, BelongsTo, Scopes } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { SocialSite } from './../../channel/models/SocialSite.entity';
import { Contact } from './Contact.entity';

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblContactSocialLink' } as IDefineOptions;
@Table(tableOptions)

@Scopes({
	profileInfo: {
		attributes: ['contactSocialLinkId', 'socialSiteId', 'contactId', 'contactSocialLink']
	}
})

export class ContactSocialLink extends Model<ContactSocialLink>{
	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	contactSocialLinkId: number

	@ForeignKey(() => SocialSite)
	@Column(DataType.BIGINT)
	socialSiteId: number

	@BelongsTo(() => SocialSite)
	socialSite: SocialSite[];

	@ForeignKey(() => Contact)
	@Column(DataType.BIGINT)
	contactId: number

	@BelongsTo(() => Contact)
	contact: Contact[];

	@Column(DataType.CHAR(255))
	contactSocialLink: string

	@Column(DataType.DATE)
	deletedAt: Date

	@Column(DataType.DATE)
	createdAt: Date

	@Column(DataType.DATE)
	updatedAt: Date
}
