import {
	Table,
	Column,
	PrimaryKey,
	Model,
	AutoIncrement,
	DataType,
	BeforeCreate,
	AllowNull,
	BeforeUpdate,
	ForeignKey,
	BelongsTo,
	Scopes
} from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Contact } from './../../user/models/Contact.entity';
import { ProductGroup } from '../../product/models/ProductGroup.entity';
import { Channel } from 'src/modules/channel/models/Channel.entity';
var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblInterestedProductGroup' } as IDefineOptions;

@Table(tableOptions)

@Scopes({
	contactProductInfom: {
		attributes: ['InterestedProductGroupId', 'contactId','channelId','productGroupId']
	}
})

export class InterestedProductGroup extends Model<InterestedProductGroup>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	InterestedProductGroupId: number

	@ForeignKey(() => Contact)
	@Column(DataType.BIGINT)
	contactId: number

	@BelongsTo(() => Contact)
	contact: Contact[];

    @ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	channelId: number

	@BelongsTo(() => Channel)
	channel: Channel[];

    @ForeignKey(() => ProductGroup)
	@Column(DataType.BIGINT)
	productGroupId: number

	@BelongsTo(() => ProductGroup)
	productGroup: ProductGroup;

	@Column(DataType.DATE)
	deletedAt: Date

	@Column(DataType.DATE)
	createdAt: Date

	@Column(DataType.DATE)
	updatedAt: Date

	@BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(InterestedProductGroup: InterestedProductGroup, options: any) { 
		InterestedProductGroup.updatedAt = Sequelize.fn('NOW'); 
	}
}
