import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, ForeignKey, BelongsTo } from 'sequelize-typescript';
import { Channel } from '../../channel/models/Channel.entity';
import { ChannelType } from '../../channel/models/ChannelType.entity';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Media } from '../../media/models/Media.entity';

const tableOptions: IDefineOptions = { timestamp: false, tableName: 'tblreferencecontact' } as IDefineOptions;

@Table(tableOptions)
export class ReferenceContact extends Model<ReferenceContact>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	referencecontactid: number;

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	channelid: number;

	@BelongsTo(() => Channel)
	channel: Channel;

	@Column
	companyname: string

	@ForeignKey(() => ChannelType)
	@Column(DataType.BIGINT)
	companytypeid: number;

	@BelongsTo(() => ChannelType)
	channeltype: ChannelType;

	@Column
	firstname: string;

	@Column
	lastname: string;

	@Column
	phone1: string;

	@Column
	phone2: string;

	@Column
	mobileno: string;

	@Column({
		type: DataType.CHAR(255),
		allowNull: false
	})
	email: string;

	@Column
	jobtitle: string;

	@Column
	linkedin: string;

	@ForeignKey(() => Media)
	@Column(DataType.BIGINT)
	avatarid: number;

	@BelongsTo(() => Media, 'avatarid')
	avatar: Media;

	@AllowNull(false)
	@Column(DataType.DATE)
	createdat: Date

	@Column(DataType.DATE)
	updatedat: Date
}
