'use strict';

import { Table, Column, Model, AllowNull, DataType, Scopes, BeforeCreate, BeforeUpdate } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { ApiProperty } from '@nestjs/swagger';
import { UUIDV4 } from 'sequelize/types';
var Sequelize = require('sequelize');

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblResponsibility' } as IDefineOptions;

@Table(tableOptions)

@Scopes({ 
	channelDetail: {
		attributes: ['responsibilityId', 'responsibility', 'deletedAt']
	}
}) 

export class Responsibility extends Model<Responsibility>{

	@Column({
		type: DataType.BIGINT,
		allowNull: false,
		// defaultValue:UUIDV4,//autoIncrement: true,
		unique: true,
		primaryKey: true,
	})
	@ApiProperty()
	responsibilityId: Number

	@AllowNull(false)
	@Column(DataType.CHAR(255))
	responsibility	: string;

	@Column(DataType.DATE)
	deletedAt: Date

	@AllowNull(false)
	@Column(DataType.DATE)
	createdAt: Date

	@Column(DataType.DATE)
	updatedAt: Date

	@BeforeCreate 
	@BeforeUpdate 
	public static async dateUpdate(responsibility: Responsibility, options: any) { 
		responsibility.updatedAt = Sequelize.fn('NOW'); 
	}
}
