import {
	Table,
	Column,
	PrimaryKey,
	Model,
	AutoIncrement,
	DataType,
	BeforeCreate,
	AllowNull,
	BeforeUpdate,
	ForeignKey,
	Scopes,
	BelongsTo,
	HasOne,
	BelongsToMany
} from 'sequelize-typescript';
import * as crypto from 'crypto';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { Channel } from '../../channel/models/Channel.entity';
import { Contact } from './Contact.entity';
import { Role } from './Role.entity';
import { UserRole } from './UserRole.entity';
import { AccessPermission } from './AccessPermission.entity';
import { UserAccessPermission } from './UserAccessPermission.entity';
const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblUser' } as IDefineOptions;

var Sequelize = require('sequelize');

@Table(tableOptions)

@Scopes(() => ({
	//Used
	userBasicInfom: {
		attributes: ['userId', 'contactId', 'channelId', 'email']
	},
	userChannelContactId: {
		attributes: ['userId', 'contactId', 'channelId']
	},
	userLogInInfom: {
		attributes: ['userId', 'contactId', 'channelId', 'email', 'password', 'isActive', 'tokenModifiedAt', 'lastLogin', 'isAlgorithm', 'suggestion_comm']
	},
	withChannel: {
		include: [Channel]
	},
	withContact: {
		include: [Contact]
	},
	channelDetail: {
		attributes: ['userId', 'contactId', 'channelId', 'email', 'isActive']
	},
	orderInfo: {
		attributes: ['contactId', 'channelId']
	}
}))

export class User extends Model<User>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	userId: number;

	@ForeignKey(() => Contact)
	@Column(DataType.BIGINT)
	contactId: number;

	@BelongsTo(() => Contact)
	contact: Contact;

	@ForeignKey(() => Channel)
	@Column(DataType.BIGINT)
	channelId: number

	@BelongsTo(() => Channel)
	channel: Channel;

	@Column(DataType.SMALLINT)
	tries: number;

	@Column({
		type: DataType.CHAR(255),
		allowNull: false,
		unique: { name: 'email', msg: 'account:singup:emailAleradytaken' }
	})
	email: string;

	@AllowNull(false)
	@Column(DataType.CHAR(255))
	password: string;

	@Column(DataType.CHAR(255))
	password1: string

	@Column(DataType.CHAR(255))
	password2: string;

	@Column(DataType.CHAR(255))
	password3: string;

	@Column(DataType.CHAR(255))
	password4: string;

	@Column(DataType.CHAR(255))
	password5: string;

	@AllowNull(false)
	@Column(DataType.CHAR(255))
	verificationCode: string

	@Column(DataType.DATE)
	verificationSentAt: Date;

	@Column(DataType.CHAR(255))
	resetPasswordToken: string;

	@Column(DataType.BIGINT)
	resetPasswordExpires: number;

	@Column(DataType.BIGINT)
	suggestion_comm: number;

	@Column(DataType.BIGINT)
	invitation_comm: number;

	@Column
	isActive: string;

	@Column(DataType.DATE)
	deletedAt: Date;

	@AllowNull(false)
	@Column(DataType.DATE)
	createdAt: Date;

	@Column(DataType.DATE)
	lastLogin: Date;

	@Column
	isAlgorithm: string;

	@Column(DataType.DATE)
	updatedAt: Date;

	@Column(DataType.DATE)
	tokenModifiedAt: Date;

	@HasOne(() => UserRole, 'userId')
	role: UserRole

	@BelongsToMany(() => Role, {
		through: () => UserRole
	})
	roles: Role[]

	@BelongsToMany(() => AccessPermission, {
		through: {
			model: () => UserAccessPermission,
			scope: {
				deletedat: null
			}
		},
	})
	permissions: AccessPermission[]

	@BeforeCreate
	@BeforeUpdate
	public static async hashPassword(user: User, options: any) {
		if (user.password) {
			var hash = crypto.createHmac('sha512', process.env.USER_PASSWORD_SALT_ROUNT); /** Hashing algorithm sha512 */
			hash.update(user.password);
			user.password = hash.digest('hex');
		}
	}

	@BeforeCreate
	@BeforeUpdate
	public static async dateUpdate(user: User, options: any) {
		user.updatedAt = Sequelize.fn('NOW');
	}
}
