'use strict';
import {Inject, Injectable } from '@nestjs/common';


import { ProcessException } from './../../../middleware/exception/Process.exception';
import { MyAddressBookDto } from './../dto/myAddressBook.dto';
import { MyAddressBook } from '../models/MyAddressBook.entity';
import { Country } from '../../common/models/Country.entity';

@Injectable()
export class MyAddressBookService {
	constructor(
		@Inject('MyAddressBookRepository') private readonly repository: typeof MyAddressBook,
		@Inject('SequelizeInstance') private readonly sequelizeInstance
	) {
		//super();
	}

	async create(data: MyAddressBookDto, user): Promise<any> {
		return await this.repository.create<MyAddressBook>(this._buildObject(new Object(), data, user));
	}



	public async getCount(user, type?) {
		let filter = {
			userid: user.userId
		};

		if (type) {
			if (type == 'billing') {
				filter = Object.assign(filter, { isbillingaddress: '1' });
			} else if (type == 'shipping') {
				filter = Object.assign(filter, { isshippingaddress: '1' });
			}
		}

		return await this.repository.count({
			where: filter
		});
	}

	public async findAll(user, type?): Promise<Array<MyAddressBook>> {
		let filter = {
			userid: user.userId
		};

		if (type) {
			if (type == 'billing') {
				filter = Object.assign(filter, { isbillingaddress: '1' });
			} else if (type == 'shipping') {
				filter = Object.assign(filter, { isshippingaddress: '1' });
			}
		}

		return await this.repository.findAll<MyAddressBook>({
			where: filter,
			include: [
				{
					model: Country,
					as: 'country',
					attributes: ['country']
				}
			],
			//limit: pagination.limit,
			//offset: pagination.offset,
			order: [['addressid', 'DESC']]
		});
	}

	public async findOne(options: Object): Promise<MyAddressBook | null> {
		let result = await this.repository.findOne<MyAddressBook>(options);
		if (!result) {
			throw new ProcessException('validate:field:notFound');
		}

		return result;
	}

	public async findById(id: number, user): Promise<MyAddressBook | null> {
		let result = await this.repository.findOne<MyAddressBook>({
			where: {
				userid: user.userId,
				addressid: id
			}
		});

		if (!result) {
			throw new ProcessException('validate:field:notFound');
		}

		return result;
	}

	public async update(id: number, newValue, user): Promise<MyAddressBook | null> {
		let result = await this.findById(id, user);

		await this._buildObject(result, newValue).save();

		return result;
	}

	public async delete(id: number, user): Promise<void> {
		let result = await this.findById(id, user);

		return await result.destroy();
	}

	private _buildObject(entity, dto, user?) {

		entity.name = dto.name;
		entity.address = dto.address;
		entity.city = dto.city;
		entity.countryid = dto.countryId;
		entity.postalcode = dto.postalCode;
		entity.contactno1 = dto.primary_contactno;
		entity.contactno2 = dto.alternative_contactno;
		entity.isshippingaddress = (dto.isshippingaddress) ? '1' : '0';
		entity.isbillingaddress = (dto.isbillingaddress) ? '1' : '0';

		if (user) {
			entity.userid = user.userId;
			entity.channelid = user.channelId;
		}

		return entity;
	}
}
