import { Injectable,Inject, InternalServerErrorException } from '@nestjs/common';
import { User } from '../models/User.entity';
import { ContextService } from './../../../context/context.service';
import { AdminUsers } from '../../admin/models/AdminUsers.entity';
import { AdminUserAccess } from '../../admin/models/AdminUserAccess.entity';
import { Channel } from '../../channel/models/Channel.entity';
import { AdminMenu } from 'src/modules/admin/models/AdminMenu.entity';
import { AdminUserCountries } from 'src/modules/admin/models/AdminUserCountries.entity';
import { RegionCountryJCT } from 'src/modules/common/models/RegionCountryJCT.entity';
import { Country } from 'src/modules/common/models/Country.entity';
import { Contact } from '../models/Contact.entity';
import { Media } from '../../media/models/Media.entity';
import { appconstant } from 'src/config/app.constant';
import { IUserService } from './interfaces/IUserService';
import { UserInBound } from '../transformation/userInBound';
import { UserRole } from '../models/UserRole.entity';
import { ContactProductGroup } from '../models/ContactProductGroup.entity';
import { ContactInterestedProductGroup } from '../models/ContactInterestedProductGroup.entity';
import { ContactInterestedProductCategory } from '../models/ContactInterestedProductCategory.entity';
import { ContactSocialLink } from '../models/ContactSocialLink.entity';
import { ContactProductCategory } from '../models/ContactProductCategory.entity';
import { UserAccessPermission } from '../models/UserAccessPermission.entity';
import { SocialSite } from './../../channel/models/SocialSite.entity';
import { ContactLocation } from '../models/ContactLocation.entity';
import { ChannelContactSuggestion } from 'src/modules/channel/models/ChannelContactSuggestion.entity';
import { BaseService } from 'src/core/service/base.service';
import { Role } from '../models/Role.entity';
import { ContactJCT } from 'src/modules/channel/models/ContactJCT.entity';
import {Op, QueryTypes } from "sequelize";
import { InterestedProductGroup } from '../models/InterestedProductGroup.entity';
import { InterestedProductCategory } from '../models/InterestedProductCategory.entity';
import { SubscriptionPlan } from 'src/modules/channel/models/SubscriptionPlan.entity';
import { ChannelSubscription } from 'src/modules/channel/models/ChannelSubscription.entity';
import { ChannelDetail } from 'src/modules/channel/models/ChannelDetail.entity';
import { Subscription } from 'src/modules/channel/models/Subscription.entity';
import { ProcessException } from 'src/middleware/exception/Process.exception';
import { ProfileList } from 'src/modules/channel/models/ProfileList.entity';
import { Sequelize } from 'sequelize-typescript';

@Injectable()
export class UserService {
    private userInBound = new UserInBound();	
	private specialChars = ['@', '#', '%', '$', '*'];
    constructor(
    @Inject('SequelizeInstance') private readonly sequelizeInstance,
    @Inject('UserRoleRepository') private readonly userRoleRepository: typeof UserRole,
    @Inject('ContactLocationRepository') private readonly contactLocationRepository: typeof ContactLocation,
    @Inject('UserRepository') private readonly userRepository: typeof User,
    @Inject('ContactRepository') private readonly contactRepository: typeof Contact,
    @Inject('ChannelRepository') private readonly channelRepository: typeof Channel,
    @Inject('AdminUserRepository') private readonly adminUserRepository: typeof AdminUsers,
    @Inject('AdminUserAccessRepository') private readonly adminUserAccessRepository: typeof AdminUserAccess,
    @Inject('MediaRepository') private readonly mediaRepository: typeof Media,
    @Inject('ContactProductGroupRepository') private readonly contactProductGroupRepository: typeof ContactProductGroup,
    @Inject('ContactInterestedProductGroupRepository') private readonly contactInterestedProductGroupRepository: typeof ContactInterestedProductGroup,
	@Inject('ContactInterestedProductCategoryRepository') private readonly contactInterestedProductCategoryRepository: typeof ContactInterestedProductCategory,
	@Inject('InterestedProductGroupRepository') private readonly interestedProductGroupRepository: typeof InterestedProductGroup,
    @Inject('InterestedProductCategoryRepository') private readonly interestedProductCategoryRepository: typeof InterestedProductCategory,
    @Inject('ContactSocialLinkRepository') private readonly contactSocialLinkRepository: typeof ContactSocialLink,
    @Inject('ContactProductCategoryRepository') private readonly contactProductCategoryRepository: typeof ContactProductCategory,
    @Inject('UserAccessPermissionRepository') private readonly userAccessPermissionRepository: typeof UserAccessPermission,
    @Inject('ContactSuggestionRepository') private readonly contactSuggestionRepository: typeof ChannelContactSuggestion,
	@Inject('CountryRepository') private readonly countryRepository: typeof Country,
    private context: ContextService,
    ) { }
    
    async findUser(email: string): Promise<any> {
        // return await this.userRepository.scope('userLogInInfom').findOne<User>({        
        //     where: { email } });
        
        console.log(email);
        const user = await this.userRepository.scope('userLogInInfom').findOne({
                include: [
					{
						model:UserRole,		
						as:'role',
						attributes: ['roleId']
					},
					{
                    model: this.channelRepository.scope('basicInfom'),
                    required: true,
                    where: { 
                        isActive: '1',
						signUpStatusId: 15,
                        deletedAt : null 
                },
				include:[
					
					{
						model: SubscriptionPlan,
						as: 'plan',
						attributes: ['subscriptionPlanId', 'subscriptionPlanType', 'rankOrder']
					},
					{
						model: ChannelSubscription,
						attributes: ['isActive', 'subscriptionId', 'startedAt', 'endAt', 'keyFeature'],
						where: { isActive: '1' },
						required: false,
						include:[
					
							{
								model: Subscription,
								as: 'subscription',
								attributes: ['subscriptionPeriodId']
							}
						]

					}
				]
				
            }, {
                model: this.contactRepository.scope('contactInfo'),
                required: true,
                include: [{
                    model: this.mediaRepository.scope('profileInfo')
                }]
            }],
			// where:Sequelize.where(Sequelize.fn('lower', Sequelize.col('email')), Sequelize.fn('lower', email))
			where: {
				email: 
				{
					[Op.iLike]: '%' +email+ '%'
				},
				deletedAt : null 
			}
            });      

        if(user){
			this.userRepository.update({ lastLogin: Date.now() }, {
                where: {
                    userId: user.userId
                }
            })
        }
        // this.context.set('user', user);
        return user;

    }
	
	async findUserById(userId: any, channelId: any): Promise<any> {
        // return await this.userRepository.scope('userLogInInfom').findOne<User>({        
        //     where: { email } });
        
        
        const user = await this.userRepository.scope('userLogInInfom').findOne({
                include: [
					{
						model:UserRole,		
						as:'role',
						attributes: ['roleId']
					},
					{
                    model: this.channelRepository.scope('basicInfom'),
                    required: true,
                    where: { 
                        isActive: '1',
                        deletedAt : null 
                },
				include:[
					
					{
						model: SubscriptionPlan,
						as: 'plan',
						attributes: ['subscriptionPlanId', 'subscriptionPlanType', 'rankOrder']
					},
					{
						model: ChannelSubscription,
						attributes: ['isActive', 'subscriptionId', 'startedAt', 'endAt', 'keyFeature'],
						where: { isActive: '1' },
						required: false,
						include:[
					
							{
								model: Subscription,
								as: 'subscription',
								attributes: ['subscriptionPeriodId']
							}
						]

					}
				]
				
            }, {
                model: this.contactRepository.scope('contactInfo'),
                required: true,
                include: [{
                    model: this.mediaRepository.scope('profileInfo')
                }]
            }],
            where: { userId: userId, channelId: channelId }
            
            });      
               
        // if(user){
        //      this.userRepository.update({lastLogin: Date.now(),isAlgorithm:'1'}, {
        //         where: {
        //             userId: user.userId
        //         }
        //     })
        // }
        // this.context.set('user', user);

		let userData = { 
			userId: user.userId,
			contactId: user.contactId,
			channelId: user.channelId,
			email: user.email,
			isActive: user.isActive,
			planId: user.channel?.planId,
			showId:user.channel?.showId,
			signUpStatusId: user.channel?.signUpStatusId,
			channelTypeId: user.channel?.channelTypeId,
			profileTypeId: user.channel?.profileTypeId,
			businessType:+user.channel?.isOEM,
			lastLogin: user.lastLogin,
			isAlgorithm:user.isAlgorithm,
			firstName: user.contact?.firstName,
			lastName: user.contact?.lastName,
			document: user.contact?.document?user.contact.document:{},
			channel: user.channel,
			roleId: user.role?.roleId,
			suggestion_comm: user.suggestion_comm,
		};

        return userData;

    }
	
	async boUser(id: number): Promise<any> {
        // return await this.userRepository.scope('userLogInInfom').findOne<User>({        
        //     where: { email } });
        
        
        const user = await this.userRepository.scope('userLogInInfom').findOne({
                include: [
					{
						model:UserRole,		
						as:'role',
						attributes: ['roleId']
					},
					{
                    model: this.channelRepository.scope('basicInfom'),
                    required: true,
                    where: { 
                        isActive: '1',
                        deletedAt : null 
                },
				include:[
					
					{
						model: SubscriptionPlan,
						as: 'plan',
						attributes: ['subscriptionPlanId', 'subscriptionPlanType', 'rankOrder']
					},
					{
						model: ChannelSubscription,
						attributes: ['isActive', 'subscriptionId', 'startedAt', 'endAt', 'keyFeature'],
						where: { isActive: '1' },
						required: false,
						include:[
					
							{
								model: Subscription,
								as: 'subscription',
								attributes: ['subscriptionPeriodId']
							}
						]

					}
				]
            }, {
                model: this.contactRepository.scope('contactInfo'),
                required: true,
                include: [{
                    model: this.mediaRepository.scope('profileInfo')
                }]
            }],
            where: {contactId: id }
            
            });      
               
        if(user){
			this.userRepository.update({ lastLogin: Date.now() }, {
                where: {
                    userId: user.userId
                }
            })
        }
        // this.context.set('user', user);
        //return user;
	
		// let users=user['dataValues'];
		let userData = {};
        userData = { 
            userId: user.userId,
            contactId: user.contactId,
            channelId: user.channelId,
            email: user.email,
            isActive: user.isActive,
            planId: user.channel?.planId,
            showId:user.channel?.showId,
            signUpStatusId: user.channel?.signUpStatusId,
            channelTypeId: user.channel?.channelTypeId,
            profileTypeId: user.channel?.profileTypeId,
            businessType:+user.channel?.isOEM,
            lastLogin: user.lastLogin,
            isAlgorithm:user.isAlgorithm,
            firstName: user.contact?.firstName,
            lastName: user.contact?.lastName,
            document: user.contact?.document?user.contact.document:{},
            channel: user.channel,
            roleId: user.role?.roleId,
			contact: user.contact,
			suggestion_comm: user.suggestion_comm,
        };

        // if(user.hasOwnProperty('adminUserId'))
        //     userData = Object.assign(userData, {adminUserId: user.adminUserId})
        
        // if(user.hasOwnProperty('userId'))
        //     userData = Object.assign(userData, {userId: user.userId})

        if (!userData) {
        //  throw new UnauthorizedException('Invalid user credentials');
        }
        return userData;
		
        // return users;

    }   

    async findAdmin(email: string): Promise<any> {
        
        const user = await this.adminUserRepository.scope().findOne({             
        //    include: [
		// 		{
        //         model: AdminUserAccess,
        //     attributes: ['allowView', 'allowModify', 'allowDelete'],                                             
        //     required: false,                           

        //     include: [{
        //         model: AdminMenu,
        //     attributes: ['moduleId','menuName','menuId','asmId','assmId'],                                             
        //     required: false, }],            
            
        // },
        // {
        //     model: AdminUserCountries,                
        //     required: false,

        //     include: [{
        //         model: Country,                
        //     attributes: ['country'],                                             
        //     required: false, }],       
            
        //     attributes: ['countryId'],          

        // }],
		

        where: { emailId:email }
        
        });
     
        
                
        // this.context.set('user', user);
        
        return user;

    }   


	async findOneById(payload: any): Promise<any> {
		if (payload.userId)
			return await this.userRepository.findOne<User>({ where: { userId: payload.userId } });
		else
			return await this.adminUserRepository.findOne<AdminUsers>({ where: { adminUserId: payload.adminUserId } });
    }



    async addUser(userDto, channelId, showId, userId, channelTypeId) {
		//generate new password
		let password: string = await this.getPassword();

		//userDto['documentId'] = null;
		// if (userDto.contact.avatar) {
		// 	let document = await Media.create({
		// 		documentTypeId: appconstant.DOCUMENTTYPE_IMAGE,
		// 		documentPath: userDto.contact.avatar,
		// 		dirName: appconstant.FOLDER_PATH_PROFILE
		// 	});

			userDto['documentId'] = userDto.contact.documentId;
//		}
		// Add User and new contact
		let uid;
		let userObject = await this.userInBound.setAddUserObject(userDto, channelId, password, showId, userId);
		let result: any = await this.sequelizeInstance.transaction(async transaction => {
			// create contact
			let user = await this.userRepository.create(userObject, {
				include: [{
					model: this.contactRepository,
					/*include: [
						{
							model: this.mediaRepository,
							required: false
						}
					]*/
				}, {
					model: this.userRoleRepository
				}],
				//returning: true,
				transaction,
			});

			uid=user;
			await this.relations(transaction, userDto, user, 'create', true);

			let ChannelcontactsuggestionObject: ChannelContactSuggestion[] = await this.userInBound.setChannelContactSuggestion(appconstant.VISIBILITY[+channelTypeId], user.contactId, channelId, user.userId, []);
			if (ChannelcontactsuggestionObject.length) {
				await this.contactSuggestionRepository.bulkCreate(ChannelcontactsuggestionObject, { returning: true, transaction });
			}

			// update empId
			user.contact.empId = showId + user.userId;
			await user.contact.save();
			
			return user;
		}).then(function(result) {
			Object.assign(result,{password:password});
			return result;
		}).catch(function(e) {
			console.log(e);
		//	throw new ProcessException(e.message);
		});

		// Send email to account owner and the new user
		//this.sendCredentialEmail(result, userDto, password);

		return uid;
    }

    async updateUser(userDto, user, interestedFamily: boolean = false,real_user) {
		var activeIds = [];		
		var realUser = real_user;//this.context.get('real-user');

		await this.sequelizeInstance.transaction(async transaction => {
			//userRole
			if (userDto.contact && userDto.contact.role && user.role && user.role.roleId != Role.ROLE_ACCOUNT_USER) {
				// validate roleId
				var roles = [
					Role.ROLE_ADMIN_USER,
					Role.ROLE_PRIMARY_USER
				];

				if (realUser && +realUser.channelTypeId === +appconstant.CHANNEL_TYPE_HUB) {
					roles.push(Role.ROLE_ACCOUNT_USER);

					if(userDto.contact.role == Role.ROLE_ACCOUNT_USER) {
						userDto.contact.workNatureId = null;					
					}
				}

				if (!roles.includes(userDto.contact.role)) {
					userDto.contact.role = Role.ROLE_PRIMARY_USER;
				}

				user.role.roleId = userDto.contact.role;
				await user.role.save({
					transaction
				});

			}

							//updating preferences
							await this.userRepository.update({
								invitation_comm: +userDto.contact.invitation_comm,
								suggestion_comm: +userDto.contact.suggestion_comm
							}, {
								where: {
									contactId: user.contactId
								},
								transaction
							}).then(x => console.log('saved', user.contactId));

			//update avatar
			let contact = await this.contactRepository.findOne({
				where: {
					contactId: user.contactId
				},
				include: [
					{
						model: this.mediaRepository,
						required: false
					}
				]
			});

			// update contact
			if (userDto.contact && Object.keys(userDto.contact).length) {
				// phone1 vs phoneNo
				if (userDto.contact.phoneNo) {
					userDto.contact.phone1 = userDto.contact.phoneNo;
				}

				if (+userDto.contact.rptContactId === 0) {
					userDto.contact.rptContactId = null
				}

				if (userDto.contact.documentId) {
					userDto.contact.documentId = userDto.contact.documentId;
				}
				userDto.contact.isActive = (userDto.contact.isActive) ? userDto.contact.isActive : '1';

				var userUpdate = false
				if(userDto.contact.email != contact.email) {
					userUpdate = true;
				} else {
					userDto.contact.email = contact.email;
				}

				await this.contactRepository.update(userDto.contact, {
					where: {
						contactId: user.contactId
					},
					transaction: transaction
				});
			}

			await this.relations(transaction, userDto, user, 'update', interestedFamily);


			if (userDto.channelContactSuggestion) {
				activeIds = [];
				let nin = [];
				userDto.channelContactSuggestion.forEach((r) => {
					nin.push(r.id)
				});

				// delete missing ids
				await this.contactSuggestionRepository.update({
					deletedat: new Date()
				}, {
						where: {
							contactid: user.contactId,
							channelid: user.channelId,
							sharetypeid: {
								[Op.notIn]: nin
							}
						}, transaction: transaction
					});

				// collect activeIds
				await this.contactSuggestionRepository.findAll({
					where: {
						contactid: user.contactId,
						channelid: user.channelId,
						deletedat: null
					}
				}).then((r) => {
					r.forEach((d) => {
						activeIds.push(+d.sharetypeid);
					})
				});


				let ChannelcontactsuggestionObject: ChannelContactSuggestion[] = await this.userInBound.setChannelContactSuggestion(userDto.channelContactSuggestion, user.contactId, user.channelId, user.userId, activeIds);
				if (ChannelcontactsuggestionObject.length) {
					await this.contactSuggestionRepository.bulkCreate(ChannelcontactsuggestionObject, { returning: true, transaction });
				}
			}


			if(userUpdate) {				
				await this.userRepository.update({
					email: userDto.contact.email,
				}, {
					where: {
						contactId: +user.contactId
					},
					transaction
				});
			}


			if(+realUser.channel.signUpStatusId == appconstant.CHANNEL_APPROVAL || userDto.ratingId){

				let defaultdata = {
					channelId : +realUser.channel.channelId,
					ratingId : 2,
					isCompleted : true,
					createdAt : new Date()
				}

				await ProfileList.findOrCreate({where: {channelId: +realUser.channel.channelId, ratingId: 2}, defaults: defaultdata}).then((result) => {
					if(!result[1]){
						result[0].update(defaultdata);
					}
				});
			}

			return user;
		}).then(async (result) => {
			if (userDto.contact.isActive && userDto.contact.isActive == '0') {
				let contactJCT: any = await ContactJCT.findAll({
					where: {
						partnerContactId: user.contactId
					}
				});

				if (contactJCT.length > 0) {
					for (var i = 0; i < contactJCT.length; i++) {
						let notificationObject = {
							senderId: +user.contactId,
							receiverId: contactJCT[i].contactId
						}
						//this.notificationService.sendNotificationRouter(notificationObject, notification.CHANGE_OF_CONTACT);
					}
				}
			}
			return result;
		}).catch(function(e) {
			console.log("updateerror:",e);
			//throw new ProcessException(e.message);
		});

		return true;
	}

    async getPassword() {
		var generator = require('generate-password');

		var password = generator.generate({
			length: 9,
			numbers: true,
			uppercase: true,
			strict: true
		});

		return password + this.specialChars[Math.floor(Math.random() * this.specialChars.length)];
    }
    
    async relations(transaction, userDto, user, action, interestedFamily: boolean = false) {
		var activeIds = [];

		// contactSocialLinks
		if (userDto.contact) {
			console.log("link:",userDto);
			let contactSocialLinkObject: ContactSocialLink[] = await this.userInBound.setSocialLinks(userDto, user.contactId);

			if (contactSocialLinkObject.length) {
				if (action == 'update') {
					let fin = [];

					if (userDto.contact.linkedInUrl  !== undefined) {
						fin.push(SocialSite.SOCIALSITE_LINKEDIN);
					}

					if (userDto.contact.googlePlusUrl !== undefined) {
						fin.push(SocialSite.SOCIALSITE_GOOGLEPLUS);
					}

					await this.contactSocialLinkRepository.destroy({
						where: {
							contactId: user.contactId,
							socialSiteId: {
								[Op.in]: fin
							}
						}, transaction: transaction
					});
				}
				await this.contactSocialLinkRepository.bulkCreate(contactSocialLinkObject, { returning: true, transaction });
			}
		}

		// tradeInformation
		// if (userDto.productFamilies) {
		// 	activeIds = []
		// 	if (action == 'update') {
		// 		let nin = [];
		// 		userDto.productFamilies.forEach((r) => {
		// 			nin.push(r.id)
		// 		});

		// 		// delete missing ids
		// 		await this.contactProductGroupRepository.update({
		// 			deletedAt: new Date()
		// 		}, {
		// 				where: {
		// 					contactId: user.contactId,
		// 					productGroupId: {
		// 						$notIn: nin
		// 					}
		// 				}, transaction: transaction
		// 			});

		// 		// collect activeIds
		// 		await this.contactProductGroupRepository.findAll({
		// 			where: {
		// 				contactId: user.contactId,
		// 				deletedAt: null
		// 			}
		// 		}).then((r) => {
		// 			r.forEach((d) => {
		// 				activeIds.push(+d.productGroupId);
		// 			})
		// 		});
		// 	}

		// 	let ContactProductGroupObject: ContactProductGroup[] = await this.userInBound.setProductGroup(userDto.productFamilies, user.contactId, activeIds);
		// 	if (ContactProductGroupObject.length) {
		// 		await this.contactProductGroupRepository.bulkCreate(ContactProductGroupObject, { returning: true, transaction });
		// 	}
		// }

		// if (userDto.productCategories) {
		// 	activeIds = [];
		// 	if (action == 'update') {
		// 		let nin = [];
		// 		userDto.productCategories.forEach((r) => {
		// 			nin.push(r.productCategoriesId)
		// 		});

		// 		// delete missing ids
		// 		await this.contactProductCategoryRepository.update({
		// 			deletedAt: new Date()
		// 		}, {
		// 				where: {
		// 					contactId: user.contactId,
		// 					productCategorieId: {
		// 						$notIn: nin
		// 					}
		// 				}, transaction: transaction
		// 			});

		// 		// collect activeIds
		// 		await this.contactProductCategoryRepository.findAll({
		// 			where: {
		// 				contactId: user.contactId,
		// 				deletedAt: null
		// 			}
		// 		}).then((r) => {
		// 			r.forEach((d) => {
		// 				activeIds.push(+d.productCategorieId);
		// 			})
		// 		});
		// 	}

		// 	let ContactProductCategorieObject: ContactProductCategory[] = await this.userInBound.setProductCategories(userDto.productCategories, user.contactId, activeIds);
		// 	if (ContactProductCategorieObject.length) {
		// 		await this.contactProductCategoryRepository.bulkCreate(ContactProductCategorieObject, { returning: true, transaction });
		// 	}
		// }

		//interested Info		
			if (userDto.productFamilies) {
				activeIds = []
				if (action == 'update') {
					let nin = [];
					userDto.productFamilies.forEach((r) => {
						nin.push(r.id)
					});
	
					// delete missing ids
					await this.interestedProductGroupRepository.update({
						deletedAt: new Date()
					}, {
							where: {
								contactId: user.contactId,
								productGroupId: {
									[Op.notIn]: nin
								}
							}, transaction: transaction
						});
	
					// collect activeIds
					await this.interestedProductGroupRepository.findAll({
						where: {
							contactId: user.contactId,
							deletedAt: null
						}
					}).then((r) => {
						r.forEach((d) => {
							activeIds.push(+d.productGroupId);
						})
					});
				}
	
				let InterestedProductGroupObject: InterestedProductGroup[] = await this.userInBound.setInterestedProductGroup(userDto.productFamilies, user.channelId,user.contactId, activeIds);
				if (InterestedProductGroupObject.length) {
					await this.interestedProductGroupRepository.bulkCreate(InterestedProductGroupObject, { returning: true, transaction });
				}
			} else {
				// delete missing ids
				await this.interestedProductGroupRepository.update({
					deletedAt: new Date()
				}, {
						where: {
							contactId: user.contactId
						}, transaction: transaction
					});
			}

			if (userDto.productCategories) {
				activeIds = [];
				if (action == 'update') {
					let nin = [];
					userDto.productCategories.forEach((r) => {
						nin.push(r.productCategoriesId)
					});
	
					// delete missing ids
					await this.interestedProductCategoryRepository.update({
						deletedAt: new Date()
					}, {
							where: {
								contactId: user.contactId,
								productCategorieId: {
									[Op.notIn]: nin
								}
							}, transaction: transaction
						});
	
					// collect activeIds
					await this.interestedProductCategoryRepository.findAll({
						where: {
							contactId: user.contactId,
							deletedAt: null
						}
					}).then((r) => {
						r.forEach((d) => {
							activeIds.push(+d.productCategorieId);
						})
					});
				}
	
				let InterestedProductCategorieObject: InterestedProductCategory[] = await this.userInBound.setInterestedProductCategories(userDto.productCategories, user.channelId,user.contactId, activeIds);
				if (InterestedProductCategorieObject.length) {
					await this.interestedProductCategoryRepository.bulkCreate(InterestedProductCategorieObject, { returning: true, transaction });
				}
			} else {
				// delete missing ids
				await this.interestedProductCategoryRepository.update({
					deletedAt: new Date()
				}, {
						where: {
							contactId: user.contactId
						}, transaction: transaction
					});
			}
		

		// access permission
		if (userDto.accessPermissions) {
			activeIds = [];
			if (action == 'update') {
				let nin = [];
				userDto.accessPermissions.forEach((r) => {
					nin.push(r.id)
				});

				// delete missing ids
				await this.userAccessPermissionRepository.update({
					deletedat: new Date()
				}, {
						where: {
							userid: user.userId,
							accesspermissionid: {
								[Op.notIn]: nin
							}
						}, transaction: transaction
					});

				// collect activeIds
				await this.userAccessPermissionRepository.findAll({
					where: {
						userid: user.userId,
						deletedat: null
					}
				}).then((r) => {
					r.forEach((d) => {
						activeIds.push(+d.accesspermissionid);
					})
				});
			}

			let UserAccessPermissionObject: UserAccessPermission[] = await this.userInBound.setAccessPermissions(userDto.accessPermissions, user.userId, activeIds);
			if (UserAccessPermissionObject.length) {
				await this.userAccessPermissionRepository.bulkCreate(UserAccessPermissionObject, { returning: true, transaction });
			}

			// update as token updated
			await User.update({
				tokenModifiedAt: this.sequelizeInstance.fn('NOW')
			}, {
					where: {
						userId: user.userId
					},
					returning: true, transaction
				});
		}

		// tradeLocation
		if (userDto.locations) {
			if (action == 'update') {
				await this.contactLocationRepository.update({
					deletedAt: new Date()
				}, {
						where: {
							contactId: user.contactId
						}, transaction: transaction
					});
			}

			let ContactLocationObject: ContactLocation[] = await this.userInBound.setContactLocations(userDto.locations, user.channelId, user.contactId);
			if (ContactLocationObject.length) {
				await this.contactLocationRepository.bulkCreate(ContactLocationObject, { returning: true, transaction });
			}
		}
    }
    
    async getPreference(typeid){


        let ids=(typeid==2)?[50,21,25,1]:[21,25,42,1];
        let left=[];let right=[];let children;
        
        let query = 'select "accesspermissionid" as id,"name","parentaccesspermissionid" as parent from "tblaccesspermission" where ("parentaccesspermissionid" in(:Ids) or "accesspermissionid" in(:Ids)) and "deletedat" is null and "isactive"=\'1\' ';
        let result=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
            replacements: {
              Ids: ids                            
            }
            }).then(results => {        
                return results;
            });

            let child=result;

            if(typeid==2)
            {
                result.map(x => {
                    if(x.parent==null && (x.id==50 || x.id==21))
                    {                      
                        children=[];
                        child.map(c =>  {
                            if(c.parent==x.id)
                            {
                                let obj={
                                    title:c.name,
									key:c.id,
									selectable:true     
                                }
                                children.push(obj);
                            }
                        })
                        left.push({title:x.name,key:x.id,selectable:true,children:children});                        
                    }

                    if(x.parent==null && (x.id==25 || x.id==1))
                    {                      
                        children=[];
                        child.map(c =>  {
                            if(c.parent==x.id)
                            {
                                let obj={
                                    title:c.name,
									key:c.id,
									selectable:true          
                                }
                                children.push(obj);
                            }
                        })
                        right.push({title:x.name,key:x.id,selectable:true,children:children});                        
                    }
                });
            }
            else            
            {
                result.map(x => {
                    if(x.parent==null && (x.id==21 || x.id==25))
                    {                      
                        children=[];
                        child.map(c =>  {
                            if(c.parent==x.id)
                            {
                                let obj={
                                    title:c.name,
									key:c.id,
									selectable:true          
                                }
                                children.push(obj);
                            }
                        })
                        left.push({title:x.name,key:x.id,selectable:true,children:children});                        
                    }

                    if(x.parent==null && (x.id==42 || x.id==1))
                    {                      
                        children=[];
                        child.map(c =>  {
                            if(c.parent==x.id)
                            {
                                let obj={
                                    title:c.name,
									key:c.id,
									selectable:true          
                                }
                                children.push(obj);
                            }
                        })
                        right.push({title:x.name,key:x.id,selectable:true,children:children});                        
                    }
                });
            }

        let data={
            left:left,
            right:right
        }
        return data;     
	}

	async getMailPreference() {

		let mail = [
			{ title: 'Channel Approve', key: 2, selectable: true }, { title: 'Channel Dismiss', key: 3, selectable: true }, { title: 'Connection Approve', key: 4, selectable: true },
			{ title: 'Connection Dismiss', key: 5, selectable: true }
		]
		let left = []; let right = [];

		left.push({ title: 'Email', key: 1, selectable: true, children: mail });

		//right.push({ title: 'Message', key: 2, selectable: true, children: children });

		let data = {
			left: left,
			right: right
		}
		return data;
	}

	async getUserPreference(contactId, channelId){

		let query = 'select cast("accesspermissionid" as varchar) from "tbluseraccesspermission" as up where "deletedat" is null and "userid" in(select "userId" from "tblUser" where "contactId"=:contactId and "channelId"=:channelId) ';
        let result=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
            replacements: {
        		contactId: contactId,                            
				channelId: channelId
            }
            }).then(results => {      				
                return results.map(x => x.accesspermissionid);
			});
			
		return result;
	}

	async getUserPrevilege(contactId){

		let query = 'select cast("accesspermissionid" as varchar),(select "name" from "tblaccesspermission" where "accesspermissionid"=up."accesspermissionid") from "tbluseraccesspermission" as up where "deletedat" is null and "userid" in(select "userId" from "tblUser" where "contactId"=:contactId) ';
        let result=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
            replacements: {
        		contactId: contactId                            
            }
            }).then(results => {      				
                return results.map(x => x.accesspermissionid);
			});
			
		return result;
	}

	async getUser(id, channelId) {
		
		let user = await this.userRepository.findOne<User>({
			where: {
				contactId: id,
				channelId: channelId
			},
			include: [
				{
					model: UserRole,
					required: true
				}
			]
		});

		if (!user) {
			console.log("error:",user,id,channelId);
			//throw new ProcessException('validate:field:notFound');
		}

		return user;
	}

	async getDefaultContact(channelId)
	{
		let user = await this.contactRepository.findOne<Contact>({
			attributes: ['firstName', 'lastName', 'jobTitle'],
			where: {				
				channelId: channelId,
				isDefaultContact:'1'
			},
			include: [
				{
					model: Country,
					attributes:['country']
				},
				{
					model: Media,
					attributes:['documentPath']
				}


			]
			
		});

		let no_of_users = await this.contactRepository.count({
			where: {
				channelId: channelId,
			}
		});
		let contact = {
			user: user,
			userCount: no_of_users
		}
		return contact;
	}

	async getAllContacts(channelId)
	{
		let user = await this.contactRepository.findAll<Contact>({
			attributes:['contactId','firstName','lastName','jobTitle'],
			where: {				
				channelId: channelId,
				deletedAt:null,
				isActive:'1'				
			},
			include: [				
				{
					model: Media,
					attributes:['documentPath']
				},	
				{
					model: Country,
					attributes:['country']
				},
				{
					model: Channel,
					attributes: ['companyName'],
					include: [
						{
							model: ChannelDetail,
							attributes: ['channelLogoId'],
							include: [
								{
									model: Media,
									as: 'channelLogo',
									attributes: ['documentPath']
								}
							]
						}
					]
				},
				
			]
		});		
		return user;
	}

	async setDefaultContact(channelId,contactId)
	{
		await this.contactRepository.update({ isDefaultContact: '0' }, {
			where: {
				channelId: channelId,				
			},
		});

		await this.contactRepository.update({ isDefaultContact: '1' }, {
			where: {
				contactId: contactId,				
			},
		});
	}

	public async Countries() {              
        const countries = await Country.findAll({             
            attributes: [['countryId','id'],['country','name']]     
        });

        return countries;
    }
	
	async getContact(Id: number): Promise<Contact> {
		try {
			let Contact = await this.contactRepository.scope('contactInfo').findOne(
				{
					include: [
						{
							model: this.countryRepository,
							attributes: ['isoCode','country', 'countryId'],
							required: false
						},
						{
							model: this.mediaRepository,
							attributes: ['documentUrl','documentPath', 'documentId'],
							required: false
						},
						{
							model: User,
							as: 'user',
							required: true,
							attributes: ['userId']
						}
					], where: { contactId: Id }
				});
			return Contact;
		} catch (e) {
			console.log("Error+", e);
			throw new InternalServerErrorException();
		}
	}
	async updateContact(contactId, contactData){
			//update avatar
			let contact = await this.contactRepository.findOne({
				where: {
					contactId: contactId
				}
			});
			if(!contact)
			throw new ProcessException('User not found');
			
			// update contact
			if (contactData) {
				let contact = {};
				if(contactData.firstName)
				contact['firstName']=contactData.firstName;
				if(contactData.lastName)
				contact['lastName']= contactData.lastName;
				if(contactData.email)
				contact['email']=contactData.email;
				if(contactData.country)
				contact['countryId']=contactData.country.id;
				if(contactData.phone)
				contact['phone1']=contactData.phone;
				if(contactData.mobileNo)
				contact['mobileNo']=contactData.mobileNo;
				if(contactData.jobTitle)
				contact['jobTitle']=contactData.jobTitle;

				if (+contactData.rptContactId === 0) {
					contact['rptContactId'] = null;
				}else{
					contact['rptContactId']=+contactData.rptContactId;
				}
				await this.contactRepository.update(contact, {
					where: {
						contactId:contactId
					}
				});
			}
	}

	async isAlgorithm(userId) {
		//update avatar
		this.userRepository.update({ isAlgorithm: '1' }, {
			where: {
				userId: userId
			}
		})
	}

	
    async checkEmail(email: string, userId) {
        let result = await this.userRepository.findOne<User>({
			where: {
				email: 
				{
					[Op.iLike]: '%' +email+ '%'
				},
				userId:{
					[Op.not]:+userId
				},
				deletedAt : null 
			}
		});
		console.log(result);
		if (!result) {
			return true;
		}

		return false;
	}
}
