'use strict';

//import { BaseTransformation } from "../../core/transformation/BaseTransformation";
import { ProcessException } from '../../../middleware/exception/Process.exception';
import { User } from '../../user/models/User.entity';
import { Contact } from '../../user/models/Contact.entity';
import { ContactLocation } from './../models/ContactLocation.entity';
import { Channel } from '../../channel/models/Channel.entity';
import { SignUpHistory } from '../../user/models/SignUpHistory.entity';
import { AddUserDto } from '../dto/addUser.dto';
import { UserRole } from '../../user/models/UserRole.entity';
import { Role } from '../../user/models/Role.entity';
import { Media } from './../../media/models/Media.entity';
import { ContactProductGroup } from './../models/ContactProductGroup.entity';
import { ContactProductCategory } from './../models/ContactProductCategory.entity';
import { ContactInterestedProductGroup } from './../models/ContactInterestedProductGroup.entity';
import { ContactInterestedProductCategory } from './../models/ContactInterestedProductCategory.entity';
import { UserAccessPermission } from './../models/UserAccessPermission.entity';
import { ContactSocialLink } from './../models/ContactSocialLink.entity';
import { SocialSite } from './../../channel/models/SocialSite.entity';
import { ChannelContactSuggestion } from "../../channel/models/ChannelContactSuggestion.entity";

import { isObject } from 'lodash';
import { InterestedProductGroup } from '../models/InterestedProductGroup.entity';
import { InterestedProductCategory } from '../models/InterestedProductCategory.entity';

export class UserInBound  {
	// constructor() {
	// 	super();
	// }

	async setAddUserObject(user: any, channelId: number, password: string, showId: string, userId: number): Promise<User> {
		try {

			/*let media = <Media>{};

			if(user.contact.avatar){
				media.documentTypeId = appconstant.DOCUMENTTYPE_IMAGE;
				media.documentPath = user.contact.avatar;
				media.dirName = appconstant.FOLDER_PATH_PROFILE;
			}*/

			let contact = <Contact>{};
			//contact Object
			contact.firstName = user.contact.firstName;
			contact.lastName = user.contact.lastName;
			contact.email = user.contact.email;
			contact.countryId = user.contact.countryId;
			contact.channelId = channelId;
			contact.showId = showId;
			contact.userId = userId;
			/*if(media){
				contact.document = media;
			}	*/
			contact.phone1 = (user.contact.phoneNo) ? user.contact.phoneNo : null;
			contact.mobileNo = (user.contact.mobileNo) ? user.contact.mobileNo : null;
			contact.jobTitle = user.contact.jobTitle;
			contact.documentId = user.documentId;
			contact.rptContactId = (+user.contact.rptContactId != 0) ? user.contact.rptContactId : null;
			// contact.linkedIn = (user.contact.rptContactId) ? user.contact.rptContactId : null;
			contact.isActive = (user.contact.isActive) ? user.contact.isActive : '1';
			contact.isShared = user.contact.isShared;
			contact.workNatureId = user.contact.workNatureId;
			contact.createdAt = new Date();

			// validate roleId
			var roles = [
				Role.ROLE_ADMIN_USER,
				Role.ROLE_PRIMARY_USER
			];

			if (!roles.includes(user.contact.role)) {
				user.contact.role = Role.ROLE_PRIMARY_USER;
			}

			// Add role based on
			let userRole = <UserRole>{};
			userRole.roleId = user.contact.role;

			let returnObject = <User>{};
			returnObject.contact = contact;
			returnObject.channelId = channelId;
			returnObject.role = userRole;
			returnObject.email = user.contact.email;
			returnObject.password = password;
			returnObject.isActive = '1';
			returnObject.createdAt = new Date();
			returnObject.verificationCode = "0000000"; // TODO: verification code should be nullable
			returnObject.verificationSentAt = new Date();

			return returnObject;

		} catch (e) {
			throw new ProcessException('account:objectCreation:User');
		}
	}


	async setProductGroup(productFamilies: any, contactId: number, activeIds: number[]): Promise<ContactProductGroup[]> {
		try {
			let resultObject = <Array<ContactProductGroup>>[];

			for (let ii = 0; ii < productFamilies.length; ii++) {
				if (!activeIds.includes(productFamilies[ii].id)) {
					let innerObject = <ContactProductGroup>{};
					innerObject.contactId = contactId;
					innerObject.productGroupId = productFamilies[ii].id;
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:productGroup');
		}
	}

	async setChannelContactSuggestion(channelContactSuggestion: any, contactId: number, channelId: number, userId: number, activeIds: number[]): Promise<ChannelContactSuggestion[]> {
		try {
			let resultObject = <Array<ChannelContactSuggestion>>[];

			for (let ii = 0; ii < channelContactSuggestion.length; ii++) {
				if (!activeIds.includes(channelContactSuggestion[ii].id)) {
					let sharetypeid: number;

					if (isObject(channelContactSuggestion[ii])) {
						sharetypeid = channelContactSuggestion[ii].id;
					} else {
						sharetypeid = channelContactSuggestion[ii];
					}

					let innerObject = <ChannelContactSuggestion>{};
					innerObject.channelid = channelId;
					innerObject.contactid = contactId;
					innerObject.sharetypeid = sharetypeid;
					innerObject.updatedby = userId;
					innerObject.createdat = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:channelContactSuggestion');
		}
	}

	async setInterestedProductGroup(productFamilies: any, channelId: number,contactId: number, activeIds: number[]): Promise<InterestedProductGroup[]> {
		try {
			let resultObject = <Array<InterestedProductGroup>>[];

			for (let ii = 0; ii < productFamilies.length; ii++) {
				if (!activeIds.includes(+productFamilies[ii].id)) {
					let innerObject = <InterestedProductGroup>{};
					innerObject.contactId = contactId;
					innerObject.channelId = channelId;
					innerObject.productGroupId = productFamilies[ii].id;
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:productGroup');
		}
	}

	async setProductCategories(productCategories: any, contactId: number, activeIds: number[]): Promise<ContactProductCategory[]> {
		try {
			let resultObject = <Array<ContactProductCategory>>[];

			for (let ii = 0; ii < productCategories.length; ii++) {
				if (!activeIds.includes(productCategories[ii].productCategoriesId)) {
					let innerObject = <ContactProductCategory>{};
					innerObject.contactId = contactId;
					innerObject.productCategorieId = productCategories[ii].productCategoriesId;
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:ProductCategories');
		}
	}

	async setInterestedProductCategories(productCategories: any, channelId: number, contactId: number, activeIds: number[]): Promise<InterestedProductCategory[]> {
		try {
			let resultObject = <Array<InterestedProductCategory>>[];

			for (let ii = 0; ii < productCategories.length; ii++) {
				if (!activeIds.includes(+productCategories[ii].productCategoriesId)) {
					let innerObject = <InterestedProductCategory>{};
					innerObject.contactId = contactId;
					innerObject.channelId = channelId;
					innerObject.productCategorieId = productCategories[ii].productCategoriesId;
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
			console.log("err:",e);
			throw new ProcessException('channel:objectCreation:ProductCategories');
		}
	}

	async setAccessPermissions(accessPermissions: any, userId: number, activeIds: number[]): Promise<UserAccessPermission[]> {
		try {
			let resultObject = <Array<UserAccessPermission>>[];

			for (let ii = 0; ii < accessPermissions.length; ii++) {
				if (!activeIds.includes(accessPermissions[ii].id)) {
					let innerObject = <UserAccessPermission>{};
					innerObject.userid = userId;
					innerObject.accesspermissionid = accessPermissions[ii].id;
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:UserAccessPermission');
		}
	}

	async setContactLocations(locations: any, channelId: number, contactId: number): Promise<ContactLocation[]> {
		try {
			let resultObject = <Array<ContactLocation>>[];

			for (let ii = 0; ii < locations.length; ii++) {
				for (let ii2 = 0; ii2 < locations[ii].location.length; ii2++) {
					let innerObject = <ContactLocation>{};
					innerObject.channelId = channelId;
					innerObject.contactId = contactId;

					innerObject.locationType = locations[ii].locationType;
					innerObject.regionCountryJCTId = locations[ii].location[ii2];
					innerObject.createdAt = new Date();

					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
			console.log(e);
			throw new ProcessException('channel:objectCreation:ContactLocation');
		}
	}

	async setSocialLinks(user: any, contactId: number): Promise<ContactSocialLink[]> {
		try {
			let resultObject = <Array<ContactSocialLink>>[];

			if (user.contact && user.contact.linkedInUrl !== undefined) {
				let innerObject = <ContactSocialLink>{};
				innerObject.contactId = contactId;
				innerObject.socialSiteId = SocialSite.SOCIALSITE_LINKEDIN;
				innerObject.contactSocialLink = user.contact.linkedInUrl;
				innerObject.createdAt = new Date();

				resultObject.push(innerObject);
			}

			if (user.contact && user.contact.googlePlusUrl !== undefined) {
				let innerObject = <ContactSocialLink>{};
				innerObject.contactId = contactId;
				innerObject.socialSiteId = SocialSite.SOCIALSITE_GOOGLEPLUS;
				innerObject.contactSocialLink = user.contact.googlePlusUrl;
				innerObject.createdAt = new Date();

				resultObject.push(innerObject);
			}
			return resultObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:SocialLink');
		}
	}
}
