import { Module } from '@nestjs/common';
import { UserService } from './service/user.service';
import { ContextService } from '../../context/context.service';
import { UserProviders } from './provider/user.provider';
import { UserRoleProviders } from './provider/userRole.providers';
import { SignUpHistoryProviders } from './provider/SignUpHistory.providers';
import { SignUpStatusProviders } from './provider/SignUpStatus.providers';
import { UserController } from './controller/user.controller';
import { databaseProviders } from 'src/core/database/database.providers';
import { ContactInterestedProductCategoryProvider, ContactInterestedProductGroupProvider, ContactProductCategoryProvider, ContactProductGroupProvider, ContactSocialLinkProvider, ContactSuggestionProvider, InterestedProductCategoryProvider, InterestedProductGroupProvider } from './provider/contact.providers';
import { ChannelTypeAccessPermissionProviders, PreferenceProviders, UserAccessPermissionProviders } from './provider/preference.providers';
import { ContactLocationProviders } from './provider/contactLocation.providers';
import { MyAddressBookService } from './service/myAddressBook.service';
import { AddressBookController } from './controller/myAddressBook.controller';
import { MyAddressBookProviders } from './provider/myAddressBook.providers';
import { CountryProviders } from '../common/provider/common.provider';

@Module({
  controllers:[UserController,AddressBookController],
  providers: [UserService,MyAddressBookService,
    ContextService,
    ...databaseProviders,
    ...UserProviders,
  ...UserRoleProviders,
  ...SignUpHistoryProviders,
  ...SignUpStatusProviders,...ContactProductCategoryProvider,
  ...ContactProductGroupProvider,
  ...ContactInterestedProductCategoryProvider,
  ...ContactInterestedProductGroupProvider,
  ...InterestedProductCategoryProvider,
  ...InterestedProductGroupProvider,
  ...PreferenceProviders,
  ...UserAccessPermissionProviders,
  ...ContactLocationProviders,
  ...ContactSocialLinkProvider,
  ...ContactSuggestionProvider,
  ...MyAddressBookProviders,
  ...ChannelTypeAccessPermissionProviders,
  ...CountryProviders],
    
    exports: [
      UserService,
      ...UserProviders,
      ...UserRoleProviders,
      ...SignUpHistoryProviders,
      ...SignUpStatusProviders,
      ...ContactProductCategoryProvider,
		...ContactProductGroupProvider,
		...ContactInterestedProductCategoryProvider,
    ...ContactInterestedProductGroupProvider,
    ...InterestedProductCategoryProvider,
    ...InterestedProductGroupProvider,
		...PreferenceProviders,
		...UserAccessPermissionProviders,
		...ContactLocationProviders,
		...ContactSocialLinkProvider,
		...ContactSuggestionProvider,
		...ChannelTypeAccessPermissionProviders,
    ...MyAddressBookProviders
    ]
 
})
export class UserModule {}
