import { Controller, Post, Get, HttpStatus, Request, Response, Param, Query, Inject, Body, UsePipes, InternalServerErrorException, UseGuards, Req } from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import { ProcessException } from 'src/middleware/exception/Process.exception';
import { ValidationPipe } from 'src/middleware/pipe/validation.pipe';
import { appconstant } from '../../../config/app.constant';
import { AuthException } from './../../../middleware/exception/Auth.exception';
import { StripeV2 } from '../../../lib/Stripe';
import { walletRechargeService } from './../service/walletRecharge.service';
import { walletRechargeDto } from './../dto/walletRecharge.dto';
@UseGuards(AuthGuard('jwt'))
@Controller('walletRecharge')
export class walletRechargeController {
    constructor(
		private readonly walletRechargeService: walletRechargeService,
	) {}
    @Get('getWallet')
	public async getWallet(@Request() req, @Response() res) {
		let transactionId = null;
		let user = req.user;
		let result = await this.walletRechargeService.getWalletRecharge(user);
		//if(result){
			return res.status(HttpStatus.OK).json({ wallet: result, confidentialAmt: +process.env.IS_CONFIDENTIAL_AMT });
		//} else {
		//	return res.status(HttpStatus.OK).json({ sent : 'error' });
		//}
		
	}

	@Post('processWalletPayment')
	public async processWalletPayment(@Request() req, @Response() res) {
		let transactionId = null;
		let user = req.user;
		let result = await this.walletRechargeService.processWalletPayment(req.body, user);
		if(result){
			return res.status(HttpStatus.OK).json({ sent : 'success' });
		} else {
			return res.status(HttpStatus.OK).json({ sent : 'error' });
		}
		
	}

	
	@Post('completeRecharge')
	public async completeRecharge(@Request() req, @Response() res) {
	
		// add to transaction table
		let user = req.user;
		let transactionId = null;
		let result = null;
		
		 result = await this.walletRechargeService.completeWalletRecharge(req.body, user);
		 
		if(req.body.paymentType === 'wallet'){
			console.log(req.body.paymentType);
		   result = await this.walletRechargeService.getWalletRecharge(user);
		}
		return res.status(HttpStatus.OK).json({
			sent: "ok",
			result : result
		});
	}

	@Post('confirmWalletRecharge')
	public async confirmWalletRecharge(@Request() req, @Response() res, @Body(new ValidationPipe()) walletRechargeDto: walletRechargeDto) {
		let user = req.user;
		
		let amount = walletRechargeDto.amount;
		let paymentId = walletRechargeDto.paymentId;
		let transactionId = null;
		let paymentIntentSecret = null;
		let stripe = new StripeV2();
		// add to transaction table
		if(paymentId == 1){
			
			paymentIntentSecret = await stripe.createPaymentIntend( +(amount * 100).toFixed(2), appconstant.CHANNELHUB_CURRENCY_CODE, 'wallet_recharge', +user.channelId);
			paymentIntentSecret = paymentIntentSecret.client_secret;
		} else if(paymentId == 4){
			paymentIntentSecret =  walletRechargeDto.captureid;
			const charge = await stripe.createchargesForSource(amount,'eur',paymentIntentSecret);

			console.log(charge);
			console.log('charge');
		}

		console.log(paymentIntentSecret);
		console.log('paymentIntentSecret');
		let transaction = await this.walletRechargeService.getTransactionIdForWalletRecharge(
			amount,
			appconstant.CHANNELHUB_CHANNEL_ID,
			user.channel.channelId,
			user.userId,
			paymentIntentSecret
		);
	//	transactionId = transaction.walletRechargeId;
		// console.log(transaction);
		return res.status(HttpStatus.OK).json({
			secret: paymentIntentSecret,
			amount: +(amount * 100).toFixed(2),
			transaction: transaction,
		});
	}
	
}