import { Table, Column, Model, DataType, BelongsTo, ForeignKey, AllowNull, Scopes, AutoIncrement, PrimaryKey } from 'sequelize-typescript';

import { Channel } from '../../channel/models/Channel.entity';
import { User } from '../../user/models/User.entity';
import { Sequelize } from 'sequelize-typescript';
import { Currency } from '../../channel/models/Currency.entity';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
const channelValidation = {
	isAvailable: async (value: string, next: Function): Promise<any> => {
		const isExist = await Channel.findOne({ where: { channelId: value } });
		if (!isExist) {
			next('channel:notFound');
		}
		next();
	},
};
const tableOptions: IDefineOptions = { timestamps: false, tableName: 'tblUserWalletAmount' } as IDefineOptions;

@Table(tableOptions)
@Scopes({
	channelDetail: {
		attributes: ['userId', 'channelId', 'amount', 'updatedAt']
	},
})

export class UserWalletAmount extends Model<UserWalletAmount> {
	

	@PrimaryKey
    @AutoIncrement
    @AllowNull(false)
    @Column(DataType.BIGINT)
	id: number

	@Column({
		type: DataType.NUMBER,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		validate: channelValidation
	})
	channelId: number

	@AllowNull(false)
	@Column(DataType.NUMBER)
	amount: number

	@Column({
		type: DataType.NUMBER,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		validate: {
			isAvailable: async (value: string, next: Function): Promise<any> => {
				const isExist = await User.findOne({ where: { userId: value } });
				if (!isExist) {
					next('account:resetPassword:userNotFound');
				}
				next();
			},
		}
	})
	userId: number

	@Column({
		type: DataType.DATE,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		defaultValue: Sequelize.literal('CURRENT_TIMESTAMP')
	})
	updatedAt: string
			
}
