import { Table, Column, Model, DataType, BelongsTo, ForeignKey, AllowNull, Scopes, AutoIncrement, PrimaryKey } from 'sequelize-typescript';
import { PaymentType } from '../../channel/models/PaymentType.entity';
import { Channel } from '../../channel/models/Channel.entity';
import { User } from '../../user/models/User.entity';
import { Sequelize } from 'sequelize-typescript';
import { Currency } from '../../channel/models/Currency.entity';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';

const channelValidation = {
	isAvailable: async (value: string, next: Function): Promise<any> => {
		const isExist = await Channel.findOne({ where: { channelId: value } });
		if (!isExist) {
			next('channel:notFound');
		}
		next();
	},
};
//const tableOptions: IDefineOptions = { timestamp: false, tableName: 'tblWalletRecharge' } as IDefineOptions;
const tableOptions: IDefineOptions = { timestamps: false, tableName: 'tblWalletRecharge' } as IDefineOptions;

@Table(tableOptions)

@Scopes({
	channelDetail: {
		attributes: ['payerChannelId', 'payeeChannelId', 'amount', 'currencyid', 'paymentIntent', 'entity', 'updatedBy']
	},
})

export class WalletRecharge extends Model<WalletRecharge> {
	

	@PrimaryKey
    @AutoIncrement
    @AllowNull(false)
    @Column(DataType.BIGINT)
	walletRechargeId: number

	@Column({
		type: DataType.NUMBER,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		validate: channelValidation
	})
	payerChannelId: number

	@Column({
		type: DataType.NUMBER,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		validate: channelValidation
	})
	payeeChannelId: number

	@AllowNull(false)
	@Column(DataType.NUMBER)
	amount: number

	@AllowNull(true)
	@Column(DataType.CHAR(255))
	transactionNumber: string

	@AllowNull(true)
	@Column(DataType.CHAR(255))
	secretKey: string

	@AllowNull(true)
	@Column(DataType.CHAR(255))
	cardType: string

	@AllowNull(true)
	@Column(DataType.CHAR(255))
	cardLastFourDigit: string

	@Column
	status: string

	@AllowNull(true)
	@Column(DataType.CHAR(255))
	message: string

	@Column({
		type: DataType.NUMBER,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		validate: {
			isAvailable: async (value: string, next: Function): Promise<any> => {
				const isExist = await User.findOne({ where: { userId: value } });
				if (!isExist) {
					next('account:resetPassword:userNotFound');
				}
				next();
			},
		}
	})
	updatedBy: number

	@Column({
		type: DataType.DATE,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		defaultValue: Sequelize.literal('CURRENT_TIMESTAMP')
	})
	paymentTransactionAt: string

	

	@Column({
		type: DataType.NUMBER,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		validate: {
			isAvailable: async (value: string, next: Function): Promise<any> => {
				const isExist = await Currency.findOne({ where: { currencyid: value } });
				if (!isExist) {
					next('currency:notFound');
				}
				next();
			},
		}
	})
	@ForeignKey(() => Currency)
	currencyid: number

	@BelongsTo(() => Currency)
	currency: Currency;

	@AllowNull(true)
	@Column(DataType.CHAR(255))
	invoiceno: string

	@AllowNull(true)
	@Column(DataType.CHAR(255))
	entity: string
 	

	@AllowNull(true)
	@Column(DataType.CHAR(255))
	paymentIntent: string
	
}
