'use strict';

import { Inject, Injectable, InternalServerErrorException } from '@nestjs/common';

import { ProcessException } from '../../../middleware/exception/Process.exception';
import { isUndefined } from 'lodash';
import { Sequelize } from 'sequelize-typescript';
import { Console } from 'console';
import { appconstant } from '../../../config/app.constant';
import { subscriptionFeatures } from '../../../config/subscriptionFeatures';
import { Country } from '../../common/models/Country.entity';
import { User } from '../../user/models/User.entity';
import { Role } from '../../user/models/Role.entity';
import * as moment from 'moment';
import { renderFile } from 'ejs';
import { S3Lib } from "src/lib/S3Lib";
let fs = require('fs');
import { join } from 'path';
const { Op } = require('sequelize')
import { ChannelJCT } from '../../channel/models/ChannelJCT.entity';
import { UserWalletAmount } from '../models/UserWalletAmount.entity';
import { OtherPayment } from '../../channel/models/OtherPayment.entity';
import { WalletTransaction } from '../models/WalletTransaction.entity';
import { WalletUsageType } from '../models/WalletUsageType.entity';
import { BaseService } from 'src/core/service/base.service';

@Injectable()
export class walletRechargeService  extends BaseService {
constructor(
	@Inject('OtherPaymentRepository') private readonly otherPaymentRepository: typeof OtherPayment,
	@Inject('UserWalletAmountRepository') private readonly userWalletAmountRepository: typeof UserWalletAmount,
	@Inject('WalletTransactionRepository') private readonly walletTransactionRepository: typeof WalletTransaction,
	 @Inject('WalletUsageTypeRepository') private readonly walletUsageTypeRepository: typeof WalletUsageType,
	 //@Inject('SequelizeInstance') private readonly sequelizeInstance
) { super(); }


public async getTransactionIdForWalletRecharge(amount, payeeChannelId, payerChannelId, userId, secretKey,paymentIntentId = null, entity = 'wallet_recharge') {
	console.log('create transaction id and return the amount and Id');
	console.log(secretKey);
			console.log('charge');
	try {
		return await this.otherPaymentRepository.create({
			payerChannelId: payerChannelId,
			payeeChannelId: payeeChannelId,
			amount: amount,
			updatedBy: userId,
			currencyid: appconstant.CHANNELHUB_CURRENCY,
			paymentIntent: (paymentIntentId) ? paymentIntentId : null,
			entity: entity,
			secretKey: secretKey
		}).then(async (invoice) => {
			// update invoice number
			let invoicenumber;
			// if (amount == 0) {
			// 	invoicenumber = await this.sequelizeInstance.query('select \'CHZ\' || to_char(now(),\'YYMM\')  || \'-9\' || case when ((count(*)+1) < 9) then \'0\' else \'\' end || (count(*) + 1) :: varchar(16) as number from "tblWalletRecharge" where to_char("paymentTransactionAt", \'mm/yyyy\') = to_char(now(), \'mm/yyyy\') and "walletRechargeId" < ' + invoice.walletRechargeId, {
			// 		type: this.sequelizeInstance.QueryTypes.SELECT,
			// 		raw: true,
			// 	});
			// } else {
			// 	invoicenumber = await this.sequelizeInstance.query('select \'CH\' || to_char(now(),\'YYMM\')  || \'-9\' || case when ((count(*)+1) < 9) then \'0\' else \'\' end || (count(*) + 1) :: varchar(16) as number from "tblWalletRecharge" where to_char("paymentTransactionAt", \'mm/yyyy\') = to_char(now(), \'mm/yyyy\') and "walletRechargeId" < ' + invoice.walletRechargeId, {
			// 		type: this.sequelizeInstance.QueryTypes.SELECT,
			// 		raw: true,
			// 	});
			// }
			invoice.invoiceno = null;
			await invoice.save();

			return invoice;
		});

		//
	} catch (e) {
		console.log(e);
	}
}

public async completeWalletRecharge(req, user){
	try {
		let paymentIntent = null;
		let reqAmount = 0;
		console.log(req);
		
		if(req.paymentMethod == 3){ //paypal
			paymentIntent = req.paymentIntent.client_secret;
			 reqAmount = +req.amount;
		} else if(req.paymentMethod == 1){ //stripe card
			paymentIntent = req.id;
			 reqAmount = +req.purchase_units[0].amount.value;
		}
		
		return await this.otherPaymentRepository.update({
			paymentIntent: (req.id) ? req.id : null,
			message: req.status,
			transactionNumber: (req.payment_method) ? req.payment_method : null
		}, {
			where: {
				secretKey: paymentIntent
			}
		}).then(async (invoice) => {

			
			if(req.paymentType == 'wallet') {
				let amountInfo = await this.userWalletAmountRepository.findOne({where: {
					userId: user.userId,
					channelId: user.channelId
				}});
				
				if(amountInfo == null){
					await this.userWalletAmountRepository.create({
						userId: user.userId,
						channelId: user.channelId,
						amount: +req.amount,
						updatedAt:Sequelize.fn('NOW')
					});
				} else { 
					let totalamt = [];
					let existamt = +amountInfo.amount;
					let reqAmt = +req.amount;
					totalamt['amount'] = (existamt + reqAmt);
					amountInfo.update(totalamt);
				}
		    } else {
				console.log('++++++++',paymentIntent);
				return   await this.otherPaymentRepository.findOne({
					where: {
						secretKey: paymentIntent
					}
				});	
				
			}
			return invoice;
		});
	} catch (e) {
		console.log(e);
	}
}

public async getWalletRecharge(user){
	try {
		return await UserWalletAmount.findOne({
			where: {
				channelId: user.channelId,
				userId: user.userId
			}
		}).then(async (res) => {
			return res;
		});
	} catch (e) {
		console.log(e);
	}
}

public async processWalletPayment(req, user){
	try {
		// return await UserWalletAmount.findAll({
		// 	attributes: [
		// 		'channelId',
		// 		[Sequelize.fn('sum', Sequelize.col('amount')), 'amount'],
		// 	  ],
		// 	  group: ['channelId'],
		// 	  where: {
		// 		channelId: user.channelId
		// 	}
		// })
		return await UserWalletAmount.findOne({
			where: {
				channelId: user.channelId,
				userId: user.userId
			}
		}).then(async (userAmount) => {

			/*console.log(userAmount);*/

			let walletTransactions: any = {};
			walletTransactions.payerUserId = +user.userId;
			walletTransactions.payeeChannelId = req.req_cid;
			walletTransactions.amount = req.amt;
			walletTransactions.currencyid = 1;
			walletTransactions.walletUsageTypeId = 1;
			walletTransactions.updatedAt = Date.now();
			userAmount.amount = +userAmount.amount - 10
			userAmount.save();

			if(req.jctId == null){
				await ChannelJCT.create({
					isConfidential: '0',
					channelId: +req.req_cid,
					partnerChannelId: +user.channelId,
					createdAt: Date.now(),
					connectionStatusTypeId: +appconstant.REMOVE_CONFIDENTIAL //suggestion
				});

			} else { 
				await ChannelJCT.update({
					isConfidential: '0'
				}, {
					where: {
						// channelId: user.channelId,
						// partnerChannelId: +req.req_cid,
						deletedat: null,
						channelJCTId: req.jctId
					}						
				});

				await ChannelJCT.update({
					isConfidential: '0'
				}, {
					where: {
						 channelId: +req.req_cid,
						 partnerChannelId: user.channelId,
						
					}						
				});
				console.log('wallet rc confirmation rrrrr');
				console.log(req.jctId);
			}
			//this.walletTransactionRepository.create(walletTransactions);
			return userAmount;
		});
	} catch (e) {
		console.log(e);
	}
}
}
