'use strict';

import { Channel } from './../../channel/models/Channel.entity';
import { ContextService } from './../../../context/context.service';
import { Put, Controller, Post, Get, HttpStatus, Response, Query, UseGuards, Res, Req, Body, Delete, Param, ParseIntPipe } from '@nestjs/common';
import { ValidationPipe } from './../../../middleware/pipe/validation.pipe';
import { MyWishlistSearchFilterDto } from './../dto/myWishlistFilter.dto';
import { ProductDto } from './../dto/product.dto';
import { ProductMoveDto } from './../dto/productMove.dto';
import { WishlistService } from './../service/wishlist.service';
import { AuthException } from './../../../middleware/exception/Auth.exception';
import { Role } from './../../user/models/Role.entity';
import {Op, QueryTypes } from "sequelize";
import { GroupDto } from '../dto/group.dto';
import { AuthGuard } from '@nestjs/passport';
import { BaseController } from 'src/core/controller/base.controller';
import { permissions } from 'src/config/permissions.constants';

@Controller('favorite')
@UseGuards(AuthGuard('jwt'))
export class IndexController extends BaseController {

	constructor(
		private wishlistService: WishlistService
	) {
		super();
	}

	async _validateWishlistModule(user) {
		if (user.channelTypeId == Channel.CHANNEL_TYPE_VENDOR) {
			throw new AuthException('Not allowed');
		}

		// let currentUserPermissions = this.context.get('dataSession').permissions;
		// let currentUserRole = this.context.get('dataSession').role;

		// if (currentUserRole.roleId != Role.ROLE_ACCOUNT_USER) {
		// 	if (!currentUserPermissions.includes(permissions.PRODUCTS_WISHLIST)) {
		// 		throw new AuthException('Not allowed');
		// 	}
		// }
	}

	@Post('')
	async getMywishlist(@Req() req, @Res() res, @Body(new ValidationPipe()) productFilters: any) {
		// MyWishlistSearchFilterDto
		let user = req.user;

		// user preference
		await this._validateWishlistModule(user);

		// skip rejected products
		let rejectedProductIds = await Channel.sequelize.query('select array_agg(productid) as ids from tblproductrejection where channelid = ' + user.channelId, {
			type: QueryTypes.SELECT,
			raw: true,
		}).then((result:any) => {
			return (result[0].ids) ? result[0].ids : [];
		});

		// get All wishlist ids
		let productIds = [];

		await this.wishlistService.findAll(user.userId, productFilters.type, user.channelId, rejectedProductIds).then((products) => {
			products.forEach((product) => {
				productIds.push(product.productid);
			});
		});
		let results=[];
		let total=productIds.length;
		if(productIds.length>0){
			results = await this.wishlistService.getSuggestedProduct(productIds,productFilters, user);
		}
		let result = {
			results:results,
			total:total
		}
		res.status(HttpStatus.OK).send(result);
	}

	@Post('add')
	async addToMywishlist(@Req() req, @Res() res, @Body(new ValidationPipe()) product: ProductDto) {
		let user = req.user;

		// user preference
		await this._validateWishlistModule(user);

		await this.wishlistService.create({
			userid: +user.userId,
			channelid: +user.channelId,
			productid: product.id,
			type: product.type
		}, user);

		res.status(HttpStatus.CREATED).json({
			"status": true
		});
	}

	@Delete('')
	async removeFromMywishlist(@Req() req, @Res() res, @Query('id', new ParseIntPipe()) id) {
		let user = req.user;

		// user preference
		await this._validateWishlistModule(user);

		await this.wishlistService.delete(id, 'WISHLIST', user);

		res.status(HttpStatus.NO_CONTENT).json({
			"status": true
		});
	}

	@Get('/count')
	async getItemCount(@Req() req, @Res() res) {
		let user = req.user;

		// user preference
		//await this._validateWishlistModule(user);

		let result = await this.wishlistService.count('WISHLIST', user.userId);

		result += await this.wishlistService.channelcount(user);

		res.status(HttpStatus.OK).json({
			"status": true,
			count: result
		});
	}

	@Get('/productcount/:productId')
	async getProductCount(@Param('productId') productId, @Req() req, @Res() res) {
		let user = req.user;

		// user preference
		//await this._validateWishlistModule(user);
		let result = await this.wishlistService.productcount(user, productId);
		res.status(HttpStatus.OK).json({
			"status": true,
			count: result
		});
	}

	@Put('')
	async MoveToOtherType(@Req() req, @Res() res, @Body(new ValidationPipe()) product: ProductMoveDto) {
		let user = req.user;

		// user preference
		await this._validateWishlistModule(user);

		// TODO : validate BUSINESS
		await this.wishlistService.move(user.channelId, user.userId, product, user);

		res.status(HttpStatus.OK).json({
			"status": true
		});
	}

	//create new channel group --@Nagaraj
	@Post('group')
	async addChannelGroup(@Req() req, @Res() res, @Body(new ValidationPipe()) group: GroupDto) {
		let user = req.user;

		// user preference
		//await this._validateWishlistModule(user);

		let result=await this.wishlistService.createChannelGroup({
			userId: +user.userId,
			channelId: +user.channelId,
			groupName: group.groupName			
		}, user);

		res.status(HttpStatus.CREATED).json({
			"status": result.channelGroupId
		});
	}

	@Get('channelcount')
	async getChannelCount(@Req() req, @Res() res) {
		let user = req.user;	

		let result = await this.wishlistService.channelcount(user);

		res.status(HttpStatus.OK).json({
			"status": true,
			count: result
		});
	}


	//get favorite group list --@Nagaraj
	@Get('group')
	public async channelGroupList(@Req() req, @Response() res) {
		let user = req.user;
		let group = await this.wishlistService.getGroupList(user);
		res.status(HttpStatus.OK).send(group);
	}

	//add favorite channel to group --@Nagaraj
	@Post('addchannel')
	async addFavoriteChannel(@Req() req, @Res() res, @Body(new ValidationPipe()) product: ProductDto) {
		let user = req.user;

		
		await this.wishlistService.createFavoriteChannel({
			userid: +user.userId,
			channelid: +product.channelid,
			channelgroupid: product.channelgroupid,
			type: product.type
		}, user,product.list_type);

		res.status(HttpStatus.CREATED).json({
			"status": true
		});
	}

	//remove favorite channel from group --@Nagaraj
	@Put('removechannel')
	async removeFavoriteChannel(@Req() req, @Res() res, @Body(new ValidationPipe()) product: ProductDto) {
		let user = req.user;
		
		await this.wishlistService.removeFavoriteChannel({
			userid: +user.userId,
			channelid: +product.channelid,
			channelgroupid: product.channelgroupid			
		}, user,product.list_type);

		res.status(HttpStatus.CREATED).json({
			"status": true
		});
	}

	//remove favorite group from group --@Nagaraj
	@Put('removegroup')
	async removeChannelGroup(@Req() req, @Res() res,  @Body(new ValidationPipe()) product: ProductDto) {
		let user = req.user;

		// user preference
		//await this._validateWishlistModule(user);

		await this.wishlistService.removeFavoriteGroup(product.channelgroupid);

		res.status(HttpStatus.NO_CONTENT).json({
			"status": true
		});
	}
}
