'use strict';

import { IsEnum, IsNotEmpty, MaxLength, IsString, IsOptional, IsInt, IsBoolean, ArrayNotEmpty, IsNumber } from 'class-validator';

export class MyWishlistSearchFilterDto {

	@IsNotEmpty({ message: "validate:field:shouldNotEmpty" })
	@IsEnum([
		'WISHLIST',
		'BUSINESS',
		'EVALVATED'
	], { message: "Field 'type' should be any one from this list(WISHLIST, BUSINESS, EVALVATED)" })
	type: string

	@IsOptional()
	@ArrayNotEmpty({ message: "validate:field:shouldBeArray" })
	@IsInt({
		each: true,
		message: "validate:field:shouldBeInt"
	})
	readonly productCategoryIds: number;

	@IsOptional()
	@ArrayNotEmpty({ message: "validate:field:shouldBeArray" })
	@IsInt({
		each: true,
		message: "validate:field:shouldBeInt"
	})
	readonly productFamilyIds: number;

	@IsOptional()
	@ArrayNotEmpty({ message: "validate:field:shouldBeArray" })
	@IsInt({
		each: true,
		message: "validate:field:shouldBeInt"
	})
	readonly brandIds: number[];

	@IsOptional()
	@ArrayNotEmpty({ message: "validate:field:shouldBeArray" })
	@IsString({
		each: true,
		message: "validate:field:shouldBeString"
	})
	readonly quality: string;

	@IsOptional()
	@IsBoolean({ message: "validate:field:shouldBeBoolean" })
	readonly is_sample_free: boolean;

	@IsOptional()
	@IsNumber({}, { message: "validate:field:shouldBeInt" })
	readonly retail_price_min: number;

	@IsOptional()
	@IsNumber({}, { message: "validate:field:shouldBeInt" })
	readonly retail_price_max: number;

	@IsOptional()
	@IsNumber({}, { message: "validate:field:shouldBeInt" })
	readonly retailer_commission_min: number;

	@IsOptional()
	@IsNumber({}, { message: "validate:field:shouldBeInt" })
	readonly retailer_commission_max: number;

	@IsOptional()
	@IsNumber({}, { message: "validate:field:shouldBeInt" })
	readonly distributor_commission_min: number;

	@IsOptional()
	@IsNumber({}, { message: "validate:field:shouldBeInt" })
	readonly distributor_commission_max: number;
}
