import {Catch,ExceptionFilter,HttpException,ArgumentsHost, Logger, HttpStatus} from '@nestjs/common';
import { ValidationException } from 'src/middleware/exception/Validation.exception';


@Catch((ValidationException))
export class HttpErrorFilter implements ExceptionFilter
{

    catch(error,host:ArgumentsHost)
    {
        
        const ctx=host.switchToHttp();
        const request=ctx.getRequest();
        const response=ctx.getResponse();
        const status=error instanceof HttpException?error.getStatus():HttpStatus.INTERNAL_SERVER_ERROR;
        
        const errorResponse ={
            code: status,
            timestamp:new Date().toLocaleDateString(),
            path:request.url,
            method: request.method,
            message:error.message||null
        };
        if (error instanceof ValidationException) {
            response.setHeader('x-httpStatus-error', HttpStatus.UNPROCESSABLE_ENTITY);
            response.status(HttpStatus.UNPROCESSABLE_ENTITY).json(error.messages);
          return;
        } 

        Logger.error(
            `${request.method} ${request.url}`,
            JSON.stringify(errorResponse),
            'ExceptionFilter',            
        );

        response.status(status).json(errorResponse);
        
    }

}