import { HttpStatus, Inject, Injectable, InternalServerErrorException, Param, ParseIntPipe, Req, Res } from '@nestjs/common';
import {Op, QueryTypes } from "sequelize";
import { appconstant } from 'src/config/app.constant';
import { AuthException } from 'src/middleware/exception/Auth.exception';
import { Country } from 'src/modules/common/models/Country.entity';
import { RegionCountryJCT } from 'src/modules/common/models/RegionCountryJCT.entity';
import { Media } from 'src/modules/media/models/Media.entity';
import { SampleOrder } from 'src/modules/order/models/SampleOrder.entity';
import { Brand } from 'src/modules/product/models/Brand.entity';
import { Product } from 'src/modules/product/models/Product.entity';
import { ProductCategory } from 'src/modules/product/models/ProductCategory.entity';
import { ProductEvaluation } from 'src/modules/product/models/ProductEvaluation.entity';
import { ProductGroup } from 'src/modules/product/models/ProductGroup.entity';
import { ContactSocialLink } from 'src/modules/user/models/ContactSocialLink.entity';
import { Role } from 'src/modules/user/models/Role.entity';
import { SignUpStatus } from 'src/modules/user/models/SignUpStatus.entity';
import { User } from 'src/modules/user/models/User.entity';
import { UserRole } from 'src/modules/user/models/UserRole.entity';
import { Utility } from '../helper/utility';
import { CertificateType } from '../models/CertificateType.entity';
import { Channel } from '../models/Channel.entity';
import { ChannelAward } from '../models/ChannelAward.entity';
import { ChannelType } from '../models/ChannelType.entity';
import { ContactJCT } from '../models/ContactJCT.entity';
import { ProfileType } from '../models/ProfileType.entity';
import { SocialSite } from '../models/SocialSite.entity';
import { Subscription } from '../models/Subscription.entity';
import { SubscriptionPlan } from '../models/SubscriptionPlan.entity';
import { ChannelDetail } from '../models/ChannelDetail.entity';
import { ChannelHubDetailOutBound } from '../transformation/channelHubDetailOutBound';
import { ChannelKeyPartnerService } from './channelKeyPartner.service';
import { ContextService } from 'src/context/context.service';
import { isBoolean, isNull } from 'lodash';
import { Contact } from 'src/modules/user/models/Contact.entity';
import { ShareType } from '../models/ShareType.entity';
import { ChannelCustomerProfile } from '../models/ChannelCustomerProfile.entity';
import { ProfileTypeClassification } from '../models/ProfileTypeClassification.entity';
import { ICustomerProfileAssaingObject } from 'src/modules/product/transformation/interface/IProductSerivceOutBound';
import { Region } from 'src/modules/common/models/Region.entity';
import { ProcessException } from 'src/middleware/exception/Process.exception';
import { Address } from '../models/Address.entity';

import {Dashboard} from '../models/Dashboard.entity';
import {DashboardReport} from '../models/DashboardReport.entity';
import { ChannelB2B } from '../models/ChannelB2B.entity';
import { CompanyMediaImages } from '../models/CompanyMediaImages';
import { EventParticipants } from '../models/EventParticipants.entity';
import { EventBo } from '../models/EventBo.entity';
import { EventPackage } from '../models/EventPackage.entity';
import { EventChannel } from '../models/EventChannel.entity';

var _ = require('lodash');

@Injectable()
export class ChannelService {
	private channelHubDetailOutBound = new ChannelHubDetailOutBound();
    constructor(
        @Inject('SequelizeInstance') private readonly sequelizeInstance,
		@Inject('ChannelRepository') private readonly channelRepository: typeof Channel,
		@Inject('SignUpStatusRepository') private readonly signUpStatusRepository: typeof SignUpStatus,
		@Inject('ChannelTypeRepository') private readonly channelTypeRepository: typeof ChannelType,
		@Inject('ProductGroupRepository') private readonly productGroupRepository: typeof ProductGroup,
		@Inject('ProductCategoryRepository') private readonly productCategoryRepository: typeof ProductCategory,
		@Inject('ProfileTypeRepository') private readonly profileTypeRepository: typeof ProfileType,
		@Inject('ContactJCTRepository') private readonly contactJCTRepository: typeof ContactJCT,
		@Inject('SubscriptionRepository') private readonly subscriptionRepository: typeof Subscription,
		@Inject('MediaRepository') private readonly mediaRepository: typeof Media,
		@Inject('ChannelCertificateTypeRepository') private readonly channelCertificateTypeRepository: typeof CertificateType,
		@Inject('ProductRepository') private readonly productRepository: typeof Product,
		@Inject('SharetypeRepository') private readonly sharetypeRepository: typeof ShareType,
		@Inject('ChannelCustomerProfileRepository') private readonly channelCustomerProfileRepository: typeof ChannelCustomerProfile,
		@Inject('ProfileTypeClassificationRepository') private readonly profileTypeClassificationRepository: typeof ProfileTypeClassification,
		@Inject('BrandRepository') private readonly brandRepository: typeof Brand,
		@Inject('ContactRepository') private readonly contactRepository: typeof Contact,
		// @Inject('AddressRepository') private readonly addressRepository: typeof Address,
		private readonly channelKeyPartnerService: ChannelKeyPartnerService,
		@Inject('CompanyMediaImagesRepository') private readonly companyMediaImagesRepository: typeof CompanyMediaImages,
		@Inject('CountryRepository') private readonly countryRepository: typeof Country,
    ){}

	async getTopCompany(keyword, typeId, btype) {
	
		let type=[];
		if (typeId == 2 && btype == 1)
			type=[3,4];
		else if (typeId == 3 && btype == 2)
			type = [3, 5];
		else if (typeId == 2 && btype == 3)
			type = [3, 4, 5];
		else if (typeId == 3 || typeId == 4 || typeId == 5)
			type = [2];
		let query='select "channelId","companyName","channelTypeId" from "tblChannel" where "channelTypeId" in (:typeId) and unaccent("companyName") ILike :search and "deletedAt" is null and "isActive"=\'1\' '

		let company=await this.sequelizeInstance.query(query , {
			type: QueryTypes.SELECT,
			raw: true,
			replacements: {
				search: '' + keyword + '%',
				typeId:type,
				}
			});
		
		query='select "productId","product","productSKU" from "tblProduct" where unaccent("product") ILike :search and "deletedat" is null and "isactive"=\'1\' limit 4 '

		let product=await this.sequelizeInstance.query(query , {
			type: QueryTypes.SELECT,
			raw: true,
			replacements: {
				search: '' + keyword + '%',
				}
			});
		let result={
			company:company,
			product:product,
			type: +typeId,
			btype: +btype
		}

		

		return result;
		
	}

	async getTopProduct(keyword){
		let query='select "productId","product" from "tblProduct" where unaccent("companyName") ILike :search limit 4 '

		let result=await this.sequelizeInstance.query(query , {
			type: QueryTypes.SELECT,
			raw: true,
			replacements: {
				search: '' + keyword + '%',
				}
			});
		return result;
	}

    async getSearchList(viewId,planId,data,userData){
		let user = userData;
        let channelFilter=data.filter;

		let contactId=user.contactId;
		let channelId=user.channelId;
		let typeid=user.channelTypeId;
		let businessType = user.businessType;
		let userId=user.userId;
		let filterch=[];let channeltype=[];
		let aggregation={};let channelquery;

		console.log("productId:", channelFilter.productId);

		if (typeid == 2 && businessType == 1)
			channeltype = [3, 4];
		else if (typeid == 2 && businessType == 2)
			channeltype = [3, 5];
		else if (typeid == 2 && businessType == 3)
			channeltype = [3, 4, 5];
		else if (typeid == 3)// && businessType == 1)
			channeltype = [2];
		// else if (typeid == 3 && businessType == 2)
		//     channeltype = [2, 3, 4];
		// else if (typeid == 3 && businessType == 3)
		//     channeltype = [2, 3, 4, 5];
		else if (typeid == 4)
			channeltype = [2];
		else if (typeid == 5)
			channeltype = [2];


		
		let order=data.order;
		//confirm		
		//console.log("viewId:",viewId,channeltype[0]);

		if(viewId==1||(channelFilter.connectionStatusId&&channelFilter.connectionStatusId==1))
		{			
			channelquery = 'select "matchingScore","connectionStatusTypeId" as relation_status,\'Confirm\' as status,"channelJCTId" as channeljct,"transToContact","note",(select "documentPath" from "tblDocument" where "documentId"="introVideo") as "introVideo" ,"isConfidential",case when "channelId"=' + channelId + ' then "partnerChannelId" when "partnerChannelId"=' + channelId + ' then "channelId" end as channel_id,case when "requested_to"=' + contactId + ' then "requested_by" when "requested_by"=' + contactId + ' then "requested_to" end as contact_id from "tblChannelJCT" where "connectionStatusTypeId"=2 and case when "requested_by"=' + contactId + ' then "channelId" in(select "channelId" from "tblChannel" '
				+ ' where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) '
				+ ' when "requested_to"=' + contactId + ' then "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6))  end and "deletedat" is null order by "updatedAt" ' + data.order + '  ';
		}
		//outbound
		else if(viewId==2||(channelFilter.connectionStatusId&&channelFilter.connectionStatusId==2))
		{
			channelquery = 'select "matchingScore","connectionStatusTypeId" as relation_status,\'OUT\' as status,"channelJCTId" as channeljct,"transToContact","note",(select "documentPath" from "tblDocument" where "documentId"="introVideo") as "introVideo" ,"isConfidential",case when "channelId"=' + channelId + ' then "partnerChannelId" when "partnerChannelId"=' + channelId + ' then "channelId" end as channel_id,case when "requested_to"=' + contactId + ' then "requested_by" when "requested_by"=' + contactId + ' then "requested_to" end as contact_id from "tblChannelJCT"  where "connectionStatusTypeId" in(7,11) and case when "requested_to"=' + contactId + ' and "connectionStatusTypeId"=11 then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) when "requested_by"=' + contactId + ' and "connectionStatusTypeId"=7 then "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"=' + contactId + ' and "connectionStatusTypeId" in(5,6))  end and "deletedat" is null order by "updatedAt" ' + data.order + ' ';
		}
		//inbound
		else if(viewId==3||(channelFilter.connectionStatusId&&channelFilter.connectionStatusId==3))
		{
			channelquery = 'select "matchingScore","connectionStatusTypeId" as relation_status,\'IN\' as status,"channelJCTId" as channeljct,"transToContact","note",(select "documentPath" from "tblDocument" where "documentId"="introVideo") as "introVideo" ,"isConfidential",case when "channelId"=' + channelId + ' then "partnerChannelId" when "partnerChannelId"=' + channelId + ' then "channelId" end as channel_id,case when "requested_to"=' + contactId + ' then "requested_by" when "requested_by"=' + contactId + ' then "requested_to" end as contact_id from "tblChannelJCT" where "connectionStatusTypeId" in(7,10) '
		+' and case when "requested_to"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15  and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) end and "deletedat" is null order by "updatedAt" '+data.order+' ';
		}
		//requestintro
		else if(viewId==4)
		{
		channelquery='select "matchingScore","connectionStatusTypeId" as relation_status,\'RequestIntro\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential",case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id,case when "requested_to"='+contactId+' then "requested_by" when "requested_by"='+contactId+' then "requested_to" end as contact_id,"updatedAt" from "tblChannelJCT" where "connectionStatusTypeId"=10 and "partnerChannelId" not in(select "partnerChannelId" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "deletedat" is null and "requested_by"='+contactId+' and "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') union all select "connectionStatusTypeId" as relation_status,\'RequestIntro\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential",case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id,case when "requested_to"='+contactId+' then "requested_by" when "requested_by"='+contactId+' then "requested_to" end as contact_id,"updatedAt"  from "tblChannelJCT" where "connectionStatusTypeId"=11 and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "deletedat" is null and "requested_to"='+contactId+' and "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') order by "updatedAt" '+data.order+'  ';
		}
		//savelater
		else if(viewId==5)
		{
		channelquery='select "matchingScore","connectionStatusTypeId" as relation_status,\'Savelater\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential",case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id,case when "requested_to"='+contactId+' then "requested_by" when "requested_by"='+contactId+' then "requested_to" end as contact_id from "tblChannelJCT" where "connectionStatusTypeId"=11 and case when "requested_by"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6)) end and "deletedat" is null order by "updatedAt" '+data.order+' ' ;
		}
		//suggestion
		else if(viewId==6||(channelFilter.connectionStatusId&&channelFilter.connectionStatusId==6))
		{
		channelquery='select "matchingScore","connectionStatusTypeId" as relation_status,\'Suggestion\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential","channelId" as channel_id,"requested_by" as contact_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"='+contactId+' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) order by "updatedAt" '+data.order+' ' ;
		}
		//vendor suggestion
		else if(viewId==8)
		{			
			channelquery = 'select "matchingScore","connectionStatusTypeId" as relation_status,\'Suggestion\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential","channelId" as channel_id,"requested_by" as contact_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"=' + contactId + ' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(2) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) order by "matchingScore" desc '; // "updatedAt" '+data.order+' ' ;
		}

		//distributor suggestion
		else if(viewId==9)
		{
			channelquery = 'select "matchingScore","connectionStatusTypeId" as relation_status,\'Suggestion\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential","channelId" as channel_id,"requested_by" as contact_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"=' + contactId + ' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(3) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) order by "matchingScore" asc '; // "updatedAt" '+data.order+' ' ;
		}

		//retailer suggestion
		else if(viewId==10)
		{
		channelquery='select "matchingScore","connectionStatusTypeId" as relation_status,\'Suggestion\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential","channelId" as channel_id,"requested_by" as contact_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"='+contactId+' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(4) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) order by "rank" asc '; // "updatedAt" '+data.order+' ' ;
		}

		//reseller suggestion
		else if(viewId==14)
		{
		channelquery='select "matchingScore","connectionStatusTypeId" as relation_status,\'Suggestion\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential","channelId" as channel_id,"requested_by" as contact_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"='+contactId+' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(5) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) order by "rank" asc '; // "updatedAt" '+data.order+' ' ;
		}
		
		else if(viewId==30)
		{	
			channelquery='select "matchingScore","connectionStatusTypeId" as relation_status,\'Confirm\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential",case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id,case when "requested_to"='+contactId+' then "requested_by" when "requested_by"='+contactId+' then "requested_to" end as contact_id,"updatedAt" from "tblChannelJCT" where "connectionStatusTypeId"=2 and case when "requested_to"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" '
			+' where  "channelTypeId" in(:channelTypeId) and "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) '
			+' when "requested_by"='+contactId+' then "partnerChannelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6))  end and "deletedat" is null'
			+' union all'
				+ ' select "matchingScore","connectionStatusTypeId" as relation_status,\'Outbound\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential",case when "channelId"=' + channelId + ' then "partnerChannelId" when "partnerChannelId"=' + channelId + ' then "channelId" end as channel_id,case when "requested_to"=' + contactId + ' then "requested_by" when "requested_by"=' + contactId + ' then "requested_to" end as contact_id,"updatedAt" from "tblChannelJCT"  where "connectionStatusTypeId" in(7,11) and case when "requested_to"=' + contactId + ' and "connectionStatusTypeId"=11 then "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) when "requested_by"=' + contactId + ' and "connectionStatusTypeId"=7 then "partnerChannelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"=' + contactId + ' and "connectionStatusTypeId" in(5,6))  end and "deletedat" is null '
			+' union all'
				+ ' select "matchingScore","connectionStatusTypeId" as relation_status,\'Inbound\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential",case when "channelId"=' + channelId + ' then "partnerChannelId" when "partnerChannelId"=' + channelId + ' then "channelId" end as channel_id,case when "requested_to"=' + contactId + ' then "requested_by" when "requested_by"=' + contactId + ' then "requested_to" end as contact_id,"updatedAt" from "tblChannelJCT" where "connectionStatusTypeId" in(7,10) '
			+' and case when "requested_to"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "deletedAt" is null and "signUpStatusId" = 15  and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) end and "deletedat" is null  '
			+' union all'
				+ ' select "matchingScore","connectionStatusTypeId" as relation_status,\'RequestIntro\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential",case when "channelId"=' + channelId + ' then "partnerChannelId" when "partnerChannelId"=' + channelId + ' then "channelId" end as channel_id,case when "requested_to"=' + contactId + ' then "requested_by" when "requested_by"=' + contactId + ' then "requested_to" end as contact_id,"updatedAt" from "tblChannelJCT" where "connectionStatusTypeId"=10 and "partnerChannelId" not in(select "partnerChannelId" from "tblChannelJCT" where "requested_by"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) and "deletedat" is null and "requested_by"=' + contactId + ' and "partnerChannelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') union all select "matchingScore","connectionStatusTypeId" as relation_status,\'RequestIntro\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential",case when "channelId"=' + channelId + ' then "partnerChannelId" when "partnerChannelId"=' + channelId + ' then "channelId" end as channel_id,case when "requested_to"=' + contactId + ' then "requested_by" when "requested_by"=' + contactId + ' then "requested_to" end as contact_id,"updatedAt"  from "tblChannelJCT" where "connectionStatusTypeId"=11 and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) and "deletedat" is null and "requested_to"=' + contactId + ' and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\')  '
			+' union all'
				+ ' select "matchingScore","connectionStatusTypeId" as relation_status,\'Savelater\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential",case when "channelId"=' + channelId + ' then "partnerChannelId" when "partnerChannelId"=' + channelId + ' then "channelId" end as channel_id,case when "requested_to"=' + contactId + ' then "requested_by" when "requested_by"=' + contactId + ' then "requested_to" end as contact_id,"updatedAt" from "tblChannelJCT" where "connectionStatusTypeId"=11 and case when "requested_by"=' + contactId + ' then "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) end and "deletedat" is null'
			+' union all'
				+ ' select "matchingScore","connectionStatusTypeId" as relation_status,\'Suggestion\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential","channelId" as channel_id,"requested_by" as contact_id,"updatedAt" from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"=' + contactId + ' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)'
			+' union all'
			
				+ ' select CAST( null AS bigint) as "matchingScore",CAST( null AS bigint) as relation_status,\'NonSuggestion\' as status,CAST( null AS bigint) as channeljct,CAST( null AS bigint) as "transToContact",CAST( null AS text) as "note",CAST( null AS bigint) as "introVideo",CAST( null AS bit) as "isConfidential",tc."channelId" as channel_id,CAST( null AS bigint)as contact_id,tc."updatedAt" from "tblChannel" as tc inner join "tblContact" as con on con."channelId"=tc."channelId" and con."isActive"=\'1\' and con."deletedAt" is null and con."contactId" not in(select case when "requested_to"=' + contactId + ' then "requested_by" when "requested_by"=' + contactId + ' then "requested_to" end from "tblChannelJCT" as jct where ("requested_by"=' + contactId + ' or "requested_to"=' + contactId + ') and "connectionStatusTypeId" in(5,6,2,7,10,11,1)) where "signUpStatusId"=15 and  "channelTypeId" not in(6) and tc."isActive"=\'1\' and tc."deletedAt" is null and "channelTypeId" in(:channelTypeId) and tc."channelId" not in(select case when "dismissedBy"=' + channelId + ' then "dismissedChId" when "dismissedChId"=' + channelId + ' then "dismissedBy" end from "tblchanneldismissreason" where "dismissedBy"=' + channelId + ' or ("dismissedChId"=' + channelId + ' and "dismissedBy"=tc."channelId")) ';
			
		}		
						
		let result= await this.sequelizeInstance.query(channelquery, {
        type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		search: '%' + data.search + '%',
		channelTypeId:channeltype
		}
		}).then(x => {
			if (viewId == 3 || viewId == 1 || viewId == 2)
				return x;
			else
				return _.uniqBy(x, 'channel_id');
		})
		if(data.search)
		{
			//console.log("page",pagination.page);//console.log("searcg",pagination.search);
			let  searchchannellist=await this.sequelizeInstance.query('select "channelId" as channel_id from "tblChannel" where "companyName" ILIKE :search', {
				type: QueryTypes.SELECT,
				raw: true,
				replacements: {
				search: '%' + data.search + '%',
				}
				})
			let chl=searchchannellist.map(x => x.channel_id);
			result = result.filter(ch => {return chl.includes(ch.channel_id);})
			//console.log(result)	;
		}
		// console.log('check',result?true:false)
		if (channelFilter.typeId && result.length > 0)
		{
			let chlist = result.map(x => +x.channel_id);		
			let ctype='select array_agg("channelId") as ch from "tblChannel" where "channelTypeId" in(:channeltypeid) and "channelId" in(:channelid)';
		await this.sequelizeInstance.query(ctype, {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:chlist,
		channeltypeid:+channelFilter.typeId
		}
		}).then((result) => {
			if(result[0].ch)
		filterch.push(...result[0].ch);
		});
		result = result.filter(ch => {return filterch.includes(ch.channel_id);})
		}
		
		if ((channelFilter.storeCount || channelFilter.turnOver || channelFilter.brandId || channelFilter.locationId || channelFilter.countryId || channelFilter.specialityId || channelFilter.intProductId || channelFilter.productId || channelFilter.retailerprofileId || channelFilter.resellerprofileId) && result.length > 0) {

		let chlist = result.map(x => +x.channel_id);		
		
		if(channelFilter.turnOver)
		{
		let query;
			if(channelFilter.turnOver=='<1M€')
			{
				query='select array_agg("channelId") as ch from "tblChannelDetail" where "channelId" in(:channelid) and case when "turnoverRange"=\'<1M€\' or cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)<1 or "turnover"<1 then cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)<1 or "turnoverRange"=\'<1M€\' or "turnover"<1 end';
			}
			else if(channelFilter.turnOver=='1-10M€')
			{					
				query='select  array_agg("channelId") as ch from "tblChannelDetail" where "channelId" in(:channelid) and case when "turnoverRange"=\'1-10M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\'))as int)>1 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<10) or ("turnover">1 and "turnover"<10) then "turnoverRange"=\'1-10M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\'))as int)>1 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<10) or ("turnover">1 and "turnover"<10) end';
			}
			else if(channelFilter.turnOver=='10-100M€')
			{
				query='select  array_agg("channelId") as ch from "tblChannelDetail" where "channelId" in(:channelid) and "turnoverRange"!=\'>100M€\' and case when "turnoverRange"=\'10-100M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\')) as int)>10 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<=100) or ("turnover">10 and "turnover"<=100) then "turnoverRange"=\'10-100M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\')) as int)>10 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<=100) or ("turnover">10 and "turnover"<=100) end';
			}
			else if(channelFilter.turnOver=='>100M€')
			{
				query='select array_agg("channelId") as ch from "tblChannelDetail" where "channelId" in(:channelid) and case when "turnoverRange"=\'>100M€\' or cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)>100 or "turnover">100 then cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)>100 or "turnoverRange"=\'>100M€\' or "turnover">100 end';
			}
			
		await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:chlist
		//channeltypeid:channelFilter.typeId
		}
		}).then((result) => {

			if(result[0].ch){
				filterch = [];
				filterch.push(...result[0].ch);
			}else
			filterch = [];
		});
		}

			if (channelFilter.storeCount) {
				let store;

				if (channelFilter.storeCount == '5')
					store = 'select array_agg("channelId") as ch from "tblChannelDetail" where "channelId" in(:channelid) and ("physicalSales"<=5 or "physicalSales" is null)';
				else
					store = 'select array_agg("channelId") as ch from "tblChannelDetail" where "channelId" in(:channelid) and "physicalSales">:count';

				await this.sequelizeInstance.query(store, {
					type: QueryTypes.SELECT,
					raw: true,
					replacements: {
						channelid: (filterch.length > 0) ? filterch : chlist,
						count: +channelFilter.storeCount
					}
				}).then((result) => {
					if (result[0].ch) {

						filterch = [];
						filterch.push(...result[0].ch);
					} else
						filterch = [];

				});
			}

		if(channelFilter.brandId) {
		let brand='select array_agg("channelId") as ch from "tblChannelBrand" where "brandId" in(:brandid) and "channelId" in(:channelid)';
		await this.sequelizeInstance.query(brand, {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:chlist,
		brandid:channelFilter.brandId
		}
		}).then((result) => {
			if(result[0].ch){
				filterch = [];
		filterch.push(...result[0].ch);
		}else
		filterch = [];

		});
		}


			if (channelFilter.productId) {
		await this.sequelizeInstance.query('select array_agg("channelId") as ch from "tblChannelProductCategorie" cpc where cpc."productCategorieId" in(:productcatid) and "channelId" in(:channelid) ', {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:(filterch.length>0)?filterch:chlist,
				productcatid: channelFilter.productId
			}
		}).then((result) => {
			if (result[0].ch) {
				filterch = [];
				filterch.push(...result[0].ch);
			} else
				filterch = [];

		});
			}

			if (channelFilter.intProductId) {
				await this.sequelizeInstance.query('select array_agg("channelId") as ch from "tblInterestedProductCategory" cpc where cpc."productCategorieId" in(:productcatid) and "contactId" in(:contactid) and "channelId" in(:channelid) ', {
					type: QueryTypes.SELECT,
					raw: true,
					replacements: {
						contactid: result.map(x => +x.contact_id),
						channelid: (filterch.length > 0) ? filterch : chlist,
						productcatid: channelFilter.intProductId
					}
				}).then((result) => {
					if (result[0].ch) {
						filterch = [];
						filterch.push(...result[0].ch);
					} else
						filterch = [];

				});
			}

			if (channelFilter.retailerprofileId || channelFilter.resellerprofileId) {
				let query;
				let ids = channelFilter.retailerprofileId ? channelFilter.retailerprofileId : channelFilter.resellerprofileId;

				if (viewId == 8 || viewId == 9)
					query = 'select array_agg("channelId") as ch from "tblChannelCustomerProfile" as ccp where "profileTypeId" in(:retailprofileid) and "channelId" in(:channelid) and ccp."profileTypeId" in(select "profileTypeId" from "tblProfileTypeClassification" where ("channelTypeId" in(3,4) and ccp."channelId" in(select "channelId" from "tblChannel" where "channelId"=ccp."channelId" and "isOEM"=\'1\')) or '
						+ '("channelTypeId" in(3,5) and ccp."channelId" in(select "channelId" from "tblChannel" where "channelId"=ccp."channelId" and "isOEM"=\'2\')) or'
						+ '("channelTypeId" in(3,4,5) and ccp."channelId" in(select "channelId" from "tblChannel" where "channelId"=ccp."channelId" and "isOEM"=\'3\')))';
				if (viewId == 10 || viewId == 14)
					query = 'select array_agg("channelId") as ch from "tblChannel" where "profileTypeId" in(:retailprofileid) and "channelId" in(:channelid)';

				await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:(filterch.length>0)?filterch:chlist,
			retailprofileid: (channelFilter.retailerprofileId && channelFilter.resellerprofileId) ? channelFilter.resellerprofileId.concat(channelFilter.retailerprofileId) : ids

		}
		}).then((result) => {
			if(result[0].ch){
				filterch = [];
		filterch.push(...result[0].ch);
			}else
			filterch = [];

		});
		}
	
		if(channelFilter.countryId) {
		await this.sequelizeInstance.query('select array_agg("channelId") as ch from "tblChannel" where "countryId" in(:countryid) and "channelId" in(:channelid)', {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:(filterch.length>0)?filterch:chlist,
		countryid:channelFilter.countryId
		}
		}).then((result) => {
			if(result[0].ch){
				filterch = [];
		filterch.push(...result[0].ch);
			}else
			filterch = [];
		});
		}
		if(channelFilter.specialityId) {
		await this.sequelizeInstance.query('select array_agg(distinct "channelId") as ch from "tblContact" where "contactId" in(Select "contactId" from "tblContactInterestedProductCategory" where "productCategorieId" in(:specialityid)) and "channelId" in(:channelid)', {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:(filterch.length>0)?filterch:chlist,
		specialityid:channelFilter.specialityId
		}
		}).then((result) => {
			if(result[0].ch){
				filterch = [];
		filterch.push(...result[0].ch);
			}else
			filterch = [];
		});
		}
		if(channelFilter.locationId) {

			let locationType = user.channelTypeId == 2 ? 'SELLING' : 'TARGET';

			let query = 'select array_agg("channelId") as ch  from "tblChannelLocation" where "locationType"=:locationType and "channelId" in(:channelid) and "deletedAt" is null and'
				+ '"regionCountryJCTId" in(select "regionCountryJCTId" from "tblRegionCountryJCT" where "countryId" in (:locationid)) '

			await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:(filterch.length>0)?filterch:chlist,
			locationType: locationType,
		locationid:channelFilter.locationId
		}
		}).then((result) => {
			if(result[0].ch){
				filterch = [];
		filterch.push(...result[0].ch);
			}else
			filterch = [];
		});
		}
		// if(!channelFilter.connectionStatusId)
			result = result.filter(ch => {return filterch.includes(ch.channel_id);})
		}

		
		let total = result?result.length:0;
		let final = []; let master = [];
				
		if (result.length > 0) {

			let ordered = await this.channelRepository.findAll({
				type: QueryTypes.SELECT,
				attributes: ['channelId'],
				raw: true,
				where: {
					channelId: result.map(x => x.channel_id)
				},
				order: [['companyName', data.order]]
			}).then(res => {
				return res.map(x => x.channelId)
			});

			result.sort(function (a, b) {
				return ordered.indexOf(a) - ordered.indexOf(b);
			});
			result = await this.mapOrder(result, ordered, 'channel_id');

			result.sort((a, b) => b.matchingScore - a.matchingScore);

			master = result.map(x => +x.channel_id);

			if (viewId == 3 || viewId == 1 || viewId == 2) {
				//result = result.map(x => +x.contact_id);
				result = result.slice(data.offset).slice(0, data.limit);
			}
			else
			result=result.slice(data.offset).slice(0,data.limit);
			
			if ((viewId == 3 || viewId == 1 || viewId == 2) && result.length > 0) {
				let clist = await this.getConnectedContactsList(result);
				let resultformat = {
					results: clist,
					master: master,
					aggregation: aggregation,
					total: total
				}
				return resultformat;
			}
			else if ((viewId == 3 || viewId == 1 || viewId == 2) && result.length == 0) {
				let resultformat = {
					results: [],
					master: [],
					aggregation: aggregation,
					total: 0
				}
				return resultformat;
			}
			else {

		 for(let i=0;i<result.length;i++)
		 {
			 let status=result[i].status;		 
			// console.log("status:",status);
			let companyquery;

			try{			
			if(result[i].channeljct!=null)
			{  
				companyquery = 'Select ' + result[i].matchingScore + ' as matchingScore,' + result[i].relation_status + ' as relation_status,:status as status,' + result[i].isConfidential + ' as isConfidential,' + result[i].channeljct + ' as channel_jct_id,' + result[i].transToContact + ' as transToContact,:note as note,ch."channelId" as cid,ch."companyName" as companyname,ch."countryId" as countryid, tc."country" as country, tc."isoCode", (select "profileType" from "tblProfileType" where "profileTypeId"=ch."profileTypeId"), cht."channelType" as channeltype,ch."channelTypeId" as channeltypeid,'
		+'chd."detailDesc" as detaildesc, chd."webSiteUrl" as websiteurl,chd."startup",chd."expectations" as sourcing,(select array_to_json(array_agg(row_to_json(a))) from (select SUBSTRING ( cast(chd."maturitylevel" as TEXT) ,2 , length(cast(chd."maturitylevel" as TEXT))-2) as item_name)a) as looking_startups, '
		+'(select doc."documentPath" from "tblDocument" as doc  where doc."documentId"='+result[i].introVideo+' ) as introvideo, '
		+'(select distinct channelgroupid as favorite from "tblproductwishlist" fav where fav."userid"='+userId+' and fav."channelid"=ch."channelId" and "productid" is null and "deletedat" is null fetch first row only), '
		+'(Select td."documentPath" as logo from "tblChannelDetail" chd inner join "tblDocument" td on td."documentId" in(chd."channelLogoId") where chd."channelId"=ch."channelId"), '

		+'(select array_to_json(array_agg(row_to_json(c))) from (Select td."documentPath",con."isDefaultContact",con."firstName",con."lastName",con."jobTitle" from "tblContact" con left join "tblDocument" td on td."documentId" in(con."documentId") where con."channelId"=ch."channelId")c)as contacts, '
		
		+'(select array_to_json(array_agg(row_to_json(x))) from (Select pc."productCategorie" as item_name from "tblChannelProductCategorie" cpc inner join "tblProductCategories" pc on pc."productCategorieId" = cpc."productCategorieId" where cpc."channelId"=ch."channelId")x) as company_cat, '
		
				  + '(select array_to_json(array_agg(row_to_json(z))) from (Select pt."profileType" as item_name from "tblProfileType" pt where "profileTypeId" in (select "profileTypeId" from "tblChannelCustomerProfile" as ccp where ccp."channelId"=ch."channelId" and "profileTypeId" in(select "profileTypeId" from '
				  + '"tblProfileTypeClassification" where ("channelTypeId" in(3,4) and ccp."channelId" in(select "channelId" from "tblChannel" where "channelId"=ch."channelId" and "isOEM"=\'1\')) or '
				  + '("channelTypeId" in(3,5) and ccp."channelId" in(select "channelId" from "tblChannel" where "channelId"=ch."channelId" and "isOEM"=\'2\')) or '
				  + '("channelTypeId" in(3,4,5) and ccp."channelId" in(select "channelId" from "tblChannel" where "channelId"=ch."channelId" and "isOEM"=\'3\')))	))z) as retail_customer_profile '
		

		+'from "tblChannel" ch inner join "tblChannelDetail" chd on chd."channelId" = ch."channelId" inner join "tblCountry" tc on tc."countryId" = ch."countryId" '
				  + 'inner join "tblChannelType"cht on cht."channelTypeId" = ch."channelTypeId" where ch."channelId" in(:channelid)';
			 }
			 else{
				companyquery = 'Select ' + result[i].matchingScore + ' as matchingScore,' + result[i].relation_status + ' as relation_status,:status as status,' + result[i].isConfidential + ' as isConfidential,' + result[i].channeljct + ' as channel_jct_id,' + result[i].transToContact + ' as transToContact,:note as note,ch."channelId" as cid,case when chd."turnover" is null then chd."turnoverRange" when chd."turnover" is not null then cast(chd."turnover" as text) end as turnover,ch."companyName" as companyname,ch."countryId" as countryid, tc."country" as country, tc."isoCode", (select "profileType" from "tblProfileType" where "profileTypeId"=ch."profileTypeId"), cht."channelType" as channeltype,ch."channelTypeId" as channeltypeid,'
				+'chd."detailDesc" as detaildesc, chd."webSiteUrl" as websiteurl, chd."startup",chd."expectations" as sourcing,(select array_to_json(array_agg(row_to_json(a))) from (select SUBSTRING ( cast(chd."maturitylevel" as TEXT) ,2 , length(cast(chd."maturitylevel" as TEXT))-2) as item_name)a) as looking_startups, '

				+'(select doc."documentPath" from "tblDocument" as doc  where doc."documentId"='+result[i].introVideo+' ) as introvideo, '

				+'(select distinct channelgroupid as favorite from "tblproductwishlist" fav where fav."userid"='+userId+' and fav."channelid"=ch."channelId" and "productid" is null and "deletedat" is null fetch first row only), '
				+'(Select td."documentPath" as logo from "tblChannelDetail" chd inner join "tblDocument" td on td."documentId" in(chd."channelLogoId") where chd."channelId"=ch."channelId"), '
		
				+'(select array_to_json(array_agg(row_to_json(c))) from (Select td."documentPath",con."isDefaultContact",con."firstName",con."lastName",con."jobTitle" from "tblContact" con left join "tblDocument" td on td."documentId" in(con."documentId") where con."channelId"=ch."channelId")c)as contacts, '

				+'(select array_to_json(array_agg(row_to_json(x))) from (Select pc."productCategorie" as item_name from "tblChannelProductCategorie" cpc inner join "tblProductCategories" pc on pc."productCategorieId" = cpc."productCategorieId" where cpc."channelId"=ch."channelId")x) as company_cat, '
		
					+ '(select array_to_json(array_agg(row_to_json(z))) from (Select pt."profileType" as item_name from "tblProfileType" pt where "profileTypeId" in (select "profileTypeId" from "tblChannelCustomerProfile" as ccp where ccp."channelId"=ch."channelId" and "profileTypeId" in(select "profileTypeId" from '
					+ '"tblProfileTypeClassification" where ("channelTypeId" in(3,4) and ccp."channelId" in(select "channelId" from "tblChannel" where "channelId"=ch."channelId" and "isOEM"=\'1\')) or '
					+ '("channelTypeId" in(3,5) and ccp."channelId" in(select "channelId" from "tblChannel" where "channelId"=ch."channelId" and "isOEM"=\'2\')) or '
					+ '("channelTypeId" in(3,4,5) and ccp."channelId" in(select "channelId" from "tblChannel" where "channelId"=ch."channelId" and "isOEM"=\'3\')))	))z) as retail_customer_profile '
				
		
				+'from "tblChannel" ch inner join "tblChannelDetail" chd on chd."channelId" = ch."channelId" inner join "tblCountry" tc on tc."countryId" = ch."countryId" '
					+ 'inner join "tblChannelType"cht on cht."channelTypeId" = ch."channelTypeId" where ch."channelId" in(:channelid)';
			 }
		let res= await this.sequelizeInstance.query(companyquery, {
            type: QueryTypes.SELECT,
				raw: true,
				replacements: {
				channelid:result[i].channel_id,
				contactid:result[i].contact_id,
				status:result[i].status,
				note:(result[i].note=='')?null:result[i].note
				}
				});

				final.push(...res);

			}
			catch(e)
			{
				console.log("error:",e);
				// console.log("status:",result[i])
	
			}
		 }
			}
		
	}else
	result=[];




	final.map(x => {		
		//console.log("typeid:",x.channeltypeid);	
		let flag = 0;

		if ((x.relation_status == null || x.relation_status == 1) && ((x.isconfidential == 1 && (viewId == 8 || viewId == 9 || viewId == 10 || viewId == 6 || viewId == 7) && planId == 1 && (typeid == 2)))) // || (typeid == 3 && x.channeltypeid == 4)))
			//|| (planId == 7 && x.isconfidential == 1 && (typeid == 2 || (typeid == 3 && x.channeltypeid == 4)) && (viewId == 6 || viewId == 7))
			//|| (planId == 7 && x.isconfidential == 1 && data.page != 1 && (typeid == 2 || (typeid == 3 && x.channeltypeid == 4)) && (viewId == 8 || viewId == 9 || viewId == 10))))
		{
			if (planId == 1 && flag == 0) {
				flag = 1;
				return false;
			}
			else if ((flag == 1 && planId == 1) || (flag == 0)) {
			x.logo=null;
			x.companyname='XXXXXXXXXXXXXXXXX';
			x.country='XXXXXXXXXXXXXXXXX';
			x.contacts.map(c => {
				c.firstName='XXXXXXXXXXXXXXXXX';
				c.lastName='XXXXXXXXXXXXXXXXX';
				c.jobTitle='XXXXXXXXXXXXXXXXX';
			})
		}
		}

	})

		let resultformat={
		results:final,
			master: master,
		aggregation:aggregation,
		total:total
		}

		return resultformat;
    }

	async nonMemberList(viewId,planId,data,userData){
		let user = userData;
        let channelFilter=data.filter;
		let contactId=user.contactId;
		let channelId=user.channelId;
		let typeid=user.channelTypeId;
		let userId=user.userId;
		let filterch=[];let channeltype=[];
		let aggregation={};let channelquery;
		if(typeid==2)
		channeltype=[3,4];
		if(typeid == 3 || typeid == 4 || typeid == 5)
		channeltype=[2];
		// else if(typeid==3)
		// channeltype=[2,3,4];
		// else if(typeid==4)
		// channeltype=[2,3];
		
		let order=data.order;			
		channelquery='select "oldChannelId" as channel_id from "tblOldChannel" where "channelTypeId" in(:channelTypeId) and "isActive"=\'1\' and "deletedAt" is null'; // "updatedAt" '+data.order+' ' ;
					 
				
		let result= await this.sequelizeInstance.query(channelquery, {
        type: QueryTypes.SELECT,
		raw: true,
		replacements: {		
		channelTypeId:channeltype
		}
		}).then(x=>{						
			return  _.uniqBy(x,'channel_id');
		})

		

		if(data.search)
		{			
			let  searchchannellist=await this.sequelizeInstance.query('select "oldChannelId" as channel_id from "tblOldChannel" where "companyName" ILIKE :search', {
				type: QueryTypes.SELECT,
				raw: true,
				replacements: {
				search: '%' + data.search + '%',
				}
				})
			let chl=searchchannellist.map(x => x.channel_id);
			result = result.filter(ch => {return chl.includes(ch.channel_id);})			
		}		
		if(channelFilter.typeId && result)
		{
			let chlist = result.map(x => +x.channel_id);		
			let ctype='select array_agg("oldChannelId") as ch from "tblOldChannel" where "channelTypeId" in(:channeltypeid) and "oldChannelId" in(:channelid)';
		await this.sequelizeInstance.query(ctype, {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:chlist,
		channeltypeid:+channelFilter.typeId
		}
		}).then((result) => {
			if(result[0].ch)
		filterch.push(...result[0].ch);
		});
		result = result.filter(ch => {return filterch.includes(ch.channel_id);})
		}
		
		if((channelFilter.turnOver||channelFilter.brandId||channelFilter.locationId||channelFilter.countryId||channelFilter.specialityId||channelFilter.productCategoryId||channelFilter.retailerprofileId)&& result) {

		let chlist = result.map(x => +x.channel_id);		
		
		if(channelFilter.turnOver)
		{
		let query;
			if(channelFilter.turnOver=='<1M€')
			{
				query='select array_agg("oldChannelId") as ch from "tblOldChannelDetail" where "oldChannelId" in(:channelid) and case when "turnoverRange"=\'<1M€\' or cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)<1 or "turnover"<1 then cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)<1 or "turnoverRange"=\'<1M€\' or "turnover"<1 end';
			}
			else if(channelFilter.turnOver=='1-10M€')
			{					
				query='select  array_agg("oldChannelId") as ch from "tblOldChannelDetail" where "oldChannelId" in(:channelid) and case when "turnoverRange"=\'1-10M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\'))as int)>1 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<10) or ("turnover">1 and "turnover"<10) then "turnoverRange"=\'1-10M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\'))as int)>1 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<10) or ("turnover">1 and "turnover"<10) end';
			}
			else if(channelFilter.turnOver=='10-100M€')
			{
				query='select  array_agg("oldChannelId") as ch from "tblOldChannelDetail" where "oldChannelId" in(:channelid) and "turnoverRange"!=\'>100M€\' and case when "turnoverRange"=\'10-100M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\')) as int)>10 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<=100) or ("turnover">10 and "turnover"<=100) then "turnoverRange"=\'10-100M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\')) as int)>10 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<=100) or ("turnover">10 and "turnover"<=100) end';
			}
			else if(channelFilter.turnOver=='>100M€')
			{
				query='select array_agg("oldChannelId") as ch from "tblOldChannelDetail" where "oldChannelId" in(:channelid) and case when "turnoverRange"=\'>100M€\' or cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)>100 or "turnover">100 then cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)>100 or "turnoverRange"=\'>100M€\' or "turnover">100 end';
			}
			
		await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:chlist
		//channeltypeid:channelFilter.typeId
		}
		}).then((result) => {

			if(result[0].ch){
				filterch = [];
				filterch.push(...result[0].ch);
			}else
			filterch = [];
		});
		}

		if(channelFilter.brandId) {
		let brand='select array_agg("oldChannelId") as ch from "tblOldChannelBrand" where "brandId" in(:brandid) and "oldChannelId" in(:channelid)';
		await this.sequelizeInstance.query(brand, {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:chlist,
		brandid:channelFilter.brandId
		}
		}).then((result) => {
			if(result[0].ch){
				filterch = [];
		filterch.push(...result[0].ch);
		}else
		filterch = [];

		});
		}
		if(channelFilter.productCategoryId) {
		await this.sequelizeInstance.query('select array_agg("oldChannelId") as ch from "tblOldChannelProductCategorie" cpc where cpc."productCategorieId" in(:productcatid) and "oldChannelId" in(:channelid) ', {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:(filterch.length>0)?filterch:chlist,
		productcatid:channelFilter.productCategoryId
		}
		}).then((result) => {
			if(result[0].ch){
				filterch = [];
		filterch.push(...result[0].ch);
			}else
			filterch = [];

		});
		}
		if(channelFilter.retailerprofileId) {
		await this.sequelizeInstance.query('select array_agg("oldChannelId") as ch from "tblOldChannelCustomerProfile" where "profileTypeId" in(:retailprofileid) and "oldChannelId" in(:channelid)', {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:(filterch.length>0)?filterch:chlist,
		retailprofileid:channelFilter.retailerprofileId
		}
		}).then((result) => {
			if(result[0].ch){
				filterch = [];
		filterch.push(...result[0].ch);
			}else
			filterch = [];

		});
		}
	
		if(channelFilter.countryId) {
		await this.sequelizeInstance.query('select array_agg("oldChannelId") as ch from "tblOldChannel" where "countryId" in(:countryid) and "oldChannelId" in(:channelid)', {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:(filterch.length>0)?filterch:chlist,
		countryid:channelFilter.countryId
		}
		}).then((result) => {
			if(result[0].ch){
				filterch = [];
		filterch.push(...result[0].ch);
			}else
			filterch = [];
		});
		}
		if(channelFilter.specialityId) {
		await this.sequelizeInstance.query('select array_agg(distinct "oldChannelId") as ch from "tblOldContact" where "oldContactId" in(Select "oldContactId" from "tblContactInterestedProductCategory" where "productCategorieId" in(:specialityid)) and "oldChannelId" in(:channelid)', {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:(filterch.length>0)?filterch:chlist,
		specialityid:channelFilter.specialityId
		}
		}).then((result) => {
			if(result[0].ch){
				filterch = [];
		filterch.push(...result[0].ch);
			}else
			filterch = [];
		});
		}
		if(channelFilter.locationId) {
		await this.sequelizeInstance.query('select array_agg("oldChannelId") as ch from "tblOldChannel" where "countryId" in(:locationid) and "oldChannelId" in(:channelid)', {
            type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:(filterch.length>0)?filterch:chlist,
		locationid:channelFilter.locationId
		}
		}).then((result) => {
			if(result[0].ch){
				filterch = [];
		filterch.push(...result[0].ch);
			}else
			filterch = [];
		});
		}
		// if(!channelFilter.connectionStatusId)
			result = result.filter(ch => {return filterch.includes(ch.channel_id);})
		}

		
		let total = result?result.length:0;
		let final=[];
				
		if(result){		

			let ordered = await this.sequelizeInstance.query('select "oldChannelId" from "tblOldChannel" where "oldChannelId" in(:channelid)  order by "companyName" asc', {
				type: QueryTypes.SELECT,
				raw: true,
				replacements: {
				channelid:result.map(x => x.channel_id),
				}						
			}).then(res => {
				return res.map(x=>x.oldChannelId)
				});

			//   result.sort(function(a, b){  
			// 	return ordered.indexOf(a) - ordered.indexOf(b);
			//   });

			result = await this.mapOrder(result, ordered, 'channel_id');

			result=result.slice(data.offset).slice(0,data.limit);
			
			
		
		 for(let i=0;i<result.length;i++)
		 {
			 let status='Non Suggestion';		 
			 let mscore=null;let relation_status=null;let isConfidential=1;let channeljct=null;let transToContact=null;let note=null;
			// console.log("status:",status);
			let companyquery;

			try{						
				companyquery='Select '+mscore+' as matchingScore,'+relation_status+' as relation_status,\'Non Member\' as status,'+isConfidential+' as isConfidential,'+channeljct+' as channel_jct_id,'+transToContact+' as transToContact,'+note+' as note,ch."oldChannelId" as cid,case when chd."turnover" is null then chd."turnoverRange" when chd."turnover" is not null then cast(chd."turnover" as text) end as turnover,ch."companyName" as companyname,ch."countryId" as countryid, tc."country" as country, tc."isoCode", pt."profileType" as profiletype, cht."channelType" as channeltype,ch."channelTypeId" as channeltypeid,'
				+'chd."detailDesc" as detaildesc, chd."webSiteUrl" as websiteurl, chd."startup",(select array_to_json(array_agg(row_to_json(a))) from (select SUBSTRING ( cast(chd."maturitylevel" as TEXT) ,2 , length(cast(chd."maturitylevel" as TEXT))-2) as item_name)a) as looking_startups, '

				//+'(select doc."documentPath" from "tblDocument" as doc  where doc."documentId"='+result[i].introVideo+' ) as introvideo, '

				+'(select distinct channelgroupid as favorite from "tblproductwishlist" fav where fav."userid"='+userId+' and fav."channelid"=ch."oldChannelId" and "productid" is null and "deletedat" is null fetch first row only), '
				+'(Select td."documentPath" as logo from "tblOldChannelDetail" chd inner join "tblDocument" td on td."documentId" in(chd."channelLogoId") where chd."oldChannelId"=ch."oldChannelId"), '
		
				+'(select array_to_json(array_agg(row_to_json(c))) from (Select td."documentPath",con."isDefaultContact",con."firstName",con."lastName",con."jobTitle" from "tblOldContact" con left join "tblDocument" td on td."documentId" in(con."documentId") where con."oldChannelId"=ch."oldChannelId")c)as contacts, '

				+'(select array_to_json(array_agg(row_to_json(x))) from (Select pc."productCategorie" as item_name from "tblOldChannelProductCategorie" cpc inner join "tblProductCategories" pc on pc."productCategorieId" = cpc."productCategorieId" where cpc."oldChannelId"=ch."oldChannelId")x) as company_cat, '
		
				+'(select array_to_json(array_agg(row_to_json(z))) from (Select pt."profileType" as item_name from "tblProfileType" pt where "profileTypeId" in (select "profileTypeId" from "tblOldChannelCustomerProfile" as ccp where ccp."oldChannelId"=ch."oldChannelId"))z) as retail_customer_profile '
				
		
				+'from "tblOldChannel" ch inner join "tblOldChannelDetail" chd on chd."oldChannelId" = ch."oldChannelId" inner join "tblCountry" tc on tc."countryId" = ch."countryId" '
				+'inner join "tblProfileType"pt on pt."profileTypeId" = ch."profileTypeId" inner join "tblChannelType" as cht on cht."channelTypeId" = ch."channelTypeId" where ch."oldChannelId" in(:channelid)';
			 
		let res= await this.sequelizeInstance.query(companyquery, {
            type: QueryTypes.SELECT,
				raw: true,
				replacements: {
				 channelid:result[i].channel_id,
				// contactid:result[i].contact_id,
				// status:result[i].status,
				// note:(result[i].note=='')?null:result[i].note
				 }
				});

				final.push(...res);

			}
			catch(e)
			{
				console.log("error:",e);
				// console.log("status:",result[i])
	
			}
		}		
		
	}else
	result=[];


	// final.map(x => {		
	// 	//console.log("typeid:",x.channeltypeid);
	// 	if((x.relation_status==null||x.relation_status==1)&&((x.isconfidential!=1 && (viewId==8||viewId==9||viewId==10||viewId==6||viewId==7) && planId==1&&(typeid==2||(typeid==3&&x.channeltypeid==4))) 
	// 	||(planId==7&&x.isconfidential!=1&&(typeid==2||(typeid==3&&x.channeltypeid==4))&&(viewId==6||viewId==7))
	// 	||(planId==7&&x.isconfidential!=1&&data.page!=1&&(typeid==2||(typeid==3&&x.channeltypeid==4))&&(viewId==8||viewId==9||viewId==10))))
	// 	{
	// 		x.logo=null;
	// 		x.companyname='XXXXXXXXXXXXXXXXX';
	// 		x.country='XXXXXXXXXXXXXXXXX';
	// 		x.contacts.map(c => {
	// 			c.firstName='XXXXXXXXXXXXXXXXX';
	// 			c.lastName='XXXXXXXXXXXXXXXXX';
	// 			c.jobTitle='XXXXXXXXXXXXXXXXX';
	// 		})
	// 	}
	// })

		let resultformat={
		results:final,
		aggregation:aggregation,
		total:total
		}

		return resultformat;
    }
    

    async getSearchAggregation(viewId,data){

		let user = data;//this.context.get('user');

		let contactId=user.contactId;
		let channelId=user.channelId;
		let typeid=user.channelTypeId;
	//	let userId=user.userId;
		//let filterch=[];
		let channeltype=[];
		let aggregation=null;let channelquery;
		if(typeid==2)
		channeltype=[3,4];
		else if(typeid==3)
		channeltype=[2,4];
		else if(typeid==4)
		channeltype=[2,3];
		//let order='desc';
		//confirm		
		if(viewId==1)
		{			
		channelquery='select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=2 and case when "requested_to"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" '
		+' where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) '
		+' when "requested_by"='+contactId+' then "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6))  end and "deletedat" is null    ';
		}
		//outbound
		else if(viewId==2)
		{
		channelquery='select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT"  where "connectionStatusTypeId" in(7,11) and case when "requested_to"='+contactId+' and "connectionStatusTypeId"=11 then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) when "requested_by"='+contactId+' and "connectionStatusTypeId"=7 then "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6))  end and "deletedat" is null ';	
		}
		//inbound
		else if(viewId==3)
		{
		channelquery='select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(7,10) '
		+' and case when "requested_to"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15  and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) end and "deletedat" is null ';
		}
		//requestintro
		else if(viewId==4)
		{
		channelquery='select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=10 and "partnerChannelId" not in(select "partnerChannelId" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "deletedat" is null and "requested_by"='+contactId+' and "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') union all select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id  from "tblChannelJCT" where "connectionStatusTypeId"=11 and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "deletedat" is null and "requested_to"='+contactId+' and "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\')  ';
		}
		//savelater
		else if(viewId==5)
		{
		channelquery='select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=11 and case when "requested_by"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6)) end and "deletedat" is null  ' ;
		}
		//suggestion
		else if(viewId==6)
		{
		channelquery='select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"='+contactId+' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)  ' ;
		}
		//vendor suggestion
		else if(viewId==8)
		{
		channelquery='select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"='+contactId+' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(2) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) ' ;
		}

		//distributor suggestion
		else if(viewId==9)
		{
		channelquery='select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"='+contactId+' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(3) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)  ' ;
		}

		//retailer suggestion
		else if(viewId==10)
		{
		channelquery='select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"='+contactId+' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(4) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)  ' ;
		}
		else if(viewId==7)
		{	
			channelquery='select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=2 and case when "requested_to"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" '
			+' where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) '
			+' when "requested_by"='+contactId+' then "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6))  end and "deletedat" is null'
			+' union all'
			+' select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT"  where "connectionStatusTypeId" in(7,11) and case when "requested_to"='+contactId+' and "connectionStatusTypeId"=11 then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) when "requested_by"='+contactId+' and "connectionStatusTypeId"=7 then "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6))  end and "deletedat" is null '
			+' union all'
			+' select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(7,10) '
			+' and case when "requested_to"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15  and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) end and "deletedat" is null  '
			+' union all'
			+' select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=10 and "partnerChannelId" not in(select "partnerChannelId" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "deletedat" is null and "requested_by"='+contactId+' and "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') union all select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id  from "tblChannelJCT" where "connectionStatusTypeId"=11 and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "deletedat" is null and "requested_to"='+contactId+' and "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\')  '
			+' union all'
			+' select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=11 and case when "requested_by"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6)) end and "deletedat" is null' 
			+' union all'
			+' select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"='+contactId+' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)'
			+' union all'
			// +' select CAST( null AS bigint) as relation_status,\'NonSuggestion\' as status,CAST( null AS bigint) as channeljct,CAST( null AS bigint) as "transToContact",CAST( null AS text) as "note",CAST( null AS bigint) as "introVideo",CAST( null AS bit) as "isConfidential","channelId" as channel_id,CAST( null AS bigint)as contact_id,"updatedAt" from "tblChannel" where "signUpStatusId"=15 and  "channelTypeId" not in(5,6) and "isActive"=\'1\' and "deletedAt" is null and "channelTypeId" in(:channelTypeId) and "channelId" not in(select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end from "tblChannelJCT" where ("requested_by"='+contactId+' or "requested_to"='+contactId+') and "connectionStatusTypeId" in(5,6,2,7,10,11,1) and "deletedat" is null) ' 
			// +' and "channelId" not in(select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end from "tblChannelJCT" where ("requested_by"='+contactId+' or "requested_to"='+contactId+') and "connectionStatusTypeId" in(5,6,2,7,10,11,1) and "deletedat" is null) order by "updatedAt" '+pagination.order+'  ';
			+' select tc."channelId" as channel_id from "tblChannel" as tc inner join "tblContact" as con on con."channelId"=tc."channelId" and con."isActive"=\'1\' and con."deletedAt" is null and con."contactId" not in(select case when "requested_to"='+contactId+' then "requested_by" when "requested_by"='+contactId+' then "requested_to" end from "tblChannelJCT" as jct where ("requested_by"='+contactId+' or "requested_to"='+contactId+') and "connectionStatusTypeId" in(5,6,2,7,10,11,1)) where "signUpStatusId"=15 and  "channelTypeId" not in(5,6) and tc."isActive"=\'1\' and tc."deletedAt" is null and "channelTypeId" in(:channelTypeId) and tc."channelId" not in(select case when "dismissedBy"='+channelId+' then "dismissedChId" when "dismissedChId"='+channelId+' then "dismissedBy" end from "tblchanneldismissreason" where "dismissedBy"='+channelId+' or ("dismissedChId"='+channelId+' and "dismissedBy"=tc."channelId")) '; 
			//+' and "channelId" not in(select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end from "tblChannelJCT" where ("requested_by"='+contactId+' or "requested_to"='+contactId+') and "connectionStatusTypeId" in(5,6,2,7,10,11,1) and "deletedat" is null) order by "updatedAt" '+pagination.order+'  ';
		}		 
				
		let result= await this.sequelizeInstance.query(channelquery, {
		type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		//search: '%' + pagination.search + '%',
		channelTypeId:channeltype
		}
		}).then(x=>{						
			return  _.uniqBy(x,'channel_id');
		})

	if(result.length>0) 
	{
			let chlist = result.map(x => +x.channel_id);

			let brandquery='select distinct on(tb."brandName") tb."brandName" as name,tb."id" as id from "tblBrand" tb '
			+'inner join "tblDocument" td on td."documentId"=tb."documentId" where id in (select "brandId" from "tblChannelBrand" where "channelId" in(:channelid)) order by tb."brandName" ';
			
			let productcatquery='Select distinct on(pc."productCategorie")pc."productCategorie" as name,pc."productCategorieId" as id from "tblChannelProductCategorie" cpc '
			+'inner join "tblProductCategories" pc on pc."productCategorieId" = cpc."productCategorieId" '
			+'where cpc."channelId" in(:channelid) order by pc."productCategorie"';
	
			let retailcustquery='Select distinct on(pt."profileType")pt."profileType" as name,pt."profileTypeId" as id from "tblProfileType" pt where "profileTypeId" in '
			+'(select "profileTypeId" from "tblChannelCustomerProfile" where "channelId" in(:channelid)) order by pt."profileType" ';
	
			let countryquery='select distinct on(tc."country")tc."country" as name, tc."priority", ch."countryId" as id from "tblChannel" ch inner join "tblCountry" tc on tc."countryId"=ch."countryId" '
			+'where ch."channelId" in(:channelid) order by tc."country" ';
	
			let intcatquery='select distinct on("productCategorie")"productCategorie" as name,"productCategorieId" as id from "tblProductCategories" where "productCategorieId" in '
			+'(Select cpc."productCategorieId" from "tblContactInterestedProductCategory" cpc where cpc."contactId" in(select "contactId" from "tblContact" where "channelId" in(:channelid) and "deletedAt" is null)) order by "productCategorie" ';
	
			let lookupquery='select distinct on("country")"country" as name,"countryId" as id, "priority" from "tblCountry" where "countryId" in '
			+'(Select pc."countryId" from "tblContactLocation" cpc inner join "tblRegionCountryJCT" pc on pc."regionCountryJCTId" = cpc."regionCountryJCTId" '
			+ 'inner join "tblChannel" ch on ch."channelId" in(:channelid) where cpc."channelId" in(:channelid) and cpc."locationType"=\'SELLING\') order by "country" ';
	
		let brandlist= await this.sequelizeInstance.query(brandquery, {
		type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:chlist
		}
		})

		let productcatlist= await this.sequelizeInstance.query(productcatquery, {
		type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:chlist
		}
		})

		let retailcustlist= await this.sequelizeInstance.query(retailcustquery, {
		type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:chlist
		}
		})

		let countrylist= await this.sequelizeInstance.query(countryquery, {
		type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:chlist
		}
		})

		let intcatlist= await this.sequelizeInstance.query(intcatquery, {
		type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:chlist
		}
		})

		let lookuplist= await this.sequelizeInstance.query(lookupquery, {
		type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		channelid:chlist
		}
		})


		aggregation={
		brandIds:brandlist,
		locactionIds:lookuplist,
		specialIds:intcatlist,
		countryIds:countrylist,
		retailIds:retailcustlist,
		productIds:productcatlist
		};

	}	
	
		return aggregation;
		
	}

	//Channel Filter Services

	async chFilterBase(viewId, channelType, user, data) {

		let channelFilter = data?.filter;
//		let user = data;//this.context.get('user');
		let chlist;

		let contactId = user.contactId;
		let channelId = user.channelId;
		let typeid = user.channelTypeId;
		let businessType = user.businessType;
		//	let userId=user.userId;
		//let filterch=[];
		let channeltype = [];
		let aggregation = null; let channelquery;

		if (channelType > 0)
			channeltype = [channelType];
		else {
			if (typeid == 2 && businessType == 1)
				channeltype = [3, 4];
			else if (typeid == 2 && businessType == 2)
				channeltype = [3, 5];
			else if (typeid == 2 && businessType == 3)
				channeltype = [3, 4, 5];
			if (typeid == 3 || typeid == 4 || typeid == 5)
				channeltype = [2];
		}


		// else if(typeid==3)
		// channeltype=[2,4];
		// else if(typeid==4)
		// channeltype=[2,3];
		//let order='desc';
		//confirm		
		if (viewId == 1) {
			channelquery = 'select case when "channelId"=' + channelId + ' then "partnerChannelId" when "partnerChannelId"=' + channelId + ' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=2 and case when "requested_to"=' + contactId + ' then "channelId" in(select "channelId" from "tblChannel" '
				+ ' where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) '
				+ ' when "requested_by"=' + contactId + ' then "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"=' + contactId + ' and "connectionStatusTypeId" in(5,6))  end and "deletedat" is null    ';
		}
		//outbound
		else if (viewId == 2) {
			channelquery = 'select case when "channelId"=' + channelId + ' then "partnerChannelId" when "partnerChannelId"=' + channelId + ' then "channelId" end as channel_id from "tblChannelJCT"  where "connectionStatusTypeId" in(7,11) and case when "requested_to"=' + contactId + ' and "connectionStatusTypeId"=11 then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) when "requested_by"=' + contactId + ' and "connectionStatusTypeId"=7 then "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"=' + contactId + ' and "connectionStatusTypeId" in(5,6))  end and "deletedat" is null ';
		}
		//inbound
		else if (viewId == 3) {
			channelquery = 'select case when "channelId"=' + channelId + ' then "partnerChannelId" when "partnerChannelId"=' + channelId + ' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(7,10) '
				+ ' and case when "requested_to"=' + contactId + ' then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15  and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) end and "deletedat" is null ';
		}
		//requestintro
		else if (viewId == 4) {
			channelquery = 'select case when "channelId"=' + channelId + ' then "partnerChannelId" when "partnerChannelId"=' + channelId + ' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=10 and "partnerChannelId" not in(select "partnerChannelId" from "tblChannelJCT" where "requested_by"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) and "deletedat" is null and "requested_by"=' + contactId + ' and "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') union all select case when "channelId"=' + channelId + ' then "partnerChannelId" when "partnerChannelId"=' + channelId + ' then "channelId" end as channel_id  from "tblChannelJCT" where "connectionStatusTypeId"=11 and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) and "deletedat" is null and "requested_to"=' + contactId + ' and "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\')  ';
		}
		//savelater
		else if (viewId == 5) {
			channelquery = 'select case when "channelId"=' + channelId + ' then "partnerChannelId" when "partnerChannelId"=' + channelId + ' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=11 and case when "requested_by"=' + contactId + ' then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) end and "deletedat" is null  ';
		}
		//suggestion
		else if (viewId == 6) {
			channelquery = 'select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"=' + contactId + ' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)  ';
		}
		//vendor suggestion
		else if (viewId == 8) {
			channelquery = 'select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"=' + contactId + ' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(2) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) ';
		}

		//distributor suggestion
		else if (viewId == 9) {
			channelquery = 'select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"=' + contactId + ' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(3) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)  ';
		}

		//retailer suggestion
		else if (viewId == 10) {
			channelquery = 'select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"=' + contactId + ' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(4) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)  ';
		}

		//reseller suggestion
		else if (viewId == 14) {
			channelquery = 'select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"=' + contactId + ' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(5) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)  ';
		}

		else if (viewId == 30) {
			channelquery = 'select "matchingScore","connectionStatusTypeId" as relation_status,\'Confirm\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential",case when "channelId"=' + channelId + ' then "partnerChannelId" when "partnerChannelId"=' + channelId + ' then "channelId" end as channel_id,case when "requested_to"=' + contactId + ' then "requested_by" when "requested_by"=' + contactId + ' then "requested_to" end as contact_id,"updatedAt" from "tblChannelJCT" where "connectionStatusTypeId"=2 and case when "requested_to"=' + contactId + ' then "channelId" in(select "channelId" from "tblChannel" '
				+ ' where  "channelTypeId" in(:channelTypeId) and "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) '
				+ ' when "requested_by"=' + contactId + ' then "partnerChannelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"=' + contactId + ' and "connectionStatusTypeId" in(5,6))  end and "deletedat" is null'
				+ ' union all'
				+ ' select "matchingScore","connectionStatusTypeId" as relation_status,\'Outbound\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential",case when "channelId"=' + channelId + ' then "partnerChannelId" when "partnerChannelId"=' + channelId + ' then "channelId" end as channel_id,case when "requested_to"=' + contactId + ' then "requested_by" when "requested_by"=' + contactId + ' then "requested_to" end as contact_id,"updatedAt" from "tblChannelJCT"  where "connectionStatusTypeId" in(7,11) and case when "requested_to"=' + contactId + ' and "connectionStatusTypeId"=11 then "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) when "requested_by"=' + contactId + ' and "connectionStatusTypeId"=7 then "partnerChannelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"=' + contactId + ' and "connectionStatusTypeId" in(5,6))  end and "deletedat" is null '
				+ ' union all'
				+ ' select "matchingScore","connectionStatusTypeId" as relation_status,\'Inbound\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential",case when "channelId"=' + channelId + ' then "partnerChannelId" when "partnerChannelId"=' + channelId + ' then "channelId" end as channel_id,case when "requested_to"=' + contactId + ' then "requested_by" when "requested_by"=' + contactId + ' then "requested_to" end as contact_id,"updatedAt" from "tblChannelJCT" where "connectionStatusTypeId" in(7,10) '
				+ ' and case when "requested_to"=' + contactId + ' then "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "deletedAt" is null and "signUpStatusId" = 15  and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) end and "deletedat" is null  '
				+ ' union all'
				+ ' select "matchingScore","connectionStatusTypeId" as relation_status,\'RequestIntro\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential",case when "channelId"=' + channelId + ' then "partnerChannelId" when "partnerChannelId"=' + channelId + ' then "channelId" end as channel_id,case when "requested_to"=' + contactId + ' then "requested_by" when "requested_by"=' + contactId + ' then "requested_to" end as contact_id,"updatedAt" from "tblChannelJCT" where "connectionStatusTypeId"=10 and "partnerChannelId" not in(select "partnerChannelId" from "tblChannelJCT" where "requested_by"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) and "deletedat" is null and "requested_by"=' + contactId + ' and "partnerChannelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') union all select "matchingScore","connectionStatusTypeId" as relation_status,\'RequestIntro\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential",case when "channelId"=' + channelId + ' then "partnerChannelId" when "partnerChannelId"=' + channelId + ' then "channelId" end as channel_id,case when "requested_to"=' + contactId + ' then "requested_by" when "requested_by"=' + contactId + ' then "requested_to" end as contact_id,"updatedAt"  from "tblChannelJCT" where "connectionStatusTypeId"=11 and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) and "deletedat" is null and "requested_to"=' + contactId + ' and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\')  '
				+ ' union all'
				+ ' select "matchingScore","connectionStatusTypeId" as relation_status,\'Savelater\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential",case when "channelId"=' + channelId + ' then "partnerChannelId" when "partnerChannelId"=' + channelId + ' then "channelId" end as channel_id,case when "requested_to"=' + contactId + ' then "requested_by" when "requested_by"=' + contactId + ' then "requested_to" end as contact_id,"updatedAt" from "tblChannelJCT" where "connectionStatusTypeId"=11 and case when "requested_by"=' + contactId + ' then "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) end and "deletedat" is null'
				+ ' union all'
				+ ' select "matchingScore","connectionStatusTypeId" as relation_status,\'Suggestion\' as status,"channelJCTId" as channeljct,"transToContact","note","introVideo","isConfidential","channelId" as channel_id,"requested_by" as contact_id,"updatedAt" from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"=' + contactId + ' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"=' + contactId + ' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)'
				+ ' union all'

				+ ' select CAST( null AS bigint) as "matchingScore",CAST( null AS bigint) as relation_status,\'NonSuggestion\' as status,CAST( null AS bigint) as channeljct,CAST( null AS bigint) as "transToContact",CAST( null AS text) as "note",CAST( null AS bigint) as "introVideo",CAST( null AS bit) as "isConfidential",tc."channelId" as channel_id,CAST( null AS bigint)as contact_id,tc."updatedAt" from "tblChannel" as tc inner join "tblContact" as con on con."channelId"=tc."channelId" and con."isActive"=\'1\' and con."deletedAt" is null and con."contactId" not in(select case when "requested_to"=' + contactId + ' then "requested_by" when "requested_by"=' + contactId + ' then "requested_to" end from "tblChannelJCT" as jct where ("requested_by"=' + contactId + ' or "requested_to"=' + contactId + ') and "connectionStatusTypeId" in(5,6,2,7,10,11,1)) where "signUpStatusId"=15 and  "channelTypeId" not in(6) and tc."isActive"=\'1\' and tc."deletedAt" is null and "channelTypeId" in(:channelTypeId) and tc."channelId" not in(select case when "dismissedBy"=' + channelId + ' then "dismissedChId" when "dismissedChId"=' + channelId + ' then "dismissedBy" end from "tblchanneldismissreason" where "dismissedBy"=' + channelId + ' or ("dismissedChId"=' + channelId + ' and "dismissedBy"=tc."channelId")) ';

		}
		if (viewId == 15) {
			channelquery = 'select "oldChannelId" as channel_id from "tblOldChannel" where "channelTypeId" in(:channelTypeId) and "isActive"=\'1\' and "deletedAt" is null'; // "updatedAt" '+data.order+' ' ;
		}
		if(viewId == 50){
			channelquery = 'select "channelid" as channel_id from "tblproductwishlist" where "userid"=:userId and "deletedat" is null' ;
		}


		let result = await this.sequelizeInstance.query(channelquery, {
			type: QueryTypes.SELECT,
			raw: true,
			replacements: {
				//search: '%' + pagination.search + '%',
				channelTypeId: channeltype,
				userId: user.userId
			}
		}).then(x => {
			return _.uniqBy(x, 'channel_id');
		})

		let filterch = [];
		if (result.length > 0)
			chlist = result.map(x => +x.channel_id);

		if (channelFilter.typeId && result.length > 0) {

			let ctype = 'select array_agg("channelId") as ch from "tblChannel" where "channelTypeId" in(:channeltypeid) and "channelId" in(:channelid)';
			await this.sequelizeInstance.query(ctype, {
				type: QueryTypes.SELECT,
				raw: true,
				replacements: {
					channelid: chlist,
					channeltypeid: +channelFilter.typeId
				}
			}).then((result) => {
				if (result[0].ch)
					filterch.push(...result[0].ch);
			});
			result = result.filter(ch => { return filterch.includes(ch.channel_id); })
		}

		if ((channelFilter.storeCount || channelFilter.turnOver || channelFilter.brandId || channelFilter.locationId || channelFilter.countryId || channelFilter.specialityId || channelFilter.productCategoryId || channelFilter.retailerprofileId || channelFilter.resellerprofileId) && result.length > 0) {

			//let chlist = result.map(x => +x.channel_id);

			if (channelFilter.turnOver) {
				let query;
				if (channelFilter.turnOver == '<1M€') {
					query = 'select array_agg("channelId") as ch from "tblChannelDetail" where "channelId" in(:channelid) and case when "turnoverRange"=\'<1M€\' or cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)<1 or "turnover"<1 then cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)<1 or "turnoverRange"=\'<1M€\' or "turnover"<1 end';
				}
				else if (channelFilter.turnOver == '1-10M€') {
					query = 'select  array_agg("channelId") as ch from "tblChannelDetail" where "channelId" in(:channelid) and case when "turnoverRange"=\'1-10M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\'))as int)>1 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<10) or ("turnover">1 and "turnover"<10) then "turnoverRange"=\'1-10M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\'))as int)>1 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<10) or ("turnover">1 and "turnover"<10) end';
				}
				else if (channelFilter.turnOver == '10-100M€') {
					query = 'select  array_agg("channelId") as ch from "tblChannelDetail" where "channelId" in(:channelid) and "turnoverRange"!=\'>100M€\' and case when "turnoverRange"=\'10-100M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\')) as int)>10 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<=100) or ("turnover">10 and "turnover"<=100) then "turnoverRange"=\'10-100M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\')) as int)>10 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<=100) or ("turnover">10 and "turnover"<=100) end';
				}
				else if (channelFilter.turnOver == '>100M€') {
					query = 'select array_agg("channelId") as ch from "tblChannelDetail" where "channelId" in(:channelid) and case when "turnoverRange"=\'>100M€\' or cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)>100 or "turnover">100 then cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)>100 or "turnoverRange"=\'>100M€\' or "turnover">100 end';
				}

				await this.sequelizeInstance.query(query, {
					type: QueryTypes.SELECT,
					raw: true,
					replacements: {
						channelid: chlist,
						//channeltypeid:channelFilter.typeId
					}
				}).then((result) => {

					if (result[0].ch) {
						filterch = [];
						filterch.push(...result[0].ch);
					} else
						filterch = [];
				});
				result = result.filter(ch => { return filterch.includes(ch.channel_id); })
			}

			if (channelFilter.storeCount) {
				let store;

				if (channelFilter.storeCount == '5')
					store = 'select array_agg("channelId") as ch from "tblChannelDetail" where "channelId" in(:channelid) and ("physicalSales"<=5 or "physicalSales" is null)';
				else
					store = 'select array_agg("channelId") as ch from "tblChannelDetail" where "channelId" in(:channelid) and "physicalSales">:count';

				await this.sequelizeInstance.query(store, {
					type: QueryTypes.SELECT,
					raw: true,
					replacements: {
						channelid: chlist,
						count: +channelFilter.storeCount
					}
				}).then((result) => {
					if (result[0].ch) {

						filterch = [];
						filterch.push(...result[0].ch);
					} else
						filterch = [];

				});
			}

			if (channelFilter.brandId) {
				let brand = 'select array_agg("channelId") as ch from "tblChannelBrand" where "brandId" in(:brandid) and "channelId" in(:channelid)';
				await this.sequelizeInstance.query(brand, {
					type: QueryTypes.SELECT,
					raw: true,
					replacements: {
						channelid: chlist,
						brandid: channelFilter.brandId
					}
				}).then((result) => {
					if (result[0].ch) {
						filterch = [];
						filterch.push(...result[0].ch);
					} else
						filterch = [];

				});
				result = result.filter(ch => { return filterch.includes(ch.channel_id); })
			}
			if (channelFilter.productCategoryId) {
				await this.sequelizeInstance.query('select array_agg("channelId") as ch from "tblChannelProductCategorie" cpc where cpc."productCategorieId" in(:productcatid) and "channelId" in(:channelid) ', {
					type: QueryTypes.SELECT,
					raw: true,
					replacements: {
						channelid: chlist,
						productcatid: channelFilter.productCategoryId
					}
				}).then((result) => {
					if (result[0].ch) {
						filterch = [];
						filterch.push(...result[0].ch);
					} else
						filterch = [];

				});
				result = result.filter(ch => { return filterch.includes(ch.channel_id); })
			}
			if (channelFilter.retailerprofileId || channelFilter.resellerprofileId) {
				let query;
				let ids = channelFilter.retailerprofileId ? channelFilter.retailerprofileId : channelFilter.resellerprofileId;
				if (viewId == 8 || viewId == 9)
					query = 'select array_agg("channelId") as ch from "tblChannelCustomerProfile" as ccp where "profileTypeId" in(:retailprofileid) and "channelId" in(:channelid) and ccp."profileTypeId" in(select "profileTypeId" from "tblProfileTypeClassification" where ("channelTypeId" in(3,4) and ccp."channelId" in(select "channelId" from "tblChannel" where "channelId"=ccp."channelId" and "isOEM"=\'1\')) or '
						+ '("channelTypeId" in(3,5) and ccp."channelId" in(select "channelId" from "tblChannel" where "channelId"=ccp."channelId" and "isOEM"=\'2\')) or'
						+ '("channelTypeId" in(3,4,5) and ccp."channelId" in(select "channelId" from "tblChannel" where "channelId"=ccp."channelId" and "isOEM"=\'3\')))';
				if (viewId == 10 || viewId == 14)
					query = 'select array_agg("channelId") as ch from "tblChannel" where "profileTypeId" in(:retailprofileid) and "channelId" in(:channelid)';

				await this.sequelizeInstance.query(query, {
					type: QueryTypes.SELECT,
					raw: true,
					replacements: {
						channelid: (filterch.length > 0) ? filterch : chlist,
						retailprofileid: (channelFilter.retailerprofileId && channelFilter.resellerprofileId) ? channelFilter.resellerprofileId.concat(channelFilter.retailerprofileId) : ids

					}
				}).then((result) => {
					if (result[0].ch) {
						filterch = [];
						filterch.push(...result[0].ch);
					} else
						filterch = [];

				});
				result = result.filter(ch => { return filterch.includes(ch.channel_id); })
			}

			if (channelFilter.countryId) {
				await this.sequelizeInstance.query('select array_agg("channelId") as ch from "tblChannel" where "countryId" in(:countryid) and "channelId" in(:channelid)', {
					type: QueryTypes.SELECT,
					raw: true,
					replacements: {
						channelid: chlist,
						countryid: channelFilter.countryId
					}
				}).then((result) => {
					if (result[0].ch) {
						filterch = [];
						filterch.push(...result[0].ch);
					} else
						filterch = [];
				});
				result = result.filter(ch => { return filterch.includes(ch.channel_id); })
			}
			if (channelFilter.specialityId) {
				await this.sequelizeInstance.query('select array_agg(distinct "channelId") as ch from "tblContact" where "contactId" in(Select "contactId" from "tblContactInterestedProductCategory" where "productCategorieId" in(:specialityid)) and "channelId" in(:channelid)', {
					type: QueryTypes.SELECT,
					raw: true,
					replacements: {
						channelid: chlist,
						specialityid: channelFilter.specialityId
					}
				}).then((result) => {
					if (result[0].ch) {
						filterch = [];
						filterch.push(...result[0].ch);
					} else
						filterch = [];
				});
				result = result.filter(ch => { return filterch.includes(ch.channel_id); })
			}
			if (channelFilter.locationId) {
				let locationType = user.channelTypeId == 2 ? 'SELLING' : 'TARGET';

				let query = 'select array_agg("channelId") as ch  from "tblChannelLocation" where "locationType"=:locationType and "channelId" in(:channelid) and "deletedAt" is null and'
					+ '"regionCountryJCTId" in(select "regionCountryJCTId" from "tblRegionCountryJCT" where "countryId" in (:locationid)) '

				await this.sequelizeInstance.query(query, {
					type: QueryTypes.SELECT,
					raw: true,
					replacements: {
						channelid: chlist,
						locationType: locationType,
						locationid: channelFilter.locationId
					}
				}).then((result) => {
					if (result[0].ch) {
						filterch = [];
						filterch.push(...result[0].ch);
					} else
						filterch = [];
				});
				result = result.filter(ch => { return filterch.includes(ch.channel_id); })
			}

		}



		chlist = result.map(x => +x.channel_id);
		return chlist;

	}


	async getchannelTypes(type) {
		let list = [];
		if (type == 2) {
			//list.push({id:2,name:'VENDOR',status:false});
			list.push({ id: 3, name: 'DISTRIBUTOR', status: false });
			list.push({ id: 4, name: 'RETAILER', status: false });
			list.push({ id: 5, name: 'RESELLER', status: false });
		}
		else if (type == 3) {
			list.push({ id: 2, name: 'VENDOR', status: false });
			list.push({ id: 3, name: 'DISTRIBUTOR', status: false });
			list.push({ id: 4, name: 'RETAILER', status: false });
			list.push({ id: 5, name: 'RESELLER', status: false });
		}
		else if (type == 4) {
			list.push({ id: 2, name: 'VENDOR', status: false });
			list.push({ id: 3, name: 'DISTRIBUTOR', status: false });
			//list.push({id:4,name:'RETAILER',status:false});			
		}
			return list;

	}

	async getBrandFilter(chlist){		

	
	let brandIds;

	try{

		if(chlist.data.length>0) 
		{			

			let brandquery='select distinct on(tb."brandName") tb."brandName" as name,tb."id" as id from "tblBrand" tb '
			+'inner join "tblDocument" td on td."documentId"=tb."documentId" where id in (select "brandId" from "tblChannelBrand" where "channelId" in(:channelid)) order by tb."brandName" ';
				
			brandIds= await this.sequelizeInstance.query(brandquery, {
			type: QueryTypes.SELECT,
			raw: true,
			replacements: {
			channelid:chlist.data
			}
			})
	
		}
	}	
	catch(e)
	{
		console.log("brandFilter:",e);
	}
	
		return brandIds;
		
	}


	async getLocationFilter(chlist){		

		let locactionIds;
		
	
		try{
	
			if(chlist.data.length>0) 
			{						
				let lookupquery='select distinct on("country")"country" as name,"countryId" as id, "priority" from "tblCountry" where "countryId" in '
					+'(Select pc."countryId" from "tblContactLocation" cpc inner join "tblRegionCountryJCT" pc on pc."regionCountryJCTId" = cpc."regionCountryJCTId" '
					+ 'inner join "tblChannel" ch on ch."channelId" in(:channelid) where cpc."channelId" in(:channelid) and cpc."locationType"=\'SELLING\') order by "country" ';
				
				locactionIds= await this.sequelizeInstance.query(lookupquery, {
				type: QueryTypes.SELECT,
				raw: true,
				replacements: {
				channelid:chlist.data
				}
				})	
				
			}
	
		}	
		catch(e)
		{
			console.log("locationFilter:",e);
		}
		
			return locactionIds;
			
	}

	async getSpecialFilter(chlist){		

		let specialIds;
		
		try{
	
			if(chlist.data.length>0) 
			{			
			
				let intcatquery='select distinct on("productCategorie")"productCategorie" as name,"productCategorieId" as id from "tblProductCategories" where "productCategorieId" in '
				+'(Select cpc."productCategorieId" from "tblContactInterestedProductCategory" cpc where cpc."contactId" in(select "contactId" from "tblContact" where "channelId" in(:channelid) and "deletedAt" is null)) order by "productCategorie" ';
								
				specialIds= await this.sequelizeInstance.query(intcatquery, {
				type: QueryTypes.SELECT,
				raw: true,
				replacements: {
				channelid:chlist.data
				}
				})	
				
			}
	
		}	
		catch(e)
		{
			console.log("specialFilter:",e);
		}
		
		return specialIds;
			
	}

	async getProductFilter(chlist, channelId) {

		let productIds;

		try {

			if (chlist.data.length > 0) {

				let productcatquery = 'select pg."productGroupId" as groupId,pg."productGroup" as groupname,pc."productCategorieId" as childId,pc."productCategorie" as childname,cast(false as boolean) as status from "tblProductGroup" as pg inner join'
					+ ' "tblProductCategories" as pc on pg."productGroupId"=pc."productGroupId" and pc."productCategorieId" in '
					+ '(select cpc."productCategorieId" from "tblChannelProductCategorie" as cpc where "channelId"=:channelId) '
					+ 'where pg."deletedAt" is null and pg."productGroup"!=\'\' ';

				// let productcatquery='Select distinct on(pc."productCategorie")pc."productCategorie" as name,pc."productCategorieId" as id from "tblChannelProductCategorie" cpc '
				// +'inner join "tblProductCategories" pc on pc."productCategorieId" = cpc."productCategorieId" '
				// +'where cpc."channelId" in(:channelid) order by pc."productCategorie"';

				productIds = await this.sequelizeInstance.query(productcatquery, {
					type: QueryTypes.SELECT,
					raw: true,
					replacements: {
						//channelid:chlist.data
						channelId: channelId
					}
				})

			}

		}
		catch (e) {
			console.log("productFilter:", e);
		}

		return productIds;

	}

	async getInterestedProduct(chlist, contactId) {

		let productIds;
		
		try{
	
			if(chlist.data.length>0) 
			{			
			
				let productcatquery = 'select pg."productGroupId" as groupId,pg."productGroup" as groupname,pc."productCategorieId" as childId,pc."productCategorie" as childname,cast(false as boolean) as status from "tblProductGroup" as pg inner join'
					+ ' "tblProductCategories" as pc on pg."productGroupId"=pc."productGroupId" and pc."productCategorieId" in '
					+ '(select cpc."productCategorieId" from "tblInterestedProductCategory" as cpc where "contactId"=:contactId) '
					+ 'where pg."deletedAt" is null and pg."productGroup"!=\'\' ';

			// let productcatquery='Select distinct on(pc."productCategorie")pc."productCategorie" as name,pc."productCategorieId" as id from "tblChannelProductCategorie" cpc '
			// +'inner join "tblProductCategories" pc on pc."productCategorieId" = cpc."productCategorieId" '
			// +'where cpc."channelId" in(:channelid) order by pc."productCategorie"';
								
			productIds= await this.sequelizeInstance.query(productcatquery, {
				type: QueryTypes.SELECT,
				raw: true,
				replacements: {
					//channelid:chlist.data
					contactId: contactId
				}
				})
				
			}
	
		}	
		catch(e)
		{
			console.log("productFilter:",e);
		}
		
		return productIds;
			
	}

	async getCountryFilter(chlist){		

		let countryIds;
		
	
		try{
	
			if(chlist.data.length>0) 
			{			
			
			let countryquery='select distinct on(tc."country")tc."country" as name, tc."priority", ch."countryId" as id from "tblChannel" ch inner join "tblCountry" tc on tc."countryId"=ch."countryId" '
			+'where ch."channelId" in(:channelid) order by tc."country" ';
								
			countryIds= await this.sequelizeInstance.query(countryquery, {
				type: QueryTypes.SELECT,
				raw: true,
				replacements: {
				channelid:chlist.data
				}
				})
				
			}
	
		}	
		catch(e)
		{
			console.log("countryFilter:",e);
		}
		
		return countryIds;
			
	}

	async getRetailFilter(chlist){		

		let retailIds;
		
		try{
	
			if(chlist.data.length>0) 
			{			
			
				let retailcustquery = 'Select distinct on(pt."profileType")pt."profileType" as name,pt."profileTypeId" as id,(select "channelTypeId" from "tblProfileTypeClassification" as pc where pc."profileTypeId"=pt."profileTypeId") from "tblProfileType" pt where "profileTypeId" in '
					+ '(select "profileTypeId" from "tblChannelCustomerProfile" as ccp where ccp."channelId" in(:channelid) and ccp."profileTypeId" in(select "profileTypeId" from "tblProfileTypeClassification" where ("channelTypeId" in(3,4) and ccp."channelId" in(select "channelId" from "tblChannel" where "channelId"=ccp."channelId" and "isOEM"=\'1\')) or '
					+ '("channelTypeId" in(3,5) and ccp."channelId" in(select "channelId" from "tblChannel" where "channelId"=ccp."channelId" and "isOEM"=\'2\')) or'
					+ '("channelTypeId" in(3,4,5) and ccp."channelId" in(select "channelId" from "tblChannel" where "channelId"=ccp."channelId" and "isOEM"=\'3\')) )) '

								
			retailIds= await this.sequelizeInstance.query(retailcustquery, {
				type: QueryTypes.SELECT,
				raw: true,
				replacements: {
				channelid:chlist.data
				}
				})
				
			}

			if (chlist.data.length > 0 && retailIds.length == 0) {
				let retailcustquery = 'Select distinct on(pt."profileType")pt."profileType" as name,pt."profileTypeId" as id,(select "channelTypeId" from "tblProfileTypeClassification" as pc where pc."profileTypeId"=pt."profileTypeId") from "tblProfileType" pt where "profileTypeId" in '
					+ '(select "profileTypeId" from "tblChannel" as ccp where ccp."channelId" in(:channelid)) ';

				retailIds = await this.sequelizeInstance.query(retailcustquery, {
					type: QueryTypes.SELECT,
					raw: true,
					replacements: {
						channelid: chlist.data
					}
				})
			}

		}	
		catch(e)
		{
			console.log("retailFilter:",e);
		}
		
		return retailIds;
			
	}

	async getTargetFilter(user, chlist) {

		let targetIds;
		//console.log("usertar:", user);
		let locationType = user.channelTypeId == 2 ? 'TARGET' : 'SELLING';

		try {

			if (chlist.data.length > 0) {
				let target = 'select distinct on("country")"country" as name, "priority", "countryId" as id  from "tblCountry" where "countryId" in ' //(select "countryId" from "tblRegionCountryJCT"  '
					//+ 'where "regionCountryJCTId" in(select "regionCountryJCTId" from "tblChannelLocation" where "locationType"=:locationType and "channelId"=:sourceId and "deletedAt" is null)) '
					+ '(select ch."countryId" from "tblChannel" ch inner join "tblCountry" tc on tc."countryId"=ch."countryId" where ch."channelId" in(:channelid) order by tc."country")'

				targetIds = await this.sequelizeInstance.query(target, {
					type: QueryTypes.SELECT,
					raw: true,
					replacements: {
						sourceId: user.channelId,
						locationType: locationType,
						channelid: chlist.data
					}
				})
			}
		}
		catch (e) {
			console.log("targetFilter:", e);
		}

		return targetIds;

	}

	//End of Channel Filter Services

	//Contact Filter Services 

	async contactFilterBase(data)
	{
		let user = data;//this.context.get('user');
		let conlist;		
		let typeid=user.channelTypeId;
			
		let channeltype=[];
		let aggregation=null;let channelquery;
		if(typeid==2)
		channeltype=[3,4];
		else if(typeid==3)
		channeltype=[2,4];
		else if(typeid==4)
		channeltype=[2,3];
		//let order='desc';
		// if(viewId==32)
		// {
			channelquery='select "contactId" from "tblContact" where "deletedAt" is null and "isActive"=\'1\' and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelType)) ';
		//}

		
		let result= await this.sequelizeInstance.query(channelquery, {
		type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		//search: '%' + pagination.search + '%',
		channelType:channeltype
		}
		}).then(x=>{						
			return  _.uniqBy(x,'contactId');
		})

	if(result.length>0) 	
		conlist = result.map(x => +x.contactId);			
	
	return conlist;
	}

	//contact filter for listing
		//Contact Filter Services

		async contactFilterBase1(userData, data, order)
		{
			let user = userData;//this.context.get('user');
			let conlist;		
			let typeid=user.channelTypeId;
				
			let channeltype=[];
			let aggregation=null;let channelquery;
			if(typeid==2)
			channeltype=[3,4];
			else if(typeid==3)
			channeltype=[2,4];
			else if(typeid==4)
			channeltype=[2,3];
			//let order='desc';
			// if(viewId==32)
			// {
				channelquery='select "contactId" from "tblContact" where "deletedAt" is null and "isActive"=\'1\' and "channelId" in(select "channelId" from "tblChannel" where "channelId" in(:channelId)) order by "firstName" ' + order;
			//}
	
			
			let result= await this.sequelizeInstance.query(channelquery, {
			type: QueryTypes.SELECT,
			raw: true,
			replacements: {
			//search: '%' + pagination.search + '%',
			channelId:data
			}
			}).then(x=>{						
				return  _.uniqBy(x,'contactId');
			})
	
		if(result.length>0) 	
			conlist = result.map(x => +x.contactId);			
		
		return conlist;
		}
	

	async getContactSpecialFilter(chlist){		

		let specialIds;		
		try{	
			if(chlist.data.length>0) 
			{						
				let intcatquery='select distinct on("productCategorie")"productCategorie" as name,"productCategorieId" as id from "tblProductCategories" where "productCategorieId" in '
				+'(Select cpc."productCategorieId" from "tblContactInterestedProductCategory" cpc where cpc."contactId" in(:contactid)) order by "productCategorie" ';
								
				specialIds= await this.sequelizeInstance.query(intcatquery, {
				type: QueryTypes.SELECT,
				raw: true,
				replacements: {
				contactid:chlist.data
				}
				})					
			}	
		}	
		catch(e)
		{
			console.log("specialFilter:",e);
		}		
		return specialIds;			
	}

	async getContactCountryFilter(chlist){		

		let countryIds;		
	
		try{
	
			if(chlist.data.length>0) 
			{			
			
			let countryquery='select distinct on(tc."country")tc."country" as name, tc."priority", ch."countryId" as id from "tblContact" ch inner join "tblCountry" tc on tc."countryId"=ch."countryId" '
			+'where ch."contactId" in(:contactid) order by tc."country" ';
								
			countryIds= await this.sequelizeInstance.query(countryquery, {
				type: QueryTypes.SELECT,
				raw: true,
				replacements: {
				contactid:chlist.data
				}
				})
				
			}
	
		}	
		catch(e)
		{
			console.log("countryFilter:",e);
		}
		
		return countryIds;
			
	}
	
	async conNameListFilter(chlist){		

		let nameList;		
		try{	
			if(chlist.length>0) 
			{						
				let listquery='select "contactId" as id,concat(ct."firstName",\' \',ct."lastName") as name from "tblContact" as ct where "contactId" in(:contactid) and "deletedAt" is null and "isActive"=\'1\' ';
								
				nameList= await this.sequelizeInstance.query(listquery, {
				type: QueryTypes.SELECT,
				raw: true,
				replacements: {
				contactid:chlist
				}
				})					
			}	
		}	
		catch(e)
		{
			console.log("specialFilter:",e);
		}		
		return nameList;			
	}
	//End of Contact Filter Services


	async getFoCompanyList(viewId,planId,data){
			let contactId=data.contactId;
		let businessType = data.businessType;
			let channelId=data.channelId;
			let typeid=data.channelTypeId;let channeltype=[];let channelquery;
			// let userId=user.userId;
			// let filterch=[];
			// let aggregation={};
		if (typeid == 2 && businessType == 1)
			channeltype = [3, 4];
		else if (typeid == 2 && businessType == 2)
			channeltype = [3, 5];
		else if (typeid == 2 && businessType == 3)
			channeltype = [3, 4, 5];
		else if (typeid == 3 || typeid == 4 || typeid == 5)
			channeltype=[2];

			//confirm
			if(viewId==1){				
			channelquery='select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=2 and case when "requested_to"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" '
			 +' where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) '
			 +' when "requested_by"='+contactId+' then "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6))  end and "deletedat" is null ';
			}
			//outbound
			else if(viewId==2){
			channelquery='select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT"  where "connectionStatusTypeId" in(7,11) and case when "requested_to"='+contactId+' and "connectionStatusTypeId"=11 then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) when "requested_by"='+contactId+' and "connectionStatusTypeId"=7 then "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6))  end and "deletedat" is null ';	
			}
			//inbound
			else if(viewId==3){
			channelquery='select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(7,10) '
			+' and case when "requested_to"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15  and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) end and "deletedat" is null  '
			}
			//requestintro
			else if(viewId==4){
			channelquery='select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=10 and "partnerChannelId" not in(select "partnerChannelId" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "deletedat" is null and "requested_by"='+contactId+' and "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') union all select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id  from "tblChannelJCT" where "connectionStatusTypeId"=11 and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "deletedat" is null and "requested_to"='+contactId+' and "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\')  ';
			}
			//savelater
			else if(viewId==5){
			channelquery='select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=11 and case when "requested_by"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6)) end and "deletedat" is null' ;
			}
			//suggestion
			else if(viewId==6){
			channelquery='select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"='+contactId+' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)' ;
			}
			else if(viewId==30){	
			if(planId==1 || planId==7)	
			{
				if(typeid==2)
				{
					channelquery='select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=2 and case when "requested_to"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" '
			 +' where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) '
			 +' when "requested_by"='+contactId+' then "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6))  end and "deletedat" is null '
			 +' union all'
			 +' select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT"  where "connectionStatusTypeId" in(7,11) and case when "requested_to"='+contactId+' and "connectionStatusTypeId"=11 then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) when "requested_by"='+contactId+' and "connectionStatusTypeId"=7 then "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\'and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6))  end and "deletedat" is null '
			 +' union all'
			 +' select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(7,10) '
			 +' and case when "requested_to"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15  and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) end and "deletedat" is null  '
			 +' union all'
			 +' select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=10 and "partnerChannelId" not in(select "partnerChannelId" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "deletedat" is null and "requested_by"='+contactId+' and "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') union all select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id  from "tblChannelJCT" where "connectionStatusTypeId"=11 and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "deletedat" is null and "requested_to"='+contactId+' and "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\')  '
			 +' union all'
			 +' select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=11 and case when "requested_by"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6)) end and "deletedat" is null' 
			 +' union all'
			 +' select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"='+contactId+' and "isConfidential"=\'1\' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)' ;
			 
				}
				else
				{
				let nonsuggcompany;
				if(typeid==3)		
					nonsuggcompany='select tc."channelId" as channel_id from "tblChannel" as tc inner join "tblContact" as con on con."channelId"=tc."channelId" and con."isActive"=\'1\' and con."deletedAt" is null and con."contactId" not in(select case when "requested_to"='+contactId+' then "requested_by" when "requested_by"='+contactId+' then "requested_to" end from "tblChannelJCT" as jct where ("requested_by"='+contactId+' or "requested_to"='+contactId+') and "connectionStatusTypeId" in(5,6,2,7,10,11,1) ) where "signUpStatusId"=15 and  "channelTypeId" not in(5,6) and tc."isActive"=\'1\' and tc."deletedAt" is null and "channelTypeId" in(2) and tc."channelId" not in(select case when "dismissedBy"='+channelId+' then "dismissedChId" when "dismissedChId"='+channelId+' then "dismissedBy" end from "tblchanneldismissreason" where "dismissedBy"='+channelId+' or ("dismissedChId"='+channelId+' and "dismissedBy"=tc."channelId")) '; 
				else if(typeid==4)
					nonsuggcompany='select tc."channelId" as channel_id from "tblChannel" as tc inner join "tblContact" as con on con."channelId"=tc."channelId" and con."isActive"=\'1\' and con."deletedAt" is null and con."contactId" not in(select case when "requested_to"='+contactId+' then "requested_by" when "requested_by"='+contactId+' then "requested_to" end from "tblChannelJCT" as jct where ("requested_by"='+contactId+' or "requested_to"='+contactId+') and "connectionStatusTypeId" in(5,6,2,7,10,11,1) ) where "signUpStatusId"=15 and  "channelTypeId" not in(5,6) and tc."isActive"=\'1\' and tc."deletedAt" is null and "channelTypeId" in(2,3) and tc."channelId" not in(select case when "dismissedBy"='+channelId+' then "dismissedChId" when "dismissedChId"='+channelId+' then "dismissedBy" end from "tblchanneldismissreason" where "dismissedBy"='+channelId+' or ("dismissedChId"='+channelId+' and "dismissedBy"=tc."channelId")) '; 
				 
					channelquery='select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=2 and case when "requested_to"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" '
			 +' where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) '
			 +' when "requested_by"='+contactId+' then "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6))  end and "deletedat" is null '
			 +' union all'
			 +' select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT"  where "connectionStatusTypeId" in(7,11) and case when "requested_to"='+contactId+' and "connectionStatusTypeId"=11 then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) when "requested_by"='+contactId+' and "connectionStatusTypeId"=7 then "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\'and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6))  end and "deletedat" is null '
			 +' union all'
			 +' select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(7,10) '
			 +' and case when "requested_to"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15  and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) end and "deletedat" is null  '
			 +' union all'
			 +' select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=10 and "partnerChannelId" not in(select "partnerChannelId" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "deletedat" is null and "requested_by"='+contactId+' and "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') union all select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id  from "tblChannelJCT" where "connectionStatusTypeId"=11 and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "deletedat" is null and "requested_to"='+contactId+' and "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\')  '
			 +' union all'
			 +' select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=11 and case when "requested_by"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6)) end and "deletedat" is null' 
			 +' union all'
			 +' select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"='+contactId+' and "isConfidential"=\'1\' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)' 
			 +' union all '+nonsuggcompany ;
				}
			// +' union all'
			// +' select tc."channelId" from "tblChannel" as tc inner join "tblContact" as con on con."channelId"=tc."channelId" and con."isActive"=\'1\' and con."deletedAt" is null and con."contactId" not in(select case when "requested_to"='+contactId+' then "requested_by" when "requested_by"='+contactId+' then "requested_to" end from "tblChannelJCT" as jct where ("requested_by"='+contactId+' or "requested_to"='+contactId+') and "connectionStatusTypeId" in(5,6,2,7,10,11,1)) where "signUpStatusId"=15 and  "channelTypeId" not in(5,6) and tc."isActive"=\'1\' and tc."deletedAt" is null and "channelTypeId" in(:channelTypeId) ';
			//  +' select "channelId" as channel_id from "tblChannel" where "signUpStatusId"=15 and  "channelTypeId" not in(5,6) and "isActive"=\'1\' and "deletedAt" is null and "channelTypeId" in(:channelTypeId) and "channelId" not in(select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end from "tblChannelJCT" where ("requested_by"='+contactId+' or "requested_by"='+contactId+') and "connectionStatusTypeId" in(5,6,2,7,10,11,1) and "deletedat" is null) ' 
			//  +' and "channelId" not in(select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end from "tblChannelJCT" where ("requested_by"='+contactId+' or "requested_by"='+contactId+') and "connectionStatusTypeId" in(5,6,2,7,10,11,1) and "deletedat" is null) ';
			}
			 else
			 	channelquery='select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=2 and case when "requested_to"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" '
			 +' where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) '
			 +' when "requested_by"='+contactId+' then "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6))  end and "deletedat" is null '
			 +' union all'
			 +' select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT"  where "connectionStatusTypeId" in(7,11) and case when "requested_to"='+contactId+' and "connectionStatusTypeId"=11 then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) when "requested_by"='+contactId+' and "connectionStatusTypeId"=7 then "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\'and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6))  end and "deletedat" is null '
			 +' union all'
			 +' select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(7,10) '
			 +' and case when "requested_to"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15  and "isActive"=\'1\') and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_by" not in(select "requested_by" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) end and "deletedat" is null  '
			 +' union all'
			 +' select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=10 and "partnerChannelId" not in(select "partnerChannelId" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "deletedat" is null and "requested_by"='+contactId+' and "partnerChannelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') union all select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id  from "tblChannelJCT" where "connectionStatusTypeId"=11 and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "deletedat" is null and "requested_to"='+contactId+' and "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\')  '
			 +' union all'
			 +' select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end as channel_id from "tblChannelJCT" where "connectionStatusTypeId"=11 and case when "requested_by"='+contactId+' then "channelId" in(select "channelId" from "tblChannel" where "deletedAt" is null and "signUpStatusId" = 15 and "isActive"=\'1\') and "requested_to" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null) and "requested_to" not in(select "requested_to" from "tblChannelJCT" where "requested_by"='+contactId+' and "connectionStatusTypeId" in(5,6)) end and "deletedat" is null' 
			 +' union all'
			 +' select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"='+contactId+' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:channelTypeId) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)'
			 +' union all'
			 +' select tc."channelId" as channel_id from "tblChannel" as tc inner join "tblContact" as con on con."channelId"=tc."channelId" and con."isActive"=\'1\' and con."deletedAt" is null and con."contactId" not in(select case when "requested_to"='+contactId+' then "requested_by" when "requested_by"='+contactId+' then "requested_to" end from "tblChannelJCT" as jct where ("requested_by"='+contactId+' or "requested_to"='+contactId+') and "connectionStatusTypeId" in(5,6,2,7,10,11,1) ) where "signUpStatusId"=15 and  "channelTypeId" not in(5,6) and tc."isActive"=\'1\' and tc."deletedAt" is null and "channelTypeId" in(:channelTypeId) and tc."channelId" not in(select case when "dismissedBy"='+channelId+' then "dismissedChId" when "dismissedChId"='+channelId+' then "dismissedBy" end from "tblchanneldismissreason" where "dismissedBy"='+channelId+' or ("dismissedChId"='+channelId+' and "dismissedBy"=tc."channelId")) '; 
			 //+' and "channelId" not in(select case when "channelId"='+channelId+' then "partnerChannelId" when "partnerChannelId"='+channelId+' then "channelId" end from "tblChannelJCT" where ("requested_by"='+contactId+' or "requested_by"='+contactId+') and "connectionStatusTypeId" in(5,6,2,7,10,11,1) and "deletedat" is null) ';
			}
			//vendor suggestion
			else if(viewId==8)
			{
				if (planId == 1 && typeid == 2)
				channelquery='select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"='+contactId+' and "isConfidential"=\'1\' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(2) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)' ;
			else
				channelquery='select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"='+contactId+' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(2) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)' ;	
			}
			//distributor suggestion
			else if(viewId==9)
			{
				if (planId == 1 && typeid == 2)
				channelquery='select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"='+contactId+' and "isConfidential"=\'1\' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(3) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)' ;
			else
				channelquery='select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"='+contactId+' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(3) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)' ;	
			}
			//retailer suggestion
			else if(viewId==10)
			{
				if (planId == 1 && typeid == 2)
				channelquery='select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"='+contactId+' and "isConfidential"=\'1\' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(4) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)' ;
			else
				channelquery='select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"='+contactId+' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(4) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)' ;
			}

			//reseller suggestion
			else if(viewId==14)
			{
			if(planId==1)
				channelquery='select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"='+contactId+' and "isConfidential"=\'1\' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(5) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)' ;
			else
				channelquery='select "channelId" as channel_id from "tblChannelJCT" where "connectionStatusTypeId" in(1,16) and "requested_to"='+contactId+' and "deletedat" is null and "channelId" not in(select "channelId" from "tblChannelJCT" where "requested_to"='+contactId+' and "connectionStatusTypeId" in(5,6)) and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(5) and "isActive"=\'1\' and "signUpStatusId"=15 and "deletedAt" is null and "planId" is not null) and "requested_by" in(select "contactId" from "tblContact" where "isActive"=\'1\' and "deletedAt" is null)' ;
			}

			else if(viewId==15){
				channelquery='select "oldChannelId" as channel_id from "tblOldChannel" where "channelTypeId" in(:channelTypeId) and "isActive"=\'1\' and "deletedAt" is null'; // "updatedAt" '+data.order+' ' ;
			}

			else if(viewId==50)
			{
				channelquery='select "channelid" as channel_id from "tblproductwishlist" where "userid"=:userId and "deletedat" is null';
			}

			let result=await this.sequelizeInstance.query(channelquery, {
			type: QueryTypes.SELECT,
			raw: true,
			replacements: {			
			channelTypeId:channeltype,
			userId: data.userId
			}
			}).then(x=>{				
				return x;
			});
			if(result.length>0){
			let chlist=result.map(x => x.channel_id);
			//console.log("cahn:",chlist);
			let query="";
			
			query='select "companyName" as name, "channelId" as id from "tblChannel" where "channelId" in(:channelIds) and "companyName" is not null and "companyName" != \'\' ';

			if(viewId == 15)
			query = 'select "companyName" as name, "oldChannelId" as id from "tblOldChannel" where "oldChannelId" in(:channelIds) and "companyName" is not null and "companyName" != \'\' ';


			
			result= await this.sequelizeInstance.query(query, {
				type: QueryTypes.SELECT,
				raw: true,
				replacements: {				
				channelIds:chlist
				},
				});
				}
				return { results:result?result:[] }	
	}

	async getChannel(channelId: number, user: any): Promise<any> {

		let channel = await this.channelRepository.scope('channelDetail').findOne<Channel>({
			where:
			{
				channelId: +channelId,
				deletedAt: null,
				signUpStatusId: appconstant.CHANNEL_APPROVED,
				isActive: '1',
				channelTypeId: {
					[Op.in]: [appconstant.CHANNEL_TYPE_VENDOR, appconstant.CHANNEL_TYPE_DISTRIBUTOR, appconstant.CHANNEL_TYPE_RETAILER, appconstant.CHANNEL_TYPE_RESELLER]
				}
			},
			include: [
				{
				model: ChannelB2B,
				as:'channelB2B',
				attributes: ['channelB2BId', 'channelId', 'channelB2B'],
				where: { deletedAt: null },
				required: false,
			},
				{ model: this.signUpStatusRepository },
				{ model: this.channelTypeRepository },
				{ model: Country, as: 'country', where: { deletedAt: null } },
				{ model: Address, as: 'regAddress'},
				{ model: ProfileType.scope('channelDetail'), as: 'profileType', attributes: ['profileType'], where: { deletedAt: null }, required: false },
				{
					model: this.productGroupRepository,
					as: 'speciality',
					attributes: ['productGroupId', 'productGroup'],
					required: false,
					where: { isActive: '1' },
					through: { attributes: [], where: { deletedAt: null } }
				},
				{
					model: this.productCategoryRepository,
					as: 'productCategory',
					attributes: ['productCategorieId', 'productCategorie'],
					required: false,
					where: { isActive: '1' },
					order:'productCategorie',
					through: { attributes: [], where: { deletedAt: null } }
				},
				{
					model: ChannelAward,
					attributes: ['awardDetail', 'issuedBy', 'validUpto', 'isLifeTime'],
					where: { deletedAt: null },
					required: false
				}
			]
		});
		if (!channel) return false;

		if (+user.channelId != +channelId && (+channel.channelTypeId == +user.channelTypeId && +user.channelTypeId != appconstant.CHANNEL_TYPE_DISTRIBUTOR)) {
			throw new AuthException('Not allowed');
		}

		if (!channel.channelJCT) {
			await new Utility().lazyload(channel, 'channelJCT', {
				where: {
					channelId: user.channelId,
					partnerChannelId: channelId,
					deletedat: null
				},
				required: false,
				include: [{
					model: this.contactJCTRepository,
					as: 'contactJCT',
					attributes: ['partnerContactId'],
					where: { contactId: user.contactId },
					required: false,
				}]
			});
		}

		if (!channel.channelJCTPartner) {
			await new Utility().lazyload(channel, 'channelJCTPartner', {
				where: {
					channelId: channelId,
					partnerChannelId: user.channelId,
					deletedat: null,
					subscription_queued: '0'
				},
				required: false,
				include: [{
					model: this.contactJCTRepository,
					as: 'contactJCT',
					attributes: ['partnerContactId'],
					where: { contactId: user.contactId },
					required: false,
				}]
			});
		}

		// check share preference
		if (!channel.shareType) {
			await new Utility().lazyload(channel, 'shareType', {
				through: { attributes: [], where: { deletedat: null, sharetypeid: appconstant.shareTypeId[+user.channelTypeId] } },
				required: false,
				attributes: ['sharetypeid', 'sharetype']
			});
		}

		if (+user.channelId != +channelId && (channel.shareType.length === 0) &&
			(channel.channelJCTPartner && channel.channelJCTPartner[0] && [2, 7].includes(+channel.channelJCTPartner[0].connectionStatusTypeId)) &&
			(channel.channelJCT && channel.channelJCT[0] && [2, 7].includes(+channel.channelJCT[0].connectionStatusTypeId))
		) {
			throw new AuthException('Not allowed');
		}

		try {

			if (!channel.channelDetail) {
				await new Utility().lazyload(channel, 'channelDetail', {
					where: { deletedAt: null },
					required: false,
					attributes: ['channelLogoId', 'channelIconId', 'webSiteUrl', 'estYear', 'regYear', 'detailDesc', 'expectations', 'companyDesc', 'empCntRange', 'empCnt', 'empCntDSale', 'empCntISale', 'empCntDMar', 'empCntIMar', 'turnoverRange', 'turnover', 'revD', 'revI', 'deletedAt', 'turnoverPhyStore', 'turnoverOnStore', 'startup', 'storeCount', 'maturitylevel', 'keyretailers', 'keyresellers', 'keydistributors', 'no_of_retailers', 'no_of_resellers', 'no_of_skus', 'no_of_distributors', 'no_of_vendors', 'introvideoId', 'channelBannerId', 'phone1', 'mobileNo', 'bannerURL'],
					include: [{
						model: Media,
						as: 'channelLogo',
						attributes: ['documentUrl', 'documentPath'],
						where: { deletedAt: null },
						required: false
					},
					{
						model: Media,
						as: 'channelIcon',
						where: { deletedAt: null },
						attributes: ['documentUrl'],
						required: false
					},
					{
						model: Media,
						as: 'introvideo',
						where: { deletedAt: null },
						// attributes: ['documentUrl'],
						required: false
					},
					{
						model: Media,
						as: 'channelBanner',
						where: { deletedAt: null },
						// attributes: ['documentUrl'],
						required: false
					}
					],
				});
			}

			if (!channel.serviceList) {
				await new Utility().lazyload(channel, 'serviceList', {
					through: { attributes: [], where: { deletedAt: null } },
					required: false,
					attributes: ['serviceListId', 'serviceList']
				});
			}

			if (!channel.channelLocation) {
				await new Utility().lazyload(channel, 'channelLocation', {
					where: { deletedAt: null },
					required: false,
					attributes: ['locationType'],
					include: [{
						model: RegionCountryJCT,
						where: { deletedAt: null },
						required: false,
						include: [
							{
								model: Country,
								where: { deletedAt: null },
								attributes: ['countryId', 'country', 'priority'],
								required: false
							}
						],
					}]
				});
			}

			if (!channel.channelBrand) {
				await new Utility().lazyload(channel, 'channelBrand', {
					where: { deletedAt: null },
					required: false,
					include: [{
						model: Brand.scope('channelDetail'),
						as: 'brand',
						where: { deletedAt: null, isActive: '1' },
						required: false,
						include: [{
							model: Media,
							as: 'document',
							where: { deletedAt: null },
							required: false
						}]
					}]
				});
			}
			if (!channel.channelSocial) {
				await new Utility().lazyload(channel, 'channelSocial', {
					where: { deletedAt: null },
					required: false,
					include: [{
						model: SocialSite.scope('channelDetail'),
						where: { deletedAt: null },
						attributes: ['socialSite'],
						required: false
					}]
				});
			}

			if (!channel.channelCertificate) {
				await new Utility().lazyload(channel, 'channelCertificate', {
					where: { deletedAt: null },
					attributes: ['cerificateTypeId', 'cerificateName', 'cerificateNumber', 'cerificateDesc'],
					required: false
				});
			}

			if (!channel.channelSubscription) {
				await new Utility().lazyload(channel, 'channelSubscription', {
					where: { isActive: '1' },
					required: false,
					include: [{
						model: this.subscriptionRepository.scope('channelDetail'),
						as: 'subscription',
						where: { deletedAt: null, isActive: '1' },
						required: false,
						include: [{
							model: SubscriptionPlan.scope('channelDetail'),
							as: 'subscriptionPlan',
							where: { deletedAt: null, isActive: '1' },
							required: false
						}],
					}]
				});
			}

			if (!channel.channelDocument) {
				await new Utility().lazyload(channel, 'channelDocument', {
					attributes: ['channelDocumentId', 'documentId', 'name', 'description'],
					where: { deletedAt: null },
					required: false,
					include: [{
						attributes: ['documentId', 'documentPath', 'documentUrl'],
						where: { deletedAt: null, documentTypeId:2 },
						model: this.mediaRepository,
						required: true,
					}]
				});
			}

			channel.toJSON();

			channel['channelKeyRetailer'] = await this.channelKeyPartnerService.getKeyRetailers(+channel.channelId, appconstant.CHANNEL_TYPE_RETAILER);
			channel['channelKeyReseller'] = await this.channelKeyPartnerService.getKeyRetailers(+channel.channelId, appconstant.CHANNEL_TYPE_RESELLER);
			channel['channelKeyDistributor'] = await this.channelKeyPartnerService.getKeyRetailers(+channel.channelId, appconstant.CHANNEL_TYPE_DISTRIBUTOR);

			let custquery = 'select tpt."profileType", tptc."channelTypeId" from "tblChannelCustomerProfile" tcp inner join "tblProfileTypeClassification" tptc on tcp."profileTypeClassificationId"=tptc."profileTypeClassificationId" inner join "tblProfileType" tpt on tpt."profileTypeId"=tptc."profileTypeId" where tcp."channelId"=:channelId and tcp."deletedAt" is null'

			channel['customerProfile'] = await await this.sequelizeInstance.query(custquery, {
				type: QueryTypes.SELECT,
				raw: true,
				replacements: {
					channelId: +channel.channelId
				}
				});

			if (channel.channelTypeId == appconstant.CHANNEL_TYPE_DISTRIBUTOR || channel.channelTypeId == appconstant.CHANNEL_TYPE_RETAILER) {
				channel['sampleOrderCount'] = await this.getSampleOrderCount(channelId);
				channel['sampleEvaluationCount'] = await this.getSampleEvaluationCount(channelId);
			}

			channel['productCount'] = await this.productRepository.count({
				where: {
					channelId: channelId,
					isactive: '1',
					deletedat: null
				}
			});

			channel['certificateType'] = await this.channelCertificateTypeRepository.findAll({
				where: { deletedAt: null }
			});

			let query='select "productCategorieId", "productCategorie" from "tblProductCategories" where "productCategorieId" in(select "productCategorieId" from "tblInterestedProductCategory" where "channelId"=:channelId and "deletedAt" is null) order by "productCategorie"';
			
			let userProductCat= await this.sequelizeInstance.query(query, {
				type: QueryTypes.SELECT,
				raw: true,
				replacements: {
					channelId: channelId
				}
				});
					let companyImages= await this.companyMediaImagesRepository.findAll<CompanyMediaImages>({
						where: {
							channelId: +channel.channelId,
							deletedAt: null
						},
						include: [{
							model: Media,
							where: { 
								documentTypeId:1,
								deletedAt: null
							},
							required: true
						}],
						order: [['companyMediaId', 'DESC']]
					});

			//wishlist
			let fav_query = 'select "channelgroupid" from "tblproductwishlist" where "userid"=:userId and "channelid"=:channelId and "deletedat" is null limit 1';
			let favorite = await this.sequelizeInstance.query(fav_query, {
				type: QueryTypes.SELECT,
				raw: true,
				replacements: {
					channelId: channelId,
					userId: user.userId
				}
			}).then(res => {
				return res[0]?.channelgroupid;
			});

			let channelTransformed: any = await this.channelHubDetailOutBound.channelDetail(channel, user, userProductCat, companyImages);
			//channelTransformed['matchingChannels'] = await this.matchingChannels(channelTransformed);
			//set profile viewer counts
			// this.sequelizeInstance.query('select * from "dashboard"(' + user.userId + ',' + channel.channelId + ','  + channel.channelId +',1)', {
			// 	type: QueryTypes.SELECT,
			// 	raw: true
			// });
			

			if (user.channelId != channel.channelId) {
				let viewcount = await Dashboard.count({
					where: {
						user_id: user.userId,
						channelid: channel.channelId,
						visitid: channel.channelId,
						visittype: 1
					}
				})
				if (viewcount > 0) {
					await Dashboard.update({ count: viewcount + 1 }, {
						where: {
							user_id: user.userId,
							channelid: channel.channelId,
							visitid: channel.channelId,
							visittype: 1
						}
					})
				}
				else {
					let defaultData = {
						user_id: user.userId,
						channelid: channel.channelId,
						visitid: channel.channelId,
						visittype: '1',
						count: 1,
						day: new Date().getDay(),
						month: new Date().getMonth(),
						year: new Date().getFullYear(),
					};
					await Dashboard.create(defaultData);
				}
			}



			// await Dashboard.findOrCreate({where: {
			// 	user_id: user.userId,
			// 	channelid: channel.channelId,
			// 	visitid: channel.channelId,
			// }, defaults: defaultData}).then(result => {
			// 	if(!result[1]){
			// 		console.log('result1', result);
			// 		let month = new Date().getMonth()+1;
			// 		let dashr = {
			// 			userId: user.userId,
			// 			dashboardId: result[0].id,
			// 			['month'+month] : '1',
			// 			yearId: new Date().getFullYear()		
			// 		}

			// 		result[0].update({count: result[0].count+1}).then(dres => {
			// 			if(dres.dashboardReport)
			// 				DashboardReport.update({['month'+month]:'1'}, {where: {
			// 					reportId: dres.dashboardReport,
			// 					dashboardId: dres.id,
			// 					yearId: new Date().getFullYear()
			// 				}}).then(res => 'updated Dashboard Report')
			// 			else{
			// 				DashboardReport.create(dashr).then(res => dres.update({dashboardReport : res.reportId}));
			// 				console.log('dashboard report created');
			// 			}

			// 			// DashboardReport.findOrCreate({
			// 			// 	where:{
			// 			// 		reportId: result[0].dashboardReport
			// 			// 	}, defaults: dashr
			// 			// }).then(dres => {
			// 			// 	if(!dres[1]){
			// 			// 		dres[0].update({['month'+month] : '1'}).then(res => console.log('updated Dashboard Report'));
			// 			// 	}else{
			// 			// 		console.log('created');
			// 			// 	}
			// 			// })
			// 		})
			// 	}
			// 	else{
			// 		console.log('result2', result);

			// 		let month = new Date().getMonth()+1;
			// 		let year = new Date().getFullYear();

			// 		DashboardReport.create({
			// 			userId: user.userId,
			// 			dashboardId: result[0].id,
			// 			['month'+month] : '1',
			// 			yearId: year
			// 		}).then(res => {
			// 			console.log('created Dashbooard Report', res, result[0].id);
			// 			result[0].update({dashboardReport: res.reportId})
			// 		});
			// 	}
			// })
			
			return Object.assign(channelTransformed, {favorite: favorite});
		} catch (e) {
			console.log("Error+ ", e);
			throw new InternalServerErrorException();
		}
	}
	
	async getSampleOrderCount(channelId: number): Promise<number> {
		try {
			return await SampleOrder.count(
				{
					where: {
						buyerchannelid: channelId,
						statusid: { [Op.ne]: appconstant.ORDER_STATUS_CANCELED }
					}
				});
		} catch (e) {
			console.log("Error+ ", e);
			throw new InternalServerErrorException();
		}
	}

	async getSampleEvaluationCount(channelId: number): Promise<number> {
		try {
			return await ProductEvaluation.count(
				{
					where: {
						channelid: channelId,
						deletedat: null
					}
				});
		} catch (e) {
			console.log("Error+ ", e);
			throw new InternalServerErrorException();
		}
	}

	public async getChannelContactCount(channelId: number, pagination, contactFilter, validateVisibility?): Promise<any> {

		let query = 'select count(DISTINCT ct."contactId") from "tblContact" ct left join "tblContactLocation" ctl on ctl."contactId" = ct."contactId" left join "tblRegionCountryJCT" rcJCT on rcJCT."regionCountryJCTId" = ctl."regionCountryJCTId" left join (select "countryId" , "country" from "tblCountry" where "deletedAt" is null ) c on c."countryId" = rcJCT."countryId" left join "tblDocument" d on d."documentId" = ct."documentId" left join "tblUser" u on u."contactId" = ct."contactId" left join "tblUserRole" ur on ur."userId" = u."userId" left join "tblRole" r on ur."roleId" = r."roleId" where ct."channelId" = ' + channelId + ' AND ct."deletedAt" is null AND ctl."deletedAt" is null AND rcJCT."deletedAt" is null and d."deletedAt" is null and u."deletedAt" is null and ur."deletedAt" is null AND ct."isActive" = \'1\' ';

		if (isNull(validateVisibility) || isBoolean(validateVisibility) && validateVisibility) {
			query += ' AND ct."isShared" = \'' + appconstant.VISIBILITY_SHARED + '\'';
		}

		// if (channelContactFilter && channelContactFilter.contactType) {
		// 	query = query + ' AND (ct."workNatureId" IN (:workNature) or ct."workNatureId" is null)';
		// }

		if (pagination.search) {
			query = query + ' AND ( concat(ct."firstName",\' \', ct."lastName") ilike :searchName';
			query = query + ' OR ct."phone1" SIMILAR TO :search';
			query = query + ' OR ct."mobileNo" SIMILAR TO :search';
			query = query + ' OR ct."email" SIMILAR TO :search';
			query = query + ' OR ct."jobTitle" SIMILAR TO :search)';
		}

		// //filter
		if (contactFilter.isActive) {
			query = query + ' AND ct."isActive" = ' + "'" + contactFilter.isActive + "'";
		}

		if (contactFilter.roleId) {
			query = query + ' AND r."roleId" = ' + contactFilter.roleId;
		}

		if (contactFilter.countryId) {
			query = query + ' AND ct."contactId"  in  (select ctl1."contactId"  from  "tblContactLocation" ctl1,"tblRegionCountryJCT" rcJCT1  where rcJCT1."regionCountryJCTId" = ctl1."regionCountryJCTId" and rcJCT1."countryId" =coalesce(' + contactFilter.countryId + ' ,rcJCT1."countryId")  )';
		}

		if (contactFilter.excludeDefault) {
			query = query + ' AND ct."isDefaultContact" != ' + "'" + 1 + "'";
		}
		//filter

		return await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT,
			replacements: {
				search: '%(' + pagination.search + ')%',
				searchName: '%' + pagination.search + '%',
				// workNature: (channelContactFilter && channelContactFilter.contactType) ? appconstant[channelContactFilter.contactType + '_CONTACT'] : null,
			}
		}).then(contacts => {
			return contacts[0].count;
		});
	}

	public async getChannelContactsList(channelId: number, pagination, contactFilter, validateVisibility?): Promise<any> {

		var query = 'select (select array_to_json(array_agg(row_to_json(z))) from (select "contactSocialLink", "socialSiteId" from "tblContactSocialLink" where "deletedAt" is null and "socialSiteId" in(1) and "contactId"=ct."contactId")z) as sociallink, u."userId", ct."contactId", ct."isEvent", ct."isShared", ct."firstName", ct."lastName", ct.phone1, ct.phone2, ct."mobileNo", ct.email, ct."jobTitle", ct."isActive", ct."countryId", con."country", ct."isDefaultContact", d."documentPath", case when d."documentPath" IS NULL OR d."documentPath" = ' + "''" + ' then ' + "''" + ' else CONCAT(' + "'" + process.env.CDN_URL + "'" + ', d."documentPath") end documentUrl, r.role, r."roleId", string_agg(c."country", ' + "', '" + ') countries, array_to_json(array_agg(row_to_json(c))) countrylist from "tblContact" ct left join "tblCountry" con on con."countryId" = ct."countryId" left join "tblContactLocation" ctl on ctl."contactId" = ct."contactId" left join "tblRegionCountryJCT" rcJCT on rcJCT."regionCountryJCTId" = ctl."regionCountryJCTId" left join (select "countryId" , "country" from "tblCountry" where "deletedAt" is null ) c on c."countryId" = rcJCT."countryId" left join "tblDocument" d on d."documentId" = ct."documentId" left join "tblUser" u on u."contactId" = ct."contactId" left join "tblUserRole" ur on ur."userId" = u."userId" left join "tblRole" r on ur."roleId" = r."roleId" where ct."channelId" = ' + channelId + ' AND ct."deletedAt" is null AND ctl."deletedAt" is null AND rcJCT."deletedAt" is null and d."deletedAt" is null and u."deletedAt" is null and ur."deletedAt" is null AND ct."isActive" = \'1\' ';

		if (isNull(validateVisibility) || isBoolean(validateVisibility) && validateVisibility) {
			query += ' AND ct."isShared" = \'' + appconstant.VISIBILITY_SHARED + '\'';
		}

		// if (channelContactFilter && channelContactFilter.contactType) {
		// 	query = query + ' AND (ct."workNatureId" IN (:workNature) or ct."workNatureId" is null)';
		// }

		if (pagination.search) {
			query = query + ' AND ( concat(ct."firstName",\' \', ct."lastName") ilike :searchName';
			query = query + ' OR ct."phone1" SIMILAR TO :search';
			query = query + ' OR ct."mobileNo" SIMILAR TO :search';
			query = query + ' OR ct."email" SIMILAR TO :search';
			query = query + ' OR ct."jobTitle" SIMILAR TO :search )';
		}

		//filter
		if (contactFilter.isActive) {
			query = query + ' AND ct."isActive" = ' + "'" + contactFilter.isActive + "'";
		}

		if (contactFilter.roleId) {
			query = query + ' AND r."roleId" IN (:roleId)';
		}

		if (contactFilter.countryId) {
			query = query + ' AND ct."contactId"  in  (select ctl1."contactId"  from  "tblContactLocation" ctl1,"tblRegionCountryJCT" rcJCT1  where rcJCT1."regionCountryJCTId" = ctl1."regionCountryJCTId" and rcJCT1."countryId" IN (:countryId))';
		}

		if (contactFilter.excludeDefault) {
			query = query + ' AND ct."isDefaultContact" != ' + "'" + 1 + "'";
		}
		//filter

		query = query + ' group by ct."contactId", d."documentId", r."roleId", u."userId", con."country"';

		if (pagination.limit) {
			query = query + " limit :limit";
		}

		if (pagination.offset) {
			query = query + " offset :offset";
		}

		return await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT,
			model: Contact,
			mapToModel: true,
			replacements: {
				search: '%(' + pagination.search + ')%',
				searchName: '%' + pagination.search + '%',
				// workNature: (channelContactFilter && channelContactFilter.contactType) ? appconstant[channelContactFilter.contactType + '_CONTACT'] : null,
				limit: pagination.limit,
				offset: pagination.offset,
				roleId: contactFilter.roleId,
				countryId: contactFilter.countryId
			}
		}).then(contacts => {
			let contactIds = [];
			if (contacts.length > 0) {
				contacts.map(function(data, index) {
					contactIds.push(data.contactId);
				});
			}
			return { list: contacts, contactIds: contactIds };
		});
	}

	public async getConnectedContactsList(contactList) {
		console.log('contactList', contactList);
		var query = 'select (select array_to_json(array_agg(row_to_json(z))) from (select "contactSocialLink", "socialSiteId" from "tblContactSocialLink" where "deletedAt" is null and "socialSiteId" in(1) and "contactId"=ct."contactId")z) as sociallink,(select "companyName" from "tblChannel" where "channelId"=ct."channelId"),u."userId", ct."contactId", ct."isEvent", ct."isShared", ct."firstName", ct."lastName", ct.phone1, ct.phone2, ct."mobileNo", ct.email, ct."jobTitle", ct."isActive", ct."countryId", con."country", ct."isDefaultContact", d."documentPath", case when d."documentPath" IS NULL OR d."documentPath" = ' + "''" + ' then ' + "''" + ' else CONCAT(' + "'" + process.env.CDN_URL + "'" + ', d."documentPath") end documentUrl, r.role, string_agg(c."country", ' + "', '" + ') countries, array_to_json(array_agg(row_to_json(c))) countrylist from "tblContact" ct left join "tblCountry" con on con."countryId" = ct."countryId" left join "tblContactLocation" ctl on ctl."contactId" = ct."contactId" left join "tblRegionCountryJCT" rcJCT on rcJCT."regionCountryJCTId" = ctl."regionCountryJCTId" left join (select "countryId" , "country" from "tblCountry" where "deletedAt" is null ) c on c."countryId" = rcJCT."countryId" left join "tblDocument" d on d."documentId" = ct."documentId" left join "tblUser" u on u."contactId" = ct."contactId" left join "tblUserRole" ur on ur."userId" = u."userId" left join "tblRole" r on ur."roleId" = r."roleId" where ct."contactId" in(:contactList) AND ct."deletedAt" is null AND ctl."deletedAt" is null AND rcJCT."deletedAt" is null and d."deletedAt" is null and u."deletedAt" is null and ur."deletedAt" is null AND ct."isActive" = \'1\' ';

		query = query + ' group by ct."contactId", d."documentId", r."roleId", u."userId", con."country"';

		return await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT,
			model: Contact,
			mapToModel: true,
			replacements: {
				contactList: contactList.map(x => x.contact_id)
				// search: '%(' + pagination.search + ')%',
				// searchName: '%' + pagination.search + '%',
				// workNature: (channelContactFilter && channelContactFilter.contactType) ? appconstant[channelContactFilter.contactType + '_CONTACT'] : null,
				// limit: pagination.limit,
				// offset: pagination.offset,
				// roleId: contactFilter.roleId,
				// countryId: contactFilter.countryId
			}
		}).then(contacts => {
			let records = [];
			if (contacts.length > 0) {
				contacts.map(x => {
					let data = contactList.filter(con => con.contact_id == x.contactId);
					console.log("contactList:", data);
					let rec = {
						channelJCTId: data[0].channeljct,
						introVideo: data[0].introVideo,
						isConfidential: data[0].isConfidential,
						relationId: +data[0].relation_status,
						channelId: +data[0].channel_id,
						channelTypeId: +data[0].channelTypeId,
						request: data[0].status,
						companyName: data[0].companyName,
						record: x
					}
					records.push(rec);
				});
			}

			return records;
		});
	}

	async getDefaultContact(partnerChannelId: number, user?: any, channelContactFilter?, onlyCompanyDefault?): Promise<any> {
		let defaultContactId = null;

		if (!onlyCompanyDefault) {
			let myDefault = 'select "partnerContactId" FROM "tblContactJCT" CONJCT inner join "tblChannelJCT" CHJCT on CHJCT."channelJCTId" =CONJCT."channelJCTId" where ((CHJCT."partnerChannelId" =:partnerChannelId and CHJCT."channelId"=:channelId) OR (CHJCT."channelId" =:partnerChannelId and CHJCT."partnerChannelId"=:channelId)) and CONJCT."contactId" = :contactId LIMIT 1';
			defaultContactId = await this.sequelizeInstance.query(myDefault, {
				type: QueryTypes.SELECT,
				replacements: {
					partnerChannelId: partnerChannelId,
					channelId: user.channelId,
					contactId: user.contactId
				}
			}).then(async (result) => {
				if (result[0]) {
					return +result[0].partnerContactId;
				}

				return null;
			});
		}
		if (isNull(defaultContactId)) {
			let query = 'select "contactId" from "tblUser" as tu where tu."contactId" in(select "contactId" from "tblContact" where "channelId"=:partnerChannelId and "isDefaultContact"=\'1\') and tu."userId" in(select "userId" from "tblUserRole" where "roleId"=\'1\')';
			let defaultq = await this.sequelizeInstance.query(query, {
						type: QueryTypes.SELECT,
						replacements: {
							partnerChannelId: partnerChannelId,
						}
					})
					if(defaultq.length>0){
						defaultContactId = defaultq[0].contactId;
					}
		}

		return defaultContactId?defaultContactId:0;
	}
	public async getrequestid(conlist:any,user:any)
	{
		let contactlist=[];
		let req;
		console.log('result', conlist);
			for(var i=0;i<conlist.length;i++){
				let relation={};
				let query = 'select "channelJCTId","connectionStatusTypeId","isConfidential",(select doc."documentPath" from "tblDocument" as doc  where doc."documentId"="introVideo") as introvideo from "tblChannelJCT" where "requested_to"=' + user.contactId + ' and "requested_by"=' + conlist[i].contactId + '  and "deletedat" is null ';
				let status=await this.sequelizeInstance.query( query, {
					type: QueryTypes.SELECT
				}).then((result) => {
					if(result.length > 0)
					{
					if(result[0].connectionStatusTypeId==7||result[0].connectionStatusTypeId==10)	
						req='IN';																
					else if(result[0].connectionStatusTypeId==11)	
						req='OUT';						
					else if(result[0].connectionStatusTypeId==1)					
						req='SUGGESTION';					
					else if(result[0].connectionStatusTypeId==2)					
						req='CONFIRM';
					}
					return result;
					});
					console.log('status', status);
					if(status.length==0){
						query = 'select "channelJCTId","connectionStatusTypeId","isConfidential",(select doc."documentPath" from "tblDocument" as doc  where doc."documentId"="introVideo") as introvideo from "tblChannelJCT" where  "requested_by"=' + user.contactId + ' and "requested_to"=' + conlist[i].contactId + '  and "deletedat" is null ';
						status = await this.sequelizeInstance.query( query, {
								type: QueryTypes.SELECT
							}).then((result) => {
								if(result.length > 0)
								{
								if(result[0].connectionStatusTypeId==7)	
									req= 'OUT';
								else if(result[0].connectionStatusTypeId==11)	
									req= 'SAVELATER';
								else if(result[0].connectionStatusTypeId==10)	
									req= 'REQUESTINTRO';								
								else if(result[0].connectionStatusTypeId==1)							
									req='SUGGESTION';								
								else if(result[0].connectionStatusTypeId==2)								
									req='CONFIRM';
								}

								return result;
							});							
					}
					console.log('statuIs', status);
					if(status.length>0 && status[0].connectionStatusTypeId!=5 && status[0].connectionStatusTypeId!=6){
						relation = {
							record: conlist[i],
							channelJCTId: status[0].channelJCTId,
							introVideo: status[0].introvideo, 
							isConfidential:status[0].isConfidential,
                        	relationId: +status[0].connectionStatusTypeId,
							request: (req)?req:null
						};
					}	
					else if(status.length==0){
						relation = {
							record: conlist[i],
							channelJCTId: null,
							isConfidential:null,
							introVideo: null, 
                        	relationId: 0,
							request: null
						};

					}		
					console.log('relation', relation);
					if(Object.keys(relation).length !== 0)
						contactlist.push(relation);
			}	

			return contactlist;					
	}
	async getShareTypeList(): Promise<any> {
		try {
			let result = await this.sharetypeRepository.findAll<ShareType>();
			return result;
		} catch (e) {
			console.log("Error+", e);
			throw new InternalServerErrorException();
		}
	}
	public async findById(id: number, productFilter: any): Promise<Channel | null> {

		let result: any;
		let productGroupFilter = { deletedAt: null };
		let productCategoryFilter = { deletedAt: null };
		let brandFilter = { deletedAt: null };

		if (productFilter && productFilter === 'true') {
			let productlist: any = await this.productRepository.findAll({
				where: {
					channelId: id,
					deletedat: null
				},
				include: [
					{
						model: this.brandRepository,
						as: 'brand',
						where: { deletedAt: null }
					}, {
						model: this.productCategoryRepository,
						as: 'productCategory',
						where: { deletedAt: null }
					}, {
						model: this.productGroupRepository,
						as: 'productGroup',
						where: { deletedAt: null }
					}
				]
			});

			let productGroupIds: any = [];
			let productCategoryIds: any = [];
			let brandIds: any = [];

			for (let ii = 0; ii < productlist.length; ii++) {
				productGroupIds.push(productlist[ii].productGroupId);
				productCategoryIds.push(productlist[ii].productCategorieId);
				brandIds.push(productlist[ii].brandId)
			}
			productGroupFilter['productGroupId'] = { [Op.in]: productGroupIds };
			productCategoryFilter['productCategorieId'] = { [Op.in]: productCategoryIds }
			brandFilter['id'] = { [Op.in]: brandIds };

		}

		result = await this.channelRepository.findOne<Channel>({
			where: {
				channelId: id,
				deletedAt: null
			},
			include: [
				{
					model: Brand, as: 'brand', required: false, where: brandFilter, through: { attributes: [], where: { deletedAt: null } }, attributes: ['id', 'brandName']
				},
				{
					model: this.channelCustomerProfileRepository.scope('channelDetail'),
					where: { deletedAt: null },
					required: false,
					include: [
						{
							model: this.profileTypeClassificationRepository.scope('profileTypeClassificationInfom'),
							where: { deletedAt: null },
							required: false,
							include: [
								{
									model: this.profileTypeRepository.scope('profileTypeInfom'),
									where: { deletedAt: null },
									required: false,
								},
								{
									model: this.channelTypeRepository.scope('channelTypeInfom')
								}
							]
						}
					]
				}
			],
			attributes: ['channelId', 'channelTypeId']
		});


		if (!result) {
			throw new ProcessException('validate:field:notFound');
		}

		// lazy load
		await new Utility().lazyload(result, 'productGroup', {
			where: productGroupFilter,
			through: {
				attributes: [],
				where: { deletedAt: null }
			},
			attributes: ['productGroupId', 'productGroup']
		});

		await new Utility().lazyload(result, 'productCategory', {
			where: productCategoryFilter,
			through: {
				attributes: [],
				where: { deletedAt: null }
			},
			attributes: ['productCategorieId', 'productGroupId', 'productCategorie']
		});

		await new Utility().lazyload(result, 'locations', {
			where: {
				deletedAt: null
			}, through: { attributes: ['locationType'], where: { deletedAt: null } }, include: [
				{ model: Country, required: false, attributes: ['countryId', 'country'] },
				{ model: Region, required: false, attributes: ['regionId', 'region'] },
			]
		});

		if (result.customerProfile) {
			result = result.toJSON();
			let returnObject = new Object;
			for (let ii = 0; ii < result.customerProfile.length; ii++) {
				for (let jj = 0; jj < result.customerProfile[ii].profileTypeClass.length; jj++) {

					let resultObject = <ICustomerProfileAssaingObject>{};
					resultObject.Channel_Customer_Profile_ID = parseInt(result.customerProfile[ii]['ChannelCustomerProfile']['customerProfileId']);
					resultObject.Channel_Profile_ID = parseInt(result.customerProfile[ii]['profileTypeId']);
					resultObject.Channel_Profile_Name = result.customerProfile[ii]['profileType'];
					resultObject.name = result.customerProfile[ii]['profileType'];
					resultObject.Channel_Type_ID = parseInt(result.customerProfile[ii].profileTypeClass[jj]['channelTypeId']);
					resultObject.Channel_Type_Name = result.customerProfile[ii].profileTypeClass[jj]['channelType']['channelType'];
					resultObject.Default = result.customerProfile[ii].profileTypeClass[jj]['isDefault'];

					let propertyName = result.customerProfile[ii].profileTypeClass[jj]['channelType']['channelType'];
					if (!returnObject[propertyName]) {
						returnObject[propertyName] = new Array<ICustomerProfileAssaingObject>();
						returnObject[propertyName].push(resultObject);
					} else {
						returnObject[propertyName].push(resultObject);
					}
				}
			}
			result['customerProfile'] = returnObject;
		}

		return result;
	}

	//all contacts
async getAllContacts(contact)
{
	let userData = await this.contactRepository.findAll<Contact>({
		attributes:['contactId','firstName','lastName','jobTitle', "channelId"],
		where: {				
			deletedAt:null,
			isActive:'1',
			contactId : {
				[Op.in] : contact
			} 
		},
		include: [				
			{
				model: Media,
				attributes:['documentPath']
			},	
			{
				model: Country,
				attributes:['country']
			},
			{
				model: Channel,
				attributes: ['companyName', "channelTypeId"],
				include: [
					{
						model: ChannelDetail,
						attributes: ['channelLogoId'],
						include: [
							{
								model: Media,
								as: 'channelLogo',
								attributes: ['documentPath']
							}
						]
					}
				]
			},
			
		]
	});		
	return userData;
}

public async getDashboardcount(user) {
	return await this.sequelizeInstance.query('select * from "getdashboardcounts"(:channelId,:contactId,:userId)', {
		type: QueryTypes.SELECT,
		replacements: {
			channelId: user.channelId,
			contactId: user.contactId,
			userId:user.userId			
		}
	}).then((results) => {
		return results[0].getdashboardcounts;
	});
}


//searchList for advanced search contact filter
async getSearchList1(data,userData){
	let user = userData;
	let channelFilter=data.filter;
	// let contactId=user.contactId;
	// let channelId=user.channelId;
	let typeid=user.channelTypeId;
	let userId=user.userId;
	let filterch=[];let channeltype=[];
	let aggregation={};let channelquery;
	if(typeid==2)
	channeltype=[3,4];
	else if(typeid==3)
	channeltype=[2,3,4];
	else if(typeid==4)
	channeltype=[2,3];

	
	//let order='desc';
	//confirm		
	//console.log("viewId:",viewId,channeltype[0]);
	channelquery = 'select "channelId" as "channel_id" from "tblChannel" where "channelTypeId" in (:channelTypeId)';
			
	let result= await this.sequelizeInstance.query(channelquery, {
	type: QueryTypes.SELECT,
	raw: true,
	replacements: {
	channelTypeId:channeltype
	}
	}).then(x=>{						

		return  _.uniqBy(x,'channel_id');
	})
	if(data.search)
	{
		//console.log("page",pagination.page);//console.log("searcg",pagination.search);
		let  searchchannellist=await this.sequelizeInstance.query('select "channelId" as channel_id from "tblContact" where CONCAT_WS(\' \',"firstName","lastName") ILIKE :search', {
			type: QueryTypes.SELECT,
			raw: true,
			replacements: {
			search: '%' + data.search + '%',
			}
			})
		let chl=searchchannellist.map(x => x.channel_id);
		result = result.filter(ch => {return chl.includes(ch.channel_id);})
		//console.log(result)	;
	}
	// console.log('check',result?true:false)
	if(channelFilter.typeId && result)
	{
		let chlist = result.map(x => +x.channel_id);		
		let ctype='select array_agg("channelId") as ch from "tblChannel" where "channelTypeId" in(:channeltypeid) and "channelId" in(:channelid)';
	await this.sequelizeInstance.query(ctype, {
		type: QueryTypes.SELECT,
	raw: true,
	replacements: {
	channelid:chlist,
	channeltypeid:+channelFilter.typeId
	}
	}).then((result) => {
		if(result[0].ch)
	filterch.push(...result[0].ch);
	});
	result = result.filter(ch => {return filterch.includes(ch.channel_id);})
	}
	
	if((channelFilter.turnOver||channelFilter.brandId||channelFilter.locationId||channelFilter.countryId||channelFilter.specialityId||channelFilter.productCategoryId||channelFilter.retailerprofileId)&& result) {

	let chlist = result.map(x => +x.channel_id);		
	
	if(channelFilter.turnOver)
	{
	let query;
		if(channelFilter.turnOver=='<1M€')
		{
			query='select array_agg("channelId") as ch from "tblChannelDetail" where "channelId" in(:channelid) and case when "turnoverRange"=\'<1M€\' or cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)<1 or "turnover"<1 then cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)<1 or "turnoverRange"=\'<1M€\' or "turnover"<1 end';
		}
		else if(channelFilter.turnOver=='1-10M€')
		{					
			query='select  array_agg("channelId") as ch from "tblChannelDetail" where "channelId" in(:channelid) and case when "turnoverRange"=\'1-10M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\'))as int)>1 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<10) or ("turnover">1 and "turnover"<10) then "turnoverRange"=\'1-10M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\'))as int)>1 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<10) or ("turnover">1 and "turnover"<10) end';
		}
		else if(channelFilter.turnOver=='10-100M€')
		{
			query='select  array_agg("channelId") as ch from "tblChannelDetail" where "channelId" in(:channelid) and "turnoverRange"!=\'>100M€\' and case when "turnoverRange"=\'10-100M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\')) as int)>10 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<=100) or ("turnover">10 and "turnover"<=100) then "turnoverRange"=\'10-100M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\')) as int)>10 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<=100) or ("turnover">10 and "turnover"<=100) end';
		}
		else if(channelFilter.turnOver=='>100M€')
		{
			query='select array_agg("channelId") as ch from "tblChannelDetail" where "channelId" in(:channelid) and case when "turnoverRange"=\'>100M€\' or cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)>100 or "turnover">100 then cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)>100 or "turnoverRange"=\'>100M€\' or "turnover">100 end';
		}
		
	await this.sequelizeInstance.query(query, {
		type: QueryTypes.SELECT,
	raw: true,
	replacements: {
	channelid:chlist
	//channeltypeid:channelFilter.typeId
	}
	}).then((result) => {

		if(result[0].ch){
			filterch = [];
			filterch.push(...result[0].ch);
		}else
		filterch = [];
	});
	}

	if(channelFilter.brandId) {
	let brand='select array_agg("channelId") as ch from "tblChannelBrand" where "brandId" in(:brandid) and "channelId" in(:channelid)';
	await this.sequelizeInstance.query(brand, {
		type: QueryTypes.SELECT,
	raw: true,
	replacements: {
	channelid:chlist,
	brandid:channelFilter.brandId
	}
	}).then((result) => {
		if(result[0].ch){
			filterch = [];
	filterch.push(...result[0].ch);
	}else
	filterch = [];

	});
	}
	if(channelFilter.productCategoryId) {
	await this.sequelizeInstance.query('select array_agg("channelId") as ch from "tblChannelProductCategorie" cpc where cpc."productCategorieId" in(:productcatid) and "channelId" in(:channelid) ', {
		type: QueryTypes.SELECT,
	raw: true,
	replacements: {
	channelid:(filterch.length>0)?filterch:chlist,
	productcatid:channelFilter.productCategoryId
	}
	}).then((result) => {
		if(result[0].ch){
			filterch = [];
	filterch.push(...result[0].ch);
		}else
		filterch = [];

	});
	}
	if(channelFilter.retailerprofileId) {
	await this.sequelizeInstance.query('select array_agg("channelId") as ch from "tblChannelCustomerProfile" where "profileTypeId" in(:retailprofileid) and "channelId" in(:channelid)', {
		type: QueryTypes.SELECT,
	raw: true,
	replacements: {
	channelid:(filterch.length>0)?filterch:chlist,
	retailprofileid:channelFilter.retailerprofileId
	}
	}).then((result) => {
		if(result[0].ch){
			filterch = [];
	filterch.push(...result[0].ch);
		}else
		filterch = [];

	});
	}

	if(channelFilter.countryId) {
	await this.sequelizeInstance.query('select array_agg("channelId") as ch from "tblChannel" where "countryId" in(:countryid) and "channelId" in(:channelid)', {
		type: QueryTypes.SELECT,
	raw: true,
	replacements: {
	channelid:(filterch.length>0)?filterch:chlist,
	countryid:channelFilter.countryId
	}
	}).then((result) => {
		if(result[0].ch){
			filterch = [];
	filterch.push(...result[0].ch);
		}else
		filterch = [];
	});
	}
	if(channelFilter.specialityId) {
	await this.sequelizeInstance.query('select array_agg(distinct "channelId") as ch from "tblContact" where "contactId" in(Select "contactId" from "tblContactInterestedProductCategory" where "productCategorieId" in(:specialityid)) and "channelId" in(:channelid)', {
		type: QueryTypes.SELECT,
	raw: true,
	replacements: {
	channelid:(filterch.length>0)?filterch:chlist,
	specialityid:channelFilter.specialityId
	}
	}).then((result) => {
		if(result[0].ch){
			filterch = [];
	filterch.push(...result[0].ch);
		}else
		filterch = [];
	});
	}
	if(channelFilter.locationId) {
	await this.sequelizeInstance.query('select array_agg("channelId") as ch from "tblChannel" where "countryId" in(:locationid) and "channelId" in(:channelid)', {
		type: QueryTypes.SELECT,
	raw: true,
	replacements: {
	channelid:(filterch.length>0)?filterch:chlist,
	locationid:channelFilter.locationId
	}
	}).then((result) => {
		if(result[0].ch){
			filterch = [];
	filterch.push(...result[0].ch);
		}else
		filterch = [];
	});
	}
	// if(!channelFilter.connectionStatusId)
		result = result.filter(ch => {return filterch.includes(ch.channel_id);})
	}

	
	let total = result?result.length:0;
	let final=[];
			
	if(result){		
		//  result=result.slice(data.offset).slice(0,data.limit);
		}else
	result=[];

	let finalres = {
		result: result,
		total: total
	}

	return finalres;
}


//searchList for profileview 
async getSearchListProfile(data,userData){
	let user = userData;
	let channelFilter=data.filter;
	// let contactId=user.contactId;
	// let channelId=user.channelId;
	let typeid=user.channelTypeId;
	let userId=user.userId;
	let filterch=[];let channeltype=[];
	let aggregation={};let channelquery;

	if(typeid==2)
	channeltype=[3,4];
	else if(typeid==3)
	channeltype=[2,3,4];
	else if(typeid==4)
	channeltype=[2,3];

	
	//let order='desc';
	//confirm		
	//console.log("viewId:",viewId,channeltype[0]);
	channelquery = 'select "channelId" as "channel_id" from "tblContact" where "contactId" in (:contactId)';
			
	let result= await this.sequelizeInstance.query(channelquery, {
	type: QueryTypes.SELECT,
	raw: true,
	replacements: {
	contactId:data.contactId
	}
	}).then(x=>{						

		return  _.uniqBy(x,'channel_id');
	})
	if(data.search)
	{
		//console.log("page",pagination.page);//console.log("searcg",pagination.search);
		let  searchchannellist=await this.sequelizeInstance.query('select "channelId" as channel_id from "tblContact" where CONCAT_WS(\' \',"firstName","lastName") ILIKE :search', {
			type: QueryTypes.SELECT,
			raw: true,
			replacements: {
			search: '%' + data.search + '%',
			}
			})
		let chl=searchchannellist.map(x => x.channel_id);
		result = result.filter(ch => {return chl.includes(ch.channel_id);})
		//console.log(result)	;
	}
	// console.log('check',result?true:false)
	if(channelFilter.typeId && result)
	{
		let chlist = result.map(x => +x.channel_id);		
		let ctype='select array_agg("channelId") as ch from "tblChannel" where "channelTypeId" in(:channeltypeid) and "channelId" in(:channelid)';
	await this.sequelizeInstance.query(ctype, {
		type: QueryTypes.SELECT,
	raw: true,
	replacements: {
	channelid:chlist,
	channeltypeid:+channelFilter.typeId
	}
	}).then((result) => {
		if(result[0].ch)
	filterch.push(...result[0].ch);
	});
	result = result.filter(ch => {return filterch.includes(ch.channel_id);})
	}
	
	if((channelFilter.turnOver||channelFilter.brandId||channelFilter.locationId||channelFilter.countryId||channelFilter.specialityId||channelFilter.productCategoryId||channelFilter.retailerprofileId)&& result) {

	let chlist = result.map(x => +x.channel_id);		
	
	if(channelFilter.turnOver)
	{
	let query;
		if(channelFilter.turnOver=='<1M€')
		{
			query='select array_agg("channelId") as ch from "tblChannelDetail" where "channelId" in(:channelid) and case when "turnoverRange"=\'<1M€\' or cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)<1 or "turnover"<1 then cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)<1 or "turnoverRange"=\'<1M€\' or "turnover"<1 end';
		}
		else if(channelFilter.turnOver=='1-10M€')
		{					
			query='select  array_agg("channelId") as ch from "tblChannelDetail" where "channelId" in(:channelid) and case when "turnoverRange"=\'1-10M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\'))as int)>1 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<10) or ("turnover">1 and "turnover"<10) then "turnoverRange"=\'1-10M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\'))as int)>1 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<10) or ("turnover">1 and "turnover"<10) end';
		}
		else if(channelFilter.turnOver=='10-100M€')
		{
			query='select  array_agg("channelId") as ch from "tblChannelDetail" where "channelId" in(:channelid) and "turnoverRange"!=\'>100M€\' and case when "turnoverRange"=\'10-100M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\')) as int)>10 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<=100) or ("turnover">10 and "turnover"<=100) then "turnoverRange"=\'10-100M€\' or (cast((substring("turnoverRange" FROM \'[0-9]+\')) as int)>10 and cast(substring("turnoverRange" FROM \'[0-9]+\') as int)<=100) or ("turnover">10 and "turnover"<=100) end';
		}
		else if(channelFilter.turnOver=='>100M€')
		{
			query='select array_agg("channelId") as ch from "tblChannelDetail" where "channelId" in(:channelid) and case when "turnoverRange"=\'>100M€\' or cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)>100 or "turnover">100 then cast(substring("turnoverRange" FROM \'[0-9]+\')as integer)>100 or "turnoverRange"=\'>100M€\' or "turnover">100 end';
		}
		
	await this.sequelizeInstance.query(query, {
		type: QueryTypes.SELECT,
	raw: true,
	replacements: {
	channelid:(filterch.length>0)?filterch:chlist
	//channeltypeid:channelFilter.typeId
	}
	}).then((result) => {

		if(result[0].ch){
			filterch = [];
			filterch.push(...result[0].ch);
		}else
		filterch = [];
	});
	}

	if(channelFilter.brandId) {
	let brand='select array_agg("channelId") as ch from "tblChannelBrand" where "brandId" in(:brandid) and "channelId" in(:channelid)';
	await this.sequelizeInstance.query(brand, {
		type: QueryTypes.SELECT,
	raw: true,
	replacements: {
	channelid:(filterch.length>0)?filterch:chlist,
	brandid:channelFilter.brandId
	}
	}).then((result) => {
		if(result[0].ch){
			filterch = [];
	filterch.push(...result[0].ch);
	}else
	filterch = [];

	});
	}
	if(channelFilter.productCategoryId) {
	await this.sequelizeInstance.query('select array_agg("channelId") as ch from "tblChannelProductCategorie" cpc where cpc."productCategorieId" in(:productcatid) and "channelId" in(:channelid) ', {
		type: QueryTypes.SELECT,
	raw: true,
	replacements: {
	channelid:(filterch.length>0)?filterch:chlist,
	productcatid:channelFilter.productCategoryId
	}
	}).then((result) => {
		if(result[0].ch){
			filterch = [];
	filterch.push(...result[0].ch);
		}else
		filterch = [];

	});
	}
	if(channelFilter.retailerprofileId) {
	await this.sequelizeInstance.query('select array_agg("channelId") as ch from "tblChannelCustomerProfile" where "profileTypeId" in(:retailprofileid) and "channelId" in(:channelid)', {
		type: QueryTypes.SELECT,
	raw: true,
	replacements: {
	channelid:(filterch.length>0)?filterch:chlist,
	retailprofileid:channelFilter.retailerprofileId
	}
	}).then((result) => {
		if(result[0].ch){
			filterch = [];
	filterch.push(...result[0].ch);
		}else
		filterch = [];

	});
	}

	if(channelFilter.countryId) {
	await this.sequelizeInstance.query('select array_agg("channelId") as ch from "tblChannel" where "countryId" in(:countryid) and "channelId" in(:channelid)', {
		type: QueryTypes.SELECT,
	raw: true,
	replacements: {
	channelid:(filterch.length>0)?filterch:chlist,
	countryid:channelFilter.countryId
	}
	}).then((result) => {
		if(result[0].ch){
			filterch = [];
	filterch.push(...result[0].ch);
		}else
		filterch = [];
	});
	}
	if(channelFilter.specialityId) {
	await this.sequelizeInstance.query('select array_agg(distinct "channelId") as ch from "tblContact" where "contactId" in(Select "contactId" from "tblContactInterestedProductCategory" where "productCategorieId" in(:specialityid)) and "channelId" in(:channelid)', {
		type: QueryTypes.SELECT,
	raw: true,
	replacements: {
	channelid:(filterch.length>0)?filterch:chlist,
	specialityid:channelFilter.specialityId
	}
	}).then((result) => {
		if(result[0].ch){
			filterch = [];
	filterch.push(...result[0].ch);
		}else
		filterch = [];
	});
	}
	if(channelFilter.locationId) {
	await this.sequelizeInstance.query('select array_agg("channelId") as ch from "tblChannel" where "countryId" in(:locationid) and "channelId" in(:channelid)', {
		type: QueryTypes.SELECT,
	raw: true,
	replacements: {
	channelid:(filterch.length>0)?filterch:chlist,
	locationid:channelFilter.locationId
	}
	}).then((result) => {
		if(result[0].ch){
			filterch = [];
	filterch.push(...result[0].ch);
		}else
		filterch = [];
	});
	}
	// if(!channelFilter.connectionStatusId)
		result = result.filter(ch => {return filterch.includes(ch.channel_id);})
	}

	
	let total = result?result.length:0;
	let final=[];
			
	if(result.length){		
	
	let cha = result.map(x => x.channel_id);
	let qu='select "contactId" from "tblContact" where "contactId" in (:contactId) and "channelId" in (:channelId) ';
	qu += ' and "contactId" not in (select "requested_to" from "tblChannelJCT" where "requested_by"=' + user.contactId +'  and "requested_to" in (:contactId) and "connectionStatusTypeId" in(5,6)and "deletedat" is null)';
	qu += ' and "contactId" not in (select "requested_by" from "tblChannelJCT" where "requested_to"=' + user.contactId +'  and "requested_by" in (:contactId) and "connectionStatusTypeId" in(5,6)and "deletedat" is null)'
	final = await this.sequelizeInstance.query(qu,{
	type: QueryTypes.SELECT,
	raw: true,
	replacements: {
		channelId: cha,
		contactId: data.contactId
		}
	 })
		}else
	result=[];

	let finalres = {
		result: final,
		total: final.length
	}

	return finalres;
}


public async billingAddress(user){
	return await Channel.findAll<Channel>({
		where: {
			channelId: user.channelId,
		},
		include: [
			{
				model: Address,
				as: 'billAddress',
				include: [
					{
						model: Country,
						as: 'country',
						attributes: ['country']
					}
				],
				required: true
			}
		]
	});
}


//helper functions

/**
 * Sort array of objects based on another array
 */

 async mapOrder (array, order, key) {
  
	array.sort( function (a, b) {
	  var A = a[key], B = b[key];
	  
	  if (order.indexOf(A) > order.indexOf(B)) {
		return 1;
	  } else {
		return -1;
	  }
	  
	});
	
	return array;
  };
  async isEvent(ContactId, isEvent){
	return await this.contactRepository.update({ isEvent:isEvent},{
		where:{
			contactId : ContactId,
		},
		returning: true
	});
}

async setEventParticipantStatus(eventId, channelId, ContactId){
	return await EventParticipants.update({ status:'1'},{
		where:{
			contactId : ContactId,
			channelId: channelId,
			eventId: eventId,
			deletedAt:null
		},
		returning: true
	});
}

async getEventListing(channelId, contactId){
	let result:any;

	let res = await EventBo.findAll({
		where: {
			deletedAt: null
		},
		include: [
			{
                model: Country,
                attributes: ['countryId', 'country'],
                required: false
            },
            {
                model: Media,
                as: 'logo',
                attributes: ['documentId', 'documentPath'],
                where: {
                    deletedAt: null
                },
                required: false
            },
            {
                model: Media,
                as: 'banner',
                attributes: ['documentId', 'documentPath'],
                where: {
                    deletedAt: null
                },
                required: false
            },
			{
                model: EventChannel,
				where:{
					channelId: channelId
				},
				include:[
					{
						model: EventPackage,
						required: false
					}
				],
                required: false
            },
		]
	}).then(res => res);

	let list = [];
	if(res){
		res.map(x => {


			let temp = {
				event_name: x.name,
				eventId: x.id,
				event_catalyst_id: x.eventCatalystId,
				date_start: x.fromDate,
				date_end: x.toDate,
				description: x.description,
				location: x.country.country,
				logo: x.logo.documentPath,
				status: "",
				statusDate: "",
				type: "",
				channel: x.channels?.length ? x.channels[0] : null
				
			};

			list.push(temp);
		});
	}

	result = {registrations: list};

	return result;

}

async getEventPackages(user, eventId){
	let result : any;

	result = await EventPackage.findAll({
		where: {
			eventId: eventId,
			channelType: user.channelTypeId,
			isSeller: user.channelTypeId=='2' ? '1' : '0',
			isBuyer: user.channelTypeId!='2' ? '1' : '0',
			deletedAt: null
		},
		order:[
			['price', 'ASC']
		]
	});

	return result;

}

//getEventChannelDetails
async getEventChannelDetails(channelId, eventId){
    let result: any={participants: [], contacts: []};

    let participantQuery = 'select tc."contactId" as "id", concat(tc."firstName",  \' \', tc."lastName") as "name", td."documentPath", tpar."eventParticipantId" from "tblEventParticipants" tpar inner join "tblContact" tc on tc."contactId"=tpar."contactId" left join "tblDocument" td on td."documentId"=tc."documentId" where tpar."channelId"=:channelId and tpar."eventId"=:eventId';

    result.participants = await this.sequelizeInstance.query(participantQuery, {
        type: QueryTypes.SELECT,
        raw: true,
        replacements: {
            channelId: channelId,
            eventId: eventId
        }
    }).then(res => res);

    let contactsQuery = 'select tc."contactId" as "id", concat(tc."firstName", \' \', tc."lastName") as "name", td."documentPath"  from "tblContact" tc left join "tblDocument" td on td."documentId"=tc."documentId" where tc."channelId"=:channelId and tc."deletedAt" is null';

    result.contacts = await this.sequelizeInstance.query(contactsQuery, {
        type: QueryTypes.SELECT,
        raw: true,
        replacements: {
            channelId: channelId,
        }
    });

    return result;

}

//post event registration details
async postEventRegistraion(user, data){

	let eventchanneldata = {
		channelId: user.channelId,
		eventId: +data.eventId,
		isNew: '0',
		packageId: data.packageId,
		additionalMembers: data.additionalMembers,
		paymenttransactionId: data.paymentTransactionId,
		paymentstatus: data.paymentStatus,
		paymentMethod: data.paymentMethod
	};

	await EventChannel.findOrCreate({where: {channelId: user.channelId, eventId: data.eventId}, defaults: eventchanneldata}).then((result) => {
		if(!result[1]){
			result[0].update(eventchanneldata);
		}

	});

	let contactId = data.contactId;
    let channelId = user.channelId;
    let eventId = data.eventId;
    let participantType = user.channelTypeId=='2' ? '1' : '2';
    let channelType = user.channelTypeId;
	let packageId = data.packageId;


        let registeredQuery = 'select "contactId", "eventParticipantId", status from "tblEventParticipants" where "channelId"=:channelId and "eventId"=:eventId  and "deletedAt" is null';

        let oldIds = await this.sequelizeInstance.query(registeredQuery, {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
                channelId: channelId,
                eventId: eventId
            }
        });

        let activeId = [];
        
        oldIds.map(x => {
            contactId.map(y => {
                if(x.contactId == y)
                    activeId.push(y);
            })
        })

        //removing deleted contacts
        await EventParticipants.destroy({where: {
            eventId: eventId,
            channelId: channelId,
            contactId: {
                [Op.notIn]: activeId
            }
        }});

        console.log('activeId', activeId)
        //adding new contacts
        let newRegisteredUsers = [];
        if(activeId.length)
          contactId.map(x => {
              if(activeId.filter(y => x == y).length == 0)
                newRegisteredUsers.push(x);
          })
        else 
         newRegisteredUsers = contactId;

        if(newRegisteredUsers.length){
            let newContacts = newRegisteredUsers.map(x => {
                let data = {contactId: +x, channelId: +channelId, eventId: +eventId, channelType: +channelType, participantType: +participantType, packageId: +packageId}

                return data;
            });
            console.log('new users', newContacts)
            await EventParticipants.bulkCreate(newContacts);
        }

    

    return true;

}

//sending data to eventCatalyst
async foToEventCatalyst(contacts){
   

    let query='select (select concat(\'https://d35w8j22j5uly8.cloudfront.net\',"documentPath") from "tblDocument" where "documentId"=tcon."documentId") as profileImage, '
    + ' "firstName","lastName","jobTitle","contactId",tcon."mobileNo",tcon."email",tc."companyName",tc."channelId" as companyId, cd."detailDesc",case when tc."channelTypeId"=2 then \'Vendor\' when tc."channelTypeId"=3 then \'Distributor\' when tc."channelTypeId"=4 then \'Retailer\' end as channelType, '
    + ' (select concat(\'https://d35w8j22j5uly8.cloudfront.net\',"documentPath") from "tblDocument" where "documentId"=cd."channelLogoId") as companyLogo, '
    + '\'3e683a61-992b-47d2-8ecf-f88c9849f499\' as eventId,\'Confirmed\' as status, '
    + ' case when tc."channelTypeId"=2 then \'Vendor\' when tc."channelTypeId" in(3,4) then \'Buyer\' end as type, '
    + ' (select "country" from "tblCountry" where "countryId"=tc."countryId"),'
    + ' (select "isoCode" from "tblCountry" where "countryId"=tc."countryId"),(select string_agg("productCategorie",\',\') from "tblProductCategories" where "productCategorieId" in '
    + '(Select cpc."productCategorieId" from "tblContactInterestedProductCategory" cpc where cpc."contactId" = tcon."contactId"))as "IntrestedCategory", '
    
    + '(select string_agg("productCategorie",\',\') from "tblProductCategories" where "productCategorieId" in '
    + '(Select cpc."productCategorieId" from "tblContactProductCategorie" cpc where cpc."contactId" = tcon."contactId")) as "ExistingCategory", '
    
    + '(select "profileType" from "tblProfileType" where "profileTypeId"=tc."profileTypeId") as "ProfileType", '
    
    + '(select string_agg(pt."profileType",\',\') from "tblProfileType" pt where "profileTypeId" in (select "profileTypeId" from "tblChannelCustomerProfile" as ccp where ccp."channelId"=tc."channelId")) as "Retail_Customer_Profile", '
    
    + '(select "password" from "tblUser" where "contactId"=tcon."contactId") '
    
    + 'from "tblContact" tcon inner join "tblChannel" tc on tc."channelId"=tcon."channelId"	inner join "tblChannelDetail" cd on cd."channelId"=tcon."channelId" where tcon."contactId" in (:contactId)';	

    let result:any;

    if(contacts.length){
        result=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
            replacements: {
            contactId:contacts
            }
            });
    }

    return result;
}

async getSelectedEventDetails(user, eventId){

	let result = await EventChannel.findOne({
		where: {
			channelId: user.channelId,
			eventId: eventId,
			deletedAt: null
		}
	}).then(res => {
			return res[0];
	});

	return result;

}

async eventRegistration(user){

}

async getPendingEvent(){
	return await EventChannel.findAndCountAll({
		where: {
			paymentstatus: '0',
			paymentMethod: 2,
			deletedAt: null
		},
		include: [
			{
				model: EventPackage,
				required: false,
				
			},
			{
				model: EventBo,
				required: false
			},
			{
				model: Channel,
				required: false
			}
			
		]
	}).then(res => res);

}

async updateEventChannel(req){
	return await EventChannel.update({ paymentstatus:req.status},{
		where:{
			id: req.eventChannelId,
		},
		returning: true
	});
	
}

async getContactDetail(contactId: number) {
	try {
		let contactDetails = await this.contactRepository.findOne<any>({
			where: {
				contactId: contactId,
				deletedAt: null
			},
			include: [
				{
					model: Media, as: 'document', required: false, where: {
						deletedAt: null
					}
				},
				{
					model: this.countryRepository,
					required: false
				},
				// {
				// 	model: ContactSocialLink, as: 'contactSocialLink', required: false, where: {
				// 		deletedAt: null,
				// 		socialSiteId: {
				// 			$in: [
				// 				SocialSite.SOCIALSITE_LINKEDIN,
				// 				SocialSite.SOCIALSITE_GOOGLEPLUS
				// 			]
				// 		}
				// 	},
				// 	attributes: [
				// 		'contactSocialLink', 'socialSiteId'
				// 	]
				// },
				{
					model: ProductGroup, as: 'productGroups', required: false, through: {
						attributes: [],
						where: {
							deletedAt: null
						}
					}, attributes: [
						'productGroupId', 'productGroup'
					]
				}
			]
		}).then(async (result) => {
			// await result.contactSocialLink.forEach((socialLink) => {
			// 	if (+socialLink.socialSiteId == SocialSite.SOCIALSITE_LINKEDIN) {
			// 		result.linkedIn = socialLink.contactSocialLink;
			// 	} else if (+socialLink.socialSiteId == SocialSite.SOCIALSITE_GOOGLEPLUS) {
			// 		result.googleplus = socialLink.contactSocialLink;
			// 	}
			// });

			return result;
		});

		//productCategories
		await new Utility().lazyload(contactDetails, 'productCategories', {
			// model: ProductCategory, as: 'productCategories', required: false,
			through: {
				attributes: [],
				where: {
					deletedAt: null
				}
			}, attributes: [
				'productCategorieId', 'productGroupId', 'productCategorie'
			]
		});

		//IntProductCategories
		await new Utility().lazyload(contactDetails, 'IntProductCategories', {
			// model: ProductCategory, as: 'IntProductCategories', required: false,
			through: {
				attributes: [],
				where: {
					deletedAt: null
				}
			}, attributes: [
				'productCategorieId', 'productGroupId', 'productCategorie'
			]
		});

		// load location info
		await new Utility().lazyload(contactDetails, 'locations', {
			through: {
				attributes: ['locationType'],
				where: {
					deletedAt: null
				}
			}, include: [
				{
					model: Country,
					required: false,
					attributes: [
						'countryId', 'country'
					]
				}
			]
		});

		return contactDetails;
	} catch (e) {
		console.log("Error+ ", e);
		throw new InternalServerErrorException();
	}
}
}
