import { Inject, Injectable, InternalServerErrorException, RequestTimeoutException } from '@nestjs/common';
//import { sequelizeInstance } from '../models/sequelizeInstance.entity';
import { ProfileRatings } from '../models/ProfileCreationRating.entity';
import { ProfileList } from '../models/ProfileList.entity';
import { ContextService } from '../../../context/context.service';
import { ChannelDetail } from '../models/ChannelDetail.entity';
import { ChannelBrand } from '../models/ChannelBrand.entity';
import { ChannelCertificate } from '../models/ChannelCertificate.entity';
import { ProcessException } from 'src/middleware/exception/Process.exception';
import { ChannelCustomerProfile } from '../models/ChannelCustomerProfile.entity';
import { ProductGroup } from 'src/modules/product/models/ProductGroup.entity';
import { Brand } from 'src/modules/product/models/Brand.entity';
import { ProductCategory } from 'src/modules/product/models/ProductCategory.entity';
import { ProfileType } from '../models/ProfileType.entity';
import { ProfileTypeClassification } from '../models/ProfileTypeClassification.entity';
import { Country } from 'src/modules/common/models/Country.entity';
import { Region } from 'src/modules/common/models/Region.entity';
import { RegionCountryJCT } from 'src/modules/common/models/RegionCountryJCT.entity';
import { ChannelProductGroup } from '../models/ChannelProductGroup.entity';
import { ChannelProductCategorie } from '../models/ChannelProductCategorie.entity';
import {Op, QueryTypes } from "sequelize";
import { ChannelDocument } from '../models/ChannelDocument.entity';
import { Media, MediaType } from '../../media/models/index';
import { ChannelLocation } from '../models/ChannelLocation.entity';
import { ContactInterestedProductGroup } from 'src/modules/user/models/ContactInterestedProductGroup.entity';
import { ContactInterestedProductCategory } from 'src/modules/user/models/ContactInterestedProductCategory.entity';
import { User } from 'src/modules/user/models/User.entity';
import { Contact } from 'src/modules/user/models/Contact.entity';
import { ProductService } from 'src/modules/product/service/product.service';
import { appconstant } from 'src/config/app.constant';
import { Channel } from '../models/Channel.entity';
import { isBoolean, isUndefined } from 'lodash';
import { ContactLocation } from 'src/modules/user/models/ContactLocation.entity';
import { ChannelLocationDto } from '../dto/channelLocation.dto';

import { ChannelSocialLink } from '../models/ChannelSocialLink.entity';
import { and } from 'sequelize';
import { Address } from '../models/Address.entity';
import { ChannelAward } from '../models/ChannelAward.entity';
import { promisify } from 'util';
import { ChannelSpeciality } from '../models/ChannelSpeciality.entity';
import { TradeInformationDto } from '../dto/tradeInformation.dto';
import { TradeInformationInBound } from '../transformation/tradeInformationInBound';
import { ContactProductGroup } from 'src/modules/user/models/ContactProductGroup.entity';
import { ContactProductCategory } from 'src/modules/user/models/ContactProductCategory.entity';
import { ChannelServiceList } from '../models/ChannelServiceList.entity';
//import { exception } from 'console';
import { CompanyUpdateDto } from '../dto/company.update.dto';
import { CompanyAwardsDto } from '../dto/companyawards.dto';
import { ChannelB2B } from '../models/ChannelB2B.entity';

import { ChannelKeyPartner } from '../models/ChannelKeyPartner.entity';
import { KeyPartnerJCT } from '../models/KeyPartnerJCT.entity';
import { OldChannel } from '../models/OldChannel.entity';
import { OldChannelDetail } from '../models/OldChannelDetail.entity';

import sanitizeHtml from 'sanitize-html';
import { OldAddress } from '../models/OldAddress.entity';
//import { Currency } from 'aws-sdk/clients/macie2';
import { Currency } from '../models/Currency.entity';
import {ServiceList} from  '../models/ServiceList.entity';
import { ChannelPaymentOptionType } from '../models/ChannelPaymentOptionType.entity';
import { ChannelPaymentTerm } from '../models/ChannelPaymentTerm.entity';
import { ChannelSharePreference } from '../models/ChannelSharePreference.entity';
import { SettingsDto } from '../dto/settings.dto';
import { ChannelSubscription } from '../models/ChannelSubscription.entity';
import { SubscriptionPlan } from '../models/SubscriptionPlan.entity';
import { Subscription } from '../models/Subscription.entity';
import { SearchRegionDto } from '../dto/searchRegion.dto';
import { Zone } from 'src/modules/common/models/Zone.entity';
import { Product } from 'src/modules/product/models/Product.entity';
import { ProductPriceDetail } from 'src/modules/product/models/ProductPriceDetail.entity';
import { ProductTag } from 'src/modules/product/models/ProductTag.entity';

import { UserWalletAmount } from '../../walletRecharge/models/UserWalletAmount.entity';
import { InterestedProductGroup } from 'src/modules/user/models/InterestedProductGroup.entity';
import { InterestedProductCategory } from 'src/modules/user/models/InterestedProductCategory.entity';
import { CompanyMediaImages } from '../models/CompanyMediaImages';
import { AuthException } from 'src/middleware/exception/Auth.exception';
import { EndCustomersB2b } from '../models/EndCustomerB2b';

import {CertificateTag} from '../models/CertificateTag.entity';
import {CertificateTagDto} from '../dto/certificateTag.dto';
import { GuestRequirement } from '../models/GuestRequirement.entity';


var _ = require('lodash');


@Injectable()
export class ChannelSettingService {
    private tradeInformationInBound = new TradeInformationInBound();
    constructor(
        private readonly productService: ProductService,
        @Inject('SequelizeInstance') private readonly sequelizeInstance,
        @Inject('ChannelRepository') private readonly channelRepository: typeof Channel,
        @Inject('ChannelDetailRepository') private readonly channelDetailRepository: typeof ChannelDetail,
        @Inject('ChannelBrandRepository') private readonly channelBrandRepository: typeof ChannelBrand,
        @Inject('ChannelCustomerProfileRepository') private readonly channelCustProfileRepository: typeof ChannelCustomerProfile,
        @Inject('ChannelCertiRepository') private readonly channelCertiRepository: typeof ChannelCertificate,
        @Inject('SettingRepository') private readonly settingRepository: typeof ProfileRatings,
        @Inject('ProfileRepository') private readonly profileRepository: typeof ProfileList,

        @Inject('UserRepository') private readonly userRepository: typeof User,
        @Inject('ContactRepository') private readonly contactRepository: typeof Contact,
        
        @Inject('ChannelProductGroupRepository') private readonly channelProductGroupRepository: typeof ChannelProductGroup,
        @Inject('ProductGroupRepository') private readonly productGroupRepository: typeof ProductGroup,
        @Inject('BrandRepository') private readonly brandRepository: typeof Brand,
        @Inject('ProductCategoryRepository') private readonly productCatRepository: typeof ProductCategory,
        @Inject('ChannelProductCategoryRepository') private readonly channelCatRepository: typeof ChannelProductCategorie,

        @Inject('ContactProductGroupRepository') private readonly contactGroupRepository: typeof ContactInterestedProductGroup,
        @Inject('ContactProductCategoryRepository') private readonly contactCatRepository: typeof ContactInterestedProductCategory,

		@Inject('InterestedProductGroupRepository') private readonly intrestedGroupRepository: typeof InterestedProductGroup,
        @Inject('InterestedProductCategoryRepository') private readonly intrestedCatRepository: typeof InterestedProductCategory,

		
        @Inject('ProfileTypeRepository') private readonly profileTypeRepository: typeof ProfileType,
        @Inject('ProfileTypeClassiRepository') private readonly profileTypeClassiRepository: typeof ProfileTypeClassification,
        @Inject('CountryRepository') private readonly countryRepository: typeof Country,
        @Inject('RegionRepository') private readonly regionRepository: typeof Region,
        @Inject('RegionCtyJctRepository') private readonly regCtyJCTRepository: typeof RegionCountryJCT,
        @Inject('ChannelLocationRepository') private readonly channelLocationRepository: typeof ChannelLocation,
        @Inject('ContactLocationRepository') private readonly contactLocationRepository: typeof ContactLocation,
        @Inject('ChannelDocumentRepository') private readonly channelDocumentRepository: typeof ChannelDocument,
        @Inject('MediaRepository') private readonly mediaRepository: typeof Media,

        @Inject('ChannelSocialLinkRepository') private readonly socialLinkRepository: typeof ChannelSocialLink,
        @Inject('AddressRepository') private readonly addressRepository: typeof Address,
        @Inject('ChannelAwardRepository') private readonly awardRepository: typeof ChannelAward,
        @Inject('ChannelProductCategorieRepository') private readonly channelProductCategorieRepository: typeof ChannelProductCategorie,
        @Inject('ChannelCustomerProfileRepository') private readonly channelCustomerProfileRepository: typeof ChannelCustomerProfile,
        @Inject('ChannelServiceListRepository') private readonly channelServiceListRepository: typeof ChannelServiceList,
        @Inject('ChannelSpecialityRepository') private readonly channelSpecialityRepository: typeof ChannelSpeciality,
		@Inject('ServiceListRepository') private readonly serviceListRepository: typeof ServiceList,

		@Inject('MediaTypeRepository') private readonly mediaTypeRepository: typeof MediaType,
		@Inject('ChannelB2BRepository') private readonly channelB2BRepository: typeof ChannelB2B,
		@Inject('CurrencyRepository') private readonly currencyRepository: typeof Currency,

		@Inject('ChannelPaymentOptionTypeRepository') private readonly channelPaymentOptionTypeRepository: typeof ChannelPaymentOptionType,
		@Inject('ChannelPaymentTermRepository') private readonly channelPaymentTermRepository: typeof ChannelPaymentTerm,
		//@Inject('ChannelSamplePreferenceTypeRepository') private readonly channelSamplePreferenceTypeRepository: typeof ChannelSamplePreferenceType,
		@Inject('ChannelSharePreferenceRepository') private readonly channelSharePreferenceRepository: typeof ChannelSharePreference,

		@Inject('ChannelSubscriptionRepository') private readonly channelSubscriptionRepository: typeof ChannelSubscription,
		@Inject('SubscriptionPlanRepository') private readonly subscriptionPlanRepository: typeof SubscriptionPlan,
		@Inject('SubscriptionRepository') private readonly subscriptionRepository: typeof Subscription,
		@Inject('ZoneRepository') private readonly zoneRepository: typeof Zone,
		@Inject('UserWalletAmountRepository') private readonly userWalletAmountRepository: typeof UserWalletAmount,
		

		@Inject('ProductRepository') private readonly productRepository: typeof Product,
		@Inject('CompanyMediaImagesRepository') private readonly companyMediaImagesRepository: typeof CompanyMediaImages,
		@Inject('EndCustomersB2bRepository') private readonly endCustomersB2bRepository: typeof EndCustomersB2b,
		@Inject('GuestRequirementRepository') private readonly guestRequirementRepository: typeof GuestRequirement,
        private context: ContextService,
    ){}
    async getProfileList(typeId,channelId){       
        
    //     let user = req.user;
    //    console.log("users:",user);


    let query='select "order","icon","heading","desc",rat."value", '
    + '(select "isCompleted" from "tblProfileList" where "ratingId"=rat."order" and "channelId"=:channelId and "deletedAt" is null) as "isCompleted" '
    + 'from "tblProfileCreationRatings" as rat where rat."channelType"=:typeId and rat."deletedAt" is null  ';

    let level=await this.sequelizeInstance.query(query, {
        type: QueryTypes.SELECT,
        replacements: {
          channelId: channelId,
          typeId:typeId    
      }
        }).then(results => {        
			let status=results.filter(x => x.isCompleted).length;
			if((typeId==3||typeId==4||typeId==5)&&status==2)
			{
				results=results.map(x => {
					x.isCompleted=true;
					return x;
				})
			}
           return results;
        }); 


	


    
        return level;
    }

	async getUserStatus(channelId, typeId) {


		let count = 'select count(*) as profile_completion_count from "tblProfileList" where "isCompleted"=true and  "channelId"=:channelId and "ratingId" in(:ratingId) and "deletedAt" is null';

		let profile_completion_count = await this.sequelizeInstance.query(count, {
			type: QueryTypes.SELECT,
			replacements: {
				channelId: channelId,
				ratingId: typeId == 2 ? [1, 2, 3, 5] : [1, 2]
			}
		}).then(x => {
			return x[0].profile_completion_count;
		});

		return profile_completion_count;
	}

    async save_Company_Profile(data){

		let ChannelSpecialityObject = <Array<ChannelSpeciality>>[];
        
        let channelDetail ={
            estYear: data.yearEstablished,
            startup: data.areYouStartUp,
            empCntRange: data.no_of_staffs,
            turnoverRange: data.annualTurnOver,
            maturitylevel: data.maturitylevel,
            channelLogoId: +data.companyLogo,
            detailDesc: data.tweetPitch,
			companyDesc: data?.companyDescription,
            introvideoId: data.introVideo,            
            no_of_skus:(data.typeId==2)?data.no_of_skus:0,
            storeCount:(data.typeId==4)?data.no_of_skus:null,
            }            
            //console.log("vat:",data.vatNumber);
        let certificate=[];
        if(data.companyRegNumber!='')
            certificate.push({channelId:data.channelId,cerificateTypeId:1,cerificateName:data.certificateName,cerificateNumber:data.companyRegNumber});
        if(data.vatNumber!=null&&data.vatNumber!='')
            certificate.push({channelId:data.channelId,cerificateTypeId:2,cerificateName:'VAT',cerificateNumber:data.vatNumber});

        return this.sequelizeInstance.transaction(async transaction => {
            return await this.channelCertiRepository.bulkCreate(certificate, {               
                transaction,
            }).then(async (certiObject) => {
                await this.channelDetailRepository.update(channelDetail, {
                    where: {
                        channelId: data.channelId
                    },
                    returning: true,
                    transaction
                }); 
               
                if(data.typeId==3 && data.relevantDistributorProfile) 
                {
                    
                // update product group & category for channel
                let ChannelProfileTypes: ChannelCustomerProfile[] = await this.setChannelProfile(data.relevantDistributorProfile, +data.channelId);
                await this.channelCustProfileRepository.bulkCreate(ChannelProfileTypes, { returning: true, transaction });

				await this.channelRepository.update({profileTypeId: data.relevantDistributorProfile[0].profileTypeId}, {
					where:{
						channelId: data.channelId
					}
				});

                // let ChannelGroupObject: ChannelProductGroup[] = await this.setChannelProductGroup(data.productGroup, +data.channelId);
                // await this.channelProductGroupRepository.bulkCreate(ChannelGroupObject, { returning: true, transaction });

                // let ChannelCatObject: ChannelProductCategorie[] = await this.setChannelProductCat(data.productCat, +data.channelId);
                // await this.channelCatRepository.bulkCreate(ChannelCatObject, { returning: true, transaction });
                }

            //saving brand
            if (data.brands.length > 0) {
                let channelBrandObject = await this.saveBrand(data.brands, data.channelId, transaction);
                
                    if (channelBrandObject.channelBrandArrayObject.length > 0) {
                        await this.channelBrandRepository.bulkCreate(channelBrandObject.channelBrandArrayObject, { returning: true, transaction });
                        await this.brandRepository.update({
                            isAssigned: '1'
                        }, {
                                where: {
                                    id: {
                                        [Op.in]: channelBrandObject.brandIds
                                    }
                                },
                                returning: true, transaction
                            }
                        );
                    }
                }


                //update product brand & customerprofile for channel
                // let ChannelBrandObject: ChannelBrand[] = await this.setChannelBrand(data.brands, +data.channelId);
                // await this.channelBrandRepository.bulkCreate(ChannelBrandObject, { returning: true, transaction });

                let ChannelProfileTypes: ChannelCustomerProfile[] = await this.setChannelProfile(data.relevantChannelProfile, +data.channelId);
                await this.channelCustProfileRepository.bulkCreate(ChannelProfileTypes, { returning: true, transaction });

				if(data.typeId=='4'){
					await this.channelRepository.update({profileTypeId: data.relevantChannelProfile[0].profileTypeId}, {
						where: {
							channelId: data.channelId
						}
					});
				}



				//Saving additional fields added in initial profile

					let channelContacts = await this.contactRepository.scope('basicInfom').findAll({
							where: {
								deletedAt: null,
								contactId: { [Op.ne]: data.contactId },
								channelId: data.channelId
							}
						});
	
						let contacts = [];
						contacts.push(data.contactId);
						if (channelContacts.length > 0) {
							channelContacts.map(function (data) {
								contacts.push(data.contactId);
							});
						}
	
	
	if (ChannelSpecialityObject.length > 0) {
	
							let speciality = await this.channelSpecialityRepository.findAll({
								where: {
									channelId: data.channelId,
									deletedAt: null
								}
							});
	
							let ChannelSpecialityIds = await this.tradeInformationInBound.setProductGroupIds(speciality);
							let ChannelSpecialityCurrentIds = [];
							let ChannelSpecialityCreateObject = [];
							for (let kk = 0; kk < ChannelSpecialityObject.length; kk++) {
								if (ChannelSpecialityIds.includes(ChannelSpecialityObject[kk].productGroupId)) {
									ChannelSpecialityCurrentIds.push(ChannelSpecialityObject[kk].productGroupId);
								} else {
									ChannelSpecialityCreateObject.push(ChannelSpecialityObject[kk]);
									ChannelSpecialityCurrentIds.push(ChannelSpecialityObject[kk].productGroupId);
								}
							}
							if (ChannelSpecialityCreateObject.length > 0) {
								await this.channelSpecialityRepository.bulkCreate(ChannelSpecialityCreateObject, { returning: true, transaction });
							}
	
							await this.channelSpecialityRepository.update({ deletedAt: new Date() }, {
								where: {
									channelId: data.channelId,
									productGroupId: { [Op.notIn]: ChannelSpecialityCurrentIds },
									deletedAt: null
								}, returning: true, transaction
							});
						}


			if (data.social_linkedin_id) {
				// update channel social linkedin profile details to channel social link table
				await this.socialLinkRepository.update({
					socialSiteId: appconstant.SOCIAL_SITE_TYPE_LINKEDIN,
					channelId: data.channelId,
					channelSocialLink: data.social_linkedin_url
				},
					{
						where: { channelSocialLinkId: data.social_linkedin_id },
						returning: true, transaction,
					});
			} else {
				// save channel social linkedin profile details to channel social link table
				if (data.social_linkedin_url && Object.keys(data.social_linkedin_url).length > 0) {
					await this.socialLinkRepository.create({
						socialSiteId: appconstant.SOCIAL_SITE_TYPE_LINKEDIN,
						channelId: data.channelId,
						channelSocialLink: data.social_linkedin_url
					},
						{
							transaction
						});
				}
			}

			// save channel social google plus profile details to channel social link table
			if (data.social_googleplus_id) {
				// update channel social google plus profile details to channel social link table
				await this.socialLinkRepository.update({
					socialSiteId: appconstant.SOCIAL_SITE_TYPE_GOOGLEPLUS,
					channelId: data.channelId,
					channelSocialLink: data.social_googleplus_url
				},
					{
						where: { channelSocialLinkId: data.social_googleplus_id },
						returning: true, transaction,
					});
			} else {
				// save channel social google plus profile details to channel social link table
				if (data.social_googleplus_url && Object.keys(data.social_googleplus_url).length > 0) {
					await this.socialLinkRepository.create({
						socialSiteId: appconstant.SOCIAL_SITE_TYPE_GOOGLEPLUS,
						channelId: data.channelId,
						channelSocialLink: data.social_googleplus_url
					},
						{
							 transaction
						});
				}
			}

			if (data.social_instagram_id) {
				// update channel social instagram profile details to channel social link table
				await this.socialLinkRepository.update({
					channelId: data.channelId,
					channelSocialLink: data.social_instagram_url
				},
					{
						where: { channelSocialLinkId: data.social_instagram_id },
						returning: true, transaction,
					});
			} else {
				// save channel social instagram profile details to channel social link table
				if (data.social_instagram_url && Object.keys(data.social_instagram_url).length > 0) {
					await this.socialLinkRepository.create({
						socialSiteId: appconstant.SOCIAL_SITE_TYPE_INSTAGRAM,
						channelId: data.channelId,
						channelSocialLink: data.social_instagram_url
					},
						{
							transaction
						});
				}
			}

			if (data.social_twitter_id) {
				// update channel social instagram profile details to channel social link table
				await this.socialLinkRepository.update({
					channelId: data.channelId,
					channelSocialLink: data.social_twitter_url
				},
					{
						where: { channelSocialLinkId: data.social_twitter_id },
						returning: true, transaction,
					});
			} else {
				// save channel social instagram profile details to channel social link table
				if (data.social_twitter_url && Object.keys(data.social_twitter_url).length > 0) {
					await this.socialLinkRepository.create({
						socialSiteId: appconstant.SOCIAL_SITE_TYPE_TWITTER,
						channelId: data.channelId,
						channelSocialLink: data.social_twitter_url
					},
						{
							transaction
						});
				}
			}

			if (data.social_facebook_id) {
				// update channel social instagram profile details to channel social link table
				await this.socialLinkRepository.update({
					channelId: data.channelId,
					channelSocialLink: data.social_facebook_url
				},
					{
						where: { channelSocialLinkId: data.social_facebook_id },
						returning: true, transaction,
					});
			} else {
				// save channel social instagram profile details to channel social link table
				if (data.social_facebook_url && Object.keys(data.social_facebook_url).length > 0) {
					await this.socialLinkRepository.create({
						socialSiteId: appconstant.SOCIAL_SITE_TYPE_FACEBOOK,
						channelId: data.channelId,
						channelSocialLink: data.social_facebook_url
					},
						{
							transaction
						});
				}
			}

			if (data.B2B_url && Object.keys(data.B2B_url).length > 0) {

				for (let ii = 0; ii < data.B2B_url.length; ii++) {
					if (data.B2B_url[ii].channel_B2B_id) {
						// update channel B2B details to channel B2B table

						await this.channelB2BRepository.update({
							channelId: data.channelId,
							channelB2B: data.B2B_url[ii].B2B_url
						},
							{
								where: { channelB2BId: data.B2B_url[ii].channel_B2B_id },
								returning: true, transaction,
							});
					} else {
						// save channel B2B details to channel B2B table

						await this.channelB2BRepository.create({
							channelId: data.channelId,
							channelB2B: data.B2B_url[ii].B2B_url
						},
							{
								 transaction
							});
					}
				}
			}

			if (data.channel_document && Object.keys(data.channel_document).length > 0) {
				for (let ii = 0; ii < data.channel_document.length; ii++) {

							// let updatedDocument = await this.mediaRepository.create({
					// 	documentPath: data.channel_document[ii].document,
					// 	documentTypeId: appconstant.DOCUMENTTYPE_DOC,
					// 	dirName: appconstant.FOLDER_PATH_COMPANY_DOCUMENT
					// },
					// 	{
					// 		 transaction,
					// 	});

					if (data.channel_document[ii].channelDocumentId) {
						// update channel documents
						await this.channelDocumentRepository.update({
							channelId: data.channelId,
							documentId: data.channel_document[ii].documentId,
							name: _.defaultTo(data.channel_document[ii].documentName, ''),
							description: _.defaultTo(data.channel_document[ii].documentDescription, ''),
							updatedBy: data.userId
						},
							{
								where: { channelDocumentId: data.channel_document[ii].channelDocumentId },
								returning: true, transaction,
							});
					} else {
						// save channel document
						await this.channelDocumentRepository.create({
							channelId: data.channelId,
							documentId: data.channel_document[ii].documentId,
							name: _.defaultTo(data.channel_document[ii].documentName, ''),
							description: _.defaultTo(data.channel_document[ii].documentDescription, ''),
							updatedBy: data.userId
						},
							{
								 transaction
							});
					}
				}
			}


			if (data.award_details && Object.keys(data.award_details).length > 0) {
				for (let ii = 0; ii < data.award_details.length; ii++) {
					let valid_upto = (data.award_details[ii].valid_upto) ? data.award_details[ii].valid_upto : null;
					if (data.award_details[ii].channel_award_id) {
						//save awards image
						let documentId = null, documentPath = null;

						if (data.award_details[ii].image) {

							if(data.award_details[ii].image.includes('/product/awards/')){
								documentId = data.award_details[ii].documentId;
							}else{

							let media = <Media>{};
							media.documentTypeId = appconstant.DOCUMENTTYPE_IMAGE;
							media.documentPath = (data.award_details[ii].image) ? data.award_details[ii].image : null;
							media.dirName = appconstant.FOLDER_PATH_PRODUCT_AWARDS;
							
							let logo = await this.imageCreate(media, transaction);
							// data.award_details[ii].documentId = logo.documentId;
							documentId = logo.documentId;

							//Deleting existing documentId
							if(data.award_details[ii].documentId){
							await this.mediaRepository.update({
								deletedAt: new Date()
							}, {where: {documentId: data.award_details[ii].documentId}})
							}
							}
						} 

						// update award details to channel award table
						await this.awardRepository.update({
							channelId: data.channelId,
							awardDetail: 'channel',
							issuedBy: 'ch',
							// validUpto: valid_upto,
							// isLifeTime: data.award_details[ii].isLifeTime,
							documentId: documentId,
						},
							{
								where: { channelAwardId: data.award_details[ii].channel_award_id },
								returning: true, transaction,
							});

							console.log('awardsave');
					} else {
						//save awards image
						let documentId = null;

						if (data.award_details[ii].image) {
								let media = <Media>{};
							media.documentTypeId = appconstant.DOCUMENTTYPE_IMAGE;
							media.documentPath = (data.award_details[ii].image) ? data.award_details[ii].image : null;
							media.dirName = appconstant.FOLDER_PATH_PRODUCT_AWARDS;
							
							let logo = await this.imageCreate(media, transaction);
							// data.award_details[ii].documentId = logo.documentId;
							documentId = logo.documentId;
						}

						// save award details to channel award table
						await this.awardRepository.create({
							channelId: data.channelId,
							awardDetail: 'channel',
							issuedBy: 'ch',
							// validUpto: valid_upto,
							// isLifeTime: data.award_details[ii].isLifeTime,
							documentId: documentId,
						},
							{
								 transaction
							});
							console.log('awardsave1');
					}
				}
			}

			//Company Images
			if (data.companyImages && Object.keys(data.companyImages).length > 0) {
				for (let ii = 0; ii < data.companyImages.length; ii++) {
						// save channel document
						if(!data.companyImages[ii].companyMediaId){
						await this.companyMediaImagesRepository.create({
							channelId: data.channelId,
							documentId: data.companyImages[ii].documentId,
							contactId:data.contactId,
							updatedBy: data.userId
						},
							{
								 transaction
							});
							}
				}
			}

			if (data.productFamilies?.length > 0) {

				let productGroup = await this.channelProductGroupRepository.findAll({
					where: {
						channelId: data.channelId,
						deletedAt: null
					}
				});

				let ChannelProductGroupIds = await this.tradeInformationInBound.setProductGroupIds(productGroup);
				let ChannelProductGroupObject: ChannelProductGroup[] = await this.tradeInformationInBound.setProductGroup(data.productFamilies, data.channelId);
				let ChannelProductGroupCurrentIds = [];
				let ChannelProductGroupCreateObject = [];

				/* Account User contact update */
				// let ContactProductGroupIds = await this.tradeInformationInBound.setProductGroupIds(contact.productGroups);
				let ContactProductGroupObject: ContactProductGroup[] = await this.tradeInformationInBound.setContactProductGroup(data.productFamilies, data.contactId);
				let ContactProductGroupCurrentIds = [];
				// let ContactProductGroupCreateObject = [];

				for (let kk = 0; kk < ChannelProductGroupObject.length; kk++) {
					if (ChannelProductGroupIds.includes(ChannelProductGroupObject[kk].productGroupId)) {
						ChannelProductGroupCurrentIds.push(ChannelProductGroupObject[kk].productGroupId);
					} else {
						ChannelProductGroupCreateObject.push(ChannelProductGroupObject[kk]);
						ChannelProductGroupCurrentIds.push(ChannelProductGroupObject[kk].productGroupId);
					}
					// if (ContactProductGroupIds.includes(ContactProductGroupObject[kk].productGroupId)) {
					// 	ContactProductGroupCurrentIds.push(ContactProductGroupObject[kk].productGroupId);
					// } else {
					// 	// ContactProductGroupCreateObject.push(ContactProductGroupObject[kk]);
					// 	ContactProductGroupCurrentIds.push(ContactProductGroupObject[kk].productGroupId);
					// }

					ContactProductGroupCurrentIds.push(ContactProductGroupObject[kk].productGroupId);
				}
				if (ChannelProductGroupCreateObject.length > 0) {
					await this.channelProductGroupRepository.bulkCreate(ChannelProductGroupCreateObject, { returning: true, transaction });
				}

				// if (ContactProductGroupCreateObject.length > 0) {
				// 	await this.contactProductGroupRepository.bulkCreate(ContactProductGroupCreateObject, { returning: true, transaction });
				// }

				await this.channelProductGroupRepository.update({ deletedAt: new Date() }, {
					where: {
						channelId: data.channelId,
						productGroupId: { [Op.notIn]: ChannelProductGroupCurrentIds },
						deletedAt: null
					}, returning: true, transaction
				});

				await this.contactGroupRepository.update({ deletedAt: new Date() }, {
					where: {
						contactId: { [Op.in]: contacts },
						productGroupId: { [Op.notIn]: ContactProductGroupCurrentIds },
						deletedAt: null
					}, returning: true, transaction
				});
			}

			if (data.productCategories?.length > 0) {

				let productCategory = await this.channelProductCategorieRepository.findAll({
					where: {
						channelId: data.channelId,
						deletedAt: null
					}
				});

				let productCategoriesIds = await this.tradeInformationInBound.setProductCategoryIds(productCategory);
				let ProductCategorieObject: ChannelProductCategorie[] = await this.tradeInformationInBound.setProductCategories(data.productCategories, data.channelId);
				let ChannelProductCategoriesCurrentIds = [];
				let ChannelProductCategoriesCreateObject = [];

				/* Account User contact update */
				// let contactProductCategoriesIds = await this.tradeInformationInBound.setProductCategoryIds(contact.productCategories);
				let contactProductCategorieObject: ContactProductCategory[] = await this.tradeInformationInBound.setContactProductCategories(data.productCategories, data.contactId);
				let ContactProductCategoriesCurrentIds = [];
				// let ContactProductCategoriesCreateObject = [];

				for (let kk = 0; kk < ProductCategorieObject.length; kk++) {
					if (productCategoriesIds.includes(ProductCategorieObject[kk].productCategorieId)) {
						ChannelProductCategoriesCurrentIds.push(ProductCategorieObject[kk].productCategorieId);
					} else {
						ChannelProductCategoriesCreateObject.push(ProductCategorieObject[kk]);
						ChannelProductCategoriesCurrentIds.push(ProductCategorieObject[kk].productCategorieId);
					}
					// if (contactProductCategoriesIds.includes(contactProductCategorieObject[kk].productCategorieId)) {
					// 	ContactProductCategoriesCurrentIds.push(contactProductCategorieObject[kk].productCategorieId);
					// } else {
					// 	// ContactProductCategoriesCreateObject.push(contactProductCategorieObject[kk]);
					// 	ContactProductCategoriesCurrentIds.push(contactProductCategorieObject[kk].productCategorieId);
					// }

					ContactProductCategoriesCurrentIds.push(contactProductCategorieObject[kk].productCategorieId);
				}
				if (ChannelProductCategoriesCreateObject.length > 0) {
					await this.channelProductCategorieRepository.bulkCreate(ChannelProductCategoriesCreateObject, { returning: true, transaction });
				}

				// if (ContactProductCategoriesCreateObject.length > 0) {
				// 	await this.contactProductCategoryRepository.bulkCreate(ContactProductCategoriesCreateObject, { returning: true, transaction });
				// }

				await this.channelProductCategorieRepository.update({ deletedAt: new Date() }, {
					where: {
						channelId: data.channelId,
						productCategorieId: { [Op.notIn]: ChannelProductCategoriesCurrentIds },
						deletedAt: null
					}, returning: true, transaction
				});

				await this.contactCatRepository.update({ deletedAt: new Date() }, {
					where: {
						contactId: { [Op.in]: contacts },
						productCategorieId: { [Op.notIn]: ContactProductCategoriesCurrentIds },
						deletedAt: null
					}, returning: true, transaction
				});
			}

			// Code fof insert completed profile to increase profileStrength
			let rating=await this.profileRepository.create({
				channelId : +data.channelId,
				ratingId : +data.ratingId,
				isCompleted : true,
				createdAt : new Date()
			}, { transaction });
			console.log('rating', rating);
                return certiObject;
            })

        })
        .catch((err) => {
            console.log(err)
            throw new ProcessException('account:singup:tryAgain');
        });
        
        //console.log("profileData:",data);

    }

    async save_User_Profile(data){
        //console.log("data:",data);

        let user = data;
        let typeid=user.channelTypeId;

        let channelLocation=[];

        if(data.sellingRegion)
        data.sellingRegion.map(x => {
            channelLocation.push({channelId: data.channelId,locationType:typeid==3?'SELLING':typeid==4?'RETAIL':typeid==5?'RESELLING':'SELLING',regionCountryJCTId:x}) })               

        if(data.targetRegion)
        data.targetRegion.map(x => {
            channelLocation.push({channelId: data.channelId,locationType:'TARGET',regionCountryJCTId:x})  })               
        
        return this.sequelizeInstance.transaction(async transaction => {
            return await this.channelLocationRepository.bulkCreate(channelLocation, {                                
                transaction,
            }).then(async (location) => {

                await this.userRepository.update({suggestion_comm:data.suggestions,invitation_comm:data.networkInvitation}, {
                    where: {
                        contactId: data.contactId
                    },
                    returning: true,
                    transaction
                }) 

                await this.contactRepository.update({documentId:data.companyLogo}, {
                    where: {
                        contactId: data.contactId
                    },
                    returning: true,
                    transaction
                }) 
                                                
            // update product brand & catego tblChannelCustomerProfile
                let ContactGroupObject: InterestedProductGroup[] = await this.setContactProductGroup(data.productGroup, +data.contactId,+data.channelId);
                await this.intrestedGroupRepository.bulkCreate(ContactGroupObject, { returning: true, transaction });

                let ContactCatObject: InterestedProductCategory[] = await this.setContactProductCat(data.productCat, +data.contactId,+data.channelId);
                await this.intrestedCatRepository.bulkCreate(ContactCatObject, { returning: true, transaction });

            // Code fof insert completed profile to increase profileStrength
                await this.profileRepository.create({
                    channelId : data.channelId,
                    ratingId : data.ratingId,
                    isCompleted : true,
                    createdAt : new Date()
                }, { transaction });

                return location;
            })
        })
        .catch((err) => {
            console.log(err)
            throw new ProcessException('account:singup:tryAgain');
        });
        
        //console.log("profileData:",data);

    }

    async setChannelBrand(channelBrand: any, channelId: number): Promise<ChannelBrand[]> {
		try {
			let resultObject = <Array<ChannelBrand>>[];
			if (channelBrand) {
				for (let ii = 0; ii < channelBrand.length; ii++) {
					let innerObject = <ChannelBrand>{};
					innerObject.channelId = channelId;
					innerObject.brandId = channelBrand[ii];
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:channelBrand');
		}
    }    

    async setChannelProfile(channelProfile: any, channelId: number): Promise<ChannelCustomerProfile[]> {
		try {
			let resultObject = <Array<ChannelCustomerProfile>>[];
			if (channelProfile) {
				for (let ii = 0; ii < channelProfile.length; ii++) {
					let innerObject = <ChannelCustomerProfile>{};
					innerObject.channelId = channelId;
                    innerObject.profileTypeId = channelProfile[ii].profileTypeId;
                    innerObject.profileTypeClassificationId = channelProfile[ii].profileTypeClassificationId;
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:channelProfile');
		}
    }
    
    async setContactProductGroup(productGroup: any, contactId: number,channelId:number): Promise<InterestedProductGroup[]> {
		try {
            console.log("producdtGroup:",productGroup);
            productGroup=_.uniq(productGroup);
            
			let resultObject = <Array<InterestedProductGroup>>[];
			if (productGroup) {
				for (let ii = 0; ii < productGroup.length; ii++) {
					let innerObject = <InterestedProductGroup>{};
					innerObject.contactId = contactId;
					innerObject.channelId = channelId;
					innerObject.productGroupId = productGroup[ii];
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
            console.log("err:",e);
			throw new ProcessException('channel:objectCreation:channelBrand');
		}
    }
    
    async setContactProductCat(productCat: any, contactId: number,channelId:number): Promise<InterestedProductCategory[]> {
		try {
			let resultObject = <Array<InterestedProductCategory>>[];
			if (productCat) {
				for (let ii = 0; ii < productCat.length; ii++) {
					let innerObject = <InterestedProductCategory>{};
					innerObject.contactId = contactId;
					innerObject.channelId = channelId;
					innerObject.productCategorieId = productCat[ii];
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:channelBrand');
		}
    }

    async setChannelProductGroup(productGroup: any, channelId: number): Promise<ChannelProductGroup[]> {
		try {
            console.log("producdtGroup:",productGroup);
            productGroup=_.uniq(productGroup);
            
			let resultObject = <Array<ChannelProductGroup>>[];
			if (productGroup) {
				for (let ii = 0; ii < productGroup.length; ii++) {
					let innerObject = <ChannelProductGroup>{};
					innerObject.channelId = channelId;
					innerObject.productGroupId = productGroup[ii];
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
            console.log("err:",e);
			throw new ProcessException('channel:objectCreation:channelBrand');
		}
    }
    
    async setChannelProductCat(productCat: any, channelId: number): Promise<ChannelProductCategorie[]> {
		try {
			let resultObject = <Array<ChannelProductCategorie>>[];
			if (productCat) {
				for (let ii = 0; ii < productCat.length; ii++) {
					let innerObject = <ChannelProductCategorie>{};
					innerObject.channelId = channelId;
					innerObject.productCategorieId = productCat[ii];
					innerObject.createdAt = new Date();
					resultObject.push(innerObject);
				}
			}

			return resultObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:channelBrand');
		}
    }

    
    public async getInitialProfile(data){
        let user = data;
		// console.log('user', user);
		let distProfile;
    let channelTypeId=user.channelTypeId;
	let businessType=user.businessType;
	let isOEM=user.channel.isOEM;
    let channelId=user.channelId;
    let type=[];
	let btype=[];

	if(businessType==1)
		btype=[1,3];
	else if(businessType==2)
		btype=[2,3];
	else if(businessType==3)
		btype=[1,2,3];
	else	
		btype=[-1]

    if((channelTypeId==2||channelTypeId==3)&&businessType==1)
	   type=[4,5];	
	else if((channelTypeId==2||channelTypeId==3)&&businessType==2)	
        type=[4, 5];			
	else if((channelTypeId==2||channelTypeId==3)&&businessType==3)	
        type=[4,5];    			
    else if(channelTypeId==4)
        type=[4];
	else if(channelTypeId==5)
        type=[5];
	else	
		type=[-1];
        
    //Code for get productgroup and its categories of company for company profile addnew
    let query='select pg."productGroupId" as groupId,pg."productGroup" as groupname,pc."productCategorieId" as childId,pc."productCategorie" as childname,cast(false as boolean) as status from "tblProductGroup" as pg inner join' 
        +' "tblProductCategories" as pc on pg."productGroupId"=pc."productGroupId" where pg."deletedAt" is null and pg."productGroup"!=\'\' and pg."businessType" in(:businessType) ';
      
        let productGroup=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,  
			replacements: {
                businessType: btype     
            }         
            }).then(results => {        
               return results;
            });

    //     // let productGroup = await this.productGroupRepository.scope('productInfo').findAll({
        //     where: {
        //         deletedAt:null
        //     },
        //     include: [
        //         {
        //           model:this.productCatRepository.scope('productCategoryInfo'),                                      
        //           where: { deletedAt: null},
        //           //required:true
        //         }],
        // });
        
        
        // //Code for get productBrand of company for company profile addnew
        let url='https://d35w8j22j5uly8.cloudfront.net/';//process.env.CDN_URL;
      query ='select "brandName" as name,"id",doc."documentId" as doc_id,concat(\''+url+'\',doc."documentPath") as documentPath from "tblBrand" as br inner join "tblDocument" as doc on doc."documentId"=br."documentId" where br."deletedAt" is null';

      let brand=await this.sequelizeInstance.query(query, {
        type: QueryTypes.SELECT,        
        }).then(results => {        
           return results;
        });
        // let productBrand = await this.brandRepository.scope('basicInfom').findAll({
        //     where: {
        //         deletedAt:null
        //     },
            
        // });

        // //Code for get profile data of company for company profile addnew

		if(channelTypeId == 4 || channelTypeId ==5){
			query='select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
			+'when ptc."channelTypeId"=2 then \'vendor\' when ptc."channelTypeId"=5 then \'Reseller\' end as groupname,pt."profileType" as childname,'
				+ 'pt."profileTypeId" as childId,cast(false as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" where ptc."channelTypeId" in(:typeId)';
		}else{
			query='select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
			+'when ptc."channelTypeId"=2 then \'vendor\' when ptc."channelTypeId"=5 then \'Reseller\' end as groupname,pt."profileType" as childname,'
				+ 'pt."profileTypeId" as childId,cast(false as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" where ptc."channelTypeId" in(:typeId)';
		}
		let profileList=await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT,           
			replacements: {
				typeId: type,
				//businessType: (!isOEM||isOEM==3)?[1,2,3]:isOEM
			}
			}).then(results => {        
			   return results;
			});
		if(channelTypeId==3)
		{
			query='select case when ptc."channelTypeId"=3 then \'Distributor\' when ptc."channelTypeId"=4 then \'Retailer\' '
			+'when ptc."channelTypeId"=2 then \'vendor\' when ptc."channelTypeId"=5 then \'Reseller\' end as groupname,pt."profileType" as childname,'
			+'pt."profileTypeId" as childId,cast(false as boolean) as status,ptc."profileTypeClassificationId" as profileTypeClassificationId from "tblProfileType" as pt inner join "tblProfileTypeClassification" as ptc on pt."profileTypeId"=ptc."profileTypeId" where ptc."channelTypeId" in(:typeId);';
		  
			distProfile=await this.sequelizeInstance.query(query, {
				type: QueryTypes.SELECT,           
				replacements: {
					typeId: [3]
				}
				}).then(results => {        
				   return results;
				});
		}

        // let profileList = await this.profileTypeRepository.scope('profileTypeInfom').findAll({
        //     where: {
        //         deletedAt:null
        //     },
        //     include: [
        //         {
        //           model:this.profileTypeClassiRepository.scope('profileTypeClassificationInfom'),
        //           where: { deletedAt: null},//channelTypeId:[3,2]},
        //           required:true
        //         }],
        // });   

        //Code for get registration number of company for company profile addnew

    query = 'select "registrationcertificate","isVat" from "tblCountry" as cty inner join "tblChannel" as ch on ch."countryId"=cty."countryId" where ch."channelId"=:channelId';
      
      let country=await this.sequelizeInstance.query(query, {
      type: QueryTypes.SELECT,
      replacements: {
        channelId: channelId     
    }
      }).then(results => {        
         return results;
      }); 
  
    

        let result={
            productGroup:productGroup,
            productBrand:brand,
            profileList:profileList,
			distProfile:distProfile,
            country:country
        }

        return result;    
    }

	public async getVat(countryId){
	let  query = 'select "registrationcertificate","isVat" from "tblCountry" as cty where cty."countryId"=:countryId';
      
      let country=await this.sequelizeInstance.query(query, {
      type: QueryTypes.SELECT,
      replacements: {
        countryId: countryId
    }
      }).then(results => {        
         return results;
      }); 

    let result={        
      country:country
    }

        return result;    
    }

    public async getContactProduct(contactId,channelId){
    //     let user = data;
    // let channelTypeId=user.channelTypeId;
    // let channelId=user.channelId;    
        
    //Code for get productgroup and its categories of company for company profile addnew
    	let query='select pg."productGroupId" as groupId,(select "productGroup" from "tblProductGroup" where "productGroupId"=pg."productGroupId") as groupname,pc."productCategorieId" as childId,pg."productCategorie" as childname,cast(true as boolean) as status from '
		+ ' "tblInterestedProductCategory" as pc inner join "tblProductCategories" as pg on pg."productCategorieId"=pc."productCategorieId"  where pc."contactId"=:contactId and pg."deletedAt" is null and pc."deletedAt" is null';
      
        let productGroup=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,           
            replacements: {contactId: contactId}
            }).then(results => {        
               return results;
            });    

			if(productGroup.length==0)
			{
				query='select pg."productGroupId" as groupId,(select "productGroup" from "tblProductGroup" where "productGroupId"=pg."productGroupId") as groupname,pc."productCategorieId" as childId,pg."productCategorie" as childname,cast(true as boolean) as status from '
			+ ' "tblChannelProductCategorie" as pc inner join "tblProductCategories" as pg on pg."productCategorieId"=pc."productCategorieId"  where pc."channelId"=:channelId and pg."deletedAt" is null and pc."deletedAt" is null';
		  
			productGroup=await this.sequelizeInstance.query(query, {
				type: QueryTypes.SELECT,           
				replacements: {channelId: channelId}
				}).then(results => {        
				   return results;
				});    
			}    

        let result={
            productGroup:productGroup,            
            //intProductGroup:intProductGroup
        }

        return result;    
    }

    public async getContactLocation(user){
            let typeId = user.channelTypeId;
        // let channelTypeId=user.channelTypeId;
        // let channelId=user.channelId;
        
        let type = [];
        if(typeId == 2)
            type = ['TARGET']
        else if(typeId == 3)
            type = ['TARGET', 'SELLING']
        else if(typeId == 4)
            type = ['RETAIL']
		else if(typeId == 5)
			type = ['RESELLING']

        //Code for get contactLocations and its categories of company for company profile addnew
        let query='select rc."regionId" as groupId ,(select "region" from "tblRegion" where "regionId"=rc."regionId") as groupname,rc."countryId" as "childId",(select "country" from "tblCountry" where "countryId"=rc."countryId") as childname,rc."regionCountryJCTId", ' + 
        ' cast(true as boolean) as status, tc."locationType" from "tblContactLocation" as tc inner join "tblRegionCountryJCT" as rc on tc."regionCountryJCTId"=rc."regionCountryJCTId" where tc."deletedAt" is null and tc."contactId"=:contactId and tc."locationType" in (:type)';
          
            let contactLocation=await this.sequelizeInstance.query(query, {
                type: QueryTypes.SELECT,           
                replacements: {contactId: user.contactId,
                type: type}
                }).then(results => {        
                   return results;
                });    
		
		if(contactLocation.length==0)
		{
			query='select rc."regionId" as groupId ,(select "region" from "tblRegion" where "regionId"=rc."regionId") as groupname,rc."countryId" as "childId",(select "country" from "tblCountry" where "countryId"=rc."countryId") as childname,rc."regionCountryJCTId", ' + 
        ' cast(true as boolean) as status, tc."locationType" from "tblChannelLocation" as tc inner join "tblRegionCountryJCT" as rc on tc."regionCountryJCTId"=rc."regionCountryJCTId" where tc."deletedAt" is null and tc."channelId"=:channelId and tc."locationType" in (:type)';
          
            contactLocation=await this.sequelizeInstance.query(query, {
                type: QueryTypes.SELECT,           
                replacements: {channelId: user.channelId,
                type: type}
                }).then(results => {        
                   return results;
                });    
		}
			
    
        
    
            let result={
                contactLocation: contactLocation
            }
    
            return result;    
        }


    public async getUserProfile(channelId, channelTypeId){

        //Code for get region list for user profile addnew
        let query='select "regionId" as id,"region" as name from "tblRegion" where "deletedAt" is null';

		// let zone = [];
		// if(channelTypeId == '2')
		// zone = [1, 3];
		// else
		// zone = [1];

        query='select tr."regionId" as groupId,tr."region" as groupname,cty."countryId" as "childId",cty."country" as childname,rc."regionCountryJCTId",cast(false as boolean) as status from "tblRegion" as tr  inner join "tblRegionCountryJCT" as rc on rc."regionId"=tr."regionId" '
            + 'inner join "tblCountry" as cty on cty."countryId"=rc."countryId" where tr."deletedAt" is null and tr."regionId" in (select "regionId" from "tblZoneRegionJCT" where "zoneId" in (1, 2, 3))';
      
        let regions=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,           
			// replacements: {
			// 	zoneId: zone
			// }
            }).then(results => {        
               return results;
            });

		query='select tr."regionId" as groupId,tr."region" as groupname,cty."countryId" as "childId",cty."country" as childname,rc."regionCountryJCTId",cast(false as boolean) as status from "tblRegion" as tr  inner join "tblRegionCountryJCT" as rc on rc."regionId"=tr."regionId" '
		+ 'inner join "tblCountry" as cty on cty."countryId"=rc."countryId" where tr."deletedAt" is null and tr."regionId" in (select "regionId" from "tblZoneRegionJCT" where "zoneId" in (1, 2, 3))';
      
        let regions1=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,           
            }).then(results => {        
               return results;
            });


        // let regions = await this.regionRepository.scope('zoneRegionJCT').findAll({
        //     where: {
        //         deletedAt:null
        //     }            
        // });            
        

        //Code for get productgroup and its categories of company for user profile addnew
        query='select  pg."productGroupId" as groupId,pg."productGroup" as groupname,pc."productCategorieId" as childId,pc."productCategorie" as childname,cast(true as boolean) as status from "tblProductGroup" as pg inner join '
        + '"tblProductCategories" as pc on pg."productGroupId"=pc."productGroupId" and pc."productCategorieId" in(select "productCategorieId" from "tblChannelProductCategorie" where "channelId"=:channelId) '
        + 'and pc."productGroupId" in(select "productGroupId" from "tblChannelProductGroup" where "channelId"=:channelId)';

            let productGroup=await this.sequelizeInstance.query(query, {
                type: QueryTypes.SELECT,  
                replacements: {
                    channelId: channelId
                }
                     
                }).then(results => {        
                   return results;
                });
    
        // let productGroup = await this.channelProductGroupRepository.scope('tradeinfo').findAll({
        //     where: {
        //         deletedAt:null,
        //         channelId:61
        //     },
        //     include: [
        //         {
        //           model:this.channelCatRepository.scope('channelDetail'),                                      
        //           where: { deletedAt: null,
        //             channelId:61    
        //            },
        //            include: [
        //             {
        //               model:this.productCatRepository.scope('productCategoryInfo'),                                      
        //               where: { deletedAt: null},
        //               required:true
        //             }],
        //           //required:true
        //         }],
        // });

        
        //Code for get country based on region for user profile addnew
        // query ='select cty."country" as name,cty."countryId" as id from "tblCountry" as cty inner join "tblRegionCountryJCT" as rc on rc."countryId"=cty."countryId" where rc."regionId"=:regionId';
        
        // let country=await sequelizeInstance.sequelize.query(query, {
        //     type: QueryTypes.SELECT,  
        //     replacements: {
        //         regionId: 4
        //     }
                 
        //     }).then(results => {        
        //        return results;
        //     });

        // let country = await this.countryRepository.findAll({
        //     attributes: [['country', 'name'], ['countryId', 'id']],
        //     where: {                
        //         deletedAt: null
        //     },
        //     include: [
        //         {
        //           model:this.regCtyJCTRepository,                  
        //           where: { regionId:2,deletedAt: null},
        //           required:true,
        //           attributes:[]
        //         }],
           
        // })

        let result={
            productGroup:productGroup,            
//            country:country,
            regions:regions,
            regions1:regions1

        }

        return result;    
    }

	public async getChannelProduct(channelId,businessType){

	let btype=[];
	if(businessType==1)
		btype=[1,3];
	else if(businessType==2)
		btype=[2,3];
	else if(businessType==3)
		btype=[1,2,3];
	else	
		btype=[-1];
        //Code for get productgroup and its categories of company for user profile addnew
        let query='select  pg."productGroupId" as groupId,pg."productGroup" as groupname,pc."productCategorieId" as childId,pc."productCategorie" as childname,cast(true as boolean) as status from "tblProductGroup" as pg inner join '
        + '"tblProductCategories" as pc on pg."productGroupId"=pc."productGroupId" and pc."productCategorieId" in(select "productCategorieId" from "tblChannelProductCategorie" where "channelId"=:channelId and "deletedAt" is null) '
        + 'and pc."productGroupId" in(select "productGroupId" from "tblChannelProductGroup" where "channelId"=:channelId) and pg."businessType" in(:businessType)' ;

            let productGroup=await this.sequelizeInstance.query(query, {
                type: QueryTypes.SELECT,  
                replacements: {
                    channelId: channelId,
					businessType:btype
                }
                     
                }).then(results => {        
                   return results;
                });
    
        
        let result={
            productGroup:productGroup
        }

        return result;    
    }

	public async getChannelBrand(channelId){

        //Code for get channel brand		
        let query='select concat(\''+process.env.CDN_URL+'\',(select "documentPath" from "tblDocument" where "documentId"=pg."documentId"))as "documentpath",cpg."brandId" as id,pg."brandName" as name,0 as "isNew" from "tblChannelBrand" as cpg inner join "tblBrand" as pg on cpg."brandId"=pg."id" where cpg."deletedAt" is null and cpg."channelId"=:channelId';

            let productGroup=await this.sequelizeInstance.query(query, {
                type: QueryTypes.SELECT,  
                replacements: {
                    channelId: channelId
                }
                     
                }).then(results => {        
                   return results;
                });
    
        
        let result={
            brand:productGroup
        }

        return result;    
    }

	public async getChannelDetail(channelId){

        //Code for get channel brand		
        let query='select "turnover","no_of_retailers","no_of_distributors","no_of_skus","startup","turnoverRange","empCnt","revD","revI","empCntRange","maturitylevel","turnoverPhyStore","turnoverOnStore","storeCount", '
		+ ' "empCntDSale","empCntISale","empCntDMar","empCntIMar", "keydistributors","keyretailers","keyresellers","no_of_resellers" from "tblChannelDetail" where "channelId"=:channelId ';

            let channelDetail=await this.sequelizeInstance.query(query, {
                type: QueryTypes.SELECT,  
                replacements: {
                    channelId: channelId
                }
                     
                }).then(results => {        
                   return results;
                });

    
        
        return channelDetail[0];    
    }

	public async getProfileAndSpeciality(channelId){
		let tradeinfo = [];

		tradeinfo=await this.channelRepository.scope().findAll<Channel>({
            attributes:['channelId', 'profileTypeId', 'isFrenchTech'],                    
            where:{
                channelId:channelId
            },
            include: [
            	{
                    model:this.channelCustProfileRepository,
                    as:'channelCustomerProfile',
                    attributes: ['profileTypeId'],
                    // include: [
                    //     {
                    //         model:this.profileTypeRepository,
                    //         attributes:['profileType']
                    //     }
                    // ]

                },
				{
					model:this.serviceListRepository.scope('tradeinfo'),
					as:'serviceList',
					where: {deletedAt: null},
					required: false
				},
				{
					model:this.channelSpecialityRepository,
					as:'channelSpeciality',
					where: {deletedAt: null},
					required: false,
					include: [
                        {
                            model:this.productGroupRepository,
                            as:'productGroup',
                            attributes:[['productGroupId','id'],['productGroup','name']]
                        }
                    ]
				},

            ]

        });

		let query = 'select * from "tblProductGroup" where "deletedAt" is null and "isActive"=\'1\'';
		let specialityOptions = await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT
		}).then(res => res);

		let final = Object.assign(tradeinfo, {specialityOptions: specialityOptions});

		return final;

	}

    public async getProductProfile(channelId){

        //Code for get prduct  for user profile addnew
        let query='select cpg."productCategorieId" as id,pg."productCategorie"  as name,pg."productGroupId" as groupid from "tblChannelProductCategorie" as cpg inner join "tblProductCategories" as pg on cpg."productCategorieId"=pg."productCategorieId"  where cpg."channelId"=:channelId';

        
        let categorie=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,           
            replacements: {
                channelId: channelId
            }
            }).then(results => {        
               return results;
            });

        query='select cpg."brandId" as id,pg."brandName" as name from "tblChannelBrand" as cpg inner join "tblBrand" as pg on cpg."brandId"=pg."id" where cpg."channelId"=:channelId';
        
        let brand=await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,           
            replacements: {
                channelId: channelId
            }
            }).then(results => {        
               return results;
            });

			query='select count(*) as "activeProducts" from "tblProduct" as tp  where tp."channelId"= :channelId and tp."deletedat" is null and "isactive"=:isactive';
        
			let activeProduct=await this.sequelizeInstance.query(query, {
				type: QueryTypes.SELECT,           
				replacements: {
					channelId: channelId,
					isactive: '1'
				}
				}).then(results => {        
				   return results;
				});	

        let result={
            productCat:categorie,            
            productBrand:brand,
			activeProduct:+activeProduct[0].activeProducts
        }

        return result;    
    }

    async getCompanyMediaLibrary(channelId, type){

		let cond = {
			...(type=="channel" && {channelId: +channelId}),
			...(type=="oldChannel" && {oldChannelId: +channelId}),
			deletedAt: null
		}

		return await this.channelDocumentRepository.findAll<ChannelDocument>({
			where: cond,
			include: [{
				model: Media,
				where: { 
                    documentTypeId:1,
                    deletedAt: null 
                },
				required: true
			}],
			order: [['channelDocumentId', 'DESC']]
        });	
    }

	async getCompanyImages(channelId){
		return await this.companyMediaImagesRepository.findAll<CompanyMediaImages>({
			where: {
                channelId: +channelId,
				deletedAt: null
			},
			include: [{
				model: Media,
				where: { 
                    documentTypeId:1,
                    deletedAt: null 
                },
				required: true
			}],
			order: [['companyMediaId', 'DESC']]
        });	
    }
	
	async getCompanyDocument(channelId){
		return await this.channelDocumentRepository.findAll<ChannelDocument>({
			where: {
                channelId: +channelId,
				deletedAt: null
			},
			include: [{
				model: Media,
				where: { 
                    documentTypeId:2,
                    deletedAt: null 
                },
				required: true
			}],
			order: [['channelDocumentId', 'DESC']]
        });	
    }

    async createCompanyLogo(doc, user){

        return this.sequelizeInstance.transaction(async transaction => {
		let updatedDocument = await this.mediaRepository.create({
			documentPath: doc.Key,
			documentTypeId: 1,
			dirName: 'company/logo'
		}, transaction);
		var document = updatedDocument;
		let companyMedia = await this.channelDocumentRepository.create({
			documentId: document.documentId,
            ...(user.channelId && {channelId: +user.channelId}),
			...(user.oldChannelId && {oldChannelId: +user.oldChannelId})
            
        }, transaction);
    //    let data = Object.assign(companyMedia, {'document':updatedDocument.documentPath});
    let data = {
        documentId: companyMedia.documentId,
        ...(user.channelId && {channelId: companyMedia.channelId}),
		...(user.oldChannelId && {oldChannelId: companyMedia.oldChannelId}),
        channelDocumentId:companyMedia.channelDocumentId,
        document:document,
        name:companyMedia.name
    };
        return data;
    })
    .catch((err) => {
        console.log(err)
        throw new ProcessException('account:singup:tryAgain');
    });
    }
    async existingBrandCheck(brand): Promise<any> {
        let result = await this.brandRepository.findOne<Brand>({
			where: {
				brandName: {
					[Op.iLike]: brand.brandName.trim()
				},
				id: {
					[Op.not]: +brand.id
				},
				deletedAt: null
			}
		});
        console.log(result)
                if (!result) {
                    return true;
                }

                return false;
    }
    
    public async saveBrand(brand: any, channelId: number, transaction: any): Promise<any> {
		try {
			let channelBrandArrayObject = <Array<ChannelBrand>>[];
			let brandIds = [];

			for (let jj = 0; jj < brand.length; jj++) {
				if (brand[jj].id&&brand[jj].isNew=="0") {

					// relate brand to channel
					let channelBrandObject = await this.setChannelBrands(brand[jj].id, channelId);
					channelBrandArrayObject.push(channelBrandObject);

					brandIds.push(brand[jj].id);

				} else {

					// create new brand
					let brandObject = <Brand>{};
					brandObject.brandName = brand[jj].name;
					brandObject.createdAt = new Date();

					if (brand[jj].documentpath) {

						// save detail to document table
						let mediaObject: Media = await this.setDocumentObject(brand[jj].documentpath, appconstant.DOCUMENTTYPE_IMAGE, appconstant.FOLDER_PATH_BRAND);
						let document = await this.mediaRepository.create(mediaObject, { transaction });

						//set brant image id
						brandObject.documentId = +document.documentId;
						console.log('brandObject',brandObject);
					}else if (brand[jj].documentPath) {

						// save detail to document table
						let mediaObject: Media = await this.setDocumentObject(brand[jj].documentPath, appconstant.DOCUMENTTYPE_IMAGE, appconstant.FOLDER_PATH_BRAND);
						let document = await this.mediaRepository.create(mediaObject, { transaction });

						//set brant image id
						brandObject.documentId = +document.documentId;
						console.log('brandObject',brandObject);
					}

					// save new brand to brand Master
					let newbrand = await this.productService.saveBrand(brandObject, transaction);

					// relate brand to channel
					let channelBrandObject = await this.setChannelBrands(newbrand.id, channelId);
					channelBrandArrayObject.push(channelBrandObject);
					brandIds.push(newbrand.id);
				}
			}
			return { "channelBrandArrayObject": channelBrandArrayObject, "brandIds": brandIds };
		} catch (e) {
			console.log("Error+ ", e);
			throw new ProcessException("channel:");
		}
    }
    
    async setChannelBrands(brandId: number, channelId: number): Promise<ChannelBrand> {
		try {
			let resultObject = <ChannelBrand>{};

			resultObject.brandId = brandId;
			resultObject.channelId = channelId;
			resultObject.createdAt = new Date();

			return resultObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:channelBrand');
		}


    }

    public async setDocumentObject(docTempPath: string, doctype: number, folderPath?: string): Promise<Media> {
		try {
			let mediaObject = <Media>{};
			mediaObject.documentPath = docTempPath;
			mediaObject.documentTypeId = doctype;
			if (folderPath) {
				mediaObject.dirName = folderPath;
			}
			return mediaObject;
		} catch (e) {
			throw new ProcessException('channel:objectCreation:media');
		}
    }

    async getCompanyProfile(user: any): Promise<any> {
        //console.log("user:",user);
		let companyProfile: any = '';
        let channelAward: any[] = [];
        let data = '';
		let channelStatusArray: Array<number> = [appconstant.ACCOUNTSETUP, appconstant.ACCOUNTSETUP_COMPANYPROFILE, appconstant.ACCOUNTSETUP_TRADEINFORMATION, appconstant.ACCOUNTSETUP_TRADELOCATION, appconstant.CHANNEL_APPROVAL, appconstant.CHANNEL_APPROVED, appconstant.CLAIM_APPROVED];
		try {
			let userSignUpStatus = channelStatusArray.includes(parseInt(user.signUpStatusId));
			if (userSignUpStatus) {
				companyProfile = await this.sequelizeInstance.query("select * from fngetchannelprofile(" + user.userId + "," + parseInt(user.signUpStatusId) + ",'" + process.env.CDN_URL + "');");
				companyProfile = companyProfile[0][0].fngetchannelprofile;

				console.log('companyProfle', companyProfile);

                if(companyProfile.channel.channelAward){
                    channelAward  = await Promise.all(companyProfile.channel.channelAward.map(async x => {
                           
                           if(x.documentId){
                               let data = await 
                                this.sequelizeInstance.query('select "documentId","documentPath" from "tblDocument" where "documentId"='+x.documentId, {
                                   type: QueryTypes.SELECT,
                                   raw: true,
                                   }).then(y=>{            
                                       console.log('img',x, y, );
                                       return Object.assign(x, {awardImg: y[0].documentPath, isOld:true, lifeTime: x.lifeTime == '1' ? true : false});
                                   })
                               return data;
                           }
                           else{
                               return Object.assign(x, {awardImg: null, isOld:true, lifeTime: x.lifeTime == '1' ? true : false});
                           }
                               
                       }))
                   }

				   let bannerImg = [];
				   console.log('banner', companyProfile.channel.channelDetail.channelBannerId)
				   if(companyProfile.channel.channelDetail.channelBannerId){
					bannerImg = await this.sequelizeInstance.query('select "documentId","documentPath" from "tblDocument" where "documentId"='+companyProfile.channel.channelDetail.channelBannerId, {
								type: QueryTypes.SELECT,
								raw: true,
								}).then(y=>{            
									console.log('img', y );
									return y[0];
								})
				   }
				   let endCustomerB2b = await this.sequelizeInstance.query('select "customerTypeId","other" from "tblEndCustomersB2b" where "channelId"='+user.channelId, {
					type: QueryTypes.SELECT,
					raw: true,
					}).then(y=>{
						return y[0];
					})
					//    let endCustomerB2b=this.endCustomersB2bRepository.findOne({where:{channelId:+user.channelId}});
                       // channelAward = await Promise.all(channelAward);
                       // console.log('channelAward', channelAward);
					   companyProfile['bannerImg'] = bannerImg;
                       companyProfile['channelAward'] = channelAward;
					   companyProfile['endCustomerB2b'] = endCustomerB2b;
					   companyProfile['linkedInBanner'] = companyProfile.channel.channelDetail.bannerURL;
               }
		} catch (e) {
			console.log("Error+", e);
			//throw new InternalServerErrorException();
		}
		if (!companyProfile) {
			throw new ProcessException('channel:getCompanyProfile:companyNotFound');
		}
		return companyProfile;
    }
    
    async getTradeInfo(user: any): Promise<any> {
        let tradeinfo: any = '';
        
        tradeinfo=await this.channelRepository.scope().findAll<Channel>({
            attributes:['channelId', 'profileTypeId', 'isFrenchTech', 'isCompanyPublic'],                    
            where:{
                channelId:user.channelId
            },
            include: [
                {
                    model:this.channelBrandRepository,
                    as: 'channelBrand',
                    where:{deletedAt: null},
                    required:false,
                    attributes:['channelBrandId','brandId'],                    
                    include: [
                        {
                        model:this.brandRepository,
                        as:'brand',
                        attributes:['brandName'],
                        include: [
                            {
                                model:this.mediaRepository,
                                as:'document',
                                attributes: ['documentPath']
                            }
                        ]
                        }
                    ]
                
                },
                {
                    model:this.channelProductGroupRepository,
                    as:'channelProductGroup',
                    attributes: ['productGroupId'],
					where: {deletedAt: null},
					required:false,
                    include: [
                        {
                            model:this.productGroupRepository,
                            as:'productGroup',
                            attributes:[['productGroupId','groupid'],['productGroup','groupname']]
                        }
                    ]
                },
                {
                    model:this.channelCatRepository,
                    as:'channelProductCategorie',
                    attributes: ['channelProductCategorieId'],
					where: {deletedAt: null},
					required:false,
                    include: [
                        {
                            model:this.productCatRepository,
                            as:'productCategorie',
                            attributes:[['productCategorieId','childid'],['productCategorie','childname'],['productGroupId','groupid']],

                        }
                    ]

                },
                {
                    model:this.channelCustProfileRepository,
                    as:'channelCustomerProfile',
                    attributes: ['profileTypeId','profileTypeClassificationId'],
					required: false,
                    // include: [
                    //     {
                    //         model:this.profileTypeRepository,
                    //         attributes:['profileType']
                    //     }
                    // ]
					where:{
						deletedAt: null
					}

                },
                {
                    model:this.channelDetailRepository,
                    as:'channelDetail',
                    attributes: ['no_of_retailers','no_of_distributors','no_of_skus','startup','turnoverRange','empCnt','revD','revI','empCntRange','maturitylevel','turnoverPhyStore','turnoverOnStore','storeCount',
						'empCntDSale','empCntISale','empCntDMar','empCntIMar', 'keydistributors', 'keyretailers']  
                },
				{
					model:this.serviceListRepository.scope('tradeinfo'),
					as:'serviceList',
					where: {deletedAt: null},
					required: false
				},
				{
					model:this.channelSpecialityRepository,
					as:'channelSpeciality',
					where: {deletedAt: null},
					required: false,
					include: [
                        {
                            model:this.productGroupRepository,
                            as:'productGroup',
                            attributes:[['productGroupId','id'],['productGroup','name']]
                        }
                    ]
				},

            ]

        })

		let query = 'select * from "tblProductGroup" where "deletedAt" is null and "isActive"=\'1\'';

		let specialityOptions = await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT
		}).then(res => res);

		let keyRetailers = [];
		let keyDistributors = [];
		let keyPartner = {retailer: [], distributor: []};
		if(user.channelTypeId == '2'){
			keyRetailers = await this.searchRetailer('',4);
			keyDistributors = await this.searchRetailer('',3);
			keyPartner['retailer'] = keyRetailers;
			keyPartner['distributor'] = keyDistributors;
		}else if(user.channelTypeId == '3'){
			keyRetailers = await this.searchRetailer('',4);
			keyPartner['retailer'] = keyRetailers;
		}

		let serviceListOptions = await this.getServiceList();
        
		tradeinfo = Object.assign(tradeinfo, {keyPartners: keyPartner, serviceListOptions: serviceListOptions, specialityOptions: specialityOptions});
		return tradeinfo;
    }
    
    async getTradeLocation(user: any): Promise<any> {
        let tradelocation: any = '';

        tradelocation=await this.channelRepository.scope().findAll<Channel>({
            attributes:['channelId'],                    
            where:{
                channelId:user.channelId,
            },
            include: [
                {
                    model:this.channelLocationRepository,
                    as: 'channelLocation',
                    attributes:['locationType'],                    
                    where: {
                        locationType: {
                            [Op.in]: ['TARGET','SELLING','RETAIL','RESELLING']
                        },
                        deletedAt: null,
                    },
                    include: [
                        {
                        model:this.regCtyJCTRepository,
                        as:'RegionCountryJCT',
                        attributes:['regionCountryJCTId'],
                        include: [
                            {
                                model:this.regionRepository,                                
                                attributes: ['regionId','region'],                               
                            },
                            {
                                model:this.countryRepository,                                
                                attributes: ['countryId','country']
                            }
                        ]
                        }
                    ]
                
                }
                
            ]


        })

		return tradelocation;
    }

    async updateChannelLocation(channelLocationDto: ChannelLocationDto, user: any, subscriptionFeatureKeys): Promise<any> {

		let channel = await this.channelRepository.findByPk(user.channelId);
		let isUpdate: boolean = true;
		// if (!isUndefined(channelLocationDto.isUpdate) && isBoolean(channelLocationDto.isUpdate) && !channelLocationDto.isUpdate) {
		// 	isUpdate = false;
		// }

		//let key = +user.channel.subscriptionId + '-' + +features.ZONE_INCLUDES;
		var verify = false;
		// if (subscriptionFeatureKeys[key]) {
		// 	verify = true;
		// }

		if (isUpdate) {
			let zoneFilter: any = 0;
			let getZone = [];

			/* Removed filter based on Subscription, need to revert in next phase */
			// if (verify) {
			// 	zoneFilter = +subscriptionFeatureKeys[key];
			// 	getZone = await this.getZoneObject(channelLocationDto.location);
			// }

			// if (getZone.length <= zoneFilter) {
				try {
					let channelLocationObject: any = [];
					let contactLocationObject: any = [];
					let locationType: any = channelLocationDto.locationType;
					let data: any;
					data = channelLocationDto.location;
					for (var i in data) {
						let channellocation = {
							channelId: user.channelId,
							locationType: locationType,
							regionCountryJCTId: data[i],
							updatedBy: user.userId
						}
						let contactlocation = channellocation;
						contactlocation["contactId"] = user.contactId;
						channelLocationObject.push(channellocation);
						contactLocationObject.push(contactlocation);
					}

					if (channelLocationObject.length > 0) {
						await this.channelLocationRepository.update({ deletedAt: new Date().toUTCString() }, { where: { channelId: user.channelId, locationType: locationType, deletedAt: null } });
						await this.contactLocationRepository.update({ deletedAt: new Date().toUTCString() }, { where: { channelId: user.channelId, locationType: locationType, contactId: user.contactId, deletedAt: null } });
					}

					let channelLocation = await this.channelLocationRepository.bulkCreate(channelLocationObject);
					let contactLocation = await this.contactLocationRepository.bulkCreate(contactLocationObject);
					// let status = {
					// 	signUpStatusId: appconstant.CHANNEL_APPROVAL,
					// 	updatedAt: this.sequelizeInstance.fn('NOW')
					// };


					// if (user.channel.signUpStatusId != appconstant.CHANNEL_APPROVED && (channelLocationDto.locationType == 'SELLING' || channelLocationDto.locationType == 'RETAIL')) {
					// 	let signUpHistoryObject: SignUpHistory = await this.channelSerivceInBound.setSingUpHistoryObject(appconstant.CHANNEL_APPROVAL, user.channelId, user.userId);
					// 	await this.signUpHistorylRepository.create(signUpHistoryObject);

					// 	await channel.update(status).then(async () => {

					// 		// add approved company to elastic search
					// 		await this.channelIndexService.updateNode(+channel.channelId, {
					// 			doc: {
					// 				company: {
					// 					status_id: appconstant.CHANNEL_APPROVAL
					// 				}
					// 			}
					// 		});
					// 	});

					// 	let requestObject: IMailNotification = {};
					// 	requestObject.channelId = +user.channelId;
					// 	requestObject.reason = '';
					// 	this.notificationService.sendNotificationRouter(requestObject, notification.CHANNEL_APPROVAL_PENDING);

					// 	return status;
					// } else {
						return true;
					//}
				} catch (e) {
					console.log("Error+", e);
					//throw new InternalServerErrorException();
				}
			// } else {
			// 	throw new ProcessException('Based on your Subscription, you are eligible to choose any ' + +subscriptionFeatureKeys[key] + ' from the given zone list');
			// }
		}

		return { signUpStatusId: channel.signUpStatusId };
	}

	async getuserLimits(user: any): Promise<any> {
		let users: any = '';

        let query='select "keyFeature" FROM "tblChannelSubscription" where "channelId" =:channelId AND "isActive" =:isActive';

        let result = await this.sequelizeInstance.query(query , {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
                channelId: user.channelId,
				isActive: '1'                
            }
		}).then(async (x) => {     
			let userLimitAccess = {};                       
			 x.forEach(async (data) => {
				userLimitAccess['allowed'] = (data.keyFeature['1'] === 'Unlimited')?'Unlimited': +data.keyFeature['1'];
			});
			console.log(userLimitAccess);
			return userLimitAccess;
		});
		
		return result;
	}
    
    async getUsers(user: any): Promise<any> {
        let users: any = '';

        let query='select case when tc."countryId" is not null then (select "country" from "tblCountry" where "countryId"=tc."countryId") '
        + 'else (select "country" from "tblCountry" where "countryId" in (select "countryId" from "tblChannel" where "channelId"=:channelId)) end as "country", '
        + '(select "roleId" from "tblUserRole" where "userId" in (select "userId" from "tblUser" where "contactId"=tc."contactId")) as "roleId" ,'
        + '(select "documentPath" from "tblDocument" where "documentId"=tc."documentId") as "documentPath" ,'
			+ 'tc."contactId",tc."firstName",tc."lastName",tc."jobTitle",tc."mobileNo",tc."email",tc."isActive" from "tblContact" as tc where tc."channelId"=:channelId and tc."deletedAt" is null and tc."isActive"=\'1\' ';

        users=await this.sequelizeInstance.query(query , {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
                channelId: user.channelId,                
            }
            }).then(x=>{                            
                console.log("users:",x);
                return x;
            })

        let accountuser=users.filter(x => x.roleId==1);
        let accuser={
            userdata:accountuser,
            role:"Account Owner",
            total:accountuser.length
        }
        
        let primary=users.filter(x => x.roleId==3);
        let puser={
            userdata:primary,
            role:"Primary",
            total:primary.length
        }
        
        let admin=users.filter(x => x.roleId==2);
        let aduser={
            userdata:admin,
            role:"Admin",
            total:admin.length
        }
		let userLimit = {};
		userLimit['allowed'] = 'unlimited'; 
		userLimit['total'] = 10; // no:of users in the account

        let data=[];
            data.push(accuser);
            data.push(aduser);
            data.push(puser);
			
       	return data;
	}

	async deleteUser(contactId: any): Promise<any> {
		await this.userRepository.update({ deletedAt: new Date().toUTCString(), isActive: '0' }, { where: { contactId: contactId } });
		await this.contactRepository.update({ deletedAt: new Date().toUTCString(), isActive: '0' }, { where: { contactId: contactId } });
		return true;
	}

    async imageCreate(image, transaction) {
        return await this.mediaRepository.create(
            image,
            {
                //returning: true,
                 transaction,
            }
        );
    }

    //updating company profile
    async update_Company_Profile(data){
        
        let channelDetail ={
            channelLogoId: +data.companyLogo,
            detailDesc: data.tweetPitch,
            // introvideoId: +data.introVideo,            
            phone1: data.phoneNumber1,
            phone2: data.phoneNumber2,
            mobileNo: data.mobile,
            webSiteUrl: data.websiteUrl,
            expectations: data.sourcing,
            }        
        
        let address = {
            address: data.legalAddress,
            city: data.city,
            countryId: +data.country,
            postalCode: data.zipCode,
            state: null
        }

        let channel = {
            companyName: data.companyLegalName,
            companyMail: data.email,
            countryId: +data.country,
        }

            console.log("vat:",data);
        let certificate=[];

        if(data.companyRegNumber!='')
            certificate.push({channelId:data.channelId,cerificateTypeId:1,cerificateName:data.certificateName,cerificateNumber:data.companyRegNumber});
        if(data.vatNumber!=null&&data.vatNumber!='')
            certificate.push({channelId:data.channelId,cerificateTypeId:2,cerificateName:'VAT',cerificateNumber:data.vatNumber});
        await this.channelCertiRepository.update({deletedAt: new Date()}, {where: {channelId: data.channelId}});
        return this.sequelizeInstance.transaction(async transaction => {
            return await this.channelCertiRepository.bulkCreate(certificate,transaction).then(async (certiObject) => {
                await this.channelDetailRepository.update(channelDetail, {
                    where: {
                        channelId: data.channelId
                    },
                    returning: true,
                    transaction
                }); 
                

                if(data.typeId==3)
                {
                    
                // update product group & category for channel
                let ChannelProfileTypes: ChannelCustomerProfile[] = await this.setChannelProfile(data.relevantDistributorProfile, +data.channelId);
                await this.channelCustProfileRepository.update(ChannelProfileTypes, transaction);

                let ChannelGroupObject: ChannelProductGroup[] = await this.setChannelProductGroup(data.productGroup, +data.channelId);
                await this.channelProductGroupRepository.update(ChannelGroupObject, transaction );

                let ChannelCatObject: ChannelProductCategorie[] = await this.setChannelProductCat(data.productCat, +data.channelId);
                await this.channelCatRepository.update(ChannelCatObject, transaction );
                }

                //channel mail and name
                await this.channelRepository.update(channel, {where: {
                    channelId: data.channelId
                },transaction});


                //linkedIn                
                if(data.linkedIn){
                    let linkedIn = {socialSiteId: appconstant.SOCIAL_SITE_TYPE_LINKEDIN,
                        channelId: data.channelId,
                        channelSocialLink: data.linkedIn};

                    await this.socialLinkRepository.findOrCreate({
                        where: {
                            channelId: data.channelId, 
                            socialSiteId: appconstant.SOCIAL_SITE_TYPE_LINKEDIN 
                        }
                    ,defaults: linkedIn, transaction}).then((result) => {
                        if(!result[1]){
                            result[0].update(linkedIn)
                        }
                    });  
                }
                
                //Insta                
                if(data.insta){
                    let insta = {socialSiteId: appconstant.SOCIAL_SITE_TYPE_INSTAGRAM,
                        channelId: data.channelId,
                        channelSocialLink: data.insta};

                    await this.socialLinkRepository.findOrCreate({
                        where: {
                            channelId: data.channelId, 
                            socialSiteId: appconstant.SOCIAL_SITE_TYPE_INSTAGRAM
                        }
                    ,defaults: insta, transaction}).then((result) => {
                        if(!result[1]){
                            result[0].update(insta)
                        }
                    });  
                }
                
        //Googleplus 
        if(data.tradeUrls){
            let trade = {socialSiteId: appconstant.SOCIAL_SITE_TYPE_GOOGLEPLUS,
                channelId: data.channelId,
                channelSocialLink: data.tradeUrls};

            await this.socialLinkRepository.findOrCreate({
                where: {
                    channelId: data.channelId, 
                    socialSiteId: appconstant.SOCIAL_SITE_TYPE_GOOGLEPLUS
                }
            ,defaults: trade, transaction}).then((result) => {
                if(!result[1]){
                    result[0].update(trade)
                }
            });  
        }

        let awards = [];
        await this.awardRepository.update({deletedAt: new Date()}, {
            where: {
                channelId: data.channelId,
                deletedAt: null
            },transaction
        })
        if (data.award_details && Object.keys(data.award_details).length > 0) {
            for (let ii = 0; ii < data.award_details.length; ii++) {
                let valid_upto = (data.award_details[ii].valid_upto) ? data.award_details[ii].valid_upto : null;

                //save awards image
                if (data.award_details[ii].image) {
                    let media = <Media>{};
                    media.documentTypeId = appconstant.DOCUMENTTYPE_IMAGE;
                    media.documentPath = (data.award_details[ii].image) ? data.award_details[ii].image : null;
                    media.dirName = appconstant.FOLDER_PATH_PRODUCT_AWARDS;
                    
                    let logo = await this.imageCreate(media, transaction);
                    data.award_details[ii].imageId = logo.documentId;

                } else {
                    data.award_details[ii].imageId = null
                }

                let award = {
                    channelAwardId: data.award_details[ii].channelAwardId?data.award_details[ii].channelAwardId:null,
                    channelId: data.channelId,
                    awardDetail: data.award_details[ii].details,
                    issuedBy: data.award_details[ii].issued_by,
                    validUpto: valid_upto,
                    isLifeTime: data.award_details[ii].isLifeTime,
                    documentId: (+data.award_details[ii].imageId > 0) ? +data.award_details[ii].imageId : null}

                    let award1 = {
                        channelId: data.channelId,
                        awardDetail: data.award_details[ii].details,
                        issuedBy: data.award_details[ii].issued_by,
                        validUpto: valid_upto,
                        isLifeTime: data.award_details[ii].isLifeTime,
                        documentId: (+data.award_details[ii].imageId > 0) ? +data.award_details[ii].imageId : null}

                await this.awardRepository.findOrCreate({
                    where: {
                        channelId: data.channelId, channelAwardId: award.channelAwardId
                    }, defaults: award1, transaction
                }).then(res => {
                    if(!res[1]){
                        res[0].update(award);
                    }
                })
            }
            // await this.awardRepository.bulkCreate(awards, {transaction});
        }

        // if (data.award_details && Object.keys(data.award_details).length > 0) {
        //     for (let ii = 0; ii < data.award_details.length; ii++) {
        //         let valid_upto = (data.award_details[ii].valid_upto) ? data.award_details[ii].valid_upto : null;
        //         if (data.award_details[ii].product_award_id) {

        //             // update article logo to document table
        //             if (data.award_details[ii].imageId && data.award_details[ii].image) {
        //                 let media = <Media>{};
        //                 media.documentTypeId = appconstant.DOCUMENTTYPE_IMAGE;
        //                 media.documentPath = (data.award_details[ii].image) ? data.award_details[ii].image : null;
        //                 media.dirName = appconstant.FOLDER_PATH_PRODUCT_AWARDS;

        //                 let logo = await this.imageUpdate(data.award_details[ii].imageId, media, transaction);
        //                 data.award_details[ii].imageId = logo.documentId;
        //             } else if (data.award_details[ii].image) {
        //                 let media = <Media>{};
        //                 media.documentTypeId = appconstant.DOCUMENTTYPE_IMAGE;
        //                 media.documentPath = (data.award_details[ii].image) ? data.award_details[ii].image : null;
        //                 media.dirName = appconstant.FOLDER_PATH_PRODUCT_AWARDS;

        //                 let logo = await this.imageCreate(media, transaction);
        //                 data.award_details[ii].imageId = logo.documentId;
        //             }


        //             // update award details to channel award table
        //             await this.awardRepository.update({
        //                 channelId: data.channelId,
        //                 awardDetail: data.award_details[ii].details,
        //                 issuedBy: data.award_details[ii].issued_by,
        //                 validUpto: valid_upto,
        //                 isLifeTime: data.award_details[ii].isLifeTime,
        //                 documentId: (+data.award_details[ii].imageId > 0) ? +data.award_details[ii].imageId : null
        //             },
        //                 {
        //                     where: { deletedAt: null, channelAwardId: data.award_details[ii].channelAwardId },
        //                     returning: true, transaction,
        //                 });
        //         } else {

        //             //save awards image
        //             if (data.award_details[ii].image) {
        //                 let media = <Media>{};
        //                 media.documentTypeId = appconstant.DOCUMENTTYPE_IMAGE;
        //                 media.documentPath = (data.award_details[ii].image) ? data.award_details[ii].image : null;
        //                 media.dirName = appconstant.FOLDER_PATH_PRODUCT_AWARDS;

        //                 let logo = await this.imageCreate(media, transaction);
        //                 data.award_details[ii].imageId = logo.documentId;
        //             } else {
        //                 data.award_details[ii].imageId = null
        //             }

        //             // save award details to channel award table
        //             await this.awardRepository.create({
        //                 channelId: data.channelId,
        //                 awardDetail: data.award_details[ii].details,
        //                 issuedBy: data.award_details[ii].issued_by,
        //                 validUpto: valid_upto,
        //                 isLifeTime: data.award_details[ii].isLifeTime,
        //                 documentId: (+data.award_details[ii].imageId > 0) ? +data.award_details[ii].imageId : null
        //             },
        //             transaction);
        //         }
        //     }
        // }

        // // updateawards
        // if (data.deleted_award_details && data.deleted_award_details.length > 0) {
        //     for (let ii = 0; ii < data.deleted_award_details.length; ii++) {
        //         if (data.deleted_award_details[ii]) {
        //             // update award details to channel award table
        //             await this.awardRepository.update({
        //                 channelId: data.channelId,
        //                 deletedAt: new Date()
        //             },
        //                 {
        //                     where: { deletedAt: null, productAwardId: data.deleted_award_details[ii] },
        //                     returning: true, transaction,
        //                 });
        //         }
        //     }
        // }



            
            let channeldet = await this.channelRepository.findByPk(data.channelId);
            await this.addressRepository.update(address, {where:{
                addressId: channeldet.billAddressId
            }, transaction});

                return certiObject;
            })
        })
        .catch((err) => {
            console.log(err)
            throw new ProcessException('account:singup:tryAgain');
        });
        
        //console.log("profileData:",data);

    }

	//update company profile old
	public async updateUserCompanyProfile(requestObject: CompanyUpdateDto, user: any): Promise<boolean> {
		let companyProfile: any = '';
		let companyProfileJson: any = '';
		let channelStatusArray: Array<number> = [appconstant.CHANNEL_APPROVED, appconstant.CHANNEL_APPROVAL];

		//validate company name
		let realUser = user;
		if (realUser && realUser.channel && +realUser.channel.channelTypeId === +appconstant.CHANNEL_TYPE_HUB) {
			// before update verify if the company name already exist
			let result = await this.channelRepository.findOne({
				where: {
					companyName: requestObject.company_name.trim(),
					channelId: {
						$not: +user.channel.channelId
					}
				}
			});

			if (result) {
				throw new ProcessException('channel:profile:companyNameAlreadyExist');
			}

			// allow edit
			channelStatusArray.push(+user.channel.signUpStatusId);
		}


		// return true;
		try {
			let result: any = '';
			console.log('user.channel.signUpStatusId', user.channel.signUpStatusId);
			let userSignUpStatus = channelStatusArray.includes(+user.channel.signUpStatusId);
			if (userSignUpStatus) {
				companyProfile = await this.userRepository.scope('userBasicInfom').findOne({
					include: [
						{
							model: this.contactRepository.scope('contactInfo'),
							where: { deletedAt: null },
							include: [
								{
									model: this.mediaRepository,
									include: [
										{
											model: this.mediaTypeRepository,
											required: false
										},
									],
									required: false
								}
							]
						},
						{
							model: this.channelRepository.scope('profileInfom'),
							where: { signUpStatusId: parseInt(user.channel.signUpStatusId), deletedAt: null },
							include: [
								{
									model: this.channelDetailRepository.scope('channelDetail'),
									where: { deletedAt: null },
									include: [
										{
											model: this.mediaRepository,
											as: 'channelLogo',
											include: [
												{
													model: this.mediaTypeRepository,
													required: false
												},
											],
											required: false
										},
										{
											model: this.mediaRepository,
											as: 'channelIcon',
											include: [
												{
													model: this.mediaTypeRepository,
													required: false
												},
											],
											required: false
										}
									]
								},
								{
									model: this.channelCertiRepository.scope('profileInfo'),
									where: { deletedAt: null },
									required: false
								},
								{
									model: this.channelDocumentRepository,
									where: { deletedAt: null },
									required: false,
									include: [
										{
											model: this.mediaRepository,
											as: 'document',
											required: false
										}
									]
								}
							]
						}
					],
					where: { userId: parseInt(user.userId), deletedAt: null }
				});
			}

			if (companyProfile) {
				companyProfileJson = companyProfile.toJSON();
				result = await this.sequelizeInstance.transaction(async transaction => {
					let billAddress: any;
					let comapnyLogo: any;
					let comapnyIcon: any;
					let contactProfileImage: any;
					let comapnyLogoId: any = null;
					let comapnyIconId: any = null;
					let comapnyRegAddressId: any = null;
					let comapnyBillAddressId: any = null;
					let contactProfileImageId: any = null;

					// update company registration address to address table
					if (requestObject.reg_address_id) {
						await this.addressRepository.update({
							address: requestObject.reg_address,
							city: requestObject.reg_city,
							countryId: requestObject.reg_country_id,
							postalCode: requestObject.reg_postal_code,
						},
							{
								where: { addressId: companyProfile.channel.regAddressId },
								returning: true, transaction,
							});
						comapnyRegAddressId = companyProfile.channel.regAddressId;
					}

					if (requestObject.is_same_address) {
						comapnyBillAddressId = comapnyRegAddressId;
					} else {
						// update company billing address to address table
						if (requestObject.bill_address_id) {
							await this.addressRepository.update({
								address: requestObject.bill_address,
								city: requestObject.bill_city,
								countryId: requestObject.bill_country_id,
								postalCode: requestObject.bill_postal_code,
							},
								{
									where: { addressId: requestObject.bill_address_id },
									returning: true, transaction,
								});
							comapnyBillAddressId = requestObject.bill_address_id;
						} else {
							// save company billing address to address table
							if (requestObject.bill_address && requestObject.bill_city && requestObject.bill_country_id && requestObject.bill_postal_code) {
								billAddress = await this.addressRepository.create(
									{
										address: requestObject.bill_address,
										city: requestObject.bill_city,
										countryId: requestObject.bill_country_id,
										postalCode: requestObject.bill_postal_code,
									},
									 {transaction }
								);
								comapnyBillAddressId = billAddress.addressId;
							}
						}
					}

					// update channel information's address to channel table
					let companyInfoUpdate: any = {
						// companyMail: requestObject.company_email || null,
						regAddressId: comapnyRegAddressId,
						billAddressId: comapnyBillAddressId,
						countryId: +requestObject.reg_country_id,
						isCompanyPublic: requestObject.is_company_public || '0',
						isOEM: requestObject.isOEM || '3'
					};

					if (realUser && realUser.channel && +realUser.channel.channelTypeId === +appconstant.CHANNEL_TYPE_HUB) {
						companyInfoUpdate.companyName = requestObject.company_name.trim();
					}

					await this.channelRepository.update(companyInfoUpdate,
						{
							where: { channelId: companyProfile.channel.channelId }, returning: true, transaction,
						});

					// if (companyProfileJson.contact.documentId) {
					// 	contactProfileImageId = companyProfileJson.contact.documentId;
					// }

					// if (requestObject.contact_profile_image_url) {
					// 	if (companyProfileJson.contact.documentId) {
					// 		companyProfile.contact.document.documentPath = requestObject.contact_profile_image_url;
					// 		companyProfile.contact.document.dirName = appconstant.FOLDER_PATH_PROFILE;
					// 		await companyProfile.contact.document.save({
					// 			returning: true, transaction,
					// 		});
					// 		contactProfileImageId = companyProfile.contact.document.updatedId;
					// 	} else {
					// 		// save contact profile detail to document table
					// 		contactProfileImage = await this.mediaRepository.create({
					// 			documentPath: requestObject.contact_profile_image_url,
					// 			documentTypeId: appconstant.DOCUMENTTYPE_IMAGE,
					// 			dirName: appconstant.FOLDER_PATH_PROFILE
					// 		},
					// 			{
					// 				returning: true, transaction,
					// 			});
					// 		contactProfileImageId = contactProfileImage.documentId;
					// 	}
					// }

					// update channel owner contact details to contact table
					// await this.contactRepository.update({
					// 	firstName: requestObject.contact_first_name,
					// 	lastName: requestObject.contact_last_name,
					// 	phone1: requestObject.contact_phone1,
					// 	phone2: requestObject.contact_phone2,
					// 	mobileNo: requestObject.contact_mobile_no,
					// 	jobTitle: requestObject.contact_job_title,
					// 	documentId: contactProfileImageId
					// },
					// 	{
					// 		where: { contactId: companyProfile.contactId }, returning: true, transaction,
					// 	});

					// if (requestObject.contact_linkedin_id) {
					// 	// update channel social linkedin profile details to channel social link table
					// 	await this.contactSocialLinkRepository.update({
					// 		contactSocialLink: requestObject.contact_linkedin_url,
					// 		socialSiteId: appconstant.SOCIAL_SITE_TYPE_LINKEDIN,
					// 		contactId: companyProfile.contactId
					// 	},
					// 		{
					// 			where: { contactSocialLinkId: requestObject.contact_linkedin_id },
					// 			returning: true, transaction,
					// 		});
					// } else {
					// 	// save contact social linkedin profile details to contact social link table
					// 	if (requestObject.contact_linkedin_url) {
					// 		await this.contactSocialLinkRepository.create({
					// 			contactSocialLink: requestObject.contact_linkedin_url,
					// 			socialSiteId: appconstant.SOCIAL_SITE_TYPE_LINKEDIN,
					// 			contactId: companyProfile.contactId
					// 		},
					// 			{
					// 				returning: true, transaction
					// 			});
					// 	}
					// }

					if (requestObject.social_linkedin_id) {
						// update channel social linkedin profile details to channel social link table
						await this.socialLinkRepository.update({
							socialSiteId: appconstant.SOCIAL_SITE_TYPE_LINKEDIN,
							channelId: companyProfile.channelId,
							channelSocialLink: requestObject.social_linkedin_url
						},
							{
								where: { channelSocialLinkId: requestObject.social_linkedin_id },
								returning: true, transaction,
							});
					} else {
						// save channel social linkedin profile details to channel social link table
						if (requestObject.social_linkedin_url && Object.keys(requestObject.social_linkedin_url).length > 0) {
							await this.socialLinkRepository.create({
								socialSiteId: appconstant.SOCIAL_SITE_TYPE_LINKEDIN,
								channelId: companyProfile.channelId,
								channelSocialLink: requestObject.social_linkedin_url
							},
								{
									transaction
								});
						}
					}

					// save channel social google plus profile details to channel social link table
					if (requestObject.social_googleplus_id) {
						// update channel social google plus profile details to channel social link table
						await this.socialLinkRepository.update({
							socialSiteId: appconstant.SOCIAL_SITE_TYPE_GOOGLEPLUS,
							channelId: companyProfile.channelId,
							channelSocialLink: requestObject.social_googleplus_url
						},
							{
								where: { channelSocialLinkId: requestObject.social_googleplus_id },
								returning: true, transaction,
							});
					} else {
						// save channel social google plus profile details to channel social link table
						if (requestObject.social_googleplus_url && Object.keys(requestObject.social_googleplus_url).length > 0) {
							await this.socialLinkRepository.create({
								socialSiteId: appconstant.SOCIAL_SITE_TYPE_GOOGLEPLUS,
								channelId: companyProfile.channelId,
								channelSocialLink: requestObject.social_googleplus_url
							},
								{
									 transaction
								});
						}
					}

					if (requestObject.social_instagram_id) {
						// update channel social instagram profile details to channel social link table
						await this.socialLinkRepository.update({
							channelId: companyProfile.channelId,
							channelSocialLink: requestObject.social_instagram_url
						},
							{
								where: { channelSocialLinkId: requestObject.social_instagram_id },
								returning: true, transaction,
							});
					} else {
						// save channel social instagram profile details to channel social link table
						if (requestObject.social_instagram_url && Object.keys(requestObject.social_instagram_url).length > 0) {
							await this.socialLinkRepository.create({
								socialSiteId: appconstant.SOCIAL_SITE_TYPE_INSTAGRAM,
								channelId: companyProfile.channelId,
								channelSocialLink: requestObject.social_instagram_url
							},
								{
									transaction
								});
						}
					}

					if (requestObject.social_twitter_id) {
						// update channel social instagram profile details to channel social link table
						await this.socialLinkRepository.update({
							channelId: companyProfile.channelId,
							channelSocialLink: requestObject.social_twitter_url
						},
							{
								where: { channelSocialLinkId: requestObject.social_twitter_id },
								returning: true, transaction,
							});
					} else {
						// save channel social instagram profile details to channel social link table
						if (requestObject.social_twitter_url && Object.keys(requestObject.social_twitter_url).length > 0) {
							await this.socialLinkRepository.create({
								socialSiteId: appconstant.SOCIAL_SITE_TYPE_TWITTER,
								channelId: companyProfile.channelId,
								channelSocialLink: requestObject.social_twitter_url
							},
								{
									transaction
								});
						}
					}
	
					if (requestObject.social_facebook_id) {
						// update channel social instagram profile details to channel social link table
						await this.socialLinkRepository.update({
							channelId: companyProfile.channelId,
							channelSocialLink: requestObject.social_facebook_url
						},
							{
								where: { channelSocialLinkId: requestObject.social_facebook_id },
								returning: true, transaction,
							});
					} else {
						// save channel social instagram profile details to channel social link table
						if (requestObject.social_facebook_url && Object.keys(requestObject.social_facebook_url).length > 0) {
							await this.socialLinkRepository.create({
								socialSiteId: appconstant.SOCIAL_SITE_TYPE_FACEBOOK,
								channelId: companyProfile.channelId,
								channelSocialLink: requestObject.social_facebook_url
							},
								{
									transaction
								});
						}
					}

					if (requestObject.B2B_url && Object.keys(requestObject.B2B_url).length > 0) {

						for (let ii = 0; ii < requestObject.B2B_url.length; ii++) {
							if (requestObject.B2B_url[ii].channel_B2B_id) {
								// update channel B2B details to channel B2B table

								await this.channelB2BRepository.update({
									channelId: companyProfile.channelId,
									channelB2B: requestObject.B2B_url[ii].B2B_url
								},
									{
										where: { channelB2BId: requestObject.B2B_url[ii].channel_B2B_id },
										returning: true, transaction,
									});
							} else {
								// save channel B2B details to channel B2B table

								await this.channelB2BRepository.create({
									channelId: companyProfile.channelId,
									channelB2B: requestObject.B2B_url[ii].B2B_url
								},
									{
										 transaction
									});
							}
						}
					}

					if (requestObject.registration_certificate && Object.keys(requestObject.registration_certificate).length > 0) {
						for (let ii = 0; ii < requestObject.registration_certificate.length; ii++) {
							if (requestObject.registration_certificate[ii].channel_certificate_id) {
								// update registration certifiacte to channel certificate table
								await this.channelCertiRepository.update({
									channelId: companyProfile.channelId,
									cerificateName: requestObject.registration_certificate[ii].name,
									cerificateNumber: requestObject.registration_certificate[ii].id,
									cerificateTypeId: appconstant.CERTIFICATETYPE_REGISTRATION
								},
									{
										where: { channelCerificateId: requestObject.registration_certificate[ii].channel_certificate_id },
										returning: true, transaction,
									});
							} else {
								// save registration certificate to channel certificate table
								await this.channelCertiRepository.create({
									channelId: companyProfile.channelId,
									cerificateName: requestObject.registration_certificate[ii].name,
									cerificateNumber: requestObject.registration_certificate[ii].id,
									cerificateTypeId: appconstant.CERTIFICATETYPE_REGISTRATION
								},
									{
										 transaction
									});
							}
						}
					}

					if (requestObject.tax_certificate && Object.keys(requestObject.tax_certificate).length > 0) {
						for (let ii = 0; ii < requestObject.tax_certificate.length; ii++) {
							if (requestObject.tax_certificate[ii].channel_certificate_id) {
								// save tax certifiacte to channel certificate table
								await this.channelCertiRepository.update({
									channelId: companyProfile.channelId,
									cerificateName: requestObject.tax_certificate[ii].name,
									cerificateNumber: requestObject.tax_certificate[ii].id,
									cerificateTypeId: appconstant.CERTIFICATETYPE_TAX
								},
									{
										where: { channelCerificateId: requestObject.tax_certificate[ii].channel_certificate_id },
										returning: true, transaction,
									});
							} else {
								// update tax certifiacte to channel certificate table
								await this.channelCertiRepository.create({
									channelId: companyProfile.channelId,
									cerificateName: requestObject.tax_certificate[ii].name,
									cerificateNumber: requestObject.tax_certificate[ii].id,
									cerificateTypeId: appconstant.CERTIFICATETYPE_TAX
								},
									{
										transaction
									});
							}
						}
					}

					if (requestObject.deleted_registration_certificate && requestObject.deleted_registration_certificate.length > 0) {
						for (let ii = 0; ii < requestObject.deleted_registration_certificate.length; ii++) {
							if (requestObject.deleted_registration_certificate[ii]) {
								// update certificate details to channel certificate table
								await this.channelCertiRepository.update({
									channelId: companyProfile.channelId,
									deletedAt: new Date()
								},
									{
										where: { channelCerificateId: requestObject.deleted_registration_certificate[ii] },
										returning: true, transaction,
									});
							}
						}
					}

					if (requestObject.deleted_tax_certificate && requestObject.deleted_tax_certificate.length > 0) {
						for (let ii = 0; ii < requestObject.deleted_tax_certificate.length; ii++) {
							if (requestObject.deleted_tax_certificate[ii]) {
								// update certificate details to channel certificate table
								await this.channelCertiRepository.update({
									channelId: companyProfile.channelId,
									deletedAt: new Date()
								},
									{
										where: { channelCerificateId: requestObject.deleted_tax_certificate[ii] },
										returning: true, transaction,
									});
							}
						}
					}

					if (requestObject.channel_document && Object.keys(requestObject.channel_document).length > 0) {
						for (let ii = 0; ii < requestObject.channel_document.length; ii++) {

									// let updatedDocument = await this.mediaRepository.create({
							// 	documentPath: requestObject.channel_document[ii].document,
							// 	documentTypeId: appconstant.DOCUMENTTYPE_DOC,
							// 	dirName: appconstant.FOLDER_PATH_COMPANY_DOCUMENT
							// },
							// 	{
							// 		 transaction,
							// 	});

							if (requestObject.channel_document[ii].channelDocumentId) {
								// update channel documents
								await this.channelDocumentRepository.update({
									channelId: companyProfile.channelId,
									documentId: requestObject.channel_document[ii].documentId,
									name: _.defaultTo(requestObject.channel_document[ii].documentName, ''),
									description: _.defaultTo(requestObject.channel_document[ii].documentDescription, ''),
									updatedBy: companyProfile.userId
								},
									{
										where: { channelDocumentId: requestObject.channel_document[ii].channelDocumentId },
										returning: true, transaction,
									});
							} else {
								// save channel document
								await this.channelDocumentRepository.create({
									channelId: companyProfile.channelId,
									documentId: requestObject.channel_document[ii].documentId,
									name: _.defaultTo(requestObject.channel_document[ii].documentName, ''),
									description: _.defaultTo(requestObject.channel_document[ii].documentDescription, ''),
									updatedBy: companyProfile.userId
								},
									{
										 transaction
									});
							}
						}
					}

					if (requestObject.deleted_channel_documents && requestObject.deleted_channel_documents.length > 0) {
						for (let ii = 0; ii < requestObject.deleted_channel_documents.length; ii++) {
							if (requestObject.deleted_channel_documents[ii]) {
								// update award details to channel award table
								await this.channelDocumentRepository.update({
									deletedAt: new Date()
								},
									{
										where: { channelDocumentId: requestObject.deleted_channel_documents[ii] },
										returning: true, transaction,
									});
							}
						}
					}

					if (requestObject.deleted_award_details && requestObject.deleted_award_details.length > 0) {
	
								// update award details to channel award table
								await this.awardRepository.update({
									// channelId: companyProfile.channelId,
									deletedAt: new Date()
								},
									{
										where: { 
											channelAwardId: { 
												[Op.in] : requestObject.deleted_award_details 
											}
										},
										returning: true, transaction,
									});
					}

					if (requestObject.deleted_B2B && requestObject.deleted_B2B.length > 0) {
						for (let ii = 0; ii < requestObject.deleted_B2B.length; ii++) {
							if (requestObject.deleted_B2B[ii]) {
								// update channel B2B details to channel B2B table
								await this.channelB2BRepository.update({
									channelId: companyProfile.channelId,
									deletedAt: new Date()
								},
									{
										where: { channelB2BId: requestObject.deleted_B2B[ii] },
										returning: true, transaction,
									});
							}
						}
					}
					if (requestObject.award_details && Object.keys(requestObject.award_details).length > 0) {
							for (let ii = 0; ii < requestObject.award_details.length; ii++) {
													// save award details to channel award table
													await this.awardRepository.create({
														channelId: companyProfile.channelId,
														awardDetail: 'channel',
														issuedBy: 'ch',
														documentId: requestObject.award_details[ii].documentId,
													},
														{
															 transaction
														});
													}
												}
					// if (requestObject.award_details && Object.keys(requestObject.award_details).length > 0) {
					// 	for (let ii = 0; ii < requestObject.award_details.length; ii++) {
					// 		let valid_upto = (requestObject.award_details[ii].valid_upto) ? requestObject.award_details[ii].valid_upto : null;
					// 		if (requestObject.award_details[ii].channel_award_id) {
					// 			//save awards image
					// 			let documentId = null, documentPath = null;

					// 			if (requestObject.award_details[ii].image) {

					// 				if(requestObject.award_details[ii].image.includes('/product/awards/')){
					// 					documentId = requestObject.award_details[ii].documentId;
					// 				}else{

					// 				let media = <Media>{};
					// 				media.documentTypeId = appconstant.DOCUMENTTYPE_IMAGE;
					// 				media.documentPath = (requestObject.award_details[ii].image) ? requestObject.award_details[ii].image : null;
					// 				media.dirName = appconstant.FOLDER_PATH_PRODUCT_AWARDS;
									
					// 				let logo = await this.imageCreate(media, transaction);
					// 				// requestObject.award_details[ii].documentId = logo.documentId;
					// 				documentId = logo.documentId;

					// 				//Deleting existing documentId
					// 				if(requestObject.award_details[ii].documentId){
					// 				await this.mediaRepository.update({
					// 					deletedAt: new Date()
					// 				}, {where: {documentId: requestObject.award_details[ii].documentId}})
					// 				}
					// 				}
					// 			} 

					// 			// update award details to channel award table
					// 			await this.awardRepository.update({
					// 				channelId: companyProfile.channelId,
					// 				awardDetail: 'channel',
					// 				issuedBy: 'ch',
					// 				// validUpto: valid_upto,
					// 				// isLifeTime: requestObject.award_details[ii].isLifeTime,
					// 				documentId: documentId,
					// 			},
					// 				{
					// 					where: { channelAwardId: requestObject.award_details[ii].channel_award_id },
					// 					returning: true, transaction,
					// 				});

					// 				console.log('awardsave');
					// 		} else {
					// 			//save awards image
					// 			let documentId = null;

					// 			if (requestObject.award_details[ii].image) {
					// 					let media = <Media>{};
					// 				media.documentTypeId = appconstant.DOCUMENTTYPE_IMAGE;
					// 				media.documentPath = (requestObject.award_details[ii].image) ? requestObject.award_details[ii].image : null;
					// 				media.dirName = appconstant.FOLDER_PATH_PRODUCT_AWARDS;
									
					// 				let logo = await this.imageCreate(media, transaction);
					// 				// requestObject.award_details[ii].documentId = logo.documentId;
					// 				documentId = logo.documentId;
					// 			}

					// 			// save award details to channel award table
					// 			await this.awardRepository.create({
					// 				channelId: companyProfile.channelId,
					// 				awardDetail: 'channel',
					// 				issuedBy: 'ch',
					// 				// validUpto: valid_upto,
					// 				// isLifeTime: requestObject.award_details[ii].isLifeTime,
					// 				documentId: documentId,
					// 			},
					// 				{
					// 					 transaction
					// 				});
					// 				console.log('awardsave1');
					// 		}
					// 	}
					// }

					if (companyProfileJson.channel.channelDetail.channelLogoId) {
						comapnyLogoId = companyProfileJson.channel.channelDetail.channelLogoId;
					}

					// save comapny logo detail to document table
					if (requestObject.company_logo_url) {
						// if (companyProfileJson.channel.channelDetail.channelLogoId) {
						// 	companyProfile.channel.channelDetail.channelLogo.documentPath = requestObject.company_logo_url;
						// 	companyProfile.channel.channelDetail.channelLogo.dirName = appconstant.FOLDER_PATH_COMPANY_LOGO;
						// 	await companyProfile.channel.channelDetail.channelLogo.save({
						// 		returning: true, transaction,
						// 	});
						// 	comapnyLogoId = companyProfile.channel.channelDetail.channelLogo.updatedId;
						// } else {
						// 	// save comapny logo detail to document table
						// 	comapnyLogo = await this.mediaRepository.create({
						// 		documentPath: requestObject.company_logo_url,
						// 		documentTypeId: appconstant.DOCUMENTTYPE_IMAGE,
						// 		dirName: appconstant.FOLDER_PATH_COMPANY_LOGO
						// 	},
						// 		{
						// 			 transaction,
						// 		});
						// 	comapnyLogoId = requestObject.company_logo_url;
						// }
						comapnyLogoId = requestObject.company_logo_url;
					}
					let bannerId: any = null;
					if (requestObject.bannerImage.length>0) {
						let companyBanner;
						if (!isUndefined(requestObject.bannerImage[0].id) && requestObject.bannerImage[0].id != '') {
							bannerId = requestObject.bannerImage[0].id;
						} else {
							// save comapny logo detail to document table

							let oldBanner = await this.channelDetailRepository.findOne({
								where: {channelId: user.channelId},
								attributes: ['channelBannerId']
							}).then( res => {
								console.log('banner', res.channelBannerId);
								return res.channelBannerId;
							}).catch(err => console.log('banner error', err));

							if(oldBanner){
								await this.mediaRepository.update({deletedAt: new Date()}, {where: {
									documentId: +oldBanner
								}});
							}

							companyBanner = await this.mediaRepository.create({
								documentPath: requestObject.bannerImage[0].image,
								documentTypeId: appconstant.DOCUMENTTYPE_IMAGE,
								dirName: appconstant.FOLDER_PATH_COMPANY_BANNER
							},
								{
									 transaction,
								});
								bannerId = companyBanner.documentId;
						}
					}

					//saving certificate tags
					if(requestObject.certificatesList?.length){
						let certificateTag : CertificateTagDto[] = requestObject.certificatesList.map(x => {
							x.channelId = user.channelId;
							x.createdat = new Date();
							return x;
						});
		
						await CertificateTag.bulkCreate(certificateTag, {
							transaction
						});
					}
		
					if(requestObject.deletedTags?.length){
						await CertificateTag.update({deletedat: new Date()}, {
							where: {
								channelId: user.channelId,
								certificatetagid: {
									[Op.in] : requestObject.deletedTags
								}
							},
							transaction, returning: true
						})
					}

					// if (companyProfileJson.channel.channelDetail.channelIconId) {
					// 	comapnyIconId = companyProfileJson.channel.channelDetail.channelIconId;
					// }

					// if (requestObject.company_icon_url) {
					// 	if (companyProfileJson.channel.channelDetail.channelIconId) {
					// 		companyProfile.channel.channelDetail.channelIcon.documentPath = requestObject.company_icon_url;
					// 		companyProfile.channel.channelDetail.channelIcon.dirName = appconstant.FOLDER_PATH_COMPANY_ICON;
					// 		await companyProfile.channel.channelDetail.channelIcon.save({
					// 			returning: true, transaction,
					// 		});
					// 		comapnyIconId = companyProfile.channel.channelDetail.channelIcon.updatedId;
					// 	} else {
					// 		// save comapny icon detail to document table
					// 		comapnyIcon = await this.mediaRepository.create({
					// 			documentPath: requestObject.company_icon_url,
					// 			documentTypeId: appconstant.DOCUMENTTYPE_IMAGE,
					// 			dirName: appconstant.FOLDER_PATH_COMPANY_ICON
					// 		},
					// 			{
					// 				returning: true, transaction,
					// 			});
					// 		comapnyIconId = comapnyIcon.documentId;
					// 	}
					//
					// }

					//Company Images
					if (requestObject.companyImages && Object.keys(requestObject.companyImages).length > 0) {
						for (let ii = 0; ii < requestObject.companyImages.length; ii++) {
								// save channel document
								if(!requestObject.companyImages[ii].companyMediaId){
								await this.companyMediaImagesRepository.create({
									channelId: companyProfile.channelId,
									documentId: requestObject.companyImages[ii].documentId,
									contactId:companyProfile.contactId,
									updatedBy: companyProfile.userId
								},
									{
										 transaction
									});
									}
						}
					}

					if (requestObject.deleted_companyImages && requestObject.deleted_companyImages.length > 0) {
						for (let ii = 0; ii < requestObject.deleted_companyImages.length; ii++) {
							if (requestObject.deleted_companyImages[ii]) {
								// update award details to channel award table
								await this.companyMediaImagesRepository.update({
									deletedAt: new Date()
								},
									{
										where: { companyMediaId: requestObject.deleted_companyImages[ii] },
										returning: true, transaction,
									});
							}
						}
					}



					// update channel detals address to channel detail table
					return await this.channelDetailRepository.update({
						channelLogoId: comapnyLogoId,
						channelBannerId:bannerId,
						// channelIconId: comapnyIconId,
						// ownerFirstName: (requestObject.company_owner_first_name) ? requestObject.company_owner_first_name : null,
						// ownerLastName: (requestObject.company_owner_last_name) ? requestObject.company_owner_last_name : null,
						estYear: requestObject.year_of_estabilishment,
						// regYear: (requestObject.year_of_registration) ? requestObject.year_of_registration : null,
						phone1: requestObject.company_phone1,
						// phone2: (isUndefined(requestObject.company_phone2))? null : requestObject.company_phone2,
						// mobileNo: (isUndefined(requestObject.company_mobile_no))? null : requestObject.company_mobile_no,
						webSiteUrl: requestObject.company_website_url,
						introvideoId: requestObject.company_introvideo,
						detailDesc: requestObject.company_about,//sanitizeHtml(requestObject.company_about, appconstant.SANITIZE_HTML),
						expectations: requestObject.company_expectation,//(requestObject.company_expectation) ? sanitizeHtml(requestObject.company_expectation, appconstant.SANITIZE_HTML) : ''
						companyDesc: requestObject.companyDesc
					},
						{
							where: { channelId: companyProfile.channelId }, returning: true, transaction,
						});

				});
				if (result) {


					if(+user.channel.signUpStatusId == appconstant.CHANNEL_APPROVAL || requestObject.ratingId){

						let defaultdata = {
							channelId : +user.channel.channelId,
							ratingId : 1,
							isCompleted : true,
							createdAt : new Date()
						}
		
						await this.profileRepository.findOrCreate({where: {channelId: +user.channel.channelId, ratingId: 1}, defaults: defaultdata}).then((result) => {
							if(!result[1]){
								result[0].update(defaultdata);
							}
						});

					}

					return true;
				} else {
					return false;
				}
			}
		} catch (e) {
			console.log("Error+", e);
			// throw new InternalServerErrorException();
			
		}
		if (!companyProfile) {
			throw new ProcessException('channel:getCompanyProfile:companyNotFound');
		}
	}

    //image Update
    	async imageUpdate(imageId, image, transaction) {
		let logo = await this.mediaRepository.update(
			{ deletedAt: new Date() },
			{
				where: { documentId: imageId },
				returning: true, transaction,
			}
		).then(async () => {
			return await this.imageCreate(image, transaction);
		});
		return logo;
	}

    //update trade info
    // public async updatetradeInformation(params: any, user: any): Promise<boolean> { //TradeInformationDto
    //     console.log('params', params);
	// 	try {
	// 		let ChannelSpecialityObject = <Array<ChannelSpeciality>>[];
	// 		let userDetails = await this.userRepository.scope('userBasicInfom').findOne({
	// 			where: { userId: parseInt(user.userId), isActive: '1' }
	// 		});
    //         console.log('channelspeciality', userDetails);
	// 		if (userDetails) {
	// 			let result: any = await this.sequelizeInstance.transaction(async transaction => {

	// 				// let contact = await this.contactRepository.scope('basicInfom').findOne({
	// 				// 	where: {
	// 				// 		contactId: user.contactId,
	// 				// 		channelId: user.channelId
	// 				// 	},
	// 				// 	include: [
	// 				// 		{
	// 				// 			model: ProductGroup, as: 'productGroups', required: false, where: {
	// 				// 				deletedAt: null
	// 				// 			}, through: { attributes: [], where: { deletedAt: null } }, attributes: ['productGroupId']
	// 				// 		},
	// 				// 		{
	// 				// 			model: ProductCategory, as: 'productCategories', required: false, where: {
	// 				// 				deletedAt: null
	// 				// 			}, through: { attributes: [], where: { deletedAt: null } }, attributes: ['productCategorieId']
	// 				// 		}
	// 				// 	]
	// 				// })

	// 				let channelContacts = await this.contactRepository.scope('basicInfom').findAll({
	// 					where: {
	// 						deletedAt: null,
	// 						contactId: { [Op.ne]: user.contactId },
	// 						channelId: user.channelId
	// 					}
	// 				});
    //                 console.log('channelcontacts', channelContacts);
	// 				let contacts = [];
	// 				contacts.push(user.contactId);
	// 				if (channelContacts?.length > 0) {
	// 					channelContacts.map(function (data) {
	// 						contacts.push(data.contactId);
	// 					});
	// 				}
                    
	// 				let ChannelDetailObject = {
    //                     empCntRange: params.no_of_staffs,
    //                     turnoverRange: params.annualTurnOver,
    //                     revD: params.domestic,
    //                     revI: params.interRev,
    //                     startup: params.areYouStartUp,
    //                     maturitylevel: params.maturitylevel,
    //                     storeCount: params.no_of_skus,
    //                     empCntDSale: params.sales,
    //                     empCntDMar: params.marketing,
    //                     no_of_distributors: params.no_of_distributors,
    //                     no_of_retailers: params.no_of_retailers,
    //                     no_of_vendors: params.no_of_vendors,

    //                  } ; //<ChannelDetail>{};
    //                  let channelCustomerProfilesCurrentIds = [];
	// 				 let ChannelCustomerProfilesCreateObject = [];
    //                 console.log('channeldetal', params.productFamilies);
	// 				if (params.productFamilies.length > 0) {
    //                     console.log('productFamilies');
	// 					let productGroup = await this.channelProductGroupRepository.findAll({
	// 						where: {
	// 							channelId: user.channelId,
	// 							deletedAt: null
	// 						}
	// 					});

	// 					let ChannelProductGroupIds = await this.tradeInformationInBound.setProductGroupIds(productGroup);
	// 					let ChannelProductGroupObject: ChannelProductGroup[] = await this.tradeInformationInBound.setProductGroup(params.productFamilies, user.channelId);
	// 					let ChannelProductGroupCurrentIds = [];
	// 					let ChannelProductGroupCreateObject = [];

	// 					/* Account User contact update */
	// 					// let ContactProductGroupIds = await this.tradeInformationInBound.setProductGroupIds(contact.productGroups);
	// 					let ContactProductGroupObject: ContactProductGroup[] = await this.tradeInformationInBound.setContactProductGroup(params.productFamilies, user.contactId);
	// 					let ContactProductGroupCurrentIds = [];
	// 					// let ContactProductGroupCreateObject = [];
    //                     console.log('productgropu', ContactProductGroupObject, ChannelProductGroupObject);
	// 					for (let kk = 0; kk < ChannelProductGroupObject.length; kk++) {
	// 						if (ChannelProductGroupIds.includes(ChannelProductGroupObject[kk].productGroupId)) {
	// 							ChannelProductGroupCurrentIds.push(ChannelProductGroupObject[kk].productGroupId);
	// 						} else {
	// 							ChannelProductGroupCreateObject.push(ChannelProductGroupObject[kk]);
	// 							ChannelProductGroupCurrentIds.push(ChannelProductGroupObject[kk].productGroupId);
	// 						}
	// 						// if (ContactProductGroupIds.includes(ContactProductGroupObject[kk].productGroupId)) {
	// 						// 	ContactProductGroupCurrentIds.push(ContactProductGroupObject[kk].productGroupId);
	// 						// } else {
	// 						// 	// ContactProductGroupCreateObject.push(ContactProductGroupObject[kk]);
	// 						// 	ContactProductGroupCurrentIds.push(ContactProductGroupObject[kk].productGroupId);
	// 						// }

	// 						ContactProductGroupCurrentIds.push(ContactProductGroupObject[kk].productGroupId);
	// 					}
	// 					if (ChannelProductGroupCreateObject.length > 0) {
	// 						await this.channelProductGroupRepository.bulkCreate(ChannelProductGroupCreateObject, { returning: true, transaction });
	// 					}

	// 					// if (ContactProductGroupCreateObject.length > 0) {
	// 					// 	await this.contactProductGroupRepository.bulkCreate(ContactProductGroupCreateObject, { returning: true, transaction });
	// 					// }
    //                     ChannelProductGroupCurrentIds = [...new Set(ChannelProductGroupCurrentIds)];
    //                     console.log('ChannelProductGroupCurrentIds', ChannelProductGroupCurrentIds);
	// 					let data = await this.channelProductGroupRepository.update({ deletedAt: new Date() }, {
	// 						where: {
	// 							channelId: user.channelId,
	// 							productGroupId: { [Op.notIn]: ChannelProductGroupCurrentIds  },
	// 							deletedAt: null
	// 						}, returning: true, transaction
	// 					});
    //                     console.log('data', data);
	// 					await this.contactGroupRepository.update({ deletedAt: new Date() }, {
	// 						where: {
	// 							contactId: { [Op.in]: contacts },
	// 							productGroupId: { [Op.notIn]: ContactProductGroupCurrentIds },
	// 							deletedAt: null
	// 						}, returning: true, transaction
	// 					});
	// 				}
                    
	// 				if (params.productCategories.length > 0) {

	// 					let productCategory = await this.channelProductCategorieRepository.findAll({
	// 						where: {
	// 							channelId: user.channelId,
	// 							deletedAt: null
	// 						}
	// 					});

	// 					let productCategoriesIds = await this.tradeInformationInBound.setProductCategoryIds(productCategory);
	// 					let ProductCategorieObject: ChannelProductCategorie[] = await this.tradeInformationInBound.setProductCategories(params.productCategories, user.channelId);
	// 					let ChannelProductCategoriesCurrentIds = [];
	// 					let ChannelProductCategoriesCreateObject = [];

	// 					/* Account User contact update */
	// 					// let contactProductCategoriesIds = await this.tradeInformationInBound.setProductCategoryIds(contact.productCategories);
	// 					let contactProductCategorieObject: ContactProductCategory[] = await this.tradeInformationInBound.setContactProductCategories(params.productCategories, user.contactId);
	// 					let ContactProductCategoriesCurrentIds = [];
	// 					// let ContactProductCategoriesCreateObject = [];

	// 					for (let kk = 0; kk < ProductCategorieObject.length; kk++) {
	// 						if (productCategoriesIds.includes(ProductCategorieObject[kk].productCategorieId)) {
	// 							ChannelProductCategoriesCurrentIds.push(ProductCategorieObject[kk].productCategorieId);
	// 						} else {
	// 							ChannelProductCategoriesCreateObject.push(ProductCategorieObject[kk]);
	// 							ChannelProductCategoriesCurrentIds.push(ProductCategorieObject[kk].productCategorieId);
	// 						}
	// 						// if (contactProductCategoriesIds.includes(contactProductCategorieObject[kk].productCategorieId)) {
	// 						// 	ContactProductCategoriesCurrentIds.push(contactProductCategorieObject[kk].productCategorieId);
	// 						// } else {
	// 						// 	// ContactProductCategoriesCreateObject.push(contactProductCategorieObject[kk]);
	// 						// 	ContactProductCategoriesCurrentIds.push(contactProductCategorieObject[kk].productCategorieId);
	// 						// }

	// 						ContactProductCategoriesCurrentIds.push(contactProductCategorieObject[kk].productCategorieId);
    //                         console.log('ChannelProductCategoriesCreateObject',ChannelProductCategoriesCreateObject)
	// 					}
	// 					if (ChannelProductCategoriesCreateObject.length > 0) {
	// 						await this.channelProductCategorieRepository.bulkCreate(ChannelProductCategoriesCreateObject, { returning: true, transaction });
	// 					}

	// 					// if (ContactProductCategoriesCreateObject.length > 0) {
	// 					// 	await this.contactCatRepository.bulkCreate(ContactProductCategoriesCreateObject, { returning: true, transaction });
	// 					// }
    //                     ChannelProductCategoriesCurrentIds = [... new Set(ChannelProductCategoriesCurrentIds)]
	// 					await this.channelProductCategorieRepository.update({ deletedAt: new Date() }, {
	// 						where: {
	// 							channelId: user.channelId,
	// 							productCategorieId: { [Op.notIn]: ChannelProductCategoriesCurrentIds },
	// 							deletedAt: null
	// 						}, returning: true, transaction
	// 					});

	// 					await this.contactCatRepository.update({ deletedAt: new Date() }, {
	// 						where: {
	// 							contactId: { [Op.in]: contacts },
	// 							productCategorieId: { [Op.notIn]: ContactProductCategoriesCurrentIds },
	// 							deletedAt: null
	// 						}, returning: true, transaction
	// 					});
	// 				}

	// 				if (params.customerProfiles && params.customerProfiles.length > 0) {

	// 					let customerProfiles = await this.channelCustomerProfileRepository.findAll({
	// 						where: {
	// 							channelId: user.channelId,
	// 							deletedAt: null
	// 						}
	// 					});

	// 					let customerProfilesIds = await this.tradeInformationInBound.setCustomerProfilesIds(customerProfiles);
	// 					let customerProfilesObject: ChannelCustomerProfile[] = await this.tradeInformationInBound.setCustomerProfiles(params.customerProfiles, user.channelId);
						
	// 					for (let kk = 0; kk < customerProfilesObject.length; kk++) {
	// 						if (customerProfilesIds.includes(customerProfilesObject[kk].profileTypeId)) {
	// 							channelCustomerProfilesCurrentIds.push(customerProfilesObject[kk].profileTypeId);
	// 						} else {
	// 							ChannelCustomerProfilesCreateObject.push(customerProfilesObject[kk]);
	// 							channelCustomerProfilesCurrentIds.push(customerProfilesObject[kk].profileTypeId);
	// 						}
	// 					}
    //                     console.log('ChannelCustomerProfilesCreateObject', ChannelCustomerProfilesCreateObject);
	// 					if (ChannelCustomerProfilesCreateObject.length > 0) {
	// 						await this.channelCustomerProfileRepository.bulkCreate(ChannelCustomerProfilesCreateObject, { returning: true, transaction });
	// 					}

	// 					await this.channelCustomerProfileRepository.update({ deletedAt: new Date() }, {
	// 						where: {
	// 							channelId: user.channelId,
	// 							profileTypeId: { [Op.notIn]: channelCustomerProfilesCurrentIds },
	// 							deletedAt: null
	// 						}, returning: true, transaction
	// 					});
	// 				}

	// 				let channelObject = <Channel>{
	// 					isFrenchTech: params.is_french_tech
	// 				};
	// 				if (params.channelType == appconstant.CHANNEL_TYPE_VENDOR) {
    //                     console.log('savingbrandforvendor');
	// 					//saving brand
	// 					if (params.brands.length > 0) {

	// 						let channelBrandObject = await this.saveBrand(params.brands, user.channelId, transaction);
    //                         console.log('savingbrand', channelBrandObject); 
	// 						if (channelBrandObject.channelBrandArrayObject.length > 0) {

	// 							let brand = await this.channelBrandRepository.findAll({
	// 								where: {
	// 									channelId: user.channelId,
	// 									deletedAt: null
	// 								}
	// 							});

	// 							let ChannelBrandCurrentIds = [];
	// 							let ChannelBrandCreateObject = [];
	// 							let brandObject = channelBrandObject.channelBrandArrayObject;
	// 							let brandIds = channelBrandObject.brandIds;

	// 							let ChannelBrandIds = await this.tradeInformationInBound.setchannelbrandIds(brand);
	// 							console.log('brands',brand,ChannelBrandIds);
	// 							let totalRemovedBrands = ChannelBrandIds.filter(e => !brandObject.find(a => e == a.brandId));

	// 							for (let kk = 0; kk < brandObject.length; kk++) {
	// 								if (!ChannelBrandIds.includes(+brandObject[kk].brandId)) {
	// 									ChannelBrandCreateObject.push(brandObject[kk]);
	// 								}

	// 								if(!totalRemovedBrands.includes(+brandObject[kk].brandId)) { 
	// 									ChannelBrandCurrentIds.push(+brandObject[kk].brandId)
	// 								}
	// 							}

	// 							if (ChannelBrandCreateObject.length > 0) {
	// 								await this.channelBrandRepository.bulkCreate(ChannelBrandCreateObject, { returning: true, transaction });
	// 							}

    //                             console.log('channelcurids',ChannelBrandCurrentIds);
    //                             console.log('ChannelBrandCreateObject',ChannelBrandCreateObject);

	// 							await this.channelBrandRepository.update({ deletedAt: new Date() }, {
	// 								where: {
	// 									channelId: user.channelId,
	// 									brandId: { [Op.notIn]: ChannelBrandCurrentIds },
	// 									deletedAt: null
	// 								}, returning: true, transaction
	// 							});

	// 							await this.brandRepository.update({
	// 								isAssigned: '1'
	// 							}, {
	// 									where: {
	// 										id: {
	// 											[Op.in]: channelBrandObject.brandIds
	// 										}
	// 									},
	// 									returning: true, transaction
	// 								}
	// 							);

	// 							await this.brandRepository.update({
	// 								isAssigned: '0'
	// 							}, {
	// 									where: {
	// 										id: {
	// 											[Op.in]: totalRemovedBrands
	// 										}
	// 									},
	// 									returning: true, transaction
	// 								}
	// 							);
	// 						}
	// 					}

	// 					if (params.tradeInformationVendor && params.tradeInformationVendor.profileTypeId) {
	// 						//updateing channel
	// 						channelObject.profileTypeId = params.tradeInformationVendor.profileTypeId;
	// 					}

	// 					// update key Retailers
	// 					await this.tradeInformationInBound.keyRetailers(appconstant.CHANNEL_TYPE_RETAILER, params.channelKeyRetailer, user.channelId, user.userId, params.deletedkeyretailer);

	// 					// create key Distributors
	// 					await this.tradeInformationInBound.keyRetailers(appconstant.CHANNEL_TYPE_DISTRIBUTOR, params.channelKeyDistributor, user.channelId, user.userId, params.deletedkeydistributor);

	// 					// ChannelDetailObject = await this.tradeInformationInBound.setChannelDetail(params.tradeInformationVendor, user.channelId);

	// 				} else if (params.channelType == appconstant.CHANNEL_TYPE_DISTRIBUTOR) {
	// 					//saving brand
	// 					if ( params.brands && params.brands.length > 0) {
	// 						let channelBrandObject = await this.saveBrand(params.brands, user.channelId, transaction);
    //                         console.log('channelBrandObject',channelBrandObject);
	// 						if (channelBrandObject.channelBrandArrayObject.length > 0) {
	// 							let brand = await this.channelBrandRepository.findAll({
	// 								where: {
	// 									channelId: user.channelId,
	// 									deletedAt: null
	// 								}
	// 							});

	// 							let ChannelBrandCurrentIds = [];
	// 							let ChannelBrandCreateObject = [];
	// 							let brandObject = channelBrandObject.channelBrandArrayObject;

	// 							let ChannelBrandIds = await this.tradeInformationInBound.setchannelbrandIds(brand);
	// 							let totalRemovedBrands = ChannelBrandIds.filter(e => !brandObject.find(a => e == a.brandId));
    //                             console.log('removebrands', totalRemovedBrands);
    //                             console.log('channelBrandIds', ChannelBrandIds);
    //                             console.log('brandObject', brandObject, brand);
	// 							for (let kk = 0; kk < brandObject.length; kk++) {
	// 								if (!ChannelBrandIds.includes(+brandObject[kk].brandId)) {
	// 									ChannelBrandCreateObject.push(brandObject[kk]);
	// 								}

	// 								if(!totalRemovedBrands.includes(+brandObject[kk].brandId)) {
	// 									ChannelBrandCurrentIds.push(+brandObject[kk].brandId);
	// 								}
	// 							}

	// 							if (ChannelBrandCreateObject.length > 0) {
	// 								await this.channelBrandRepository.bulkCreate(ChannelBrandCreateObject, { returning: true, transaction });
	// 							}

	// 							await this.channelBrandRepository.update({ deletedAt: new Date() }, {
	// 								where: {
	// 									channelId: user.channelId,
	// 									brandId: { [Op.notIn]: ChannelBrandCurrentIds },
	// 									deletedAt: null
	// 								}, returning: true, transaction
	// 							});

	// 						    console.log('brandasa',JSON.stringify(ChannelBrandCurrentIds));

	// 							await this.brandRepository.update({
	// 								isAssigned: '1'
	// 							}, {
	// 									where: {
	// 										id: {
	// 											[Op.in]: channelBrandObject.brandIds
	// 										}
	// 									},
	// 									returning: true, transaction
	// 								}
	// 							);

                                
                                
	// 							await this.brandRepository.update({
	// 								isAssigned: '0'
	// 							}, {
	// 									where: {
	// 										id: {
	// 											[Op.in]: totalRemovedBrands
	// 										}
	// 									},
	// 									returning: true, transaction
	// 								}
	// 							);

	// 							console.log(JSON.stringify(totalRemovedBrands));
	// 						}
	// 					}

	// 					//ChannelDetail Object
	// 					// ChannelDetailObject = {} //await this.tradeInformationInBound.setChannelDetail(params.tradeInformationDistributor, user.channelId);

	// 					//Speciality Object
	// 					// ChannelSpecialityObject = await this.tradeInformationInBound.setSpeciality(params.distributorSpeciality, user.channelId);

	// 					//saving service List
	// 					if (params.listOfServices.length > 0) {

	// 						let serviceList = await this.channelServiceListRepository.findAll({
	// 							where: {
	// 								channelId: user.channelId,
	// 								deletedAt: null
	// 							}
	// 						});

	// 						let ChannelServiceListIds = await this.tradeInformationInBound.setServiceListIds(serviceList);
	// 						let ChannelServiceListObject: ChannelServiceList[] = await this.tradeInformationInBound.setServiceList(params.listOfServices, user.channelId);
	// 						let ChannelServiceListCurrentIds = [];
	// 						let ChannelServiceListCreateObject = [];
	// 						for (let kk = 0; kk < ChannelServiceListObject.length; kk++) {
	// 							if (ChannelServiceListIds.includes(ChannelServiceListObject[kk].serviceListId)) {
	// 								ChannelServiceListCurrentIds.push(ChannelServiceListObject[kk].serviceListId);
	// 							} else {
	// 								ChannelServiceListCreateObject.push(ChannelServiceListObject[kk]);
	// 								ChannelServiceListCurrentIds.push(ChannelServiceListObject[kk].serviceListId);
	// 							}
	// 						}
	// 						if (ChannelServiceListCreateObject.length > 0) {
	// 							await this.channelServiceListRepository.bulkCreate(ChannelServiceListCreateObject, { returning: true, transaction });
	// 						}

	// 						await this.channelServiceListRepository.update({ deletedAt: new Date() }, {
	// 							where: {
	// 								channelId: user.channelId,
	// 								serviceListId: { $notIn: ChannelServiceListCurrentIds },
	// 								deletedAt: null
	// 							}, returning: true, transaction
	// 						});
	// 					}

	// 					// update key Retailers
	// 					await this.tradeInformationInBound.keyRetailers(appconstant.CHANNEL_TYPE_RETAILER, params.channelKeyRetailer, user.channelId, user.userId, params.deletedkeyretailer);

	// 					//updateing channel
	// 					channelObject.profileTypeId = params.channelTypeId;
	// 				} else if (params.channelType == appconstant.CHANNEL_TYPE_RETAILER) {

	// 					//ChannelDetail Object
	// 					// ChannelDetailObject = await this.tradeInformationInBound.setChannelDetail(ChannelDetailObject, user.channelId);

	// 					//Speciality Object
	// 					/*ChannelSpecialityObject = await this.tradeInformationInBound.setSpeciality(params.tradeInformationRetailer.speciality, user.channelId);*/
	// 					channelObject.profileTypeId = ChannelCustomerProfilesCreateObject[0].profileTypeId;
	// 				}

	// 				if (ChannelSpecialityObject.length > 0) {

	// 					let speciality = await this.channelSpecialityRepository.findAll({
	// 						where: {
	// 							channelId: user.channelId,
	// 							deletedAt: null
	// 						}
	// 					});

	// 					let ChannelSpecialityIds = await this.tradeInformationInBound.setProductGroupIds(speciality);
	// 					let ChannelSpecialityCurrentIds = [];
	// 					let ChannelSpecialityCreateObject = [];
	// 					for (let kk = 0; kk < ChannelSpecialityObject.length; kk++) {
	// 						if (ChannelSpecialityIds.includes(ChannelSpecialityObject[kk].productGroupId)) {
	// 							ChannelSpecialityCurrentIds.push(ChannelSpecialityObject[kk].productGroupId);
	// 						} else {
	// 							ChannelSpecialityCreateObject.push(ChannelSpecialityObject[kk]);
	// 							ChannelSpecialityCurrentIds.push(ChannelSpecialityObject[kk].productGroupId);
	// 						}
	// 					}
	// 					if (ChannelSpecialityCreateObject.length > 0) {
	// 						await this.channelSpecialityRepository.bulkCreate(ChannelSpecialityCreateObject, { returning: true, transaction });
	// 					}

	// 					await this.channelSpecialityRepository.update({ deletedAt: new Date() }, {
	// 						where: {
	// 							channelId: user.channelId,
	// 							productGroupId: { $notIn: ChannelSpecialityCurrentIds },
	// 							deletedAt: null
	// 						}, returning: true, transaction
	// 					});
	// 				}

	// 				if (ChannelDetailObject) {
	// 					await this.channelDetailRepository.update(ChannelDetailObject, { where: { channelId: user.channelId }, returning: true, transaction });
	// 				}

	// 				// update channel information
	// 				return await this.channelRepository.update(channelObject, { where: { channelId: user.channelId }, returning: true, transaction });
	// 			});

	// 			if (result) {
	// 				return true;
	// 			} else {
	// 				return false;
	// 			}
	// 		} else {
	// 			// throw new exception('channel:subscripion:userNotFound');
    //             console.log('errorintradeinfor')
	// 		}
	// 	} catch (e) {
	// 		console.log("Error+", e);
	// 		// throw new InternalServerErrorException();
	// 	}
	// }
    

    //update trade Info 1

    public async updatetradeInformation(params: any, user: any): Promise<boolean> {
		try {
			let ChannelSpecialityObject = <Array<ChannelSpeciality>>[];
			let userDetails = await this.userRepository.scope('userBasicInfom').findOne({
				where: { userId: parseInt(user.userId), isActive: '1' }
			});

			if (userDetails) {
				let result: any = await this.sequelizeInstance.transaction(async transaction => {

					// let contact = await this.contactRepository.scope('basicInfom').findOne({
					// 	where: {
					// 		contactId: user.contactId,
					// 		channelId: user.channelId
					// 	},
					// 	include: [
					// 		{
					// 			model: ProductGroup, as: 'productGroups', required: false, where: {
					// 				deletedAt: null
					// 			}, through: { attributes: [], where: { deletedAt: null } }, attributes: ['productGroupId']
					// 		},
					// 		{
					// 			model: ProductCategory, as: 'productCategories', required: false, where: {
					// 				deletedAt: null
					// 			}, through: { attributes: [], where: { deletedAt: null } }, attributes: ['productCategorieId']
					// 		}
					// 	]
					// })

					let channelContacts = await this.contactRepository.scope('basicInfom').findAll({
						where: {
							deletedAt: null,
							contactId: { [Op.ne]: user.contactId },
							channelId: user.channelId
						}
					});

					let contacts = [];
					contacts.push(user.contactId);
					if (channelContacts.length > 0) {
						channelContacts.map(function (data) {
							contacts.push(data.contactId);
						});
					}

					let ChannelDetailObject = <ChannelDetail>{};

					if (params.productFamilies?.length > 0) {

						let productGroup = await this.channelProductGroupRepository.findAll({
							where: {
								channelId: user.channelId,
								deletedAt: null
							}
						});

						let ChannelProductGroupIds = await this.tradeInformationInBound.setProductGroupIds(productGroup);
						let ChannelProductGroupObject: ChannelProductGroup[] = await this.tradeInformationInBound.setProductGroup(params.productFamilies, user.channelId);
						let ChannelProductGroupCurrentIds = [];
						let ChannelProductGroupCreateObject = [];

						/* Account User contact update */
						// let ContactProductGroupIds = await this.tradeInformationInBound.setProductGroupIds(contact.productGroups);
						let ContactProductGroupObject: ContactProductGroup[] = await this.tradeInformationInBound.setContactProductGroup(params.productFamilies, user.contactId);
						let ContactProductGroupCurrentIds = [];
						// let ContactProductGroupCreateObject = [];

						for (let kk = 0; kk < ChannelProductGroupObject.length; kk++) {
							if (ChannelProductGroupIds.includes(ChannelProductGroupObject[kk].productGroupId)) {
								ChannelProductGroupCurrentIds.push(ChannelProductGroupObject[kk].productGroupId);
							} else {
								ChannelProductGroupCreateObject.push(ChannelProductGroupObject[kk]);
								ChannelProductGroupCurrentIds.push(ChannelProductGroupObject[kk].productGroupId);
							}
							// if (ContactProductGroupIds.includes(ContactProductGroupObject[kk].productGroupId)) {
							// 	ContactProductGroupCurrentIds.push(ContactProductGroupObject[kk].productGroupId);
							// } else {
							// 	// ContactProductGroupCreateObject.push(ContactProductGroupObject[kk]);
							// 	ContactProductGroupCurrentIds.push(ContactProductGroupObject[kk].productGroupId);
							// }

							ContactProductGroupCurrentIds.push(ContactProductGroupObject[kk].productGroupId);
						}
						if (ChannelProductGroupCreateObject.length > 0) {
							await this.channelProductGroupRepository.bulkCreate(ChannelProductGroupCreateObject, { returning: true, transaction });
						}

						// if (ContactProductGroupCreateObject.length > 0) {
						// 	await this.contactProductGroupRepository.bulkCreate(ContactProductGroupCreateObject, { returning: true, transaction });
						// }

						await this.channelProductGroupRepository.update({ deletedAt: new Date() }, {
							where: {
								channelId: user.channelId,
								productGroupId: { [Op.notIn]: ChannelProductGroupCurrentIds },
								deletedAt: null
							}, returning: true, transaction
						});

						await this.contactGroupRepository.update({ deletedAt: new Date() }, {
							where: {
								contactId: { [Op.in]: contacts },
								productGroupId: { [Op.notIn]: ContactProductGroupCurrentIds },
								deletedAt: null
							}, returning: true, transaction
						});
					}

					if (params.productCategories?.length > 0) {

						let productCategory = await this.channelProductCategorieRepository.findAll({
							where: {
								channelId: user.channelId,
								deletedAt: null
							}
						});

						let productCategoriesIds = await this.tradeInformationInBound.setProductCategoryIds(productCategory);
						let ProductCategorieObject: ChannelProductCategorie[] = await this.tradeInformationInBound.setProductCategories(params.productCategories, user.channelId);
						let ChannelProductCategoriesCurrentIds = [];
						let ChannelProductCategoriesCreateObject = [];

						/* Account User contact update */
						// let contactProductCategoriesIds = await this.tradeInformationInBound.setProductCategoryIds(contact.productCategories);
						let contactProductCategorieObject: ContactProductCategory[] = await this.tradeInformationInBound.setContactProductCategories(params.productCategories, user.contactId);
						let ContactProductCategoriesCurrentIds = [];
						// let ContactProductCategoriesCreateObject = [];

						for (let kk = 0; kk < ProductCategorieObject.length; kk++) {
							if (productCategoriesIds.includes(ProductCategorieObject[kk].productCategorieId)) {
								ChannelProductCategoriesCurrentIds.push(ProductCategorieObject[kk].productCategorieId);
							} else {
								ChannelProductCategoriesCreateObject.push(ProductCategorieObject[kk]);
								ChannelProductCategoriesCurrentIds.push(ProductCategorieObject[kk].productCategorieId);
							}
							// if (contactProductCategoriesIds.includes(contactProductCategorieObject[kk].productCategorieId)) {
							// 	ContactProductCategoriesCurrentIds.push(contactProductCategorieObject[kk].productCategorieId);
							// } else {
							// 	// ContactProductCategoriesCreateObject.push(contactProductCategorieObject[kk]);
							// 	ContactProductCategoriesCurrentIds.push(contactProductCategorieObject[kk].productCategorieId);
							// }

							ContactProductCategoriesCurrentIds.push(contactProductCategorieObject[kk].productCategorieId);
						}
						if (ChannelProductCategoriesCreateObject.length > 0) {
							await this.channelProductCategorieRepository.bulkCreate(ChannelProductCategoriesCreateObject, { returning: true, transaction });
						}

						// if (ContactProductCategoriesCreateObject.length > 0) {
						// 	await this.contactProductCategoryRepository.bulkCreate(ContactProductCategoriesCreateObject, { returning: true, transaction });
						// }

						await this.channelProductCategorieRepository.update({ deletedAt: new Date() }, {
							where: {
								channelId: user.channelId,
								productCategorieId: { [Op.notIn]: ChannelProductCategoriesCurrentIds },
								deletedAt: null
							}, returning: true, transaction
						});

						await this.contactCatRepository.update({ deletedAt: new Date() }, {
							where: {
								contactId: { [Op.in]: contacts },
								productCategorieId: { [Op.notIn]: ContactProductCategoriesCurrentIds },
								deletedAt: null
							}, returning: true, transaction
						});
					}

					if (params.customerProfiles && params.customerProfiles?.length > 0) {

						let customerProfiles = await this.channelCustomerProfileRepository.findAll({
							where: {
								channelId: user.channelId,
								deletedAt: null
							}
						});

						let customerProfilesIds = await this.tradeInformationInBound.setCustomerProfilesIds(customerProfiles);
						let customerProfilesObject: ChannelCustomerProfile[] = await this.tradeInformationInBound.setCustomerProfiles(params.customerProfiles, user.channelId);
						let channelCustomerProfilesCurrentIds = [];
						let ChannelCustomerProfilesCreateObject = [];
						for (let kk = 0; kk < customerProfilesObject.length; kk++) {
							if (customerProfilesIds.includes(customerProfilesObject[kk].profileTypeId)) {
								channelCustomerProfilesCurrentIds.push(customerProfilesObject[kk].profileTypeId);
							} else {
								ChannelCustomerProfilesCreateObject.push(customerProfilesObject[kk]);
								channelCustomerProfilesCurrentIds.push(customerProfilesObject[kk].profileTypeId);
							}
						}
						if (ChannelCustomerProfilesCreateObject.length > 0) {
							await this.channelCustomerProfileRepository.bulkCreate(ChannelCustomerProfilesCreateObject, { returning: true, transaction });
						}

						await this.channelCustomerProfileRepository.update({ deletedAt: new Date() }, {
							where: {
								channelId: user.channelId,
								profileTypeId: { [Op.notIn]: channelCustomerProfilesCurrentIds },
								deletedAt: null
							}, returning: true, transaction
						});
					}

					let channelObject = <Channel>{
						isFrenchTech: params.is_french_tech
					};
					if (params.channelType == appconstant.CHANNEL_TYPE_VENDOR) {

						//saving brand
						if (params.tradeInformationVendor.brand?.length > 0) {

							let channelBrandObject = await this.saveBrand(params.tradeInformationVendor.brand, user.channelId, transaction);

							if (channelBrandObject.channelBrandArrayObject.length > 0) {

								let brand = await this.channelBrandRepository.findAll({
									where: {
										channelId: user.channelId,
										deletedAt: null
									}
								});

								let ChannelBrandCurrentIds = [];
								let ChannelBrandCreateObject = [];
								let brandObject = channelBrandObject.channelBrandArrayObject;
								let brandIds = channelBrandObject.brandIds;

								let ChannelBrandIds = await this.tradeInformationInBound.setchannelbrandIds(brand);
								let totalRemovedBrands = ChannelBrandIds.filter(e => !brandIds.find(a => e == a));

								for (let kk = 0; kk < brandObject.length; kk++) {
									if (!ChannelBrandIds.includes(brandObject[kk].brandId)) {
										ChannelBrandCreateObject.push(brandObject[kk]);
									}

									if(!totalRemovedBrands.includes(+brandObject[kk].brandId)) { 
										ChannelBrandCurrentIds.push(brandObject[kk].brandId)
									}
								}

								if (ChannelBrandCreateObject.length > 0) {
									await this.channelBrandRepository.bulkCreate(ChannelBrandCreateObject, { returning: true, transaction });
								}

								await this.channelBrandRepository.update({ deletedAt: new Date() }, {
									where: {
										channelId: user.channelId,
										brandId: { [Op.notIn]: ChannelBrandCurrentIds },
										deletedAt: null
									}, returning: true, transaction
								});

								await this.brandRepository.update({
									isAssigned: '1'
								}, {
										where: {
											id: {
												[Op.in]: brandIds
											}
										},
										returning: true, transaction
									}
								);

								await this.brandRepository.update({
									isAssigned: '0'
								}, {
										where: {
											id: {
												[Op.in]: totalRemovedBrands
											}
										},
										returning: true, transaction
									}
								);
							}
						}

						if (params.tradeInformationVendor && params.tradeInformationVendor.profileTypeId) {
							//updateing channel
							channelObject.profileTypeId = params.tradeInformationVendor.profileTypeId;
						}

						// update key Retailers
						await this.tradeInformationInBound.keyRetailers(appconstant.CHANNEL_TYPE_RETAILER, params.tradeInformationVendor.channelKeyRetailer, user.channelId, user.userId, params.tradeInformationVendor.deletedkeyretailer);

						// create key Distributors
						await this.tradeInformationInBound.keyRetailers(appconstant.CHANNEL_TYPE_DISTRIBUTOR, params.tradeInformationVendor.channelKeyDistributor, user.channelId, user.userId, params.tradeInformationVendor.deletedkeydistributor);

						ChannelDetailObject = await this.tradeInformationInBound.setChannelDetail(params.tradeInformationVendor, user.channelId, user.channelTypeId);

					} else if (params.channelType == appconstant.CHANNEL_TYPE_DISTRIBUTOR) {
						//saving brand
						if (params.tradeInformationDistributor && params.tradeInformationDistributor?.brand && params.tradeInformationDistributor.brand.length > 0) {
							let channelBrandObject = await this.saveBrand(params.tradeInformationDistributor.brand, user.channelId, transaction);

							if (channelBrandObject.channelBrandArrayObject.length > 0) {
								let brand = await this.channelBrandRepository.findAll({
									where: {
										channelId: user.channelId,
										deletedAt: null
									}
								});

								let ChannelBrandCurrentIds = [];
								let ChannelBrandCreateObject = [];
								let brandObject = channelBrandObject.channelBrandArrayObject;

								let ChannelBrandIds = await this.tradeInformationInBound.setchannelbrandIds(brand);
								let totalRemovedBrands = ChannelBrandIds.filter(e => !brandObject.find(a => e == a.brandId));

								for (let kk = 0; kk < brandObject.length; kk++) {
									if (!ChannelBrandIds.includes(brandObject[kk].brandId)) {
										ChannelBrandCreateObject.push(brandObject[kk]);
									}

									if(!totalRemovedBrands.includes(+brandObject[kk].brandId)) {
										ChannelBrandCurrentIds.push(brandObject[kk].brandId);
									}
								}

								if (ChannelBrandCreateObject.length > 0) {
									await this.channelBrandRepository.bulkCreate(ChannelBrandCreateObject, { returning: true, transaction });
								}

								await this.channelBrandRepository.update({ deletedAt: new Date() }, {
									where: {
										channelId: user.channelId,
										brandId: { [Op.notIn]: ChannelBrandCurrentIds },
										deletedAt: null
									}, returning: true, transaction
								});

								console.log(JSON.stringify(ChannelBrandCurrentIds));

								await this.brandRepository.update({
									isAssigned: '1'
								}, {
										where: {
											id: {
												[Op.in]: channelBrandObject.brandIds
											}
										},
										returning: true, transaction
									}
								);

								await this.brandRepository.update({
									isAssigned: '0'
								}, {
										where: {
											id: {
												[Op.in]: totalRemovedBrands
											}
										},
										returning: true, transaction
									}
								);

								console.log(JSON.stringify(totalRemovedBrands));
							}
						}

						//ChannelDetail Object
						ChannelDetailObject = await this.tradeInformationInBound.setChannelDetail(params.tradeInformationDistributor, user.channelId, user.channelTypeId);

						//Speciality Object
						ChannelSpecialityObject = await this.tradeInformationInBound.setSpeciality(params.tradeInformationDistributor.distributorSpeciality, user.channelId);

						//saving service List
						if (params.tradeInformationDistributor.listOfServices?.length > 0) {

							let serviceList = await this.channelServiceListRepository.findAll({
								where: {
									channelId: user.channelId,
									deletedAt: null
								}
							});

							let ChannelServiceListIds = await this.tradeInformationInBound.setServiceListIds(serviceList);
							let ChannelServiceListObject: ChannelServiceList[] = await this.tradeInformationInBound.setServiceList(params.tradeInformationDistributor.listOfServices, user.channelId);
							let ChannelServiceListCurrentIds = [];
							let ChannelServiceListCreateObject = [];
							for (let kk = 0; kk < ChannelServiceListObject.length; kk++) {
								if (ChannelServiceListIds.includes(ChannelServiceListObject[kk].serviceListId)) {
									ChannelServiceListCurrentIds.push(ChannelServiceListObject[kk].serviceListId);
								} else {
									ChannelServiceListCreateObject.push(ChannelServiceListObject[kk]);
									ChannelServiceListCurrentIds.push(ChannelServiceListObject[kk].serviceListId);
								}
							}
							if (ChannelServiceListCreateObject.length > 0) {
								await this.channelServiceListRepository.bulkCreate(ChannelServiceListCreateObject, { returning: true, transaction });
							}

							await this.channelServiceListRepository.update({ deletedAt: new Date() }, {
								where: {
									channelId: user.channelId,
									serviceListId: { [Op.notIn]: ChannelServiceListCurrentIds },
									deletedAt: null
								}, returning: true, transaction
							});
						}

						// update key Retailers
						await this.tradeInformationInBound.keyRetailers(appconstant.CHANNEL_TYPE_RETAILER, params.tradeInformationDistributor.channelKeyRetailer, user.channelId, user.userId, params.tradeInformationDistributor.deletedkeyretailer);

						//updateing channel
						channelObject.profileTypeId = params.tradeInformationDistributor.profileTypeId;
					} else if (params.channelType == appconstant.CHANNEL_TYPE_RETAILER) {

						//ChannelDetail Object
						ChannelDetailObject = await this.tradeInformationInBound.setChannelDetail(params.tradeInformationRetailer, user.channelId, user.channelTypeId);

						//Speciality Object
						/*ChannelSpecialityObject = await this.tradeInformationInBound.setSpeciality(params.tradeInformationRetailer.speciality, user.channelId);*/
						channelObject.profileTypeId = params.tradeInformationRetailer.profileTypeId;
					}
					else if (params.channelType == appconstant.CHANNEL_TYPE_RESELLER) {

						//ChannelDetail Object
						ChannelDetailObject = await this.tradeInformationInBound.setChannelDetail(params.tradeInformationRetailer, user.channelId, user.channelTypeId);

						//Speciality Object
						/*ChannelSpecialityObject = await this.tradeInformationInBound.setSpeciality(params.tradeInformationRetailer.speciality, user.channelId);*/
						channelObject.profileTypeId = params.tradeInformationRetailer.profileTypeId;
					}

					if (ChannelSpecialityObject.length > 0) {

						let speciality = await this.channelSpecialityRepository.findAll({
							where: {
								channelId: user.channelId,
								deletedAt: null
							}
						});

						let ChannelSpecialityIds = await this.tradeInformationInBound.setProductGroupIds(speciality);
						let ChannelSpecialityCurrentIds = [];
						let ChannelSpecialityCreateObject = [];
						for (let kk = 0; kk < ChannelSpecialityObject.length; kk++) {
							if (ChannelSpecialityIds.includes(ChannelSpecialityObject[kk].productGroupId)) {
								ChannelSpecialityCurrentIds.push(ChannelSpecialityObject[kk].productGroupId);
							} else {
								ChannelSpecialityCreateObject.push(ChannelSpecialityObject[kk]);
								ChannelSpecialityCurrentIds.push(ChannelSpecialityObject[kk].productGroupId);
							}
						}
						if (ChannelSpecialityCreateObject.length > 0) {
							await this.channelSpecialityRepository.bulkCreate(ChannelSpecialityCreateObject, { returning: true, transaction });
						}

						await this.channelSpecialityRepository.update({ deletedAt: new Date() }, {
							where: {
								channelId: user.channelId,
								productGroupId: { [Op.notIn]: ChannelSpecialityCurrentIds },
								deletedAt: null
							}, returning: true, transaction
						});
					}

					if (ChannelDetailObject) {
						await this.channelDetailRepository.update(ChannelDetailObject, { where: { channelId: user.channelId }, returning: true, transaction });
					}

					// update channel information
					return await this.channelRepository.update(channelObject, { where: { channelId: user.channelId }, returning: true, transaction });
				});

				if (result) {
					return true;
				} else {
					return false;
				}
			} else {
				// throw new AuthException('channel:subscripion:userNotFound');
			}
		} catch (e) {
			console.log("Error+", e);
			// throw new InternalServerErrorException();
		}
	}

    async reportingusers(user: any): Promise<any> {
        let users: any = '';

        let query='select "contactId" as id,concat(ct."firstName",\' \',ct."lastName") as name from "tblContact" as ct where "deletedAt" is null and "channelId" =:channelId and'
        + ' "contactId" in(select "contactId" from "tblUser" where "userId" in(select "userId" from "tblUserRole" where "roleId" in(1,2))) ';

        users=await this.sequelizeInstance.query(query , {
            type: QueryTypes.SELECT,
            raw: true,
            replacements: {
                channelId: user.channelId,                
            }
            }).then(x=>{                            
                console.log("users:",x);
                return x;
            })

                          

       	return users;
	}


        //generalInfo for usermanage form
        async generalInfo(req: any) : Promise<any> {
            let users = '';
    
            let query = 'select (select array_to_json(array_agg(row_to_json(z))) from (select "contactSocialLink" from "tblContactSocialLink" where "deletedAt" is null and "socialSiteId" in(1,2) and "contactId"=:contactId)z) as sociallink,"rptContactId",(select "roleId" from "tblUserRole" where "userId" in(select "userId" from "tblUser" where "contactId"=:contactId)),(select "userId" from "tblUser" where "contactId"=:contactId) as"userId", "isShared","countryId","firstName","lastName","email","mobileNo","phone1","jobTitle","linkedIn","empId","hubspotContactId","documentId","isActive",(select "documentPath" from "tblDocument" where "documentId"=tc."documentId") from "tblContact" as tc' + 
             ' where "contactId"=:contactId ' ;
            users = await this.sequelizeInstance.query(query, {
                type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                    contactId: req.contactId,                
                }
                }).then(x=>{                            
                    //console.log("users:",x);
                    return Object.assign(x[0], {documentPath: x[0].documentPath? process.env.CDN_URL + x[0].documentPath:null});
                })

			query = 'select "suggestion_comm", "invitation_comm" from "tblUser" where "contactId"=:contactId';
			let pref = await this.sequelizeInstance.query(query, {
                type: QueryTypes.SELECT,
                raw: true,
                replacements: {
                    contactId: req.contactId,                
                }
                }).then(x=>{                            
                    //console.log("users:",x);
                    return x[0];
                });
    
			users = Object.assign(users, {preference: pref});

            return users;
    
        }

		public async searchRetailer(pagination, partner_type) {
			return await this.sequelizeInstance.query('select * from "fnfindkeyretailer"(:keyword, :channelType,\':limit\', \':offset\', :cdnUrl)', {
				type: QueryTypes.SELECT,
				replacements: {
					keyword: (pagination) ? pagination : '',
					channelType: partner_type,
					limit: 1,
					offset: 1,
					cdnUrl: process.env.CDN_URL
				}
			}).then((results) => {
				return results[0].fnfindkeyretailer;
			});
		}

//keypartners

public async getKeyRetailers(channelId, partnerType) {
	return await this.sequelizeInstance.query('select * from "fnGetKeyPartnersbyChannelId"(:channelId, :partnerType, :productId, :cdnUrl)', {
		type: QueryTypes.SELECT,
		replacements: {
			channelId: channelId,
			productId: null,
			cdnUrl: process.env.CDN_URL,
			partnerType: partnerType
		}
	}).then((results) => {
		return (results && results[0] && Array.isArray(results[0].fnGetKeyPartnersbyChannelId) && results[0].fnGetKeyPartnersbyChannelId.length > 0) ? results[0].fnGetKeyPartnersbyChannelId.map((x) => {
			if(x.active != null) {
				x.weight = 3
			} else if(x.old != null) {
				x.weight = 2
			} else {
				x.weight = 1
			}

			return x;
		}).sort((a, b) => (a.weight > b.weight) ? -1 : 1) : results[0].fnGetKeyPartnersbyChannelId;
	});
}

public async existingKeyRetailerCheck(KeyRetailer, CountryId, channelId, partner_type, partner_id) {
	let partnerData: any = null;
	if(partner_id) {
		partnerData = await this.getKeyPartner(partner_id);
	}
	

	let newQuery = 'select count(*) from "tblchannelkeypartner" where LOWER(partnername) = LOWER(:KeyRetailer) and "countryid" = :countryid and deletedat is null and "channelId" = :channelId and partner_type = :partnerType';
	if(partnerData && partnerData.channelkeypartner && partnerData.channelkeypartner.channelkeypartnerid) {
		newQuery += ' and channelkeypartnerid NOT IN (:channelkeypartnerid)';
	}

	let oldQuery = 'SELECT count(*) FROM "tblOldChannel" toc WHERE LOWER(toc."companyName") = LOWER(:KeyRetailer) AND toc."countryId" = :countryId AND toc."deletedAt" is null AND toc."isActive" = \'1\' AND toc."channelTypeId" = :partnerType AND NOT EXISTS(SELECT 1 FROM "tblChannel" WHERE "oldChannelId" = toc."oldChannelId" AND "deletedAt" is null AND "signUpStatusId" = 15 AND "isActive" = \'1\' AND "channelTypeId" = :partnerType )';
	if(partnerData && partnerData.oldchannel && partnerData.oldchannel.oldChannelId) {
		oldQuery += ' AND toc."oldChannelId" NOT IN (:oldChannelId)';
	}

	if (await this.sequelizeInstance.query(newQuery, {
		type: QueryTypes.SELECT,
		replacements: {
			channelId: channelId,
			KeyRetailer: KeyRetailer,
			countryid:CountryId,
			partnerType: partner_type,
			channelkeypartnerid: (partnerData && partnerData.channelkeypartner && partnerData.channelkeypartner.channelkeypartnerid) ? [+partnerData.channelkeypartner.channelkeypartnerid] : []
		}
	}).then(async (results) => {
		if (+results[0].count > 0) {
			return true
		} else {
			return await this.sequelizeInstance.query('SELECT count(*) FROM "tblChannel" WHERE LOWER("companyName") = LOWER(:KeyRetailer) AND "countryId" = :countryId AND "deletedAt" is null AND "signUpStatusId" = 15 AND "isActive" = \'1\' AND "channelTypeId" = :partnerType', {
				type: QueryTypes.SELECT,
				replacements: {
					KeyRetailer: KeyRetailer,
					countryId:CountryId,
					partnerType: partner_type
				}
			}).then(async (results) => {
				if (+results[0].count > 0) {
					return true
				} else {
					if (await this.sequelizeInstance.query(oldQuery, {
						type: QueryTypes.SELECT,
						replacements: {
							KeyRetailer: KeyRetailer,
							countryId:CountryId,
							partnerType: partner_type,
							oldChannelId: (partnerData && partnerData.oldchannel && partnerData.oldchannel.oldChannelId) ? [+partnerData.oldchannel.oldChannelId] : []
						}
					}).then(async (results) => {
						if (+results[0].count > 0) {
							return true
						} else {
							return false
						}
					})) {
						return true
					} else {
						return false
					}
				}
			});
		}
	})) {
		// throw new ValidationException([{
		// 	value: KeyRetailer.trim(),
		// 	property: 'KeyRetailer',
		// 	children: [],
		// 	constraints:
		// 		{ unique: 'validate:field:shouldBeUnique' }
		// }]);
		console.error('validation exception key retailer')
		return false;
	}

	return true;
}

public async getKeyPartner(partnerId) {
	let keyPartner = await KeyPartnerJCT.findOne({
		include: [
			{
				model: OldChannel,
				as: 'oldchannel',
				attributes: ['oldChannelId', 'companyName', 'channelTypeId', 'countryId', 'regAddressId'],
				include: [
					{
						model: OldChannelDetail,
						as: 'oldChannelDetail',
						attributes: ['OldChannelDetailId', 'channelLogoId'],
						include: [
							{
								model: Media,
								as: 'channelLogo',
								required: false,
								attributes: ['documentPath', 'documentUrl']
							}
						],
						required: true
					},
					{
						model: OldAddress,
						as: 'regAddress',
						required: false,
						attributes: ['oldAddressId', 'countryId'],
						include: [
							{
								model: Country,
								as: 'country',
								required: false,
								attributes: ['country', 'countryId']
							}
						]
					}
				],
				required: false
			},
			{
				model: ChannelKeyPartner,
				as: 'channelkeypartner',
				required: false,
				attributes: ['channelkeypartnerid', 'channelId', 'documentid', 'countryid', 'partnername'],
				include: [
					{
						model: Media,
						as: 'partnerImage',
						required: false,
						attributes: ['documentPath', 'documentUrl']
					},
					{
						model: Country,
						as: 'country',
						required: false,
						attributes: ['country', 'countryId']
					}
				]
			}
		],
		attributes: {
			exclude: ['createdat', 'updatedat', 'deletedat']
		},
		where: {
			keypartnerid: partnerId,
			existchannelid: null
		}
	});

	if (!keyPartner) {
		// throw new NotFoundException('NotFound');
		console.log('not found');
	}

	return keyPartner;
}

//payment

async getPaymentOption(): Promise<any> {
	let users: any = '';
	
	let query='select "paymentoptiontypeid","paymenttype",cast(false as boolean) as status from "tblpaymentoptiontype"';
	
	let paymentOption=await this.sequelizeInstance.query(query , {
	type: QueryTypes.SELECT,
	raw: true,
	});
	
	let paymentTerm=await this.sequelizeInstance.query('select "paymenttermid","paymentterm",cast(false as boolean) as status from "tblpaymentterm"' , {
	type: QueryTypes.SELECT,
	raw: true,
	});
	
	let payment={
	paymentOption:paymentOption,
	paymentTerm:paymentTerm
	}
	
	
	return payment;
	}

async getCurrency():Promise<any>{
	let currency = await this.currencyRepository.findAll<Currency>({
		attributes:['currencyid','currency','currencycode'],		
	})
	return currency;
}

//get service list
public async getServiceList(): Promise<Array<ServiceList>> {
	try {
		let result = await this.serviceListRepository.scope('serviceListInfom').findAll<ServiceList>();
		return result;
	} catch (e) {
		console.log("Error+ ", e);
		// throw new InternalServerErrorException();

	}
}

public async updateSettings(settingsDto: SettingsDto, user: any): Promise<any> {
	try {
		// if (settingsDto.paymentType.length > 0) {
		// 	await this.updateIndividualSetting(settingsDto.paymentType, 'paymentoptiontypeid', this.channelPaymentOptionTypeRepository, user);
		// }

		// if (settingsDto.paymentTerms.length > 0) {
		// 	await this.updateIndividualSetting(settingsDto.paymentTerms, 'paymenttermid', this.channelPaymentTermRepository, user);
		// }

		let channelDetail = await this.channelDetailRepository.findOne({ where: { channelId: user.channelId } });

		if (settingsDto.currencyId) {
			await channelDetail.update({ currencyid: settingsDto.currencyId });
		}

		if (settingsDto.shareStatus==true || settingsDto.shareStatus==false) {
			await channelDetail.update({ shareStatus: settingsDto.shareStatus });
		}

		if (settingsDto.otherOptions) {
			await channelDetail.update({ paymentoptionothers: settingsDto.otherOptions });
		}

		// if (settingsDto.userVisibility.length > 0) {
		// 	await this.updateIndividualSetting(settingsDto.userVisibility, 'uservisibilitytypeid', this.channelUserVisibilityRepository, user);
		// }

		if (settingsDto.share.length > 0) { // && isBoolean(settingsDto.shareStatus) && settingsDto.shareStatus) {
			//await this.channelSharePreferenceRepository.update({ deletedat: new Date().toUTCString() }, { where: { channelid: user.channelId, deletedat: null } });
			await this.updateIndividualSetting(settingsDto.share, 'sharetypeid', this.channelSharePreferenceRepository, user);
		}

		// if (isBoolean(settingsDto.shareStatus) && !settingsDto.shareStatus) {
		// 	await this.channelSharePreferenceRepository.update({ deletedat: new Date().toUTCString() }, { where: { channelid: user.channelId, deletedat: null } });
		// }

		// if (settingsDto.sampleRequests && settingsDto.sampleRequests.length > 0) {
		// 	await this.updateIndividualSetting(settingsDto.sampleRequests, 'preferencetypeid', this.channelSamplePreferenceTypeRepository, user);
		// }

		// if (isBoolean(settingsDto.autoRenewalOff)) {
		// 	await this.channelRepository.update({
		// 		autoRenewalOff: (settingsDto.autoRenewalOff === true) ? '1': '0'
		// 	}, {
		// 		where: {
		// 			channelId: user.channelId
		// 		}
		// 	});
		// }

		return true;

	} catch (e) {
		console.log("Internal Server error at ChannelSettingsService: " + e);		
		//throw new InternalServerErrorException();
	}
}

public async updateIndividualSetting(data, settingId, repository, user): Promise<any> {
	try {
		let setting: any = [];
		for (var i in data) {
			let settingObj = {
				channelid: user.channelId,
				updatedby: user.userId
			}
			settingObj[settingId] = data[i][settingId] - 1
			setting.push(settingObj);
		}
		if (setting.length > 0) {
			await repository.update({ deletedat: new Date().toUTCString() }, { where: { channelid: user.channelId, deletedat: null } });
			let data = await repository.bulkCreate(setting);

		}
	} catch (e) {
		console.log("Internal Server error at ChannelSettingsService: " + e);
		//throw new InternalServerErrorException();
	}
}

public async settings(user: any): Promise<any> {
	try {
		let settings: any = {};
		settings['paymentType'] = await this.channelPaymentOptionTypeRepository.scope('settings').findAll({ where: { channelid: user.channelId, deletedat: null } });
		settings['paymentTerms'] = await this.channelPaymentTermRepository.scope('settings').findAll({ where: { channelid: user.channelId, deletedat: null } });
		// settings['userVisibility'] = await this.channelUserVisibilityRepository.scope('settings').findAll({ where: { channelid: user.channelId, deletedat: null } });
		settings['share'] = await this.channelSharePreferenceRepository.scope('settings').findAll({ where: { channelid: user.channelId, deletedat: null } });
		//settings['sampleRequests'] = await this.channelSamplePreferenceTypeRepository.scope('settings').findAll({ where: { channelid: user.channelId, deletedat: null } });

		// settings['shareStatus'] = false;
		// if (settings['share'] && settings['share'].length > 0) {
		// 	settings['shareStatus'] = true;
		// }

		let channel = await this.channelRepository.findOne({ where: { channelId: user.channelId } });
		settings['autoRenewalOff'] = false;
		if(channel.autoRenewalOff == '1') {
			settings['autoRenewalOff'] = true;
		}

		let channelDetail = await this.channelDetailRepository.findOne({ where: { channelId: user.channelId } });
		if (channelDetail) {
			settings['currencyId'] = channelDetail.currencyid;
			settings['otherOptions'] = channelDetail.paymentoptionothers;
			settings['shareStatus'] = channelDetail.shareStatus;
		}

		//ChannelSubscriptionRepository
		//SubscriptionPlanRepository


		    let channelSubscription = await this.channelSubscriptionRepository.findOne({
			//attributes: ['endAt',  'subscriptionPlanType'],
            where: {
                isActive:'1',
				isPaid:'1',
				channelId: user.channelId
            },
            include: [
				{
					model:this.subscriptionRepository,
					include: [
						{
						  model:this.subscriptionPlanRepository
						}
					]	
				}],
				order:[['channelSubscriptionId' , 'desc']]
        });
		
		if (channelSubscription) {

			
			settings['channelSubscription'] = {};
			settings['channelSubscription']['endat'] = new Date(channelSubscription['endAt']).toLocaleString() //;
			settings['channelSubscription']['plan'] = channelSubscription['subscription']['subscriptionPlan']['subscriptionPlanType'];
			settings['channelSubscription']['planId'] = +channelSubscription['subscription']['subscriptionPlan']['subscriptionPlanId'];
			settings['channelSubscription']['channelSubscriptionId'] = +channelSubscription['channelSubscriptionId'];
		}

		let userWalletAmount =  await this.userWalletAmountRepository.findOne({
			where: {
				channelId: user.channelId
				//userId: user.userId
			}
		});
		console.log('userWalletAmount');
		console.log('=================');
		console.log(userWalletAmount);
	
		if(userWalletAmount){
			settings['userWalletAmount'] = userWalletAmount.amount;
		} else {
			settings['userWalletAmount'] = 0;
		}


		return settings;

	} catch (e) {
		console.log("Internal Server error at ChannelSettingsService: " + e);
		//throw new InternalServerErrorException();
	}
}	

async searchRegion(searchRegionDto: SearchRegionDto, user: any): Promise<any> {
	// let result: any;
	// let excluded: boolean = true;
	// if (isBoolean(searchRegionDto.exclude) && !searchRegionDto.exclude) {
	// 	excluded = false;
	// }

	// try {
	// 	let region: any = '';
	// 	if (excluded) {
	// 		let channelLocation = await this.channelLocationRepository.scope('regionCountryJCT').findAll<any>({
	// 			where: {
	// 				channelId: user.channelId,
	// 				deletedAt: null,
	// 				locationType: searchRegionDto.locationType
	// 			},
	// 			include: [{
	// 				model: RegionCountryJCT,
	// 			}]
	// 		});

	// 		let regionArr: any = [];
	// 		for (var i in channelLocation) {
	// 			regionArr.push(channelLocation[i].RegionCountryJCT.regionId);
	// 		}

	// 		if (regionArr && regionArr.length > 0) {
	// 			var uniqueRegion = regionArr.filter((item, i, ar) => ar.indexOf(item) === i);
	// 			region = uniqueRegion;
	// 		}
	// 	}

	// 	result = await this.zoneRepository.scope('noZones').findAll<any>({
	// 		include: [{
	// 			model: Region.scope('zoneRegionJCT'),
	// 			as: 'region',
	// 			where:
	// 			{
	// 				deletedAt: null,
	// 				channelId: { [Op.or]: [null, user.channelId] },
	// 				regionId: { [Op.notIn]: region },
	// 				region: { [Op.like]: '%' + searchRegionDto.region + '%' },
	// 			},
	// 			include: [{
	// 				model: Country.scope('regionCountryJCT'),
	// 				as: 'country',
	// 				where:
	// 				{
	// 					deletedAt: null
	// 				}
	// 			}]
	// 		}],
	// 		where:
	// 		{
	// 			deletedAt: null,
	// 			zoneId: { [Op.in]: searchRegionDto.zone }
	// 		}
	// 	});
	// } catch (e) {
	// 	console.log("Error+", e);
	// }
	// if (!result || result.length == 0) throw new ProcessException('common:searchRegion:noRegionFound');
	// return result;

	    let tradelocation: any = '';

		let query = 'select "regionId" from "tblZoneRegionJCT" where "zoneId" in (:zoneId)';
		let regionIds=[];
		let zoneId = await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT,
			replacements: {
				zoneId: searchRegionDto.zone
			}
		}).then(res => {
			if(res){
				res.map(x => {
					regionIds.push(x.regionId);
				})
			}
		});

        tradelocation=await this.regCtyJCTRepository.findAll<RegionCountryJCT>({
			attributes:['regionCountryJCTId'],
            include: [
                            {
                                model:this.regionRepository,                                
                                attributes: ['regionId','region'],                               
                            },
                            {
                                model:this.countryRepository,                                
                                attributes: ['countryId','country']
                            }
                    ],
					where:{
						regionId: {
							[Op.in] : regionIds
						}
					}
		})

		return tradelocation;

}

//Buyer Highlights
async buyerHighlights(viewId){
		
		let channeltype=[];
		let final=[];
		let aggregation={};let channelquery;
		if(viewId==1)
		channeltype=[3,4];
		else if(viewId==2)
		channeltype=[2];
		else if(viewId==3)
		channeltype=[3];
		else if(viewId==4)
		channeltype=[4];

		//let order='desc';
		//confirm		
		//console.log("viewId:",viewId,channeltype[0]);

		if(viewId == 1 || viewId == 2 || viewId == 3 || viewId == 4)
		{
			channelquery='select "channelId",(select "contactId" from "tblContact" where "isDefaultContact"=\'1\' and "channelId"=tc."channelId") from "tblChannel" as tc where tc."deletedAt" is null and "isActive"=\'1\' and "signUpStatusId"=15 and "channelTypeId" in (:type) order by tc."updatedAt" desc limit 9';
		
			let result= await this.sequelizeInstance.query(channelquery, {
			type: QueryTypes.SELECT,
			raw: true,
			replacements: {
			type:channeltype
			}
			}).then(x=>{	
				//console.log('x',x);					
				return  _.uniqBy(x,'channelId');
			})
	
			
			console.log('result', result)		
			if(result){		
				//  result=result;		
			
			 for(let i=0;i<result.length;i++)
			 {
				let companyquery;
	
				try{			
					companyquery='Select '
					+'chd."detailDesc" as detaildesc, chd."webSiteUrl" as websiteurl, chd."startup", '
	
					+'(select array_to_string(array_agg("productCategorie"), \',\') from "tblProductCategories" where "productCategorieId" in (select "productCategorieId" from "tblInterestedProductCategory" where "contactId"=:contactid) )as lookingfor, '
	
					+'(Select td."documentPath" as logo from "tblChannelDetail" chd inner join "tblDocument" td on td."documentId" in(chd."channelLogoId") where chd."channelId"=ch."channelId"), '
			
					+'(select array_to_json(array_agg(row_to_json(c))) from (Select td."documentPath",con."isDefaultContact",con."firstName",con."lastName",con."jobTitle" from "tblContact" con left join "tblDocument" td on td."documentId" in(con."documentId") where con."channelId"=ch."channelId" and con."isDefaultContact"=\'1\')c)as contacts, '				
					
					+'ch."companyName", tc."country", ch."channelId", ch."channelTypeId", pt."profileType", pt."profileTypeId" from "tblChannel" ch inner join "tblChannelDetail" chd on chd."channelId" = ch."channelId" inner join "tblCountry" tc on tc."countryId" = ch."countryId" inner join "tblProfileType" pt on pt."profileTypeId"=ch."profileTypeId" '
					
					+' where ch."channelId" in(:channelid)';
				 
			let res= await this.sequelizeInstance.query(companyquery, {
				type: QueryTypes.SELECT,
					raw: true,
					replacements: {
					channelid:+result[i].channelId,
					contactid:+result[i].contactId,
					}
					});
					final.push(...res);
	
				}
				catch(e)
				{
					console.log("error:",e);
					// console.log("status:",result[i])
		
				}
			}		
			
		}else{
			final=[];
		}
		
		}
		
	return final;
    }


//buyers aggr
	async buyersaggr(viewId, parent, data) {
		
	let channeltype=[];
	let final=[];
	let aggregation={};let channelquery;
	if(viewId==1)
	channeltype=[3,4];
	else if(viewId==2)
	channeltype=[2];
	else if(viewId==3)
	channeltype=[3];
	else if(viewId==4)
	channeltype=[4];

	//let order='desc';
	//confirm		
	//console.log("viewId:",viewId,channeltype[0]);

	if(viewId == 1 || viewId == 2 || viewId == 3 || viewId == 4)
	{
		channelquery = 'select "channelId",(select "contactId" from "tblContact" where "isDefaultContact"=\'1\' and "channelId"=tc."channelId") from "tblChannel" as tc where tc."deletedAt" is null and "isActive"=\'1\' and "isPublic"=\'1\' and "signUpStatusId"=15 and "channelTypeId" in (:type) ';

		if (parent == 'categories' && data.categoryId?.length)
			channelquery += ' and "channelId" in (select "channelId" from "tblChannelProductCategorie" where "productCategorieId" in   (:category)) '

		if (parent == 'brand' && data.brandId?.length)
			channelquery += ' and "channelId" in (select "channelId" from "tblChannelBrand" where "brandId" in   (:brand)) '

		if (parent == 'location' && data.locationId?.length)
			channelquery += 'and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(:type) and "countryId" in(:locationId)) '


		channelquery += ' order by tc."updatedAt"';



		let result= await this.sequelizeInstance.query(channelquery, {
		type: QueryTypes.SELECT,
		raw: true,
		replacements: {
		type:channeltype,
			category: data.categoryId,
			brand: data.brandId,
			locationId: data.locationId,
		}
		}).then(x=>{	
			//console.log('x',x);					
			return  _.uniqBy(x,'channelId');
		})
		
		//console.log('result', result)		
		if(result){		
			final = result.map(x => x.channelId);
		}		
		
	}
	
	return final;
}

//productsHighlights
public async productsHighlights(btype){	

	
	let query1 = 'select count(*) from "tblProduct" where "isactive"=\'1\' and "deletedat" is null';
	
	let count = await this.sequelizeInstance.query(query1, {
		type: QueryTypes.SELECT,
		raw: true,
		// replacements: {
		// 	channelId: channels
		// }
	}).then(res => res[0].count);
	let final = {result  : {}, total: count}
	return await this.sequelizeInstance.query('select * from "fnGetProductHighlight"(:btype)',
		{
		type: QueryTypes.SELECT,
		raw: true,
		replacements: {
			btype: btype
		}
	}).then((results) => {
		console.log(results);
		
		if(results){
			final = {result: results[0].fnGetProductHighlight, total: count}
		}
		return final;

	});

	

	// let type=[];
	// if(btype==1)
	// 	type=['1','3'];
	// if(btype==2)
	// 	type=['2','3'];
	// if(btype==3)
	// 	type=['1','2','3'];
 
	// let query = 'select "channelId" from "tblChannel" where "isActive"=\'1\' and "deletedAt" is null and "isOEM" in (:type) order by "updatedAt" desc';

	// let channels = await this.sequelizeInstance.query(query, {
	// 	type: QueryTypes.SELECT,
	// 	raw: true,
	// 	replacements: {
	// 	type:type
	// 	}
	// }).then(res => {
	// 	let temp = res.map(x => x.channelId);
	// 	return temp;
	// });

	// let query1 = 'select count(*) from "tblProduct" where "channelId" in (:channelId) and "isactive"=\'1\' and "deletedat" is null and "isFeatured"=\'1\' and "isPublic"=\'1\'';

	// let count = await this.sequelizeInstance.query(query1, {
	// 	type: QueryTypes.SELECT,
	// 	raw: true,
	// 	replacements: {
	// 		channelId: channels
	// 	}
	// }).then(res => res[0].count);

	
	// let res =await this.productRepository.findAll<Product>({
	// 	where: {
	// 		isactive: '1',
	// 		deletedat: null,
	// 		isPublic: '1',
	// 		channelId: {
	// 			[Op.in] : channels
	// 		},
	// 		isFeatured : '1',
	// 		show_in_home_page : '1'
	// 	},
	// 	order: [['updatedat', 'DESC']],
	// 	limit: 12,
	// 	include: [
	// 		{
	// 			model: Brand,
	// 			attributes: ['brandName'],
	// 			include: [
	// 				{
	// 					model: Media,
	// 				}
	// 			]
	// 		},
	// 		{
	// 			model: ProductPriceDetail,
	// 			attributes: ['retailerprice', 'retailerpriceunitid', 'businesscurrencyid']
	// 		},
	// 		{
	// 			model: ProductTag,
	// 			attributes: ['tag'],
	// 			where: {
	// 				deletedat: null
	// 			},
	// 			required: false
	// 		},
	// 		{
	// 			model: Media
	// 		},
	// 	]
	// });

	// let final = {result: [], total: 0}

	// if(res){
	// 	final = {result: res, total: count}
	// }
	// return final;
}

//product aggr
	async productsaggr(parent, data) {
	let query = 'select "productId" from "tblProduct" where "isactive"=\'1\' and "deletedat" is null ';

		if (parent == 'family' && data.familyId.length)
		query += ' and "productGroupId" in (:familyId) '
		if (parent == 'categories' && data.categoryId.length)
		query += ' and "productCategorieId" in (:categoryId) ' 
		if (parent == 'brand' && data.brandId.length)
			query += ' and "brandId" in (:brandId) '

	query += 'order by updatedat ';

	let result= await this.sequelizeInstance.query(query, {
		type: QueryTypes.SELECT,
		raw: true,
		replacements:{
			familyId: data.familyId,
			categoryId: data.categoryId,
			brandId: data.brandId,
		}
		}).then(x=>{				
			return  _.uniqBy(x,'productId');
		})

	if(result)
		result = result.map(x => x.productId);

	return result;

}

//get Buyers Highlight search list
async getBuyerSearchList(viewId, data){

	let channelFilter = data.filter;
	let total = 0;

	let channeltype=[];
		let final=[];
		let aggregation={};let channelquery;
		if(viewId==1)
		channeltype=[3,4];
		else if(viewId==2)
		channeltype=[2];
		else if(viewId==3)
		channeltype=[3];
		else if(viewId==4)
		channeltype=[4];

		let filterch = [];

		let order=data.order?data.order:' asc';

		let orderby = ' order by "'+ data.orderby +'" ';

		if(data.orderby == 'isFeatured')
			order = ' desc';

		
	//channelquery = 'select "channelId",(select "contactId" from "tblContact" where "isDefaultContact"=\'1\' and "channelId"=tc."channelId") from "tblChannel" as tc where tc."deletedAt" is null and "isActive"=\'1\' and "signUpStatusId"=15 and "channelTypeId" in (:type) ';

	channelquery = 'select ch."channelId",(select "contactId" from "tblContact" where "isDefaultContact"=\'1\' and "channelId"=ch."channelId"), ' +
		'ch."companyName", tc."country", ch."channelId" from "tblChannel" ch inner join "tblChannelDetail" chd on chd."channelId" = ch."channelId" inner join "tblCountry" tc on tc."countryId" = ch."countryId" ' +
		' where ch."deletedAt" is null and "isActive"=\'1\' and "signUpStatusId"=15 and "channelTypeId" in (:type) and "isPublic" = \'1\''; //and "isCompanyPublic" = \'1\'  order by tc."updatedAt"
		


		channelquery += orderby + order;

			let result= await this.sequelizeInstance.query(channelquery, {
			type: QueryTypes.SELECT,
			raw: true,
			replacements: {
			type:channeltype
			}
			}).then(x=>{				
				return  _.uniqBy(x,'channelId');
			})

			if(result)
				total = result.length;



			if((channelFilter.brandId||channelFilter.countryId||channelFilter.productCategoryId)&& result) {
				let chlist = result.map(x => +x.channelId);
			
			if(channelFilter.brandId) {
				//console.log("totals:", channelFilter.brandId);
				let brand='select array_agg("channelId") as ch from "tblChannelBrand" where "brandId" in(:brandid) and "channelId" in(:channelid)';
				await this.sequelizeInstance.query(brand, {
					type: QueryTypes.SELECT,
				raw: true,
				replacements: {
				channelid:chlist,
				brandid:channelFilter.brandId
				}
				}).then((result) => {
				//	console.log("chfilter:", result);
					if(result[0].ch){
						filterch = [];
				filterch.push(...result[0].ch);

				}else
				filterch = [];
		
				});
				}

			if(channelFilter.countryId) {
				await this.sequelizeInstance.query('select array_agg("channelId") as ch from "tblChannel" where "countryId" in(:countryid) and "channelId" in(:channelid)', {
					type: QueryTypes.SELECT,
				raw: true,
				replacements: {
				channelid:(filterch.length>0)?filterch:chlist,
				countryid:channelFilter.countryId
				}
				}).then((result) => {
					if(result[0].ch){
						filterch = [];
				filterch.push(...result[0].ch);
					}else
					filterch = [];
				});
				}
		
				if(channelFilter.productCategoryId) {
					await this.sequelizeInstance.query('select array_agg("channelId") as ch from "tblChannelProductCategorie" cpc where cpc."productCategorieId" in(:productcatid) and "channelId" in(:channelid) ', {
						type: QueryTypes.SELECT,
					raw: true,
					replacements: {
					channelid:(filterch.length>0)?filterch:chlist,
					productcatid:channelFilter.productCategoryId
					}
					}).then((result) => {
						if(result[0].ch){
							filterch = [];
					filterch.push(...result[0].ch);
						}else
						filterch = [];
			
					});
					}

				result = result.filter(ch => {return filterch.includes(ch.channelId);})
				total = result.length;
			}	

			if(result){		

				result=result.slice(data.offset).slice(0,data.limit);

				for(let i=0;i<result.length;i++)
				{
				   let companyquery;
	   
				   try{			
					   companyquery='Select '
					   +'chd."detailDesc" as detaildesc, chd."webSiteUrl" as websiteurl, chd."startup", '
	   
					   +'(select array_to_string(array_agg("productCategorie"), \',\') from "tblProductCategories" where "productCategorieId" in (select "productCategorieId" from "tblInterestedProductCategory" where "contactId"=:contactid) )as lookingfor, '
	   
					   +'(Select td."documentPath" as logo from "tblChannelDetail" chd inner join "tblDocument" td on td."documentId" in(chd."channelLogoId") where chd."channelId"=ch."channelId"), '
			   
					   +'(select array_to_json(array_agg(row_to_json(c))) from (Select td."documentPath",con."isDefaultContact",con."firstName",con."lastName",con."jobTitle" from "tblContact" con left join "tblDocument" td on td."documentId" in(con."documentId") where con."channelId"=ch."channelId" and con."isDefaultContact"=\'1\')c)as contacts, '				
					   
					   +'ch."companyName", tc."country", ch."channelId", ch."isFeatured" from "tblChannel" ch inner join "tblChannelDetail" chd on chd."channelId" = ch."channelId" inner join "tblCountry" tc on tc."countryId" = ch."countryId" '
					   
					   +' where ch."channelId" in(:channelid) ';
					
			   let res= await this.sequelizeInstance.query(companyquery, {
				   type: QueryTypes.SELECT,
					   raw: true,
					   replacements: {
					   channelid:+result[i].channelId,
					   contactid:+result[i].contactId,
					   }
					   });
					   final.push(...res);
	   
				   }
				   catch(e)
				   {
					   console.log("error:",e);
					   // console.log("status:",result[i])
		   
				   }
			   }	

			} 
			else{
				final = [];
			}

		let finaldata = {data: final, total: total};

		return finaldata;

}

//categories filter for guest
async getCategoriesFilter(data){

	let query = 'select "productCategorieId" from "tblChannelProductCategorie" where "channelId" in (:channelId)';


	let catId = await this.sequelizeInstance.query(query, {
		type: QueryTypes.SELECT,
		replacements: {
			channelId: data.data
		}
	}).then(res => {
		console.log('res', res);
		return _.uniqBy(res,'productCategorieId');
	})

	let result = [];

	if(catId){

		let cId = catId.map(x => x.productCategorieId);

		query = 'select "productCategorieId" as "id" , "productCategorie" as "name" from "tblProductCategories" where "isActive"=\'1\' and "deletedAt" is null and "productCategorieId" in (:pId)';

		 result = await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT,
			replacements: {
				pId: cId	
			}
		}).then(res => {
			console.log('res', res);
			return res;
		})
	}
	

	return result;

}

//brands filter for guest
async getBrandFilter(data){
	let query = 'select "brandId" from "tblChannelBrand" where "channelId" in (:channelId)';


	let brandIds = await this.sequelizeInstance.query(query, {
		type: QueryTypes.SELECT,
		replacements: {
			channelId: data.data
		}
	}).then(res => {
		return _.uniqBy(res, 'brandId');
	})

	let result = [];

	if(brandIds){
		query = 'select "id", "brandName"  as "name" from "tblBrand" where "isActive"=\'1\' and "isAssigned"=\'1\' and "deletedAt" is null and "id" in (:brandIds)'

		let brandId = brandIds.map(x => x.brandId);

		 result = await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT,
			replacements: {
				brandIds: brandId
			}
		}).then(res => {
			return res;
		})
	
	}

	return result;
}

	//country filter for guest
	async getCountryFilter(data) {
		let query = 'select "countryId" from "tblChannel" as ch where "channelId" in (:channelId)';


		let countryIds = await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT,
			replacements: {
				channelId: data.data
			}
		}).then(res => {
			return _.uniqBy(res, 'countryId');
		})

		let result = [];

		if (countryIds) {
			query = 'select "countryId" as id, "country"  as "name" from "tblCountry" where  "countryId" in (:countryIds)'

			let countryId = countryIds.map(x => x.countryId);

			result = await this.sequelizeInstance.query(query, {
				type: QueryTypes.SELECT,
				replacements: {
					countryIds: countryId
				}
			}).then(res => {
				return res;
			})

		}

		return result;
	}

		//update company profile old
		public async editBasicCompany(requestObject: any, user: any): Promise<boolean> {
			let companyProfile: any = '';
			let companyProfileJson: any = '';
			let channelStatusArray: Array<number> = [appconstant.CHANNEL_APPROVED];
	
			//validate company name
			let realUser = user;
			console.log('log', user);
			// if (realUser && realUser.channel && +realUser.channel.channelTypeId === +appconstant.CHANNEL_TYPE_HUB) {
				// before update verify if the company name already exist

				let result = await this.channelRepository.findOne({
					where: {
						companyName: requestObject.company_name.trim(),
						channelId: {
							[Op.not]: +user.channelId
						}
					}
				});
				if (result) {
					throw new ProcessException('channel:profile:companyNameAlreadyExist');
				}
				// allow edit
				channelStatusArray.push(+user.channel.signUpStatusId);
			// }
			// return true;
			try {
				let result: any = '';
				console.log('user.channel.signUpStatusId', user.channel.signUpStatusId);
				let userSignUpStatus = channelStatusArray.includes(+user.channel.signUpStatusId);
				if (userSignUpStatus) {
					companyProfile = await this.userRepository.scope('userBasicInfom').findOne({
						include: [
							{
								model: this.contactRepository.scope('contactInfo'),
								where: { deletedAt: null },
								include: [
									{
										model: this.mediaRepository,
										include: [
											{
												model: this.mediaTypeRepository,
												required: false
											},
										],
										required: false
									}
								]
							},
							{
								model: this.channelRepository.scope('profileInfom'),
								where: { signUpStatusId: parseInt(user.channel.signUpStatusId), deletedAt: null },
								include: [
									{
										model: this.channelDetailRepository.scope('channelDetail'),
										where: { deletedAt: null },
										include: [
											{
												model: this.mediaRepository,
												as: 'channelLogo',
												include: [
													{
														model: this.mediaTypeRepository,
														required: false
													},
												],
												required: false
											},
											{
												model: this.mediaRepository,
												as: 'channelIcon',
												include: [
													{
														model: this.mediaTypeRepository,
														required: false
													},
												],
												required: false
											}
										]
									},
									{
										model: this.channelCertiRepository.scope('profileInfo'),
										where: { deletedAt: null },
										required: false
									},
									{ 
										model: this.channelDocumentRepository,
										where: { deletedAt: null },
										required: false,
										include: [
											{
												model: this.mediaRepository,
												as: 'document',
												required: false
											}
										]
									}
								]
							}
						],
						where: { userId: parseInt(user.userId), deletedAt: null }
					});
				}
	console.log('companyProfile', companyProfile);
				if (companyProfile) {
					companyProfileJson = companyProfile.toJSON();
					result = await this.sequelizeInstance.transaction(async transaction => {
						let comapnyLogoId: any = null;
						let comapnyRegAddressId: any = null;
	
						// update company registration address to address table
						if (requestObject.reg_address_id) {
							await this.addressRepository.update({
								address: requestObject.reg_address,
								city: requestObject.reg_city,
								countryId: requestObject.reg_country_id,
								postalCode: requestObject.reg_postal_code,
							},
								{
									where: { addressId: companyProfile.channel.regAddressId },
									returning: true, transaction,
								});
							comapnyRegAddressId = companyProfile.channel.regAddressId;
						}
	
						// update channel information's address to channel table
						let companyInfoUpdate: any = {
							companyMail: requestObject.company_email || null,
							regAddressId: comapnyRegAddressId,
							countryId: +requestObject.reg_country_id,
							companyName : requestObject.company_name.trim()
						};
	
						await this.channelRepository.update(companyInfoUpdate,
							{
								where: { channelId: companyProfile.channel.channelId }, returning: true, transaction,
							});
	
						if (requestObject.social_linkedin_id) {
							// update channel social linkedin profile details to channel social link table
							await this.socialLinkRepository.update({
								socialSiteId: appconstant.SOCIAL_SITE_TYPE_LINKEDIN,
								channelId: companyProfile.channelId,
								channelSocialLink: requestObject.social_linkedin_url
							},
								{
									where: { channelSocialLinkId: requestObject.social_linkedin_id },
									returning: true, transaction,
								});
						} else {
							// save channel social linkedin profile details to channel social link table
							if (requestObject.social_linkedin_url && Object.keys(requestObject.social_linkedin_url).length > 0) {
								await this.socialLinkRepository.create({
									socialSiteId: appconstant.SOCIAL_SITE_TYPE_LINKEDIN,
									channelId: companyProfile.channelId,
									channelSocialLink: requestObject.social_linkedin_url
								},
									{
										transaction
									});
							}
						}
	
						// save channel social google plus profile details to channel social link table
						if (requestObject.social_googleplus_id) {
							// update channel social google plus profile details to channel social link table
							await this.socialLinkRepository.update({
								socialSiteId: appconstant.SOCIAL_SITE_TYPE_GOOGLEPLUS,
								channelId: companyProfile.channelId,
								channelSocialLink: requestObject.social_googleplus_url
							},
								{
									where: { channelSocialLinkId: requestObject.social_googleplus_id },
									returning: true, transaction,
								});
						} else {
							// save channel social google plus profile details to channel social link table
							if (requestObject.social_googleplus_url && Object.keys(requestObject.social_googleplus_url).length > 0) {
								await this.socialLinkRepository.create({
									socialSiteId: appconstant.SOCIAL_SITE_TYPE_GOOGLEPLUS,
									channelId: companyProfile.channelId,
									channelSocialLink: requestObject.social_googleplus_url
								},
									{
										 transaction
									});
							}
						}
	
						if (requestObject.social_instagram_id) {
							// update channel social instagram profile details to channel social link table
							await this.socialLinkRepository.update({
								channelId: companyProfile.channelId,
								channelSocialLink: requestObject.social_instagram_url
							},
								{
									where: { channelSocialLinkId: requestObject.social_instagram_id },
									returning: true, transaction,
								});
						} else {
							// save channel social instagram profile details to channel social link table
							if (requestObject.social_instagram_url && Object.keys(requestObject.social_instagram_url).length > 0) {
								await this.socialLinkRepository.create({
									socialSiteId: appconstant.SOCIAL_SITE_TYPE_INSTAGRAM,
									channelId: companyProfile.channelId,
									channelSocialLink: requestObject.social_instagram_url
								},
									{
										transaction
									});
							}
						}

						if (requestObject.social_twitter_id) {
							// update channel social twitter profile details to channel social link table
							await this.socialLinkRepository.update({
								channelId: companyProfile.channelId,
								channelSocialLink: requestObject.social_twitter_url
							},
								{
									where: { channelSocialLinkId: requestObject.social_twitter_id },
									returning: true, transaction,
								});
						} else {
							// save channel social twitter profile details to channel social link table
							if (requestObject.social_twitter_url && Object.keys(requestObject.social_twitter_url).length > 0) {
								await this.socialLinkRepository.create({
									socialSiteId: appconstant.SOCIAL_SITE_TYPE_TWITTER,
									channelId: companyProfile.channelId,
									channelSocialLink: requestObject.social_twitter_url
								},
									{
										transaction
									});
							}
						}

						if (requestObject.social_facebook_id) {
							// update channel social facebook profile details to channel social link table
							await this.socialLinkRepository.update({
								channelId: companyProfile.channelId,
								channelSocialLink: requestObject.social_facebook_url
							},
								{
									where: { channelSocialLinkId: requestObject.social_facebook_id },
									returning: true, transaction,
								});
						} else {
							// save channel social facebook profile details to channel social link table
							if (requestObject.social_facebook_url && Object.keys(requestObject.social_facebook_url).length > 0) {
								await this.socialLinkRepository.create({
									socialSiteId: appconstant.SOCIAL_SITE_TYPE_FACEBOOK,
									channelId: companyProfile.channelId,
									channelSocialLink: requestObject.social_facebook_url
								},
									{
										transaction
									});
							}
						}
						


	
						if (requestObject.B2B_url && Object.keys(requestObject.B2B_url).length > 0) {
	
							for (let ii = 0; ii < requestObject.B2B_url.length; ii++) {
								if (requestObject.B2B_url[ii].channel_B2B_id) {
									// update channel B2B details to channel B2B table
	
									await this.channelB2BRepository.update({
										channelId: companyProfile.channelId,
										channelB2B: requestObject.B2B_url[ii].channelB2B
									},
										{
											where: { channelB2BId: requestObject.B2B_url[ii].channel_B2B_id },
											returning: true, transaction,
										});
								} else {
									// save channel B2B details to channel B2B table
	
									await this.channelB2BRepository.create({
										channelId: companyProfile.channelId,
										channelB2B: requestObject.B2B_url[ii].channelB2B
									},
										{
											 transaction
										});
								}
							}
						}
						if (requestObject.deleted_B2B && requestObject.deleted_B2B.length > 0) {
							for (let ii = 0; ii < requestObject.deleted_B2B.length; ii++) {
								if (requestObject.deleted_B2B[ii]) {
									// update channel B2B details to channel B2B table
									await this.channelB2BRepository.update({
										channelId: companyProfile.channelId,
										deletedAt: new Date()
									},
										{
											where: { channelB2BId: requestObject.deleted_B2B[ii] },
											returning: true, transaction,
										});
								}
							}
						}
						
						if (companyProfileJson.channel.channelDetail.channelLogoId) {
							comapnyLogoId = companyProfileJson.channel.channelDetail.channelLogoId;
						}
						// save comapny logo detail to document table
						if (requestObject.company_logo_url) {
							comapnyLogoId = requestObject.company_logo_url;
						}	
	
						// update channel detals address to channel detail table
						return await this.channelDetailRepository.update({
							channelLogoId: comapnyLogoId,
							phone1: requestObject.company_phone1,
							phone2: (isUndefined(requestObject.company_phone2))? null : requestObject.company_phone2,
							mobileNo: (isUndefined(requestObject.company_mobile_no))? null : requestObject.company_mobile_no,
							webSiteUrl: requestObject.company_website_url,
							introvideoId: requestObject.company_introvideo,
							detailDesc: requestObject.company_about,//sanitizeHtml(requestObject.company_about, appconstant.SANITIZE_HTML),
							companyDesc: requestObject.company_full,
						},
							{
								where: { channelId: companyProfile.channelId }, returning: true, transaction,
							});
	
					});
					if (result) {
						return true;
					} else {
						return false;
					}
				}
			} catch (e) {
				console.log("Error+", e);
				throw new InternalServerErrorException();
				
			}
			if (!companyProfile) {
				throw new ProcessException('channel:getCompanyProfile:companyNotFound');
			}
		}

		public async updateBannerImage(requestObject: any, user: any): Promise<boolean> {
			let bannerId: any = null;
				let companyBanner;
					let oldBanner = await this.channelDetailRepository.findOne({
						where: {channelId: user.channelId},
						attributes: ['channelBannerId']
					})
					return await this.sequelizeInstance.transaction(async transaction => {
						console.log('oldBanner',oldBanner);
					if(oldBanner){
						await this.mediaRepository.update({deletedAt: new Date()}, {where: {
							documentId: +oldBanner.channelBannerId
						},returning: true, transaction});
					}

					companyBanner = await this.mediaRepository.create({
						documentPath: requestObject.bannerUrl,
						documentTypeId: appconstant.DOCUMENTTYPE_IMAGE,
						dirName: appconstant.FOLDER_PATH_COMPANY_BANNER
					},
						{
							 transaction,
						});
						bannerId = companyBanner.documentId;
						await this.channelDetailRepository.update({channelBannerId:bannerId}, {
							where: {channelId: user.channelId},returning: true, transaction
						});
						return companyBanner;
					});
		}

		public async editProfileSave( user: any, params: any): Promise<boolean>{
			try {
				let ChannelSpecialityObject = <Array<ChannelSpeciality>>[];
				let userDetails = await this.userRepository.scope('userBasicInfom').findOne({
					where: { userId: parseInt(user.userId), isActive: '1' }
				});
	
				if (userDetails) {
					let result: any = await this.sequelizeInstance.transaction(async transaction => {
	
						// let contact = await this.contactRepository.scope('basicInfom').findOne({
						// 	where: {
						// 		contactId: user.contactId,
						// 		channelId: user.channelId
						// 	},
						// 	include: [
						// 		{
						// 			model: ProductGroup, as: 'productGroups', required: false, where: {
						// 				deletedAt: null
						// 			}, through: { attributes: [], where: { deletedAt: null } }, attributes: ['productGroupId']
						// 		},
						// 		{
						// 			model: ProductCategory, as: 'productCategories', required: false, where: {
						// 				deletedAt: null
						// 			}, through: { attributes: [], where: { deletedAt: null } }, attributes: ['productCategorieId']
						// 		}
						// 	]
						// })
	
						let channelContacts = await this.contactRepository.scope('basicInfom').findAll({
							where: {
								deletedAt: null,
								contactId: { [Op.ne]: user.contactId },
								channelId: user.channelId
							}
						});
	
						let contacts = [];
						contacts.push(user.contactId);
						if (channelContacts.length > 0) {
							channelContacts.map(function (data) {
								contacts.push(data.contactId);
							});
						}
	
						let ChannelDetailObject = <ChannelDetail>{};
	
						if (params.productFamilies?.length > 0) {
	
							let productGroup = await this.channelProductGroupRepository.findAll({
								where: {
									channelId: user.channelId,
									deletedAt: null
								}
							});
	
							let ChannelProductGroupIds = await this.tradeInformationInBound.setProductGroupIds(productGroup);
							let ChannelProductGroupObject: ChannelProductGroup[] = await this.tradeInformationInBound.setProductGroup(params.productFamilies, user.channelId);
							let ChannelProductGroupCurrentIds = [];
							let ChannelProductGroupCreateObject = [];
	
							/* Account User contact update */
							// let ContactProductGroupIds = await this.tradeInformationInBound.setProductGroupIds(contact.productGroups);
							let ContactProductGroupObject: ContactProductGroup[] = await this.tradeInformationInBound.setContactProductGroup(params.productFamilies, user.contactId);
							let ContactProductGroupCurrentIds = [];
							// let ContactProductGroupCreateObject = [];
	
							for (let kk = 0; kk < ChannelProductGroupObject.length; kk++) {
								if (ChannelProductGroupIds.includes(ChannelProductGroupObject[kk].productGroupId)) {
									ChannelProductGroupCurrentIds.push(ChannelProductGroupObject[kk].productGroupId);
								} else {
									ChannelProductGroupCreateObject.push(ChannelProductGroupObject[kk]);
									ChannelProductGroupCurrentIds.push(ChannelProductGroupObject[kk].productGroupId);
								}
								// if (ContactProductGroupIds.includes(ContactProductGroupObject[kk].productGroupId)) {
								// 	ContactProductGroupCurrentIds.push(ContactProductGroupObject[kk].productGroupId);
								// } else {
								// 	// ContactProductGroupCreateObject.push(ContactProductGroupObject[kk]);
								// 	ContactProductGroupCurrentIds.push(ContactProductGroupObject[kk].productGroupId);
								// }
	
								ContactProductGroupCurrentIds.push(ContactProductGroupObject[kk].productGroupId);
							}
							if (ChannelProductGroupCreateObject.length > 0) {
								await this.channelProductGroupRepository.bulkCreate(ChannelProductGroupCreateObject, { returning: true, transaction });
							}
	
							// if (ContactProductGroupCreateObject.length > 0) {
							// 	await this.contactProductGroupRepository.bulkCreate(ContactProductGroupCreateObject, { returning: true, transaction });
							// }
	
							await this.channelProductGroupRepository.update({ deletedAt: new Date() }, {
								where: {
									channelId: user.channelId,
									productGroupId: { [Op.notIn]: ChannelProductGroupCurrentIds },
									deletedAt: null
								}, returning: true, transaction
							});
	
							await this.contactGroupRepository.update({ deletedAt: new Date() }, {
								where: {
									contactId: { [Op.in]: contacts },
									productGroupId: { [Op.notIn]: ContactProductGroupCurrentIds },
									deletedAt: null
								}, returning: true, transaction
							});
						}
	
						if (params.productCategories?.length > 0) {
	
							let productCategory = await this.channelProductCategorieRepository.findAll({
								where: {
									channelId: user.channelId,
									deletedAt: null
								}
							});
	
							let productCategoriesIds = await this.tradeInformationInBound.setProductCategoryIds(productCategory);
							let ProductCategorieObject: ChannelProductCategorie[] = await this.tradeInformationInBound.setProductCategories(params.productCategories, user.channelId);
							let ChannelProductCategoriesCurrentIds = [];
							let ChannelProductCategoriesCreateObject = [];
	
							/* Account User contact update */
							// let contactProductCategoriesIds = await this.tradeInformationInBound.setProductCategoryIds(contact.productCategories);
							let contactProductCategorieObject: ContactProductCategory[] = await this.tradeInformationInBound.setContactProductCategories(params.productCategories, user.contactId);
							let ContactProductCategoriesCurrentIds = [];
							// let ContactProductCategoriesCreateObject = [];
	
							for (let kk = 0; kk < ProductCategorieObject.length; kk++) {
								if (productCategoriesIds.includes(ProductCategorieObject[kk].productCategorieId)) {
									ChannelProductCategoriesCurrentIds.push(ProductCategorieObject[kk].productCategorieId);
								} else {
									ChannelProductCategoriesCreateObject.push(ProductCategorieObject[kk]);
									ChannelProductCategoriesCurrentIds.push(ProductCategorieObject[kk].productCategorieId);
								}
								// if (contactProductCategoriesIds.includes(contactProductCategorieObject[kk].productCategorieId)) {
								// 	ContactProductCategoriesCurrentIds.push(contactProductCategorieObject[kk].productCategorieId);
								// } else {
								// 	// ContactProductCategoriesCreateObject.push(contactProductCategorieObject[kk]);
								// 	ContactProductCategoriesCurrentIds.push(contactProductCategorieObject[kk].productCategorieId);
								// }
	
								ContactProductCategoriesCurrentIds.push(contactProductCategorieObject[kk].productCategorieId);
							}
							if (ChannelProductCategoriesCreateObject.length > 0) {
								await this.channelProductCategorieRepository.bulkCreate(ChannelProductCategoriesCreateObject, { returning: true, transaction });
							}
	
							// if (ContactProductCategoriesCreateObject.length > 0) {
							// 	await this.contactProductCategoryRepository.bulkCreate(ContactProductCategoriesCreateObject, { returning: true, transaction });
							// }
	
							await this.channelProductCategorieRepository.update({ deletedAt: new Date() }, {
								where: {
									channelId: user.channelId,
									productCategorieId: { [Op.notIn]: ChannelProductCategoriesCurrentIds },
									deletedAt: null
								}, returning: true, transaction
							});
	
							await this.contactCatRepository.update({ deletedAt: new Date() }, {
								where: {
									contactId: { [Op.in]: contacts },
									productCategorieId: { [Op.notIn]: ContactProductCategoriesCurrentIds },
									deletedAt: null
								}, returning: true, transaction
							});
						}
	
						if (params.customerProfiles && params.customerProfiles?.length > 0) {
	
							let customerProfiles = await this.channelCustomerProfileRepository.findAll({
								where: {
									channelId: user.channelId,
									deletedAt: null
								}
							});
	
							let customerProfilesIds = await this.tradeInformationInBound.setCustomerProfilesIds(customerProfiles);
							let customerProfilesObject: ChannelCustomerProfile[] = await this.tradeInformationInBound.setCustomerProfiles(params.customerProfiles, user.channelId);
							let channelCustomerProfilesCurrentIds = [];
							let ChannelCustomerProfilesCreateObject = [];
							for (let kk = 0; kk < customerProfilesObject.length; kk++) {
								if (customerProfilesIds.includes(customerProfilesObject[kk].profileTypeId)) {
									channelCustomerProfilesCurrentIds.push(customerProfilesObject[kk].profileTypeId);
								} else {
									ChannelCustomerProfilesCreateObject.push(customerProfilesObject[kk]);
									channelCustomerProfilesCurrentIds.push(customerProfilesObject[kk].profileTypeId);
								}
							}
							if (ChannelCustomerProfilesCreateObject.length > 0) {
								await this.channelCustomerProfileRepository.bulkCreate(ChannelCustomerProfilesCreateObject, { returning: true, transaction });
							}
	
							await this.channelCustomerProfileRepository.update({ deletedAt: new Date() }, {
								where: {
									channelId: user.channelId,
									profileTypeId: { [Op.notIn]: channelCustomerProfilesCurrentIds },
									deletedAt: null
								}, returning: true, transaction
							});
						}
	
						let channelObject = <Channel>{
							isFrenchTech: params.is_french_tech
						};
						if (params.channelType == appconstant.CHANNEL_TYPE_VENDOR) {
	
							//saving brand
							if (params.tradeInformationVendor.brand?.length > 0) {
	
								let channelBrandObject = await this.saveBrand(params.tradeInformationVendor.brand, user.channelId, transaction);
	
								if (channelBrandObject.channelBrandArrayObject.length > 0) {
	
									let brand = await this.channelBrandRepository.findAll({
										where: {
											channelId: user.channelId,
											deletedAt: null
										}
									});
	
									let ChannelBrandCurrentIds = [];
									let ChannelBrandCreateObject = [];
									let brandObject = channelBrandObject.channelBrandArrayObject;
									let brandIds = channelBrandObject.brandIds;
	
									let ChannelBrandIds = await this.tradeInformationInBound.setchannelbrandIds(brand);
									let totalRemovedBrands = ChannelBrandIds.filter(e => !brandIds.find(a => e == a));
	
									for (let kk = 0; kk < brandObject.length; kk++) {
										if (!ChannelBrandIds.includes(brandObject[kk].brandId)) {
											ChannelBrandCreateObject.push(brandObject[kk]);
										}
	
										if(!totalRemovedBrands.includes(+brandObject[kk].brandId)) { 
											ChannelBrandCurrentIds.push(brandObject[kk].brandId)
										}
									}
	
									if (ChannelBrandCreateObject.length > 0) {
										await this.channelBrandRepository.bulkCreate(ChannelBrandCreateObject, { returning: true, transaction });
									}
	
									await this.channelBrandRepository.update({ deletedAt: new Date() }, {
										where: {
											channelId: user.channelId,
											brandId: { [Op.notIn]: ChannelBrandCurrentIds },
											deletedAt: null
										}, returning: true, transaction
									});
	
									await this.brandRepository.update({
										isAssigned: '1'
									}, {
											where: {
												id: {
													[Op.in]: brandIds
												}
											},
											returning: true, transaction
										}
									);
	
									await this.brandRepository.update({
										isAssigned: '0'
									}, {
											where: {
												id: {
													[Op.in]: totalRemovedBrands
												}
											},
											returning: true, transaction
										}
									);
								}
							}
	
							if (params.tradeInformationVendor && params.tradeInformationVendor.profileTypeId) {
								//updateing channel
								channelObject.profileTypeId = params.tradeInformationVendor.profileTypeId;
							}
	
							// update key Retailers
							await this.tradeInformationInBound.keyRetailers(appconstant.CHANNEL_TYPE_RETAILER, params.tradeInformationVendor.channelKeyRetailer, user.channelId, user.userId, params.tradeInformationVendor.deletedkeyretailer);
	
							// create key Distributors
							await this.tradeInformationInBound.keyRetailers(appconstant.CHANNEL_TYPE_DISTRIBUTOR, params.tradeInformationVendor.channelKeyDistributor, user.channelId, user.userId, params.tradeInformationVendor.deletedkeydistributor);
	
							ChannelDetailObject = await this.tradeInformationInBound.setChannelDetail(params.tradeInformationVendor, user.channelId, user.channelTypeId);
	
						} else if (params.channelType == appconstant.CHANNEL_TYPE_DISTRIBUTOR) {
							//saving brand
							if (params.tradeInformationDistributor && params.tradeInformationDistributor?.brand && params.tradeInformationDistributor.brand.length > 0) {
								let channelBrandObject = await this.saveBrand(params.tradeInformationDistributor.brand, user.channelId, transaction);
	
								if (channelBrandObject.channelBrandArrayObject.length > 0) {
									let brand = await this.channelBrandRepository.findAll({
										where: {
											channelId: user.channelId,
											deletedAt: null
										}
									});
	
									let ChannelBrandCurrentIds = [];
									let ChannelBrandCreateObject = [];
									let brandObject = channelBrandObject.channelBrandArrayObject;
	
									let ChannelBrandIds = await this.tradeInformationInBound.setchannelbrandIds(brand);
									let totalRemovedBrands = ChannelBrandIds.filter(e => !brandObject.find(a => e == a.brandId));
	
									for (let kk = 0; kk < brandObject.length; kk++) {
										if (!ChannelBrandIds.includes(brandObject[kk].brandId)) {
											ChannelBrandCreateObject.push(brandObject[kk]);
										}
	
										if(!totalRemovedBrands.includes(+brandObject[kk].brandId)) {
											ChannelBrandCurrentIds.push(brandObject[kk].brandId);
										}
									}
	
									if (ChannelBrandCreateObject.length > 0) {
										await this.channelBrandRepository.bulkCreate(ChannelBrandCreateObject, { returning: true, transaction });
									}
	
									await this.channelBrandRepository.update({ deletedAt: new Date() }, {
										where: {
											channelId: user.channelId,
											brandId: { [Op.notIn]: ChannelBrandCurrentIds },
											deletedAt: null
										}, returning: true, transaction
									});
	
									console.log(JSON.stringify(ChannelBrandCurrentIds));
	
									await this.brandRepository.update({
										isAssigned: '1'
									}, {
											where: {
												id: {
													[Op.in]: channelBrandObject.brandIds
												}
											},
											returning: true, transaction
										}
									);
	
									await this.brandRepository.update({
										isAssigned: '0'
									}, {
											where: {
												id: {
													[Op.in]: totalRemovedBrands
												}
											},
											returning: true, transaction
										}
									);
	
									console.log(JSON.stringify(totalRemovedBrands));
								}
							}
	
							//Speciality Object
							ChannelSpecialityObject = await this.tradeInformationInBound.setSpeciality(params.tradeInformationDistributor.distributorSpeciality, user.channelId);
	
							//updateing channel
							channelObject.profileTypeId = params.tradeInformationDistributor.profileTypeId;
						} else if (params.channelType == appconstant.CHANNEL_TYPE_RETAILER || params.channelType == appconstant.CHANNEL_TYPE_RESELLER) {
	
							//Speciality Object
							/*ChannelSpecialityObject = await this.tradeInformationInBound.setSpeciality(params.tradeInformationRetailer.speciality, user.channelId);*/
							channelObject.profileTypeId = params.tradeInformationRetailer.profileTypeId;
						}
						else if (params.channelType == appconstant.CHANNEL_TYPE_RESELLER) {
	
						//Speciality Object
						/*ChannelSpecialityObject = await this.tradeInformationInBound.setSpeciality(params.tradeInformationRetailer.speciality, user.channelId);*/
						channelObject.profileTypeId = params.tradeInformationReseller.profileTypeId;
						}
	
						if (ChannelSpecialityObject.length > 0) {
	
							let speciality = await this.channelSpecialityRepository.findAll({
								where: {
									channelId: user.channelId,
									deletedAt: null
								}
							});
	
							let ChannelSpecialityIds = await this.tradeInformationInBound.setProductGroupIds(speciality);
							let ChannelSpecialityCurrentIds = [];
							let ChannelSpecialityCreateObject = [];
							for (let kk = 0; kk < ChannelSpecialityObject.length; kk++) {
								if (ChannelSpecialityIds.includes(ChannelSpecialityObject[kk].productGroupId)) {
									ChannelSpecialityCurrentIds.push(ChannelSpecialityObject[kk].productGroupId);
								} else {
									ChannelSpecialityCreateObject.push(ChannelSpecialityObject[kk]);
									ChannelSpecialityCurrentIds.push(ChannelSpecialityObject[kk].productGroupId);
								}
							}
							if (ChannelSpecialityCreateObject.length > 0) {
								await this.channelSpecialityRepository.bulkCreate(ChannelSpecialityCreateObject, { returning: true, transaction });
							}
	
							await this.channelSpecialityRepository.update({ deletedAt: new Date() }, {
								where: {
									channelId: user.channelId,
									productGroupId: { [Op.notIn]: ChannelSpecialityCurrentIds },
									deletedAt: null
								}, returning: true, transaction
							});
						}
	
						if (ChannelDetailObject) {
							await this.channelDetailRepository.update(ChannelDetailObject, { where: { channelId: user.channelId }, returning: true, transaction });
						}
	
						// update channel information
						return await this.channelRepository.update(channelObject, { where: { channelId: user.channelId }, returning: true, transaction });
					});
	
					console.log('resulteditsave', result);
					if (result) {
						console.log('resulteditsave', result);
						return true;
					} else {
						return false;
					}
				} else {
					// throw new AuthException('channel:subscripion:userNotFound');
				}
			} catch (e) {
				console.log("Error+", e);
				throw new InternalServerErrorException();
			}
		}
		public async updateFacts(params: any, user: any) {
			try {
				let userDetails = await this.userRepository.scope('userBasicInfom').findOne({
					where: { userId: parseInt(user.userId), isActive: '1' }
				});
				if (userDetails) {
					return await this.sequelizeInstance.transaction(async transaction => {
						let contacts = [];
						contacts.push(user.contactId);
						let ChannelDetailObject = <ChannelDetail>{};
						ChannelDetailObject = await this.tradeInformationInBound.setFactsChannelDetail(params);
	
						if (ChannelDetailObject) {
							return await this.channelDetailRepository.update(ChannelDetailObject, { where: { channelId: user.channelId }, returning: true, transaction });
						}
					});
				} else {
					throw new AuthException('channel:subscripion:userNotFound');
				}
			} catch (e) {
				console.log("Error+", e);
				throw new InternalServerErrorException();
			}
		}
		public async getFacts(user: any) {
			try { 
				return await this.channelDetailRepository.findOne({
				where: {channelId: user.channelId},
				attributes: ['empCnt','empCntRange', 'turnoverRange', 'turnover', 'startup', 'estYear', 'storeCount', 'no_of_skus', 'maturitylevel']
			})
		} catch (e) {
				console.log("Error+", e);
				throw new InternalServerErrorException();
			}
		}



		public async getKeyPartnerOptions(user: any){
			let keyRetailerOptions = [];
			let keyDistributorOptions = [];
			let keyResellerOptions = [];

			let keyPartner = {retailer : [], reseller : [], distributor : []};

			if(user.channelTypeId == '2'){
				keyRetailerOptions = await this.searchRetailer('',4);
				keyDistributorOptions = await this.searchRetailer('',3);
				keyResellerOptions = await this.searchRetailer('', 5);
				keyPartner['retailer'] = keyRetailerOptions;
				keyPartner['distributor'] = keyDistributorOptions;
				keyPartner['reseller'] = keyResellerOptions;
			}else if(user.channelTypeId == '3'){
				keyRetailerOptions = await this.searchRetailer('',4);
				keyResellerOptions = await this.searchRetailer('', 5);
				keyPartner['reseller'] = keyResellerOptions;
				keyPartner['retailer'] = keyRetailerOptions;
			}
	
			return Object.assign({keyPartners: keyPartner});

		}

		async getChannelProfile(user: any): Promise<any> {
			let tradeinfo: any = '';
			
			tradeinfo=await this.channelRepository.scope().findAll<Channel>({
				attributes:['channelId', 'profileTypeId'],                    
				where:{
					channelId:user.channelId
				},
				include: [
				
					{
						model:this.channelCustProfileRepository,
						as:'channelCustomerProfile',
						attributes: ['profileTypeId','profileTypeClassificationId'],
						// include: [
						//     {
						//         model:this.profileTypeRepository,
						//         attributes:['profileType']
						//     }
						// ]
						where:{
							deletedAt: null
						}
	
					},
				
	
				]
	
			})

			return tradeinfo;
		}


	public async update_company_new(user: any, data: any): Promise<boolean>{

		let companyProfile: any = '';
		let companyProfileJson: any = '';
		let channelStatusArray: Array<number> = [appconstant.CHANNEL_APPROVED, appconstant.CHANNEL_APPROVAL];

		//validate company name
		let realUser = user;
		if (realUser && realUser.channel && +realUser.channel.channelTypeId === +appconstant.CHANNEL_TYPE_HUB) {
			// before update verify if the company name already exist
			let result = await this.channelRepository.findOne({
				where: {
					companyName: data.company_name.trim(),
					channelId: {
						$not: +user.channel.channelId
					}
				}
			});

			if (result) {
				throw new ProcessException('channel:profile:companyNameAlreadyExist');
			}

			// allow edit
			channelStatusArray.push(+user.channel.signUpStatusId);
		}


		// return true;
		try {
			let result: any = '';
			console.log('user.channel.signUpStatusId', user.channel.signUpStatusId);
			let userSignUpStatus = channelStatusArray.includes(+user.channel.signUpStatusId);
			if (userSignUpStatus) {
				companyProfile = await this.userRepository.scope('userBasicInfom').findOne({
					include: [
						{
							model: this.contactRepository.scope('contactInfo'),
							where: { deletedAt: null },
							include: [
								{
									model: this.mediaRepository,
									include: [
										{
											model: this.mediaTypeRepository,
											required: false
										},
									],
									required: false
								}
							]
						},
						{
							model: this.channelRepository.scope('profileInfom'),
							where: { signUpStatusId: parseInt(user.channel.signUpStatusId), deletedAt: null },
							include: [
								{
									model: this.channelDetailRepository.scope('channelDetail'),
									where: { deletedAt: null },
									include: [
										{
											model: this.mediaRepository,
											as: 'channelLogo',
											include: [
												{
													model: this.mediaTypeRepository,
													required: false
												},
											],
											required: false
										},
										{
											model: this.mediaRepository,
											as: 'channelIcon',
											include: [
												{
													model: this.mediaTypeRepository,
													required: false
												},
											],
											required: false
										}
									]
								},
								{
									model: this.channelCertiRepository.scope('profileInfo'),
									where: { deletedAt: null },
									required: false
								},
								{
									model: this.channelDocumentRepository,
									where: { deletedAt: null },
									required: false,
									include: [
										{
											model: this.mediaRepository,
											as: 'document',
											required: false
										}
									]
								}
							]
						}
					],
					where: { userId: parseInt(user.userId), deletedAt: null }
				});
			}

			if (companyProfile) {
				companyProfileJson = companyProfile.toJSON();
				await this.save_company_registration(Object.assign(data.companyRegistration, {user: user}), companyProfile, realUser);
				await this.save_trade_information(Object.assign(data.tradeInformation, {user: user}), companyProfile, realUser);

				if(data.channels)
				await this.save_channel_info(Object.assign(data.channels, {user: user}), companyProfile, realUser);


				await this.save_marketing_info(Object.assign(data.marketing, {user: user}), companyProfile, realUser);
				await this.save_facts_figures(Object.assign(data.facts, {user: user}), companyProfile, realUser);
				await this.save_settingsdocuments(Object.assign(data.document, {user: user}), companyProfile, realUser);

				if(+user.channel.signUpStatusId == appconstant.CHANNEL_APPROVAL || data.ratingId){

					let defaultdata = {
						channelId : +user.channel.channelId,
						ratingId : 1,
						isCompleted : true,
						createdAt : new Date()
					}
	
					await this.profileRepository.findOrCreate({where: {channelId: +user.channel.channelId, ratingId: 1}, defaults: defaultdata}).then((result) => {
						if(!result[1]){
							result[0].update(defaultdata);
						}
					});

				}

			}

			return true
		}
		catch(err){
			console.log(err);
		}
	}

	public async save_company_registration(data: any, companyProfile: any, realUser: any){
		
        let certificate=[];
		let result: any = '';
		let requestObject = {...data.certificates};
        // if(data.certificate.companyRegNumber!='')
        //     certificate.push({channelId:data.user.channelId,cerificateTypeId:1,cerificateName:data.certificate.certificateName,cerificateNumber:data.certificate.companyRegNumber});
        // if(data.certificate.vatNumber!=null&&data.certificate.vatNumber!=''){
		// 	certificate.push({channelId:data.user.channelId,cerificateTypeId:2,cerificateName:'VAT',cerificateNumber:data.certificate.vatNumber, vat_verified: data.certificate.vat_verified});	
		// }
            
		try{
			result = this.sequelizeInstance.transaction(async transaction => {
				// await this.channelCertiRepository.bulkCreate(certificate, {               
				// 	transaction,returning: true
				// });

				let billAddress: any;
				let comapnyLogoId: any = null;			
				let comapnyRegAddressId: any = null;
				let comapnyBillAddressId: any = null;
				if (data.reg_address_id) {
					await this.addressRepository.update({
						address: data.reg_address,
						city: data.reg_city,
						countryId: data.reg_country_id,
						postalCode: data.reg_postal_code,
					},
						{
							where: { addressId: companyProfile.channel.regAddressId },
							returning: true, transaction,
						});
					comapnyRegAddressId = companyProfile.channel.regAddressId;
				}

				if (data.is_same_address) {
					comapnyBillAddressId = comapnyRegAddressId;
				} else {
					// update company billing address to address table
					if (data.bill_address_id) {
						await this.addressRepository.update({
							address: data.bill_address,
							city: data.bill_city,
							countryId: data.bill_country_id,
							postalCode: data.bill_postal_code,
						},
							{
								where: { addressId: data.bill_address_id },
								returning: true, transaction,
							});
						comapnyBillAddressId = data.bill_address_id;
					} else {
						// save company billing address to address table
						if (data.bill_address && data.bill_city && data.bill_country_id && data.bill_postal_code) {
							billAddress = await this.addressRepository.create(
								{
									address: data.bill_address,
									city: data.bill_city,
									countryId: data.bill_country_id,
									postalCode: data.bill_postal_code,
								},
								 {	
									 transaction 
								 }
							);
							comapnyBillAddressId = billAddress.addressId;
						}
					}
				}

				if (requestObject.registration_certificate && Object.keys(requestObject.registration_certificate).length > 0) {
					for (let ii = 0; ii < requestObject.registration_certificate.length; ii++) {
						if (requestObject.registration_certificate[ii].channel_certificate_id) {
							// update registration certifiacte to channel certificate table
							await this.channelCertiRepository.update({
								channelId: companyProfile.channelId,
								cerificateName: requestObject.registration_certificate[ii].name,
								cerificateNumber: requestObject.registration_certificate[ii].id,
								cerificateTypeId: appconstant.CERTIFICATETYPE_REGISTRATION
							},
								{
									where: { channelCerificateId: requestObject.registration_certificate[ii].channel_certificate_id },
									returning: true, transaction,
								});
						} else {
							// save registration certificate to channel certificate table
							console.log('creating cert')
							await this.channelCertiRepository.create({
								channelId: companyProfile.channelId,
								cerificateName: requestObject.registration_certificate[ii]?.name,
								cerificateNumber: requestObject.registration_certificate[ii].id,
								cerificateTypeId: appconstant.CERTIFICATETYPE_REGISTRATION
							},
								{
									 transaction
								});
						}
					}
				}

				if (requestObject.tax_certificate && Object.keys(requestObject.tax_certificate).length > 0) {
					for (let ii = 0; ii < requestObject.tax_certificate.length; ii++) {
						if (requestObject.tax_certificate[ii].channel_certificate_id) {
							// save tax certifiacte to channel certificate table
							await this.channelCertiRepository.update({
								channelId: companyProfile.channelId,
								cerificateName: requestObject.tax_certificate[ii].name,
								cerificateNumber: requestObject.tax_certificate[ii].id,
								cerificateTypeId: appconstant.CERTIFICATETYPE_TAX,
								vat_verified: requestObject.tax_certificate[ii].vat_verified
							},
								{
									where: { channelCerificateId: requestObject.tax_certificate[ii].channel_certificate_id },
									returning: true, transaction,
								});
						} else {
							// update tax certifiacte to channel certificate table
							await this.channelCertiRepository.create({
								channelId: companyProfile.channelId,
								cerificateName: requestObject.tax_certificate[ii].name,
								cerificateNumber: requestObject.tax_certificate[ii].id,
								cerificateTypeId: appconstant.CERTIFICATETYPE_TAX
							},
								{
									transaction
								});
						}
					}
				}

				if (requestObject.deleted_registration_certificate && requestObject.deleted_registration_certificate.length > 0) {
					for (let ii = 0; ii < requestObject.deleted_registration_certificate.length; ii++) {
						if (requestObject.deleted_registration_certificate[ii]) {
							// update certificate details to channel certificate table
							await this.channelCertiRepository.update({
								channelId: companyProfile.channelId,
								deletedAt: new Date()
							},
								{
									where: { channelCerificateId: requestObject.deleted_registration_certificate[ii] },
									returning: true, transaction,
								});
						}
					}
				}

				if (requestObject.deleted_tax_certificate && requestObject.deleted_tax_certificate.length > 0) {
					for (let ii = 0; ii < requestObject.deleted_tax_certificate.length; ii++) {
						if (requestObject.deleted_tax_certificate[ii]) {
							// update certificate details to channel certificate table
							await this.channelCertiRepository.update({
								channelId: companyProfile.channelId,
								deletedAt: new Date()
							},
								{
									where: { channelCerificateId: requestObject.deleted_tax_certificate[ii] },
									returning: true, transaction,
								});
						}
					}
				}

				// update channel information's address to channel table
				let companyInfoUpdate: any = {
					regAddressId: comapnyRegAddressId,
					billAddressId: comapnyBillAddressId,
					countryId: +data.reg_country_id,
					...(data.profileTypeId && {profileTypeId: +data.profileTypeId}),
					...(data.isOEM && {isOEM: +data.isOEM})
				};

				console.log('realUSer', realUser.channel, +realUser.channel.channelTypeId, +appconstant.CHANNEL_TYPE_HUB)

				if (realUser && realUser.channel && +realUser.channel.channelTypeId === +appconstant.CHANNEL_TYPE_HUB) {
					console.log('company_name', data.company_name);
					companyInfoUpdate.companyName = data.company_name.trim();
				}

				await this.channelRepository.update(companyInfoUpdate,
					{
						where: { channelId: companyProfile.channel.channelId }, returning: true, transaction,
					});

					if (companyProfile.channel.channelDetail.channelLogoId) {
						comapnyLogoId = companyProfile.channel.channelDetail.channelLogoId;
					}

					// save comapny logo detail to document table
					if (data.company_logo_url) {
						comapnyLogoId = data.company_logo_url;
					}

					// update channel detals address to channel detail table
					 await this.channelDetailRepository.update({
						channelLogoId: comapnyLogoId,
						phone1: data.company_phone1,
						},
						{
							where: { channelId: companyProfile.channelId }, returning: true, transaction,
						});						

			 });

			if (result) {
				return true;
			} else {
				return false;
			}

			}
			catch(err){
				console.log("Error+", err);
			}
			if (!companyProfile) {
				throw new ProcessException('channel:getCompanyProfile:companyNotFound');
			}

	}

	public async save_trade_information(data: any, companyProfile: any, user: any){
		let result: any = '';
		let userDetails = await this.userRepository.scope('userBasicInfom').findOne({
			where: { userId: parseInt(user.userId), isActive: '1' }
		});

		try{

		
		if (userDetails) {
		result = await this.sequelizeInstance.transaction(async transaction => {
		// update channel detals address to channel detail table
		let details={}
		if(user.channelTypeId==4 || user.channelTypeId==5){
			console.log('data.companyDesc',data.companyDesc);
		 details={
			detailDesc: data.company_about,//sanitizeHtml(requestObject.company_about, appconstant.SANITIZE_HTML),
			expectations: data.company_expectation,//(requestObject.company_expectation) ? sanitizeHtml(requestObject.company_expectation, appconstant.SANITIZE_HTML) : ''
			companyDesc:data.companyDesc,
			isPhysicalSales:data.isPhysicalSales,
			physicalSales:data.physicalSales?+data.physicalSales:null
		}
		}else{
			 details={
				detailDesc: data.company_about,//sanitizeHtml(requestObject.company_about, appconstant.SANITIZE_HTML),
				expectations: data.company_expectation,//(requestObject.company_expectation) ? sanitizeHtml(requestObject.company_expectation, appconstant.SANITIZE_HTML) : ''
				companyDesc:data.companyDesc
			}
		}
		let con=await this.channelDetailRepository.update(details,
		{
				where: { channelId: companyProfile.channelId }, returning: true, transaction,
		});
		console.log('con', con);
		let channelContacts = await this.contactRepository.scope('basicInfom').findAll({
			where: {
				deletedAt: null,
				contactId: { [Op.ne]: user.contactId },
				channelId: user.channelId
			}
		});

		let contacts = [];
		contacts.push(user.contactId);
		if (channelContacts.length > 0) {
			channelContacts.map(function (data) {
				contacts.push(data.contactId);
			});
		}

		if (data.productFamilies?.length > 0) {

			let productGroup = await this.channelProductGroupRepository.findAll({
				where: {
					channelId: user.channelId,
					deletedAt: null
				}
			});

			let ChannelProductGroupIds = await this.tradeInformationInBound.setProductGroupIds(productGroup);
			let ChannelProductGroupObject: ChannelProductGroup[] = await this.tradeInformationInBound.setProductGroup(data.productFamilies, user.channelId);
			let ChannelProductGroupCurrentIds = [];
			let ChannelProductGroupCreateObject = [];

			/* Account User contact update */
			// let ContactProductGroupIds = await this.tradeInformationInBound.setProductGroupIds(contact.productGroups);
			let ContactProductGroupObject: ContactProductGroup[] = await this.tradeInformationInBound.setContactProductGroup(data.productFamilies, user.contactId);
			let ContactProductGroupCurrentIds = [];
			// let ContactProductGroupCreateObject = [];

			for (let kk = 0; kk < ChannelProductGroupObject.length; kk++) {
				if (ChannelProductGroupIds.includes(ChannelProductGroupObject[kk].productGroupId)) {
					ChannelProductGroupCurrentIds.push(ChannelProductGroupObject[kk].productGroupId);
				} else {
					ChannelProductGroupCreateObject.push(ChannelProductGroupObject[kk]);
					ChannelProductGroupCurrentIds.push(ChannelProductGroupObject[kk].productGroupId);
				}
				// if (ContactProductGroupIds.includes(ContactProductGroupObject[kk].productGroupId)) {
				// 	ContactProductGroupCurrentIds.push(ContactProductGroupObject[kk].productGroupId);
				// } else {
				// 	// ContactProductGroupCreateObject.push(ContactProductGroupObject[kk]);
				// 	ContactProductGroupCurrentIds.push(ContactProductGroupObject[kk].productGroupId);
				// }

				ContactProductGroupCurrentIds.push(ContactProductGroupObject[kk].productGroupId);
			}
			if (ChannelProductGroupCreateObject.length > 0) {
				await this.channelProductGroupRepository.bulkCreate(ChannelProductGroupCreateObject, { returning: true, transaction });
			}

			// if (ContactProductGroupCreateObject.length > 0) {
			// 	await this.contactProductGroupRepository.bulkCreate(ContactProductGroupCreateObject, { returning: true, transaction });
			// }

			await this.channelProductGroupRepository.update({ deletedAt: new Date() }, {
				where: {
					channelId: user.channelId,
					productGroupId: { [Op.notIn]: ChannelProductGroupCurrentIds },
					deletedAt: null
				}, returning: true, transaction
			});

			await this.contactGroupRepository.update({ deletedAt: new Date() }, {
				where: {
					contactId: { [Op.in]: contacts },
					productGroupId: { [Op.notIn]: ContactProductGroupCurrentIds },
					deletedAt: null
				}, returning: true, transaction
			});
		}

		if (data.productCategories?.length > 0) {

			let productCategory = await this.channelProductCategorieRepository.findAll({
				where: {
					channelId: user.channelId,
					deletedAt: null
				}
			});

			

			let productCategoriesIds = await this.tradeInformationInBound.setProductCategoryIds(productCategory);
			let ProductCategorieObject: ChannelProductCategorie[] = await this.tradeInformationInBound.setProductCategories(data.productCategories, user.channelId);
			let ChannelProductCategoriesCurrentIds = [];
			let ChannelProductCategoriesCreateObject = [];

			/* Account User contact update */
			// let contactProductCategoriesIds = await this.tradeInformationInBound.setProductCategoryIds(contact.productCategories);
			let contactProductCategorieObject: ContactProductCategory[] = await this.tradeInformationInBound.setContactProductCategories(data.productCategories, user.contactId);
			let ContactProductCategoriesCurrentIds = [];
			// let ContactProductCategoriesCreateObject = [];

			for (let kk = 0; kk < ProductCategorieObject.length; kk++) {
				if (productCategoriesIds.includes(ProductCategorieObject[kk].productCategorieId)) {
					ChannelProductCategoriesCurrentIds.push(ProductCategorieObject[kk].productCategorieId);
				} else {
					ChannelProductCategoriesCreateObject.push(ProductCategorieObject[kk]);
					ChannelProductCategoriesCurrentIds.push(ProductCategorieObject[kk].productCategorieId);
				}
				// if (contactProductCategoriesIds.includes(contactProductCategorieObject[kk].productCategorieId)) {
				// 	ContactProductCategoriesCurrentIds.push(contactProductCategorieObject[kk].productCategorieId);
				// } else {
				// 	// ContactProductCategoriesCreateObject.push(contactProductCategorieObject[kk]);
				// 	ContactProductCategoriesCurrentIds.push(contactProductCategorieObject[kk].productCategorieId);
				// }

				ContactProductCategoriesCurrentIds.push(contactProductCategorieObject[kk].productCategorieId);
			}
			
			if (ChannelProductCategoriesCreateObject.length > 0) {
				console.log('ChannelProductCategoriesCreateObject', ChannelProductCategoriesCreateObject)
				await this.channelProductCategorieRepository.bulkCreate(ChannelProductCategoriesCreateObject, { returning: true, transaction });
			}

			// if (ContactProductCategoriesCreateObject.length > 0) {
			// 	await this.contactProductCategoryRepository.bulkCreate(ContactProductCategoriesCreateObject, { returning: true, transaction });
			// }

			await this.channelProductCategorieRepository.update({ deletedAt: new Date() }, {
				where: {
					channelId: user.channelId,
					productCategorieId: { [Op.notIn]: ChannelProductCategoriesCurrentIds },
					deletedAt: null
				}, returning: true, transaction
			});

			await this.contactCatRepository.update({ deletedAt: new Date() }, {
				where: {
					contactId: { [Op.in]: contacts },
					productCategorieId: { [Op.notIn]: ContactProductCategoriesCurrentIds },
					deletedAt: null
				}, returning: true, transaction
			});


		}

		//saving brand
		if (data.brand?.length > 0) {

			let channelBrandObject = await this.saveBrand(data.brand, user.channelId, transaction);

			if (channelBrandObject.channelBrandArrayObject.length > 0) {

				let brand = await this.channelBrandRepository.findAll({
					where: {
						channelId: user.channelId,
						deletedAt: null
					}
				});

				let ChannelBrandCurrentIds = [];
				let ChannelBrandCreateObject = [];
				let brandObject = channelBrandObject.channelBrandArrayObject;
				let brandIds = channelBrandObject.brandIds;

				let ChannelBrandIds = await this.tradeInformationInBound.setchannelbrandIds(brand);
				let totalRemovedBrands = ChannelBrandIds.filter(e => !brandIds.find(a => e == a));

				for (let kk = 0; kk < brandObject.length; kk++) {
					if (!ChannelBrandIds.includes(brandObject[kk].brandId)) {
						ChannelBrandCreateObject.push(brandObject[kk]);
					}

					if(!totalRemovedBrands.includes(+brandObject[kk].brandId)) { 
						ChannelBrandCurrentIds.push(brandObject[kk].brandId)
					}
				}

				if (ChannelBrandCreateObject.length > 0) {
					await this.channelBrandRepository.bulkCreate(ChannelBrandCreateObject, { returning: true, transaction });
				}

				await this.channelBrandRepository.update({ deletedAt: new Date() }, {
					where: {
						channelId: user.channelId,
						brandId: { [Op.notIn]: ChannelBrandCurrentIds },
						deletedAt: null
					}, returning: true, transaction
				});

				await this.brandRepository.update({
					isAssigned: '1'
				}, {
						where: {
							id: {
								[Op.in]: brandIds
							}
						},
						returning: true, transaction
					}
				);

				await this.brandRepository.update({
					isAssigned: '0'
				}, {
						where: {
							id: {
								[Op.in]: totalRemovedBrands
							}
						},
						returning: true, transaction
					}
				);
			}
		}

		if(user.channelTypeId==4 || user.channelTypeId==5){
			if(data.endCustomers){
				let b2b=this.endCustomersB2bRepository.findOne({
					where:{channelId:+user.channelId}
				});
				if(b2b){
				this.endCustomersB2bRepository.update({
					customerTypeId:data.endCustomers,
					other:data.otherEndCustomer
				}, {where:{channelId:+user.channelId}});
				}else{
					this.endCustomersB2bRepository.create({
						channelId:+user.channelId,
						customerTypeId:data.endCustomers,
						other:data.otherEndCustomer
					})
				}
			}
		}


					if(data.distributorSpeciality?.length){
						let ChannelSpecialityObject = <Array<ChannelSpeciality>>[];

						//Speciality Object
						ChannelSpecialityObject = await this.tradeInformationInBound.setSpeciality(data.distributorSpeciality, user.channelId);
						console.log('speciality dist', ChannelSpecialityObject, data.distributorSpeciality)
	
						if (ChannelSpecialityObject.length > 0) {
	
							let speciality = await this.channelSpecialityRepository.findAll({
								where: {
									channelId: user.channelId,
									deletedAt: null
								}
							});
	
							let ChannelSpecialityIds = await this.tradeInformationInBound.setProductGroupIds(speciality);
							let ChannelSpecialityCurrentIds = [];
							let ChannelSpecialityCreateObject = [];
							for (let kk = 0; kk < ChannelSpecialityObject.length; kk++) {
								if (ChannelSpecialityIds.includes(ChannelSpecialityObject[kk].productGroupId)) {
									ChannelSpecialityCurrentIds.push(ChannelSpecialityObject[kk].productGroupId);
								} else {
									ChannelSpecialityCreateObject.push(ChannelSpecialityObject[kk]);
									ChannelSpecialityCurrentIds.push(ChannelSpecialityObject[kk].productGroupId);
								}
							}
							if (ChannelSpecialityCreateObject.length > 0) {
								await this.channelSpecialityRepository.bulkCreate(ChannelSpecialityCreateObject, { returning: true, transaction });
							}
	
							await this.channelSpecialityRepository.update({ deletedAt: new Date() }, {
								where: {
									channelId: user.channelId,
									productGroupId: { [Op.notIn]: ChannelSpecialityCurrentIds },
									deletedAt: null
								}, returning: true, transaction
							});
						}
					}
					
		
		});
	}
	if (result) {
		return true;
	} else {
		return false;
	}

	}
	catch(err){
		console.log("Error+", err);
	}
	}


	public async save_channel_info(data: any, companyProfile: any, user: any){

		console.log('datadata,',data);

		let result: any = '';
		try{
			result = this.sequelizeInstance.transaction(async transaction => {

				if (data.customerProfiles && data.customerProfiles?.length > 0) {

					let customerProfiles = await this.channelCustomerProfileRepository.findAll({
						where: {
							channelId: user.channelId,
							deletedAt: null
						}
					});

					let customerProfilesIds = await this.tradeInformationInBound.setCustomerProfilesIds(customerProfiles);
					let customerProfilesObject: ChannelCustomerProfile[] = await this.tradeInformationInBound.setCustomerProfiles(data.customerProfiles, user.channelId);
					let channelCustomerProfilesCurrentIds = [];
					let ChannelCustomerProfilesCreateObject = [];
					for (let kk = 0; kk < customerProfilesObject.length; kk++) {
						if (customerProfilesIds.includes(customerProfilesObject[kk].profileTypeId)) {
							channelCustomerProfilesCurrentIds.push(customerProfilesObject[kk].profileTypeId);
						} else {
							ChannelCustomerProfilesCreateObject.push(customerProfilesObject[kk]);
							channelCustomerProfilesCurrentIds.push(customerProfilesObject[kk].profileTypeId);
						}
					}
					if (ChannelCustomerProfilesCreateObject.length > 0) {
						await this.channelCustomerProfileRepository.bulkCreate(ChannelCustomerProfilesCreateObject, { returning: true, transaction });
					}
					console.log('ChannelCustomerProfilesCreateObject',ChannelCustomerProfilesCreateObject)
					await this.channelCustomerProfileRepository.update({ deletedAt: new Date() }, {
						where: {
							channelId: user.channelId,
							profileTypeId: { [Op.notIn]: channelCustomerProfilesCurrentIds },
							deletedAt: null
						}, returning: true, transaction
					});
				}
				if(user.channelTypeId!==4 && user.channelTypeId!==5){

				if(user.channelTypeId==2){
				// update key Dist
				await this.tradeInformationInBound.keyRetailers(appconstant.CHANNEL_TYPE_DISTRIBUTOR, data.channelKeyDistributor, user.channelId, user.userId, data.deletedkeydistributor);	
				}

				// update key Retailers
				await this.tradeInformationInBound.keyRetailers(appconstant.CHANNEL_TYPE_RETAILER, data.channelKeyRetailer, user.channelId, user.userId, data.deletedkeyretailer);

				// create key Resellers
				await this.tradeInformationInBound.keyRetailers(appconstant.CHANNEL_TYPE_RESELLER, data.channelKeyReseller, user.channelId, user.userId, data.deletedkeyreseller);

				let ChannelDetailObject = await this.tradeInformationInBound.setChannelDetail(data, user.channelId, user.channelTypeId);

				console.log('channelDetailObject', ChannelDetailObject)
				if (ChannelDetailObject) {
					await this.channelDetailRepository.update(ChannelDetailObject, { where: { channelId: user.channelId }, returning: true, transaction });
				}

				await this.channelRepository.update({isOEM: data.isOEM,  isCompanyPublic: data.isCompanyPublic}, { where: { channelId: user.channelId }, returning: true, transaction });				
}
			});
			
			if(result){
				return true;
			}else{
				return false;
			}
		}catch(err){
			console.log('error', err);
		}

	}

	public async save_marketing_info(data: any, companyProfile: any, user: any){

		let result: any = '';

		let requestObject={...data.socialProfile, ...data.award, ...data};

		try{
			result = await this.sequelizeInstance.transaction(async transaction => {
				if (requestObject.social_linkedin_id) {
					// update channel social linkedin profile details to channel social link table
					await this.socialLinkRepository.update({
						socialSiteId: appconstant.SOCIAL_SITE_TYPE_LINKEDIN,
						channelId: companyProfile.channelId,
						channelSocialLink: requestObject.social_linkedin_url
					},
						{
							where: { channelSocialLinkId: requestObject.social_linkedin_id },
							returning: true, transaction,
						});
				} else {
					// save channel social linkedin profile details to channel social link table
					if (requestObject.social_linkedin_url && Object.keys(requestObject.social_linkedin_url).length > 0) {
						await this.socialLinkRepository.create({
							socialSiteId: appconstant.SOCIAL_SITE_TYPE_LINKEDIN,
							channelId: companyProfile.channelId,
							channelSocialLink: requestObject.social_linkedin_url
						},
							{
								transaction
							});
					}
				}
		
				// save channel social google plus profile details to channel social link table
				if (requestObject.social_googleplus_id) {
					// update channel social google plus profile details to channel social link table
					await this.socialLinkRepository.update({
						socialSiteId: appconstant.SOCIAL_SITE_TYPE_GOOGLEPLUS,
						channelId: companyProfile.channelId,
						channelSocialLink: requestObject.social_googleplus_url
					},
						{
							where: { channelSocialLinkId: requestObject.social_googleplus_id },
							returning: true, transaction,
						});
				} else {
					// save channel social google plus profile details to channel social link table
					if (requestObject.social_googleplus_url && Object.keys(requestObject.social_googleplus_url).length > 0) {
						await this.socialLinkRepository.create({
							socialSiteId: appconstant.SOCIAL_SITE_TYPE_GOOGLEPLUS,
							channelId: companyProfile.channelId,
							channelSocialLink: requestObject.social_googleplus_url
						},
							{
								 transaction
							});
					}
				}
		
				if (requestObject.social_instagram_id) {
					// update channel social instagram profile details to channel social link table
					await this.socialLinkRepository.update({
						channelId: companyProfile.channelId,
						channelSocialLink: requestObject.social_instagram_url
					},
						{
							where: { channelSocialLinkId: requestObject.social_instagram_id },
							returning: true, transaction,
						});
				} else {
					// save channel social instagram profile details to channel social link table
					if (requestObject.social_instagram_url && Object.keys(requestObject.social_instagram_url).length > 0) {
						await this.socialLinkRepository.create({
							socialSiteId: appconstant.SOCIAL_SITE_TYPE_INSTAGRAM,
							channelId: companyProfile.channelId,
							channelSocialLink: requestObject.social_instagram_url
						},
							{
								transaction
							});
					}
				}
		
				if (requestObject.social_twitter_id) {
					// update channel social instagram profile details to channel social link table
					await this.socialLinkRepository.update({
						channelId: companyProfile.channelId,
						channelSocialLink: requestObject.social_twitter_url
					},
						{
							where: { channelSocialLinkId: requestObject.social_twitter_id },
							returning: true, transaction,
						});
				} else {
					// save channel social instagram profile details to channel social link table
					if (requestObject.social_twitter_url && Object.keys(requestObject.social_twitter_url).length > 0) {
						await this.socialLinkRepository.create({
							socialSiteId: appconstant.SOCIAL_SITE_TYPE_TWITTER,
							channelId: companyProfile.channelId,
							channelSocialLink: requestObject.social_twitter_url
						},
							{
								transaction
							});
					}
				}

				if (requestObject.social_facebook_id) {
					// update channel social instagram profile details to channel social link table
					await this.socialLinkRepository.update({
						channelId: companyProfile.channelId,
						channelSocialLink: requestObject.social_facebook_url
					},
						{
							where: { channelSocialLinkId: requestObject.social_facebook_id },
							returning: true, transaction,
						});
				} else {
					// save channel social instagram profile details to channel social link table
					if (requestObject.social_facebook_url && Object.keys(requestObject.social_facebook_url).length > 0) {
						await this.socialLinkRepository.create({
							socialSiteId: appconstant.SOCIAL_SITE_TYPE_FACEBOOK,
							channelId: companyProfile.channelId,
							channelSocialLink: requestObject.social_facebook_url
						},
							{
								transaction
							});
					}
				}
				 

				if (requestObject.B2B_url && Object.keys(requestObject.B2B_url).length > 0) {
		
					for (let ii = 0; ii < requestObject.B2B_url.length; ii++) {
						if (requestObject.B2B_url[ii].channel_B2B_id) {
							// update channel B2B details to channel B2B table
		
							await this.channelB2BRepository.update({
								channelId: companyProfile.channelId,
								channelB2B: requestObject.B2B_url[ii].B2B_url
							},
								{
									where: { channelB2BId: requestObject.B2B_url[ii].channel_B2B_id },
									returning: true, transaction,
								});
						} else {
							// save channel B2B details to channel B2B table
		
							await this.channelB2BRepository.create({
								channelId: companyProfile.channelId,
								channelB2B: requestObject.B2B_url[ii].B2B_url
							},
								{
									 transaction
								});
						}
					}
				}
		
				if (requestObject.deleted_B2B && requestObject.deleted_B2B.length > 0) {
					for (let ii = 0; ii < requestObject.deleted_B2B.length; ii++) {
						if (requestObject.deleted_B2B[ii]) {
							// update channel B2B details to channel B2B table
							await this.channelB2BRepository.update({
								channelId: companyProfile.channelId,
								deletedAt: new Date()
							},
								{
									where: { channelB2BId: requestObject.deleted_B2B[ii] },
									returning: true, transaction,
								});
						}
					}
				}

				//deleting deleted Awards

				if (requestObject.deleted_award_details && requestObject.deleted_award_details.length > 0) {
	
					// update award details to channel award table
					await this.awardRepository.update({
						// channelId: companyProfile.channelId,
						deletedAt: new Date()
					},
						{
							where: { 
								channelAwardId: { 
									[Op.in] : requestObject.deleted_award_details 
								}
							},
							returning: true, transaction,
						});
				}

				if (requestObject.award_details && Object.keys(requestObject.award_details).length > 0) {
					for (let ii = 0; ii < requestObject.award_details.length; ii++) {
											await this.awardRepository.create({
												channelId: companyProfile.channelId,
												awardDetail: 'channel',
												issuedBy: 'ch',
												documentId: requestObject.award_details[ii].documentId,
											},
												{
													 transaction
												});
											}
									}

			//Company Images
			if (requestObject.companyImages && Object.keys(requestObject.companyImages).length > 0) {
				for (let ii = 0; ii < requestObject.companyImages.length; ii++) {
						// save channel document
						if(!requestObject.companyImages[ii].companyMediaId){
						await this.companyMediaImagesRepository.create({
							channelId: companyProfile.channelId,
							documentId: requestObject.companyImages[ii].documentId,
							contactId:companyProfile.contactId,
							updatedBy: companyProfile.userId
						},
							{
								 transaction
							});
							}
				}
			}

			if (requestObject.deleted_companyImages && requestObject.deleted_companyImages.length > 0) {
				for (let ii = 0; ii < requestObject.deleted_companyImages.length; ii++) {
					if (requestObject.deleted_companyImages[ii]) {
						// update award details to channel award table
						await this.companyMediaImagesRepository.update({
							deletedAt: new Date()
						},
							{
								where: { companyMediaId: requestObject.deleted_companyImages[ii] },
								returning: true, transaction,
							});
					}
				}
			}


			//updating banner image
			let bannerId: any = null;
					if (requestObject.bannerImage.length>0) {
						let companyBanner;
						if (!isUndefined(requestObject.bannerImage[0].id) && requestObject.bannerImage[0].id != '') {
							bannerId = requestObject.bannerImage[0].id;
						} else {
							// save comapny logo detail to document table

							let oldBanner = await this.channelDetailRepository.findOne({
								where: {channelId: user.channelId},
								attributes: ['channelBannerId']
							}).then( res => {
								console.log('banner', res.channelBannerId);
								return res.channelBannerId;
							}).catch(err => console.log('banner error', err));

							if(oldBanner){
								await this.mediaRepository.update({deletedAt: new Date()}, {where: {
									documentId: +oldBanner
								}});
							}

							companyBanner = await this.mediaRepository.create({
								documentPath: requestObject.bannerImage[0].image,
								documentTypeId: appconstant.DOCUMENTTYPE_IMAGE,
								dirName: appconstant.FOLDER_PATH_COMPANY_BANNER
							},
								{
									 transaction,
								});
								bannerId = companyBanner.documentId;


								await this.channelDetailRepository.update({
										channelBannerId:bannerId}, {where: {
										channelId: user.channelId
									}});

						}
					}

			//saving certificate tags

			if(data.certificatesList?.length){
				let certificateTag : CertificateTagDto[] = data.certificatesList.map(x => {
					x.channelId = user.channelId;
					x.createdat = new Date();
					return x;
				});

				console.log('certificateTag', certificateTag);
				await CertificateTag.bulkCreate(certificateTag, {
					transaction
				});
			}

			if(data.deletedTags?.length){
				await CertificateTag.update({deletedat: new Date()}, {
					where: {
						channelId: user.channelId,
						certificatetagid: {
							[Op.in] : data.deletedTags
						}
					},
					transaction, returning: true
				})
			}

			if(data.company_email)
				await this.channelRepository.update({
					companyMail: data.company_email
				}, {where:{
					channelId: user.channelId
				}});


			let channelDetailData = {
				...(data.company_mobile_no && {mobileNo: data.company_mobile_no}),
				...(bannerId && {channelBannerId:bannerId}),
				...(data.company_website_url && {webSiteUrl: data.company_website_url})
			}

			
				await this.channelDetailRepository.update(channelDetailData, {where: {
					channelId: user.channelId
				}});

			});

			if(result)
				return true;
			else 
				return false;

		}catch(err){
			console.log('error ', err);
		}

		

	}

	public async save_facts_figures(data: any, companyProfile: any, user: any){

		let result: any = '';
		try{
			result = await this.sequelizeInstance.transaction(async transaction => {
				let channelDetailObject = await this.tradeInformationInBound.setChannelDetail(data, user.channelId, user.channelTypeId);
				await this.channelDetailRepository.update(channelDetailObject, {
					where: {
						channelId: user.channelId
					},
					returning: true, transaction
				});
			});

			if(result)
				return true;
			else
				return false;
		}catch(err){
			console.log('error', err);
		}

	}

	public async save_settingsdocuments(data: any, companyProfile:any, user: any){

		let result: any = '';
		let requestObject = {...data};

		try{
			result = await this.sequelizeInstance.transaction(async transaction => {
				if (requestObject.channel_document && Object.keys(requestObject.channel_document).length > 0) {
					for (let ii = 0; ii < requestObject.channel_document.length; ii++) {

								// let updatedDocument = await this.mediaRepository.create({
						// 	documentPath: requestObject.channel_document[ii].document,
						// 	documentTypeId: appconstant.DOCUMENTTYPE_DOC,
						// 	dirName: appconstant.FOLDER_PATH_COMPANY_DOCUMENT
						// },
						// 	{
						// 		 transaction,
						// 	});

						if (requestObject.channel_document[ii].channelDocumentId) {
							// update channel documents
							await this.channelDocumentRepository.update({
								channelId: companyProfile.channelId,
								documentId: requestObject.channel_document[ii].documentId,
								name: _.defaultTo(requestObject.channel_document[ii].documentName, ''),
								description: _.defaultTo(requestObject.channel_document[ii].documentDescription, ''),
								updatedBy: companyProfile.userId
							},
								{
									where: { channelDocumentId: requestObject.channel_document[ii].channelDocumentId },
									returning: true, transaction,
								});
						} else {
							// save channel document
							await this.channelDocumentRepository.create({
								channelId: companyProfile.channelId,
								documentId: requestObject.channel_document[ii].documentId,
								name: _.defaultTo(requestObject.channel_document[ii].documentName, ''),
								description: _.defaultTo(requestObject.channel_document[ii].documentDescription, ''),
								updatedBy: companyProfile.userId
							},
								{
									 transaction
								});
						}
					}
				}

				if (requestObject.deleted_channel_documents && requestObject.deleted_channel_documents.length > 0) {
					for (let ii = 0; ii < requestObject.deleted_channel_documents.length; ii++) {
						if (requestObject.deleted_channel_documents[ii]) {
							// update award details to channel award table
							await this.channelDocumentRepository.update({
								deletedAt: new Date()
							},
								{
									where: { channelDocumentId: requestObject.deleted_channel_documents[ii] },
									returning: true, transaction,
								});
						}
					}
				}

				//updating introvideo
				await this.channelDetailRepository.update({introvideoId: data.company_introvideo}, {
					where: {
						channelId: user.channelId
					}, returning: true, transaction
				});
			});

			if(result)
				return true;
			else
				return false;

		}catch(err){
			console.log('error ',err);
		}

	}

	public async getChannelCertificateTags(user: any){
		let result = await CertificateTag.findAll({where: {
			channelId : user.channelId,
			deletedat : null
		}});

		return result;
	}

	public async getCategoryDetails(groupId){
		let query = 'select "productCategorieId", count(*), (select "productCategorie" from "tblProductCategories" as tp where tp."productCategorieId"=tc."productCategorieId") from "tblProduct" as tc where tc."productCategorieId" in (select "productCategorieId" from "tblProductCategories" where "productGroupId"=:groupId) and tc."isactive"=\'1\' group by "productCategorieId"';

		let result = await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT,
			replacements: {
			  groupId: groupId
		  }	
		}).then(res => console.log('catresult', res));

	}

	public async getProducts(businessType) {                    
		let type = [] ;

		if(businessType==1)
			type=[1,3]
		if(businessType==2)
			type=[2,3]
		if(businessType==3)
			type=[1,2,3]

        let query='select pg."productGroupId" as groupId,pg."productGroup" as groupname from "tblProductGroup" as pg where pg."deletedAt" is null and pg."productGroup"!=\'\' and pg."businessType" in (:businessType) and pg."isActive"=\'1\' ';

         let productGroup=await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT,	
            replacements: {
				businessType:type
			}	
		}).then((results) => {  
			return results; 
        });

		let productGroup1 = [];
		let final = [];

		if(productGroup)
		{

			query =' select "productGroupId" as groupId, count(*), (select "productGroup" as groupname from "tblProductGroup" as pg '+
			'where pg."productGroupId"=tc."productGroupId"' +
			') from "tblProduct" as tc where tc."productGroupId" in (:productGroup) and tc."isactive"=\'1\' and tc."deletedat" is null  group by "productGroupId" order by count desc'


			// query = 'select "productGroupId" as groupId, "productGroup" as groupname from "tblProductGroup" where "isActive"=\'1\' ' +
			// 'and "deletedAt" is null and "businessType" in (:businessType) and "productGroup"!=\'\' and "productGroup" is not null and'+
			// '"productGroupId" not in (:productGroup) ';

			
			productGroup1=await this.sequelizeInstance.query(query, {
				type: QueryTypes.SELECT,	
				replacements: {
					businessType:type,
					productGroup:productGroup.map(x=>x.groupid)
				}	
			}).then((results) => {  
				return results; 
			});

			final = productGroup.filter(x=> !productGroup1.find(y => y.groupid==x.groupid));
			if(final)
				final = final.map(x => ({groupid: x.groupid, groupname: x.groupname, count:0}));
		
		}
        
        let data={
            productGroup:[...productGroup1, ...final],
        }

        
        return data;
    }

	public async getCategories(businessType) {                    
		let type = [] ;

		if(businessType==1)
			type=[1,3]
		if(businessType==2)
			type=[2,3]
		if(businessType==3)
			type=[1,2,3]

        let query='select pg."productCategorieId" as id,pg."productCategorie" as name from "tblProductCategories" as pg where pg."deletedAt" is null and pg."productCategorie"!=\'\' and pg."businessType" in (:businessType) and pg."isActive"=\'1\' ';

         let productCat=await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT,	
            replacements: {
				businessType:type
			}	
		}).then((results) => {  
			return results; 
        });
        
        let data={
            productCat:productCat,
        }

        
        return data;
    }

	public async guestUserRequirement(data){

		let res = {
			email: data.email,
			note:data.note,
			type:data.type
		}
		if(data.type==2)
		res['productSKU']=data.productSKU;
		else
		res['channelId']=data.channelId;
		return await this.guestRequirementRepository.create(res);
	}
	async getTopCompany(keyword,typeId){
	
		let type=[];
		if(typeId==1)
			type=['3','1'];
		else if(typeId==2)
			type=['2','3'];
		// else if(typeId==4)
		// 	type=[2,3];

		let query='select "channelId","companyName","channelTypeId" from "tblChannel" where "isOEM" in (:typeId) and unaccent("companyName") ILike :search and "deletedAt" is null and "isActive"=\'1\'  ' //and "isPublic"=\'1\'

		let company=await this.sequelizeInstance.query(query , {
			type: QueryTypes.SELECT,
			raw: true,
			replacements: {
				search: '' + keyword + '%',
				typeId:type,
				}
			});
		
		query='select "productId","product","productSKU" from "tblProduct" where unaccent("product") ILike :search and "deletedat" is null and "isactive"=\'1\' and ' //"isPublic"=\'1\' and
		+' "channelId" in (select "channelId" from "tblChannel" where "isactive"=\'1\' and "isOEM" in (:typeId)) limit 4' //and "isPublic"=\'1\'

		let product=await this.sequelizeInstance.query(query , {
			type: QueryTypes.SELECT,
			raw: true,
			replacements: {
				search: '' + keyword + '%',
				typeId: type
				}
			});
		
		let result={
			company:company,
			product:product,
			type:+typeId
		}

		

		return result;
		
	}

	// async getTopProduct(keyword){
	// 	let query='select "productId","product" from "tblProduct" where unaccent("companyName") ILike :search limit 4 and "isActive"=\'1\' and "isPublic"=\'1\'  '

	// 	let result=await this.sequelizeInstance.query(query , {
	// 		type: QueryTypes.SELECT,
	// 		raw: true,
	// 		replacements: {
	// 			search: '' + keyword + '%',
	// 			}
	// 		});
	// 	return result;
	// }

	async getMatchingVendors(channelId, data){
		let query = 'select tc."channelId",tc."companyName", (select td."documentPath" as logo from "tblChannelDetail" chd inner join "tblDocument" td on td."documentId" in (chd."channelLogoId") where chd."channelId"=tc."channelId") ' + 
		'from "tblChannel" tc where tc."channelId" in (select "channelId" from "tblProduct" where "productCategorieId" in (select "productCategorieId" from "tblProduct" where "channelId"=:channelId)) and ' +
		'tc."isActive"=\'1\' and tc."deletedAt" is null and tc."channelId"!=:channelId'

		let result = await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT,
			raw: true,
			replacements: {
				channelId: channelId
			},
		})

		let finalRes = {result : []};

		if(result){
			finalRes = {result : result.slice(data.offset).slice(0,data.limit)};
		}

		return finalRes;
	}

	async getMatchingBrands(channelId, data){
		
		let query = 'select tc."channelId" '+ 
		'from "tblChannel" tc where tc."channelId" in (select "channelId" from "tblProduct" where "productCategorieId" in (select "productCategorieId" from "tblProduct" where "channelId"=:channelId)) and ' +
		'tc."isActive"=\'1\' and tc."deletedAt" is null and tc."channelId"!=:channelId and tc."countryId"=(select "countryId" from "tblChannel" where "channelId"=:channelId)'

		let result = await this.sequelizeInstance.query(query, {
			type: QueryTypes.SELECT,
			raw: true,
			replacements: {
				channelId: channelId
			},
		}).then(x => {return  _.uniqBy(x,'channelId')})

		let result1=[];

		if(result)
		{
			let channelList = result.map(x => +x.channelId)


			// let query1 = 'select distinct on(tb."brandName") tb."brandName" as name, tb."id" as id, td."documentPath" as logo from "tblBrand" tb '
			// +'inner join "tblDocument" td on td."documentId"=tb."documentId" where id in (select "brandId" from "tblChannelBrand" where "channelId" in(:channelId)) order by tb."brandName" '

			let query1 = 'select distinct on(tc."companyName") tc."companyName" as name, tc."channelId" as id, td."documentPath" as logo from "tblChannel" tc inner join "tblChannelDetail" tcd on tc."channelId"=tcd."channelId"  ' +
			'left join "tblDocument" td on td."documentId"=tcd."channelLogoId" where tc."isActive"=\'1\' and tc."deletedAt" is null and tc."isPublic"=\'1\' and tc."channelId" in (:channelId)'

			result1 = await this.sequelizeInstance.query(query1, {
				type: QueryTypes.SELECT,
				raw: true,
				replacements: {
					channelId: channelList.length ? channelList : [-1]
				},
			}).then(res => res);
		}

		let finalRes = {result : []};

		if(result1){

			finalRes = {result : result1.slice(data.offset).slice(0,data.limit)};
		}

		return finalRes;
	}

	async getFeaturedVendors(data){

		// let type = [];
		// if (data.btype==1)
		// 	type=['1', '3']
		// else if (data.btype==2)
		// 	type=['2', '3']
		// else if (data.btype==3)
		// 	type=['1','2','3']
		// else type=['2']
		/*********** * old code **************/
		// let query = 'select tc."channelId", tc."companyName", chd."detailDesc",(select "documentPath" from "tblDocument" where "documentId"=chd."introvideoId") as "documentUrl" '
		// + 'from "tblChannel" tc inner join "tblChannelDetail" chd on tc."channelId"=chd."channelId" where tc."channelId" in (select "channelId" from "tblChannel" where "isActive"=\'1\' '
		// + 'and "deletedAt" is null and "isFeatured"=\'1\' and "isOEM" in (:typeId) and "channelTypeId"=2) ';
		/*********** * old code **************/
		
		// let query = 'select  tc."channelId", tc."companyName", chd."detailDesc", (select "documentPath" from "tblDocument" where "documentId"=chd."introvideoId") as "documentUrl" '
		// + 'from "tblChannel" tc inner join "tblChannelDetail" chd on tc."channelId"=chd."channelId" left join "tblChannelSubscription" cs on tc."channelId"=cs."channelId" '
		// +' where tc."channelId" in (select "channelId" from "tblChannel" where "isActive"=\'1\' and cs."isActive" = \'1\' and cs."isPaid" = \'1\' and "deletedAt" is null and "isFeatured"=\'1\' and "isOEM" in (:typeId) and "channelTypeId"=2) '
		// + ' and json_extract_path_text(cs."keyFeature", :isFeaturedByPlan ) = \'1\' '
		// +' AND  tc."channelId" not in (select csht."channelId" from "tblProductChannelHighlights" csht where tc."channelId"=csht."channelId" and csht."deletedAt" is null and csht."type" = \'c\'  and csht."createdAt" >= NOW() - interval \'1 week\' ) limit 5';
	
		// let	result = await this.sequelizeInstance.query(query, {
		// 	type: QueryTypes.SELECT,
		// 	raw: true,
		// 	replacements: {
		// 		typeId: type,
		// 		isFeaturedByPlan: (appconstant.SubscriptionCharcteristicFeature['COMPANY_FEATURE_BRAND_HOMEPAGE']).toString()
		// 	},
		// }).then(res => res);

		// let finalRes={result: []}
		// //console.log('result >> ', result);
		// if(result)
		// 	finalRes={result : result.slice(data.offset).slice(0, data.limit)}
		
		let res:any = await this.sequelizeInstance.query('select * from "fnGetFeatureVendor"(:btype)', {
			type: QueryTypes.SELECT,
			raw: true,
			replacements: {
				btype: data.btype
			}
		}).then((results) => {
			return results[0].fnGetFeatureVendor;
		});
		
		let finalRes={result: []}
		if(res)
			finalRes={result : res}
		return finalRes;

	}

	async getBuyerSearchCount(data){

		let filterch = [];
		let total = 0;
		

		let channelquery='select "channelId" from "tblChannel" as tc where tc."deletedAt" is null and "isActive"=\'1\' and "signUpStatusId"=15 ';

		if(data.filter.type){
			channelquery += ' and "channelTypeId" = '+data.filter.type;
		}
		
			let result= await this.sequelizeInstance.query(channelquery, {
			type: QueryTypes.SELECT,
			raw: true,
			replacements: {
			}
			}).then(x=>{				
				return  _.uniqBy(x,'channelId');
			})

			if(result){
				total = result.length;
			}
				

		if((data.filter.countryId||data.filter.categoryId)&& result) {
				let chlist = result.map(x => x.channelId);

		if(data.filter.countryId) {
			await this.sequelizeInstance.query('select array_agg("channelId") as ch from "tblChannel" where "countryId" in(:countryid) and "channelId" in (:channelid)', {
				type: QueryTypes.SELECT,
			raw: true,
			replacements: {
			countryid:data.filter.countryId,
			channelid: chlist
			}
			}).then((result) => {
				
				if(result[0].ch){
					filterch = [];
			filterch.push(...result[0].ch);
				}else
				filterch = [];
			});
			}
			
	
			if(data.filter.categoryId) {
				await this.sequelizeInstance.query('select array_agg("channelId") as ch from "tblChannelProductCategorie" cpc where cpc."productCategorieId" in(:productcatid) and "channelId" in(:channelid) ', {
					type: QueryTypes.SELECT,
				raw: true,
				replacements: {
				channelid:filterch.length>0 ? filterch : chlist,
				productcatid:data.filter.categoryId
				}
				}).then((result) => {
					
					if(result[0].ch){
						filterch = [];
				filterch.push(...result[0].ch);
					}else
					filterch = [];
		
				});
			}
			

			
			

			result = result.filter(ch => {return filterch.includes(ch.channelId);})
			result = result.map(x=>+x.channelId)
			total = result.length;
			
		}
		
		let finalRes={result: {}, product: 0}
		let result1 = {};
		let prod = 0;
		
		if(result.length){

			if(result[0].hasOwnProperty('channelId'))
				result = result.map(x => x.channelId)

			let query = 'select count(case when tc."channelTypeId"=2 then 1 else null end) as vendor, ' + 
			'count(case when tc."channelTypeId"=3 then 1 else null end) as distributor, ' +
			'count(case when tc."channelTypeId"=4 then 1 else null end) as retailer, ' +
			'count(case when tc."channelTypeId"=5 then 1 else null end) as reseller ' +
			'from "tblChannel" tc where tc."channelId" in (:channelId) and tc."isActive"=\'1\' and tc."deletedAt" is null';
	
				result1 = await this.sequelizeInstance.query(query, {
				type: QueryTypes.SELECT,
				raw: true,
				replacements: {
					channelId: result.length ? result : [null]
				}
			}).then(res => res[0]);

			let query1 = 'select count(*) from "tblProduct" where "channelId" in (select "channelId" from "tblChannel" where "channelId" in (:channelId) and "channelTypeId"=2) and "isactive"=\'1\' and "deletedat" is null';

				prod= await this.sequelizeInstance.query(query1, {
					type: QueryTypes.SELECT,
					raw: true,
					replacements: {
						channelId: result.length ? result : [null]
					}
				}).then(res => res[0]);
		}
		

		if(result1 || prod)
			finalRes={result : result1, product: prod}

		return finalRes;

	}


	async getFamilyFilter(data) {
		let productIds;

		try {

			if (data.length > 0) {

				let productcatquery = 'select pc."productGroupId" as id,pc."productGroup" as name from'
					+ ' "tblProductGroup" as pc where pc."productGroupId" in '
					+ '(select cpc."productGroupId" from "tblProduct" as cpc where "productId" in(:productId)) ';

				productIds = await this.sequelizeInstance.query(productcatquery, {
					type: QueryTypes.SELECT,
					raw: true,
					replacements: {
						//channelid:chlist.data
						productId: data
					}
				})

			}

		}
		catch (e) {
			console.log("productFilter:", e);
		}

		return productIds;

}

async getChannelAwards(user){
	// return await this.awardRepository.findAll({
	// 	where:{
	// 		deletedAt: null,
	// 		channelId: user.channelId
	// 	},
	// 	// include: [{
	// 	// 	model: Media,
	// 	// 	where: { 
	// 	// 		documentTypeId:1,
	// 	// 		deletedAt: null 
	// 	// 	},
	// 	// 	required: true
	// 	// }],
	// });

	let query = 'select ad."channelId", ad."channelAwardId" as id, ad."documentId", ad."isLifeTime", ad."issuedBy" as "awardName", (select concat( \'' + process.env.CDN_URL + '\',"documentPath") as "documentUrl" from "tblDocument" where "documentId"=ad."documentId" and "deletedAt" is null) from "tblChannelAward" as ad where "channelId"=:channelId and ad."documentId" is not null and "libraryDeteledAt" is null';
		return await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
			raw: true,
		replacements: {
			channelId:+user.channelId,
		}
		})
}

async getProductAwards(user){
    let query = 'select ad."productId", ad."productAwardId" as id, ad."documentId", ad."isLifeTime", ad."issuedBy" as "awardName", (select concat( \'' + process.env.CDN_URL + '\',"documentPath") as "documentUrl" from "tblDocument" where "documentId"=ad."documentId" and "deletedAt" is null) from "tblProductAward" as ad where "productId" in(select "productId" from "tblProduct" where "channelId"=:channelId and "deletedat" is null) and ad."documentId" is not null and "libraryDeteledAt" is null';
    return await this.sequelizeInstance.query(query, {
        type: QueryTypes.SELECT,
        raw: true,
    replacements: {
        channelId:+user.channelId,
    }
    })
}


async deleteChannelAwards(data){
	let query = 'update "tblChannelAward" set "libraryDeteledAt"=now() where "channelAwardId"=:channelAwardId';
		return await this.sequelizeInstance.query(query, {
            type: QueryTypes.SELECT,
			raw: true,
		replacements: {
			channelAwardId:data.id
		}
		})
}

async deleteProductAwards(data){
    let query = 'update "tblProductAward" set "libraryDeteledAt"=now() where "productAwardId"=:productAwardId';
    return await this.sequelizeInstance.query(query, {
        type: QueryTypes.SELECT,
        raw: true,
    replacements: {
		productAwardId:data.id
    }
    })
}
}
